//---------------------------------------------------------------------------
//
//  FILE NAME:          SCSClient.cc
//
//  DEVELOPMENT:        CXCDS, User Infrastructure
//
//  DESCRIPTION:
//
//
//---------------------------------------------------------------------------
// Copyright (c) 2006, Smithsonian Astrophysical Observatory
//---------------------------------------------------------------------------
 


#include <string>
#include <sstream>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <unistd.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <iostream>
#include <errno.h>

#include "SCSClient.hh"
#include "SCSConnection.hh"
#include "SCSMessage.hh"
#include "SCSLog.hh"

SCSClient::SCSClient()
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::SCSClient() enter",2);
#endif 

  connection=0;

#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::SCSClient() exit",2);
#endif 

};

SCSClient::~SCSClient()
{ 
#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::~SCSClient() enter",2);
#endif  


  if ( connection )
    delete connection;

  
#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::~SCSClient() exit",2);
#endif  
};



//
// Pre: Connection is not open. 
//
//
// Post: Connection using INET socket is opened to 
//       a server running on inHostName host
//       and listenning on port inPortId.  Returns 1 on success,
//       0 on timeout, -1 on error.  
//
//
// Desc: An attempt is made to establish INET socket connection to
//       specified port on specified host within inTimeoutUs useconds.
//
//
//


int SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId)
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId) enter",2);
#endif  
  
  if ( connection )
    CloseConnection();
  
  int tmpSock;

  struct hostent *hostData;
  struct sockaddr_in socketData;
  struct timeval timeStart,timeEnd;
  int stat;

  if (!(hostData=gethostbyname(inHostName.c_str())))
    {
      
#ifdef SCSDEBUG
      ostringstream tmpStr;
      tmpStr<<"ERROR : SCSClient::OpenConnection - gethostbyname failed for host '"<<inHostName<<"'";
      SCSLog::Write(tmpStr.str(),1);
#endif
      close(tmpSock);
      
#ifdef SCSDEBUG
      SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId) gethostbyname failed exit -1",2); 
#endif  
      return -1;
    };
  
  memset((void*)&socketData,0,sizeof(socketData));
  socketData.sin_family=AF_INET;
  memcpy((void*)&socketData.sin_addr,hostData->h_addr,hostData->h_length);
  
  socketData.sin_port=htons(inPortId);
  




  if ( gettimeofday(&timeStart,0) )
    {
#ifdef SCSDEBUG
      SCSLog::Write("ERROR : SCSClient::OpenConnection - gettimeofday failed",errno,1);
      
#endif
	  
      
#ifdef SCSDEBUG
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId) exit -1",2);
#endif  
	  return -1;
	};
      


  while(1)
    {


      //create socket
      
      if ( (tmpSock=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP)) < 0)
	{
	  
#ifdef SCSDEBUG
	  SCSLog::Write("ERROR : SCSClient::OpenConnection - socket create failed",errno,1);
	  
#endif
	  
#ifdef SCSDEBUG
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId) exit -1",2);
#endif  
	  continue;
	  return -1;
	};
      
      
      
      
      int tmpErr;
      
      // attempts to connect to server
      
      stat=connect(tmpSock,(struct sockaddr*)&socketData,sizeof(socketData));
      
      if ( stat )
	{
	  tmpErr=errno;
	  close(tmpSock);
	}
      else
	{
	  connection=new SCSConnection(tmpSock);
	  break;
	};

      if (gettimeofday(&timeEnd,0) )
	{
#ifdef SCSDEBUG
	  SCSLog::Write("ERROR : SCSClient::OpenConnection - gettimeofday failed",errno,1);
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId) exit -1",2);
#endif  
	  close(tmpSock);
	  continue;
	  return -1;
	};
      
      if ( ((timeEnd.tv_sec-timeStart.tv_sec)*1000000+(timeEnd.tv_usec-timeStart.tv_usec))> inTimeoutUs)
	{
#ifdef SCSDEBUG
	  SCSLog::Write("SCSClient::OpenConnection - connect failed",tmpErr,1);
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId) exit -1",2);
#endif  
	  close(tmpSock);
	  return 0;
	};
      close(tmpSock);
    };
  

#ifdef SCSDEBUG
      SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId) exit 1",2);
#endif  
  return 1;
};



//
// Pre: Connection is not open.
//
//
// Post: Connection using UNIX(local) socket inSocketName is opened to 
//       a server running on local host 
//       and listenning on socket inSocketName.  Returns 1 on success,
//       0 on timeout after successfull connection to server socket but with
//          no response from the server
//      -1 on error or timeout conncting to server socket.  
//
//
// Desc: An attempt is made to establish UNIX socket connection to
//       specified socket on local host.
//
//
//



int SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName)
{

#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) enter",2);
#endif  

  struct timeval timeStart,timeEnd;
  int stat;
  int tmpErr=0;
  int tmpSock;
  
  struct sockaddr_un socketData;
  struct sockaddr_un srvSocketData;
  int addrLen;
  int srvAddrLen;

 
  

      



  if ( gettimeofday(&timeStart,0) )
    {
#ifdef SCSDEBUG
      SCSLog::Write("ERROR : SCSClient::OpenConnection - gettimeofday failed",errno,1);
      SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit -1",2);
#endif  
      return -1;
    };


      
  memset((void*)&srvSocketData,0,sizeof(srvSocketData));
  srvSocketData.sun_family=AF_UNIX;
  strcpy(srvSocketData.sun_path,inSocketName.c_str());
  srvAddrLen=strlen(srvSocketData.sun_path)+sizeof(srvSocketData.sun_family);
  
#ifdef __APPLE__
  srvAddrLen+=+sizeof(srvSocketData.sun_len);
  srvSocketData.sun_len=srvAddrLen;
      
#endif
      
      
      


  if ( connection )
    CloseConnection();

  while(1)
    {

      
      
      //create socket
      
      if ( (tmpSock=socket(AF_UNIX,SOCK_STREAM,0)) < 0)
	{
#ifdef SCSDEBUG
	  SCSLog::Write("ERROR : SCSClient::OpenConnection - socket create failed",errno,1);

#endif
	  
#ifdef SCSDEBUG
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit -1",2);
#endif  
	  stat=-1;
	  continue;
	  return -1;
	};
      
      
      int yes=1;
      if (setsockopt(tmpSock,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(int)) == -1) 
	{
#ifdef SCSDEBUG
	  SCSLog::Write("ERROR : SCSClient::OpenConnection - setsockopt failed",errno,1);
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit -1",2);
#endif 

	  close(tmpSock);
	  stat=-1;
	  continue;
	  return -1;
	  
	} 
      
      
      
      //create connection
      
      connection=new SCSConnection(tmpSock);
      
      
      //bind socket
      
      ostringstream tmpStrStr;
      memset((void*)&socketData,0,sizeof(socketData));
      socketData.sun_family=AF_UNIX;
      tmpStrStr<<inSocketName<<"_"<<tmpSock<<"_"<<getpid();
      strcpy(socketData.sun_path,tmpStrStr.str().c_str());
      addrLen=strlen(socketData.sun_path)+sizeof(socketData.sun_family);
      
#ifdef __APPLE__
      addrLen+=+sizeof(socketData.sun_len);
      socketData.sun_len=addrLen;
#endif


      unlink(socketData.sun_path);
      if ( bind (connection->connectionSocket,(struct sockaddr*)&socketData,addrLen)<0)
	{
#ifdef SCSDEBUG
	  tmpErr=errno;
	  ostringstream tmpStr;
	  tmpStr<<"ERROR : SCSClient::OpenConnection - bind failed for socket '"<<tmpStr.str()<<"'";
	  SCSLog::Write(tmpStr.str(),tmpErr,1);
#endif

	  close(connection->connectionSocket);
	  connection->connectionSocket=-1;
	  delete connection;
	  connection=0;
	  
#ifdef SCSDEBUG
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit -1",2);
#endif  
	  stat=-1;
	  continue;
	  return -1;
	};	
      
      if ( chmod(socketData.sun_path,S_IRWXU) < 0 )
	{
#ifdef SCSDEBUG
	  tmpErr=errno;
	  ostringstream tmpStr;
	  tmpStr<<"ERROR : SCSClient::OpenConnection - chmod failed for socket '"<<tmpStr.str()<<"'";
	  SCSLog::Write(tmpStr.str(),tmpErr,1);
#endif

	  close(connection->connectionSocket);
	  unlink(socketData.sun_path);
	  connection->connectionSocket=-1;
	  delete connection;
	  connection=0;
	  
#ifdef SCSDEBUG
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit -1",2);
#endif  
	  stat=-1;
	  continue;
	  return -1;
	};
      
      
      
      
      
      // attempt to connect to server

      
      stat= connect(connection->connectionSocket,(struct sockaddr*)&srvSocketData,sizeof(socketData));

      tmpErr=errno;      


      if ( !stat)
	{
	  SCSMessage tmpMessage;
	  
	  stat = ReceiveMessage(inTimeoutUs,tmpMessage);
	  
	  if ( stat > 0 )
	    {
	      if ( tmpMessage.Type() != SCSMessage::PROTOCOL_ACK )
		{
		  stat =-2;
		}
	      else
		break;
	    };
	}
      else
	{
	 unlink(socketData.sun_path);
	};
  

	

      if (gettimeofday(&timeEnd,0))
	{
#ifdef SCSDEBUG
	  SCSLog::Write("ERROR : SCSClient::OpenConnection - gettimeofday failed",errno,1);
#endif
	  close(connection->connectionSocket);
	  connection->connectionSocket=-1;
	  unlink(socketData.sun_path);
	  delete connection;
	  connection=0;
	  
#ifdef SCSDEBUG
	  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit -1",2);
#endif  
	  stat=-1;
	  continue;
	  return -1;
	};
  

    
      if (((timeEnd.tv_sec-timeStart.tv_sec)*1000000+(timeEnd.tv_usec-timeStart.tv_usec))> inTimeoutUs)
	{
	  if ( stat != -1 )
	    {	    
	      stat=0;
	    };
	  break;
	}
      close(connection->connectionSocket);
      connection->connectionSocket=-1;
      unlink(socketData.sun_path);
      delete connection;
      
    };
  

  if ( stat <= 0 )
    {
#ifdef SCSDEBUG
      if ( stat == -1 )
	{
	  ostringstream tmpStr;
	  tmpStr<<"ERROR : SCSClient::OpenConnection - connect failed for socket '" << inSocketName<<"'";
	  SCSLog::Write(tmpStr.str(),tmpErr,1);
	}
      else if ( stat == -2 )
	{
	  SCSLog::Write("ERROR : SCSClient::OpenConnection - receive ACK from server failed",1);
	}
      
#endif
      close(connection->connectionSocket);
      unlink(socketData.sun_path);
      connection->connectionSocket=-1;
      delete connection;
      connection=0;

#ifdef SCSDEBUG
      SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit -1",2);
#endif  
      if ( stat < 0 )
	{
	  return -1;
	};

      return stat;
    };
  

  
#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName) exit 1",2);
#endif  

  return 1;
};



//
// Pre: Connection is open
//
//
// Post: Connection is closed. Returns -1 on failure, 1 on success.
//
//
// Desc: Connection to a sever is closed
//
//
//



int SCSClient::CloseConnection()
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::CloseConnection() enter",2);
#endif  

  if ( connection )
    {
      delete connection;
      connection=0;
#ifdef SCSDEBUG
      SCSLog::Write("SCSClient::CloseConnection() exit 1",2);
#endif  
      return 1;
    };
  
#ifdef SCSDEBUG
  SCSLog::Write("ERROR : SCSClient::CloseConnection failed - connection not open",1);
#endif

#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::CloseConnection() exit -1",2);
#endif  
  return -1;
};


//
// Pre: Client is connected to a server
//
//
// Post: Data contained in inMessage is sent to connected server.
//       Returns 1 on success, 0 on timeout, -1 on failure.  If
//       timeout is negative sending will block untill data is sent.
//
//
// Desc:
//
//
//



int SCSClient::SendMessage(long inTimeoutUs,const SCSMessage& inMessage)
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::SendMessage(long inTimeoutUs,const SCSMessage& inMessage) enter",2);
#endif 
  
  if ( connection )
    {
      int stat= connection->SendMessage(inTimeoutUs,inMessage);
#ifdef SCSDEBUG
      if ( stat <= 0 )
	SCSLog::Write("ERROR : SCSClient::SendMessage failed",1);
#endif

#ifdef SCSDEBUG
      ostringstream tmpStr;
      tmpStr<<"SCSClient::SendMessage(long inTimeoutUs,const SCSMessage& inMessage) exit "<<stat;
      SCSLog::Write(tmpStr.str(),2);
#endif 
  
      return stat;
      
    };

#ifdef SCSDEBUG
  SCSLog::Write("ERROR : SCSClient::SendMessage failed - connection not open",1);
  
#endif

#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::SendMessage(long inTimeoutUs,const SCSMessage& inMessage) exit -1",2);
#endif 
  
  return -1;

};


//
// Pre: Client is connected to a server
//
//
// Post: Data is received from server and put into outMessage. Returns
//       1 on success, 0 on timeout ,-1 on failure. If timeout is negative
//       reading will block untill message is received.
//
//
// Desc:
//
//
//



int SCSClient::ReceiveMessage(long inTimeoutUs,SCSMessage& outMessage)
{

#ifdef SCSDEBUG
  SCSLog::Write("SCSClient::ReceiveMessage(long inTimeoutUs,SCSMessage& outMessage) enter",2);
#endif 
  
  int stat=0;;
  if ( connection )
    {
      // receive message from connection

      stat= connection->ReceiveMessage(inTimeoutUs,outMessage);

      if ( stat>0)
	{

	  // if server closed connection, close our end too

	  if  ( outMessage.Type() == SCSMessage::PROTOCOL_CLOSE )
	    {
	      connection->Close();
	      delete connection;
	      connection=0;
	    };
	};
    };

#ifdef SCSDEBUG
      if ( stat <= 0 )
	SCSLog::Write("ERROR : SCSClient::ReceiveMessage failed",1);
#endif


#ifdef SCSDEBUG
      ostringstream tmpStr;
      tmpStr<<"SCSClient::ReceiveMessage(long inTimeoutUs,SCSMessage& outMessage) exit "<<stat;
      SCSLog::Write(tmpStr.str(),2);
#endif 

  return stat;

};
  
  

int SCSClient::Test()
{
  SCSClient tmpClient;
  int stat =1;
  cout <<" Testing SCSClient::OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId)...";
  
  if ( tmpClient.OpenConnection(0,"localhost1",1000000) >0 )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";
  
  cout<<endl;
  


  cout <<" Testing SCSClient::OpenConnection(long inTimeoutUs,const string& inSocketName)...";
  
  if ( tmpClient.OpenConnection(0,"/tmp/Some_nonexistant_socket") >0 )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";
  
  cout<<endl;


  cout <<" Testing SCSClient::CloseConnection()...";
  
  if ( tmpClient.CloseConnection() >0 )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";
  
  cout<<endl;


  cout <<" Testing SCSClient::SendMessage(long inTimeoutUs,const SCSMessage& inMessage)...";
  
  SCSMessage tmpMessage;
  tmpMessage.Set("1234",4);
  if ( tmpClient.SendMessage(0,tmpMessage) >0 )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";
  
  cout<<endl;


  cout <<" Testing SCSClient::ReceiveMessage(long inTimeoutUs,SCSMessage& outMessage)...";
  

  if ( tmpClient.ReceiveMessage(0,tmpMessage) >0 )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";

  cout<<endl;

  cout <<" Testing SCSClient::IsConnectionOpen()...";
  

  if ( tmpClient.IsConnectionOpen() )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";

  
  cout<<endl;

  return stat;
};


