//---------------------------------------------------------------------------
//
//  FILE NAME:          SCSClient.hh
//
//  DEVELOPMENT:        CXCDS, User Infrastructure
//
//  DESCRIPTION:
//
//
//---------------------------------------------------------------------------
// Copyright (c) 2006, Smithsonian Astrophysical Observatory
//---------------------------------------------------------------------------
 
#ifndef _SCSCLIENT_HH_
#define _SCSCLIENT_HH_


#include <string>
#include <sys/socket.h>

using namespace std;

class SCSConnection;
class SCSMessage;

class SCSClient
{

public:
  SCSClient();
  virtual ~SCSClient();
  
  //
  // Pre: Connection is not open. 
  //
  //
  // Post: Connection using INET socket is opened to 
  //       a server running on inHostName host
  //       and listenning on port inPortId.  Returns 1 on success,
  //       0 on timeout, -1 on error.  
  //
  //
  // Desc: An attempt is made to establish INET socket connection to
  //       specified port on specified host within inTimeoutUs useconds.
  //
  //
  //

  int OpenConnection(long inTimeoutUs,const string& inHostName,int inPortId);
 

  //
  // Pre: Connection is not open.
  //
  //
  // Post: Connection using UNIX(local) socket inSocketName is opened to 
  //       a server running on local host 
  //       and listenning on socket inSocketName.  Returns 1 on success,
  //       0 on timeout after successfull connection to server socket but with
  //          no response from the server
  //      -1 on error or timeout conncting to server socket.  
  //
  //
  // Desc: An attempt is made to establish UNIX socket connection to
  //       specified socket on local host.
  //
  //



  int OpenConnection(long inTimeoutUs,const string& inSocketName);


  //
  // Pre: Connection is open
  //
  //
  // Post: Connection is closed. Returns -1 on failure, 1 on success.
  //
  //
  // Desc: Connection to a sever is closed
  //
  //
  //

  
  int CloseConnection();


  //
  // Pre: Client is connected to a server
  //
  //
  // Post: Data contained in inMessage is sent to connected server.
  //       Returns 1 on success, 0 on timeout, -1 on failure.  If
  //       timeout is negative sending will block untill data is sent.
  //
  //
  // Desc:
  //
  //
  //


  virtual int SendMessage(long inTimeoutUs,const SCSMessage& inMessage);


  //
  // Pre: Client is connected to a server
  //
  //
  // Post: Data is received from server and put into outMessage. Returns
  //       1 on success, 0 on timeout ,-1 on failure. If timeout is negative
  //       reading will block untill message is received.
  //
  //
  // Desc:
  //
  //
  //


  virtual int ReceiveMessage(long inTimeoutUs,SCSMessage& outMessage);

  //
  // Pre:
  //
  //
  // Post: Returns true if connection is open, false otherwise
  //
  //
  // Desc:
  //
  //
  //

  
  bool IsConnectionOpen(){return (connection!=0);};


  //
  // Pre:
  //
  //
  // Post: 
  //
  //
  // Desc:
  //
  //
  //

  
  static int Test();


protected:
private:

  // actual connection

  SCSConnection* connection;
  
};



#endif
