//---------------------------------------------------------------------------
//
//  FILE NAME:          SCSConnection.hh
//
//  DEVELOPMENT:        CXCDS, User Infrastructure
//
//  DESCRIPTION:
//
//
//---------------------------------------------------------------------------
// Copyright (c) 2006, Smithsonian Astrophysical Observatory
//---------------------------------------------------------------------------
 
#ifndef _SCSONNECTION_HH_
#define _SCSONNECTION_HH_



class SCSMessage;
class SCSConnection
{
  // Only these clases can instantiate SCSConnection

  friend class SCSServer;
  friend class SCSClient;
public:

  //
  // Pre: Connection is initialized( it contains an open socket)
  //
  //
  // Post: data in inMessage is send through a socket.
  //       Returns 1 on success , 0 on timeout, -1 error.
  //       If timeout is negative write will block untill message 
  //       is sent. 
  //
  //
  // Desc:
  //

  int SendMessage(long inTimeoutUs,const SCSMessage& inMessage);
 
  //
  // Pre: Connection is initialized( it contains an open socket)
  //
  //
  //
  // Post: data is read from the socket and put in outMessage 
  //       Returns 1 on success , 0 on timeout, -1 error.
  //       If timeout is negative write will block untill message 
  //       is received. 
  //
  //
  // Desc:
  //

  int ReceiveMessage(long inTimeoutUs,SCSMessage& outMessage);
 
  //
  // Pre: 
  //
  //
  // Post: Check if connection is read ready within specified timeout.
  //
  //
  // Desc:
  //

  int IsReadReady(long inTimeoutUs);

  //
  // Pre:
  //
  //
  // Post: Check if connection is read ready within specified timeout.
  //
  //
  // Desc:
  //

  int IsWriteReady(long inTimeoutUs);


  //
  // Pre:
  //
  //
  // Post: 
  //
  //
  // Desc: Unit test
  //

  static int Test();


protected:

private:
  // These are private so only friends can instantiate it
  SCSConnection(const SCSConnection& inConnection);
  SCSConnection(int inSocket);
  virtual ~SCSConnection();

  //
  // Pre: Socket is write ready
  //
  //
  // Post: Message is sent through write ready socket
  //       Returns 1 on success , -1 otherwise
  //
  // Desc:
  //

  int SendMessage(const SCSMessage& inMessage); 
  
  //
  // Pre: Socket is read ready
  //
  //
  // Post: Message is received through read ready socket
  //       Returns 1 on success , -1 otherwise
  //
  // Desc:
  //


  int ReceiveMessage(SCSMessage& outMessage);

  //
  // Pre: 
  //
  //
  // Post: Returns socket descriptor
  //       
  //
  // Desc:
  //

  int GetSocketId(){return connectionSocket;};

  //
  // Pre: socket is read ready
  //
  //
  // Post: Sets flag indicating connection is in read ready state
  //       
  //
  // Desc:
  //


  void SetReadReady(bool inFlag){readReadyFlag=inFlag;};

  
  //
  // Pre: socket is write ready
  //
  //
  // Post: Sets flag indicating connection is in write ready state
  //       
  //
  // Desc:
  //

  void SetWriteReady(bool inFlag){writeReadyFlag=inFlag;};

  //
  // Pre: Socket is open
  //
  //
  // Post: Sends close message to the other end of connection and 
  //       closes local socket.
  //       
  //
  // Desc:
  //


  int CloseProtocol();



  //
  // Pre: 
  //
  //
  // Post: Closes local socket.
  //       
  //
  // Desc:
  //

  void Close();



  
  //
  // Pre: 
  //
  //
  // Post: Return 1 if host is little endian,
  //       0 otherwise
  //
  // Desc:
  //

  int Endian();


  //
  // Pre: 
  //
  //
  // Post: 
  //
  //
  // Desc:
  //

  int PackMsgSize(char* outBuffer,int inBufferSize,long inMessageSize);



  //
  // Pre: 
  //
  //
  // Post: 
  //
  //
  // Desc:
  //

  int UnpackMsgSize(char* inBuffer,int inBufferSize,long* outMessageSize);


  // Actual connection socket
  int connectionSocket;

  // Socket read ready flag
  bool readReadyFlag;

  
  // Socket write ready flag
  bool writeReadyFlag;

};

#endif
