//---------------------------------------------------------------------------
//
//  FILE NAME:          SCSMessage.cc
//
//  DEVELOPMENT:        CXCDS, User Infrastructure
//
//  DESCRIPTION:
//
//
//---------------------------------------------------------------------------
// Copyright (c) 2006, Smithsonian Astrophysical Observatory
//---------------------------------------------------------------------------


#include <string.h> 
#include <iostream>
#include <sstream>

#include "SCSMessage.hh"
#include "SCSLog.hh"

using namespace std;

const int SCSMessage::SCS_MESSAGE_HEADER_SIZE=4;
const long SCSMessage::SCS_MAX_MESSAGE_SIZE=0x7FFFFFFF;

const char SCSMessage::PROTOCOL_DATA='d';
const char SCSMessage::PROTOCOL_ACK='a';
const char SCSMessage::PROTOCOL_CLOSE='c';

SCSMessage::SCSMessage()
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::SCSMessage() enter",2);
#endif 

  data=new char[1];
  data[0]=SCSMessage::PROTOCOL_DATA;
  size=0;

#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::SCSMessage() exit",2);
#endif 
};

SCSMessage::~SCSMessage()
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::~SCSMessage() enter",2);
#endif  

  if ( data)
    delete[] data;
  size=0;

#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::~SCSMessage() exit",2);
#endif  

};


//
// Pre: 
//
//
// Post: message is filled with inSize chars of data from inData buffer.
//       
//
// Desc:
//
//

  

  
int  SCSMessage::Set(const char* inData,unsigned long inSize)
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Set(const char* inData,unsigned long inSize) enter",2);
#endif 

  if ( inData && inSize )
    {
      if ( !Resize(inSize) )
	{
#ifdef SCSDEBUG
	  SCSLog::Write("ERROR : SCSMessage::Set failed",1);
#endif

#ifdef SCSDEBUG
	  SCSLog::Write("SCSMessage::Set(const char* inData,unsigned long inSize) exit 0",2);
#endif 
	  return 0;
	};
      memcpy(data+1,inData,size*sizeof(char));
      data[0]=SCSMessage::PROTOCOL_DATA;
    }
  else
    Resize(0);


#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Set(const char* inData,unsigned long inSize) exit 1",2);
#endif 
  return 1 ;
};


//
// Pre: 
//
//
// Post: inSize chars of data from in inData are appended to the end
//       of message.  
//
//
// Desc:
//
//
//

  
int  SCSMessage::Append(const char* inData,unsigned long inSize)
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Append(const char* inData,unsigned long inSize) enter",2);
#endif 

  if ( inData && inSize)
    {
      char* tmpData= new char[size+inSize+1];
      
      memcpy(tmpData+1,data+1,size*sizeof(char));
      
      memcpy(tmpData+1+size,inData,inSize*sizeof(char));
      tmpData[0]=SCSMessage::PROTOCOL_DATA;
      delete[] data;
      data=tmpData;
      size+=inSize;

#ifdef SCSDEBUG
      SCSLog::Write("SCSMessage::Append(const char* inData,unsigned long inSize) exit 1",2);
#endif 
      return 1;
    };

#ifdef SCSDEBUG
  SCSLog::Write("ERROR : SCSMessage::Append failed",1);
#endif

#ifdef SCSDEBUG
      SCSLog::Write("SCSMessage::Append(const char* inData,unsigned long inSize) exit 0",2);
#endif 
  return 0;
};


//
// Pre:
//
//
// Post:  Message is filled with inSize chars from inData and message
//        type is set to inType.  Memory is reallocated if necessary.
//
//
// Desc:
// 

int  SCSMessage::Set(const char* inData,unsigned long inSize,char inType)
{
#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Set(const char* inData,unsigned long inSize,char inType) enter",2);
#endif 
  
  int stat;
  
  stat = Set(inData,inSize);
  if ( stat)
    data[0]=inType;

#ifdef SCSDEBUG
  ostringstream tmpStr;
  tmpStr<<"SCSMessage::Set(const char* inData,unsigned long inSize,char inType) exit "<<stat;
  SCSLog::Write(tmpStr.str(),2);
#endif 
  

  return stat;
};


//
// Pre:
//
//
// Post:  Message is cleared. Internal memory is released.
//
//
// Desc:
//
//
//
  
  

void  SCSMessage::Clear()
{

#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Clear() enter",2);
#endif 

  Resize(0);

#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Clear() exit",2);
#endif 
  
};


//
// Pre:
//
//
// Post:  Message is resized to inSize chars. Internal memory is reallocated.
//
//
// Desc:
//
//
//
  

int SCSMessage::Resize(unsigned long inSize)
{

#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Resize(unsigned long inSize) enter",2);
#endif 

  if ( data)
    {
      delete[] data;
      size=0;
      data=new char[inSize+1];
      if(!data)
	{
#ifdef SCSDEBUG
	  SCSLog::Write("ERROR : SCSMessage::Resize failed",1);
#endif

#ifdef SCSDEBUG
	  SCSLog::Write("SCSMessage::Resize(unsigned long inSize) exit 0",2);
#endif 
	  return 0;
	};
    };
  size = inSize;
  data[0]=SCSMessage::PROTOCOL_DATA;

#ifdef SCSDEBUG
  SCSLog::Write("SCSMessage::Resize(unsigned long inSize) exit 1",2);
#endif 

  return 1; 
};

int SCSMessage::Test()
{

  int stat =1;
  SCSMessage tmpMessage;
  
  cout <<" Testing SCSMessage::Size()...";
  if ( tmpMessage.Size() !=0 )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";
  
  cout<<endl;
  

  cout <<" Testing SCSMessage::Type()...";
  if ( tmpMessage.Type() !=SCSMessage::PROTOCOL_DATA )
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";
  
  cout<<endl;
  
  tmpMessage.Set("123456789",10);

  cout <<" Testing SCSMessage::Data()...";
  if (memcmp(tmpMessage.Data(),"123456789",10)!=0 )
     {
       stat = 0;
       cout<<"FAILED";
     }
  else
    cout<<"OK";
  
  cout<<endl;


  cout <<" Testing SCSMessage::Set(const char* inData,unsigned long inSize)...";
  tmpMessage.Set("123456789",10);
  if ( ( tmpMessage.Type() !=SCSMessage::PROTOCOL_DATA )
       || (tmpMessage.Size()!=10)
       || (memcmp(tmpMessage.Data(),"123456789",10)!=0 ))
    {
      stat = 0;
      cout<<"FAILED";
    }
  else
    cout<<"OK";
  
  cout<<endl;

  cout <<" Testing SCSMessage::Append(const char* inData,unsigned long inSize)...";
  tmpMessage.Set("123456789",9);
  tmpMessage.Append("abcd",5);
  if ( ( tmpMessage.Type() !=SCSMessage::PROTOCOL_DATA )
       || (tmpMessage.Size()!=14)
       || (memcmp(tmpMessage.Data(),"123456789abcd",14)!=0 ))
     {
       stat = 0;
       cout<<"FAILED";
     }
  else
    cout<<"OK";
  
  cout<<endl;
  
  cout <<" Testing SCSMessage::Clear()...";
  tmpMessage.Clear();
  if ( ( tmpMessage.Type() !=SCSMessage::PROTOCOL_DATA )
       || (tmpMessage.Size()!=0))
     {
       stat = 0;
       cout<<"FAILED";
     }
  else
    cout<<"OK";
  
  cout<<endl;
  
  cout <<" Testing SCSMessage::Resize(unsigned long inSize)...";
  tmpMessage.Resize(100);
  if( ( tmpMessage.Type() !=SCSMessage::PROTOCOL_DATA )
      || (tmpMessage.Size()!=100))
     {
       cout <<tmpMessage.Size()<<endl;
       stat = 0;
       cout<<"FAILED";
     }
  else
    cout<<"OK";
  
   cout<<endl;

  return stat; 
};
