//---------------------------------------------------------------------------
//
//  FILE NAME:          SCSMessage.hh
//
//  DEVELOPMENT:        CXCDS, User Infrastructure
//
//  DESCRIPTION:
//
//
//---------------------------------------------------------------------------
// Copyright (c) 2006, Smithsonian Astrophysical Observatory
//---------------------------------------------------------------------------
 
#ifndef _SCSMESSAGE_HH_
#define _SCSMESSAGE_HH_



class SCSMessage
{
  friend class SCSServer;
  friend class SCSClient;
  friend class SCSConnection;
public:
  SCSMessage();
  virtual ~SCSMessage();
  
  //
  // Pre: 
  //
  //
  // Post: Returns pointer to data stored in the message.
  //
  //
  // Desc:
  //
  //
  //

  const char* Data(){if (data) return data+1; return 0; };

  //
  // Pre:
  //
  //
  // Post: returns size of data in the message
  //
  //
  // Desc:
  //
  //
  //
  

  long Size(){  return size;};

  //
  // Pre:
  //
  //
  // Post: returns message type one of {SCSMessage::PROTOCOL_DATA,
  //       SCSMessage::PROTOCOL_ACK,SCSMessage::PROTOCOL_CLOSE}
  //
  //
  // Desc:
  //
  //
  //
  

  char Type() { if (data) return data[0]; return SCSMessage::PROTOCOL_DATA;};

  //
  // Pre: 
  //
  //
  // Post: message is filled with inSize chars of data from inData buffer.
  //       
  //
  // Desc:
  //
  //
  //
  
  int  Set(const char* inData,unsigned long inSize);

  //
  // Pre: 
  //
  //
  // Post: inSize chars of data from in inData are appended to the end
  //       of message.  
  //
  //
  // Desc:
  //
  //
  //
  


  int  Append(const char* inData,unsigned long inSize);

  //
  // Pre:
  //
  //
  // Post:  Message is cleared. Internal memory is released.
  //
  //
  // Desc:
  //
  //
  //
  
  

  void Clear();

  // Constants used for internal typing of messages sent
  static const char PROTOCOL_DATA;
  static const char PROTOCOL_ACK;
  static const char PROTOCOL_CLOSE;



  //
  // Pre:
  //
  //
  // Post:  
  //
  //
  // Desc:
  //
  //
  //
  
  static int Test();

  
private:
  //
  // Pre:
  //
  //
  // Post:  Message is resized to inSize chars. Internal memory is reallocated.
  //
  //
  // Desc:
  //
  //
  //
  int Resize(unsigned long inSize);

  
  //
  // Pre:
  //
  //
  // Post:  Message is filled with inSize chars from inData and message
  //        type is set to inType.  Memory is reallocated if necessary.
  //
  //
  // Desc:
  //
  //
  //
  int  Set(const char* inData,unsigned long inSize,char inType);

  // Actual message data
  char* data;

  // size of data
  long  size;

  // constatnt used to describe size of number describing message size.

  static const int SCS_MESSAGE_HEADER_SIZE;
  static const long SCS_MAX_MESSAGE_SIZE;
};



#endif
