#include <iostream>
#include <sys/types.h>
#include <sstream>
#include <unistd.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <map>

#include "SCSClient.hh"
#include "SCSServer.hh"
#include "SCSMessage.hh"
#include "SCSConnection.hh"

void usage(char* inAppName);


void PrintInfo(const string& inTestId, 
	       const string& inTestDesc,
	       const string& inExpectedResult);

int timerOn(struct timeval& inTimeStart);
long timerOff(const struct timeval& inTimeStart);

//
//Test SCSMessage
//
int TestSCSMessage(int inTestIndx,int action); 

//
//Test SCSClient
//
int TestSCSClient(int inTestIndx,int action); 

//
//Test SCSServer
//
int TestSCSServer(int inTestIndx,int action); 

//
//Test SCSConnection
//
int TestSCSConnection(int inTestIndx,int action); 

//
//Test 1 Client 1 Server using local sockets
//
int Test1Client1ServerLocalSockets(int inTestIndx,int action); 

//
//Test 1 Client 1 Server using Inet sockets
//
int Test1Client1ServerInetSockets(int inTestIndx,int action); 



//
//Test N Client 1 Server using local sockets
//
int TestNClients1ServerLocalSockets(int inTestIndx,int action); 


//
//Test N Client 1 Server using Inet sockets
//

int TestNClients1ServerInetSockets(int inTestIndx,int action); 





//
//Test Client Timeout using local sockets
//
int TestClientTimeoutLocalSockets(int inTestIndx,int action); 


//
//Test Client Timeout using Inet sockets
//

int TestClientTimeoutInetSockets(int inTestIndx,int action); 







int (*testSet[])(int ,int)=
{
  TestSCSMessage,
  TestSCSClient,
  TestSCSServer,
  TestSCSConnection,
  Test1Client1ServerLocalSockets,
  Test1Client1ServerInetSockets,
  TestNClients1ServerLocalSockets,
  TestNClients1ServerInetSockets,
  TestClientTimeoutLocalSockets,
  TestClientTimeoutInetSockets,
  0
};


int main(int argc , char* argv[])
{
  string tmpStr;
  int testCase;
  int totalSuccess=0;
  int totalTests;
  int totalTests1;


  if ( argc > 1 )
    {
      for(int indx=1;indx<argc;indx++)
	{
	  tmpStr=argv[indx];
	  if ( tmpStr=="-h")
	    {
	      usage(argv[0]);
	      break;
	    }
	  else if ( tmpStr=="-l")
	    {
	      totalTests=0;
	      while(testSet[totalTests] )
		{
		  testSet[totalTests](totalTests+1,1);
		  totalTests++;
		};
	      break;
	    }
	  else 
	    {
	      totalTests=0;
	      while(testSet[totalTests] )
		  totalTests++;
	      totalTests1=0;
	      for( int indx1=indx;indx1<argc;indx1++)
		{
		  testCase = atoi(argv[indx1]);
		  if (( testCase > 0) && (testCase <=totalTests))
		    {
		      totalSuccess +=testSet[testCase-1](testCase,0);
		      totalTests1++;
		    }
		  else
		    {
		      usage(argv[0]);
		      return -1;
		    }
		};
	      cout <<"Summary : "<<totalSuccess <<" out of " <<totalTests1<<" passed"<<endl;
	      break;
	      
	    };
	};
    }
  else
    {
      totalTests=0;
      while(testSet[totalTests])
	{
	  totalSuccess +=testSet[totalTests](totalTests+1,0);
	  totalTests++;
	};

      cout <<"Summary : "<<totalSuccess <<" out of " <<totalTests<<" passed"<<endl;
    }

    return 0;
}

void usage(char* inAppName)
{
  cout<<"usage: "<<inAppName<<" [option]"<<endl;
  cout<<"       -h - show this info"<<endl;
  cout<<"       -l - list all tests"<<endl;
  cout<<"       num [num1...] - run tests number 'num num1...'"<<endl<<endl;
  cout<<"Available tests:"<<endl;
  int indx1=0;
  while(testSet[indx1])
    {
      testSet[indx1](indx1+1,1);
      indx1++;
    };
};



void PrintInfo(const string& inTestId, 
	       const string& inTestDesc,
	       const string& inExpectedResult)
{
  cout<<"Test "<<inTestId<<" "<<inTestDesc<<" "<<inExpectedResult<<endl;


};

int timerOn(struct timeval& inTimeStart)
{
  if ( gettimeofday(&inTimeStart,0) )
    {
      return -1;
    };

  return 1;
};

long timerOff(const struct timeval& inTimeStart)
{
  struct timeval timeEnd;
  if (  gettimeofday(&timeEnd,0) )
    {
      return -1;
    };
 
  return (timeEnd.tv_sec-inTimeStart.tv_sec)*1000000+timeEnd.tv_usec-inTimeStart.tv_usec;
};


int TestSCSMessage(int inTestIndx,int action)
{
  ostringstream tmpOStr;
  tmpOStr<<inTestIndx;
  int stat=0;

  PrintInfo(tmpOStr.str(),"Unit Testing For SCSMessage","");

  if ( action != 0 )
    return 1;

  return SCSMessage::Test();

};

int TestSCSClient(int inTestIndx,int action)
{
  ostringstream tmpOStr;
  tmpOStr<<inTestIndx;
  int stat=0;

  PrintInfo(tmpOStr.str(),"Unit Testing For SCSClient","");

  if ( action != 0 )
    return 1;

  return SCSClient::Test();

};

int TestSCSServer(int inTestIndx,int action)
{
  ostringstream tmpOStr;
  tmpOStr<<inTestIndx;
  int stat=0;

  PrintInfo(tmpOStr.str(),"Unit Testing For SCSServer","");

  if ( action != 0 )
    return 1;

  return SCSServer::Test();

};

int TestSCSConnection(int inTestIndx,int action)
{
  ostringstream tmpOStr;
  tmpOStr<<inTestIndx;
  int stat=0;

  PrintInfo(tmpOStr.str(),"Unit Testing For SCSConnection","");

  if ( action != 0 )
    return 1;

  return SCSConnection::Test();

};




int Test1Client1ServerLocalSockets(int inTestIndx,int action)
{

  ostringstream tmpOStr;
  int res=0;
  struct timeval timeStart;
  long duration;
  int pid;
  int procStat;
  ostringstream  socketPath;
  struct stat  fileStat;
  int messageCount=10000;

  tmpOStr<<inTestIndx;
  PrintInfo(tmpOStr.str(),"Regression Testing 1 client 1 server on local sockets","");

  if ( action != 0 )
    return 1;


  socketPath<<"/tmp/test1."<<getpid();
  
  
  pid=fork();
  
  if ( pid<0)
    {
      cout<<"ERROR: fork failed"<<endl;
      return res;
    };
  
  
  if (! pid)
    {
      SCSServer testServer;
      SCSMessage tmpMessage;
      string tmpStr;
      long clientId;
      testServer.Start(10,socketPath.str());

      
      while((clientId=testServer.ReceiveMessage(100000000,tmpMessage))>0)
	{

	  tmpStr=tmpMessage.Data() ;

	  if (( tmpStr == "quit") || (tmpMessage.Type()== SCSMessage::PROTOCOL_CLOSE))
	    {
	      testServer.Shutdown();
	      exit(0);
	    };

	  tmpStr+="ACK";
	  
	  tmpMessage.Set(tmpStr.c_str(),tmpStr.size()+1);
	  
	  if ( testServer.SendMessage(100000000,clientId,tmpMessage)<=0)
	    {
	      testServer.Shutdown();
	      exit(-1);
	    };
	
	};

      testServer.Shutdown();
      exit(res);
    }
  else
    {
      

      SCSClient testClient;
      SCSMessage tmpMessage;
      string tmpStr;
      int okCount=0;


      cout <<" Testing open connection to server...";

      if ( testClient.OpenConnection(100000000,socketPath.str())<=0)
	{
	  cout<<"FAILED"<<endl;
	  res=0;
	}
      else
	{
	  cout<<"OK"<<endl;
	  cout <<" Testing send/receive  "<<messageCount<<" messages...";
	  for(int indx=0;indx<messageCount;indx++)
	    {
	      ostringstream tmpOStr;
	      
	      tmpOStr<<"Test1-"<<indx;
	      tmpMessage.Set(tmpOStr.str().c_str(),tmpOStr.str().size()+1);
	      if ( testClient.SendMessage(100000000,tmpMessage) <= 0)
		break;
	      
	      if ( testClient.ReceiveMessage(100000000,tmpMessage) <=0)
		break;
	      
	      tmpStr=tmpMessage.Data();
	      
	      if ( tmpStr == (tmpOStr.str()+"ACK"))
		okCount++;
	    };

	  if ( messageCount == okCount )
	    cout <<"OK"<<endl;
	  else
	    cout <<"FAILED"<<endl;


	  cout <<" Testing close connection...";
	  if (testClient.CloseConnection()>0)
	    res=1;

	  if ( res ) 
	    cout <<"OK"<<endl;
	  else
	    cout <<"FAILED"<<endl;

	  cout <<" Testing server socket cleanup...";

	  sleep(1);
	  if ( stat( socketPath.str().c_str(),&fileStat) == -1 )
	    cout <<"OK"<<endl;
	  else
	    {
	      res=0;
	      cout <<"FAILED"<<endl;
	    };

	};
    };

  wait(&procStat);

  return res;



};


int Test1Client1ServerInetSockets(int inTestIndx,int action)
{

  ostringstream tmpOStr;
  int res=0;
  struct timeval timeStart;
  long duration;
  int pid;
  int procStat;
  ostringstream  socketPath;
  struct stat  fileStat;
  int messageCount=100;

  tmpOStr<<inTestIndx;
  PrintInfo(tmpOStr.str(),"Regression Testing 1 client 1 server on Inet sockets","");

  if ( action != 0 )
    return 1;


  
  pid=fork();
  
  if ( pid<0)
    {
      cout<<"ERROR: fork failed"<<endl;
      return res;
    };
  
  
  if (! pid)
    {
      SCSServer testServer;
      SCSMessage tmpMessage;
      string tmpStr;
      testServer.Start(10,10000);
      long clientId;
      
      
      while((clientId=testServer.ReceiveMessage(100000000,tmpMessage))>0)
	{
	  if ( clientId == 0 )
	    {
	      testServer.Shutdown();
	      exit(-1);
	    };

	  tmpStr=tmpMessage.Data() ;

	  if (( tmpStr == "quit") || (tmpMessage.Type()== SCSMessage::PROTOCOL_CLOSE))
	    {
	      testServer.Shutdown();
	      exit(0);
	    };

	  tmpStr+="ACK";
	  
	  tmpMessage.Set(tmpStr.c_str(),tmpStr.size()+1);
	  
	  if ( testServer.SendMessage(100000000,clientId,tmpMessage)<=0)
	    {
	      testServer.Shutdown();
	      exit(-1);
	    };
	
	};
      testServer.Shutdown();
      exit(res);
    }
  else
    {
      

      SCSClient testClient;
      SCSMessage tmpMessage;
      string tmpStr;
      int okCount=0;

      cout <<" Testing open connection to server...";
      
      if ( testClient.OpenConnection(100000000,"localhost",10000)<=0)
	{
	  cout<<"FAILED"<<endl;
	  res=0;
	}
      else
	{
	  cout<<"OK"<<endl;

	  cout <<" Testing send/receive "<<messageCount<<" messages...";

	  for(int indx=0;indx<messageCount;indx++)
	    {
	      ostringstream tmpOStr;
	      
	      tmpOStr<<"Test1-"<<indx;
	      tmpMessage.Set(tmpOStr.str().c_str(),tmpOStr.str().size()+1);
	      if ( testClient.SendMessage(100000000,tmpMessage) <= 0)
		break;
	      
	      if ( testClient.ReceiveMessage(100000000,tmpMessage) <=0)
		break;
	      
	      tmpStr=tmpMessage.Data();
	      
	      if ( tmpStr == (tmpOStr.str()+"ACK"))
		okCount++;
	    };

	  if ( messageCount == okCount )
	    cout <<"OK"<<endl;
	  else
	    cout <<"FAILED"<<endl;


	  cout <<" Testing close connection...";
	  if (testClient.CloseConnection()>0)
	    res=1;

	  if ( res ) 
	    cout <<"OK"<<endl;
	  else
	    cout <<"FAILED"<<endl;

	};
    };

  wait(&procStat);

  return res;



};













int TestNClients1ServerLocalSockets(int inTestIndx,int action)
{ 

  ostringstream tmpOStr,tmpOStr1;
  int res=0;
    int pid=1;
  int procStat;
  int messageCount=10000;
  int clientCount=10;
  ostringstream  socketPath;

  

  tmpOStr<<inTestIndx;
  tmpOStr1<<"Regression Testing "<<clientCount<<" clients 1 server on local sockets"; 
  PrintInfo(tmpOStr.str(),tmpOStr1.str(),"");

  if ( action != 0 )
    return 1;



  socketPath<<"/tmp/test1."<<getpid();
  

  for(int cnt=0;cnt<clientCount;cnt++)
    {
      pid=fork();
      if ( pid< 0 )
	{
	  cout <<"ERROR: fork failed"<<endl;
	  return res;
	};
      if (! pid)
	{
	  break;
	};
    };


  if (pid)
    {


      
      SCSServer testServer;
      SCSMessage tmpMessage;
      string tmpStr;
      long clientId;
      int quitCount=0;
      int clCount=0;
      map<int,int> clMessages;


      res=1;
      cout <<" Testing server startup...";
      if ( testServer.Start(10,socketPath.str())>0)
	cout<<"OK"<<endl;
      else
	cout<<"FAILED"<<endl;
      
      
      cout <<" Testing send/receive "<<messageCount<<" messages per client with "<<clientCount <<" clients...";

      while((clientId=testServer.ReceiveMessage(10000000,tmpMessage))>0)
	{
	  if ( clientId == 0 )
	    {
	      break;
	    };
	  
	  if ( tmpMessage.Type() == SCSMessage::PROTOCOL_CLOSE )
	    {
	      quitCount++;
	      if ( quitCount == clientCount )
		break;
	      else
		continue;
	    };

	  tmpStr=tmpMessage.Data() ;
	  
	  if ( tmpStr == "ACK")
	    {
	      clMessages[clientId]+=1;
	    }
	  else
	    {
	      tmpStr+="ACK";
	      
	      tmpMessage.Set(tmpStr.c_str(),tmpStr.size()+1);
	      
	      if ( testServer.SendMessage(100000000,clientId,tmpMessage)>0)
		clMessages[clientId]+=1;
	    };
	};

            
      clCount=0;
      for(map<int,int>::iterator tmpPtr=clMessages.begin();tmpPtr!=clMessages.end();tmpPtr++)
	{
	  if(tmpPtr->second !=(messageCount+1))
	    res=0;
	  clCount++;

	};

      if ( clCount != clientCount )
	res=0;

      
      if ( res )
	cout <<"OK"<<endl;
      else
	cout <<"FAILED"<<endl;
      

      cout <<" Testing server shutdown...";      
      if ( testServer.Shutdown() >0)
	cout <<"OK"<<endl;
      else
	{
	  res=0;
	  cout <<"FAILED"<<endl;
	};

      for(int indx=0;indx<clientCount;indx++)
	wait(&procStat);
      

    }
  else
    {
      SCSClient testClient;
      SCSMessage tmpMessage;
      string tmpStr;
      int okCount=0;
      
      if ( testClient.OpenConnection(100000000,socketPath.str())<=0)
	res = 0;
      else
	{
	  for(int indx=0;indx<messageCount;indx++)
	    {
	      ostringstream tmpOStr;
	      
	      tmpOStr<<"Test1-"<<indx<<"-"<<indx;
	      tmpMessage.Set(tmpOStr.str().c_str(),tmpOStr.str().size()+1);
	      
	      
	      if ( testClient.SendMessage(100000000,tmpMessage) <= 0)
		{
		  break;
		};
	      
	      if ( testClient.ReceiveMessage(100000000,tmpMessage) <=0)
		{
		  break;
		};
	      tmpStr=tmpMessage.Data();
	      
	      if ( tmpStr == (tmpOStr.str()+"ACK"))
		okCount++;
	    };
	  
	  
	  tmpMessage.Set("ACK",4);
	  if ( testClient.SendMessage(1000000000,tmpMessage) > 0)
	    res=1;
	  if (( messageCount != okCount ) || (!res))
	    res= 0;
	};
      exit(res);
    };
  

  
  
  
  
  return res;
  
};


int TestNClients1ServerInetSockets(int inTestIndx,int action)
{ 

  ostringstream tmpOStr,tmpOStr1;
  int res=0;
    int pid=1;
  int procStat;
  int messageCount=100;
  int clientCount=10;

  

  tmpOStr<<inTestIndx;
  tmpOStr1<<"Regression Testing "<<clientCount<<" clients 1 server on Inet sockets"; 
  PrintInfo(tmpOStr.str(),tmpOStr1.str(),"");


  if ( action != 0 )
    return 1;




  for(int cnt=0;cnt<clientCount;cnt++)
    {
      pid=fork();
      if ( pid< 0 )
	{
	  cout <<"ERROR: fork failed"<<endl;
	  return res;
	};
      if (! pid)
	{
	  break;
	};
    };


  if (pid)
    {


      
      SCSServer testServer;
      SCSMessage tmpMessage;
      string tmpStr;
      long clientId;
      int quitCount=0;
      int clCount=0;
      map<int,int> clMessages;


      res=1;
      cout <<" Testing server startup...";
      if ( testServer.Start(10,10000)>0)
	cout<<"OK"<<endl;
      else
	cout<<"FAILED"<<endl;
      
      
      cout <<" Testing send/receive "<<messageCount<<" messages per client with "<<clientCount <<" clients...";

      while((clientId=testServer.ReceiveMessage(10000000,tmpMessage))>0)
	{
	  if ( clientId == 0 )
	    {
	      break;
	    };
	  
	  if ( tmpMessage.Type() == SCSMessage::PROTOCOL_CLOSE )
	    {
	      quitCount++;
	      if ( quitCount == clientCount )
		break;
	      else
		continue;
	    };

	  tmpStr=tmpMessage.Data() ;
	  
	  if ( tmpStr == "ACK")
	    {
	      clMessages[clientId]+=1;
	    }
	  else
	    {
	      tmpStr+="ACK";
	      
	      tmpMessage.Set(tmpStr.c_str(),tmpStr.size()+1);
	      
	      if ( testServer.SendMessage(100000000,clientId,tmpMessage)>0)
		clMessages[clientId]+=1;
	    };
	};

            
      clCount=0;
      for(map<int,int>::iterator tmpPtr=clMessages.begin();tmpPtr!=clMessages.end();tmpPtr++)
	{
	  if(tmpPtr->second !=(messageCount+1))
	    res=0;
	  clCount++;

	};

      if ( clCount != clientCount )
	res=0;

      
      if ( res )
	cout <<"OK"<<endl;
      else
	cout <<"FAILED"<<endl;
      

      cout <<" Testing server shutdown...";      
      if ( testServer.Shutdown() >0)
	cout <<"OK"<<endl;
      else
	{
	  res=0;
	  cout <<"FAILED"<<endl;
	};


      for(int indx=0;indx<clientCount;indx++)
	wait(&procStat);


      return(res);
    }
  else
    {

      
      SCSClient testClient;
      SCSMessage tmpMessage;
      string tmpStr;
      int okCount=0;
      
      if ( testClient.OpenConnection(100000000,"localhost",10000)<=0)
	res = 0;
      else
	{
	  for(int indx=0;indx<messageCount;indx++)
	    {
	      ostringstream tmpOStr;
	      
	      tmpOStr<<"Test1-"<<indx<<"-"<<indx;
	      tmpMessage.Set(tmpOStr.str().c_str(),tmpOStr.str().size()+1);
	      
	      
	      if ( testClient.SendMessage(100000000,tmpMessage) <= 0)
		{
		  break;
		};
	      
	      if ( testClient.ReceiveMessage(100000000,tmpMessage) <=0)
		{
		  break;
		};
	      tmpStr=tmpMessage.Data();
	      
	      if ( tmpStr == (tmpOStr.str()+"ACK"))
		okCount++;
	    };
	  
	  
	  tmpMessage.Set("ACK",4);
	  if ( testClient.SendMessage(1000000000,tmpMessage) > 0)
	    res=1;
	  if (( messageCount != okCount ) || (!res))
	    res= 0;

	  
	};

      testClient.CloseConnection();
      exit(res);
    };
  

  
  
  
  
  
  return res;
  
};






















int TestClientTimeoutLocalSockets(int inTestIndx,int action)
{

  ostringstream tmpOStr;
  int res=0;
  struct timeval timeStart;
  long duration;
  int pid;
  int procStat;
  ostringstream  socketPath;


  tmpOStr<<inTestIndx;
  PrintInfo(tmpOStr.str(),"Regression Testing Client Timeout on local sockets","");

  if ( action != 0 )
    return 1;


  socketPath<<"/tmp/test1."<<getpid();
  
  
  pid=fork();
  
  if ( pid<0)
    {
      cout<<"ERROR: fork failed"<<endl;
      return res;
    };
  
  
  if (! pid)
    {
      SCSServer testServer;
      testServer.Start(10,socketPath.str());
      sleep(5);
      testServer.Shutdown();
      exit(res);
    }
  else
    {
      

      SCSClient testClient;
      int okCount=0;


      cout <<" Testing open connection timeout ...";
      sleep(1);

      if ( testClient.OpenConnection(100,socketPath.str())!=0)
	{
	  cout<<"FAILED"<<endl;
	  res=0;
	}
      else
	{
	  cout<<"OK"<<endl;
	  res=1;
	};
    };

  wait(&procStat);

  return res;



};


int TestClientTimeoutInetSockets(int inTestIndx,int action)
{

  ostringstream tmpOStr;
  int res=0;
  struct timeval timeStart;
  long duration;
  int pid;
  int procStat;
  ostringstream  socketPath;


  tmpOStr<<inTestIndx;
  PrintInfo(tmpOStr.str(),"Regression Client Timeout on Inet sockets","");

  if ( action != 0 )
    return 1;


  
  pid=fork();
  
  if ( pid<0)
    {
      cout<<"ERROR: fork failed"<<endl;
      return res;
    };
  
  
  if (! pid)
    {
      SCSServer testServer;
      testServer.Start(10,10000);
      testServer.Shutdown();
      exit(res);
    }
  else
    {
      

      SCSClient testClient;

      string tmpStr;
      int okCount=0;

      cout <<" Testing open connection timeout...";
      sleep(1);
      int st;
      if ( (st=testClient.OpenConnection(100,"localhost",10000))!=0)
	{
	  cout<<"FAILED "<<st<<endl;
	  res=0;
	}
      else
	{
	  cout<<"OK"<<endl;
	  res=1;
	};
    };

  wait(&procStat);

  return res;



};


