// 
//  Copyright (C) 2006,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsServer.hh"

#include <iostream>
#include <sstream>
#include <stdio.h>
#include <string>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

using namespace std;

static struct sigaction sigint_act;
static void sigint_hdlr(int sig)
{
   sig = errno;                                 // save errno
   sigaction(SIGINT,&sigint_act,NULL);          // restore handler
   write(STDERR_FILENO,"\n",1);                 // POSIX signal-safe output
   errno = sig;                                 // restore errno
}


int main(int argc, char *argv[])
{
   bool load_prefs = false;
   ChipsServer *server = NULL;
   string       sname = ""; 
   char *arg;

   // add signal handler to catch <control><c>
   if (isatty(STDIN_FILENO)) 
   {
      sigemptyset(&sigint_act.sa_mask);
      sigint_act.sa_handler = sigint_hdlr;
      sigint_act.sa_flags   = 0;
      sigaction(SIGINT,&sigint_act,NULL);
   }
   argv++;
   argc--;

   // To run regression tests set default to starting server

   while (argc > 0)
   {
      arg = argv[0];
      if (!strcmp(arg, "-d"))  
      {
         // run application using defaults
         load_prefs = true;
      }
      else if ((!strcmp(arg, "-s")) && (argc>1))
      {
         // run as background server only- no application
         sname = argv[1];
         argc--; 
         argv++;
      }
      else
      {
         cerr << " ERROR: Unknown or unsupported commandline option- " << 
            argv[0] << endl;
         exit(1); 
      }
      argc--; 
      argv++;
   }

   if ((server = new ChipsServer(load_prefs)) == NULL)
   {
      cerr << "ERROR: UNable to initialize the chips server process." << endl;
      exit(1); 
   }
   else
   {
      if (sname.length() == 0)
      {
         ostringstream tmpStr1;
	 tmpStr1<< "ChipsServer_"<<getpid()<<"_1";
	 sname = tmpStr1.str();
      }
      server->create_server(sname);
   }
   exit(0); 
}

