#! /bin/sh

TESTID=chips-smoke001
export TESTID

#
# Replace with variable via configure
#
ASCDS_QUIET=1; export ASCDS_QUIET
ASCDS_OVERRIDE=1; export ASCDS_OVERRIDE
locale -a | grep -i "en_US" > /dev/null
if test $? = 0; then
   LANG="en_US.UTF-8"; export LANG
fi
. setup.sh

#
# Setup test env
#
TESTDIR=$ASCDS_INSTALL/test/smoke/data; export TESTDIR
if [ "$ASCDS_SMOKE_DIR" = "" ]; then
   OUTDIR=$ASCDS_WORK_PATH/smoke.${LOGNAME}/$TESTID; export OUTDIR
else
   OUTDIR=$ASCDS_SMOKE_DIR/smoke.${LOGNAME}/$TESTID; export OUTDIR
fi
SAVEDIR=$TESTDIR/; export SAVEDIR

INDIR=$TESTDIR
MYSCRIPT=${OUTDIR}/${TESTID}.py
OUTFILE=${OUTDIR}/${TESTID}.txt

if test -d $OUTDIR
then
   \rm -rf $OUTDIR
fi
mkdir -p $OUTDIR


#
# set up test script to run
echo "from pychips import *" > ${MYSCRIPT}
echo "strip_chart(3)" >> ${MYSCRIPT}
echo "add_contour([1,2,5,3,2,1,5,1,9,0,2,3,4,1,3,8,0,0,2,2,0,0,0,2,0],5,5,'color=coral')" >>${MYSCRIPT}
echo "add_contour([0,1,0,0,2,.3,.5,.2,0,1,0,1,2,1,0,0,0,2,0,0,0,2,3,0,0],5,5,'color=gold style=dot depth=77')" >>${MYSCRIPT}
echo "current_plot('plot2')" >>${MYSCRIPT}
echo "add_histogram([1,1.5,2,2.5,3,3.5,4,4.5,5],None,[8,-4,9,17,12,3,0,5,8])" >>${MYSCRIPT}
echo "set_preference('label.size','14')" >>${MYSCRIPT}
echo "add_label(1.45,-3,'\\pi')" >>${MYSCRIPT}
echo "add_label(2.45,12,'\\Sigma','color=plum depth=200')" >>${MYSCRIPT}
echo "add_label(3.45,.5,'\\Beta','color=darkred depth=200')" >>${MYSCRIPT}
echo "add_label(4.45,2,'\\heartsuit','color=forest depth=200')" >>${MYSCRIPT}
echo "set_histogram(['fill.style',1,'fill.color','cornflower','dropline',1])" >>${MYSCRIPT}
echo "current_plot('plot3')" >>${MYSCRIPT}
echo "add_curve([1,1.5,2,2.5,3,3.5,4,4.5,5], [8,-4,9,17,12,3,0,5,8], 'line.color=lime depth=32')" >>${MYSCRIPT}
echo "add_curve([0,1.5,2,2.5,3,3.5,4,4.5,5], [2,3,5,7,9,8,6,3.5,2], 'line.color=red symbol.style=none')" >>${MYSCRIPT}
echo "limits(X_AXIS,.4,AUTO)" >>${MYSCRIPT}
echo "print(info())" >>${MYSCRIPT}
echo "print(info_depth())" >>${MYSCRIPT}

#
# Run command
#
echo " CHIPS1 SMOKE TEST LOG " > $OUTFILE
echo "" >> $OUTFILE
printf "Running test $TESTID "
printf  "."
python ${MYSCRIPT} | grep -v "missing on display" >> ${OUTFILE}  2>&1
printf  "."

#
# Run checks on the output files.
#
state=0
diff "$OUTFILE" "$SAVEDIR/${TESTID}.txt" 2>&1 >> $OUTDIR/diff.log
if test $? != 0
then
   state=1
fi

#
# Exit condition
#
printf ". "
if test $state = 0
then
  echo "PASS"

  if echo "$OUTDIR" | grep "$TESTID" >/dev/null
  then
     rm $OUTDIR/${TESTID}.*
     rm $OUTDIR/diff.log
     rmdir $OUTDIR
  fi
else
  echo "FAIL"
fi

exit $state
