#! /bin/sh

TESTID=chips-smoke002
export TESTID

#
# Replace with variable via configure
#
ASCDS_QUIET=1; export ASCDS_QUIET
ASCDS_OVERRIDE=1; export ASCDS_OVERRIDE
locale -a | grep -i "en_US" > /dev/null
if test $? = 0; then
   LANG="en_US.UTF-8"; export LANG
fi
. setup.sh

#
# Setup test env
#
TESTDIR=$ASCDS_INSTALL/test/smoke/data; export TESTDIR
if [ "$ASCDS_SMOKE_DIR" = "" ]; then
   OUTDIR=$ASCDS_WORK_PATH/smoke.${LOGNAME}/$TESTID; export OUTDIR
else
   OUTDIR=$ASCDS_SMOKE_DIR/smoke.${LOGNAME}/$TESTID; export OUTDIR
fi
SAVEDIR=$TESTDIR/; export SAVEDIR

INDIR=$TESTDIR
MYSCRIPT=${OUTDIR}/${TESTID}.py
OUTFILE=${OUTDIR}/${TESTID}.txt

if test -d $OUTDIR
then
   \rm -rf $OUTDIR
fi
mkdir -p $OUTDIR

#
# set up test script to run
echo "from pychips import *"               >  $MYSCRIPT
echo "set_preferences(['foreground.display','navy','background.display','beige', 'curve.line.style','noline', 'curve.err.color','red', 'region.fill.style', '1', 'info.current', 'true','info.depth','true'])"   >> $MYSCRIPT
echo "add_curve([10,38,32,20,30,50,25,37,19,42],[40,37,28,10,15,33,21,44,17,19],[3,2,0,0,1,3,5,2,2,0])"                          >> $MYSCRIPT
echo "set_axis ('majorgrid.visible=1')"           >> $MYSCRIPT
echo "add_region([10,37,50,42],[40,44,33,19], 'fill.color=blue')"  >> $MYSCRIPT
echo "add_region(30,32,28,6.7,'fill.color=green depth=50')" >> $MYSCRIPT
echo "add_label(10,46,'\\Omega zone', 'color=sienna')"  >> $MYSCRIPT
echo "add_label(33,28, 'overlap', 'fontstyle=italic')"  >> $MYSCRIPT
echo "add_label(21,10, 'sample A', 'angle=45')" >> $MYSCRIPT
echo "add_line([16,22,28],[46.5,46.5, 42.5])"   >> $MYSCRIPT
echo "add_point(32,22,'style=diamond color=gray')"      >> $MYSCRIPT
echo "add_point(36,32,'style=diamond color=gray')"      >> $MYSCRIPT
echo "add_point(19,37,'style=arrow color=white depth=200 angle=-30 size=18')"   >> $MYSCRIPT
echo "print(info())"          >> $MYSCRIPT

 
#
# Run command
#
echo " CHIPS2 SMOKE TEST LOG " > $OUTFILE
echo "" >> $OUTFILE
printf "Running test $TESTID "

printf "."

python ${MYSCRIPT} | grep -v "missing on display" >> ${OUTFILE}  2>&1

#
# Run checks on the output files.
#
printf "."
state=0
diff "$OUTFILE" "$SAVEDIR/${TESTID}.txt" 2>&1 >> $OUTDIR/diff.log
if test $? != 0
then
   state=1
fi

printf ". "
#
# Exit condition
#
if test $state = 0
then
  echo "PASS"
  if echo "$OUTDIR" | grep "$TESTID" >/dev/null
  then
     rm $OUTDIR/${TESTID}.*
     rm $OUTDIR/diff.log
     rmdir $OUTDIR
  fi
else
  echo "FAIL"
fi

exit $state
