// 
//  Copyright (C) 2004,2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsAnnotation.hh"

#include <stdio.h>
#include <math.h>

extern ChipsDefaults* theDefaults; 

ChipsAnnotation::ChipsAnnotation(ChipsObject* par, 
   string val):ChipsObject(par, val)
{
   id = val; 
   initialized = false; 
   nx1 = ny1 = 0.0;
   nx2 = ny2 = 1.0;
   angle = 0; 
}

ChipsAnnotation::ChipsAnnotation (const ChipsAnnotation &ca):ChipsObject (ca)
{
   angle = ca.angle;
   color = ca.color;
   initialized = ca.initialized;

}


ChipsAnnotation::~ChipsAnnotation()
{

}


int ChipsAnnotation::set_angle(float in_angle)
{
   float old_angle = angle;
   int rval = ERR_STATUS_GOOD; 

   // convert angle (in degrees) to range 0:360
   angle = fmod(in_angle, (float)360.0);
   if (angle < 0.0)  angle = 360.0 + angle;

   // if angle has changed, call virtual to handle change
   if (initialized && (old_angle != angle))  rval = change_rotation();

   return rval;
}


int ChipsAnnotation::set_color(string in_color)
{
   int rval = ERR_STATUS_GOOD;
   if ((rval = color.set_foreground_color(in_color)) == ERR_STATUS_GOOD)
      if (initialized)  rval = change_color();
   return rval;
}


int ChipsAnnotation::change_color()
{
   return ERR_FEATURE_DNE;
}

int ChipsAnnotation::change_rotation()
{
   return ERR_FEATURE_DNE;
}

