// 
//  Copyright (C) 2004-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSAXIS_HH
#define CHIPSAXIS_HH

#include <string>
#include <stdlib.h>
#include <stdio.h>
#include "ChipsObject.hh"
#include "ChipsDefaults.hh"
#include "ChipsLabel.hh"
#include "ChipsColor.hh"
#include "ChipsStateStream.hh"
#include "ChipsTextSupport.hh"
#include "ChipsCommand.hh"
#include "ChipsAxisAlgorithmFactory.hh"

class ChipsAxisAlgorithm;
class ChipsDataSystem;
class Transform;

using namespace std;

#define AUTOLIM_DEF      0
#define AUTOLIM_OFF      1
#define AUTOMIN_ON       2
#define AUTOMAX_ON       4
#define AUTOLIM_ON       (AUTOMAX_ON | AUTOMIN_ON)


class ChipsAxis: public ChipsObject
{
  public:

   ChipsAxis(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, 
                ChipsAxisAlgorithmFactory::TransformType alg_type, string val);
   ChipsAxis(ChipsObject* par,  ChipsCoordinate *inCoord, COORD_SYS coord_sys,
                ChipsStateStream& in);
   ChipsAxis(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys,
                double inMinXPos, double inMinYPos, double inMaxXPos,
                double inMaxYPos, int inOrientation,
                ChipsAxisAlgorithmFactory::TransformType alg_type,
                string val);
   ChipsAxis(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys,
                double inMinXPos, double inMinYPos, double inMaxXPos, 
                double inMaxYPos, int inOrientation, double inmin, 
                double inmax, ChipsAxisAlgorithmFactory::TransformType alg_type,
                string val);

   void       object_delete ();

   void       initialize(); 
   void       post_initialize (ChipsCoordinate *inCoord, COORD_SYS coord_sys);
   void       synchronize(const ChipsAxis& other);
   int        reset_values();

   int        set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj);
   int        set_property(void* in_val, int property, bool rec=true);
   int        get_property(void* out_val, int property);

   string     get_color()    { return color.get_name(); }
   int        set_color(string in_color); 

   ChipsLabel* get_label()   { return label; } 
   string     get_label_text();
   int        set_label_text(string val);

   string     get_label_color(); 
   int        set_label_color(string in_color);
   int        get_label_size(); 
   int        set_label_size(int sz);
   float      get_label_angle();
   int        set_label_angle(float angle);
   float      get_label_halign();
   int        set_label_halign(float angle);
   float      get_label_valign();
   int        set_label_valign(float angle);
   string     get_label_font();
   int        set_label_font(string font);
   V_FONTSTYLE_TYPE get_label_fontstyle();
   int        set_label_fontstyle(string fontstyle);
   int        set_label_visible(bool viz);
   int        get_label_visible();
   float      get_thickness()   { return thickness; }
   int        set_thickness(float val); 

   float      get_major_thickness()   { return major_thickness; }
   int        set_major_thickness(float val); 

   float      get_minor_thickness()   { return minor_thickness; }
   int        set_minor_thickness(float val); 

   double     get_pad()               { return pad; }
   int        set_pad(double in_val); 

   bool       get_use_pad()           { return use_pad; }
   int        set_use_pad(bool in_val){ this->use_pad = in_val; return ERR_STATUS_GOOD; };

   double     get_absolute_min (bool transform=true);
   double     get_absolute_max (bool transform=true);
   double     get_min (bool transform=true);
   double     get_max (bool transform=true);

   bool       get_transform_valid ();
   void       transform_tick_limits ();
                  

   V_TICKSTYLE_TYPE get_major_tickstyle()   { return major_tickstyle; } 
   int        set_major_tickstyle(V_TICKSTYLE_TYPE val);

   V_TICKSTYLE_TYPE get_minor_tickstyle()   { return minor_tickstyle; } 
   int        set_minor_tickstyle(V_TICKSTYLE_TYPE val);

   V_TICKSTYLE_TYPE get_ticklabelstyle()   { return ticklabstyle; }
   int        set_ticklabelstyle(V_TICKSTYLE_TYPE val);
   string     get_ticklabel_color();
   int        set_ticklabel_color(string in_color);
   int        get_ticklabel_size();
   int        set_ticklabel_size(int sz);
   float      get_ticklabel_angle();
   int        set_ticklabel_angle(float angle);
   float      get_ticklabel_halign(bool computed=false); // compute true will calculate auto
   int        set_ticklabel_halign(float angle);
   float      get_ticklabel_valign(bool computed=false); // compute true will calculate auto
   int        set_ticklabel_valign(float angle);
   string     get_ticklabel_font();
   int        set_ticklabel_font(string font);
   V_FONTSTYLE_TYPE get_ticklabel_fontstyle();
   int        set_ticklabel_fontstyle(string fontstyle);
   int        get_ticklabel_offset();
   int        set_ticklabel_offset(int sz);
   int        set_ticklabel_visible(bool viz);
   int        get_ticklabel_visible()    { return ticklabvis; }
   int        adjust_autologmin(double minval, bool rec=true); 
   int        adjust_logmin(double minval, bool rec=true); 
   int        adjust_min(double minval, bool rec=true);
   int        adjust_max(double maxval, bool rec=true);
   int        adjust_minmax(double minval, double maxval, int autos=0, bool rec=true, bool transformed=true);
   int        adjust_autominmax(double minval, double maxval, bool rec=true);
   int        adjust_from_aspect(double minval, double maxval); 
   void       check_minmax (bool &reversed);
//   void       check_minmax (double &minval, double &maxval, bool &reversed);

   int        reverse_axis(); 

   double     get_log_minimum();
   int        set_log_minimum(double val);

   double     get_minor_int();
   int        set_minor_int(double val);

   double     get_major_int();
   int        set_major_int(double val);

   float      get_parallel_offset() { return offset_par; }
   int        set_parallel_offset(float in_offset); 

   float      get_perpendicular_offset() { return offset_perp; }
   int        set_perpendicular_offset(float in_offset); 

   bool       get_title_flip() { return title_flip;};
   int        set_title_flip(bool inFlag);

   V_TICKMODE_TYPE get_major_mode();
   int        set_major_mode(V_TICKMODE_TYPE inmode);
   int        set_major_mode(string inmode);

   V_TICKMODE_TYPE get_minor_mode();
   int        set_minor_mode(string inmode);

   void       show_minor()      { minor_vis = true; change_minor_visible(); }
   void       hide_minor()      { minor_vis = false; change_minor_visible(); }
   bool       minor_visible()   { return minor_vis; }

   void       show_major()      { major_vis = true; change_major_visible(); }
   void       hide_major()      { major_vis = false; change_major_visible(); }
   bool       major_visible()   { return major_vis; }

   string     get_tickformat()      { return tickformat; }
   int        set_tickformat(string val);  

   int        get_major_num();
   int        set_major_num(int val);

   int        get_minor_num();
   int        set_minor_num(int val);

   string     get_major_color()    { return major_color.get_name(); }
   int        set_major_color(string in_color); 

   string     get_minor_color()    { return minor_color.get_name(); }
   int        set_minor_color(string in_color); 

   int        get_major_len()   { return major_len; }
   int        set_major_len(int val);

   int        get_minor_len()   { return minor_len; }
   int        set_minor_len(int val);

   bool       get_reverse()       { return reverse; }

   bool       get_scalable();

   bool       get_wrote_limits()  { return wrote_limits; }
   void       set_wrote_limits(bool val) { wrote_limits = val; }

   void       set_automin_used(bool val=true) { automin_used = val; }
   bool       get_automin_used()      { return automin_used; }
   bool       get_automin()      { return automin; }
   void       set_automin(bool val);

   void       set_automax_used(bool val=true) { automax_used = val; }
   bool       get_automax_used()      { return automax_used; }
   bool       get_automax()      { return automax; }
   void       set_automax(bool val);

   bool       get_major_grid_visible()    { return grid_use_vis; }
   int        set_major_grid_visible(bool val);  
   float      get_major_grid_thickness()  { return grid_use_thick; }
   int        set_major_grid_thickness(float val);
   V_LINE_TYPE get_major_grid_style()     { return grid_use_style; }
   int        set_major_grid_style(V_LINE_TYPE val);
   string     get_major_grid_color()      { return grid_use_clr.get_name(); }
   int        set_major_grid_color(string in_color);

   bool       get_minor_grid_visible()    { return grid_min_vis; }
   int        set_minor_grid_visible(bool val);  
   float      get_minor_grid_thickness()  { return grid_min_thick; }
   int        set_minor_grid_thickness(float val);
   V_LINE_TYPE get_minor_grid_style()     { return grid_min_style; }
   int        set_minor_grid_style(V_LINE_TYPE val);
   string     get_minor_grid_color()      { return grid_min_clr.get_name(); }
   int        set_minor_grid_color(string in_color);
   // Start and stop position of the grid for this axis
   // the coordinates are expected to be in frame normalized
   int        set_grid_position (double start_pos, double stop_pos); 

   int        log_scale();
   int        linear_scale(); 
   int        get_scaletype();
          
   void       get_range(double& outMin,double& outMax) 
                 { outMin = this->get_min (); 
                   outMax = this->get_max (); }
   void       get_linear_range(double& outMin,double& outMax, bool transformed=true);


   int        set_ll_corner(V_CORNERSTYLE_TYPE val); 
   int        set_ur_corner(V_CORNERSTYLE_TYPE val); 
   int        set_corners(V_CORNERSTYLE_TYPE val, double lv, double rv); 
   int        set_ll_corner_width(double val); 
   int        set_ur_corner_width(double val); 
   int        set_alignment(V_TICKSTYLE_TYPE val);
   double     get_ll_corner_width()  { return ll_corner_width; }
   double     get_ur_corner_width()  { return ur_corner_width; }
   double     get_orig_pos();
   int        set_tick_positions(unsigned int arraysize, double* pos, 
                                 string* vals = NULL, bool rec=true); 
   void       get_tick_positions(unsigned int &arraysize, double** pos,
                                 string** text);
   void       get_arbitrary_tick_positions (unsigned int &arraysize, double** pos,
                                 string** text);

   int        get_orientation ();

   int        set_data_coord_id (const string &id);
   int        remove_data_coord_id (const string &id);
   string     get_data_coord_id (int index);
   void       get_data_coord_id (list<string> &ids);


   // QUESTION FOR DEPTH. IS THIS NECESSARY!!
   double get_norm_pos();
   virtual int set_depth(int newdepth);
   int         move_object(V_SUBOBJECT_TYPE obj, COORD_SYS coord_sys, const string &coord_id, double xx, double yy, int mode);
   int        resize_object(V_SUBOBJECT_TYPE subobj, COORD_SYS coord_sys, const string &coord_id, double *bounds);
   int        rotate_object(V_SUBOBJECT_TYPE subobj, float angle);
   int        select_object(V_SUBOBJECT_TYPE subobj, bool selected);

   virtual int move(COORD_SYS coord_sys, const string &coord_id, double xx,
                    double yy, int mode, bool rec=true);
   virtual int reset_position ();
   int         adjust_default_colors();

   int         compile_synch_data(double** d_p, int** i_p, string** s_p);
   int         unsync_data(double* d_p, int* i_p, string* s_p);

   virtual int change_depth() { return ERR_FEATURE_DNE; }
   virtual int change_color();
   virtual int change_position(); 
   virtual int change_range();
   virtual int change_reverse();
   virtual int change_scale_factor();
   virtual int change_scale();
   virtual int change_thickness(); 
   virtual int change_tick_format(); 
   virtual int change_ticklabel_style();
   virtual int change_ticklabel_color();
   virtual int change_ticklabel_font();
   virtual int change_ticklabel_fontstyle();
   virtual int change_ticklabel_size();
   virtual int change_ticklabel_angle();
   virtual int change_ticklabel_offset();
   virtual int change_ticklabel_visible();
   virtual int change_ticklabel_alignment();
   virtual int change_visible(); 
   virtual int change_log_minimum();
   virtual int change_major_color();
   virtual int change_major_interval(); 
   virtual int change_major_length(); 
   virtual int change_major_mode(); 
   virtual int change_major_num(); 
   virtual int change_major_style(); 
   virtual int change_major_thickness(); 
   virtual int change_major_visible(); 
   virtual int change_minor_color();
   virtual int change_minor_interval(); 
   virtual int change_minor_length(); 
   virtual int change_minor_mode(); 
   virtual int change_minor_num(); 
   virtual int change_minor_style(); 
   virtual int change_minor_thickness(); 
   virtual int change_minor_visible(); 
   virtual int change_parallel_offset(); 
   virtual int change_perpendicular_offset(); 
   virtual int change_title_flip();
   virtual int change_title_visible();
   virtual int change_grid_property();
   virtual int change_corner(); 
   virtual int change_alignment(); 
   virtual int set_plot_location (double x1, double y1, double x2, double y2);

   virtual int set_major_grid (double ***grid, int slices, int lines);
   virtual int set_minor_grid (double ***grid, int slices, int lines);

   void       serialize(ChipsStateStream& out);
   virtual void serialize_ascii(ChipsStateStream& out);
   void       deserialize(ChipsStateStream& in);
   string     build_axis_property_string(int set_flag=0);
   string     build_axis_scale_string();
   string     build_set_axis_string(int setatts=1);
   void       set_axis_num (int val) {axis_num = val;};
   int        get_axis_num () {return axis_num;};
   string     get_alter_id()  { return alterid; }

   virtual int hide ();
   virtual int show ();

   void       set_algorithm_type (ChipsAxisAlgorithmFactory::TransformType type);
   void       set_algorithm_type (Transform *transform);
   ChipsAxisAlgorithmFactory::TransformType get_algorithm_type ();
   bool       check_transform_compatibility (Transform *transform);

   int        update_tick_positions ();
   int        update_tick_limits ();

   inline void set_shared_ds (bool flag) { this->shared_ds = flag; };
   inline bool get_shared_ds () { return this->shared_ds; };

   inline void set_border_axis (bool flag) { this->border_axis = flag; };
   inline bool get_border_axis () { return this->border_axis; };

   int        format_axis_point (double pnt, string &formated_pnt);


   ChipsDataSystem *get_data_system (int index=0);
   ChipsDataSystem *get_data_system (const string &ds_id);

   int        validate_limits (const ChipsVector &coord, bool transformed);
   int        validate_limits (double xmin, double xmax, double ymin, double ymax, int dim, bool transformed);

  protected:
   virtual ~ChipsAxis();

   void       invert_labels(); // does axis need to invert label/ticklabels
   void       set_use_major_tickstyle(); // set up majortick style to use 
   void       set_use_minor_tickstyle(); // set up majortick style to use 
   int        calculate_norm_coordinate ();
   void       update_tick_text ();
   bool       is_sexigesimal_format(string val);


   ChipsVector orig_pos[2];  // axis position in its original coordinates
   ChipsVector norm_pos[2];  // axis position in frame normalized coordinates
   ELEM_P_T*  tick_text;     // latex parsed tick trees 
   unsigned int ttext_cnt;   // number of text elements  

   double     pad;           // pad factor to apply to each side of axis
   bool       use_pad;       // flag to turn on/off padding -- overrides automin/max when off
   double     label_angle;   // angle for axis label 
   double     use_halign;    // horizontal alignment (including auto) 
   double     use_valign;    // vertical alignment (including auto) 
   float      offset_par;    // parallel offset for axis label 
   float      offset_perp;   // perpendicular offset for axis label 
   bool       title_flip;    // flip of labels inside/outside of axis
   bool       major_vis;     // are major ticks visible? 
   bool       minor_vis;     // are minor ticks visible? 
   bool       reverse;       // reverse the axis 
   bool       reverse_set;   // user explicitly set the reverse (ascii state)
   float      thickness;     // axis thickness 
   float      major_thickness; // major tick thickness 
   float      minor_thickness; // minor tick thickness 
   int        major_len;   // length of major ticks  
   int        minor_len;   // length of minor ticks  
   V_TICKSTYLE_TYPE major_tickstyle; // type of major tickmarks 
   V_TICKSTYLE_TYPE minor_tickstyle; // type of minor tickmarks 
   V_TICKSTYLE_TYPE use_major_tickstyle; // adjusted tickstyle to use 
   V_TICKSTYLE_TYPE use_minor_tickstyle; // adjusted tickstyle to use 
   V_TICKSTYLE_TYPE ticklabstyle; // type of tick labels- inside/outside
   ChipsColor       ticklabcolor; // color of tick labels
   string           ticklabfontfile;  // fontfile for tick labels
   string           ticklabfont;  // font of tick labels
   V_FONTSTYLE_TYPE ticklabfontstyle; // font style of tick labels 
   float            ticklabangle; // angle of tick labels
   int              ticklabsize;  // size of tick labels
   int              ticklaboffset; // offset of tick labels
   bool             ticklabvis;    // tick label visibility
   double           ticklabvertalignment; // vertical alignment of tick label
   double           ticklabhorizalignment; // horiz alignment of tick label

   bool       automin;       // automatically set min limit 
   bool       automax;       // automatically set max limit 
   bool       automin_used;  // limits were setting using automin   
   bool       automax_used;  // limits were setting using automax  
   bool       wrote_limits;  // have limits for this axis been serialized?
   bool       initialized;   // have we initialized everything 
   bool       sexagesimal;   // are we displaying sexagesimal (hh:mm:ss)
   string     tickformat;    // tick format  
   ChipsColor color;         // color of axis 
   ChipsColor major_color;   // color of major ticks 
   ChipsColor minor_color;   // color of minor ticks 
   ChipsLabel* label;        // label for the axis 

   ChipsColor  grid_use_clr;  // major grid color user setting 
   V_LINE_TYPE grid_use_style;// major grid style user setting 
   bool        grid_use_vis;  // display major grid user setting 
   float       grid_use_thick;// major grid line thickness user setting 
   ChipsColor  grid_maj_clr;  // major grid color internal setting 
   V_LINE_TYPE grid_maj_style;// major grid style internal setting 
   bool        grid_maj_vis;  // display major grid internal setting 
   float       grid_maj_thick;// major grid line thickness internal setting 
   ChipsColor  grid_min_clr;  // minor grid color 
   V_LINE_TYPE grid_min_style;// minor grid style 
   bool        grid_min_vis;  // display minor grid 
   float       grid_min_thick;// minor grid line thickness 
   double      grid_start_pos;// where plot area starts
   double      grid_stop_pos; // where plot area ends

   int        label_size;    // size of text in label 
   string     label_color;   // color of label text
   string     label_font;   // font of label 
   double     label_vertalignment; // vertical alignment of tick label
   double     label_horizalignment; // horiz alignment of tick labe
   V_FONTSTYLE_TYPE label_fontstyle;
   bool       labvis;        // axis label visibility
   int        deserialize_label; // deserialize label in derived constructor
   string     temp_label;    // temporary string to hold label text

   int        axis_num;       // this axis_x order 0th, 1st, 2nd, etc.
   string     alterid;        // altered id for state file

   double     ll_corner_width;    // thickness of lower/left corner joint 
   double     ur_corner_width;    // thickness of upper/right corner joint 
   V_CORNERSTYLE_TYPE ll_corner;  // style of lower/left corner joint
   V_CORNERSTYLE_TYPE ur_corner;  // style of upper/right corner joint
   V_TICKSTYLE_TYPE alignment;    // axis alignment support (in/out/centered)


   ChipsAxisAlgorithm  *axis_alg; // the type of axis being created

   list<string>     data_coord_ids;      // the id of an associated data systems

   bool       shared_ds;          // using shared data systems
   bool       border_axis;        // indicate border axis

   

  private:
   bool add_command (ChipsCommand *inCmd, int &outStatus);

};

#endif 
