#include "ChipsAxisAlgorithm.hh"
#include "ChipsAxis.hh"

ChipsAxisAlgorithm::ChipsAxisAlgorithm (int orientation, ChipsAxis *axis)
{
   this->modified = true;
   this->scalable = true;
   this->scaletype = 0;

   this->ttext_cnt = 0;
   this->tickcnt = 0;
   this->arbcnt = 0;

   this->tickpos = NULL;
   this->arbpos = NULL;
   this->arbtext = NULL;

   this->tick_text = NULL;
   this->ttext_cnt = 0;

   this->minortickpos = NULL;
   this->minortickcnt = 0;

   this->minor_mode = chips_count;
   this->minor_mode_adjusted = chips_count;
   this->major_mode = chips_nice;
   this->minor_num = 3;
   this->minor_num_adjusted = 3;
   this->major_num = 4;
   this->major_int = 3;
   this->minor_int = 5;

   this->arbitrary_set = false;

   this->orientation = orientation;

   this->axis = axis;

   this->transform_valid = false;

}

ChipsAxisAlgorithm::ChipsAxisAlgorithm (ChipsAxisAlgorithm &copy)
{

   // dont copy axis

   this->scalable = copy.scalable;
   this->scaletype = copy.scaletype;

   this->ttext_cnt = copy.ttext_cnt;
   this->tickcnt = copy.tickcnt;
   this->arbcnt = copy.arbcnt;


   this->tickpos = NULL;
   this->arbpos = NULL;
   this->arbtext = NULL;
   this->tick_text = NULL;

   if ((this->tickcnt > 0) && copy.tickpos)
   {
      this->tickpos = new double[this->tickcnt];
      for (int ii=this->tickcnt; ii--;)
         this->tickpos[ii] = copy.tickpos[ii];
   }

   if ((this->arbcnt > 0) && copy.arbpos)
   {
      this->arbpos = new double[this->arbcnt];
      for (int ii=this->arbcnt; ii--;)
         this->arbpos[ii] = copy.arbpos[ii];

      if (copy.arbtext)
      {
         this->arbtext = new string[this->arbcnt];
         for (int ii=this->arbcnt; ii--;)
            this->arbtext[ii] = copy.arbtext[ii];
      }
   }
    
   if ((this->ttext_cnt > 0) && copy.tick_text)
   {
      this->tick_text = new string[this->ttext_cnt];
      for (int ii=this->ttext_cnt; ii--;)
         this->tick_text[ii] = copy.tick_text[ii];
   }

   this->minortickcnt = copy.minortickcnt;
   this->minortickpos = NULL;
   if (this->minortickcnt > 0)
   {
      this->minortickpos = new double[this->minortickcnt];
      for (int ii=this->minortickcnt; ii--;)
         this->minortickpos[ii] = copy.minortickpos[ii];
   }

   this->minor_mode = copy.minor_mode;
   this->minor_mode_adjusted = copy.minor_mode_adjusted;
   this->major_mode = copy.major_mode;
   this->minor_num = copy.minor_num;
   this->minor_num_adjusted = copy.minor_num_adjusted;
   this->major_num = copy.major_num;
   this->major_int = copy.major_int;
   this->minor_int = copy.minor_int;

   this->orientation = copy.orientation;

   this->arbitrary_set = false;

   this->modified = true;

   this->transform_valid = copy.transform_valid;

}


ChipsAxisAlgorithm::~ChipsAxisAlgorithm ()
{

   if (this->tickpos)
   {
      delete []this->tickpos;
      this->tickpos = NULL;
   }

   if (this->minortickpos)
   {
      delete []this->minortickpos;
      this->minortickpos = NULL;
   }

   if (this->arbpos)
   {
      delete []this->arbpos;
      this->arbpos = NULL;
   }

   if (this->arbtext)
   {
      delete []this->arbtext;
      this->arbtext = NULL;
   }

   if (this->tick_text)
   {
      delete []this->tick_text;
      this->tick_text = NULL;
   }

}

void ChipsAxisAlgorithm::operator = (ChipsAxisAlgorithm &copy)
{
   // don't copy axis

   this->scalable = copy.scalable;
   this->scaletype = copy.scaletype;

   this->ttext_cnt = copy.ttext_cnt;
   this->tickcnt = copy.tickcnt;
   this->arbcnt = copy.arbcnt;

   if (this->tickpos)
   {
      delete []this->tickpos;
      this->tickpos = NULL;
   }

   if (this->tickcnt > 0)
   {
      this->tickpos = new double[this->tickcnt];
      for (int ii=this->tickcnt; ii--;)
         this->tickpos[ii] = copy.tickpos[ii];
   }

   if (this->arbpos)
   {
      delete []this->arbpos;
      this->arbpos = NULL;
   }

   if (this->arbtext)
   {
      delete []this->arbtext;
      this->arbtext = NULL;
   }

   if (this->arbcnt > 0)
   {
      this->arbpos = new double[this->arbcnt];
      for (int ii=this->arbcnt; ii--;)
         this->arbpos[ii] = copy.arbpos[ii];
   }

   if (this->tick_text)
   {
      delete []this->tick_text;
      this->tick_text = NULL;
   }

   if (this->ttext_cnt > 0)
   {
      this->tick_text = new string[this->ttext_cnt];
      for (int ii=this->ttext_cnt; ii--;)
         this->tick_text[ii] = copy.tick_text[ii];
   }

   this->minortickcnt = copy.minortickcnt;

   if (this->minortickpos)
   {
      delete []this->minortickpos;
      this->minortickpos = NULL;
   }

   if (this->minortickcnt > 0)
   {
      this->minortickpos = new double[this->minortickcnt];
      for (int ii=this->minortickcnt; ii--;)
         this->minortickpos[ii] = copy.minortickpos[ii];
   }

   this->set_minor_mode (copy.minor_mode);
   this->minor_mode_adjusted = copy.minor_mode_adjusted;
   this->set_major_mode (copy.major_mode);
   this->minor_num = copy.minor_num;
   this->minor_num_adjusted = copy.minor_num_adjusted;
   this->major_num = copy.major_num;
   this->major_int = copy.major_int;
   this->minor_int = copy.minor_int;
   this->orientation = copy.orientation;

   this->arbitrary_set = false;
   this->modified = true;

   this->transform_valid = copy.transform_valid;

}

void ChipsAxisAlgorithm::set_minor_num (int num)
{
   if (this->minor_num != num)
      this->modified=true; 
   this->minor_num = num;
}

void ChipsAxisAlgorithm::set_major_num (int num) 
{
   if (this->major_num != num)
      this->modified=true; 
   this->major_num = num;
}

void ChipsAxisAlgorithm::set_minor_interval (double interval) 
{
   if (this->minor_int != interval)
      this->modified=true;
    this->minor_int = interval;
}

void ChipsAxisAlgorithm::set_major_interval (double interval) 
{
   if (this->major_int != interval)
      this->modified=true;
   this->major_int = interval;
}

