// 
//  Copyright (C) 2005-2007,2009,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSAXISBIND_HH
#define CHIPSAXISBIND_HH

class ChipsPlot;
class ChipsAxisBind;
class ChipsAxisGraph;
class ChipsFrame;

#include "ChipsAxis.hh"
#include "ChipsStateStream.hh"
#include <string>
#include <stdio.h>
#include <list>
#include <map>

using namespace std;


typedef struct axisbind_elem_t
{
   ChipsAxis*  axis1;
   ChipsAxis*  axis2; 
} AXISBIND_ELEM_T, *AXISBIND_ELEM_P_T;

class ChipsRecordElem
{
  public:
   ChipsRecordElem () {
      track_updates = 0;
      updates_recorded = 0;
      property_recorded = 0;
   }

   map <string, list <string> > updated_axes;  // plots and axes which were
                                               //   touched during an update
   bool track_updates;         // indicate track updates
   int updates_recorded;       // mask indicating what was updated: 
                               //   1 = limits, 2 = property
   int property_recorded;
};


class ChipsAxisBind
{
  public:

   ChipsAxisBind();
  ~ChipsAxisBind();

   int      add_aspect_ratio(ChipsAxis* a1, ChipsAxis*a2, double xx, double yy);
   int      remove_aspect_ratio(ChipsAxis* a1, ChipsAxis*a2);
   int      update_connected_nodes(ChipsAxis* a1, double min, double max,
                                   bool transformed,
                                   int autolim=AUTOLIM_OFF, int center = 0,
                                   bool rec=true, double *altmin=NULL, double *altmax=NULL,
                                   string altid="");
   int      update_connected_node_properties(ChipsAxis* a1, bool rec=true);
   int      update_all_nodes(int dim, double min, double max,
                                   bool transformed,
                                   int autolim=AUTOLIM_OFF, int center = 0,
                                   bool rec=true, double *altmin=NULL, double *altmax=NULL); 
   bool     aspect_ratio_set(ChipsAxis* ax);

   /*** Temporary Code ******/
   int      adjust_axis_minmax (ChipsAxis* a1, double min, double max, bool transformed, int autos=0, bool rec=false);
   int      adjust_axis_min_or_max (ChipsAxis *a1, double val, int flag, bool rec=false);

   int      update_property (list<ChipsAxis*> &axis_list, void *val, int property, bool rec=true);
   int      update_property (ChipsAxis *a1, void *val, int property, bool rec=true);


   int      add_binding(ChipsAxis* a1, ChipsAxis* a2);
   int      delete_binding(ChipsAxis* a1, ChipsAxis* a2);
   int      add_axis(ChipsAxis* ax); 
   int      remove_axis(ChipsAxis* ax);
   int      synch_axes(ChipsAxis* ax1, ChipsAxis* ax2, bool rec=true);
   int      compile_synch_data (ChipsAxis *ax, double** d_p, int** i_p, string** s_p);
   int      check_binding(ChipsAxis* a1, ChipsAxis* a2=NULL, bool cross=false);
   int      check_valid_binding(ChipsAxis* a1, ChipsAxis* a2);
   string   get_bindings_list(ChipsAxis* ax); 

   int      record_updates (bool flag);
   int      get_recorded_plots (list <string> &plot_ids);
   int      get_recorded_axes (string plot_id, list <string> &axis_ids);
   int      get_updates_recorded () { return this->record_elem.updates_recorded; };
   int      get_property_recorded () { return this->record_elem.property_recorded; };

   int      find_axis_min_max(ChipsAxis* axis, int dim,
               double* min, double* max, double* lmin, bool apply_transform);


   void     serialize(ChipsStateStream& out);
   void     serialize_ascii(ChipsStateStream& out);
   int      serialize_all_nodes(ChipsStateStream& out);
   void     deserialize (ChipsFrame *cf, ChipsStateStream &out);

   list <ChipsAxis*>  get_axes(ChipsAxis* ax);
   list <ChipsAxis*>  get_connections(ChipsAxis* ax);

   void     set_access_plot(ChipsPlot* plot=NULL) { access_plot = plot; }
   void     reset_access_list(ChipsPlot* cplot=NULL); 

  protected:

   // convert axis ranges from logical to data transformed or
   // vice versa. direction = 0 converting to transformed and
   // direction = 1 convert to logical
   void compute_data_range (ChipsDataSystem *ds, double xrange[], double yrange[], int direction, double out_xrange[], double out_yrange[]);

   list <AXISBIND_ELEM_P_T> bindings;
   string   listings; 
   ChipsAxisGraph* graph; 
   map<ChipsAxis*, int>accessed;
   ChipsPlot* access_plot;

   ChipsRecordElem record_elem;
};

#endif 
