// 
//  Copyright (C) 2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSAXISGRAPH_HH
#define CHIPSAXISGRAPH_HH

#include <vector>
#include <string>
#include <iostream>
#include <map>

#include "ChipsStateStream.hh"

using namespace std;

class ChipsAxisNode;
class ChipsAxis; 
class ChipsId;
class ChipsPlot;
class ChipsPlotData;
class ChipsAnnotation;
class ChipsFrame;

class ChipsAxisEdge   // graph edges for Axis Graph
{
  public:

   ChipsAxisEdge(ChipsAxisNode* from, ChipsAxisNode* to, int typ,
                 double val1 = 0, double val2 = 0);
   ~ChipsAxisEdge(); 
   ChipsAxisNode* get_source()  { return source; }
   ChipsAxisNode* get_dest()    { return dest; }
   int          get_type()    { return type; }
   double       get_value1()  { return v1; }
   double       get_value2()  { return v2; }
   void         list_nodes(); 

  private:
   ChipsAxisNode *source;
   ChipsAxisNode *dest;
   int          type;  // 0 = unknown   1 = ratio   2 = bind
   double       v1;
   double       v2;
}; 

class ChipsAxisNode    // graph vertices for Axis Graph 
{
  public:
   ChipsAxisNode(ChipsAxis* ax);
   ~ChipsAxisNode()  { edges.clear(); }
   string get_id()    { return id; }
   ChipsAxis*   get_axis()   { return axis; }
   int get_status()   { return status; }
   void set_status(int st) { status = st; }
   void add_edge(ChipsAxisNode *node, int type=0, double val1=0.0, 
                 double val2=0.0); 
   void remove_edge(ChipsAxisNode *node); 
   int  check_edge(ChipsAxisNode *node); 
   int  get_edge_values(ChipsAxisNode *node, double &v1, double &v2);
   void get_max_edge_value(double &val);
   vector<ChipsAxisEdge>& get_edges()  { return edges; } 
   int traverse_edges(int val=-1); 
   void list_edges(); 
   void list_associated_axes_dfs(vector<ChipsAxis*> &vec);
   void serialize (ChipsStateStream& out);
   void deserialize (ChipsFrame *cf, ChipsStateStream& in, vector<ChipsAxisNode*> &out_nodes);

  private:
   ChipsAxis* axis; 
   string id;
   vector<ChipsAxisEdge> edges;
   int    status;
};

class ChipsAxisGraph   // graph of related axes for Axis Graph 
{
  public:
   ChipsAxisGraph(); 
   ~ChipsAxisGraph();
   void print();
   void add_node(ChipsAxis* axis);
   int add_edge(ChipsAxis* ax1, ChipsAxis* ax2, int type=0, double val1=0.0,
                double val2=0.0); 
   void remove_edge(ChipsAxis* ax1, ChipsAxis* ax2); 
   void remove_node(ChipsAxis* axis);
   ChipsAxisNode* find_node(ChipsAxis* axis);
   bool check_for_cycle();
   int  get_edge_values(ChipsAxis* ax1, ChipsAxis* ax2,double &min, double&max);
   int  get_edge_max(ChipsAxis* ax1, double &max); 
   vector<ChipsAxis*> list_attached_vertices_dfs(ChipsAxis* entry); 
   vector<ChipsAxis*> list_attached_vertices_bfs(ChipsAxis* entry);
   vector<ChipsAxis*> list_all_vertices();

   void serialize (ChipsStateStream& out);
   void deserialize (ChipsFrame *cf, ChipsStateStream& in);

  private:
   vector<ChipsAxisNode*> nodes;
};

#endif  // do not delete 
