// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsAxisTickCommand.hh"
#include "ChipsStateStream.hh"
#include "ChipsAxis.hh"

ChipsAxisTickCommand::ChipsAxisTickCommand(ChipsAxis* obj, int ocnt,
   double *ovals, string *olabs, int ncnt, double *nvals, string *nlabs)
{
   co = (ChipsObject*)obj;
   weight = 1;

   if ((ocnt > 0) && ovals)
   {
      o_cnt = ocnt;
      o_vals = new double[o_cnt]; 
      if (o_vals)  for (int rr = o_cnt; rr--; )    o_vals[rr] = ovals[rr];
      if (olabs)
      {
         o_labs = new string[o_cnt];
         if (o_labs)  for (int rr = o_cnt; rr--; )    o_labs[rr] = olabs[rr];
      }
      else   o_labs =  NULL;
   }
   else 
   {
      o_cnt = 0; 
      o_vals = NULL;
      o_labs = NULL; 
   }
   if ((ncnt > 0) && nvals)
   {
      n_cnt = ncnt;
      n_vals = new double[n_cnt];
      if (n_vals)  for (int rr = n_cnt; rr--; )    n_vals[rr] = nvals[rr];
      if (nlabs)
      {
         n_labs = new string[n_cnt];
         if (n_labs)  for (int rr = n_cnt; rr--; )    n_labs[rr] = nlabs[rr];
      }
      else   n_labs =  NULL;
   }
   else 
   {
      n_cnt = 0; 
      n_vals = NULL;
      n_labs = NULL; 
   }
}

ChipsAxisTickCommand::~ChipsAxisTickCommand()
{
   if (n_vals)   delete[]  n_vals;
   if (o_vals)   delete[]  o_vals;
   if (n_labs)   delete[]  n_labs;
   if (o_labs)   delete[]  o_labs;

}


ChipsAxisTickCommand::ChipsAxisTickCommand(ChipsObject *cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsAxisTickCommand::execute ()
{
   int rval = ERR_STATUS_BAD;
   if (co && n_vals && (n_cnt > 0))
   {
      ChipsAxis* ca = dynamic_cast <ChipsAxis *>(co);
      rval = ca->set_tick_positions(n_cnt, n_vals, n_labs, false);
   }
   return rval;
}

int ChipsAxisTickCommand::unexecute ()
{
   int rval = ERR_STATUS_BAD;
   if (co && o_vals && (o_cnt > 0))
   {
      ChipsAxis* ca = dynamic_cast <ChipsAxis *>(co);
      rval = ca->set_tick_positions(o_cnt, o_vals, o_labs, false);
   }
   return rval;
}

void ChipsAxisTickCommand::serialize(ChipsStateStream& out)
{
   int val = 21;
   // Standard serialize output for a command
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   out.write_int(&n_cnt);
   if (n_cnt > 0)  out.write_double(n_vals, n_cnt);
   val = (n_labs != NULL);
   out.write_int(&val);
   if (val)
      for (val = n_cnt; val--; )  out.write_string(&n_labs[val]);

   out.write_int(&o_cnt);
   if (o_cnt > 0)  out.write_double(o_vals, o_cnt);
   val = (o_labs != NULL);
   out.write_int(&val);
   if (val)
      for (val = o_cnt; val--; )  out.write_string(&o_labs[val]);
}

void ChipsAxisTickCommand::deserialize(ChipsStateStream& in)
{
   int val; 

   // Standard serialize input for a command
   in.read_int(&ref);
   in.read_int(&block_ref);

   in.read_int(&n_cnt);
   if (n_cnt > 0)
   {
      n_vals = new double[n_cnt];
      in.read_double(n_vals, n_cnt);
   }
   in.read_int(&val);
   if (val)
   {
      n_labs = new string[n_cnt];
      for (val = n_cnt; val--; )  in.read_string(&n_labs[val]);
   }
   else   n_labs = NULL; 
   in.read_int(&o_cnt);
   if (o_cnt > 0)
   {
      o_vals = new double[o_cnt];
      in.read_double(o_vals, o_cnt);
   }
   else 
      o_vals = NULL; 
   in.read_int(&val);
   if (val)
   {
      o_labs = new string[o_cnt];
      for (val = o_cnt; val--; )  in.read_string(&o_labs[val]);
   }
   else   o_labs = NULL; 
}

void ChipsAxisTickCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Axis TickLabels" << endl;
};



