// 
//  Copyright (C) 2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSCARTESIANAXISALGORITHM_HH
#define CHIPSCARTESIANAXISALGORITHM_HH

#include <string>
#include <stdlib.h>
#include <stdio.h>
#include "ChipsAxisAlgorithm.hh"

class Transform;

using namespace std;

class ChipsCartesianAxisAlgorithm : public ChipsAxisAlgorithm
{
  public:

   ChipsCartesianAxisAlgorithm (int orientation, ChipsAxis *axis);
   ChipsCartesianAxisAlgorithm (ChipsCartesianAxisAlgorithm &copy);
   virtual ~ChipsCartesianAxisAlgorithm ();

   virtual ChipsCartesianAxisAlgorithm * clone ();

   virtual void       set_scalable(bool val);
   virtual void       set_scaletype (int val);

   virtual double     get_min (bool transformed=true);
   virtual double     get_max (bool transformed=true);

   virtual double     get_absolute_min (int scaletype=-1, bool transformed=true);
   virtual int        set_absolute_min (double val, int scaletype=-1, bool transformed=true);
   virtual double     get_absolute_max (int scaletype=-1, bool transformed=true);
   virtual int        set_absolute_max (double val, int scaletype=-1, bool transformed=true);
   virtual int        set_absolute_minmax (double min, double max, int scaletype=-1, bool transformed=true);


   void               set_log_minimum (double val);
   double             get_log_minimum () {return this->log_minimum;};

   double             get_log_min () { return this->log_min; };
   double             get_lin_min () { return this->lin_max; };
   double             get_log_max () { return this->log_min; };
   double             get_lin_max () { return this->lin_max; };

   virtual double*    get_tick_positions (bool transformed);
   virtual void       set_tick_positions (double *ticks, int count);
   virtual void       set_arbitrary_positions (double *positions, string *text, int count);
   virtual int        set_major_mode (V_TICKMODE_TYPE mode);
   virtual int        set_minor_mode (V_TICKMODE_TYPE mode);

   virtual int        calculate_tick_limits(bool automin, bool automax,
                                  double pad);
   virtual int        update_tick_positions(bool &out_updates);
   virtual void       reset_string_labels(const string& tickformat);
   virtual double *** calculate_grid (int &out_line_count, int &out_slices, int tick_type);

   virtual void       transform_tick_limits () {};

   virtual void       format_point (const string &tickformat, double pnt, string &formatted_pnt);

   virtual bool       fix_minmax (double &minval, double &maxval, bool transformed);
   virtual bool       validate_coordinates (const ChipsVector &coord, const string &ds_id);

   virtual void       serialize (ChipsStateStream &out);
   virtual void       deserialize (ChipsStateStream &in);

  protected:

   void       reset_adjusted_parameters ();

/********** TEMPORARY CODE ***************/
   int        compute_sexagesimal_locations(int dec);
   int        sexagesimal_dec_ticks(const string& tickformat);
   int        sexagesimal_ra_ticks(const string& tickformat);
/********** END TEMPORARY CODE **********/
   

   int        linear_nice_tick_positions();
   int        linear_count_tick_positions();
   int        linear_limit_tick_positions();
   int        linear_arbitrary_tick_positions();
   int        linear_interval_tick_positions();

   int        log_nice_tick_positions();
   int        log_count_tick_positions();
   int        log_limit_tick_positions();
   int        log_arbitrary_tick_positions();
   int        log_interval_tick_positions();

   int        linear_nice_tick_limits();
   int        linear_count_tick_limits(bool automin, bool automax,
                                             double pad);
   int        linear_limit_tick_limits(bool automin, bool automax,
                                             double pad);
   int        linear_arbitrary_tick_limits(bool automin, bool automax,
                                             double pad);
   int        linear_interval_tick_limits(bool automin, bool automax,
                                             double pad);

   int        log_nice_tick_limits();
   int        log_count_tick_limits();
   int        log_limit_tick_limits(bool automin, bool automax,
                                             double pad);
   int        log_arbitrary_tick_limits(bool automin, bool automax,
                                             double pad);
   int        log_interval_tick_limits(bool automin, bool automax,
                                             double pad);


              // nice tick methods (C.R. Lewart, CACM, Oct 73,v16 n10)
   void       scale1(double inmin, double inmax, int slices, double* outmin,
                     double* outmax, double* dist, double* base=NULL);
   void       scale2(double min, double max, int slices, double* outmin,
                     double* outmax, double* dist);

   void       remove_transform (double *data, int data_size);
   void       apply_transform (double *data, int data_size);
   void       get_axis_position (bool use_transform, ChipsVector &min_coord, ChipsVector &max_coord);
   bool       use_transform ();



   double     padded_min;    // minimum range including padding
   double     padded_max;    // maximum range including padding
   double     lin_min;       // minimum range for linear scale
   double     lin_max;       // minimum range for linear scale
   double     log_min;       // maximum range for log scale
   double     log_max;       // maximum range for log scale

   double     log_minimum;   // minimum log value

   double     last_pad;
   bool       last_automin;
   bool       last_automax;

   double*    untransformed_tickpos;
   bool       transform_applied;


};

#endif

