// 
//  Copyright (C) 2007-2009,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#include <dirent.h>
#include <pwd.h>
#include <sstream>

#include "ChipsClient.hh"
#include "ChipsId.hh"
#include "ChipsMessages.hh"
#include "SCSClient.hh"
#include "SCSMessage.hh"
#include "Transform.hh"
#include "ChipsTransformFactory.hh"


// define static members
double ChipsClient::buffer[1024];
SCSClient chipsClient;
string ChipsClient::serverId="";
string ChipsClient::clientId="";
string ChipsClient::tmpdir="/tmp/.chips";

// constructor 
ChipsClient::ChipsClient(bool load_prefs)
{
   struct passwd *pw = getpwuid(getuid());
   if (pw && pw->pw_name[0] != '\0') 
   {
      tmpdir.append(".");
      tmpdir.append(pw->pw_name);
   }

   // check that the temp directory exists- create it if not
   if (access(tmpdir.c_str(), W_OK) != 0)
   {
      mkdir(tmpdir.c_str(), S_IRUSR | S_IWUSR | S_IXUSR);
      if (access(tmpdir.c_str(), W_OK) != 0)  tmpdir = "/tmp";
   }
   tmpdir.append("/");
}


// destructor 
ChipsClient::~ChipsClient()
{
}


// client method packs ChipsId object & msg id and sent it to the server
int ChipsClient::send(ChipsId* cid,   // chips id object 
   int   msgid,                       // message id value
   void* item1,                       // value for parameter 1 
   void* item2,                       // value for parameter 2 
   void* item3,                       // value for parameter 3
   void* item4,                       // value for parameter 4
   void* item5,                       // value for parameter 5
   void* item6,                       // value for parameter 6
   void* item7,                       // value for parameter 7
   void* item8,                       // value for parameter 8
   void* item9,                       // value for parameter 9
   void* item10,                      // value for parameter 10
   void* item11)                      // value for parameter 11
{

   int rval = ERR_STATUS_GOOD;

   if ((rval = pack_it(cid, msgid, item1, item2, item3, item4, item5, 
		       item6, item7, item8, item9, item10, item11)) ==  ERR_STATUS_GOOD)
     {
       int* lp = (int*)buffer;
       char* cp = (char*)buffer;
       unsigned int len; 
       lp++;
       len = *lp;
       SCSMessage tmpMessage;
       
       long* i1p = static_cast<long*>(item1);
       
       // send client request to server
       if (msgid == MSG_LOAD_CMAP_ARRAY)
         {
	   // message may be too big for buffer - handle individually
	    unsigned int datalen = (*i1p) * sizeof(double);
            *lp = *lp + datalen * 4; //4 because there are 4 arrays 
            char* data_p = (char*)(item3); //Red
	    
	    tmpMessage.Set(cp,len);
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item4);       //Green 	    
	    tmpMessage.Append(data_p,datalen);
	    data_p = (char*)(item5); //Blue
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item6); //Alpha
            tmpMessage.Append(data_p,datalen);
	    
	    if ( chipsClient.SendMessage(-1,tmpMessage) > 0 )
	      rval= ERR_STATUS_GOOD;
	    else
	      rval= ERR_NO_SOCKET;
	 }
       else if (msgid == MSG_ADD_CURVE_A)
	 {
	   // message may be too big for buffer- handle individually
	   unsigned int datalen = (*i1p) * sizeof(double);
            *lp = *lp + datalen * 2; 
            char* data_p = (char*)(item2);
	    
	    tmpMessage.Set(cp,len);
	    tmpMessage.Append(data_p,datalen); // send x data 
            data_p = (char*)(item3);       // send y data 	    
	    tmpMessage.Append(data_p,datalen);
	    
	    if ( chipsClient.SendMessage(-1,tmpMessage) > 0 )
	      rval= ERR_STATUS_GOOD;
	    else
	      rval= ERR_NO_SOCKET;

         }
       else if (msgid == MSG_ADD_CURVE_B)
         {
	   // message may be too big for buffer- handle individually
	   unsigned int datalen = (*i1p) * sizeof(double);
	   int colcnt = 2;
	   if ((char*)(item4) != NULL) colcnt++; 
	   if ((char*)(item5) != NULL) colcnt++; 
	   if ((char*)(item6) != NULL) colcnt++; 
	   if ((char*)(item7) != NULL) colcnt++; 
	   if ((char*)(item8) != NULL) colcnt++; 
	   if ((char*)(item9) != NULL) colcnt++; 
	   *lp = *lp + datalen * colcnt; 
            char* data_p = (char*)(item2);
	    
	    tmpMessage.Set(cp,len);
	    tmpMessage.Append(data_p,datalen); // send x data 
            data_p = (char*)(item3);       // send y data 	    
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item4);       // send xup data 
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item5);       // send xdown data 
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item6);       // send yup data 
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item7);       // send ydown data 
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item8);       // send y upper/lower limits data 
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item9);       // send x upper/lower limits data 
	    tmpMessage.Append(data_p,datalen);

	    if ( chipsClient.SendMessage(-1,tmpMessage) > 0 ) // send header info
	      rval= ERR_STATUS_GOOD;
	    else
	      rval= ERR_NO_SOCKET;
	    tmpMessage.Clear();

         }
       else if (msgid == MSG_ADD_HISTOGRAM)
         {
	   // message may be too big for buffer- handle individually
	   unsigned int datalen = (*i1p) * sizeof(double);
	   int colcnt = 1;
            if ((char*)(item3) != NULL) colcnt++;
            if ((char*)(item4) != NULL) colcnt++;
            if ((char*)(item5) != NULL) colcnt++;
            if ((char*)(item6) != NULL) colcnt++;
            *lp = *lp + datalen * colcnt;
            char* data_p = (char*)(item2);

	    tmpMessage.Set(cp,len);
	    tmpMessage.Append(data_p,datalen); // send x data 
            data_p = (char*)(item3);       // send y data 	    
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item4);       // send xup data 
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item5);       // send xdown data 
	    tmpMessage.Append(data_p,datalen);
            data_p = (char*)(item6);       // send yup data 
	    tmpMessage.Append(data_p,datalen);


	    if ( chipsClient.SendMessage(-1,tmpMessage) > 0 ) // send header info
	      rval= ERR_STATUS_GOOD;
	    else
	      rval= ERR_NO_SOCKET;
	    
         }
         else if (msgid == MSG_ADD_CONTOUR)
         {
            // message may be too big for buffer- handle individually
            unsigned int datalen = (*i1p) * sizeof(double);
            *lp = *lp + datalen*3;

            char* data_p = (char*)(item2);
	    tmpMessage.Set(cp,len);
	    tmpMessage.Append(data_p,datalen);
	    data_p = (char*)(item3);
	    tmpMessage.Append(data_p,datalen); 
	    data_p = (char*)(item4);       
	    tmpMessage.Append(data_p,datalen); 

	    if ( chipsClient.SendMessage(-1,tmpMessage) > 0 ) 
	      rval= ERR_STATUS_GOOD;
	    else
	      rval= ERR_NO_SOCKET;

         }
         else if (msgid == MSG_ADD_CONTOUR_B)
         {
            // message may be too big for buffer- handle individually
            long* i2p = static_cast<long*>(item2);
            long* i3p = static_cast<long*>(item3);
            unsigned int datalen = (*i2p) * (*i3p) * sizeof(double);
            *lp = *lp + datalen;

            char* data_p = (char*)(item1);
	    tmpMessage.Set(cp,len);
	    tmpMessage.Append(data_p,datalen);
	    if ( chipsClient.SendMessage(-1,tmpMessage) > 0 )  // send header info
		rval= ERR_STATUS_GOOD;
	      else
		rval= ERR_NO_SOCKET;

         }
         else if (msgid == MSG_ADD_IMAGE_C)
         {
            // message may be too big for buffer- handle individually
            int* i2p = static_cast<int*>(item5);
            int* i3p = static_cast<int*>(item6);
            unsigned int datalen = (*i2p) * (*i3p) * sizeof(double);

            int image_cnt = 0;

            if (item1) image_cnt++;
            if (item2) image_cnt++;
            if (item3) image_cnt++;
            if (item4) image_cnt++;

            *lp = *lp + (datalen*image_cnt);

            tmpMessage.Set(cp,len);
            char* data_p;
            if (item1)
            {
               data_p = (char*)(item1);
               tmpMessage.Append(data_p,datalen);
            }
            if (item2)
            {
               data_p = (char*)(item2);
               tmpMessage.Append(data_p,datalen);
            }
            if (item3)
            {
               data_p = (char*)(item3);
               tmpMessage.Append(data_p,datalen);
            }
            if (item4)
            {
               data_p = (char*)(item4);
               tmpMessage.Append(data_p,datalen);
            }


	    if ( chipsClient.SendMessage(-1,tmpMessage) > 0 )
		rval= ERR_STATUS_GOOD;
	      else
		rval= ERR_NO_SOCKET;

         }
         else if (msgid == MSG_ADD_IMAGE_CHANNEL)
         {
            // message may be too big for buffer- handle individually
            int* i2p = static_cast<int*>(item2);
            int* i3p = static_cast<int*>(item3);
            unsigned int datalen = (*i2p) * (*i3p) * sizeof(double);

            *lp = *lp + datalen;

            char* data_p = (char*)(item1);
            tmpMessage.Set(cp,len);
            tmpMessage.Append(data_p,datalen);
            if ( chipsClient.SendMessage(-1,tmpMessage) > 0 )  // send header info
                rval= ERR_STATUS_GOOD;
              else
                rval= ERR_NO_SOCKET;
         }
         else if ((msgid == MSG_ADD_LINE_B) || (msgid == MSG_ADD_REGION))
         {
            // message may be too big for buffer- handle individually
            int* i1p = static_cast<int*>(item3);
            unsigned int datalen = (*i1p) * sizeof(double);
            *lp = *lp + datalen * 2;

            char* data_p = (char*)(item1);
            tmpMessage.Set(cp,len);
            tmpMessage.Append(data_p,datalen); // send x data
            data_p = (char*)(item2);       // send y data
            tmpMessage.Append(data_p,datalen);

            if ( chipsClient.SendMessage(-1,tmpMessage) > 0 )
                rval= ERR_STATUS_GOOD;
              else
                rval= ERR_NO_SOCKET;

         }
         else 
         {
	   tmpMessage.Set(cp,len);
	   if ( chipsClient.SendMessage(-1,tmpMessage) > 0 )  // send header info
	     rval= ERR_STATUS_GOOD;
	   else
	     rval= ERR_NO_SOCKET;
	   
            // regular sized message- send packed buffer 
         }
   
         // get response back from server- to avoid a blocking read (in
         // case the server dies before writing to the pipe), use a loop
         // to check if data is on the pipe before trying to read from it
         int bytes = 0;
	 if (msgid != MSG_QUIT)
	   rval = get_response(cid, &bytes);
	 else
	   serverId="";
     }
   
   return (rval);
}


int ChipsClient::get_response(ChipsId* cid, int* bytes)
{
   unsigned int buff[2];            // buffer for message header (id,size)
   unsigned int msglen = 0,         // specified message length w/o header
                bytes_in_buffer;    // number of bytes in pipe
   int          rval = ERR_STATUS_GOOD;  // return status 
   char*        pos = (char*)buff;  // pointer to message buffer
   char         pbuf[PIPE_BUF];     // copy buffer for short messages
   int          overflow = 0;       // indicator for long messages

   SCSMessage tmpMessage;
   // read the message id and message length (fixed length of 8 bytes)
   buff[0] = buff[1] = 0;
   
   if (chipsClient.ReceiveMessage(-1,tmpMessage) <=0)
     return ERR_NO_SOCKET;
   else  if (tmpMessage.Type() == SCSMessage::PROTOCOL_CLOSE )
    {
      return ERR_STATUS_BAD;
    };
     

   memcpy(pos,tmpMessage.Data(),8);
   msglen = buff[1] - 8;
   bytes_in_buffer = msglen;
   
   // if a byte param passed in, pass back btyes read
   if (bytes) *bytes = buff[1];


   memcpy(pos,tmpMessage.Data(),8);
   msglen = buff[1] - 8;
   bytes_in_buffer = msglen;
   
   // if a byte param passed in, pass back btyes read
   if (bytes) *bytes = buff[1];

   // make sure the message length is appropriate for the data
   if (((msglen <= bytes_in_buffer) || (overflow = (msglen > PIPE_BUF))) &&
        (rval == ERR_STATUS_GOOD))
   {
      char *bf = pbuf;            // buffer pointer

      if (overflow)   // too large for preset buffer- allocate new buffer
      {
         bf = (char*) malloc(msglen*sizeof(char));
      }

      // load all data from pipe to our buffer
      memcpy(bf,tmpMessage.Data()+8,msglen);


      // handle message based on id
      switch(buff[0])
      {
         case 0:      // valid status- get cid
            rval = buff[0]; 
            unpack_it(bf, cid);
         break;
         default:     
            rval = buff[0]; 
         break;
      }
      if (overflow)
      {
         free(bf);   // cleanup any allocated buffer
      }
   };

   return rval; 
}

// pack message id and ChipsId object
int ChipsClient::pack_it(ChipsId* cid,  // chips id object
   int   msgid,                         // message id value
   void* item1,                         // value for parameter 1
   void* item2,                         // value for parameter 2
   void* item3,                         // value for parameter 3
   void* item4,                         // value for parameter 4
   void* item5,                         // value for parameter 5
   void* item6,                         // value for parameter 6
   void* item7,                         // value for parameter 7
   void* item8,                         // value for parameter 8
   void* item9,                         // value for parameter 9
   void* item10,                        // value for parameter 10
   void* item11)                        // value for parameter 11
{
   int *intp, *msg_sz, idlen, pos;
   char* cp = (char*)buffer; 
   string id;
   int *csys;
   unsigned int item_sz; 
   int rval = ERR_STATUS_GOOD;

   if (!cid) return ERR_NO_ID_OBJ; 

   // set message id and storage for size of message
   intp = (int*)buffer;
   *intp = msgid;
   msg_sz = ++intp;
   pos = 2*sizeof(int);

   // set window id 
   id = cid->get_window_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set frame id 
   id = cid->get_frame_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set plot id 
   id = cid->get_plot_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set image id 
   id = cid->get_image_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set coord sys id 
   id = cid->get_coord_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set curve id 
   id = cid->get_curve_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set contour id
   id = cid->get_contour_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set histogram id
   id = cid->get_histogram_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set axis id 
   id = cid->get_axis_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set xaxis id 
   id = cid->get_xaxis_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set yaxis id 
   id = cid->get_yaxis_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set lbid id 
   id = cid->get_label_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set lnid id 
   id = cid->get_line_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set ptid id 
   id = cid->get_point_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // set region id 
   id = cid->get_region_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   // get colorbar id
   id = cid->get_colorbar_id();
   if ((idlen = id.length()) != 0)
   {
      strcpy(&cp[pos], id.c_str());
      pos += idlen;
   }
   cp[pos++] = '\0';
   csys = (int*)&cp[NEXT_INT_POS(pos)];
   *csys = (int)cid->get_coord_sys ();
   pos = NEXT_INT_POS(pos) + sizeof(int);
   // axis dim
   csys = (int*)&cp[NEXT_INT_POS(pos)];
   *csys = (int)cid->get_axis_dim ();
   pos = NEXT_INT_POS(pos) + sizeof(int);

   switch(msgid)
   {
      case MSG_CURRENT_REQUEST:  break;  // handled- cid is only param
      case MSG_DEL_WINDOW:       break;  // handled- cid is only param
      case MSG_DEL_LEGEND:       break;  // handled- cid is only param
      case MSG_DEL_ALL_WINS:     break;  // handled- cid is only param
      case MSG_LOAD_PREFS_A:     break;  // handled- cid is only param
      case MSG_ADD_WINDOW_A:     break;  // handled- cid is only param
      case MSG_ICONIFY_WIN:      break;  // handled- cid is only param
      case MSG_DEICONIFY_WIN:    break;  // handled- cid is only param
      case MSG_SET_CURRENT_WIN:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_FRM:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_IMG:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_CBAR: break;  // handled- cid is only param
      case MSG_SET_CURRENT_PLT:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_CRV:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_CNTR: break;  // handled- cid is only param
      case MSG_SET_CURRENT_HST:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_AXS:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_LAB:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_LINE: break;  // handled- cid is only param
      case MSG_SET_CURRENT_PNT:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_LGD:  break;  // handled- cid is only param
      case MSG_SET_CURRENT_REGN: break;  // handled- cid is only param
      case MSG_ADD_FRAME_A:      break;  // handled- cid is only param
      case MSG_PLACED_FRAME:     break;  // handled- cid is only param
      case MSG_DEL_FRAME:        break;  // handled- cid is only param
      case MSG_DEL_ALL_FRAMES:   break;  // handled- cid is only param
      case MSG_SINGLE_FRAME:     break;  // handled- cid is only param
      case MSG_NEXT_FRAME:       break;  // handled- cid is only param
      case MSG_PREV_FRAME:       break;  // handled- cid is only param
      case MSG_STOP_BLINK:       break;  // handled- cid is only param
      case MSG_CLEAR_PLOT:       break;  // handled- cid is only param
      case MSG_DEL_PLOT:         break;  // handled- cid is only param
      case MSG_DEL_CURVE:        break;  // handled- cid is only param
      case MSG_DEL_CONTOUR:      break;  // handled- cid is only param
      case MSG_DEL_HISTOGRAM:    break;  // handled- cid is only param
      case MSG_DEL_AXIS:         break;  // handled- cid is only param
      case MSG_DEL_LINE:         break;  // handled- cid is only param
      case MSG_DEL_REGION:       break;  // handled- cid is only param
      case MSG_DEL_POINT:        break;  // handled- cid is only param
      case MSG_DEL_ALL_PLOTS:    break;  // handled- cid is only param
      case MSG_DEL_IMAGE:        break;  // handled- cid is only param
      case MSG_DEL_ALL_IMAGES:   break;  // handled- cid is only param
      case MSG_HIDE_GRID:        break;  // handled- cid is only param
      case MSG_SHOW_LAYOUT:      break;  // handled- cid is only param
      case MSG_HIDE_LAYOUT:      break;  // handled- cid is only param
      case MSG_DEL_CBAR:         break;  // handled- cid is only param
      case MSG_DEL_LABEL:        break;  // handled- cid is only param
      case MSG_SET_CV_SYM_FILL:  break;  // handled- cid is only param
      case MSG_SET_CV_SYM_UNFILL:break;  // handled- cid is only param
      case MSG_SET_HST_SYM_FILL: break;  // handled- cid is only param
      case MSG_SET_HST_SYM_UNFIL:break;  // handled- cid is only param
      case MSG_SET_WIN_DRAW_ON:  break;  // handled- cid is only param
      case MSG_SET_WIN_DRAW_OFF: break;  // handled- cid is only param
      case MSG_GET_PLOT_LMARGIN: break;  // handled- cid is only param
      case MSG_GET_PLOT_RMARGIN: break;  // handled- cid is only param
      case MSG_GET_PLOT_BMARGIN: break;  // handled- cid is only param
      case MSG_GET_PLOT_TMARGIN: break;  // handled- cid is only param
      case MSG_GET_PLOT_DEPTH:   break;  // handled- cid is only param
      case MSG_GET_AXIS_DEPTH:   break;  // handled- cid is only param
      case MSG_GET_IMG_DEPTH:    break;  // handled- cid is only param
      case MSG_GET_CBAR_LENGTH:  break;  // handled- cid is only param
      case MSG_GET_CBAR_WIDTH:   break;  // handled- cid is only param
      case MSG_GET_CBAR_DEPTH:   break;  // handled- cid is only param
      case MSG_GET_CV_DEPTH:     break;  // handled- cid is only param
      case MSG_GET_CNTR_DEPTH:   break;  // handled- cid is only param
      case MSG_GET_DEF_DEPTH:    break;  // handled- cid is only param
      case MSG_GET_HST_DEPTH:    break;  // handled- cid is only param
      case MSG_GET_WIN_TITLE:    break;  // handled- cid is only param
      case MSG_GET_WINDOW_COLOR: break;  // handled- cid is only param
      case MSG_GET_FRAME_COLOR:  break;  // handled- cid is only param
      case MSG_GET_FRAME_BCOLOR: break;  // handled- cid is only param
      case MSG_GET_FRAME_TRANS:  break;  // handled- cid is only param
      case MSG_GET_FRAME_SCALE:  break;  // handled- cid is only param
      case MSG_GET_FRAME_BVIS:   break;  // handled- cid is only param
      case MSG_GET_FRAME_VIS:    break;  // handled- cid is only param
      case MSG_GET_WIN_REDRAW:   break;  // handled- cid is only param
      case MSG_GET_CBAR_TITLE:   break;  // handled- cid is only param
      case MSG_GET_LAB_TEXT:     break;  // handled- cid is only param
      case MSG_GET_LAB_COLOR:    break;  // handled- cid is only param
      case MSG_GET_LAB_STYLE:    break;  // handled- cid is only param
      case MSG_GET_LAB_FONT:     break;  // handled- cid is only param
      case MSG_GET_LAB_SIZE:     break;  // handled- cid is only param
      case MSG_GET_LAB_DEPTH:    break;  // handled- cid is only param
      case MSG_GET_LAB_ANGLE:    break;  // handled- cid is only param
      case MSG_GET_LINE_STYLE:   break;  // handled- cid is only param
      case MSG_GET_LINE_EXTEND:  break;  // handled- cid is only param
      case MSG_GET_LINE_THICK:   break;  // handled- cid is only param
      case MSG_GET_LINE_DEPTH:   break;  // handled- cid is only param
      case MSG_GET_REGN_EDGCOL:  break;  // handled- cid is only param
      case MSG_GET_REGN_FILCOL:  break;  // handled- cid is only param
      case MSG_GET_LINE_COLOR:   break;  // handled- cid is only param
      case MSG_GET_REGN_THICK:   break;  // handled- cid is only param
      case MSG_GET_REGN_STYLE:   break;  // handled- cid is only param
      case MSG_GET_REGN_FILSTYL: break;  // handled- cid is only param
      case MSG_GET_POINT_COLOR:  break;  // handled- cid is only param
      case MSG_GET_POINT_SYMBOL: break;  // handled- cid is only param
      case MSG_GET_POINT_SIZE:   break;  // handled- cid is only param
      case MSG_GET_POINT_FILL:   break;  // handled- cid is only param
      case MSG_GET_POINT_DEPTH:  break;  // handled- cid is only param
      case MSG_GET_REGN_ANGLE:   break;  // handled- cid is only param
      case MSG_GET_REGN_DEPTH:   break;  // handled- cid is only param
      case MSG_GET_CV_LIMLENGTH: break;  // handled- cid is only param
      case MSG_GET_CV_LIMOVRIDE: break;  // handled- cid is only param
      case MSG_GET_CV_SYM_ANGLE: break;  // handled- cid is only param
      case MSG_GET_HST_SYM_ANGLE:break;  // handled- cid is only param
      case MSG_GET_HST_FILL_STYL:break;  // handled- cid is only param
      case MSG_GET_POINT_ANGLE:  break;  // handled- cid is only param
      case MSG_GET_PLOT_AX_STYLE:break;  // handled- cid is only param
      case MSG_GET_IMG_INVCMAP:  break;  // handled- cid is only param
      case MSG_GET_IMG_CMAPINTRP:break;  // handled- cid is only param
      case MSG_GET_IMG_SCL_CHNNL:break;  // handled- cid is only param
      case MSG_GET_DATA_ASPRATIO:break;  // handled- cid is only param
      case MSG_GET_PLOT_ASPRATIO:break;  // handled- cid is only param
      case MSG_GET_PLOT_ASPW:    break;  // handled- cid is only param
      case MSG_GET_PLOT_ASPH:    break;  // handled- cid is only param
      case MSG_GET_PLOT_TITLE:   break;  // handled- cid is only param
      case MSG_GET_IMG_CMAP:     break;  // handled- cid is only param
      case MSG_GET_IMG_CMAPSIZE: break;  // handled- cid is only param
      case MSG_GET_IMG_INTERP:   break;  // handled- cid is only param
      case MSG_GET_PLOT_CORNER:  break;  // handled- cid is only param
      case MSG_GET_CV_ERR_CAPLEN:break;  // handled- cid is only param
      case MSG_GET_CV_SYM_SIZE:  break;  // handled- cid is only param
      case MSG_GET_CV_SYM_FILL:  break;  // handled- cid is only param
      case MSG_GET_CV_SYM_COLOR: break;  // handled- cid is only param
      case MSG_GET_CV_ERR_COLOR: break;  // handled- cid is only param
      case MSG_GET_CV_ERR_STYLE: break;  // handled- cid is only param
      case MSG_GET_CV_ERR_UP:    break;  // handled- cid is only param
      case MSG_GET_CV_ERR_DOWN:  break;  // handled- cid is only param
      case MSG_GET_CV_ERR_LEFT:  break;  // handled- cid is only param
      case MSG_GET_CV_ERR_RIGHT: break;  // handled- cid is only param
      case MSG_GET_CV_SYMBOL:    break;  // handled- cid is only param
      case MSG_GET_CV_LINE_SIZE: break;  // handled- cid is only param
      case MSG_GET_CV_LINE_COLOR:break;  // handled- cid is only param
      case MSG_GET_CV_LINE_STYLE:break;  // handled- cid is only param
      case MSG_GET_CV_BADDATA:   break;  // handled- cid is only param
      case MSG_GET_CNTR_THICK:   break;  // handled- cid is only param
      case MSG_GET_CNTR_COLOR:   break;  // handled- cid is only param
      case MSG_GET_CNTR_STYLE:   break;  // handled- cid is only param
      case MSG_GET_CNTR_MODE:    break;  // handled- cid is only param
      case MSG_GET_CNTR_ALG:     break;  // handled- cid is only param
      case MSG_GET_HST_SYM_SIZE: break;  // handled- cid is only param
      case MSG_GET_HST_SYM_COLOR:break;  // handled- cid is only param
      case MSG_GET_HST_SYM_FILL: break;  // handled- cid is only param
      case MSG_GET_HST_ERR_CPLEN:break;  // handled- cid is only param
      case MSG_GET_CV_ERR_SIZE:  break;  // handled- cid is only param
      case MSG_GET_HST_ERR_SIZE: break;  // handled- cid is only param
      case MSG_GET_HST_ERR_COLOR:break;  // handled- cid is only param
      case MSG_GET_HST_ERR_STYLE:break;  // handled- cid is only param
      case MSG_GET_HST_SYMBOL:   break;  // handled- cid is only param
      case MSG_GET_HST_LINE_SIZE:break;  // handled- cid is only param
      case MSG_GET_HST_LINE_COLR:break;  // handled- cid is only param
      case MSG_GET_HST_LINE_STYL:break;  // handled- cid is only param
      case MSG_GET_HST_DROPLINE: break;  // handled- cid is only param
      case MSG_GET_HST_ERR_UP:   break;  // handled- cid is only param
      case MSG_GET_HST_ERR_DOWN: break;  // handled- cid is only param
      case MSG_GET_HST_FILL_COLR:break;  // handled- cid is only param
      case MSG_GET_HST_FILL_OPAC:break;  // handled- cid is only param
      case MSG_GET_TITLE_COLOR:  break;  // handled- cid is only param
      case MSG_GET_TITLE_ANGLE:  break;  // handled- cid is only param
      case MSG_GET_TITLE_X_POS:  break;  // handled- cid is only param
      case MSG_GET_TITLE_Y_POS:  break;  // handled- cid is only param
      case MSG_GET_TITLE_SIZE:   break;  // handled- cid is only param
      case MSG_GET_CBAR_LAB_FONT:break;  // handled- cid is only param
      case MSG_GET_CBAR_LAB_TXT: break;  // handled- cid is only param
      case MSG_GET_CBAR_TL_FONT: break;  // handled- cid is only param
      case MSG_GET_TITLE_FONT:   break;  // handled- cid is only param
      case MSG_GET_CBAR_LAB_FSTY:break;  // handled- cid is only param
      case MSG_GET_CBAR_TL_FSTY: break;  // handled- cid is only param
      case MSG_GET_TITLE_FNTSTYL:break;  // handled- cid is only param
      case MSG_GET_AXIS_COLOR:   break;  // handled- cid is only param
      case MSG_GET_CBAR_LAB_CLR: break;  // handled- cid is only param
      case MSG_GET_CBAR_TL_CLR:  break;  // handled- cid is only param
      case MSG_GET_AXIS_MAJ_CLR: break;  // handled- cid is only param
      case MSG_GET_AXIS_MIN_CLR: break;  // handled- cid is only param 
      case MSG_GET_AXIS_TEXT:    break;  // handled- cid is only param 
      case MSG_GET_AXIS_XLAB_TXT:break;  // handled- cid is only param 
      case MSG_GET_AXIS_YLAB_TXT:break;  // handled- cid is only param 
      case MSG_GET_TCKLB_COLOR:  break;  // handled- cid is only param 
      case MSG_GET_AXSLB_COLOR:  break;  // handled- cid is only param 
      case MSG_GET_AXSG_MJ_CLR:  break;  // handled- cid is only param 
      case MSG_GET_AXSG_MN_CLR:  break;  // handled- cid is only param 
      case MSG_GET_AXIS_TICKFMT: break;  // handled- cid is only param  
      case MSG_GET_CBAR_TK_MODE: break;  // handled- cid is only param
      case MSG_GET_AXIS_MAJ_MODE:break;  // handled- cid is only param 
      case MSG_GET_AXIS_MIN_MODE:break;  // handled- cid is only param 
      case MSG_GET_AXIS_TICKSTYL:break;  // handled- cid is only param 
      case MSG_GET_CBAR_LAB_LOC: break;  // handled- cid is only param 
      case MSG_GET_CBAR_TK_STY:  break;  // handled- cid is only param 
      case MSG_GET_CBAR_TK_LOC:  break;  // handled- cid is only param
      case MSG_GET_CBAR_TK_CLR:  break;  // handled- cid is only param
      case MSG_GET_CBAR_TK_LEN:  break;  // handled- cid is only param
      case MSG_GET_CBAR_TK_VIS:  break;  // handled- cid is only param
      case MSG_GET_AXIS_MAJ_STYL:break;  // handled- cid is only param 
      case MSG_GET_AXIS_MIN_STYL:break;  // handled- cid is only param 
      case MSG_GET_CBAR_LAB_VIS: break;  // handled- cid is only param 
      case MSG_GET_CBAR_ORIENT:  break;  // handled- cid is only param 
      case MSG_GET_CBAR_BVIS:    break;  // handled- cid is only param 
      case MSG_GET_CBAR_TL_VIS:  break;  // handled- cid is only param 
      case MSG_GET_AXIS_MAJ_VIS: break;  // handled- cid is only param 
      case MSG_GET_AXIS_MIN_VIS: break;  // handled- cid is only param 
      case MSG_GET_AXIS_AUTOMAX: break;  // handled- cid is only param 
      case MSG_GET_AXIS_AUTOMIN: break;  // handled- cid is only param 
      case MSG_GET_AXIS_VISIBLE: break;  // handled- cid is only param 
      case MSG_GET_IMG_VISIBLE:  break;  // handled- cid is only param 
      case MSG_GET_CBAR_VISIBLE: break;  // handled- cid is only param 
      case MSG_GET_CV_VISIBLE:   break;  // handled- cid is only param 
      case MSG_GET_CNTR_VISIBLE: break;  // handled- cid is only param 
      case MSG_GET_HST_VISIBLE:  break;  // handled- cid is only param 
      case MSG_GET_LAB_VISIBLE:  break;  // handled- cid is only param 
      case MSG_GET_LINE_VISIBLE: break;  // handled- cid is only param 
      case MSG_GET_POINT_VISIBLE:break;  // handled- cid is only param 
      case MSG_GET_REGN_VISIBLE: break;  // handled- cid is only param
      case MSG_GET_PLOT_VISIBLE: break;  // handled- cid is only param 
      case MSG_GET_TCKLB_FONT:   break;  // handled- cid is only param  
      case MSG_GET_TCKLB_FNTSTYL:break;  // handled- cid is only param 
      case MSG_GET_TCKLB_VIS:    break;  // handled- cid is only param 
      case MSG_GET_AXSLB_VIS:    break;  // handled- cid is only param 
      case MSG_GET_AXSLB_FONT:   break;  // handled- cid is only param  
      case MSG_GET_AXSLB_FNTSTYL:break;  // handled- cid is only param 
      case MSG_GET_AXSTCKLB_STYL:break;  // handled- cid is only param 
      case MSG_GET_AXIS_THICK:   break;  // handled- cid is only param 
      case MSG_GET_CBAR_TK_THK:  break;  // handled- cid is only param 
      case MSG_GET_AXIS_MAJ_THK: break;  // handled- cid is only param 
      case MSG_GET_AXIS_MIN_THK: break;  // handled- cid is only param 
      case MSG_GET_AXIS_MAJ_NUM: break;  // handled- cid is only param 
      case MSG_GET_AXIS_MIN_NUM: break;  // handled- cid is only param 
      case MSG_GET_AXIS_MAJ_LEN: break;  // handled- cid is only param 
      case MSG_GET_AXIS_MIN_LEN: break;  // handled- cid is only param 
      case MSG_GET_TCKLB_SIZE:   break;  // handled- cid is only param 
      case MSG_GET_TCKLB_OFFSET: break;  // handled- cid is only param 
      case MSG_GET_CBAR_LAB_SZ:  break;  // handled- cid is only param 
      case MSG_GET_CBAR_TL_SZ:   break;  // handled- cid is only param 
      case MSG_GET_CBAR_TL_OFF:  break;  // handled- cid is only param 
      case MSG_GET_CNTR_NUMLEV:  break;  // handled- cid is only param 
      case MSG_GET_SCALE:        break;  // handled- cid is only param 
      case MSG_GET_AXSLB_SIZE:   break;  // handled- cid is only param 
      case MSG_GET_AXSG_MJ_SZ:   break;  // handled- cid is only param 
      case MSG_GET_AXSG_MN_SZ:   break;  // handled- cid is only param 
      case MSG_GET_AXSG_MJ_STL:  break;  // handled- cid is only param 
      case MSG_GET_AXSG_MN_STL:  break;  // handled- cid is only param 
      case MSG_GET_AXSG_MJ_VIS:  break;  // handled- cid is only param 
      case MSG_GET_AXSG_MN_VIS:  break;  // handled- cid is only param 
      case MSG_GET_AXIS_PAR_OFF: break;  // handled- cid is only param 
      case MSG_GET_AXIS_PERP_OFF:break;  // handled- cid is only param 
      case MSG_GET_CBAR_PAR_OFF: break;  // handled- cid is only param
      case MSG_GET_CBAR_PERP_OFF:break;  // handled- cid is only param
      case MSG_GET_AXIS_XLAB_ANG:break;  // handled- cid is only param 
      case MSG_GET_AXIS_YLAB_ANG:break;  // handled- cid is only param 
      case MSG_GET_AXSLB_ANGLE:  break;  // handled- cid is only param 
      case MSG_GET_TCKLB_ANGLE:  break;  // handled- cid is only param 
      case MSG_GET_AXIS_PAD:     break;  // handled- cid is only param
      case MSG_GET_REGN_OPACITY: break;  // handled- cid is only param
      case MSG_GET_AXIS_MAJ_INT: break;  // handled- cid is only param
      case MSG_GET_AXIS_MIN_INT: break;  // handled- cid is only param
      case MSG_GET_CNTR_INT:     break;  // handled- cid is only param
      case MSG_GET_CBAR_LAB_ANG: break;  // handled- cid is only param
      case MSG_GET_CBAR_LAB_HAL: break;  // handled- cid is only param
      case MSG_GET_CBAR_LAB_VAL: break;  // handled- cid is only param
      case MSG_GET_CBAR_TL_ANG:  break;  // handled- cid is only param
      case MSG_GET_CBAR_TL_HAL:  break;  // handled- cid is only param
      case MSG_GET_CBAR_HALIGN:  break;  // handled- cid is only param
      case MSG_GET_CBAR_VALIGN:  break;  // handled- cid is only param
      case MSG_GET_CBAR_TL_VAL:  break;  // handled- cid is only param
      case MSG_GET_LAB_HALIGN:   break;  // handled- cid is only param
      case MSG_GET_LAB_VALIGN:   break;  // handled- cid is only param
      case MSG_GET_TCKLB_HALIGN: break;  // handled- cid is only param
      case MSG_GET_TCKLB_VALIGN: break;  // handled- cid is only param
      case MSG_GET_AXSLB_HALIGN: break;  // handled- cid is only param
      case MSG_GET_AXSLB_VALIGN: break;  // handled- cid is only param
      case MSG_GET_TITLE_HALIGN: break;  // handled- cid is only param
      case MSG_GET_TITLE_VALIGN: break;  // handled- cid is only param
      case MSG_INFO_BOUND_REQUEST:break; // handled- cid is only param
      case MSG_INFO_COORD_REQUEST:break; // handled- cid is only param
      case MSG_GET_HST_RANGE:    break;  // handled- cid is only param
      case MSG_GET_CNTR_RANGE:   break;  // handled- cid is only param
      case MSG_GET_IMG_RANGE:    break;  // handled- cid is only param
      case MSG_GET_CV_RANGE:     break;  // handled- cid is only param
      case MSG_GET_PLOT_RANGE:   break;  // handled- cid is only param
      case MSG_GET_IMG_ALPHA:    break;  // handled- cid is only param
      case MSG_GET_IMG_THRESH:   break;  // handled- cid is only param
      case MSG_GET_PLOT_XRANGE:  break;  // handled- cid is only param
      case MSG_GET_PLOT_YRANGE:  break;  // handled- cid is only param
      case MSG_GET_AXIS_RANGE:   break;  // handled- cid is only param
      case MSG_GET_CV_XRANGE:    break;  // handled- cid is only param
      case MSG_GET_CV_YRANGE:    break;  // handled- cid is only param
      case MSG_GET_HST_XRANGE:   break;  // handled- cid is only param
      case MSG_GET_HST_YRANGE:   break;  // handled- cid is only param
      case MSG_GET_IMG_XRANGE:   break;  // handled- cid is only param
      case MSG_GET_IMG_YRANGE:   break;  // handled- cid is only param
      case MSG_GET_CNTR_XRANGE:  break;  // handled- cid is only param
      case MSG_GET_CNTR_YRANGE:  break;  // handled- cid is only param
      case MSG_GET_CNTR_ZRANGE:  break;  // handled- cid is only param
      case MSG_INVERT_TRANS:     break;  // handled- cid is only param
      case MSG_GET_SERVER_ID:    break;  // handled- cid is only param 
      case MSG_CLEAR_ERRS:       break;  // handled- cid is only param 
      case MSG_LOCK:             break;  // handled- cid is only param 
      case MSG_UNLOCK:           break;  // handled- cid is only param 
      case MSG_QUIT:             break;  // handled- cid is only param 
      case MSG_OPEN_UNDO_BLK:    break;  // handled- cid is only param
      case MSG_CLOSE_UNDO_BLK:   break;  // handled- cid is only param
      case MSG_OPEN_UNDO_BUFF:   break;  // handled- cid is only param
      case MSG_CLOSE_UNDO_BUFF:  break;  // handled- cid is only param
      case MSG_CLEAR_UNDO_BUFF:  break;  // handled- cid is only param
      case MSG_GET_ERR_VERB:     break;  // handled- cid is only param
      case MSG_GET_ERR_TYPES:    break;  // handled- cid is only param
      case MSG_GET_CNTR_LEVELS:  break;  // handled- cid is only param
      case MSG_HIDE_GUI:         break;  // handled- cid is only param
      case MSG_GET_CWD:          break;  // handled- cid is only param
      case MSG_SET_WIN_TITLE:     // fallthrough intended
      case MSG_SET_CWD:           // fallthrough intended
      case MSG_LOAD_PREFS_B:      // fallthrough intended
      case MSG_ADD_WINDOW_B:      // fallthrough intended
      case MSG_ADD_FRAME_B:       // fallthrough intended
      case MSG_SET_DATA_ASPRATIO: // fallthrough intended 
      case MSG_SET_PLOT_ASPRATIO: // fallthrough intended 
      case MSG_SET_PLOT_TITLE:    // fallthrough intended 
      case MSG_SET_PLOT_XLABEL:   // fallthrough intended 
      case MSG_SET_PLOT_YLABEL:   // fallthrough intended 
      case MSG_SET_WINDOW_COLOR:  // fallthrough intended 
      case MSG_SET_FRAME_COLOR:   // fallthrough intended 
      case MSG_SET_FRAME_BCOLOR:  // fallthrough intended 
      case MSG_SET_GRID_COLOR:    // fallthrough intended 
      case MSG_SET_LAYOUT_COLOR:  // fallthrough intended
      case MSG_SET_TITLE_COLOR:   // fallthrough intended
      case MSG_SET_CBAR_LAB_FONT: // fallthrough intended
      case MSG_SET_CBAR_TL_FONT:  // fallthrough intended
      case MSG_SET_TITLE_FONT:    // fallthrough intended
      case MSG_SET_LAB_FONT:      // fallthrough intended
      case MSG_SET_LAB_STYLE:     // fallthrough intended
      case MSG_SET_CBAR_LAB_FSTY: // fallthrough intended
      case MSG_SET_CBAR_TL_FSTY:  // fallthrough intended
      case MSG_SET_TITLE_FNTSTYL: // fallthrough intended
      case MSG_SET_TCKLB_COLOR:   // fallthrough intended
      case MSG_SET_AXSLB_COLOR:   // fallthrough intended
      case MSG_SET_TCKLB_FONT:    // fallthrough intended
      case MSG_SET_TCKLB_FNTSTYL: // fallthrough intended
      case MSG_SET_AXSLB_FONT:    // fallthrough intended
      case MSG_SET_AXSLB_FNTSTYL: // fallthrough intended
      case MSG_SET_AXIS_COLOR:    // fallthrough intended
      case MSG_SET_CBAR_TK_MODE:  // fallthrough intended
      case MSG_SET_AXIS_MAJ_MODE: // fallthrough intended
      case MSG_SET_AXIS_MIN_MODE: // fallthrough intended
      case MSG_SET_CBAR_LAB_CLR:  // fallthrough intended
      case MSG_SET_CBAR_TL_CLR:   // fallthrough intended
      case MSG_SET_AXIS_MAJ_CLR:  // fallthrough intended
      case MSG_SET_AXIS_MIN_CLR:  // fallthrough intended
      case MSG_SET_AXIS_TICKFMT:  // fallthrough intended
      case MSG_SET_AXIS_TEXT:     // fallthrough intended
      case MSG_SET_AXIS_XLAB_TXT: // fallthrough intended
      case MSG_SET_AXIS_YLAB_TXT: // fallthrough intended
      case MSG_SET_LINE_COLOR:    // fallthrough intended
      case MSG_SET_POINT_COLOR:   // fallthrough intended
      case MSG_SET_REGN_EDGCOL:   // fallthrough intended
      case MSG_SET_REGN_FILCOL:   // fallthrough intended
      case MSG_SET_LEGEND_COLOR:  // fallthrough intended 
      case MSG_SET_CV_SYM_COLOR:  // fallthrough intended
      case MSG_SET_CV_ERR_STYLE:  // fallthrough intended 
      case MSG_SET_CV_ERR_COLOR:  // fallthrough intended
      case MSG_SET_CV_LINE_COLOR: // fallthrough intended
      case MSG_SET_HST_SYM_COLOR: // fallthrough intended
      case MSG_SET_HST_ERR_STYLE: // fallthrough intended 
      case MSG_SET_HST_ERR_COLOR: // fallthrough intended
      case MSG_SET_HST_LINE_COLR: // fallthrough intended
      case MSG_SET_HST_FILL_COLR: // fallthrough intended
      case MSG_SET_AXSG_MJ_CLR:   // fallthrough intended
      case MSG_SET_AXSG_MN_CLR:   // fallthrough intended
      case MSG_SET_CBAR_TK_CLR:   // fallthrough intended
      case MSG_SET_CBAR_LAB_TXT:  // fallthrough intended
      case MSG_SET_CBAR_TITLE:    // fallthrough intended 
      case MSG_SET_LAB_TEXT:      // fallthrough intended 
      case MSG_SET_LAB_COLOR:     // fallthrough intended 
      case MSG_PRINT_FRAME:       // fallthrough intended
      case MSG_ADD_PLOT:          // fallthrough intended 
      case MSG_GET_PREFERENCE:    // fallthrough intended
      case MSG_SET_CNTR_MODE:     // fallthrough intended
      case MSG_SET_CNTR_COLOR:    // fallthrough intended
      case MSG_SET_CNTR_STYLE:    // fallthrough intended
      case MSG_UNDO_TAG:          // fallthrough intended
      case MSG_REDO_TAG:          // fallthrough intended
      case MSG_SET_UNDO_TAG:      // fallthrough intended
      case MSG_CLEAR_UNDO_TAG:    // fallthrough intended
      case MSG_LOAD_STATE:        // fallthrough intended
      case MSG_CMDLNE_CONN:       // fallthrough intended
      case MSG_SHOW_GUI:
      if (item1) 
      {
         char *cval = (char*)item1;

         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz; 
         }
      }
      cp[pos++] = '\0';
      break; 
      case MSG_SAVE_PREFS:        // fallthrough intended
      case MSG_SAVE_STATE:        
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival = static_cast<int*>(item2);
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         if (item1)
         {
            char *cval = (char*)item1;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_SAVE_STATE_B:
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival = static_cast<int*>(item2);
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         ip = (int*)&cp[NEXT_INT_POS(pos)];
         ival = static_cast<int*>(item3);
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         if (item1)
         {
            char *cval = (char*)item1;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_GET_PICK:
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival = static_cast<int*>(item1);    // store  count 
         *ip = *ival; 
         pos = NEXT_INT_POS(pos) + sizeof(int);
         unsigned int* up = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         unsigned int* uival = static_cast<unsigned int*>(item2); 
         *up = *uival;    // store col count
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);
      }
      break; 
      case MSG_PRINT_WINDOW:      // fallthrough intended
      case MSG_PRINT_IMAGE:       // fallthrough intended
      case MSG_LOAD_FONT:
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival = static_cast<int*>(item3);   
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         char *cval1 = (char*)item1;
         char *cval2 = (char*)item2;
         if ((item_sz = strlen(cval1)) != 0)
         {
            strcpy(&cp[pos], cval1);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         if ((item_sz = strlen(cval2)) != 0)
         {
            strcpy(&cp[pos], cval2);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break; 
      case MSG_AXIS_UNBIND:       // fallthrough intended
      case MSG_AXIS_BIND:
      {
         char *cval1 = (char*)item1;
         char *cval2 = (char*)item2;
         if ((item_sz = strlen(cval1)) != 0)
         {
            strcpy(&cp[pos], cval1);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         if ((item_sz = strlen(cval2)) != 0)
         {
            strcpy(&cp[pos], cval2);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break; 
      case MSG_SET_LAB_VALIGN:    // fallthrough intended
      case MSG_SET_CBAR_LENGTH:   // fallthrough intended
      case MSG_SET_CBAR_WIDTH:    // fallthrough intended
      case MSG_SET_CBAR_LAB_ANG:  // fallthrough intended
      case MSG_SET_CBAR_LAB_HAL:  // fallthrough intended
      case MSG_SET_CBAR_LAB_VAL:  // fallthrough intended
      case MSG_SET_CBAR_TL_ANG:   // fallthrough intended
      case MSG_SET_CBAR_TL_HAL:   // fallthrough intended
      case MSG_SET_CBAR_TL_VAL:   // fallthrough intended
      case MSG_SET_CBAR_HALIGN:   // fallthrough intended
      case MSG_SET_CBAR_VALIGN:   // fallthrough intended
      case MSG_SET_LAB_HALIGN:    // fallthrough intended
      case MSG_SET_AXSLB_VALIGN:  // fallthrough intended
      case MSG_SET_AXSLB_HALIGN:  // fallthrough intended
      case MSG_SET_TCKLB_VALIGN:  // fallthrough intended
      case MSG_SET_TCKLB_HALIGN:  // fallthrough intended
      case MSG_SET_TITLE_VALIGN:  // fallthrough intended
      case MSG_SET_TITLE_HALIGN:  // fallthrough intended
      case MSG_SET_PLOT_LMARGIN:  // fallthrough intended
      case MSG_SET_PLOT_RMARGIN:  // fallthrough intended
      case MSG_SET_PLOT_TMARGIN:  // fallthrough intended
      case MSG_SET_PLOT_BMARGIN:  // fallthrough intended
      case MSG_SET_AXIS_PAR_OFF:  // fallthrough intended
      case MSG_SET_CBAR_PAR_OFF:  // fallthrough intended
      case MSG_SET_AXIS_PERP_OFF: // fallthrough intended
      case MSG_SET_CBAR_PERP_OFF: // fallthrough intended
      case MSG_SET_TITLE_X_POS:   // fallthrough intended
      case MSG_SET_TITLE_Y_POS:   // fallthrough intended
      case MSG_SET_TITLE_ANGLE:   // fallthrough intended
      case MSG_SET_AXIS_XLAB_ANG: // fallthrough intended
      case MSG_SET_AXIS_YLAB_ANG: // fallthrough intended
      case MSG_SET_AXSLB_ANGLE:   // fallthrough intended
      case MSG_SET_TCKLB_ANGLE:   // fallthrough intended
      case MSG_SET_LAB_ANGLE:     // fallthrough intended
      case MSG_SET_REGN_ANGLE:    // fallthrough intended
      case MSG_SET_CV_SYM_ANGLE:  // fallthrough intended
      case MSG_SET_HST_SYM_ANGLE: // fallthrough intended
      case MSG_SET_LINE_THICK:    // fallthrough intended 
      case MSG_SET_CV_LINE_SIZE:  // fallthrough intended 
      case MSG_SET_HST_LINE_SIZE: // fallthrough intended
      case MSG_SET_CV_ERR_SIZE:   // fallthrough intended 
      case MSG_SET_HST_ERR_SIZE:  // fallthrough intended
      case MSG_SET_CBAR_TK_THK:   // fallthrough intended 
      case MSG_SET_AXIS_MAJ_THK:  // fallthrough intended 
      case MSG_SET_AXIS_MIN_THK:  // fallthrough intended 
      case MSG_SET_REGN_THICK:    // fallthrough intended
      case MSG_SET_GRID_THICK:    // fallthrough intended 
      case MSG_SET_LAYOUT_THICK:  // fallthrough intended 
      case MSG_SET_AXIS_THICK:    // fallthrough intended 
      case MSG_SET_LEGEND_THICK:  // fallthrough intended 
      case MSG_SET_CNTR_THICK:    // fallthrough intended 
      case MSG_SET_AXSG_MJ_SZ:    // fallthrough intended 
      case MSG_SET_AXSG_MN_SZ:    // fallthrough intended 
      case MSG_SET_POINT_ANGLE:   // fallthrough intended
      case MSG_GRID_XGAP:         // fallthrough intended 
      case MSG_GRID_YGAP:         // fallthrough intended
      case MSG_ZOOM_PLOT:
      if (item1) 
      {
         float* fp = (float*)&cp[NEXT_FLOAT_POS(pos)];
         float* fval = static_cast<float*>(item1);
         *fp = *fval;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float); 
      }
      break;
      case MSG_SET_REGN_OPACITY:  // fallthrough intended
      case MSG_SET_AXIS_PAD:      // fallthrough intended
      case MSG_SET_AXIS_MAJ_INT:  // fallthrough intended
      case MSG_SET_AXIS_MIN_INT:  // fallthrough intended 
      case MSG_SET_CV_LIMLENGTH:  // fallthrough intended 
      case MSG_SET_HST_FILL_OPAC: // fallthrough intended
      case MSG_SET_CNTR_INT: 
      if (item1)
      {
         double* dp = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dval = static_cast<double*>(item1);
         *dp = *dval;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
      }
      break;
      case MSG_SET_IMG_ALPHA:      // fallthrough intended
      case MSG_SET_IMG_THRESH:
      {
         double *dp2, *dp = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dmin = static_cast<double*>(item1);
         double* dmax = static_cast<double*>(item2);
         *dp = *dmin;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         dp2 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *dp2 = *dmax;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
      }
      break;
      case MSG_GRID_GAP:
      {
         float *fp2, *fp = (float*)&cp[NEXT_FLOAT_POS(pos)];
         float* fmin = static_cast<float*>(item1);
         float* fmax = static_cast<float*>(item2);
         *fp = *fmin;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         fp2 = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *fp2 = *fmax;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
      }
      break;

      case MSG_GRID_OBJS:
      {
         float *fp2, *fp = (float*)&cp[NEXT_FLOAT_POS(pos)];
         float* fmin = static_cast<float*>(item3);
         float* fmax = static_cast<float*>(item4);
         int lct; 
         *fp = *fmin;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         fp2 = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *fp2 = *fmax;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);

         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival = static_cast<int*>(item1);    // store col count
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         ip = (int*)&cp[NEXT_INT_POS(pos)];
         ival = static_cast<int*>(item2);
         *ip = *ival;    // store row count
         pos = NEXT_INT_POS(pos) + sizeof(int);

         ip = (int*)&cp[NEXT_INT_POS(pos)];
         ival = static_cast<int*>(item5);
         *ip = *ival;    // store usage 
         pos = NEXT_INT_POS(pos) + sizeof(int);

         ip = (int*)&cp[NEXT_INT_POS(pos)];
         ival = static_cast<int*>(item6);
         lct = *ival;
         if (lct < 0) lct = 0; 
         *ip = lct;     // store plot count 
         pos = NEXT_INT_POS(pos) + sizeof(int);
         ip = (int*)&cp[NEXT_INT_POS(pos)];

         if (lct && item7)
         {
            string* sval1 = (string*)item7;
            int rr;

            for (rr = 0; rr < lct; rr++)
            {
               strcpy(&cp[pos], sval1[rr].c_str());
               pos += strlen(&cp[pos]);
               cp[pos++] = '\0';
            }
         }
      }
      break;
      case MSG_GRID_OBJS_B:
      {
         float *fp2, *fp = (float*)&cp[NEXT_FLOAT_POS(pos)];
         float* fmin = static_cast<float*>(item4);
         float* fmax = static_cast<float*>(item5);
         int lct; 
         *fp = *fmin;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         fp2 = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *fp2 = *fmax;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);

         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival = static_cast<int*>(item1);    // store orient
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         ip = (int*)&cp[NEXT_INT_POS(pos)];
         ival =  static_cast<int*>(item2);
         lct = *ival;
         if (lct < 0)  lct = 0;
	 *ip = lct;           // store val count
         pos = NEXT_INT_POS(pos) + sizeof(int);

         ip = (int*)&cp[NEXT_INT_POS(pos)];
         ival =  static_cast<int*>(item6);  // store usage 
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         ip = (int*)&cp[NEXT_INT_POS(pos)];
         if (lct && item3)
         {
            int rr; 
            for (rr = 0; rr < lct; rr++)
            {
               ip = (int*)&cp[NEXT_INT_POS(pos)];
               *ip = ((int*)item3)[rr];    // store array vals 
               pos = NEXT_INT_POS(pos) + sizeof(int);
            } 
         }

         ip = (int*)&cp[NEXT_INT_POS(pos)];
         ival =  static_cast<int*>(item7);
         lct = *ival;
         if (lct < 0) lct = 0; 
         *ip = lct;           // store plot count
         pos = NEXT_INT_POS(pos) + sizeof(int);
         ip = (int*)&cp[NEXT_INT_POS(pos)];
 
         if (lct && item8)
         {
            string* sval1 = (string*)item8;
            int rr;

            for (rr = 0; rr < lct; rr++)
            {
               strcpy(&cp[pos], sval1[rr].c_str());
               pos += strlen(&cp[pos]);
               cp[pos++] = '\0';
            }
         }
      }
      break;
      case MSG_SWAP_OBJ_POS:
      if (item1)
      {
         char *cval = (char*)item1;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_GRID_XRELSIZES:    // fallthrough intended
      case MSG_GRID_YRELSIZES:
      if (item1 && item2)
      {
         int rr;
         int* cnt = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *cnt = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         float* ps = (float*)&cp[NEXT_FLOAT_POS(pos)];
         float* dval = static_cast<float*>(item2);

         for (rr = 0; rr < *cnt; rr++)
         {
            *ps = dval[rr];
             pos = NEXT_FLOAT_POS(pos) + sizeof(float);
             ps = (float*)&cp[NEXT_FLOAT_POS(pos)];
         }
      }
      break;
      case MSG_GRID_XRELSIZE:     // fallthrough intended
      case MSG_GRID_YRELSIZE:
      if (item1 && item2)
      {
         float *fp = (float*)&cp[NEXT_FLOAT_POS(pos)];
         float* ratio = static_cast<float*>(item2);
         *fp = *ratio;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         int* col = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *col = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_COLLAPSE_DEPTH:
      {
         int cnt; 
         int *ip = (int*)&cp[NEXT_INT_POS(pos)];
         int *ival = static_cast<int*>(item1);    // store dest
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         ip = (int*)&cp[NEXT_INT_POS(pos)];      // store src count
         ival =  static_cast<int*>(item2);
	 *ip = *ival;
         cnt = *ival; 
         pos = NEXT_INT_POS(pos) + sizeof(int);
         ip = (int*)&cp[NEXT_INT_POS(pos)];      // store src count
         int* ia = static_cast<int*>(item3);
         if (cnt && item3)
         {
            for (int rr = 0; rr < cnt; rr++)
            {
               *ip = ia[rr];
               pos = NEXT_INT_POS(pos) + sizeof(int);
	       ip = (int*)&cp[NEXT_INT_POS(pos)];
            }
         }
      }
      break;
      case MSG_PANTO_PLOT:        // fallthrough intended
      case MSG_SET_LIMITS_XY:     // fallthrough intended
      case MSG_SET_LIMITS_Y:      // fallthrough intended
      case MSG_SET_LIMITS_X:
      if (item1 && item2)
      {
         double *dp2, *dp = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dmin = static_cast<double*>(item1);
         double* dmax = static_cast<double*>(item2);
         *dp = *dmin;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         dp2 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *dp2 = *dmax;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
      }
      break; 
      case MSG_SET_LIMITS_XX_YY:
      if (item1 && item2 && item3 && item4)
      {
         double *dpx2, *dpx;
         double *dpy2, *dpy;
         double* dxmin = static_cast<double*>(item1);
         double* dxmax = static_cast<double*>(item2);
         double* dymin = static_cast<double*>(item3);
         double* dymax = static_cast<double*>(item4);

         dpx = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *dpx = *dxmin;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

         dpx2 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *dpx2 = *dxmax;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

         dpy = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *dpy = *dymin;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

         dpy2 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *dpy2 = *dymax;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
      }
      break;
      case MSG_ZOOM_PLOT_GUI:     // fallthrough intended
      case MSG_PAN_PLOT:          // fallthrough intended
      case MSG_SET_LIMITS_XY_AA:  // fallthrough intended
      case MSG_SET_LIMITS_Y_AA:   // fallthrough intended
      case MSG_SET_LIMITS_X_AA:
         // do nothing
      break; 
      case MSG_SET_LIMITS_XY_MA:  // fallthrough intended
      case MSG_SET_LIMITS_Y_MA:   // fallthrough intended
      case MSG_SET_LIMITS_X_MA:
      {
         double *dp = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dmin = static_cast<double*>(item1);
         *dp = *dmin;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
      }
      break;
      case MSG_SET_LIMITS_XY_AM:  // fallthrough intended
      case MSG_SET_LIMITS_Y_AM:   // fallthrough intended
      case MSG_SET_LIMITS_X_AM:
      {
         double *dp = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dmin = static_cast<double*>(item1);
         *dp = *dmin;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
      }
      break;
      case MSG_SET_LIMITS_OID:
      {
         int* dim = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *dim = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         int* obj = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival2 =  static_cast<int*>(item2);
         *obj = *ival2;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         char *cval = (char*)item3;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_SET_LIMITS_CID:
      {
         int* dim = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *dim = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         int* obj = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival2 =  static_cast<int*>(item2);
         *obj = *ival2;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         char *cval = (char*)item3;  // window id
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item4;  // frame id
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item5;  // plot id
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item6;  // object id
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_SET_PREFERENCE:
      if (item1 && item2)
      {
         char *cval = (char*)item1;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item2;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_MOVE:
      {
         int* obj = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *obj = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         double* dp = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dval = static_cast<double*>(item2);
         *dp = *dval;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         double* dp2 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dval2 = static_cast<double*>(item3);
         *dp2 = *dval2;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival2 = static_cast<int*>(item4);
         *ip = *ival2;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_REARRANGE_DEPTH:
      {
         int* obj = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *obj = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival2 = static_cast<int*>(item2);
         *ip = *ival2;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_HILITE_OBJ:
      {
         int* obj = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *obj = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_SET_CASCADE:
      {
         int* cnt = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *cnt = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         char *cval = (char*)item2;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item3;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_GET_OBJ_COUNT:
      {
         int* cnt = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *cnt = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_GET_OBJ_COORD:
      {
         int* cnt = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *cnt = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_SET_LINE_STIPPLE:
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* val =  static_cast<int*>(item1);
         *ip = *val;    
         pos = NEXT_INT_POS(pos) + sizeof(int);
         ip = (int*)&cp[NEXT_INT_POS(pos)];
         val =  static_cast<int*>(item2);
         *ip = *val;   
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_GET_LINE_STIPPLE:
      {
         int* cnt = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *cnt = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_GET_ERRS:        // fallthrough intended
      case MSG_GET_ERR_CNT:
      {
         int* cnt = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *cnt = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_SHOW_GRID:         // fallthrough intended 
      case MSG_SET_CNTR_NUMLEV:   // fallthrough intended 
      case MSG_SET_CBAR_LAB_SZ:   // fallthrough intended 
      case MSG_SET_CBAR_TL_SZ:    // fallthrough intended 
      case MSG_SET_CBAR_TL_OFF:   // fallthrough intended 
      case MSG_SET_TCKLB_OFFSET:  // fallthrough intended 
      case MSG_SET_TCKLB_SIZE:    // fallthrough intended 
      case MSG_SET_AXSLB_SIZE:    // fallthrough intended 
      case MSG_SET_TITLE_SIZE:    // fallthrough intended 
      case MSG_SET_LAB_SIZE:      // fallthrough intended 
      case MSG_SET_LAB_DEPTH:     // fallthrough intended 
      case MSG_SET_LAB_VISIBLE:   // fallthrough intended 
      case MSG_SET_LINE_VISIBLE:  // fallthrough intended
      case MSG_SET_POINT_VISIBLE: // fallthrough intended
      case MSG_SET_AXIS_VISIBLE:  // fallthrough intended 
      case MSG_SET_CV_VISIBLE:    // fallthrough intended 
      case MSG_SET_CBAR_VISIBLE:  // fallthrough intended 
      case MSG_SET_IMG_VISIBLE:   // fallthrough intended 
      case MSG_SET_CNTR_VISIBLE:  // fallthrough intended 
      case MSG_SET_HST_VISIBLE:   // fallthrough intended 
      case MSG_SET_LEGEND_VIS:    // fallthrough intended 
      case MSG_SET_REGN_VISIBLE:  // fallthrough intended
      case MSG_SET_PLOT_VISIBLE:  // fallthrough intended 
      case MSG_SET_PLOT_DEPTH:    // fallthrough intended 
      case MSG_SET_IMG_DEPTH:     // fallthrough intended 
      case MSG_SET_CBAR_DEPTH:    // fallthrough intended 
      case MSG_SET_CV_DEPTH:      // fallthrough intended 
      case MSG_SET_CNTR_DEPTH:    // fallthrough intended 
      case MSG_SET_DEF_DEPTH:     // fallthrough intended 
      case MSG_SET_HST_DEPTH:     // fallthrough intended 
      case MSG_SET_AXIS_DEPTH:    // fallthrough intended 
      case MSG_SET_LINE_DEPTH:    // fallthrough intended 
      case MSG_SET_FRAME_VIS:     // fallthrough intended 
      case MSG_SET_FRAME_BVIS:    // fallthrough intended 
      case MSG_SET_CBAR_ORIENT:   // fallthrough intended 
      case MSG_SET_CBAR_BVIS:     // fallthrough intended 
      case MSG_SET_CBAR_TL_VIS:   // fallthrough intended 
      case MSG_SET_CBAR_TK_VIS:   // fallthrough intended
      case MSG_SET_AXIS_MAJ_VIS:  // fallthrough intended 
      case MSG_SET_AXIS_MIN_VIS:  // fallthrough intended 
      case MSG_SET_GRID_SCALE:    // fallthrough intended 
      case MSG_SET_REGN_DEPTH:    // fallthrough intended 
      case MSG_SET_REGN_STYLE:    // fallthrough intended
      case MSG_SET_REGN_FILSTYL:  // fallthrough intended
      case MSG_SET_GRID_STYLE:    // fallthrough intended 
      case MSG_SET_AXSG_MJ_STL:   // fallthrough intended 
      case MSG_SET_AXSG_MN_STL:   // fallthrough intended 
      case MSG_SET_POINT_SIZE:    // fallthrough intended 
      case MSG_SET_POINT_SYMBOL:  // fallthrough intended 
      case MSG_SET_POINT_DEPTH:   // fallthrough intended 
      case MSG_SET_LAYOUT_STYLE:  // fallthrough intended 
      case MSG_SET_AXIS_TICKSTYL: // fallthrough intended 
      case MSG_SET_AXSTCKLB_STYL: // fallthrough intended
      case MSG_SET_LINE_STYLE:    // fallthrough intended 
      case MSG_SET_LINE_EXTEND:   // fallthrough intended
      case MSG_SET_CV_SYMBOL:     // fallthrough intended 
      case MSG_SET_CV_BADDATA:    // fallthrough intended
      case MSG_SET_CV_LINE_STYLE: // fallthrough intended 
      case MSG_SET_CV_ERR_CAPLEN: // fallthrough intended 
      case MSG_SET_CV_SYM_SIZE:   // fallthrough intended 
      case MSG_SET_CNTR_ALG:      // fallthrough intended
      case MSG_SET_HST_SYMBOL:    // fallthrough intended
      case MSG_SET_HST_LINE_STYL: // fallthrough intended
      case MSG_SET_HST_FILL_STYL: // fallthrough intended
      case MSG_SET_HST_SYM_SIZE:  // fallthrough intended
      case MSG_SET_HST_ERR_CPLEN: // fallthrough intended 
      case MSG_SET_FRAMEMODE:     // fallthrough intended
      case MSG_SET_IMG_CMAP:      // fallthrough intended 
      case MSG_SET_IMG_CMAPSIZE:  // fallthrough intended
      case MSG_SET_IMG_INTERP:    // fallthrough intended 
      case MSG_SET_PLOT_CORNER:   // fallthrough intended 
      case MSG_PICK_LIMITS:       // fallthrough intended
      case MSG_SET_LOGSCALE:      // fallthrough intended
      case MSG_SET_LINEARSCALE:   // fallthrough intended
      case MSG_SET_CBAR_LAB_LOC:  // fallthrough intended
      case MSG_SET_CBAR_TK_STY:   // fallthrough intended
      case MSG_SET_CBAR_TK_LOC:   // fallthrough intended
      case MSG_SET_CBAR_TK_LEN:   // fallthrough intended
      case MSG_SET_AXIS_MAJ_STYL: // fallthrough intended
      case MSG_SET_AXIS_MIN_STYL: // fallthrough intended
      case MSG_SET_PLOT_AX_STYLE: // fallthrough intended
      case MSG_SET_REDRAW:        // fallthrough intended
      case MSG_REVERSE_AXIS:      // fallthrough intended
      case MSG_DISPLAY_DEPTH:     // fallthrough intended
      case MSG_HIDE_DEPTH:        // fallthrough intended
      case MSG_INFO_DEPTH_REQUEST:// fallthrough intended
      case MSG_SET_ERR_VERB:      // fallthrough intended
      case MSG_UNDO:              // fallthrough intended
      case MSG_REDO:              
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* val =  static_cast<int*>(item1);
         *ip = *val;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break; 
      case MSG_INFO_REQUEST:
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* val =  static_cast<int*>(item1);
         *ip = *val;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         ip = (int*)&cp[NEXT_INT_POS(pos)];
         val =  static_cast<int*>(item2);
         *ip = *val;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_SET_AXSG_MJ_VIS:   // fallthrough intended
      case MSG_SET_AXSG_MN_VIS:   // fallthrough intended
      case MSG_SET_AXIS_AUTOMIN:  // fallthrough intended
      case MSG_SET_AXIS_AUTOMAX:  // fallthrough intended
      case MSG_SET_AXIS_SCALABLE: // fallthrough intended 
      case MSG_SET_CV_ERR_UP:     // fallthrough intended 
      case MSG_SET_CV_ERR_DOWN:   // fallthrough intended 
      case MSG_SET_CV_ERR_LEFT:   // fallthrough intended 
      case MSG_SET_CV_ERR_RIGHT:  // fallthrough intended 
      case MSG_SET_CV_LIMOVRIDE:  // fallthrough intended
      case MSG_SET_FRAME_SCALE:   // fallthrough intended 
      case MSG_SET_FRAME_TRANS:   // fallthrough intended 
      case MSG_SET_HST_DROPLINE:  // fallthrough intended
      case MSG_SET_HST_ERR_UP:    // fallthrough intended
      case MSG_SET_HST_ERR_DOWN:  // fallthrough intended
      case MSG_SET_LEGEND_TRANS:  // fallthrough intended 
      case MSG_SET_LEGEND_BORDER: // fallthrough intended 
      case MSG_SET_POINT_FILL:    // fallthrough intended 
      case MSG_SET_AXSLB_VIS:     // fallthrough intended 
      case MSG_SET_CBAR_LAB_VIS:  // fallthrough intended
      case MSG_SET_IMG_INVCMAP:   // fallthrough intended
      case MSG_SET_IMG_CMAPINTRP: // fallthrough intended
      case MSG_SET_IMG_SCL_CHNNL: // fallthrough intended
      case MSG_SET_TCKLB_VIS:     
      {
         bool* bp = (bool*)&cp[NEXT_BOOL_POS(pos)];
         bool* bval =  static_cast<bool*>(item1);
         *bp = *bval;
         pos = NEXT_BOOL_POS(pos) + sizeof(bool);
      }
      break; 
      case MSG_ROW_TILED_FRAME:   // fallthrough intended
      case MSG_COL_TILED_FRAME:   
      {
         unsigned int* ip = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         unsigned int* uval =  static_cast<unsigned int*>(item1);
         *ip = *uval;
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);
      }
      break;

      case MSG_SET_AXIS_MAJ_LEN:  // fallthrough intended
      case MSG_SET_AXIS_MIN_LEN:  // fallthrough intended
      case MSG_SET_AXIS_MAJ_NUM:  // fallthrough intended 
      case MSG_SET_AXIS_MIN_NUM: 
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *ip = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_TILED_FRAME:    // tile - specified rows and cols 
      if (item1 && item2)
      {
         unsigned int* ip = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         unsigned int* uval =  static_cast<unsigned int*>(item1);
         *ip = *uval;    // store row count 
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);

         ip = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         uval =  static_cast<unsigned int*>(item2);
         *ip = *uval;    // store col count
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);
      }
      break;
      case MSG_PICK:  
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *ip = *ival;    // store  count 
         pos = NEXT_INT_POS(pos) + sizeof(int);

         unsigned int* up = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         unsigned int* uval =  static_cast<unsigned int*>(item2);
         *up = *uval;    // store col count
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);
      }
      break;
      case MSG_BLINK_FRAMES:
      if (item1 && item2)
      {
         float* fval = static_cast<float*>(item2);
         float *fp;
         fp = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *fp = *fval;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         char *cval = (char*)item1;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break; 
      case MSG_ARRANGE_FRAMES:
      if (item1 && item2)
      {
         string* sval1 = (string*)item2; 
         int rr;

         // pack message to contain cnt, and frame ids
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *ip = *ival;    // store num entries
         pos = NEXT_INT_POS(pos) + sizeof(int);

         for (rr = 0; rr < *ival; rr++)
         {
            strcpy(&cp[pos], sval1[rr].c_str());
            pos += strlen(&cp[pos]);
            cp[pos++] = '\0';
         }
      }
      break;
      case MSG_ADD_CLIENT_ERR:
      if (item1 && item2)
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *ip = *ival;    // store num entries
         pos = NEXT_INT_POS(pos) + sizeof(int);
         char *cval = (char*)item2;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_LOAD_CMAP_ARRAY:
      {//ASDFQWER
	 //Item 1: Long, Length of array
	 //Item 2: Int, Slot for colormap
	 //Item 3: Red, pointer to array of doubles - set in send
	 //Item 4: Green, pointer to array of doubles - set in send
	 //Item 5: Blue, pointer to array of doubles - set in send
	 //Item 6: Alpha, pointer to array of doubles - set in send
	 long* cnt = (long*)&cp[NEXT_LONG_POS(pos)];
         long* lval = static_cast<long*>(item1);
         *cnt = *lval;
         pos = NEXT_LONG_POS(pos) + sizeof(long);
	 
	 int* ip = (int*)&cp[NEXT_INT_POS(pos)];
	 int* ival = static_cast<int*>(item2);
	 *ip = *ival;
	 pos = NEXT_INT_POS(pos) + sizeof(int);
	 
	 //set position to start of data segment - populated in send
	 pos = NEXT_DOUBLE_POS(pos);
      }
      break;
      case MSG_LOAD_CMAP:  // fallthrough intended
      case MSG_LOAD_FILL:
      if (item1 && item2)
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item2);
         *ip = *ival;    // store num entries
         pos = NEXT_INT_POS(pos) + sizeof(int);
         char *cval = (char*)item1;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_LEGEND:
      if (item1 && item2)
      {
         char **cval1 = (char**)item1;
         char **cval2 = (char**)item2;

         int rr, cnt = 0;
         while (cval1[cnt] && cval2[cnt]) cnt++;

         // pack message to contain cnt, all curve ids, all curve labs, lgid
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         *ip = (int)cnt;    // store num entries 
         pos = NEXT_INT_POS(pos) + sizeof(int);

         for (rr = 0; rr < cnt; rr++)
         {
            strcpy(&cp[pos], cval1[rr]);
            pos += strlen(cval1[rr]);
            cp[pos++] = '\0';
         }

         for (rr = 0; rr < cnt; rr++)
         {
            strcpy(&cp[pos], cval2[rr]);
            pos += strlen(cval2[rr]);
            cp[pos++] = '\0';
         }

         if (item3)
         {
            char *lgid = (char*)item3;
            if ((item_sz = strlen(lgid)) != 0)
            {
               strcpy(&cp[pos], lgid);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_WINDOW_C:    // add window w/ dimensions
      if (item1 && item2 && item3)
      {
         float *height, *width;
         char *cval = (char*)item3;

         float* fval1 = static_cast<float*>(item1);
         float* fval2 = static_cast<float*>(item2);
         height = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *height = *fval1;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float); 
         width = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *width = *fval2;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float); 

         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';

         if (item4)
         {
            cval = (char*)item4;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_POINT:
      {
         double *xx, *yy;
         char *ptid = (char*)item3;

         double* dval1 = static_cast<double*>(item1);
         double* dval2 = static_cast<double*>(item2);
         xx = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *xx = *dval1;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         yy = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *yy = *dval2;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

         if (item3 && ((item_sz = strlen(ptid)) != 0))
         {
            strcpy(&cp[pos], ptid);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break; 
      case MSG_ADD_POINT_B:
      {
         double *xx, *yy, *zz;
         char *ptid = (char*)item4;

         double* dval1 = static_cast<double*>(item1);
         double* dval2 = static_cast<double*>(item2);
         double* dval3 = static_cast<double*>(item3);
         xx = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *xx = *dval1;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         yy = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *yy = *dval2;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         zz = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *zz = *dval3;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

         if (item4 && ((item_sz = strlen(ptid)) != 0))
         {
            strcpy(&cp[pos], ptid);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_LABEL:
      {
         double *xx, *yy;
         char *lid, *txt = (char*)item3;

         double* fval1 = static_cast<double*>(item1);
         double* fval2 = static_cast<double*>(item2);
         xx = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *xx = *fval1;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         yy = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *yy = *fval2;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

         if ((item_sz = strlen(txt)) != 0)
         {
            strcpy(&cp[pos], txt);
            pos+= item_sz;
         }
         cp[pos++] = '\0';

         if (item4)
         {
            lid = (char*)item4;
            if ((item_sz = strlen(lid)) != 0)
            {
               strcpy(&cp[pos], lid);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_CBAR:
      {
         double *xx, *yy;
     
         char *lid;
         char *txt = (char*)item3;

         double* fval1 = static_cast<double*>(item1);
         double* fval2 = static_cast<double*>(item2);
         xx = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *xx = *fval1;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         yy = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *yy = *fval2;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         int* ival =  static_cast<int*>(item4);
         int* obj = (int*)&cp[NEXT_INT_POS(pos)];
         *obj = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         if ((item_sz = strlen(txt)) != 0)
         {
            strcpy(&cp[pos], txt);
            pos+= item_sz;
         }
         cp[pos++] = '\0';

         if (item5)
         {
            lid = (char*)item5;
            if ((item_sz = strlen(lid)) != 0)
            {
               strcpy(&cp[pos], lid);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_PLOT_B:       // fallthrough intended 
      case MSG_REPOSITION_PLOT:  // fallthrough intended 
      case MSG_REPOSITION_FRAME: // fallthrough intended 
      case MSG_ADD_FRAME_C:   
      if (item1 && item2 && item3 && item4)
      {
         float* fval1 = static_cast<float*>(item1);
         float* fval2 = static_cast<float*>(item2);
         float* fval3 = static_cast<float*>(item3);
         float* fval4 = static_cast<float*>(item4);
         float *x1, *y1, *x2, *y2;

         x1 = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *x1 = *fval1;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         y1 = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *y1 = *fval2;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         x2 = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *x2 = *fval3;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         y2 = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *y2 = *fval4;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);

         if (item5)
         {
            char *cval = (char*)item5;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_AXIS:
      if (item1 && item2 && item3 && item4 && item5 && item6)
      {
         int*   ival  =  static_cast<int*>(item1);
         float* fval1 = static_cast<float*>(item2);
         float* fval2 = static_cast<float*>(item3);
         float* fval3 = static_cast<float*>(item4);
         float* fval4 = static_cast<float*>(item5);
         float* fval5 = static_cast<float*>(item6);

         float *ps, *minx, *maxx, *miny, *maxy;

         int* dim = (int*)&cp[NEXT_INT_POS(pos)];
         pos = NEXT_INT_POS(pos) + sizeof(int);
         *dim = *ival;
         ps = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *ps = *fval1;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         minx = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *minx = *fval2;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         maxx = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *maxx = *fval3;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         miny = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *miny = *fval4;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);
         maxy = (float*)&cp[NEXT_FLOAT_POS(pos)];
         *maxy = *fval5;
         pos = NEXT_FLOAT_POS(pos) + sizeof(float);

         if (item7)
         {
            Transform *trans = static_cast<Transform*>(item7);
            ChipsTransformFactory transform_factory;
            unsigned char *serial_transform;
            int serial_transform_size = 0;

            // turn the transform into a byte stream
            serial_transform = transform_factory.serialize (trans,
                                                        serial_transform_size);

            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = serial_transform_size;
            pos = NEXT_INT_POS(pos) + sizeof(int);
            memcpy (&cp[pos], serial_transform, serial_transform_size);
            pos += serial_transform_size;
         }
         else
         {
            // Indicate there was no transform
            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = 0;
            pos = NEXT_INT_POS(pos) + sizeof(int);
         }

         if (item8)
         {
            char *cval = (char*)item8;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break; 
      case MSG_ADD_LINE:
      {
         double* dval1 = static_cast<double*>(item1);
         double* dval2 = static_cast<double*>(item2);
         double* dval3 = static_cast<double*>(item3);
         double* dval4 = static_cast<double*>(item4);
         double *x1, *y1, *x2, *y2;
         x1 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *x1 = *dval1;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         y1 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *y1 = *dval2;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         x2 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *x2 = *dval3;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         y2 = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *y2 = *dval4;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

         if (item5)
         {
            char *cval = (char*)item5;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_REGION_B:
      if (item1)
      {
         int* side     = static_cast<int*>(item1);
         double* dval1 = static_cast<double*>(item2);
         double* dval2 = static_cast<double*>(item3);
         double* rad   = static_cast<double*>(item4);
         double *xx, *yy, *rads;
         int* cnt = (int*)&cp[NEXT_INT_POS(pos)];
         pos = NEXT_INT_POS(pos) + sizeof(int);
         *cnt = *side;
         xx = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *xx = *dval1;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         yy = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *yy = *dval2;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         rads = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         *rads = *rad;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
         if (item5)
         {
            char *cval = (char*)item5;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_ADD_REGION:   // fallthrough intended
      case MSG_ADD_LINE_B:
      if (item1 && item2 && item3)
      {
         unsigned int* uval  =  static_cast<unsigned int*>(item3);
         unsigned int* cnt = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         pos = NEXT_INT_POS(pos) + sizeof(int);
         *cnt = *uval;
         if (item4)
         {
            char *cval = (char*)item4;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';

         //set position to start of data segment
         pos = NEXT_DOUBLE_POS(pos);
      }
      else  if (item3) rval = ERR_CURVE_COL_DNE;
         else   rval = ERR_CURVE_DATA_DNE;
      break;
      case MSG_SET_CNTR_LEVELS:  
      if (item1 && item2)
      {
         unsigned int rr; 
         unsigned int* uval  = static_cast<unsigned int*>(item1);
         unsigned int* cnt = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         *cnt = *uval;
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);
         double* ps = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dval = static_cast<double*>(item2);

         for (rr = 0; rr < *cnt; rr++)
         {
            *ps = dval[rr];   
             pos = NEXT_DOUBLE_POS(pos) + sizeof(double); 
             ps = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         }
      }
      break;
      case MSG_SET_AXIS_TICKPOS:
      if (item1 && item2)
      {
         unsigned int rr;
         unsigned int* uval  = static_cast<unsigned int*>(item1);
         unsigned int* cnt = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         *cnt = *uval;
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);
         double* ps = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double* dval = static_cast<double*>(item2);

         for (rr = 0; rr < *cnt; rr++)
         {
            *ps = dval[rr];
             pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
             ps = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         }
      }
      if (item3)
      {
         string* sval1 = (string*)item3;
         int rr;

         // pack message to contain cnt, and frame ids
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         int* ival =  static_cast<int*>(item1);
         *ip = *ival;    // store num entries
         pos = NEXT_INT_POS(pos) + sizeof(int);

         for (rr = 0; rr < *ival; rr++)
         {
            strcpy(&cp[pos], sval1[rr].c_str());
            pos += strlen(&cp[pos]);
            cp[pos++] = '\0';
         }
      }
      else
      {
         int* ip = (int*)&cp[NEXT_INT_POS(pos)];
         *ip = 0;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_ADD_CURVE_A: 
      if (!item1) 
         rval = ERR_CURVE_DATA_DNE;
      else if (item2 && item3)
      {
         int *ival;
         if (item4)
         {
            Transform *trans = static_cast<Transform*>(item4);
            ChipsTransformFactory transform_factory;
            unsigned char *serial_transform;
            int serial_transform_size = 0;

            // turn the transform into a byte stream
            serial_transform = transform_factory.serialize (trans,
                                                        serial_transform_size);

            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = serial_transform_size;
            pos = NEXT_INT_POS(pos) + sizeof(int);
            memcpy (&cp[pos], serial_transform, serial_transform_size);
            pos += serial_transform_size;
         }
         else
         {
            // Indicate there was no transform
            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = 0;
            pos = NEXT_INT_POS(pos) + sizeof(int);
         }

         if (item5)
         {
            char *cval = (char*)item5;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';

         long* cnt = (long*)&cp[NEXT_LONG_POS(pos)];
         long* lval = static_cast<long*>(item1);
         *cnt = *lval;
         pos = NEXT_LONG_POS(pos) + sizeof(long);

         //set position to start of data segment 
         pos = NEXT_DOUBLE_POS(pos);
      }
      else   
         rval = ERR_CURVE_COL_DNE;
      break;
      case MSG_ADD_CURVE_B:
      if (item1 && item2 && item3)
      {
         int *ival;
         if (item10)
         {
            Transform *trans = static_cast<Transform*>(item10);
            ChipsTransformFactory transform_factory;
            unsigned char *serial_transform;
            int serial_transform_size = 0;

            // turn the transform into a byte stream
            serial_transform = transform_factory.serialize (trans,
                                                        serial_transform_size);

            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = serial_transform_size;
            pos = NEXT_INT_POS(pos) + sizeof(int);
            memcpy (&cp[pos], serial_transform, serial_transform_size);
            pos += serial_transform_size;
         }
         else
         {
            // Indicate there was no transform
            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = 0;
            pos = NEXT_INT_POS(pos) + sizeof(int);
         }


         if (item11)
         {
            char *cval = (char*)item11;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';

         char mask = 0;
         if (item4)  mask |= 0x01;
         if (item5)  mask |= 0x02;
         if (item6)  mask |= 0x04;
         if (item7)  mask |= 0x08;
         if (item8)  mask |= 0x010;
         if (item9)  mask |= 0x020;

         char* mask_p = (char*)&cp[NEXT_CHAR_POS(pos)];
         *mask_p = mask;
         pos = NEXT_CHAR_POS(pos) + sizeof(char);

         long* cnt = (long*)&cp[NEXT_LONG_POS(pos)];
         long* lval = static_cast<long*>(item1);
         *cnt = *lval;
         pos = NEXT_LONG_POS(pos) + sizeof(long);

         //set position to start of data segment
         pos = NEXT_DOUBLE_POS(pos);
      }
      else  if (item1) rval = ERR_CURVE_COL_DNE;
         else   rval = ERR_CURVE_DATA_DNE;
      break;
      case MSG_ADD_HISTOGRAM:
      if (item1 && item2)
      {
         if (item7)
         {
            char *cval = (char*)item7;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';

         char mask = 0;
         if (item3)  mask |= 0x02;
         if (item4)  mask |= 0x04;
         if (item5)  mask |= 0x08;
         if (item6)  mask |= 0x10;

         char* mask_p = (char*)&cp[NEXT_CHAR_POS(pos)];
         *mask_p = mask;
         pos = NEXT_CHAR_POS(pos) + sizeof(char);

         long* cnt = (long*)&cp[NEXT_LONG_POS(pos)];
         long* lval = static_cast<long*>(item1);
         *cnt = *lval;
         pos = NEXT_LONG_POS(pos) + sizeof(long);

         //set position to start of data segment
         pos = NEXT_DOUBLE_POS(pos);
      }
      else  if (item1) rval = ERR_CURVE_COL_DNE;
         else   rval = ERR_CURVE_DATA_DNE;
      break;
      case MSG_ADD_CONTOUR:
      if (item1 && item2 && item3 && item4)
      {
         int *ival;
         if (item8)
         {
            char *cval = (char*)item8;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';

         if (item7)
         {
            Transform *trans = static_cast<Transform*>(item7);
            ChipsTransformFactory transform_factory;
            unsigned char *serial_transform;
            int serial_transform_size = 0;
      
            // turn the transform into a byte stream 
            serial_transform = transform_factory.serialize (trans, 
                                                    serial_transform_size);

            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = serial_transform_size;
            pos = NEXT_INT_POS(pos) + sizeof(int);
            memcpy (&cp[pos], serial_transform, serial_transform_size);
            pos += serial_transform_size;
         }
         else
         {
            // Indicate there was no transform
            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = 0;
            pos = NEXT_INT_POS(pos) + sizeof(int);
         }

         long* cnt = (long*)&cp[NEXT_LONG_POS(pos)];
         long* lval  =  static_cast<long*>(item1);

         *cnt = *lval;
         pos = NEXT_LONG_POS(pos) + sizeof(long);

         unsigned int* num = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         unsigned int* uval  =  static_cast<unsigned int*>(item5);

         *num = *uval;
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);

         if (*num != 0)
         {
	    if (item6) 
	    {
               unsigned int rr; 
               pos = NEXT_DOUBLE_POS(pos);
               double* ps = (double*)&cp[pos];
               double* dval = static_cast<double*>(item6);
               for (rr = 0; rr < *num; rr++)
               {
                  *ps = dval[rr];
                   pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
                   ps = (double*)&cp[NEXT_DOUBLE_POS(pos)];
               }
	    }
	    else 
	    {
	       // no levels provided- set count to 0 to avoid segv
	       *num = 0;
	    }
         }
         //set position to start of data segment
         pos = NEXT_DOUBLE_POS(pos);
      }
      else  if (item1) rval = ERR_CURVE_COL_DNE;
         else   rval = ERR_CURVE_DATA_DNE;
      break;
      case MSG_ADD_CONTOUR_B:
      if (item1 && item2 && item3)
      {
         int *ival;
         if (item7)
         {
            char *cval = (char*)item7;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';

         if (item6)
         {
            Transform *trans = static_cast<Transform*>(item6);
            ChipsTransformFactory transform_factory;
            unsigned char *serial_transform;
            int serial_transform_size = 0;

            // turn the transform into a byte stream
            serial_transform = transform_factory.serialize (trans, 
                                                        serial_transform_size);

            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = serial_transform_size;
            pos = NEXT_INT_POS(pos) + sizeof(int);
            memcpy (&cp[pos], serial_transform, serial_transform_size);
            pos += serial_transform_size;
         }
         else
         {
            // Indicate there was no transform
            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = 0;
            pos = NEXT_INT_POS(pos) + sizeof(int);
         }

         long* cnt = (long*)&cp[NEXT_LONG_POS(pos)];
         long* lval  =  static_cast<long*>(item2);
         *cnt = *lval;
         pos = NEXT_LONG_POS(pos) + sizeof(long);
         cnt = (long*)&cp[NEXT_LONG_POS(pos)];
         lval  =  static_cast<long*>(item3);
         *cnt = *lval;
         pos = NEXT_LONG_POS(pos) + sizeof(long);
         unsigned int* num = (unsigned int*)&cp[NEXT_INT_POS(pos)];
         unsigned int* uval  =  static_cast<unsigned int*>(item4);
         *num = *uval;
         pos = NEXT_INT_POS(pos) + sizeof(unsigned int);

         if (*num != 0)
         {
            if (item5)
            {
               unsigned int rr;
               pos = NEXT_DOUBLE_POS(pos);
               double* ps = (double*)&cp[pos];
               double* dval = static_cast<double*>(item5);
               for (rr = 0; rr < *num; rr++)
               {
                  *ps = dval[rr];
                   pos = NEXT_DOUBLE_POS(pos) + sizeof(double);
                   ps = (double*)&cp[NEXT_DOUBLE_POS(pos)];
               }
            }
            else
            {
               // no levels provided- set count to 0 to avoid segv
               *num = 0;
            }
         }
         //set position to start of data segment
         pos = NEXT_DOUBLE_POS(pos);
      }
      else  if (!item1) rval = ERR_CURVE_COL_DNE;
      break;
      case MSG_ADD_IMAGE:
      if (item1 && item2)
      {
         int *ival;
         char *cval = (char*)item1;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item2;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         if (item3)
         {
            Transform *trans = static_cast<Transform*>(item3);
            ChipsTransformFactory transform_factory;
            unsigned char *serial_transform;
            int serial_transform_size = 0;

            // turn the transform into a byte stream
            serial_transform = transform_factory.serialize (trans,
                                                        serial_transform_size);

            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = serial_transform_size;
            pos = NEXT_INT_POS(pos) + sizeof(int);
            memcpy (&cp[pos], serial_transform, serial_transform_size);
            pos += serial_transform_size;
         }
         else
         {
            // Indicate there was no transform
            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = 0;
            pos = NEXT_INT_POS(pos) + sizeof(int);
         }

      }
      break; 
      case MSG_ADD_IMAGE_C:
      if ((item1 || item2 || item3 || item4) &&
          (item5 && item6))
      {
         int *ival;
         if (item8)
         {
            char *cval = (char*)item8;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';

         if (item7)
         {
            Transform *trans = static_cast<Transform*>(item7);
            ChipsTransformFactory transform_factory;
            unsigned char *serial_transform;
            int serial_transform_size = 0;

            // turn the transform into a byte stream
            serial_transform = transform_factory.serialize (trans,
                                                        serial_transform_size);

            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = serial_transform_size;
            pos = NEXT_INT_POS(pos) + sizeof(int);
            memcpy (&cp[pos], serial_transform, serial_transform_size);
            pos += serial_transform_size;
         }
         else
         {
            // Indicate there was no transform
            ival = (int*)&cp[NEXT_INT_POS(pos)];
            *ival = 0;
            pos = NEXT_INT_POS(pos) + sizeof(int);
         }

         ival  =  static_cast<int*>(item5);
         int* xaxis = (int*)&cp[NEXT_INT_POS(pos)];
         *xaxis = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         int* yaxis = (int*)&cp[NEXT_INT_POS(pos)];
         ival  =  static_cast<int*>(item6);
         *yaxis = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         // count the number of images
         int* image_count = (int*)&cp[NEXT_INT_POS(pos)];
         *image_count = 0;
         if (item1) *image_count |= 1;
         if (item2) *image_count |= 2;
         if (item3) *image_count |= 4;
         if (item4) *image_count |= 8;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         //set position to start of data segment 
         pos = NEXT_DOUBLE_POS(pos);
      }
      break;
      case MSG_ADD_IMAGE_CHANNEL:
      if (item1 && item2 && item3 && item4)
      {
         int*   ival  =  static_cast<int*>(item2);
         int* xaxis = (int*)&cp[NEXT_INT_POS(pos)];
         *xaxis = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         int* yaxis = (int*)&cp[NEXT_INT_POS(pos)];
         ival  =  static_cast<int*>(item3);
         *yaxis = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         ival  =  static_cast<int*>(item4);
         int* channel = (int*)&cp[NEXT_INT_POS(pos)];
         *channel = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);


         //set position to start of data segment
         pos = NEXT_DOUBLE_POS(pos);
      }
      break;
      case MSG_DEL_IMAGE_CHANNEL:
      if (item1)
      {
         int*   ival  =  static_cast<int*>(item1);
         int* channel = (int*)&cp[NEXT_INT_POS(pos)];
         *channel = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
      }
      break;
      case MSG_ADD_TRANSFORM:
      if (item1 && item2 && item3)
      {

	 if (item4){
	    char *cval = (char*)item4;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
	 }
         cp[pos++] = '\0';
         int* row = (int*)&cp[NEXT_INT_POS(pos)];
	 int *ival  =  static_cast<int*>(item1);
         *row = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         int* col = (int*)&cp[NEXT_INT_POS(pos)];
         ival  =  static_cast<int*>(item2);
         *col = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         //set position to start of data segment
	 double *data_p = (double *)&cp[NEXT_DOUBLE_POS(pos)];
         pos = NEXT_DOUBLE_POS(pos) + (*row * (*col))*sizeof (double);
         for (int ii=0; ii<*row; ii++)
            for (int jj=0; jj<*col; jj++)
               data_p[ii*(*col)+jj] = ((double **)(item3))[ii][jj];
      }
      break;
      case MSG_REMOVE_TRANSFORM:
      if (item1)
      {
         char *cval = (char*)item1;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
      }
      break;
      case MSG_APPLY_TRANS_A:
      {
	 if (item1){
            char *cval = (char*)item1;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
	    cp[pos++]='\0';
	 }
      }
      break;
      case MSG_APPLY_TRANS_B:
      if (item1 && item2 && item3)
      {
         if (item4)
	 {
            char *cval = (char*)item4;
            if ((item_sz = strlen(cval)) != 0)
            {
               strcpy(&cp[pos], cval);
               pos+= item_sz;
            }
         }
         cp[pos++] = '\0';
         int* row = (int*)&cp[NEXT_INT_POS(pos)];
         int *ival  =  static_cast<int*>(item1);
         *row = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
         int* col = (int*)&cp[NEXT_INT_POS(pos)];
         ival  =  static_cast<int*>(item2);
         *col = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);

         //set position to start of data segment
         double *data_p = (double *)&cp[NEXT_DOUBLE_POS(pos)];
         pos = NEXT_DOUBLE_POS(pos) + (*row * (*col))*sizeof (double);
         for (int ii=0; ii<*row; ii++)
            for (int jj=0; jj<*col; jj++)
               data_p[ii*(*col)+jj] = ((double **)(item3))[ii][jj];
      }
      break;
      case MSG_CONVERT_COORD_A:
      if (item1 && item2 && item3 && item4 &&
	  item5 && item6 && item7 && item8)
      {
         // Write the strings out
         char *cval = (char*)item3;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
	 cval = (char*)item4;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
	 cval = (char*)item5;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
	 cval = (char*)item6;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';

         // Write the integers out
         int* count = (int*)&cp[NEXT_INT_POS(pos)];
         int *ival  =  static_cast<int*>(item2);
         *count = *ival;
         pos = NEXT_INT_POS(pos) + sizeof(int);
	 int *coord_sys = (int*)&cp[NEXT_INT_POS(pos)];
         ival  =  static_cast<int*>(item7);
         *coord_sys = *ival;
	 pos = NEXT_INT_POS(pos) + sizeof(int);
	 int *axis = (int*)&cp[NEXT_INT_POS(pos)];
         ival  =  static_cast<int*>(item8);
         *axis = *ival;
	 pos = NEXT_INT_POS(pos) + sizeof(int);


	 // Write the arrays out
         double* data_p = (double*)&cp[NEXT_DOUBLE_POS(pos)];
	 pos = NEXT_DOUBLE_POS(pos) + (*count)*sizeof (double);

         for (int ii=0; ii<*count; ii++)
            data_p[ii] = ((double *)(item1))[ii];

      }
      break;
      case MSG_CONVERT_COORD_B:
      if (item1 && item2 && item3 && item4 &&
          item5 && item6 && item7)
      {
         // Write the strings out
         char *cval = (char*)item2;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item3;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item4;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';
         cval = (char*)item5;
         if ((item_sz = strlen(cval)) != 0)
         {
            strcpy(&cp[pos], cval);
            pos+= item_sz;
         }
         cp[pos++] = '\0';

         // Write the integers out
         int *coord_sys = (int*)&cp[NEXT_INT_POS(pos)];
         int *ival  =  static_cast<int*>(item6);
         *coord_sys = *ival;
	 pos = NEXT_INT_POS(pos) + sizeof(int);
         int *axis = (int*)&cp[NEXT_INT_POS(pos)];
         ival  =  static_cast<int*>(item7);
         *axis = *ival;
	 pos = NEXT_INT_POS(pos) + sizeof(int);

	 double* in_val = (double*)&cp[NEXT_DOUBLE_POS(pos)];
         double *dval  =  static_cast<double*>(item1);
         *in_val = *dval;
         pos = NEXT_DOUBLE_POS(pos) + sizeof(double);

      }
      break;
      default:
         rval = ERR_MESSAGE_UNKNOWN; 
      break; 
   }
   *msg_sz = pos;   // set message size

   return rval; 
}


int ChipsClient::unpack_it(const char* buf, ChipsId* cid)
{
   int rval = 0;   // return current index in char* buffer

   if (buf && cid)
   {
      const char* val = buf;
      int ln, pos = 0;
      int *ival;

      if ((ln=strlen(&val[pos]))!=0) {cid->set_window_id(&val[pos]);pos+=ln;}
      else  cid->set_window_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_frame_id(&val[pos]);pos+=ln;}
      else  cid->set_frame_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_plot_id(&val[pos]);pos+=ln;}
      else  cid->set_plot_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_image_id(&val[pos]);pos+=ln;}
      else  cid->set_image_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_coord_id(&val[pos]);pos+=ln;}
      else  cid->set_coord_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_curve_id(&val[pos]);pos+=ln;}
      else  cid->set_curve_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_contour_id(&val[pos]);pos+=ln;}
      else  cid->set_contour_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_histogram_id(&val[pos]);pos+=ln;}
      else  cid->set_histogram_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_axis_id(&val[pos]);pos+=ln;}
      else  cid->set_axis_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_xaxis_id(&val[pos]);pos+=ln;}
      else  cid->set_xaxis_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_yaxis_id(&val[pos]);pos+=ln;}
      else  cid->set_yaxis_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_label_id(&val[pos]);pos+=ln;}
      else  cid->set_label_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_line_id(&val[pos]);pos+=ln;}
      else  cid->set_line_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_point_id(&val[pos]);pos+=ln;}
      else  cid->set_point_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_region_id(&val[pos]);pos+=ln;}
      else  cid->set_region_id("");
      pos++;
      if ((ln=strlen(&val[pos]))!=0) {cid->set_colorbar_id(&val[pos]);pos+=ln;}
      else  cid->set_colorbar_id("");
      pos++;
      ival = (int*)&val[NEXT_INT_POS(pos)];
      cid->set_coord_sys ((COORD_SYS)(*ival));
      pos = NEXT_INT_POS(pos) + sizeof(int);
      // axis dim
      ival = (int*)&val[NEXT_INT_POS(pos)];
      cid->set_axis_dim((*ival));
      pos = NEXT_INT_POS(pos) + sizeof(int);
      rval = pos; 
   }
   return rval; 
}


bool ChipsClient::does_server_exist(const string& inServerId)
{
   bool bv = 0;

   string tmpServerSocket= tmpdir;
   tmpServerSocket+=inServerId;
   SCSClient tmpClient;

   if ( tmpClient.OpenConnection(10000000,tmpServerSocket)  > 0)
   {
      // server exists
      tmpClient.CloseConnection();
      bv = 1;
   }
   return bv; 
}


list <string> ChipsClient::list_servers()
{
   list <string> slist;
   string tmp = "";
   // open up the chips server directory to search for socket files
   DIR* dirp = opendir(tmpdir.c_str());
   if (dirp)
   {
      struct dirent *entry;
      // check each item in the directory 
      SCSClient tmpClient;

      while((entry = readdir(dirp)) != NULL)
      {
         if (strcmp(entry->d_name, ".") && strcmp(entry->d_name, ".."))
         {
	    string tmpname = tmpdir;
	    tmpname.append(entry->d_name);
            struct stat status;
            if (!stat(tmpname.c_str(), &status))
            {
	       // don't add directories or regular files to our potential list
	       if (!(S_ISDIR(status.st_mode) || S_ISREG(status.st_mode)))
	       {
                  tmpname = entry->d_name;
                  string tmpsck = tmpdir + "/" + tmpname; 
                  // add servers we can connect to (1) or that are locked (1)
                  // socket files w/o servers return -1- don't add those 
                  if (tmpClient.OpenConnection(40000,tmpsck) != -1)
                  {
	             slist.push_back(tmpname);
                     tmpClient.CloseConnection(); 
                  }
	       }
            }
         }
      }
      closedir(dirp);
   }
   return slist;
}


int ChipsClient::connect_server(const string& inServerId, int attempts)
{
   int rval = ERR_STATUS_GOOD;
   int retry = 10;
   string tmpServerSocket= tmpdir;
   tmpServerSocket+=inServerId;

   if (attempts < 1) retry = 10;
      else  retry = attempts;

   if (!serverId.empty())
      disconnect_server ();

   while (retry)
   {
      if ( chipsClient.OpenConnection(1000000,tmpServerSocket) <=0)
      {
         serverId="";
         rval = ERR_SOCK_UNAVAIL;
         retry--; 
      }
      else
      {
         serverId=inServerId;
         retry = 0;  
         rval = ERR_STATUS_GOOD;
      }
   }
   return rval;
}


int ChipsClient::disconnect_server()
{
   if (serverId.empty())   return EER_SOCK_NOT_CONN;
   serverId="";
   if ( chipsClient.CloseConnection()<=0)  return  ERR_SOCK_CLOSE;
      else   return ERR_STATUS_GOOD;
}

int ChipsClient::create_client()
{
   if (!clientId.empty())   return ERR_SOCK_CONN;
   ostringstream tmpStr;
   tmpStr<<"ChipsClient"<<"_"<<getpid();
   clientId=tmpStr.str();

   return ERR_STATUS_GOOD;
}


// method to retrieve information from the server process 
int ChipsClient::get_data(
   ChipsId* cid,          // chips id object 
   int      msgid,        // message id value
   void***  out_data,     // info array returned from server  
   char**   out_typ,      // types of info in array 
   int**    out_len,      // length of info in array 
   int*     out_cnt,      // number of data items returned 
   void*    in_item1,     // value for optional parameter 1 
   void*    in_item2,     // value for optional parameter 2 
   void*    in_item3,     // value for optional parameter 3
   void*    in_item4,     // value for optional parameter 4
   void*    in_item5,     // value for optional parameter 5
   void*    in_item6,     // value for optional parameter 6
   void*    in_item7,     // value for optional parameter 7
   void*    in_item8,     // value for optional parameter 8
   void*    in_item9,     // value for optional parameter 9
   void*    in_item10,    // value for optional parameter 10
   void*    in_item11)    // value for optional parameter 11
{
   int rval = ERR_STATUS_GOOD;

   if ((rval = pack_it(cid, msgid, in_item1, in_item2, in_item3, in_item4, 
       in_item5, in_item6, in_item7, in_item8, in_item9, in_item10, in_item11)) == 
       ERR_STATUS_GOOD)
   {
      int* lp = (int*)buffer;
      char* cp = (char*)buffer;
      unsigned int msglen = 0;
      lp++;
      msglen = *lp;
 
      unsigned int buff[2] = {0,0};   // buffer for mesg header (id,size)
      char*        pos = (char*)buff; // pointer to message buffer
      double       pbuf[PIPE_BUF];    // copy buffer for short messages
      int          overflow = 0;      // indicator for long messages
      SCSMessage tmpMessage;

      tmpMessage.Set(cp,msglen);
      chipsClient.SendMessage(-1,tmpMessage);

      if ( chipsClient.ReceiveMessage(-1,tmpMessage) <= 0 )
         return ERR_NO_SOCKET;
      else if (tmpMessage.Type() == SCSMessage::PROTOCOL_CLOSE )
         return ERR_STATUS_BAD;

      memcpy(pos,tmpMessage.Data(),8);
      msglen = buff[1] - 8;

      if (msglen > 0)
      {
         // make sure the message length is appropriate for the data
         overflow=(msglen > PIPE_BUF);
         if (rval == ERR_STATUS_GOOD)
         {
            char *bf = (char*)pbuf;     // buffer pointer
	      
            // too large for preset buffer- allocate new one
            if (overflow)      bf = (char*) malloc(msglen*sizeof(char));
	      
            // load all data from pipe to our buffer
            memcpy(bf,tmpMessage.Data()+8,msglen);
            tmpMessage.Clear();

            // unpack the data 
            if (buff[0] == MSG_INFO_RESPONSE)
            {
               void **dd = NULL;
               char *tt = NULL, *dp = NULL, **pp = NULL;
               int rr, cc, *ll = NULL;
               short* sp = (short*)bf;  
               unsigned int*   sz;
		  
               *out_cnt = sp[0];
               sz = (unsigned int*)&bf[((5 + *out_cnt)/4)*4];  
               dp = (char*) &sz[*out_cnt]; 
		  
               // allocate memory for returned info 
               dd = (void**)calloc(*out_cnt,sizeof(void*));
               tt = (char*)calloc(*out_cnt,sizeof(char));
               ll = (int*)calloc(*out_cnt,sizeof(int));
               pp = (char**)malloc(*out_cnt * sizeof(char*)); 
		  
               pp[0] = (char*)&sz[*out_cnt];
               if (((dp - bf) % 8) == 4)   pp[0] += 4; 
               tt[0] = bf[2];
               ll[0] = sz[0];
               for (rr = 1; rr < *out_cnt; rr++)
               {
                  tt[rr] = bf[2+rr];
                  ll[rr] = sz[rr];
                  switch(tt[rr-1])
                  {
                     case MSG_DATATYPE_DOUBLE: 
                        pp[rr] = (char*)pp[rr-1] + 
			   ((ll[rr-1]*sizeof(double))+7)/8*8;
                     break; 
                     case MSG_DATATYPE_FLOAT: 
                        pp[rr] = (char*)pp[rr-1] + 
                           ((ll[rr-1]*sizeof(float))+7)/8*8;
                     break; 
                     case MSG_DATATYPE_LONG: 
                        pp[rr] = (char*)pp[rr-1] + 
                           ((ll[rr-1]*sizeof(long))+7)/8*8;
                     break; 
                     case MSG_DATATYPE_INT: 
                        pp[rr] = (char*)pp[rr-1] + 
                           ((ll[rr-1]*sizeof(int))+7)/8*8;
                     break; 
                     case MSG_DATATYPE_CHAR: 
                        pp[rr] = (char*)pp[rr-1] + 
                           ((ll[rr-1]*sizeof(char))+7)/8*8;
                     break; 
                     case MSG_DATATYPE_UCHAR:
                        pp[rr] = (char*)pp[rr-1] +
                           ((ll[rr-1]*sizeof(unsigned char))+7)/8*8;
                     break;
		     default:
		     break; 
                  }
               }
 
               // get data
               for(rr = *out_cnt; rr--; )
               {
                  switch(tt[rr])
                  {
                     case MSG_DATATYPE_DOUBLE : 
                     {
                        double *dval = (double*)pp[rr];
   
                        dd[rr] = (double*)calloc(ll[rr],sizeof(double));
  
                        for (cc = 0; cc < ll[rr]; cc++)
                           ((double*) dd[rr])[cc] = dval[cc];  
                     }
                     break; 
                     case MSG_DATATYPE_FLOAT : 
                     {
                        float *fval = (float*)pp[rr];

                        dd[rr] = (float*)calloc(ll[rr],sizeof(float));

                        for (cc = 0; cc < ll[rr]; cc++)
                           ((float*) dd[rr])[cc] = fval[cc];
                     }
                     break; 
                     case MSG_DATATYPE_LONG : 
                           dd[rr] = (long*)calloc(ll[rr], sizeof(long));
                     break; 
                     case MSG_DATATYPE_ULONG : 
                        dd[rr] = (unsigned long*)calloc(ll[rr],
                           sizeof(unsigned long));
                     break; 
                     case MSG_DATATYPE_INT : 
                     {
                        int *ival = (int*)pp[rr];
   
                        dd[rr] = (int*)calloc(ll[rr],sizeof(int));
   
                        for (cc = 0; cc < ll[rr]; cc++)
                           ((int*) dd[rr])[cc] = ival[cc];
                     }
                     break; 
                     case MSG_DATATYPE_UINT : 
                        dd[rr] = (unsigned int*)calloc(ll[rr],
                           sizeof(unsigned int));
                     break; 
                     case MSG_DATATYPE_SHORT : 
                        dd[rr] = (short*)calloc(ll[rr],sizeof(short));
                     break; 
                     case MSG_DATATYPE_USHORT : 
                        dd[rr] = (unsigned short*)calloc(ll[rr],
                           sizeof(unsigned short));
                     break; 
                     case MSG_DATATYPE_CHAR :
                     {
                        char *cval = (char*)pp[rr];
   
                        dd[rr] = (char*)calloc(ll[rr],sizeof(char));
   
                        for (cc = 0; cc < ll[rr]; cc++)
                           ((char*) dd[rr])[cc] = cval[cc];
                     }
                     break; 
                     case MSG_DATATYPE_UCHAR :
                     {
                        unsigned char *ucval = (unsigned char*)pp[rr];

                        dd[rr] = (unsigned char*)calloc(ll[rr],
                           sizeof(unsigned char));

                        for (cc = 0; cc < ll[rr]; cc++)
                           ((unsigned char*) dd[rr])[cc] = ucval[cc];
                     }
                     break;
                     default:
                     break; 
                  }
               }  

               // set pointers to data
               *out_len = ll;
               *out_typ = tt;
               *out_data = dd;
               free(pp); 
            }
            else if ((buff[0] < ERR_MAX_NUM) && (buff[1] == 8))
            {
               // the get_data request has failed and the message
               // returned was a status message, so set the err value
               rval = buff[0]; 
            }
            else   rval = ERR_MESSAGE_INVALID;  // unexpected message id
   
            if (overflow)   free(bf);   // cleanup any allocated buffer
         }
      }
      else if ((buff[1] == 8) && (buff[0] < ERR_MAX_NUM) &&
	       (buff[0] >= ERR_STATUS_GOOD))
      {
         rval = buff[0];     // return value only returned in response 
      }
      else   rval = ERR_RESPONSE_TIMEOUT;   // response timed out
   }
   return (rval);
}

