// 
//  Copyright (C) 2007,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSCLIENT_HH
#define CHIPSCLIENT_HH

#include <string>
#include <list>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include <iostream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "SCSMessage.hh"

using namespace std;


// macros to determine the byte offset in the buffer for positions
// of various standard types
//
#define NEXT_DOUBLE_POS(p) (((p + sizeof(double) -1)/sizeof(double)) \
                            * sizeof(double))
#define NEXT_FLOAT_POS(p)  (((p + sizeof(float) -1)/sizeof(float)) \
                            * sizeof(float))
#define NEXT_LONG_POS(p)   (((p + sizeof(long) -1)/sizeof(long)) \
                            * sizeof(long))
#define NEXT_INT_POS(p)    (((p + sizeof(int) -1)/sizeof(int)) \
                            * sizeof(int))
#define NEXT_BOOL_POS(p)   (((p + sizeof(bool) -1)/sizeof(bool)) \
                            * sizeof(bool))
#define NEXT_SHORT_POS(p)    (((p + sizeof(short) -1)/sizeof(short)) \
                            * sizeof(short))
#define NEXT_CHAR_POS(p)   (((p + sizeof(char) -1)/sizeof(char)) \
                            * sizeof(char))

class ChipsId;

class ChipsClient 
{
  public:

   ChipsClient(bool load_prefs=true);
   virtual ~ChipsClient();

   static int     create_client();
   static bool    does_server_exist(const string& inClientId);
   list <string>  list_servers();
   static int     connect_server(const string& inClientId, int attempts=10);
   static int     disconnect_server();
   static int     pack_it(ChipsId* cid, int msgid, void* item1=NULL,
                          void* item2=NULL, void* item3=NULL, 
                          void* item4=NULL, void* item5=NULL,
                          void* item6=NULL, void* item7=NULL,
                          void* item8=NULL, void* item9=NULL,
                          void* item10=NULL, void* item11=NULL);
   static int     send(ChipsId* cid, int msgid, void* item1=NULL,
                       void* item2=NULL, void* item3=NULL,
                       void* item4=NULL, void* item5=NULL,
                       void* item6=NULL, void* item7=NULL,
                       void* item8=NULL, void* item9=NULL,
                       void* item10=NULL, void* item11=NULL);
   static int     get_response(ChipsId* cid, int* bytes=NULL);
   static int     unpack_it(const char* buf, ChipsId* cid);
   static int     get_data(ChipsId* cid, int msgid, 
                       void*** data, char** typ, int** len, int* cnt,
                       void* item1=NULL, void* item2=NULL, void* item3=NULL,
                       void* item4=NULL, void* item5=NULL, void* item6=NULL,
                       void* item7=NULL, void* item8=NULL, void* item9=NULL,
                       void* item10=NULL, void* item11=NULL);

  protected:

  private:

   static double  buffer[1024]; 

   static string serverId;
   static string clientId;
   static string tmpdir;
};


#endif 
