// 
//  Copyright (C) 2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsColor.hh"


#include <stdio.h>

#ifndef NO_CMD_MANAGER
#include "ChipsCmdManager.hh"
extern ChipsCmdManager* cmd_manager;
#endif

// set up static members 
string ChipsColor::fg_name = "white";
float ChipsColor::fg_red = 1.0;
float ChipsColor::fg_green = 1.0;
float ChipsColor::fg_blue = 1.0;
string ChipsColor::bg_name = "black";
float ChipsColor::bg_red = 0.0;
float ChipsColor::bg_green = 0.0;
float ChipsColor::bg_blue = 0.0;


ChipsColor::ChipsColor()
{
   if (!fg_name.length())  set_default_foreground("gold"); 
   if (!bg_name.length())  set_default_background("navy"); 
   name = "";
   red_p = &bg_red;
   green_p = &bg_green;
   blue_p = &bg_blue; 
}


ChipsColor::ChipsColor(string val)
{
   if (set_color(val) != ERR_STATUS_GOOD)
   {
      name = "default";
      red_p = &bg_red;
      green_p = &bg_green;
      blue_p = &bg_blue; 
   }
}


ChipsColor::~ChipsColor()
{

}


int ChipsColor::set_color(string val)
{
   int rval = 0;
   float rr, gg, bb;

   if (strcmp_cis(val.c_str(), "default") == 0)
   {
      name = val;
      red_p = &bg_red;
      green_p = &bg_green;
      blue_p = &bg_blue;
   }
   else if ((rval = get_color_components(val, rr, gg, bb)) == ERR_STATUS_GOOD)
   {
      name = val; 
      my_red = rr;
      my_green = gg;
      my_blue = bb;
      red_p = &my_red;
      green_p = &my_green;
      blue_p = &my_blue;
   }
   return (rval); 
}


int ChipsColor::set_foreground_color(string val)
{
   int rval = 0;
   float rr, gg, bb;

   if (strcmp_cis(val.c_str(), "default") == 0)
   {
      name = val;
      red_p = &fg_red;
      green_p = &fg_green;
      blue_p = &fg_blue;
   }
   else if ((rval = get_color_components(val, rr, gg, bb)) == ERR_STATUS_GOOD)
   {
      name = val; 
      my_red = rr;
      my_green = gg;
      my_blue = bb;
      red_p = &my_red;
      green_p = &my_green;
      blue_p = &my_blue;
   }
   return (rval);
}

int ChipsColor::set_background_color(string val)
{
   int rval = 0;
   float rr, gg, bb;

   if (strcmp_cis(val.c_str(), "default") == 0)
   {
      name = val;
      red_p = &bg_red;
      green_p = &bg_green;
      blue_p = &bg_blue;
   }
   else if ((rval = get_color_components(val, rr, gg, bb)) == ERR_STATUS_GOOD)
   {
      name = val; 
      my_red = rr;
      my_green = gg;
      my_blue = bb;
      red_p = &my_red;
      green_p = &my_green;
      blue_p = &my_blue;
   }
   return (rval);
}


int ChipsColor::set_default_foreground(string val)
{
   int rval = 0;
   float rr, gg, bb;

   if ((rval = get_color_components(val, rr, gg, bb)) == ERR_STATUS_GOOD)
   {
      fg_red = rr;
      fg_green = gg;
      fg_blue = bb;
      fg_name = val; 
   }
   return (rval); 
}


int ChipsColor::set_default_background(string val)
{
   int rval = 0;
   float rr, gg, bb;

   if ((rval = get_color_components(val, rr, gg, bb)) == ERR_STATUS_GOOD)
   {
      bg_red = rr;
      bg_green = gg;
      bg_blue = bb;
      bg_name = val; 
   }
   return (rval); 
}


int ChipsColor::valid_color(string val)
{
   int rval = 0;
   float rr, gg, bb;
 
   if (!strcmp_cis(val.c_str(), "default"))
      rval = ERR_STATUS_GOOD;
   else 
      rval = get_color_components(val, rr, gg, bb);
   return rval;
}


int ChipsColor::hexchar2int(char inchar)
{
   int val; 
   switch(inchar)
   {
      case '0': val = 0; break; 
      case '1': val = 1; break; 
      case '2': val = 2; break; 
      case '3': val = 3; break; 
      case '4': val = 4; break; 
      case '5': val = 5; break; 
      case '6': val = 6; break; 
      case '7': val = 7; break; 
      case '8': val = 8; break; 
      case '9': val = 9; break; 
      case 'a': case 'A': val = 10; break; 
      case 'b': case 'B': val = 11; break; 
      case 'c': case 'C': val = 12; break; 
      case 'd': case 'D': val = 13; break; 
      case 'e': case 'E': val = 14; break; 
      case 'f': case 'F': val = 15; break; 
      default: val = 0; break;
   }
   return val; 
}

int ChipsColor::get_color_components(string val, 
   float& comp_r,    // red color component 
   float& comp_g,    // green color component
   float& comp_b)    // blue color component
{
   int rval = ERR_STATUS_GOOD;

   if (val.length())
   {
      string errstr;
      if (!strcmp_cis(val.c_str(), "red")) 
      { 
         comp_r = 1.0;  
         comp_g = 0.0;  
         comp_b = 0.0; 
      }
      else if (!strcmp_cis(val.c_str(), "green")) 
      { 
         comp_r = 0.0;  
         comp_g = 1.0;  
         comp_b = 0.0; 
      }
      else if (!strcmp_cis(val.c_str(), "blue")) 
      { 
         comp_r = 0.0;  
         comp_g = 0.0;  
         comp_b = 1.0; 
      }
      else if (!strcmp_cis(val.c_str(), "yellow")) 
      { 
         comp_r = 1.0;  
         comp_g = 1.0;  
         comp_b = 0.0; 
      }
      else if (!strcmp_cis(val.c_str(), "orange")) 
      { 
         comp_r = 1.0;  
         comp_g = 0.64; 
         comp_b = 0.0; 
      }
      else if (!strcmp_cis(val.c_str(), "purple")) 
      { 
         comp_r = 0.63; 
         comp_g = 0.13;  
         comp_b = 0.94;
      }
      else if (!strcmp_cis(val.c_str(), "white")) 
      { 
         comp_r = 1.0;  
         comp_g = 1.0;  
         comp_b = 1.0; 
      }
      else if (!strcmp_cis(val.c_str(), "black")) 
      { 
         comp_r = 0.0;  
         comp_g = 0.0;  
         comp_b = 0.0; 
      }
      else if (!strcmp_cis(val.c_str(), "cyan")) 
      { 
         comp_r = 0.0;  
         comp_g = 1.0; 
         comp_b = 1.0;
      }
      else if (!strcmp_cis(val.c_str(), "magenta")) 
      { 
         comp_r = 1.0;  
         comp_g = 0.0;  
         comp_b = 1.0; 
      }
      else if (!strcmp_cis(val.c_str(), "brown")) 
      { 
         comp_r = 0.65; 
         comp_g = 0.16; 
         comp_b = 0.16;
      }
      else if (!strcmp_cis(val.c_str(), "beige"))
      {
         comp_r = 0.96;
         comp_g = 0.96;
         comp_b = 0.86;
      }
      else if (!strcmp_cis(val.c_str(), "gold"))
      {
         comp_r = 1.0;
         comp_g = 0.84;
         comp_b = 0.0;
      }
      else if (!strcmp_cis(val.c_str(), "gray"))
      {
         comp_r = 0.75;
         comp_g = 0.75;
         comp_b = 0.75;
      }
      else if (!strcmp_cis(val.c_str(), "pink"))
      {
         comp_r = 1.0;
         comp_g = 0.75;
         comp_b = 0.8;
      }
      else if (!strcmp_cis(val.c_str(), "ivory"))
      {
         comp_r = 1.0;
         comp_g = 1.0;
         comp_b = 0.94;
      }
      else if (!strcmp_cis(val.c_str(), "navy"))
      {
         comp_r = 0.0;
         comp_g = 0.0;
         comp_b = 0.5;
      }
      else if (!strcmp_cis(val.c_str(), "khaki"))
      {
         comp_r = 0.94;
         comp_g = 0.90;
         comp_b = 0.55;
      }
      else if (!strcmp_cis(val.c_str(), "salmon"))
      {
         comp_r = 0.98;
         comp_g = 0.50;
         comp_b = 0.45;
      }
      else if (!strcmp_cis(val.c_str(), "coral"))
      {
         comp_r = 1.0;
         comp_g = 0.5;
         comp_b = 0.5;
      }
      else if (!strcmp_cis(val.c_str(), "lime"))
      {
         comp_r = 0.2;
         comp_g = 0.8;
         comp_b = 0.2;
      }
      else if (!strcmp_cis(val.c_str(), "plum"))
      {
         comp_r = 0.87;
         comp_g = 0.63;
         comp_b = 0.87;
      }
      else if (!strcmp_cis(val.c_str(), "tan"))
      {
         comp_r = 0.82;
         comp_g = 0.71;
         comp_b = 0.55;
      }
      else if (!strcmp_cis(val.c_str(), "orchid"))
      {
         comp_r = 0.85;
         comp_g = 0.44;
         comp_b = 0.84;
      }
      else if (!strcmp_cis(val.c_str(), "thistle"))
      {
         comp_r = 0.85;
         comp_g = 0.75;
         comp_b = 0.85;
      }
      else if (!strcmp_cis(val.c_str(), "linen"))
      {
         comp_r = 0.98;
         comp_g = 0.94;
         comp_b = 0.90;
      }
      else if (!strcmp_cis(val.c_str(), "peach"))
      {
         comp_r = 1.0;
         comp_g = 0.85;
         comp_b = 0.73;
      }
      else if (!strcmp_cis(val.c_str(), "azure"))
      {
         comp_r = 0.94;
         comp_g = 1.0;
         comp_b = 1.0;
      }
      else if (!strcmp_cis(val.c_str(), "forest"))
      {
         comp_r = 0.13;
         comp_g = 0.55;
         comp_b = 0.13;
      }
      else if (!strcmp_cis(val.c_str(), "olive"))
      {
         comp_r = 0.42;
         comp_g = 0.56;
         comp_b = 0.14;
      }
      else if (!strcmp_cis(val.c_str(), "lavender"))
      {
         comp_r = 0.9;
         comp_g = 0.9;
         comp_b = 0.99;
      }
      else if (!strcmp_cis(val.c_str(), "snow"))
      {
         comp_r = 1.0;
         comp_g = 0.98;
         comp_b = 0.98;
      }
      else if (!strcmp_cis(val.c_str(), "bisque"))
      {
         comp_r = 1.0;
         comp_g = 0.89;
         comp_b = 0.77;
      }
      else if (!strcmp_cis(val.c_str(), "honeydew"))
      {
         comp_r = 0.94;
         comp_g = 1.0;
         comp_b = 0.94;
      }
      else if (!strcmp_cis(val.c_str(), "turquoise"))
      {
         comp_r = 0.25;
         comp_g = 0.88;
         comp_b = 0.82;
      }
      else if (!strcmp_cis(val.c_str(), "aquamarine"))
      {
         comp_r = 0.50;
         comp_g = 1.0;
         comp_b = 0.89;
      }
      else if (!strcmp_cis(val.c_str(), "seagreen"))
      {
         comp_r = 0.18;
         comp_g = 0.55;
         comp_b = 0.34;
      }
      else if (!strcmp_cis(val.c_str(), "steelblue"))
      {
         comp_r = 0.27;
         comp_g = 0.51;
         comp_b = 0.71;
      }
      else if (!strcmp_cis(val.c_str(), "cornflower"))
      {
         comp_r = 0.39;
         comp_g = 0.58;
         comp_b = 0.93;
      }
      else if (!strcmp_cis(val.c_str(), "skyblue"))
      {
         comp_r = 0.53;
         comp_g = 0.81;
         comp_b = 0.92;
      }
      else if (!strcmp_cis(val.c_str(), "goldenrod"))
      {
         comp_r = 0.85;
         comp_g = 0.65;
         comp_b = 0.13;
      }
      else if (!strcmp_cis(val.c_str(), "sienna"))
      {
         comp_r = 0.63;
         comp_g = 0.32;
         comp_b = 0.18;
      }
      else if (!strcmp_cis(val.c_str(), "maroon"))
      {
         comp_r = 0.69;
         comp_g = 0.19;
         comp_b = 0.38;
      }
      else if (!strcmp_cis(val.c_str(), "violet"))
      {
         comp_r = 0.94;
         comp_g = 0.51;
         comp_b = 0.94;
      }
      else if (!strcmp_cis(val.c_str(), "mistyrose"))
      {
         comp_r = 1.0;
         comp_g = 0.89;
         comp_b = 0.88;
      }
      else if (!strcmp_cis(val.c_str(), "tomato"))
      {
         comp_r = 1.0;
         comp_g = 0.39;
         comp_b = 0.28;
      }
      else if (!strcmp_cis(val.c_str(), "darkred"))   // 8b0000
      {
         comp_r = 0.55;
         comp_g = 0.00;
         comp_b = 0.00;
      }
      else if (!strcmp_cis(val.c_str(), "firebrick"))
      {
         comp_r = 0.70;
         comp_g = 0.12;
         comp_b = 0.12;
      }
      else if (!strcmp_cis(val.c_str(), "chocolate"))
      {
         comp_r = 0.82;
         comp_g = 0.41;
         comp_b = 0.12;
      }
      else if (!strcmp_cis(val.c_str(), "wheat"))  // f5deb3
      {
         comp_r = 0.96;
         comp_g = 0.87;
         comp_b = 0.70;
      }
      else if (!strcmp_cis(val.c_str(), "mediumblue"))  // 0000cd
      {
         comp_r = 0.00;
         comp_g = 0.00;
         comp_b = 0.80;
      }
      else if (!strcmp_cis(val.c_str(), "peru"))
      {
         comp_r = 0.80;
         comp_g = 0.52;
         comp_b = 0.25;
      }
      else if (!strcmp_cis(val.c_str(), "chartreuse"))
      {
         comp_r = 0.50;
         comp_g = 1.00;
         comp_b = 0.00;
      }
      else if (!strcmp_cis(val.c_str(), "springgreen"))  // 00ff7f
      {
         comp_r = 0.00;
         comp_g = 1.00;
         comp_b = 0.50;
      }
      else if (!strcmp_cis(val.c_str(), "seashell"))
      {
         comp_r = 1.00;
         comp_g = 0.96;
         comp_b = 0.93;
      }
      else if (!strcmp_cis(val.c_str(), "slateblue"))
      {
         comp_r = 0.42;
         comp_g = 0.35;
         comp_b = 0.80;
      }
      else if (!strcmp_cis(val.c_str(), "slategray"))
      {
         comp_r = 0.44;
         comp_g = 0.50;
         comp_b = 0.56;
      }
      else if (val.length() == 6)
      {
         // verify that string only contains hex digits 
         if (isxdigit(val[0]) && isxdigit(val[1]) && isxdigit(val[2]) && 
             isxdigit(val[3]) && isxdigit(val[4]) && isxdigit(val[5]))
         {
            // convert hex rrggbb to composites
            comp_r = (float)((hexchar2int(val[0]) << 4) + 
                                hexchar2int(val[1])) / 255.0;
            comp_g = (float)((hexchar2int(val[2]) << 4) +
                                hexchar2int(val[3])) / 255.0;
            comp_b = (float)((hexchar2int(val[4]) << 4) + 
                                hexchar2int(val[5])) / 255.0;
         }
         else
	 {
	    rval = ERR_COLOR_NONHEX_DIGIT;
#ifndef NO_CMD_MANAGER
	    errstr = "The color '";
	    errstr.append(val);
	    errstr.append("' is unknown or not a six digit hex specification.");
	    cmd_manager->add_err(errstr, ERR_COLOR_NONHEX_DIGIT);
#endif
         }
      }
      else
      {
	 rval = ERR_UNKNOWN_COLOR;
#ifndef NO_CMD_MANAGER
         if (cmd_manager)
	 {
	    errstr = "The color '";
	    errstr.append(val);
	    errstr.append("' is unknown or not supported");
            cmd_manager->add_err(errstr, ERR_UNKNOWN_COLOR);
	 }
#endif
      }
   }
   else
   {
      rval = ERR_UNKNOWN_COLOR; 
#ifndef NO_CMD_MANAGER
      if (cmd_manager)
         cmd_manager->add_err(" An empty string was specified for color.", 
	    ERR_UNKNOWN_COLOR);
#endif
   }
   return rval; 
}




int ChipsColor::strcmp_cis(const char *aa,  /* i: input string */
                 const char *bb)  /* i: input string */
{
   unsigned int ii;   /* loop variable */
   int zz = 1;        /* comparison status */
   if (aa && bb)
   {
      if ((ii = strlen(aa)) == strlen(bb))
      {
	 zz = 0;
	 while (ii-- && !zz)  
	    if (tolower(aa[ii]) != tolower(bb[ii]))   zz = 1;
      }
   }
   return(zz);
}



