// 
//  Copyright (C) 2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsColorBar.hh"
#include "ChipsFontList.hh"
#include "ChipsPlot.hh"

#include <math.h>
#include <float.h>
#include <stdio.h>
#include <sstream>
#include "ChipsCmdManager.hh"
#include "ChipsColorMapTable.hh"
#include "ChipsPlotSystem.hh"

extern ChipsDefaults* theDefaults; 
extern ChipsFontList* theFonts;
extern ChipsCmdManager* cmd_manager;
extern ChipsColorMapTable *colormap_table;


ChipsColorBar::ChipsColorBar(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, ChipsStateStream& in):ChipsObject(par, "")
{
   double range[2];

   initialized = false;
   initialize();
   deserialize(in);

   post_initialize (inCoord, coord_system);

   colormap_table->get_colormap_scalar_range (colormap_id, range);
   this->calculate_tick_positions (range[0], range[1]);
}

ChipsColorBar::ChipsColorBar(ChipsObject* par, 
   ChipsCoordinate *inCoord, COORD_SYS coord_sys, 
   double xpos, double ypos, string val):ChipsObject(par, val)
{
   initialized = false;
   
   double range[] = {0,1};
   colormap_id = colormap_table->add_colormap ();
   colormap_table->set_colormap_type (colormap_id, 
      ((V_COLORMAP_TYPE)chips_grayscale));
   colormap_table->set_colormap_scalar_range (colormap_id, range);


   initialize();
   orig = ChipsVector (xpos, ypos);
   norm_pos = ChipsVector (0, 0);
   
   post_initialize (inCoord, coord_sys);

   calculate_tick_positions (range[0], range[1]);

}

ChipsColorBar::ChipsColorBar(ChipsObject* par, 
   ChipsCoordinate *inCoord, COORD_SYS coord_sys, 
   double xpos, double ypos, unsigned short cmap_id,
   string val):ChipsObject(par, val)
{
   double range[2];

   initialized = false;
   
   colormap_id = cmap_id;

   initialize();
   orig = ChipsVector (xpos, ypos);
   norm_pos = ChipsVector (0, 0);
   post_initialize (inCoord, coord_sys);

   colormap_table->get_colormap_scalar_range (colormap_id, range);
   this->calculate_tick_positions (range[0], range[1]);
}


ChipsColorBar::~ChipsColorBar()
{
   if (tickpos && tickcnt)
   {
      free(tickpos);
      tickpos = NULL;
      tickcnt = 0;
   }
   if (coord) coord->remove (this);
}

void ChipsColorBar::object_delete ()
{
   delete this;
}

void ChipsColorBar::initialize()
{  
   int tmp_orient;
   bar_flip = false;
   flipped = false; 
   tickpos = NULL;
   tickcnt = 0; 
   ticklabfontfile = "DEFAULT_TIMES";
   label_format = NULL;
   orient = 0;
   if (theDefaults)
   {
      horiz_alignment = theDefaults->get_colorbarhalign();
      vert_alignment = theDefaults->get_colorbarvalign();
      depth      = theDefaults->get_colorbardepth(); 
      tmp_orient = theDefaults->get_colorbarorient();
      border_vis = theDefaults->get_colorbarbordervis();
      width = theDefaults->get_colorbarwidth ();
      length = theDefaults->get_colorbarlength ();
      label_text = theDefaults->get_colorbarlabtext ();
      label_angle = theDefaults->get_colorbarlabangle();
      label_size = theDefaults->get_colorbarlabsize();
      set_label_color (theDefaults->get_colorbarlabcolor());
      label_font = theDefaults->get_colorbarlabfont();
      label_fontstyle = theDefaults->get_colorbarlabfontstyle();
      set_label_halign (theDefaults->get_colorbarlabhalign()); 
      set_label_valign (theDefaults->get_colorbarlabvalign()); 
      set_label_location (theDefaults->get_colorbarlabloc ());
      offset_par = theDefaults->get_colorbarparalleloffset ();
      offset_perp = theDefaults->get_colorbarperpendicularoffset();

      tickmode = theDefaults->get_colorbartickmode ();
      ticklocation = theDefaults->get_colorbartickloc();
      tickstyle = theDefaults->get_colorbartickstyle();
      tickthickness = theDefaults->get_colorbartickthickness(); 
      ticklength = theDefaults->get_colorbarticklength(); 
      tickcolor.set_foreground_color (theDefaults->get_colorbartickcolor());

      ticklabcolor.set_foreground_color (theDefaults->get_colorbarticklabcolor());
      ticklabfont = theDefaults->get_colorbarticklabfont();
      ticklabfontstyle = theDefaults->get_colorbarticklabfontstyle();
      ticklabangle = theDefaults->get_colorbarticklabangle();
      ticklabsize = theDefaults->get_colorbarticklabsize();
      ticklaboffset = theDefaults->get_colorbarticklaboffset();
      set_ticklabel_valign (theDefaults->get_colorbarticklabvalign());
      set_ticklabel_halign (theDefaults->get_colorbarticklabhalign());
      labvis = theDefaults->get_colorbarlabvisible ();
      ticklabvis = theDefaults->get_colorbarticklabvisible ();
      tickvis = theDefaults->get_colorbartickvisible ();

   }
   else
   {
      set_tick_style(chips_inside);
      set_label_location(chips_inside);
      tmp_orient = 0;
      tickthickness = 1.0; 
      ticklength = 4; 
      tickcolor.set_foreground_color ("white");
      label_text = "";
      label_angle = 0; 
      label_size = 10;
      set_label_color ("yellow");
      label_font = "helvetica";
      label_fontstyle = chips_normal; 
      set_label_halign(0.0);
      set_label_halign(0.5);
      offset_par = 0;
      offset_perp = 5;
      set_ticklabel_color("white");
      tickmode = 0;
      ticklocation = chips_outside; 
      ticklaboffset = 2;
      ticklabangle = 0.0;
      ticklabsize = 10;
      ticklabfont = "helvetica";
      ticklabfontstyle = chips_normal;
      depth = V_USE_DEFAULT_DEPTH;
      border_vis = true;
      width = .1;
      length = 1;
      labvis = true;
      ticklabvis = true;
      tickvis = true;
   }
   if (depth == V_USE_DEFAULT_DEPTH)  depth = ((ChipsPlot*)parent)->get_depth();
   nx1 = ny1 = 0.0;
   nx2 = ny2 = 1.0;
   cbar_num = 0;
   initialized = true;

   set_orientation (tmp_orient);
   set_tick_mode(tickmode); 
   set_label_visible(labvis); 
   set_ticklabel_visible(ticklabvis); 
   set_ticklabel_font (ticklabfont);
   set_tick_visible (tickvis);
   update_label_text ();

   object_type = chips_colorbar;

   coord = NULL;
}

void ChipsColorBar::post_initialize (ChipsCoordinate *inCoord, COORD_SYS coord_sys)
{
   // Cant process if there's no chips coordinate
   if (inCoord)
   {
      coord = inCoord;
      coord_system = coord_sys;
      // This is temporarily hard coded until a better method of searching
      // down the coordinate system tree is implemented. Otherwise objects
      // in window system must be kept in the frame coord
      if ((coord_sys == WINDOW_NORM) || (coord_sys == PIXEL) ||
          (coord_sys == PIXEL_REAL))
         data_system_id = "";
      else
         data_system_id = coord->get_id ();
      status = coord->insert (this);

      // convert the data to logical coordinates if necessary
      if (coord_sys == DATA)
      {
         ChipsDataSystem *ds_coord = dynamic_cast <ChipsDataSystem *>(coord);
         if (ds_coord->get_default_transform_id () != "")
         {
            data_system_id = ds_coord->get_id_no_wcs ();
            ds_coord->convert_cs_to_cs (&orig, &orig, 1, DATA, "", DATA, data_system_id);
         }

      }
   }
   else
      cmd_manager->add_err(status=ERR_COORD_DNE, id, 1, coord_sys);

   if (!status)
      status = calculate_norm_coordinate (orig, norm_pos);

   // Set the axis direction
   invert_labels();

   // reset the width and height since they deal with coords
   set_width (width);
   set_length (length);

}


int ChipsColorBar::set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj)
{
   int rval = ERR_STATUS_GOOD;

   if ((obj == chips_window) || (obj == chips_frame) || (obj == chips_plot) ||
       (obj == chips_colorbar))
   {
      int mask = theDefaults->get_allpropmask();

      if ((mask & PREF_ALL_COLOR) == PREF_ALL_COLOR)
      {
         string clr = theDefaults->get_allcolor();

         // set plot properties
         rval |= set_property((void*)clr.c_str(), SET_CBAR_TICK_COLOR);
         rval |= set_property((void*)clr.c_str(), SET_CBAR_LABEL_COLOR);
         rval |= set_property((void*)clr.c_str(), SET_CBAR_TICKLABEL_COLOR);
      }
      else if ((mask & PREF_ALL_FONT) == PREF_ALL_FONT)
      {
         string newfont = theDefaults->get_allfont();
         rval =  set_property((void*)newfont.c_str(), SET_CBAR_LABEL_FONT);
         rval |=  set_property((void*)newfont.c_str(), SET_CBAR_TICKLABEL_FONT);
      }
      else if ((mask & PREF_ALL_FONTSTYLE) == PREF_ALL_FONTSTYLE)
      {
         V_FONTSTYLE_TYPE newfontstyle = theDefaults->get_allfontstyle();
         string style = v_fontstyle_to_string(newfontstyle);
         rval =  set_property((void*)style.c_str(), SET_CBAR_LABEL_FONTSTYLE);
         rval |=  set_property((void*)style.c_str(), SET_CBAR_TICKLABEL_FNTSTL);
      }
      else if ((mask & PREF_ALL_VISIBLE) == PREF_ALL_VISIBLE)
      {
         int vis = theDefaults->get_allvisible();
         rval = set_property(static_cast<void*>(&vis), SET_CBAR_VISIBLE); 
      }
      else if ((mask & PREF_ALL_ANGLE) == PREF_ALL_ANGLE)
      {
         float newangle = theDefaults->get_allangle();
         rval = set_property(static_cast<void*>(&newangle), SET_CBAR_LABEL_ANGLE);
         rval |= set_property(static_cast<void*>(&newangle), SET_CBAR_TICKLABEL_ANGLE);
      }
      else if ((mask & PREF_ALL_THICKNESS) == PREF_ALL_THICKNESS)
      {
         float thick = theDefaults->get_allthickness();
         rval = set_property(static_cast<void*>(&thick), SET_CBAR_TICK_THICKNESS);
      }
      else if ((mask & PREF_ALL_DEPTH) == PREF_ALL_DEPTH)
      {
	 int dep = theDefaults->get_alldepth();
         rval = set_property(static_cast<void*>(&dep), SET_CBAR_DEPTH);
      }
      else if ((mask & PREF_ALL_LENGTH) == PREF_ALL_LENGTH)
      {
         int newlength = theDefaults->get_alllength();
         rval = set_property(static_cast<void*>(&newlength), SET_CBAR_TICK_LENGTH);
      }

      if (cmd_manager && !rval)
         cmd_manager->update_mask (STATE_CHNG_PROP_CLBAR);
   }
   return rval;
}

int ChipsColorBar::set_property(void* in_val, int property, bool rec)
{
   int rval = ERR_STATUS_GOOD;
   ChipsSetCommand *cmd = NULL;
   string save_current = ((ChipsPlot*)parent)->get_current_colorbar(); 
   switch(property)
   {
      case SET_CBAR_HALIGN:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_horizalignment(), *fval, save_current);
         if ((rval = set_horizalignment(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;

      }
      break;
      case SET_CBAR_VALIGN:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_vertalignment(), *fval, save_current);
         if ((rval = set_vertalignment(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;

      }
      break;
      case SET_CBAR_BORDERVIS:
      {
         bool* bval = static_cast<bool*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_border_visible(), *bval, save_current);
         if ((rval = set_border_visible(*bval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_ORIENTATION:
      {
         int* ival = static_cast<int*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_orientation(), *ival, save_current);
         if ((rval = set_orientation(*ival)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_WIDTH:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_width(), *fval, save_current);
         if ((rval = set_width(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_LENGTH:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_length(), *fval, save_current);
         if ((rval = set_length(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TEXT:
      {
	 string val = (const char*)in_val; 
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_label_text(), val, save_current);
         if ((rval = set_label_text(val)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_LABEL_ANGLE:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_label_angle(), *fval, save_current);
         if ((rval = set_label_angle(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_LABEL_HALIGN:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_label_halign(), *fval, save_current);
         if ((rval = set_label_halign(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_LABEL_VALIGN:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_label_valign(), *fval, save_current);
         if ((rval = set_label_valign(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICK_STYLE:
      {
         int* ival = static_cast<int*>(in_val);
	 int oval = (int)tickstyle; 
         if (rec)  cmd = new ChipsSetCommand(this, SET_CBAR_TICK_STYLE,
            oval, *ival, save_current);
         if ((rval = set_tick_style((V_TICKSTYLE_TYPE)*ival)) == 
	    ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_VISIBLE:
      {
         int* ival = static_cast<int*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            (int)visible, *ival, save_current);
         if (*ival == 1)
         {
            if (initialized)  rval = show();
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (*ival == 0)
         {
            if (initialized)  rval = hide();
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else 
	 {
            cmd_manager->add_err(rval=ERR_INT_MINMAX, id, 0, "cbar visibility", *ival,0, 1);
	    if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
      }
      break;
      case SET_CBAR_DEPTH:
      {
         int*   ival  =  static_cast<int*>(in_val);
         int newval;
         if ((*ival) == V_USE_DEFAULT_DEPTH)    
            newval = ((ChipsPlot*)parent)->get_depth();
         else
            newval = *ival;
         if (rec)
         {
            cmd = new ChipsSetCommand(this, property,
              (int)depth, newval, save_current);
            ((ChipsPlot*)parent)->remove_object_from_depth_table(this);
         }
         if ((rval = set_depth(newval)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command(cmd, rval))   delete cmd;
         }
         else  delete cmd;
         if (rec) ((ChipsPlot*)parent)->add_object_to_depth_table(this);
      }
      break;
      case SET_CBAR_LABEL_COLOR:
      {
         string clr = (const char*)in_val;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_label_color(), clr, save_current);
         if ((rval = set_label_color(clr)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_COLOR:
      {
         string clr = (const char*)in_val;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_ticklabel_color(), clr, save_current);
         if ((rval = set_ticklabel_color(clr)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_OFFSET:
      {
         int* ival = static_cast<int*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            ticklaboffset, *ival, save_current);
         if ((rval = set_ticklabel_offset(*ival)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_SIZE:
      {
         int* ival = static_cast<int*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            ticklabsize, *ival, save_current);
         if ((rval = set_ticklabel_size(*ival)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_ANGLE:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            ticklabangle, *fval, save_current);
         if ((rval = set_ticklabel_angle(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_HALIGN:
      {
         float* fval = static_cast<float*>(in_val);
	 float oval = ticklabhorizalignment;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            oval, *fval, save_current);
         if ((rval = set_ticklabel_halign(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_VALIGN:
      {
         float* fval = static_cast<float*>(in_val);
	 float oval = ticklabvertalignment;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            oval, *fval, save_current);
         if ((rval = set_ticklabel_valign(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_VIS:
      {
         bool* bval = static_cast<bool*>(in_val);
         bool oval = ticklabvis;
         if (rec)  cmd = new ChipsSetCommand(this, property,
              oval, *bval, save_current);
         if ((rval = set_ticklabel_visible(*bval)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)
            delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_FONT:
      {
         string val = (const char*)in_val;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_ticklabel_font(), val, save_current);
         if ((rval = set_ticklabel_font(val)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICKLABEL_FNTSTL:
      {
         string val = (const char*)in_val;
	 string oval = v_fontstyle_to_string(get_ticklabel_fontstyle());
         if (rec)  cmd = new ChipsSetCommand(this, property,
            oval, val, save_current);
         if ((rval = set_ticklabel_fontstyle(val)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break; 
      case SET_CBAR_LABEL_SIZE:
      {
         int* ival = static_cast<int*>(in_val);
	 int oval = get_label_size();
         if (rec)  cmd = new ChipsSetCommand(this, property,
            oval, *ival, save_current);
         if ((rval = set_label_size(*ival)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_LABEL_FONT:
      {
         string val = (const char*)in_val;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_label_font(), val, save_current);
         if ((rval = set_label_font(val)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_LABEL_VIS:
      {
         bool* bval = static_cast<bool*>(in_val);
         bool oval = labvis;
         if (rec)  cmd = new ChipsSetCommand(this, property,
              oval, *bval, save_current);
         if ((rval = set_label_visible(*bval)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)
            delete cmd;
      }
      break;
      case SET_CBAR_LABEL_LOCATION:
      {
         int* ival = static_cast<int*>(in_val);
         int oval = (int)label_location;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            oval, *ival, save_current);
         if ((rval = set_label_location((V_TICKSTYLE_TYPE)*ival)) ==
            ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_LABEL_FONTSTYLE:
      {
         string val = (const char*)in_val;
	 string oval = v_fontstyle_to_string(get_label_fontstyle());
         if (rec)  cmd = new ChipsSetCommand(this, property,
            oval, val, save_current);
         if ((rval = set_label_fontstyle(val)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICK_MODE:
      {
         string val = (const char*)in_val;
         string oval;
         if (tickmode == 0)
            oval = "none";
         else if (tickmode == 2)
            oval = "nice";
         else
            oval = "ends";
         if (rec)  cmd = new ChipsSetCommand(this, property,
            oval, val, save_current);
         if ((rval = set_tick_mode(val)) == ERR_STATUS_GOOD)
         {
            if (rec && !add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICK_THICKNESS:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_tick_thickness (), *fval, save_current);
         if ((rval = set_tick_thickness(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;

      case SET_CBAR_TICK_LOCATION:
      {
         int* ival = static_cast<int*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_tick_location (), *ival, save_current);
         if ((rval = set_tick_location((V_TICKSTYLE_TYPE)*ival)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      } 
      break;
      case SET_CBAR_TICK_VIS:
      {
         bool* bval = static_cast<bool*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_tick_visible (), *bval, save_current);
         if ((rval = set_tick_visible(*bval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_TICK_COLOR:
      {
         string clr = (const char*)in_val;
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_tick_color(), clr, save_current);
         if ((rval = set_tick_color(clr)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;

      }
      break;
      case SET_CBAR_TICK_LENGTH:
      {
         int* ival = static_cast<int*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            get_tick_length (), *ival, save_current);
         if ((rval = set_tick_length(*ival)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;

      }
      break;
      case SET_CBAR_OFFSET_PAR:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            offset_par, *fval, save_current);
         if ((rval = set_parallel_offset(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      case SET_CBAR_OFFSET_PERP:
      {
         float* fval = static_cast<float*>(in_val);
         if (rec)  cmd = new ChipsSetCommand(this, property,
            offset_perp, *fval, save_current);
         if ((rval = set_perpendicular_offset(*fval)) == ERR_STATUS_GOOD)
         {
            if (rec)
              if (!add_command (cmd, rval))  delete cmd;
         }
         else if (cmd)  delete cmd;
      }
      break;
      default:
         cmd_manager->add_err (rval=ERR_UNKNOWN_PROPERTY,id, 0, property);
      break;
   }

   // need to add object id for errors
   if (cmd_manager && rval)  cmd_manager->add_object_id_to_errs(id);

   if (cmd_manager && !rval)
      cmd_manager->update_mask (STATE_CHNG_PROP_CLBAR);


   return rval;
}

int ChipsColorBar::get_property(void* out_val, int property)
{
   int rval = ERR_STATUS_GOOD;

   switch(property)
   {
      case SET_CBAR_HALIGN:
      {
         float *fv = static_cast<float*> (out_val);
         *fv = get_horizalignment ();
      }
      break;
      case SET_CBAR_VALIGN:
      {
         float *fv = static_cast<float*> (out_val);
         *fv = get_vertalignment ();
      }
      break;
      case SET_CBAR_BORDERVIS:
      {
         bool *bv = static_cast<bool*> (out_val);
         *bv = get_border_visible ();
      }
      break;
      case SET_CBAR_ORIENTATION:
      {
         int *iv = static_cast<int*> (out_val);
         *iv = get_orientation ();
      }
      break;
      case SET_CBAR_WIDTH:
      {
         float *fv = static_cast<float*> (out_val);
         *fv = get_width ();
      }
      break;
      case SET_CBAR_LENGTH:
      {
         float *fv = static_cast<float*> (out_val);
         *fv = get_length ();
      }
      break;
      case SET_CBAR_LABEL_ANGLE:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_label_angle();
      }
      break;
      case SET_CBAR_LABEL_HALIGN:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_label_halign();
      }
      break;
      case SET_CBAR_LABEL_VALIGN:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_label_valign();
      }
      break;
      case SET_CBAR_TEXT:
      {
         string *sval =  (string*)out_val;
         *sval = get_label_text();
      }
      break;
      case SET_CBAR_DEPTH:
      {
         int *iv = (int*)out_val;
         *iv = get_depth();
      }
      break;
      case SET_CBAR_TICK_MODE:
      {
         string *sval =  (string*)out_val;
         if (tickmode == 0)
            *sval = "none";
         else if (tickmode == 2)
            *sval = "nice";
         else 
            *sval = "ends";
      }
      break;
      case SET_CBAR_TICK_THICKNESS:
      {
         float *fv = (float*)out_val;
         *fv = get_tick_thickness();
      }
      break;
      case SET_CBAR_TICK_STYLE:
      {
         string *sval =  (string*)out_val;
         *sval = v_tickstyle_to_string(tickstyle);
      }
      break;
      case SET_CBAR_TICK_LOCATION:
      {
         string *sval =  (string*)out_val;
         *sval = v_tickstyle_to_string(ticklocation);
      }
      break;
      case SET_CBAR_TICK_VIS:
      {
         bool *bv = (bool*)out_val;
         *bv = get_tick_visible();
      }
      break;
      case SET_CBAR_TICK_COLOR:
      {
         string *sval =  (string*)out_val;
         *sval = get_tick_color();
      }
      break;
      case SET_CBAR_TICK_LENGTH:
      {
         int *iv = (int*)out_val;
         *iv = get_tick_length();
      }
      break;
      case SET_CBAR_LABEL_COLOR:
      {
         string *sval =  (string*)out_val;
         *sval = get_label_color();
      }
      break;
      case SET_CBAR_LABEL_VIS:
      {
         int *iv = (int*)out_val;
         *iv = get_label_visible();
      }
      break;
      case SET_CBAR_LABEL_LOCATION:
      {
         string *sval =  (string*)out_val;
         *sval = v_tickstyle_to_string(get_label_location());
      }
      break;
      case SET_CBAR_TICKLABEL_COLOR:
      {
         string *sval =  (string*)out_val;
         *sval = get_ticklabel_color();
      }
      break;
      case SET_CBAR_TICKLABEL_OFFSET:
      {
         int *iv = (int*)out_val;
         *iv = get_ticklabel_offset();
      }
      break;
      case SET_CBAR_TICKLABEL_SIZE:
      {
         int *iv = (int*)out_val;
         *iv = get_ticklabel_size();
      }
      break;
      case SET_CBAR_TICKLABEL_ANGLE:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_ticklabel_angle();
      }
      break;
      case SET_CBAR_TICKLABEL_HALIGN:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_ticklabel_halign();
      }
      break;
      case SET_CBAR_TICKLABEL_VALIGN:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_ticklabel_valign();
      }
      break;
      case SET_CBAR_TICKLABEL_VIS:
      {
         int *iv = (int*)out_val;
         *iv = get_ticklabel_visible();
      }
      break;
      case SET_CBAR_TICKLABEL_FONT:
      {
         string *sval =  (string*)out_val;
         *sval = get_ticklabel_font();
      }
      break;
      case SET_CBAR_TICKLABEL_FNTSTL:
      {
         string *sval =  (string*)out_val;
         *sval = v_fontstyle_to_string(get_ticklabel_fontstyle());
      }
      break; 
      case SET_CBAR_LABEL_SIZE:
      {
         int *iv = (int*)out_val;
         *iv = get_label_size();
      }
      break;
      case SET_CBAR_LABEL_FONT:
      {
         string *sval =  (string*)out_val;
         *sval = get_label_font();
      }
      break;
      case SET_CBAR_LABEL_FONTSTYLE:
      {
         string *sval =  (string*)out_val;
         *sval = v_fontstyle_to_string(get_label_fontstyle());
      }
      break;
      case SET_CBAR_VISIBLE:
      {
         int *iv = (int*)out_val;
         *iv = is_visible();
      }
      break;
      case SET_CBAR_OFFSET_PAR:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_parallel_offset();
      }
      break;
      case SET_CBAR_OFFSET_PERP:
      {
         float* fv = static_cast<float*>(out_val);
         *fv = get_perpendicular_offset();
      }
      break;

      default:
         cmd_manager->add_err (rval=ERR_UNKNOWN_PROPERTY,id, 0, property);
      break;
   }
   return rval;
}

float ChipsColorBar::get_width ()
{
   return width;
}

int ChipsColorBar::set_width (float val)
{
   int rval = ERR_STATUS_GOOD;

   if (val > 0)
   {
      width = val;

      if (initialized)
      {

         ChipsVector low_bound(orig);
         ChipsVector up_bound(orig);
         string cd_id = this->data_system_id;

         if (orient == 0)
         {
            low_bound[0] = orig[0] - (length * horiz_alignment);
            low_bound[1] = orig[1] - (width * vert_alignment);
            up_bound[0] = low_bound[0] + length;
            up_bound[1] = low_bound[1] + width;
         }
         else
         {
            low_bound[0] = orig[0] - (width * vert_alignment);
            low_bound[1] = orig[1] - (length * horiz_alignment);
            up_bound[0] = low_bound[0] + width;
            up_bound[1] = low_bound[1] + length;
         }

         if (coord_system == DATA)
            cd_id = (dynamic_cast<ChipsDataSystem *>(coord))->get_id_no_wcs ();

         coord->convert_cs_to_cs (&low_bound, &low_bound, 1,
                            coord_system, cd_id,
                            FRAME_NORM, "");
         coord->convert_cs_to_cs (&up_bound, &up_bound, 1,
                            coord_system, cd_id,
                            FRAME_NORM, "");

         // if horizontal then the length is horizontal
         if (orient == 0)
         {
            frame_length = up_bound[0] - low_bound[0];
            frame_width = up_bound[1] - low_bound[1];
         }
         else
         {
            frame_length = up_bound[1] - low_bound[1];
            frame_width = up_bound[0] - low_bound[0];
         }

         rval =  change_size();
      }
   }
   else
      cmd_manager->add_err (rval=ERR_FLT_MIN,id, 0, "colorbar width",val,0);
   return rval;
}


float ChipsColorBar::get_length ()
{
   return length;
}

int ChipsColorBar::set_length (float val)
{
   int rval = ERR_STATUS_GOOD;

   if (val > 0)
   {
      length = val;

      if (initialized)
      {
         ChipsVector low_bound(orig);
         ChipsVector up_bound(orig);
         string cd_id = this->data_system_id;

         if (orient == 0)
         {
            low_bound[0] = orig[0] - (length * horiz_alignment);
            low_bound[1] = orig[1] - (width * vert_alignment);
            up_bound[0] = low_bound[0] + length;
            up_bound[1] = low_bound[1] + width;
         }
         else
         {
            low_bound[0] = orig[0] - (width * vert_alignment);
            low_bound[1] = orig[1] - (length * horiz_alignment);
            up_bound[0] = low_bound[0] + width;
            up_bound[1] = low_bound[1] + length;
         }

         if (coord_system == DATA)
            cd_id = (dynamic_cast<ChipsDataSystem *>(coord))->get_id_no_wcs ();

         coord->convert_cs_to_cs (&low_bound, &low_bound, 1,
                            coord_system, cd_id, 
                            FRAME_NORM, "");
         coord->convert_cs_to_cs (&up_bound, &up_bound, 1,
                            coord_system, cd_id,
                            FRAME_NORM, "");

         // if horizontal then the length is horizontal
         if (orient == 0)
         {
            frame_length = up_bound[0] - low_bound[0];
            frame_width = up_bound[1] - low_bound[1];
         }
         else
         {
            frame_length = up_bound[1] - low_bound[1];
            frame_width = up_bound[0] - low_bound[0];
         }

         rval =  change_size();
      }

   }
   else
      cmd_manager->add_err (rval=ERR_FLT_MIN,id, 0, "colorbar length",val,0);
   return rval;
}

int ChipsColorBar::set_orientation  (int val)
{
   int rval = ERR_STATUS_GOOD;

   if ((val == 0) || (val == 1))
   {
      bool orient_changed = !(orient == val);
      orient = val;

      if (initialized)
      {
         invert_labels();
         change_orientation();
      }
   }
   else
      cmd_manager->add_err(rval=ERR_STATUS_BAD, "Invalid values for orientation should be 0 or 1");
   return rval; 
}

string  ChipsColorBar::get_label_text()
{
   return label_text;
}

string  ChipsColorBar::get_label_color()
{
   return label_color.get_name();
}


int ChipsColorBar::get_label_visible()
{
   return labvis;
}


string  ChipsColorBar::get_label_font()
{
   return label_font;
}


V_FONTSTYLE_TYPE ChipsColorBar::get_label_fontstyle()
{
   return label_fontstyle;
}


int ChipsColorBar::get_label_size()
{
   return label_size;
}


int  ChipsColorBar::set_label_text(string val)
{
   label_text = val; 
   update_label_text ();
   if (initialized)  change_title_text ();
   return ERR_STATUS_GOOD;
}


int  ChipsColorBar::set_label_angle(float val)
{
   int rval = ERR_STATUS_GOOD;

   label_angle = val;
   if (initialized)
      change_title_angle ();

   return rval;
}


float ChipsColorBar::get_label_angle()
{
   return label_angle;
}


int  ChipsColorBar::set_label_halign(float val)
{
   int rval = ERR_STATUS_GOOD;

   use_label_halign = label_horizalignment = val;
   if (label_horizalignment == AUTO_ALIGN)
      use_label_halign = 0.5;
   if (initialized)  rval = change_title_alignment();


   return rval;
}


float ChipsColorBar::get_label_halign(bool computed)
{
   if (computed)
      return use_label_halign;

   return label_horizalignment;
}


int  ChipsColorBar::set_label_valign(float val)
{
   int rval = ERR_STATUS_GOOD;

   use_label_valign = label_vertalignment = val;
   if (label_vertalignment == AUTO_ALIGN)
   {
      if (((orient == 0) && 
           ((!bar_flip && (label_location == chips_outside)) ||
            (bar_flip && (label_location == chips_inside)))) ||
          ((orient == 1) && (label_location == chips_inside)))
         use_label_valign = 1.0;
      else
         use_label_valign = 0.0;
   }
   if (initialized)  rval = change_title_alignment();


   return rval;

}


float ChipsColorBar::get_label_valign(bool computed)
{
   if (computed)
      return use_label_valign;

   return label_vertalignment;
}


int ChipsColorBar::set_label_visible(bool viz)
{
   int rval = ERR_STATUS_GOOD;

   labvis = viz;
   if (initialized)  rval = change_title_visible();

   return rval;
}


int ChipsColorBar::set_label_color(string val)
{
   int rval = ERR_STATUS_GOOD;

   if ((rval = label_color.set_foreground_color(val)) == ERR_STATUS_GOOD)
   {
      update_label_text ();
      if (initialized) change_title_color();
   }
   return rval;


}

int ChipsColorBar::set_label_font(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (val.length())
   {
      if (theFonts && theFonts->font_exists(val))
      {
         string tmpfont = theFonts->get_path(val, label_fontstyle);
         if (tmpfont.length())
         {
            label_fontfile = tmpfont;
            label_font = val;
            update_label_text ();
            if (initialized)   rval = change_title_font();
         }
      }
      else
         cmd_manager->add_err(rval=ERR_STR_SUBSET, id, 0, "font", val.c_str (), "helvetica|courier|times|greek|user loaded fonts");
   }
   else
      cmd_manager->add_err(rval=ERR_STR_SUBSET, id, 0, "font", val.c_str (), "helvetica|courier|times|greek|user loaded fonts");

   return rval;

}


int ChipsColorBar::set_label_size(int sz)
{
   int rval = ERR_STATUS_GOOD;

   if ((sz >= V_SIZE_MIN_VAL) || (sz <= V_SIZE_MAX_VAL))
   {
      label_size = sz;
      update_label_text ();
      if (initialized)  rval = change_title_size();
   }
   else
      cmd_manager->add_err(rval=ERR_INT_MINMAX, id, 0, "colorbar label size", sz,
                                  V_SIZE_MIN_VAL, V_SIZE_MAX_VAL);

   return rval;
}


int ChipsColorBar::set_label_fontstyle(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (val.length())
   {
      V_FONTSTYLE_TYPE tmp;
      if ((rval = c2_string_to_fontstyle(val.c_str(), &tmp)) == ERR_STATUS_GOOD)
      {
         label_fontstyle = tmp;
         set_label_font(label_font);
      }
   }
   else
      cmd_manager->add_err(rval=ERR_STR_SUBSET, id, 0, "fontstyle", val.c_str (), "normal|bold|italic|bolditalic");

   return rval;
}

int ChipsColorBar::set_label_location(V_TICKSTYLE_TYPE val)
{
   int rval = ERR_STATUS_GOOD;

   if ((val == chips_inside) || (val == chips_outside))
   {
      label_location = val;
      if (initialized)
      {
         set_label_halign(label_horizalignment);
         set_label_valign(label_vertalignment);
         rval = change_title_location();
      }
   }
   else
      cmd_manager->add_err(rval=ERR_INT_SUBSET, id, 0, "label location", (int)val, "chips_inside|chips_outside");
   return rval;
}


int ChipsColorBar::set_parallel_offset(float in_offset)
{
   int rval = ERR_STATUS_GOOD;

   if (in_offset != offset_par)
   {
      if ((in_offset >= V_MIN_PAR_OFFSET)  && (in_offset <= V_MAX_PAR_OFFSET))
      {
         offset_par = in_offset;
         if (initialized)  rval = change_parallel_offset();
      }
      else
         cmd_manager->add_err(rval=ERR_FLT_MINMAX, id, 0, "color bar parallel offset", in_offset, V_MIN_PAR_OFFSET, V_MAX_PAR_OFFSET);

   }
   return rval;
}

float ChipsColorBar::get_parallel_offset()
{
   return offset_par;
}

int ChipsColorBar::set_perpendicular_offset(float in_offset)
{
   int rval = ERR_STATUS_GOOD;

   if (in_offset != offset_perp)
   {
      if ((in_offset >= V_MIN_PERP_OFFSET)  && (in_offset <= V_MAX_PERP_OFFSET))
      {
         offset_perp = in_offset;
         if (initialized)  rval = change_perpendicular_offset();
      }
      else
         cmd_manager->add_err(rval=ERR_FLT_MINMAX, id, 0, "color bar perpendicular offset", in_offset, (float)V_MIN_PERP_OFFSET, (float)V_MAX_PERP_OFFSET);
   }
   return rval;
}

float ChipsColorBar::get_perpendicular_offset()
{
   return offset_perp;
}



int ChipsColorBar::set_tick_mode(string inmode)
{
   if (!chips_strcmp(inmode.c_str(), "none")) 
      return set_tick_mode(0);
   else if  (!chips_strcmp(inmode.c_str(), "ends"))
      return set_tick_mode(1);
   else if  (!chips_strcmp(inmode.c_str(), "nice"))
      return set_tick_mode(2);
   return ERR_DATA_SUBSET;
}


int ChipsColorBar::set_tick_mode (int inmode)
{
   int rval = ERR_STATUS_GOOD;
   tickmode = inmode;
   if (initialized)
   {
      double range[2];
      colormap_table->get_colormap_scalar_range (colormap_id, range);
      calculate_tick_positions (range[0], range[1]);
      change_tick_mode ();
   }

   return rval;
}

int ChipsColorBar::set_tick_thickness(float val)
{
   int rval = ERR_STATUS_GOOD;
   // need to determine thickness of ticks
   if ((val < V_THICK_MIN_VAL) || (val > V_THICK_MAX_VAL))
      cmd_manager->add_err(rval=ERR_FLT_MINMAX, id, 0, "colorbar tick thickness", val, V_THICK_MIN_VAL, (float)V_THICK_MAX_VAL);
   else
   {
      tickthickness = val;
      if (initialized)  rval = change_tick_thickness();
   }
   return rval;
}


int ChipsColorBar::set_tick_style(V_TICKSTYLE_TYPE val)
{
   int rval = ERR_STATUS_GOOD;

   if (valid_tickstyle_type(val))
   {
      tickstyle = val; 
      if (initialized)  rval = change_tick_style(); 
   }
   else
   {
      stringstream errstr;
      errstr << "The colorbar tickstyle value (" << val << ") must be one of [inside|outside]";
      cmd_manager->add_err(errstr.str ());
   }
   return rval;
}

int ChipsColorBar::set_tick_location(V_TICKSTYLE_TYPE val)
{
   int rval = ERR_STATUS_GOOD;

   if (valid_tickstyle_type(val))
   {
      ticklocation = val;
      set_ticklabel_halign (ticklabhorizalignment);
      set_ticklabel_valign (ticklabvertalignment);
      if (initialized)  rval = change_tick_location();
   }
   else
   {
      stringstream errstr;
      errstr << "The colorbar ticklocation value (" << val << ") must be one of [inside|outside]";
      cmd_manager->add_err(errstr.str ());
   }
   return rval;
}

int ChipsColorBar::set_tick_visible(bool viz)
{
   int rval = ERR_STATUS_GOOD;

   tickvis = viz;
   if (initialized)  rval = change_tick_visible();

   return rval;
}

int ChipsColorBar::set_tick_color(string val)
{
   int rval = ERR_STATUS_GOOD;

   if ((rval = tickcolor.set_foreground_color(val)) == ERR_STATUS_GOOD)
   {
      update_label_text ();
      if (initialized) change_tick_color();
   }
   return rval;


}

string  ChipsColorBar::get_tick_color()
{
   return tickcolor.get_name ();
}

int ChipsColorBar::set_tick_length(int val)
{
   int rval = ERR_STATUS_GOOD;

   if (val >= 0)
   {
      ticklength = val;
      if (initialized)  rval =  change_tick_length();
   }
   else
      cmd_manager->add_err (rval=ERR_NO_INT_NEGATIVE,id, 0, val);
   return rval;
}

int  ChipsColorBar::set_ticklabel_angle(float val)
{
   int rval = ERR_STATUS_GOOD;

   ticklabangle = val;
   if (initialized)  rval = change_ticklabel_angle();
   return rval;
}


float ChipsColorBar::get_ticklabel_angle()
{
   return ticklabangle;
}


int  ChipsColorBar::set_ticklabel_halign(float val)
{
   int rval = ERR_STATUS_GOOD;
   use_ticklab_halign = ticklabhorizalignment = val;
   if (ticklabhorizalignment == AUTO_ALIGN)
   {
      if (orient == 0) 
         use_ticklab_halign = 0.5; 
      else if ((bar_flip && (ticklocation == chips_outside)) || 
                (!bar_flip && (ticklocation == chips_inside))) 
          use_ticklab_halign = 0.0;
      else 
          use_ticklab_halign = 1.0;
   }
   if (initialized)  rval = change_ticklabel_alignment();
   return rval;
}


float ChipsColorBar::get_ticklabel_halign(bool computed)
{
   if (computed)
      return use_ticklab_halign;

   return ticklabhorizalignment;
}


int  ChipsColorBar::set_ticklabel_valign(float val)
{
   int rval = ERR_STATUS_GOOD;
   use_ticklab_valign = ticklabvertalignment = val;
   if (ticklabvertalignment == AUTO_ALIGN) 
   {
      if (orient != 0) 
         use_ticklab_valign = 0.5;
       else if ((bar_flip && (ticklocation == chips_inside)) || 
                (!bar_flip && (ticklocation == chips_outside))) 
          use_ticklab_valign = 1.0;
      else 
          use_ticklab_valign = 0.0;
   }
   if (initialized)  rval = change_ticklabel_alignment();
   return rval;
}


float ChipsColorBar::get_ticklabel_valign(bool computed)
{
   if (computed)
      return use_ticklab_valign;
   return ticklabvertalignment;
}


int ChipsColorBar::set_ticklabel_color(string val)
{
   int rval = ERR_STATUS_GOOD;
   if ((rval = ticklabcolor.set_foreground_color(val)) == ERR_STATUS_GOOD)
      if (initialized) change_ticklabel_color();
   return rval;
}

string  ChipsColorBar::get_ticklabel_color()
{
   return ticklabcolor.get_name(); 
}


int ChipsColorBar::set_ticklabel_font(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (val.length())
   {
      if (theFonts && theFonts->font_exists(val))
      {
         string tmpfont = theFonts->get_path(val, ticklabfontstyle); 
         if (tmpfont.length())
         {
            ticklabfontfile = tmpfont;
            ticklabfont = val;
	    if (initialized)   rval = change_ticklabel_font();
         }
      }
      else
         cmd_manager->add_err(rval=ERR_STR_SUBSET, id, 0, "font", val.c_str (), "helvetica|courier|times|greek|user loaded fonts");
   }
   else
      cmd_manager->add_err(rval=ERR_STR_SUBSET, id, 0, "font", val.c_str (), "helvetica|courier|times|greek|user loaded fonts");

   return rval;
}


string  ChipsColorBar::get_ticklabel_font()
{
   return ticklabfont;
}

int ChipsColorBar::set_ticklabel_size(int sz)
{
   int rval = ERR_STATUS_GOOD;

   if (sz >= 0)
   {
      ticklabsize = sz;
      if (initialized)  rval = change_ticklabel_size();
   }
   return rval;
}


int ChipsColorBar::set_ticklabel_visible(bool viz)
{
   int rval = ERR_STATUS_GOOD;
   ticklabvis = viz;
   if (initialized)  rval = change_ticklabel_visible();
   return rval;
}


int ChipsColorBar::get_ticklabel_size()
{
   return ticklabsize;
}


int ChipsColorBar::set_position (double inXPos, double inYPos)
{
   return ERR_FEATURE_DNE; 
}

int ChipsColorBar::set_vertalignment(float val)
{
   int rval = ERR_STATUS_GOOD;

   if (vert_alignment != AUTO_ALIGN)
   {
      vert_alignment = val;
      if (initialized)
         rval = change_alignment();
   }
   else
      rval = ERR_STATUS_BAD;
   return rval;

}

int ChipsColorBar::set_horizalignment(float val)
{
   int rval = ERR_STATUS_GOOD;

   if (horiz_alignment != AUTO_ALIGN)
   {
      horiz_alignment = val;
      if (horiz_alignment != AUTO_ALIGN)
         rval = change_alignment();
   }
   else 
      rval = ERR_STATUS_BAD;
   return rval;

}

int ChipsColorBar::set_border_visible(bool val)
{
   int rval = ERR_STATUS_GOOD;

   rval = ERR_STATUS_BAD;
   border_vis = val;
   rval = change_border_visible ();
   return rval;

}

int ChipsColorBar::set_colormap_id(unsigned short cmap_id)
{
   colormap_id = cmap_id;
   this->change_colormap ();
   return ERR_STATUS_GOOD;
}

int ChipsColorBar::set_ticklabel_fontstyle(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (val.length())
   {
      V_FONTSTYLE_TYPE tmp;
      if ((rval = c2_string_to_fontstyle(val.c_str(), &tmp)) == ERR_STATUS_GOOD)
      {
         ticklabfontstyle = tmp; 
         set_ticklabel_font(ticklabfont); 
      }
   }
   else 
      cmd_manager->add_err(rval=ERR_STR_SUBSET, id, 0, "fontstyle", val.c_str (), "normal|bold|italic|bolditalic");

   return rval;
}


V_FONTSTYLE_TYPE ChipsColorBar::get_ticklabel_fontstyle()
{
   return ticklabfontstyle;
}


int ChipsColorBar::set_ticklabel_offset(int sz)
{
   int rval = ERR_STATUS_GOOD;

   if (sz >= 0)
   {
      ticklaboffset = sz;
      if (initialized)  rval = change_ticklabel_offset();
   }
   return rval;
}

int ChipsColorBar::get_ticklabel_offset()
{
   return ticklaboffset;
}

// SCALE1 algorithm (C.R. Lewart, CACM, Oct 73, v16 n10) 
// finds a new range to nicely divide data into approx n linear intervals 
void ChipsColorBar::scale1(double inmin, double inmax, int slices, 
   double* outmin, double* outmax, double* dist, double *base)
{
   double interval, scaled, fm, baseval;
   int nal, m;

   if (inmax == inmin)  ++inmax;
   if (!slices) slices = 5;     // avoid div by 0- use 5 as desired num ticks

   interval = fabs((inmax-inmin)/(double)slices);
   nal = (int) log10(interval);
   if (interval < 1) --nal;

   // scale interval between 1-10 and find value- e
   if ((scaled = interval/pow(10.0, nal)) < 1.41421356237) baseval = 1.0;
      else if (scaled < 3.16227766017)                     baseval = 2.0;
         else  if (scaled < 7.07106781187)                 baseval = 5.0;
            else                                           baseval = 10;
   if (base)  *base = baseval; 

   if ((*dist = copysign(baseval * pow(10.0, nal), (inmax - inmin))) != 0)
   {
      // compute interval size
      fm = inmin/(*dist);
      if (fm < 0.0)   m = (int)fm - 1;
         else   m = (int)fm;
      if (fabs((double)m+1.0-fm) < 0.0000000000000001)  ++m;

      // find new minimum and maximum
      *outmin = (*dist) * (double)m;
      fm = inmax/(*dist);
      if (fm < -1.0)  m = (int)fm; 
         else m = (int)fm + 1;
      if (fabs(fm+1.0-(double)m) < 0.0000000000000001)  --m;
      *outmax = (*dist) * (double)m;

      // adjust for roundoff
      if (*outmin > inmin)   *outmin = inmin;
      if (*outmax < inmax)   *outmax = inmax;
   }
   else
   {
      *outmin = inmin;
      *outmax = inmax;
   }
}

int ChipsColorBar::calculate_tick_positions(double min_value, double max_value)
{
   double xpmin, xpmax, dist, base;
   int rval = ERR_STATUS_GOOD;
   scale1(min_value, max_value, 5, &xpmin, &xpmax, &dist, &base);

   // set number of minor ticks- if the unit base is 2, setup 
   // minor tick intervals of 1/4, otherwise use 1/5 
/*   if (base == 2.0)     minor_num = 3; 
      else              minor_num = 4; 
*/
   if (xpmax > xpmin)   tickcnt = (unsigned int)((xpmax - xpmin) / dist + 1.5); 
      else              tickcnt = 1; 

   // clearup any old tick positions 
   if (tickpos)  
   {
      free(tickpos);   
      tickpos = NULL;
   }

   // make sure we have a reasonable number of ticks 
   if (tickmode ==1)
   {
      tickcnt = 2;
      tickpos = (double*)malloc(tickcnt * sizeof(double));
      if (tickpos)
      {
         tickpos[0] = tickmin = min_value;
         tickpos[1] = tickmax = max_value;
      }
   }
   else if (tickcnt < 0 || tickcnt > 500)
   //if ((tickcnt < 0) || (tickcnt > 500) || (tickmode == 1))
   {
      // setup new tick locations 
      tickcnt = 2;
      tickpos = (double*)malloc(tickcnt * sizeof(double));
      if (tickpos)
      {
         tickpos[0] = xpmin;
         tickpos[1] = xpmax;
         tickmin = xpmin; 
         tickmax = xpmax; 
/*         minor_mode = chips_count; 
         change_minor_mode(); 
         change_minor_num(); 
         rval = update_range(); 
*/
      }
   }
   else
   {
      // setup new tick locations 
      tickpos = (double*)malloc(tickcnt * sizeof(double));
      if (tickpos)
      {
         int rr; 
         if ((dist > DBL_EPSILON) && (xpmin < DBL_EPSILON) &&
             (xpmin > -1*DBL_EPSILON))
            tickpos[0] = 0;
         else
            tickpos[0] = xpmin;  
         for (rr = 1; rr < tickcnt; rr++) 
         {
            tickpos[rr] = tickpos[rr-1] + dist; 
            if ((dist > DBL_EPSILON) && (tickpos[rr] < DBL_EPSILON) && 
               (tickpos[rr] > -1*DBL_EPSILON))
               tickpos[rr] = 0; 
         }
         tickmin = xpmin; 
         tickmax = xpmax; 
      }
   }
   return rval;
}

int ChipsColorBar::move_object (V_SUBOBJECT_TYPE obj, COORD_SYS coord_sys, const string &coord_id, double xx, double yy, int mode)
{

   int rval = ERR_STATUS_GOOD;

   switch (obj)
   {
      case chips_colorbar_label:
      {
         ChipsVector pos1 = norm_pos;
         ChipsVector pos2 (xx, yy);
         float tmp_offset_perp = 0;
         float tmp_offset_par = 0;
         int par_index = (this->get_orientation () == 0) ? 0 : 1;
         int perp_index = (this->get_orientation () == 0) ? 1 : 0;

         coord->convert_cs_to_cs (&pos2, &pos2, 1, coord_sys, coord_id, PIXEL_REAL, "");

         if (mode == V_MOVE_DELTA)
         {
            int flip = 1;

            if (((!this->bar_flip) && (label_location == chips_outside)) ||
                ((this->bar_flip) && (label_location == chips_inside)))
               flip = -1;

            pos1[par_index] = (this->offset_par+.5) * (frame_length);
            coord->convert_cs_to_cs (&pos1, &pos1, 1, FRAME_NORM, "", PIXEL_REAL, "");
            pos1 += pos2;
            tmp_offset_perp = this->offset_perp + (pos2[perp_index] * flip);

            coord->convert_cs_to_cs(&pos1,&pos1,1,PIXEL_REAL,"",FRAME_NORM,"");
   
            tmp_offset_par = pos1[par_index];

         }
         else
         {
            ChipsVector low_bound (2);
            ChipsVector up_bound (2);
            ChipsVector pix_low_bound (2);
            ChipsVector pix_up_bound (2);

            low_bound[par_index] = this->norm_pos[par_index] - (this->frame_length * horiz_alignment);
            low_bound[perp_index] = this->norm_pos[perp_index] - (this->frame_width * vert_alignment);
            up_bound[par_index] = low_bound[par_index] + this->frame_length;
            up_bound[perp_index] = low_bound[perp_index] + this->frame_width;

            coord->convert_cs_to_cs (&low_bound, &pix_low_bound, 1,
                            FRAME_NORM, "",
                            PIXEL_REAL, "");
            coord->convert_cs_to_cs (&up_bound, &pix_up_bound, 1,
                            FRAME_NORM, "",
                            PIXEL_REAL, "");

            if (((!this->bar_flip) && (label_location == chips_outside)) ||
                ((this->bar_flip) && (label_location == chips_inside)))
            {
               tmp_offset_perp =  pix_low_bound[perp_index] - pos2[perp_index]; 
               tmp_offset_par = low_bound[par_index] - pos2[par_index];
            }
            else
            {
               tmp_offset_perp =  pos2[perp_index] - pix_up_bound[perp_index];
               tmp_offset_par = pos2[par_index] - up_bound[par_index];
            }
         }

         if (dbl_equal((frame_length), 0))
            tmp_offset_par = 0;
         else
            tmp_offset_par = (tmp_offset_par/frame_length) - .5;

        

         this->set_property (&tmp_offset_par, SET_CBAR_OFFSET_PAR);
         this->set_property (&tmp_offset_perp, SET_CBAR_OFFSET_PERP);
      }
      break;
      default:
         rval = ERR_CANT_MOVE;
   }
   return rval;
}

int ChipsColorBar::resize_object(V_SUBOBJECT_TYPE subobj, COORD_SYS coord_sys, const string &coord_id, double *bounds)
{
   int rval = ERR_STATUS_GOOD;

   ChipsVector lower (bounds[0], bounds[1]);
   ChipsVector upper (bounds[2], bounds[3]);
   double boundingBox[4];

   if (!coord)
   {
      cmd_manager->add_err(ERR_COORD_OBJ_DNE, id, 0);
      return ERR_COORD_OBJ_DNE;
   }

   if (!coord->coord_exists (coord_sys, coord_id))
   {
      if (coord_sys == DATA)
         cmd_manager->add_err(rval=ERR_COORD_DS_DNE, id, 0, v_coord_sys_to_string(DATA), coord_id.c_str ());
      else
         cmd_manager->add_err(rval=ERR_COORD_DNE, id, 0, v_coord_sys_to_string(DATA));
      return ERR_COORD_DNE;
   }

   // convert to pixel real which is what compute_resize uses
   rval = coord->convert_cs_to_cs(&lower, &lower, 1, coord_sys,
                                     coord_id, PIXEL_REAL, "");
   rval = coord->convert_cs_to_cs(&upper, &upper, 1, coord_sys,
                                     coord_id, PIXEL_REAL, "");

   boundingBox[0] = lower[0];
   boundingBox[1] = lower[1];
   boundingBox[2] = upper[0];
   boundingBox[3] = upper[1];


   switch (subobj)
   {
      case chips_colorbar_label:
         rval = this->compute_resize_title (boundingBox);

      break;
      default:
         rval = ERR_FEATURE_DNE;
   }

   return rval;
}

int ChipsColorBar::rotate_object(V_SUBOBJECT_TYPE subobj, float angle)
{
   int rval = ERR_STATUS_GOOD;

   switch (subobj)
   {
      case chips_colorbar_label:
         rval = this->set_property (&angle, SET_CBAR_LABEL_ANGLE);
      break;
      default:
         rval = ERR_FEATURE_DNE;
   }

   return rval;
}

int ChipsColorBar::select_object(V_SUBOBJECT_TYPE subobj, bool selected)
{
   int rval = ERR_STATUS_GOOD;

   switch (subobj)
   {
      case chips_colorbar_label:
      {
         if (selected) this->select_title ();
           else this->unselect_title ();
      }
      break;
      default:
         rval = ERR_FEATURE_DNE;
   }

   return rval;

}


int ChipsColorBar::move(COORD_SYS coord_sys, 
   const string &coord_id, double xx, double yy, int mode, bool rec)
{
   int rval = ERR_STATUS_GOOD;
   ChipsMoveCommand* cmd = NULL;
   ChipsVector tmp_n (0.0, 0.0);
   ChipsVector tmp_org (0.0, 0.0);

   string csid = coord_id;

   if (!coord)
   {
      cmd_manager->add_err(ERR_COORD_OBJ_DNE, id, 0);
      return ERR_COORD_OBJ_DNE;
   }

   if (coord_sys == COORD_NONE)
   {
      coord_sys = coord_system;
      csid = coord->get_id ();
   }

   if (!coord->coord_exists (coord_sys, csid))
   {
      if (coord_sys == DATA)
         cmd_manager->add_err(rval=ERR_COORD_DS_DNE, id, 0, v_coord_sys_to_string(DATA), csid.c_str ());
      else
	 cmd_manager->add_err(rval=ERR_COORD_DNE, id, 0, v_coord_sys_to_string(DATA));
      return ERR_COORD_DNE;
   }
   if (rec)  cmd = new ChipsMoveCommand(this, orig[0], orig[1], xx, yy, coord_sys, coord_id, mode, ((ChipsPlot*)parent)->get_current_colorbar());

   if (mode != V_MOVE_DELTA)
      tmp_org = ChipsVector(xx, yy);
   else
   {
      // convert offset and (0,0) to frame norm to get offset
      rval = coord->convert_cs_to_cs(&orig, &tmp_org, 1, coord_system,
                                     data_system_id, coord_sys, csid);
      if (rval == ERR_STATUS_GOOD)
      {
         // adjust norm pos by offset
         tmp_org[0] += xx;
         tmp_org[1] += yy;
      }
   }
   // convert new pos to frame norm
   rval = coord->convert_cs_to_cs(&tmp_org, &tmp_org, 1, coord_sys,
                                  csid, coord_system, data_system_id);
   if (rval == ERR_NEG_LOG_VAL)
      cmd_manager->add_err("Cannot move an object to a position less than or equal to zero when in log scale.", rval=ERR_EDGE_BOUNDARY, id, 0);
   if (!rval)
   {
      orig = tmp_org;
      rval = calculate_norm_coordinate (orig, norm_pos);
      invert_labels ();
      if ((rval == ERR_STATUS_GOOD) && initialized)
         rval = change_position();
   }
   if ((rval == ERR_STATUS_GOOD) && rec)
   {
      if (!add_command(cmd, rval))   delete cmd;
   }
   else if (rec)
      delete(cmd);

   if (!rval)
      cmd_manager->update_mask (STATE_CHNG_PROP_CLBAR);

   return rval;
}

int ChipsColorBar::resize(COORD_SYS coord_sys, const string &coord_id,
                           double *bounds, bool rec)
{

   ChipsVector lower (bounds[0], bounds[1]);
   ChipsVector upper (bounds[2], bounds[3]);
   int rval;
   bool lock = cmd_manager->cmd_lock ();
   float tmp_length, tmp_width;
   double origin[2];
   string cd_id = this->data_system_id;
   

   if (!rec)
      cmd_manager->set_cmd_lock (true);

   if (!coord)
   {
      cmd_manager->add_err(ERR_COORD_OBJ_DNE, id, 0);
      return ERR_COORD_OBJ_DNE;
   }

   if (!coord->coord_exists (coord_sys, coord_id))
   {
      if (coord_sys == DATA)
         cmd_manager->add_err(rval=ERR_COORD_DS_DNE, id, 0, v_coord_sys_to_string(DATA), coord_id.c_str ());
      else
         cmd_manager->add_err(rval=ERR_COORD_DNE, id, 0, v_coord_sys_to_string(DATA));
      return ERR_COORD_DNE;
   }

   if (coord_system == DATA)
      cd_id = (dynamic_cast<ChipsDataSystem *>(coord))->get_id_no_wcs ();


   // convert to pixel real which is what compute_resize uses
   rval = coord->convert_cs_to_cs(&lower, &lower, 1, coord_sys,
                                     coord_id, coord_system, cd_id);
   rval = coord->convert_cs_to_cs(&upper, &upper, 1, coord_sys,
                                     coord_id, coord_system, cd_id);

   if (!rec)
      cmd_manager->set_cmd_lock (true);
   else
      cmd_manager->open_block ();

   if (this->orient == 0)
   {
      tmp_length = (upper[0] - lower[0]);
      tmp_width = (upper[1] - lower[1]);
      origin[0] = lower[0] + tmp_length * this->horiz_alignment;
      origin[1] = lower[1] + tmp_width * this->vert_alignment;
   }
   else
   {
      tmp_length = (upper[1] - lower[1]);
      tmp_width = (upper[0] - lower[0]);
      origin[0] = lower[0] + tmp_width * this->vert_alignment;
      origin[1] = lower[1] + tmp_length * this->horiz_alignment;
   }

   if ((tmp_length > 0)  && (tmp_width > 0))
   {
      this->set_property (&tmp_length, SET_CBAR_LENGTH, rec);
      this->set_property (&tmp_width, SET_CBAR_WIDTH, rec);
      this->move (coord_system, cd_id, origin[0], origin[1], V_MOVE_ABSOLUTE, rec);
   }

   if (!rec)
      cmd_manager->set_cmd_lock (lock);
   else
      cmd_manager->close_block ();

   return rval;

}

int ChipsColorBar::change_coordinate (COORD_SYS coordsys, const string& coordid)
{

   ChipsCoordinate *new_coord;
   ChipsVector dim[2];
   int rval = ERR_STATUS_GOOD;

   // This is temporarily hard coded until a better method of searching
   // down the coordinate system tree is implemented. Otherwise objects
   // in window system must be kept in the frame coord
   if ((coordsys == WINDOW_NORM) || (coordsys == PIXEL) ||
       (coordsys == PIXEL_REAL))
      new_coord = coord->get_coord (FRAME_NORM, "");
   else
      new_coord = coord->get_coord (coordsys, coordid);

   if (!new_coord)
   {
      cmd_manager->add_err(ERR_COORD_DNE, id, 0, v_coord_sys_to_string(coordsys));
      return ERR_COORD_DNE;
   }
   coord->convert_cs_to_cs (&orig, &orig, 1, coord_system, data_system_id, coordsys, coordid);

   // recompute the length and width in the new coordinate system
   if (orient == 0)
   {
      ChipsVector new_dim;
      dim[0] = ChipsVector (2);
      dim[1] = ChipsVector (2);

      dim[0] = ChipsVector (norm_pos[0] - (frame_length * horiz_alignment),
                            norm_pos[1] - (frame_width * vert_alignment));
      dim[1] = ChipsVector (dim[0][0] + frame_length,
                            dim[0][1] + frame_width);

      coord->convert_cs_to_cs (&dim[0], &dim[0], 2, FRAME_NORM, "", coordsys, coordid);

      new_dim = dim[1] - dim[0];
      length = new_dim[0];
      width = new_dim[1];
   }
   else
   {
      ChipsVector new_dim;
      dim[0] = ChipsVector (2);
      dim[1] = ChipsVector (2);

      dim[0] = ChipsVector (norm_pos[0] - (frame_width * vert_alignment),
                            norm_pos[1] - (frame_length * horiz_alignment));
      dim[1] = ChipsVector (dim[0][0] + frame_width,
                            dim[0][1] + frame_length);

      coord->convert_cs_to_cs (&dim[0], &dim[0], 2, FRAME_NORM, "", coordsys, coordid);

      new_dim = dim[1] - dim[0];
      length = new_dim[1];
      width = new_dim[0];
   }


   rval = coord->remove (this);

   if (!rval)
      rval = new_coord->insert (this, false);

   if (!rval)
   {
      this->set_chips_coord (new_coord);
      this->set_coord_system (coordsys, coordid);

      this->change_position ();

      cmd_manager->update_mask (STATE_CHNG_PROP_CLBAR);
   }

   return ERR_STATUS_GOOD;
}

int ChipsColorBar::get_position (ChipsVector &pos)
{
   int rval = ERR_STATUS_GOOD;
   pos = orig;

   // convert the data to logical coordinates if necessary
   if (coord_system == DATA)
   {
      ChipsDataSystem *ds_coord = dynamic_cast <ChipsDataSystem *>(coord);
      if (ds_coord->get_default_transform_id () != "")
         rval = ds_coord->convert_cs_to_cs (&pos, &pos, 1, DATA, data_system_id, DATA, "");
   }

   return rval;
}



int ChipsColorBar::adjust_default_colors()
{
   if (get_label_color() == "default")  change_title_color();
   if (get_ticklabel_color() == "default")      change_ticklabel_color();
   if (get_tick_color() == "default")  change_tick_color();
   return ERR_STATUS_GOOD; 
}

int ChipsColorBar::change_colormap()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_depth()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_size()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_position()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_reverse()
{
   return ERR_FEATURE_DNE;
}


int ChipsColorBar::change_visible()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_tick_style()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_tick_color()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_tick_thickness()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_tick_length()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_tick_mode()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_tick_location()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_tick_visible()
{
   return ERR_FEATURE_DNE;
}



int ChipsColorBar::change_ticklabel_color()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_ticklabel_font()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_ticklabel_angle()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_ticklabel_size()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_ticklabel_visible()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_ticklabel_alignment()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_ticklabel_offset()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_text ()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_color()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_font()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_fontstyle()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_size()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_angle()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_alignment()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_location()
{
   return ERR_FEATURE_DNE;
}


int ChipsColorBar::change_parallel_offset()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_perpendicular_offset()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_title_visible()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_border_visible()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_bar_flip()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_alignment()
{
   return ERR_FEATURE_DNE;
}

int ChipsColorBar::change_orientation()
{
   return ERR_FEATURE_DNE;
}

void ChipsColorBar::serialize(ChipsStateStream& out)
{
   int sz;
   int len;
   string col;
   short sh_val;

   out.write_string(&id);
   sz = (int) ref_cnt;
   out.write_int(&sz);
   sz = (int)coord_system;
   out.write_int(&sz);
   out.write_string(&data_system_id);

   sz = orig.get_size ();
   out.write_int (&sz);
   for (int ii=0; ii<sz; ii++)
      out.write_double(&(orig[ii]));

   out.write_int (&orient);
   out.write_int (&tickcnt);
   out.write_double (tickpos, tickcnt);
   out.write_double (&tickmin);
   out.write_double (&tickmax);

   out.write_double (&label_angle);
   len = (int)ticklocation;
   out.write_int (&len);
   len = (int)tickstyle;
   out.write_int (&len);
   out.write_float (&tickthickness);
   out.write_int (&ticklength);
   col = get_tick_color ();
   out.write_string (&col);
   out.write_int (&tickmode);
   out.write_bool (&tickvis);
   col = get_ticklabel_color ();
   out.write_string (&col);
   out.write_string (&ticklabfontfile);
   out.write_string (&ticklabfont);
   len = (int)ticklabfontstyle;
   out.write_int (&len);
   out.write_float (&ticklabangle);
   out.write_int (&ticklabsize);
   out.write_int (&ticklaboffset);
   out.write_bool (&ticklabvis);
   out.write_double (&ticklabvertalignment);
   out.write_double (&ticklabhorizalignment);

   out.write_float (&horiz_alignment);
   out.write_float (&vert_alignment);

   out.write_bool (&border_vis);
   out.write_bool (&bar_flip);

   out.write_int (&label_size);
   col = get_label_color ();
   out.write_string (&col);

   out.write_string (&label_font);
   out.write_string (&label_fontfile);
   out.write_double (&label_vertalignment);
   out.write_double (&label_horizalignment);
   len = (int)label_location;
   out.write_int (&len);
   len = (int)label_fontstyle;
   out.write_int (&len);
   out.write_bool (&labvis);
   out.write_string (&label_text);

   out.write_float (&offset_par);
   out.write_float (&offset_perp);

   out.write_int (&cbar_num);
   out.write_double (&width);
   out.write_double (&length);
   out.write_double (&frame_width);
   out.write_double (&frame_length);
   sh_val = (short)colormap_id;
   out.write_short (&sh_val);
   
   out.write_int(&depth);
}
                                                                                        
void ChipsColorBar::serialize_ascii(ChipsStateStream& out)
{
   string att = "", tmp;
   char buf[80];
   const char *cp;


   if (horiz_alignment != theDefaults->get_colorbarhalign())
   {
      sprintf(buf, "halign=%2.1f", horiz_alignment);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (vert_alignment != theDefaults->get_colorbarvalign())
   {
      sprintf(buf, "valign=%2.1f", vert_alignment);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (border_vis != theDefaults->get_colorbarbordervis())
   {
      if (att.length()) att.append(" ");
      if (border_vis)  att.append("border.visible=true");
         else          att.append("border.visible=false");
   }
   if (width != theDefaults->get_colorbarwidth())
   {
      if (att.length()) att.append(" ");
      sprintf(buf, "width=%3.2f", width);
      att.append(buf);
   }
   if (length != theDefaults->get_colorbarlength())
   {
      if (att.length()) att.append(" ");
      sprintf(buf, "length=%3.2f", length);
      att.append(buf);
   }
   if (label_text != theDefaults->get_colorbarlabtext())
   {
      if (att.length()) att.append(" ");
      att.append("label.text=\\\"");
      if (label_text.length())   att.append(label_text);
      att.append("\\\"");
   }
#ifndef WLM_ORI
   if (label_angle != theDefaults->get_colorbarlabangle())
   {
      if (orient == 1)
      {
         if (bar_flip)
            sprintf(buf, "label.angle=%2.1f", -1*(label_angle+90));
         else
            sprintf(buf, "label.angle=%2.1f", label_angle-90);
      }
      else
         sprintf(buf, "label.angle=%2.1f", label_angle);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
#else
   if (orient == 1 && label_angle == label_angle0 -90)
   {
      sprintf(buf, "label.angle=%2.1f", label_angle0);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   else if (orient == 1 && label_angle != theDefaults->get_colorbarlabangle())
   {
      sprintf(buf, "label.angle=%2.1f", label_angle-90);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   else
   {
      sprintf(buf, "label.angle=%2.1f", label_angle);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
#endif
   if (label_size != theDefaults->get_colorbarlabsize())
   {
      sprintf(buf, "label.size=%1d", label_size);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   tmp = get_label_color();
   if (tmp != theDefaults->get_colorbarlabcolor())
   {
      if (att.length()) att.append(" ");
      att.append("label.color=");
      att.append(tmp);
   }
   if (label_font != theDefaults->get_colorbarlabfont())
   {
      if (att.length()) att.append(" ");
      att.append("label.font=");
      att.append(label_font);
   }
   if (label_fontstyle != theDefaults->get_colorbarlabfontstyle() &&
       ((cp = v_fontstyle_to_string(label_fontstyle))!=NULL))
   {
      if (att.length()) att.append(" ");
      att.append("label.fontstyle=");
      att.append(cp);
   }
   if (label_horizalignment != theDefaults->get_colorbarlabhalign())
   {
      sprintf(buf, "label.halign=%2.1f", label_horizalignment);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (label_vertalignment != theDefaults->get_colorbarlabvalign())
   {
      sprintf(buf, "label.valign=%2.1f", label_vertalignment);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (label_location != theDefaults->get_colorbarlabloc () &&
       ((cp = v_tickstyle_to_string(label_location))!=NULL))
   {
      if (att.length()) att.append(" ");
      att.append("label.location=");
      att.append(cp);
   }
   if (offset_par != theDefaults->get_colorbarparalleloffset ())
   {
      if (att.length()) att.append(" ");
      sprintf(buf,"offset.parallel=%3.2f", offset_par);
      att.append(buf);
   }
   if (offset_perp != theDefaults->get_colorbarperpendicularoffset())
   {
      if (att.length()) att.append(" ");
      sprintf(buf,"offset.perpendicular=%3.2f", offset_perp);
      att.append(buf);
   }
   if (tickmode != theDefaults->get_colorbartickmode ()) 
   {
//   there is no convience function to turn int to string for this mode
//       ((cp = v_tickmode_to_string((V_TICKMODE_TYPE)tickmode))!=NULL))
      string mode="";
      if ((int)tickmode == 0)
         mode = "none";
      else if  ((int)tickmode == 1)
         mode = "ends";
      else if  ((int)tickmode == 2)
         mode = "nice";

      if (mode.length () > 0)
      {
         if (att.length()) att.append(" ");
         att.append("tick.mode=");
         att.append(mode);
      }
   }
   if (ticklocation != theDefaults->get_colorbartickloc() &&
       ((cp = v_tickstyle_to_string(ticklocation))!=NULL))
   {
      if (att.length()) att.append(" ");
      att.append("tick.location=");
      att.append(cp);
   }
   if (ticklength != theDefaults->get_colorbarticklength())
   {
      if (att.length()) att.append(" ");
      sprintf(buf,"tick.length=%1d", ticklength);
      att.append(buf);
   }
   tmp = get_ticklabel_color();
   if (tmp != theDefaults->get_colorbarticklabcolor())
   {
      if (att.length()) att.append(" ");
      att.append("ticklabel.color=");
      att.append(tmp);
   }
   if (ticklabfont != theDefaults->get_colorbarticklabfont())
   {
      if (att.length()) att.append(" ");
      att.append("ticklabel.font=");
      att.append(ticklabfont);
   }
   if (ticklabfontstyle != theDefaults->get_colorbarticklabfontstyle() &&
       ((cp = v_fontstyle_to_string(ticklabfontstyle))!=NULL))
   {
      if (att.length()) att.append(" ");
      att.append("ticklabel.fontstyle=");
      att.append(cp);
   }
   if (ticklabangle != theDefaults->get_colorbarticklabangle())
   {
      sprintf(buf, "ticklabel.angle=%2.1f", ticklabangle);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (ticklabsize != theDefaults->get_colorbarticklabsize())
   {
      sprintf(buf, "ticklabel.size=%1d", ticklabsize);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (ticklaboffset != theDefaults->get_colorbarticklaboffset())
   {
      sprintf(buf, "ticklabel.offset=%1d", ticklaboffset);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (ticklabvertalignment != theDefaults->get_colorbarticklabvalign())
   {
      sprintf(buf, "ticklabel.valign=%2.1f", ticklabvertalignment);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (ticklabhorizalignment != theDefaults->get_colorbarticklabhalign())
   {
      sprintf(buf, "ticklabel.halign=%2.1f", ticklabhorizalignment);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (labvis != theDefaults->get_colorbarlabvisible())
   {
      if (att.length()) att.append(" ");
      if (labvis)  att.append("label.visible=true");
         else      att.append("label.visible=false");
   }
   if (ticklabvis != theDefaults->get_colorbarticklabvisible ())
   {
      if (att.length()) att.append(" ");
      if (ticklabvis)  att.append("ticklabel.visible=true");
         else      att.append("ticklabel.visible=false");
   }
   if (tickvis != theDefaults->get_colorbartickvisible ())
   {
      if (att.length()) att.append(" ");
      if (tickvis)  att.append("tick.visible=true");
         else      att.append("tick.visible=false");
   }
   tmp = get_tick_color();
   if (theDefaults->get_colorbartickcolor() != tmp)
   {
      if (att.length()) att.append(" ");
      att.append("tick.color=");
      att.append(tmp);
   }
   if (tickthickness != theDefaults->get_colorbartickthickness())
   {
      if (att.length()) att.append(" ");
      if (tickthickness != 0.5)
      {
         sprintf(buf,"tick.thickness=%-1.0f", tickthickness);
         att.append(buf);
      }
      else
         att.append("tick.thickness=0.5");
   }
   int dep = theDefaults->get_colorbardepth();
   if (depth != dep &&
       (dep != V_USE_DEFAULT_DEPTH || depth!=((ChipsPlot*)parent)->get_depth()))
   {
      sprintf(buf, "depth=%d", depth);
      if (att.length()) att.append(" ");
      att.append(buf);
   }
   if (theDefaults->get_colorbartickstyle() != tickstyle &&
       ((cp = v_tickstyle_to_string(tickstyle))!=NULL))
   {
      if (att.length()) att.append(" ");
      att.append("tick.style=");
      att.append(cp);
   }
   if (orient != theDefaults->get_colorbarorient())
   {
      if (att.length()) att.append(" ");
      if (orient == 0)  att.append("orientation=horizontal");
         else           att.append("orientation=vertical");
   }
   if (coord_system != DATA &&
       (cp = v_coord_sys_to_enum_string(coord_system)) != NULL)
   {
      if (att.length()) att.append(" ");
      att.append("coordsys=");
      att.append(cp);
   }
   if (att.length()) att.append(" ");
   att.append("id=");
   att.append(id);

   ChipsVector tmp_pos = orig;
   // convert the data to wcs coordinates if necessary
   if (coord_system == DATA)
   {
      ChipsDataSystem *ds_coord = dynamic_cast <ChipsDataSystem *>(coord);
      if (ds_coord->get_default_transform_id () != "")
      {
         data_system_id = ds_coord->get_id_no_wcs ();
         ds_coord->convert_cs_to_cs (&tmp_pos, &tmp_pos, 1, DATA, data_system_id, DATA, "")
;
      }
   }

   out <<"add_colorbar("<<tmp_pos[0]<<", "<<tmp_pos[1];
   if (att.length()) out <<", \""<<att<<"\"";
   out <<")"<< endl;
}

void ChipsColorBar::deserialize(ChipsStateStream& in)
{
   int len;
   string col;
   short sh_val;

   in.read_string(&id);
   in.read_int(&len);
   ref_cnt = len;
   in.read_int(&len);
   coord_system = (COORD_SYS)len;
   in.read_string(&data_system_id);

   if (((in.get_major_version() == 3) && (in.get_minor_version () >= 4)) ||
      (in.get_major_version() > 3))
   {
      in.read_int((int*)&len);
      orig = ChipsVector (len);
      for (int ii=0; ii<len; ii++)
         in.read_double(&(orig[ii]));

      in.read_int (&orient);
      in.read_int (&tickcnt);

      // clearup any old tick positions
      if (tickpos)
      {
         free(tickpos);
         tickpos = NULL;
      }
      tickpos = (double*)malloc(tickcnt * sizeof(double));

      in.read_double (tickpos, tickcnt);
      in.read_double (&tickmin);
      in.read_double (&tickmax);

      in.read_double (&label_angle);
      in.read_int (&len);
      ticklocation = (V_TICKSTYLE_TYPE)len;
      in.read_int (&len);
      tickstyle = (V_TICKSTYLE_TYPE)len;
      in.read_float (&tickthickness);
      in.read_int (&ticklength);
      in.read_string (&col);
      set_tick_color (col);
      in.read_int (&tickmode);
      in.read_bool (&tickvis);
      in.read_string (&col);
      set_ticklabel_color (col);
      in.read_string (&ticklabfontfile);
      in.read_string (&ticklabfont);
      in.read_int (&len);
      ticklabfontstyle = (V_FONTSTYLE_TYPE)len;
      in.read_float (&ticklabangle);
      in.read_int (&ticklabsize);
      in.read_int (&ticklaboffset);
      in.read_bool (&ticklabvis);
      in.read_double (&ticklabvertalignment);
      in.read_double (&ticklabhorizalignment);

      in.read_float (&horiz_alignment);
      in.read_float (&vert_alignment);

      in.read_bool (&border_vis);
      in.read_bool (&bar_flip);
      in.read_int (&label_size);
      in.read_string (&col);
      set_label_color(col);
      
      
      in.read_string (&label_font);
      in.read_string (&label_fontfile);
      in.read_double (&label_vertalignment);
      in.read_double (&label_horizalignment);
      in.read_int (&len);
      label_location = (V_TICKSTYLE_TYPE)len;
      in.read_int (&len);
      label_fontstyle = (V_FONTSTYLE_TYPE)len;
      in.read_bool (&labvis);
      in.read_string (&label_text);
      set_label_text (label_text);

      in.read_float (&offset_par);
      in.read_float (&offset_perp);

      in.read_int (&cbar_num);
      in.read_double (&width);
      in.read_double (&length);
      in.read_double (&frame_width);
      in.read_double (&frame_length);
      in.read_short (&sh_val);
      colormap_id = (unsigned short)sh_val;

      in.read_int(&depth);

   }
}

int ChipsColorBar::show()
{
   visible = true;
   return change_visible ();
}

int ChipsColorBar::hide()
{
   visible = false;
   return change_visible ();
}

int ChipsColorBar::set_depth(int newdepth)
{
   int rval = ERR_STATUS_GOOD;
   if (newdepth < 0)
      cmd_manager->add_err(rval=ERR_NEG_DEPTH,id,0,newdepth);
   else
   {
      depth = newdepth;
      if (initialized)  rval = change_depth();
   }
   return rval;
}


bool ChipsColorBar::add_command (ChipsCommand *inCmd, int &outStatus)
{
   bool success = true;
   if ((outStatus = cmd_manager->add_cmd(inCmd)) != ERR_STATUS_GOOD)
   {
      if (outStatus == ERR_CMD_LOCKED) outStatus = ERR_STATUS_GOOD;
      success = false;
   }
   return success;
}

int ChipsColorBar::set_bar_flip(bool inFlag)
{
   int rval = ERR_STATUS_GOOD;
   if (inFlag != bar_flip)
   {
      bar_flip = inFlag;
      if (initialized)  rval = change_bar_flip();
   }
   return rval;
}

int ChipsColorBar::calculate_norm_coordinate (ChipsVector &in_vector, ChipsVector &out_vector)
{
   int rval = ERR_STATUS_GOOD;
   if (coord)
      rval = coord->convert_cs_to_cs (&in_vector, &out_vector, 1, coord_system,
                            data_system_id, FRAME_NORM, "");

   return rval;
}

int ChipsColorBar::reset_position ()
{
   int rval;

   rval = calculate_norm_coordinate (orig, norm_pos);
   if (!rval)
   {

      if (coord_system == DATA)
      {
         ChipsPlotSystem *tmp_coord = dynamic_cast <ChipsPlotSystem *>(coord->get_coord (PLOT_NORM));
         set_clip_location (tmp_coord->get_lower_bounds ()[0], 
                            tmp_coord->get_lower_bounds ()[1],
                            tmp_coord->get_upper_bounds ()[0],
                            tmp_coord->get_upper_bounds ()[1]);
      }

      rval = change_position ();

      // reset the width and the length since they are dependent on 
      // coordinates as well
      set_width (width);
      set_length (length);
   }

   return rval;
}


int ChipsColorBar::set_clip_location (double x1, double y1, double x2, double y2)
{
   return ERR_FEATURE_DNE;
}


// do we need to invert the axis label or ticklabels?
void ChipsColorBar::invert_labels()
{
   ChipsVector tmp_pnt (.5,.5);
   if (!coord)
      return;

   if ((coord_system == PLOT_NORM) || (coord_system == DATA))
      coord->convert_cs_to_cs (&tmp_pnt, &tmp_pnt, 1, PLOT_NORM, 
                              parent->get_id (), FRAME_NORM, "");

   if (orient == 0)
   {
      set_bar_flip (norm_pos[1] > tmp_pnt[1]);
      if (flipped)    flipped = 0;
   }
   else
   {
      set_bar_flip (norm_pos[0] > tmp_pnt[0]);
      if (bar_flip && !flipped)   flipped = 1;
      else if (flipped && !bar_flip)   flipped = 0;
   }

   // update the color bar with new layout
   set_ticklabel_valign (ticklabvertalignment);
   set_ticklabel_halign (ticklabhorizalignment);
   set_label_valign (label_vertalignment);
   set_label_halign (label_horizalignment);
   set_label_angle (label_angle);

}

void ChipsColorBar::update_label_text ()
{
   delete_tree(label_format);   // cleanup old format tree
   format_text(label_text, label_font, label_fontstyle, get_label_color(), 
               label_size, &label_format);
}


