// 
//  Copyright (C) 2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSCOLORBAR_HH
#define CHIPSCOLORBAR_HH

#include <string>
#include <stdlib.h>
#include <stdio.h>
#include "ChipsObject.hh"
#include "ChipsDefaults.hh"
#include "ChipsColor.hh"
#include "ChipsStateStream.hh"
#include "ChipsTextSupport.hh"
#include "ChipsCommand.hh"
#include "ChipsId.hh"

using namespace std;


class ChipsColorBar: public ChipsObject
{
  public:

   ChipsColorBar(ChipsObject* par, ChipsCoordinate *inCoord, 
                     COORD_SYS coord_sys, ChipsStateStream& in);
   ChipsColorBar(ChipsObject* par, ChipsCoordinate *inCoord, 
                     COORD_SYS coord_sys, double xpos, double ypos,
                     unsigned short cmap_id, string val);
   ChipsColorBar(ChipsObject* par, ChipsCoordinate *inCoord, 
                     COORD_SYS coord_sys, double xpos, double ypos,
                     string val);

   void       object_delete ();

   void       initialize(); 
   int        set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj);
   int        set_property(void* in_val, int property, bool rec=true);
   int        get_property(void* out_val, int property);

   int        set_width (float value);
   float      get_width ();
   int        set_length (float value);
   float      get_length ();

   string     get_label_text();
   int        set_label_text(string val);
   string     get_label_color(); 
   int        set_label_color(string in_color);
   int        get_label_size(); 
   int        set_label_size(int sz);
   float      get_label_angle();
   int        set_label_angle(float angle);
   float      get_label_halign(bool computed=false); // compute true will calculate auto
   int        set_label_halign(float angle);
   float      get_label_valign(bool computed=false); // compute true will calculate auto
   int        set_label_valign(float angle);
   string     get_label_font();
   int        set_label_font(string font);
   V_FONTSTYLE_TYPE get_label_fontstyle();
   int        set_label_fontstyle(string fontstyle);
   int        set_label_visible(bool viz);
   int        get_label_visible();
   int        set_label_location(V_TICKSTYLE_TYPE val);
   V_TICKSTYLE_TYPE get_label_location () { return label_location; };
   float      get_parallel_offset();
   int        set_parallel_offset(float in_offset);
   float      get_perpendicular_offset();
   int        set_perpendicular_offset(float in_offset);


   int        get_tick_mode () { return tickmode; } 
   int        set_tick_mode(string inmode);
   int        set_tick_mode (int val);
   float      get_tick_thickness()   { return tickthickness; }
   int        set_tick_thickness(float val); 
   V_TICKSTYLE_TYPE get_tick_style()   { return tickstyle; } 
   int        set_tick_style(V_TICKSTYLE_TYPE val);

   V_TICKSTYLE_TYPE get_tick_location()   { return ticklocation; }
   int        set_tick_location(V_TICKSTYLE_TYPE val);
   int        set_tick_visible(bool viz);
   bool       get_tick_visible() { return tickvis; };
   int        set_tick_color(string color);
   string     get_tick_color();
   int        set_tick_length(int val);
   int        get_tick_length() { return ticklength; };



   string     get_ticklabel_color();
   int        set_ticklabel_color(string in_color);
   int        get_ticklabel_size();
   int        set_ticklabel_size(int sz);
   float      get_ticklabel_angle();
   int        set_ticklabel_angle(float angle);
   float      get_ticklabel_halign(bool computed=false); // compute true will calculate auto
   int        set_ticklabel_halign(float angle);
   float      get_ticklabel_valign(bool computed=false); // compute true will calculate auto
   int        set_ticklabel_valign(float angle);
   string     get_ticklabel_font();
   int        set_ticklabel_font(string font);
   V_FONTSTYLE_TYPE get_ticklabel_fontstyle();
   int        set_ticklabel_fontstyle(string fontstyle);
   int        get_ticklabel_offset();
   int        set_ticklabel_offset(int sz);
   int        set_ticklabel_visible(bool viz);
   int        get_ticklabel_visible()    { return ticklabvis; }

   int        set_position (double inXPos, double inYPos);

   int        set_vertalignment(float val);
   int        set_horizalignment(float val);
   float      get_vertalignment(){return vert_alignment;};
   float      get_horizalignment(){return horiz_alignment;};

   int        set_border_visible(bool val);
   bool       get_border_visible() { return border_vis; }

   int        set_orientation (int val);
   int        get_orientation () {return orient;};
   int        set_alignment(V_TICKSTYLE_TYPE val);

   bool       get_bar_flip() { return bar_flip;};
   int        set_bar_flip(bool inFlag);

              // nice tick methods (C.R. Lewart, CACM, Oct 73,v16 n10)
   void       scale1(double min, double max, int n, double* outmin, 
                     double* outmax, double* dist, double* base=NULL);

   virtual int set_depth(int newdepth);
   virtual int move(COORD_SYS coord_sys, const string &coord_id, double xx,
                    double yy, int mode, bool rec=true);
   int  resize (COORD_SYS coord_sys, const string &coord_id,
                    double *bounds, bool rec=true);
   int        change_coordinate (COORD_SYS coordsys,
                                          const string& coordid);
   int        get_position (ChipsVector &pos);


   int        set_colormap_id(unsigned short colormap_id); 
   int        adjust_default_colors();

   int        reset_position ();

   int         move_object(V_SUBOBJECT_TYPE obj, COORD_SYS coord_sys, const string &coord_id, double xx, double yy, int mode);
   int        resize_object(V_SUBOBJECT_TYPE subobj, COORD_SYS coord_sys, const string &coord_id, double *bounds);
   int        rotate_object(V_SUBOBJECT_TYPE subobj, float angle);
   int        select_object(V_SUBOBJECT_TYPE subobj, bool selected);



   virtual int change_depth();
   virtual int change_position(); 
   virtual int change_reverse(); 
   virtual int change_ticklabel_color();
   virtual int change_ticklabel_font();
   virtual int change_ticklabel_size();
   virtual int change_ticklabel_angle();
   virtual int change_ticklabel_offset();
   virtual int change_ticklabel_visible();
   virtual int change_ticklabel_alignment();
   virtual int change_visible(); 

   virtual int change_tick_style(); 
   virtual int change_tick_color();
   virtual int change_tick_thickness();
   virtual int change_tick_length();
   virtual int change_tick_mode();
   virtual int change_tick_location();
   virtual int change_tick_visible ();

   virtual int change_title_text ();
   virtual int change_title_color();
   virtual int change_title_font();
   virtual int change_title_fontstyle();
   virtual int change_title_size();
   virtual int change_title_angle();
   virtual int change_title_alignment();
   virtual int change_title_location ();
   virtual int change_parallel_offset();
   virtual int change_perpendicular_offset();


   virtual int change_colormap();
   virtual int change_size();               // width or length 
   virtual int change_alignment();          // halign or valign
   virtual int change_orientation ();
   virtual int change_bar_flip ();
   virtual int change_title_visible();
   virtual int change_border_visible();
   virtual int set_clip_location (double x1, double y1, double x2, double y2);

   void       serialize(ChipsStateStream& out);
   void       serialize_ascii(ChipsStateStream& out);
   void       deserialize(ChipsStateStream& in);
   void       set_cbar_num (int val) {cbar_num = val;};
   int        get_cbar_num () {return cbar_num;};

   virtual int hide ();
   virtual int show ();

   virtual inline void select_title () {};
   virtual inline void unselect_title () {};

   virtual inline int compute_resize_title (double *bounds) {return ERR_FEATURE_DNE;};



  protected:
   virtual ~ChipsColorBar();
   int calculate_norm_coordinate (ChipsVector &in_vector, ChipsVector &out_vector);
   int calculate_tick_positions (double min_value, double max_value);
   void post_initialize (ChipsCoordinate *inCoord, COORD_SYS coord_sys);
   void invert_labels();

   ChipsVector orig;  // cbar position in its original coordinates
   ChipsVector norm_pos; // cbar position in its frame norm coordinates
   int	      orient;        // orientation of the cbar
   int        tickcnt; 
   double*    tickpos;       // nice tick locations 
   double     tickmin;       // minimum tick value
   double     tickmax;       // maximum tick value

   double     label_angle;   // angle for cbar label 
   double     use_ticklab_halign; // horizontal tick label alignment (including auto)
   double     use_ticklab_valign; // vertical tick label alignment (including auto)
   double     use_label_halign; // horizontal label alignment (including auto)
   double     use_label_valign; // vertical label alignment (including auto)

   V_TICKSTYLE_TYPE ticklocation; // ticks location relative to plot center
   V_TICKSTYLE_TYPE tickstyle; // type of major tickmarks 
   float            tickthickness; // tick thickness 
   int              ticklength;   
   ChipsColor       tickcolor; // tick color
   int              tickmode;
   bool             tickvis;  // ticks visible

   ChipsColor       ticklabcolor; // color of tick labels
   string           ticklabfontfile;  // fontfile for tick labels
   string           ticklabfont;  // font of tick labels
   V_FONTSTYLE_TYPE ticklabfontstyle; // font style of tick labels 
   float            ticklabangle; // angle of tick labels
   int              ticklabsize;  // size of tick labels
   int              ticklaboffset; // offset of tick labels
   bool             ticklabvis;    // tick label visibility
   double           ticklabvertalignment; // vertical alignment of tick label
   double           ticklabhorizalignment; // horiz alignment of tick label

   float      horiz_alignment; 
   float      vert_alignment; 

   bool       initialized;   // have we initialized everything 
   bool       border_vis;     // border visible 
   bool       bar_flip;    // flip of labels inside/outside of axis
   bool       flipped; 

   ELEM_P_T   label_format;        // tree containing text format
   int        label_size;    // size of text in label 
   ChipsColor label_color;   // color of label text
   string     label_font;   // font of label 
   string     label_fontfile; // fontfile for labels
   double     label_vertalignment; // vertical alignment of tick label
   double     label_horizalignment; // vertical alignment of tick label
   V_TICKSTYLE_TYPE label_location; // which side of the bar the label is located
   
   V_FONTSTYLE_TYPE label_fontstyle;
   bool       labvis;        // cbar label visibility
   string     label_text;    //string to hold label text

   float      offset_par;    // parallel offset for axis label
   float      offset_perp;   // perpendicular offset for axis label

   int        cbar_num;       // this axis_x order 0th, 1st, 2nd, etc.
   double     width;         // size of a slice of the cbar
   double     length;          // size of the span of the cbar 
   double     frame_width;   // size of width in frame coords
   double     frame_length;   // size of length in frame coords


//   ChipsObject* mapping;      // associated object 
   unsigned short colormap_id; // reference id for the colormap
  private:
   bool add_command (ChipsCommand *inCmd, int &outStatus);
   void update_label_text ();

};


#endif 
