#ifndef CHIPSCOLORMAP_HH
#define CHIPSCOLORMAP_HH

#include "ChipsStateStream.hh"

class ChipsColorMap
{
   public:
      ChipsColorMap ();
      virtual ~ChipsColorMap ();

      void set_interpolate (bool flag);
      bool get_interpolate ();

      bool get_inverted ();

      void set_colormap_type (int type);
      int  get_colormap_type ();

      virtual void set_table_size (int size) = 0;
      virtual int  get_table_size () = 0;

      virtual void set_alpha_range (double alpha_range[]) = 0;
      virtual void get_alpha_range (double alpha_range[]) = 0;

      virtual void set_color_scale (int scale) = 0;
      virtual int  get_color_scale () = 0;

      virtual void set_color_ramp (int ramp) = 0;
      virtual int  get_color_ramp () = 0;

      virtual void set_scalar_threshold (double threshold[]) = 0;
      virtual void get_scalar_threshold (double threshold[]) = 0;

      virtual void set_scalar_range (double range[]) = 0;
      virtual void get_scalar_range (double range[]) = 0;

      virtual void clear_positions (int position=15) = 0;

      virtual void set_blue_position (double value, double position) = 0;
      virtual void set_green_position (double value, double position) = 0;
      virtual void set_red_position (double value, double position) = 0;
      virtual void set_alpha_position (double value, double position) = 0;

      virtual void set_blue_range (double min, double max) = 0;
      virtual void set_green_range (double min, double max) = 0;
      virtual void set_red_range (double min, double max) = 0;

      virtual void set_hue_position (double position, double value) = 0;
      virtual void set_saturation_position (double position, double value) = 0;
      virtual void set_value_position (double position, double value) = 0;

      virtual void set_hue_range (double min, double max) = 0;
      virtual void set_value_range (double min, double max) = 0;
      virtual void set_saturation_range (double min, double max) = 0;

      virtual void set_table (double **table, int table_size) = 0;

      virtual void set_data_range (double min, double max) = 0;

      virtual void invert_table () = 0;

      virtual void update () = 0;

      virtual unsigned char *get_computed_table (int &out_size) = 0;

      virtual void serialize (ChipsStateStream& out) = 0;
      virtual void deserialize (ChipsStateStream& in) = 0;

   protected:
      bool inverted;
      bool interpolate;
 
      int colormap_type;
};

#endif
