#ifndef CHIPSCOLORMAPTABLE_HH
#define CHIPSCOLORMAPTABLE_HH

#include <string>
#include <map>
#include <list>

#include "ChipsColorMap.hh"
#include "ChipsDefines.hh"
#include "ChipsStateStream.hh"

using namespace std;

typedef map <unsigned short, ChipsColorMap*> colormap_map;

class ChipsColorMapTable
{
   public:
      class UserDefinedMap
      {
         public:
            UserDefinedMap () {
               this->table = NULL;
               this->table_size = 0;
               this->map_type = TABLE;
               this->fname = "";
            }
            enum MAP_TYPE {TABLE, DS9};

            double **table;
            int table_size;

            list <pair <double, double> > red_map;
            list <pair <double, double> > green_map;
            list <pair <double, double> > blue_map;
            list <pair <double, double> > alpha_map;

            MAP_TYPE map_type;
            string fname;      // original filename
      };


      ChipsColorMapTable ();
      virtual ~ChipsColorMapTable ();

      void set_colormap_type (unsigned short id, V_COLORMAP_TYPE colormap_enum);
      V_COLORMAP_TYPE get_colormap_type (unsigned short id);

      void set_invert_colormap (unsigned short id, bool invert);
      bool get_invert_colormap (unsigned short id);

      void set_colormap_alpha_range (unsigned short id, double alpha_range[]);
      void get_colormap_alpha_range (unsigned short id, double alpha_range[]);

      void set_colormap_size (unsigned short id, int size);
      int  get_colormap_size (unsigned short id);

      void set_colormap_scale (unsigned short id, int scale);
      int  get_colormap_scale (unsigned short id);
     
      void set_colormap_ramp (unsigned short id, int ramp);
      int  get_colormap_ramp (unsigned short id);

      void set_colormap_scalar_threshold (unsigned short id, double threshold[]);
      void get_colormap_scalar_threshold (unsigned short id, double threshold[]);

      void set_colormap_scalar_range (unsigned short id, double range[]);
      void get_colormap_scalar_range (unsigned short id, double range[]);

      void set_colormap_interpolate (unsigned short id, bool interpolate);
      bool  get_colormap_interpolate (unsigned short id);

      int  load_file (string filename, V_COLORMAP_TYPE slot);

      unsigned short add_colormap ();
      ChipsColorMap *get_colormap (unsigned short id);
      
      virtual void serialize(ChipsStateStream& out);
      virtual void serialize_ascii(ChipsStateStream& out);
      virtual void deserialize(ChipsStateStream& in);
      
      int read_array_list (long len,V_COLORMAP_TYPE slot, double* r, double* g, 
		double* b, double* alpha);
      
   protected:
      virtual ChipsColorMap *new_colormap ();
      int read_ds9_file (ifstream &file, UserDefinedMap &user_map, string filename);
      int read_table_file (ifstream &file, UserDefinedMap &user_map, string filename);
      
      colormap_map colormap_table;
      unsigned short ref_count;

      UserDefinedMap userdefined1;
      UserDefinedMap userdefined2;
      UserDefinedMap userdefined3;
};

#endif
