// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsContourLevelCommand.hh"
#include "ChipsStateStream.hh"
#include "ChipsContour.hh"

ChipsContourLevelCommand::ChipsContourLevelCommand(ChipsContour* obj, int ocnt,
   double *ovals, int ncnt, double *nvals)
{
   co = (ChipsObject*)obj;
   weight = 1;

   if ((ocnt > 0) && ovals)
   {
      o_cnt = ocnt;
      o_vals = new double[o_cnt]; 
      if (o_vals)  for (int rr = o_cnt; rr--; )    o_vals[rr] = ovals[rr];
   }
   else 
   {
      o_cnt = 0; 
      o_vals = NULL;
   }
   if ((ncnt > 0) && nvals)
   {
      n_cnt = ncnt;
      n_vals = new double[n_cnt];
      if (n_vals)  for (int rr = n_cnt; rr--; )    n_vals[rr] = nvals[rr];
   }
   else 
   {
      n_cnt = 0; 
      n_vals = NULL;
   }
}

ChipsContourLevelCommand::~ChipsContourLevelCommand()
{
   if (n_vals)   delete[]  n_vals;
   if (o_vals)   delete[]  o_vals;

}


ChipsContourLevelCommand::ChipsContourLevelCommand(ChipsObject *cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsContourLevelCommand::execute ()
{
   int rval = ERR_STATUS_BAD;
   if (co && n_vals && (n_cnt > 0))
   {
      ChipsContour* ct = dynamic_cast <ChipsContour *>(co);
      rval = ct->set_levels(n_cnt, n_vals, false);
   }
   return rval;
}

int ChipsContourLevelCommand::unexecute ()
{
   int rval = ERR_STATUS_BAD;
   if (co && o_vals && (o_cnt > 0))
   {
      ChipsContour* ct = dynamic_cast <ChipsContour *>(co);
      rval = ct->set_levels(o_cnt, o_vals, false);
   }
   return rval;
}

void ChipsContourLevelCommand::serialize(ChipsStateStream& out)
{
   int val = 20;
   // Standard serialize output for a command
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   out.write_int(&n_cnt);
   if (n_cnt > 0)  out.write_double(n_vals, n_cnt);
   out.write_int(&o_cnt);
   if (o_cnt > 0)  out.write_double(o_vals, o_cnt);
}

void ChipsContourLevelCommand::deserialize(ChipsStateStream& in)
{
   // Standard serialize input for a command
   in.read_int(&ref);
   in.read_int(&block_ref);

   in.read_int(&n_cnt);
   if (n_cnt > 0)
   {
      n_vals = new double[n_cnt];
      in.read_double(n_vals, n_cnt);
   }
   in.read_int(&o_cnt);
   if (o_cnt > 0)
   {
      o_vals = new double[o_cnt];
      in.read_double(o_vals, o_cnt);
   }
}

void ChipsContourLevelCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Contour Level" << endl;
};



