// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsCoordinate.hh"
#include <bitset>
#include <math.h>
#include <sstream>

ChipsCoordinate::ChipsCoordinate () 
{
   parent = NULL;
   id = "";
   store_object_local=false;
   visible=true;
}

ChipsCoordinate::ChipsCoordinate (ChipsCoordinate *par)
{
   parent=par;
   id="";
   store_object_local = false;
   visible = true;
}

ChipsCoordinate::ChipsCoordinate (const ChipsCoordinate &cs)
{
   this->parent = cs.parent;
   this->id = cs.id;
   this->store_object_local = cs.store_object_local;
   this->visible = cs.visible;
}

ChipsCoordinate::ChipsCoordinate (ChipsStateStream &in)
{
   parent = NULL;
   id = "";
   store_object_local=false;
   visible=true;
}



ChipsCoordinate::~ChipsCoordinate ()
{
   list <ChipsObject *>::iterator ii;
   for (ii = chips_objects.begin (); ii != chips_objects.end (); ++ii)
   {
      (*ii)->set_coord_system (COORD_NONE, "");
      (*ii)->set_chips_coord (NULL);
   }
}

int ChipsCoordinate::update_obj_positions (bool update_all)
{
   int rval = ERR_STATUS_GOOD;

   list <ChipsObject *>::iterator ii;
   for (ii = chips_objects.begin (); ii != chips_objects.end (); ++ii){
       rval = (*ii)->reset_position ();
       if ((rval != ERR_STATUS_GOOD) && (rval != ERR_FEATURE_DNE))
          return rval;
   }

   return rval;
};

int ChipsCoordinate::get_object_list (list <ChipsObject *> &outList)
{
   outList = chips_objects;
   return ERR_STATUS_GOOD;
}

bool ChipsCoordinate::is_visible (bool check_parent)
{
   bool parent_visible = true;
   if (check_parent && parent)
      parent_visible = parent->is_visible (check_parent);
  
   return (visible && parent_visible);
}

void ChipsCoordinate::set_visible (bool vis)
{
   visible = vis;
}

int ChipsCoordinate::get_num_objs (int obj)
{
   // we want a count of all objects  
   if (obj == chips_not_object)  return  chips_objects.size();

   // we want a count of a specific object type (ie. chips_curve)
   int cnt = 0; 

   list <ChipsObject *>::iterator ii;
   for (ii = chips_objects.begin (); ii != chips_objects.end (); ++ii)
      if ((*ii)->get_object_type() == obj)   cnt++;
 
   return cnt;
}

// extract a specific coordinates id from an full id name
string ChipsCoordinate::extract_id (COORD_SYS coordsys, const string &coordid)
{
   int pos = -1;
   switch (coordsys)
   {
      case DATA:
         pos = coordid.length ();
      break;
      case PLOT_NORM:
         pos = coordid.find_first_of (".", pos+1);  // fall through intended
      case FRAME_NORM:
         pos = coordid.find_first_of (".", pos+1);  // fall through intended
      case WINDOW_NORM:
      case PIXEL:
      case PIXEL_REAL:
         pos = coordid.find_first_of (".", pos+1); 
      break;
      default:
      break;
   }

   if (pos < 0)
      return coordid;

   return coordid.substr (0, pos);
}


      



