// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSCOORDINATE_HH
#define CHIPSCOORDINATE_HH

class ChipsCoordinate;

#include <list>
#include <string>
#include <map>
#include "ChipsObject.hh"
#include "ChipsDefines.hh"
#include "ChipsMatrix.hh"
#include "ChipsStateStream.hh"

class ChipsCoordinate
{
   public:
      ChipsCoordinate ();
      ChipsCoordinate (ChipsCoordinate *par);
      ChipsCoordinate (const ChipsCoordinate &);
      ChipsCoordinate (ChipsStateStream &in);
      virtual ~ChipsCoordinate ();

      virtual int    convert_cs_to_cs (const ChipsVector *in, ChipsVector *out, 
                                 int size, COORD_SYS from_cs, 
                                 const string &from_id, COORD_SYS to_cs, 
                                 const string &to_id) 
                             {return ERR_STATUS_BAD;};
      virtual int    add_coord_sys (ChipsCoordinate *) 
                             {return ERR_STATUS_BAD;};
      virtual int    remove_coord_sys (const string &csid) 
                             {return ERR_STATUS_BAD;};
      virtual ChipsCoordinate *get_coord (COORD_SYS coord_sys, 
                                 const string &csid="") 
                             {return NULL;};
      virtual bool   coord_exists (COORD_SYS coord_sys, const string &csid)
                             {return false;};
      virtual void   get_coord_list (list <string> &outList) {};

      virtual int    insert (ChipsObject *co, bool use_co_info=true) 
                             {return ERR_STATUS_GOOD;};
      virtual int    remove (ChipsObject *co, bool use_co_info=true) 
                             {return ERR_STATUS_GOOD;};
      virtual int    update_obj_positions (bool update_all=true);
      virtual int    get_object_list (list <ChipsObject *> &outList);

      virtual ChipsVector get_lower_bounds (){return ChipsVector(1);};
      virtual ChipsVector get_upper_bounds (){return ChipsVector(1);};
      virtual COORD_SYS   get_bounds_coord_sys () {return COORD_NONE;};

      virtual int    deserialize (ChipsStateStream &in) 
                             { return ERR_STATUS_BAD; };
      virtual void   serialize (ChipsStateStream &out) {};

      // These are temporary functions to work around visibility of objects
      // whose coordinate system is not visible but whose parent is (in
      // particular annotations).
      virtual bool   is_visible (bool check_parent=true);
      virtual void   set_visible (bool vis);


      inline void store_local (bool val) {store_object_local = val;};
      int get_num_objs (int obj = chips_not_object);

      string get_id () {return id;};
      ChipsCoordinate * get_parent () {return parent;};
      void set_parent (ChipsCoordinate *par) { parent = par; };
      void remove_parent () { parent = static_cast <ChipsCoordinate *> (0); };

      // create an id from all parent information
      virtual void create_id (string &curr_id) {};

      // extract a specific coordinates id from an full id name
      static string extract_id (COORD_SYS coordsys, const string &coordid);

   protected:

      ChipsCoordinate *parent;
      string id;
      list <ChipsObject *> chips_objects;   //List of associated objects

      bool store_object_local;

      bool visible;
};

#endif
