// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsDataAspectCommand.hh"
#include "ChipsStateStream.hh"
#include "ChipsPlot.hh"
#include "ChipsId.hh"


ChipsDataAspectCommand::ChipsDataAspectCommand(ChipsObject *new_co, 
   string xax, string yax, string new_val, string old_val, string oldcurr)
{
   co = new_co;
   weight=1;
   xaxis = xax;
   yaxis = yax;
   new_ratio = new_val;
   old_ratio = old_val;
   current = oldcurr;
}

ChipsDataAspectCommand::~ChipsDataAspectCommand()
{
}


ChipsDataAspectCommand::ChipsDataAspectCommand(ChipsObject* cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsDataAspectCommand::execute ()
{
   if (co && (co->get_object_type() == chips_plot))
   {
      ChipsId cid;
      cid.set_xaxis_id(xaxis);
      cid.set_yaxis_id(yaxis);
      dynamic_cast<ChipsPlot *>(co)->set_property(&cid, 
         (void*)new_ratio.c_str(), SET_DATA_ASPECT_RATIO, false);
   }
   return ERR_STATUS_GOOD;
}

int ChipsDataAspectCommand::unexecute ()
{
   if (co && (co->get_object_type() == chips_plot))
   {
      ChipsId cid;
      cid.set_xaxis_id(xaxis);
      cid.set_yaxis_id(yaxis);
      dynamic_cast<ChipsPlot *>(co)->set_property(&cid, 
         (void*)old_ratio.c_str(), SET_DATA_ASPECT_RATIO, false);
   }
   return ERR_STATUS_GOOD;
}

void ChipsDataAspectCommand::serialize(ChipsStateStream& out)
{
   int val = 24;
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   // write out data fields
   out.write_string(&xaxis);
   out.write_string(&yaxis);
   out.write_string(&new_ratio);
   out.write_string(&old_ratio);
   out.write_string(&current);
}

void ChipsDataAspectCommand::deserialize(ChipsStateStream& in)
{
   in.read_int(&ref);
   in.read_int(&block_ref);
   in.read_string(&xaxis);
   in.read_string(&yaxis);
   in.read_string(&new_ratio);
   in.read_string(&old_ratio);
   in.read_string(&current);
}

void ChipsDataAspectCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"DataAspect" << endl;
};



