// 
//  Copyright (C) 2004,2005,2007-2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSDEFAULTS_HH
#define CHIPSDEFAULTS_HH

#include <string>
#include <stdio.h>
#include <vector>
#include <sstream>
#include <stdlib.h>
#include "ChipsStateStream.hh"
#include "ChipsDefinesP.hh"

using namespace std;

typedef struct defvals
{
   // window defaults
   bool       windowdisplay;     // always display window
   bool       windowsmoothing;   // smooth to hide aliasing effects
   float      windowheight;      // height of window
   float      windowwidth;       // width of window
   float      windowscaleheight;      // height of window
   float      windowscalewidth;       // width of window
   V_UNITS_TYPE windowunits;     // units of height/width (pixels|inches|cm)
   string     windowgeom;        // window geometry for location +/-x+/-y
   string     windowprefix;      // prefix for window titlebar (ie 'chips:')
   string     windowstem;        // stem for default window ids
   string     windowid;          // window id if specified
   string     windowcolor;       // color of window background
   bool       windowfocus;       // new chips window has focus 
   bool       showgrid;          // should window grids be displayed?
   int        gridscale;         // 1 = 1/4in, 2=1/2in, 3=cm, 4=normalized
   float      gridthickness;     // how thick to draw the grids
   string     gridcolor;         // color of grid
   V_LINE_TYPE gridlinestyle;    // type of line for grids
   bool       showlayout;        // display frame boundaries/layouts?
   float      layoutthickness;   // how thick to draw the frame borders/layout
   string     layoutcolor;       // color of layout/frame borders
   V_LINE_TYPE layoutlinestyle;  // type of line for layout
   string     printmethod;       // print window renderer - framebuffer|xoffscreen|none

   // frame defaults
   bool       frameaspect;       // keep frame aspect when tiling
   bool       framescale;        // scale labels when tiling
   bool       framebordervis;    // frame border visible
   int        framemode;         // placed = 0, single = 1, tiled = 2
   string     framestem;         // stem for default frame ids
   string     frameid;           // frame id if specified
   string     framecolor;        // color of frame
   string     framebordercolor;  // color of frame border
   bool       frametransparent;  // is frame transparent?

   // plot defaults
   string     plotstem;          // stem for default plot ids
   string     plotid;            // plot id if specified
   float      plotlmargin;       // left margin of plot
   float      plotrmargin;       // right margin of plot
   float      plottmargin;       // top margin of plot
   float      plotbmargin;       // bottom margin of plot
   float      titlexpos;         // plot title default x normalized pos
   float      titleangle;        // plot title default angle
   float      titlehalign;       // plot title default horizontal alignment
   float      titlevalign;       // plot title default vertical alignment
   float      titleypos;         // plot title default x normalized pos
   float      plotaspectw;       // plot's aspect width
   float      plotaspecth;       // plot's aspect height
   string     titlecolor;        // color of plot title string
   int        titlesize;         // size of plot title
   string     titlefont;         // font of plot title
   int        titledepth;        // depth for plot 
   string     titlelabel;        // plot title label if provided
   string     plotratio;         // plot's aspect ratio  w:h
   V_FONTSTYLE_TYPE titlefontstyle;  // font style- italic, bold, etc...

   // image defaults
   string     imagestem;         // stem for default image ids
   string     imageid;           // image id if specified
   int        imagedepth;        // depth of image
   V_COLORMAP_TYPE imagecmap;    // image color map
   V_INTERPOLATION_TYPE image_interp; // image interpolation method
   double     imagealpha[2];     // image alpha range
   double     imagethreshold[2]; // image threshold
   bool       imageinvertcmap;   // image flag for inverting colormaps
   int        imagecmapsize;     // number of colors in a colormap
   bool       imagecmapinterp;   // image flag for interpolating colormaps
   bool       imagescalechannels; // image flag for scaling channels
   string     cbarstem;          // stem for default colorbar ids
   string     cbarid;            // colorbar id if provided 
   float      cbarwidth;        // colorbar height
   float      cbarlength;         // colorbar width
   bool       cbarbordervis;     // colorbar border visible 
   int        cbardepth;         // colorbar depth 
   float      cbarhalign;        // colorbar horizontal alignment 
   float      cbarvalign;        // colorbar horizontal alignment 
   int        cbarorient;        // colorbar orientation 
   int        cbartickmode;      // none, ends, nice
   V_TICKSTYLE_TYPE cbartickstyle;   // inside, outside 
   string     cbartickcolor;     // tick color
   int        cbarticklength;    // tick length 
   float      cbartickthickness;  // tick thickness
   V_TICKSTYLE_TYPE  cbartickloc; // tick pos relative to plot center inside/outside
   bool       cbartickvisible;   // tick visibility
   string     cbarticklabcolor;  // color of tick label string
   float      cbarticklabangle;  // angle of tick label
   float      cbarticklabhalign; // horizontal alignment of tick label
   float      cbarticklabvalign; // vertical alignment of tick label
   int        cbarticklabsize;   // size of tick label
   int        cbarticklaboffset; // offset of tick label
   string     cbarticklabfont;   // font of tick label
   V_FONTSTYLE_TYPE cbarticklabfontstyle; // tick label fontstyle
   bool       cbarticklabvisible;// tick label visibility
   int        cbarlabsize;       // size of colorbar title label
   float      cbarlabangle;      // angle of colorbar title label
   string     cbarlabcolor;      // font of colorbar title label
   string     cbarlabfont;       // font of colorbar title label
   float      cbarlabhalign;     // horizontal alignment 
   float      cbarlabvalign;     // vertical alignment 
   bool       cbarlabvisible;    // label visibility
   V_TICKSTYLE_TYPE cbarlabloc;  // inside or outside
   V_FONTSTYLE_TYPE cbarlabfontstyle; // label fontstyle
   string     cbarlabtext;       // color bar label
   float      cbaroffsetperp;    // label perpendicular offset
   float      cbaroffsetpar;     // axis label parallel offset

   // axis defaults
   string     axisxstem;          // stem for default x axis ids
   string     axisxid;            // x axis id if provided
   string     axisxlabel;         // x axis label if provided
   string     axisystem;          // stem for default y axis ids
   string     axisyid;            // y axis id if provided
   string     axisylabel;         // y axis label if provided
   V_AXISSTYLE_TYPE axisstyle;   // type of axis
   V_CORNERSTYLE_TYPE cornerstyle;  // type of axis corner joint
   string     axiscolor;         // color of axis
   string     tickformat;        // tick format string (ie. "%2d")
   V_TICKSTYLE_TYPE ticklabstyle;// inside or outside
   string     ticklabcolor;      // color of tick label string
   float      ticklabangle;      // angle of tick label
   float      ticklabhalign;     // horizontal alignment of tick label
   float      ticklabvalign;     // vertical alignment of tick label
   int        ticklabsize;       // size of tick label
   int        ticklaboffset;     // offset of tick label
   int        axisdepth;         // depth level
   string     ticklabfont;       // font of tick label
   V_FONTSTYLE_TYPE ticklabfontstyle; // tick label fontstyle
   bool       axis_automin;      // auto minimum enabled
   bool       axis_automax;      // auto maximum enabled
   double     axis_pad;          // pad factor % for each side of axis
   float      axis_offsetperp;   // axis label perpendicular offset
   float      axis_offsetpar;    // axis label parallel offset
   float      axisthickness;     // size of axis
   string     majortickmode;     // count or interval based tick marks
   float      majortickinterval; // interval spacing of ticks
   unsigned int majortickcount;  // number of major ticks to display
   float      majortickthickness; // thickness of major tick marks
   int        majorticklength;   // length of major tick marks
   string     majortickcolor;    // color of major ticks
   V_TICKSTYLE_TYPE majortickstyle;    // inside, outside, or centered
   string     minortickmode;     // count or interval based tick marks
   float      minortickinterval; // interval spacing of minor ticks
   unsigned int minortickcount;  // number of minor ticks to display
   float      minortickthickness; // thickness of minor tick marks
   int        minorticklength;   // length of minor tick marks
   string     minortickcolor;    // color of minor ticks
   V_TICKSTYLE_TYPE minortickstyle;    // inside, outside, or centered
   bool       majortickvisible;  // display or hide major ticks
   bool       minortickvisible;  // display or hide minor ticks
   string     axlabcolor;        // color of axis label string
   int        axlabsize;         // size of axis label
   string     axlabfont;         // font of axis label
   float      axlabhalign;       // horizontal alignment of axis label
   float      axlabvalign;       // vertical alignment of axis label
   float      axlabxangle;       // x axis label angle  
   float      axlabyangle;       // x axis label angle   
   V_FONTSTYLE_TYPE axlabfontstyle;  // font style- italic, bold, etc...
   bool       majorgridvisible;  // display or hide major grid
   string     majorgridcolor;    // color of major grid
   float      majorgridthickness;// thickness of major grid
   V_LINE_TYPE majorgridstyle;   // type of line for major grid
   bool       minorgridvisible;  // display or hide minor grid
   string     minorgridcolor;    // color of minor grid
   float      minorgridthickness;// thickness of minor grid
   V_LINE_TYPE minorgridstyle;   // type of line for minor grid

   // point defaults
   V_SYMBOL_TYPE pointstyle;     // type of symbol
   string     pointcolor;        // color of symbol
   string     pointstem;         // stem for default curve ids
   string     pointid;           // point id if provided
   float      pointangle;        // point default angle
   bool       pointfill;         // filled or open symbol
   int        pointsize;         // size of symbol
   int        pointdepth;         // depth level

   // line defaults
   V_LINE_TYPE linestyle;        // type of line
   string     linecolor;         // color of line
   float      linethickness;     // thickness of line
   string     linestem;          // stem for default curve ids
   string     lineid;            // line id if provided
   int        linedepth;         // depth level
   V_LINE_EXTENSION_TYPE lineextend; // extend line endpoints
 
   // curve defaults
   string     curvestem;         // stem for default curve ids
   string     curveid;           // curve id if provided
   string     curvelinecolor;    // color of curve line
   V_LINE_TYPE curvelinestyle;   // type of line
   float      curvelinethickness;// thickness of line
   V_SYMBOL_TYPE curvesymbolstyle; // type of symbol
   string     curvesymbolcolor;  // color of curve symbol
   bool       curvesymbolfill;   // filled or open symbol
   int        curvesymbolsize;   // size of symbol
   float      curvesymbolangle;  // symbol default angle
   int        curvedepth;         // depth level
   bool       errup;             // up errors on/off
   bool       errdown;           // down errors on/off
   bool       errleft;           // left errors on/off
   bool       errright;          // right errors on/off
   V_ERRSTYLE_TYPE errstyle;     // curve error style
   string     errcolor;          // curve error color
   int        errcaplen;         // length of curve error caps
   float      errthickness;      // thickness of curve errs
   float      limitlength;       // upper/lower limits length (normalized)
   bool       limitoverride;     // upper/lower limits overrides up & down errs
   V_BADDATAMODE_TYPE curvebaddata;  // how to handle bad data in curve
  

   // histogram defaults
   string     histstem;          // stem for default histogram ids
   string     histid;            // histogram id if provided
   V_SYMBOL_TYPE  histsymbolstyle;  // histogram symbol style
   string     histsymbolcolor;   // histogram symbol color
   bool       histsymbolfill;    // histogram symbol fill
   int        histsymbolsize;    // histogram symbol size
   float      histsymbolangle;   // histogram symbol angle
   V_LINE_TYPE histlinestyle;    // histogram line style
   string     histlinecolor;     // histogram line color
   float      histlinethickness; // histogram line thickness
   bool       histerrup;         // histogram error up
   bool       histerrdown;       // histogram error down
   bool       histdropline;      // histogram drop lines (bar graph)
   V_ERRSTYLE_TYPE histerrstyle; // histogram error style
   string     histerrcolor;      // histogram error color
   int        histerrcaplen;     // length of histogram error caps
   float      histerrthickness;  // histogram error thickness
   int        histdepth;         // depth level
   string     histfillcolor;     // histogram fill color
   double     histfillopacity;   // histogram fill opacity
   V_FILL_TYPE histfillstyle;    // histogram fill style

   // contour defaults
   string     contourstem;       // stem for default contour ids
   string     contourid;         // contour id if provided 
   string     contourcolor;      // default contour color
   V_LINE_TYPE contourstyle;     // line style for contours
   float      contourthickness;  // thickness of contour line
   vector<double> contourlevels; // the contour levels
   unsigned int contournumlevels;   // number of contour levels (count mode)
   V_CONTOUR_TYPE contouralgorithm; // contour algorithm to use
   V_TICKMODE_TYPE contourmode;  // mode for determining contour levels
   float      contourinterval;   // interval for contour levels (interval mode)
   int        contourdepth;      // depth level

   // label defaults
   string     labelstem;         // stem for default labels
   string     labelid;           // label id if provided 
   string     labelfont;         // font for labels
   V_FONTSTYLE_TYPE labelstyle;  // bold, italic, normal, bolditalic
   int        labelsize;         // size of label
   int        labeldepth;        // depth level
   string     labelcolor;        // color of label
   float      labelangle;        // angle of tick label
   float      labelhalign;       // horizontal alignment of label
   float      labelvalign;       // vertical alignment of label

   // legend defaults
   string     legendstem;        // stem for default legend ids
   string     legendcolor;       // default legend color
   bool       legendtransparent; // is legend transparent
   bool       legendborders;     // does legend display borders
   float      legendthickness;   // thickness of legend border
   int        legenddepth;       // depth level

   // region defaults
   string     regionstem;        // stem for default region ids
   string     regionid;          // region id if provided 
   float      regionthickness;   // thickness of region edges
   V_LINE_TYPE regionstyle;      // edge style of region 
   V_FILL_TYPE regionfillstyle;  // region fill style
   string     regioncolor;       // region edge color
   string     regionfillcolor;   // region fill color 
   float      regionangle;       // region angle
   double     regionopacity;     // region opacity
   int        regiondepth;       // depth level

   // export defaults
   bool       exportclobber;     // overwrite file if it exists
   bool       exportfonts;       // export fonts to ps file
   int        exportorient;      // portrait/landscape
   string     exportformat;      // export format- ps, eps, png, pdf, jpg
   V_PRINTFILE_TYPE exportcolorscheme;  // color scheme- rgb, cmyk, grayscale
   int        exportdpi;         // export dots per inch
   float      pagelmargin;       // page bottom margin
   float      pagermargin;       // page bottom margin
   float      pagetmargin;       // page bottom margin
   float      pagebmargin;       // page bottom margin
   float      pageheight;        // height of page
   float      pagewidth;         // width of page
   V_UNITS_TYPE pageunits;       // units of the page height/width
   float      pagescaleheight;   // scale page height
   float      pagescalewidth;    // scale page width
   bool       fittopage;         // fit the window to the page
   bool       pageaspect;        // keep window aspect when fitting to page
   bool       printdialog;       // display print dialog window 

   // gui defaults 
   bool       toolbarvis;        // display toolbar in gui 
   bool       navpanevis;        // display navigation panel in gui 
   bool       infpanevis;        // display object panel in gui 
   bool       proppanevis;       // display action panel in gui 
   bool       shellpanevis;      // display shell panel in gui 
   string     tbarmode;          // default mode- pan/range/select/add  
   string     tbarstyle;         // icons/labels/both 
   bool       zoombtn;           // show zoom button
   bool       delbtn;            // show remove button
   bool       guitips;           // show gui tool tips 

   string     meta1;             // meta data
   string     pickcolor;         // color for pick limits bounding box

} DEFVALS_T, *DEFVALS_P_T;


#define  PREF_MAX_LEN  256

#define  PREF_ALL_COLOR         1
#define  PREF_ALL_BGCOLOR       2
#define  PREF_ALL_FONT          4 
#define  PREF_ALL_FONTSTYLE     8
#define  PREF_ALL_THICKNESS    16
#define  PREF_ALL_COUNT        32          
#define  PREF_ALL_LENGTH       64          
#define  PREF_ALL_VISIBLE     128          
#define  PREF_ALL_FILL        256          
#define  PREF_ALL_ASPECT      512          
#define  PREF_ALL_ANGLE      1024          
#define  PREF_ALL_DEPTH      2048


class ChipsDefaults
{
  public:

   ChipsDefaults(bool load=true);
   ChipsDefaults(ChipsStateStream& in);
   virtual ~ChipsDefaults();

   virtual void serialize(ChipsStateStream& out);
   void deserialize(ChipsStateStream& in);

   int    load_file(string in_fname="", bool rec=true);
   int    save_file(string in_fname="", bool override=false); 
   int    set_preference(string pref, string value); 
   char*  get_preference(string pref);

   int    set_overrides(string inlist, string defobj="");
   int    reset_defaults();

   int    get_file_version()            { return file_version; }
   int    get_supported_version()       { return min_version; }

   int    get_allpropmask()             { return allpropmask; }
   string get_allbgcolor()              { return allbgcolor; }
   string get_allcolor()                { return allcolor; }
   string get_allfont()                 { return allfont; }
   V_FONTSTYLE_TYPE get_allfontstyle()  { return allfontstyle; }
   float  get_allthickness()            { return allthickness; } 
   int    get_allcount()                { return allcount; } 
   int    get_alllength()               { return alllength; } 
   V_LINE_TYPE get_alllinestyle()       { return alllinestyle; }
   bool   get_allvisible()              { return allvisible;  }
   bool   get_allfill()                 { return allfill;  }
   bool   get_allaspect()               { return allaspect;  }
   float  get_allangle()                { return allangle; }
   int    get_alldepth()                { return alldepth; }
 
   bool   show_window()    { return curr->windowdisplay; }
   bool   get_windowdisplay() { return curr->windowdisplay; }
   int    set_windowdisplay(bool val) { curr->windowdisplay=val; return 0; }
   bool   get_windowsmoothing() { return curr->windowsmoothing; }
   float  get_windowwidth() { return curr->windowwidth; }
   float  get_windowheight() { return curr->windowheight; }
   float  get_windowscalewidth() { return curr->windowscalewidth; }
   float  get_windowscaleheight() { return curr->windowscaleheight; }
   V_UNITS_TYPE get_windowunits() { return curr->windowunits; }
   string get_windowgeom() { return curr->windowgeom; }
   string get_windowstem() { return curr->windowstem; }
   string get_windowprefix() { return curr->windowprefix; }
   string get_windowid() { return curr->windowid; }
   string get_windowcolor() { return curr->windowcolor; }
   int    set_windowcolor(string val);
   bool   show_grid()    { return curr->showgrid; }
   bool   get_windowfocus() { return curr->windowfocus; }
   bool   get_showgrid() { return curr->showgrid; }
   int    set_showgrid(bool val) { curr->showgrid=val; return 0; }
   bool   show_layout()    { return curr->showlayout; }
   bool   get_showlayout() { return curr->showlayout; }
   int    set_showlayout(bool val) { curr->showlayout=val; return 0; }
   int    get_gridscale() { return curr->gridscale; }
   int    set_gridscale(int val); 
   string get_gridcolor() { return curr->gridcolor; }
   int    set_gridcolor(string val);
   float  get_gridthickness() { return curr->gridthickness; }
   int    set_gridthickness(float val);
   V_LINE_TYPE get_gridlinestyle() { return curr->gridlinestyle; }
   int    set_gridlinestyle(string val);
   string get_layoutcolor() { return curr->layoutcolor; }
   int    set_layoutcolor(string val);
   float  get_layoutthickness() { return curr->layoutthickness; }
   int    set_layoutthickness(float val);
   V_LINE_TYPE get_layoutlinestyle() { return curr->layoutlinestyle; }
   int    set_layoutlinestyle(string val);
   string get_printmethod() { return curr->printmethod; }


   string get_framecolor() { return curr->framecolor; }
   string get_framebordercolor() { return curr->framebordercolor; }
   int    set_framecolor(string val);
   bool   get_framebordervisible() { return curr->framebordervis; }
   bool   frame_aspect() { return curr->frameaspect; }
   bool   get_frameaspect() { return curr->frameaspect; }
   bool   get_framescale()  { return curr->framescale; }
   int    set_frameaspect(bool val)  { curr->frameaspect=val; return 0; }
   int    get_framemode() { return curr->framemode; }
   int    set_framemode(int val);
   int    set_framemode(string val);
   string get_framestem() { return curr->framestem; }
   int    set_framestem(string val);
   string get_frameid() { return curr->frameid; }
   bool   frame_transparent() { return curr->frametransparent; }
   bool   get_frametransparent() { return curr->frametransparent; }
   int    set_frametransparent(bool val) { curr->frametransparent=val; return 0; }

   string get_plotstem() { return curr->plotstem; }
   int    set_plotstem(string val);
   string get_plotid()   { return curr->plotid; }
   float  get_plotleftmargin() { return curr->plotlmargin; }
   float  get_plotrightmargin() { return curr->plotrmargin; }
   float  get_plottopmargin() { return curr->plottmargin; }
   float  get_plotbottommargin() { return curr->plotbmargin; }
   int    get_plottitledepth()       { return curr->titledepth; }
   float  get_plottitleangle()    { return curr->titleangle;   } 
   float  get_plottitlehalign()   { return curr->titlehalign;   } 
   float  get_plottitlevalign()   { return curr->titlevalign;   } 
   float  get_plottitlexpos()    { return curr->titlexpos;    } 
   float  get_plottitleypos()    { return curr->titleypos;    } 
   string get_plottitlecolor()   { return curr->titlecolor;   }
   int    get_plottitlesize()    { return curr->titlesize;    }
   string get_plottitlefont() { return curr->titlefont;  }
   string get_plottitlelabel() { return curr->titlelabel; }
   V_FONTSTYLE_TYPE get_plottitlefontstyle() { return curr->titlefontstyle;  }
   float  get_plotaspect_width() { return curr->plotaspectw; }
   float  get_plotaspect_height() { return curr->plotaspecth; }
   string get_plotratio()  { return curr->plotratio; }

   string get_imagestem() { return curr->imagestem; }
   int    set_imagestem(string val);
   string get_imageid()   { return curr->imageid; }
   int    get_image_depth() { return curr->imagedepth; } 
   V_COLORMAP_TYPE  get_image_colormap() { return curr->imagecmap; } 
   V_INTERPOLATION_TYPE get_image_interpolation() { return curr->image_interp;}
   double get_image_alpha_lower() { return curr->imagealpha[0]; }
   double get_image_alpha_upper() { return curr->imagealpha[1]; }
   double get_image_threshold_lower() { return curr->imagethreshold[0]; }
   double get_image_threshold_upper() { return curr->imagethreshold[1]; }
   bool   get_image_invert_colormap() { return curr->imageinvertcmap; }
   int    get_image_colormap_size() { return curr->imagecmapsize; }
   bool   get_image_colormap_interpolate() { return curr->imagecmapinterp; }
   bool   get_image_scale_channels() { return curr->imagescalechannels; }

   string get_colorbarstem() { return curr->cbarstem; }
   string get_colorbarid()   { return curr->cbarid; }
   float  get_colorbarwidth ()    { return curr->cbarwidth; }
   float  get_colorbarlength ()   { return curr->cbarlength; }
   bool   get_colorbarbordervis() { return curr->cbarbordervis; }
   int    get_colorbardepth()     { return curr->cbardepth; }
   float  get_colorbarhalign()    { return curr->cbarhalign; }
   float  get_colorbarvalign()    { return curr->cbarvalign; }
   int    get_colorbarorient()    { return curr->cbarorient; }
   int    get_colorbartickmode()  { return curr->cbartickmode; }
   V_TICKSTYLE_TYPE get_colorbartickstyle() { return curr->cbartickstyle; }
   string get_colorbartickcolor() { return curr->cbartickcolor; }
   int    get_colorbarticklength() { return curr->cbarticklength; }
   float  get_colorbartickthickness() { return curr->cbartickthickness; }
   V_TICKSTYLE_TYPE get_colorbartickloc() { return curr->cbartickloc; }
   bool   get_colorbartickvisible() { return curr->cbartickvisible; }
   V_TICKSTYLE_TYPE get_colorbarlabloc() { return curr->cbarlabloc; }
   string get_colorbarticklabcolor()  { return curr->cbarticklabcolor; }
   float  get_colorbarticklabangle()  { return curr->cbarticklabangle; } 
   float  get_colorbarticklabhalign()  { return curr->cbarticklabhalign; } 
   float  get_colorbarticklabvalign()  { return curr->cbarticklabvalign; } 
   int    get_colorbarticklabsize()  { return curr->cbarticklabsize; } 
   bool   get_colorbarticklabvisible()  { return curr->cbarticklabvisible; }
   int    get_colorbarticklaboffset()  { return curr->cbarticklaboffset; } 
   string get_colorbarticklabfont()  { return curr->cbarticklabfont; }
   V_FONTSTYLE_TYPE get_colorbarticklabfontstyle() { return curr->cbarticklabfontstyle; }
   int    get_colorbarlabsize()  { return curr->cbarlabsize; }
   float  get_colorbarlabangle()  { return curr->cbarlabangle; }
   string get_colorbarlabcolor()  { return curr->cbarlabcolor; }
   string get_colorbarlabfont()  { return curr->cbarlabfont; }
   float  get_colorbarlabhalign()  { return curr->cbarlabhalign; }
   float  get_colorbarlabvalign()  { return curr->cbarlabvalign; }
   V_FONTSTYLE_TYPE get_colorbarlabfontstyle() { return curr->cbarlabfontstyle; }
   string get_colorbarlabtext() { return curr->cbarlabtext; }
   bool   get_colorbarlabvisible () { return curr->cbarlabvisible; }
   float  get_colorbarparalleloffset()  { return curr->cbaroffsetpar; }
   float  get_colorbarperpendicularoffset()  { return curr->cbaroffsetperp; }


   V_AXISSTYLE_TYPE get_axisstyle() { return curr->axisstyle; }
   V_CORNERSTYLE_TYPE get_cornerstyle() { return curr->cornerstyle; }
   string get_axiscolor() { return curr->axiscolor; }
   int    set_tickformat(string val);
   string get_tickformat() { return curr->tickformat; }
   V_TICKSTYLE_TYPE get_ticklabelstyle()       { return curr->ticklabstyle; }
   string get_ticklabelcolor()   { return curr->ticklabcolor;   }
   float  get_ticklabelhalign()   { return curr->ticklabhalign;   }
   float  get_ticklabelvalign()   { return curr->ticklabvalign;   }
   float  get_ticklabelangle()   { return curr->ticklabangle;   }
   int    get_ticklabelsize()    { return curr->ticklabsize;    }
   int    get_ticklabeloffset()  { return curr->ticklaboffset;  }
   string get_ticklabelfont() { return curr->ticklabfont;  }
   V_FONTSTYLE_TYPE get_ticklabelfontstyle() { return curr->ticklabfontstyle;  }
   int    set_axiscolor(string val);
   float  get_axisthickness() { return curr->axisthickness; }
   int    set_axisthickness(float val);
   string get_axisxstem() { return curr->axisxstem; }
   string get_axisystem() { return curr->axisystem; }
   int    set_axisxstem(string val);
   int    set_axisystem(string val);
   string get_axisxid() { return curr->axisxid; }
   string get_axisyid() { return curr->axisyid; }
   string get_axisxlabel() { return curr->axisxlabel; }
   string get_axisylabel() { return curr->axisylabel; }
   float  get_axisparalleloffset()  { return curr->axis_offsetpar; }
   float  get_axisperpendicularoffset()  { return curr->axis_offsetperp; }
   bool   get_axisautomin()         { return curr->axis_automin; }
   bool   get_axisautomax()         { return curr->axis_automax; }
   int    get_axisdepth()           { return curr->axisdepth; }
   double get_axispad()             { return curr->axis_pad; }
   string get_majortickcolor() { return curr->majortickcolor; }
   string get_minortickcolor() { return curr->minortickcolor; }
   string get_majortickmode() { return curr->majortickmode; } 
   string get_minortickmode() { return curr->minortickmode; } 
   float  get_majortickinterval() { return curr->majortickinterval; }
   float  get_minortickinterval() { return curr->minortickinterval; }
   unsigned int get_majortickcount() { return curr->majortickcount; }
   unsigned int get_minortickcount() { return curr->minortickcount; }
   float  get_majortickthickness()   { return curr->majortickthickness; }
   float  get_minortickthickness()   { return curr->minortickthickness; }
   int    get_majorticklength()      { return curr->majorticklength; }
   int    get_minorticklength()      { return curr->minorticklength; }
   V_TICKSTYLE_TYPE get_majortickstyle()       { return curr->majortickstyle; }
   V_TICKSTYLE_TYPE get_minortickstyle()       { return curr->minortickstyle; }
   bool   get_majortickvisible()     { return curr->majortickvisible; }
   bool   get_minortickvisible()     { return curr->minortickvisible; }
   string get_axlabcolor()   { return curr->axlabcolor;   }
   float  get_axlabhalign()   { return curr->axlabhalign;   }
   float  get_axlabvalign()   { return curr->axlabvalign;   }
   float  get_axlabxangle()   { return curr->axlabxangle;   }
   float  get_axlabyangle()   { return curr->axlabyangle;   }
   int    get_axlabsize()    { return curr->axlabsize;    }
   string get_axlabfont() { return curr->axlabfont;  }
   V_FONTSTYLE_TYPE get_axlabfontstyle() { return curr->axlabfontstyle;  }
   string get_majorgridcolor()       { return curr->majorgridcolor; }
   string get_minorgridcolor()       { return curr->minorgridcolor; }
   float  get_majorgridthickness()   { return curr->majorgridthickness; }
   float  get_minorgridthickness()   { return curr->minorgridthickness; }
   bool   get_majorgridvisible()     { return curr->majorgridvisible; }
   bool   get_minorgridvisible()     { return curr->minorgridvisible; }
   V_LINE_TYPE get_majorgridstyle()  { return curr->majorgridstyle;  }
   V_LINE_TYPE get_minorgridstyle()  { return curr->minorgridstyle;  }

   string get_curvestem() { return curr->curvestem; }
   string get_curveid() { return curr->curveid; }
   int    set_curvestem(string val);
   V_SYMBOL_TYPE get_curvesymbolstyle() { return curr->curvesymbolstyle; }
   int    set_curvesymbolstyle(string val);
   int    set_curvesymbolstyle(V_SYMBOL_TYPE val);
   string get_curvesymbolcolor() { return curr->curvesymbolcolor; }
   int    set_curvesymbolcolor(string val);
   bool   curvesymbol_filled() { return curr->curvesymbolfill; }
   bool   get_curvesymbolfill() { return curr->curvesymbolfill; }
   int    set_curvesymbolfill(bool val)  { curr->curvesymbolfill=val; return 0; }
   int    get_curvedepth()           { return curr->curvedepth; }
   int    get_curvesymbolsize() { return curr->curvesymbolsize; }
   float  get_curvesymbolangle()   { return curr->curvesymbolangle;   }
   V_LINE_TYPE get_curvelinestyle() { return curr->curvelinestyle; }
   int    set_curvelinestyle(string val);
   int    set_curvelinestyle(V_LINE_TYPE val);
   string get_curvelinecolor() { return curr->curvelinecolor; }
   int    set_curvelinecolor(string val);
   float  get_curvelinethickness() { return curr->curvelinethickness; }
   bool   get_errup()     { return curr->errup; }
   bool   get_errdown()   { return curr->errdown; }
   bool   get_errleft()   { return curr->errleft; }
   bool   get_errright()  { return curr->errright; }
   V_ERRSTYLE_TYPE get_errstyle() { return curr->errstyle; }
   string get_errcolor() { return curr->errcolor; }
   int    get_errcaplength() { return curr->errcaplen; }
   float  get_errthickness() { return curr->errthickness; }
   float  get_limitlength() { return curr->limitlength; }
   float  get_limitoverride() { return curr->limitoverride; }
   V_BADDATAMODE_TYPE get_curvebaddatamode() { return curr->curvebaddata;}

   // Histogram attributes 
   string get_histogramstem() { return curr->histstem; }
   string get_histogramid() { return curr->histid; }
   V_SYMBOL_TYPE get_histogramsymbolstyle() { return curr->histsymbolstyle; }
   string get_histogramsymbolcolor() { return curr->histsymbolcolor; }
   bool   histogramsymbol_filled() { return curr->histsymbolfill; }
   bool   get_histogramsymbolfill() { return curr->histsymbolfill; }
   int    get_histogramsymbolsize() { return curr->histsymbolsize; }
   int    get_histogramdepth()      { return curr->histdepth; }
   float  get_histogramsymbolangle()   { return curr->histsymbolangle;   }
   V_LINE_TYPE get_histogramlinestyle() { return curr->histlinestyle; }
   string get_histogramlinecolor() { return curr->histlinecolor; }
   float  get_histogramlinethickness() { return curr->histlinethickness; }
   bool   get_histogramerrup()     { return curr->histerrup; }
   bool   get_histogramerrdown()   { return curr->histerrdown; }
   bool   get_histogramdropline()   { return curr->histdropline; }
   V_ERRSTYLE_TYPE get_histogramerrstyle() { return curr->histerrstyle; }
   string get_histogramerrcolor() { return curr->histerrcolor; }
   float  get_histogramerrcaplength() { return curr->histerrcaplen; }
   float  get_histogramerrthickness() { return curr->histerrthickness; }
   string get_histogramfillcolor() { return curr->histfillcolor; }
   float  get_histogramfillopacity() { return curr->histfillopacity; }
   V_FILL_TYPE get_histogramfillstyle(){ return curr->histfillstyle; }


   // Contour attributes
   string get_contourstem() { return curr->contourstem; }
   string get_contourid() { return curr->contourid; }
   float  get_contourthickness() { return curr->contourthickness; }
   string get_contourcolor() { return curr->contourcolor; }
   V_LINE_TYPE get_contourstyle() { return curr->contourstyle; }
   void get_contourlevels(vector<double> &out_levels) { out_levels = curr->contourlevels; }
   unsigned int get_contournumlevels() { return curr->contournumlevels; }
   V_CONTOUR_TYPE get_contouralgorithm() { return curr->contouralgorithm; }
   V_TICKMODE_TYPE get_contourmode() { return curr->contourmode; }
   float  get_contourinterval()     { return curr->contourinterval; }
   int    get_contourdepth()        { return curr->contourdepth; }

   // Region attributes
   string get_regionstem()          { return curr->regionstem; }
   string get_regionid()            { return curr->regionid; }
   string get_regionfillcolor()     { return curr->regionfillcolor; }
   float  get_regionedgethickness() { return curr->regionthickness; }
   V_LINE_TYPE get_regionedgestyle(){ return curr->regionstyle; }
   V_FILL_TYPE get_regionfillstyle(){ return curr->regionfillstyle; }
   string get_regionedgecolor()     { return curr->regioncolor; }
   float  get_regionangle()         { return curr->regionangle;   }
   double get_regionopacity()       { return curr->regionopacity; }
   int    get_regiondepth()         { return curr->regiondepth; }

   // Label attributes
   string get_labelstem()    { return curr->labelstem;    }
   string get_labelid()      { return curr->labelid;    }
   string get_labelcolor()   { return curr->labelcolor;   }
   int    get_labelsize()    { return curr->labelsize;    }
   int    get_labeldepth()   { return curr->labeldepth; }
   float  get_labelangle()   { return curr->labelangle;   }
   float  get_labelhalign()  { return curr->labelhalign;   }
   float  get_labelvalign()  { return curr->labelvalign;   }
   string get_labelfont()    { return curr->labelfont;    }
   V_FONTSTYLE_TYPE get_labelfontstyle() { return curr->labelstyle;  }

   V_SYMBOL_TYPE get_pointstyle() { return curr->pointstyle; }
   int    set_pointstyle(string val);
   int    set_pointstyle(V_SYMBOL_TYPE val);
   string get_pointcolor() { return curr->pointcolor; }
   int    set_pointcolor(string val);
   bool   point_filled() { return curr->pointfill; }
   bool   get_pointfill() { return curr->pointfill; }
   int    set_pointfill(bool val)  { curr->pointfill=val; return 0; }
   int    get_pointsize() { return curr->pointsize; }
   string get_pointstem() { return curr->pointstem; }
   int    set_pointstem(string val);
   int    get_pointdepth() { return curr->pointdepth; }
   string get_pointid() { return curr->pointid; }
   float  get_pointangle()   { return curr->pointangle;   }

   V_LINE_TYPE get_linestyle() { return curr->linestyle; }
   int    set_linestyle(string val);
   int    set_linestyle(V_LINE_TYPE val);
   string get_linecolor() { return curr->linecolor; }
   int    set_linecolor(string val);
   float  get_linethickness() { return curr->linethickness; }
   string get_linestem() { return curr->linestem; }
   string get_lineid() { return curr->lineid; }
   int    set_linestem(string val);
   int    get_linedepth()   { return curr->linedepth; }
   V_LINE_EXTENSION_TYPE get_lineextend () { return curr->lineextend; };

   string get_legendstem()  { return curr->legendstem; }
   int    set_legendstem(string val);
   string get_legendcolor() { return curr->legendcolor; }
   int    set_legendcolor(string val);
   float  get_legendthickness() { return curr->legendthickness; }
   int    set_legendthickness(float val);
   bool   legend_transparent() { return curr->legendtransparent; }
   bool   get_legendtransparent() { return curr->legendtransparent; }
   int    set_legendtransparent(bool val) { curr->legendtransparent=val; return 0; }
   bool   legend_borders() { return curr->legendborders; }
   bool   get_legendborders() { return curr->legendborders; }
   int    set_legendborders(bool val) { curr->legendborders=val; return 0; }
   int    get_legenddepth()   { return curr->legenddepth; }

   string get_defaultfgps()       { return defaultfgps; }
   string get_defaultbgps()       { return defaultbgps; }
   string get_defaultfgdisp()     { return defaultfgdisp; }
   string get_defaultbgdisp()     { return defaultbgdisp; }
   int    get_defaultdepth()      { return defaultdepth; }
   int    get_defaultverbosity()  { return defaultverb; }
   
   bool   get_exportclobber()     { return curr->exportclobber; }
   bool   get_exportfonts()       { return curr->exportfonts; }
   int    get_exportorient()      { return curr->exportorient; }
   string get_exportformat()      { return curr->exportformat; }
   V_PRINTFILE_TYPE get_exportcolorscheme() { return curr->exportcolorscheme; }
   int    get_exportdpi()         { return curr->exportdpi; }
   float  get_pageleftmargin() { return curr->pagelmargin; }
   int    set_pageleftmargin(float val);
   float  get_pagerightmargin() { return curr->pagermargin; }
   int    set_pagerightmargin(float val);
   float  get_pagetopmargin() { return curr->pagetmargin; }
   int    set_pagetopmargin(float val);
   float  get_pagebottommargin() { return curr->pagebmargin; }
   int    set_pagebottommargin(float val);
   float  get_pagewidth() { return curr->pagewidth; }
   int    set_pagewidth(float val);
   float  get_pageheight() { return curr->pageheight; }
   int    set_pageheight(float val);
   float  get_pagescalewidth() { return  curr->pagescalewidth; }
   int    set_pagescalewidth(float val);
   float  get_pagescaleheight() { return  curr->pagescaleheight; }
   int    set_pagescaleheight(float val);
   V_UNITS_TYPE get_pageunits () { return curr->pageunits; }
   int    set_pageunits (V_UNITS_TYPE val);
   int    set_pageunits (string val);
   bool   get_fittopage() { return curr->fittopage; }
   int    set_fittopage(bool val);
   bool   get_pageaspect() { return curr->pageaspect; }
   int    set_pageaspect(bool val);
   bool   get_printdialog() { return curr->printdialog; }

   bool   get_toolbarvis() { return curr->toolbarvis; }
   bool   get_navpanevis() { return curr->navpanevis; }
   bool   get_infpanevis() { return curr->infpanevis; }
   bool   get_proppanevis() { return curr->proppanevis; }
   bool   get_shellpanevis() { return curr->shellpanevis; }
   string get_tbarmode()     { return curr->tbarmode; }
   string get_tbarstyle()    { return curr->tbarstyle; }
   bool   get_tbarzoombtn()  { return curr->zoombtn; }
   bool   get_tbardelbtn()   { return curr->delbtn; }
   bool   get_guitips()      { return curr->guitips; }
  
   unsigned int  get_pickmask()   { return pickmask; }
   string get_pickcolor()    { return curr->pickcolor; }
   bool   get_infocurrent()  { return infocurrent; }
   bool   get_infodepth()    { return infodepth; }
   bool   get_infocoord()    { return infocoord; }

   string get_meta()         { return curr->meta1; }

   int    parseLine(const char *in_txt, char del=':', bool rec=true);

   int    get_version_from_file(string fname); 
 
  protected:

   int    default_load_overrides(); 
   int    set_boolean_value(bool& bv, char* val);
   int    set_aspect_ratio(DEFVALS_P_T* cp, char* val); 

   int    processLine(bool ignore_checks=false);

   template <class TemplateType>
   int    string_to_array (vector<TemplateType> &out_array);
   template <class TemplateType>
   int    string_to_string_array (vector<TemplateType> &out_array);

   void   get_window_prefs(string &pref);
   void   get_frame_prefs(string &pref);
   void   get_plot_prefs(string &pref);
   void   get_axis_prefs(string &pref);
   void   get_image_prefs(string &pref);
   void   get_curve_prefs(string &pref);
   void   get_histogram_prefs(string &pref);
   void   get_contour_prefs(string &pref);
   void   get_region_prefs(string &pref);
   void   get_label_prefs(string &pref);
   void   get_line_prefs(string &pref);
   void   get_point_prefs(string &pref);
   void   get_legend_prefs(string &pref);
   void   get_devicecolor_prefs(string &pref);
   void   get_export_prefs(string &pref);
   void   get_gui_prefs(string &pref);
   void   get_pick_prefs(string &pref);
   void   get_info_prefs(string &pref);
   void   get_default_prefs(string &pref);
   void   get_colorbar_prefs(string &pref);
 
   int    unknown_property_error(); 
   int    thickness_value_error(float val);
   int    no_property_value_error();
   int    data_range_error(float val, float min, float max);
   int    data_range_error(int val, int min, int max);
   int    negative_value_error(float val, int eval=ERR_NO_FLT_NEGATIVE);
   int    negative_value_error(int val, int eval=ERR_NO_INT_NEGATIVE);
   int    data_subset_error(const char* val, const char* subset,
                            int eval=ERR_DATA_SUBSET);

   string    filename;           // name of parameter file to use
   int       file_version;       // version info for current rcfile 
   int       min_version;        // minimal usable version for rcfile 
   bool      version_exclude;    // if true, exclude based on file version

   // application defaults 
   string    defaultfgps; 
   string    defaultbgps; 
   string    defaultfgdisp; 
   string    defaultbgdisp; 
   int       defaultdepth; 
   int       defaultverb; 

   int       allpropmask;        // mask of all.properties to set
   string    allbgcolor;         // all background color 
   string    allcolor;           // all foreground color 
   V_LINE_TYPE alllinestyle;     // all linestyle
   float     allthickness;       // all line thickess 
   int       allcount;           // all tick count 
   int       alllength;          // all tick length
   int       alldepth;           // all depth values
   string    allfont;            // all label font
   V_FONTSTYLE_TYPE allfontstyle; // all font style- italic, bold, etc...
   bool      allvisible;         // all visibility settings
   bool      allfill;            // all symbol fills
   bool      allaspect;          // all aspect settings 
   float     allangle;           // set all label angles 

   unsigned int  pickmask;       // mask of events for pickpoints
   bool      infocurrent;        // add reverse video for currency to info
   bool      infocoord;          // add coord system to info
   bool      infodepth;          // add depth values to info

  private:
   char       errbuf[PREF_MAX_LEN]; // error message buffer 
   char       buf[PREF_MAX_LEN]; // line entry buffer 
   char      *obj_p;             // object portion of line entry
   char      *prop_p;            // property portion of line entry
   char      *value_p;           // value portion of line entry

   bool       load_overrides;    // initalize the override values

   DEFVALS_P_T  curr; 
   DEFVALS_T    defvals;
   DEFVALS_T    overrides; 
};


template <class TemplateType>
int ChipsDefaults::string_to_array (vector<TemplateType> &out_array)
{
   string value_string = value_p;
   int rval = ERR_STATUS_GOOD;

   if (value_string[value_string.length ()-1] != ']')
      rval = ERR_INVALID_ARRAY;
   else
   { 
      // parse the string and populate the output vector

      int last_pos = 1;
      int curr_pos = 1;
      istringstream value;
      TemplateType tmp_value;

      while (curr_pos > 0)
      {
         // find the next delimiter. ignore delimiters that are escaped
         // by back slashes
         curr_pos = last_pos-1;
         while (1)
         {
            curr_pos = value_string.find_first_of(",]", curr_pos+1);

            // check to see if the delimeter is being escaped
            if ((curr_pos > 0) && (value_string[curr_pos-1] == '\\'))
            {

               // find all the back slashes. if there are an even
               // number of back slashes then break otherwise the
               // delimiter is being escaped

               int ii = curr_pos;
               while (ii>0 && value_string[--ii] == '\\');
               if ((((curr_pos-1) - (ii+1))%2) == 1)
                  break;
               else
               {
                  // remove the back slash
                  curr_pos--;
                  value_string.erase (curr_pos, 1);
               }
            }
            else
               break;
         }


         // check if its the end of the string
         if (curr_pos > 1)
         {

            // format the string into the actual data type
            value.clear ();
            value.str(value_string.substr(last_pos, curr_pos-last_pos));
            value >> tmp_value;
            if (value.fail () || value.bad () || !value.eof ())
            {
               rval = ERR_INVALID_ARRAY_ELEM;
               break;
            }
            out_array.push_back (tmp_value);
         }

         last_pos = curr_pos+1;
      }
   }

   return rval;
}

template <class TemplateType>
int ChipsDefaults::string_to_string_array (vector<TemplateType> &out_array)
{
   string value_string = value_p;
   int rval = ERR_STATUS_GOOD;

   if (value_string[value_string.length ()-1] != ']')
      rval = ERR_INVALID_ARRAY;
   else
   {
      // parse the string and populate the output vector

      int last_pos = 1;
      int curr_pos = 1;
      istringstream value;
      TemplateType tmp_value;

      while (curr_pos > 0)
      {
         // find the next delimiter. ignore delimiters that are escaped
         // by back slashes
         curr_pos = last_pos-1;
         while (1)
         {
            curr_pos = value_string.find_first_of(",]", curr_pos+1);

            // check to see if the delimeter is being escaped
            if ((curr_pos > 0) && (value_string[curr_pos-1] == '\\'))
            {

               // find all the back slashes. if there are an even
               // number of back slashes then break otherwise the
               // delimiter is being escaped

               int ii = curr_pos;
               while (ii>0 && value_string[--ii] == '\\');
               if ((((curr_pos-1) - (ii+1))%2) == 1)
                  break;
               else
               {
                  // remove the back slash
                  curr_pos--;
                  value_string.erase (curr_pos, 1);
               }
            }
            else
               break;
         }


         // check if its the end of the string
         if (curr_pos > 1)
         {

            // format the string into the actual data type
            value.clear ();
            value.str(value_string.substr(last_pos, curr_pos-last_pos));
            value >> tmp_value;
            if (value.fail () || value.bad ())
            {
               rval = ERR_INVALID_ARRAY_ELEM;
               break;
            }
            
            // add the spaces back into the array element
            if (!value.eof ())
            {
               TemplateType spaced_value = tmp_value;
               do
               {
                  spaced_value += " ";
                  value >> tmp_value;
                  spaced_value += tmp_value;
               } while (!value.eof ());
               tmp_value = spaced_value;
            }

            out_array.push_back (tmp_value);
         }

         last_pos = curr_pos+1;
      }
   }

   return rval;
}



#endif 
