// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <string>
#include <string.h>
#include <ChipsErrs.hh>

#include "cxcds_version.h"

using namespace std;


#ifndef CHIPSDEFINES_HH
#define CHIPSDEFINES_HH

#define  V_USE_DEFAULT_DEPTH         0

// frame placement modes 
#define  V_FRAMEMODE_PLACED        0
#define  V_FRAMEMODE_SINGLE        1 
#define  V_FRAMEMODE_TILED         2 

#define  X_AXIS                     0x01
#define  Y_AXIS                     0x02
#define  XY_AXIS                    (X_AXIS | Y_AXIS)

#define  RA_AXIS                     0x01
#define  DEC_AXIS                    0x02
#define  RADEC_AXIS                  (RA_AXIS | DEC_AXIS)


#define  V_SCALE_LINEAR             0
#define  V_SCALE_LOG                1

#define V_MOVE_ABSOLUTE             0
#define V_MOVE_DELTA                1 

#define V_LOAD_STATE                0x01
#define V_LOAD_COMMANDS             0x02
#define V_CLEAR_ALL                 0x80
#define V_LOAD_ALL                  (V_LOAD_STATE | V_LOAD_COMMANDS) 

#define V_CHECKPOINT_VAL            52801
#define V_STATE_MAJ_VER             3
#define V_STATE_MIN_VER             16

#define CHIPS_VERSION_STR           "4.6"
#define CHIPS_COPYRIGHT_STR         CXCDS_COPYRIGHT_STRING
#define CHIPS_VERSION               40601

// enumerate coordinated systems
enum COORD_SYS 
{
   COORD_NONE, PIXEL, FRAME_NORM, DATA, PLOT_NORM, WINDOW_NORM, PIXEL_REAL
};

// enumerated type specifying valid line types 
enum V_LINE_TYPE
{ 
   chips_noline, 
   chips_solid, 
   chips_dot, 
   chips_dotlongdash, 
   chips_dotshortdash, 
   chips_longdash, 
   chips_shortdash, 
   chips_shortdashlongdash 
}; 

// enumerated type specifying fill patterns 
enum V_FILL_TYPE
{
   chips_nofill, 
   chips_solidfill,
   chips_updiagonal,
   chips_downdiagonal, 
   chips_horizontal,
   chips_vertical,
   chips_crisscross,
   chips_brick, 
   chips_grid, 
   chips_hexagon, 
   chips_polkadot,
   chips_zigzag,
   chips_wave,
   chips_flower, 
   chips_userfill1,
   chips_userfill2,
   chips_userfill3, 
   chips_fill_size
};

// enumerated type specifying rgba
enum V_RGBA_TYPE
{
   chips_red,
   chips_green,
   chips_blue,
   chips_alpha
};

// enumerated type specifying color maps
enum V_COLORMAP_TYPE
{
   // to account for overlap in names start this enumeration at 4-
   // red,green,blue from rga and grayscale from print enumerations
   chips_rainbow = 4,
   chips_hsv,
   chips_heat,
   chips_cool,
   chips_usercmap1,
   chips_usercmap2,
   chips_usercmap3,
   chips_cmap_size
};


// enumeration specifying image interpolation method
enum V_INTERPOLATION_TYPE
{
   chips_neighbor,
   chips_bilinear,
   chips_bicubic
};

// enumerated type specifying object types
enum V_OBJECT_TYPE
{
   chips_not_object,
   chips_object,
   chips_window,
   chips_frame,
   chips_image,
   chips_plot,
   chips_curve,
   chips_histogram,
   chips_contour,
   chips_axis,
   chips_label,
   chips_line,
   chips_legend,
   chips_point,
   chips_region, 
   chips_colorbar, 
   chips_coordinate_system
};

// enumerated type specifying sub-object types
enum V_SUBOBJECT_TYPE
{
   chips_not_subobject,
   chips_plot_title,
   chips_axis_label,
   chips_colorbar_label
};



// enumerated type specifying valid symbol types 
enum V_SYMBOL_TYPE 
{
   chips_none, 
   chips_cross, 
   chips_diamond, 
   chips_downtriangle, 
   chips_circle, 
   chips_plus, 
   chips_square, 
   chips_uptriangle,
   chips_point_type,
   chips_arrow
};


// enumerated type specifying valid unit types 
enum V_UNITS_TYPE
{
   chips_nounit,
   chips_cm, 
   chips_mm, 
   chips_inch, 
   chips_pixels
};


// enumerated type specifying valid font styles
enum V_FONTSTYLE_TYPE
{
   chips_normal, 
   chips_bold, 
   chips_italic,
   chips_bolditalic 
};


// enumerated type specifying valid fonts
enum V_FONT_TYPE
{
   chips_helvetica, 
   chips_courier, 
   chips_times,
   chips_greek
};


// enumerated type specifying valid contour algorithms
enum V_CONTOUR_TYPE
{
   chips_standard,
   chips_marching
};


// enumerated type for specifying tick mode
enum V_TICKMODE_TYPE
{
   chips_count,
   chips_interval,
   chips_nice,
   chips_limits, 
   chips_arbitrary
};


// enumerated type for specifying tick style
enum V_TICKSTYLE_TYPE
{
   chips_inside,
   chips_outside,
   chips_centered
};


// enumerated type specifying curve errbar styles
enum V_ERRSTYLE_TYPE
{
   chips_bar,
   chips_cap
};


// enumerated type specifying bad data handling styles
enum V_BADDATAMODE_TYPE
{
   chips_ignore,
   chips_omit
};


// enumerated type for specifying axis style
enum V_AXISSTYLE_TYPE
{
   chips_open,
   chips_box,
   chips_closed
};


// enumerated type for specifying axis corner styles
enum V_CORNERSTYLE_TYPE
{
   chips_miter, 
   chips_bevel, 
   chips_fill,
   chips_notch
};


enum V_LINE_EXTENSION_TYPE
{
   chips_extend_none,
   chips_extend_start,
   chips_extend_end,
   chips_extend_both
};


// enumerated type for specifying print file type
enum V_PRINTFILE_TYPE
{
   chips_nofiletype,
   chips_rgb,
   chips_cmyk,
   chips_grayscale,
   chips_cmyksep
};


enum V_SHUFFLE_TYPE
{
   chips_forward,
   chips_backward,
   chips_front,
   chips_back
};

// pickpoints event structure 
typedef struct chips_pick_evt 
{
   double       x;     // x position 
   double       y;     // y position 
   string       x_str; // x string position 
   string       y_str; // y string position
   unsigned int event;  // event mask 
   char         key;   // key of KEY_PRESS event  
} CHIPS_PICK_EVT, *CHIPS_PICK_EVT_P;  



// type to indicate autolimits use on axis limits command
typedef string V_AUTOLIMIT_TYPE; 
#define        AUTO              "auto"

// value to indicate auto-position of ticklabels
#define        AUTO_ALIGN        -99.0

#define KEY_PRESS     0x0001
#define BTN1_DOWN     0x0002
#define BTN2_DOWN     0x0004
#define BTN3_DOWN     0x0008
#define KEY_RELEASE   0x0010
#define BTN1_UP       0x0020 
#define BTN2_UP       0x0040 
#define BTN3_UP       0x0080 
#define BTNS_DOWN     (BTN1_DOWN | BTN2_DOWN | BTN3_DOWN)
#define BTNS_UP       (BTN1_UP | BTN2_UP | BTN3_UP)
#define MOTION_NOTIFY 0x0100


// converters from enums to strings  
extern const char* v_symbol_to_string(V_SYMBOL_TYPE val);
extern const char* v_line_to_string(V_LINE_TYPE val);
extern const char* v_fill_to_string(V_FILL_TYPE val);
extern const char* v_colormap_to_string(V_COLORMAP_TYPE val);
extern const char* v_rgba_to_string(V_RGBA_TYPE val);
extern const char* v_interpolation_to_string(V_INTERPOLATION_TYPE val); 
extern const char* v_units_to_string(V_UNITS_TYPE val);
extern const char* v_font_to_string(V_FONT_TYPE val);
extern const char* v_contour_to_string(V_CONTOUR_TYPE val);
extern const char* v_fontstyle_to_string(V_FONTSTYLE_TYPE val);
extern const char* v_errstyle_to_string(V_ERRSTYLE_TYPE val);
extern const char* v_baddatamode_to_string(V_BADDATAMODE_TYPE val);
extern const char* v_axisstyle_to_string(V_AXISSTYLE_TYPE val);
extern const char* v_cornerstyle_to_string(V_CORNERSTYLE_TYPE val);
extern const char* v_tickmode_to_string(V_TICKMODE_TYPE val);
extern const char* v_tickstyle_to_string(V_TICKSTYLE_TYPE val);
extern const char* hoffset_to_string(float val); 
extern const char* voffset_to_string(float val); 
extern const char* pickmask_to_string(unsigned int mask);
extern const char* v_printfile_to_string(V_PRINTFILE_TYPE val);
extern const char* v_pagesize_to_string(float w, float h, V_UNITS_TYPE u);
extern const char* v_object_to_string(V_OBJECT_TYPE);
extern const char* v_line_extension_to_string (V_LINE_EXTENSION_TYPE val);
extern const char* v_coord_sys_to_string (COORD_SYS val);
extern const char* v_coord_sys_to_enum_string(COORD_SYS val);


// converters from strings to enums
extern int string_to_v_symbol(const char* val, V_SYMBOL_TYPE* out_symbol); 
extern int string_to_v_units(const char* val, V_UNITS_TYPE* unt);
extern int string_to_v_line(const char* val, V_LINE_TYPE* out_line);
extern int string_to_v_fill(const char* val, V_FILL_TYPE* out_line);
extern int string_to_v_colormap(const char* val, V_COLORMAP_TYPE* out_cmap);
extern int string_to_v_rgba(const char* val, V_RGBA_TYPE* out_cmap);
extern int string_to_v_interpolation(const char* val, V_INTERPOLATION_TYPE* out_int);
extern int string_to_v_font(const char* val, V_FONT_TYPE* out_font);
extern int string_to_v_contour(const char* val, V_CONTOUR_TYPE* out_contour);
extern int string_to_v_fontstyle(const char* val, V_FONTSTYLE_TYPE* out_fontstyle);
extern int string_to_v_errstyle(const char* val, V_ERRSTYLE_TYPE* out_errstyle); 
extern int string_to_v_baddatamode(const char* val, V_BADDATAMODE_TYPE* out_baddatamode); 
extern int string_to_v_axisstyle(const char* val, V_AXISSTYLE_TYPE* out_axisstyle); 
extern int string_to_v_cornerstyle(const char* val, V_CORNERSTYLE_TYPE* out_cornerstyle); 
extern int string_to_v_tickmode(const char* val, V_TICKMODE_TYPE* out_tickmode); 
extern int string_to_v_tickstyle(const char* val, V_TICKSTYLE_TYPE* out_tickstyle); 
extern int string_to_hoffset(const char* val, float* pos);
extern int string_to_voffset(const char* val, float* pos);
extern int string_to_v_printfile(const char* val, V_PRINTFILE_TYPE* type); 
extern int string_to_pickmask(const char* val, unsigned int* mask); 
extern int string_to_pagesize(const char* val, float &w, float &h, V_UNITS_TYPE &u);
extern int string_to_v_object(const char* val, V_OBJECT_TYPE* out_objecttype); 
extern int string_to_v_line_extension (const char* val, V_LINE_EXTENSION_TYPE *type);
extern int string_to_v_coord_sys (const char* val, COORD_SYS *type);

extern int string_to_ratio_values(const char* val, float &w, float &h); 

// checks if valids are legit 
extern bool valid_line_type(V_LINE_TYPE val);
extern bool valid_fill_type(V_FILL_TYPE val);
extern bool valid_colormap_type(V_COLORMAP_TYPE val);
extern bool valid_rgba_type(V_RGBA_TYPE val);
extern bool valid_interpolation_type(V_INTERPOLATION_TYPE val);
extern bool valid_symbol_type(V_SYMBOL_TYPE val);
extern bool valid_units_type(V_UNITS_TYPE val);
extern bool valid_font_type(V_FONT_TYPE val);
extern bool valid_contour_type(V_CONTOUR_TYPE val);
extern bool valid_fontstyle_type(V_FONTSTYLE_TYPE val);
extern bool valid_errstyle_type(V_ERRSTYLE_TYPE val);
extern bool valid_baddatamode_type(V_BADDATAMODE_TYPE val);
extern bool valid_axisstyle_type(V_AXISSTYLE_TYPE val);
extern bool valid_cornerstyle_type(V_CORNERSTYLE_TYPE val);
extern bool valid_tickmode_type(V_TICKMODE_TYPE val);
extern bool valid_tickstyle_type(V_TICKSTYLE_TYPE val);
extern bool valid_printfile_type(V_PRINTFILE_TYPE val);
extern bool valid_object_type(V_OBJECT_TYPE val); 
extern bool valid_line_extension_type (V_LINE_EXTENSION_TYPE val);
extern bool valid_coord_sys_type (COORD_SYS val);
extern bool valid_shuffle_type(V_SHUFFLE_TYPE val);
extern bool valid_color(string in_color);
extern int chips_strcmp(const char *aa, const char *bb);

extern bool approx_equal(double d1, double d2, double relerr = 0.00001, double maxerr= 0.000000001);

#endif
