// 
//  Copyright (C) 2004-2009,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <iostream>
#include "ChipsDefinesP.hh"
#include "ChipsColor.hh"
#include "ChipsCmdManager.hh"

extern ChipsCmdManager* cmd_manager;

int chips_solid_pattern=0xFFFF;
int chips_dot_pattern=0x5555;
int chips_dotlongdash_pattern=0xE4FF;
int chips_dotshortdash_pattern=0x2727;
int chips_longdash_pattern=0x07FF;
int chips_shortdash_pattern=0x0F0F;
int chips_shortdashlongdash_pattern=0x3CFF;



int c2_string_to_line(const char* val, V_LINE_TYPE* line, const char* prop)
{
   int rval = string_to_v_line(val, line);
   if (cmd_manager && (rval == ERR_BAD_LINESTYLE))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [noline|solid|dot|dotlongdash|dotshortdash|longdash|shortdash|shortdashlongdash]");
      cmd_manager->add_err(errstr);
   }
   return rval; 
}

int c2_string_to_fill(const char* val, V_FILL_TYPE* fill, const char* prop)
{
   int rval = string_to_v_fill(val, fill);
   if (cmd_manager && (rval == ERR_BAD_FILLSTYLE))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [nofill|solidfill|updiagonal|downdiagonal|horizontal|vertical|crisscross|brick|grid|hexagon|polkadot|zigzag|wave|flower|userfill1|userfill2|userfill3]");
      cmd_manager->add_err(errstr);
   }
   return rval; 
}


int c2_string_to_symbol(const char* val, V_SYMBOL_TYPE* sym, const char* prop)
{
   int rval = string_to_v_symbol(val, sym);
   if (cmd_manager && (rval == ERR_BAD_SYMBOL))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [none|cross|diamond|downtriangle|circle|plus|square|uptriangle|point|arrow]");
      cmd_manager->add_err(errstr);
   }
   return rval; 
}

int c2_string_to_units(const char* val, V_UNITS_TYPE* unt, const char* prop)
{
   int rval = string_to_v_units(val, unt);
   if (cmd_manager && (rval == ERR_BAD_UNITS))
   {
      string errstr = "The ";
      if (prop) 
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [cm|mm|inches|pixels]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_fontstyle(const char* val, V_FONTSTYLE_TYPE* fst, const char* prop)
{
   int rval = string_to_v_fontstyle(val, fst);
   if (cmd_manager && (rval == ERR_BAD_FONTSTYLE))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [normal|bold|italic|bolditalic]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_hoffset(const char* val, float* fval, const char* prop)
{
   int rval = string_to_hoffset(val, fval);
   if (cmd_manager && (rval == ERR_INVALID_NUMERIC_ARG))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be numeric or [left|center|right|auto]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_voffset(const char* val, float* fval, const char* prop)
{
   int rval = string_to_voffset(val, fval);
   if (cmd_manager && (rval == ERR_INVALID_NUMERIC_ARG))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be numeric or [base|center|top|auto]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}


int c2_string_to_errstyle(const char* val, V_ERRSTYLE_TYPE* styl, const char* prop)
{
   int rval = string_to_v_errstyle(val, styl);
   if (cmd_manager && (rval == ERR_BAD_ERRSTYLE))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [bar|line|cap|capped]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_axisstyle(const char* val, V_AXISSTYLE_TYPE* astyl, const char* prop)
{
   int rval = string_to_v_axisstyle(val, astyl);
   if (cmd_manager && (rval == ERR_BAD_PLOTSTYLE))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [open|box|closed]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_tickstyle(const char* val, V_TICKSTYLE_TYPE* tstyl, const char* prop)
{
   int rval = string_to_v_tickstyle(val, tstyl);
   if (cmd_manager && (rval == ERR_DATA_RANGE))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [inside|outside|centered]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_tickmode(const char* val, V_TICKMODE_TYPE* tmode, const char* prop)
{
   int rval = string_to_v_tickmode(val, tmode);
   if (cmd_manager && (rval == ERR_BAD_TICKMODE))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [count|interval|nice|limits|arbitrary]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_object(const char* val, V_OBJECT_TYPE* obj, const char* prop)
{
   int rval = string_to_v_object(val, obj);
   if (cmd_manager && (rval == ERR_BAD_OBJECT))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [window|frame|plot|curve|histogram|contour|axis|label|line|point|region|coordsys]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}


int c2_string_to_line_extension(const char* val, V_LINE_EXTENSION_TYPE* line, 
                                const char* prop)
{
   int rval = string_to_v_line_extension(val, line);
   if (cmd_manager && (rval == ERR_BAD_LINEEXTENSION))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [none|start|end|both]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}



bool c2_valid_color(string in_color)
{
   ChipsColor clr;
   return (clr.valid_color(in_color) == ERR_STATUS_GOOD);
}

int c2_string_to_interpolation(const char* val, V_INTERPOLATION_TYPE* line, 
                               const char* prop)
{
   int rval = string_to_v_interpolation(val, line);
   if (cmd_manager && (rval == ERR_BAD_IMG_INTERP))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [neighbor|bilinear|bicubic]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_colormap(const char* val, V_COLORMAP_TYPE* line, 
                          const char* prop)
{
   int rval = string_to_v_colormap(val, line);
   if (cmd_manager && (rval == ERR_BAD_CMAP))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [red|green|blue|grayscale|rainbow|hsv|heat|cool|usercmap1|usercmap2|usercmap3]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_rgba(const char* val, V_RGBA_TYPE* color,
                      const char* prop)
{
   int rval = string_to_v_rgba(val, color);
   if (cmd_manager && (rval == ERR_BAD_RGBA))
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [red|green|blue|alpha]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}

int c2_string_to_cbarmode(const char* val, int* tmode, const char* prop)
{
   int rval = ERR_STATUS_GOOD;
   if (val == NULL || tmode == NULL)   
      rval = ERR_DATA_SUBSET;
   else if (!chips_strcmp(val, "none") || !chips_strcmp(val, "0")) 
      *tmode = 0;
   else if (!chips_strcmp(val, "ends") || !chips_strcmp(val, "1")) 
      *tmode = 1;
   else if (!chips_strcmp(val, "nice") || !chips_strcmp(val, "2")) 
      *tmode = 2;
   else if (cmd_manager) 
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [none|ends|nice|0|1|2]");
      cmd_manager->add_err(errstr);
   }
   return rval;
}


int c2_string_to_gridscale(const char* val, int* scale, const char* prop)
{
   int rval = ERR_STATUS_GOOD;
   if (val == NULL || scale == NULL)
      rval = ERR_DATA_SUBSET;
   else if (!chips_strcmp(val, "default") || !chips_strcmp(val, "0"))
      *scale = 0;
   else if (!chips_strcmp(val, "quarterinch") || !chips_strcmp(val, "1"))
      *scale = 1;
   else if (!chips_strcmp(val, "halfinch") || !chips_strcmp(val, "2"))
      *scale = 2;
   else if (!chips_strcmp(val, "centimeter") || !chips_strcmp(val, "cm") ||
            !chips_strcmp(val, "3"))
      *scale = 3;
   else if (!chips_strcmp(val, "normalized") || !chips_strcmp(val, "4"))
      *scale = 4;
   else if (cmd_manager)
   {
      string errstr = "The ";
      if (prop)
      {
         errstr.append(prop);
         errstr.append(" ");
      }
      errstr.append("value (");
      errstr.append(val);
      errstr.append(") must be one of [default|quarterinch|halfinch|cm|normalized|0|1|2|3|4]");
      cmd_manager->add_err(errstr);
      rval = ERR_DATA_SUBSET;
   }
   return rval;
}

const char*  c2_gridscale_to_string(int val)
{
   const char* gv[6] ={"default","quarterinch","halfinch","cm","normalized",""};
   const char *ans = NULL;
   if (val > -1 && val < 5)
      ans = gv[val]; 
   else 
      ans = gv[5]; 
   return ans;      
} 



bool flt_equal (float A, float B)
{
   float maxAbsoluteError = FLT_MIN;
   float maxRelativeError = FLT_EPSILON;
   float relativeError;

// TEMPORARY 
//   return (fabs(A-B) < maxRelativeError);
      

   if (fabs(A - B) < maxAbsoluteError)
      return true;

   if (fabs(B) > fabs(A))
      relativeError = fabs((A - B) / B);
   else
      relativeError = fabs((A - B) / A);

   return (relativeError <= maxRelativeError);
}

bool dbl_equal (double A, double B)
{
   double maxAbsoluteError = DBL_MIN;
   double maxRelativeError = DBL_EPSILON;
   double relativeError;

   if (fabs(A - B) < maxAbsoluteError)
      return true;

   if (fabs(B) > fabs(A))
      relativeError = fabs((A - B) / B);
   else
      relativeError = fabs((A - B) / A);

   return (relativeError <= maxRelativeError);
}

