// 
//  Copyright (C) 2004-2009,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSDEFINESP_HH
#define CHIPSDEFINESP_HH

#include <string>
#include <ChipsDefines.hh>

using namespace std;


// conversion factors
#define  V_INCH2PIX_CONV            72.00
#define  V_CM2PIX_CONV              28.35
#define  V_MM2PIX_CONV              2.835
#define  V_DEF_WINDOW_SCALE	    1.25

#define  V_DEF_WINDOW_DIM          360 
#define  V_MAX_TILE_COLS           10
#define  V_MAX_TILE_ROWS           10
#define  V_MAX_MARGIN_SIZE          0.99
#define  V_MIN_PAR_OFFSET          -0.5
#define  V_MAX_PAR_OFFSET          0.5
#define  V_MIN_PERP_OFFSET        -100
#define  V_MAX_PERP_OFFSET         100
#define  V_DEFAULT_DEPTH           100

// window grid values 
#define  V_GRID_SCALE_DEFAULT      0
#define  V_GRID_SCALE_QTR_INCH     1 
#define  V_GRID_SCALE_HALF_INCH    2 
#define  V_GRID_SCALE_CENTIMETER   3 
#define  V_GRID_SCALE_NORMALIZED   4

// label/symbol sizes 
#define  V_SIZE_MIN_VAL            1
#define  V_SIZE_DEF_VAL            5
#define  V_SIZE_MAX_VAL            100

// line thickness sizes
#define  V_THICK_MIN_VAL           0.5
#define  V_THICK_DEF_VAL           1
#define  V_THICK_MAX_VAL           10

#define  V_PI                      3.141592653589793238462643

#define  Z_AXIS                     0x04 
#define  XZ_AXIS                    (X_AXIS | Z_AXIS)
#define  YZ_AXIS                    (Y_AXIS | Z_AXIS)
#define  XYZ_AXIS                   (X_AXIS | Y_AXIS | Z_AXIS)

// set method classifications
#define  SET_POINT_SYMBOL_STYLE        0
#define  SET_POINT_SYMBOL_COLOR        1
#define  SET_POINT_SYMBOL_SIZE         2
#define  SET_POINT_SYMBOL_FILL         3 
#define  SET_POINT_SYMBOL_ANGLE        4
#define  SET_POINT_VISIBLE             5
#define  SET_POINT_DEPTH               6

#define  SET_CURVE_LIMLENGTH           7
#define  SET_CURVE_LIMOVERRIDE         8
#define  SET_CURVE_SYMBOL_ANGLE        9
#define  SET_CURVE_SYMBOL_STYLE       10
#define  SET_CURVE_SYMBOL_COLOR       11  
#define  SET_CURVE_SYMBOL_SIZE        12 
#define  SET_CURVE_SYMBOL_FILL        13
#define  SET_CURVE_LINE_STYLE         14 
#define  SET_CURVE_LINE_COLOR         15
#define  SET_CURVE_LINE_THICKNESS     16
#define  SET_CURVE_ERR_STYLE          17 
#define  SET_CURVE_ERR_COLOR          18
#define  SET_CURVE_ERR_THICKNESS      19 
#define  SET_CURVE_ERR_UP             20
#define  SET_CURVE_ERR_DOWN           21
#define  SET_CURVE_ERR_LEFT           22
#define  SET_CURVE_ERR_RIGHT          23
#define  SET_CURVE_VISIBLE            24

#define  SET_LINE_STYLE               25
#define  SET_LINE_COLOR               26
#define  SET_LINE_THICKNESS           27
#define  SET_LINE_VISIBLE             28
#define  SET_LINE_DEPTH               29

#define  SET_AXIS_COLOR               30
#define  SET_AXIS_THICKNESS           31
#define  SET_AXIS_TEXT                32
#define  SET_AXIS_MAJ_STYLE           33
#define  SET_AXIS_TICKFORMAT          34
#define  SET_AXIS_MAJOR_NUM           35
#define  SET_AXIS_MINOR_NUM           36
#define  SET_AXIS_MAJOR_COLOR         37
#define  SET_AXIS_MINOR_COLOR         38
#define  SET_AXIS_MAJORINT            39
#define  SET_AXIS_MINORINT            40
#define  SET_AXIS_MAJORLENGTH         41
#define  SET_AXIS_MINORLENGTH         42
#define  SET_AXIS_VISIBLE             43
#define  SET_AXIS_SCALABLE            44
#define  SET_AXIS_MAJ_VIS             45
#define  SET_AXIS_MIN_VIS             46
#define  SET_AXIS_MAJ_THICKNESS       47 
#define  SET_AXIS_MIN_THICKNESS       48 
#define  SET_AXIS_MIN_STYLE           49
#define  SET_AXIS_MAJ_MODE            50
#define  SET_AXIS_MIN_MODE            51
#define  SET_AXIS_REVERSE             52
#define  SET_AXIS_OFFSET_PAR          53 
#define  SET_AXIS_OFFSET_PERP         54 
#define  SET_AXIS_LOG_MIN             55
#define  SET_AXIS_TICKLABEL_STYLE     56
#define  SET_AXIS_PAD                 57
#define  SET_AXIS_AUTOMIN             58
#define  SET_AXIS_AUTOMAX             59
#define  SET_GRID_THICKNESS            60
#define  SET_GRID_COLOR                61
#define  SET_GRID_LINESTYLE            62
#define  SET_GRID_SCALE                63
#define  SET_LAYOUT_COLOR              64
#define  SET_LAYOUT_LINESTYLE          65
#define  SET_LAYOUT_THICKNESS          66
#define  SET_AXIS_TICKSTYLE            67
#define  SET_AXIS_DEPTH                68
#define  SET_CURVE_DEPTH               69
#define  SET_PLOT_LMARGIN              70 
#define  SET_PLOT_RMARGIN              71 
#define  SET_PLOT_TMARGIN              72 
#define  SET_PLOT_BMARGIN              73 
#define  SET_PLOT_LOGSCALE             75 
#define  SET_PLOT_LINEARSCALE          76 
#define  SET_PLOT_TITLE                77 
#define  SET_PLOT_VISIBLE              78
#define  SET_PLOT_XLABEL               79 
#define  SET_PLOT_YLABEL               80 
#define  SET_PLOT_ZLABEL               81 
#define  SET_PLOT_TITLE_COLOR          86
#define  SET_PLOT_TITLE_X              87
#define  SET_PLOT_TITLE_Y              88
#define  SET_PLOT_TITLE_FONT           89
#define  SET_PLOT_TITLE_FONTSTYLE      90
#define  SET_PLOT_TITLE_SIZE           91
#define  SET_PLOT_AXIS_STYLE           92
#define  SET_PLOT_TITLE_ANGLE          93 
#define  SET_PLOT_TITLE_HALIGN         94 
#define  SET_PLOT_TITLE_VALIGN         95 
#define  SET_PLOT_CORNER               96
#define  SET_PLOT_DEPTH                97
#define  SET_PLOT_OFFSCREEN            98

#define  SET_WINDOW_COLOR             100 
#define  SET_WINDOW_TITLE             101 
#define  SET_WINDOW_REDRAW            102
#define  SET_WINDOW_VISIBLE           103

#define  SET_FRAME_COLOR              110 
#define  SET_FRAME_TRANSPARENT        112 
#define  SET_FRAME_SCALE              113 
#define  SET_FRAME_BORDER_COLOR       114
#define  SET_FRAME_BORDER_VIS         115
#define  SET_FRAME_VISIBLE            116 

#define  SET_LABEL_BBOX               125
#define  SET_LABEL_DEPTH              126
#define  SET_LABEL_TEXT               127
#define  SET_LABEL_FONT               128
#define  SET_LABEL_HALIGN             129
#define  SET_LABEL_VALIGN             130
#define  SET_LABEL_STYLE              131 
#define  SET_LABEL_SIZE               132 
#define  SET_LABEL_COLOR              133
#define  SET_LABEL_ANGLE              134
#define  SET_LABEL_VISIBLE            135

#define  SET_AXIS_LABEL_COLOR         136
#define  SET_AXIS_LABEL_HALIGN        137
#define  SET_AXIS_LABEL_VALIGN        138
#define  SET_AXIS_LABEL_ANGLE         139
#define  SET_AXIS_LABEL_FONT          140
#define  SET_AXIS_LABEL_FONTSTYLE     141
#define  SET_AXIS_LABEL_SIZE          142

#define  SET_AXIS_TICKLABEL_COLOR     143
#define  SET_AXIS_TICKLABEL_OFFSET    144
#define  SET_AXIS_TICKLABEL_FONT      145
#define  SET_AXIS_TICKLABEL_FNTSTL    146
#define  SET_AXIS_TICKLABEL_SIZE      147
#define  SET_AXIS_TICKLABEL_HALIGN    148
#define  SET_AXIS_TICKLABEL_VALIGN    149
#define  SET_AXIS_TICKLABEL_ANGLE     150

#define  SET_AXIS_MAJGRD_VIS          151
#define  SET_AXIS_MAJGRD_COLOR        152
#define  SET_AXIS_MAJGRD_STYLE        153
#define  SET_AXIS_MAJGRD_THICK        154
#define  SET_AXIS_MINGRD_VIS          155
#define  SET_AXIS_MINGRD_COLOR        156
#define  SET_AXIS_MINGRD_STYLE        157
#define  SET_AXIS_MINGRD_THICK        158
#define  SET_AXIS_TICKLABEL_VIS       159

#define  SET_CONTOUR_COLOR            160
#define  SET_CONTOUR_STYLE            161
#define  SET_CONTOUR_THICKNESS        162
#define  SET_CONTOUR_MODE             163 
#define  SET_CONTOUR_TICK_VIS         164
#define  SET_CONTOUR_TICK_COLOR       165
#define  SET_CONTOUR_TICK_THICK       166
#define  SET_CONTOUR_TICK_LENGTH      167
#define  SET_CONTOUR_TICK_DIR         168
#define  SET_CONTOUR_LAB_VIS          169
#define  SET_CONTOUR_LAB_COLOR        170
#define  SET_CONTOUR_LAB_SIZE         171
#define  SET_CONTOUR_LAB_FONT         172
#define  SET_CONTOUR_LAB_FONTSTYLE    173
#define  SET_CONTOUR_VISIBLE          174
#define  SET_CONTOUR_ALGORITHM        175
#define  SET_CONTOUR_INTERVAL         176
#define  SET_CONTOUR_NUMLEVELS        177
#define  SET_CONTOUR_DEPTH            178
#define  SET_HISTO_SYMBOL_STYLE       190   
#define  SET_HISTO_SYMBOL_COLOR       191  
#define  SET_HISTO_SYMBOL_SIZE        192 
#define  SET_HISTO_SYMBOL_FILL        193
#define  SET_HISTO_LINE_STYLE         194
#define  SET_HISTO_LINE_COLOR         195
#define  SET_HISTO_LINE_THICKNESS     196
#define  SET_HISTO_ERR_STYLE          197
#define  SET_HISTO_ERR_COLOR          198
#define  SET_HISTO_ERR_THICKNESS      199
#define  SET_HISTO_ERR_UP             200
#define  SET_HISTO_ERR_DOWN           201
#define  SET_HISTO_DROPLINE           202
#define  SET_HISTO_VISIBLE            203
#define  SET_HISTO_SYMBOL_ANGLE       204
#define  SET_HISTO_DEPTH              205
#define  SET_HISTO_FILL_COLOR         206
#define  SET_HISTO_FILL_OPACITY       207
#define  SET_HISTO_FILL_STYLE         209
#define  SET_HISTO_FILL_STYLE2        210

#define  SET_LEGEND_COLOR             230
#define  SET_LEGEND_THICKNESS         231
#define  SET_LEGEND_TRANSPARENT       232
#define  SET_LEGEND_VISIBLE           233
#define  SET_LEGEND_BORDERS           234

#define  SET_EXPORT_FONTS             240
#define  SET_EXPORT_ORIENT            241
#define  SET_PAGE_LMARGIN             242
#define  SET_PAGE_RMARGIN             243
#define  SET_PAGE_TMARGIN             244
#define  SET_PAGE_BMARGIN             245
#define  SET_PAGE_WIDTH               246
#define  SET_PAGE_HEIGHT              247
#define  SET_PAGE_UNITS               248
#define  SET_PAGE_SIZE                249
#define  SET_PAGE_SCALE_WIDTH         250
#define  SET_PAGE_SCALE_HEIGHT        251
#define  SET_PAGE_SCALE_UNITS         252
#define  SET_PAGE_FIT                 253
#define  SET_PAGE_KEEP_ASPECT         254

#define  SET_IMAGE_CMAP               260
#define  SET_IMAGE_INV_CMAP           261
#define  SET_IMAGE_CMAP_SIZE          262
#define  SET_IMAGE_CMAP_INTERPOLATE   263
#define  SET_IMAGE_INTERPOLATION      264
#define  SET_IMAGE_ALPHA              265
#define  SET_IMAGE_THRESHOLD          266
#define  SET_IMAGE_DEPTH              267
#define  SET_IMAGE_VISIBLE            268
#define  SET_IMAGE_SCALE_CHANNELS     269
#define  GET_IMAGE_CMAP_ID            270

#define  SET_DEFAULT_DEPTH            275

#define  GET_PLOT_DATA_RANGE          295
#define  GET_PLOT_RANGE               296
#define  GET_HISTO_RANGE              297
#define  GET_CONTOUR_RANGE            298
#define  GET_CURVE_RANGE              299
#define  GET_CURVE_XRANGE             300
#define  GET_CURVE_YRANGE             301
#define  GET_CONTOUR_XRANGE           302
#define  GET_CONTOUR_YRANGE           303
#define  GET_CONTOUR_ZRANGE           304
#define  GET_HISTO_XRANGE             305
#define  GET_HISTO_YRANGE             306
#define  GET_PLOT_XRANGE              307
#define  GET_PLOT_YRANGE              308
#define  GET_AXIS_RANGE               309
#define  GET_IMAGE_RANGE              310
#define  GET_IMAGE_XRANGE             311
#define  GET_IMAGE_YRANGE             312
#define  SET_AXIS_LABEL_VIS           313
#define  SET_CURVE_BADDATA            314
#define  SET_AXIS_XLAB_TXT            316
#define  SET_AXIS_YLAB_TXT            317
#define  SET_AXIS_XLAB_ANG            318
#define  SET_AXIS_YLAB_ANG            319
#define  SET_AXIS_LLCORNER            320
#define  SET_AXIS_LLCORNWID           321
#define  SET_AXIS_URCORNER            322
#define  SET_AXIS_URCORNWID           323
#define  SET_AXIS_SCALE               325
#define  GET_AXIS_SCALE               326
#define  SET_LINE_EXTEND              327

#define  SET_PLOT_ASPECT_RATIO        330
#define  SET_PLOT_ASPECT_WIDTH        331
#define  SET_PLOT_ASPECT_HEIGHT       332
#define  SET_DATA_ASPECT_RATIO        340

#define  SET_REGN_EDGE_COLOR          350
#define  SET_REGN_EDGE_STYLE          351
#define  SET_REGN_EDGE_THICKNESS      352  
#define  SET_REGN_FILL_COLOR          353
#define  SET_REGN_ANGLE               354
#define  SET_REGN_VISIBLE             356
#define  SET_REGN_OPACITY             357
#define  SET_REGN_DEPTH               358
#define  SET_REGN_FILL_STYLE          359
#define  SET_REGN_FILL_STYLE2         360

#define  SET_CBAR_BORDERVIS           370 
#define  SET_CBAR_DEPTH               371
#define  SET_CBAR_HALIGN              372
#define  SET_CBAR_LENGTH              373 
#define  SET_CBAR_LABEL_ANGLE         374
#define  SET_CBAR_LABEL_COLOR         375
#define  SET_CBAR_LABEL_FONT          376
#define  SET_CBAR_LABEL_FONTSTYLE     377
#define  SET_CBAR_LABEL_HALIGN        378
#define  SET_CBAR_LABEL_VALIGN        379
#define  SET_CBAR_LABEL_SIZE          380
#define  SET_CBAR_LABEL_VIS           381 
#define  SET_CBAR_LABEL_LOCATION      382
#define  SET_CBAR_ORIENTATION         383 
#define  SET_CBAR_TEXT                384
#define  SET_CBAR_TICK_THICKNESS      385 
#define  SET_CBAR_TICK_LENGTH         386
#define  SET_CBAR_TICK_COLOR          387 
#define  SET_CBAR_TICK_MODE           388 
#define  SET_CBAR_TICK_STYLE          389
#define  SET_CBAR_TICK_LOCATION       390
#define  SET_CBAR_TICK_VIS            391
#define  SET_CBAR_TICKLABEL_ANGLE     392
#define  SET_CBAR_TICKLABEL_COLOR     393
#define  SET_CBAR_TICKLABEL_FONT      394
#define  SET_CBAR_TICKLABEL_FNTSTL    395
#define  SET_CBAR_TICKLABEL_OFFSET    396
#define  SET_CBAR_TICKLABEL_SIZE      397
#define  SET_CBAR_TICKLABEL_HALIGN    398
#define  SET_CBAR_TICKLABEL_VALIGN    399
#define  SET_CBAR_TICKLABEL_VIS       400
#define  SET_CBAR_VISIBLE             401
#define  SET_CBAR_VALIGN              402
#define  SET_CBAR_WIDTH               403           
#define  SET_CBAR_OFFSET_PAR          404
#define  SET_CBAR_OFFSET_PERP         405

#define  SET_CURVE_ERR_CAPLEN         420
#define  SET_HISTO_ERR_CAPLEN         421


#define  CMD_MNGER_MAX_WEIGHT         200

#define  CLEAR_ALL_ERRS                -1
#define  CLEAR_DNE_ERRS                -2

// stencil patterns for lines 
extern int chips_solid_pattern;
extern int chips_dot_pattern;
extern int chips_dotlongdash_pattern;
extern int chips_dotshortdash_pattern;
extern int chips_longdash_pattern;
extern int chips_shortdash_pattern;
extern int chips_shortdashlongdash_pattern;


extern int c2_string_to_line(const char* val, V_LINE_TYPE* out_line,
                             const char* prop = NULL);
extern int c2_string_to_fill(const char* val, V_FILL_TYPE* out_fill,
                             const char* prop = NULL);
extern int c2_string_to_symbol(const char* val, V_SYMBOL_TYPE* out_symbol,
                               const char* prop = NULL);
extern int c2_string_to_units(const char* val, V_UNITS_TYPE* unt,
                               const char* prop = NULL);
extern int c2_string_to_fontstyle(const char* val, V_FONTSTYLE_TYPE* fontstyle,
                               const char* prop = NULL);
extern int c2_string_to_errstyle(const char* val, V_ERRSTYLE_TYPE* errstyle,
                               const char* prop = NULL);
extern int c2_string_to_axisstyle(const char* val, V_AXISSTYLE_TYPE* astyle,
                               const char* prop = NULL);
extern int c2_string_to_tickstyle(const char* val, V_TICKSTYLE_TYPE* tstyle,
                               const char* prop = NULL);
extern int c2_string_to_tickmode(const char* val, V_TICKMODE_TYPE* tmode,
                               const char* prop = NULL);
extern int c2_string_to_object(const char* val, V_OBJECT_TYPE* obj,
                               const char* prop = NULL);

extern int c2_string_to_hoffset(const char* val, float* pos, const char* prop = NULL);
extern int c2_string_to_voffset(const char* val, float* pos, const char* prop = NULL);

extern int c2_string_to_line_extension(const char* val, 
                               V_LINE_EXTENSION_TYPE* line, const char* prop);


extern bool c2_valid_color(string in_color);

extern int c2_string_to_colormap(const char* val, V_COLORMAP_TYPE* obj,
                               const char* prop = NULL);
extern int c2_string_to_rgba(const char* val, V_RGBA_TYPE* obj,
                               const char* prop = NULL);
extern int c2_string_to_interpolation(const char* val, V_INTERPOLATION_TYPE* obj,
                               const char* prop = NULL);

extern int c2_string_to_cbarmode(const char* val, int* mode, const char* prop = NULL);

extern int c2_string_to_gridscale(const char* val, int* scale, const char* prop = NULL);
extern const char*  c2_gridscale_to_string(int val);

// alternative approx equal functions
extern bool flt_equal (float A, float B);
extern bool dbl_equal (double A, double B);


#endif
