// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsDeleteCommand.hh"
#include "ChipsPlot.hh"
#include "ChipsWindow.hh"
#include "ChipsFrame.hh"

ChipsDeleteCommand::ChipsDeleteCommand(ChipsObject* cob, const string &curr)
{
   co = cob;
   if (co->get_object_type() == chips_window)
      weight = 25; 
   else 
      weight = 5;
   current = curr;

}

ChipsDeleteCommand::ChipsDeleteCommand(ChipsObject* cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   if (co->get_object_type() == chips_window)
      weight = 25; 
   else 
      weight = 5;

   // detach chips object from the coordinate system
   if (co && co->get_chips_coord ())
      co->get_chips_coord ()->remove (co);
}


ChipsDeleteCommand::ChipsDeleteCommand(ChipsStateStream& in)
{
   co = NULL;
   deserialize(in);
   if (co && co->get_object_type() == chips_window)
      weight = 25; 
   else 
      weight = 5;

}

ChipsDeleteCommand::~ChipsDeleteCommand()
{
}


int ChipsDeleteCommand::execute ()
{
   if (!co)
      return ERR_STATUS_BAD;
   if (co->get_parent ())
   {
      return co->get_parent ()->dereference_object(co,false);
   }
   else
   {
      return co->dereference_object(co,false);
   }
   return ERR_STATUS_BAD;
}

int ChipsDeleteCommand::unexecute ()
{
   if (!co)
      return ERR_STATUS_BAD;
   if (co->get_parent ())
      return co->get_parent ()->reference_object(co,false,current);
   else
      return co->reference_object(co,false,current);
   return ERR_STATUS_BAD;
}

int ChipsDeleteCommand::clear ()
{
   co = NULL;
   return ERR_STATUS_BAD;
}


void ChipsDeleteCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Delete " << v_object_to_string(co->get_object_type ()) << endl;
};

void ChipsDeleteCommand::serialize(ChipsStateStream& out)
{
   int val = 2;
   out.write_int(&val);

   serialize_object(out);
     
   out.write_int(&ref);
   out.write_int(&block_ref);

   out.write_string(&current);
}

void ChipsDeleteCommand::deserialize(ChipsStateStream& in)
{
   in.read_int(&ref);
   in.read_int(&block_ref);

   in.read_string(&current);

}


void ChipsDeleteCommand::clear_objects()
{
   if (co)
   {
      if (co->get_parent())
         co->get_parent()->clear_cmd (co);
      else
         co->clear_cmd(co);
      co = NULL;
   }
}

