// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSDEPTHCONTROL_HH
#define CHIPSDEPTHCONTROL_HH

#include <string>
#include <iostream>
#include <stdio.h>
#include <map>

#include "ChipsDefines.hh"
#include "ChipsDepthCollapseCommand.hh"

class ChipsObject;
class ChipsStateStream;
class ChipsFrame;

using namespace std;


typedef map<int, bool, less<int> > depth_vis_map;
typedef multimap<int, ChipsObject*, less<int> > depth_obj_map;


class ChipsDepthControl
{
  public:
   ChipsDepthControl(int val=V_DEFAULT_DEPTH);
   ChipsDepthControl(ChipsFrame* frm, ChipsStateStream& in);

   virtual ~ChipsDepthControl();

  public:
   int        add_object(ChipsObject* obj);
   int        remove_object(ChipsObject* obj);
   int        rearrange_object(ChipsObject* obj, int manner); 
   int        display_depth(int depth); 
   int        hide_depth(int depth);
   int        reset_addition_count(int* start);
   int        collapse_depths(int* sources, int src_cnt, int dest, 
                                      ChipsDepthCollapseCommand* cmd);
   int        uncollapse_depths(multimap<int, int> &dest_table, int src_depth);
/*   int        set_depths (int *obj_ref_cnt, int obj_cnt, int depth);
*/
   int        lowest_depth(); 
   char*      list_depths(int depth=0);
   int        set_default_depth(int val);
   int        get_default_depth()   { return default_depth; }
   int        sort_check();

   bool       get_depth_visible (int depth=0);
   int        get_shuffle_cnt()  { return shuffle_cnt; }

   // retrieve the position of an object with respect to other
   //  objects at the same depth
   int        position_in_depth (int obj_ref_cnt, int obj_depth);
   int        total_objects_in_depth (int depth);
  
   virtual int sort_depth (int start_layer=0);
   virtual int sort_single_item ();
   virtual int sort_last_item ();

   void       serialize(ChipsStateStream& out);
   void       serialize_ascii(ChipsStateStream& out, string plid="");
   int        deserialize_obj_table(ChipsFrame* frm, ChipsStateStream& in);

  private:
   string list_at_depth(int dep); 

   depth_obj_map  obj_table;
   depth_vis_map  vis_table;
   int            add_cnt;       // number of added objects to sort 
   int            start_depth;   // determines which depth to sort at
   int            default_depth; // depth to utilize for 'default' depth
   int            shuffle_cnt;   // number of object shuffles performed

};


#endif 
