// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsDepthUpdateCommand.hh"
#include "ChipsStateStream.hh"
#include "ChipsPlot.hh"

ChipsDepthUpdateCommand::ChipsDepthUpdateCommand(ChipsObject* parent, ChipsObject* obj, int adddel, int pos)
{
   co = parent;
   weight = 1;
   addrm = adddel;
   if (obj)
   {
      otype = obj->get_object_type();
      oid   = obj->get_id();
      position = pos;
   }
}

ChipsDepthUpdateCommand::~ChipsDepthUpdateCommand()
{
}


ChipsDepthUpdateCommand::ChipsDepthUpdateCommand(ChipsObject *cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsDepthUpdateCommand::execute ()
{
   int rval = ERR_STATUS_BAD;
   if (co)
   {
      V_OBJECT_TYPE tp; 
      if ((tp = co->get_object_type()) == chips_frame)
      {
         ChipsFrame *cf = dynamic_cast <ChipsFrame *>(co);
         rval = cf->update_depth_table(otype, oid, addrm, position);
      }
      else if (tp == chips_plot)
      {
         ChipsPlot *cp = dynamic_cast <ChipsPlot *>(co);
         rval = cp->update_depth_table(otype, oid, addrm, position);
      }
   }
   return rval;
}

int ChipsDepthUpdateCommand::unexecute ()
{
   int rval = ERR_STATUS_BAD;
   if (co)
   {
      V_OBJECT_TYPE tp; 
      int unex = (addrm == 0);
      if ((tp = co->get_object_type()) == chips_frame)
      {
         ChipsFrame *cf = dynamic_cast <ChipsFrame *>(co);
         rval = cf->update_depth_table(otype, oid, unex, position);
      }
      else if (tp == chips_plot)
      {
         ChipsPlot *cp = dynamic_cast <ChipsPlot *>(co);
         rval = cp->update_depth_table(otype, oid, unex, position);
      }
   }
   return rval;
}

void ChipsDepthUpdateCommand::serialize(ChipsStateStream& out)
{
   int val = 19;
   // Standard serialize output for a command
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   // Specific output for depth vis command
   out.write_int(&addrm);
   val = (int)otype;
   out.write_int(&val);
   out.write_int(&position);
   out.write_string(&oid);
}

void ChipsDepthUpdateCommand::deserialize(ChipsStateStream& in)
{
   // Standard serialize input for a command
   in.read_int(&ref);
   in.read_int(&block_ref);

   // Specific input for depth vis command
   in.read_int(&addrm);
   int val;
   in.read_int(&val);
   in.read_int(&position);
   otype = (V_OBJECT_TYPE)val; 
   in.read_string(&oid);
}

void ChipsDepthUpdateCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Depth Update" << endl;
};



