// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsDepthVisCommand.hh"
#include "ChipsStateStream.hh"

ChipsDepthVisCommand::ChipsDepthVisCommand(ChipsFrame *frame, int inDepth, bool inVisible)
{
   co = frame;
   weight = 1;
   depth = inDepth;
   visible = inVisible;
}

ChipsDepthVisCommand::~ChipsDepthVisCommand()
{
}


ChipsDepthVisCommand::ChipsDepthVisCommand(ChipsObject *cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsDepthVisCommand::execute ()
{
   int rval = ERR_STATUS_BAD;
   if (co)
   {
      ChipsFrame *cf = dynamic_cast <ChipsFrame *>(co);
      if (visible)
         rval = cf->display_depth (depth, false);
      else
         rval = cf->hide_depth (depth, false);
   }
   return rval;
}

int ChipsDepthVisCommand::unexecute ()
{
   int rval = ERR_STATUS_BAD;
   if (co)
   {
      ChipsFrame *cf = dynamic_cast <ChipsFrame *>(co);
      if (visible)
         rval = cf->hide_depth (depth, false);
      else
         rval = cf->display_depth (depth, false);
   }
   return rval;
}

void ChipsDepthVisCommand::serialize(ChipsStateStream& out)
{
   int val = 15;
   // Standard serialize output for a command
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   // Specific output for depth vis command
   out.write_int (&depth);
   out.write_bool (&visible);
}

void ChipsDepthVisCommand::deserialize(ChipsStateStream& in)
{
   // Standard serialize input for a command
   in.read_int(&ref);
   in.read_int(&block_ref);

   // Specific input for depth vis command
   in.read_int (&depth);
   in.read_bool (&visible);
}

void ChipsDepthVisCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Depth Visible" << endl;
};



