// 
//  Copyright (C) 2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsError.hh"
#include "ChipsErrs.hh"

ChipsError::ChipsError(string oid, int errid, string txt, int typ)
{
   obj_id = oid;
   id = errid;
   if (txt == "default")
   {
      text = get_error_message(id); 
   } 
   else  
      text = txt;
   type = typ; 
}

ChipsError::ChipsError(string oid, int errid, int typ, va_list ap)
{
   char buffer[1024];

   obj_id = oid;
   id = errid;
   type = typ;

   // generate string from variable length arg list 
   vsprintf(buffer, get_error_message(id), ap);
   text = buffer; 
}


ChipsError::~ChipsError()
{

}

string ChipsError::get_message(int verbosity)
{
   if (!type)   msg_buffer = "chips ERROR: ";
      else      msg_buffer = "chips WARNING: ";
   msg_buffer.append(text); 
   if (verbosity > 2)
   {
      static string delim = "->";
      string loc = " {object: ";
      int added = 0;
      if (win_id != "")
      {
         loc.append(win_id);   
         added = 1;
      }
      if (frame_id != "")
      {
         if (added)  loc.append(delim);
         loc.append(frame_id);   
         added = 1; 
      }
      if (plot_id != "")
      {
         if (added)  loc.append(delim);
         loc.append(plot_id);   
         added = 1; 
      }
      if (obj_id != "")
      {
         if (added)  loc.append(delim);
         loc.append(obj_id);   
      }
      loc.append("}");
      if (loc != " {object: }")  msg_buffer.append(loc);
   }
   msg_buffer.append("\n");
   return msg_buffer; 
}




