// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSERROR_HH
#define CHIPSERROR_HH

#include <string>
#include <iostream>
#include <stdio.h>
#include <stdarg.h>
#include "ChipsDefines.hh"

using namespace std;


class ChipsError 
{
  public:

   ChipsError(string oid, int errid, string txt="default", int typ=0);
   ChipsError(string oid, int errid, int typ, va_list ap);
   virtual ~ChipsError(); 

   string  get_message(int verbosity=1);
   int     get_id()    { return id;   }
   string  get_text()  { return text; } 
   int     get_type()  { return type; }
   int     get_count() { return cnt;  }

   void    set_window(string val)  { if (win_id == "")  win_id = val; }
   void    set_frame(string val)   { if (frame_id == "")  frame_id = val; }
   void    set_plot(string val)    { if (plot_id == "")  plot_id = val; }
   void    set_object(string val)  { if (obj_id == "")  obj_id = val; }
   
  private:
   int     id;
   int     type;      // 0 = error, 1 = warning 
   string  text;      // error text
   int     cnt;       // number of occurrances of this exact error 

   string win_id;     // window where this occurred
   string frame_id;   // frame where this occurred
   string plot_id;    // plot where this occurred
   string obj_id;     // object where this occurred


   string msg_buffer;  
};


#endif 
