// 
//  Copyright (C) 2004-2007,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsErrs.hh"

#include <stdio.h>
#include <unistd.h>

const char* ChipsErrMessages[ERR_MAX_NUM] = {
   "Request successful",
   "Request failed",
   "Specified Value not in a member of valid subset values",
   "The %s value (%s) must be one of [%s]",
   "The %s value (%1d) must be one of [%s]", 
   "The %s value (%s) must be between %s and %s",
   "The %s value (%1d) must be between %1d and %1d",
   "The %s value (%g) must be between %g and %g",
   "The %s value (%4.3f) must be between %4.3f and %4.3f", 
   "The %s value (%1d) must be greater than %1d",
   "The %s value (%g) must be greater than %g",
   "The %s value (%4.3f) must be greater than %4.3f",
   "The %s value (%1d) must be less than %1d",
   "The %s value (%g) must be less than %g",
   "The %s value (%4.3f) must be less than %4.3f",
   "The %s value (%1d) must be greater than or equal to %1d",
   "The %s value (%1d) must be less than or equal to %1d",
   "The %s value (%g) must be greater than or equal to %1g",
   "The %s value (%g) must be less than or equal to %g",
   "The %s value (%4.3f) must be greater than or equal to %4.3f",
   "The %s value (%4.3f) must be less than or equal to %4.3f",
   "The specified %s (%s) does not exist",
   "No %s id was provided",
   "The id %s is currently being used by another %s",  
   "There are currently no %s to operate on",
   "The requested operation failed- no %s contain the specified object",
   "Attempt to create %s failed",
   "ChipsId Object is not instantiated or is NULL",
   "An object id was not provided or is blank",
   "The use of 'all' is inappropriate for the given context",
   "No return pointer was provided",
   "The specified data value was outside of the valid input range",
   "The reorder list contains an incorrect number of entries (%1d vs %1d)",
   "The specified %s does not exist",
   "An internal memory allocation failed",
   "Unable to access the requested property (%d)",
   "The specified color is unknown or contains a non-hex digit",
   "The specified color is unknown or is not 6 hex digits in length",
   "Cannot take log negative number",
   "Unable to determine location of default resource file. Please specify an explicit path/filename",
   "'All %s' is inappropriate for the current context",
   "Cannot print multiple windows at once",
   "Axis is not supported",
   "The %s command only work in single frame mode (1). Current mode is %1d",
   "No previous frame to iterate to",
   "No next frame to iterate to",
   "Cannot change the visibility property of plot borders explicitly- try setting the plot style property (open, boxed, closed)",
   "Must explicitly specify the axis to move",
   "Data dimension size does not match coordinate systems",
   "The internal coordinate object does not exist- unable to perform requested operation",
   "The axis dimension must be one of [X_AXIS|Y_AXIS|XY_AXIS]",
   "A %s scale request was made for an unscalable axis", 
   "The %s value (%g) must be greater than the %s value (%g)",
   "No current %s- unable to create %s",
   "The selected plot already contains a legend", 
   "The server was unable to unpack the client request message", 
   "An invalid message was received", 
   "An unknown/unsupported request was made to the server", 
   "The server response timed out", 
   "The fork to create the server process failed", 
   "A filename was not supplied", 
   "The access permission for the file %s is incorrect", 
   "The output file %s already exists but export.clobber is set to false", 
   "Output file %s creation failed", 
   "The coordinate value is outside of the normalized range (0:1)", 
   "Requested operation pushes the %s out of the viewable area", 
   "The requested functionality is currently not implemented", 
   " Request not possible while the window is in 'blink' mode", 
   "Unable to open the specified input file",
   "Cannot display multiple 'current' frames in single mode",
   "The specified style (%1d) must be one of [chips_solid|chips_dot|chips_dotlongdash|chips_dotshortdash|chips_longdash|chips_shortdash|chips_shortdashlongdash]",
   "The specified symbol style was invalid or unsupported",
   "The specified number of tile rows (%1d) must be between %1d and %1d",
   "The specified number of tile columns (%1d) must be between %1d and %1d",
   "Parallel offset value must be between -0.5 and 0.5",
   "Perpendicular offset must be between -100 and 100",
   "Specified argument is not numeric",
   "The %s coordinate system does not exist",
   "The %s coordinate system, %s, does not exist",
   "Data coordinate system limits cannot be equal",
   "Pixel coordinate range cannot be zero",
   "Data coordinate system could not be created",
   "Unable to switch to arbitrary tick mode- positions not specified",
   "The axis label property, %s, cannot be set as the axis label does not exist",
   "Insufficient data to plot- one or more missing column(s)",
   "Insufficient data to plot- zero rows of data provided",
   "The specified x2 value must be greater than x1 to define a valid area",
   "The specified y2 value must be greater than y1 to define a valid area",
   "Invalid lines from the loaded resource file were ignored",
   "The plot currently contains no data- cannot use 'auto' limits",
   "The requested font (%s) has not been loaded, using default font (%s)",
   "Unable to load the font file %s", 
   "The %s margin (%g) plus the %s margin (%g) must be smaller than 1.0",
   "X and Y axis ids must be different", 
   "The string 'all' cannot be used as an object id",
   "Cannot delete boxed axes- set visiblity or change plot style", 
   "The specified axis is reserved for plot boxed/closed edges",
   "The page size is smaller than zero after margins applied.",
   "Unable to delete an axis which contains an associated %s",
   "The plot currently contains no %s axes- unable to apply limits",
   "The internal preferences object does not exist- unable to perform requested operation",
   "No current window to perform operation on",
   "No current frame to perform operation on",
   "No frames exist to determine row tiling",
   "No frames exist to determine column tiling",
   "Unable to load the specified preferences file", 
   "Data for contours must be regularly gridded", 
   "'All' is unsupported in get operations when more than one object exists",
   "Unknown property id (%1d) was encountered",
   "No values were supplied for row/column gridding", 
   "Array element %s[%1d] contains an invalid value (%1d)", 
   "Cannot adjust relative x size on split rows", 
   "Cannot adjust relative y size on split columns", 
   "The count (%1d) does not equal the number of elements expected (%1d)",
   "The specified xgap (%g) is too large for the margins and number of columns",
   "The specified ygap (%g) is too large for the margins and number of rows", 
   "Unable to swap the specified plot (%s) with itself",
   "The default depth value cannot be 'default' (0)",
   "The specified depth value (%1d) cannot be negative",
   "Unable to access the associated depth table",
   "The %s component file of the cmyksep output (%s) already exists", 
   "Cmyk colorscheme is only supported for ps/eps/pdf output files",
   "The specified export colorscheme does not exist or is invalid",
   "Supported error styles are: line, and capped",
   "Supported pick events: key_[press|release], btn[1|2|3|s]_[up|down]",
   "Supported line styles are: solid, dot, noline, [long|short]dash, dot[long|short]dash, and shortdashlongdash",
   "Supported symbols are: circle, cross, diamond, none, plus, square, point, and [up|down]triangle", 
   "Supported fonts are: helvetica, courier, times, greek, and user loaded fonts",
   "Supported plot styles are: open, boxed, and closed",
   "Invalid/unknown line extension",
   "Attempted to bind axis (%s->%s) to itself",
   "Supported export orientations are: portrait and landscape",
   "The pick event mask is not set or blank",
   "Supported modes are: nice, limits, interval, count, and arbitrary",
   "Supported units are: cm, mm, inches, and pixels",
   "The value (%d) must be non-negative",
   "The value (%f) must be non-negative",
   "Supported fontstyles are: normal, bold, italic, and bolditalic",
   "Supported objects are: window, frame, plot, curve, histogram, contour, axis, label, line, coordsys",
   "The value to assign to the property was not specified",
   "The requested object cannot be repositioned via the move command",
   "Cannot move %s using %s coordinate system values",
   "Cannot convert coordinates from different windows (%s and %s)",
   "Cannot close state when undo block is open",
   "Cannot mark an undo tag inside an open undo buffer block",
   "Command is locked",  // internal error should not reach user
   "There are no actions available to undo", 
   "There are no actions available to redo", 
   "The specified count must be at least one",
   "Unable to load state file (incompatible version or corrupted?)",
   "The file is not a state file or is an incompatible version",
   "Incomplete state file load (incompatible version or corrupted?)",
   "Unable to load incompatible state file (unsupported major version)",
   "Coordinate transform %s does not exist",
   "Coordinate transform %s already exists",
   "The specified opacity setting is invalid (0.0:1.0)", 
   "Region creation attempt failed- degenerate polygons (duplicate or intersecting edges) are not supported",
   "Supported corner styles are: miter, bevel, fill, and notch",
   "Supported bad data handling modes are: ignore and omit",
   "Contouring of data containing NaNs is currently not supported",
   "Supported contour algorithms are: standard and marching",
   "The specified undo/redo tag is already in use",
   "Commands for the specified redo tag have not been undone yet",
   "Commands for the specified undo tag have not been done yet",
   "The specified undo/redo tag does not exist",
   "No commands to tag in the undo/redo stack",
   "The client-server communication has failed. Check for connection",
   "Unable to connect to server", 
   "There is currently no server connected", 
   "A failure occurred while disconnecting from the server", 
   "Client is already connected to a server",
   "Specified object must be a curve, contour, histogram, or image for use as limits range",
   "No write permission for the directory (%s)",
   "Cannot apply transform. Number of rows and columns in the matrix do not\n  match the length of the point vector",
   "The specified transform is not a square matrix (%1d x %1d)",
   "The specified transform cannot be NULL",
   "No transform was found",
   "Tranform is not invertable",
   "Thickness values must be between 0.5 and 10.0",
   "Unknown or unsupported object attribute entry: %s",
   "There are less than %d actions to %s", 
   "The axes %s:%s and %s:%s are already bound to each other.",
   "The %s range is too large to support an interval mode of %4.3f",
   "The %s axis major_num attribute (%1d) is too large to support count mode",
   "The %s functionality has been deprecated.",
   "Supported tick styles are: inside, outside, and centered",
   "Supported tick label styles are: inside and outside",
   "Supported color schemes are: rgb, cmyk, grayscale, and cmyksep",
   "Supported page sizes are: letter, A3, A4, A5, legal, and executive",
   "Unknown error status.",
   "The specified server does not exists. Could not connect to server",
   "There are multiple servers available to connect to. Please specify a specific server. Use list_servers () to see a list of existing servers",
   "The preference array, %s, does not terminate with a bracket, ']'",
   "The preference array, %s, contains missing or invalid values. Make sure all array types are %s",
   "Supported fill styles are: nofill, solidfill, updiagonal, downdiagonal, horizontal, vertical, crisscross, brick, grid, hexagon, polkadot, flower, userfill1, userfill2, userfill3", 
   "Unable to access font table",
   "Unable to load the fill xbm file %s", 
   "Invalid slot specified for custom loaded fill file. Please use either chips_userfill1, chips_userfill2, or chips_userfill3.",
   "The specified fill file %s does not appear to be a supported format (basic xbm).",
   "Cannot set the axis, %s, current because plot borders cannot be current.",
   "The fill style value (%d) must be one of [chips_nofill|chips_solidfill|chips_updiagonal|chips_downdiagonal|chips_horizontal|chips_vertical|chips_crisscross|chips_brick|chips_grid|chips_hexagon|chips_polkadot|chips_zigzag|chips_wave|chips_flower|chips_userfill1|chips_userfill2|chips_userfill3]",
   "Plot already contains an image",
   "Cannot perform pick operation because window redraw is set to false.",
   "No current axes to operate on. Please specify an id or 'all' to set border axis properties.",
   "Can not set plot label for border axes. Try using the set_axis command with the border axis id and the 'x.label' or 'y.label' property",
   "The colormap style value (%d) must be one of [chips_red|chips_green|chips_blue|chips_grayscale|chips_rainbow|chips_hsv|chips_heat|chips_cool|chips_usercmap1|chips_usercmap2|chips_usercmap3]",
   "The image interpolation method (%d) must be one of [chips_neighbor|chips_bilinear|chips_bicubic]",
   "The specified minimum (%d) value exceeds the specified maximum (%d)",
   "The specified minimum (%g) value exceeds the specified maximum (%g)",
   "Unable to load the colormap file %s",
   "The specified colormap file, %s, does not appear to follow sao image standards.",
   "The specified colormap file, %s, has an invalid line, %s",
   "Invalid slot specified for custom loaded colormap file. Please use either chips_usercmap1, chips_usercmap2, or chips_usercmap3.",
   "Unable to use the limits command w/ XY_AXIS when data aspect ratio is set",
   "The image rgba value (%d) must be one of [chips_red|chips_green|chips_blue|chips_alpha]",
   "Cannot perform %s when the image is true color",
   "Cannot perform %s when the image is pseudo color",
   "Specified channel data size (%ld) does not match current image size (%ld- %d x %d)",
   "Channel %d has no image to remove",
   "Unable to set the aspect ratio- please verify that the data system exists",
   "Unable to bind axes since it would result in a cyclic dependency",
   "Unable to set the data aspect ratio since it would result in a cyclic dependency",
   "Binding of axes (%s, %s) across X and Y dimenions is not supported.",
   "Unable to set the aspect ratio- please verify that axes are in linear scale",
   "Log scale can not be applied to axes which have a data aspect ratio set",
   "Unable to create server- the requested server Id is already in use.",
   "The transform type provided (%s, %s) is unsupported - proceeding with logical coordinates."
};


const char* get_error_message(int errval)
{
   const char* msg = NULL;

   if ((errval < 0) || (errval >= ERR_MAX_NUM))
      errval = ERR_UNKNOWN_VALUE;
   msg = ChipsErrMessages[errval];
   return msg; 
}


bool err_type_object_DNE(int val)
{
   bool rv; 
   switch(val)
   {
      // all fallthroughs intentional 
      case ERR_OBJ_DNE: 
      case ERR_OBJ_LIST_EMPTY:
      case ERR_NO_CUR_WINDOW:
      case ERR_NO_CUR_FRAME:
      case ERR_OBJ_VAL_DNE:
      case ERR_NO_OBJ_ID:
      case ERR_AXIS_LABELS_DNE:
      case ERR_ALL_TASK_FAIL:
         rv = true; 
      break; 
      default:
         rv = false;
      break;
   }
   return rv; 
}


