// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSERRS_HH
#define CHIPSERRS_HH

// Error Status Values 
//
// The following macros define the stnadard error/return values
// of general functions

#define ERR_STATUS_GOOD          0  // command successful
#define ERR_STATUS_BAD           1  // generic failure value 
#define ERR_DATA_SUBSET          2  // value not in subset
#define ERR_STR_SUBSET           3  // format for string subset err  
#define ERR_INT_SUBSET           4  // format for string subset err  
#define ERR_STR_MINMAX           5  // format for string min/max err
#define ERR_INT_MINMAX           6  // format for int min/max err
#define ERR_DBL_MINMAX           7  // format for dbl min/max err
#define ERR_FLT_MINMAX           8  // format for float min/max err
#define ERR_INT_MIN              9  // format for int min range err 
#define ERR_DBL_MIN             10  // format for flt min range err 
#define ERR_FLT_MIN             11  // format for flt min range err 
#define ERR_INT_MAX             12  // format for int max range err 
#define ERR_DBL_MAX             13  // format for flt max range err 
#define ERR_FLT_MAX             14  // format for flt max range err 
#define ERR_INT_GTE             15  // format - integer >=
#define ERR_INT_LTE             16  // format - integer <=
#define ERR_DBL_GTE             17  // format - double  >=
#define ERR_DBL_LTE             18  // format - double  <=
#define ERR_FLT_GTE             19  // format - float   >=
#define ERR_FLT_LTE             20  // format - float   <=
#define ERR_OBJ_VAL_DNE         21  // format for specified a (b) dne 
#define ERR_NO_OBJ_ID           22  // format for no obj id provided
#define ERR_ID_IN_USE           23  // format id in use by obj 
#define ERR_OBJ_LIST_EMPTY      24  // format object list is empty
#define ERR_ALL_TASK_FAIL       25  // an operation on 'all' failed
#define ERR_OBJ_NOT_CREATED     26  // format object creation failed 
#define ERR_NO_ID_OBJ           27  // ChipsId object is NULL  
#define ERR_BLANK_ID            28  // object id was not provided/blank
#define ERR_INVALID_ALL_USE     29  // use of 'all' inappropriate  
#define ERR_RETURN_PTR_NULL     30  // no return object pointer provided
#define ERR_DATA_RANGE          31  // data is out of valid range
#define ERR_REORDER_LIST_LEN    32  // reorder list is wrong size
#define ERR_OBJ_DNE             33  // object does not exist 
#define ERR_ALLOC_FAIL          34  // memory allocation failure
#define ERR_UNKNOWN_PROPERTY    35  // access request for an unknown property
#define ERR_COLOR_NONHEX_DIGIT  36  // color contains a non-hex digit
#define ERR_UNKNOWN_COLOR       37  // color unknown or wrong length for hex
#define ERR_NEG_LOG_VAL         38  // Cannot take log negative number
#define ERR_RESFLE_PATH         39  // unable to find $HOME/.newchipsrc
#define ERR_OBJ_INVALID_ALL     40  // format- 'All' obj inappropriate  
#define ERR_PRINT_ALL           41  // Cannot print multiple windows at once
#define ERR_INVALID_AXIS        42  // Axis is not supported
#define ERR_FRAME_INVALID_MODE  43  // mode of frame is invalid for request
#define ERR_FRAME_NO_PREV       44  // previous frame missing/DNE 
#define ERR_FRAME_NO_NEXT       45  // next frame missing/DNE 
#define ERR_BOX_AXIS_VIS        46  // cannot change visibility of plot border
#define ERR_SPECIFY_AXIS        47  // must specify axis to move
#define ERR_BAD_COORD_DIM_SZ    48  // Pt dim size does not match coord sys
#define ERR_COORD_OBJ_DNE       49  // coordinate object DNE (internal err)
#define ERR_AXIS_INVALID_DIM    50  // specified an invalid axis dimension
#define ERR_AXIS_UNSCALABLE     51  // scale request on unscalable axis 
#define ERR_FLT_V1_GT_V2        52  // format- float v1 must be > v2 
#define ERR_NO_CURRENT_AXIS     53  // current axis not specified
#define ERR_PLOT_HAS_LEGEND     54  // add attempt to plot containing legend
#define ERR_SERVER_UNPACK       55  // server unable to unpack request
#define ERR_MESSAGE_INVALID     56  // invalid message  
#define ERR_MESSAGE_UNKNOWN     57  // unknown request made to server 
#define ERR_RESPONSE_TIMEOUT    58  // server response timed out 
#define ERR_FORK_FAIL           59  // fork to create server failed 
#define ERR_FILE_NOT_SUPPLIED   60  // no file was provided
#define ERR_ACCESS_PERMISSION   61  // file access permission is incorrect  
#define ERR_FILE_CLOBBER_OFF    62  // file exists and clobber is not set
#define ERR_FILE_CREATE_FAIL    63  // file creation failed 
#define ERR_NORM_DATA_RANGE     64  // out of normalized data range (0:1) 
#define ERR_EDGE_BOUNDARY       65  // op pushes object out of bounds
#define ERR_FEATURE_DNE         66  // requested function not implemented yet
#define ERR_BLINK_ACTIVE        67  // request not possible while blink on
#define ERR_FILE_OPEN           68  // unable to open file
#define ERR_MULT_SINGLE_FRM     69  // can't use single mode w/ 'all' frames
#define ERR_LINETYPE_DNE        70  // unsupported line style 
#define ERR_SYMBOLTYPE_DNE      71  // unsupported symbol style 
#define ERR_ROW_TILE_BOUND      72  // number of rows to tile too large
#define ERR_COL_TILE_BOUND      73  // number of cols to tile too large
#define ERR_PAR_OFFSET_RANGE    74  // out of range value for parallel offset
#define ERR_PERP_OFFSET_RANGE   75  // invalid value for perpendicular offset
#define ERR_INVALID_NUMERIC_ARG 76  // Specified argument is not numeric
#define ERR_COORD_DNE           77  // Coordinate system does not exist
#define ERR_COORD_DS_DNE        78  // Data coord system does not exist.
#define ERR_COORD_DATA_LIM_EQ   79  // Data coord system limits can not be equal
#define ERR_COORD_RANGE_ZERO    80  // Pixel coordinate range can not be zero
#define ERR_COORD_DS_NO_ADD     81  // Data coord system could not be created
#define ERR_ARBTICKPOS_DNE      82
#define ERR_AXIS_LABELS_DNE     83  // axis labels do not exist
#define ERR_CURVE_COL_DNE       84  // missing a required column (x or y)  
#define ERR_CURVE_DATA_DNE      85  // curve row cnt is 0 
#define ERR_NORM_XMAX_LE_XMIN   86  // x2 <= x1  (max <= min)
#define ERR_NORM_YMAX_LE_YMIN   87  // y2 <= y1  (max <= min)
#define ERR_IGNORE_RC_LINE      88  // ignoring line from resource file
#define ERR_NO_DATA_FOR_AUTO    89  // auto limits on plot w/o data
#define ERR_FONT_DNE            90  // warning- switch to default font
#define ERR_FONT_FILE_LOAD      91  // unable to load font file
#define ERR_MARGIN_SIZE         92  // opposite margins sum larger than 1
#define ERR_AXIS_XY_ID          93  // attempt to use same and y axis id 
#define ERR_USED_ALL_AS_ID      94  // attempt to use 'all' as an object id 
#define ERR_BOX_AXIS_DEL        95  // invalid delete attempt of boxed axes 
#define ERR_BOX_AXIS_ID         96  // invalid use of box ids for axis ids
#define ERR_MARGIN_VS_PAGE      97  // Page size smaller than zero /w margins
#define ERR_AXIS_OBJ_EXISTS     98  // can't delete axis w/objects
#define ERR_LIMITS_NO_AXES      99  // can't use limits w/o axes
#define ERR_NO_PREFS_OBJ       100  // preferences object DNE (internal err)
#define ERR_NO_CUR_WINDOW      101  // no current window 
#define ERR_NO_CUR_FRAME       102  // no current frame 
#define ERR_NO_ROW_TILES       103  // no frames exist to row tile 
#define ERR_NO_COL_TILES       104  // no frames exist to column tile 
#define ERR_PREF_LOAD          105  // unable to load prefs file
#define ERR_CNTR_REG_GRID      106  // contours must be regularly gridded
#define ERR_INVALID_GET_ALL    107  // all not supported for gets
#define ERR_UNKNOWN_PROP_ID    108  // prop id is unknown- internal err
#define ERR_NO_VALS_SUPPLIED   109  // array data not provided
#define ERR_ARRAY_VALUES       110  // values in array are invalid 
#define ERR_SPLITROW_X         111  // can't adjust xrel of split rows
#define ERR_SPLITCOL_Y         112  // can't adjust yrel of split cols
#define ERR_EXPECTED_CNT       113  // cnt does not match expected num entries
#define ERR_XGAP_VS_COLS       114  // gap too big for number of cols 
#define ERR_YGAP_VS_ROWS       115  // gap too big for number of rows
#define ERR_SWAP_SELF          116  // attempt to swap object with self 
#define ERR_DEF_DEPTH          117  // default depth != default
#define ERR_NEG_DEPTH          118  // depth must be >= 0 
#define ERR_DEPTH_TABLE_DNE    119  // depth table doesn't exist
#define ERR_CMYK_EXISTS        120  // format cmyk file already exists
#define ERR_CMYK_NONPS         121  // cymk only supported for ps/eps
#define ERR_COLORSCHEME_DNE    122  // unknown or unspecified colorscheme
#define ERR_BAD_ERRSTYLE       123  // invalid/unknown error style 
#define ERR_BAD_PICKMASK       124  // invalid/unknown pick mask component
#define ERR_BAD_LINESTYLE      125  // invalid/unknown line style 
#define ERR_BAD_SYMBOL         126  // invalid/unknown symbol 
#define ERR_BAD_FONT           127  // invalid/unknown font
#define ERR_BAD_PLOTSTYLE      128  // invalid plot/axis style
#define ERR_BAD_LINEEXTENSION  129  // invalid/unknown line extension
#define ERR_SELF_BIND          130  // Cannot bind axis to itself
#define ERR_EXPORT_ORIENT      131  // unknown/unsupported orientation
#define ERR_NO_PICK_MASK       132  // no pick mask specified 
#define ERR_BAD_TICKMODE       133  // invalid/unknown tickmode 
#define ERR_BAD_UNITS          134  // invalid/unknown units
#define ERR_NO_INT_NEGATIVE    135  // no negative value allowed
#define ERR_NO_FLT_NEGATIVE    136  // no negative value allowed
#define ERR_BAD_FONTSTYLE      137  // invalid/unknown fontstyle 
#define ERR_BAD_OBJECT         138  // invalid/unknown object type
#define ERR_NO_PROP_VALUE      139  // no value specified for property
#define ERR_CANT_MOVE          140  // object does not have move ability
#define ERR_INV_MOVE           141  // format- invalid obj move in coord sys
#define ERR_COORD_MULT_WIN     142  // Can't convert coord from multiple windows
#define ERR_UNDO_BLOCK_OPN     143  // Can't close state w/ open undo block 
#define ERR_UNDO_BUFFER_TAG    144  // Can't mark undo tag in open buffer block
#define ERR_CMD_LOCKED         145  // command is locked
#define ERR_NO_UNDO            146  // no items to undo
#define ERR_NO_REDO            147  // no items to redo
#define ERR_ZERO_CNT           148  // need to specify value of at least 1
#define ERR_STATEFILE_COR      149  // statefile corrupted
#define ERR_NOT_STATEFILE      150  // supplied file is not a state file
#define ERR_STATEFILE_INC      151  // incomplete state file load
#define ERR_STATEFILE_MAJ      152  // incompatible state file version number
#define ERR_COORD_TRANS_DNE    153  // transform does not exsist
#define ERR_COORD_TRANS_EXISTS 154  // transform already exists
#define ERR_REGION_OPACITY     155  // invalid opacity value 
#define ERR_REGION_CREATE_FAIL 156  // region creation failed 
#define ERR_BAD_CORNERSTYLE    157  // invalid corner style
#define ERR_BAD_BADDATASTYLE   158  // invalid baddata mode 
#define ERR_CONTOUR_NAN        159  // contour contains a nan
#define ERR_BAD_CONTOUR_ALG    160  // invalid/unknown contour algorithm
#define ERR_TAG_USED           161  // undo/redo tag already in use
#define ERR_BAD_TAG_REDO       162  // redo tag comands not undone yet
#define ERR_BAD_TAG_UNDO       163  // undo tag comands not done yet
#define ERR_TAG_DNE            164  // specified tag does not exist
#define ERR_TAG_NOTHING        165  // nothing to tag
#define ERR_NO_SOCKET          166  // socket connection failure
#define ERR_SOCK_UNAVAIL       167  // unable to connect to server 
#define EER_SOCK_NOT_CONN      168  // no server connected 
#define ERR_SOCK_CLOSE         169  // error disconnection from server
#define ERR_SOCK_CONN          170  // already connected to socket
#define ERR_LIMIT_OBJ          171  // invalid obj to use for limits spec
#define ERR_DIR_WRITE_PERM     172  // no write permission for directory
#define ERR_BAD_TRANSFORM_SIZE 173  // transform size does not match vector size
#define ERR_TRANS_NOT_SQUARE   174  // transform is not a square matrix
#define ERR_TRANSFORM_NULL     175  // transform is NULL
#define ERR_NO_TRANFORM_FOUND  176  // transform not found
#define ERR_TRANS_NOT_INVERT   177  // transform is not invertable
#define ERR_THICKNESS_VAL      178  // bad thickness value specified
#define ERR_ATTRIBUTE_DNE      179  // format- unknown attribute/property 
#define ERR_UNDO_COUNT         180  // foramt- unable to undo count actions
#define ERR_DUP_BINDING        181  // axes already bound to each other
#define ERR_INT_MODE_RANGE     182  // axis range too large for interval mode
#define ERR_COUNT_MODE_RANGE   183  // major_num too large for count mode
#define ERR_DEPRECATED         184  // %s feature deprecated

#define ERR_BAD_TICKSTYLE      185  // invalid tick style-inside,outside,center
#define ERR_BAD_TICKLABSTYLE   186  // invalid tick label-inside, outside 
#define ERR_BAD_COLORSCHEME    187  // invalid color scheme-cmyk,rgb,... 
#define ERR_BAD_PAGESIZE       188  // invalid page size-letter, a4, ...
#define ERR_UNKNOWN_VALUE      189  // unknown error message 
#define ERR_SOCK_SERVERID      190  // specified server does not exist ...
#define ERR_MULT_SERVERID      191  // there are multiple servers available ...
#define ERR_INVALID_ARRAY      192  // the preference array %s does not termin...
#define ERR_INVALID_ARRAY_ELEM 193  // the preference array %s contains missing or ...
#define ERR_BAD_FILLSTYLE      194  // invalid fill pattern style string 
#define ERR_FILL_TABLE_ACCESS  195  // internal error- fill table DNE
#define ERR_FILL_LOAD          196  // unable to load fill file 
#define ERR_FILL_SLOT_INV      197  // invalid slot specified for fill file 
#define ERR_FILL_FORMAT        198  // fill file not xbm 
#define ERR_PLOT_BORDER_CURR   199  // Cannot set the axis, %s, current...
#define ERR_BAD_FILLSTYLE2     200  // invalid fill pattern style enum 
#define ERR_PLOT_HAS_IMAGE     201  // plot already contains image 
#define ERR_PICK_DISPLAY       202  // pick request w/ display off 
#define ERR_BORDER_AXIS_PROP   203  // set axis for border axis w/o using id 
#define ERR_BORDER_AXIS_LBL    204  // used set_plot_xlabel for border axis
#define ERR_BAD_CMAP           205  // invalid colormap style string 
#define ERR_BAD_IMG_INTERP     206  // invalid image interpolation string
#define ERR_IMIN_GT_IMAX       207  // interger min > max 
#define ERR_DMIN_GT_DMAX       208  // double min > max 
#define ERR_CMAP_LOAD          209  // unable to load colormap file
#define ERR_CMAP_DS9           210  // invalid line in sao image colormap file
#define ERR_CMAP_TABLE         211  // invalid line in colormap table file
#define ERR_CMAP_SLOT_INV      212  // invalid slot specified for fill file
#define ERR_XY_LIM_RATIO       213  // can't use limits(XY) with ratios
#define ERR_BAD_RGBA           214  // invalid rbga value
#define ERR_IMAGE_TC_MODE      215  // cannot perform %s when image is true color
#define ERR_IMAGE_PC_MODE      216  // cannot perform %s when image is psuedo color
#define ERR_BAD_CHANNEL_DIM    217  // cannot add an image channel with ...
#define ERR_DEL_NULL_CHANNEL   218  // channel %d has no image to remove
#define ERR_DASP_NO_DS         219  // can't set data aspect ratio w/o axes 
#define ERR_CYC_DEP_BIND       220  // can't add bind- cyclic dependency 
#define ERR_CYC_DEP_ASP        221  // can't add asp ratio- cyclic dependency 
#define ERR_CROSSDIM_BIND      222  // don't support bind of axes in different dim
#define ERR_DASP_LOG           223  // can't set data aspect ratio on logscale axis
#define ERR_LOG_DASP           224  // can't use log scale on axes w/ data asp ratio
#define ERR_DUP_SRVRID         225  // duplicate chips server id requested
#define ERR_TRANS_NOT_SUPP     226  // transform type is unsupported
#define ERR_MAX_NUM            227  

extern const char* get_error_message(int errval);

extern bool err_type_object_DNE(int val);

#endif 
