// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <string.h>

#include "ChipsFillCommand.hh"
#include "ChipsFillPattern.hh"
#include "ChipsStateStream.hh"

extern ChipsFillPattern* fill_patterns; 


ChipsFillCommand::ChipsFillCommand(int slotval, unsigned char* oldval, unsigned char* newval)
{
   weight=1;
   co = NULL;
   slot = slotval; 
   if (oldval)  memcpy(oval, oldval, 128); 
   if (newval)  memcpy(nval, newval, 128); 
}

ChipsFillCommand::~ChipsFillCommand()
{
}


ChipsFillCommand::ChipsFillCommand(ChipsStateStream& in)
{
   deserialize(in);
   weight=1;
}

int ChipsFillCommand::execute ()
{
   return fill_patterns->set_fill(nval, slot);
}

int ChipsFillCommand::unexecute ()
{
   return fill_patterns->set_fill(oval, slot);
}

void ChipsFillCommand::serialize(ChipsStateStream& out)
{
   int val = 22;
   int type = (int)chips_not_object;
   out.write_int(&val);
   out.write_int(&type);
   out.write_int(&ref);
   out.write_int(&block_ref);

   out.write_int(&slot); 
   out.write_char((char*)&oval[0], 128);
   out.write_char((char*)&nval[0], 128);
}

void ChipsFillCommand::deserialize(ChipsStateStream& in)
{
   in.read_int(&ref);
   in.read_int(&block_ref);

   in.read_int(&slot);
   in.read_char((char*)&oval[0], 128);
   in.read_char((char*)&nval[0], 128);
}

void ChipsFillCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Set Fill " << endl;
};



