// 
//  Copyright (C) 2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsFontList.hh"
#include "ChipsCmdManager.hh"

#include <stdio.h>

extern ChipsCmdManager* cmd_manager;


ChipsFontList::ChipsFontList()
{
   FONTLIST_ELEM_P_T  fp = new FONTLIST_ELEM_T;

   if (fp)
   {
      fp->fontname = "helvetica";
      fp->path_n   = "DEFAULT_HELVETICA";
      fp->path_b   = "DEFAULT_HELVETICA_BOLD";
      fp->path_i   = "DEFAULT_HELVETICA_ITALIC";
      fp->path_bi  = "DEFAULT_HELVETICA_BOLDITALIC";
      fonts.push_back(fp);
   }
   if ((fp = new FONTLIST_ELEM_T) != NULL)
   {
      fp->fontname = "courier";
      fp->path_n   = "DEFAULT_COURIER";
      fp->path_b   = "DEFAULT_COURIER_BOLD";
      fp->path_i   = "DEFAULT_COURIER_ITALIC";
      fp->path_bi  = "DEFAULT_COURIER_BOLDITALIC";
      fonts.push_back(fp);
   }
   if ((fp = new FONTLIST_ELEM_T) != NULL)
   {
      fp->fontname = "times";
      fp->path_n   = "DEFAULT_TIMES";
      fp->path_b   = "DEFAULT_TIMES_BOLD";
      fp->path_i   = "DEFAULT_TIMES_ITALIC";
      fp->path_bi  = "DEFAULT_TIMES_BOLDITALIC";
      fonts.push_back(fp);
   }
   if ((fp = new FONTLIST_ELEM_T) != NULL)
   {
      fp->fontname = "greek";
      fp->path_n   = "DEFAULT_GREEK";
      fp->path_b   = "DEFAULT_GREEK_BOLD";
      fp->path_i   = "DEFAULT_GREEK_ITALIC";
      fp->path_bi  = "DEFAULT_GREEK_BOLDITALIC";
      fonts.push_back(fp);
   }
}


ChipsFontList::~ChipsFontList()
{
   list<FONTLIST_ELEM_P_T>::iterator ii;

   for(ii=fonts.begin(); ii != fonts.end(); ++ii)
      delete(*ii);
   fonts.clear();
}


int ChipsFontList::load_font(string fname, string fpath, 
   V_FONTSTYLE_TYPE fstyle)
{
   int rval = ERR_STATUS_GOOD;
   FONTLIST_ELEM_P_T  fp = NULL;

   if (fname.length() && fpath.length() && valid_fontstyle_type(fstyle)) 
   {
      if ((chips_strcmp(fname.c_str(), "helvetica") == 0) ||
          (chips_strcmp(fname.c_str(), "courier") == 0) ||
          (chips_strcmp(fname.c_str(), "greek") == 0) ||
          (chips_strcmp(fname.c_str(), "times") == 0))
      {
         string estr = "The specified font name ("; 
         estr.append(fname);
         estr.append(") can not be one of the predefined fonts [helvetica, courier, greek, or times]"); 
         cmd_manager->add_err(estr.c_str(), rval=ERR_STATUS_BAD);
      } 
      else if ((fp = get_font(fname)) != NULL)
      {
         switch(fstyle)
         {
            case chips_bold:
               fp->path_b = fpath;
            break;
            case chips_italic:
               fp->path_i = fpath;
            break;
            case chips_bolditalic:
               fp->path_bi = fpath;
            break;
            default:
               fp->path_n = fpath;
            break; 
         }
      }
      else 
      {
         if ((fp = new FONTLIST_ELEM_T) != NULL)
         {
            fp->fontname = fname;

            switch(fstyle)
            {
               case chips_bold:
                  fp->path_b = fpath;
               break;
               case chips_italic:
                  fp->path_i = fpath;
               break;
               case chips_bolditalic:
                  fp->path_bi = fpath;
               break;
               default:
                  fp->path_n = fpath;
               break; 
            }
            fonts.push_back(fp);
         }
      }
   }
   else 
   {
      cmd_manager->add_err("A font name to access the specified font was not provided" , rval=ERR_STATUS_BAD);
   }
   return rval; 
}

string ChipsFontList::get_path(string& fname, V_FONTSTYLE_TYPE styl)
{
   string path = ""; 
   FONTLIST_ELEM_P_T fp = NULL;

   if (fname.length())
   {
      list<FONTLIST_ELEM_P_T>::iterator ii;

      for(ii=fonts.begin(); (!fp) && (ii != fonts.end()); ++ii)
      {
         if (chips_strcmp((*ii)->fontname.c_str(), fname.c_str()) == 0)
         {
            fp = *ii; 
            switch(styl)
            {
               case chips_bold:
                  path = fp->path_b;
                  // if we don't have a bold font, use the normal font
                  if (!path.length())   path = fp->path_n; 
               break;
               case chips_italic:
                  path = fp->path_i;
                  // if we don't have a italic font, use the normal font
                  if (!path.length())   path = fp->path_n; 
               break;
               case chips_bolditalic:
                  path = fp->path_bi;
                  // if we don't have a bolditalic font, try using italic,
                  // then bold, then the normal font
                  if (!path.length()) 
                  {  
                     path = fp->path_i; 
                     if (!path.length())   
                     {
                        path = fp->path_b; 
                        if (!path.length())   path = fp->path_n; 
                     }
                  }
               break;
               default:
                  path = fp->path_n;
               break;
            }
         }
      }
   }
   return path; 
}

string ChipsFontList::get_normal_path(string& fname)
{
   string path = "";
   FONTLIST_ELEM_P_T fp = NULL;

   if (fname.length())
   {
      list<FONTLIST_ELEM_P_T>::iterator ii;

      for(ii=fonts.begin(); (!fp) && (ii != fonts.end()); ++ii)
      {
         if (chips_strcmp((*ii)->fontname.c_str(), fname.c_str()) == 0)
         {
            fp = *ii; 
            path = fp->path_n;
         }
      }
   }
   return path;
}

string ChipsFontList::get_bold_path(string& fname)
{
   string path = "";
   FONTLIST_ELEM_P_T fp = NULL;

   if (fname.length())
   {
      list<FONTLIST_ELEM_P_T>::iterator ii;

      for(ii=fonts.begin(); (!fp) && (ii != fonts.end()); ++ii)
      {
         if (chips_strcmp((*ii)->fontname.c_str(), fname.c_str()) == 0)
         {
            fp = *ii; 
            path = fp->path_b;
         }
      }
   }
   return path;
}

string ChipsFontList::get_italic_path(string& fname)
{
   string path = "";
   FONTLIST_ELEM_P_T fp = NULL;

   if (fname.length())
   {
      list<FONTLIST_ELEM_P_T>::iterator ii;

      for(ii=fonts.begin(); (!fp) && (ii != fonts.end()); ++ii)
      {
         if (chips_strcmp((*ii)->fontname.c_str(), fname.c_str()) == 0)
         {
            fp = *ii; 
            path = fp->path_i;
         }
      }
   }
   return path;
}


string ChipsFontList::get_bolditalic_path(string& fname)
{
   string path = "";
   FONTLIST_ELEM_P_T fp = NULL;

   if (fname.length())
   {
      list<FONTLIST_ELEM_P_T>::iterator ii;

      for(ii=fonts.begin(); (!fp) && (ii != fonts.end()); ++ii)
      {
         if (chips_strcmp((*ii)->fontname.c_str(), fname.c_str()) == 0)
         {
            fp = *ii; 
            path = fp->path_bi;
         }
      }
   }
   return path;
}


bool ChipsFontList::font_exists(string& fname)
{
   bool rval = false;

   if (fname.length())
   {
      list<FONTLIST_ELEM_P_T>::iterator ii;

      for(ii=fonts.begin(); ((ii != fonts.end()) && !rval); ++ii)
      {
         rval = (chips_strcmp( (*ii)->fontname.c_str(), fname.c_str()) == 0);
      }
   }
   return rval; 
}


FONTLIST_ELEM_P_T ChipsFontList::get_font(string& fname)
{
   FONTLIST_ELEM_P_T fp = NULL;

   if (fname.length())
   {
      list<FONTLIST_ELEM_P_T>::iterator ii;

      for(ii=fonts.begin(); (!fp) && (ii != fonts.end()); ++ii)
      {
         if (chips_strcmp((*ii)->fontname.c_str(), fname.c_str()) == 0)
         {
            fp = *ii;
         }
      }
   }
   return fp;
}

void ChipsFontList::get_font_names (list<string> &out_fnames)
{
   list<FONTLIST_ELEM_P_T>::iterator ii;

   for(ii=fonts.begin(); ii != fonts.end(); ++ii)
      out_fnames.push_back ((*ii)->fontname);
}

