// 
//  Copyright (C) 2005-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSHISTOGRAM_HH
#define CHIPSHISTOGRAM_HH

#include <string>
#include <stdio.h>
#include "ChipsObject.hh"
#include "ChipsCurve.hh"
#include "ChipsDefaults.hh"
#include "ChipsAxis.hh"
#include "ChipsStateStream.hh"

using namespace std;

class ChipsHistogram: public ChipsCurve
{
  public:

   ChipsHistogram(ChipsObject* par, unsigned long cnt, double* bin_lo, 
                  double* bin_hi, double* bincnts, string hid);
   ChipsHistogram(ChipsObject* par, unsigned long cnt, double* bin_lo, 
                  double* bin_hi, double* bincnts, double* ydown, double* yup, 
                  string hid);
   ChipsHistogram(ChipsObject* par, ChipsStateStream& in);

   void object_delete ();

   virtual void initialize(); 

   unsigned long get_cnt()               { return num_pts; }

   int        set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj);
   int        set_property(void* in_val, int property, bool rec=true);

   int        get_counts_data(double* dat, unsigned long* cnt);
   int        get_binlo_data(double* dat, unsigned long* cnt);
   int        get_binhi_data(double* dat, unsigned long* cnt);
   int        set_dropline(bool val); 
   bool       get_dropline()   { return drop_line; } 

   int        set_fillcolor (string in_color);
   string     get_fillcolor ()     {return fillcolor.get_name ();};
   int        set_fillopacity (double in_val);
   double     get_fillopacity ()     {return fillopacity;};
   V_FILL_TYPE get_fillstyle()        { return fillstyle; }
   int        set_fillstyle(V_FILL_TYPE val, bool record=false);
   int        set_fillstyle(unsigned char* bpattern);

   void       set_histogram_num (int val) {histogram_num = val;};
   int        get_histogram_num () {return histogram_num;};

   virtual int show ();
   virtual int hide ();

   virtual int reset_position ();
   virtual int  set_depth(int newdepth);

   virtual int change_linethickness(); 
   virtual int change_linestyle(); 
   virtual int change_linecolor(); 
   virtual int change_symbolsize(); 
   virtual int change_symbolstyle(); 
   virtual int change_symbolcolor(); 
   virtual int change_symbolfilled(); 
   virtual int change_dropline(); 
   virtual int change_fillcolor ();
   virtual int change_fillopacity ();
   virtual int change_fillstyle(); 
   virtual int change_visible();
   virtual int change_depth();
   virtual int set_plot_location (double x1, double y1, double x2, double y2);
   virtual int change_range (const ChipsVector &low_range, const ChipsVector&high_range);
   virtual int change_scale (int inXScale, int inYScale);


   virtual void serialize(ChipsStateStream& out);
   virtual void serialize_ascii(ChipsStateStream& out);
   virtual void deserialize(ChipsStateStream& in);

   
  protected:

   virtual ~ChipsHistogram();

   void        calculate_limits();


   double*     binlo;        // low bin values  
   double*     binhi;        // high bin values  

   bool        force_drop;   // do we need to draw drop lines?
   bool        drop_line;    // should line extend to bottom of plot 
   bool        lo_supplied;  // low bin info supplied
   bool        hi_supplied;  // high bin info supplied

   int        histogram_num;       // the histogram's order 0th, 1st, 2nd, etc.

   ChipsColor fillcolor;     // color of the fill
   double     fillopacity;   // opacity of the fill
   V_FILL_TYPE fillstyle;    // style of fill in histogram 

   unsigned char fillpattern[128]; // fill pattern

  private:
   bool add_command (ChipsCommand *inCmd, int &outStatus);
};

#endif 
