// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsId.hh"
#include "ChipsStateStream.hh"

#include <stdio.h>
#include <sstream>

ChipsId::ChipsId()
{
   wid = "";
   fid = "";
   pid = "";
   gid = ""; 
   iid = "";
   aid = "";
   xaid = "";
   yaid = "";
   cbid = "";
   lbid = "";
   lnid = "";
   ptid = "";
   ctid = ""; 
   cvid = ""; 
   hid  = ""; 
   lgid = ""; 
   rgid = ""; 
   coordid = ""; 
   coordsys = COORD_NONE;
   ref_wid = 0;
   ref_fid = 0;
   ref_pid = 0;
   ref_iid = 0;
   ref_aid = 0;
   ref_xaid = 0;
   ref_yaid = 0;
   ref_cbid = 0;
   ref_lbid = 0;
   ref_lnid = 0;
   ref_ptid = 0;
   ref_ctid = 0;
   ref_cvid = 0;
   ref_hid  = 0;
   ref_lgid = 0;
   ref_rgid = 0;

   axisdim = XY_AXIS; 
}


ChipsId::ChipsId(const char* ww, const char* ff, const char* pp)
{
   if (ww)    wid = ww;   // window id passed in, so use it 
      else    wid = "";
   if (ff)    fid = ff;   // frame id passed in, so use it 
      else    fid = "";   
   if (pp)    pid = pp;   // plot id passed in, so use it 
      else    pid = "";   
   gid = "";
   iid = "";
   aid = "";
   xaid = "";
   yaid = "";
   cbid = "";
   lbid = "";
   lnid = "";
   ptid = "";
   ctid = "";
   cvid = "";
   hid  = ""; 
   lgid = "";
   rgid = "";
   coordid = ""; 
   coordsys = COORD_NONE;
   axisdim = XY_AXIS; 

   ref_wid = 0;
   ref_fid = 0;
   ref_pid = 0;
   ref_iid = 0;
   ref_aid = 0;
   ref_xaid = 0;
   ref_yaid = 0;
   ref_cbid = 0;
   ref_lbid = 0;
   ref_lnid = 0;
   ref_ptid = 0;
   ref_ctid = 0;
   ref_cvid = 0;
   ref_hid  = 0;
   ref_lgid = 0;
   ref_rgid = 0;

}


ChipsId::ChipsId(const ChipsId& orig)
{
   if (&orig)
   {
      wid = orig.wid;
      fid = orig.fid;
      pid = orig.pid;
      gid = orig.gid; 
      iid = orig.iid;
      aid = orig.aid;
      xaid = orig.xaid;
      yaid = orig.yaid;
      cbid = orig.cbid;
      lbid = orig.lbid;
      lnid = orig.lnid;
      ptid = orig.ptid;
      ctid = orig.ctid;
      cvid = orig.cvid;
      hid  = orig.hid; 
      lgid = orig.lgid;
      rgid = orig.rgid; 

      ref_wid = orig.ref_wid;
      ref_fid = orig.ref_fid;
      ref_pid = orig.ref_pid;
      ref_iid = orig.ref_iid;
      ref_aid = orig.ref_aid;
      ref_xaid = orig.ref_xaid;
      ref_yaid = orig.ref_yaid;
      ref_cbid = orig.ref_cbid;
      ref_lbid = orig.ref_lbid;
      ref_lnid = orig.ref_lnid;
      ref_ptid = orig.ref_ptid;
      ref_ctid = orig.ref_ctid;
      ref_cvid = orig.ref_cvid;
      ref_hid  = orig.ref_hid;
      ref_lgid = orig.ref_lgid;
      ref_rgid = orig.ref_rgid;

      coordid =  orig.coordid; 
      coordsys = orig.coordsys;
      axisdim = orig.axisdim; 
   }
}


ChipsId::ChipsId (ChipsStateStream &in)
{
   deserialize (in);
}


ChipsId::~ChipsId()
{

}

void ChipsId::operator = (const ChipsId &orig)
{
   if (&orig != this)
   {
      wid = orig.wid;
      fid = orig.fid;
      pid = orig.pid;
      gid = orig.gid;
      iid = orig.iid;
      aid = orig.aid;
      xaid = orig.xaid;
      yaid = orig.yaid;
      cbid = orig.cbid;
      lbid = orig.lbid;
      lnid = orig.lnid;
      ptid = orig.ptid;
      ctid = orig.ctid;
      cvid = orig.cvid;
      hid  = orig.hid;
      lgid = orig.lgid;
      rgid = orig.rgid;

      ref_wid = orig.ref_wid;
      ref_fid = orig.ref_fid;
      ref_pid = orig.ref_pid;
      ref_iid = orig.ref_iid;
      ref_aid = orig.ref_aid;
      ref_xaid = orig.ref_xaid;
      ref_yaid = orig.ref_yaid;
      ref_cbid = orig.ref_cbid;
      ref_lbid = orig.ref_lbid;
      ref_lnid = orig.ref_lnid;
      ref_ptid = orig.ref_ptid;
      ref_ctid = orig.ref_ctid;
      ref_cvid = orig.ref_cvid;
      ref_hid  = orig.ref_hid;
      ref_lgid = orig.ref_lgid;
      ref_rgid = orig.ref_rgid;

      coordid =  orig.coordid;
      coordsys = orig.coordsys;
      axisdim = orig.axisdim;
   }
}



void ChipsId::reset_window_id(string val)
{
   wid = val;
   fid = ""; 
   pid = "";
   iid = "";
   cbid = "";  
   lbid = "";  
   lnid = "";
   ptid = ""; 
   aid = ""; 
   xaid = ""; 
   yaid = ""; 
   ctid = ""; 
   cvid = ""; 
   hid  = ""; 
   gid = ""; 
   lgid = ""; 
   rgid = ""; 
   ref_wid = 0;
   ref_fid = 0;
   ref_pid = 0;
   ref_iid = 0;
   ref_aid = 0;
   ref_xaid = 0;
   ref_yaid = 0;
   ref_cbid = 0;
   ref_lbid = 0;
   ref_lnid = 0;
   ref_ptid = 0;
   ref_ctid = 0;
   ref_cvid = 0;
   ref_hid  = 0;
   ref_lgid = 0;
   ref_rgid = 0;

   coordid = ""; 
   coordsys = COORD_NONE;
   axisdim = XY_AXIS; 
}


void ChipsId::reset_frame_id(string val)
{
   fid = val;
   pid = "";
   iid = "";
   cbid = "";  
   lbid = "";  
   lnid = "";
   ptid = ""; 
   gid = ""; 
   aid = ""; 
   xaid = ""; 
   yaid = ""; 
   ctid = ""; 
   cvid = ""; 
   hid  = ""; 
   lgid = ""; 
   rgid = ""; 
   coordid = ""; 
   coordsys = COORD_NONE;
   axisdim = XY_AXIS; 

   ref_fid = 0;
   ref_pid = 0;
   ref_iid = 0;
   ref_aid = 0;
   ref_xaid = 0;
   ref_yaid = 0;
   ref_cbid = 0;
   ref_lbid = 0;
   ref_lnid = 0;
   ref_ptid = 0;
   ref_ctid = 0;
   ref_cvid = 0;
   ref_hid  = 0;
   ref_lgid = 0;
   ref_rgid = 0;

}


void ChipsId::reset_plot_id(string val)
{
   pid = val;
   gid = ""; 
   aid = ""; 
   xaid = ""; 
   yaid = ""; 
   ctid = ""; 
   cvid = ""; 
   hid  = ""; 
   iid = "";
   cbid = "";  
   lbid = "";  
   lnid = "";
   ptid = ""; 
   lgid = ""; 
   rgid = ""; 
   coordid = ""; 
   coordsys = COORD_NONE;
   axisdim = XY_AXIS; 

   ref_pid = 0;
   ref_iid = 0;
   ref_aid = 0;
   ref_xaid = 0;
   ref_yaid = 0;
   ref_cbid = 0;
   ref_lbid = 0;
   ref_lnid = 0;
   ref_ptid = 0;
   ref_ctid = 0;
   ref_cvid = 0;
   ref_hid  = 0;
   ref_lgid = 0;
   ref_rgid = 0;

}

void ChipsId::set_all()
{
   wid = "all";
   fid = "all";
   pid = "all";
   gid = "all";
   iid = "all";
   aid = "all";
   xaid = "all";
   yaid = "all";
   cbid = "all";
   lbid = "all";
   lnid = "all";
   ptid = "all";
   ctid = "all";
   cvid = "all";
   hid  = "all";
   lgid = "all";
   rgid = "all";
   coordid = "";
   coordsys = COORD_NONE;
   axisdim = XY_AXIS; 

   ref_wid = 0;
   ref_fid = 0;
   ref_pid = 0;
   ref_iid = 0;
   ref_aid = 0;
   ref_xaid = 0;
   ref_yaid = 0;
   ref_cbid = 0;
   ref_lbid = 0;
   ref_lnid = 0;
   ref_ptid = 0;
   ref_ctid = 0;
   ref_cvid = 0;
   ref_hid  = 0;
   ref_lgid = 0;
   ref_rgid = 0;

}

void ChipsId::set_axis_dim(int val)
{
   if ((val == XY_AXIS) || (val == X_AXIS) || (val == Y_AXIS)) 
      axisdim = val; 
}

void ChipsId::serialize (ChipsStateStream &out)
{
   int val, tmp_ref;
   out.write_string (&wid);
   out.write_string (&fid);
   out.write_string (&pid);
   out.write_string (&gid);
   out.write_string (&iid);
   out.write_string (&aid);
   out.write_string (&xaid);
   out.write_string (&yaid);
   out.write_string (&cbid);
   out.write_string (&lbid);
   out.write_string (&lnid);
   out.write_string (&ptid);
   out.write_string (&ctid);
   out.write_string (&cvid);
   out.write_string (&hid); 
   out.write_string (&lgid);
   out.write_string (&rgid);
   out.write_string (&coordid);
   val = static_cast <int> (coordsys);
   out.write_int (&val);
   out.write_int (&axisdim);

   tmp_ref = (int)ref_wid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_fid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_pid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_iid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_aid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_xaid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_yaid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_cbid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_lbid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_lnid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_ptid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_ctid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_cvid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_hid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_lgid;
   out.write_int (&tmp_ref);
   tmp_ref = (int)ref_rgid;
   out.write_int (&tmp_ref);
}

void ChipsId::deserialize (ChipsStateStream &in)
{
   int val, tmp_ref;
   in.read_string (&wid);
   in.read_string (&fid);
   in.read_string (&pid);
   in.read_string (&gid);
   in.read_string (&iid);
   in.read_string (&aid);
   in.read_string (&xaid);
   in.read_string (&yaid);
   in.read_string (&cbid);
   in.read_string (&lbid);
   in.read_string (&lnid);
   in.read_string (&ptid);
   in.read_string (&ctid);
   in.read_string (&cvid);
   in.read_string (&hid);
   in.read_string (&lgid);
   in.read_string (&rgid);
   in.read_string (&coordid);
   in.read_int (&val);
   coordsys = static_cast <COORD_SYS> (val);
   in.read_int (&axisdim);
   in.read_int (&tmp_ref);
   ref_wid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_fid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_pid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_iid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_aid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_xaid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_yaid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_cbid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_lbid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_lnid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_ptid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_ctid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_cvid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_hid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_lgid = (unsigned int)tmp_ref;
   in.read_int (&tmp_ref);
   ref_rgid = (unsigned int)tmp_ref;
}

void ChipsId::serialize (string &out)
{
   ostringstream stream;

   stream << wid.length () << " " << wid << " ";
   stream << fid.length () << " " << fid << " ";
   stream << pid.length () << " " << pid << " ";
   stream << gid.length () << " " << gid << " ";
   stream << iid.length () << " " << iid << " ";
   stream << aid.length () << " " << aid << " ";
   stream << xaid.length () << " " << xaid << " ";
   stream << yaid.length () << " " << yaid << " ";
   stream << cbid.length () << " " << cbid << " ";
   stream << lbid.length () << " " << lbid << " ";
   stream << lnid.length () << " " << lnid << " ";
   stream << ptid.length () << " " << ptid << " ";
   stream << ctid.length () << " " << ctid << " ";
   stream << cvid.length () << " " << cvid << " ";
   stream << hid.length () << " " << hid << " ";
   stream << rgid.length () << " " << rgid << " ";
   stream << coordid.length () << " " << coordid << " ";
   stream << (int)coordsys << " ";
   stream << axisdim;
   stream << ends;

   out += stream.str ();
}

void ChipsId::deserialize (string in)
{
   int val;
   istringstream stream;

   stream.str (in);

   stream >> val;
   if (val != 0)
       stream >> wid;
   stream >> val;
   if (val != 0)
       stream >> fid;
   stream >> val;
   if (val != 0)
       stream >> pid;
   stream >> val;
   if (val != 0)
       stream >> gid;
   stream >> val;
   if (val != 0)
       stream >> iid;
   stream >> val;
   if (val != 0)
       stream >> aid;
   stream >> val;
   if (val != 0)
       stream >> xaid;
   stream >> val;
   if (val != 0)
       stream >> yaid;
   stream >> val;
   if (val != 0)
       stream >> cbid;
   stream >> val;
   if (val != 0)
       stream >> lbid;
   stream >> val;
   if (val != 0)
       stream >> lnid;
   stream >> val;
   if (val != 0)
       stream >> ptid;
   stream >> val;
   if (val != 0)
       stream >> ctid;
   stream >> val;
   if (val != 0)
       stream >> cvid;
   stream >> val;
   if (val != 0)
       stream >> hid;
   stream >> val;
   if (val != 0)
       stream >> rgid;
   stream >> val;
   if (val != 0)
       stream >> coordid;
   stream >> val;
   coordsys = static_cast <COORD_SYS> (val);
   stream >> axisdim;

}



int ChipsId::set_object_id(V_OBJECT_TYPE obj, string val)
{
   int rval = ERR_STATUS_GOOD;
   switch (obj)
   {
      case chips_window:
         wid = val; 
      break;
      case chips_frame:
         fid = val;
      break;
      case chips_plot:
         pid = val; 
      break;
      case chips_axis:
         aid = val; 
      break;
      case chips_colorbar:
         cbid = val; 
      break;
      case chips_label:
         lbid = val; 
      break;
      case chips_line:
         lnid = val; 
      break;
      case chips_point:
         ptid = val; 
      break;
      case chips_region:
        rgid = val; 
      break;
      case chips_curve:
         cvid = val; 
      break;
      case chips_image:
         iid = val; 
      break;
      case chips_contour:
         ctid = val; 
      break;
      case chips_histogram:
         hid = val; 
      break;
      default:
         rval = ERR_STATUS_BAD;
      break;
   }
   return rval; 
}

int ChipsId::get_object_id(V_OBJECT_TYPE obj, string &val) const
{
   int rval = ERR_STATUS_GOOD;
   switch (obj)
   {
      case chips_window:
         val = wid;
      break;
      case chips_frame:
         val = fid;
      break;
      case chips_plot:
         val = pid;
      break;
      case chips_axis:
         val = aid;
      break;
      case chips_colorbar:
         val = cbid;
      break;
      case chips_label:
         val = lbid;
      break;
      case chips_line:
         val = lnid;
      break;
      case chips_point:
         val = ptid;
      break;
      case chips_region:
        val = rgid;
      break;
      case chips_curve:
         val = cvid;
      break;
      case chips_image:
         val = iid;
      break;
      case chips_contour:
         val = ctid;
      break;
      case chips_histogram:
         val = hid;
      break;
      default:
         rval = ERR_STATUS_BAD;
      break;
   }
   return rval;
}




int ChipsId::set_object_ref_id(V_OBJECT_TYPE obj, unsigned int val)
{
   int rval = ERR_STATUS_GOOD;
   switch (obj)
   {
      case chips_window:
         ref_wid = val;
      break;
      case chips_frame:
         ref_fid = val;
      break;
      case chips_plot:
         ref_pid = val;
      break;
      case chips_axis:
         ref_aid = val;
      break;
      case chips_colorbar:
         ref_cbid = val;
      break;
      case chips_label:
         ref_lbid = val;
      break;
      case chips_line:
         ref_lnid = val;
      break;
      case chips_point:
         ref_ptid = val;
      break;
      case chips_region:
        ref_rgid = val;
      break;
      case chips_curve:
         ref_cvid = val;
      break;
      case chips_image:
         ref_iid = val;
      break;
      case chips_contour:
         ref_ctid = val;
      break;
      case chips_histogram:
         ref_hid = val;
      break;
      default:
         rval = ERR_STATUS_BAD;
      break;
   }
   return rval;
}

int ChipsId::get_object_ref_id(V_OBJECT_TYPE obj, unsigned int &val) const
{
   int rval = ERR_STATUS_GOOD;
   switch (obj)
   {
      case chips_window:
         val = ref_wid;
      break;
      case chips_frame:
         val = ref_fid;
      break;
      case chips_plot:
         val = ref_pid;
      break;
      case chips_axis:
         val = ref_aid;
      break;
      case chips_colorbar:
         val = ref_cbid;
      break;
      case chips_label:
         val = ref_lbid;
      break;
      case chips_line:
         val = ref_lnid;
      break;
      case chips_point:
         val = ref_ptid;
      break;
      case chips_region:
        val = ref_rgid;
      break;
      case chips_curve:
         val = ref_cvid;
      break;
      case chips_image:
         val = ref_iid;
      break;
      case chips_contour:
         val = ref_ctid;
      break;
      case chips_histogram:
         val = ref_hid;
      break;
      default:
         rval = ERR_STATUS_BAD;

      break;
   }
   return rval;
}

