// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSID_HH
#define CHIPSID_HH

#include <string>
#include <iostream>
#include <stdio.h>
#include "ChipsDefines.hh"


class ChipsStateStream;

using namespace std;


class ChipsId 
{
  public:

   ChipsId();
   ChipsId(const char* ww, const char* ff=NULL, const char* pp=NULL);
   ChipsId(const ChipsId& orig);
   ChipsId (ChipsStateStream &in);
   virtual ~ChipsId(); 

   void operator = (const ChipsId &orig);


   void        set_all(); 
   void        clear()   { reset_window_id(""); }

   const char* get_window_id() const   { return wid.c_str(); }
   void        set_window_id(string val)   { wid = val; }
   void        reset_window_id(string val);

   unsigned int get_window_ref_id() const   { return ref_wid; }
   void        set_window_ref_id(unsigned int val)   { ref_wid = val; }

   const char* get_frame_id() const   { return fid.c_str(); }
   void        set_frame_id(string val)    { fid = val; }
   void        reset_frame_id(string val);

   unsigned int get_frame_ref_id() const   { return ref_fid; }
   void        set_frame_ref_id(unsigned int val)    { ref_fid = val; }

   const char* get_group_id() const   { return gid.c_str(); }
   void        set_group_id(string val)    { gid = val; }

   const char* get_plot_id() const   { return pid.c_str(); }
   void        set_plot_id(string val)    { pid = val; }
   void        reset_plot_id(string val);

   unsigned int get_plot_ref_id() const   { return ref_pid; }
   void        set_plot_ref_id(unsigned int val)    { ref_pid = val; }

   const char* get_image_id() const   { return iid.c_str(); }
   void        set_image_id(string val)    { iid = val; }

   unsigned int get_image_ref_id() const   { return ref_iid; }
   void        set_image_ref_id(unsigned int val)    { ref_iid = val; }

   const char* get_axis_id() const   { return aid.c_str(); }
   void        set_axis_id(string val)    { aid = val; xaid = yaid = ""; }

   unsigned int get_axis_ref_id() const   { return ref_aid; }
   void        set_axis_ref_id(unsigned int val)    { ref_aid = val; ref_xaid = ref_yaid = 0; }

   const char* get_xaxis_id() const   { return xaid.c_str(); }
   void        set_xaxis_id(string val)    { xaid = val; }

   unsigned int get_xaxis_ref_id() const   { return ref_xaid; }
   void        set_xaxis_ref_id(unsigned int val)    { ref_xaid = val; }

   const char* get_yaxis_id() const   { return yaid.c_str(); }
   void        set_yaxis_id(string val)    { yaid = val; }

   unsigned int get_yaxis_ref_id() const   { return ref_yaid; }
   void        set_yaxis_ref_id(unsigned int val)    { ref_yaid = val; }

   const char* get_curve_id() const   { return cvid.c_str(); }
   void        set_curve_id(string val)    { cvid = val; }

   unsigned int get_curve_ref_id() const   { return ref_cvid; }
   void        set_curve_ref_id(unsigned int val)    { ref_cvid = val; }

   const char* get_contour_id() const   { return ctid.c_str(); }
   void        set_contour_id(string val)    { ctid = val; }

   unsigned int get_contour_ref_id() const   { return ref_ctid; }
   void        set_contour_ref_id(unsigned int val)    { ref_ctid = val; }

   const char* get_histogram_id() const   { return hid.c_str(); }
   void        set_histogram_id(string val)    { hid = val; }

   unsigned int get_histogram_ref_id() const   { return ref_hid; }
   void        set_histogram_ref_id(unsigned int val)    { ref_hid = val; }

   const char* get_colorbar_id() const   { return cbid.c_str(); }
   void        set_colorbar_id(string val)    { cbid = val; }

   unsigned int get_colorbar_ref_id() const   { return ref_cbid; }
   void        set_colorbar_ref_id(unsigned int val)  { ref_cbid = val; }

   const char* get_label_id() const   { return lbid.c_str(); }
   void        set_label_id(string val)    { lbid = val; }

   unsigned int get_label_ref_id() const   { return ref_lbid; }
   void        set_label_ref_id(unsigned int val)    { ref_lbid = val; }

   const char* get_line_id() const   { return lnid.c_str(); }
   void        set_line_id(string val)    { lnid = val; }

   unsigned int get_line_ref_id() const   { return ref_lnid; }
   void        set_line_ref_id(unsigned int val)    { ref_lnid = val; }

   const char* get_point_id() const   { return ptid.c_str(); }
   void        set_point_id(string val)    { ptid = val; }

   unsigned int get_point_ref_id() const   { return ref_ptid; }
   void        set_point_ref_id(unsigned int val)    { ref_ptid = val; }

   const char* get_legend_id() const   { return lgid.c_str(); }
   void        set_legend_id(string val)    { lgid = val; }

   unsigned int get_legend_ref_id() const   { return ref_lgid; }
   void        set_legend_ref_id(unsigned int val)    { ref_lgid = val; }

   const char* get_region_id() const   { return rgid.c_str(); }
   void        set_region_id(string val)    { rgid = val; }

   unsigned int get_region_ref_id()  const  { return ref_rgid; }
   void        set_region_ref_id(unsigned int val)    { ref_rgid = val; }

   const char* get_coord_id() const   { return coordid.c_str(); }
   void        set_coord_id(string val)    { coordid = val; }

   COORD_SYS   get_coord_sys() const   { return coordsys; }
   void        set_coord_sys(COORD_SYS val)    { coordsys = val; }

   int         get_axis_dim() const    { return axisdim; }
   void        set_axis_dim(int dim);

   void        serialize (ChipsStateStream &out);
   void        deserialize (ChipsStateStream &in);

   void        serialize (string &string);
   void        deserialize (string string);


   int         set_object_id(V_OBJECT_TYPE obj, string val);
   int         set_object_ref_id(V_OBJECT_TYPE obj, unsigned int val);

   int         get_object_id(V_OBJECT_TYPE obj, string &val) const;
   int         get_object_ref_id(V_OBJECT_TYPE obj, unsigned int &val) const;


  private:
   string     wid;           // window id
   string     fid;           // frame id
   string     pid;           // plot id
   string     iid;           // image id 
   string     gid;           // group id  
   string     ctid;          // contour id 
   string     cvid;          // curve id 
   string     hid;           // histogram id 
   string     aid;           // axis id 
   string     xaid;          // x axis id 
   string     yaid;          // y axis id 
   string     cbid;          // colorbar id 
   string     lbid;          // label id 
   string     lnid;          // line id
   string     ptid;          // point id 
   string     lgid;          // legend id 
   string     rgid;          // region id 

   int        ref_wid;           // window ref id
   int        ref_fid;           // frame ref id
   int        ref_pid;           // plot  ref id
   int        ref_iid;           // image ref id
   int        ref_ctid;          // contour ref id
   int        ref_cvid;          // curve ref id
   int        ref_hid;           // histogram ref id
   int        ref_aid;           // axis ref id
   int        ref_xaid;          // x axis ref id
   int        ref_yaid;          // y axis ref id
   int        ref_cbid;          // colorbar ref id
   int        ref_lbid;          // label ref id
   int        ref_lnid;          // line ref id
   int        ref_ptid;          // point ref id
   int        ref_lgid;          // legend ref id
   int        ref_rgid;          // region ref id

   string     coordid;       // coordinate system id 
   COORD_SYS  coordsys;	     // coordinate system
   int        axisdim;       // axis dimension
};


#endif 
