// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSLABEL_HH
#define CHIPSLABEL_HH

#include <string>
#include <stdio.h>
#include "ChipsObject.hh"
#include "ChipsDefaults.hh"
#include "ChipsAnnotation.hh"
#include "ChipsCoordinate.hh"
#include "ChipsStateStream.hh"
#include "ChipsId.hh"

#include "ChipsTextSupport.hh"

using namespace std;

class ChipsCommand;

class ChipsLabel: public ChipsAnnotation
{
  public:

   ChipsLabel(ChipsObject* par, string lbid, ChipsCoordinate *inCoord, 
              COORD_SYS coord_sys, double inXPos, double inYPos, string val);
   ChipsLabel(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, ChipsStateStream& in);
   
   virtual void object_delete ();

   int          set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj);
   int          set_property(void* in_val,  int property, bool rec=true);
   int          get_property(void* out_val, int property);

   string       get_text()    { return text; }
   int          set_text(string val); 
   string       get_font()    { return font; }
   int          set_font(string in_font);
   V_FONTSTYLE_TYPE  get_fontstyle()    { return style; }
   int          set_fontstyle(V_FONTSTYLE_TYPE in_style);
   int          get_size()    { return size; }
   int          set_size(int val);
   int          set_position (const ChipsVector &pos);
   int          get_position (ChipsVector &pos);
   int          change_coordinate (COORD_SYS coordsys,
                                          const string& coordid);
   int          set_bounding_box(int val);
   
   int          set_vertalignment(double val);
   int          set_horizalignment(double val);
   double       get_vertalignment(){return vert_alignment;};
   double       get_horizalignment(){return horiz_alignment;};

   void       set_label_num (int val) {label_num = val;};
   int        get_label_num () {return label_num;};

   int  resize (COORD_SYS coord_sys, const string &coord_id, 
                    double *bounds, bool rec=true);

   virtual int  set_depth(int newdepth);
   virtual int  move(COORD_SYS coord_sys, const string &coord_id, double xx,
                    double yy, int mode, bool rec=true);
   virtual int  compute_resize (double *bounds);

//   virtual int  reset_position () {return set_position (orig[0], orig[1]);};
   virtual int  reset_position ();
   virtual int  redraw(); 
   virtual int set_color(string in_color);
   virtual int  change_text();
   virtual int  change_font();
   virtual int  change_size(); 
   virtual int  change_scale_factor(); 
   virtual int  change_position(); 
   virtual int  change_alignment();
   virtual int  change_visible();
   virtual int  change_depth();
   virtual int  change_bbox();

   virtual int  show ();
   virtual int  hide ();



   void       initialize();
   virtual void serialize(ChipsStateStream& out);
   virtual void serialize_ascii(ChipsStateStream& out);
   virtual void deserialize(ChipsStateStream& in);

  protected:

   virtual ~ChipsLabel();
   bool add_command (ChipsCommand *inCmd, int &outStatus);


   string       text;          // text of label 
   int          size;          // angle or rotation
   string       font;          // font of label
   V_FONTSTYLE_TYPE style;     // normal, bold, italic 
   double       vert_alignment;
   double       horiz_alignment;

   ChipsVector orig;

   ELEM_P_T     format;        // tree containing text format
   int        label_num;       // this labels order 0th, 1st, 2nd, etc.
   int          bbox;          // display bounding box?
};


#endif 
