// 
//  Copyright (C) 2004,2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsLegend.hh"

#include <stdio.h>

extern ChipsDefaults* theDefaults; 


ChipsLegend::ChipsLegend(ChipsObject* par, const char** cvids, 
   const char** in_labels, string lgid):ChipsAnnotation(par, lgid)
{
   int rr = 0;
   string cv, lb;

   id = lgid; 
   initialize();
   cnt = 0;
   while (cvids[cnt] && in_labels[cnt]) cnt++;

   for (rr = 0; rr < cnt; rr++)
   {
      cv = cvids[rr]; 
      lb = in_labels[rr]; 
      curves.push_back(cv);
      labels.push_back(lb); 
   }

}


ChipsLegend::~ChipsLegend()
{
   curves.clear();
   labels.clear();
}


void ChipsLegend::initialize()
{
   string bkgd_color;

   if (theDefaults)
   {
      thickness    = theDefaults->get_legendthickness();
      opaque       = !theDefaults->get_legendtransparent();
      borders      = theDefaults->get_legendborders();
      bkgd_color   = theDefaults->get_legendcolor();
   }
   else
   {
      thickness    = 1;
      opaque       = false;
      borders      = false;
      bkgd_color   = "black"; 
   }
   object_type = chips_legend;
   set_color(bkgd_color);
}


int ChipsLegend::set_property(void* in_val, int in_property) 
{
   int rval = ERR_STATUS_GOOD;

   switch(in_property)
   {
      case SET_LEGEND_COLOR:
      {
         string clr = (const char*)in_val;
         rval = set_color(clr);
      }
      break; 
      case SET_LEGEND_THICKNESS:
      {
         float* fval = static_cast<float*>(in_val);
         rval = set_thickness(*fval);
      }
      break; 
      case SET_LEGEND_TRANSPARENT:
      {
         int* ival = static_cast<int*>(in_val);
         set_transparency(*ival == 1);
      }
      break;
      case SET_LEGEND_VISIBLE:
      {
         int* ival = static_cast<int*>(in_val);
         if (*ival == 1)
            show();
         else 
            hide();
      }
      break;
      case SET_LEGEND_BORDERS:
      {
         int* ival = static_cast<int*>(in_val);
         set_borders(*ival == 1);
      }
      break;
      default:
         rval = ERR_UNKNOWN_PROPERTY;
      break;
   }
   return rval;
}



int ChipsLegend::set_thickness(float val)
{
   int rval = ERR_DATA_RANGE;

   // need to determine thickness of point
   if ((val < V_THICK_MIN_VAL) || (val > V_THICK_MAX_VAL))
      rval = ERR_THICKNESS_VAL;
   else
   {
      thickness = val;
      rval = change_thickness(); 
   }
   return rval;
}


int ChipsLegend::set_transparency(bool transparency)
{
   int rval = ERR_STATUS_GOOD;

   if (opaque == transparency)
   {
      opaque = !transparency;
      rval = change_transparency();
   }
   return rval;
}


int ChipsLegend::set_borders(bool show)
{
   int rval = ERR_STATUS_GOOD;

   if (borders != show)
   {
      borders = show; 
      rval = change_borders();
   }
   return rval;
}



int ChipsLegend::redraw()
{
   cout << " ChipsLegend - virtual redraw() " << endl; 
   return ERR_FEATURE_DNE;
}


int ChipsLegend::change_thickness()
{
   cout << " ChipsLegend - virtual change_thickness() " << endl; 
   return ERR_FEATURE_DNE;
}


int ChipsLegend::change_transparency()
{
   cout << " ChipsLegend - virtual change_transparency() " << endl; 
   return ERR_FEATURE_DNE;
}


int ChipsLegend::change_position()
{
   cout << " ChipsLegend - virtual change_position() " << endl; 
   return ERR_FEATURE_DNE;
}


int ChipsLegend::change_borders()
{
   cout << " ChipsLegend - virtual change_borders() " << endl;
   return ERR_FEATURE_DNE;
}


