// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSLINE_HH
#define CHIPSLINE_HH

#include <string>
#include <stdio.h>
#include "ChipsObject.hh"
#include "ChipsDefaults.hh"
#include "ChipsAnnotation.hh"
#include "ChipsStateStream.hh"
#include "ChipsId.hh"

using namespace std;

class ChipsCommand;

class ChipsLine: public ChipsAnnotation
{
  public:

   ChipsLine(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys,
	     double startx, double starty, 
             double endx, double endy, string lnid);
   ChipsLine(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys,
             ChipsStateStream& in);
   ChipsLine(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, 
             double* xcoords, double* ycoords,
             unsigned int cnt, string lnid);
   ChipsLine (const ChipsLine &inLine);

   void       object_delete ();

   int        change_coordinate (COORD_SYS coordsys,
                                          const string& coordid);
   int        get_position (ChipsVector &pos);

   int        set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj);
   int        set_property(void* in_val, int in_property, bool rec=true);
   int        get_property(void* out_val, int property);

   float      get_thickness()    { return thickness; }
   int        set_thickness(float val);

   V_LINE_TYPE get_linestyle()   { return linestyle; }
   int        set_linestyle(V_LINE_TYPE val); 
   int        set_linestyle(string val);

   int        get_extend () { return extend_line; };
   int        set_extend (V_LINE_EXTENSION_TYPE val);
   int        set_extend (string val);

   void       initialize();
   void       set_line_num (int val) {line_num = val;};
   int        get_line_num () {return line_num;};
   virtual void serialize(ChipsStateStream& out);
   virtual void serialize_ascii(ChipsStateStream& out);
   virtual void deserialize(ChipsStateStream& in);


   virtual int set_depth(int newdepth);
   virtual int redraw(); 
   virtual int move(COORD_SYS coord_sys, const string &coord_id, double xx,
                    double yy, int mode, bool rec=true);
   virtual int reset_position ();
   virtual int change_position(); 
   virtual int change_linestyle(); 
   virtual int change_thickness(); 
   virtual int change_visible();
   virtual int change_depth();
   virtual int update_data();
   virtual int update_clipping ();

   virtual int show ();
   virtual int hide ();

  protected:

   virtual ~ChipsLine();

   int calculate_end_points ();
   bool add_command (ChipsCommand *inCmd, int &outStatus);

   ChipsVector*     pnts;        // coordinate points
   ChipsVector      endpoints[2]; // user specified end points

   unsigned int num_pnts;    // total number of points in line 
   float      thickness;     // thickness of line 
   V_LINE_TYPE linestyle;    // type of line  

   int        line_num;       // this point order 0th, 1st, 2nd, etc.
   int        extend_line;    // direction to extend the endpoints
};


#endif 
