// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSMATRIX_HH
#define CHIPSMATRIX_HH

#include "ChipsErrs.hh"

using namespace std;

class ChipsMatrix;

class ChipsVector
{
  public:
   ChipsVector ();
   ChipsVector (double x, double y);
   ChipsVector (double x, double y, double z);
   ChipsVector (int size);
   ChipsVector (const ChipsVector &v);
   ~ChipsVector ();
   void operator = (double elem);
   void operator = (const ChipsVector &v);
   void operator = (double *elem);
   double &operator [] (unsigned int i) {return elements[i];};
   const double &operator [] (unsigned int i) const {return elements[i];};
   ChipsVector operator + (const ChipsVector &v);
   ChipsVector operator - (const ChipsVector &v);
   void operator += (const ChipsVector &v) {*this = *this + v;};
   void operator -= (const ChipsVector &v) {*this = *this - v;};
   bool operator == (const ChipsVector &v);
   bool operator != (const ChipsVector &v);
   ChipsVector operator * (double scale);
   const int get_size () const {return size;};

   void print ();

  protected:
   double *elements;
   int size;
};

class ChipsMatrix
{
  public:
   ChipsMatrix ();
   ChipsMatrix (int r, int c);
   ChipsMatrix (const ChipsMatrix &m);
   ~ChipsMatrix ();

   void operator = (const ChipsMatrix &m);
   ChipsVector operator * (const ChipsVector &v);
   ChipsMatrix operator * (const ChipsMatrix &m);
   double *operator [] (unsigned int i) { return (elements[i]);};
   const double *operator [] (unsigned int i) const { return (elements[i]);};
 
   void identity (); 
   double determinant (); 
   bool invertable () {return (determinant () != 0);};
   ChipsMatrix inverse ();
   void set_size (int r, int c);
   int set_elements (double *elem);
   int set_elements (double **elem);
   const int get_row () const {return this->row;};
   const int get_col () const {return this->col;};

   void print ();

  protected:
   double **elements;
   int row, col;
};

#endif
