// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsObject.hh"

#include <stdio.h>
#include <cmath>
#include <sstream>

unsigned int ChipsObject::cur_ref_cnt = 0;

ChipsObject::ChipsObject(ChipsObject* par, string val)
{
   id = val; 
   parent = par; 

   nx1 = 0.0;
   ny1 = 0.0;
   nx2 = 1.0;
   ny2 = 1.0;

   visible = true;
   vis_override = false;
   depth = V_USE_DEFAULT_DEPTH;
   coord_system = COORD_NONE;
   data_system_id = "";
   coord = NULL;
   object_type = chips_object;
   status = ERR_STATUS_GOOD;
   cur_ref_cnt++;
   ref_cnt = cur_ref_cnt;

   interaction_blocked=false;
}


ChipsObject::ChipsObject(const ChipsObject &co)
{
   id = co.id;
   parent = co.parent;
   object_type = co.object_type;

   nx1 = co.nx1;
   ny1 = co.ny1;
   nx2 = co.nx2;
   ny2 = co.ny2;

   coord_system = co.coord_system;
   data_system_id = co.data_system_id;
   coord = co.coord;

   visible = co.visible;
   vis_override = co.vis_override;
   depth = co.depth; 
   status = ERR_STATUS_GOOD;
   cur_ref_cnt = co.cur_ref_cnt;
   ref_cnt = co.ref_cnt;
}


ChipsObject::~ChipsObject()
{

}

void ChipsObject::object_delete ()
{
   delete (this);
}


int ChipsObject::get_norm_pos(float &x1, float &y1, float &x2, float &y2)
{
   x1 = nx1;
   x2 = nx2;
   y1 = ny1;
   y2 = ny2;
   return 1;
}


ChipsAxisBind* ChipsObject::get_bind_table()
{
   if (parent != NULL)
      return parent->get_bind_table();
   return NULL;
}


double ChipsObject::get_scale_factor()
{
   double factor = 1.0; 
   if (parent != NULL)  factor =  parent->get_scale_factor();
   return factor;
}

int ChipsObject::set_depth(int newdepth)
{
   if (newdepth != depth)
   {
      depth = newdepth;
   }
   return ERR_STATUS_GOOD; 
}

bool ChipsObject::set_vis_override(bool val)
{
   bool oval = vis_override;
   vis_override = val;
   return oval;   // pass back the old value 
}

bool ChipsObject::is_visible()
{ 
   bool vis = true;
  
   if (coord)
      vis = coord->is_visible ();

   return (visible && !vis_override && vis); 
}

int ChipsObject::force_hide ()
{
   int rval;
   bool curr_visible = visible;
   rval = this->hide ();
   visible = curr_visible;

   return rval;
}

int ChipsObject::force_show ()
{
   int rval;
   bool curr_visible = visible;
   rval = this->show ();
   visible = curr_visible;

   return rval;
}

void ChipsObject::populate_chips_id (ChipsId &cid)
{
   cid.set_object_id(this->object_type, this->id);
   cid.set_object_ref_id(this->object_type, this->ref_cnt);
   if (this->parent != NULL)
      this->parent->populate_chips_id (cid);

}

bool ChipsObject::is_interaction_blocked()
{
   bool blocked = this->interaction_blocked;
   if (parent != NULL)  blocked |=  parent->is_interaction_blocked();
   return blocked;
}

