// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsPanCommand.hh"
#include "ChipsStateStream.hh"
#include "ChipsId.hh"
#include "ChipsFrame.hh"

ChipsPanCommand::ChipsPanCommand(ChipsFrame* obj, double o_x, double o_y,
                              double n_x, double n_y, ChipsId *cid_)
{
   co = (ChipsObject*)obj;
   weight = 1;

   this->old_x = o_x;
   this->old_y = o_y;
   this->new_x = n_x;
   this->new_y = n_y;
   if (cid)
      this->cid = new ChipsId (*cid_);
   else
      this->cid = NULL;
}

ChipsPanCommand::~ChipsPanCommand()
{
   if (this->cid)   delete  this->cid;
}


ChipsPanCommand::ChipsPanCommand(ChipsObject *cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsPanCommand::execute ()
{
   int rval = ERR_STATUS_BAD;
   ChipsFrame* cf = dynamic_cast <ChipsFrame *>(co);
   if (cf)
      rval = cf->pan(this->cid, this->old_x, this->old_y, this->new_x, this->new_y, false);
   return rval;
}

int ChipsPanCommand::unexecute ()
{
   int rval = ERR_STATUS_BAD;
   ChipsFrame* cf = dynamic_cast <ChipsFrame *>(co);
   if (cf)
      rval = cf->pan(this->cid, this->new_x, this->new_y, this->old_x, this->old_y, false);
   return rval;
}

void ChipsPanCommand::serialize(ChipsStateStream& out)
{
   int val = 25;
   bool cid_exists = true;
   
   // Standard serialize output for a command
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   out.write_double(&this->old_x);
   out.write_double(&this->old_y);
   out.write_double(&this->new_x);
   out.write_double(&this->new_y);

   if (this->cid)
   {
      out.write_bool (&cid_exists);
      this->cid->serialize (out);
   }
   else
   {
      cid_exists = false;
      out.write_bool (&cid_exists);
   }
}

void ChipsPanCommand::deserialize(ChipsStateStream& in)
{
   bool cid_exists;

   // Standard serialize input for a command
   in.read_int(&ref);
   in.read_int(&block_ref);

   in.read_double(&this->old_x);
   in.read_double(&this->old_y);
   in.read_double(&this->new_x);
   in.read_double(&this->new_y);
   in.read_bool (&cid_exists);
   if (cid_exists)
      this->cid = new ChipsId (in);
}

void ChipsPanCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Pan" << endl;
};



