// 
//  Copyright (C) 2004-2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSPLOT_HH
#define CHIPSPLOT_HH

#include <list>
#include <string>
#include <map>
#include <vector>
#include <stdio.h>
#include "ChipsId.hh"
#include "ChipsObject.hh"
#include "ChipsDefaults.hh"
#include "ChipsAxis.hh"
#include "ChipsAxisBind.hh"
#include "ChipsContour.hh"
#include "ChipsCurve.hh"
#include "ChipsHistogram.hh"
#include "ChipsImage.hh"
#include "ChipsLegend.hh"
#include "ChipsCoordinate.hh"
#include "ChipsDataSystem.hh"
#include "ChipsStateStream.hh"
#include "ChipsDepthControl.hh"
#include "ChipsAxisAlgorithmFactory.hh"

class ChipsImage; 
class ChipsColorBar; 
class ChipsLine; 
class ChipsLabel;
class ChipsPoint;
class ChipsRegion; 

class Transform;

using namespace std;

class ChipsPlot: public ChipsObject
{
  public:

   ChipsPlot(ChipsObject* par, const string& val);
   ChipsPlot(ChipsObject* par, ChipsStateStream& in);
   ChipsPlot(ChipsObject* par, float x1, float x2, float y1, float y2,
            const string& val);

   void        object_delete ();
   int         load_depth_table(ChipsDepthControl*);
   int         rearrange_object(ChipsId* cid, V_OBJECT_TYPE obj, int manner, bool rec=false); 
   int         get_object_count(ChipsId* cid, V_OBJECT_TYPE obj, int& out_val);
   int         get_object_coordinfo(ChipsId* cid, V_OBJECT_TYPE obj, 
                                    COORD_SYS& out_val, string& out_id);
   int         set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj); 
   int         set_property(ChipsId* cid, void* in_val, int in_property,
                            bool rec=true);
   int         set_image_property(ChipsId* cid, void* in_val, int in_property);
   int         set_curve_property(ChipsId* cid, void* in_val, int in_property);
   int         set_contour_property(ChipsId* cid, void* in_val,int in_property);
   int         set_histogram_property(ChipsId* cid, void* in_val,
                                      int in_property);
   int         set_label_property(ChipsId* cid, void* in_val, int in_property);
   int         set_colorbar_property(ChipsId* cid, void* in_val, int in_property);
   int         set_line_property(ChipsId* cid, void* in_val, int in_property);
   int         set_point_property(ChipsId* cid, void* in_val, int in_property);
   int         set_region_property(ChipsId* cid, void* in_val, int in_property);
   int         set_axis_property(ChipsId* cid, void* in_val, int in_property);
   int         set_dimension_property(ChipsId* cid, int dim, int in_property, bool rec=true);
   int         get_property(ChipsId* cid, void* out_val, int property);
   int         get_image_property(ChipsId* cid, void* out_val, int property);
   int         get_curve_property(ChipsId* cid, void* out_val, int property);
   int         get_contour_property(ChipsId* cid, void* out_val, int property);
   int         get_histogram_property(ChipsId* cid, void* out_val,
                                      int property);
   int         get_label_property(ChipsId* cid, void* out_val, int property);
   int         get_line_property(ChipsId* cid, void* out_val, int property);
   int         get_colorbar_property(ChipsId* cid, void* out_val, int property);
   int         get_point_property(ChipsId* cid, void* out_val, int property);
   int         get_region_property(ChipsId* cid, void* out_val, int property);
   int         get_axis_property(ChipsId* cid, void* out_val, int property);
   int         reposition(float x1, float y1, float x2, float y2);
   ChipsLabel* get_title()   { return title; }
   string      get_title_text(); 
   int         set_title_text(const string& val); 
   int         get_title_depth();
   int         set_title_depth(int dep);
   void        initialize(bool create_chipscoord=true); 
   void        post_initialize ();
   void        add_border_axes();
   void        remove_box_axes(ChipsAxis* ax);

   int         set_plot_lmargin(float margin_sz);
   float       get_plot_lmargin()    {  return  left_margin; }  
   int         set_plot_rmargin(float margin_sz);
   float       get_plot_rmargin()    {  return  right_margin; }  
   int         set_plot_tmargin(float margin_sz);
   float       get_plot_tmargin()    {  return  top_margin; }  
   int         set_plot_bmargin(float margin_sz);
   float       get_plot_bmargin()    {  return  bottom_margin; }  
   int         set_title_xpos(float pos);
   float       get_title_xpos() {  return  title_xpos; }
   int         set_title_ypos(float pos);
   float       get_title_ypos() {  return  title_ypos; }
   string      get_title_color();
   int         set_title_color(string in_color);
   int         get_title_size();
   int         set_title_size(int sz);
   string      get_title_font();
   int         set_title_font(string font);
   int         set_title_angle(float ang);
   float       get_title_angle();      
   int         set_title_halign(float ang);
   float       get_title_halign();      
   int         set_title_valign(float ang);
   float       get_title_valign();      
   string      get_title_fontstyle();
   int         set_title_fontstyle(string fontstyle); 
   V_AXISSTYLE_TYPE get_axis_style()  { return axis_style; }
   int         set_axis_style(V_AXISSTYLE_TYPE val, bool rec=true); 
   int         set_corner_style(V_CORNERSTYLE_TYPE val); 
   string      get_corner_style();

   int        resize_object(const ChipsId* cid, V_OBJECT_TYPE obj, V_SUBOBJECT_TYPE subobj, double *bounds);
   int        rotate_object(ChipsId* cid, V_OBJECT_TYPE obj, V_SUBOBJECT_TYPE subobj, float angle);
   int        select_object(const ChipsId* cid, V_OBJECT_TYPE obj, V_SUBOBJECT_TYPE subobj, bool selected);
   int        move_object(ChipsId* cid, V_OBJECT_TYPE obj, V_SUBOBJECT_TYPE subobj, double xx, double yy, int mode);

   int        get_annotation_position (const ChipsId* cid,
                                          V_OBJECT_TYPE obj, ChipsVector &pos);
   int        change_annotation_coordinate (const ChipsId* cid,
                                          V_OBJECT_TYPE obj,
                                          COORD_SYS coordsys,
                                          const string& coordid);


   int         add_curve(ChipsId* cid, long cnt, double* x, double* y,
                         double* ydown, double* yup, double* xdown, 
                         double* xup, double* ylims, double* xlims,
                         Transform *transform);
   int         delete_curve(ChipsId* cid, bool force_del=false);
   string      get_current_curve()   {return currcurv; }  
   int         get_current_curve(ChipsId* cid); 
   int         set_current_curve(ChipsId* cid, bool rec=true); 
   ChipsCurve* get_curve(const string& val=""); 
   int         set_xlabel_text(ChipsId* cid, const string& val);
   int         set_ylabel_text(ChipsId* cid, const string& val);

   int         add_contour(ChipsId* cid, long cnt, double* x, double* y,
                           double* z,  unsigned int count, double *levels, 
                           Transform *trans);
   int         add_contour(ChipsId* cid, double* dat, long xdim, long ydim,
                           unsigned int numlev, double* levels, Transform *trans);
   int         delete_contour(ChipsId* cid, bool force_del=false);
   int         set_contour_levels(ChipsId* cid, unsigned int arraysize,  
                                  double* pos);
   int         get_contour_levels(ChipsId* cid, vector<double> &pos);
   string      get_current_contour()   {return currcont; }
   int         get_current_contour(ChipsId* cid);
   int         set_current_contour(ChipsId* cid, bool rec=true);
   ChipsContour* get_contour(const string& val=""); 

   int         add_histogram(ChipsId* cid, long cnt, 
                             double* lo, double* hi, double* counts,
                             double* yd = NULL, double* yu = NULL);
   int         delete_histogram(ChipsId* cid, bool force_del=false);
   ChipsHistogram* get_histogram(const string& val=""); 
   string      get_current_histogram()   {return currhist; }  
   int         get_current_histogram(ChipsId* cid);
   int         set_current_histogram( ChipsId* cid, bool rec=true);

   int         add_axis (ChipsId* cid, int dim, double pos,double min,double max,
                         Transform *transform, bool setup_axis=true);
   int         add_axis (ChipsId* cid, int dim, double pos,double minx,double maxx,
                         double miny,double maxy, Transform *transform,
                         bool setup_axis=true);
   int         delete_axis(ChipsId* cid, bool force_del=false);
   int         delete_xaxis(ChipsId* cid, bool force_del=false);
   int         delete_yaxis(ChipsId* cid, bool force_del=false);
   int         set_arbitrary_tick_positions(ChipsId* cid, unsigned int  
                                            arraysize, double* pos,
                                            string* vals=NULL);
   ChipsAxis*  get_xaxis(const string& val="");
   ChipsAxis*  get_yaxis(const string& val="");
   int         get_xaxis_cnt();
   int         get_yaxis_cnt();
   ChipsAxis*  get_current_xaxis()  { return cur_x; }  
   ChipsAxis*  get_current_yaxis()  { return cur_y; }  
   string      get_current_xaxis_id();
   string      get_current_yaxis_id();
   int         get_current_xaxis(ChipsId* cid);
   int         get_current_yaxis(ChipsId* cid);
   int         set_current_xaxis( ChipsId* cid, bool rec=false);
   int         set_current_yaxis(ChipsId* cid, bool rec=false);
   int         set_current_axis(ChipsId* cid, bool rec=true);

   int         limits(ChipsId* cid, int dim, double xmin, double xmax, 
                      double ymin, double ymax);
   int         limits(ChipsId* cid, int dim, double min, double max);
   int         limits(ChipsId* cid, int dim, V_AUTOLIMIT_TYPE automin,
                      V_AUTOLIMIT_TYPE automax);
   int         limits(ChipsId* cid, int dim, double min, 
                      V_AUTOLIMIT_TYPE automax);
   int         limits(ChipsId* cid, int dim, V_AUTOLIMIT_TYPE automin, 
                      double max);
   int         zoom (ChipsId* cid, float scale);
   int         panto (ChipsId* cid, double x, double y);
   int         pan (ChipsId* cid, double x1, double y1, double x2, double y2);

   int         format_axis_point (const ChipsId *cid, double pnt, string &formated_pnt);
   int         format_axis_point (const string &axis_id, double pnt, string &formated_pnt);


   int         add_legend(ChipsId* cid, const char** curve_ids,
                          const char** curve_labels, const string& lgid = "");
   int         delete_legend(ChipsId* cid);

   //methods for image object interactions
   int        add_image(ChipsId* cid, string filename, Transform *transform);
   int        add_image(ChipsId* cid, string filename,
                       float ix1, float iy1, float ix2, float iy2, Transform *transform);
   int        add_image(ChipsId* cid, double* rvals, double *gvals, double *bvals, double *avals, int xaxis, int yaxis, Transform *transform);
   
   int        delete_image(ChipsId* cid, bool force_del=false);
   int        set_image_channel(ChipsId* cid, double* vals, int xaxis, int yaxis, int channel);
   int        remove_image_channel(ChipsId* cid, int channel);
   string     get_current_image()  { return currimage; }
   int        set_current_image(ChipsId* cid, bool rec=true);
   ChipsImage* get_image(const string& val="");
   int        print_image (ChipsId *cid, const string &filename, int fmt);

   //methods for color bar interactions 
   int        add_colorbar(ChipsId* cid, double xpos, double ypos, 
                            string obj_id="", V_OBJECT_TYPE obj=chips_not_object);
   int        delete_colorbar(ChipsId* cid, bool force_del=false);
   string     get_current_colorbar()  { return currcbar; }
   int        set_current_colorbar(ChipsId* cid, bool rec=true);
   ChipsColorBar* get_colorbar(const string& val="");

   //methods for label object interactions
   int        add_label(ChipsId* cid, double xpos, double ypos, string text);
   int        delete_label(ChipsId* cid, bool force_del=false);
   string     get_current_label()  { return currlabel; }
   int        set_current_label(ChipsId* cid, bool rec=true);
   ChipsLabel* get_label(const string& val="");

   //methods for line object interactions
   int        add_line(ChipsId* cid, double startx, double starty,
                       double endx, double endy);
   int        add_line(ChipsId* cid, double* xx, double* yy,
                       unsigned int count);
   int        delete_line(ChipsId* cid,  bool force_del=false);
   ChipsLine* get_line(const string& val="");
   int        set_current_line(ChipsId* cid, bool rec=true);
   string     get_current_line()  { return currline; }

   // methods for point object interactions
   int        add_point(ChipsId* cid, double xx, double yy);
   int        add_point(ChipsId* cid, double xx, double yy, double zz);
   int        delete_point(ChipsId* cid, bool force_del=false);
   string     get_current_point()  { return currpoint; }
   int        set_current_point(ChipsId* cid, bool rec=true);
   ChipsPoint* get_point(const string& val="");

   // methods for region object interactions
   int        add_region(ChipsId* cid, double* xx, double* yy,
                         unsigned int count);
   int        add_region(ChipsId* cid, int sides, double xx, double yy,
                         double radius);
   int        delete_region(ChipsId* cid, bool force_del=false);
   string     get_current_region()  { return currregion; }
   int        set_current_region(ChipsId* cid, bool rec=true);
   ChipsRegion* get_region(const string& val="");

   // methods for coordinate system
   int         add_coordinate_system (ChipsAxis *ax, ChipsAxis *ay, Transform *transform, bool create_data_coord=true);
   int	       update_current_coord (ChipsId *cid);

   ChipsCoordinate *find_coordinate_system (ChipsId &cid, int &err_status);
   bool        get_coord_initialized () {return coord_initialized; };
   void        set_coord_initialized (bool val) { coord_initialized = val; };

   ChipsDataSystem* get_datasystem(ChipsAxis *ax, ChipsAxis *ay);
   int update_limits (const list <string> &axis_ids);
   int update_limits (const string &axis_id);
   int update_property (const list <string> &axis_ids, int property);
   int update_property (const string &axis_id, int property);

   // aspect ratio routines 
   bool        use_ratio()  { return aspect_use; }
   int         set_aspect_ratio(string val, bool rec=false); 
   string      get_aspect_ratio() { return aspect_ratio; } 
   float       get_aspect_width() { return aspect_w; }
   float       get_aspect_height() { return aspect_h; }
   double      get_plot_width(); 
   double      get_plot_height(); 
   double      get_scale_factor(int dim); 


   char*       info(ChipsId* cid, V_OBJECT_TYPE obj, bool current_obj=false);
   char*       info_coordinate ();
   char*       list_current();

   int         find_axis_min_max(ChipsAxis* axis, int dim,
                                 double* min, double* max, double* lmin,
                                 double* tmin, double* tmax,
                                 bool init=true, ChipsAxis* altax = NULL);

   ChipsObject* get_object(const ChipsId* cid, V_OBJECT_TYPE otype,
                            bool include_deref=false);
   int         get_plot_object_count()
               { return (curves.size() + contours.size() + 
                        histograms.size() + images.size()); }

   int         adjust_default_colors(); 
   int         update_print_scale (double scale);

   void        move_offscreen(bool rec=false);
   void        move_onscreen(bool rec=false);
   int         is_offscreen()       { return offscreen; }

   int         clear();

   virtual void update_parent (ChipsObject *cl) {parent = cl;};
   virtual int get_depth();
   virtual int redraw();
   virtual int move(COORD_SYS coord_sys, const string &coord_id, double xx,
                    double yy, int mode, bool rec=true);
   virtual int change_axis_style(); 
   virtual int change_visible ();

   virtual bool is_visible()  { return (visible && !(vis_override || offscreen)); }

   virtual int show ();
   virtual int hide ();
   virtual int show_current ();
   virtual int hide_current ();

   virtual bool empty (){return true;};

   virtual ChipsCurve* new_curve(long cnt, double* x, double* y,const string &transform_id, const string& cvid="");
   virtual ChipsCurve* new_curve(long cnt, double* x, double* y,
      double* ydown, double* yup, double* xdown, double* xup, double* ylims,
      double* xlims, const string &transform_id, const string& cvid="");
   virtual ChipsContour* new_contour(long cnt, double* x, double* y,
      double* z,  unsigned int count, double *levels, string transform_id,
      const string& ctid="");
   virtual ChipsHistogram* new_histogram(long cnt, double* bincnts, 
      double* lo, double* hi, double* yd, double* yu, 
      const string& hid="");
   virtual ChipsImage* new_image(ChipsStateStream& in);
   virtual ChipsImage* new_image(string fname, string transform_id, string iid="");
   virtual ChipsImage* new_image(string fname, float posx1, float posy1,
                                 float posx2, float posy2, string transform_id, string iid="");
   virtual ChipsImage* new_image(//WLMChipsCoordinate *inCoord, COORD_SYS sys,
                                 double* rvals, double *gvals, double *bvals,
                                 double *avals, int xaxis, int yaxis, 
                                 string transform_id, string iid="");
   virtual ChipsColorBar* new_colorbar(ChipsCoordinate *inCoord, COORD_SYS sys,
      double xpos, double ypos, string tid="");
   virtual ChipsColorBar* new_colorbar(ChipsCoordinate *inCoord, COORD_SYS sys,
      double xpos, double ypos, unsigned short cmap_id, string tid="");
   virtual ChipsColorBar* new_colorbar(ChipsStateStream& in);
   virtual ChipsLabel* new_label(ChipsCoordinate *inCoord, COORD_SYS sys,
      double xpos, double ypos, string text, string tid="");
   virtual ChipsLabel* new_label(ChipsStateStream& in);
   virtual ChipsLine* new_line(ChipsCoordinate *inCoord, COORD_SYS sys,
      double startx, double starty, double endx, double endy, string lnid="");
   virtual ChipsLine* new_line(ChipsCoordinate *inCoord, COORD_SYS sys,
      double* xx, double* yy, unsigned int count, string lnid="");
   virtual ChipsLine* new_line(ChipsStateStream& in);
   virtual ChipsPoint* new_point(ChipsCoordinate *inCoord, COORD_SYS sys,
      double xx, double yy, string ptid);
   virtual ChipsPoint* new_point(ChipsCoordinate *inCoord, COORD_SYS sys,
      double xx, double yy, double zz, string ptid);
   virtual ChipsPoint* new_point(ChipsStateStream& in);
   virtual ChipsRegion* new_polygon(ChipsCoordinate *inCoord, COORD_SYS sys,
      double* xx, double* yy, unsigned int count, string rid);
   virtual ChipsRegion* new_polygon(ChipsCoordinate *inCoord, COORD_SYS sys,
      int sides, double xx, double yy, double radius, string rid);
   virtual ChipsRegion* new_polygon(ChipsStateStream& in);


   virtual ChipsAxis* new_axis(ChipsCoordinate *inCoord, COORD_SYS sys,
                              double inMinX, double inMinY, double inMaxX,
                              double inMaxY,int inOrientation,
                              ChipsAxisAlgorithmFactory::TransformType type,
                              const string& aid="");

   virtual ChipsAxis* new_axis(ChipsCoordinate *inCoord, COORD_SYS sys,
			      double inMinX, double inMinY, double inMaxX,
                              double inMaxY,int inOrientation,
			      double inMin,double inMax, 
                              ChipsAxisAlgorithmFactory::TransformType type,
                              const string& aid="");
   virtual ChipsAxis* new_axis(ChipsStateStream& in);
   virtual ChipsCurve* new_curve(ChipsStateStream& in);
   virtual ChipsContour* new_contour(ChipsStateStream& in);
   virtual ChipsHistogram* new_histogram(ChipsStateStream& in);
   virtual ChipsLegend* new_legend(const char** cids, const char** clabs, 
      const string& lgid="");

   int adjust_corners(bool rec=true);
   int adjust_corner_style();
   int adjust_corner_widths(ChipsAxis* changed, bool rec=true); 

   virtual void serialize(ChipsStateStream& out);
   void         serialize(ChipsStateStream& out, bool write_coord);
   void         serialize_ascii(ChipsStateStream& out, int keep=1);
   void         serialize_axes(ChipsStateStream& out);
   void         serialize_objects(ChipsStateStream& out);
   void         serialize_objects_ascii(ChipsStateStream& out);
   int          serialize_currency_ascii(ChipsStateStream& out, int obcnt=1);

   virtual void deserialize(ChipsStateStream& in);
   int          deserialize_axis(ChipsStateStream& in);
   int          deserialize_image(ChipsStateStream& in);
   int          deserialize_curve(ChipsStateStream& in);
   int          deserialize_contour(ChipsStateStream& in);
   int          deserialize_histogram(ChipsStateStream& in);
   int          deserialize_annotations(ChipsStateStream& in);

   virtual int reference_object (ChipsObject *co, bool change_cnt, const string &curr="");
   virtual int dereference_object (ChipsObject *co, bool change_cnt, const string &curr="");
   virtual int clear_cmd (ChipsObject *co);
   int update_object_positions ();
   int reset_object_positions ();
   int update_title_position(bool redrawFlag=true);
   virtual int recalculate_position(); 
   int adjust_border_visibility();
   int update_depth_table(V_OBJECT_TYPE obj, string oid, int how, int pos=0);

   int    add_object_to_depth_table(ChipsObject* ob, bool rec=true);
   int    remove_object_from_depth_table(ChipsObject* ob);
   int    remove_plot_from_depth_table();
   int highlight_object(ChipsId* cid, V_OBJECT_TYPE obj, bool vis);

   int update_data_coord_limits (ChipsAxis *xaxis, double xmin, double xmax,
                              ChipsAxis *yaxis, double ymin, double ymax,
                              bool transformed);

   int update_axis_coordinates (ChipsAxis *ax, list<string> &ds_ids);
   void clone_data_systems (ChipsAxis *ax);

   void   log_updates();
   void   handle_updates(); 

   virtual void enable_picklimits (bool enabled) {};
   virtual void enable_addannotation (bool enabled) {};

   virtual void disable_all_interaction ();
   virtual void enable_all_interaction ();

  protected:

   virtual ~ChipsPlot();

   int         move_axis(ChipsId *cid, double xx, double yy, int mode);
   int         move_label(ChipsId* cid, double xx, double yy, int mode);
   int         move_line(ChipsId* cid, double xx, double yy, int mode);
   int         move_colorbar(ChipsId* cid, double xx, double yy, int mode);
   int         move_point(ChipsId* cid, double xx, double yy, int mode);
   int         move_region(ChipsId* cid, double xx, double yy, int mode);


   int    handle_new_xaxis(ChipsAxis* ax, bool change_cnt, ChipsId* cid=NULL);
   int    handle_new_yaxis(ChipsAxis* ay, bool change_cnt, ChipsId* cid=NULL);

   int    set_axis_mode_property (ChipsAxis *ax, void *value, int in_property);

   int    change_objects_visibility (); // change the plot objects visibility

   bool   check_axes_for_ratios(ChipsId* cid);
   void   adjust_plot_anchors();  // set aspect border locations based on margins 
   void   reset_title_defaults();
   int    check_for_matching_attribute_list(string bstr, string astr); 
   void   initialize_title();
   int    dereference_axis (ChipsAxis *ax);
   ChipsObject *deserialize_object (V_OBJECT_TYPE co_type,
                                                    ChipsStateStream& in);


   // calculate an axis limits after changing the limits on a plot object
   int calculate_limits ( ChipsAxis *ax[], double co_min[],
                          double co_max[], double co_lmin[],
                          bool axis_created[], int num_axes,
                          bool single_object);

   int update_data_coord_positions (ChipsAxis *xaxis, ChipsAxis *yaxis, list<string> &exclude_axes);
   void remove_data_coord (ChipsAxis *ax, ChipsAxis *ay);
   bool check_for_data_sys_objs (ChipsAxis *ax);
   void combine_axes_ascii_serialize(ChipsStateStream& out, ChipsAxis* &ax1, 
                                     ChipsAxis* &ax2);
   int        create_axes (Transform *transform, int &out_created, double *min=NULL, double *max=NULL); // creates axes if none exists for a new plot object
   int        setup_current_axes (Transform *transform, int &out_created);

   int change_dim (int dim_type, int adjust_dim);

   virtual void set_data_aspect_ratio (const ChipsId *cid) {};
 
   void disable_object_interaction (const string &oid, V_OBJECT_TYPE obj);
   void enable_object_interaction (const string &oid, V_OBJECT_TYPE obj);



   // previous pixel boundaries
   double     px0;           // plot lower left pixel x value
   double     px1;           // plot upper right pixel x value
   double     py0;           // plot lower left pixel y value
   double     py1;           // plot upper right pixel y value

   float  left_margin;
   float  right_margin;
   float  top_margin;
   float  bottom_margin;
   int    title_size;
   int    title_depth;
   float  title_xpos; 
   float  title_ypos;
   float  title_ypos_norm;
   float  title_xpos_norm;
   float  title_angle; 
   float  title_halign; 
   float  title_valign; 

   float  bx1;                          // aspect updated position 
   float  by1;                          // aspect updated position 
   float  bx2;                          // aspect updated position 
   float  by2;                          // aspect updated position 
   float  aspect_w;                     // width of aspect ratio
   float  aspect_h;                     // height of aspect ratio
   string aspect_ratio;                 // aspect ratio as "w:h"

   string title_color;
   string title_font;
   V_FONTSTYLE_TYPE title_fontstyle;
   V_AXISSTYLE_TYPE axis_style; 

   char*      create_curve_id(); 
   char*      create_contour_id(); 
   char*      create_histogram_id(); 
   char*      create_image_id(); 
   char*      create_cbar_id(); 
   char*      create_label_id(); 
   char*      create_line_id(); 
   char*      create_point_id(); 
   char*      create_region_id(); 
   char*      create_axis_id(int dim); 
   char*      create_legend_id(); 

   bool       curves_exist ();
   bool       images_exist ();
   bool       axes_x_exist ();
   bool       axes_y_exist ();
   bool       histograms_exist ();
   bool       contours_exist ();
   int        prescreen_contour(unsigned long num_pts, double* xx,
                                double*yy, double* zz);
   void       cover_box_axes(ChipsAxis *ax, bool rec=true);
   void       cover_box_axes(int dim = XY_AXIS, bool rec=true);

   ChipsAxisBind* bindings; 

   ChipsDepthControl* depth_table;

   map <string, ChipsCurve* >curves;          // map of this plot's curves
   map <string, ChipsContour* >contours;      // map of this plot's contours 
   map <string, ChipsHistogram* >histograms;  // map of this plot's histograms
   map <string, ChipsImage*>images;           // map of this plot's images
   map <string, ChipsColorBar*>cbars;     // map of this plot's colorbars
   map <string, ChipsLabel*>labels;           // map of this plot's labels
   map <string, ChipsLine*>lines;             // map of this plot's lines
   map <string, ChipsPoint*>points;           // map of this plot's points
   map <string, ChipsRegion*>regions;         // map of this plot's regions
   map <string, ChipsAxis* >axes_x;           // map of this plot's x axes
   map <string, ChipsAxis* >axes_y;           // map of this plot's y axes

   map <unsigned int, ChipsCurve* >dcurves;       // map of plot's curves
   map <unsigned int, ChipsContour* >dcontours;   // map of plot's contours
   map <unsigned int, ChipsHistogram* >dhistograms; // map of plot's histograms
   map <unsigned int, ChipsImage*>dimages;
   map <unsigned int, ChipsColorBar*>dcbars;
   map <unsigned int, ChipsLabel*>dlabels;
   map <unsigned int, ChipsLine*>dlines;
   map <unsigned int, ChipsPoint*>  dpoints;
   map <unsigned int, ChipsRegion*>  dregions;
   map <unsigned int, ChipsAxis* >daxes_x;  // x axes list
   map <unsigned int, ChipsAxis* >daxes_y;  // y axes list

   map <pair <string, string>, ChipsDataSystem*> data_systems;
   map <pair <string, string>, ChipsDataSystem*> deleted_data_systems;

   ChipsLabel* title;        // title of plot 
   ChipsAxis* bottom;        // plot area bottom axis
   ChipsAxis* top;           // plot area bottom axis
   ChipsAxis* left;          // plot area bottom axis
   ChipsAxis* right;         // plot area bottom axis

   ChipsAxis* cur_x;         // current x axis
   ChipsAxis* cur_y;         // current x axis
   bool       cur_xall;      // current set to all x axes 
   bool       cur_yall;      // current set to all y axes 
   bool       bound_x;       // has x border been initially bound 
   bool       bound_y;       // has x border been initially bound 
   bool       offscreen;     // split pushed off screen
   ChipsLegend* legend;      // plot's legend box 

   string     currcurv;      // current curve 
   string     currcont;      // current contour 
   string     currhist;      // current histogram 
   string     currimage;     // id of current image
   string     currcbar;      // id of current color bar
   string     currlabel;     // id of current label 
   string     currline;      // id of current line
   string     currpoint;     // id of current point
   string     currregion;    // id of current region
   bool       aspect_use;    // preserve aspect ratio?
   bool       initialized;   // have we initialized everything

   int        curve_cnt;     // ref count for curve ids
   int        contour_cnt;   // ref count for contour ids
   int        histogram_cnt; // ref count for histograms ids
   int        image_cnt;     // reference count for image id creation
   int        cbar_cnt;      // ref count for color bars
   int        label_cnt;     // ref count for label ids
   int        line_cnt;      // ref count for line ids 
   int        point_cnt;     // ref count for point ids 
   int        region_cnt;    // ref count for region ids 
   int        axis_x_cnt;    // ref count for x axes ids
   int        axis_y_cnt;    // ref count for y axes ids
   int        legend_cnt;    // ref count for legend ids

   V_CORNERSTYLE_TYPE  corner_style;   // corner joint of axes borders 

   string     current_lyr;   // current layer
   int        deserialize_title;  // do we need to deserialize title
   bool       deserialize_coord;  // do we need to deserialize coord
   bool       coord_initialized;  // all coordinate information initialized

};


#endif 
