// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSPLOTSYSTEM_HH
#define CHIPSPLOTSYSTEM_HH

class ChipsPlotSystem;

#include <list>
#include <string>
#include <map>
#include "ChipsObject.hh"
#include "ChipsDefines.hh"
#include "ChipsMatrix.hh"
#include "ChipsCoordinate.hh"

class ChipsDataSystem;

/**************************************************************
*
*    Plot System
*
****************************************************************/
class ChipsPlotSystem: public ChipsCoordinate
{
   public:
      /* Constructors */
      ChipsPlotSystem (ChipsCoordinate *par);
      ChipsPlotSystem (ChipsCoordinate *par,
                  const ChipsVector &min, const ChipsVector &max);
      ChipsPlotSystem (const ChipsPlotSystem &);
      ChipsPlotSystem (ChipsStateStream &in, ChipsCoordinate *par);
      virtual ~ChipsPlotSystem ();

      /* Overloaded operators */
      void operator = (const ChipsPlotSystem &);

      virtual int convert_cs_to_cs (const ChipsVector *in, ChipsVector *out, int size,
                                 COORD_SYS from_cs, const string &from_id,
                                 COORD_SYS to_cs, const string &to_id);
      virtual int add_coord_sys (ChipsCoordinate *);
      virtual int remove_coord_sys (const string &id);
      virtual ChipsCoordinate *get_coord (COORD_SYS coord_sys, const string &id="");
      ChipsDataSystem *get_data_coord (COORD_SYS coord_sys, const string &xaxis_id, const string &yaxis_id);
      virtual bool coord_exists (COORD_SYS coord_sys, const string &id);
      virtual void get_coord_list (list <string> &outList);

      virtual int insert (ChipsObject *co, bool use_co_info=true);
      virtual int remove (ChipsObject *co, bool use_co_info=true);


      /* Update plot position and move objects accordingly */
      int modify_position (double position_min, double position_max, int axis);

      virtual int update_obj_positions (bool update_all=true);

      void delete_all_data_systems ();
      void set_min_pos (const ChipsVector &min) {min_position = min;};
      void set_max_pos (const ChipsVector &max) {max_position = max;};

      virtual ChipsVector get_lower_bounds (){return min_position;};
      virtual ChipsVector get_upper_bounds (){return max_position;};
      virtual COORD_SYS   get_bounds_coord_sys () {return FRAME_NORM;};

      // create an id from all parent information
      virtual void create_id (string &curr_id);

      string get_old_id () {return this->old_id;};

      virtual int deserialize (ChipsStateStream &in);
      virtual void serialize (ChipsStateStream &out);


      /* These functions should be deprecated. Use get_lower/upper_bounds */
      ChipsVector get_min_pos () {return min_position;};
      ChipsVector get_max_pos () {return max_position;};

      int set_id (const string &id, bool update=true);

   protected:
      map <string, ChipsDataSystem *, less<string> > data_systems;
//      map <string, ChipsDataSystem *>::iterator curr_ds;

      string strip_transform_id (const string &data_id, bool old=false);

      ChipsVector min_position;               //Min x/y plot position
      ChipsVector max_position;               //Max x/y data coordinate

      bool remove_data;	

      unsigned long ref_id;               // reference id
      static unsigned long global_ref_id; // reference id
      string old_id;                      // the old id tag for backwards compat.

};


#endif
