// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSPOINT_HH
#define CHIPSPOINT_HH

#include <string>
#include <stdio.h>
#include "ChipsObject.hh"
#include "ChipsDefines.hh"
#include "ChipsStateStream.hh"
#include "ChipsAnnotation.hh"
#include "ChipsId.hh"

using namespace std;

class ChipsCommand;

class ChipsPoint: public ChipsAnnotation
{
  public:

   ChipsPoint(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys,
	      double  xx, double yy, string ptid);
   ChipsPoint(ChipsObject* par, ChipsStateStream& in);
   ChipsPoint(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys,
              ChipsStateStream& in);
   ChipsPoint(ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys,
              double  xx, double yy, double zz, string ptid);

   void       object_delete ();

   int        set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj);
   int        set_property(void* in_val, int in_property, bool rec=true);
   int        get_property(void* out_val, int property);

   int        get_size()    { return size; }
   int        set_size(int val);
   bool       get_filled()    { return filled; }
   int        set_filled(bool val);

   V_SYMBOL_TYPE get_symbolstyle()   { return style; }
   int        set_symbolstyle(V_SYMBOL_TYPE val);
   void       initialize(); 

   void       set_point_num (int val) {point_num = val;};
   int        get_point_num () {return point_num;};

   int  resize (COORD_SYS coord_sys, const string &coord_id,
                    double *bounds, bool rec=true);

   int        change_coordinate (COORD_SYS coordsys,
                                          const string& coordid);
   int        get_position (ChipsVector &pos);


   virtual int move(COORD_SYS coord_sys, const string &coord_id, double xx,
                    double yy, int mode, bool rec=true); 
   virtual int reset_position ();
   virtual int  set_depth(int newdepth);

   virtual int  compute_resize (double *bounds);

   virtual int redraw(); 
   virtual int change_position(); 
   virtual int change_style(); 
   virtual int change_filled(); 
   virtual int change_size(); 
   virtual int change_visible();
   virtual int change_depth();

   virtual int show ();
   virtual int hide ();


   virtual void serialize(ChipsStateStream& out);
   virtual void serialize_ascii(ChipsStateStream& out);
   virtual void deserialize(ChipsStateStream& in);

  protected:

   virtual ~ChipsPoint();
   bool add_command (ChipsCommand *inCmd, int &outStatus);

   ChipsVector pnt;          // coordinate point
   int        size;          // size of point 
   bool       filled;        // solid or hollow point 
   V_SYMBOL_TYPE style;      // type of point  
   int        point_num;       // this points order 0th, 1st, 2nd, etc.
};


#endif 
