// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsPrintDialog.hh"
#include "ChipsDefaults.hh"
#include "ChipsCmdManager.hh"

#include <fstream>
#include <iostream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

using namespace std;

extern ChipsDefaults* theDefaults;
extern ChipsCmdManager* cmd_manager;

const char *ChipsPrintDialog::extensions[6] = { ".ps", ".eps", ".pdf", ".png", ".jpg", ".jpeg"};
const int ChipsPrintDialog::jpeg_pos = 5;


ChipsPrintDialog::ChipsPrintDialog()
{
   add_extension = true;
   use_extended_jpg = false;
}

ChipsPrintDialog::~ChipsPrintDialog()
{
}

void ChipsPrintDialog::generate_form()
{
   string ss; 
   base_w = gtk_dialog_new_with_buttons (_title.c_str(), NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_PRINT, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   gtk_widget_set_size_request(base_w, 475, 300);
   _notebook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_notebook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_notebook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_notebook), GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _notebook);

   _tab_gen = gtk_table_new(8,13, true);

   _label[0] = gtk_label_new("Print to"); 
   _toggle[0] = gtk_radio_button_new_with_label(NULL, "File");
   _toggle[1] = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_toggle[0]),"Printer");
   g_signal_connect(_toggle[0],"toggled",G_CALLBACK(dest_toggle_callback),
                    (void*)this);
   g_signal_connect(_toggle[1],"toggled",G_CALLBACK(dest_toggle_callback),
                    (void*)this);

   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _label[0], 1,3,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _toggle[0], 4,6,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _toggle[1], 8,10,1,2);
   _label[1] = gtk_label_new("Filename");
   _file_w = gtk_entry_new();
   gtk_entry_set_activates_default(GTK_ENTRY(_file_w), TRUE);
   g_signal_connect(_file_w, "activate", G_CALLBACK(entry_activate_callback),
            (void*)this);

   _browse_w =  gtk_button_new_with_label("browse"); 
   g_signal_connect(_browse_w, "clicked", G_CALLBACK(browse_button_callback),
            (void*)this);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _label[1], 1,3,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _file_w, 4,9,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _browse_w, 10,12,2,3);
   _label[2] = gtk_label_new("Format");
   _format_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_format_w), 0, "ps");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_format_w), 1, "eps");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_format_w), 2, "pdf");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_format_w), 3, "png");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_format_w), 4, "jpg");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_format_w), 5, "jpeg");
   if (export_format == "ps")   
      gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), 0);
   else if (export_format == "eps")   
      gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), 1);
   else if (export_format == "pdf")   
      gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), 2);
   else if (export_format == "png")   
      gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), 3);
   else if (export_format == "jpg")   
      gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), 4);
   else if (export_format == "jpeg")   
      gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), 5);
   else
      gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), 0);

   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _label[2], 1,3,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _format_w, 4,7,3,4);
   g_signal_connect(_format_w, "changed", G_CALLBACK(format_toggle_callback),
            (void*)this);
   g_signal_connect(_file_w, "changed", G_CALLBACK(filename_value_callback),
            (void*)this);
   check_extension (_filename, true);

   _label[3] = gtk_label_new("Color scheme");
   _color_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_color_w), 0, "rgb");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_color_w), 1, "cmyk");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_color_w), 2, "cmyk separate");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_color_w), 3, "grayscale");
   if (export_scheme == chips_rgb) 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_color_w), 0);
   else if (export_scheme == chips_cmyk) 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_color_w), 1);
   else if (export_scheme == chips_cmyksep) 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_color_w), 2);
   else if (export_scheme == chips_grayscale)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_color_w), 3);
   else 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_color_w), 0);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _label[3], 1,4,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _color_w, 4,7,4,5);

   _label[15] = gtk_label_new("Image DPI");
   _export_dpi_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_export_dpi_w), 0, "72");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_export_dpi_w), 1, "150");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_export_dpi_w), 2, "300");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_export_dpi_w), 3, "600");
   if (export_dpi == 72)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_export_dpi_w), 0);
   else if (export_dpi == 150)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_export_dpi_w), 1);
   else if (export_dpi == 300)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_export_dpi_w), 2);
   else if (export_dpi == 600)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_export_dpi_w), 3);
   else
      gtk_combo_box_set_active(GTK_COMBO_BOX(_export_dpi_w), 0);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _label[15], 1,4,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _export_dpi_w, 4,7,5,6);


   _toggle[2] = gtk_check_button_new_with_label("Overwrite existing file");
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _toggle[2], 1,7,6,7);
   _toggle[3] = gtk_check_button_new_with_label("Export fonts to ps");
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _toggle[3], 1,7,7,8);

   if (!_filename.length()) 
   {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_toggle[1]), true); 
   }
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_toggle[2]), export_clobber); 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_toggle[3]), export_font); 

   _tab_fmt = gtk_table_new(8,8, true);
   _label[4] = gtk_label_new("Orientation"); 
   _toggle[4] = gtk_radio_button_new_with_label(NULL, "Portrait");
   _toggle[5] = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_toggle[4]),"Landscape");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_toggle[5]), export_orient); 

   GtkAdjustment* adjh = (GtkAdjustment *)gtk_adjustment_new(page_scale_height,0.0,1000.0,1.0,5.0,0.0);
   GtkAdjustment* adjw = (GtkAdjustment *)gtk_adjustment_new(page_scale_width,0.0,1000.0,1.0,5.0,0.0);
   _label[5] = gtk_label_new("scale height"); 
   _label[6] = gtk_label_new("scale width"); 
   _scaleh_w = gtk_spin_button_new(adjh, 100.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_scaleh_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_scaleh_w),
	       GTK_UPDATE_IF_VALID);
   _scalew_w = gtk_spin_button_new(adjw, 100.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_scalew_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_scalew_w),
	       GTK_UPDATE_IF_VALID);
   _toggle[6] = gtk_check_button_new_with_label("fit to page");
   g_signal_connect(_toggle[6],"toggled",G_CALLBACK(scale_toggle_callback),
                    (void*)this);
   _toggle[7] = gtk_check_button_new_with_label("keep aspect");

   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _label[4], 1,3,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _toggle[4], 3,5,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _toggle[5], 5,7,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _label[5], 1,4,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _scaleh_w, 4,6,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _label[6], 1,4,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _scalew_w, 4,6,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _toggle[6], 2,6,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_tab_fmt), _toggle[7], 2,6,5,6);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_toggle[6]), fit_to_page); 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_toggle[7]), keep_aspect); 

   _tab_mrgn = gtk_table_new(10,10, true);
   _label[11] = gtk_label_new("top");
   _adj[4] = (GtkAdjustment *)gtk_adjustment_new(topmargin,0.0,1000.0,1.0,5.0,0.0);
   _spin[0] = gtk_spin_button_new(_adj[4], 5.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_spin[0]), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_spin[0]), GTK_UPDATE_IF_VALID);
   _label[12] = gtk_label_new("bottom");
   _adj[5] = (GtkAdjustment *)gtk_adjustment_new(bottommargin,0.0,1000.0,1.0,5.0,0.0);
   _spin[1] = gtk_spin_button_new(_adj[5], 5.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_spin[1]), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_spin[1]), GTK_UPDATE_IF_VALID);
   _label[13] = gtk_label_new("left");
   _adj[6] = (GtkAdjustment *)gtk_adjustment_new(leftmargin,0.0,1000.0,1.0,5.0,0.0);
   _spin[2] = gtk_spin_button_new(_adj[6], 5.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_spin[2]), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_spin[2]), GTK_UPDATE_IF_VALID);
   _label[14] = gtk_label_new("right");
   _adj[7] = (GtkAdjustment *)gtk_adjustment_new(rightmargin,0.0,1000.0,1.0,5.0,0.0);
   _spin[3] = gtk_spin_button_new(_adj[7], 5.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_spin[3]), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_spin[3]), GTK_UPDATE_IF_VALID);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _label[11],4,6,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _spin[0],4,6,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _label[12],4,6,7,8);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _spin[1],4,6,8,9);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _label[13],1,3,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _spin[2],1,3,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _label[14],7,9,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _spin[3],7,9,5,6);
   _da_w = gtk_drawing_area_new();
   gtk_table_attach_defaults(GTK_TABLE(_tab_mrgn), _da_w,3,7,3,7);

   _tab_psz = gtk_table_new(8,8, true);
   _label[7] = gtk_label_new("page size"); 
   _ps_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 0, "default");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 1, "custom");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 2, "letter");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 3, "legal");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 4, "executive");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 5, "A3");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 6, "A4");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_ps_w), 7, "A5");
   const char* psz = v_pagesize_to_string(page_width, page_height, page_units);
   if (!psz)  
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 1);
   else if (!strcmp(psz, "custom"))
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 1);
   else if (!strcmp(psz, "letter"))
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 2);
   else if (!strcmp(psz, "legal"))
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 3);
   else if (!strcmp(psz, "executive"))
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 4);
   else if (!strcmp(psz, "A3"))
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 5);
   else if (!strcmp(psz, "A4"))
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 6);
   else if (!strcmp(psz, "A5"))
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 7);
   else 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_ps_w), 0);
   g_signal_connect(_ps_w,"changed",G_CALLBACK(page_value_callback),
                    (void*)this);
   _label[8] = gtk_label_new("page height"); 
   _adj[2] = (GtkAdjustment *)gtk_adjustment_new(page_height,0.0,1000.0,1.0,5.0,0.0);
   _ph_w = gtk_spin_button_new(_adj[2], 5.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_ph_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_ph_w), GTK_UPDATE_IF_VALID);
   _adj[3] = (GtkAdjustment *)gtk_adjustment_new(page_width,0.0,1000.0,1.0,5.0,0.0);
   _pw_w = gtk_spin_button_new(_adj[3], 5.0, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_pw_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_pw_w), GTK_UPDATE_IF_VALID);
   _label[9] = gtk_label_new("page width"); 
   _label[10] = gtk_label_new("page units"); 
   _pu_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_pu_w), 0, "cm");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_pu_w), 1, "mm");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_pu_w), 2, "inches");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_pu_w), 3, "pixels");
   if (page_units == chips_inch)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_pu_w), 2);
   else if (page_units == chips_cm)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_pu_w), 0);
   else if (page_units == chips_mm)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_pu_w), 1);
   else if (page_units == chips_pixels)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_pu_w), 3);
   else
      gtk_combo_box_set_active(GTK_COMBO_BOX(_pu_w), 2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _label[7], 1,3,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _ps_w, 3,5,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _label[8], 1,3,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _ph_w, 3,5,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _label[9], 1,3,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _pw_w, 3,5,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _label[10], 1,3,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_tab_psz), _pu_w, 3,5,4,5);

   GtkWidget *label = gtk_label_new("General");
   gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), _tab_gen, label),
   label = gtk_label_new("Format");
   gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), _tab_fmt, label),
   label = gtk_label_new("Margins");
   gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), _tab_mrgn, label),
   label = gtk_label_new("Page Size");
   gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), _tab_psz, label),

   set_fileopts_sensitive(); 
   set_scaleopts_sensitive(); 
   set_pageopts_sensitive(); 

   gtk_widget_show_all(base_w);
}

int ChipsPrintDialog::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int ChipsPrintDialog::doit(ChipsWindow* cw, string tname, string fname, bool dialog)
{
   if (!cw)  return 1; 
   _cw = cw;
   _title = tname; 
   _filename = fname; 
   load_preferences(); 

   check_extension (_filename, false);

   if (dialog)
   {
#ifndef HEADLESS
      generate_form();
      gtk_widget_grab_focus(_file_w);
      initialize_filename(); 
      if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
      {
         export_orient = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[5])); 
         export_font = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[3])); 
         export_clobber = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[2]));
         fit_to_page = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[6]));
         keep_aspect = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[7]));
         topmargin = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_spin[0]));
         bottommargin = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_spin[1]));
         leftmargin = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_spin[2]));
         rightmargin = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_spin[3]));
         page_scale_height = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_scaleh_w));
         page_scale_width = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_scalew_w));
         export_format = gtk_combo_box_get_active_text(GTK_COMBO_BOX(_format_w)); 
	 switch (gtk_combo_box_get_active(GTK_COMBO_BOX(_color_w)))
	 {
            case 0:    export_scheme = chips_rgb;        break; 
            case 1:    export_scheme = chips_cmyk;       break; 
            case 2:    export_scheme = chips_cmyksep;    break; 
            case 3:    export_scheme = chips_grayscale;  break; 
	    default:   export_scheme = chips_rgb;        break;  
	    break; 
	 }
         switch (gtk_combo_box_get_active(GTK_COMBO_BOX(_color_w)))
         {
            case 0:    export_dpi = 72;        break;
            case 1:    export_dpi = 150;       break;
            case 2:    export_dpi = 300;    break;
            case 3:    export_dpi = 600;  break;
            default:   export_dpi = 72;        break;
            break;
         }
	 int bal; 
	 switch (bal = gtk_combo_box_get_active(GTK_COMBO_BOX(_ps_w)))
	 {
	    case 0: 
	       page_width = page_height = 0;     
	    break;
	    case 1: 
	    {
	       page_height = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_ph_w)); 
	       page_width = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_pw_w)); 
	       string tunits = gtk_combo_box_get_active_text(GTK_COMBO_BOX(_pu_w));
	       string_to_v_units(tunits.c_str(), &page_units); 
	    }
	    break; 
	    case 2:
	    case 3:  // fallthrough intended 
	    case 4:  // fallthrough intended 
	    case 5:  // fallthrough intended 
	    case 6:  // fallthrough intended 
	    case 7:  // fallthrough intended 
	    {
	       string psize = gtk_combo_box_get_active_text(GTK_COMBO_BOX(_ps_w));
	       string_to_pagesize(psize.c_str(), page_width, page_height,
	           page_units); 
	    }
	    break; 
            default:
	       page_width = page_height = 0;
	    break; 
	 }
         // pass in empty string if we are dumping to the printer
         if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[0])))
            _filename = gtk_entry_get_text(GTK_ENTRY(_file_w));
         else 
            _filename = "";
         int rv = generate_print(1);
         gtk_widget_destroy(base_w);
         return rv; 
      }
      gtk_widget_destroy(base_w);
      return 0; 
#else
      cmd_manager->add_err ("Print dialog cannot be used when running in offscreen mode", ERR_STATUS_BAD, "", 0);
      return ERR_STATUS_BAD;
#endif
   }
   return generate_print();
}

int ChipsPrintDialog::generate_print(int dialog)
{
   _cw->set_export_orient(export_orient);
   _cw->set_export_clobber(export_clobber);
   _cw->set_export_font(export_font);
   _cw->set_page_lmargin(leftmargin);
   _cw->set_page_rmargin(rightmargin);
   _cw->set_page_tmargin(topmargin);
   _cw->set_page_bmargin(bottommargin);
   _cw->set_page_units(v_units_to_string(page_units));
   _cw->set_page_keep_aspect(keep_aspect);
   _cw->set_fit_to_page(fit_to_page); 
   _cw->set_page_scale_width(page_scale_width); 
   _cw->set_page_scale_height(page_scale_height); 
   _cw->set_page_width(page_width); 
   _cw->set_page_height(page_height); 
   _cw->set_export_dpi(export_dpi);
   _cw->set_printmethod (print_method);

   string outname; 
   int rval = _cw->print(_filename, export_format, export_scheme, &outname); 
   if (rval == 0 && dialog && outname.length() && !access(outname.c_str(), F_OK))
   {
      GtkRecentManager *recmgr = gtk_recent_manager_get_default();
      string recfile = "file://" + outname;
      gtk_recent_manager_add_item(recmgr, recfile.c_str());
   }
   return rval;  
}

void ChipsPrintDialog::load_preferences()
{
   export_font = theDefaults->get_exportfonts();
   export_clobber = theDefaults->get_exportclobber();
   export_orient = theDefaults->get_exportorient();
   export_format = theDefaults->get_exportformat();
   export_scheme = theDefaults->get_exportcolorscheme();
   leftmargin = theDefaults->get_pageleftmargin();
   rightmargin = theDefaults->get_pagerightmargin();
   topmargin = theDefaults->get_pagetopmargin();
   bottommargin = theDefaults->get_pagebottommargin();
   page_width = theDefaults->get_pagewidth();
   page_height = theDefaults->get_pageheight();
   page_units = theDefaults->get_pageunits();
   if (page_units == chips_nounit)
      page_units = theDefaults->get_windowunits();
   page_scale_width = theDefaults->get_pagescalewidth ();
   page_scale_height = theDefaults->get_pagescaleheight ();
   fit_to_page = theDefaults->get_fittopage();
   keep_aspect = theDefaults->get_pageaspect();
   export_dpi = theDefaults->get_exportdpi ();
   print_method = theDefaults->get_printmethod ();
}


int ChipsPrintDialog::dest_toggle_callback(GObject *object, gpointer user_data)
{
   ChipsPrintDialog* cmd = (ChipsPrintDialog*)user_data;
   if (!(cmd))  return 1;
   cmd->set_fileopts_sensitive(); 
   return 0;
}

void ChipsPrintDialog::set_fileopts_sensitive()
{
   bool sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[0]));
   gtk_widget_set_sensitive(GTK_WIDGET(_file_w), sens);
   gtk_widget_set_sensitive(GTK_WIDGET(_browse_w), sens);
   gtk_widget_set_sensitive(GTK_WIDGET(_format_w), sens);
   gtk_widget_set_sensitive(GTK_WIDGET(_color_w), sens);
   gtk_widget_set_sensitive(GTK_WIDGET(_toggle[2]), sens);
   gtk_widget_set_sensitive(GTK_WIDGET(_toggle[3]), sens);
}

int ChipsPrintDialog::scale_toggle_callback(GObject *object, gpointer user_data)
{
   ChipsPrintDialog* cmd = (ChipsPrintDialog*)user_data;
   if (!(cmd))  return 1;
   cmd->set_scaleopts_sensitive(); 
   return 0;
}

void ChipsPrintDialog::set_scaleopts_sensitive()
{
   bool sens = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_toggle[6]));
   gtk_widget_set_sensitive(GTK_WIDGET(_scaleh_w), sens);
   gtk_widget_set_sensitive(GTK_WIDGET(_scalew_w), sens);
}

int ChipsPrintDialog::page_value_callback(GObject *object, gpointer user_data)
{
   ChipsPrintDialog* cmd = (ChipsPrintDialog*)user_data;
   if (!(cmd))  return 1;
   cmd->set_pageopts_sensitive(); 
   return 0;
}

void ChipsPrintDialog::set_pageopts_sensitive()
{
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(_ps_w));

   switch (val)
   {
      case 1: 
         gtk_widget_set_sensitive(GTK_WIDGET(_ph_w), 1);
         gtk_widget_set_sensitive(GTK_WIDGET(_pw_w), 1);
         gtk_widget_set_sensitive(GTK_WIDGET(_pu_w), 1);
      break; 
      default:
         gtk_widget_set_sensitive(GTK_WIDGET(_ph_w), 0);
         gtk_widget_set_sensitive(GTK_WIDGET(_pw_w), 0);
         gtk_widget_set_sensitive(GTK_WIDGET(_pu_w), 0);
      break; 
   }
}

int ChipsPrintDialog::browse_button_callback(GObject *object, gpointer user_data)
{
   ChipsPrintDialog* cmd = (ChipsPrintDialog*)user_data;
   if (!(cmd))  return 1;
   GtkWidget* dialog = gtk_file_chooser_dialog_new("Browse Files", NULL,
      GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
      GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);

   string defpath = cmd->get_filename(); 
   if (defpath.length() && cmd->_is_directory(defpath))  defpath.append("/");
   gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog),defpath.c_str()); 
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
   {
      // statically allocated memory- don't free
      gchar * filename = (char*) 
         gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
      if (filename)   cmd->set_filename(filename);
   }
   gtk_widget_destroy (dialog);
   return 0;
}

int ChipsPrintDialog::filename_value_callback (GObject *object, gpointer user_data)
{
   ChipsPrintDialog* cmd = (ChipsPrintDialog*)user_data;
   GtkEntry* entry = (GtkEntry *)object;

   cmd->check_extension (gtk_entry_get_text (entry), true);

   return 0;
}

int ChipsPrintDialog::format_toggle_callback(GObject *object, gpointer user_data)
{
   GtkComboBox *combobox = (GtkComboBox *)object;
   ChipsPrintDialog* cmd = (ChipsPrintDialog*)user_data;

   int active = gtk_combo_box_get_active (combobox);
 
   if (cmd->use_extended_jpg)
   {
      active = jpeg_pos;
      cmd->use_extended_jpg = false;
   }
   if (active != -1)
      cmd->change_extension (extensions[active]);

   return 0;
}

void ChipsPrintDialog::set_filename(string fname)
{
   _filename = fname; 
   if (_file_w)
      gtk_entry_set_text(GTK_ENTRY(_file_w), _filename.c_str());
   gtk_editable_set_position(GTK_EDITABLE(_file_w),-1);
}

void ChipsPrintDialog::initialize_filename()
{
   if (_filename.length() == 0) return; 
   if (_is_directory(_filename))
   {
      char buf[512];
      int pos1, done = 0, cnt = 1; 
      int active = gtk_combo_box_get_active(GTK_COMBO_BOX(_format_w));

      while (!done && cnt < 10) 
      {
         if (_filename[(pos1 = _filename.length()-1)] == '/')
         {
            sprintf(buf,"%suntitled%1d%s", _filename.c_str(),cnt, extensions[active]); 
            pos1++; 
         }
         else 
         {
            sprintf(buf,"%s/untitled%1d%s", _filename.c_str(),cnt, extensions[active]); 
            pos1 += 2; 
         }
   
         if (access(buf, F_OK) != 0)
         {
            _filename = buf;
            gtk_entry_set_text(GTK_ENTRY(_file_w), _filename.c_str()); 
            gtk_editable_set_position(GTK_EDITABLE(_file_w),-1);
            gtk_editable_select_region(GTK_EDITABLE(_file_w), pos1, _filename.length());
            done = 1; 
         }
         else    
         {
            cnt ++; 
         }
      }
      if (!done) 
      {
         // couldn't find good filename- just use directory
         gtk_entry_set_text(GTK_ENTRY(_file_w), _filename.c_str()); 
         gtk_editable_set_position(GTK_EDITABLE(_file_w),-1);
      }
   }
   else 
   {
      gtk_entry_set_text(GTK_ENTRY(_file_w), _filename.c_str()); 
      gtk_editable_set_position(GTK_EDITABLE(_file_w),-1);
   }
}

bool ChipsPrintDialog::_is_directory(string path)
{
   struct stat   sbuf;
   mode_t        mode;

   if (!stat(path.c_str(), &sbuf))
      mode = sbuf.st_mode;
   else
      return 0;
   return (S_ISDIR(mode) && !(S_ISREG(mode) || S_ISCHR(mode) ||
           S_ISBLK(mode) || S_ISFIFO(mode)));
}

void ChipsPrintDialog::check_extension (string filename, bool update)
{
   // search for the extension
   int ext_pos = -1;
   int fmt = 0;
   if (filename.length () != 0)
      ext_pos = filename.find_last_of (".", filename.length ());

   add_extension = true;
   if ((ext_pos != string::npos) && (ext_pos+1 != filename.length ()))
   {
      // verify the extension is valid
      string filename_ext = filename.substr (ext_pos, filename.length ());
      for (int ii=0; ii<6; ii++)
      {
         if (filename_ext == extensions[ii])
         {
            export_format = filename_ext;
            add_extension = false;
            fmt = ii;

            break;
         }
      }

      if (!add_extension && update)
      {
         if (fmt >= jpeg_pos)
         {
            use_extended_jpg = (fmt == jpeg_pos);
            fmt--;
         }
         gtk_combo_box_set_active(GTK_COMBO_BOX(_format_w), fmt);
      }
   }  
}

void ChipsPrintDialog::change_extension (string extension)
{
   if (_file_w)
   {
      string filename = gtk_entry_get_text(GTK_ENTRY(_file_w));
      if (add_extension)
         filename.append (extension);
      else
      {
         int ext_pos = filename.find_last_of (".", filename.length ());
         filename = filename.substr (0, ext_pos);
         filename += extension;
      }

      set_filename (filename);
   }
}

int ChipsPrintDialog::entry_activate_callback(GObject *object, gpointer user_data)
{
   ChipsPrintDialog* cmd = (ChipsPrintDialog*)user_data;
   gtk_dialog_response(GTK_DIALOG(cmd->base_widget()), GTK_RESPONSE_OK);
   return 0;
}

         
         
         
