// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSPRINTDIALOG_HH
#define CHIPSPRINTDIALOG_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>

#include "ChipsDefines.hh"
#include "ChipsWindow.hh"

using namespace std;

class ChipsPrintDialog
{
  public:
   ChipsPrintDialog();
   ~ChipsPrintDialog();
   int show();
   int doit(ChipsWindow* cw, string tname, string fname, bool dialog); 

    GtkWidget*  base_widget()   { return base_w; }

  protected:
   void generate_form(); 
   void load_preferences();
   int  generate_print(int dialog=0);     
   void initialize_filename();

   static int dest_toggle_callback(GObject *object, gpointer user_data);
   static int scale_toggle_callback(GObject *object, gpointer user_data);
   static int page_value_callback(GObject *object, gpointer user_data);
   static int browse_button_callback(GObject *object, gpointer user_data);
   static int format_toggle_callback(GObject *object, gpointer user_data);
   static int filename_value_callback(GObject *object, gpointer user_data);
   static int entry_activate_callback(GObject *object, gpointer user_data);

   void set_fileopts_sensitive(); 
   void set_scaleopts_sensitive(); 
   void set_pageopts_sensitive(); 
   void set_filename(string fname); 
   string get_filename()    { return _filename; }
   bool _is_directory(string path); 

   void check_extension (string extension, bool update);
   void change_extension (string extension);



   GtkWidget *base_w;   
   GtkWidget* _scrolled; 
   GtkWidget* _notebook; 
   GtkWidget* _tab_gen;
   GtkWidget* _tab_fmt;
   GtkWidget* _tab_mrgn;
   GtkWidget* _tab_psz;

   GtkAdjustment* _adj[8];
   GtkWidget* _label[16]; 
   GtkWidget* _toggle[8]; 
   GtkWidget* _spin[4]; 
   GtkWidget* _file_w; 
   GtkWidget* _browse_w; 
   GtkWidget* _format_w; 
   GtkWidget* _color_w;
   GtkWidget* _export_dpi_w;
   GtkWidget* _scaleh_w; 
   GtkWidget* _scalew_w; 
   GtkWidget* _ps_w;
   GtkWidget* _ph_w;
   GtkWidget* _pw_w;
   GtkWidget* _pu_w;
   GtkWidget* _da_w; 

   ChipsWindow* _cw;
   string     _title;             // title of dialog  
   string     _filename;          // name of file to display  

   bool       export_clobber;     // overwrite file if it exists 
   bool       export_font; // export font to output ps file
   int        export_orient; // portrait == 0, landscape == 1
   V_PRINTFILE_TYPE export_scheme; // export color scheme
   string     export_format;
   // page info
   float      leftmargin;
   float      rightmargin;
   float      topmargin;
   float      bottommargin;
   float      page_width;
   float      page_height;
   V_UNITS_TYPE page_units;
   float      page_scale_width;
   float      page_scale_height;
   bool       fit_to_page;
   bool       keep_aspect;
   int        export_dpi;
   bool       add_extension;
   bool       use_extended_jpg;
   string     print_method;
   
   static     const char *extensions[6];
   static     const int jpeg_pos;
   
};

#endif 
