// 
//  Copyright (C) 2004,2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSREGION_HH
#define CHIPSREGION_HH

#include <string>
#include <iostream>
#include <stdio.h>
#include "ChipsObject.hh"
#include "ChipsDefaults.hh"
#include "ChipsLabel.hh"
#include "ChipsColor.hh"
#include "ChipsStateStream.hh"

using namespace std;

class ChipsCommand;

class ChipsRegion: public ChipsObject
{
  public:

   ChipsRegion(ChipsObject* par, ChipsCoordinate *inCoord, 
                COORD_SYS coord_sys, string val);
   ChipsRegion(ChipsObject* par, ChipsStateStream& in);

   void       object_delete ();

   virtual int change_coordinate (COORD_SYS coordsys,
                                          const string& coordid);
   virtual int get_position (ChipsVector &pos);

   int        set_cascading_property(ChipsId* cid, V_OBJECT_TYPE obj);
   int        set_property(void* in_val, int in_property, bool rec=true);
   int        get_property(void* out_val, int property);

   void       initialize();

   // methods to handle angle settings 
   float      get_angle()    { return angle; }
   int        set_angle(float val);

   double     get_opacity()  { return opacity; }
   int        set_opacity(double in_val);

   // methods to handle region border/edge colors 
   string     get_color()    { return color.get_name(); }
   int        set_color(string);

   // methods to handle region fill colors 
   string     get_fillcolor()    { return fillcolor.get_name(); }
   int        set_fillcolor(string);

   // methods for border/edge line styles 
   V_LINE_TYPE get_linestyle()        { return linestyle; }
   int        set_linestyle(V_LINE_TYPE val);
   int        set_linestyle(string val);

   // methods for fill styles 
   V_FILL_TYPE get_fillstyle()        { return fillstyle; }
   int        set_fillstyle(V_FILL_TYPE val, bool record=false);
   int        set_fillstyle(string val);
   int        set_fillstyle(unsigned char* bpattern); 

   // methods for label interactions 
   ChipsLabel* get_label()   { return label; } 
   string     get_label_text();
   int        set_label_text(string in_val);

   // methods for region border/edge thickness 
   int        set_thickness(float val);
   float      get_thickness()   { return thickness; }

   // methods for region inclusion/exclusion
   void       set_include(bool in_val)  { include = in_val; }
   bool       get_include()  {  return include; }

   // methods for region source/background specification
   void       set_source(bool in_val)  { source = in_val; }
   bool       get_source()  {  return source; }

   void       set_region_num (int val) {region_num = val;};
   int        get_region_num () {return region_num;};


   bool       check_degenerate();

   virtual int  set_depth(int newdepth);
   virtual int move(COORD_SYS coord_sys, const string &coord_id, double xx,
                    double yy, int mode, bool rec=true);
   virtual int reset_position ();

   virtual int change_position();
   virtual int change_color();
   virtual int change_opacity();
   virtual int change_fillcolor();
   virtual int change_rotation(); 
   virtual int change_thickness(); 
   virtual int change_linestyle(); 
   virtual int change_fillstyle(); 
   virtual int change_visible(); 
   virtual int change_depth();

   virtual int hide();
   virtual int show();
   virtual int redraw();
   virtual int update_data();

   virtual int update_clipping ();

   virtual void serialize(ChipsStateStream& out);
   virtual void deserialize(ChipsStateStream& in);

  protected:

   virtual ~ChipsRegion();

   bool check_segment_intersection(ChipsVector& pnta, ChipsVector& pntb,
                                   ChipsVector& pntc, ChipsVector& pntd);
   bool add_command (ChipsCommand *inCmd, int &outStatus);

   ChipsVector* pnts;        // coordinate points
   ChipsVector center;       // center of the region 
   ChipsVector lower_bounds; // min value
   ChipsVector upper_bounds; // max value

   unsigned char fillpattern[128]; // fill pattern 
   unsigned int num_pnts;    // total number of points in polygon

   ChipsLabel* label;        // label for region 
   double     opacity;       // alpha channel 
   float      angle;         // angle or rotation
   ChipsColor color;         // color of edges 
   ChipsColor fillcolor;     // color to fill/shade with
   V_LINE_TYPE linestyle;    // style of lines/edges
   V_FILL_TYPE fillstyle;    // style of fill pattern 
   float      thickness;     // thickness of lines/edges
   int        region_num;    // this region's order 0th, 1st, 2nd, etc.
   bool       source;        // source or background region
   bool       include;       // inclusion or exclusion
   bool       initialized;   // have we initialized everything

   bool       center_set;    // the region was created with a center defined
};

#endif 
