// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsRemoteMod.hh"
#include <stdio.h>

ChipsRemoteMod::ChipsRemoteMod(bool use_server, bool load_prefs):ChipsRemote(use_server, load_prefs)
{
}

ChipsRemoteMod::ChipsRemoteMod(const string& inServerId, bool load_prefs):ChipsRemote(inServerId, load_prefs)
{
}

ChipsRemoteMod::ChipsRemoteMod(bool load_prefs):ChipsRemote(load_prefs)
{
}


ChipsRemoteMod::~ChipsRemoteMod()
{
}

int ChipsRemoteMod::bind_axes(string aid1, string aid2)
{
   ChipsId tmp;
   string tlid = "\0";
   tmp.reset_window_id(tlid);
   tmp.set_axis_id(aid1);
   return ChipsRemote::bind_axes(&tmp, tlid, aid2);
}

int ChipsRemoteMod::bind_axes(string lid1, string aid1, string lid2,string aid2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(lid1);
   tmp.set_axis_id(aid1);
   return ChipsRemote::bind_axes(&tmp, lid2, aid2);
}

int ChipsRemoteMod::unbind_axes(string aid1, string aid2)
{
   ChipsId tmp;
   string tlid = "\0";
   tmp.reset_window_id(tlid);
   tmp.set_axis_id(aid1);
   return ChipsRemote::unbind_axes(&tmp, tlid, aid2);
}

int ChipsRemoteMod::unbind_axes(string lid1, string aid1, string lid2,string aid2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(lid1);
   tmp.set_axis_id(aid1);
   return ChipsRemote::unbind_axes(&tmp, lid2, aid2);
}

int ChipsRemoteMod::set_cascading_property(string idstr, V_OBJECT_TYPE start, string property, string value) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_object_id(start, idstr);
   return ChipsRemote::set_cascading_property(&tmp, start,property,value);
}

int ChipsRemoteMod::set_current_window(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::set_current_window(&tmp);
}

int ChipsRemoteMod::set_window_title(string idstr, string title) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::set_window_title(&tmp, title);
}

int ChipsRemoteMod::set_window_bgcolor(string idstr, string in_color) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::set_window_bgcolor(&tmp, in_color);
}

int ChipsRemoteMod::set_window_redraw(string idstr, bool draw) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::set_window_redraw(&tmp, draw);
}

int ChipsRemoteMod::set_frame_bgcolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::set_frame_bgcolor(&tmp, in_color);
}

int ChipsRemoteMod::set_frame_border_color(string idstr, string in_color) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::set_frame_border_color(&tmp, in_color);
}

int ChipsRemoteMod::set_default_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::set_default_depth(&tmp, sz);
}

int ChipsRemoteMod::set_framemode(string idstr, int mode)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::set_framemode(&tmp, mode);
}

int ChipsRemoteMod::set_frame_transparency(string idstr, bool seethrough)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::set_frame_transparency(&tmp, seethrough);
}

int ChipsRemoteMod::set_frame_scale(string idstr, bool val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::set_frame_scale(&tmp, val);
}

int ChipsRemoteMod::display_frame(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::display_frame(&tmp);
}

int ChipsRemoteMod::hide_frame(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::hide_frame(&tmp);
}

int ChipsRemoteMod::display_depth(string idstr, int depth)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::display_depth(&tmp, depth);
}

int ChipsRemoteMod::hide_depth(string idstr, int depth)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::hide_depth(&tmp, depth);
}

int ChipsRemoteMod::set_current_frame(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::set_current_frame(&tmp);
}

int ChipsRemoteMod::set_plot_title(string idstr, string in_title)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title(&tmp, in_title);
}

int ChipsRemoteMod::set_plot_title_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_color(&tmp, in_color);
}

int ChipsRemoteMod::set_plot_title_angle(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_angle(&tmp, in_val);
}

int ChipsRemoteMod::set_plot_title_halign(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_halign(&tmp, in_val);
}

int ChipsRemoteMod::set_plot_title_valign(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_valign(&tmp, in_val);
}

int ChipsRemoteMod::set_plot_title_size(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_size(&tmp, sz);
}

int ChipsRemoteMod::set_plot_title_font(string idstr, string font)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_font(&tmp, font);
}

int ChipsRemoteMod::set_plot_title_fontstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_fontstyle(&tmp, style);
}

int ChipsRemoteMod::set_plot_title_xpos(string idstr, float in_norm)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_xpos(&tmp, in_norm);
}

int ChipsRemoteMod::set_plot_title_ypos(string idstr, float in_norm)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_ypos(&tmp, in_norm);
}

int ChipsRemoteMod::set_plot_xlabel(string idstr, string in_lab)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_xlabel(&tmp, in_lab);
}

int ChipsRemoteMod::set_plot_ylabel(string idstr, string in_lab)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_ylabel(&tmp, in_lab);
}

int ChipsRemoteMod::set_plot_axisstyle(string idstr, string style) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_axisstyle(&tmp, style);
}

int ChipsRemoteMod::set_plot_axisstyle(string idstr, V_AXISSTYLE_TYPE style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_axisstyle(&tmp, style);
}

int ChipsRemoteMod::set_plot_cornerstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_cornerstyle(&tmp, style);
}

int ChipsRemoteMod::set_plot_cornerstyle(string idstr, V_CORNERSTYLE_TYPE style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_cornerstyle(&tmp, style);
}

int ChipsRemoteMod::set_plot_leftmargin(string idstr, float margin_sz) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_leftmargin(&tmp, margin_sz);
}

int ChipsRemoteMod::set_plot_rightmargin(string idstr, float margin_sz) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_rightmargin(&tmp, margin_sz);
}

int ChipsRemoteMod::set_plot_topmargin(string idstr, float margin_sz) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_topmargin(&tmp, margin_sz);
}

int ChipsRemoteMod::set_plot_bottommargin(string idstr, float margin_sz) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_bottommargin(&tmp, margin_sz);
}

int ChipsRemoteMod::set_plot_title_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_title_depth(&tmp, sz);
}

int ChipsRemoteMod::set_axis_scalable(string idstr, bool scalable) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_scalable(&tmp, scalable);
}

int ChipsRemoteMod::hide_plot(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::hide_plot(&tmp);
}

int ChipsRemoteMod::display_plot(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::display_plot(&tmp);
}

int ChipsRemoteMod::set_current_plot(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_current_plot(&tmp);
}

int ChipsRemoteMod::set_arbitrary_tick_positions(string idstr, unsigned int arraysize, double* pos, string* vals) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_arbitrary_tick_positions(&tmp, arraysize, pos, vals);
}

int ChipsRemoteMod::set_axis_major_mode(string idstr, string in_mode) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_mode(&tmp, in_mode);
}

int ChipsRemoteMod::set_axis_major_thickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_thickness(&tmp, in_thickness);
}

int ChipsRemoteMod::set_axis_major_style(string idstr, string in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_style(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_major_style(string idstr, int in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_style(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_minor_mode(string idstr, string in_mode)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_mode(&tmp, in_mode);
}

int ChipsRemoteMod::set_axis_minor_thickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_thickness(&tmp, in_thickness);
}

int ChipsRemoteMod::set_axis_minor_style(string idstr, string in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_style(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_minor_style(string idstr, int in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_style(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_parallel_offset(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_parallel_offset(&tmp, in_val);
}

int ChipsRemoteMod::get_axis_parallel_offset(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_parallel_offset(&tmp, out_val);
}

int ChipsRemoteMod::set_axis_perpendicular_offset(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_perpendicular_offset(&tmp, in_val);
}

int ChipsRemoteMod::get_axis_perpendicular_offset(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_perpendicular_offset(&tmp, out_val);
}

int ChipsRemoteMod::set_axis_automin(string idstr, bool on) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_automin(&tmp, on);
}

int ChipsRemoteMod::set_axis_automax(string idstr, bool on) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_automax(&tmp, on);
}

int ChipsRemoteMod::set_axis_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_depth(&tmp, sz);
}

int ChipsRemoteMod::set_axis_pad(string idstr, double in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_pad(&tmp, in_val);
}

int ChipsRemoteMod::set_axis_label_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_color(&tmp, in_color);
}

int ChipsRemoteMod::set_axis_label_size(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_size(&tmp, sz);
}

int ChipsRemoteMod::set_axis_label_angle(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_angle(&tmp, sz);
}

int ChipsRemoteMod::set_axis_xlabel_angle(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_xlabel_angle(&tmp, sz);
}

int ChipsRemoteMod::set_axis_ylabel_angle(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ylabel_angle(&tmp, sz);
}

int ChipsRemoteMod::set_axis_label_halign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_halign(&tmp, sz);
}

int ChipsRemoteMod::set_axis_label_valign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_valign(&tmp, sz);
}

int ChipsRemoteMod::set_axis_label_font(string idstr, string font)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_font(&tmp, font);
}

int ChipsRemoteMod::set_axis_label_fontstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_fontstyle(&tmp, style);
}

int ChipsRemoteMod::set_axis_label_visible(string idstr, bool vis)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_label_visible(&tmp, vis);
}

int ChipsRemoteMod::set_axis_ticklabel_style(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_style(&tmp, style);
}

int ChipsRemoteMod::set_axis_ticklabel_style(string idstr, int in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_style(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_ticklabel_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_color(&tmp, in_color);
}

int ChipsRemoteMod::set_axis_ticklabel_size(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_size(&tmp, sz);
}

int ChipsRemoteMod::set_axis_ticklabel_angle(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_angle(&tmp, sz);
}

int ChipsRemoteMod::set_axis_ticklabel_halign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_halign(&tmp, sz);
}

int ChipsRemoteMod::set_axis_ticklabel_valign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_valign(&tmp, sz);
}

int ChipsRemoteMod::set_axis_ticklabel_font(string idstr, string font)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_font(&tmp, font);
}

int ChipsRemoteMod::set_axis_ticklabel_fontstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_fontstyle(&tmp, style);
}

int ChipsRemoteMod::set_axis_ticklabel_offset(string idstr, int in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_offset(&tmp, in_val);
}

int ChipsRemoteMod::get_axis_ticklabel_offset(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_offset(&tmp, out_val);
}

int ChipsRemoteMod::set_axis_ticklabel_visible(string idstr, bool vis)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ticklabel_visible(&tmp, vis);
}

int ChipsRemoteMod::set_axis_majorgrid_visible(string idstr, bool vis)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_majorgrid_visible(&tmp, vis);
}

int ChipsRemoteMod::set_axis_majorgrid_thickness(string idstr, float in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_majorgrid_thickness(&tmp, in_size);
}

int ChipsRemoteMod::set_axis_majorgrid_style(string idstr, V_LINE_TYPE in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_majorgrid_style(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_majorgrid_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_majorgrid_color(&tmp, in_color);
}

int ChipsRemoteMod::set_axis_minorgrid_visible(string idstr, bool vis)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minorgrid_visible(&tmp, vis);
}

int ChipsRemoteMod::set_axis_minorgrid_thickness(string idstr, float in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minorgrid_thickness(&tmp, in_size);
}

int ChipsRemoteMod::set_axis_minorgrid_style(string idstr, V_LINE_TYPE in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minorgrid_style(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_minorgrid_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minorgrid_color(&tmp, in_color);
}

int ChipsRemoteMod::display_axis(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::display_axis(&tmp);
}

int ChipsRemoteMod::hide_axis(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::hide_axis(&tmp);
}

int ChipsRemoteMod::reverse_axes(string idstr, int dim)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::reverse_axes(&tmp, dim);
}

int ChipsRemoteMod::display_major_ticks(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::display_major_ticks(&tmp);
}

int ChipsRemoteMod::hide_major_ticks(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::hide_major_ticks(&tmp);
}

int ChipsRemoteMod::display_minor_ticks(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::display_minor_ticks(&tmp);
}

int ChipsRemoteMod::hide_minor_ticks(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::hide_minor_ticks(&tmp);
}


int ChipsRemoteMod::set_curve_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_depth(&tmp, sz);
}

int ChipsRemoteMod::set_curve_symbolstyle(string idstr, V_SYMBOL_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_symboltype(&tmp, val);
}

int ChipsRemoteMod::set_curve_symbolfill(string idstr, bool in_fill)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_symbolfill(&tmp, in_fill);
}

int ChipsRemoteMod::set_curve_symbolsize(string idstr, int in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_symbolsize(&tmp, in_size);
}

int ChipsRemoteMod::set_curve_symbolangle(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_symbolangle(&tmp, in_val);
}

int ChipsRemoteMod::set_curve_symbolcolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_symbolcolor(&tmp, in_color);
}

int ChipsRemoteMod::set_curve_linestyle(string idstr, V_LINE_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_linestyle(&tmp, val);
}

int ChipsRemoteMod::set_curve_linethickness(string idstr, float in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_linethickness(&tmp, in_size);
}

int ChipsRemoteMod::set_curve_linecolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_linecolor(&tmp, in_color);
}

int ChipsRemoteMod::set_curve_baddatamode(string idstr, V_BADDATAMODE_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_baddatamode(&tmp, val);
}

int ChipsRemoteMod::set_curve_errstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errstyle(&tmp, style);
}

int ChipsRemoteMod::set_curve_errup(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errup(&tmp, val);
}

int ChipsRemoteMod::set_curve_errdown(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errdown(&tmp, val);
}

int ChipsRemoteMod::set_curve_errleft(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errleft(&tmp, val);
}

int ChipsRemoteMod::set_curve_errright(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errright(&tmp, val);
}

int ChipsRemoteMod::set_curve_errthickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errthickness(&tmp, in_thickness);
}

int ChipsRemoteMod::set_curve_errcolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errcolor(&tmp, in_color);
}

int ChipsRemoteMod::set_curve_limitoverride(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_limitoverride(&tmp, val);
}

int ChipsRemoteMod::set_curve_limitlength(string idstr, double in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_limitlength(&tmp, in_val);
}

int ChipsRemoteMod::hide_curve(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::hide_curve(&tmp);
}

int ChipsRemoteMod::display_curve(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::display_curve(&tmp);
}

int ChipsRemoteMod::set_current_curve(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_current_curve(&tmp);
}

int ChipsRemoteMod::hide_image(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::hide_image(&tmp);
}

int ChipsRemoteMod::display_image(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::display_image(&tmp);
}

int ChipsRemoteMod::set_image_colormap(string idstr, V_COLORMAP_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_colormap(&tmp, val);
}

int ChipsRemoteMod::set_image_invert_colormap(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_invert_colormap(&tmp, val);
}

int ChipsRemoteMod::set_image_colormap_interpolate(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_colormap_interpolate(&tmp, val);
}

int ChipsRemoteMod::set_image_scale_channels(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_scale_channels(&tmp, val);
}

int ChipsRemoteMod::set_image_colormap_size(string idstr, int val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_colormap_size(&tmp, val);
}

int ChipsRemoteMod::set_image_interpolation(string idstr, V_INTERPOLATION_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_interpolation(&tmp, val);
}

int ChipsRemoteMod::set_image_alpha(string idstr, double min, double max)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_alpha(&tmp, min, max);
}

int ChipsRemoteMod::set_image_threshold(string idstr, double min, double max)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_threshold(&tmp, min, max);
}

int ChipsRemoteMod::set_image_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_depth(&tmp, sz);
}

int ChipsRemoteMod::set_contour_mode(string idstr, string mode) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_mode(&tmp, mode);
}

int ChipsRemoteMod::set_contour_style(string idstr, string style) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_style(&tmp, style);
}

int ChipsRemoteMod::set_contour_color(string idstr, string in_color) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_color(&tmp, in_color);
}

int ChipsRemoteMod::set_contour_interval(string idstr, double in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_interval(&tmp, in_val);
}

int ChipsRemoteMod::set_contour_numlevels(string idstr, int in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_numlevels(&tmp, in_val);
}


int ChipsRemoteMod::set_contour_levels(string idstr, unsigned int arraysize,double* pos)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_levels(&tmp, arraysize, pos);
}

int ChipsRemoteMod::set_contour_thickness(string idstr, float in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_thickness(&tmp, in_size);
}

int ChipsRemoteMod::set_current_contour(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_current_contour(&tmp);
}

int ChipsRemoteMod::set_contour_algorithm(string idstr, V_CONTOUR_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_algorithm(&tmp, val);
}

int ChipsRemoteMod::set_contour_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_depth(&tmp, sz);
}
/*
DEPRECATED
int ChipsRemoteMod::set_contour_transform(string idstr, Transform *trans)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::set_contour_transform(&tmp, trans);
}


int ChipsRemoteMod::get_contour_transform(string idstr, Transform **trans)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_transform(&tmp, trans);
}


int ChipsRemoteMod::apply_contour_transform(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::apply_contour_transform(&tmp);
}

int ChipsRemoteMod::apply_contour_transform_inverse(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::apply_contour_transform_inverse(&tmp);
}
*/
int ChipsRemoteMod::hide_contour(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::hide_contour(&tmp);
}

int ChipsRemoteMod::display_contour(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::display_contour(&tmp);
}

int ChipsRemoteMod::set_histogram_symbolstyle(string idstr, V_SYMBOL_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_symboltype(&tmp, val);
}

int ChipsRemoteMod::set_histogram_symbolfill(string idstr, bool in_fill)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_symbolfill(&tmp, in_fill);
}

int ChipsRemoteMod::set_histogram_symbolsize(string idstr, int in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_symbolsize(&tmp, in_size);
}

int ChipsRemoteMod::set_histogram_symbolangle(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_symbolangle(&tmp, in_val);
}

int ChipsRemoteMod::set_histogram_symbolcolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_symbolcolor(&tmp, in_color);
}

int ChipsRemoteMod::set_histogram_linestyle(string idstr, V_LINE_TYPE val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_linestyle(&tmp, val);
}

int ChipsRemoteMod::set_histogram_linethickness(string idstr, float in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_linethickness(&tmp, in_size);
}

int ChipsRemoteMod::set_histogram_linecolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_linecolor(&tmp, in_color);
}

int ChipsRemoteMod::set_histogram_errstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_errstyle(&tmp, style);
}

int ChipsRemoteMod::set_histogram_errup(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_errup(&tmp, val);
}

int ChipsRemoteMod::set_histogram_errdown(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_errdown(&tmp, val);
}

int ChipsRemoteMod::set_histogram_dropline(string idstr, bool val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_dropline(&tmp, val);
}

int ChipsRemoteMod::set_histogram_errthickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_errthickness(&tmp, in_thickness);
}

int ChipsRemoteMod::set_histogram_errcolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_errcolor(&tmp, in_color);
}

int ChipsRemoteMod::set_histogram_fillopacity(string idstr, double in_opacity)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_fillopacity(&tmp, in_opacity);
}

int ChipsRemoteMod::set_histogram_fillcolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_fillcolor(&tmp, in_color);
}

int ChipsRemoteMod::hide_histogram(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::hide_histogram(&tmp);
}

int ChipsRemoteMod::display_histogram(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::display_histogram(&tmp);
}

int ChipsRemoteMod::set_current_histogram(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_current_histogram(&tmp);
}

int ChipsRemoteMod::set_histogram_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_depth(&tmp, sz);
}

int ChipsRemoteMod::set_axis_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_color(&tmp, in_color);
}

int ChipsRemoteMod::set_axis_major_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_color(&tmp, in_color);
}

int ChipsRemoteMod::set_axis_minor_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_color(&tmp, in_color);
}

int ChipsRemoteMod::set_axis_text(string idstr, string in_text)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_text(&tmp, in_text);
}

int ChipsRemoteMod::set_axis_xlabel_text(string idstr, string in_text)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_xlabel_text(&tmp, in_text);
}

int ChipsRemoteMod::set_axis_ylabel_text(string idstr, string in_text)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_ylabel_text(&tmp, in_text);
}

int ChipsRemoteMod::set_axis_tickformat(string idstr, string in_format)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_tickformat(&tmp, in_format);
}

int ChipsRemoteMod::set_axis_thickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_thickness(&tmp, in_thickness);
}

int ChipsRemoteMod::set_axis_tickstyle(string idstr, int in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_tickstyle(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_tickstyle(string idstr, string in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_tickstyle(&tmp, in_style);
}

int ChipsRemoteMod::set_axis_major_num(string idstr, int in_num)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_num(&tmp, in_num);
}

int ChipsRemoteMod::set_axis_minor_num(string idstr, int in_num)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_num(&tmp, in_num);
}

int ChipsRemoteMod::set_axis_major_length(string idstr, int in_len)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_length(&tmp, in_len);
}

int ChipsRemoteMod::set_axis_minor_length(string idstr, int in_len)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_length(&tmp, in_len);
}

int ChipsRemoteMod::set_axis_major_interval(string idstr, double in_len)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_major_interval(&tmp, in_len);
}

int ChipsRemoteMod::set_axis_minor_interval(string idstr, double in_len)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_axis_minor_interval(&tmp, in_len);
}

int ChipsRemoteMod::set_current_axis(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::set_current_axis(&tmp);
}

int ChipsRemoteMod::set_current_image(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_current_image(&tmp);
}

int ChipsRemoteMod::set_current_colorbar(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_current_colorbar(&tmp);
}

int ChipsRemoteMod::hide_colorbar(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::hide_colorbar(&tmp);
}

int ChipsRemoteMod::display_colorbar(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::display_colorbar(&tmp);
}

int ChipsRemoteMod::set_colorbar_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_depth(&tmp, sz);
}

int ChipsRemoteMod::get_colorbar_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_depth(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_width(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_width(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_width(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_width(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_length(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_length(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_length(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_length(&tmp, out_val);
}

int ChipsRemoteMod::hide_colorbar_border(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::hide_colorbar_border(&tmp);
}

int ChipsRemoteMod::display_colorbar_border(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::display_colorbar_border(&tmp);
}

int ChipsRemoteMod::get_colorbar_border_visible(string idstr, bool& val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_border_visible(&tmp, val);
}

int ChipsRemoteMod::set_colorbar_halign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_halign(&tmp, sz);
}

int ChipsRemoteMod::get_colorbar_halign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_halign(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_valign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_valign(&tmp, sz);
}

int ChipsRemoteMod::get_colorbar_valign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_valign(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_angle(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_angle(&tmp, sz);
}

int ChipsRemoteMod::get_colorbar_label_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_angle(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_halign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_halign(&tmp, sz);
}

int ChipsRemoteMod::get_colorbar_label_halign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_halign(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_valign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_valign(&tmp, sz);
}

int ChipsRemoteMod::get_colorbar_label_valign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_valign(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_orientation(string idstr, int inval)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_orientation(&tmp, inval);
}

int ChipsRemoteMod::get_colorbar_orientation(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_orientation(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_text(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_text(&tmp, in_color);
}

int ChipsRemoteMod::get_colorbar_label_text(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_text(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_color(&tmp, in_color);
}

int ChipsRemoteMod::get_colorbar_label_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_color(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_size(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_size(&tmp, sz);
}

int ChipsRemoteMod::get_colorbar_label_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_size(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_visible(string idstr, bool vis)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_visible(&tmp, vis);
}

int ChipsRemoteMod::get_colorbar_label_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_visible(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_font(string idstr, string font)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_font(&tmp, font);
}

int ChipsRemoteMod::get_colorbar_label_font(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_font(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_fontstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_fontstyle(&tmp, style);
}

int ChipsRemoteMod::get_colorbar_label_fontstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_fontstyle(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_color(&tmp, in_color);
}

int ChipsRemoteMod::get_colorbar_ticklabel_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_color(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_size(string idstr, int in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_size(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_ticklabel_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_size(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_font(string idstr, string in_font)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_font(&tmp, in_font);
}

int ChipsRemoteMod::get_colorbar_ticklabel_font(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_font(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_fontstyle(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_fontstyle(&tmp, style);
}

int ChipsRemoteMod::get_colorbar_ticklabel_fontstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_fontstyle(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_visible(string idstr, bool vis)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_visible(&tmp, vis);
}

int ChipsRemoteMod::get_colorbar_ticklabel_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_visible(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_angle(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_angle(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_ticklabel_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_angle(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_halign(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_halign(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_ticklabel_halign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_halign(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_valign(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_valign(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_ticklabel_valign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_ticklabel_valign(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_ticklabel_offset(string idstr, int in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_offset(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_ticklabel_offset(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_ticklabel_offset(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_label_location(string idstr, string style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_location(&tmp, style);
}

int ChipsRemoteMod::set_colorbar_label_location(string idstr, int in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_label_location(&tmp, in_style);
}

int ChipsRemoteMod::get_colorbar_label_location(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_label_location(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_tick_style(string idstr, string in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_style(&tmp, in_style);
}

int ChipsRemoteMod::set_colorbar_tick_style(string idstr, int in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_style(&tmp, in_style);
}

int ChipsRemoteMod::get_colorbar_tick_style(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_tick_style(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_tick_thickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_thickness(&tmp, in_thickness);
}

int ChipsRemoteMod::get_colorbar_tick_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_tick_thickness(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_tick_mode(string idstr, string in_mode)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_mode(&tmp, in_mode);
}

int ChipsRemoteMod::get_colorbar_tick_mode(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_tick_mode(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_tick_location(string idstr, string in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_location(&tmp, in_style);
}

int ChipsRemoteMod::set_colorbar_tick_location(string idstr, int in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_location(&tmp, in_style);
}

int ChipsRemoteMod::get_colorbar_tick_location(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_tick_location(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_tick_visible(string idstr, bool vis)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_visible(&tmp, vis);
}

int ChipsRemoteMod::get_colorbar_tick_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_tick_visible(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_tick_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_color(&tmp, in_color);
}

int ChipsRemoteMod::get_colorbar_tick_color(string idstr, string& out_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_tick_color(&tmp, out_color);
}

int ChipsRemoteMod::set_colorbar_tick_length(string idstr, int in_length)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_tick_length(&tmp, in_length);
}

int ChipsRemoteMod::get_colorbar_tick_length(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_tick_length(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_parallel_offset(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_parallel_offset(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_parallel_offset(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_parallel_offset(&tmp, out_val);
}

int ChipsRemoteMod::set_colorbar_perpendicular_offset(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::set_colorbar_perpendicular_offset(&tmp, in_val);
}

int ChipsRemoteMod::get_colorbar_perpendicular_offset(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::get_colorbar_perpendicular_offset(&tmp, out_val);
}



int ChipsRemoteMod::set_label_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_color(&tmp, in_color);
}

int ChipsRemoteMod::set_label_text(string idstr, string in_text)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_text(&tmp, in_text);
}

int ChipsRemoteMod::set_label_size(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_size(&tmp, sz);
}

int ChipsRemoteMod::set_label_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_depth(&tmp, sz);
}

int ChipsRemoteMod::set_label_angle(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_angle(&tmp, sz);
}

int ChipsRemoteMod::set_label_halign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_halign(&tmp, sz);
}

int ChipsRemoteMod::set_label_valign(string idstr, float sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_valign(&tmp, sz);
}

int ChipsRemoteMod::set_label_font(string idstr, string in_font)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_font(&tmp, in_font);
}

int ChipsRemoteMod::set_label_fontstyle(string idstr, string in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_label_fontstyle(&tmp, in_style);
}

int ChipsRemoteMod::hide_label(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::hide_label(&tmp);
}

int ChipsRemoteMod::display_label(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::display_label(&tmp);
}

int ChipsRemoteMod::set_current_label(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::set_current_label(&tmp);
}

int ChipsRemoteMod::set_line_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::set_line_color(&tmp, in_color);
}

int ChipsRemoteMod::set_line_thickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::set_line_thickness(&tmp, in_thickness);
}

int ChipsRemoteMod::set_line_style(string idstr, V_LINE_TYPE in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::set_line_style(&tmp, in_style);
}

int ChipsRemoteMod::set_line_extend(string idstr, V_LINE_EXTENSION_TYPE in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::set_line_extend(&tmp, in_style);
}


int ChipsRemoteMod::set_line_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::set_line_depth(&tmp, sz);
}

int ChipsRemoteMod::hide_line(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::hide_line(&tmp);
}

int ChipsRemoteMod::display_line(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::display_line(&tmp);
}

int ChipsRemoteMod::set_current_line(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::set_current_line(&tmp);
}

int ChipsRemoteMod::set_point_color(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::set_point_color(&tmp, in_color);
}

int ChipsRemoteMod::set_point_size(string idstr, int in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::set_point_size(&tmp, in_size);
}

int ChipsRemoteMod::set_point_angle(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::set_point_angle(&tmp, in_val);
}

int ChipsRemoteMod::set_point_fill(string idstr, bool in_fill)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::set_point_fill(&tmp, in_fill);
}

int ChipsRemoteMod::set_point_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::set_point_depth(&tmp, sz);
}

int ChipsRemoteMod::set_point_style(string idstr, V_SYMBOL_TYPE in_symbol)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::set_point_symbol(&tmp, in_symbol);
}

int ChipsRemoteMod::hide_point(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::hide_point(&tmp);
}

int ChipsRemoteMod::display_point(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::display_point(&tmp);
}

int ChipsRemoteMod::set_current_point(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::set_current_point(&tmp);
}

int ChipsRemoteMod::set_region_depth(string idstr, int sz)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_depth(&tmp, sz);
}

int ChipsRemoteMod::set_region_angle(string idstr, float in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_angle(&tmp, in_val);
}

int ChipsRemoteMod::set_region_edgethickness(string idstr, float in_thickness)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_edgethickness(&tmp, in_thickness);
}

int ChipsRemoteMod::set_region_edgestyle(string idstr, V_LINE_TYPE in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_edgestyle(&tmp, in_style);
}

int ChipsRemoteMod::set_region_fillstyle(string idstr, V_FILL_TYPE in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_fillstyle(&tmp, in_style);
}

int ChipsRemoteMod::set_region_edgecolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_edgecolor(&tmp, in_color);
}

int ChipsRemoteMod::set_region_fillcolor(string idstr, string in_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_fillcolor(&tmp, in_color);
}

int ChipsRemoteMod::set_region_opacity(string idstr, double in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_region_opacity(&tmp, in_val);
}

int ChipsRemoteMod::hide_region(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::hide_region(&tmp);
}

int ChipsRemoteMod::display_region(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::display_region(&tmp);
}

int ChipsRemoteMod::set_current_region(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::set_current_region(&tmp);
}

int ChipsRemoteMod::get_object_count(string idstr, V_OBJECT_TYPE obj, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_object_id(obj, idstr);
   return ChipsRemote::get_object_count(&tmp, obj,out_val);
}

int ChipsRemoteMod::get_object_coordinfo(string idstr, V_OBJECT_TYPE obj, COORD_SYS& out_val, string& out_id)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_object_id(obj, idstr);
   return ChipsRemote::get_object_coordinfo(&tmp, obj,out_val,out_id);
}

int ChipsRemoteMod::get_window_title(string idstr, string& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::get_window_title(&tmp, out_val);
}

int ChipsRemoteMod::get_window_bgcolor(string idstr, string& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::get_window_bgcolor(&tmp, out_val);
}

int ChipsRemoteMod::get_window_redraw(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::get_window_redraw(&tmp, out_val);
}

int ChipsRemoteMod::get_frame_bgcolor(string idstr, string& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::get_frame_bgcolor(&tmp, out_val);
}

int ChipsRemoteMod::get_frame_border_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::get_frame_border_color(&tmp, out_val);
}

int ChipsRemoteMod::get_frame_border_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::get_frame_border_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_default_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::get_default_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_frame_transparency(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::get_frame_transparency(&tmp, out_val);
}

int ChipsRemoteMod::get_frame_scale(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::get_frame_scale(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_color(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_halign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_halign(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_valign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_valign(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_size(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_font(string idstr, string &out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_font(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_fontstyle(string idstr, string &out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_fontstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_xpos(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_xpos(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_ypos(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_ypos(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_range(&tmp, outx1,outx2,outy1,outy2);
}

int ChipsRemoteMod::get_plot_xrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_xrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_plot_yrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_yrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_plot_axisstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_axisstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_cornerstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_cornerstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_image_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_colorbar_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_colorbar_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_contour_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_leftmargin(string idstr, float& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_leftmargin(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_rightmargin(string idstr, float& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_rightmargin(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_topmargin(string idstr, float& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_topmargin(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_bottommargin(string idstr, float& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_bottommargin(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_title_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_title_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_major_ticks_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_major_ticks_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_minor_ticks_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_minor_ticks_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_major_mode(string idstr, string& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_major_mode(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_major_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_major_thickness(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_major_style(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_major_style(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minor_mode(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minor_mode(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minor_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minor_thickness(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minor_style(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minor_style(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_automin(string idstr, bool& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_automin(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_automax(string idstr, bool& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_automax(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_pad(string idstr, double& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_pad(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_scale(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_scale(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_color(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_size(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_xlabel_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_xlabel_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ylabel_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ylabel_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_halign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_halign(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_valign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_valign(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_font(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_font(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_fontstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_fontstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_label_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_label_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_style(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_style(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_color(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_size(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_halign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_halign(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_valign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_valign(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_font(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_font(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_fontstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_fontstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ticklabel_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ticklabel_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_majorgrid_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_majorgrid_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_majorgrid_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_majorgrid_thickness(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_majorgrid_style(string idstr, V_LINE_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_majorgrid_style(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_majorgrid_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_majorgrid_color(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minorgrid_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minorgrid_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minorgrid_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minorgrid_thickness(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minorgrid_style(string idstr, V_LINE_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minorgrid_style(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minorgrid_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minorgrid_color(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_symbolstyle(string idstr, V_SYMBOL_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_symboltype(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_symbolfill(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_symbolfill(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_symbolsize(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_symbolsize(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_symbolangle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_symbolangle(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_symbolcolor(string idstr, string& out_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_symbolcolor(&tmp, out_color);
}

int ChipsRemoteMod::get_curve_linestyle(string idstr, V_LINE_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_linestyle(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_linethickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_linethickness(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_linecolor(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_linecolor(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_baddatamode(string idstr, V_BADDATAMODE_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_baddatamode(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_errstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_errup(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errup(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_errdown(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errdown(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_errleft(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errleft(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_errright(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errright(&tmp, out_val);
}

int ChipsRemoteMod::set_curve_errcaplength(string idstr, int in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_curve_errcaplength(&tmp, in_size);
}

int ChipsRemoteMod::get_curve_errcaplength(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errcaplength(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_errthickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errthickness(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_errcolor(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_errcolor(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_limitoverride(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_limitoverride(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_limitlength(string idstr, double& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_limitlength(&tmp, out_val);
}

int ChipsRemoteMod::get_curve_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_range(&tmp, outx1,outx2,outy1,outy2);
}

int ChipsRemoteMod::get_curve_xrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_xrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_curve_yrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_curve_yrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_image_colormap(string idstr, V_COLORMAP_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_colormap(&tmp, out_val);
}

int ChipsRemoteMod::get_image_interpolation(string idstr, 
   V_INTERPOLATION_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_interpolation(&tmp, out_val);
}

int ChipsRemoteMod::get_image_invert_colormap(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_invert_colormap(&tmp, out_val);
}

int ChipsRemoteMod::get_image_colormap_interpolate(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_colormap_interpolate(&tmp, out_val);
}

int ChipsRemoteMod::get_image_scale_channels(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_scale_channels(&tmp, out_val);
}

int ChipsRemoteMod::get_image_colormap_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_colormap_size(&tmp, out_val);
}

int ChipsRemoteMod::get_image_alpha(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_alpha(&tmp, out1,out2);
}

int ChipsRemoteMod::get_image_threshold(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_threshold(&tmp, out1,out2);
}

int ChipsRemoteMod::get_image_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_image_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_range(&tmp, outx1,outx2,outy1,outy2);
}

int ChipsRemoteMod::get_image_xrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_xrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_image_yrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::get_image_yrange(&tmp, out1,out2);
}


int ChipsRemoteMod::get_contour_mode(string idstr, string& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_mode(&tmp, out_val);
}

int ChipsRemoteMod::get_contour_style(string idstr, string& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_style(&tmp, out_val);
}

int ChipsRemoteMod::get_contour_color(string idstr, string& out_val) 
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_color(&tmp, out_val);
}

int ChipsRemoteMod::get_contour_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_thickness(&tmp, out_val);
}

int ChipsRemoteMod::get_contour_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_range(&tmp, outx1,outx2,outy1,outy2);
}

int ChipsRemoteMod::get_contour_xrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_xrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_contour_yrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_yrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_contour_zrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_zrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_contour_algorithm(string idstr, V_CONTOUR_TYPE& out_contour)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_algorithm(&tmp, out_contour);
}

int ChipsRemoteMod::get_contour_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::get_contour_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_contour_levels(ChipsId* cid, unsigned int &arraysize,
                              double** pos)
{
   vector<double> cntr_levels(0);
   int rval = ERR_STATUS_GOOD;
   rval = ChipsRemote::get_contour_levels (cid, cntr_levels);

   arraysize = cntr_levels.size ();

   (*pos) = new double [arraysize];
   for (unsigned int ii=0; ii<arraysize; ii++)
      (*pos)[ii] = cntr_levels[ii];

   return rval;
}

int ChipsRemoteMod::get_contour_levels(unsigned int &arraysize, double** pos)
{
   return ChipsRemoteMod::get_contour_levels (NULL, arraysize, pos);
}




int ChipsRemoteMod::get_contour_levels(string idstr, unsigned int &arraysize,double** pos)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemoteMod::get_contour_levels(&tmp, arraysize, pos);
}


int ChipsRemoteMod::get_histogram_symbolstyle(string idstr, V_SYMBOL_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_symboltype(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_symbolfill(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_symbolfill(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_symbolsize(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_symbolsize(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_symbolangle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_symbolangle(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_symbolcolor(string idstr, string& out_color)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_symbolcolor(&tmp, out_color);
}

int ChipsRemoteMod::get_histogram_linestyle(string idstr, V_LINE_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_linestyle(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_linethickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_linethickness(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_linecolor(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_linecolor(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_errstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_errstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_errup(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_errup(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_errdown(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_errdown(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_dropline(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_dropline(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_errthickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_errthickness(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_errcolor(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_errcolor(&tmp, out_val);
}

int ChipsRemoteMod::set_histogram_errcaplength(string idstr, int in_size)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::set_histogram_errcaplength(&tmp, in_size);
}

int ChipsRemoteMod::get_histogram_errcaplength(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::get_histogram_errcaplength(&tmp, out_val);
}


int ChipsRemoteMod::get_histogram_fillopacity(string idstr, double& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_fillopacity(&tmp, out_val);
}

int ChipsRemoteMod::get_histogram_fillcolor(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_fillcolor(&tmp, out_val);
}

int ChipsRemoteMod::set_histogram_fillstyle(string idstr, V_FILL_TYPE in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::set_histogram_fillstyle(&tmp, in_style);
}

int ChipsRemoteMod::get_histogram_fillstyle(string idstr, V_FILL_TYPE &in_style)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_fillstyle(&tmp, in_style);
}

int ChipsRemoteMod::get_histogram_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_range(&tmp, outx1,outx2,outy1,outy2);
}

int ChipsRemoteMod::get_histogram_xrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_xrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_histogram_yrange(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_yrange(&tmp, out1,out2);
}

int ChipsRemoteMod::get_histogram_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::get_histogram_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_color(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_major_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_major_color(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minor_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minor_color(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_text(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_text(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_xlabel_text(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_xlabel_text(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_ylabel_text(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_ylabel_text(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_tickformat(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_tickformat(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_thickness(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_tickstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_tickstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_major_num(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_major_num(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minor_num(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minor_num(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_major_length(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_major_length(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minor_length(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minor_length(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_major_interval(string idstr, double& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_major_interval(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_minor_interval(string idstr, double& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_minor_interval(&tmp, out_val);
}

int ChipsRemoteMod::get_axis_range(string idstr, double& out1, double& out2)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::get_axis_range(&tmp, out1,out2);
}

int ChipsRemoteMod::get_label_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_color(&tmp, out_val);
}

int ChipsRemoteMod::get_label_text(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_text(&tmp, out_val);
}

int ChipsRemoteMod::get_label_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_size(&tmp, out_val);
}

int ChipsRemoteMod::get_label_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_label_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_label_halign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_halign(&tmp, out_val);
}

int ChipsRemoteMod::get_label_valign(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_valign(&tmp, out_val);
}

int ChipsRemoteMod::get_label_font(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_font(&tmp, out_val);
}

int ChipsRemoteMod::get_label_fontstyle(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_fontstyle(&tmp, out_val);
}

int ChipsRemoteMod::get_label_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::get_label_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_line_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::get_line_color(&tmp, out_val);
}

int ChipsRemoteMod::get_line_thickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::get_line_thickness(&tmp, out_val);
}

int ChipsRemoteMod::get_line_style(string idstr, V_LINE_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::get_line_style(&tmp, out_val);
}

int ChipsRemoteMod::get_line_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::get_line_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_line_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::get_line_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_point_color(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::get_point_color(&tmp, out_val);
}

int ChipsRemoteMod::get_point_size(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::get_point_size(&tmp, out_val);
}

int ChipsRemoteMod::get_point_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::get_point_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_point_fill(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::get_point_fill(&tmp, out_val);
}

int ChipsRemoteMod::get_point_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::get_point_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_point_style(string idstr, V_SYMBOL_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::get_point_symbol(&tmp, out_val);
}

int ChipsRemoteMod::get_point_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::get_point_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_region_visible(string idstr, bool& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_visible(&tmp, out_val);
}

int ChipsRemoteMod::get_region_depth(string idstr, int& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_depth(&tmp, out_val);
}

int ChipsRemoteMod::get_region_angle(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_angle(&tmp, out_val);
}

int ChipsRemoteMod::get_region_edgethickness(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_edgethickness(&tmp, out_val);
}

int ChipsRemoteMod::get_region_edgestyle(string idstr, V_LINE_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_edgestyle(&tmp, out_val);
}

int ChipsRemoteMod::get_region_edgecolor(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_edgecolor(&tmp, out_val);
}

int ChipsRemoteMod::get_region_fillcolor(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_fillcolor(&tmp, out_val);
}

int ChipsRemoteMod::get_region_fillstyle(string idstr, V_FILL_TYPE& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_fillstyle(&tmp, out_val);
}


int ChipsRemoteMod::get_region_opacity(string idstr, double& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::get_region_opacity(&tmp, out_val);
}

int ChipsRemoteMod::delete_window(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_window_id(idstr);
   return ChipsRemote::delete_window(&tmp);
}

int ChipsRemoteMod::delete_frame(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_frame_id(idstr);
   return ChipsRemote::delete_frame(&tmp);
}

int ChipsRemoteMod::delete_plot(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::delete_plot(&tmp);
}

int ChipsRemoteMod::clear_plot(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::clear_plot(&tmp);
}

int ChipsRemoteMod::delete_curve(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_curve_id(idstr);
   return ChipsRemote::delete_curve(&tmp);
}

int ChipsRemoteMod::delete_contour(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_contour_id(idstr);
   return ChipsRemote::delete_contour(&tmp);
}

int ChipsRemoteMod::delete_histogram(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_histogram_id(idstr);
   return ChipsRemote::delete_histogram(&tmp);
}

int ChipsRemoteMod::delete_axis(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_axis_id(idstr);
   return ChipsRemote::delete_axis(&tmp);
}

int ChipsRemoteMod::delete_image(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::delete_image(&tmp);
}

int ChipsRemoteMod::set_image_channel(string idstr, double* vals, int xaxis, int yaxis, int channel)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::set_image_channel(&tmp, vals, xaxis, yaxis, channel);
}

int ChipsRemoteMod::remove_image_channel(string idstr, int channel)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_image_id(idstr);
   return ChipsRemote::remove_image_channel(&tmp, channel);
}

int ChipsRemoteMod::delete_colorbar(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_colorbar_id(idstr);
   return ChipsRemote::delete_colorbar(&tmp);
}

int ChipsRemoteMod::delete_label(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_label_id(idstr);
   return ChipsRemote::delete_label(&tmp);
}

int ChipsRemoteMod::delete_line(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_line_id(idstr);
   return ChipsRemote::delete_line(&tmp);
}

int ChipsRemoteMod::delete_point(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_point_id(idstr);
   return ChipsRemote::delete_point(&tmp);
}

int ChipsRemoteMod::delete_region(string idstr)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_region_id(idstr);
   return ChipsRemote::delete_region(&tmp);
}

int ChipsRemoteMod::shuffle(string idstr, V_OBJECT_TYPE obj, V_SHUFFLE_TYPE way)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_object_id(obj,idstr);
   return ChipsRemote::shuffle(&tmp, obj, way);
}

int ChipsRemoteMod::move(string idstr, V_OBJECT_TYPE obj, double xx, double yy,
   int mode)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_object_id(obj,idstr);
   return ChipsRemote::move(&tmp, obj, xx, yy, mode);
}

int ChipsRemoteMod::get_data_aspect_ratio(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_coord_id(idstr);
   tmp.set_coord_sys(DATA); 
   return ChipsRemote::get_data_aspect_ratio(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_aspect_ratio(string idstr, string& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_aspect_ratio(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_aspect_height(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_aspect_height(&tmp, out_val);
}

int ChipsRemoteMod::get_plot_aspect_width(string idstr, float& out_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::get_plot_aspect_width(&tmp, out_val);
}

int ChipsRemoteMod::set_data_aspect_ratio(string idstr, string in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_coord_id(idstr);
   tmp.set_coord_sys(DATA); 
   return ChipsRemote::set_data_aspect_ratio(&tmp, in_val);
}

int ChipsRemoteMod::set_plot_aspect_ratio(string idstr, string in_val)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::set_plot_aspect_ratio(&tmp, in_val);
}

int ChipsRemoteMod::log_scale(string idstr, int dim)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::log_scale(&tmp, dim);
}

int ChipsRemoteMod::logarithmic_scale(string idstr, int dim)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::log_scale(&tmp, dim);
}

int ChipsRemoteMod::lin_scale(string idstr, int dim)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::linear_scale(&tmp, dim);
}

int ChipsRemoteMod::linear_scale(string idstr, int dim)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_plot_id(idstr);
   return ChipsRemote::linear_scale(&tmp, dim);
}

int ChipsRemoteMod::highlight_object(string idstr, V_OBJECT_TYPE obj)
{
   ChipsId tmp;
   tmp.clear();
   tmp.set_object_id(obj, idstr);
   return ChipsRemote::highlight_object(&tmp, obj);
}


