// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsRepoCommand.hh"
#include "ChipsDefaults.hh"
#include "ChipsStateStream.hh"
#include "ChipsWindow.hh"
#include "ChipsFrame.hh"
#include "ChipsPlot.hh"

extern ChipsDefaults* theDefaults; 

ChipsRepoCommand::ChipsRepoCommand(ChipsObject *new_co, float ollx, float olly,
    float ourx, float oury, float nllx, float nlly, float nurx, float nury,
    string oldcurr)
{
   co = new_co;
   weight=1;
   o_llx = ollx;
   o_lly = olly;
   o_urx = ourx;
   o_ury = oury;
   n_llx = nllx;
   n_lly = nlly;
   n_urx = nurx;
   n_ury = nury;
   current = oldcurr;
}

ChipsRepoCommand::~ChipsRepoCommand()
{
}


ChipsRepoCommand::ChipsRepoCommand(ChipsObject* cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsRepoCommand::execute ()
{
   int rval = ERR_STATUS_GOOD;
   if (co && (co->get_object_type () == chips_frame))
   {
      rval = dynamic_cast<ChipsFrame *>(co)->reposition(n_llx, n_lly, n_urx, n_ury, false);
      if (rval == ERR_STATUS_GOOD) 
        dynamic_cast<ChipsWindow *>(co->get_parent())->update_display();
   }
   else if (co && (co->get_object_type () == chips_plot))
   {
      ChipsPlot* cp = dynamic_cast<ChipsPlot *>(co);
      cp->log_updates();
      rval = dynamic_cast<ChipsPlot *>(co)->reposition(n_llx, n_lly, n_urx, n_ury);
      cp->handle_updates(); 
   }

   return rval;
}

int ChipsRepoCommand::unexecute ()
{
   int rval = ERR_STATUS_GOOD;
   if (co && (co->get_object_type () == chips_frame))
   {
      rval = dynamic_cast<ChipsFrame *>(co)->reposition(o_llx, o_lly, o_urx, o_ury, false);
      if (rval == ERR_STATUS_GOOD)
      {
         dynamic_cast<ChipsWindow *>(co->get_parent())->update_display();
         dynamic_cast<ChipsWindow *>(co->get_parent())->set_current(current);
      }
   }
   else if (co && (co->get_object_type () == chips_plot))
   {
      ChipsPlot* cp = dynamic_cast<ChipsPlot *>(co); 
      cp->log_updates(); 
      rval = dynamic_cast<ChipsPlot *>(co)->reposition(o_llx, o_lly, o_urx, o_ury);
      cp->handle_updates(); 
   }
   return rval;
}

void ChipsRepoCommand::serialize(ChipsStateStream& out)
{
   int val = 13;
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   out.write_float(&o_llx);
   out.write_float(&o_lly);
   out.write_float(&o_urx);
   out.write_float(&o_ury);
   out.write_float(&n_llx);
   out.write_float(&n_lly);
   out.write_float(&n_urx);
   out.write_float(&n_ury);
   out.write_string(&current);
}

void ChipsRepoCommand::deserialize(ChipsStateStream& in)
{
   in.read_int(&ref);
   in.read_int(&block_ref);

   in.read_float(&o_llx);
   in.read_float(&o_lly);
   in.read_float(&o_urx);
   in.read_float(&o_ury);
   in.read_float(&n_llx);
   in.read_float(&n_lly);
   in.read_float(&n_urx);
   in.read_float(&n_ury);
   in.read_string(&current);
}

void ChipsRepoCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Reposition Frame" << endl;
};



