// 
//  Copyright (C) 2004,2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSSERVER_HH
#define CHIPSSERVER_HH

#include <string>
#include <list>
#include <iostream>
#include <fstream>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "gtk/gtk.h"

#include "ChipsSystem.hh"

// macros to determine the byte offset in the buffer for positions
// of various standard types
//
#define NEXT_DOUBLE_POS(p) (((p + sizeof(double) -1)/sizeof(double)) \
                            * sizeof(double))
#define NEXT_FLOAT_POS(p)  (((p + sizeof(float) -1)/sizeof(float)) \
                            * sizeof(float))
#define NEXT_LONG_POS(p)   (((p + sizeof(long) -1)/sizeof(long)) \
                            * sizeof(long))
#define NEXT_INT_POS(p)    (((p + sizeof(int) -1)/sizeof(int)) \
                            * sizeof(int))
#define NEXT_BOOL_POS(p)   (((p + sizeof(bool) -1)/sizeof(bool)) \
                            * sizeof(bool))
#define NEXT_SHORT_POS(p)    (((p + sizeof(short) -1)/sizeof(short)) \
                            * sizeof(short))
#define NEXT_CHAR_POS(p)   (((p + sizeof(char) -1)/sizeof(char)) \
                            * sizeof(char))



class ChipsServer 
{
  public:

   ChipsServer(bool load_prefs=true);
   virtual ~ChipsServer();

   int            create_server(const string& inServerId);

   static int     pack_it(ChipsId* cid, int msgid);
   static int     unpack_it(const char* buf, ChipsId* cid);
   static int     clean_xvfb();
   static int     clean_gui();

   string         get_server_id() { return serverId; }

   int            remove_gui(); 
   static void    lock(long id);
   static void    unlock(long id);
   int            is_locked();

  protected:

   static int     handle_server_signals(); 

   static pid_t   pid; 

  private:

   static ChipsSystem* chipsSys;
   static int     server_process(gpointer);
   static void    sig_handler(int); 
   static double  buffer[1024]; 
   static string  serverId;
   static string  xvfb_pid;
   static string  serverCwd; 
   static long    lockFlag;
   static long    lockClient;
   static long    currClientId;
   static int     cmdlineConn;
};


#endif 
