// 
//  Copyright (C) 2006,2007, 2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSSTATESTREAM_HH
#define CHIPSSTATESTREAM_HH

using namespace std;

#include <ios>
#include <iostream>
#include <fstream>
#include <string>


class ChipsStateStream: public fstream
{
   public: 
     ChipsStateStream(const char *s, ios_base::openmode mode, int opts=0);
     virtual ~ChipsStateStream();

     int read_double(double* dd, int cnt=1);
     int read_float(float* ff, int cnt=1);
     int read_long(long* ii, int cnt=1);
     int read_int(int* ii, int cnt=1);
     int read_short(short* ii, int cnt=1);
     int read_bool(bool* bb, int cnt=1);
     int read_char(char* ii, int cnt=1);
     int read_string(string* ss);
     int read_checkpoint(); 

     int write_double(double* dd, int cnt=1);
     int write_float(float* ff, int cnt=1);
     int write_long(long* ii, int cnt=1);
     int write_int(int* ii, int cnt=1);
     int write_short(short* ii, int cnt=1);
     int write_bool(bool* bb, int cnt=1);
     int write_char(char* ii, int cnt=1);
     int write_string(string* ss); 
     int write_checkpoint(); 

     int get_error()  { return error; } 
     int get_major_version()  { return vmajor; }
     int get_minor_version()  { return vminor; }
     bool version_compatible(int maj, int min)
        { return ((maj == vmajor) && (min >= vminor)); } 
     bool is_binary()         { return (mymode == 0); }

     // ascii state file specific api
     string get_data_template()    { return tempdata; }
     void  append_to_template(string fileinfo);
     string get_filename()    { return fname; }
     string use_tempname();
     int    use_tempfnum()      { return tempfnum++; }
     int    get_tempnum()      { return tempnum; }



   private:

     int  error;               // error status 
     int  vmajor;              // major version number
     int  vminor;              // minor version
     int  mymode;              // binary (0) or ascii (1)

     string fname;             // supplied filename
     string tempdata;          // template data for ascii save
     string tempname;
     int    tempnum;           // template data object number 
     int    tempfnum;          // template file number

     static int  file_endian;  // file format little (0) or big endian (1) 
     static int  dbl_sz;       // file format double size
     static int  flt_sz;       // file format float size
     static int  lng_sz;       // file format long size
     static int  int_sz;       // file format integer size
     static int  sht_sz;       // file format short size
     static int  bln_sz;       // file format boolean size
     static int  chr_sz;       // file format char size
     static int  ckpt_sz;      // file format checkpoint size
     static int  my_endian;    // application is little (0) or big endian (1) 
     static int  my_dbl_sz;    // application's double size
     static int  my_flt_sz;    // application's float size
     static int  my_lng_sz;    // application's long size
     static int  my_int_sz;    // application's integer size
     static int  my_sht_sz;    // application's short size
     static int  my_bln_sz;    // application's boolean size
     static int  my_chr_sz;    // application's char size
     static int  my_ckpt_sz;   // application's checkpoint size
};

#endif
