// 
//  Copyright (C) 2006,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSWINDOWSYSTEM_HH
#define CHIPSWINDOWSYSTEM_HH

#include <list>
#include <string>
#include <map>
#include "ChipsCoordinate.hh"
#include "ChipsEngine.hh"
#include "ChipsMatrix.hh"
#include "ChipsStateStream.hh"

class ChipsWindowSystem: public ChipsCoordinate
{
   public:
      ChipsWindowSystem (ChipsEngine *en);
      ChipsWindowSystem (ChipsEngine *en,
                         int wdth, int hght, double scale_wdth, 
			 double scale_hgth);
      ChipsWindowSystem (const ChipsWindowSystem &ws);
      ChipsWindowSystem (ChipsStateStream &in, ChipsEngine *en);
      virtual ~ChipsWindowSystem ();

      /* Overloaded operators */
      void operator = (const ChipsWindowSystem &);

      virtual int convert_cs_to_cs (const ChipsVector *in, ChipsVector *out, int size,
                                 COORD_SYS from_cs, const string &from_id,
                                 COORD_SYS to_cs, const string &to_id);
      virtual int add_coord_sys (ChipsCoordinate *);
      virtual int remove_coord_sys (const string &id);
      virtual ChipsCoordinate *get_coord (COORD_SYS coord_sys, const string &id="");
      virtual bool coord_exists (COORD_SYS coord_sys, const string &id);
      virtual void get_coord_list (list <string> &outList);

      virtual int insert (ChipsObject *co, bool use_co_info=true);
      virtual int remove (ChipsObject *co, bool use_co_info=true);
      virtual int update_obj_positions (bool update_all=true);

      virtual ChipsVector get_lower_bounds (){return ChipsVector (0,0);};
      virtual ChipsVector get_upper_bounds (){return ChipsVector (pix_width, pix_height);};
      virtual COORD_SYS   get_bounds_coord_sys () {return PIXEL;};

      virtual int deserialize (ChipsStateStream &in);
      virtual void serialize (ChipsStateStream &out);

      // create an id from all parent information
      virtual void create_id (string &curr_id);

      void set_dim (int win_width, int win_height);
      void get_dim (int &win_width, int &win_height);
      void set_scale (double width, double height);
      void get_scale (double &width, double &height);

   protected:
      ChipsEngine *eng;
      map <string, ChipsFrameSystem *> frame_systems;
      int pix_width;
      int pix_height;
      double scale_width;
      double scale_height;

      unsigned long ref_id;
      static unsigned long global_ref_id;
};

#endif
