// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsZoomCommand.hh"
#include "ChipsStateStream.hh"
#include "ChipsId.hh"
#include "ChipsFrame.hh"

ChipsZoomCommand::ChipsZoomCommand(ChipsFrame* obj, float scale_,
   ChipsId *cid_)
{
   co = (ChipsObject*)obj;
   weight = 1;

   this->scale = scale_;
   if (cid)
      this->cid = new ChipsId (*cid_);
   else
      this->cid = NULL;
}

ChipsZoomCommand::~ChipsZoomCommand()
{
   if (this->cid)   delete  this->cid;
}


ChipsZoomCommand::ChipsZoomCommand(ChipsObject *cob, ChipsStateStream& in)
{
   co = cob;
   deserialize(in);
   weight=1;
}

int ChipsZoomCommand::execute ()
{
   int rval = ERR_STATUS_BAD;
   ChipsFrame* cf = dynamic_cast <ChipsFrame *>(co);
   if (cf)
      rval = cf->zoom(this->cid, this->scale, false);
   return rval;
}

int ChipsZoomCommand::unexecute ()
{
   int rval = ERR_STATUS_BAD;
   ChipsFrame* cf = dynamic_cast <ChipsFrame *>(co);
   if (cf)
      rval = cf->zoom(this->cid, 1/this->scale, false);
   return rval;
}

void ChipsZoomCommand::serialize(ChipsStateStream& out)
{
   int val = 25;
   bool cid_exists = true;
   
   // Standard serialize output for a command
   out.write_int(&val);
   serialize_object(out);
   out.write_int(&ref);
   out.write_int(&block_ref);

   out.write_float(&this->scale);
   if (this->cid)
   {
      out.write_bool (&cid_exists);
      this->cid->serialize (out);
   }
   else
   {
      cid_exists = false;
      out.write_bool (&cid_exists);
   }
}

void ChipsZoomCommand::deserialize(ChipsStateStream& in)
{
   bool cid_exists;

   // Standard serialize input for a command
   in.read_int(&ref);
   in.read_int(&block_ref);

   in.read_float (&this->scale);
   in.read_bool (&cid_exists);
   if (cid_exists)
      this->cid = new ChipsId (in);
}

void ChipsZoomCommand::print ()
{
   cerr << "   [ "<<ref<<", "<< block_ref <<"] " <<"Zoom" << endl;
};



