// 
//  Copyright (C) 2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "ChipsEventManager.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsGuiAddLabelDialog.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;
extern ChipsEventManager* event_manager;

ChipsGuiAddLabelDialog::ChipsGuiAddLabelDialog()
{
   register_event(ChipsEventManager::AddEvent);
}

ChipsGuiAddLabelDialog::~ChipsGuiAddLabelDialog()
{
}

void ChipsGuiAddLabelDialog::build_form ()
{
   GtkWidget *label;
   GtkWidget *vbox;

   // we will actually add the close button separately below so we can
   // hook up our own callback 
   _dialog = gtk_dialog_new_with_buttons("Add Label", NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE, GTK_STOCK_CLOSE, NULL);
//   gtk_widget_set_size_request(base_w, 450, 400);
   vbox = GTK_DIALOG(_dialog)->vbox;

   label = gtk_label_new("Enter Text:");
   gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
   _text = gtk_entry_new ();
   gtk_box_pack_start(GTK_BOX(vbox), _text, TRUE, TRUE, 0);


   // destroy the window when user hits close
   g_signal_connect (_dialog,
                             "response",
                             G_CALLBACK (cleanup_callback),
                             this);

   // add text when user types
   _text_id = g_signal_connect(G_OBJECT(_text), "changed", G_CALLBACK(text_change_callback), (void *)this);
   _text_activate_id = g_signal_connect(G_OBJECT(_text), "activate", G_CALLBACK(text_activate_callback), (void *)this);

   setup_tooltip(_text,"Raw text (includes limited latex support) to use for the label");

}

void ChipsGuiAddLabelDialog::text_change_callback(GtkWidget *widget, gpointer data)
{

   ChipsGuiAddLabelDialog *labelWidget = static_cast <ChipsGuiAddLabelDialog*>(data);
   const char *val = gtk_entry_get_text (GTK_ENTRY(widget));
   string text;

   if (val)
   {
      text = val;
      labelWidget->set_text(text);
   }

}

void ChipsGuiAddLabelDialog::text_activate_callback (GtkWidget *widget, gpointer data)
{
   ChipsGuiAddLabelDialog *labelWidget = static_cast <ChipsGuiAddLabelDialog*>(data);
   ChipsGuiAddLabelDialog::text_change_callback (widget, data);
   labelWidget->close_dialog ();
}

bool ChipsGuiAddLabelDialog::cleanup_callback(GtkWidget *wid, GdkEvent *event, gpointer data)
{
   ChipsGuiAddLabelDialog *labelWidget = static_cast <ChipsGuiAddLabelDialog*>(data);

   labelWidget->close_dialog ();
   return true;

}

void ChipsGuiAddLabelDialog::close_dialog ()
{
   ChipsEngine *eng = chipsGui->get_engine();

   if (eng)
   {
      double pos[2] = {0,0};
      event_manager->invoke_event (pos, ChipsEventManager::EndAddEvent, NULL);
   }
      
   gtk_widget_destroy (this->_dialog);
}

void ChipsGuiAddLabelDialog::execute_event (int event, void *data)
{

   switch (event)
   {
      case ChipsEventManager::AddEvent:
      {
         if (event_manager->get_add_substate () != ChipsEventManager::LabelAdd)
            break;

         this->build_form ();
         gtk_window_set_position (GTK_WINDOW(this->_dialog), GTK_WIN_POS_MOUSE);
         gtk_window_set_modal(GTK_WINDOW(this->_dialog), true);
         gtk_widget_show_all (this->_dialog);
      }
      break;
      default:
      break;
   }

}

void ChipsGuiAddLabelDialog::set_text (const string &text)
{
   ChipsId cid;

   ChipsEngine *eng = chipsGui->get_engine();
   
   if (!eng)
      return;

   eng->set_property(&cid, static_cast<void*>(const_cast<char*>(text.c_str ())),
      SET_LABEL_TEXT);
}
