// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "ChipsGuiAnnotateMenu.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEventManager.hh"
#include <stdlib.h>

using namespace std;

extern ChipsGuiMain* chipsGui;
extern ChipsEventManager* event_manager;


ChipsGuiAnnotateMenu::ChipsGuiAnnotateMenu() : ChipsEventObserver(), ChipsGuiTooltipObject()
{
   int pos = 0; 
   GtkWidget *img_icon;
   string dname = "", fname;
   AtkObject *accessible;

   _enabled = true;

   base_w = gtk_menu_item_new_with_mnemonic("_Annotate");

   // set path to icons
   char *tmpdir = getenv("ASCDS_INSTALL");
   if (tmpdir)   dname = tmpdir;
   dname.append("/config/chips/");

   menu = gtk_menu_new();

   menuItemAddLabel = gtk_image_menu_item_new_with_label (("Label"));
   fname = dname;
   fname.append("icon_addlabel.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddLabel), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddLabel), TRUE);
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddLabel, pos++);

   menuItemAddPoint = gtk_image_menu_item_new_with_label (("Point"));
   fname = dname;
   fname.append("icon_addpoint.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddPoint), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddPoint), TRUE);
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddPoint, pos++);

   menuItemAddLine = gtk_image_menu_item_new_with_label (("Polyline"));
   fname = dname;
   fname.append("icon_addline.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddLine), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddLine), TRUE);
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddLine, pos++);

   menuItemAddHLine = gtk_image_menu_item_new_with_label (("Horizontal Line"));
   fname = dname;
   fname.append("icon_addhline.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddHLine), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddHLine), TRUE);
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddHLine, pos++);

   menuItemAddVLine = gtk_image_menu_item_new_with_label (("Vertical Line"));
   fname = dname;
   fname.append("icon_addvline.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddVLine), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddVLine), TRUE);

   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddVLine, pos++);

   menuItemAddPolygon = gtk_image_menu_item_new_with_label (("Polygon"));
   fname = dname;
   fname.append("icon_addpolygon.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddPolygon), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddPolygon), TRUE);

   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddPolygon, pos++);

   menuItemAddEllipse = gtk_image_menu_item_new_with_label (("Ellipse"));
   fname = dname;
   fname.append("icon_addellipse.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddEllipse), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddEllipse), TRUE);

   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddEllipse, pos++);

   menuItemAddBox = gtk_image_menu_item_new_with_label (("Box"));
   fname = dname;
   fname.append("icon_addbox.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddBox), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddBox), TRUE);

   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddBox, pos++);

   menuItemAddColorBar = gtk_image_menu_item_new_with_label (("Colorbar"));
   fname = dname;
   fname.append("icon_addcolorbar.png");
   img_icon = gtk_image_new_from_file (fname.c_str());
   gtk_image_menu_item_set_image  (GTK_IMAGE_MENU_ITEM(menuItemAddColorBar), GTK_WIDGET(img_icon));
   gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM(menuItemAddColorBar), TRUE);

   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAddColorBar, pos++);



   g_signal_connect(G_OBJECT(menuItemAddLabel), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddPoint), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddLine), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddHLine), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddVLine), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddPolygon), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddEllipse), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddBox), "activate",
                    G_CALLBACK (add_cb), this);
   g_signal_connect(G_OBJECT(menuItemAddColorBar), "activate",
                    G_CALLBACK (add_cb), this);


   setup_tooltip(menuItemAddLabel, "Add a label in the specified plot");
   setup_tooltip(menuItemAddPoint, "Add a point in the specified plot");
   setup_tooltip(menuItemAddLine, "Add a line in the specified plot");
   setup_tooltip(menuItemAddHLine, "Add a horizontal line in the specified plot");
   setup_tooltip(menuItemAddVLine, "Add a vertical line in the specified plot");
   setup_tooltip(menuItemAddPolygon, "Add a polygon in the specified plot");
   setup_tooltip(menuItemAddEllipse, "Add an ellipse in the specified plot");
   setup_tooltip(menuItemAddBox, "Add a box in the specified plot");
   setup_tooltip(menuItemAddColorBar, "Add a colorbar in the specified plot");



   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
}

ChipsGuiAnnotateMenu::~ChipsGuiAnnotateMenu()
{
   
}

int ChipsGuiAnnotateMenu::show()
{
   gtk_widget_show(base_w); 
   return 0; 
}

int ChipsGuiAnnotateMenu::hide()
{
   gtk_widget_hide(base_w);
   return 0;
}


int ChipsGuiAnnotateMenu::add_cb(GtkMenuItem *mitem, gpointer user_data)
{
   ChipsGuiAnnotateMenu* annoMenu = (ChipsGuiAnnotateMenu*)user_data;
   int rv = -1;

   if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddLabel))
      rv = annoMenu->add_object(ChipsEventManager::LabelAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddPoint))
      rv = annoMenu->add_object(ChipsEventManager::PointAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddLine))
      rv = annoMenu->add_object(ChipsEventManager::PolyLineAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddHLine))
      rv = annoMenu->add_object(ChipsEventManager::HLineAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddVLine))
      rv = annoMenu->add_object(ChipsEventManager::VLineAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddPolygon))
      rv = annoMenu->add_object(ChipsEventManager::PolygonAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddEllipse))
      rv = annoMenu->add_object(ChipsEventManager::EllipseAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddBox))
      rv = annoMenu->add_object(ChipsEventManager::BoxAdd);
   else if (mitem == GTK_MENU_ITEM(annoMenu->menuItemAddColorBar))
      rv = annoMenu->add_object(ChipsEventManager::ColorBarAdd);



   return rv;
}



int ChipsGuiAnnotateMenu::add_object(int substate)
{
   if (!chipsGui)  return -1;

   double pos[2] = {0.0,0.0};
   event_manager->invoke_event(pos, ChipsEventManager::ChangeAddStateEvent,
      (void*)&substate, this);


   return 0;
}

void ChipsGuiAnnotateMenu::execute_event (int event, void *data)
{
}

void ChipsGuiAnnotateMenu::set_sensitivity (bool enable)
{
   if (_enabled == enable)
      return;

   gtk_widget_set_sensitive(base_w, enable);
   _enabled = enable;
}

