// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiMain.hh"
#include "ChipsGuiAnnotationToolbar.hh"
#include "ChipsEventObserver.hh"
#include "ChipsGuiSettings.hh"
#include <stdlib.h>

#include <atk/atk.h>


extern ChipsEventManager* event_manager;
extern ChipsGuiSettings* GuiSettings;
extern ChipsGuiMain* chipsGui;


using namespace std;


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiAnnotationToolbar::ChipsGuiAnnotationToolbar():ChipsEventObserver(),ChipsGuiTooltipObject()
{
   GtkWidget *img_icon;
   string dname = "", fname; 
   AtkObject *accessible;

   // create the buttons in toolbar
   _toolbar = gtk_toolbar_new ();

   _btnStyle = 2;
   _annotation_enabled = true;

   // set path to icons
   char *tmpdir = getenv("ASCDS_INSTALL");
   if (tmpdir)   dname = tmpdir;
   dname.append("/config/chips/");


   // set path to icons 
   if ((_emptyBtn = gtk_radio_tool_button_new(NULL)) != NULL)
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _emptyBtn, -1);
   if ((_pointBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _pointBtn, -1);
      fname = dname;
      fname.append("icon_addpoint.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_pointBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_pointBtn),"point"); 
      _pointBtn_id = g_signal_connect(G_OBJECT(_pointBtn), "clicked",
         G_CALLBACK(addpoint_cb), (void*)this);
   }
   if ((_labelBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _labelBtn, -1);
      fname = dname;
      fname.append("icon_addlabel.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_labelBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_labelBtn),"Label");
      _labelBtn_id = g_signal_connect(G_OBJECT(_labelBtn), "clicked",
         G_CALLBACK(addlabel_cb), (void*)this);
   }
   if ((_lineBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _lineBtn, -1);
      fname = dname;
      fname.append("icon_addline.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_lineBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_lineBtn),"Polyline");
      _lineBtn_id = g_signal_connect(G_OBJECT(_lineBtn), "clicked",
         G_CALLBACK(addline_cb), (void*)this);
   }
   if ((_hlineBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _hlineBtn, -1);
      fname = dname;
      fname.append("icon_addhline.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_hlineBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_hlineBtn),"Horizontal Line");
      _hlineBtn_id = g_signal_connect(G_OBJECT(_hlineBtn), "clicked",
         G_CALLBACK(addhline_cb), (void*)this);
   }
   if ((_vlineBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _vlineBtn, -1);
      fname = dname;
      fname.append("icon_addvline.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_vlineBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_vlineBtn),"Vertical Line");
      _vlineBtn_id = g_signal_connect(G_OBJECT(_vlineBtn), "clicked",
         G_CALLBACK(addvline_cb), (void*)this);
   }



   setup_tooltip(GTK_WIDGET(_pointBtn), "Annotate the plot with a point"); 
   setup_tooltip(GTK_WIDGET(_labelBtn), "Annotate the plot with a label");
   setup_tooltip(GTK_WIDGET(_lineBtn), "Annotate the plot with a polyline");
   setup_tooltip(GTK_WIDGET(_vlineBtn), "Annotate the plot with a vertical line");
   setup_tooltip(GTK_WIDGET(_hlineBtn), "Annotate the plot with a horizontal line");


   // testing accessiblity
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_pointBtn));
   atk_object_set_name (accessible, "pointButton");
   atk_object_set_description (accessible, "Annotate with a point");
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_labelBtn));
   atk_object_set_name (accessible, "labelButton");
   atk_object_set_description (accessible, "Annotate with a label");
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_lineBtn));
   atk_object_set_name (accessible, "lineButton");
   atk_object_set_description (accessible, "Annotate with a line");
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_vlineBtn));
   atk_object_set_name (accessible, "vlineButton");
   atk_object_set_description (accessible, "Annotate with a vertical line");
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_hlineBtn));
   atk_object_set_name (accessible, "hlineButton");
   atk_object_set_description (accessible, "Annotate with a horizontal line");


   // register w/ event observer to get changes in mode
   register_event(ChipsEventManager::ChangeStateEvent);
   register_event(ChipsEventManager::ChangeAddStateEvent);

}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiAnnotationToolbar::~ChipsGuiAnnotationToolbar()
{
  gtk_widget_destroy(_toolbar);
}


int ChipsGuiAnnotationToolbar::addpoint_cb(GObject *object, gpointer user_data)
{
   ChipsGuiAnnotationToolbar *tbar = static_cast <ChipsGuiAnnotationToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
   {
      tbar->invoke_mode_change(ChipsEventManager::PointAdd); 
   }
   return 0;
}

int ChipsGuiAnnotationToolbar::addlabel_cb(GObject *object, gpointer user_data)
{
   ChipsGuiAnnotationToolbar *tbar = static_cast <ChipsGuiAnnotationToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
   {
      tbar->invoke_mode_change(ChipsEventManager::LabelAdd);
   }
   return 0;
}

int ChipsGuiAnnotationToolbar::addline_cb(GObject *object, gpointer user_data)
{
   ChipsGuiAnnotationToolbar *tbar = static_cast <ChipsGuiAnnotationToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
   {
      tbar->invoke_mode_change(ChipsEventManager::PolyLineAdd);
   }
   return 0;
}

int ChipsGuiAnnotationToolbar::addvline_cb(GObject *object, gpointer user_data)
{
   ChipsGuiAnnotationToolbar *tbar = static_cast <ChipsGuiAnnotationToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
   {
      tbar->invoke_mode_change(ChipsEventManager::VLineAdd);
   }
   return 0;
}

int ChipsGuiAnnotationToolbar::addhline_cb(GObject *object, gpointer user_data)
{
   ChipsGuiAnnotationToolbar *tbar = static_cast <ChipsGuiAnnotationToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
   {
      tbar->invoke_mode_change(ChipsEventManager::HLineAdd);
   }
   return 0;
}



void ChipsGuiAnnotationToolbar::set_style(int style)
{
   if (style == 0)
      gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_ICONS);    
   else if (style == 1)
      gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_TEXT);    
   else if (style == 2)
      gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_BOTH);    
}

void ChipsGuiAnnotationToolbar::show()
{
   // set appearance based on loaded prefs 
   update_gui_prefs(); 

   gtk_widget_show(GTK_WIDGET(_toolbar));
   gtk_widget_hide (GTK_WIDGET(_emptyBtn));
}

void ChipsGuiAnnotationToolbar::hide()
{
   gtk_widget_hide(GTK_WIDGET(_toolbar));
   gtk_widget_hide (GTK_WIDGET(_emptyBtn));
}

int ChipsGuiAnnotationToolbar::get_height()
{
   int ht = 0; 
   
   if (_toolbar)   ht = _toolbar->allocation.height; 
   return ht; 
}

// event observer to handle mode change from event manager
void ChipsGuiAnnotationToolbar::execute_event (int event, void *data)
{
   g_signal_handler_block (_pointBtn, _pointBtn_id);
   g_signal_handler_block (_labelBtn, _labelBtn_id);
   g_signal_handler_block (_lineBtn, _lineBtn_id);
   g_signal_handler_block (_hlineBtn, _hlineBtn_id);
   g_signal_handler_block (_vlineBtn, _vlineBtn_id);


   switch (event)
   {
      case ChipsEventManager::ChangeStateEvent:
      {
         int* states = (int*)data;
         if (states[1] != ChipsEventManager::AddState)
            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_emptyBtn), TRUE);

      }
      break;
      case ChipsEventManager::ChangeAddStateEvent:
      {
         int state = *((int *)data);

         if (state == ChipsEventManager::PointAdd)
            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_pointBtn), TRUE);
         else if (state == ChipsEventManager::PolyLineAdd)
            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_lineBtn), TRUE);
         else if (state == ChipsEventManager::HLineAdd)
            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_hlineBtn), TRUE);
         else if (state == ChipsEventManager::VLineAdd)
            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_vlineBtn), TRUE);
         else
            gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_emptyBtn), TRUE);
      }
      break;
      default:
      break;
   }

   g_signal_handler_unblock (_pointBtn, _pointBtn_id);
   g_signal_handler_unblock (_labelBtn, _labelBtn_id);
   g_signal_handler_unblock (_lineBtn, _lineBtn_id);
   g_signal_handler_unblock (_hlineBtn, _hlineBtn_id);
   g_signal_handler_unblock (_vlineBtn, _vlineBtn_id);

}


// passes mode change event to event manager 
void ChipsGuiAnnotationToolbar::invoke_mode_change(int val)
{
   if (event_manager) 
   {
      double pos[2] = {0.0,0.0};
      event_manager->invoke_event(pos, ChipsEventManager::ChangeAddStateEvent,
         (void*)&val, this);
   }
}

void ChipsGuiAnnotationToolbar::update_gui_prefs()
{
   // reload latest gui settings 
   if (GuiSettings)
      _btnStyle = GuiSettings->get_tbar_style();
   set_style(_btnStyle); 
   
}

void ChipsGuiAnnotationToolbar::set_annotate_sensitivity (bool enable)
{
   if (_annotation_enabled == enable)
      return;

   gtk_widget_set_sensitive(GTK_WIDGET(_pointBtn), enable);
   gtk_widget_set_sensitive(GTK_WIDGET(_labelBtn), enable);  
   gtk_widget_set_sensitive(GTK_WIDGET(_lineBtn), enable);
   gtk_widget_set_sensitive(GTK_WIDGET(_hlineBtn), enable);
   gtk_widget_set_sensitive(GTK_WIDGET(_vlineBtn), enable);

   _annotation_enabled = enable;

}


