// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIANNOTATIONTOOLBAR_HH
#define CHIPSGUIANNOTATIONTOOLBAR_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsEventObserver.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;


//---------------------------------------------------------------------------

class ChipsGuiAnnotationToolbar: public ChipsEventObserver, public ChipsGuiTooltipObject
{
public:

  ChipsGuiAnnotationToolbar();
  ~ChipsGuiAnnotationToolbar();

  GtkWidget* base_widget() { return _toolbar; } 

  void set_style(int style); 
  void invoke_mode_change(int val); 

  virtual void execute_event (int event, void *data); 

  void update_gui_prefs();

  void show();
  void hide();
  int  get_height(); 

  void set_annotate_sensitivity (bool enable);

private:
  
  // mode buttons 
  GtkWidget*   _toolbar; 
  GtkToolItem* _emptyBtn;
  GtkToolItem* _pointBtn;
  GtkToolItem* _labelBtn;
  GtkToolItem* _lineBtn;
  GtkToolItem* _hlineBtn;
  GtkToolItem* _vlineBtn;
/*  GtkToolItem* _polygonBtn; 
*/
  int  _btnStyle; 

  bool _annotation_enabled;

  // signals for callbacks
  unsigned long _pointBtn_id;
  unsigned long _labelBtn_id;
  unsigned long _lineBtn_id;
  unsigned long _hlineBtn_id;
  unsigned long _vlineBtn_id;


  // callbacks 
  static int addpoint_cb(GObject *object, gpointer user_data);
  static int addlabel_cb(GObject *object, gpointer user_data);
  static int addline_cb(GObject *object, gpointer user_data);
  static int addhline_cb(GObject *object, gpointer user_data);
  static int addvline_cb(GObject *object, gpointer user_data);


};

#endif
