// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIAXISFRAME_HH
#define CHIPSGUIAXISFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <map>

#include "ChipsGuiPropFrame.hh"
#include "ChipsAxisAlgorithmFactory.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

class ChipsAxis;
class ChipsGuiColorWidget;
class ChipsGuiLabelWidget;
class ChipsGuiLevelsWidget;
class ChipsGuiPivotPointWidget;

//---------------------------------------------------------------------------

class ChipsGuiAxisFrame : public ChipsGuiPropFrame, public ChipsGuiTooltipObject
{
public:

  ChipsGuiAxisFrame();
  virtual ~ChipsGuiAxisFrame();

  static void color_callback(ChipsGuiColorWidget *widget, void *data);
  static void thick_callback(GtkWidget *widget, gpointer data);
  static void tickstyle_callback(GtkWidget *widget, gpointer data);
  static void automin_callback(GtkWidget *widget, gpointer data);
  static void automax_callback(GtkWidget *widget, gpointer data);
  static void padding_callback(GtkWidget *widget, gpointer data);
  static void label_text_callback(GtkWidget *widget, gpointer data);
  static void apply_axis_callback(GtkWidget *widget, gpointer data);
  static void change_tab_callback (GtkNotebook *notebook, 
                 GtkNotebookPage *notebook_page, int page, void *data);

  static void label_color_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_angle_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_text_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_font_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_fontstyle_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_align_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_size_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_position_change_callback(ChipsGuiLabelWidget *widget, void *data);

  static void label_visible_change_callback(ChipsGuiLabelWidget *widget, void *data);

  static void majortick_color_callback(ChipsGuiColorWidget *widget, void *data);
  static void majortick_ticks_callback(ChipsGuiLevelsWidget *widget, void *data);
  static void majortick_thick_callback(GtkWidget *widget, gpointer data);
  static void majortick_style_callback(GtkWidget *widget, gpointer data);
  static void majortick_length_callback(GtkWidget *widget, gpointer data);
  static void majortick_visible_callback(GtkWidget *widget, gpointer data);
  static void majortick_mode_callback(GtkWidget *widget, gpointer data);
  static void majortick_interval_callback(GtkWidget *widget, gpointer data);
  static void majortick_count_callback(GtkWidget *widget, gpointer data);

  static void minortick_color_callback(ChipsGuiColorWidget *widget, void *data);
  static void minortick_thick_callback(GtkWidget *widget, gpointer data);
  static void minortick_style_callback(GtkWidget *widget, gpointer data);
  static void minortick_length_callback(GtkWidget *widget, gpointer data);
  static void minortick_visible_callback(GtkWidget *widget, gpointer data);
  static void minortick_mode_callback(GtkWidget *widget, gpointer data);
  static void minortick_interval_callback(GtkWidget *widget, gpointer data);
  static void minortick_count_callback(GtkWidget *widget, gpointer data);

  static void majorgrid_color_callback(ChipsGuiColorWidget *widget, void *data);
  static void majorgrid_thick_callback(GtkWidget *widget, gpointer data);
  static void majorgrid_style_callback(GtkWidget *widget, gpointer data);
  static void majorgrid_visible_callback(GtkWidget *widget, gpointer data);

  static void minorgrid_color_callback(ChipsGuiColorWidget *widget, void *data);
  static void minorgrid_thick_callback(GtkWidget *widget, gpointer data);
  static void minorgrid_style_callback(GtkWidget *widget, gpointer data);
  static void minorgrid_visible_callback(GtkWidget *widget, gpointer data);

  static void ticklabel_color_callback (ChipsGuiColorWidget *widget, void *data);
  static void ticklabel_font_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_fontstyle_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_size_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_angle_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_offset_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_style_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_visible_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_align_callback(ChipsGuiPivotPointWidget *widget, void *data);
  static void ticklabel_halign_auto_callback(ChipsGuiPivotPointWidget *widget, void *data);
  static void ticklabel_valign_auto_callback(ChipsGuiPivotPointWidget *widget, void *data);

  static void ticklabel_format_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_format_custom_callback(GtkWidget *widget, gpointer data);
  static bool ticklabel_format_custom_focusout_callback(GtkWidget *widget, GdkEvent *event, gpointer data);

  static void depth_callback(GtkWidget *widget, gpointer data);
  static void visible_callback(GtkWidget *widget, gpointer data);

  virtual void initialize(ChipsId* id);
  virtual void build_form(); 

  virtual void display_subobject (V_SUBOBJECT_TYPE subtype);


private:

   void build_format_list (GtkListStore *format_list);
   void build_majortick_mode_list (GtkListStore *mode_list, ChipsAxisAlgorithmFactory::TransformType axis_type);
   void build_minortick_mode_list (GtkListStore *mode_list, ChipsAxisAlgorithmFactory::TransformType axis_type);
   void reset_arbitrary_tick_list ();
   void reload_arbitrary_tick_list(int page);
   bool get_tickmode_iter (GtkListStore *mode_list, GtkTreeIter &iter, int value);
   void set_apply_xy (bool enable);
   void set_engine_axis_property (void *val, int property, bool ignore=true);

   GtkWidget*  get_axistab_majortick_widget() { return _majortick_mode; };
   GtkWidget*  get_ticktab_majortick_widget() { return _majortick_modeB; };
   
   ChipsGuiColorWidget*   _color;
   GtkWidget*   _thick;
   GtkWidget*   _tickstyle;
   GtkWidget*   _automin;
   GtkWidget*   _automax; 
   GtkWidget*   _padding;
   GtkWidget*   _label_text;
   GtkWidget*   _majorgridvisible;
   GtkWidget*   _minorgridvisible;
   GtkWidget*   _apply_current;
   GtkWidget*   _apply_xy;

   // tick label
   ChipsGuiLabelWidget*   _label;
   
   // major tick
   ChipsGuiColorWidget* _majortick_color;
   GtkWidget*   _majortick_thick;
   GtkWidget*   _majortick_style;
   GtkWidget*   _majortick_length;
   GtkWidget*   _majortick_visible;
   ChipsGuiLevelsWidget* _majortick_ticks;
   GtkWidget*   _majortick_mode;
   GtkWidget*   _majortick_interval;
   GtkWidget*   _majortick_count;

   ChipsGuiLevelsWidget* _majortick_ticksB;
   GtkWidget*   _majortick_modeB;
   GtkWidget*   _majortick_intervalB;
   GtkWidget*   _majortick_countB;


   // minor tick
   ChipsGuiColorWidget* _minortick_color;
   GtkWidget*   _minortick_thick;
   GtkWidget*   _minortick_style;
   GtkWidget*   _minortick_length;
   GtkWidget*   _minortick_visible;
   GtkWidget*   _minortick_mode;
   GtkWidget*   _minortick_interval;
   GtkWidget*   _minortick_count;


   // grids
   ChipsGuiColorWidget* _majorgrid_color;
   GtkWidget*   _majorgrid_thick;
   GtkWidget*   _majorgrid_style;
   GtkWidget*   _majorgrid_visible;

   ChipsGuiColorWidget* _minorgrid_color;
   GtkWidget*   _minorgrid_thick;
   GtkWidget*   _minorgrid_style;
   GtkWidget*   _minorgrid_visible;

   // tick labels
   GtkWidget*   _ticklabel_font;
   GtkWidget*   _ticklabel_size;
   GtkWidget*   _ticklabel_bold;
   GtkWidget*   _ticklabel_italic;
   ChipsGuiColorWidget*  _ticklabel_color;
   GtkWidget*   _ticklabel_offset;
   GtkWidget*   _ticklabel_angle;
   GtkWidget*   _ticklabel_style;
   GtkWidget*   _ticklabel_visible;
   GtkWidget*   _ticklabel_format_box;
   GtkWidget*   _ticklabel_format_custom;
   ChipsGuiPivotPointWidget*  _ticklabel_alignment;

   GtkWidget*   _depth;
   GtkWidget*   _visible;

   GtkListStore *_format_list;
   GtkListStore *_majortick_mode_list;
   GtkListStore *_majortick_mode_listB;
   GtkListStore *_minortick_mode_list;


   unsigned long _thick_id;
   unsigned long _tickstyle_id;
   unsigned long _automin_id;
   unsigned long _automax_id;
   unsigned long _padding_id;
   unsigned long _majorgridvisible_id;
   unsigned long _minorgridvisible_id; 
   unsigned long _apply_current_id;
   unsigned long _apply_xy_id;
   unsigned long _label_text_id;
   unsigned long _majortick_thick_id;
   unsigned long _majortick_style_id;
   unsigned long _majortick_length_id;
   unsigned long _majortick_visible_id;
   unsigned long _majortick_mode_id;
   unsigned long _majortick_modeB_id;
   unsigned long _majortick_interval_id;
   unsigned long _majortick_intervalB_id;
   unsigned long _majortick_count_id;
   unsigned long _majortick_countB_id;
   unsigned long _minortick_thick_id;
   unsigned long _minortick_style_id;
   unsigned long _minortick_length_id;
   unsigned long _minortick_visible_id;
   unsigned long _minortick_mode_id;
   unsigned long _minortick_interval_id;
   unsigned long _minortick_count_id;
   unsigned long _majorgrid_thick_id;
   unsigned long _majorgrid_style_id;
   unsigned long _majorgrid_visible_id;
   unsigned long _minorgrid_thick_id;
   unsigned long _minorgrid_style_id;
   unsigned long _minorgrid_visible_id;
   unsigned long _ticklabel_font_id;
   unsigned long _ticklabel_bold_id;
   unsigned long _ticklabel_italic_id;
   unsigned long _ticklabel_size_id;
   unsigned long _ticklabel_angle_change_id;
   unsigned long _ticklabel_offset_id;
   unsigned long _ticklabel_style_id;
   unsigned long _ticklabel_visible_change_id;
   unsigned long _ticklabel_format_box_id;
   unsigned long _ticklabel_format_custom_id;
   unsigned long _ticklabel_format_custom_focusout_id;
   unsigned long _depth_id;
   unsigned long _visible_id;


   float        _thicknessval;
   V_TICKSTYLE_TYPE _tickstyleval;
   bool         _autominval;
   bool         _automaxval;
   double       _paddingval;
   string       _labelval;
   int          _depthval;
   string       _colorval;
   string       _majortick_colorval;
   string       _minortick_colorval;
   float        _majortick_thicknessval;
   float        _minortick_thicknessval;
   V_TICKSTYLE_TYPE _majortick_styleval;
   V_TICKSTYLE_TYPE _minortick_styleval;
   int          _majortick_numval;
   int          _minortick_numval;
   double       _majortick_intval;
   double       _minortick_intval;
   string       _majortick_modeval;
   string       _minortick_modeval;
   bool         _majortick_visibleval;
   bool         _minortick_visibleval;
   bool         _visibleval;
   int          _majortick_lengthval;
   int          _minortick_lengthval;
   float        _label_offsetparval;
   float        _label_offsetperpval;
   int          _ticklabel_offsetval;
   string       _ticklabel_fontval;
   string       _ticklabel_fontstyleval;
   int          _ticklabel_sizeval;
   string       _ticklabel_colorval;
   float        _ticklabel_angleval;
   float        _ticklabel_halignval;
   float        _ticklabel_valignval;
   bool         _ticklabel_visibleval;
   V_TICKSTYLE_TYPE _ticklabel_styleval;
   string       _ticklabel_formatval;
   bool         _label_visibleval;
   string       _label_fontval;
   string       _label_fontstyleval;
   int          _label_sizeval;
   string       _label_colorval;
   float        _label_angleval;
   float        _label_halignval;
   float        _label_valignval;
   string       _majorgrid_colorval;
   V_LINE_TYPE  _majorgrid_styleval;
   bool         _majorgrid_visibleval;
   float        _majorgrid_thicknessval;
   string       _minorgrid_colorval;
   V_LINE_TYPE  _minorgrid_styleval;
   bool         _minorgrid_visibleval;
   float        _minorgrid_thicknessval;

   int          _label_page_num;
   map<unsigned int, bool> _apply_xy_map;

   ChipsId      _set_cid;

};

#endif
