// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiColorBarFrame.hh"
#include "ChipsGuiColorBarPropWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsColorBar.hh"
#include "ChipsId.hh"
#include "ChipsCmdManager.hh"
#include "config.h"

#include <algorithm>

#if HAVE_IEEEFP_H 
#include <ieeefp.h>
#endif 

using namespace std;

extern ChipsGuiMain* chipsGui;

class ChipsColorBarMoveCallback : public ChipsGuiPropFrameCallback
{
   public:
      virtual void execute (ChipsEngine *eng, ChipsId &cid, void *data)
      {
         double *pos = static_cast<double *>(data);
         if ((pos != NULL) && eng)
            eng->move(&cid, chips_colorbar, pos[0], pos[1], V_MOVE_ABSOLUTE);
      }
};



// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiColorBarFrame::ChipsGuiColorBarFrame():ChipsGuiPropFrame(),ChipsGuiTooltipObject()
{

   _cbarprop = NULL;
   _object_states = STATE_CHNG_PROP_CLBAR;
   _object_states |= STATE_CHNG_AXS_FMT;

}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiColorBarFrame::~ChipsGuiColorBarFrame()
{
//  gtk_widget_destroy(_frame);

   if (_cbarprop)
      delete _cbarprop;
}

void ChipsGuiColorBarFrame::build_form()
{

   _frame = gtk_frame_new("ColorBar Properties");

   _cbarprop = new ChipsGuiColorBarPropWidget ();
   _cbarprop->build_form ();

   gtk_container_add(GTK_CONTAINER(_frame), _cbarprop->base_widget ());

   _cbarprop->add_callback ("change_perpendicular_offset", new ChipsGuiColorBarPropWidgetCallback (label_perpendicular_offset_callback, this));
   _cbarprop->add_callback ("change_parallel_offset", new ChipsGuiColorBarPropWidgetCallback (label_parallel_offset_callback, this));

   _cbarprop->add_callback ("change_ticklabel_visible", new ChipsGuiColorBarPropWidgetCallback (ticklabel_visible_callback, this));
   _cbarprop->add_callback ("change_ticklabel_halign", new ChipsGuiColorBarPropWidgetCallback (ticklabel_halign_callback, this));
   _cbarprop->add_callback ("change_ticklabel_valign", new ChipsGuiColorBarPropWidgetCallback (ticklabel_valign_callback, this));
   _cbarprop->add_callback ("change_ticklabel_size", new ChipsGuiColorBarPropWidgetCallback (ticklabel_size_callback, this));
   _cbarprop->add_callback ("change_ticklabel_offset", new ChipsGuiColorBarPropWidgetCallback (ticklabel_offset_callback, this));
   _cbarprop->add_callback ("change_ticklabel_fontstyle", new ChipsGuiColorBarPropWidgetCallback (ticklabel_fontstyle_callback, this));
   _cbarprop->add_callback ("change_ticklabel_font", new ChipsGuiColorBarPropWidgetCallback (ticklabel_font_callback, this));
   _cbarprop->add_callback ("change_ticklabel_color", new ChipsGuiColorBarPropWidgetCallback (ticklabel_color_callback, this));
   _cbarprop->add_callback ("change_ticklabel_angle", new ChipsGuiColorBarPropWidgetCallback (ticklabel_angle_callback, this));

   _cbarprop->add_callback ("change_tick_location", new ChipsGuiColorBarPropWidgetCallback (tick_location_callback, this));
   _cbarprop->add_callback ("change_tick_visible", new ChipsGuiColorBarPropWidgetCallback (tick_visible_callback, this));
   _cbarprop->add_callback ("change_tick_style", new ChipsGuiColorBarPropWidgetCallback (tick_style_callback, this));
   _cbarprop->add_callback ("change_tick_color", new ChipsGuiColorBarPropWidgetCallback (tick_color_callback, this));
   _cbarprop->add_callback ("change_tick_length", new ChipsGuiColorBarPropWidgetCallback (tick_length_callback, this));
   _cbarprop->add_callback ("change_tick_thickness", new ChipsGuiColorBarPropWidgetCallback (thick_callback, this));
   _cbarprop->add_callback ("change_tick_mode", new ChipsGuiColorBarPropWidgetCallback (tickmode_callback, this));

   _cbarprop->add_callback ("change_halign", new ChipsGuiColorBarPropWidgetCallback (halign_change_callback, this));
   _cbarprop->add_callback ("change_valign", new ChipsGuiColorBarPropWidgetCallback (valign_change_callback, this));
   _cbarprop->add_callback ("change_border_visible", new ChipsGuiColorBarPropWidgetCallback (border_visible_callback, this));
   _cbarprop->add_callback ("change_visible", new ChipsGuiColorBarPropWidgetCallback (visible_callback, this));
   _cbarprop->add_callback ("change_depth", new ChipsGuiColorBarPropWidgetCallback (depth_callback, this));
   _cbarprop->add_callback ("change_orientation", new ChipsGuiColorBarPropWidgetCallback (orientation_callback, this));
   _cbarprop->add_callback ("change_width", new ChipsGuiColorBarPropWidgetCallback (width_callback, this));
  _cbarprop->add_callback ("change_length", new ChipsGuiColorBarPropWidgetCallback (length_callback, this));

  _cbarprop->add_callback ("change_label_color", new ChipsGuiColorBarPropWidgetCallback (label_color_change_callback, this));
  _cbarprop->add_callback ("change_label_visible", new ChipsGuiColorBarPropWidgetCallback (label_visible_change_callback, this));
  _cbarprop->add_callback ("change_label_size", new ChipsGuiColorBarPropWidgetCallback (label_size_change_callback, this));
  _cbarprop->add_callback ("change_label_valign", new ChipsGuiColorBarPropWidgetCallback (label_valign_change_callback, this));
  _cbarprop->add_callback ("change_label_halign", new ChipsGuiColorBarPropWidgetCallback (label_halign_change_callback, this));
  _cbarprop->add_callback ("change_label_angle", new ChipsGuiColorBarPropWidgetCallback (label_angle_change_callback, this));
  _cbarprop->add_callback ("change_label_font", new ChipsGuiColorBarPropWidgetCallback (label_font_change_callback, this));
  _cbarprop->add_callback ("change_label_fontstyle", new ChipsGuiColorBarPropWidgetCallback (label_fontstyle_change_callback, this));
  _cbarprop->add_callback ("change_text", new ChipsGuiColorBarPropWidgetCallback (label_text_change_callback, this));


   _cbarprop->add_callback ("change_position", new ChipsGuiColorBarPropWidgetCallback (position_callback, this));
   _cbarprop->add_callback ("change_coordinate", new ChipsGuiColorBarPropWidgetCallback (coordinate_callback, this));


}

void ChipsGuiColorBarFrame::thick_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float val = widget->get_tick_thickness ();

   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICK_THICKNESS);
}

void ChipsGuiColorBarFrame::tickmode_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   string text = widget->get_tick_mode ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(text.c_str())), SET_CBAR_TICK_MODE);

}

void ChipsGuiColorBarFrame::width_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float val = widget->get_width ();
   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_WIDTH);

}

void ChipsGuiColorBarFrame::length_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float val = widget->get_length ();
   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_LENGTH);
}

void ChipsGuiColorBarFrame::orientation_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_orientation ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_ORIENTATION);

}



void ChipsGuiColorBarFrame::label_color_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   string color = widget->get_label_color ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(color.c_str())), SET_CBAR_LABEL_COLOR);

}

void ChipsGuiColorBarFrame::label_angle_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float angle = widget->get_label_angle ();
   if (cbarFrame) cbarFrame->set_engine_property(static_cast<void*>(&angle),
                                 SET_CBAR_LABEL_ANGLE);

}

void ChipsGuiColorBarFrame::label_text_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   string text = widget->get_text ();

   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(text.c_str())), SET_CBAR_TEXT);

}

void ChipsGuiColorBarFrame::label_font_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   string font = widget->get_label_font ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(font.c_str())), SET_CBAR_LABEL_FONT);

}

void ChipsGuiColorBarFrame::label_fontstyle_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_label_fontstyle ();
   const char *fontstyle = v_fontstyle_to_string((V_FONTSTYLE_TYPE)val);
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(fontstyle)), SET_CBAR_LABEL_FONTSTYLE);

}

void ChipsGuiColorBarFrame::label_halign_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float halign = widget->get_label_halign ();
   if (cbarFrame)
     cbarFrame->set_engine_property(static_cast<void*>(&halign),
                                 SET_CBAR_LABEL_HALIGN);
}

void ChipsGuiColorBarFrame::label_valign_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float valign = widget->get_label_valign ();
   if (cbarFrame)
     cbarFrame->set_engine_property(static_cast<void*>(&valign),
                                 SET_CBAR_LABEL_VALIGN);
}


void ChipsGuiColorBarFrame::label_size_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int size = widget->get_label_size ();
   if (cbarFrame) cbarFrame->set_engine_property(static_cast<void*>(&size),
                                 SET_CBAR_LABEL_SIZE);
}

void ChipsGuiColorBarFrame::label_perpendicular_offset_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float offset = widget->get_perpendicular_offset ();

   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(&offset),
                                    SET_CBAR_OFFSET_PERP);
}

void ChipsGuiColorBarFrame::label_parallel_offset_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float offset = widget->get_parallel_offset ();

   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(&offset),
                                    SET_CBAR_OFFSET_PAR);
}



void ChipsGuiColorBarFrame::label_visible_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   bool visible = widget->get_label_visible ();
   if (cbarFrame) cbarFrame->set_engine_property(static_cast<void*>(&visible),
                                 SET_CBAR_LABEL_VIS);

}


void ChipsGuiColorBarFrame::tick_color_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   string color = widget->get_tick_color ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(color.c_str())), SET_CBAR_TICK_COLOR);
   
}


void ChipsGuiColorBarFrame::tick_style_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_tick_style ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICK_STYLE);

}

void ChipsGuiColorBarFrame::tick_length_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_tick_length ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICK_LENGTH);

}

void ChipsGuiColorBarFrame::tick_visible_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = (int)widget->get_tick_visible ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICK_VIS);

}

void ChipsGuiColorBarFrame::tick_location_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_tick_location();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICK_LOCATION);
}

void ChipsGuiColorBarFrame::ticklabel_color_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   string color = widget->get_ticklabel_color ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(color.c_str())), SET_CBAR_TICKLABEL_COLOR);

}

void ChipsGuiColorBarFrame::ticklabel_font_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   string font = widget->get_ticklabel_font ();

   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(font.c_str ())), SET_CBAR_TICKLABEL_FONT);

}


void ChipsGuiColorBarFrame::ticklabel_fontstyle_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
     
   int style = widget->get_ticklabel_fontstyle ();
   const char *str_style = v_fontstyle_to_string((V_FONTSTYLE_TYPE)style);
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(const_cast<char *>(str_style)), SET_CBAR_TICKLABEL_FNTSTL);

}

void ChipsGuiColorBarFrame::ticklabel_size_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);

   int size = widget->get_ticklabel_size ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&size), SET_CBAR_TICKLABEL_SIZE);
}

void ChipsGuiColorBarFrame::ticklabel_angle_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float val = widget->get_ticklabel_angle ();

   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICKLABEL_ANGLE);
}

void ChipsGuiColorBarFrame::ticklabel_offset_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_ticklabel_offset ();

   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICKLABEL_OFFSET);

}

void ChipsGuiColorBarFrame::ticklabel_halign_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float halign;
   ChipsColorBar *cbar;
   bool auto_align;

   auto_align = widget->get_ticklabel_auto_halign ();
   halign = (auto_align) ? AUTO_ALIGN : widget->get_ticklabel_halign ();

   cbarFrame->set_engine_property(static_cast<void*>(&halign), SET_CBAR_TICKLABEL_HALIGN);

   if (auto_align)
   {
      cbar = dynamic_cast<ChipsColorBar *>(cbarFrame->_eng->get_object (&(cbarFrame->_cid), chips_colorbar));

      if (cbar)
         halign = cbar->get_ticklabel_halign (true);
      widget->set_ticklabel_halign (halign);
   }
}

void ChipsGuiColorBarFrame::ticklabel_valign_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float valign;
   ChipsColorBar *cbar;
   bool auto_align;

   auto_align = widget->get_ticklabel_auto_valign ();
   valign = (auto_align) ? AUTO_ALIGN : widget->get_ticklabel_valign ();

   cbarFrame->set_engine_property(static_cast<void*>(&valign), SET_CBAR_TICKLABEL_VALIGN);

   if (auto_align)
   {
      cbar = dynamic_cast<ChipsColorBar *>(cbarFrame->_eng->get_object (&(cbarFrame->_cid), chips_colorbar));

      if (cbar)
         valign = cbar->get_ticklabel_valign (true);
      widget->set_ticklabel_valign (valign);
   }
}


void ChipsGuiColorBarFrame::ticklabel_visible_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   bool val = widget->get_ticklabel_visible ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_TICKLABEL_VIS);

}

void ChipsGuiColorBarFrame::halign_change_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float val = widget->get_halign ();

   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_HALIGN);
}

void ChipsGuiColorBarFrame::valign_change_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   float val = widget->get_valign ();

   if (cbarFrame)
      cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_VALIGN);
}

void ChipsGuiColorBarFrame::depth_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_depth ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_DEPTH);

}

void ChipsGuiColorBarFrame::visible_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_visibility ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_VISIBLE);

}

void ChipsGuiColorBarFrame::border_visible_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   int val = widget->get_border_visibility ();
   if (cbarFrame)  cbarFrame->set_engine_property(static_cast<void*>(&val), SET_CBAR_BORDERVIS);

}

void ChipsGuiColorBarFrame::position_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);
   ChipsColorBarMoveCallback move_callback;
   double pos[2];

   if (!cbarFrame)  return; //ensure we have needed pointer
   widget->get_position (pos[0],pos[1]);

   cbarFrame->update_property (&move_callback, &pos[0]);

}

void ChipsGuiColorBarFrame::coordinate_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarFrame *cbarFrame = static_cast <ChipsGuiColorBarFrame*>(data);

   if (!cbarFrame)  return; //ensure we have needed pointer

   COORD_SYS coordsys = (COORD_SYS)widget->get_coordinate ();
   ChipsId cid;
   string coordid = "";

   cbarFrame->_eng->get_current_coord (&cid);

   if ((coordsys == DATA) && (cid.get_coord_sys () != DATA))
      chipsGui->log_error ("Data coordinate system does not exists.");
   else
      coordid = cid.get_coord_id ();

   coordid = ChipsCoordinate::extract_id (coordsys, coordid);

   cbarFrame->_eng->change_annotation_coordinate (&cid, chips_colorbar, coordsys, coordid);
}


void ChipsGuiColorBarFrame::initialize(ChipsId* id)
{
   V_FONTSTYLE_TYPE tmp_fontstyle;
   ChipsColorBar *cbar = NULL;
   string tmp_tickstyle;
   COORD_SYS coordsys;
   string coordid;
   ChipsVector pos(3);

   float      widthval;
   float      offset_perpval;
   float      offset_parval;
   bool       ticklabel_visibleval;
   float      ticklabel_halignval;
   float      ticklabel_valignval;
   int        ticklabel_sizeval;
   int        ticklabel_offsetval;
   string     ticklabel_fontstyleval;
   string     ticklabel_fontval;
   string     ticklabel_colorval;
   float      ticklabel_angleval;
   string     textval;
   V_TICKSTYLE_TYPE tick_locationval;
   bool       tick_visibleval;
   V_TICKSTYLE_TYPE tick_styleval;
   string     tick_colorval;
   int        tick_lengthval;
   float      tick_thicknessval;
//   V_TICKSTYLE_TYPE label_locationval;
   bool       label_visibleval;
   int        label_sizeval;
   float      label_valignval;
   float      label_halignval;
   string     label_fontstyleval;
   string     label_fontval;
   string     label_colorval;
   float      label_angleval;
   string     tick_modeval;
   float      lengthval;
   float      valignval;
   float      halignval;
   bool       border_visibleval;
   int        visibleval;
   int        depthval;
   int        orientationval;

   bool       ticklabel_halign_auto = false;
   bool       ticklabel_valign_auto = false;

   if (!id)  return; 

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   cbar = dynamic_cast<ChipsColorBar *>(_eng->get_object (id, chips_colorbar));

   if (!cbar)
      return;

   if (&_cid != id)
   {
      _cid.reset_window_id(id->get_window_id()); 
      _cid.set_frame_id(id->get_frame_id()); 
      _cid.set_plot_id(id->get_plot_id()); 
      _cid.set_axis_id(id->get_axis_id()); 
   }

   _eng->get_property(&_cid, (void*)&depthval, SET_CBAR_DEPTH);
   _eng->get_property(&_cid, (void*)&visibleval, SET_CBAR_VISIBLE);
   _eng->get_property(&_cid, (void*)&border_visibleval, SET_CBAR_BORDERVIS);
   _eng->get_property(&_cid, (void*)&halignval, SET_CBAR_HALIGN);
   _eng->get_property(&_cid, (void*)&valignval, SET_CBAR_VALIGN);
   _eng->get_property(&_cid, (void*)&lengthval, SET_CBAR_LENGTH);
   _eng->get_property(&_cid, (void*)&orientationval, SET_CBAR_ORIENTATION);
   _eng->get_property(&_cid, (void*)&tick_modeval,SET_CBAR_TICK_MODE);
   _eng->get_property(&_cid, (void*)&label_angleval,SET_CBAR_LABEL_ANGLE);
   _eng->get_property(&_cid, (void*)&label_colorval,SET_CBAR_LABEL_COLOR);
   _eng->get_property(&_cid, (void*)&label_fontval,SET_CBAR_LABEL_FONT);
   _eng->get_property(&_cid, (void*)&label_fontstyleval,SET_CBAR_LABEL_FONTSTYLE);
   _eng->get_property(&_cid, (void*)&label_halignval,SET_CBAR_LABEL_HALIGN);
   _eng->get_property(&_cid, (void*)&label_valignval,SET_CBAR_LABEL_VALIGN);
   _eng->get_property(&_cid, (void*)&label_sizeval,SET_CBAR_LABEL_SIZE);
   _eng->get_property(&_cid, (void*)&label_visibleval,SET_CBAR_LABEL_VIS);
//   _eng->get_property(&_cid, (void*)&tmp_tickstyle,SET_CBAR_LABEL_LOCATION);
//   string_to_v_tickstyle (tmp_tickstyle.c_str (), &label_locationval);
   _eng->get_property(&_cid, (void*)&textval,SET_CBAR_TEXT);
   _eng->get_property(&_cid, (void*)&tick_thicknessval,SET_CBAR_TICK_THICKNESS);
   _eng->get_property(&_cid, (void*)&tick_lengthval,SET_CBAR_TICK_LENGTH);
   _eng->get_property(&_cid, (void*)&tick_colorval,SET_CBAR_TICK_COLOR);
   _eng->get_property(&_cid, (void*)&tmp_tickstyle,SET_CBAR_TICK_STYLE);
   string_to_v_tickstyle (tmp_tickstyle.c_str (), &tick_styleval);
   _eng->get_property(&_cid, (void*)&tick_visibleval,SET_CBAR_TICK_VIS);
   _eng->get_property(&_cid, (void*)&tmp_tickstyle,SET_CBAR_TICK_LOCATION);
   string_to_v_tickstyle (tmp_tickstyle.c_str (), &tick_locationval);
   _eng->get_property(&_cid, (void*)&ticklabel_angleval,SET_CBAR_TICKLABEL_ANGLE);
   _eng->get_property(&_cid, (void*)&ticklabel_colorval,SET_CBAR_TICKLABEL_COLOR);
   _eng->get_property(&_cid, (void*)&ticklabel_fontval,SET_CBAR_TICKLABEL_FONT);
   _eng->get_property(&_cid, (void*)&ticklabel_fontstyleval,SET_CBAR_TICKLABEL_FNTSTL);
   _eng->get_property(&_cid, (void*)&ticklabel_offsetval,SET_CBAR_TICKLABEL_OFFSET);
   _eng->get_property(&_cid, (void*)&ticklabel_sizeval,SET_CBAR_TICKLABEL_SIZE);
   _eng->get_property(&_cid, (void*)&ticklabel_valignval,SET_CBAR_TICKLABEL_VALIGN);
   _eng->get_property(&_cid, (void*)&ticklabel_halignval,SET_CBAR_TICKLABEL_HALIGN);
   _eng->get_property(&_cid, (void*)&ticklabel_visibleval,SET_CBAR_TICKLABEL_VIS);
   _eng->get_property(&_cid, (void*)&offset_parval,SET_CBAR_OFFSET_PAR);
   _eng->get_property(&_cid, (void*)&offset_perpval,SET_CBAR_OFFSET_PERP);
   _eng->get_property(&_cid, (void*)&widthval,SET_CBAR_WIDTH);
   _eng->get_object_coordinfo (&_cid, chips_colorbar, coordsys, coordid);
   _eng->get_annotation_position (&_cid, chips_colorbar, pos);

   if (label_valignval == AUTO_ALIGN)
      label_valignval = cbar->get_label_valign (true);
   
   if (label_halignval == AUTO_ALIGN)
      label_halignval = cbar->get_label_halign (true);

   if (ticklabel_valignval == AUTO_ALIGN)
   {
      ticklabel_valignval = cbar->get_ticklabel_valign (true);
      ticklabel_valign_auto = true;
   }

   if (ticklabel_halignval == AUTO_ALIGN)
   {
      ticklabel_halignval = cbar->get_ticklabel_halign (true);
      ticklabel_halign_auto = true;
   }

   _cbarprop->set_depth (depthval);
   _cbarprop->set_visibility (visibleval);
   _cbarprop->set_border_visibility (border_visibleval);
   _cbarprop->set_halign (halignval);
   _cbarprop->set_valign (valignval);
   _cbarprop->set_length (lengthval);
   _cbarprop->set_orientation (orientationval);
   _cbarprop->set_tick_mode (tick_modeval);
   _cbarprop->set_label_angle (label_angleval);
   _cbarprop->set_label_color (label_colorval);
   _cbarprop->set_label_font (label_fontval);

   string_to_v_fontstyle(label_fontstyleval.c_str (), &tmp_fontstyle);
   _cbarprop->set_label_fontstyle ((int)tmp_fontstyle);
   _cbarprop->set_label_halign (label_halignval);
   _cbarprop->set_label_valign (label_valignval);
   _cbarprop->set_label_size (label_sizeval);
   _cbarprop->set_label_visible (label_visibleval);
//   _cbarprop->set_label_location ((int)label_locationval);
   _cbarprop->set_text (textval);
   _cbarprop->set_tick_thickness (tick_thicknessval);
   _cbarprop->set_tick_length (tick_lengthval);
   _cbarprop->set_tick_color (tick_colorval);
   _cbarprop->set_tick_style ((int)tick_styleval);
   _cbarprop->set_tick_visible (tick_visibleval);
   _cbarprop->set_tick_location ((int)tick_locationval);
   _cbarprop->set_ticklabel_angle (ticklabel_angleval);
   _cbarprop->set_ticklabel_color (ticklabel_colorval);
   _cbarprop->set_ticklabel_font (ticklabel_fontval);

   string_to_v_fontstyle(ticklabel_fontstyleval.c_str (), &tmp_fontstyle);
   _cbarprop->set_ticklabel_fontstyle (tmp_fontstyle);
   _cbarprop->set_ticklabel_offset (ticklabel_offsetval);
   _cbarprop->set_ticklabel_size (ticklabel_sizeval);
   _cbarprop->set_ticklabel_valign (ticklabel_valignval);
   _cbarprop->set_ticklabel_auto_valign (ticklabel_valign_auto);
   _cbarprop->set_ticklabel_halign (ticklabel_halignval);
   _cbarprop->set_ticklabel_auto_halign (ticklabel_halign_auto);
   _cbarprop->set_ticklabel_visible (ticklabel_visibleval);
   _cbarprop->set_offsets (offset_parval, offset_perpval);
   _cbarprop->set_width (widthval);
   _cbarprop->set_position (pos[0],pos[1]);
   _cbarprop->set_coordinate ((int)coordsys);

   // get tick formats
   int xmode = 0, ymode = 0;
   string xformat, yformat;
   ChipsId tmpid (_cid);
   tmpid.set_axis_id(_cid.get_xaxis_id());
   tmpid.set_axis_dim (X_AXIS);
   _eng->get_property(&tmpid, (void*)&xformat, SET_AXIS_TICKFORMAT);
   tmpid.set_axis_id(_cid.get_yaxis_id());
   tmpid.set_axis_dim (Y_AXIS);
   _eng->get_property(&tmpid, (void*)&yformat, SET_AXIS_TICKFORMAT);

   transform(xformat.begin(), xformat.end(), xformat.begin(), ::tolower);
   transform(yformat.begin(), yformat.end(), yformat.begin(), ::tolower);

   if (xformat.find("ra") != string::npos ||
       xformat.find("dec") != string::npos)
      xmode = 1;
   if (yformat.find("ra") != string::npos ||
       yformat.find("dec") != string::npos)
      ymode = 1;

   _cbarprop->set_position_format (xmode, ymode);
   
   // enable/disable the data coord pulldown entry based on existing coords
   ChipsId mcid;
   mcid.set_coord_sys(DATA);
   if (_eng->check_coordinate_exists(&mcid))  
      _cbarprop->enable_data_coordinate();
   else 
      _cbarprop->disable_data_coordinate();
}

void ChipsGuiColorBarFrame::display_subobject (V_SUBOBJECT_TYPE subtype)
{
  _cbarprop->display_subobject (subtype);
}

