// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiColorBarPrefFrame.hh"
#include "ChipsGuiColorBarPropWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsCmdManager.hh"
#include "ChipsDefines.hh"
#include "config.h"

#include <algorithm>
#include <sstream>

#if HAVE_IEEEFP_H 
#include <ieeefp.h>
#endif 

using namespace std;

extern ChipsGuiMain* chipsGui;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiColorBarPrefFrame::ChipsGuiColorBarPrefFrame():ChipsGuiPrefFrame(),ChipsGuiTooltipObject()
{

   _cbarprop = NULL;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiColorBarPrefFrame::~ChipsGuiColorBarPrefFrame()
{
//  gtk_widget_destroy(_frame);

   if (_cbarprop)
      delete _cbarprop;
}

void ChipsGuiColorBarPrefFrame::build_form()
{

   _frame = gtk_frame_new("ColorBar Properties");

   _cbarprop = new ChipsGuiColorBarPropWidget ();
   _cbarprop->build_form ();

   _cbarprop->disable_position ();
   _cbarprop->disable_coordinate ();
   _cbarprop->disable_visibility ();

   gtk_container_add(GTK_CONTAINER(_frame), _cbarprop->base_widget ());

   _cbarprop->add_callback ("change_perpendicular_offset", new ChipsGuiColorBarPropWidgetCallback (label_perpendicular_offset_callback, this));
   _cbarprop->add_callback ("change_parallel_offset", new ChipsGuiColorBarPropWidgetCallback (label_parallel_offset_callback, this));
   _cbarprop->add_callback ("change_ticklabel_visible", new ChipsGuiColorBarPropWidgetCallback (ticklabel_visible_callback, this));
   _cbarprop->add_callback ("change_ticklabel_halign", new ChipsGuiColorBarPropWidgetCallback (ticklabel_halign_callback, this));
   _cbarprop->add_callback ("change_ticklabel_valign", new ChipsGuiColorBarPropWidgetCallback (ticklabel_valign_callback, this));
   _cbarprop->add_callback ("change_ticklabel_size", new ChipsGuiColorBarPropWidgetCallback (ticklabel_size_callback, this));
   _cbarprop->add_callback ("change_ticklabel_offset", new ChipsGuiColorBarPropWidgetCallback (ticklabel_offset_callback, this));
   _cbarprop->add_callback ("change_ticklabel_fontstyle", new ChipsGuiColorBarPropWidgetCallback (ticklabel_fontstyle_callback, this));
   _cbarprop->add_callback ("change_ticklabel_font", new ChipsGuiColorBarPropWidgetCallback (ticklabel_font_callback, this));
   _cbarprop->add_callback ("change_ticklabel_color", new ChipsGuiColorBarPropWidgetCallback (ticklabel_color_callback, this));
   _cbarprop->add_callback ("change_ticklabel_angle", new ChipsGuiColorBarPropWidgetCallback (ticklabel_angle_callback, this));

   _cbarprop->add_callback ("change_tick_location", new ChipsGuiColorBarPropWidgetCallback (tick_location_callback, this));
   _cbarprop->add_callback ("change_tick_visible", new ChipsGuiColorBarPropWidgetCallback (tick_visible_callback, this));
   _cbarprop->add_callback ("change_tick_style", new ChipsGuiColorBarPropWidgetCallback (tick_style_callback, this));
   _cbarprop->add_callback ("change_tick_color", new ChipsGuiColorBarPropWidgetCallback (tick_color_callback, this));
   _cbarprop->add_callback ("change_tick_length", new ChipsGuiColorBarPropWidgetCallback (tick_length_callback, this));
   _cbarprop->add_callback ("change_tick_thickness", new ChipsGuiColorBarPropWidgetCallback (thick_callback, this));
   _cbarprop->add_callback ("change_tick_mode", new ChipsGuiColorBarPropWidgetCallback (tickmode_callback, this));

   _cbarprop->add_callback ("change_text", new ChipsGuiColorBarPropWidgetCallback (label_text_change_callback, this));
   _cbarprop->add_callback ("change_label_visible", new ChipsGuiColorBarPropWidgetCallback (label_visible_change_callback, this));
   _cbarprop->add_callback ("change_label_size", new ChipsGuiColorBarPropWidgetCallback (label_size_change_callback, this));
   _cbarprop->add_callback ("change_label_valign", new ChipsGuiColorBarPropWidgetCallback (label_valign_change_callback, this));
   _cbarprop->add_callback ("change_label_halign", new ChipsGuiColorBarPropWidgetCallback (label_halign_change_callback, this));
   _cbarprop->add_callback ("change_label_angle", new ChipsGuiColorBarPropWidgetCallback (label_angle_change_callback, this));
   _cbarprop->add_callback ("change_label_font", new ChipsGuiColorBarPropWidgetCallback (label_font_change_callback, this));
   _cbarprop->add_callback ("change_label_fontstyle", new ChipsGuiColorBarPropWidgetCallback (label_fontstyle_change_callback, this));
   _cbarprop->add_callback ("change_label_color", new ChipsGuiColorBarPropWidgetCallback (label_color_change_callback, this));


   _cbarprop->add_callback ("change_halign", new ChipsGuiColorBarPropWidgetCallback (halign_change_callback, this));
   _cbarprop->add_callback ("change_valign", new ChipsGuiColorBarPropWidgetCallback (valign_change_callback, this));
   _cbarprop->add_callback ("change_border_visible", new ChipsGuiColorBarPropWidgetCallback (border_visible_callback, this));
   _cbarprop->add_callback ("change_depth", new ChipsGuiColorBarPropWidgetCallback (depth_callback, this));
   _cbarprop->add_callback ("change_orientation", new ChipsGuiColorBarPropWidgetCallback (orientation_callback, this));
   _cbarprop->add_callback ("change_width", new ChipsGuiColorBarPropWidgetCallback (width_callback, this));
   _cbarprop->add_callback ("change_length", new ChipsGuiColorBarPropWidgetCallback (length_callback, this));


/*   _cbarprop->add_callback ("change_coordinate", new ChipsGuiColorBarPropWidgetCallback (coordinate_callback, this));
*/

}

void ChipsGuiColorBarPrefFrame::thick_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float val = widget->get_tick_thickness ();

   if (cbarFrame)
      cbarFrame->set_tick_thickness (val);
}

void ChipsGuiColorBarPrefFrame::tickmode_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   string text = widget->get_tick_mode ();
   if (cbarFrame)  cbarFrame->set_tick_mode (text);

}

void ChipsGuiColorBarPrefFrame::width_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float val = widget->get_width ();
   if (cbarFrame)
      cbarFrame->set_width (val);

}

void ChipsGuiColorBarPrefFrame::length_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float val = widget->get_length ();
   if (cbarFrame)
      cbarFrame->set_length (val);
}

void ChipsGuiColorBarPrefFrame::orientation_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_orientation ();
   if (cbarFrame)  cbarFrame->set_orientation (val);

}

void ChipsGuiColorBarPrefFrame::label_color_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   string color = widget->get_label_color ();
   if (cbarFrame)  cbarFrame->set_label_color (color);

}

void ChipsGuiColorBarPrefFrame::label_angle_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float angle = widget->get_label_angle ();
   if (cbarFrame) cbarFrame->set_label_angle (angle);

}

void ChipsGuiColorBarPrefFrame::label_text_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   string text = widget->get_text ();

   if (cbarFrame)
      cbarFrame->set_text (text);

}

void ChipsGuiColorBarPrefFrame::label_font_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   string font = widget->get_label_font ();
   if (cbarFrame)  cbarFrame->set_label_font (font);

}

void ChipsGuiColorBarPrefFrame::label_fontstyle_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_label_fontstyle ();
   if (cbarFrame)  cbarFrame->set_label_fontstyle (val);

}

void ChipsGuiColorBarPrefFrame::label_halign_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float halign = widget->get_label_halign ();
   if (cbarFrame)
     cbarFrame->set_label_halign (halign);
}

void ChipsGuiColorBarPrefFrame::label_valign_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float valign = widget->get_label_valign ();
   if (cbarFrame)
     cbarFrame->set_label_valign (valign);
}


void ChipsGuiColorBarPrefFrame::label_size_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int size = widget->get_label_size ();
   if (cbarFrame) cbarFrame->set_label_size (size);
}

void ChipsGuiColorBarPrefFrame::label_perpendicular_offset_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float offset = widget->get_perpendicular_offset ();

   if (cbarFrame)
      cbarFrame->set_perpendicular_offset (offset);
}

void ChipsGuiColorBarPrefFrame::label_parallel_offset_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float offset = widget->get_parallel_offset ();

   if (cbarFrame)
      cbarFrame->set_parallel_offset (offset);
}



void ChipsGuiColorBarPrefFrame::label_visible_change_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   bool visible = widget->get_label_visible ();
   if (cbarFrame) cbarFrame->set_label_visible (visible);

}


void ChipsGuiColorBarPrefFrame::tick_color_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   string color = widget->get_tick_color ();
   if (cbarFrame)  cbarFrame->set_tick_color (color);
   
}


void ChipsGuiColorBarPrefFrame::tick_style_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_tick_style ();
   if (cbarFrame)  cbarFrame->set_tick_style (val);

}

void ChipsGuiColorBarPrefFrame::tick_length_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_tick_length ();
   if (cbarFrame)  cbarFrame->set_tick_length (val);

}

void ChipsGuiColorBarPrefFrame::tick_visible_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = (int)widget->get_tick_visible ();
   if (cbarFrame)  cbarFrame->set_tick_visible (val);

}

void ChipsGuiColorBarPrefFrame::tick_location_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_tick_location();
   if (cbarFrame)  cbarFrame->set_tick_location (val);
}

void ChipsGuiColorBarPrefFrame::ticklabel_color_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   string color = widget->get_ticklabel_color ();
   if (cbarFrame)  cbarFrame->set_ticklabel_color(color);

}

void ChipsGuiColorBarPrefFrame::ticklabel_font_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   string font = widget->get_ticklabel_font ();

   if (cbarFrame)  cbarFrame->set_ticklabel_font (font);

}


void ChipsGuiColorBarPrefFrame::ticklabel_fontstyle_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
     
   int style = widget->get_ticklabel_fontstyle ();
   if (cbarFrame)  cbarFrame->set_ticklabel_fontstyle (style);

}

void ChipsGuiColorBarPrefFrame::ticklabel_size_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);

   int size = widget->get_ticklabel_size ();
   if (cbarFrame)  cbarFrame->set_ticklabel_size (size);
}

void ChipsGuiColorBarPrefFrame::ticklabel_angle_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float val = widget->get_ticklabel_angle ();

   if (cbarFrame)  cbarFrame->set_ticklabel_angle (val);
}

void ChipsGuiColorBarPrefFrame::ticklabel_offset_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_ticklabel_offset ();

   if (cbarFrame)  cbarFrame->set_ticklabel_offset (val);

}

void ChipsGuiColorBarPrefFrame::ticklabel_halign_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float halign;
   bool auto_align;

   auto_align = widget->get_ticklabel_auto_halign ();
   halign = (auto_align) ? AUTO_ALIGN : widget->get_ticklabel_halign ();

   cbarFrame->set_ticklabel_halign (halign);

}

void ChipsGuiColorBarPrefFrame::ticklabel_valign_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float valign;
   bool auto_align;

   auto_align = widget->get_ticklabel_auto_valign ();
   valign = (auto_align) ? AUTO_ALIGN : widget->get_ticklabel_valign ();

   cbarFrame->set_ticklabel_valign (valign);

}


void ChipsGuiColorBarPrefFrame::ticklabel_visible_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   bool val = widget->get_ticklabel_visible ();
   if (cbarFrame)  cbarFrame->set_ticklabel_visible (val);

}

void ChipsGuiColorBarPrefFrame::halign_change_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float val = widget->get_halign ();

   if (cbarFrame)
      cbarFrame->set_halign (val);
}

void ChipsGuiColorBarPrefFrame::valign_change_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   float val = widget->get_valign ();

   if (cbarFrame)
      cbarFrame->set_valign (val);
}

void ChipsGuiColorBarPrefFrame::depth_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_depth ();
   if (cbarFrame)  cbarFrame->set_depth (val);

}

void ChipsGuiColorBarPrefFrame::border_visible_callback(ChipsGuiColorBarPropWidget *widget, gpointer data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);
   int val = widget->get_border_visibility ();
   if (cbarFrame)  cbarFrame->set_border_visibility (val);

}

/*
void ChipsGuiColorBarPrefFrame::coordinate_callback(ChipsGuiColorBarPropWidget *widget, void *data)
{
   ChipsGuiColorBarPrefFrame *cbarFrame = static_cast <ChipsGuiColorBarPrefFrame*>(data);

   if (!cbarFrame)  return; //ensure we have needed pointer

   COORD_SYS coordsys = (COORD_SYS)widget->get_coordinate ();
   ChipsId cid;
   string coordid = "";

   cbarFrame->_eng->get_current_coord (&cid);

   if ((coordsys == DATA) && (cid.get_coord_sys () != DATA))
      chipsGui->log_error ("Data coordinate system does not exists.");
   else
      coordid = cid.get_coord_id ();

   coordid = ChipsCoordinate::extract_id (coordsys, coordid);

   cbarFrame->_eng->change_annotation_coordinate (&cid, chips_point, coordsys, coordid);
}
*/

void ChipsGuiColorBarPrefFrame::set_width(float val)
{
   stringstream pref;
   pref << "width=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_perpendicular_offset(float val)
{
   stringstream pref;
   pref << "offset.perpendicular=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_parallel_offset(float val)
{
   stringstream pref;
   pref << "offset.parallel=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_visible(bool val)
{
   stringstream pref;
   pref << "ticklabel.visible=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_halign(float val)
{
   stringstream pref;

   if (val == AUTO_ALIGN)
      pref << "ticklabel.halign=auto";
   else
      pref << "ticklabel.halign=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_valign(float val)
{
   stringstream pref;
   if (val == AUTO_ALIGN)
      pref << "ticklabel.valign=auto";
   else
      pref << "ticklabel.valign=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_size(int val)
{
   stringstream pref;
   pref << "ticklabel.size=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_offset(int val)
{
   stringstream pref;
   pref << "ticklabel.offset=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_fontstyle(int val)
{
   stringstream pref;
   pref << "ticklabel.fontstyle=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_font(const string &val)
{
   stringstream pref;
   pref << "ticklabel.font=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_color(const string &val)
{
   stringstream pref;
   pref << "ticklabel.color=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_ticklabel_angle(float val)
{
   stringstream pref;
   pref << "ticklabel.angle=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_text(const string &val)
{
   stringstream pref;
   pref << "label.text=" << val;
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_tick_location(int val)
{
   stringstream pref;
   pref << "tick.location=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_tick_visible(bool val)
{
   stringstream pref;
   pref << "tick.visible=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_tick_color(const string &val)
{
   stringstream pref;
   pref << "tick.color=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_tick_length(int val)
{
   stringstream pref;
   pref << "tick.length=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_tick_style(int val)
{
   stringstream pref;
   pref << "tick.style=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_tick_thickness(float val)
{
   stringstream pref;
   pref << "tick.thickness=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_visible(bool val)
{
   stringstream pref;
   pref << "label.visible=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_size(int val)
{
   stringstream pref;
   pref << "label.size=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_valign(float val)
{
   stringstream pref;
   pref << "label.valign=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_halign(float val)
{
   stringstream pref;
   pref << "label.halign=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_fontstyle(int val)
{
   stringstream pref;
   pref << "label.fontstyle=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_font(const string &val)
{
   stringstream pref;
   pref << "label.font=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_color(const string &val)
{
   stringstream pref;
   pref << "label.color=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_label_angle(float val)
{
   stringstream pref;
   pref << "label.angle=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_tick_mode(const string &val)
{
   stringstream pref;
   pref << "tick.mode=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_length(float val)
{
   stringstream pref;
   pref << "length=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_valign(float val)
{
   stringstream pref;
   pref << "valign=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_halign(float val)
{
   stringstream pref;
   pref << "halign=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_depth(int val)
{
   stringstream pref;
   pref << "depth=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_border_visibility(bool val)
{
   stringstream pref;
   pref << "border.visible=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}

void ChipsGuiColorBarPrefFrame::set_orientation(int val)
{
   stringstream pref;
   pref << "orientation=" << val; 
   set_preference_override (pref.str (), chips_colorbar);
}



void ChipsGuiColorBarPrefFrame::initialize()
{
   COORD_SYS coordsys;
   string coordid;
   ChipsVector pos(3);
   stringstream pref_str_val;
   ChipsId cid;
   char *pref_cstr;

   float      widthval;
   float      offset_perpval;
   float      offset_parval;
   bool       ticklabel_visibleval;
   float      ticklabel_halignval;
   float      ticklabel_valignval;
   int        ticklabel_sizeval;
   int        ticklabel_offsetval;
   V_FONTSTYLE_TYPE ticklabel_fontstyleval;
   string     ticklabel_fontval;
   string     ticklabel_colorval;
   float      ticklabel_angleval;
   string     textval;
   V_TICKSTYLE_TYPE tick_locationval;
   bool       tick_visibleval;
   V_TICKSTYLE_TYPE tick_styleval;
   string     tick_colorval;
   int        tick_lengthval;
   float      tick_thicknessval;
//   V_TICKSTYLE_TYPE label_locationval;
   bool       label_visibleval;
   int        label_sizeval;
   float      label_valignval;
   float      label_halignval;
   V_FONTSTYLE_TYPE label_fontstyleval;
   string     label_fontval;
   string     label_colorval;
   float      label_angleval;
   string     tick_modeval;
   float      lengthval;
   float      valignval;
   float      halignval;
   bool       border_visibleval;
   int        visibleval;
   int        depthval;
   int        orientationval;

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   if ((pref_cstr = _eng->get_preference("colorbar.depth")) != NULL)
   {
      if (chips_strcmp (pref_cstr, "default") == 0)
      {
         free (pref_cstr);
         if ((pref_cstr = _eng->get_preference ("default.depth")) == NULL)
            depthval = V_DEFAULT_DEPTH;
      }

      if (pref_cstr)
      {
         stringstream(pref_cstr) >> depthval;
         free (pref_cstr);
      }
   }

   if ((pref_cstr = _eng->get_preference("colorbar.border.visible")) != NULL)
   {
      if ((chips_strcmp(pref_cstr, "false") == 0) || (strcmp(pref_cstr, "0") == 0) ||
          (chips_strcmp(pref_cstr, "no") == 0) || (chips_strcmp(pref_cstr, "off") == 0) ||
          (chips_strcmp(pref_cstr, "f") == 0))
         border_visibleval = false;
      else 
         border_visibleval = true;

      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.halign")) != NULL)
   {
      c2_string_to_hoffset(pref_cstr, &halignval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.valign")) != NULL)
   {
      c2_string_to_voffset (pref_cstr, &valignval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.length")) != NULL)
   {
      stringstream(pref_cstr) >> lengthval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.orientation")) != NULL)
   {
      if (!chips_strcmp(pref_cstr,"horizontal")||!chips_strcmp(pref_cstr,"0"))
         orientationval = 0;
      else if (!chips_strcmp(pref_cstr,"vertical")||!chips_strcmp(pref_cstr,"1"))
         orientationval = 1;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.tick.mode")) != NULL)
   {
      tick_modeval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.angle")) != NULL)
   {
      stringstream(pref_cstr) >> label_angleval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.color")) != NULL)
   {
      label_colorval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.font")) != NULL)
   {
      label_fontval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.fontstyle")) != NULL)
   {
      c2_string_to_fontstyle (pref_cstr, &label_fontstyleval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.halign")) != NULL)
   {
      c2_string_to_hoffset(pref_cstr, &label_halignval);
      pref_str_val >> label_halignval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.valign")) != NULL)
   {
      c2_string_to_voffset(pref_cstr, &label_valignval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.size")) != NULL)
   {
      stringstream(pref_cstr) >> label_sizeval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.visible")) != NULL)
   {
      if ((chips_strcmp(pref_cstr, "false") == 0) || (strcmp(pref_cstr, "0") == 0) ||
          (chips_strcmp(pref_cstr, "no") == 0) || (chips_strcmp(pref_cstr, "off") == 0) ||
          (chips_strcmp(pref_cstr, "f") == 0))
         label_visibleval = false;
      else
         label_visibleval = true;

      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.label.text")) != NULL)
   {
      textval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.tick.thickness")) != NULL)
   {
      stringstream(pref_cstr) >> tick_thicknessval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.tick.length")) != NULL)
   {
      stringstream(pref_cstr) >> tick_lengthval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.tick.color")) != NULL)
   {
      tick_colorval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.tick.style")) != NULL)
   {
      c2_string_to_tickstyle (pref_cstr, &tick_styleval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.tick.visible")) != NULL)
   {
      if ((chips_strcmp(pref_cstr, "false") == 0) || (strcmp(pref_cstr, "0") == 0) ||
          (chips_strcmp(pref_cstr, "no") == 0) || (chips_strcmp(pref_cstr, "off") == 0) ||
          (chips_strcmp(pref_cstr, "f") == 0))
         tick_visibleval = false;
      else
         tick_visibleval = true;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.tick.location")) != NULL)
   {
      c2_string_to_tickstyle (pref_cstr, &tick_locationval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.angle")) != NULL)
   {
      stringstream(pref_cstr) >> ticklabel_angleval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.color")) != NULL)
   {
      ticklabel_colorval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.font")) != NULL)
   {
      ticklabel_fontval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.fontstyle")) != NULL)
   {
      c2_string_to_fontstyle (pref_cstr, &ticklabel_fontstyleval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.offset")) != NULL)
   {
      stringstream(pref_cstr) >> ticklabel_offsetval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.size")) != NULL)
   {
      stringstream(pref_cstr) >> ticklabel_sizeval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.valign")) != NULL)
   {
      c2_string_to_voffset(pref_cstr, &ticklabel_valignval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.halign")) != NULL)
   {
      c2_string_to_hoffset(pref_cstr, &ticklabel_halignval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.ticklabel.visible")) != NULL)
   {
      if ((chips_strcmp(pref_cstr, "false") == 0) || (strcmp(pref_cstr, "0") == 0) ||
          (chips_strcmp(pref_cstr, "no") == 0) || (chips_strcmp(pref_cstr, "off") == 0) ||
          (chips_strcmp(pref_cstr, "f") == 0))
         ticklabel_visibleval = false;
      else
         ticklabel_visibleval = true;

      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.offset.parallel")) != NULL)
   {
      stringstream(pref_cstr) >> offset_parval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.offset.perpendicular")) != NULL)
   {
      stringstream(pref_cstr) >> offset_perpval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference("colorbar.width")) != NULL)
   {
      stringstream(pref_cstr) >> widthval;
      free (pref_cstr);
   }

   _eng->get_object_coordinfo (&cid, chips_colorbar, coordsys, coordid);

   _cbarprop->set_depth (depthval);
   _cbarprop->set_visibility (visibleval);
   _cbarprop->set_border_visibility (border_visibleval);
   _cbarprop->set_halign (halignval);
   _cbarprop->set_valign (valignval);
   _cbarprop->set_length (lengthval);
   _cbarprop->set_orientation (orientationval);
   _cbarprop->set_tick_mode (tick_modeval);
   _cbarprop->set_label_angle (label_angleval);
   _cbarprop->set_label_color (label_colorval);
   _cbarprop->set_label_font (label_fontval);

   _cbarprop->set_label_fontstyle (label_fontstyleval);
   _cbarprop->set_label_halign (label_halignval);
   _cbarprop->set_label_valign (label_valignval);
   _cbarprop->set_label_size (label_sizeval);
   _cbarprop->set_label_visible (label_visibleval);
//   _cbarprop->set_label_location ((int)label_locationval);
   _cbarprop->set_text (textval);
   _cbarprop->set_tick_thickness (tick_thicknessval);
   _cbarprop->set_tick_length (tick_lengthval);
   _cbarprop->set_tick_color (tick_colorval);
   _cbarprop->set_tick_style ((int)tick_styleval);
   _cbarprop->set_tick_visible (tick_visibleval);
   _cbarprop->set_tick_location ((int)tick_locationval);
   _cbarprop->set_ticklabel_angle (ticklabel_angleval);
   _cbarprop->set_ticklabel_color (ticklabel_colorval);
   _cbarprop->set_ticklabel_font (ticklabel_fontval);

   _cbarprop->set_ticklabel_fontstyle (ticklabel_fontstyleval);
   _cbarprop->set_ticklabel_offset (ticklabel_offsetval);
   _cbarprop->set_ticklabel_size (ticklabel_sizeval);
   _cbarprop->set_ticklabel_valign (ticklabel_valignval);
   _cbarprop->set_ticklabel_auto_valign (ticklabel_valignval == AUTO_ALIGN);
   _cbarprop->set_ticklabel_halign (ticklabel_halignval);
   _cbarprop->set_ticklabel_auto_halign (ticklabel_halignval == AUTO_ALIGN);
   _cbarprop->set_ticklabel_visible (ticklabel_visibleval);
   _cbarprop->set_offsets (offset_parval, offset_perpval);
   _cbarprop->set_width (widthval);
   _cbarprop->set_coordinate ((int)coordsys);


}

