// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiColorBarPropWidget.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiPositionWidget.hh"
#include "ChipsGuiLabelWidget.hh"
#include "ChipsGuiPivotPointWidget.hh"
#include "ChipsGuiPropPane.hh"
#include "ChipsGuiMain.hh"
#include "config.h"
#include "ChipsDefinesP.hh"

#include <sstream>
#include <cstdlib>

#if HAVE_IEEEFP_H 
#include <ieeefp.h>
#endif 

using namespace std;

extern ChipsGuiMain* chipsGui;

ChipsGuiColorBarPropWidgetCallback::ChipsGuiColorBarPropWidgetCallback (void (*callback)(ChipsGuiColorBarPropWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;

}

void ChipsGuiColorBarPropWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiColorBarPropWidget *cbarWidget = dynamic_cast<ChipsGuiColorBarPropWidget *>(caller);
   _callback (cbarWidget, _callbackData);
}



// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiColorBarPropWidget::ChipsGuiColorBarPropWidget()
{
    _valid_callback_ids.push_back ("change_width");
    _valid_callback_ids.push_back ("change_length");
    _valid_callback_ids.push_back ("change_perpendicular_offset");
    _valid_callback_ids.push_back ("change_parallel_offset");
    _valid_callback_ids.push_back ("change_ticklabel_visible");
    _valid_callback_ids.push_back ("change_ticklabel_halign");
    _valid_callback_ids.push_back ("change_ticklabel_valign");
    _valid_callback_ids.push_back ("change_ticklabel_size");
    _valid_callback_ids.push_back ("change_ticklabel_offset");
    _valid_callback_ids.push_back ("change_ticklabel_fontstyle");
    _valid_callback_ids.push_back ("change_ticklabel_font");
    _valid_callback_ids.push_back ("change_ticklabel_color");
    _valid_callback_ids.push_back ("change_ticklabel_angle");
    _valid_callback_ids.push_back ("change_text");
    _valid_callback_ids.push_back ("change_label_color");
    _valid_callback_ids.push_back ("change_label_halign");
    _valid_callback_ids.push_back ("change_label_angle");
    _valid_callback_ids.push_back ("change_label_font");
    _valid_callback_ids.push_back ("change_label_fontstyle");
    _valid_callback_ids.push_back ("change_label_visible");
    _valid_callback_ids.push_back ("change_label_size");
    _valid_callback_ids.push_back ("change_label_valign");
    _valid_callback_ids.push_back ("change_tick_location");
    _valid_callback_ids.push_back ("change_tick_visible");
    _valid_callback_ids.push_back ("change_tick_style");
    _valid_callback_ids.push_back ("change_tick_color");
    _valid_callback_ids.push_back ("change_tick_length");
    _valid_callback_ids.push_back ("change_tick_mode");
    _valid_callback_ids.push_back ("change_tick_thickness");
    _valid_callback_ids.push_back ("change_halign");
    _valid_callback_ids.push_back ("change_valign");
    _valid_callback_ids.push_back ("change_border_visible");
    _valid_callback_ids.push_back ("change_visible");
    _valid_callback_ids.push_back ("change_depth");
    _valid_callback_ids.push_back ("change_orientation");
    _valid_callback_ids.push_back ("change_position");
    _valid_callback_ids.push_back ("change_coordinate");


   _ticklabel_color = NULL;
   _tick_color = NULL;
   _label = NULL;
   _color = NULL;
   _position = NULL;

}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiColorBarPropWidget::~ChipsGuiColorBarPropWidget()
{

   if (_ticklabel_color)
      delete _ticklabel_color;
   if (_ticklabel_alignment)
      delete _ticklabel_alignment; 
   if (_tick_color)
      delete _tick_color;
   if (_label)
      delete _label;
   if (_color)
      delete _color;
   if (_position)
      delete _position;
}

void ChipsGuiColorBarPropWidget::build_form()
{

   GtkWidget*   label;

   _base_widget = _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);

   label = gtk_label_new("ColorBar");
   

   // setup axis notebook
   {
      GtkWidget*   frame1;
      GtkWidget*   table1;
      GtkWidget*   frame2;
      GtkWidget*   positionframe;

      GtkWidget*   vbox;
      GtkWidget*   app_hbox1;
      GtkWidget*   app_hbox2;
      GtkWidget*   app_hbox3;
      GtkWidget*   app_hbox4;
      GtkWidget*   app_hbox5;


      vbox = gtk_vbox_new (FALSE, 3);
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), vbox, label);

      // setup appearance

      frame1 = gtk_frame_new("Appearance");
      gtk_frame_set_label_align(GTK_FRAME(frame1), .0, .5);
      gtk_box_pack_start(GTK_BOX(vbox), frame1, FALSE, FALSE,5);

      table1 = gtk_table_new (3,2,FALSE);
      gtk_container_add(GTK_CONTAINER(frame1), table1);


      _color = new ChipsGuiColorWidget();
      _color->build_form ();

      _thick = gtk_spin_button_new_with_range(0.5, 10, 1);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_thick), 1);

      _tickmode = gtk_combo_box_new_text();
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tickmode), "none");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tickmode), "nice");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tickmode), "ends");
      gtk_combo_box_set_active(GTK_COMBO_BOX(_tickmode),1);

      _length = gtk_spin_button_new_with_range (.000001, G_MAXFLOAT,0.01);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_length), 5);

      _width = gtk_spin_button_new_with_range (.000001, G_MAXFLOAT,0.01);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_width), 5);

      _orientation = gtk_combo_box_new_text();
      gtk_combo_box_append_text(GTK_COMBO_BOX(_orientation), "horizontal");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_orientation), "vertical");
      gtk_combo_box_set_active(GTK_COMBO_BOX(_orientation),1);

      app_hbox1 = gtk_hbox_new (FALSE, 2);
      label = gtk_label_new("Color");
      gtk_box_pack_start (GTK_BOX (app_hbox1), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox1), _color->base_widget (), FALSE, FALSE, 5);
      gtk_table_attach(GTK_TABLE(table1), app_hbox1, 0, 1, 0, 1,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);

      app_hbox2 = gtk_hbox_new (FALSE, 2);
      label = gtk_label_new("Tick Mode");
      gtk_box_pack_start (GTK_BOX (app_hbox2), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox2), _tickmode, TRUE, TRUE, 5);

      gtk_table_attach(GTK_TABLE(table1), app_hbox2, 1, 2, 1, 2,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);

      app_hbox3 = gtk_hbox_new (FALSE, 2);
      label = gtk_label_new("Thickness");
      gtk_box_pack_start (GTK_BOX (app_hbox3), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox3), _thick, FALSE, FALSE, 5);
      gtk_table_attach(GTK_TABLE(table1), app_hbox3, 1, 2, 0, 1,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);



      app_hbox4 = gtk_hbox_new (FALSE,2);
      label = gtk_label_new("Length"); 
      gtk_box_pack_start (GTK_BOX (app_hbox4), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox4), _length, FALSE, FALSE, 5);

      gtk_table_attach(GTK_TABLE(table1), app_hbox4, 0, 1, 1, 2,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);

      app_hbox5 = gtk_hbox_new (FALSE,2);
      label = gtk_label_new("Width");
      gtk_box_pack_start (GTK_BOX (app_hbox5), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox5), _width, FALSE, FALSE, 5);

      gtk_table_attach(GTK_TABLE(table1), app_hbox5, 0, 1, 2, 3,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);

      gtk_table_attach(GTK_TABLE(table1), _orientation, 1, 2, 2, 3,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);



      // setup label
      
      frame2 = gtk_frame_new("Label");
      gtk_frame_set_label_align(GTK_FRAME(frame2), .0, .5);
      gtk_box_pack_start(GTK_BOX(vbox), frame2, FALSE, FALSE,5);

      _label_text = gtk_entry_new ();
      gtk_container_add (GTK_CONTAINER(frame2), _label_text);

      // add position frame
      _position = new ChipsGuiPositionWidget();
      _position->build_form (false);

      positionframe = gtk_frame_new("Position");
      gtk_frame_set_label_align(GTK_FRAME(positionframe), .0, .5);
      gtk_box_pack_start(GTK_BOX(vbox), positionframe, FALSE, FALSE,5);

      gtk_container_add(GTK_CONTAINER(positionframe), _position->base_widget ());

   }

   // setup the label page
   {
      _label = new ChipsGuiLabelWidget();
      _label->build_form ();
      _label->set_position_type (ChipsGuiLabelWidget::FLOAT,
                              ChipsGuiLabelWidget::INT);

      label = gtk_label_new("Label");
      _label_page_num = gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _label->base_widget (), label);

   }

   // setup the tick label page
   {
      GtkWidget*   table;
      GtkWidget*   fontframe;
      GtkWidget*   positionframe;
      GtkWidget*   font_hbox1;
      GtkWidget*   font_hbox2;
      GtkCellRenderer* cell1;
      GtkCellRenderer* cell2;

      GtkWidget*   pos_table;
      GtkWidget*   pos_hbox1;
      GtkWidget*   pos_hbox2;
      GtkWidget*   vis_hbox;

      label = gtk_label_new ("Tick Label");
      table = gtk_table_new (4,2,FALSE);
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), table, label);

      fontframe = gtk_frame_new("Font");
      gtk_frame_set_label_align(GTK_FRAME(fontframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), fontframe, 0, 2, 0, 1,
                                        (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5,5);

      font_hbox1 = gtk_hbox_new (FALSE, 4);
      gtk_container_add(GTK_CONTAINER(fontframe), font_hbox1);

      _ticklabel_font = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL( ChipsGuiPropPane::font_info.get_font_list ()), 0);
      gtk_box_pack_start (GTK_BOX (font_hbox1), _ticklabel_font, FALSE, FALSE, 2);

      cell1 = gtk_cell_renderer_text_new();
      gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( _ticklabel_font ), cell1, FALSE );

      _ticklabel_size = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL( ChipsGuiPropPane::font_info.get_size_list ()), 0);
      gtk_widget_set_size_request (_ticklabel_size, 65, -1);
      gtk_box_pack_start (GTK_BOX (font_hbox1), _ticklabel_size, FALSE, FALSE, 2);
      cell2 = gtk_cell_renderer_text_new();
      gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( _ticklabel_size ), cell2, FALSE );

      font_hbox2 = gtk_hbutton_box_new ();
      gtk_button_box_set_layout (GTK_BUTTON_BOX(font_hbox2), GTK_BUTTONBOX_CENTER);
      gtk_box_pack_start (GTK_BOX (font_hbox1), font_hbox2, FALSE, FALSE, 2);

      _ticklabel_bold = gtk_toggle_button_new_with_label (GTK_STOCK_BOLD);
      gtk_button_set_use_stock (GTK_BUTTON(_ticklabel_bold), TRUE);
      gtk_box_pack_start (GTK_BOX (font_hbox1), _ticklabel_bold, FALSE, FALSE, 0);


      _ticklabel_italic = gtk_toggle_button_new_with_label (GTK_STOCK_ITALIC);
      gtk_button_set_use_stock (GTK_BUTTON(_ticklabel_italic), TRUE);
      gtk_box_pack_start (GTK_BOX (font_hbox1), _ticklabel_italic, FALSE, FALSE, 0);

      _ticklabel_color = new ChipsGuiColorWidget();
      _ticklabel_color->build_form ();
      gtk_box_pack_start (GTK_BOX (font_hbox1), _ticklabel_color->base_widget (), FALSE, FALSE, 5);

      

      // add position frame
      positionframe = gtk_frame_new("Position");
      gtk_frame_set_label_align(GTK_FRAME(positionframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), positionframe, 0, 1, 1, 2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5, 5);

      pos_table = gtk_table_new (4, 2, FALSE);
      gtk_container_add(GTK_CONTAINER(positionframe), pos_table);

      pos_hbox1 = gtk_hbox_new (FALSE, 2);
      gtk_table_attach(GTK_TABLE(pos_table), pos_hbox1, 0,1,0,1, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5, 5);

      label = gtk_label_new("Offset");
      gtk_box_pack_start(GTK_BOX(pos_hbox1), label, FALSE, FALSE,5);

      _ticklabel_offset = gtk_spin_button_new_with_range (-9999, 9999,1.0);
      gtk_box_pack_start(GTK_BOX(pos_hbox1), _ticklabel_offset, FALSE, FALSE,5);
      gtk_spin_button_set_digits (GTK_SPIN_BUTTON(_ticklabel_offset), 0);

      pos_hbox2 = gtk_hbox_new (FALSE, 2);
      gtk_table_attach(GTK_TABLE(pos_table), pos_hbox2, 0,1,1,2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5, 5);

      label = gtk_label_new("Angle");
      gtk_box_pack_start(GTK_BOX(pos_hbox2), label, FALSE, FALSE,5);

      _ticklabel_angle = gtk_spin_button_new_with_range (0, 359.99999, 1);
      gtk_box_pack_start(GTK_BOX(pos_hbox2), _ticklabel_angle, FALSE, FALSE,5);
      gtk_spin_button_set_wrap (GTK_SPIN_BUTTON(_ticklabel_angle), TRUE);

      // setup alignment
      _ticklabel_alignment = new ChipsGuiPivotPointWidget();
      _ticklabel_alignment->build_form (true);
      gtk_table_attach(GTK_TABLE(pos_table), _ticklabel_alignment->base_widget (), 1,2,0,4, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL),5, 5);

      vis_hbox = gtk_hbox_new (FALSE, 2);
      gtk_table_attach(GTK_TABLE(table), vis_hbox, 0, 1, 3, 4, GTK_FILL, GTK_SHRINK,5, 5);

      _ticklabel_visible = gtk_check_button_new_with_label ("Visible");
      gtk_box_pack_start (GTK_BOX (vis_hbox), _ticklabel_visible, FALSE, FALSE, 0);

   }

   // setup tick page
   {
      GtkWidget*   app_frame;
      GtkWidget*   vbox;
      GtkWidget*   app_table;

      GtkWidget*   app_hbox1;
      GtkWidget*   app_hbox2;
      GtkWidget*   app_hbox3;
      GtkWidget*   app_hbox5;
      GtkWidget*   vis_hbox;


      label = gtk_label_new ("Ticks");
      vbox = gtk_vbox_new (FALSE, 3);
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), vbox, label);

      app_frame = gtk_frame_new("Appearance");
      gtk_frame_set_label_align(GTK_FRAME(app_frame), .0, .5);
      gtk_box_pack_start(GTK_BOX(vbox), app_frame, FALSE, FALSE,5);

      app_table = gtk_table_new (2,2,FALSE);
      gtk_container_add(GTK_CONTAINER(app_frame), app_table);


      _tick_color = new ChipsGuiColorWidget();
      _tick_color->build_form ();

      _tick_length = gtk_spin_button_new_with_range(0, 100000, 1);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_tick_length), 1);

      _tick_visible = gtk_check_button_new_with_label ("Visible");

      _tick_style = gtk_combo_box_new_text();
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tick_style), "inside");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tick_style), "outside");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tick_style), "centered");
      gtk_combo_box_set_active(GTK_COMBO_BOX(_tick_style),1);

      _tick_location = gtk_combo_box_new_text();
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tick_location), "inside");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_tick_location), "outside");
      gtk_combo_box_set_active(GTK_COMBO_BOX(_tick_location),1);


      app_hbox1 = gtk_hbox_new (FALSE, 2);
      label = gtk_label_new("Color");
      gtk_box_pack_start (GTK_BOX (app_hbox1), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox1), _tick_color->base_widget (), FALSE, FALSE, 5);
      gtk_table_attach(GTK_TABLE(app_table), app_hbox1, 0, 1, 0, 1,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);

      app_hbox2 = gtk_hbox_new (FALSE, 2);
      label = gtk_label_new("Length");
      gtk_box_pack_start (GTK_BOX (app_hbox2), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox2), _tick_length, FALSE, FALSE, 5);
      gtk_table_attach(GTK_TABLE(app_table), app_hbox2, 1, 2, 0, 1,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);

      app_hbox3 = gtk_hbox_new (FALSE, 2);
      label = gtk_label_new("Location");
      gtk_box_pack_start (GTK_BOX (app_hbox3), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox3), _tick_location, FALSE, FALSE, 5);
      gtk_table_attach(GTK_TABLE(app_table), app_hbox3, 0, 1, 1, 2,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);


      app_hbox5 = gtk_hbox_new (FALSE, 2);
      label = gtk_label_new("Style");
      gtk_box_pack_start (GTK_BOX (app_hbox5), label, FALSE, FALSE, 5);
      gtk_box_pack_start (GTK_BOX (app_hbox5), _tick_style, TRUE, TRUE, 5);
      gtk_table_attach(GTK_TABLE(app_table), app_hbox5, 1, 2, 1, 2,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);


      // visible
      vis_hbox = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(vbox), vis_hbox, FALSE, FALSE,5);
      gtk_box_pack_start (GTK_BOX (vis_hbox), _tick_visible, FALSE, FALSE, 0);
   }

   // setup alignment page
   {
      GtkWidget*   vbox2;
      GtkWidget*   hbox_halign;
      GtkWidget*   hbox_valign;
      GtkWidget*   hbox_hbalign;
      GtkWidget*   hbox_vbalign;
      GtkWidget*   vbox_halign;
      GtkWidget*   vbox_valign;



      vbox2 = gtk_vbox_new (FALSE, 2);
      label = gtk_label_new("Alignment");
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), vbox2, label);

      hbox_halign = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start (GTK_BOX (vbox2), hbox_halign, FALSE, FALSE, 5);
      hbox_valign = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start (GTK_BOX (vbox2), hbox_valign, FALSE, FALSE, 5);

      label = gtk_label_new("Horizontal");
      gtk_box_pack_start(GTK_BOX(hbox_halign), label, FALSE, FALSE,5);

      label = gtk_label_new("Vertical");
      gtk_box_pack_start(GTK_BOX(hbox_valign), label, FALSE, FALSE,5);

      vbox_halign = gtk_vbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(hbox_halign), vbox_halign, FALSE, FALSE,5);

      vbox_valign = gtk_vbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(hbox_valign), vbox_valign, FALSE, FALSE,5);

      hbox_hbalign = gtk_hbutton_box_new();
      gtk_box_pack_start (GTK_BOX (vbox_halign), hbox_hbalign, FALSE, FALSE, 5);

      _halign_left = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_LEFT);
      gtk_button_set_use_stock (GTK_BUTTON(_halign_left), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_hbalign), _halign_left, FALSE, FALSE, 0);

      _halign_center = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_CENTER);
      gtk_button_set_use_stock (GTK_BUTTON(_halign_center), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_hbalign), _halign_center, FALSE, FALSE, 0);

      _halign_right = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_RIGHT);
      gtk_button_set_use_stock (GTK_BUTTON(_halign_right), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_hbalign), _halign_right, FALSE, FALSE, 0);

      _halign = gtk_hscale_new_with_range (0, 1, .1);
      gtk_scale_set_digits(GTK_SCALE(_halign), 2);
      gtk_scale_set_draw_value(GTK_SCALE(_halign), true);
      gtk_box_pack_start (GTK_BOX (vbox_halign), _halign, FALSE, FALSE, 0);

      hbox_vbalign = gtk_hbutton_box_new ();
      gtk_box_pack_start (GTK_BOX (vbox_valign), hbox_vbalign, FALSE, FALSE, 5);

      _valign_bottom = gtk_toggle_button_new_with_label (GTK_STOCK_GOTO_BOTTOM);
      gtk_button_set_use_stock (GTK_BUTTON(_valign_bottom), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_vbalign), _valign_bottom, FALSE, FALSE, 0);

      _valign_center = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_CENTER);
      gtk_button_set_use_stock (GTK_BUTTON(_valign_center), TRUE);
      gtk_button_set_use_stock (GTK_BUTTON(_valign_center), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_vbalign), _valign_center, FALSE, FALSE, 0);

      _valign_top = gtk_toggle_button_new_with_label (GTK_STOCK_GOTO_TOP);
      gtk_button_set_use_stock (GTK_BUTTON(_valign_top), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_vbalign), _valign_top, FALSE, FALSE, 0);

      _valign = gtk_hscale_new_with_range (0, 1, .1);
      gtk_scale_set_digits(GTK_SCALE(_valign), 2);
      gtk_scale_set_draw_value(GTK_SCALE(_valign), true);
      gtk_box_pack_start (GTK_BOX (vbox_valign), _valign, FALSE, FALSE, 0);


   }
/***  There was an initial thought that we might support adjusting the 
 ***  colormaps. However, due to the complexity of what it would mean
 ***  in changing the library I've left it out for now.

   // setup color page
   {
      GtkWidget *vbox;
      GtkWidget *tabcmap;
      GtkWidget *tabalpha;
      GtkWidget *tabthresh;
      GtkWidget *frmcmap;
      GtkWidget *frmalpha;
      GtkWidget *frmthresh;
      GtkWidget *tmp;

      vbox = gtk_vbox_new (FALSE, 5);
      tabcmap = gtk_table_new(7,7,FALSE);
      tabalpha = gtk_table_new(7,4,FALSE);
      tabthresh = gtk_table_new(7,7,FALSE);
      frmcmap = gtk_frame_new("Colormap");
      gtk_frame_set_label_align(GTK_FRAME(frmcmap), .5, .5);
      gtk_container_add (GTK_CONTAINER (frmcmap), tabcmap);
      gtk_box_pack_start(GTK_BOX(vbox), frmcmap, FALSE, FALSE, 5);

      tmp = gtk_label_new("Color");
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), vbox, tmp);

      _cmap = gtk_combo_box_new_text();
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "red");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "green");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "blue");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "grayscale");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "rainbow");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "hsv");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "heat");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "cool");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "usercmap1");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "usercmap2");
      gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "usercmap3");

      _pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 256, 16);

      _cmapimg = gtk_image_new_from_pixbuf(_pixbuf);
      _cmapsz = gtk_spin_button_new_with_range(1, 9999, 1);
      _invert = gtk_check_button_new();
      _cinterp = gtk_check_button_new();
      _scale = gtk_check_button_new();

      gtk_table_attach(GTK_TABLE(tabcmap), _cmapimg, 4, 6, 2, 3,
         GTK_SHRINK, GTK_SHRINK, 5, 5);
      tmp = gtk_label_new("Colormap");
      gtk_table_attach(GTK_TABLE(tabcmap), tmp, 1, 2, 2, 3,
         GTK_FILL, GTK_FILL, 5, 5);
      gtk_table_attach(GTK_TABLE(tabcmap), _cmap, 2, 4, 2, 3,
         GTK_FILL, GTK_FILL, 5, 5);
      tmp = gtk_label_new("Size");
      gtk_table_attach(GTK_TABLE(tabcmap), tmp, 1, 2, 3, 4,
         GTK_FILL, GTK_FILL, 5, 5);
      gtk_table_attach(GTK_TABLE(tabcmap), _cmapsz, 2, 4, 3, 4,
         GTK_FILL, GTK_FILL, 5, 5);
      tmp = gtk_label_new("Invert");
      gtk_table_attach(GTK_TABLE(tabcmap), tmp, 4, 5, 3, 4,
         GTK_FILL, GTK_FILL, 5, 5);
      gtk_table_attach(GTK_TABLE(tabcmap), _invert, 5, 6, 3, 4,
         GTK_FILL, GTK_FILL, 5, 5);
      tmp = gtk_label_new("Interpolate");
      gtk_table_attach(GTK_TABLE(tabcmap), tmp, 4, 5, 4, 5,
         GTK_FILL, GTK_FILL, 5, 5);
      gtk_table_attach(GTK_TABLE(tabcmap), _cinterp, 5, 6, 4, 5,
         GTK_FILL, GTK_FILL, 5, 5);

      // alpha frame
      frmalpha = gtk_frame_new("Alpha");
      gtk_frame_set_label_align(GTK_FRAME(frmalpha), .5, .5);
      gtk_container_add (GTK_CONTAINER (frmalpha), tabalpha);
      gtk_box_pack_start(GTK_BOX(vbox), frmalpha, FALSE, FALSE,5);

      _alphamin = gtk_hscale_new_with_range(0,1,.01);
      gtk_scale_set_digits(GTK_SCALE(_alphamin), 2);
      gtk_scale_set_draw_value(GTK_SCALE(_alphamin), true);
      _alphamax = gtk_hscale_new_with_range(0,1,.01);
      gtk_scale_set_digits(GTK_SCALE(_alphamax), 2);
      gtk_scale_set_draw_value(GTK_SCALE(_alphamax), true);
      _alphalock = gtk_check_button_new();
      tmp = gtk_label_new("Minimum");
      gtk_table_attach(GTK_TABLE(tabalpha), tmp, 0, 2, 1, 2,
         GTK_FILL, GTK_FILL, 5, 5);
      gtk_table_attach(GTK_TABLE(tabalpha), _alphamin, 2, 7, 1, 2,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
      tmp = gtk_label_new("Maximum");
      gtk_table_attach(GTK_TABLE(tabalpha), tmp, 0, 2, 2, 3,
         GTK_FILL, GTK_FILL, 5, 5);
      gtk_table_attach(GTK_TABLE(tabalpha), _alphamax, 2, 7, 2, 3,
         (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
      tmp = gtk_label_new("Uniform");
      gtk_table_attach(GTK_TABLE(tabalpha), tmp, 3, 4, 3, 4,
         GTK_FILL, GTK_FILL, 10, 10);
      gtk_table_attach(GTK_TABLE(tabalpha), _alphalock,  4, 5, 3, 4,
         GTK_FILL, GTK_FILL, 10, 10);

      // threshold frame
      frmthresh = gtk_frame_new("Threshold");
      gtk_frame_set_label_align(GTK_FRAME(frmthresh), .5, .5);
      gtk_container_add (GTK_CONTAINER (frmthresh), tabthresh);
       gtk_box_pack_start(GTK_BOX(vbox), frmthresh, FALSE, FALSE,5);
      tmp = gtk_label_new("Min");
      gtk_table_attach(GTK_TABLE(tabthresh), tmp, 1, 2, 1, 2,
        (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
         //GTK_FILL, GTK_FILL, 5, 5);
      _threshmin = gtk_spin_button_new_with_range(-9999, 9999, 1);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_threshmin), 3);
      gtk_table_attach(GTK_TABLE(tabthresh), _threshmin, 3 ,4, 1, 2,
        (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
        // GTK_FILL, GTK_FILL, 5, 5);
      tmp = gtk_label_new("Max");
      gtk_table_attach(GTK_TABLE(tabthresh), tmp, 4,5, 1, 2,
        (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
        // GTK_FILL, GTK_FILL, 5, 5);
      _threshmax = gtk_spin_button_new_with_range(-9999, 9999, 1);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_threshmax), 3);
      gtk_table_attach(GTK_TABLE(tabthresh), _threshmax, 5, 6, 1, 2,
        (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
        // GTK_FILL, GTK_FILL, 5, 5);

   }
*/
   // Setup General Page
   {
      GtkWidget*   hbox1;
      GtkWidget*   hbox2;
      GtkWidget*   hbox3;
      GtkWidget*   hbox4;
      GtkWidget*   vbox1;
      
      vbox1 = gtk_vbox_new (FALSE, 2);
      label = gtk_label_new("General");
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), vbox1, label);

      hbox1 = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE,10);
      hbox2 = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox2, FALSE, FALSE,10);
      hbox3 = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox3, FALSE, FALSE,10);
      hbox4 = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox4, FALSE, FALSE,10);



      _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);

      label = gtk_label_new("Depth");
      gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE,10);
      gtk_box_pack_start(GTK_BOX(hbox1), _depth, FALSE, FALSE,10);

      _visible = gtk_check_button_new();

      label = gtk_label_new("Visible");
      gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE,10);
      gtk_box_pack_start(GTK_BOX(hbox2), _visible, FALSE, FALSE,10);

      _border_visible = gtk_check_button_new ();
      label = gtk_label_new("Border Visible");
      gtk_box_pack_start(GTK_BOX(hbox3), label, FALSE, FALSE,10);
      gtk_box_pack_start(GTK_BOX(hbox3), _border_visible, FALSE, FALSE,10);


   }




   _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_callback, this));
   _thick_id = g_signal_connect(G_OBJECT(_thick), "value-changed", G_CALLBACK(thick_callback), (void *)this);
   _tickmode_id = g_signal_connect(G_OBJECT(_tickmode), "changed", G_CALLBACK(tickmode_callback), (void*)this);
   _label_text_id = g_signal_connect(G_OBJECT(_label_text), "changed", G_CALLBACK(label_text_callback), (void*)this);
   _width_id = g_signal_connect(G_OBJECT(_width), "value-changed", G_CALLBACK(width_callback), (void *)this);
   _length_id = g_signal_connect(G_OBJECT(_length), "value-changed", G_CALLBACK(length_callback), (void *)this);
   _orientation_id = g_signal_connect(G_OBJECT(_orientation), "changed", G_CALLBACK(orientation_callback), (void*)this);

   _position->add_callback ("change_position", new ChipsGuiPositionWidgetCallback (position_callback, this));
   _position->add_callback ("change_coordinate", new ChipsGuiPositionWidgetCallback (coordinate_callback, this));


   _label->add_callback ("change_color", new ChipsGuiLabelWidgetCallback (label_color_change_callback, this));
   _label->add_callback ("change_angle", new ChipsGuiLabelWidgetCallback (label_angle_change_callback, this));
   _label->add_callback ("change_text", new ChipsGuiLabelWidgetCallback (label_text_change_callback, this));
   _label->add_callback ("change_fontstyle", new ChipsGuiLabelWidgetCallback (label_fontstyle_change_callback, this));
   _label->add_callback ("change_font", new ChipsGuiLabelWidgetCallback (label_font_change_callback, this));
   _label->add_callback ("change_align", new ChipsGuiLabelWidgetCallback (label_align_change_callback, this));
   _label->add_callback ("change_size", new ChipsGuiLabelWidgetCallback (label_size_change_callback, this));
   _label->add_callback ("change_position", new ChipsGuiLabelWidgetCallback (label_position_change_callback, this));
   _label->add_callback ("change_visible", new ChipsGuiLabelWidgetCallback (label_visible_change_callback, this));

   _tick_color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (tick_color_callback, this));
   _tick_style_id = g_signal_connect(G_OBJECT(_tick_style), "changed", G_CALLBACK(tick_style_callback), (void*)this);
   _tick_length_id = g_signal_connect(G_OBJECT(_tick_length), "value-changed", G_CALLBACK(tick_length_callback), (void*)this);
   _tick_visible_id = g_signal_connect(G_OBJECT(_tick_visible), "toggled", G_CALLBACK(tick_visible_callback), (void*)this);
   _tick_location_id = g_signal_connect(G_OBJECT(_tick_location), "changed", G_CALLBACK(tick_location_callback), (void*)this);

   _ticklabel_color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (ticklabel_color_callback, this));
   _ticklabel_font_id = g_signal_connect(G_OBJECT(_ticklabel_font), "changed", G_CALLBACK(ticklabel_font_callback), (void*)this);
   _ticklabel_bold_id = g_signal_connect(G_OBJECT(_ticklabel_bold), "toggled", G_CALLBACK(ticklabel_fontstyle_callback), (void*)this);
   _ticklabel_italic_id = g_signal_connect(G_OBJECT(_ticklabel_italic), "toggled", G_CALLBACK(ticklabel_fontstyle_callback), (void*)this);
   _ticklabel_size_id = g_signal_connect(G_OBJECT(_ticklabel_size), "changed", G_CALLBACK(ticklabel_size_callback), (void*)this);
   _ticklabel_angle_id = g_signal_connect(G_OBJECT(_ticklabel_angle), "value-changed", G_CALLBACK(ticklabel_angle_callback), (void*)this);
   _ticklabel_offset_id = g_signal_connect(G_OBJECT(_ticklabel_offset), "value-changed", G_CALLBACK(ticklabel_offset_callback), (void*)this);

   _ticklabel_alignment->add_callback ("change_pivot", new ChipsGuiPivotPointWidgetCallback (ticklabel_align_callback, this));
   _ticklabel_alignment->add_callback ("enable_xauto", new ChipsGuiPivotPointWidgetCallback (ticklabel_halign_auto_callback, this));
   _ticklabel_alignment->add_callback ("enable_yauto", new ChipsGuiPivotPointWidgetCallback (ticklabel_valign_auto_callback, this));

   _ticklabel_visible_change_id = g_signal_connect(G_OBJECT(_ticklabel_visible), "toggled", G_CALLBACK(ticklabel_visible_callback), (void*)this);


   _halign_left_id = g_signal_connect(G_OBJECT(_halign_left), "toggled", G_CALLBACK(halign_change_callback), (void*)this);
   _halign_center_id = g_signal_connect(G_OBJECT(_halign_center), "toggled", G_CALLBACK(halign_change_callback), (void*)this);
   _halign_right_id = g_signal_connect(G_OBJECT(_halign_right), "toggled", G_CALLBACK(halign_change_callback), (void*)this);

   _valign_top_id = g_signal_connect(G_OBJECT(_valign_top), "toggled", G_CALLBACK(valign_change_callback), (void*)this);
   _valign_center_id = g_signal_connect(G_OBJECT(_valign_center), "toggled", G_CALLBACK(valign_change_callback), (void*)this);
   _valign_bottom_id = g_signal_connect(G_OBJECT(_valign_bottom), "toggled", G_CALLBACK(valign_change_callback), (void*)this);

   _halign_scale_id = g_signal_connect(G_OBJECT(_halign), "value-changed", G_CALLBACK(halign_scale_change_callback), (void*)this);
   _valign_scale_id = g_signal_connect(G_OBJECT(_valign), "value-changed", G_CALLBACK(valign_scale_change_callback), (void*)this);

   _depth_id = g_signal_connect(G_OBJECT(_depth), "value-changed", G_CALLBACK(depth_callback), (void*)this);
   _visible_id = g_signal_connect(G_OBJECT(_visible), "toggled", G_CALLBACK(visible_callback), (void*)this);
   _border_visible_id = g_signal_connect(G_OBJECT(_border_visible), "toggled", G_CALLBACK(border_visible_callback), (void*)this);

   setup_tooltip(_color->base_widget(), "Sets the color of the colorbar border, tick marks, and labels"); 

   setup_tooltip(_tickmode,"Controls whether tickmarks are not shown, shown at the endpoints, or evenly spaced out on the colorbar");
   setup_tooltip(_thick,"Sets the thickness of the tick marks and borders");
   setup_tooltip(_length,"Sets the plot normalized length of the colorbar");
   setup_tooltip(_width, "Sets the plot normalized width of the colorbar");
   setup_tooltip(_orientation, "Specifies whether the colorbar is drawn horizontally or vetically");
   setup_tooltip(_ticklabel_color->base_widget(), "Sets the color of the colorbar tick labels"); 
   setup_tooltip(_ticklabel_bold, "Specifies whether a bold fontstyle should be used on the ticklabels");
   setup_tooltip(_ticklabel_italic, "Specifies whether the tick label text should be italicized");
   setup_tooltip(_ticklabel_size, "Sets the point size of the text used for tick labels");
   setup_tooltip(_ticklabel_font, "Sets the font of the text used for tick labels");
   setup_tooltip(_ticklabel_angle, "Sets the angle of text used for tick labels");
   setup_tooltip(_ticklabel_visible, "Should tick labels be displayed or hidden");
   setup_tooltip(_halign_left,"Left justifies the anchor for the colorbar's location");
   setup_tooltip(_halign_center,"Horizontally centers the anchor for the colorbar's location");
   setup_tooltip(_halign_right,"Right justifies the anchor for the colorbar's location");
   setup_tooltip(_halign,"Sets the colorbar's horizontal anchor position");
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the colorbar is drawn");
   setup_tooltip(_visible,"Should the colorbar be displayed or hidden");
   setup_tooltip(_border_visible,"Should the border around the colorbar be displayed or hidden");
   setup_tooltip(_valign_top,"Top justifies the anchor for the colorbar's location");
   setup_tooltip(_valign_center,"Vertically centers the anchor for the colorbar's location");
   setup_tooltip(_valign_bottom,"Bottom justifies the anchor for the colorbar's location");
   setup_tooltip(_valign,"Sets the colorbar's vertical anchor position");
   setup_tooltip(_tick_color->base_widget(),"Sets the color of the colorbar tick marks and border");
   setup_tooltip(_tick_length,"Sets the length of the colorbar tick marks");
   setup_tooltip(_tick_visible,"Should the colorbar tick marks be displayed or hidden");
   setup_tooltip(_tick_style,"Controls the placement of the colorbar tick marks");
   setup_tooltip(_tick_location, "Controls tick and label locations with reference to the plot center");
}

void ChipsGuiColorBarPropWidget::show ()
{
   gtk_widget_show_all (_base_widget);
}

void ChipsGuiColorBarPropWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}

void ChipsGuiColorBarPropWidget::coordinate_callback(ChipsGuiPositionWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   if (!cbarWidget)  return; //ensure we have needed pointer

   if (cbarWidget->_callbacks.count ("change_coordinate") > 0)
      cbarWidget->_callbacks["change_coordinate"]->execute (cbarWidget, NULL);

}

void ChipsGuiColorBarPropWidget::position_callback(ChipsGuiPositionWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   if (!cbarWidget)  return; //ensure we have needed pointer

   if (cbarWidget->_callbacks.count ("change_position") > 0)
      cbarWidget->_callbacks["change_position"]->execute (cbarWidget, NULL);

}


void ChipsGuiColorBarPropWidget::color_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   string color = widget->get_chips_color ();
   if (cbarWidget)
   {

      cbarWidget->_tick_color->set_chips_color (color);
      cbarWidget->_ticklabel_color->set_chips_color (color);
      cbarWidget->_label->set_color (color);

      if (cbarWidget->_callbacks.count ("change_ticklabel_color") > 0)
         cbarWidget->_callbacks["change_ticklabel_color"]->execute (cbarWidget, NULL);
      if (cbarWidget->_callbacks.count ("change_tick_color") > 0)
         cbarWidget->_callbacks["change_tick_color"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::thick_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   float fval = val;
   if (val == 0) fval = 0.5;

   if (cbarWidget)
   {
      cbarWidget->set_tick_thickness (fval);
      if (cbarWidget->_callbacks.count ("change_tick_thickness") > 0)
         cbarWidget->_callbacks["change_tick_thickness"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::tickmode_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   char *chr_text = gtk_combo_box_get_active_text (GTK_COMBO_BOX(widget));
   string text = chr_text;
   if (cbarWidget)
   {
      cbarWidget->set_tick_mode (text);
      if (cbarWidget->_callbacks.count ("change_tick_mode") > 0)
         cbarWidget->_callbacks["change_tick_mode"]->execute (cbarWidget, NULL);
   }

   g_free (chr_text);

}

void ChipsGuiColorBarPropWidget::label_text_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   string text = gtk_entry_get_text(GTK_ENTRY(widget));
   if (cbarWidget)
   {
      cbarWidget->set_text (text);
      if (cbarWidget->_callbacks.count ("change_text") > 0)
         cbarWidget->_callbacks["change_text"]->execute (cbarWidget, NULL);
   }
}

void ChipsGuiColorBarPropWidget::width_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   float val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (cbarWidget)
   {
      cbarWidget->set_width (val);
      if (cbarWidget->_callbacks.count ("change_width") > 0)
         cbarWidget->_callbacks["change_width"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::length_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   float val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (cbarWidget)
   {
      cbarWidget->set_length (val);
      if (cbarWidget->_callbacks.count ("change_length") > 0)
         cbarWidget->_callbacks["change_length"]->execute (cbarWidget, NULL);
   }
}

void ChipsGuiColorBarPropWidget::orientation_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (cbarWidget)
   {
      cbarWidget->set_orientation (val);
      if (cbarWidget->_callbacks.count ("change_orientation") > 0)
         cbarWidget->_callbacks["change_orientation"]->execute (cbarWidget, NULL);
   }
}



void ChipsGuiColorBarPropWidget::label_color_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   if (cbarWidget)
      if (cbarWidget->_callbacks.count ("change_label_color") > 0)
         cbarWidget->_callbacks["change_label_color"]->execute (cbarWidget, NULL);

}

void ChipsGuiColorBarPropWidget::label_angle_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   if (cbarWidget)
      if (cbarWidget->_callbacks.count ("change_label_angle") > 0)
         cbarWidget->_callbacks["change_label_angle"]->execute (cbarWidget, NULL);


}

void ChipsGuiColorBarPropWidget::label_text_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   string text = widget->get_text ();

   if (cbarWidget)
   {

      g_signal_handler_block(GTK_OBJECT(cbarWidget->_label_text), cbarWidget->_label_text_id);
      gtk_entry_set_text(GTK_ENTRY(cbarWidget->_label_text), text.c_str ());
      g_signal_handler_unblock(GTK_OBJECT(cbarWidget->_label_text), cbarWidget->_label_text_id);

      if (cbarWidget->_callbacks.count ("change_text") > 0)
         cbarWidget->_callbacks["change_text"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::label_font_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   if (cbarWidget)
      if (cbarWidget->_callbacks.count ("change_label_font") > 0)
         cbarWidget->_callbacks["change_label_font"]->execute (cbarWidget, NULL);


}

void ChipsGuiColorBarPropWidget::label_fontstyle_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   if (cbarWidget)
      if (cbarWidget->_callbacks.count ("change_label_fontstyle") > 0)
         cbarWidget->_callbacks["change_label_fontstyle"]->execute (cbarWidget, NULL);


}

void ChipsGuiColorBarPropWidget::label_align_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   if (cbarWidget)
   {
      if (cbarWidget->_callbacks.count ("change_label_halign") > 0)
         cbarWidget->_callbacks["change_label_halign"]->execute (cbarWidget, NULL);
      if (cbarWidget->_callbacks.count ("change_label_valign") > 0)
         cbarWidget->_callbacks["change_label_valign"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::label_size_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   if (cbarWidget)
      if (cbarWidget->_callbacks.count ("change_label_size") > 0)
         cbarWidget->_callbacks["change_label_size"]->execute (cbarWidget, NULL);

}

void ChipsGuiColorBarPropWidget::label_position_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   if (cbarWidget)
   {
      if (cbarWidget->_callbacks.count ("change_perpendicular_offset") > 0)
         cbarWidget->_callbacks["change_perpendicular_offset"]->execute (cbarWidget, NULL);
      if (cbarWidget->_callbacks.count ("change_parallel_offset") > 0)
         cbarWidget->_callbacks["change_parallel_offset"]->execute (cbarWidget, NULL);
   }

}


void ChipsGuiColorBarPropWidget::label_visible_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   if (cbarWidget)
      if (cbarWidget->_callbacks.count ("change_label_visible") > 0)
         cbarWidget->_callbacks["change_label_visible"]->execute (cbarWidget, NULL);


}


void ChipsGuiColorBarPropWidget::tick_color_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   if (cbarWidget)
      if (cbarWidget->_callbacks.count ("change_tick_color") > 0)
         cbarWidget->_callbacks["change_tick_color"]->execute (cbarWidget, NULL);

   
}


void ChipsGuiColorBarPropWidget::tick_style_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (cbarWidget)
   {
      cbarWidget->set_tick_style (val);
      if (cbarWidget->_callbacks.count ("change_tick_style") > 0)
         cbarWidget->_callbacks["change_tick_style"]->execute (cbarWidget, NULL);
   }
}

void ChipsGuiColorBarPropWidget::tick_length_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (cbarWidget)
   {
      cbarWidget->set_tick_length (val);
      if (cbarWidget->_callbacks.count ("change_tick_length") > 0)
         cbarWidget->_callbacks["change_tick_length"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::tick_visible_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = (int)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (cbarWidget)
   {
      cbarWidget->set_tick_visible (val);
      if (cbarWidget->_callbacks.count ("change_tick_visible") > 0)
         cbarWidget->_callbacks["change_tick_visible"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::tick_location_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (cbarWidget)
   {
      cbarWidget->set_tick_location (val);
      if (cbarWidget->_callbacks.count ("change_tick_location") > 0)
         cbarWidget->_callbacks["change_tick_location"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::ticklabel_color_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   string color = widget->get_chips_color ();
   if (cbarWidget)
   {
      cbarWidget->set_ticklabel_color (color);
      if (cbarWidget->_callbacks.count ("change_ticklabel_color") > 0)
         cbarWidget->_callbacks["change_ticklabel_color"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::ticklabel_font_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   char *str_font = gtk_combo_box_get_active_text(GTK_COMBO_BOX (widget));

   if (str_font)
   {
      string font = str_font;
      if (cbarWidget)
      {
         cbarWidget->set_ticklabel_font (font);
         if (cbarWidget->_callbacks.count ("change_ticklabel_font") > 0)
         cbarWidget->_callbacks["change_ticklabel_font"]->execute (cbarWidget, NULL);
      }

      g_free (str_font);
   }

}


void ChipsGuiColorBarPropWidget::ticklabel_fontstyle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int set_style = 0;

   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cbarWidget->_ticklabel_bold)))
      set_style += 1;
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cbarWidget->_ticklabel_italic)))
      set_style += 2;

   if (cbarWidget)
   {
      cbarWidget->set_ticklabel_fontstyle (set_style);
      if (cbarWidget->_callbacks.count ("change_ticklabel_fontstyle") > 0)
         cbarWidget->_callbacks["change_ticklabel_fontstyle"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::ticklabel_size_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   char *str_size = gtk_combo_box_get_active_text(GTK_COMBO_BOX (widget));
   int int_size;

   if (str_size)
   {
      int_size = atoi (str_size);

      if (cbarWidget)
      {
         cbarWidget->set_ticklabel_size (int_size);
         if (cbarWidget->_callbacks.count ("change_ticklabel_size") > 0)
            cbarWidget->_callbacks["change_ticklabel_size"]->execute (cbarWidget, NULL);
      }

      g_free (str_size);
   }

}

void ChipsGuiColorBarPropWidget::ticklabel_angle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   float val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));

   if (cbarWidget)
   {
      cbarWidget->set_ticklabel_angle (val);
      if (cbarWidget->_callbacks.count ("change_ticklabel_angle") > 0)
         cbarWidget->_callbacks["change_ticklabel_angle"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::ticklabel_offset_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));

   if (cbarWidget)
   {
      cbarWidget->set_ticklabel_offset (val);
      if (cbarWidget->_callbacks.count ("change_ticklabel_offset") > 0)
         cbarWidget->_callbacks["change_ticklabel_offset"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::ticklabel_align_callback(ChipsGuiPivotPointWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   float halign, valign;

   widget->get_pivot_position (halign, valign);

   if (cbarWidget)
   {
      if (cbarWidget->_callbacks.count ("change_ticklabel_halign") > 0)
         cbarWidget->_callbacks["change_ticklabel_halign"]->execute (cbarWidget, NULL);
      if (cbarWidget->_callbacks.count ("change_ticklabel_valign") > 0)
         cbarWidget->_callbacks["change_ticklabel_valign"]->execute (cbarWidget, NULL);
   }
}

void ChipsGuiColorBarPropWidget::ticklabel_halign_auto_callback(ChipsGuiPivotPointWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   if (cbarWidget)
   {
      cbarWidget->set_ticklabel_auto_halign (true);
      if (cbarWidget->_callbacks.count ("change_ticklabel_halign") > 0)
         cbarWidget->_callbacks["change_ticklabel_halign"]->execute (cbarWidget, NULL);
   }
}

void ChipsGuiColorBarPropWidget::ticklabel_valign_auto_callback(ChipsGuiPivotPointWidget *widget, void *data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   if (cbarWidget)
   {
      cbarWidget->set_ticklabel_auto_valign (true);
      if (cbarWidget->_callbacks.count ("change_ticklabel_valign") > 0)    
         cbarWidget->_callbacks["change_ticklabel_valign"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::ticklabel_visible_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

   if (cbarWidget)
   {
      cbarWidget->set_ticklabel_visible (val);
      if (cbarWidget->_callbacks.count ("change_ticklabel_visible") > 0)
         cbarWidget->_callbacks["change_ticklabel_visible"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::halign_scale_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);

   float val = gtk_range_get_value(GTK_RANGE(cbarWidget->_halign));
 
   if (cbarWidget)
   {
      cbarWidget->set_halign (val);
      if (cbarWidget->_callbacks.count ("change_halign") > 0)
         cbarWidget->_callbacks["change_halign"]->execute (cbarWidget, NULL);
   }
  
}

void ChipsGuiColorBarPropWidget::valign_scale_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   float val = gtk_range_get_value(GTK_RANGE(cbarWidget->_valign));

   if (cbarWidget)
   {
      cbarWidget->set_valign (val);
      if (cbarWidget->_callbacks.count ("change_valign") > 0)
         cbarWidget->_callbacks["change_valign"]->execute (cbarWidget, NULL);
   }
}

void ChipsGuiColorBarPropWidget::halign_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   float val;
   if (widget == cbarWidget->_halign_left)
      val = 0;
   else if (widget == cbarWidget->_halign_center)
      val = .5;
   else
      val = 1;

   if (cbarWidget)
   {
      cbarWidget->set_halign (val);
      if (cbarWidget->_callbacks.count ("change_halign") > 0)
         cbarWidget->_callbacks["change_halign"]->execute (cbarWidget, NULL);
   }

}

void ChipsGuiColorBarPropWidget::valign_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   float val;
   if (widget == cbarWidget->_valign_bottom)
      val = 0;
   else if (widget == cbarWidget->_valign_center)
      val = .5;
   else
      val = 1;

   if (cbarWidget)
   {
      cbarWidget->set_valign (val);
      if (cbarWidget->_callbacks.count ("change_valign") > 0)
         cbarWidget->_callbacks["change_valign"]->execute (cbarWidget, NULL);
   }

}


void ChipsGuiColorBarPropWidget::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (cbarWidget)
   {
      cbarWidget->set_depth (val);
      if (cbarWidget->_callbacks.count ("change_depth") > 0)
         cbarWidget->_callbacks["change_depth"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::visible_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = (int)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (cbarWidget)
   {
      cbarWidget->set_visibility (val);
      if (cbarWidget->_callbacks.count ("change_visible") > 0)
         cbarWidget->_callbacks["change_visible"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::border_visible_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiColorBarPropWidget *cbarWidget = static_cast <ChipsGuiColorBarPropWidget*>(data);
   int val = (int)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

   if (cbarWidget)
   {
      cbarWidget->set_border_visibility (val);
      if (cbarWidget->_callbacks.count ("change_border_visible") > 0)
         cbarWidget->_callbacks["change_border_visible"]->execute (cbarWidget, NULL);
   }


}

void ChipsGuiColorBarPropWidget::set_width(float val)
{
   _widthval = val;
   g_signal_handler_block(GTK_OBJECT(_width), _width_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_width), val);
   g_signal_handler_unblock(GTK_OBJECT(_width), _width_id);
}

void ChipsGuiColorBarPropWidget::set_perpendicular_offset(float val)
{
   if (_label)
   {
      float x,y;
      _label->get_position (x,y);
      this->set_offsets (x, val);
   }

}

void ChipsGuiColorBarPropWidget::set_parallel_offset(float val)
{
   if (_label)
   {
      float x,y;
      _label->get_position (x,y);
      this->set_offsets (val, y); 
   }

}

void ChipsGuiColorBarPropWidget::set_offsets(float parallel_offset, float perpendicular_offset)
{
   if (_label)
   {
      if (_orientationval == 0)
         _label->set_position (parallel_offset, perpendicular_offset);
      else
         _label->set_position (perpendicular_offset, parallel_offset);
   }
}

void ChipsGuiColorBarPropWidget::set_ticklabel_visible(bool val)
{
   _ticklabel_visibleval = val;
   g_signal_handler_block(GTK_OBJECT(_ticklabel_visible), _ticklabel_visible_change_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_ticklabel_visible), val);
   g_signal_handler_unblock(GTK_OBJECT(_ticklabel_visible), _ticklabel_visible_change_id);

}

void ChipsGuiColorBarPropWidget::set_ticklabel_halign(float val)
{
   float x,y;
   _ticklabel_alignment->get_pivot_position (x, y);
   _ticklabel_alignment->set_pivot_position (val, y);
}

void ChipsGuiColorBarPropWidget::set_ticklabel_auto_halign(bool val)
{
   _ticklabel_alignment->set_auto_x (val);
}

void ChipsGuiColorBarPropWidget::set_ticklabel_valign(float val)
{
   float x,y;
   _ticklabel_alignment->get_pivot_position (x, y);
   _ticklabel_alignment->set_pivot_position (x, val);
}

void ChipsGuiColorBarPropWidget::set_ticklabel_auto_valign(bool val)
{
   _ticklabel_alignment->set_auto_y (val);
}


void ChipsGuiColorBarPropWidget::set_ticklabel_size(int val)
{
   stringstream sstream;

   _ticklabel_sizeval = val;

   g_signal_handler_block(GTK_COMBO_BOX(_ticklabel_size), _ticklabel_size_id);

   sstream << val << ends;
   gtk_entry_set_text (GTK_ENTRY (GTK_BIN (_ticklabel_size)->child), sstream.str ().c_str ());
   g_signal_handler_unblock(GTK_COMBO_BOX(_ticklabel_size), _ticklabel_size_id);
}

void ChipsGuiColorBarPropWidget::set_ticklabel_offset(int val)
{
   _ticklabel_offsetval = val;
   g_signal_handler_block(GTK_OBJECT(_ticklabel_offset), _ticklabel_offset_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_ticklabel_offset), val);
   g_signal_handler_unblock(GTK_OBJECT(_ticklabel_offset), _ticklabel_offset_id);
}

void ChipsGuiColorBarPropWidget::set_ticklabel_fontstyle(int val)
{
   _ticklabel_fontstyleval = val;
   g_signal_handler_block(GTK_OBJECT(_ticklabel_bold), _ticklabel_bold_id);
   g_signal_handler_block(GTK_OBJECT(_ticklabel_italic), _ticklabel_italic_id);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_ticklabel_bold), FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_ticklabel_italic), FALSE);
   if ((_ticklabel_fontstyleval == chips_bold) || (_ticklabel_fontstyleval == chips_bolditalic))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_ticklabel_bold), TRUE);
   if ((_ticklabel_fontstyleval == chips_italic) || (_ticklabel_fontstyleval == chips_bolditalic))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_ticklabel_italic), TRUE);
   g_signal_handler_unblock(GTK_OBJECT(_ticklabel_bold), _ticklabel_bold_id);
   g_signal_handler_unblock(GTK_OBJECT(_ticklabel_italic), _ticklabel_italic_id);

}

void ChipsGuiColorBarPropWidget::set_ticklabel_font(const string &val)
{

   _ticklabel_fontval = val;

   g_signal_handler_block(GTK_COMBO_BOX(_ticklabel_font), _ticklabel_font_id);

   gtk_entry_set_text (GTK_ENTRY (GTK_BIN (_ticklabel_font)->child), _ticklabel_fontval.c_str ());
   g_signal_handler_unblock(GTK_COMBO_BOX(_ticklabel_font), _ticklabel_font_id);

}

void ChipsGuiColorBarPropWidget::set_ticklabel_color(const string &val)
{
   if (_ticklabel_color)
      _ticklabel_color->set_chips_color (val);

}

void ChipsGuiColorBarPropWidget::set_ticklabel_angle(float val)
{
   _ticklabel_angleval = val;
   g_signal_handler_block(GTK_OBJECT(_ticklabel_angle), _ticklabel_angle_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_ticklabel_angle), val);
   g_signal_handler_unblock(GTK_OBJECT(_ticklabel_angle), _ticklabel_angle_id);
}

void ChipsGuiColorBarPropWidget::set_text(const string &val)
{

   g_signal_handler_block(GTK_OBJECT(_label_text), _label_text_id);
   gtk_entry_set_text (GTK_ENTRY (_label_text), val.c_str ());
   if (_label)
      _label->set_text (val);
   g_signal_handler_unblock(GTK_OBJECT(_label_text), _label_text_id);

}

void ChipsGuiColorBarPropWidget::set_tick_location(int val)
{
   _tick_locationval = val;
   g_signal_handler_block(GTK_OBJECT(_tick_location), _tick_location_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_tick_location), _tick_locationval);
   g_signal_handler_unblock(GTK_OBJECT(_tick_location), _tick_location_id);
}

void ChipsGuiColorBarPropWidget::set_tick_visible(bool val)
{
   _tick_visibleval = val;
   g_signal_handler_block(GTK_OBJECT(_tick_visible), _tick_visible_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tick_visible), val);
   g_signal_handler_unblock(GTK_OBJECT(_tick_visible), _tick_visible_id);

}

void ChipsGuiColorBarPropWidget::set_tick_color(const string &val)
{
   if (_color)
      _color->set_chips_color(val);
   if (_tick_color)
      _tick_color->set_chips_color (val);
}

void ChipsGuiColorBarPropWidget::set_tick_length(int val)
{
   _tick_lengthval = val;
   g_signal_handler_block(GTK_OBJECT(_tick_length), _tick_length_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_tick_length), _tick_lengthval);
   g_signal_handler_unblock(GTK_OBJECT(_tick_length), _tick_length_id);
}

void ChipsGuiColorBarPropWidget::set_tick_style(int val)
{
   _tick_styleval = val;
   g_signal_handler_block(GTK_OBJECT(_tick_style), _tick_style_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_tick_style),val);
   g_signal_handler_unblock(GTK_OBJECT(_tick_style), _tick_style_id);
}

void ChipsGuiColorBarPropWidget::set_tick_thickness(float val)
{
   _tick_thicknessval = val;
   g_signal_handler_block(GTK_OBJECT(_thick), _thick_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_thick), val);
   g_signal_handler_unblock(GTK_OBJECT(_thick), _thick_id);
  
}

void ChipsGuiColorBarPropWidget::set_label_visible(bool val)
{
   if (_label)
      _label->set_visible (val);
}

void ChipsGuiColorBarPropWidget::set_label_size(int val)
{
   if (_label)
      _label->set_size (val);
}

void ChipsGuiColorBarPropWidget::set_label_valign(float val)
{
   if (_label)
      _label->set_align (_label->get_halign (), val);
}

void ChipsGuiColorBarPropWidget::set_label_halign(float val)
{
   if (_label)
      _label->set_align (val, _label->get_valign ());
}

void ChipsGuiColorBarPropWidget::set_label_align(float hval, float vval)
{
   if (_label)
      _label->set_align (hval, vval);
}


void ChipsGuiColorBarPropWidget::set_label_fontstyle(int val)
{
   if (_label)
      _label->set_fontstyle (val);
}

void ChipsGuiColorBarPropWidget::set_label_font(const string &val)
{
   if (_label)
      _label->set_font (val);
}

void ChipsGuiColorBarPropWidget::set_label_color(const string &val)
{
   if (_label)
      _label->set_color (val);
}

void ChipsGuiColorBarPropWidget::set_label_angle(float val)
{
   if (_label)
      _label->set_angle (val);
}

void ChipsGuiColorBarPropWidget::set_tick_mode(const string &val)
{
   _tick_modeval = val;
   g_signal_handler_block(GTK_OBJECT(_tickmode), _tickmode_id);

   if (_tick_modeval == "none")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_tickmode),0);
   else if (_tick_modeval == "nice")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_tickmode),1);
   else
      gtk_combo_box_set_active(GTK_COMBO_BOX(_tickmode),2);

   g_signal_handler_unblock(GTK_OBJECT(_tickmode), _tickmode_id);
}

void ChipsGuiColorBarPropWidget::set_length(float val)
{
   _lengthval = val;
   g_signal_handler_block(GTK_OBJECT(_length), _length_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_length), _lengthval);
   g_signal_handler_unblock(GTK_OBJECT(_length), _length_id);
}

void ChipsGuiColorBarPropWidget::set_valign(float val)
{
   _valignval = val;
   g_signal_handler_block(GTK_OBJECT(_valign), _valign_scale_id);
   g_signal_handler_block(GTK_OBJECT(_valign_bottom), _valign_bottom_id);
   g_signal_handler_block(GTK_OBJECT(_valign_top), _valign_top_id);
   g_signal_handler_block(GTK_OBJECT(_valign_center), _valign_center_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_bottom),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_top) ,FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_center),FALSE);

   if (flt_equal (val, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_bottom),TRUE);
   else if (flt_equal (val, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_center),TRUE);
   else if (flt_equal (val, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_top),TRUE);

   gtk_range_set_value(GTK_RANGE(_valign), val);

   g_signal_handler_unblock(GTK_OBJECT(_valign), _valign_scale_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_bottom), _valign_bottom_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_top), _valign_top_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_center), _valign_center_id);


}

void ChipsGuiColorBarPropWidget::set_halign(float val)
{
   _halignval = val;
   g_signal_handler_block(GTK_OBJECT(_halign), _halign_scale_id);
   g_signal_handler_block(GTK_OBJECT(_halign_left), _halign_left_id);
   g_signal_handler_block(GTK_OBJECT(_halign_right), _halign_right_id);
   g_signal_handler_block(GTK_OBJECT(_halign_center), _halign_center_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_left),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_right),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_center),FALSE);

   if (flt_equal (val, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_left),TRUE);
   else if (flt_equal (val, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_center),TRUE);
   else if (flt_equal (val, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_right),TRUE);


   gtk_range_set_value(GTK_RANGE(_halign), val);

   g_signal_handler_unblock(GTK_OBJECT(_halign_left), _halign_left_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_right), _halign_right_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_center), _halign_center_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign), _halign_scale_id);

}

void ChipsGuiColorBarPropWidget::set_depth(int val)
{
   _depthval = val;
   g_signal_handler_block(GTK_OBJECT(_depth), _depth_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depthval);
   g_signal_handler_unblock(GTK_OBJECT(_depth), _depth_id);
}

void ChipsGuiColorBarPropWidget::set_visibility(bool val)
{
   _visibleval = val;

   g_signal_handler_block(GTK_OBJECT(_visible), _visible_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_visible), val);
   g_signal_handler_unblock(GTK_OBJECT(_visible), _visible_id);

}

void ChipsGuiColorBarPropWidget::set_border_visibility(bool val)
{
   _border_visibleval = val;

   g_signal_handler_block(GTK_OBJECT(_border_visible), _border_visible_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_border_visible), val);
   g_signal_handler_unblock(GTK_OBJECT(_border_visible), _border_visible_id);

}

void ChipsGuiColorBarPropWidget::set_orientation(int val)
{
   float perp_offset;
   float par_offset;

   this->get_offsets (par_offset, perp_offset);

   _orientationval = val;
   if (val == 0)
   {
      double label_xrange[] = {-.5, .5};
      double label_yrange[] = {-9999, 9999};
      _label->set_position_type (ChipsGuiLabelWidget::FLOAT,
                              ChipsGuiLabelWidget::INT);
      _label->set_position_range (label_xrange, .01, label_yrange, 1);
   }
   else
   {
      double label_yrange[] = {-.5, .5};
      double label_xrange[] = {-9999, 9999};

      _label->set_position_type (ChipsGuiLabelWidget::INT,
                              ChipsGuiLabelWidget::FLOAT);
      _label->set_position_range (label_xrange, 1, label_yrange, .01);
   }
   this->set_offsets (par_offset, perp_offset);

   g_signal_handler_block(GTK_OBJECT(_orientation), _orientation_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_orientation),_orientationval);
   g_signal_handler_unblock(GTK_OBJECT(_orientation), _orientation_id);
}

void ChipsGuiColorBarPropWidget::set_position (double xpos, double ypos)
{
   if (_position)
      _position->set_position (xpos,ypos);
}

void ChipsGuiColorBarPropWidget::set_coordinate (int coordinate)
{
   if (_position)
      _position->set_coordinate (coordinate);
}

void ChipsGuiColorBarPropWidget::set_position_format (int xformat, int yformat)
{
   if (_position)
      _position->set_format (xformat, yformat);

}

float ChipsGuiColorBarPropWidget::get_width()
{
   return _widthval;
}

float ChipsGuiColorBarPropWidget::get_perpendicular_offset()
{
   float par, perp;
   this->get_offsets (par,perp);
   return perp; 
}

float ChipsGuiColorBarPropWidget::get_parallel_offset()
{
   float par, perp;
   this->get_offsets (par,perp);
   return par;
}

void ChipsGuiColorBarPropWidget::get_offsets(float &parallel_offset, float &perpendicular_offset)
{
   if (_orientationval == 0)
      _label->get_position (parallel_offset, perpendicular_offset);
   else
      _label->get_position (perpendicular_offset, parallel_offset);

}


bool ChipsGuiColorBarPropWidget::get_ticklabel_visible()
{
   return _ticklabel_visibleval;
}

float ChipsGuiColorBarPropWidget::get_ticklabel_halign()
{
   float halign, valign; 
   _ticklabel_alignment->get_pivot_position (halign,valign);

   return halign;
}

bool ChipsGuiColorBarPropWidget::get_ticklabel_auto_halign()
{
   return _ticklabel_alignment->get_auto_x ();
}

float ChipsGuiColorBarPropWidget::get_ticklabel_valign()
{

   float halign, valign;
   _ticklabel_alignment->get_pivot_position (halign,valign);

   return valign;
}

bool ChipsGuiColorBarPropWidget::get_ticklabel_auto_valign()
{
   return _ticklabel_alignment->get_auto_y ();
}

int ChipsGuiColorBarPropWidget::get_ticklabel_size()
{
   return _ticklabel_sizeval;
}

int ChipsGuiColorBarPropWidget::get_ticklabel_offset()
{
   return _ticklabel_offsetval;
}

int ChipsGuiColorBarPropWidget::get_ticklabel_fontstyle()
{
   return _ticklabel_fontstyleval;
}

string ChipsGuiColorBarPropWidget::get_ticklabel_font()
{
   return _ticklabel_fontval;
}

string ChipsGuiColorBarPropWidget::get_ticklabel_color()
{
   return _ticklabel_color->get_chips_color ();
}

float ChipsGuiColorBarPropWidget::get_ticklabel_angle()
{
   return _ticklabel_angleval;
}

string ChipsGuiColorBarPropWidget::get_text()
{
   return _label->get_text ();
}

int ChipsGuiColorBarPropWidget::get_tick_location()
{
   return _tick_locationval;
}

bool ChipsGuiColorBarPropWidget::get_tick_visible()
{
   return _tick_visibleval;
}

string ChipsGuiColorBarPropWidget::get_tick_color()
{
   return _tick_color->get_chips_color ();
}

int ChipsGuiColorBarPropWidget::get_tick_length()
{
   return _tick_lengthval;
}

int ChipsGuiColorBarPropWidget::get_tick_style()
{
   return _tick_styleval;
}

float ChipsGuiColorBarPropWidget::get_tick_thickness()
{
   return _tick_thicknessval;
}

bool ChipsGuiColorBarPropWidget::get_label_visible()
{
   return _label->get_visible ();;
}

int ChipsGuiColorBarPropWidget::get_label_size()
{
   return _label->get_size ();
}

float ChipsGuiColorBarPropWidget::get_label_valign()
{
   return _label->get_valign ();
}

float ChipsGuiColorBarPropWidget::get_label_halign()
{
   return _label->get_halign ();
}

int ChipsGuiColorBarPropWidget::get_label_fontstyle()
{
   return _label->get_fontstyle ();
}

string ChipsGuiColorBarPropWidget::get_label_font()
{
   return _label->get_font ();
}

string ChipsGuiColorBarPropWidget::get_label_color()
{
   return _label->get_color ();
}

float ChipsGuiColorBarPropWidget::get_label_angle()
{
   return _label->get_angle ();
}

string ChipsGuiColorBarPropWidget::get_tick_mode()
{
   return _tick_modeval;
}

float ChipsGuiColorBarPropWidget::get_length()
{
   return _lengthval;
}

float ChipsGuiColorBarPropWidget::get_valign()
{
   return _valignval;
}

float ChipsGuiColorBarPropWidget::get_halign()
{
   return _halignval;
}

int ChipsGuiColorBarPropWidget::get_depth()
{
   return _depthval;
}

bool ChipsGuiColorBarPropWidget::get_visibility()
{
   return _visibleval;
}

bool ChipsGuiColorBarPropWidget::get_border_visibility()
{
   return _border_visibleval;
}

int ChipsGuiColorBarPropWidget::get_orientation()
{
   return _orientationval;
}

void ChipsGuiColorBarPropWidget::get_position (double &xpos, double &ypos)
{
   _position->get_position (xpos,ypos);
}

int ChipsGuiColorBarPropWidget::get_coordinate ()
{
   return _position->get_coordinate ();
}

void ChipsGuiColorBarPropWidget::get_position_format (int &xformat, int &yformat)
{
   _position->get_format (xformat, yformat);
}


void ChipsGuiColorBarPropWidget::enable_width()
{
   gtk_widget_set_sensitive (_width, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_offsets()
{
   _label->enable_position ();
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_visible()
{
   gtk_widget_set_sensitive (_ticklabel_visible, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_align()
{
   _ticklabel_alignment->enable ();
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_size()
{
   gtk_widget_set_sensitive (_ticklabel_size, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_offset()
{
   gtk_widget_set_sensitive (_ticklabel_offset, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_fontstyle()
{
   gtk_widget_set_sensitive (_ticklabel_bold, TRUE);
   gtk_widget_set_sensitive (_ticklabel_italic, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_font()
{
   gtk_widget_set_sensitive (_ticklabel_font, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_color()
{
   _ticklabel_color->enable ();
}

void ChipsGuiColorBarPropWidget::enable_ticklabel_angle()
{
   gtk_widget_set_sensitive (_ticklabel_angle, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_text()
{
   _label->enable_text ();
}

void ChipsGuiColorBarPropWidget::enable_tick_location()
{
   gtk_widget_set_sensitive (_tick_location, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_tick_visible()
{
   gtk_widget_set_sensitive (_tick_location, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_tick_color()
{
   _tick_color->enable ();
}

void ChipsGuiColorBarPropWidget::enable_tick_length()
{
   gtk_widget_set_sensitive (_tick_length, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_tick_style()
{
   gtk_widget_set_sensitive (_tick_style, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_tick_thickness()
{
   gtk_widget_set_sensitive (_thick, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_label_visible()
{
   _label->enable_visible ();
}

void ChipsGuiColorBarPropWidget::enable_label_size()
{
   _label->enable_size ();
}

void ChipsGuiColorBarPropWidget::enable_label_align()
{
   _label->enable_align();
}

void ChipsGuiColorBarPropWidget::enable_label_fontstyle()
{
   _label->enable_fontstyle ();
}

void ChipsGuiColorBarPropWidget::enable_label_font()
{
   _label->enable_font ();
}

void ChipsGuiColorBarPropWidget::enable_label_color()
{
   _label->enable_color ();
}

void ChipsGuiColorBarPropWidget::enable_label_angle()
{
   _label->enable_angle ();
}

void ChipsGuiColorBarPropWidget::enable_tick_mode()
{
   gtk_widget_set_sensitive (_tickmode, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_length()
{
   gtk_widget_set_sensitive (_length, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_valign()
{
   gtk_widget_set_sensitive (_valign, TRUE);
   gtk_widget_set_sensitive (_valign_top, TRUE);
   gtk_widget_set_sensitive (_valign_center, TRUE);
   gtk_widget_set_sensitive (_valign_bottom, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_halign()
{
   gtk_widget_set_sensitive (_halign, TRUE);
   gtk_widget_set_sensitive (_halign_left, TRUE);
   gtk_widget_set_sensitive (_halign_center, TRUE);
   gtk_widget_set_sensitive (_halign_right, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_depth()
{
   gtk_widget_set_sensitive (_depth, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_visibility()
{
   gtk_widget_set_sensitive (_visible, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_border_visibility()
{
   gtk_widget_set_sensitive (_border_visible, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_orientation ()
{
   gtk_widget_set_sensitive (_orientation, TRUE);
}

void ChipsGuiColorBarPropWidget::enable_position ()
{
   _position->enable_position ();
}

void ChipsGuiColorBarPropWidget::enable_coordinate ()
{
   _position->enable_coordinate ();
}

void ChipsGuiColorBarPropWidget::enable_data_coordinate ()
{
   _position->enable_data_coordinate ();
}

void ChipsGuiColorBarPropWidget::disable_width()
{
   gtk_widget_set_sensitive (_width, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_offsets()
{
   _label->disable_position ();
}

void ChipsGuiColorBarPropWidget::disable_ticklabel_visible()
{
   gtk_widget_set_sensitive (_ticklabel_visible, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_ticklabel_align()
{
   _ticklabel_alignment->disable ();
}

void ChipsGuiColorBarPropWidget::disable_ticklabel_size()
{
   gtk_widget_set_sensitive (_ticklabel_size, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_ticklabel_offset()
{
   gtk_widget_set_sensitive (_ticklabel_offset, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_ticklabel_fontstyle()
{
   gtk_widget_set_sensitive (_ticklabel_bold, FALSE);
   gtk_widget_set_sensitive (_ticklabel_italic, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_ticklabel_font()
{
   gtk_widget_set_sensitive (_ticklabel_font, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_ticklabel_color()
{
   _ticklabel_color->disable ();
}
void ChipsGuiColorBarPropWidget::disable_ticklabel_angle()
{
   gtk_widget_set_sensitive (_ticklabel_angle, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_text()
{
   _label->disable_text ();
}

void ChipsGuiColorBarPropWidget::disable_tick_location()
{
   gtk_widget_set_sensitive (_tick_location, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_tick_visible()
{
   gtk_widget_set_sensitive (_tick_location, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_tick_color()
{
   _tick_color->disable ();
}

void ChipsGuiColorBarPropWidget::disable_tick_length()
{
   gtk_widget_set_sensitive (_tick_length, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_tick_style()
{
   gtk_widget_set_sensitive (_tick_style, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_tick_thickness()
{
   gtk_widget_set_sensitive (_thick, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_label_visible()
{
   _label->disable_visible ();
}
void ChipsGuiColorBarPropWidget::disable_label_size()
{
   _label->disable_size ();
}

void ChipsGuiColorBarPropWidget::disable_label_align()
{
   _label->disable_align();
}

void ChipsGuiColorBarPropWidget::disable_label_fontstyle()
{
   _label->disable_fontstyle ();
}

void ChipsGuiColorBarPropWidget::disable_label_font()
{
   _label->disable_font ();
}

void ChipsGuiColorBarPropWidget::disable_label_color()
{
   _label->disable_color ();
}

void ChipsGuiColorBarPropWidget::disable_label_angle()
{
   _label->disable_angle ();
}

void ChipsGuiColorBarPropWidget::disable_tick_mode()
{
   gtk_widget_set_sensitive (_tickmode, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_length()
{
   gtk_widget_set_sensitive (_length, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_valign()
{
   gtk_widget_set_sensitive (_valign, FALSE);
   gtk_widget_set_sensitive (_valign, FALSE);
   gtk_widget_set_sensitive (_valign_top, FALSE);
   gtk_widget_set_sensitive (_valign_center, FALSE);
   gtk_widget_set_sensitive (_valign_bottom, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_halign()
{
   gtk_widget_set_sensitive (_halign, FALSE);
   gtk_widget_set_sensitive (_halign_left, FALSE);
   gtk_widget_set_sensitive (_halign_center, FALSE);
   gtk_widget_set_sensitive (_halign_right, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_depth()
{
   gtk_widget_set_sensitive (_depth, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_visibility()
{
   gtk_widget_set_sensitive (_visible, FALSE);
}
void ChipsGuiColorBarPropWidget::disable_border_visibility()
{
   gtk_widget_set_sensitive (_border_visible, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_orientation ()
{
   gtk_widget_set_sensitive (_orientation, FALSE);
}

void ChipsGuiColorBarPropWidget::disable_position ()
{
   _position->disable_position ();
}

void ChipsGuiColorBarPropWidget::disable_coordinate ()
{
   _position->disable_coordinate ();
}

void ChipsGuiColorBarPropWidget::disable_data_coordinate ()
{
   _position->disable_data_coordinate ();
}


void ChipsGuiColorBarPropWidget::display_subobject (V_SUBOBJECT_TYPE subtype)
{
   if (subtype == chips_colorbar_label)
      gtk_notebook_set_current_page (GTK_NOTEBOOK(_nbook), _label_page_num);
}

