// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUICOLORBARPROPWIDGET_HH
#define CHIPSGUICOLORBARPROPWIDGET_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiWidget.hh"
#include "ChipsGuiTooltipObject.hh"
#include "ChipsDefines.hh"

using namespace std;

//  * change_width;
//  * change_perpendicular_offset;
//  * change_parallel_offset;
//  * change_ticklabel_visible;
//  * change_ticklabel_halign;
//  * change_ticklabel_valign;
//  * change_ticklabel_size;
//  * change_ticklabel_offset;
//  * change_ticklabel_fontstyle;
//  * change_ticklabel_font;
//  * change_ticklabel_color;
//  * change_ticklabel_angle;
//  * change_text;
//  * change_tick_location;
//  * change_tick_visible;
//  * change_tick_style;
//  * change_tick_color;
//  * change_tick_length;
//  * change_tick_mode;
//  * change_tick_thickness;
//  * change_label_visible;
//  * change_label_size;
//  * change_label_valign;
//  * change_label_halign;
//  * change_label_color;
//  * change_label_angle;
//  * change_label_font;
//  * change_label_fontstyle;
//  * change_label_size;
//  * change_halign;
//  * change_valign;
//  * change_border_visible;
//  * change_visible;
//  * change_depth;
//  * change_orientation;


class ChipsGuiColorBarPropWidget;
class ChipsGuiPositionWidget;
class ChipsGuiColorWidget;
class ChipsGuiPivotPointWidget;
class ChipsGuiLabelWidget;

//---------------------------------------------------------------------------

class ChipsGuiColorBarPropWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiColorBarPropWidgetCallback (void (*callback)(ChipsGuiColorBarPropWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiColorBarPropWidget *, void *);
     void *_callbackData;
};


class ChipsGuiColorBarPropWidget : public ChipsGuiWidget, public ChipsGuiTooltipObject
{
public:

  ChipsGuiColorBarPropWidget();
  virtual ~ChipsGuiColorBarPropWidget();

  virtual void build_form(); 

  virtual void show ();
  virtual void hide ();

  void display_subobject (V_SUBOBJECT_TYPE subtype);

  void set_width(float val);
  void set_perpendicular_offset(float val);
  void set_offsets (float parallel_offset, float perpendicular_offset);
  void set_parallel_offset(float val);
  void set_ticklabel_visible(bool val);
  void set_ticklabel_halign(float val);
  void set_ticklabel_valign(float val);
  void set_ticklabel_auto_valign(bool val);
  void set_ticklabel_auto_halign(bool val);
  void set_ticklabel_size(int val);
  void set_ticklabel_offset(int val);
  void set_ticklabel_fontstyle(int val);
  void set_ticklabel_font(const string &val);
  void set_ticklabel_color(const string &val);
  void set_ticklabel_angle(float val);
  void set_text(const string &val);
  void set_tick_location(int val);
  void set_tick_visible(bool val);
  void set_tick_color(const string &val);
  void set_tick_length(int val);
  void set_tick_style(int val);
  void set_tick_thickness(float val);
  void set_label_visible(bool val);
  void set_label_size(int val);
  void set_label_valign(float val);
  void set_label_halign(float val);
  void set_label_align (float hval, float vval);
  void set_label_fontstyle(int val);
  void set_label_font(const string &val);
  void set_label_color(const string &val);
  void set_label_angle(float val);
  void set_tick_mode(const string &val);
  void set_length(float val);
  void set_valign(float val);
  void set_halign(float val);
  void set_depth(int val);
  void set_visibility(bool val);
  void set_border_visibility(bool val);
  void set_orientation(int val);
  void set_position (double xpos, double ypos);
  void set_coordinate (int coordinate);
  void set_position_format (int xformat, int yformat);

  float  get_width();
  float  get_perpendicular_offset();
  float  get_parallel_offset();
  void   get_offsets (float &perpendicular_offset, float &parallel_offset);
  bool   get_ticklabel_visible();
  float  get_ticklabel_halign();
  float  get_ticklabel_valign();
  bool   get_ticklabel_auto_valign();
  bool   get_ticklabel_auto_halign();
  int    get_ticklabel_size();
  int    get_ticklabel_offset();
  int    get_ticklabel_fontstyle();
  string get_ticklabel_font();
  string get_ticklabel_color();
  float  get_ticklabel_angle();
  string get_text();
  int    get_tick_location();
  bool   get_tick_visible();
  string get_tick_color();
  int    get_tick_length();
  int    get_tick_style();
  float  get_tick_thickness();
  bool   get_label_visible();
  int    get_label_size();
  float  get_label_valign();
  float  get_label_halign();
  int    get_label_fontstyle();
  string get_label_font();
  string get_label_color();
  float  get_label_angle();
  string get_tick_mode();
  float  get_length();
  float  get_valign();
  float  get_halign();
  int    get_depth();
  bool   get_visibility();
  bool   get_border_visibility();
  int    get_orientation();
  void   get_position (double &xpos, double &ypos);
  int    get_coordinate ();
  void   get_position_format (int &xformat, int &yformat);

  void enable_width();
  void enable_offsets();
  void enable_ticklabel_visible();
  void enable_ticklabel_align();
  void enable_ticklabel_size();
  void enable_ticklabel_offset();
  void enable_ticklabel_fontstyle();
  void enable_ticklabel_font();
  void enable_ticklabel_color();
  void enable_ticklabel_angle();
  void enable_text();
  void enable_tick_location();
  void enable_tick_visible();
  void enable_tick_color();
  void enable_tick_length();
  void enable_tick_style();
  void enable_tick_thickness();
  void enable_label_visible();
  void enable_label_size();
  void enable_label_align();
  void enable_label_fontstyle();
  void enable_label_font();
  void enable_label_color();
  void enable_label_angle();
  void enable_tick_mode();
  void enable_length();
  void enable_valign();
  void enable_halign();
  void enable_depth();
  void enable_visibility();
  void enable_border_visibility();
  void enable_orientation ();
  void enable_position ();
  void enable_coordinate ();
  void enable_data_coordinate ();

  void disable_width();
  void disable_offsets();
  void disable_ticklabel_visible();
  void disable_ticklabel_align();
  void disable_ticklabel_size();
  void disable_ticklabel_offset();
  void disable_ticklabel_fontstyle();
  void disable_ticklabel_font();
  void disable_ticklabel_color();
  void disable_ticklabel_angle();
  void disable_text();
  void disable_tick_location();
  void disable_tick_visible();
  void disable_tick_color();
  void disable_tick_length();
  void disable_tick_style();
  void disable_tick_thickness();
  void disable_label_visible();
  void disable_label_size();
  void disable_label_align();
  void disable_label_fontstyle();
  void disable_label_font();
  void disable_label_color();
  void disable_label_angle();
  void disable_tick_mode();
  void disable_length();
  void disable_valign();
  void disable_halign();
  void disable_depth();
  void disable_visibility();
  void disable_border_visibility();
  void disable_orientation ();
  void disable_position ();
  void disable_coordinate ();
  void disable_data_coordinate ();



private:

  static void color_callback(ChipsGuiColorWidget *widget, void *data);
  static void thick_callback(GtkWidget *widget, gpointer data);
  static void tickmode_callback(GtkWidget *widget, gpointer data);
  static void label_text_callback(GtkWidget *widget, gpointer data);
  static void width_callback(GtkWidget *widget, gpointer data);
  static void length_callback(GtkWidget *widget, gpointer data);
  static void orientation_callback(GtkWidget *widget, gpointer data);

  static void label_color_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_angle_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_text_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_font_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_fontstyle_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_align_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_size_change_callback(ChipsGuiLabelWidget *widget, void *data);
  static void label_position_change_callback(ChipsGuiLabelWidget *widget, void *data);

  static void label_visible_change_callback(ChipsGuiLabelWidget *widget, void *data);

  static void tick_color_callback(ChipsGuiColorWidget *widget, void *data);
  static void tick_style_callback(GtkWidget *widget, gpointer data);
  static void tick_length_callback(GtkWidget *widget, gpointer data);
  static void tick_visible_callback(GtkWidget *widget, gpointer data);
  static void tick_location_callback(GtkWidget *widget, gpointer data);

  static void ticklabel_color_callback (ChipsGuiColorWidget *widget, void *data);
  static void ticklabel_font_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_fontstyle_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_size_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_angle_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_offset_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_align_callback(ChipsGuiPivotPointWidget *widget, void *data);
  static void ticklabel_visible_callback(GtkWidget *widget, gpointer data);
  static void ticklabel_halign_auto_callback(ChipsGuiPivotPointWidget *widget, void *data);
  static void ticklabel_valign_auto_callback(ChipsGuiPivotPointWidget *widget, void *data);
  static void halign_scale_change_callback(GtkWidget *widget, gpointer data);
  static void valign_scale_change_callback(GtkWidget *widget, gpointer data);
  static void halign_change_callback(GtkWidget *widget, gpointer data);
  static void valign_change_callback(GtkWidget *widget, gpointer data);

  static void depth_callback(GtkWidget *widget, gpointer data);
  static void visible_callback(GtkWidget *widget, gpointer data);
  static void border_visible_callback(GtkWidget *widget, gpointer data);

  static void position_callback(ChipsGuiPositionWidget *widget, void *data);
  static void coordinate_callback(ChipsGuiPositionWidget *widget, void *data);


   ChipsGuiColorWidget*   _color;
   GtkWidget*   _thick;
   GtkWidget*   _label_text;
   GtkWidget*   _length;
   GtkWidget*   _tickmode;
   GtkWidget*   _width;
   GtkWidget*   _orientation;

   // tick label
   ChipsGuiLabelWidget*   _label;
   
   // tick
   ChipsGuiColorWidget* _tick_color;
   GtkWidget*   _tick_style;
   GtkWidget*   _tick_length;
   GtkWidget*   _tick_visible;
   GtkWidget*   _tick_location;

   // tick labels
   GtkWidget*   _ticklabel_font;
   GtkWidget*   _ticklabel_size;
   GtkWidget*   _ticklabel_bold;
   GtkWidget*   _ticklabel_italic;
   ChipsGuiColorWidget*  _ticklabel_color;
   GtkWidget*   _ticklabel_offset;
   GtkWidget*   _ticklabel_angle;
   ChipsGuiPivotPointWidget*  _ticklabel_alignment;
   GtkWidget*   _ticklabel_visible;

   // alignment
   GtkWidget*   _halign_left;
   GtkWidget*   _halign_right;
   GtkWidget*   _halign_center;
   GtkWidget*   _halign;
   GtkWidget*   _valign_top;
   GtkWidget*   _valign_center;
   GtkWidget*   _valign_bottom;
   GtkWidget*   _valign;

   //color map
   GtkWidget* _alphamin;
   GtkWidget* _alphamax;
   GtkWidget* _alphalock;
   GdkPixbuf* _pixbuf;
   GtkWidget* _cmapimg;
   GtkWidget* _cmap;
   GtkWidget* _cmapsz;
   GtkWidget* _interp;    // pixel interpolation
   GtkWidget* _cinterp;   // color map interpolation
   GtkWidget* _invert;
   GtkWidget* _scale;
   GtkWidget* _threshmin;
   GtkWidget* _threshmax;


   GtkWidget* _depth;
   GtkWidget* _visible; 
   GtkWidget* _border_visible;

   ChipsGuiPositionWidget*  _position;
   GtkWidget*   _nbook;

   unsigned long _thick_id;
   unsigned long _tickmode_id;
   unsigned long _label_text_id;
   unsigned long _width_id;
   unsigned long _length_id;
   unsigned long _orientation_id;
   unsigned long _tick_style_id;
   unsigned long _tick_length_id;
   unsigned long _tick_visible_id; 
   unsigned long _tick_location_id;
   unsigned long _ticklabel_font_id;
   unsigned long _ticklabel_bold_id;
   unsigned long _ticklabel_italic_id;
   unsigned long _ticklabel_size_id;
   unsigned long _ticklabel_angle_id;
   unsigned long _ticklabel_offset_id;
   unsigned long _ticklabel_visible_change_id;
   unsigned long _halign_left_id;
   unsigned long _halign_right_id;
   unsigned long _halign_center_id;
   unsigned long _valign_top_id;
   unsigned long _valign_bottom_id;
   unsigned long _valign_center_id;
   unsigned long _halign_scale_id;
   unsigned long _valign_scale_id;
   unsigned long _depth_id;
   unsigned long _visible_id;
   unsigned long _border_visible_id;

   int        _label_page_num;

   float      _widthval;
   bool       _ticklabel_visibleval;
   int        _ticklabel_sizeval;
   int        _ticklabel_offsetval;
   int        _ticklabel_fontstyleval;
   string     _ticklabel_fontval;
   float      _ticklabel_angleval;
   int        _tick_locationval;
   bool       _tick_visibleval;
   int        _tick_styleval;
   int        _tick_lengthval;
   float      _tick_thicknessval;
   string     _tick_modeval;
   float      _lengthval;
   float      _valignval;
   float      _halignval;
   bool       _border_visibleval;
   int        _visibleval;
   int        _depthval;
   int        _orientationval;

};

#endif
