// 
//  Copyright (C) 2011,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUICOLORWIDGET_HH
#define CHIPSGUICOLORWIDGET_HH

#include <gtk/gtk.h>
#include <string>
#include <map>
#include <list>

#include "ChipsGuiWidget.hh"

using namespace std;

class ChipsGuiColorImage;

// valid callback ids are
//  * change_color

class ChipsGuiColorWidget;

class ChipsGuiColorWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiColorWidgetCallback (void (*callback)(ChipsGuiColorWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiColorWidget *, void *);
     void *_callbackData;     
};

class ChipsGuiColorWidget : public ChipsGuiWidget
{
   public:

      ChipsGuiColorWidget ();
      ~ChipsGuiColorWidget ();

      virtual void build_form ();

      void set_chips_color (const string &color);
      string get_chips_color ();

      void enable ();
      void disable ();

      virtual void show ();
      virtual void hide ();

      void set_foreground_color (bool flag) {_foregroundColor = flag;};

     
   protected:

      static void custom_color_clicked_callback(GtkWidget *widget, gpointer *data);
      static void default_color_clicked_callback(GtkWidget *widget, gpointer *data);
      static void standard_color_clicked_callback(GtkWidget *widget, gpointer *data);
      static void menu_clicked_callback(GtkWidget *widget, gpointer *data);

      static void free_pixbuf_data (guchar *data, void *) { delete []data;};

      void load_standard_colors ();
      void highlight_color(); 
      void update_default_color_string(); 

      GtkWidget *_toolBar;
      GtkWidget *_menu; 
      GtkWidget **_menuItem; 
      GtkWidget *_prevsel; 
      GtkWidget *_defaultItem;
      GtkToolItem *_colorButton;
      ChipsGuiColorImage *_currentColor;
      void (*_callback)(ChipsGuiColorWidget *, void *); 
      void *_callbackData;


      map<string, ChipsGuiColorImage *> _standardColors;
      list<string> _colorNames;

      bool _foregroundColor;
      string _prevclr; 


};

#endif

