// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiLevelsWidget.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiContourFrame.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsContour.hh"
#include "ChipsCmdManager.hh"

#include <config.h>

#ifdef HAVE_IEEEFP_H
#include <ieeefp.h>        // finite defined in ieeefp.h (solaris)
#else
#include <math.h>          // finite() defined here      (linux, OSX)
#endif


using namespace std;

extern ChipsGuiMain* chipsGui;

class ChipsContourLevelCallback : public ChipsGuiPropFrameCallback
{
   public:
      class ChipsContourLevelCallbackData
      {
         public:
            double *dd;
            int cnt;
      };

      virtual void execute (ChipsEngine *eng, ChipsId &cid, void *data)
      {
         ChipsContourLevelCallbackData *levels = static_cast<ChipsContourLevelCallbackData *>(data);

         if (levels && levels->cnt > 0 && eng)
            eng->set_contour_levels(&cid, levels->cnt, levels->dd);
      }
};



// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiContourFrame::ChipsGuiContourFrame():ChipsGuiPropFrame(),ChipsGuiTooltipObject()
{
   _color=NULL;
   _object_states = STATE_CHNG_PROP_CNTR;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiContourFrame::~ChipsGuiContourFrame()
{
   if (_color)   delete _color;
   if (_lw)     delete _lw; 
}

void ChipsGuiContourFrame::build_form()
{
   AtkObject *accessible;

   GtkWidget* tmp; 
   _frame = gtk_frame_new("Contour Properties");
   _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(_frame), _nbook);

   _table1 = gtk_table_new(7,7,FALSE);
   _tab = gtk_table_new(7,7,FALSE);
   _frm = gtk_frame_new("Appearance");
   gtk_frame_set_label_align(GTK_FRAME(_frm), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frm), _tab);
   gtk_table_attach(GTK_TABLE(_table1), _frm, 1, 7, 1, 3,
      GTK_EXPAND, GTK_FILL, 10, 10);
   tmp = gtk_label_new("Appearance");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table1, tmp);

   _table2 = gtk_table_new(7,7,FALSE);
   _tablvl = gtk_table_new(7,7,FALSE);
   _frmlvl = gtk_frame_new("Levels");
   gtk_frame_set_label_align(GTK_FRAME(_frmlvl), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmlvl), _tablvl);
   gtk_table_attach(GTK_TABLE(_table2), _frmlvl, 1, 7, 1, 3,
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);
   tmp = gtk_label_new("Levels");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table2, tmp);

   _table3 = gtk_table_new(7,7,FALSE);
   tmp = gtk_label_new("General");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table3, tmp);

   //  frame
   _color = new ChipsGuiColorWidget();
   _color->build_form ();
   _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_callback, this));

   _style = gtk_combo_box_new_text();
   _thick = gtk_spin_button_new_with_range(0.5, 10, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_thick), 1);
   gtk_table_attach(GTK_TABLE(_tab), _color->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tab), _style, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "no ");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "solid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "dot");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "dot longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "dot shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "shortdash longdash");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_style),1);
   tmp = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_tab), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_tab), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tab), _thick, 2, 3, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Thickness");
   gtk_table_attach(GTK_TABLE(_tab), tmp, 1, 2, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   // levels
   GType types[] = {G_TYPE_DOUBLE};
   _lw = new ChipsGuiLevelsWidget(1,types);
   if (_lw)
   {
      _lw->build_form(); 
      _lw->set_column_title ("Levels", 0);
      _lw->set_min_level (1);
      GtkWidget* bw = _lw->base_widget();
      if (bw)
      { 
         gtk_table_attach(GTK_TABLE(_tablvl), bw, 3, 7, 0, 7,
            (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5, 5);
         gtk_widget_show(bw); 
      }

      _lw->add_callback ("change_levels", new ChipsGuiLevelsWidgetCallback (levels_callback, this));
      _lw->add_callback ("delete_levels", new ChipsGuiLevelsWidgetCallback (levels_callback, this));
   }
   _mode = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_mode), "count");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_mode), "interval");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_mode), "nice");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_mode), "limits");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_mode), "arbitrary");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_mode),2);
   _interval = gtk_spin_button_new_with_range(G_MINDOUBLE, G_MAXDOUBLE, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_interval), 1);
   _count = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_count), 0);

   tmp = gtk_label_new("Mode");
   gtk_table_attach(GTK_TABLE(_tablvl), tmp, 0, 1, 1, 2,
      GTK_EXPAND, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tablvl), _mode, 1, 3, 1, 2,
      GTK_EXPAND, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Interval");
   gtk_table_attach(GTK_TABLE(_tablvl), tmp, 0, 1, 3, 4,
      GTK_EXPAND, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tablvl), _interval, 1, 3, 3, 4,
      GTK_EXPAND, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Num Levels");
   gtk_table_attach(GTK_TABLE(_tablvl), tmp, 0, 1, 5, 6,
      GTK_EXPAND, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tablvl), _count, 1, 3, 5, 6,
      GTK_EXPAND, GTK_SHRINK, 5, 5);

   // general frame
   tmp = gtk_label_new("Depth");
   gtk_table_attach(GTK_TABLE(_table3), tmp, 4, 5, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);
   gtk_table_attach(GTK_TABLE(_table3), _depth, 5, 6, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   _vis = gtk_check_button_new();
   tmp = gtk_label_new("Visible");
   gtk_table_attach(GTK_TABLE(_table3), tmp, 4, 5, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table3), _vis, 5, 6, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   int ndx = 1; 
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_style), "changed",
        G_CALLBACK(style_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_thick), "value-changed",
        G_CALLBACK(thick_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_vis), "toggled",
        G_CALLBACK(vis_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_depth), "value-changed",
        G_CALLBACK(depth_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_mode), "changed",
        G_CALLBACK(mode_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_interval), "value-changed",
        G_CALLBACK(interval_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_count), "value-changed",
        G_CALLBACK(count_callback), (void*)this);

   setup_tooltip(_color->base_widget(), "Sets the color of the contour lines");
   setup_tooltip(_style,"Sets the pattern used to display line segments");
   setup_tooltip(_thick,"Controls how thick the line segments are drawn");
   setup_tooltip(_mode,"Specifies the mode used to determine contour levels");
   setup_tooltip(_interval,"Specifies the distance to use between levels in Interval mode");
   setup_tooltip(_count,"Suggests the number of levels to use in Count mode");
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the contour is drawn");
   setup_tooltip(_vis,"Should the contour be displayed or hidden");


   // add accessibility info
   accessible = gtk_widget_get_accessible (_frame);
   atk_object_set_name (accessible, "contourProperties");
   atk_object_set_description (accessible, "Contour properties frame\n");
   accessible = gtk_widget_get_accessible (_style);
   atk_object_set_name (accessible, "style");
   atk_object_set_description (accessible, "Contour line styles\n");
   accessible = gtk_widget_get_accessible (_thick);
   atk_object_set_name (accessible, "thickness");
   atk_object_set_description (accessible, "Contour line thickness\n");
   accessible = gtk_widget_get_accessible (_color->base_widget ());
   atk_object_set_name (accessible, "color");
   atk_object_set_description (accessible, "Contour line color\n");
   accessible = gtk_widget_get_accessible (_mode);
   atk_object_set_name (accessible, "mode");
   atk_object_set_description (accessible, "Contour levels mode \n");
   accessible = gtk_widget_get_accessible (_interval);
   atk_object_set_name (accessible, "interval");
   atk_object_set_description (accessible, "Contour level intervals \n");
   accessible = gtk_widget_get_accessible (_count);
   atk_object_set_name (accessible, "count");
   atk_object_set_description (accessible, "Contour requested number of levels \n");
   accessible = gtk_widget_get_accessible (_depth);
   atk_object_set_name (accessible, "depth");
   atk_object_set_description (accessible, "Contour depth \n");
   accessible = gtk_widget_get_accessible (_vis);
   atk_object_set_name (accessible, "visible");
   atk_object_set_description (accessible, "Contour visibility\n");





}

void ChipsGuiContourFrame::set_color(string color)
{
   set_engine_property((void*)color.c_str(), SET_CONTOUR_COLOR); 
}

void ChipsGuiContourFrame::set_thick(float val)
{
   set_engine_property(static_cast<void*>(&val), 
      SET_CONTOUR_THICKNESS); 
   if (val < .75)  _thick_v = 0.5;
      else         _thick_v = (int)val;  
   g_signal_handler_block(GTK_SPIN_BUTTON(_thick), _cb_sigs[2]);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_thick), _thick_v);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_thick), _cb_sigs[2]);
   
}

void ChipsGuiContourFrame::set_style(string val)
{
   _style_v = val; 
   set_engine_property((void*)(_style_v.c_str()),
      SET_CONTOUR_STYLE);
}

void ChipsGuiContourFrame::set_visibility(bool val)
{
   _vis_v = val; 
   int ival = val; 
   set_engine_property(static_cast<void*>(&ival),
      SET_CONTOUR_VISIBLE);
}

void ChipsGuiContourFrame::set_depth(int val)
{
   _depth_v = val; 
   set_engine_property(static_cast<void*>(&val),
      SET_CONTOUR_DEPTH);
}

void ChipsGuiContourFrame::set_mode(string val)
{
   _mode_v = val; 
   set_engine_property((void*)(_mode_v.c_str()),
      SET_CONTOUR_MODE);
   reset_levels(); 
}

void ChipsGuiContourFrame::set_interval(double val)
{
   _interval_v = val; 
   _mode_v = "interval"; 
   set_engine_property(static_cast<void*>(&val),
      SET_CONTOUR_INTERVAL);
   reset_levels(); 
   g_signal_handler_block(GTK_COMBO_BOX(_mode), _cb_sigs[5]);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_mode),(int)1);
   g_signal_handler_unblock(GTK_COMBO_BOX(_mode), _cb_sigs[5]);

}

void ChipsGuiContourFrame::set_count(int val)
{
   _count_v = val; 
   _mode_v = "count"; 
   set_engine_property(static_cast<void*>(&val),
      SET_CONTOUR_NUMLEVELS);
   reset_levels(); 
   g_signal_handler_block(GTK_COMBO_BOX(_mode), _cb_sigs[5]);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_mode),(int)0);
   g_signal_handler_unblock(GTK_COMBO_BOX(_mode), _cb_sigs[5]);
}


void ChipsGuiContourFrame::color_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);

   if (!cntrFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
   cntrFrame->set_color(clr);
}

void ChipsGuiContourFrame::style_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   const char* tv = v_line_to_string((V_LINE_TYPE)val);
   if (cntrFrame && tv) 
   {
      string sv = tv; 
      cntrFrame->set_style(sv); 
   }
}

void ChipsGuiContourFrame::thick_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   float fval = val;
   if (val == 0) fval = 0.5;

   if (cntrFrame)  cntrFrame->set_thick(fval);
}

void ChipsGuiContourFrame::vis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (cntrFrame)  cntrFrame->set_visibility(val);
}

void ChipsGuiContourFrame::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (cntrFrame)  cntrFrame->set_depth(val); 
}

void ChipsGuiContourFrame::mode_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);
   char* tv = gtk_combo_box_get_active_text(GTK_COMBO_BOX(widget));
   if (cntrFrame && tv)
   {
      string sv = tv;
      cntrFrame->set_mode(sv);
      g_free (tv);
   }
}

void ChipsGuiContourFrame::interval_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);
   double val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (cntrFrame)  cntrFrame->set_interval(val);
}

void ChipsGuiContourFrame::count_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiContourFrame *cntrFrame = static_cast <ChipsGuiContourFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (cntrFrame)  cntrFrame->set_count(val);
}

void ChipsGuiContourFrame::reset_levels()
{
   vector<double> levs;
   _eng->get_contour_levels(&_cid, levs);
   if (levs.size())
   {
      double *dbl = new double[levs.size()];
      int cnt = 0;
      vector<double>::iterator vi;
      for(vi=levs.begin(); vi != levs.end(); vi++)
         dbl[cnt++] = (*vi);
      _lw->initialize_column(dbl, cnt, 0);
      delete []dbl;
   }
}

void ChipsGuiContourFrame::initialize(ChipsId* id)
{
   if (!id)  return; 

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   if (id != &_cid)
   {
      _cid.reset_window_id(id->get_window_id()); 
      _cid.set_frame_id(id->get_frame_id()); 
      _cid.set_plot_id(id->get_plot_id()); 
      _cid.set_contour_id(id->get_contour_id()); 
   }

   // get values
   _eng->get_property(&_cid, (void*)&_color_v, SET_CONTOUR_COLOR);
   _eng->get_property(&_cid, (void*)&_thick_v, SET_CONTOUR_THICKNESS);
   _eng->get_property(&_cid, static_cast<void*>(&_style_v), 
      SET_CONTOUR_STYLE);
   _eng->get_property(&_cid, (void*)&_depth_v, SET_CONTOUR_DEPTH);
   _eng->get_property(&_cid, (void*)&_vis_v, SET_CONTOUR_VISIBLE);
   vector<double> levs;
   _eng->get_contour_levels(&_cid, levs);
   _eng->get_property(&_cid, static_cast<void*>(&_mode_v),
      SET_CONTOUR_MODE);
   _eng->get_property(&_cid, (void*)&_interval_v, SET_CONTOUR_INTERVAL);
   _eng->get_property(&_cid, (void*)&_count_v, SET_CONTOUR_NUMLEVELS);

   //  values
   g_signal_handler_block(GTK_COMBO_BOX(_style), _cb_sigs[1]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_thick), _cb_sigs[2]);

   V_LINE_TYPE tmpa; 
   if (string_to_v_line(_style_v.c_str(), &tmpa) != 0)  tmpa = chips_solid;
   gtk_combo_box_set_active(GTK_COMBO_BOX(_style),(int)tmpa);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_thick), _thick_v);
   _color->set_chips_color (_color_v);

   g_signal_handler_unblock(GTK_COMBO_BOX(_style), _cb_sigs[1]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_thick), _cb_sigs[2]);

   // general values 
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_vis), _cb_sigs[3]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_depth), _cb_sigs[4]);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_vis), _vis_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depth_v);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_vis), _cb_sigs[3]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_depth), _cb_sigs[4]);

   // limits values 
   if (levs.size()) 
   {
      double *dbl = new double[levs.size()];
      int cnt = 0; 
      vector<double>::iterator vi;
      for(vi=levs.begin(); vi != levs.end(); vi++)
         dbl[cnt++] = (*vi); 
      _lw->initialize_column(dbl, cnt, 0);  
      delete []dbl;
   }

   g_signal_handler_block(GTK_COMBO_BOX(_mode), _cb_sigs[5]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_interval), _cb_sigs[6]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_count), _cb_sigs[7]);

   V_TICKMODE_TYPE tmt;
   if (string_to_v_tickmode(_mode_v.c_str(), &tmt) == 0) 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_mode),(int)tmt);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_interval), _interval_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_count), _count_v);

   g_signal_handler_unblock(GTK_COMBO_BOX(_mode), _cb_sigs[5]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_interval), _cb_sigs[6]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_count), _cb_sigs[7]);

}

void ChipsGuiContourFrame::levels_callback(ChipsGuiLevelsWidget *widget, void *data)
{
   ChipsGuiContourFrame *self = static_cast <ChipsGuiContourFrame *>(data);  
   ChipsContourLevelCallback::ChipsContourLevelCallbackData level_data;
   ChipsContourLevelCallback set_levels_cb;
   double *dd = NULL;
   int max_cnt = 0;
   int cnt = 0;

   widget->get_column (&dd, max_cnt, 0);

   if (!dd)
      return;

   // remove out NaN values
   for (int ii = 0; ii < max_cnt; ii++)
   {
      if (finite(dd[ii]))
         dd[cnt++] = dd[ii];
   }
   level_data.dd = dd;
   level_data.cnt = cnt;
   self->update_property (&set_levels_cb, static_cast<void *>(&level_data));

   
   delete []dd;
}

