// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUICONTOURFRAME_HH
#define CHIPSGUICONTOURFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiPropFrame.hh"
#include "ChipsGuiTooltipObject.hh"

class ChipsGuiLevelsWidget; 
class ChipsGuiColorWidget;

using namespace std;

//---------------------------------------------------------------------------

class ChipsGuiContourFrame: public ChipsGuiPropFrame, public ChipsGuiTooltipObject
{
public:

   ChipsGuiContourFrame();
   virtual ~ChipsGuiContourFrame();

   static void style_callback(GtkWidget *widget, gpointer data);
   static void thick_callback(GtkWidget *widget, gpointer data);
   static void vis_callback(GtkWidget *widget, gpointer data);
   static void depth_callback(GtkWidget *widget, gpointer data);
   static void mode_callback(GtkWidget *widget, gpointer data);
   static void interval_callback(GtkWidget *widget, gpointer data);
   static void count_callback(GtkWidget *widget, gpointer data);

   static void color_callback(ChipsGuiColorWidget *widget, void *data);
   static void levels_callback(ChipsGuiLevelsWidget *widget, void *data);

   virtual void initialize(ChipsId* id);
   virtual void build_form(); 

   virtual void display_subobject (V_SUBOBJECT_TYPE subtype) {};

   void set_color(string color); 
   void set_style(string val); 
   void set_thick(float val);
   void set_visibility(bool val); 
   void set_depth(int val); 
   void set_mode(string mode); 
   void set_interval(double interval);
   void set_count(int count); 
   void reset_levels(); 

private:
  
   GtkWidget*   _table1;
   GtkWidget*   _frm;
   GtkWidget*   _tab;
   GtkWidget*   _table2;
   GtkWidget*   _frmlvl;
   GtkWidget*   _tablvl;
   GtkWidget*   _table3;

   GtkWidget*   _style;
   GtkWidget*   _thick;

   GtkWidget*   _mode;
   GtkWidget*   _interval;
   GtkWidget*   _count;

   ChipsGuiLevelsWidget* _lw; 
   ChipsGuiColorWidget*  _color;

   GtkWidget*   _vis; 
   GtkWidget*   _depth;

   // signals for callbacks 
   unsigned long _cb_sigs[20];

   string     _color_v;     // color  
   string     _style_v;     // style  
   string     _mode_v;      // mode value 
   double     _interval_v;  // interval value 
   float      _thick_v;     // thickness 
   int        _count_v;     // suggested number of levels 
   int        _depth_v;     // depth/layer  
   bool       _vis_v; 

};

#endif
