// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "ChipsGuiEditMenu.hh"
#include "ChipsGuiSettingsDialog.hh"
#include "ChipsGuiMain.hh"
#include "ChipsErrs.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;


ChipsGuiEditMenu::ChipsGuiEditMenu() : ChipsGuiTooltipObject()
{
   int pos = 0; 
   base_w = gtk_menu_item_new_with_mnemonic("_Edit");

   menu = gtk_menu_new();

   menuItemUndo = gtk_menu_item_new_with_mnemonic(("_Undo"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemUndo, pos++);

   menuItemRedo = gtk_menu_item_new_with_mnemonic(("_Redo"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemRedo, pos++);

   menuItemClear = gtk_menu_item_new_with_mnemonic(("_Clear"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemClear, pos++);

   menuItemClearP = gtk_menu_item_new_with_mnemonic(("Clear _Plot"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemClearP, pos++);

   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, pos++);

   menuItemPrefs = gtk_menu_item_new_with_mnemonic(("_Default Preferences"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemPrefs, pos++);

   sep2 = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep2, pos++);

   menuItemConfig = gtk_menu_item_new_with_mnemonic(("_Gui Configuration"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemConfig, pos++);

   g_signal_connect_swapped(G_OBJECT(menuItemUndo), "activate",
                    G_CALLBACK (undo_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemRedo), "activate",
                    G_CALLBACK (redo_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemClear), "activate",
                    G_CALLBACK (clear_cb), (gpointer) 0);
   g_signal_connect_swapped(G_OBJECT(menuItemClearP), "activate",
                    G_CALLBACK (clear_cb), (gpointer) 1);
   g_signal_connect_swapped(G_OBJECT(menuItemPrefs), "activate",
                    G_CALLBACK (prefs_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemConfig), "activate",
                    G_CALLBACK (config_cb), (gpointer) NULL);

   setup_tooltip(menuItemUndo, "Undo the last command/commands");
   setup_tooltip(menuItemRedo, "Execute the last undone command");
   setup_tooltip(menuItemClear, "Clear - delete all windows");
   setup_tooltip(menuItemClearP, "Clear Plot - keeps the plot but removes objects contained in it");
   setup_tooltip(menuItemPrefs, "Edit the default ChIPS preference values");
   setup_tooltip(menuItemConfig, "Edit the default GUI configuration");

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);

   // gray out unimplemented features 
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemPrefs), false); 
}

ChipsGuiEditMenu::~ChipsGuiEditMenu()
{
   
}

int ChipsGuiEditMenu::show()
{
   gtk_widget_show(menuItemUndo);
   gtk_widget_show(menuItemRedo);
   gtk_widget_show(menuItemClear);
   gtk_widget_show(menuItemClearP);
   gtk_widget_show(menuItemPrefs);
   gtk_widget_show(menuItemConfig);
   gtk_widget_show(base_w); 
   return 0; 
}

int ChipsGuiEditMenu::undo_cb()
{
   if (chipsGui) chipsGui->undo_command();
   return 0;
}

int ChipsGuiEditMenu::redo_cb()
{
   if (chipsGui) chipsGui->redo_command();
   return 0;
}

int ChipsGuiEditMenu::clear_cb(gpointer val)
{
   gint plot = GPOINTER_TO_INT(val);

   if (chipsGui) chipsGui->clear_command(plot);
   return 0;
}


int ChipsGuiEditMenu::prefs_cb()
{
   cout << " edit -> preferences" << endl; 
   return 0;
}

int ChipsGuiEditMenu::config_cb()
{
   ChipsGuiSettingsDialog* dialog = new ChipsGuiSettingsDialog();

   dialog->doit();  
   return 0;
}

void ChipsGuiEditMenu::set_undo_redo_sensitivity(bool undo, bool redo)
{
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemUndo), undo); 
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemRedo), redo); 
}




