// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "ChipsGuiFileMenu.hh"
#include "ChipsGuiSaveDialog.hh"
#include "ChipsGuiMain.hh"

#include <atk/atk.h>

#define CHIPSGUI_FLIST_BUF_SZ   256

using namespace std;

extern ChipsGuiMain* chipsGui;

ChipsGuiFileMenu::ChipsGuiFileMenu(): ChipsGuiTooltipObject()
{
   AtkObject *accessible;

   int pos = 0; 
   base_w = gtk_menu_item_new_with_mnemonic("_File");

   menu = gtk_menu_new();

   menuItemPrint = gtk_menu_item_new_with_mnemonic(("_Print"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemPrint, pos++);

   menuItemSaveAs = gtk_menu_item_new_with_mnemonic(("Save _As"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemSaveAs, pos++);

   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, pos++);

   menuItemQuit = gtk_menu_item_new_with_mnemonic(("_Quit"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemQuit, pos++);

   /* We can attach the Quit menu item to our exit function */
   g_signal_connect_swapped(G_OBJECT(menuItemPrint), "activate",
                    G_CALLBACK (print_cb), (gpointer) NULL);
   g_signal_connect(G_OBJECT(menuItemSaveAs), "activate",
                    G_CALLBACK(saveas_cb), (gpointer) this);
   g_signal_connect_swapped(G_OBJECT(menuItemQuit), "activate",
                    G_CALLBACK (quit_cb), (gpointer) NULL);

   setup_tooltip(menuItemPrint, "Print the window to the printer/file");
   setup_tooltip(menuItemSaveAs, "Save current state to file or script"); 
   setup_tooltip(menuItemQuit, "Exit Chips Gui" ); 

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
   _prev_dir = "";

   // add accessibility info 
   accessible = gtk_widget_get_accessible (base_w);
   atk_object_set_name (accessible, "fileMenuItem");
   atk_object_set_description (accessible, "File menu item\n");
   accessible = gtk_widget_get_accessible (menuItemSaveAs);
   atk_object_set_name (accessible, "saveasMenuItem");
   atk_object_set_description (accessible, "SaveAs menu item\n");
   accessible = gtk_widget_get_accessible (menuItemPrint);
   atk_object_set_name (accessible, "printMenuItem");
   atk_object_set_description (accessible, "Print menu item\n");
   accessible = gtk_widget_get_accessible (menuItemQuit);
   atk_object_set_name (accessible, "quitMenuItem");
   atk_object_set_description (accessible, "Quit menu item\n");

  

}

ChipsGuiFileMenu::~ChipsGuiFileMenu()
{
   
}

int ChipsGuiFileMenu::show()
{
   gtk_widget_show(menuItemQuit);
   gtk_widget_show(base_w); 
   return 0; 
}

int ChipsGuiFileMenu::print_cb()
{
   if (chipsGui)  chipsGui->print_command(); 
   return 0;
}


int ChipsGuiFileMenu::saveas_cb(GObject *object, gpointer user_data)
{
   int status=0;
   ChipsGuiSaveDialog* fd = new ChipsGuiSaveDialog();
   status = fd->doit();
   return status; 
}


int ChipsGuiFileMenu::quit_cb()
{
   if (chipsGui)  chipsGui->exit_cb(); 
   return 0;
}




