// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiFontInfo.hh"
#include "ChipsFontList.hh"

extern ChipsFontList* theFonts;

ChipsGuiFontInfo::ChipsGuiFontInfo()
{
   _font_list = NULL;
   _size_list = NULL;
   _initialized = false;
   _table_size = 0; 
}

ChipsGuiFontInfo::~ChipsGuiFontInfo ()
{
}

void ChipsGuiFontInfo::initialize ()
{
   if (this->_initialized)
      return;

   _font_list = gtk_list_store_new (1, G_TYPE_STRING);
   _size_list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_INT);

   this->update_font_list ();
   this->build_size_list (_size_list);

   this->_initialized = true;
}


void ChipsGuiFontInfo::update_font_list ()
{
   if (!theFonts)
      return;

   _font_names.clear ();
   theFonts->get_font_names (_font_names);
   _font_names.sort ();
   this->build_font_list (_font_list);
}

void ChipsGuiFontInfo::build_font_list (GtkListStore *font_list)
{
   GtkTreeIter iter;
   list<string>::iterator ii;
   int cnt = 0;

   if (!font_list)
      return;

   // since we retain the loaded fonts for the length of the session,
   // the table size only changes when a new entry is added. We only
   // need to rebuild our font list when there is a change
   if (_font_names.size() == _table_size) return;  

   // emptying out the list store such as via gtk_list_store_clear(font_list)
   // results in Gtk-CRITICAL warnings- so for now count the number of entries
   // in the list and delete them after inserting the new list- alternatively
   // we could merge the new list with the existing store
   if (_table_size > 0)
   {
      cnt = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(font_list), NULL);
      _table_size = 0;
   }
   for (ii=_font_names.begin (); ii!=_font_names.end (); ii++)
   {
      gtk_list_store_append( font_list, &iter );
      gtk_list_store_set( font_list, &iter, 0, ii->c_str (), -1 );
      _table_size++; //set table_size to size of new table 
   }

   if (cnt)  // remove the old entries from the table
   {
      gtk_tree_model_get_iter_first(GTK_TREE_MODEL(font_list), &iter);
      for (int rr = 0; rr < cnt; rr++)  gtk_list_store_remove(font_list, &iter);
   }
}

void ChipsGuiFontInfo::build_size_list (GtkListStore *size_list)
{
   GtkTreeIter iter;

   if (!size_list)
      return;

   gtk_list_store_clear (size_list);

   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "2", 1, 2, -1 );
   gtk_list_store_append( size_list, &iter);
   gtk_list_store_set( size_list, &iter, 0, "4", 1, 4, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "6", 1, 6, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "8", 1, 8, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "9", 1, 9, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "10", 1, 10, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "11", 1, 11, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "12", 1, 12, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "13", 1, 13,  -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "14", 1, 14, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "15", 1, 15, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "16", 1, 16, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "18", 1, 18, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "20", 1, 20, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "22", 1, 22, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "24", 1, 24, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "28", 1, 28, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "32", 1, 32, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "36", 1, 36, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "40", 1, 40, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "44", 1, 44, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "48", 1, 48, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "56", 1, 56, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "64", 1, 64, -1 );
   gtk_list_store_append( size_list, &iter );
   gtk_list_store_set( size_list, &iter, 0, "72", 1, 72, -1 );



}

