// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiFrameFrame.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsFrame.hh"
#include "ChipsCmdManager.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiFrameFrame::ChipsGuiFrameFrame():ChipsGuiPropFrame(),ChipsGuiTooltipObject()
{
   _color = NULL;
   _bordercolor = NULL;
   _object_states = STATE_CHNG_PROP_FRM;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiFrameFrame::~ChipsGuiFrameFrame()
{
   if (_color)
      delete _color;
   if (_bordercolor)
      delete _bordercolor;
}

void ChipsGuiFrameFrame::build_form()
{
   AtkObject *accessible;

   GtkWidget* tmplabel;
   _frame = gtk_frame_new("Frame Properties");
   _table = gtk_table_new(7,7,FALSE);
   gtk_container_add (GTK_CONTAINER (_frame), _table);
   _color = new ChipsGuiColorWidget();
   _color->build_form ();
   _color->set_foreground_color (false);
   _trans =  gtk_check_button_new();
   _bordercolor = new ChipsGuiColorWidget();
   _bordercolor->build_form ();
   _bordervis =  gtk_check_button_new();
   _scale =  gtk_check_button_new();

   _frame_bg = gtk_frame_new("Background");
   gtk_frame_set_label_align(GTK_FRAME(_frame_bg), .5, .5); 
   _frame_bdr = gtk_frame_new("Border");
   gtk_frame_set_label_align(GTK_FRAME(_frame_bdr), .5, .5); 
   _table_bg = gtk_table_new(7,3,FALSE);
   gtk_container_add (GTK_CONTAINER (_frame_bg), _table_bg);
   _table_bdr = gtk_table_new(7,3,FALSE);
   gtk_container_add (GTK_CONTAINER (_frame_bdr), _table_bdr);

   gtk_table_attach(GTK_TABLE(_table), _frame_bg, 1, 7, 1, 3,
      GTK_EXPAND, GTK_FILL, 10, 10);
   gtk_table_attach(GTK_TABLE(_table), _frame_bdr, 1, 7, 3, 5,
      GTK_EXPAND, GTK_FILL, 10, 10);

   tmplabel = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_table_bg), tmplabel, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table_bg), _color->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Transparent");
   gtk_table_attach(GTK_TABLE(_table_bg), tmplabel, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table_bg), _trans, 6, 7, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_table_bdr), tmplabel, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table_bdr), _bordercolor->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table_bdr), _bordervis, 6, 7, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Visible");
   gtk_table_attach(GTK_TABLE(_table_bdr), tmplabel, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   gtk_table_attach(GTK_TABLE(_table), _scale, 5, 6, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   tmplabel = gtk_label_new("Scale Labels");
   gtk_table_attach(GTK_TABLE(_table), tmplabel, 2, 5, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

  _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_callback, this));
  _bordercolor->add_callback ("change_color", new ChipsGuiColorWidgetCallback (bordercolor_callback, this));

  setup_tooltip(_color->base_widget(), "Sets the frame's background color");
  setup_tooltip(_bordercolor->base_widget(), "Sets the frame's border color");
  setup_tooltip(_trans, "Sets the frame background transparent/opaque");
  setup_tooltip(_bordervis,"Displays border edges around the frame");
  setup_tooltip(_scale,"Resize text when when the frame size is changed");

  _trans_id = g_signal_connect(G_OBJECT(_trans),"toggled",
        G_CALLBACK(trans_callback), (void*)this);
  _scale_id = g_signal_connect(G_OBJECT(_scale),"toggled",
        G_CALLBACK(scale_callback), (void*)this);
  _bordervis_id = g_signal_connect(G_OBJECT(_bordervis),"toggled",
        G_CALLBACK(bordervis_callback), (void*)this);

   // add accessibility info
   accessible = gtk_widget_get_accessible (_frame);
   atk_object_set_name (accessible, "frameProperties");
   atk_object_set_description (accessible, "Frame properties frame");
   accessible = gtk_widget_get_accessible (_scale);
   atk_object_set_name (accessible, "scale");
   atk_object_set_description (accessible, "Frame scale flag");
   accessible = gtk_widget_get_accessible (_trans);
   atk_object_set_name (accessible, "transparent");
   atk_object_set_description (accessible, "Frame transparency flag");
   accessible = gtk_widget_get_accessible (_bordervis);
   atk_object_set_name (accessible, "bordervis");
   atk_object_set_description (accessible, "Frame border visible\n");
   accessible = gtk_widget_get_accessible (_color->base_widget ());
   atk_object_set_name (accessible, "color");
   atk_object_set_description (accessible, "Frame background color\n");
   accessible = gtk_widget_get_accessible (_bordercolor->base_widget ());
   atk_object_set_name (accessible, "bordercolor");
   atk_object_set_description (accessible, "Frame border color\n");





}

void ChipsGuiFrameFrame::set_color(string color)
{
   set_engine_property((void*)color.c_str(), SET_FRAME_COLOR); 
}

void ChipsGuiFrameFrame::set_bordercolor(string color)
{
   set_engine_property((void*)color.c_str(), SET_FRAME_BORDER_COLOR); 
}

void ChipsGuiFrameFrame::set_trans(bool trans)
{
   int ival = trans; 
   set_engine_property(static_cast<void*>(&ival),
      SET_FRAME_TRANSPARENT);
   _transval = trans; 
}

void ChipsGuiFrameFrame::set_bordervis(bool val)
{
   int ival = val; 
   set_engine_property(static_cast<void*>(&ival),
      SET_FRAME_BORDER_VIS);
   _bordervisval = val; 
}

void ChipsGuiFrameFrame::set_scale(bool val)
{
   int ival = val; 
   set_engine_property(static_cast<void*>(&ival),
      SET_FRAME_SCALE);
   _scaleval = val; 
}

void ChipsGuiFrameFrame::color_callback(ChipsGuiColorWidget *widget, gpointer data)
{
   ChipsGuiFrameFrame *frmFrame = static_cast <ChipsGuiFrameFrame*>(data);

   if (!frmFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
   frmFrame->set_color(clr);
}


void ChipsGuiFrameFrame::bordercolor_callback(ChipsGuiColorWidget *widget, gpointer data)
{
   ChipsGuiFrameFrame *frmFrame = static_cast <ChipsGuiFrameFrame*>(data);

   if (!frmFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
   frmFrame->set_bordercolor(clr);
}

void ChipsGuiFrameFrame::trans_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiFrameFrame *frmFrame = static_cast <ChipsGuiFrameFrame*>(data);
   bool trans = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (frmFrame)  frmFrame->set_trans(trans);
}

void ChipsGuiFrameFrame::scale_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiFrameFrame *frmFrame = static_cast <ChipsGuiFrameFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (frmFrame)  frmFrame->set_scale(val);
}

void ChipsGuiFrameFrame::bordervis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiFrameFrame *frmFrame = static_cast <ChipsGuiFrameFrame*>(data);
   bool vis = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (frmFrame)  frmFrame->set_bordervis(vis);
}

void ChipsGuiFrameFrame::initialize(ChipsId* id)
{
   int val; 
   string colorval;
   string bordercolorval;
   if (chipsGui && !_eng)
      _eng = chipsGui->get_engine();

   if (id != &_cid)
      _cid.set_frame_id(id->get_frame_id());

   _eng->get_property(&_cid, (void*)&colorval, SET_FRAME_COLOR);
   _eng->get_property(&_cid, (void*)&bordercolorval, SET_FRAME_BORDER_COLOR);
   _eng->get_property(&_cid, (void*)&val, SET_FRAME_TRANSPARENT);
   _transval = val; 
   _eng->get_property(&_cid, (void*)&val, SET_FRAME_SCALE);
   _scaleval = val; 
   _eng->get_property(&_cid, (void*)&val, SET_FRAME_BORDER_VIS);
   _bordervisval = val;

   g_signal_handler_block(GTK_TOGGLE_BUTTON(_bordervis), _bordervis_id);
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_scale), _scale_id);
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_trans), _trans_id);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scale), _scaleval);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_bordervis), _bordervisval);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_trans), _transval);

   _color->set_chips_color (colorval);
   _bordercolor->set_chips_color (bordercolorval);

   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_bordervis), _bordervis_id);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_scale), _scale_id);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_trans), _trans_id);

}

