// 
//  Copyright (C) 2011,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiGridPropDialog.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsDefaults.hh"
#include "ChipsEngine.hh"
#include "ChipsCmdManager.hh"
#include "ChipsId.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;
extern ChipsDefaults *theDefaults; 
extern ChipsCmdManager *cmd_manager;

ChipsGuiGridPropDialog::ChipsGuiGridPropDialog()
{
   _style_sigid = _thick_sigid = _scale_sigid = 0; 
   _color = NULL;
}

ChipsGuiGridPropDialog::~ChipsGuiGridPropDialog()
{
   if (_color)
      delete _color;
}

void ChipsGuiGridPropDialog::generate_form()
{
   base_w = gtk_dialog_new_with_buttons("Window Grid Properties", NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT, "Close", GTK_RESPONSE_OK,
	       "Reset", GTK_RESPONSE_CANCEL, NULL);

   gtk_widget_set_size_request(base_w, 400, 300);

   _table = gtk_table_new(10, 10, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _table); 

   _label[0] = gtk_label_new("scale");
   _scale = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_scale), 0, "1/4 inch");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_scale), 1, "1/2 inch");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_scale), 2, "centimeter");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_scale), 3, "1/10 window");

   _label[1] = gtk_label_new("color");
   _color = new ChipsGuiColorWidget();
   _color->build_form ();

   _label[2] = gtk_label_new("style");
   _style = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_style), 0, "solid");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_style), 1, "dot");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_style), 2, "dot longdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_style), 3, "dot shortdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_style), 4, "longdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_style), 5, "shortdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_style), 6, "shortdash longdash");

   _label[3] = gtk_label_new("thickness");
    _thickness = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 0, "0.5");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 1, "1");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 2, "2");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 3, "3");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 4, "4");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 5, "5");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 6, "6");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 7, "7");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 8, "8");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_thickness), 9, "9");

   gtk_table_attach_defaults(GTK_TABLE(_table), _label[0], 1,4,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_table), _label[1], 1,4,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_table), _label[2], 1,4,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_table), _label[3], 1,4,8,9);
   gtk_table_attach_defaults(GTK_TABLE(_table), _scale, 5,9,2,3);
   gtk_table_attach(GTK_TABLE(_table), _color->base_widget (), 5,9,4,5, GTK_SHRINK, GTK_SHRINK, 0, 0);
   gtk_table_attach_defaults(GTK_TABLE(_table), _style, 5,9,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_table), _thickness, 5,9,8,9);

   initialize(); 

   _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_cb, this));
   _style_sigid = g_signal_connect ((gpointer) _style, "changed",
                     G_CALLBACK(style_changed_cb), (void*)this);
   _scale_sigid = g_signal_connect ((gpointer) _scale, "changed",
                     G_CALLBACK(scale_changed_cb), (void*)this);
   _thick_sigid = g_signal_connect ((gpointer) _thickness, "changed",
                     G_CALLBACK(thickness_changed_cb), (void*)this);

   gtk_widget_show_all(base_w);
}

int ChipsGuiGridPropDialog::show()
{
   generate_form(); 
   return 0;
}

int ChipsGuiGridPropDialog::doit()
{
   generate_form();

   int resp, done = 0;
   while (!done) {
      resp = gtk_dialog_run(GTK_DIALOG(base_w));
      
      if (resp == GTK_RESPONSE_CANCEL)
      {
         if (_stylecv != _styleval)  set_style_value(_styleval); 
         if (_scalecv != _scaleval)  set_scale_value(_scaleval); 
         if (_thickcv != _thickval)  set_thickness_value(_thickval); 
         if (_colorv != _colorval)   change_color_value(_colorval.c_str()); 
         initialize(0);
      }
      else 
      {
         _styleval = _stylecv; 
         _scaleval = _scalecv; 
         _thickval = _thickcv; 
         done = 1;
      }
   }

   delete _color;
   _color=NULL;
   gtk_widget_destroy(base_w);
   return 0; 
}

void ChipsGuiGridPropDialog::change_color_value(const char *val)
{
   if (!val) return;

   if (!chipsGui) return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng) return;

   if (_colorv == val)  return; // no change 

   _colorv = val; 

   if (theDefaults->get_gridcolor() != _colorv)
      theDefaults->set_gridcolor(_colorv);
}

void ChipsGuiGridPropDialog::initialize(int load)
{
   int tmp; 

   if (load)
   {
      if (theDefaults)
      {
         _scaleval = theDefaults->get_gridscale();
         _styleval = theDefaults->get_gridlinestyle();
         _thickval = theDefaults->get_gridthickness(); 
         _colorval = theDefaults->get_gridcolor(); 
      }
      else 
      { 
         _styleval = 1;
         _scaleval = 0; 
         _thickval = 0.5;
         _colorval = "gray";
      }
   }

   // need to convert color string (ie. 'lime') to gdk rgb value 
   _color->set_chips_color (_colorval);

   gtk_combo_box_set_active(GTK_COMBO_BOX(_thickness),(int)_thickval);

   switch(_scaleval)
   {
      case V_GRID_SCALE_QTR_INCH:    tmp = 0;   break;
      case V_GRID_SCALE_HALF_INCH:   tmp = 1;   break;
      case V_GRID_SCALE_CENTIMETER:  tmp = 2;   break;
      case V_GRID_SCALE_NORMALIZED:  tmp = 3;   break;
      default:                       tmp = 0;   break;
   }
   gtk_combo_box_set_active(GTK_COMBO_BOX(_scale), tmp);
  
   switch(_styleval)
   {
      case chips_solid:              tmp = 0;   break; 
      case chips_dot:                tmp = 1;   break;
      case chips_dotlongdash:        tmp = 2;   break; 
      case chips_dotshortdash:       tmp = 3;   break;
      case chips_longdash:           tmp = 4;   break;
      case chips_shortdash:          tmp = 5;   break; 
      case chips_shortdashlongdash:  tmp = 6;   break; 
      default:                       tmp = 1;   break; 
   } 
   gtk_combo_box_set_active(GTK_COMBO_BOX(_style), tmp);

   // store defaults
   _stylecv = _styleval; 
   _scalecv = _scaleval; 
   _thickcv = _thickval; 
   _colorv  = _colorval; 
}

void ChipsGuiGridPropDialog::color_cb(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiGridPropDialog *dlg = static_cast <ChipsGuiGridPropDialog *>(data);
   string clr = widget->get_chips_color ();
   dlg->change_color_value(clr.c_str ()); 
}

void ChipsGuiGridPropDialog::style_changed_cb(GtkWidget *widget, gpointer data)
{
   ChipsGuiGridPropDialog *dlg = static_cast <ChipsGuiGridPropDialog *>(data);
   if (!dlg) return; 

   int val = chips_dot; 
   switch (gtk_combo_box_get_active(GTK_COMBO_BOX(widget)))
   {
      case 0:    val = chips_solid;  break;
      case 1:    val = chips_dot;  break;
      case 2:    val = chips_dotlongdash;  break;
      case 3:    val = chips_dotshortdash;  break;
      case 4:    val = chips_longdash;  break;
      case 5:    val = chips_shortdash;  break;
      case 6:    val = chips_shortdashlongdash;  break;
      default:   val = chips_dot;  break;
   }
   dlg->set_style_value(val);
}

void ChipsGuiGridPropDialog::scale_changed_cb(GtkWidget *widget, gpointer data)
{
   ChipsGuiGridPropDialog *dlg = static_cast <ChipsGuiGridPropDialog *>(data);
   if (!dlg) return; 

   int val = V_GRID_SCALE_QTR_INCH; 
   switch (gtk_combo_box_get_active(GTK_COMBO_BOX(widget)))
   {
      case 0:    val = V_GRID_SCALE_QTR_INCH;    break;
      case 1:    val = V_GRID_SCALE_HALF_INCH;   break;
      case 2:    val = V_GRID_SCALE_CENTIMETER;  break;
      case 3:    val = V_GRID_SCALE_NORMALIZED;  break;
      default:   val = V_GRID_SCALE_QTR_INCH;    break;
   }
   dlg->set_scale_value(val);
}


void ChipsGuiGridPropDialog::set_scale_value(int val)
{
   if (!chipsGui) return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng) return; 
   _scalecv = val;

   if (theDefaults->get_gridscale() != _scalecv)
      theDefaults->set_gridscale(_scalecv);
}

void ChipsGuiGridPropDialog::thickness_changed_cb(GtkWidget *widget, gpointer data)
{
   ChipsGuiGridPropDialog *dlg = static_cast <ChipsGuiGridPropDialog *>(data);
   if (!dlg) return; 
   int tmp = gtk_combo_box_get_active(GTK_COMBO_BOX(widget)); 
   float val;
   switch (tmp)
   {
      case 0:    val = 0.5;  break;
      default:   val = tmp;  break;
   }
   dlg->set_thickness_value(val);
}

void ChipsGuiGridPropDialog::set_style_value(int val)
{
   if (!chipsGui) return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng) return;
   _stylecv = val; 

   if (theDefaults->get_gridlinestyle() != _stylecv)
   {
      const char* tv = v_line_to_string((V_LINE_TYPE)_stylecv);
      if (tv)  theDefaults->set_gridlinestyle(tv);
   }
}

void ChipsGuiGridPropDialog::set_thickness_value(float val)
{
   if (!chipsGui) return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng) return;
   _thickcv = val; 

   if (theDefaults->get_gridthickness() != _thickcv)
      theDefaults->set_gridthickness(_thickcv);
}



