// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiMain.hh"
#include "ChipsGuiHelpMenu.hh"
#include "ChipsGuiRefMan.hh"
#include "ChipsDefines.hh"

#include <stdlib.h>
#include <unistd.h>

using namespace std;

ChipsGuiHelpMenu::ChipsGuiHelpMenu() : ChipsGuiTooltipObject()
{
  //HELP (base_w)
    // Reference Manual (menuref)
    // FAQ (menuFaq)
    // About (menuAbout)
    //Acknowledgements (menuAck)
  //------------------

   int pos = 0; 
   base_w = gtk_menu_item_new_with_mnemonic("_Help");

   menu = gtk_menu_new();

   menuRef = gtk_menu_item_new_with_mnemonic("Reference _Manual");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuRef, pos++);

#ifdef WLM_USE_FAQ
   menuFaq = gtk_menu_item_new_with_mnemonic("_FAQ");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuFaq, pos++);
#else
   menuFaq = NULL;
#endif

   menuAbout = gtk_menu_item_new_with_label("About");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuAbout, pos++);

#ifdef WLM_USE_ACK_MENU
   menuAck = gtk_menu_item_new_with_label("Acknowledgements");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuAck, pos++);
   g_signal_connect_swapped (G_OBJECT(menuAck), "activate",
                             G_CALLBACK(acknowledgement_cb),
                             (gpointer) NULL);
#else 
   menuAck = NULL; 
#endif

   g_signal_connect_swapped (G_OBJECT(menuRef), "activate",
                             G_CALLBACK(reference_manual_cb),
                             (gpointer) this);
#ifdef WLM_USE_FAQ
   g_signal_connect_swapped (G_OBJECT(menuFaq), "activate",
                             G_CALLBACK(faq_cb),
                             (gpointer) NULL);
#endif
   g_signal_connect_swapped (G_OBJECT(menuAbout), "activate",
                             G_CALLBACK(about_cb),
                             (gpointer) NULL);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);

   refMan = new ChipsGuiRefMan(); 

   setup_tooltip(menuRef, "Display contents of the reference manual");
   setup_tooltip(menuAbout, "Display version info");
#ifdef WLM_USE_FAQ
   setup_tooltip(menuFaq, "Display Frequently Asked Questions");
   // gray out unimplemented features
   gtk_widget_set_sensitive(GTK_WIDGET(menuFaq), false);
#endif
#ifdef WLM_USE_ACK_MENU
   setup_tooltip(menuAck, "Display acknowledgements");
#endif


}

ChipsGuiHelpMenu::~ChipsGuiHelpMenu()
{
   if (refMan)  
   {
      delete refMan; 
      refMan = NULL; 
   } 
}

void ChipsGuiHelpMenu::cleanup()
{
   // delete/hide any open dialogs 
   if (refMan) refMan->remove_dialog(); 
}

int ChipsGuiHelpMenu::show()
{
   gtk_widget_show_all(base_w); 
   return 0; 
}

void ChipsGuiHelpMenu::launch_ref_manual()
{
   if (refMan)  refMan->doit();
}

int ChipsGuiHelpMenu::reference_manual_cb(gpointer data)
{
   ChipsGuiHelpMenu *my = static_cast<ChipsGuiHelpMenu *>(data);
   if (my)  my->launch_ref_manual(); 
   return 0; 
}

int ChipsGuiHelpMenu::faq_cb()
{
   return 0; 
}

int ChipsGuiHelpMenu::about_cb()
{
   gtk_show_about_dialog(NULL, 
      "program-name", "Chips Gui", 
      "title", "About Chips Gui",
      "copyright", CHIPS_COPYRIGHT_STR,
      "version", CHIPS_VERSION_STR,
      NULL);  
   return 0; 
}

int ChipsGuiHelpMenu::acknowledgement_cb()
{
   return 0; 
}


