// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiHistogramFrame.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsHistogram.hh"
#include "ChipsCmdManager.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiHistogramFrame::ChipsGuiHistogramFrame():ChipsGuiPropFrame(),ChipsGuiTooltipObject()
{
   _linecolor = NULL;
   _symcolor = NULL;
   _fillcolor = NULL;
   _errcolor = NULL;
   _object_states = STATE_CHNG_PROP_HSTG;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiHistogramFrame::~ChipsGuiHistogramFrame()
{
   if (_linecolor)
      delete _linecolor;
   if (_symcolor)
      delete _symcolor;
   if (_errcolor)
      delete _errcolor;
   if (_fillcolor)
      delete _fillcolor;
}

void ChipsGuiHistogramFrame::build_form()
{
   AtkObject *accessible;

   GtkWidget* tmp; 
   _frame = gtk_frame_new("Histogram Properties");
   _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(_frame), _nbook);

   _vbox = gtk_vbox_new (FALSE, 5);
   _tabline = gtk_table_new(7,7,FALSE);
   _tabfill = gtk_table_new(7,7,FALSE);
   _tabsym = gtk_table_new(7,7,FALSE);
   _frmline = gtk_frame_new("Lines");
   gtk_frame_set_label_align(GTK_FRAME(_frmline), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmline), _tabline);
   gtk_box_pack_start(GTK_BOX(_vbox), _frmline, FALSE, FALSE,5);
   _frmfill = gtk_frame_new("Fill");
   gtk_frame_set_label_align(GTK_FRAME(_frmfill), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmfill), _tabfill);
   gtk_box_pack_start(GTK_BOX(_vbox), _frmfill, FALSE, FALSE,5);
   tmp = gtk_label_new("Lines / Fill");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _vbox, tmp);

   _table2 = gtk_table_new(7,7,FALSE);
   _frmsym = gtk_frame_new("Symbols");
   gtk_frame_set_label_align(GTK_FRAME(_frmsym), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmsym), _tabsym);
   gtk_table_attach(GTK_TABLE(_table2), _frmsym, 1, 7, 3, 5,
      GTK_EXPAND, GTK_FILL, 10, 10);
   tmp = gtk_label_new("Symbols");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table2, tmp);

   _table3 = gtk_table_new(7,7,FALSE);
   _taberr = gtk_table_new(7,7,FALSE);
   _frmerr = gtk_frame_new("Error Bars");
   gtk_frame_set_label_align(GTK_FRAME(_frmerr), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmerr), _taberr);
   gtk_table_attach(GTK_TABLE(_table3), _frmerr, 1, 7, 1, 3,
      GTK_EXPAND, GTK_FILL, 10, 10);
   tmp = gtk_label_new("Error Bars");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table3, tmp);

   _table4 = gtk_table_new(7,7,FALSE);
   tmp = gtk_label_new("General");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table4, tmp);

   // line frame
   _linecolor = new ChipsGuiColorWidget();
   _linecolor->build_form ();
   _linestyle = gtk_combo_box_new_text();
   _linethick = gtk_spin_button_new_with_range(0.5, 10, 1);
   _dropline  = gtk_check_button_new();
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_linethick), 1);
   gtk_table_attach(GTK_TABLE(_tabline), _linecolor->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabline), _linestyle, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabline), _dropline, 5, 6, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "no line");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "solid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "dot");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "dot longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "dot shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "shortdash longdash");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_linestyle),1);
   tmp = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_tabline), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_tabline), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabline), _linethick, 2, 3, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Thickness");
   gtk_table_attach(GTK_TABLE(_tabline), tmp, 1, 2, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Droplines");
   gtk_table_attach(GTK_TABLE(_tabline), tmp, 4, 5, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
 
   // fill frame
   _fillcolor = new ChipsGuiColorWidget();
   _fillcolor->build_form ();
   _fillstyle = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "no fill");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "solid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "up diagonal");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "down diagonal");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "horizontal");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "vertical");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "crisscross");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "brick");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "grid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "hexagon");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "polkadot");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "zigzag");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "wave");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "flower");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "custom 1");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "custom 2");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "custom 3");

   _opacity = gtk_hscale_new_with_range(0,1,.01);
   gtk_scale_set_digits(GTK_SCALE(_opacity), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_opacity), true);

   tmp = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_tabfill), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabfill), _fillcolor->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_tabfill), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabfill), _fillstyle, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabfill), _opacity, 1, 6, 4, 5,
      GTK_FILL, GTK_FILL, 10, 10);
   tmp = gtk_label_new("Opacity");
   gtk_table_attach(GTK_TABLE(_tabfill), tmp, 3, 5, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   // symbol frame 
   _symcolor = new ChipsGuiColorWidget();
   _symcolor->build_form ();
   _symstyle = gtk_combo_box_new_text();
   _symsize = gtk_spin_button_new_with_range(1, 100, 1);
   _symfill =  gtk_check_button_new();
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_symsize), 0);
   _symangle = gtk_spin_button_new_with_range(-359.99, 359.99, 10.0);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_symangle), 2);
   gtk_table_attach(GTK_TABLE(_tabsym), _symcolor->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabsym), _symstyle, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "no symbol");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "cross");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "diamond");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "down triangle");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "circle");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "plus");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "square");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "up triangle");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "point");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_symstyle), "arrow");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_symstyle),1);
   tmp = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_tabsym), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_tabsym), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabsym), _symsize, 2, 3, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabsym), _symangle, 2, 3, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabsym), _symfill, 5, 6, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Size");
   gtk_table_attach(GTK_TABLE(_tabsym), tmp, 1, 2, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Fill");
   gtk_table_attach(GTK_TABLE(_tabsym), tmp, 4, 5, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Angle");
   gtk_table_attach(GTK_TABLE(_tabsym), tmp, 1, 2, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   // error frame
   _errcolor = new ChipsGuiColorWidget();
   _errcolor->build_form ();
   _errcap = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_errcap), "bar");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_errcap), "cap");
   _errcaplen = gtk_spin_button_new_with_range(1, 100, 1);
   _errthick = gtk_spin_button_new_with_range(0.5, 10, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_errthick), 1);

   _errup = gtk_check_button_new();
   _errdown = gtk_check_button_new();

   tmp = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_taberr), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_taberr), _errcolor->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_taberr), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_taberr), _errcap, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Cap Length");
   gtk_table_attach(GTK_TABLE(_taberr), tmp, 1, 2, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_taberr), _errcaplen, 2, 3, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Thickness");
   gtk_table_attach(GTK_TABLE(_taberr), tmp, 4, 5, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_taberr), _errthick, 5, 6, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Up");
   gtk_table_attach(GTK_TABLE(_taberr), tmp, 1, 2, 4, 5,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_taberr), _errup, 2, 3, 4, 5,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Down");
   gtk_table_attach(GTK_TABLE(_taberr), tmp, 4, 5, 4, 5,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_taberr), _errdown, 5, 6, 4, 5,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   // general frame
   tmp = gtk_label_new("Depth");
   gtk_table_attach(GTK_TABLE(_table4), tmp, 4, 5, 2, 3,
      GTK_FILL, GTK_FILL, 10, 10);
   _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);
   gtk_table_attach(GTK_TABLE(_table4), _depth, 5, 6, 2, 3,
      GTK_FILL, GTK_FILL, 10, 10);
   _vis = gtk_check_button_new();
   tmp = gtk_label_new("Visible");
   gtk_table_attach(GTK_TABLE(_table4), tmp, 4, 5, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);
   gtk_table_attach(GTK_TABLE(_table4), _vis, 5, 6, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);

   int ndx = 0; 
   ndx++;
   _linecolor->add_callback ("change_color", new ChipsGuiColorWidgetCallback (linecolor_callback, this));
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_linestyle), "changed",
        G_CALLBACK(linestyle_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_linethick), "value-changed",
        G_CALLBACK(linethick_callback), (void*)this);

   ndx++;
   _symcolor->add_callback ("change_color", new ChipsGuiColorWidgetCallback (symcolor_callback, this));
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_symstyle), "changed",
      G_CALLBACK(symstyle_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_symsize), "value-changed",
      G_CALLBACK(symsize_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_symangle), "value-changed",
      G_CALLBACK(symangle_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_symfill),"toggled",
      G_CALLBACK(symfill_callback), (void*)this);

   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_errup),"toggled",
      G_CALLBACK(errup_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_errdown),"toggled",
      G_CALLBACK(errdown_callback), (void*)this);
   ndx++;
   _errcolor->add_callback ("change_color", new ChipsGuiColorWidgetCallback (errcolor_callback, this));
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_errcap), "changed",
      G_CALLBACK(errcap_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_errthick), "value-changed",
      G_CALLBACK(errthick_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_errcaplen), "value-changed",
      G_CALLBACK(errcaplen_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_vis), "toggled",
      G_CALLBACK(vis_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_depth), "value-changed",
      G_CALLBACK(depth_callback), (void*)this);

  _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_dropline), "toggled",
        G_CALLBACK(dropline_callback), (void*)this);
  _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_opacity), "value-changed",
        G_CALLBACK(opacity_callback), (void*)this);
  ndx++;
  _fillcolor->add_callback ("change_color", new ChipsGuiColorWidgetCallback (fillcolor_callback, this));
  _cb_sigs[ndx] = g_signal_connect(G_OBJECT(_fillstyle), "changed",
        G_CALLBACK(fillstyle_callback), (void*)this);

   // setup tooltips 
   setup_tooltip(_linecolor->base_widget(),"Sets the color to use for the histogram's line segments");
   setup_tooltip(_linestyle, "Sets the pattern used to display line segments");
   setup_tooltip(_linethick, "Controls how thick the line segments are drawn");
   setup_tooltip(_dropline, "Should individual bin edges be drawn to zero");
   setup_tooltip(_fillcolor->base_widget(),"Sets the color to use to fill the bins");
   setup_tooltip(_fillstyle, "Sets the fill pattern to use for the bins"); 
   setup_tooltip(_opacity, "Sets the transparency level for the fill");
   setup_tooltip(_symcolor->base_widget(),"Sets the color to use for the data point markers");
   setup_tooltip(_symstyle,"Sets the shape used to display data points");
   setup_tooltip(_symsize,"Controls how large the data point symbols are drawn");
   setup_tooltip(_symfill,"Specifies whether data point symbols are filled or hollow");
   setup_tooltip(_symangle,"Specifies a rotation angle to apply to data point symbols");
   setup_tooltip(_errcolor->base_widget(),"Sets the color to use for error bars, caps, and limits");
   setup_tooltip(_errcap,"Specifies whether the error bars should be capped or not");
   setup_tooltip(_errcaplen,"Specifies how wide caps should be drawn");
   setup_tooltip(_errthick,"Controls how thick the error bars and caps are drawn");
   setup_tooltip(_errup, "Specifies whether existing vertical upper error markers should be displayed");
   setup_tooltip(_errdown, "Specifies whether existing vertical lower error markers should be displayed");
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the histogram is drawn"); 
   setup_tooltip(_vis,"Should the histogram be displayed or hidden"); 

   // add accessibility info
   accessible = gtk_widget_get_accessible (_frame);
   atk_object_set_name (accessible, "histogramProperties");
   atk_object_set_description (accessible, "Histogram properties frame\n");
   accessible = gtk_widget_get_accessible (_linestyle);
   atk_object_set_name (accessible, "lineStyle");
   atk_object_set_description (accessible, "Histogram line styles\n");
   accessible = gtk_widget_get_accessible (_linethick);
   atk_object_set_name (accessible, "lineThickness");
   atk_object_set_description (accessible, "Histogram line thickness\n");
   accessible = gtk_widget_get_accessible (_linecolor->base_widget ());
   atk_object_set_name (accessible, "lineColor");
   atk_object_set_description (accessible, "Histogram line color\n");
   accessible = gtk_widget_get_accessible (_symstyle);
   atk_object_set_name (accessible, "symbolStyle");
   atk_object_set_description (accessible, "Histogram symbol style\n");
   accessible = gtk_widget_get_accessible (_symsize);
   atk_object_set_name (accessible, "symbolSize");
   atk_object_set_description (accessible, "Histogram symbol size\n");
   accessible = gtk_widget_get_accessible (_symangle);
   atk_object_set_name (accessible, "symbolAngle");
   atk_object_set_description (accessible, "Histogram symbol angle\n");
   accessible = gtk_widget_get_accessible (_symfill);
   atk_object_set_name (accessible, "symbolFill");
   atk_object_set_description (accessible, "Histogram symbol fill\n");
   accessible = gtk_widget_get_accessible (_symcolor->base_widget ());
   atk_object_set_name (accessible, "symbolColor");
   atk_object_set_description (accessible, "Histogram symbol color\n");
   accessible = gtk_widget_get_accessible (_errup);
   atk_object_set_name (accessible, "errorUp");
   atk_object_set_description (accessible, "Histogram error up visible\n");
   accessible = gtk_widget_get_accessible (_errdown);
   atk_object_set_name (accessible, "errorDown");
   atk_object_set_description (accessible, "Histogram error down visible\n");
   accessible = gtk_widget_get_accessible (_errcap);
   atk_object_set_name (accessible, "errorCap");
   atk_object_set_description (accessible, "Histogram error cap visible\n");
   accessible = gtk_widget_get_accessible (_errcaplen);
   atk_object_set_name (accessible, "errorCapLength");
   atk_object_set_description (accessible, "Histogram error cap length\n");
   accessible = gtk_widget_get_accessible (_errthick);
   atk_object_set_name (accessible, "errorThickness");
   atk_object_set_description (accessible, "Histogram error bar thickness\n");
   accessible = gtk_widget_get_accessible (_errcolor->base_widget ());
   atk_object_set_name (accessible, "errorColor");
   atk_object_set_description (accessible, "Histogram error bar color\n");
   accessible = gtk_widget_get_accessible (_dropline);
   atk_object_set_name (accessible, "dropLine");
   atk_object_set_description (accessible, "Histogram drop line\n");
   accessible = gtk_widget_get_accessible (_fillcolor->base_widget ());
   atk_object_set_name (accessible, "fillColor");
   atk_object_set_description (accessible, "Histogram fill color\n");
   accessible = gtk_widget_get_accessible (_fillstyle);
   atk_object_set_name (accessible, "fillStyle");
   atk_object_set_description (accessible, "Histogram fill style\n");
   accessible = gtk_widget_get_accessible (_opacity);
   atk_object_set_name (accessible, "opacity");
   atk_object_set_description (accessible, "Histogram fill opacity\n");
   accessible = gtk_widget_get_accessible (_vis);
   atk_object_set_name (accessible, "visible");
   atk_object_set_description (accessible, "Histogram visible\n");
   accessible = gtk_widget_get_accessible (_depth);
   atk_object_set_name (accessible, "depth");
   atk_object_set_description (accessible, "Histogram depth\n");
}

void ChipsGuiHistogramFrame::set_linecolor(string color)
{
   set_engine_property((void*)color.c_str(), SET_HISTO_LINE_COLOR); 
}

void ChipsGuiHistogramFrame::set_linethick(float val)
{
   set_engine_property(static_cast<void*>(&val), 
      SET_HISTO_LINE_THICKNESS); 
   if (val < .75)  _linethick_v = 0.5;
      else         _linethick_v = (int)val;  
   g_signal_handler_block(GTK_SPIN_BUTTON(_linethick), _cb_sigs[2]);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_linethick), _linethick_v);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_linethick), _cb_sigs[2]);
}

void ChipsGuiHistogramFrame::set_linestyle(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_LINE_STYLE);
}

void ChipsGuiHistogramFrame::set_symcolor(string color)
{
   set_engine_property((void*)color.c_str(), SET_HISTO_SYMBOL_COLOR); 
}

void ChipsGuiHistogramFrame::set_symsize(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_SYMBOL_SIZE);
}

void ChipsGuiHistogramFrame::set_symfill(bool fill)
{
   set_engine_property(static_cast<void*>(&fill),
      SET_HISTO_SYMBOL_FILL);
}

void ChipsGuiHistogramFrame::set_symstyle(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_SYMBOL_STYLE);
}

void ChipsGuiHistogramFrame::set_symangle(float val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_SYMBOL_ANGLE);
}

void ChipsGuiHistogramFrame::set_errcolor(string color)
{
   set_engine_property((void*)color.c_str(), SET_HISTO_ERR_COLOR); 
}

void ChipsGuiHistogramFrame::set_errbars(int pos, bool val)
{
   set_engine_property(static_cast<void*>(&val),
      pos);
}

void ChipsGuiHistogramFrame::set_errcap(int val)
{
   if (!_eng) return;
   string sval = "bar";
   if (val == 1)  sval = "capped"; 
   set_engine_property((void*)sval.c_str(),
      SET_HISTO_ERR_STYLE);
}

void ChipsGuiHistogramFrame::set_errcaplen(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_ERR_CAPLEN);
}

void ChipsGuiHistogramFrame::set_errthick(float val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_ERR_THICKNESS);
   if (val < .75)  _errthick_v = 0.5;
      else         _errthick_v = (int)val;
   g_signal_handler_block(GTK_SPIN_BUTTON(_errthick), _cb_sigs[12]);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_errthick), _errthick_v);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_errthick), _cb_sigs[12]);
}

void ChipsGuiHistogramFrame::set_visibility(bool val)
{
   int ival = val; 
   set_engine_property(static_cast<void*>(&ival),
      SET_HISTO_VISIBLE);
}

void ChipsGuiHistogramFrame::set_depth(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_DEPTH);
}

void ChipsGuiHistogramFrame::set_dropline(bool val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_DROPLINE);
}

void ChipsGuiHistogramFrame::set_opacity(float val)
{
   double dval = val; 
   set_engine_property(static_cast<void*>(&dval),
      SET_HISTO_FILL_OPACITY);
}

void ChipsGuiHistogramFrame::set_fillcolor(string color)
{
   set_engine_property((void*)color.c_str(), SET_HISTO_FILL_COLOR); 
}

void ChipsGuiHistogramFrame::set_fillstyle(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_HISTO_FILL_STYLE);
}

void ChipsGuiHistogramFrame::linecolor_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);

   if (!crvFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
   crvFrame->set_linecolor(clr);
}

void ChipsGuiHistogramFrame::linestyle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (crvFrame)  crvFrame->set_linestyle(val); 
}

void ChipsGuiHistogramFrame::linethick_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   float fval = val;
   if (val == 0) fval = 0.5;

   if (crvFrame)  crvFrame->set_linethick(fval);
}

void ChipsGuiHistogramFrame::symcolor_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);

   if (!crvFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
   crvFrame->set_symcolor(clr);
}

void ChipsGuiHistogramFrame::symstyle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (crvFrame)  crvFrame->set_symstyle(val);
}

void ChipsGuiHistogramFrame::symsize_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (crvFrame)  crvFrame->set_symsize(val);
}

void ChipsGuiHistogramFrame::symangle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   float val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (crvFrame)  crvFrame->set_symangle(val);
}

void ChipsGuiHistogramFrame::symfill_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   bool fill = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (crvFrame)  crvFrame->set_symfill(fill);
}

void ChipsGuiHistogramFrame::errup_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (crvFrame)  crvFrame->set_errbars(SET_HISTO_ERR_UP, val);
}

void ChipsGuiHistogramFrame::errdown_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (crvFrame)  crvFrame->set_errbars(SET_HISTO_ERR_DOWN, val);
}

void ChipsGuiHistogramFrame::errcolor_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   if (!crvFrame)  return; //ensure we have needed pointer
   string clr = widget->get_chips_color ();
   crvFrame->set_errcolor(clr);
}

void ChipsGuiHistogramFrame::errcap_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (crvFrame)  crvFrame->set_errcap(val);
}

void ChipsGuiHistogramFrame::errthick_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   float fval = val;
   if (crvFrame)  crvFrame->set_errthick(fval);
}

void ChipsGuiHistogramFrame::errcaplen_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (crvFrame)  crvFrame->set_errcaplen(val);
}

void ChipsGuiHistogramFrame::vis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (crvFrame)  crvFrame->set_visibility(val);
}

void ChipsGuiHistogramFrame::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (crvFrame)  crvFrame->set_depth(val); 
}

void ChipsGuiHistogramFrame::fillcolor_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);

   if (!crvFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
   crvFrame->set_fillcolor(clr);
}

void ChipsGuiHistogramFrame::dropline_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (crvFrame)  crvFrame->set_dropline(val);
}

void ChipsGuiHistogramFrame::opacity_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   float val = gtk_range_get_value(GTK_RANGE(widget));
   if (crvFrame)  crvFrame->set_opacity(val);
}

void ChipsGuiHistogramFrame::fillstyle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiHistogramFrame *crvFrame = static_cast <ChipsGuiHistogramFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (crvFrame)  crvFrame->set_fillstyle(val);
}

void ChipsGuiHistogramFrame::initialize(ChipsId* id)
{
   if (!id)  return; 

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   if (id != &_cid)
   {
      _cid.reset_window_id(id->get_window_id()); 
      _cid.set_frame_id(id->get_frame_id()); 
      _cid.set_plot_id(id->get_plot_id()); 
      _cid.set_histogram_id(id->get_histogram_id()); 
   }

   // get values
   _eng->get_property(&_cid, (void*)&_linecolor_v, SET_HISTO_LINE_COLOR);
   _eng->get_property(&_cid, (void*)&_linethick_v, SET_HISTO_LINE_THICKNESS);
   _eng->get_property(&_cid, static_cast<void*>(&_linestyle_v), 
      SET_HISTO_LINE_STYLE);
   _eng->get_property(&_cid, (void*)&_depth_v, SET_HISTO_DEPTH);
   _eng->get_property(&_cid, (void*)&_dropline_v, SET_HISTO_DROPLINE);

   _eng->get_property(&_cid, (void*)&_symcolor_v, SET_HISTO_SYMBOL_COLOR);
   _eng->get_property(&_cid, (void*)&_symangle_v, SET_HISTO_SYMBOL_ANGLE);
   int ival;
   _eng->get_property(&_cid, (void*)&ival, SET_HISTO_SYMBOL_FILL);
   _symfill_v = ival; 
   _eng->get_property(&_cid, (void*)&_vis_v, SET_HISTO_VISIBLE);
   _eng->get_property(&_cid, (void*)&_symsize_v, SET_HISTO_SYMBOL_SIZE);
   _eng->get_property(&_cid, static_cast<void*>(&_symstyle_v),
      SET_HISTO_SYMBOL_STYLE);
   double dval;
   _eng->get_property(&_cid, static_cast<void*>(&dval),
      SET_HISTO_FILL_OPACITY);
   _opacity_v = dval; 
   _eng->get_property(&_cid, (void*)&_fillcolor_v, SET_HISTO_FILL_COLOR);
   _eng->get_property(&_cid, static_cast<void*>(&_fillstyle_v),
      SET_HISTO_FILL_STYLE);
   _eng->get_property(&_cid, (void*)&_errup_v, SET_HISTO_ERR_UP);
   _eng->get_property(&_cid, (void*)&_errdown_v, SET_HISTO_ERR_DOWN);
   _eng->get_property(&_cid, (void*)&_errcolor_v, SET_HISTO_ERR_COLOR);
   string tmpstr;
   _eng->get_property(&_cid, (void*)&tmpstr, SET_HISTO_ERR_STYLE);
   if (tmpstr == "capped")   _errstyle_v = 1;
      else                   _errstyle_v = 0;
   _eng->get_property(&_cid, (void*)&_errcaplen_v, SET_HISTO_ERR_CAPLEN);

   // line values
   g_signal_handler_block(GTK_COMBO_BOX(_linestyle), _cb_sigs[1]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_linethick), _cb_sigs[2]);

   gtk_combo_box_set_active(GTK_COMBO_BOX(_linestyle),(int)_linestyle_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_linethick), _linethick_v);
   _linecolor->set_chips_color (_linecolor_v);
   g_signal_handler_unblock(GTK_COMBO_BOX(_linestyle), _cb_sigs[1]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_linethick), _cb_sigs[2]);

   // symbol values 
   g_signal_handler_block(GTK_COMBO_BOX(_symstyle), _cb_sigs[4]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_symsize), _cb_sigs[5]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_symangle), _cb_sigs[6]);
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_symfill), _cb_sigs[7]);

   gtk_combo_box_set_active(GTK_COMBO_BOX(_symstyle),(int)_symstyle_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_symsize), _symsize_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_symangle), _symangle_v);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_symfill), _symfill_v);
   _symcolor->set_chips_color (_symcolor_v);
   g_signal_handler_unblock(GTK_COMBO_BOX(_symstyle), _cb_sigs[4]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_symsize), _cb_sigs[5]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_symangle), _cb_sigs[6]);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_symfill), _cb_sigs[7]);

   // error values 
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_errup), _cb_sigs[8]);
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_errdown), _cb_sigs[9]);
   g_signal_handler_block(GTK_COMBO_BOX(_errcap), _cb_sigs[11]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_errthick), _cb_sigs[12]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_errcaplen), _cb_sigs[13]);
   _errcolor->set_chips_color (_errcolor_v);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_errup), _errup_v);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_errdown), _errdown_v);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_errcap),(int)_errstyle_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_errthick), _errthick_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_errcaplen), _errcaplen_v);

   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_errup), _cb_sigs[8]);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_errdown), _cb_sigs[9]);
   g_signal_handler_unblock(GTK_COMBO_BOX(_errcap), _cb_sigs[11]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_errthick), _cb_sigs[12]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_errcaplen), _cb_sigs[13]);

   // general values 
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_vis), _cb_sigs[14]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_depth), _cb_sigs[15]);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_vis), _vis_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depth_v);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_vis), _cb_sigs[14]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_depth), _cb_sigs[15]);

   // fill values 
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_dropline), _cb_sigs[16]);
   g_signal_handler_block(GTK_SCALE(_opacity), _cb_sigs[17]);
   g_signal_handler_block(GTK_COMBO_BOX(_fillstyle), _cb_sigs[19]);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_dropline), _dropline_v);
   gtk_range_set_value(GTK_RANGE(_opacity), _opacity_v);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_fillstyle),(int)_fillstyle_v);
   _fillcolor->set_chips_color (_fillcolor_v);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_dropline), _cb_sigs[16]);
   g_signal_handler_unblock(GTK_SCALE(_opacity), _cb_sigs[17]);
   g_signal_handler_unblock(GTK_COMBO_BOX(_fillstyle), _cb_sigs[19]);
}

