// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIHISTOGRAMFRAME_HH
#define CHIPSGUIHISTOGRAMFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiPropFrame.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

class ChipsGuiColorWidget;

//---------------------------------------------------------------------------

class ChipsGuiHistogramFrame: public ChipsGuiPropFrame, public ChipsGuiTooltipObject
{
public:

   ChipsGuiHistogramFrame();
   virtual ~ChipsGuiHistogramFrame();

   static void linecolor_callback(ChipsGuiColorWidget *widget, void *data);
   static void linestyle_callback(GtkWidget *widget, gpointer data);
   static void linethick_callback(GtkWidget *widget, gpointer data);
   static void symcolor_callback(ChipsGuiColorWidget *widget, void *data);
   static void symstyle_callback(GtkWidget *widget, gpointer data);
   static void symsize_callback(GtkWidget *widget, gpointer data);
   static void symangle_callback(GtkWidget *widget, gpointer data);
   static void symfill_callback(GtkWidget *widget, gpointer data);
   static void errup_callback(GtkWidget *widget, gpointer data);
   static void errdown_callback(GtkWidget *widget, gpointer data);
   static void errcolor_callback(ChipsGuiColorWidget *widget, void *data);
   static void errcap_callback(GtkWidget *widget, gpointer data);
   static void errthick_callback(GtkWidget *widget, gpointer data);
   static void errcaplen_callback(GtkWidget *widget, gpointer data);
   static void vis_callback(GtkWidget *widget, gpointer data);
   static void depth_callback(GtkWidget *widget, gpointer data);
   static void dropline_callback(GtkWidget *widget, gpointer data);
   static void opacity_callback(GtkWidget *widget, gpointer data);
   static void fillcolor_callback(ChipsGuiColorWidget *widget, void *data);
   static void fillstyle_callback(GtkWidget *widget, gpointer data);

   virtual void initialize(ChipsId* id);
   virtual void build_form(); 

   virtual void display_subobject (V_SUBOBJECT_TYPE subtype) {};

   void set_linecolor(string color); 
   void set_linestyle(int val); 
   void set_linethick(float val);
   void set_symcolor(string color);
   void set_symstyle(int val); 
   void set_symsize(int val); 
   void set_symangle(float val);
   void set_symfill(bool val); 
   void set_depth(int val); 
   void set_visibility(bool val); 
   void set_errcolor(string color); 
   void set_errbars(int pos, bool val); 
   void set_errcap(int val);
   void set_errcaplen(int val);
   void set_errthick(float val);
   void set_fillcolor(string color); 
   void set_fillstyle(int val);
   void set_opacity(float val); 
   void set_dropline(bool val); 

private:
  
   GtkWidget*   _vbox;
   GtkWidget*   _frmline;
   GtkWidget*   _tabline;
   GtkWidget*   _frmfill;
   GtkWidget*   _tabfill;
   GtkWidget*   _table2;
   GtkWidget*   _frmsym;
   GtkWidget*   _tabsym;

   GtkWidget*   _table3;
   GtkWidget*   _frmerr;
   GtkWidget*   _taberr;
   GtkWidget*   _table4;

   ChipsGuiColorWidget*   _linecolor;
   GtkWidget*   _linestyle;
   GtkWidget*   _linethick;
   ChipsGuiColorWidget*   _symcolor;
   GtkWidget*   _symstyle;
   GtkWidget*   _symsize;
   GtkWidget*   _symangle;
   GtkWidget*   _symfill;

   GtkWidget*   _errleft; 
   GtkWidget*   _errright; 
   GtkWidget*   _errup; 
   GtkWidget*   _errdown; 
   ChipsGuiColorWidget*   _errcolor;
   GtkWidget*   _errcap;
   GtkWidget*   _errcaplen;
   GtkWidget*   _errthick;

   GtkWidget*   _dropline; 
   
   ChipsGuiColorWidget*   _fillcolor;
   GtkWidget*   _fillstyle;
   GtkWidget*   _opacity;

   GtkWidget*   _vis; 
   GtkWidget*   _depth;

   // signals for callbacks 
   unsigned long _cb_sigs[20];

   bool       _errup_v;         // display up error bar?
   bool       _errdown_v;       // display down error bar?
   int        _errstyle_v;      // error bar style- capped or line
   string     _errcolor_v;      // error bar color
   float      _errthick_v;      // error bar thickness
   int        _errcaplen_v;     // error bar cap length
   string     _linecolor_v;     // color of line
   V_LINE_TYPE _linestyle_v;    // style of line
   float      _linethick_v;     // thickness of line
   int        _dropline_v; 
   V_FILL_TYPE _fillstyle_v;
   float      _opacity_v;
   string     _fillcolor_v; 
   string     _symcolor_v;      // color of symbol
   V_SYMBOL_TYPE _symstyle_v;
   int        _symsize_v;       // size of symbol
   float      _symangle_v;      // angle of rotation
   bool       _symfill_v;       // solid or hollow point
   int        _depth_v;         // depth/layer  
   bool       _vis_v; 

};

#endif
