// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiImageFrame.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsImage.hh"
#include "ChipsColorMapTable.hh"
#include "ChipsColorMap.hh"
#include "ChipsCmdManager.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;
extern ChipsColorMapTable *colormap_table;


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiImageFrame::ChipsGuiImageFrame():ChipsGuiPropFrame(),ChipsGuiTooltipObject()
{
   _object_states = STATE_CHNG_PROP_IMG;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiImageFrame::~ChipsGuiImageFrame()
{
}

void ChipsGuiImageFrame::build_form()
{
   AtkObject *accessible;

   GtkWidget *tmp; 
   _frame = gtk_frame_new("Image Properties");
   _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(_frame), _nbook);

   _vbox = gtk_vbox_new (FALSE, 5);
   _tabcmap = gtk_table_new(7,7,FALSE);
   _tabalpha = gtk_table_new(7,4,FALSE);
   _tabthresh = gtk_table_new(7,7,FALSE);
   _frmcmap = gtk_frame_new("Colormap");
   gtk_frame_set_label_align(GTK_FRAME(_frmcmap), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmcmap), _tabcmap);
   gtk_box_pack_start(GTK_BOX(_vbox), _frmcmap, FALSE, FALSE, 5);
 
   tmp = gtk_label_new("Color");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _vbox, tmp);

   _cmap = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "red");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "green");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "blue");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "grayscale");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "rainbow");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "hsv");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "heat");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "cool");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "usercmap1");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "usercmap2");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_cmap), "usercmap3");

   _pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 256, 16);

   _cmapimg = gtk_image_new_from_pixbuf(_pixbuf); 
   _cmapsz = gtk_spin_button_new_with_range(1, 9999, 1);
   _invert = gtk_check_button_new();
   _cinterp = gtk_check_button_new();
   _scale = gtk_check_button_new();
   
   gtk_table_attach(GTK_TABLE(_tabcmap), _cmapimg, 4, 6, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Colormap");
   gtk_table_attach(GTK_TABLE(_tabcmap), tmp, 1, 2, 2, 3,
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabcmap), _cmap, 2, 4, 2, 3,
      GTK_FILL, GTK_FILL, 5, 5);
   tmp = gtk_label_new("Size");
   gtk_table_attach(GTK_TABLE(_tabcmap), tmp, 1, 2, 3, 4,
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabcmap), _cmapsz, 2, 4, 3, 4,
      GTK_FILL, GTK_FILL, 5, 5);
   tmp = gtk_label_new("Invert");
   gtk_table_attach(GTK_TABLE(_tabcmap), tmp, 4, 5, 3, 4,
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabcmap), _invert, 5, 6, 3, 4,
      GTK_FILL, GTK_FILL, 5, 5);
   tmp = gtk_label_new("Interpolate");
   gtk_table_attach(GTK_TABLE(_tabcmap), tmp, 4, 5, 4, 5, 
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabcmap), _cinterp, 5, 6, 4, 5,
      GTK_FILL, GTK_FILL, 5, 5);

   // alpha frame 
   _frmalpha = gtk_frame_new("Alpha");
   gtk_frame_set_label_align(GTK_FRAME(_frmalpha), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmalpha), _tabalpha);
   gtk_box_pack_start(GTK_BOX(_vbox), _frmalpha, FALSE, FALSE,5);

   _alphamin = gtk_hscale_new_with_range(0,1,.01);
   gtk_scale_set_digits(GTK_SCALE(_alphamin), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_alphamin), true);
   _alphamax = gtk_hscale_new_with_range(0,1,.01);
   gtk_scale_set_digits(GTK_SCALE(_alphamax), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_alphamax), true);
   _alphalock = gtk_check_button_new();
   tmp = gtk_label_new("Minimum");
   gtk_table_attach(GTK_TABLE(_tabalpha), tmp, 0, 2, 1, 2,
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabalpha), _alphamin, 2, 7, 1, 2,
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Maximum");
   gtk_table_attach(GTK_TABLE(_tabalpha), tmp, 0, 2, 2, 3,
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabalpha), _alphamax, 2, 7, 2, 3,
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Uniform");
   gtk_table_attach(GTK_TABLE(_tabalpha), tmp, 3, 4, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabalpha), _alphalock,  4, 5, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);
   
   // threshold frame 
   _frmthresh = gtk_frame_new("Threshold");
   gtk_frame_set_label_align(GTK_FRAME(_frmthresh), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmthresh), _tabthresh);
    gtk_box_pack_start(GTK_BOX(_vbox), _frmthresh, FALSE, FALSE,5);
   tmp = gtk_label_new("Min");
   gtk_table_attach(GTK_TABLE(_tabthresh), tmp, 1, 2, 1, 2,
     (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
      //GTK_FILL, GTK_FILL, 5, 5);
   _threshmin = gtk_spin_button_new_with_range(-G_MAXDOUBLE, G_MAXDOUBLE, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_threshmin), 3);
   gtk_table_attach(GTK_TABLE(_tabthresh), _threshmin, 3 ,4, 1, 2,
     (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
     // GTK_FILL, GTK_FILL, 5, 5);
   tmp = gtk_label_new("Max");
   gtk_table_attach(GTK_TABLE(_tabthresh), tmp, 4,5, 1, 2,
     (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
     // GTK_FILL, GTK_FILL, 5, 5);
   _threshmax = gtk_spin_button_new_with_range(-G_MAXDOUBLE, G_MAXDOUBLE, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_threshmax), 3);
   gtk_table_attach(GTK_TABLE(_tabthresh), _threshmax, 5, 6, 1, 2,
     (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK, 5, 5);
     // GTK_FILL, GTK_FILL, 5, 5);

   // general frame
   _tabgen = gtk_table_new(7,7,FALSE);
   tmp = gtk_label_new("General");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _tabgen, tmp);
   tmp = gtk_label_new("Depth");
   gtk_table_attach(GTK_TABLE(_tabgen), tmp, 4, 5, 2, 3,
      GTK_FILL, GTK_FILL, 10, 10);
   _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);
   gtk_table_attach(GTK_TABLE(_tabgen), _depth, 5, 6, 2, 3,
      GTK_FILL, GTK_FILL, 10, 10);
   _vis = gtk_check_button_new();
   tmp = gtk_label_new("Visible");
   gtk_table_attach(GTK_TABLE(_tabgen), tmp, 4, 5, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabgen), _vis, 5, 6, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);

   tmp = gtk_label_new("Interpolation");
   _interp = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_interp), "neighbor");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_interp), "bilinear");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_interp), "bicubic");
   gtk_table_attach(GTK_TABLE(_tabgen), tmp, 4, 5, 4, 5,
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabgen), _interp, 5, 6, 4, 5,
      GTK_FILL, GTK_FILL, 5, 5);
   tmp = gtk_label_new("Scale RGB");
   gtk_table_attach(GTK_TABLE(_tabgen), tmp, 4, 5, 5, 6,
      GTK_FILL, GTK_FILL, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabgen), _scale, 5, 6, 5, 6,
      GTK_FILL, GTK_FILL, 5, 5);

   int ndx = 0; 

   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_alphamin), "value-changed",
        G_CALLBACK(alpha_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_alphamax), "value-changed",
        G_CALLBACK(alpha_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_alphalock), "toggled",
      G_CALLBACK(alphalock_callback), (void*)this);

   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_vis), "toggled",
      G_CALLBACK(vis_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_depth), "value-changed",
      G_CALLBACK(depth_callback), (void*)this);

   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_scale), "toggled",
      G_CALLBACK(scale_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_invert), "toggled",
      G_CALLBACK(invert_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_cinterp), "toggled",
      G_CALLBACK(cinterp_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_cmapsz), "value-changed",
      G_CALLBACK(cmapsz_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_cmap), "changed",
        G_CALLBACK(cmap_callback), (void*)this);

   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_threshmin), "value-changed",
        G_CALLBACK(threshmin_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_threshmax), "value-changed",
        G_CALLBACK(threshmax_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_interp), "changed",
        G_CALLBACK(interp_callback), (void*)this);

   setup_tooltip(_cmap,"Specifies the colormap to use to display the image"); 
   setup_tooltip(_cmapsz,"Set the number of entries to use from the color map");
   setup_tooltip(_invert,"Invert the colormap"); 
   setup_tooltip(_cinterp,"Interpolate the colormap over the size of the colormap"); 
   setup_tooltip(_scale,"Should R,G,B channels be normalized to their own ranges"); 
   setup_tooltip(_interp,"Set the interpolation method for displaying image pixels"); 
   setup_tooltip(_threshmin,"Set the lowerbound value for image thresholding"); 
   setup_tooltip(_threshmax,"Set the upperbound value for image thresholding"); 
   setup_tooltip(_alphamin,"Normalized minimum value for alpha channel"); 
   setup_tooltip(_alphamax,"Normalized maximum value for alpha channel"); 
   setup_tooltip(_alphalock,"Adjusts minimum and maximum values together"); 
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the image is drawn"); 
   setup_tooltip(_vis,"Should the image be displayed or hidden"); 

   // add accessibility info
   accessible = gtk_widget_get_accessible (_frame);
   atk_object_set_name (accessible, "imageProperties");
   atk_object_set_description (accessible, "Image properties frame\n");
   accessible = gtk_widget_get_accessible (_cmap);
   atk_object_set_name (accessible, "colormap");
   atk_object_set_description (accessible, "Image color map\n");
   accessible = gtk_widget_get_accessible (_cmapsz);
   atk_object_set_name (accessible, "colormapSize");
   atk_object_set_description (accessible, "Number of entries in the color map\n");
   accessible = gtk_widget_get_accessible (_invert);
   atk_object_set_name (accessible, "invert");
   atk_object_set_description (accessible, "Invert color map\n");
   accessible = gtk_widget_get_accessible (_cinterp);
   atk_object_set_name (accessible, "colormapInterp");
   atk_object_set_description (accessible, "Interpolate flag for color map\n");
   accessible = gtk_widget_get_accessible (_scale);
   atk_object_set_name (accessible, "rgbScale");
   atk_object_set_description (accessible, "RGB channel scale\n");
   accessible = gtk_widget_get_accessible (_interp);
   atk_object_set_name (accessible, "interpolate");
   atk_object_set_description (accessible, "Image interpolation method\n");
   accessible = gtk_widget_get_accessible (_threshmin);
   atk_object_set_name (accessible, "minThreshold");
   atk_object_set_description (accessible, "Color map minimum threshold\n");
   accessible = gtk_widget_get_accessible (_threshmax);
   atk_object_set_name (accessible, "maxThreshold");
   atk_object_set_description (accessible, "Color map maximum threshold\n");
   accessible = gtk_widget_get_accessible (_alphamin);
   atk_object_set_name (accessible, "minAlpha");
   atk_object_set_description (accessible, "Color map minimum alpha channel value\n");
   accessible = gtk_widget_get_accessible (_alphamax);
   atk_object_set_name (accessible, "maxAlpha");
   atk_object_set_description (accessible, "Color map minimum alpha channel value\n");
   accessible = gtk_widget_get_accessible (_alphalock);
   atk_object_set_name (accessible, "alphaLock");
   atk_object_set_description (accessible, "Lock alpha min/max spinners\n");
   accessible = gtk_widget_get_accessible (_depth);
   atk_object_set_name (accessible, "depth");
   atk_object_set_description (accessible, "Image depth\n");
   accessible = gtk_widget_get_accessible (_vis);
   atk_object_set_name (accessible, "visible");
   atk_object_set_description (accessible, "Image visible\n");



}

void ChipsGuiImageFrame::set_visibility(bool val)
{
   int ival = val; 
   set_engine_property(static_cast<void*>(&ival), SET_IMAGE_VISIBLE);
   _vis_v = val; 
}

void ChipsGuiImageFrame::set_scale(bool val)
{
   set_engine_property(static_cast<void*>(&val), SET_IMAGE_SCALE_CHANNELS);
   _scale_v = val; 
}

void ChipsGuiImageFrame::set_invert(bool val)
{
   set_engine_property(static_cast<void*>(&val), SET_IMAGE_INV_CMAP);
   _invert_v = val; 
   display_colormap();   // recalculate colormap preview graphic
}

void ChipsGuiImageFrame::set_cinterp(bool val)
{
   set_engine_property(static_cast<void*>(&val), SET_IMAGE_CMAP_INTERPOLATE);
   _cinterp_v = val;
   display_colormap();   // recalculate colormap preview graphic
}

void ChipsGuiImageFrame::set_depth(int val)
{
   set_engine_property(static_cast<void*>(&val), SET_IMAGE_DEPTH);
   _depth_v = val; 
}

void ChipsGuiImageFrame::set_colormap(int val)
{
   V_COLORMAP_TYPE vv = (V_COLORMAP_TYPE)val; 
   set_engine_property(static_cast<void*>(&vv), SET_IMAGE_CMAP);
   _cmap_v = vv;
   display_colormap();   // recalculate colormap preview graphic
}

void ChipsGuiImageFrame::set_interpolation(int val)
{
   _interp_v = (V_INTERPOLATION_TYPE)val;
   set_engine_property(static_cast<void*>(&_interp_v), SET_IMAGE_INTERPOLATION);
}

void ChipsGuiImageFrame::set_alpha()
{
   double drng[2]; 

   drng[0] = gtk_range_get_value(GTK_RANGE(_alphamin));
   drng[1] = gtk_range_get_value(GTK_RANGE(_alphamax));
   
   set_engine_property(static_cast<void*>(&drng), SET_IMAGE_ALPHA);
   display_colormap();   // recalculate colormap preview graphic
}

void ChipsGuiImageFrame::set_colormap_size(int val)
{
   
   set_engine_property(static_cast<void*>(&val), SET_IMAGE_CMAP_SIZE);
   _cmapsz_v = val;
   display_colormap();   // recalculate colormap preview graphic
}

void ChipsGuiImageFrame::set_threshold(double val, int pos)
{
   double range[2];
   if (pos)
   {
      range[0] = _threshmin_v;
      range[1] = _threshmax_v = val;
   }
   else
   {
      range[0] = _threshmin_v = val; 
      range[1] = _threshmax_v; 
   }
   set_engine_property(static_cast<void*>(&range), SET_IMAGE_THRESHOLD);
}

void ChipsGuiImageFrame::vis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (imgFrame)  imgFrame->set_visibility(val);
}

void ChipsGuiImageFrame::scale_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (imgFrame)  imgFrame->set_scale(val);
}

void ChipsGuiImageFrame::invert_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (imgFrame)  imgFrame->set_invert(val);
}

void ChipsGuiImageFrame::cinterp_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (imgFrame)  imgFrame->set_cinterp(val);
}

void ChipsGuiImageFrame::alphalock_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (imgFrame)  imgFrame->set_alphalock(val);
}

void ChipsGuiImageFrame::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (imgFrame)  imgFrame->set_depth(val); 
}

void ChipsGuiImageFrame::cmap_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (imgFrame)  imgFrame->set_colormap(val);
}

void ChipsGuiImageFrame::cmapsz_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (imgFrame)  imgFrame->set_colormap_size(val);
}

void ChipsGuiImageFrame::interp_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (imgFrame)  imgFrame->set_interpolation(val);
}

void ChipsGuiImageFrame::threshmin_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   double val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (imgFrame)  imgFrame->set_threshold(val, 0);
}

void ChipsGuiImageFrame::threshmax_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);
   double val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (imgFrame)  imgFrame->set_threshold(val, 1);
}



void ChipsGuiImageFrame::match_alphas(float val)
{
   g_signal_handler_block(GTK_SCALE(_alphamin), _cb_sigs[0]);
   g_signal_handler_block(GTK_SCALE(_alphamax), _cb_sigs[1]);
   gtk_range_set_value(GTK_RANGE(_alphamin), val);
   gtk_range_set_value(GTK_RANGE(_alphamax), val);
   g_signal_handler_unblock(GTK_SCALE(_alphamin), _cb_sigs[0]);
   g_signal_handler_unblock(GTK_SCALE(_alphamax), _cb_sigs[1]);
}


void ChipsGuiImageFrame::alpha_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiImageFrame *imgFrame = static_cast <ChipsGuiImageFrame*>(data);

   if (!imgFrame) return; 

   float val = gtk_range_get_value(GTK_RANGE(widget));

   if (imgFrame->is_alphalocked())  imgFrame->match_alphas(val);  

   imgFrame->set_alpha();
}

void ChipsGuiImageFrame::display_colormap()
{
   if (_eng && colormap_table)
   {
      unsigned int ui;
      int cmsize;
      _eng->get_property(&_cid, (void*)&ui, GET_IMAGE_CMAP_ID);
      ChipsColorMap* cm = colormap_table->get_colormap(ui);
      if (cm)
      {
         guchar buf[1024];    // 256 x 8 bits per channel (r,g,b,a)

         guchar* cmvals = cm->get_computed_table(cmsize);

         // remap cmap size to buffer 
         int ndx;
         float pos = 0.0, step = cmsize/256.0; 
         for (int jj = 0; jj < 256; jj++)
         {
            // adjust step on 4 byte (32 bits- rgba) boundaries 
            ndx = (int)(pos + FLT_EPSILON)/4*4;
            buf[jj*4] = cmvals[ndx]; 
            buf[jj*4+1] = cmvals[ndx+1]; 
            buf[jj*4+2] = cmvals[ndx+2]; 
            buf[jj*4+3] = cmvals[ndx+3]; 
            pos += step; 
         }

         guchar *pp, *pixels = gdk_pixbuf_get_pixels(_pixbuf);
         int rowstride = gdk_pixbuf_get_rowstride(_pixbuf);
         for (int yy = 0; yy< 16;yy++)
            for (int zz=0; zz < 256;zz++)
            {
               pp = pixels + yy * rowstride + zz * 4;
               pp[0] = buf[zz * 4];
               pp[1] = buf[zz * 4 + 1];  //g
               pp[2] = buf[zz * 4 + 2];  //b
               pp[3] = buf[zz * 4 + 3];  //a
            }
      }
      gtk_image_set_from_pixbuf(GTK_IMAGE(_cmapimg), _pixbuf);
   }
}


void ChipsGuiImageFrame::initialize(ChipsId* id)
{
   if (!id)  return; 

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   if (id != &_cid)
   {
      _cid.reset_window_id(id->get_window_id()); 
      _cid.set_frame_id(id->get_frame_id()); 
      _cid.set_plot_id(id->get_plot_id()); 
      _cid.set_image_id(id->get_image_id()); 
   }

   // get values
   double dval[2];

   _eng->get_property(&_cid, (void*)dval, SET_IMAGE_ALPHA);
   _alphamin_v = dval[0]; 
   _alphamax_v = dval[1]; 
   _alphalock_v = (_alphamin_v == _alphamax_v); 
   _eng->get_property(&_cid, (void*)&_depth_v, SET_IMAGE_DEPTH);
   _eng->get_property(&_cid, (void*)&_vis_v, SET_IMAGE_VISIBLE);

   _eng->get_property(&_cid, (void*)dval, SET_IMAGE_THRESHOLD);
   _threshmin_v = dval[0];
   _threshmax_v = dval[1];

   // cmap values
   _eng->get_property(&_cid, (void*)&_cmap_v, SET_IMAGE_CMAP);
   unsigned int ui;
   _eng->get_property(&_cid, (void*)&ui, GET_IMAGE_CMAP_ID);
   _eng->get_property(&_cid, (void*)&_invert_v, SET_IMAGE_INV_CMAP);
   _eng->get_property(&_cid, (void*)&_scale_v, SET_IMAGE_SCALE_CHANNELS);
   _eng->get_property(&_cid, (void*)&_cinterp_v, SET_IMAGE_CMAP_INTERPOLATE);
   _eng->get_property(&_cid, (void*)&_cmapsz_v, SET_IMAGE_CMAP_SIZE);
   _eng->get_property(&_cid, (void*)&_interp_v, SET_IMAGE_INTERPOLATION);

   display_colormap(); 

   // alpha values
   g_signal_handler_block(GTK_SCALE(_alphamin), _cb_sigs[0]);
   g_signal_handler_block(GTK_SCALE(_alphamax), _cb_sigs[1]);
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_alphalock), _cb_sigs[2]);

   gtk_range_set_value(GTK_RANGE(_alphamin), _alphamin_v);
   gtk_range_set_value(GTK_RANGE(_alphamax), _alphamax_v);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_alphalock), _alphalock_v);

   g_signal_handler_unblock(GTK_SCALE(_alphamin), _cb_sigs[0]);
   g_signal_handler_unblock(GTK_SCALE(_alphamax), _cb_sigs[1]);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_alphalock), _cb_sigs[2]);

   // general values 
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_vis), _cb_sigs[3]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_depth), _cb_sigs[4]);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_vis), _vis_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depth_v);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_vis), _cb_sigs[3]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_depth), _cb_sigs[4]);

   // colormap values 
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_scale), _cb_sigs[5]);
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_invert), _cb_sigs[6]);
   g_signal_handler_block(GTK_TOGGLE_BUTTON(_cinterp), _cb_sigs[7]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_cmapsz), _cb_sigs[8]);
   g_signal_handler_block(GTK_COMBO_BOX(_cmap), _cb_sigs[9]);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scale), _scale_v);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_invert), _invert_v);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_cinterp), _cinterp_v);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_cmap),(int)_cmap_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_cmapsz), _cmapsz_v);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_scale), _cb_sigs[5]);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_invert), _cb_sigs[6]);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_cinterp), _cb_sigs[7]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_cmapsz), _cb_sigs[8]);
   g_signal_handler_unblock(GTK_COMBO_BOX(_cmap), _cb_sigs[9]);

   g_signal_handler_block(GTK_SPIN_BUTTON(_threshmin), _cb_sigs[10]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_threshmax), _cb_sigs[11]);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_threshmin), _threshmin_v);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_threshmax), _threshmax_v);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_threshmin), _cb_sigs[10]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_threshmax), _cb_sigs[11]);

   g_signal_handler_block(GTK_COMBO_BOX(_interp), _cb_sigs[12]);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_interp),(int)_interp_v);
   g_signal_handler_unblock(GTK_COMBO_BOX(_interp), _cb_sigs[12]);
}

