// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIIMAGEFRAME_HH
#define CHIPSGUIIMAGEFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <iostream>

#include "ChipsGuiPropFrame.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

class ChipsGuiColorWidget;

//---------------------------------------------------------------------------

class ChipsGuiImageFrame: public ChipsGuiPropFrame, public ChipsGuiTooltipObject
{
public:

   ChipsGuiImageFrame();
   virtual ~ChipsGuiImageFrame();

   static void cmap_callback(GtkWidget *widget, gpointer data);
   static void cmapsz_callback(GtkWidget *widget, gpointer data);
   static void alpha_callback(GtkWidget *widget, gpointer data);
   static void alphalock_callback(GtkWidget *widget, gpointer data);
   static void invert_callback(GtkWidget *widget, gpointer data);
   static void interp_callback(GtkWidget *widget, gpointer data);
   static void cinterp_callback(GtkWidget *widget, gpointer data);
   static void scale_callback(GtkWidget *widget, gpointer data);
   static void vis_callback(GtkWidget *widget, gpointer data);
   static void depth_callback(GtkWidget *widget, gpointer data);
   static void threshmin_callback(GtkWidget *widget, gpointer data);
   static void threshmax_callback(GtkWidget *widget, gpointer data);

   virtual void initialize(ChipsId* id);
   virtual void build_form(); 

   virtual void display_subobject (V_SUBOBJECT_TYPE subtype) {};

   void display_colormap(); 
   void set_alpha();
   void set_colormap(int val); 
   void set_colormap_size(int val); 
   void set_interpolation(int val);
   void set_cinterp(bool val);
   void set_invert(bool val); 
   void set_scale(bool val); 
   void set_threshold(double val, int pos);
   void set_depth(int val); 
   void set_visibility(bool val); 

   void set_alphalock(bool val) { _alphalock_v = val; } 
   bool is_alphalocked()  { return _alphalock_v; }
   void match_alphas(float val); 

private:
  
   GtkWidget* _vbox;
   GtkWidget* _frmcmap;
   GtkWidget* _frmalpha;
   GtkWidget* _frmthresh;
   GtkWidget* _tabcmap;
   GtkWidget* _tabalpha;
   GtkWidget* _tabthresh;
   GtkWidget* _tabgen;

   GtkWidget* _alphamin; 
   GtkWidget* _alphamax; 
   GtkWidget* _alphalock; 
   GdkPixbuf* _pixbuf; 
   GtkWidget* _cmapimg; 
   GtkWidget* _cmap; 
   GtkWidget* _cmapsz;
   GtkWidget* _interp;    // pixel interpolation
   GtkWidget* _cinterp;   // color map interpolation 
   GtkWidget* _invert;
   GtkWidget* _scale;
   GtkWidget* _threshmin;
   GtkWidget* _threshmax; 

   GtkWidget* _vis; 
   GtkWidget* _depth;

   // signals for callbacks 
   unsigned long _cb_sigs[20];

   float      _alphamin_v; 
   float      _alphamax_v; 
   bool       _alphalock_v; 
   V_COLORMAP_TYPE _cmap_v;
   int        _cmapsz_v; 
   V_INTERPOLATION_TYPE _interp_v;
   bool       _invert_v;
   bool       _scale_v;
   bool       _cinterp_v; 
   double     _threshmin_v; 
   double     _threshmax_v; 
   int        _depth_v;         // depth/layer  
   bool       _vis_v; 

};

#endif
