// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiInfoPane.hh"
#include "ChipsEventObserver.hh"
#include "ChipsEventManager.hh"
#include "ChipsGuiMain.hh"
#include <gdk/gdkkeysyms.h>
#include <atk/atk.h>

extern ChipsEventManager* event_manager;
extern ChipsGuiMain* chipsGui;

#include <stdlib.h>
#include <string.h>

#define FGINDEX  1
#define BGINDEX  3 

extern string cid_to_id(ChipsId* cid);

GdkPixbuf* ChipsGuiInfoPane::pixbufs[13] = { NULL, NULL, NULL, NULL, NULL,
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}; 

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiInfoPane::ChipsGuiInfoPane():ChipsEventObserver(),ChipsGuiTooltipObject()
{
   AtkObject *accessible;

   _frame = gtk_frame_new("Info");
   _table = gtk_table_new (2,2, FALSE);
   gtk_container_add (GTK_CONTAINER(_frame), _table);

   _scrolltv = gtk_scrolled_window_new(NULL, NULL);
   _scrolliv = gtk_scrolled_window_new(NULL, NULL);
   gtk_widget_set_size_request(GTK_WIDGET(_scrolltv), 200, 200);
   gtk_widget_set_size_request(GTK_WIDGET(_scrolliv), 200, 200);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolltv), 
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolliv), 
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

   gtk_table_attach(GTK_TABLE(_table),_scrolltv ,0,2,0,2,
      (GtkAttachOptions)(GTK_EXPAND |GTK_FILL),
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL),0,0);
   gtk_table_attach(GTK_TABLE(_table),_scrolliv ,0,2,0,2,
      (GtkAttachOptions)(GTK_EXPAND |GTK_FILL),
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL),0,0);

   GError *err = NULL;
   lastsel.object_type = chips_not_object; 

   string dname = "", fname; 
   char *tmpdir = getenv("ASCDS_INSTALL");
   if (tmpdir)  dname = tmpdir;
   dname.append("/config/chips/");
   
   if (pixbufs[0] == NULL)
   {
      fname = dname; 
      fname.append("icon_window.png");
      pixbufs[0] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
      atexit(ChipsGuiInfoPane::free_pixbufs); 
   }
   if (pixbufs[1] == NULL)
   {
      fname = dname; 
      fname.append("icon_frame.png");
      pixbufs[1] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }     
   if (pixbufs[2] == NULL)
   {
      fname = dname; 
      fname.append("icon_image.png");
      pixbufs[2] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[3] == NULL)
   {
      fname = dname; 
      fname.append("icon_plot.png");
      pixbufs[3] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[4] == NULL)
   {
      fname = dname; 
      fname.append("icon_curve.png");
      pixbufs[4] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[5] == NULL)
   {
      fname = dname; 
      fname.append("icon_histogram.png");
      pixbufs[5] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[6] == NULL)
   {
      fname = dname; 
      fname.append("icon_contour.png");
      pixbufs[6] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[7] == NULL)
   {
      fname = dname; 
      fname.append("icon_axis.png");
      pixbufs[7] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[8] == NULL)
   {
      fname = dname; 
      fname.append("icon_label.png");
      pixbufs[8] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[9] == NULL)
   {
      fname = dname; 
      fname.append("icon_line.png");
      pixbufs[9] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[10] == NULL)
   {
      fname = dname; 
      fname.append("icon_point.png");
      pixbufs[10] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[11] == NULL)
   {
      fname = dname; 
      fname.append("icon_region.png");
      pixbufs[11] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }
   if (pixbufs[12] == NULL)
   {
      fname = dname; 
      fname.append("icon_colorbar.png");
      pixbufs[12] = gdk_pixbuf_new_from_file (fname.c_str(), &err);
   }

   // set up icon mode settings 
   iconstore = gtk_list_store_new(I_NUM_COLS,
                  G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF);
   icon_view = gtk_icon_view_new();

   gtk_icon_view_set_text_column(GTK_ICON_VIEW (icon_view), I_IDVAL);
   gtk_icon_view_set_pixbuf_column(GTK_ICON_VIEW (icon_view), I_ICONVAL);
   gtk_icon_view_set_selection_mode(GTK_ICON_VIEW (icon_view),
                                    GTK_SELECTION_SINGLE);
   gtk_icon_view_set_model(GTK_ICON_VIEW(icon_view), GTK_TREE_MODEL(iconstore));
   gtk_widget_show(icon_view);
   gtk_container_add (GTK_CONTAINER(_scrolliv), icon_view); 


   // set up tree view mode settings 
   treestore = gtk_tree_store_new(T_NUM_COLS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
   tree_view = gtk_tree_view_new();
   col = gtk_tree_view_column_new();
   col2 = gtk_tree_view_column_new();

   gtk_tree_view_column_set_title(col, "Objects");
   gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), col);
   gtk_tree_view_column_set_title(col2, "Id");
   gtk_tree_view_set_search_column(GTK_TREE_VIEW(tree_view), T_IDVAL);
   gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), col2);
   renderer = gtk_cell_renderer_text_new();
   gtk_tree_view_column_pack_start(col, renderer, TRUE);
   gtk_tree_view_column_add_attribute(col, renderer, "text", T_TYPEVAL);
   gtk_tree_view_column_pack_start(col2, renderer, TRUE);
   gtk_tree_view_column_add_attribute(col2, renderer, "text", T_IDVAL);
   gtk_tree_view_set_model(GTK_TREE_VIEW(tree_view), GTK_TREE_MODEL(treestore));

   gtk_container_add (GTK_CONTAINER(_scrolltv), tree_view); 

   gtk_widget_show(tree_view);
   gtk_widget_show(GTK_WIDGET(_scrolltv));
   gtk_widget_show(GTK_WIDGET(_scrolliv));

   gtk_widget_show(GTK_WIDGET(_table));
   set_mode(TREE_MODEL_MODE); 

   // add callbacks 
   g_signal_connect((gpointer)tree_view, "key-press-event", 
      G_CALLBACK(keypress_cb), (void*)this);
   g_signal_connect((gpointer)icon_view, "key-press-event", 
      G_CALLBACK(keypress_cb), (void*)this);
   g_signal_connect((gpointer)tree_view, "button-press-event", 
      G_CALLBACK(handle_button_cb), (void*)this);
   g_signal_connect((gpointer)icon_view, "button-press-event",
       G_CALLBACK(handle_button_cb), (void*)this);
   _sigchgi = g_signal_connect((gpointer)icon_view, "selection-changed",
       G_CALLBACK (icon_changed_cb), (void*)this);
   _sigchgt = g_signal_connect((gpointer)tree_view, "cursor-changed",
       G_CALLBACK (tree_changed_cb), (void*)this);

   // add tooltips 
   setup_tooltip(icon_view, "Click on an icon to select that object");
   setup_tooltip(tree_view, "Click on a row to select that object");

   // add accessibility info
   accessible = gtk_widget_get_accessible (_frame);
   atk_object_set_name (accessible, "infoFrame");
   atk_object_set_description (accessible, "Info frame\n");
   accessible = gtk_widget_get_accessible (icon_view);
   atk_object_set_name (accessible, "iconView");
   atk_object_set_description (accessible, "Info displayed in icon form\n");
   accessible = gtk_widget_get_accessible (tree_view);
   atk_object_set_name (accessible, "treeView");
   atk_object_set_description (accessible, "Info displayed in tree form\n");

   

   // register w/ event observer to get changes in selection
   register_event(ChipsEventManager::SelectEvent);
   register_event(ChipsEventManager::UnselectEvent);
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiInfoPane::~ChipsGuiInfoPane()
{
  gtk_widget_destroy(_frame);
}

int ChipsGuiInfoPane::update_list(const char* values, bool force)
{
   int ln;
   char  *buf, *word, *pos, *id0, *id1;
   string tmpw, tmpf, root = ""; 

   // no changes if the list is the same
   if ((strcmp(values,curlist.c_str()) == 0) && !force)    return 0;

   curlist = values;

   gtk_list_store_clear(iconstore);
   gtk_tree_store_clear(treestore);

   if (mode == TREE_MODEL_MODE)
   {
      if ((ln = strlen(values)) > 0)
      {
         GtkTreeIter toplevel, frame, plot, child;
         buf = new char[ln + 1];
         strcpy(buf, values);
         word = pos = buf;
         while (word)
         {
            word = strtok(pos, "\n\0");
            pos = NULL;
            if (word)
            {
               id0 = strstr(word, " [");
               id1 = strstr(word, "]");
               if (id0 && id1)
               {
                  id0 +=2;
                  *id1 = '\0';
               }
               if (!strncmp(word, "Window [", 8))
               {
                  tmpw = root = id0;
                  gtk_tree_store_append(treestore, &toplevel, NULL);
                  gtk_tree_store_set(treestore, &toplevel,
                     T_TYPEVAL, "Window",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "  Frame [", 9))
               {
                  root = tmpw + " " + (tmpf = id0);
                  gtk_tree_store_append(treestore, &frame, &toplevel);
                  gtk_tree_store_set(treestore, &frame,
                     T_TYPEVAL, "Frame",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "    Plot [", 10))
               {
                  root = tmpw + " " + tmpf + " " + id0;
                  gtk_tree_store_append(treestore, &plot, &frame);
                  gtk_tree_store_set(treestore, &plot,
                     T_TYPEVAL, "Plot",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Curve [", 13))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Curve",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Region [", 14))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Region",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Contour [", 15))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Contour",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Point [", 13))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Point",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Line [", 12))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Line",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Image [", 13))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Image",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Label [", 13))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Label",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      X Axis [", 14))
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Axis",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Y Axis [", 14)) 
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Axis",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      ColorBar [", 16)) 
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "ColorBar",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
               else if (!strncmp(word, "      Histogram [", 17)) 
               {
                  gtk_tree_store_append(treestore, &child, &plot);
                  gtk_tree_store_set(treestore, &child,
                     T_TYPEVAL, "Histogram",
                     T_IDVAL, id0,
                     T_PARVAL, root.c_str(),
                     -1);
               }
            }
         }
         delete []buf;
      }
      gtk_tree_view_expand_all(GTK_TREE_VIEW(tree_view));
   }
   else
   {
      if ((ln = strlen(values)) > 0)
      {
         GtkTreeIter toplevel;
         buf = new char[ln + 1];
         strcpy(buf, values);
         word = pos = buf;
         while (word)
         {
            word = strtok(pos, "\n\0");
            pos = NULL;
            if (word)
            {
               id0 = strstr(word, " [");
               id1 = strstr(word, "]");
               if (id0 && id1)
               {
                  id0 +=2;
                  *id1 = '\0';
               }
               if (!strncmp(word, "Window [", 8))
               {
                  tmpw = root = id0;
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Window",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(), 
                     I_ICONVAL, pixbufs[0],
                     -1);
               }
               else if (!strncmp(word, "  Frame [", 9))
               {
                  root = tmpw + " " + (tmpf = id0); 
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Frame",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(), 
                     I_ICONVAL, pixbufs[1],
                     -1);
               }
               else if (!strncmp(word, "    Plot [", 10))
               {
                  root = tmpw + " " + tmpf + " " + id0; 
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Plot",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[3],
                     -1);
               }
               else if (!strncmp(word, "      Curve [", 13))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Curve",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[4],
                     -1);
               }
               else if (!strncmp(word, "      Histogram [", 17)) 
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Histogram",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[5],
                     -1);
               }
               else if (!strncmp(word, "      Image [", 13))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Image",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[2],
                     -1);
               }
               else if (!strncmp(word, "      Region [", 14))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Region",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[11],
                     -1);
               }
               else if (!strncmp(word, "      Contour [", 15))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Contour",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[6],
                     -1);
               }
               else if (!strncmp(word, "      Point [", 13))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Point",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[10],
                     -1);
               }
               else if (!strncmp(word, "      Line [", 12))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Line",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[9],
                     -1);
               }
               else if (!strncmp(word, "      Label [", 13))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Label",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[8],
                     -1);
               }
               else if (!strncmp(word, "      X Axis [", 14))
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Axis",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[7],
                     -1);
               }
               else if (!strncmp(word, "      Y Axis [", 14)) 
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "Axis",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[7],
                     -1);
               }
               else if (!strncmp(word, "      ColorBar [", 16)) 
               {
                  gtk_list_store_append(iconstore, &toplevel);
                  gtk_list_store_set(iconstore, &toplevel,
                     I_TYPEVAL, "ColorBar",
                     I_IDVAL, id0,
                     I_PARVAL, root.c_str(),
                     I_ICONVAL, pixbufs[12],
                     -1);
               }
            }
         }
         delete []buf;
      }
   }
   // if selection hasn't changed- keep selected highlighted 
   if (lastsel.object_type != chips_not_object)  set_iterator(&lastsel); 
   return 1;
}

void ChipsGuiInfoPane::set_mode(MODEL_MODE md)
{
   bool force_update = md != mode; 
   if (md == TREE_MODEL_MODE)
   {
      mode = TREE_MODEL_MODE;
      if (force_update)   update_list(curlist.c_str(), true); 
      gtk_widget_hide(_scrolliv);
      gtk_widget_show(_scrolltv);
   }
   else
   {
      mode = ICON_MODEL_MODE;
      if (force_update)   update_list(curlist.c_str(), true); 
      gtk_widget_show(_scrolliv);
      gtk_widget_hide(_scrolltv);
   }
}

void ChipsGuiInfoPane::show()
{
   gtk_widget_show(GTK_WIDGET(_frame));
}

void ChipsGuiInfoPane::hide()
{
   gtk_widget_hide(GTK_WIDGET(_frame));
}

int ChipsGuiInfoPane::get_height()
{
   int ht = 0;

   if (_frame)   ht = _frame->allocation.height;
   return ht;
}

void ChipsGuiInfoPane::popup_menu()
{
   GtkWidget *menu, *mitem1, *mitem2;
   AtkObject *accessible;

   menu = gtk_menu_new();
   mitem1 =  gtk_radio_menu_item_new_with_label(NULL, "Tree");
   mitem2 =  gtk_radio_menu_item_new_with_label(NULL, "Icon");

   g_signal_connect(mitem1, "activate", (GCallback)set_treemode_cb,  this);
   g_signal_connect(mitem2, "activate", (GCallback)set_iconmode_cb,  this);

   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem1);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), mitem2);
   gtk_widget_show_all(menu);

   // add accessibility info
   accessible = gtk_widget_get_accessible (menu);
   atk_object_set_name (accessible, "infoPopupMenu");
   atk_object_set_description (accessible, "Info frame popup menu\n");
   accessible = gtk_widget_get_accessible (menu);
   atk_object_set_name (accessible, "treePopupMenuItem");
   atk_object_set_description (accessible, "Tree option\n");
   accessible = gtk_widget_get_accessible (menu);
   atk_object_set_name (accessible, "iconPopupMenuItem");
   atk_object_set_description (accessible, "Icon option\n");


   gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0,
                  gdk_event_get_time(NULL));

}

void ChipsGuiInfoPane::set_treemode_cb(GtkWidget *widget, gpointer data)
{
   ChipsGuiInfoPane *gp = static_cast <ChipsGuiInfoPane*>(data); 
   if (gp)  gp->set_mode(TREE_MODEL_MODE);
}

void ChipsGuiInfoPane::set_iconmode_cb(GtkWidget *widget, gpointer data)
{
   ChipsGuiInfoPane *gp = static_cast <ChipsGuiInfoPane*>(data); 
   if (gp)  gp->set_mode(ICON_MODEL_MODE);
}

bool ChipsGuiInfoPane::keypress_cb (GtkWidget *wid, GdkEventKey *event,
   gpointer data)
{
   ChipsGuiInfoPane *gp = static_cast <ChipsGuiInfoPane*>(data);

   if ((event->keyval == GDK_Delete) ||
       (event->keyval == GDK_KP_Delete) ||
       (event->keyval == GDK_BackSpace))
   {
      gp->delete_selected_object(); 
      return 1; 
   }
   return 0; 
}

bool ChipsGuiInfoPane::handle_button_cb(GtkWidget *wid, 
   GdkEventButton *event, gpointer data)
{
   if (event->type == GDK_BUTTON_PRESS  &&  event->button == 3)
   {
      // RMB down = popup menu for icon/tree view 
      ChipsGuiInfoPane *gp = static_cast <ChipsGuiInfoPane*>(data); 
      gp->popup_menu(); 
      return true; // press handled via popup- don't forward event  
   }
   else if (event->type == GDK_BUTTON_PRESS  &&  event->button == 1)
   {
      // LMB down = select icon (needed if icon already selected) 
      ChipsGuiInfoPane *gp = static_cast <ChipsGuiInfoPane*>(data);
      if (gp && (gp->get_mode() == ICON_MODEL_MODE))
      {
         GtkTreePath* cpath = 0;
         if ((cpath = gtk_icon_view_get_path_at_pos(GTK_ICON_VIEW(gp->icon_view),
             event->x, event->y)) != NULL)
         {
            GtkTreeIter iter;
            gtk_tree_model_get_iter(GTK_TREE_MODEL(gp->iconstore),&iter,
               (GtkTreePath *)cpath);
            if (gtk_icon_view_path_is_selected(GTK_ICON_VIEW(gp->icon_view),(GtkTreePath*)cpath))
            {
               gtk_icon_view_unselect_path (GTK_ICON_VIEW(gp->icon_view),(GtkTreePath *)cpath);
               gtk_icon_view_select_path (GTK_ICON_VIEW(gp->icon_view),(GtkTreePath *)cpath);
               return true; 
            }
         }
      }
   }
   return false; // need to pass button press to tree  
}

bool ChipsGuiInfoPane::icon_changed_cb(GtkWidget *widget, gpointer data)
{
   GtkIconView *iv = GTK_ICON_VIEW(widget);
   ChipsGuiInfoPane *gp = static_cast <ChipsGuiInfoPane*>(data);
   GtkTreeIter iter;
   GList *list;
   GValue tval = {0}, idval = {0}, parval = {0};
   const gchar *str_a, *str_b, *str_c;

   if (gp)
   {
      list = gtk_icon_view_get_selected_items(iv);

      int ln = 0;
      if ((ln = g_list_length (list)) == 1)
      {
         gtk_tree_model_get_iter(GTK_TREE_MODEL(gp->iconstore),&iter,(GtkTreePath *)(list->data));
         gtk_tree_model_get_value(GTK_TREE_MODEL(gp->iconstore), &iter, I_TYPEVAL, &tval);
         gtk_tree_model_get_value(GTK_TREE_MODEL(gp->iconstore), &iter, I_IDVAL, &idval);
         gtk_tree_model_get_value(GTK_TREE_MODEL(gp->iconstore), &iter, I_PARVAL, &parval);
         str_a = g_value_get_string(&tval);
         str_b = g_value_get_string(&idval);
         str_c = g_value_get_string(&parval);
         gp->invoke_event(str_a, str_b, str_c);  
         g_value_unset(&tval);
         g_value_unset(&idval);
         g_value_unset(&parval);
         return TRUE; 
      }
   }
   return FALSE; // pass signal to parent 
}

bool ChipsGuiInfoPane::tree_changed_cb(GtkWidget *widget, gpointer data)
{
   GtkTreeView *tv = GTK_TREE_VIEW(widget);
   ChipsGuiInfoPane *gp = static_cast <ChipsGuiInfoPane*>(data);

   if (gp)
   {
      GtkTreeIter iter;
      GValue tval = {0}, idval = {0}, parval = {0};
      const gchar *str_a, *str_b, *str_c;

      GtkTreeSelection* sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv)); 

      if (gtk_tree_selection_get_selected(sel, NULL, &iter)) 

      {
         gtk_tree_model_get_value(GTK_TREE_MODEL(gp->treestore), &iter, T_TYPEVAL, &tval);
         gtk_tree_model_get_value(GTK_TREE_MODEL(gp->treestore), &iter, T_IDVAL, &idval);
         gtk_tree_model_get_value(GTK_TREE_MODEL(gp->treestore), &iter, T_PARVAL, &parval);
         str_a = g_value_get_string(&tval);
         str_b = g_value_get_string(&idval);
         str_c = g_value_get_string(&parval);
         gp->invoke_event(str_a, str_b, str_c);  
         g_value_unset(&tval);
         g_value_unset(&idval);
         g_value_unset(&parval);
         return TRUE; 
      }
   }
   return FALSE; // pass signal to parent 
}

// event observer to handle mode change from event manager
void ChipsGuiInfoPane::execute_event (int event, void *data)
{

   if (event == ChipsEventManager::SelectEvent)
   {
      ChipsEventObjectInfo* sel = (ChipsEventObjectInfo*)data;
      lastsel = *sel;
      set_iterator(sel);
   }
   else if (event == ChipsEventManager::UnselectEvent)
   {
      // should we unhighlight the object?
   }
}

// passes object selection to event manager to handle 
void ChipsGuiInfoPane::invoke_event(string type, string id, string parents)
{
   ChipsEventObjectInfo *sel;
   ChipsId cid; 
   V_OBJECT_TYPE typ; 
   if (string_to_v_object(type.c_str(), &typ) == ERR_STATUS_GOOD)
   { 
      size_t pos, pos2; 
 
      if ((pos = parents.find(" ")) != string::npos)
      {
         cid.reset_window_id(parents.substr(0,pos));
         if ((pos = parents.find(" ",(pos2=pos+1))) != string::npos)
         {
            cid.set_frame_id(parents.substr(pos2, pos-pos2));
            if ((pos = parents.find(" ",(pos2=pos+1))) != string::npos)
            {
               cid.set_plot_id(parents.substr(pos2,pos-pos2));
            }
            else 
               cid.set_plot_id(parents.substr(pos2)); 
         }
         else 
            cid.set_frame_id(parents.substr(pos2)); 
         cid.set_object_id(typ, id); 
      } 
      else 
      {
         cid.reset_window_id(parents); 
      }

      sel = new ChipsEventObjectInfo(cid, typ); 
      double evtpos[2] = {0.0,0.0};
      event_manager->invoke_event(evtpos, ChipsEventManager::SelectEvent,
            (void*)sel, this);
      delete sel; 
   }
}

// set the appropriate entry in the tree/icon view  
void ChipsGuiInfoPane::set_iterator(ChipsEventObjectInfo* sel)
{
   if (mode == ICON_MODEL_MODE)
      gtk_tree_model_foreach(GTK_TREE_MODEL(iconstore), &(ChipsGuiInfoPane::tree_model_highlight_selected), (void*)this);
   else 
      gtk_tree_model_foreach(GTK_TREE_MODEL(treestore), &(ChipsGuiInfoPane::tree_model_highlight_selected), (void*)this);
}

string cid_to_id(ChipsId* cid)
{
   static string val = "";
   if (cid)  
   {
      val = cid->get_window_id(); 
      val.append(" ");  
      val.append(cid->get_frame_id()); 
      val.append(" ");  
      val.append(cid->get_plot_id()); 
   }
   return val; 
}

void ChipsGuiInfoPane::set_cursor(GtkTreePath *path)
{
   if (mode == TREE_MODEL_MODE)
   {
      g_signal_handler_block(GTK_TREE_VIEW(tree_view), _sigchgt);
      gtk_tree_view_set_cursor(GTK_TREE_VIEW(tree_view), path, NULL, false);
      g_signal_handler_unblock(GTK_TREE_VIEW(tree_view), _sigchgt);
   }
   else
   {
      g_signal_handler_block(GTK_ICON_VIEW(icon_view),_sigchgi);
      gtk_icon_view_set_cursor(GTK_ICON_VIEW(icon_view), path, NULL, false);
      g_signal_handler_unblock(GTK_ICON_VIEW(icon_view), _sigchgi);
   }
}

gboolean ChipsGuiInfoPane::tree_model_highlight_selected(GtkTreeModel  *model,
              GtkTreePath *path, GtkTreeIter *iter, gpointer userdata)
{
   ChipsGuiInfoPane* me = (ChipsGuiInfoPane*)userdata; 
   ChipsEventObjectInfo* sel = &(me->lastsel); 
   string oid, otype, oroot, itid, ittype, itroot;
   ChipsId* cid = &sel->cid; 
   switch(sel->object_type)
   {
      case chips_region:
         oroot = cid_to_id(cid);
         oid = cid->get_region_id();
         otype = "Region";
      break;
      case chips_curve:
         oroot = cid_to_id(cid);
         oid = cid->get_curve_id();
         otype = "Curve";
      break;
      case chips_contour:
         oroot = cid_to_id(cid);
         oid = cid->get_contour_id();
         otype = "Contour";
      break;
      case chips_histogram:
         oroot = cid_to_id(cid);
         oid = cid->get_histogram_id();
         otype = "Histogram";
      break;
      case chips_image:
         oroot = cid_to_id(cid);
         oid = cid->get_image_id();
         otype = "Image";
      break;
      case chips_point:
         oroot = cid_to_id(cid);
         oid = cid->get_point_id();
         otype = "Point";
      break;
      case chips_axis:
         oroot = cid_to_id(cid);
         oid = cid->get_axis_id();
         otype = "Axis";
      break;
      case chips_line:
         oroot = cid_to_id(cid);
         oid = cid->get_line_id();
         otype = "Line";
      break;
      case chips_label:
         oroot = cid_to_id(cid);
         oid = cid->get_label_id();
         otype = "Label";
      break;
      case chips_colorbar:
         oroot = cid_to_id(cid);
         oid = cid->get_colorbar_id();
         otype = "ColorBar";
      break;
      case chips_plot:
         oroot = cid_to_id(cid);
         oid = cid->get_plot_id();
         otype = "Plot";
      break;
      case chips_frame:
         oroot = cid->get_window_id();
         oroot.append(" ");
         oroot.append(cid->get_frame_id());
         oid = cid->get_frame_id();
         otype = "Frame";
      break;
      case chips_window:
         oid = oroot = cid->get_window_id();
         otype = "Window";
      break;
      default:
      break; 
   }

   GValue tval = {0}, idval = {0}, rootval = {0};
   gtk_tree_model_get_value(GTK_TREE_MODEL(model), iter, T_TYPEVAL, &tval);
   gtk_tree_model_get_value(GTK_TREE_MODEL(model), iter, T_IDVAL, &idval);
   gtk_tree_model_get_value(GTK_TREE_MODEL(model), iter, T_PARVAL, &rootval);
   ittype = g_value_get_string(&tval);
   itid = g_value_get_string(&idval);
   itroot = g_value_get_string(&rootval);

   g_value_unset(&tval);
   g_value_unset(&idval);
   g_value_unset(&rootval);

   if (ittype == otype && itid == oid && itroot == oroot)
   {
      // found entry that was selected- highlight appropriate entry 
      if (me->get_mode() == TREE_MODEL_MODE)
      {
         GtkTreeSelection* sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(me->tree_view));
         gtk_tree_selection_select_iter(GTK_TREE_SELECTION(sel), iter); 
      }
      else 
      {
         gtk_icon_view_select_path(GTK_ICON_VIEW(me->icon_view), path);
      }
      me->set_cursor(path); 
      return true;   // end iterations 
   }
   return false; // iterate through other entries  
}

void ChipsGuiInfoPane::delete_selected_object()
{
   GtkTreeIter iter;
   string type, id, parents; 
   GValue tval = {0}, idval = {0}, parval = {0};

   if (mode == TREE_MODEL_MODE)
   {
      GtkTreeSelection* sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));

      if (gtk_tree_selection_get_selected(sel, NULL, &iter))
      {
         gtk_tree_model_get_value(GTK_TREE_MODEL(treestore), &iter, T_TYPEVAL, &tval);
         gtk_tree_model_get_value(GTK_TREE_MODEL(treestore), &iter, T_IDVAL, &idval);
         gtk_tree_model_get_value(GTK_TREE_MODEL(treestore), &iter, T_PARVAL, &parval);
         type = g_value_get_string(&tval);
         id = g_value_get_string(&idval);
         parents = g_value_get_string(&parval);
         g_value_unset(&tval);
         g_value_unset(&idval);
         g_value_unset(&parval);
      }
      else return; 
   }
   else 
   {
      GList *list = gtk_icon_view_get_selected_items(GTK_ICON_VIEW (icon_view));

      if ((g_list_length (list)) != 1)  return;
      
      gtk_tree_model_get_iter(GTK_TREE_MODEL(iconstore),&iter,(GtkTreePath *)(list->data));
      gtk_tree_model_get_value(GTK_TREE_MODEL(iconstore), &iter, I_TYPEVAL, &tval);
      gtk_tree_model_get_value(GTK_TREE_MODEL(iconstore), &iter, I_IDVAL, &idval);
      gtk_tree_model_get_value(GTK_TREE_MODEL(iconstore), &iter, I_PARVAL, &parval);
      type = g_value_get_string(&tval);
      id = g_value_get_string(&idval);
      parents = g_value_get_string(&parval);
      g_value_unset(&tval);
      g_value_unset(&idval);
      g_value_unset(&parval);
   }

   V_OBJECT_TYPE typ;
   if (string_to_v_object(type.c_str(), &typ) == ERR_STATUS_GOOD)
   {
      ChipsId cid;
      size_t pos, pos2;

      if ((pos = parents.find(" ")) != string::npos)
      {
         cid.reset_window_id(parents.substr(0,pos));
         if ((pos = parents.find(" ",(pos2=pos+1))) != string::npos)
         {
            cid.set_frame_id(parents.substr(pos2, pos-pos2));
            if ((pos = parents.find(" ",(pos2=pos+1))) != string::npos)
            {
               cid.set_plot_id(parents.substr(pos2,pos-pos2));
            }
            else
               cid.set_plot_id(parents.substr(pos2));
         }
         else
            cid.set_frame_id(parents.substr(pos2));
         cid.set_object_id(typ, id);
      }
      else
      {
         cid.reset_window_id(parents);
      }
      if (chipsGui)  chipsGui->delete_command(&cid, typ); 
   }
}

 
void ChipsGuiInfoPane::free_pixbufs()
{
   for (int rr = 0; rr < 13; rr++)
      if (pixbufs[rr]) 
      {
         g_object_unref(pixbufs[rr]);
         pixbufs[rr] = NULL;
      }
}
 
