// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIINFOPANE_HH
#define CHIPSGUIINFOPANE_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsEventObserver.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

class ChipsId; 


//---------------------------------------------------------------------------

class ChipsGuiInfoPane: public ChipsEventObserver, public ChipsGuiTooltipObject
{
public:

  enum MODEL_MODE { TREE_MODEL_MODE=0, ICON_MODEL_MODE };
  enum IMODEL_FIELDS { I_TYPEVAL=0, I_IDVAL, I_PARVAL, I_ICONVAL, I_NUM_COLS };
  enum TMODEL_FIELDS { T_TYPEVAL=0, T_IDVAL, T_PARVAL, T_NUM_COLS };

  ChipsGuiInfoPane();
  ~ChipsGuiInfoPane();

  GtkWidget* base_widget() { return _frame; } 

  void show();
  void hide();
  int  get_height(); 

  int update_list(const char* values, bool force = false);
  void set_iterator(ChipsEventObjectInfo* sel);
  void delete_selected_object(); 

   void       set_mode(MODEL_MODE mode);
   MODEL_MODE get_mode()   { return mode; }
  void popup_menu(); 
  void set_cursor(GtkTreePath *path); 

   static bool handle_button_cb(GtkWidget *wid, GdkEventButton *event, gpointer data);
   static void set_treemode_cb(GtkWidget *widget, gpointer data);
   static void set_iconmode_cb(GtkWidget *widget, gpointer data);
   static bool icon_changed_cb(GtkWidget *widget, gpointer data);
   static bool tree_changed_cb(GtkWidget *widget, gpointer data);
   static bool keypress_cb(GtkWidget *widget, GdkEventKey *event,gpointer data);

   virtual void execute_event (int event, void *data);
   void  invoke_event(string type, string id, string parents);  

   static gboolean tree_model_highlight_selected(GtkTreeModel  *model,
              GtkTreePath *path, GtkTreeIter *iter, gpointer userdata); 


   static void free_pixbufs(); 

private:
  
   MODEL_MODE  mode;      // display data as iconlist or tree
   GtkWidget *icon_view;
   GtkWidget *tree_view;
   GtkWidget *container;
   GtkTreeViewColumn *col, *col2;
   GtkCellRenderer *renderer;

   GtkTreeStore *treestore;
   GtkListStore *iconstore;
   string      curlist;

   GtkWidget*   _frame; 
   GtkWidget*   _table;
   GtkWidget*   _scrolltv; 
   GtkWidget*   _scrolliv; 

   unsigned long _sigchgt;  
   unsigned long _sigchgi;  

   ChipsEventObjectInfo lastsel; 

   static GdkPixbuf* pixbufs[13]; 

};

#endif
