// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiLabelPrefFrame.hh"
#include "ChipsGuiLabelPropWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsId.hh"
#include "ChipsGuiPropPane.hh"
#include "ChipsGuiFontInfo.hh"
#include "ChipsCmdManager.hh"
#include "ChipsMatrix.hh"

#include <sstream>

using namespace std;

extern ChipsGuiMain* chipsGui;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiLabelPrefFrame::ChipsGuiLabelPrefFrame():ChipsGuiPrefFrame(),ChipsGuiTooltipObject()
{
   _labelprop = NULL;
   _object_states = STATE_CHNG_PROP_LBL;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiLabelPrefFrame::~ChipsGuiLabelPrefFrame()
{
   if (_labelprop)  delete _labelprop; 
}

void ChipsGuiLabelPrefFrame::build_form()
{

   _frame = gtk_frame_new("Label Properties");

   _labelprop = new ChipsGuiLabelPropWidget ();
   _labelprop->build_form ();

   _labelprop->disable_position ();
   _labelprop->disable_coordinate ();
   _labelprop->disable_visibility ();
   _labelprop->disable_text ();

   gtk_container_add(GTK_CONTAINER(_frame), _labelprop->base_widget ());


   _labelprop->add_callback ("change_color", new ChipsGuiLabelPropWidgetCallback (color_change_callback, this));
   _labelprop->add_callback ("change_font", new ChipsGuiLabelPropWidgetCallback (font_change_callback, this));
   _labelprop->add_callback ("change_fontstyle", new ChipsGuiLabelPropWidgetCallback (fontstyle_change_callback, this));
   _labelprop->add_callback ("change_size", new ChipsGuiLabelPropWidgetCallback (size_change_callback, this));
   _labelprop->add_callback ("change_angle", new ChipsGuiLabelPropWidgetCallback (angle_change_callback, this));
/*   _labelprop->add_callback ("change_coordinate", new ChipsGuiLabelPropWidgetCallback (coordinate_callback, this));
*/
   _labelprop->add_callback ("change_halign", new ChipsGuiLabelPropWidgetCallback (halign_change_callback, this));
   _labelprop->add_callback ("change_valign", new ChipsGuiLabelPropWidgetCallback (valign_change_callback, this));
   _labelprop->add_callback ("change_depth", new ChipsGuiLabelPropWidgetCallback (depth_callback, this));

}


void ChipsGuiLabelPrefFrame::set_color(const string &color)
{
   stringstream pref;
   pref << "color=" << color;
   set_preference_override(pref.str (), chips_label);
}

void ChipsGuiLabelPrefFrame::set_angle(float val)
{
   stringstream pref;
   pref << "angle=" << val;
   set_preference_override(pref.str (), chips_label);
}

void ChipsGuiLabelPrefFrame::set_font(const string &val)
{
   stringstream pref;
   pref << "font=" << val;
   set_preference_override(pref.str (), chips_label);

}

void ChipsGuiLabelPrefFrame::set_depth(int val)
{
   stringstream pref;
   pref << "depth=" << val;
   set_preference_override(pref.str (), chips_label);
}

void ChipsGuiLabelPrefFrame::set_fontstyle(int val)
{
   stringstream pref;
   pref << "fontstyle=" << val;
   set_preference_override(pref.str (), chips_label);
}

void ChipsGuiLabelPrefFrame::set_halign(float val)
{
   stringstream pref;
   pref << "halign=" << val;
   set_preference_override(pref.str (), chips_label);
}

void ChipsGuiLabelPrefFrame::set_valign(float val)
{
   stringstream pref;
   pref << "valign=" << val;
   set_preference_override(pref.str (), chips_label);
}

void ChipsGuiLabelPrefFrame::set_size(int val)
{
   stringstream pref;
   pref << "size=" << val;
   set_preference_override(pref.str (), chips_label);
}

/*
void ChipsGuiLabelPrefFrame::set_coordinate(int coordinate)
{
}
*/



void ChipsGuiLabelPrefFrame::color_change_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_color ();
     
   labelFrame->set_color(clr);
}

void ChipsGuiLabelPrefFrame::depth_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame) return; //ensure we have needed pointer
   int depth = widget->get_depth ();
   labelFrame->set_depth(depth);   

}

void ChipsGuiLabelPrefFrame::font_change_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame) return; //ensure we have needed pointer
   string font = widget->get_font ();
   labelFrame->set_font (font);
}

void ChipsGuiLabelPrefFrame::fontstyle_change_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame) return; //ensure we have needed pointer
   int fontstyle = widget->get_fontstyle ();
   labelFrame->set_fontstyle (fontstyle);

}

void ChipsGuiLabelPrefFrame::size_change_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame) return; //ensure we have needed pointer
   int size = widget->get_size ();
   labelFrame->set_size (size);

}

void ChipsGuiLabelPrefFrame::angle_change_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame) return; //ensure we have needed pointer
   float angle = widget->get_angle ();
   labelFrame->set_angle (angle);

}

/*
void ChipsGuiLabelPrefFrame::coordinate_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame)  return; //ensure we have needed pointer
   int coordinate = widget->get_coordinate ();
   labelFrame->set_coordinate (coordinate);
}
*/

void ChipsGuiLabelPrefFrame::halign_change_callback(ChipsGuiLabelPropWidget *widget, void *data)
{
   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);

   if (!labelFrame)  return; //ensure we have needed pointer
   float halign = widget->get_halign ();
   labelFrame->set_halign (halign);
}

void ChipsGuiLabelPrefFrame::valign_change_callback(ChipsGuiLabelPropWidget *widget, void *data)
{

   ChipsGuiLabelPrefFrame *labelFrame = static_cast <ChipsGuiLabelPrefFrame*>(data);
   
   if (!labelFrame)  return; //ensure we have needed pointer
   float valign = widget->get_valign ();
   labelFrame->set_valign (valign);

}

void ChipsGuiLabelPrefFrame::initialize()
{
   ChipsId cid;
   int depthval;
   string fontval;
   V_FONTSTYLE_TYPE fontstyleval;
   float halignval;
   float valignval;
   int sizeval;
   COORD_SYS coordsys;
   string coordid;
   ChipsVector pos(3);
   string colorval;
   float angleval;
   string tmp_fontstyleval;
   stringstream pref_str_val;
   char *pref_cstr;


   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 


   if ((pref_cstr = _eng->get_preference ("label.color")) != NULL)
   {
      colorval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("label.angle")) != NULL)
   {
      stringstream(pref_cstr) >> angleval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("label.size")) != NULL)
   {
      stringstream(pref_cstr) >> sizeval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("label.font")) != NULL)
   {
      fontval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("label.fontstyle")) != NULL)
   {
      c2_string_to_fontstyle (pref_cstr, &fontstyleval);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("label.depth")) != NULL)
   {
      if (chips_strcmp (pref_cstr, "default") == 0)
      {
         free (pref_cstr);
         if ((pref_cstr = _eng->get_preference ("default.depth")) == NULL)
            depthval = V_DEFAULT_DEPTH;
      }

      if (pref_cstr)
      {
         stringstream(pref_cstr) >> depthval;
         free (pref_cstr);
      }
   }

   if ((pref_cstr = _eng->get_preference ("label.halign")) != NULL)
   {
      stringstream(pref_cstr) >> halignval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("label.valign")) != NULL)
   {
      stringstream(pref_cstr) >> valignval;
      free (pref_cstr);
   }

   _eng->get_object_coordinfo (&cid, chips_label, coordsys, coordid);

   _labelprop->set_color (colorval);
   _labelprop->set_coordinate ((int)coordsys);
   _labelprop->set_angle (angleval);
   _labelprop->set_font (fontval);
   _labelprop->set_fontstyle (fontstyleval);
   _labelprop->set_size (sizeval);
   _labelprop->set_halign (halignval);
   _labelprop->set_valign (valignval);
   _labelprop->set_depth (depthval);
   _labelprop->set_visibility (true);
   _labelprop->set_position (0,0);

}


