// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiLabelPropWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiPositionWidget.hh"
#include "ChipsGuiPropPane.hh"
#include "ChipsGuiFontInfo.hh"
#include "ChipsMatrix.hh"

#include <sstream>

using namespace std;

extern ChipsGuiMain* chipsGui;

ChipsGuiLabelPropWidgetCallback::ChipsGuiLabelPropWidgetCallback (void (*callback)(ChipsGuiLabelPropWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;

}

void ChipsGuiLabelPropWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiLabelPropWidget *labelWidget = dynamic_cast<ChipsGuiLabelPropWidget *>(caller);
   _callback (labelWidget, _callbackData);
}


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiLabelPropWidget::ChipsGuiLabelPropWidget()
{
   _valid_callback_ids.push_back ("change_color");
   _valid_callback_ids.push_back ("change_angle");
   _valid_callback_ids.push_back ("change_depth");
   _valid_callback_ids.push_back ("change_font");
   _valid_callback_ids.push_back ("change_fontstyle");
   _valid_callback_ids.push_back ("change_halign");
   _valid_callback_ids.push_back ("change_valign");
   _valid_callback_ids.push_back ("change_text");
   _valid_callback_ids.push_back ("change_position");
   _valid_callback_ids.push_back ("change_visibility");
   _valid_callback_ids.push_back ("change_coordinate");
   _valid_callback_ids.push_back ("change_size");

   _color = NULL;
   _position = NULL;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiLabelPropWidget::~ChipsGuiLabelPropWidget()
{
   if (_color)  delete _color; 
   if (_position) delete _position;
}

void ChipsGuiLabelPropWidget::build_form()
{

   GtkWidget*   table;
   GtkWidget*   label;

   _base_widget = _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);

   table = gtk_table_new (2, 3, FALSE);
   label = gtk_label_new("Text");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), table, label);
   
   ChipsGuiPropPane::font_info.initialize ();

   // setup font frame
   {
      GtkWidget*   fontframe;
      GtkWidget*   hbox_font;
      GtkWidget*   hbox1;
      GtkCellRenderer* cell1;
      GtkCellRenderer* cell2;


      fontframe = gtk_frame_new("Font");
      gtk_frame_set_label_align(GTK_FRAME(fontframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), fontframe, 0, 2, 0, 1,
                                        (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5,5);

      hbox1 = gtk_hbox_new (FALSE, 4);
      gtk_container_add(GTK_CONTAINER(fontframe), hbox1);

      _font = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL( ChipsGuiPropPane::font_info.get_font_list ()), 0);
      gtk_box_pack_start (GTK_BOX (hbox1), _font, FALSE, FALSE, 5);

      cell1 = gtk_cell_renderer_text_new();
      gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( _font ), cell1, FALSE );
//      gtk_cell_layout_add_attribute( GTK_CELL_LAYOUT( _font ), cell1, "text", 0);

      _size = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL( ChipsGuiPropPane::font_info.get_size_list ()), 0);
      gtk_widget_set_size_request (_size, 65, -1);
      gtk_box_pack_start (GTK_BOX (hbox1), _size, FALSE, FALSE, 5);

      cell2 = gtk_cell_renderer_text_new();
      gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( _size ), cell2, FALSE );
//      gtk_cell_layout_add_attribute( GTK_CELL_LAYOUT( _size ), cell2, "text", 0 );

      hbox_font = gtk_hbutton_box_new ();
      gtk_box_pack_start (GTK_BOX (hbox1), hbox_font, FALSE, FALSE, 5);

      _bold = gtk_toggle_button_new_with_label (GTK_STOCK_BOLD);
      gtk_button_set_use_stock (GTK_BUTTON(_bold), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_font), _bold, FALSE, FALSE, 0);
   

      _italic = gtk_toggle_button_new_with_label (GTK_STOCK_ITALIC);
      gtk_button_set_use_stock (GTK_BUTTON(_italic), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_font), _italic, FALSE, FALSE, 0);

      _color = new ChipsGuiColorWidget();
      _color->build_form ();
      _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_change_callback, this));
      gtk_box_pack_start (GTK_BOX (hbox1), _color->base_widget (), FALSE, FALSE, 5);
   }

   // add text and position
   {
      GtkWidget*   hbox2;

      GtkWidget*   textframe;
      GtkWidget*   positionframe;


      // add text frame
      textframe = gtk_frame_new("Text");
      gtk_frame_set_label_align(GTK_FRAME(textframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), textframe, 0, 2, 1, 2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_SHRINK),5, 5);

      _text = gtk_entry_new ();
      gtk_container_add(GTK_CONTAINER(textframe), _text);

      // add position frame
      _position = new ChipsGuiPositionWidget();
      _position->build_form ();

      positionframe = gtk_frame_new("Position");
      gtk_frame_set_label_align(GTK_FRAME(positionframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), positionframe, 0, 2, 2, 3, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_SHRINK),5, 5);

      gtk_container_add(GTK_CONTAINER(positionframe), _position->base_widget ());

   }

   _text_change_id = g_signal_connect(G_OBJECT(_text), "changed", G_CALLBACK(text_changed_callback), (void *)this);
   _font_change_id = g_signal_connect(G_OBJECT(_font), "changed", G_CALLBACK(font_change_callback), (void*)this);
   _fontstyle_bold_change_id = g_signal_connect(G_OBJECT(_bold), "toggled", G_CALLBACK(fontstyle_change_callback), (void*)this);
   _fontstyle_italic_change_id = g_signal_connect(G_OBJECT(_italic), "toggled", G_CALLBACK(fontstyle_change_callback), (void*)this);
   _size_change_id = g_signal_connect(G_OBJECT(_size), "changed", G_CALLBACK(size_change_callback), (void*)this);

   _position->add_callback ("change_position", new ChipsGuiPositionWidgetCallback (position_callback, this));
   _position->add_callback ("change_angle", new ChipsGuiPositionWidgetCallback (angle_callback, this));
   _position->add_callback ("change_coordinate", new ChipsGuiPositionWidgetCallback (coordinate_callback, this));


   // Setup Alignment Page
   {
      GtkWidget*   vbox2;
      GtkWidget*   hbox_halign;
      GtkWidget*   hbox_valign;
      GtkWidget*   hbox_hbalign;
      GtkWidget*   hbox_vbalign;
      GtkWidget*   vbox_halign;
      GtkWidget*   vbox_valign;



      vbox2 = gtk_vbox_new (FALSE, 2);
      label = gtk_label_new("Alignment");
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), vbox2, label);

      hbox_halign = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start (GTK_BOX (vbox2), hbox_halign, FALSE, FALSE, 5);
      hbox_valign = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start (GTK_BOX (vbox2), hbox_valign, FALSE, FALSE, 5);

      label = gtk_label_new("Horizontal");
      gtk_box_pack_start(GTK_BOX(hbox_halign), label, FALSE, FALSE,5);

      label = gtk_label_new("Vertical");
      gtk_box_pack_start(GTK_BOX(hbox_valign), label, FALSE, FALSE,5);

      vbox_halign = gtk_vbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(hbox_halign), vbox_halign, FALSE, FALSE,5);

      vbox_valign = gtk_vbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(hbox_valign), vbox_valign, FALSE, FALSE,5);

      hbox_hbalign = gtk_hbutton_box_new();
      gtk_box_pack_start (GTK_BOX (vbox_halign), hbox_hbalign, FALSE, FALSE, 5);

      _halign_left = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_LEFT);
      gtk_button_set_use_stock (GTK_BUTTON(_halign_left), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_hbalign), _halign_left, FALSE, FALSE, 0);

      _halign_center = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_CENTER);
      gtk_button_set_use_stock (GTK_BUTTON(_halign_center), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_hbalign), _halign_center, FALSE, FALSE, 0);

      _halign_right = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_RIGHT);
      gtk_button_set_use_stock (GTK_BUTTON(_halign_right), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_hbalign), _halign_right, FALSE, FALSE, 0);

      _halign = gtk_hscale_new_with_range (0, 1, .05);
      gtk_range_set_increments (GTK_RANGE(_halign), .05, .10);
      gtk_scale_set_digits(GTK_SCALE(_halign), 2);
      gtk_scale_set_draw_value(GTK_SCALE(_halign), true);
      gtk_box_pack_start (GTK_BOX (vbox_halign), _halign, FALSE, FALSE, 0);

      hbox_vbalign = gtk_hbutton_box_new ();
      gtk_box_pack_start (GTK_BOX (vbox_valign), hbox_vbalign, FALSE, FALSE, 5);

      _valign_bottom = gtk_toggle_button_new_with_label (GTK_STOCK_GOTO_BOTTOM);
      gtk_button_set_use_stock (GTK_BUTTON(_valign_bottom), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_vbalign), _valign_bottom, FALSE, FALSE, 0);

      _valign_center = gtk_toggle_button_new_with_label (GTK_STOCK_JUSTIFY_CENTER);
      gtk_button_set_use_stock (GTK_BUTTON(_valign_center), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_vbalign), _valign_center, FALSE, FALSE, 0);

      _valign_top = gtk_toggle_button_new_with_label (GTK_STOCK_GOTO_TOP);
      gtk_button_set_use_stock (GTK_BUTTON(_valign_top), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_vbalign), _valign_top, FALSE, FALSE, 0);

      _valign = gtk_hscale_new_with_range (0, 1, .05);
      gtk_range_set_increments (GTK_RANGE(_valign), .05, .10);
      gtk_scale_set_digits(GTK_SCALE(_valign), 2);
      gtk_scale_set_draw_value(GTK_SCALE(_valign), true);
      gtk_box_pack_start (GTK_BOX (vbox_valign), _valign, FALSE, FALSE, 0);

   }

   _halign_left_change_id = g_signal_connect(G_OBJECT(_halign_left), "toggled", G_CALLBACK(halign_change_callback), (void*)this);
   _halign_center_change_id = g_signal_connect(G_OBJECT(_halign_center), "toggled", G_CALLBACK(halign_change_callback), (void*)this);
   _halign_right_change_id = g_signal_connect(G_OBJECT(_halign_right), "toggled", G_CALLBACK(halign_change_callback), (void*)this);

   _valign_top_change_id = g_signal_connect(G_OBJECT(_valign_top), "toggled", G_CALLBACK(valign_change_callback), (void*)this);
   _valign_center_change_id = g_signal_connect(G_OBJECT(_valign_center), "toggled", G_CALLBACK(valign_change_callback), (void*)this);
   _valign_bottom_change_id = g_signal_connect(G_OBJECT(_valign_bottom), "toggled", G_CALLBACK(valign_change_callback), (void*)this);

   _halign_scale_change_id = g_signal_connect(G_OBJECT(_halign), "value-changed", G_CALLBACK(halign_scale_change_callback), (void*)this);
   _valign_scale_change_id = g_signal_connect(G_OBJECT(_valign), "value-changed", G_CALLBACK(valign_scale_change_callback), (void*)this);


   // Setup General Page
   {
      GtkWidget* tmptab = gtk_table_new(7,7,FALSE);

      label = gtk_label_new("General");
      gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), tmptab, label);

      _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
      gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);
      
      label = gtk_label_new("Depth");
      gtk_table_attach(GTK_TABLE(tmptab), label, 4, 5, 2, 3,
         GTK_FILL, GTK_FILL, 10, 10);
      gtk_table_attach(GTK_TABLE(tmptab), _depth, 5, 6, 2, 3,
         GTK_FILL, GTK_FILL, 10, 10);
      _vis = gtk_check_button_new();
      label = gtk_label_new("Visible");
      gtk_table_attach(GTK_TABLE(tmptab), label, 4, 5, 3, 4,
         GTK_FILL, GTK_FILL, 10, 10);
      gtk_table_attach(GTK_TABLE(tmptab), _vis, 5, 6, 3, 4,
         GTK_FILL, GTK_FILL, 10, 10);
   }

   _depth_id = g_signal_connect(G_OBJECT(_depth), "value-changed", 
      G_CALLBACK(depth_callback), (void*)this);
   _vis_id = g_signal_connect(G_OBJECT(_vis), "toggled",
      G_CALLBACK(vis_callback), (void*)this);

   setup_tooltip(_font,"Specifies the name of the font to use for the text");
   setup_tooltip(_size,"Specifies the point size to use for the text");
   setup_tooltip(_bold,"Specifies whether a bold fontstyle should be used");
   setup_tooltip(_italic,"Specifies whether the text should be italicized");
   setup_tooltip(_color->base_widget(), "Specifies the color of the label text");
   setup_tooltip(_text,"Raw text (includes limited latex support) to use for the label");
   setup_tooltip(_halign_left,"Left justifies the anchor for the label's location");
   setup_tooltip(_halign_center,"Horizontally centers the anchor for the label's location");
   setup_tooltip(_halign_right,"Right justifies the anchor for the label's location");
   setup_tooltip(_halign,"Sets the label's horizontal anchor position");
   setup_tooltip(_valign_top,"Top justifies the anchor for the label's location");
   setup_tooltip(_valign_center,"Vertically centers the anchor for the label's location");
   setup_tooltip(_valign_bottom,"Bottom justifies the anchor for the label's location");
   setup_tooltip(_valign,"Sets the label's vertical anchor position");
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the text is drawn");
   setup_tooltip(_vis,"Should the label text be displayed or hidden");
}

void ChipsGuiLabelPropWidget::show ()
{
   gtk_widget_show_all (_base_widget);
}

void ChipsGuiLabelPropWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}


void ChipsGuiLabelPropWidget::set_position (double x, double y)
{
   if (_position)
      _position->set_position (x,y);

}

void ChipsGuiLabelPropWidget::set_color(const string &color)
{
   if (_color)
      _color->set_chips_color (color);
}

void ChipsGuiLabelPropWidget::set_angle(float val)
{
   if (_position)
      _position->set_angle (val);
}

void ChipsGuiLabelPropWidget::set_coordinate (int val)
{
   if (_position)
      _position->set_coordinate (val);
}

void ChipsGuiLabelPropWidget::set_position_format (int xformat, int yformat)
{
   if (_position)
      _position->set_format (xformat, yformat);
}


void ChipsGuiLabelPropWidget::set_font(const string &val)
{
   GtkTreeIter iter;
   int index = 0;
   const gchar *tmp_font;
   GValue value={0,};
   bool font_set = false;
   GtkListStore *font_list;

   _fontval = val;
   g_signal_handler_block(GTK_OBJECT(_font), _font_change_id);

   ChipsGuiPropPane::font_info.update_font_list();

   font_list = ChipsGuiPropPane::font_info.get_font_list ();

   // this doesn't work if the default value is not in the model tree
   // it's not clear from the documentation how to make this work
   gtk_tree_model_get_iter_root (GTK_TREE_MODEL(font_list), &iter);
   do
   {
      gtk_tree_model_get_value (GTK_TREE_MODEL(font_list), &iter,
                                    0, &value);
      tmp_font = g_value_get_string (&value);
      if (chips_strcmp (tmp_font,_fontval.c_str ()) == 0)
      {
         font_set = true;
         break;
      }
      index++;
      g_value_unset (&value);
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(font_list), &iter));

   if (font_set)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_font),index);
   else
      gtk_combo_box_set_active(GTK_COMBO_BOX(_font), -1);

   g_signal_handler_unblock(GTK_OBJECT(_font), _font_change_id);
}

void ChipsGuiLabelPropWidget::set_depth(int val)
{
   _depthval = val;
   g_signal_handler_block(GTK_OBJECT(_depth), _depth_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depthval);
   g_signal_handler_unblock(GTK_OBJECT(_depth), _depth_id);
}

void ChipsGuiLabelPropWidget::set_fontstyle(int val)
{
   _fontstyleval = val;
   g_signal_handler_block(GTK_OBJECT(_bold), _fontstyle_bold_change_id);
   g_signal_handler_block(GTK_OBJECT(_italic), _fontstyle_italic_change_id);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_bold), FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_italic), FALSE);
   if ((_fontstyleval == chips_bold) || (_fontstyleval == chips_bolditalic))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_bold), TRUE);
   if ((_fontstyleval == chips_italic) || (_fontstyleval == chips_bolditalic))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_italic), TRUE);
   g_signal_handler_unblock(GTK_OBJECT(_bold), _fontstyle_bold_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_italic), _fontstyle_italic_change_id);
}

void ChipsGuiLabelPropWidget::set_halign(float val)
{
   _halignval = val;

   g_signal_handler_block(GTK_OBJECT(_halign), _halign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(_halign_left), _halign_left_change_id);
   g_signal_handler_block(GTK_OBJECT(_halign_right), _halign_right_change_id);
   g_signal_handler_block(GTK_OBJECT(_halign_center), _halign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_left),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_right),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_center),FALSE);


   if (flt_equal (_halignval, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_left),TRUE);
   else if (flt_equal (_halignval, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_center),TRUE);
   else if (flt_equal (_halignval, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_right),TRUE);
   gtk_range_set_value(GTK_RANGE(_halign), _halignval);

   g_signal_handler_unblock(GTK_OBJECT(_halign), _halign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_left), _halign_left_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_right), _halign_right_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_center), _halign_center_change_id);

}

void ChipsGuiLabelPropWidget::set_valign(float val)
{
   _valignval = val;

   g_signal_handler_block(GTK_OBJECT(_valign), _valign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(_valign_top), _valign_top_change_id);
   g_signal_handler_block(GTK_OBJECT(_valign_bottom), _valign_bottom_change_id);
   g_signal_handler_block(GTK_OBJECT(_valign_center), _valign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_top),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_bottom),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_center),FALSE);

   if (flt_equal (_valignval, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_bottom),TRUE);
   else if (flt_equal (_valignval, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_center),TRUE);
   else if (flt_equal (_valignval, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_top),TRUE);
   gtk_range_set_value(GTK_RANGE(_valign), _valignval);

   g_signal_handler_unblock(GTK_OBJECT(_valign), _valign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_top), _valign_top_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_bottom), _valign_bottom_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_center), _valign_center_change_id);

}

void ChipsGuiLabelPropWidget::set_size(int val)
{
   stringstream sstream;

   _sizeval = val;

   g_signal_handler_block(GTK_COMBO_BOX(_size), _size_change_id);

   sstream << val << ends;
   gtk_entry_set_text (GTK_ENTRY (GTK_BIN (_size)->child), sstream.str ().c_str ());
   g_signal_handler_unblock(GTK_COMBO_BOX(_size), _size_change_id);
}

void ChipsGuiLabelPropWidget::set_text(const string &val)
{
   _textval = val;

   g_signal_handler_block(GTK_ENTRY(_text), _text_change_id);
   gtk_entry_set_text(GTK_ENTRY(_text),(const gchar *)_textval.c_str());
   g_signal_handler_unblock(GTK_ENTRY(_text), _text_change_id);
}

void ChipsGuiLabelPropWidget::set_visibility(bool val)
{
   _visval = val;

   g_signal_handler_block(GTK_OBJECT(_vis), _vis_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_vis), val);
   g_signal_handler_unblock(GTK_OBJECT(_vis), _vis_id);
}

float  ChipsGuiLabelPropWidget::get_angle()
{
   return _position->get_angle ();
}

int    ChipsGuiLabelPropWidget::get_depth()
{
   return _depthval;
}

string ChipsGuiLabelPropWidget::get_color()
{
   return _color->get_chips_color ();
}

string ChipsGuiLabelPropWidget::get_font()
{
   return _fontval;
}

int    ChipsGuiLabelPropWidget::get_fontstyle()
{
   return _fontstyleval;
}

float  ChipsGuiLabelPropWidget::get_halign()
{
   return _halignval;
}

float  ChipsGuiLabelPropWidget::get_valign()
{
   return _valignval;
}

int    ChipsGuiLabelPropWidget::get_size()
{
   return _sizeval;
}

string ChipsGuiLabelPropWidget::get_text()
{
   return _textval;
}

void   ChipsGuiLabelPropWidget::get_position (double &xpos, double &ypos)
{
   _position->get_position (xpos,ypos);
}

int    ChipsGuiLabelPropWidget::get_coordinate ()
{
   return _position->get_coordinate ();
}

void   ChipsGuiLabelPropWidget::get_position_format (int &xformat, int &yformat)
{
   _position->get_format (xformat, yformat);
}

bool   ChipsGuiLabelPropWidget::get_visibility()
{
   return _visval;
}


void ChipsGuiLabelPropWidget::enable_angle()
{
   _position->enable_angle ();
}

void ChipsGuiLabelPropWidget::enable_depth()
{
   gtk_widget_set_sensitive (_depth, TRUE);
}

void ChipsGuiLabelPropWidget::enable_color()
{
   _color->enable ();
}

void ChipsGuiLabelPropWidget::enable_font()
{
   gtk_widget_set_sensitive (_font, TRUE);
}

void ChipsGuiLabelPropWidget::enable_fontstyle()
{
   gtk_widget_set_sensitive (_bold, FALSE);
   gtk_widget_set_sensitive (_italic, FALSE);  
}

void ChipsGuiLabelPropWidget::enable_halign()
{
   gtk_widget_set_sensitive (_halign_left, TRUE);
   gtk_widget_set_sensitive (_halign_right, TRUE);
   gtk_widget_set_sensitive (_halign_center, TRUE);
   gtk_widget_set_sensitive (_halign, TRUE);
}

void ChipsGuiLabelPropWidget::enable_valign()
{
   gtk_widget_set_sensitive (_valign_top, TRUE);
   gtk_widget_set_sensitive (_valign_bottom, TRUE);
   gtk_widget_set_sensitive (_valign_center, TRUE);
   gtk_widget_set_sensitive (_valign, TRUE);
}

void ChipsGuiLabelPropWidget::enable_size()
{
   gtk_widget_set_sensitive (_size, TRUE);
}

void ChipsGuiLabelPropWidget::enable_text()
{
   gtk_widget_set_sensitive (_text, TRUE);
}

void ChipsGuiLabelPropWidget::enable_position ()
{
   _position->enable_position ();
}

void ChipsGuiLabelPropWidget::enable_coordinate ()
{
   _position->enable_coordinate ();
}

void ChipsGuiLabelPropWidget::enable_data_coordinate ()
{
   _position->enable_data_coordinate ();
}

void ChipsGuiLabelPropWidget::enable_visibility()
{
   gtk_widget_set_sensitive (_vis, TRUE);
}


void ChipsGuiLabelPropWidget::disable_angle()
{
   _position->disable_angle ();
}

void ChipsGuiLabelPropWidget::disable_depth()
{
   gtk_widget_set_sensitive (_depth, FALSE);
}

void ChipsGuiLabelPropWidget::disable_color()
{
   _color->disable ();
}

void ChipsGuiLabelPropWidget::disable_font()
{
   gtk_widget_set_sensitive (_font, FALSE);
}

void ChipsGuiLabelPropWidget::disable_fontstyle()
{
   gtk_widget_set_sensitive (_bold, FALSE);
   gtk_widget_set_sensitive (_italic, FALSE);
}

void ChipsGuiLabelPropWidget::disable_halign()
{
   gtk_widget_set_sensitive (_halign_left, FALSE);
   gtk_widget_set_sensitive (_halign_right, FALSE);
   gtk_widget_set_sensitive (_halign_center, FALSE);
   gtk_widget_set_sensitive (_halign, FALSE);
}

void ChipsGuiLabelPropWidget::disable_valign()
{
   gtk_widget_set_sensitive (_valign_top, FALSE);
   gtk_widget_set_sensitive (_valign_bottom, FALSE);
   gtk_widget_set_sensitive (_valign_center, FALSE);
   gtk_widget_set_sensitive (_valign, FALSE);
}

void ChipsGuiLabelPropWidget::disable_size()
{
   gtk_widget_set_sensitive (_size, FALSE);
}
void ChipsGuiLabelPropWidget::disable_text()
{
   gtk_widget_set_sensitive (_text, FALSE);
}

void ChipsGuiLabelPropWidget::disable_position ()
{
   _position->disable_position ();
}

void ChipsGuiLabelPropWidget::disable_coordinate ()
{
   if (_position)  _position->disable_coordinate ();
}

void ChipsGuiLabelPropWidget::disable_data_coordinate ()
{
   if (_position)  _position->disable_data_coordinate ();
}

void ChipsGuiLabelPropWidget::disable_visibility()
{
   gtk_widget_set_sensitive (_vis, FALSE);
}


void ChipsGuiLabelPropWidget::color_change_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);

   if (!labelWidget)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
     
   if (labelWidget->_callbacks.count ("change_color") > 0)
      labelWidget->_callbacks["change_color"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::position_callback(ChipsGuiPositionWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   if (!labelWidget)  return; //ensure we have needed pointer
   double x,y;
   widget->get_position (x,y);

   if (labelWidget->_callbacks.count ("change_position") > 0)
      labelWidget->_callbacks["change_position"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::coordinate_callback(ChipsGuiPositionWidget *widget, void *data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);

   if (!labelWidget)  return; //ensure we have needed pointer

   if (labelWidget->_callbacks.count ("change_coordinate") > 0)
      labelWidget->_callbacks["change_coordinate"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::angle_callback(ChipsGuiPositionWidget *widget, void *data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);

   if (!labelWidget)  return; //ensure we have needed pointer
   float angle = widget->get_angle ();
   labelWidget->set_angle (angle);

   if (labelWidget->_callbacks.count ("change_angle") > 0)
      labelWidget->_callbacks["change_angle"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::text_changed_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   const char *val = gtk_entry_get_text (GTK_ENTRY(widget));
   string text;

   if (val)
   {
      text = val;
      labelWidget->set_text(text); 
   }

   if (labelWidget->_callbacks.count ("change_text") > 0)
      labelWidget->_callbacks["change_text"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (labelWidget)  labelWidget->set_depth(val); 

   if (labelWidget->_callbacks.count ("change_depth") > 0)
      labelWidget->_callbacks["change_depth"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::font_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   string font;
   char *str_font = gtk_combo_box_get_active_text(GTK_COMBO_BOX (widget));

   if (str_font)
   {
      font = str_font;
      labelWidget->set_font (font);

      g_free (str_font);
   }

   if (labelWidget->_callbacks.count ("change_font") > 0)
      labelWidget->_callbacks["change_font"]->execute (labelWidget, NULL);


}

void ChipsGuiLabelPropWidget::fontstyle_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   int set_style = 0;

   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (labelWidget->_bold)))
      set_style += 1;
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (labelWidget->_italic)))
      set_style += 2;

   if (set_style == 1)
      labelWidget->set_fontstyle ((int)chips_bold);
   else if (set_style == 2)
      labelWidget->set_fontstyle ((int)chips_italic);
   else if (set_style == 3)
      labelWidget->set_fontstyle ((int)chips_bolditalic);
   else
      labelWidget->set_fontstyle ((int)chips_normal);

   if (labelWidget->_callbacks.count ("change_fontstyle") > 0)
      labelWidget->_callbacks["change_fontstyle"]->execute (labelWidget, NULL);



}

void ChipsGuiLabelPropWidget::size_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);

   char *str_size = gtk_combo_box_get_active_text(GTK_COMBO_BOX (widget));
   int int_size;

   if (str_size)
   {
      int_size = atoi (str_size);
      labelWidget->set_size (int_size);

      g_free (str_size);
   }

   if (labelWidget->_callbacks.count ("change_size") > 0)
      labelWidget->_callbacks["change_size"]->execute (labelWidget, NULL);


}

void ChipsGuiLabelPropWidget::halign_scale_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_left),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_right),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_center),FALSE);

   val = gtk_range_get_value(GTK_RANGE(labelWidget->_halign));

   if (flt_equal (val, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_left),TRUE);
   else if (flt_equal (val, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_center),TRUE);
   else if (flt_equal (val, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_right),TRUE);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   labelWidget->set_halign (val);

   if (labelWidget->_callbacks.count ("change_halign") > 0)
      labelWidget->_callbacks["change_halign"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::valign_scale_change_callback(GtkWidget *widget, gpointer data)
{

   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_top),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_bottom),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_center),FALSE);


   val = gtk_range_get_value(GTK_RANGE(labelWidget->_valign));

   if (flt_equal (val, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_bottom),TRUE);
   else if (flt_equal (val, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_center),TRUE);
   else if (flt_equal (val, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_top),TRUE);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);

   labelWidget->set_valign (val);     

   if (labelWidget->_callbacks.count ("change_valign") > 0)
      labelWidget->_callbacks["change_valign"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::halign_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign), labelWidget->_halign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_left),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_right),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_center),FALSE);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
   if (widget == labelWidget->_halign_left)
      val = 0;
   else if (widget == labelWidget->_halign_center)
      val = .5;
   else
      val = 1;

   gtk_range_set_value(GTK_RANGE(labelWidget->_halign), val);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign), labelWidget->_halign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   labelWidget->set_halign (val);

   if (labelWidget->_callbacks.count ("change_halign") > 0)
      labelWidget->_callbacks["change_halign"]->execute (labelWidget, NULL);

   
}

void ChipsGuiLabelPropWidget::valign_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign), labelWidget->_valign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_bottom),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_top),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_center), FALSE);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
   if (widget == labelWidget->_valign_bottom)
      val = 0;
   else if (widget == labelWidget->_valign_center)
      val = .5;
   else
      val = 1;

   gtk_range_set_value(GTK_RANGE(labelWidget->_valign), val);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign), labelWidget->_valign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);


   labelWidget->set_valign (val);

   if (labelWidget->_callbacks.count ("change_valign") > 0)
      labelWidget->_callbacks["change_valign"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelPropWidget::vis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelPropWidget *labelWidget = static_cast <ChipsGuiLabelPropWidget*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (labelWidget)  labelWidget->set_visibility(val);

   if (labelWidget->_callbacks.count ("change_visibility") > 0)
      labelWidget->_callbacks["change_visibility"]->execute (labelWidget, NULL);

}


