// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSGUILABELPROPWIDGET_HH
#define CHIPSGUILABELPROPWIDGET_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiWidget.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

//  * change_color
//  * change_angle
//  * change_depth
//  * change_font
//  * change_fontstyle
//  * change_halign
//  * change_valign
//  * change_text
//  * change_position
//  * change_visibility
//  * change_coordinate
//  * change_size

class ChipsGuiLabelPropWidget;
class ChipsGuiColorWidget;
class ChipsGuiPositionWidget;

//---------------------------------------------------------------------------

class ChipsGuiLabelPropWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiLabelPropWidgetCallback (void (*callback)(ChipsGuiLabelPropWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiLabelPropWidget *, void *);
     void *_callbackData;
};


class ChipsGuiLabelPropWidget : public ChipsGuiWidget, public ChipsGuiTooltipObject
{
public:

  ChipsGuiLabelPropWidget();
  virtual ~ChipsGuiLabelPropWidget();

  virtual void build_form(); 

  virtual void show ();
  virtual void hide ();

  void set_angle(float val); 
  void set_depth(int val); 
  void set_color(const string &color); 
  void set_font(const string &val);
  void set_fontstyle(int val);
  void set_halign(float val);
  void set_valign(float val);
  void set_size(int val);
  void set_text(const string &val);
  void set_position (double xpos, double ypos);
  void set_coordinate (int coordinate);
  void set_position_format (int xformat, int yformat);
  void set_visibility(bool val);

  float  get_angle();
  int    get_depth();
  string get_color();
  string get_font();
  int    get_fontstyle();
  float  get_halign();
  float  get_valign();
  int    get_size();
  string get_text();
  void   get_position (double &xpos, double &ypos);
  int    get_coordinate ();
  void   get_position_format (int &xformat, int &yformat);
  bool   get_visibility();

  void enable_angle();
  void enable_depth();
  void enable_color();
  void enable_font();
  void enable_fontstyle();
  void enable_halign();
  void enable_valign();
  void enable_size();
  void enable_text();
  void enable_position ();
  void enable_coordinate ();
  void enable_data_coordinate(); 
  void enable_visibility();

  void disable_angle();
  void disable_depth();
  void disable_color();
  void disable_font();
  void disable_fontstyle();
  void disable_halign();
  void disable_valign();
  void disable_size();
  void disable_text();
  void disable_position ();
  void disable_coordinate ();
  void disable_data_coordinate(); 
  void disable_visibility();


protected:

   static void text_changed_callback(GtkWidget *widget, gpointer data);
   static void font_change_callback(GtkWidget *widget, gpointer data);
   static void fontstyle_change_callback(GtkWidget *widget, gpointer data);
   static void size_change_callback(GtkWidget *widget, gpointer data);
   static void angle_callback(ChipsGuiPositionWidget *widget, gpointer data);
   static void position_callback(ChipsGuiPositionWidget *widget, gpointer data);
   static void coordinate_callback(ChipsGuiPositionWidget *widget, void *data);
   static void halign_change_callback(GtkWidget *widget, gpointer data);
   static void valign_change_callback(GtkWidget *widget, gpointer data);
   static void halign_scale_change_callback(GtkWidget *widget, gpointer data);
   static void valign_scale_change_callback(GtkWidget *widget, gpointer data);
   static void depth_callback(GtkWidget *widget, gpointer data);
   static void vis_callback(GtkWidget *widget, gpointer data);
   static void color_change_callback(ChipsGuiColorWidget *widget, void *data);


   GtkWidget*   _text;
   GtkWidget*   _bold;
   GtkWidget*   _italic;
   GtkWidget*   _font;
   GtkWidget*   _size;
   GtkWidget*   _halign_left;
   GtkWidget*   _halign_right;
   GtkWidget*   _halign_center;
   GtkWidget*   _halign;
   GtkWidget*   _valign_top;
   GtkWidget*   _valign_center;
   GtkWidget*   _valign_bottom;
   GtkWidget*   _valign;
   GtkWidget*   _depth;
   GtkWidget*   _vis;
   GtkWidget*   _nbook;

   ChipsGuiColorWidget*   _color;
   ChipsGuiPositionWidget* _position;



  // signals for callbacks 
  unsigned long _text_change_id;
  unsigned long _font_change_id;
  unsigned long _fontstyle_bold_change_id;
  unsigned long _fontstyle_italic_change_id;
  unsigned long _size_change_id;
  unsigned long _halign_left_change_id;
  unsigned long _halign_right_change_id;
  unsigned long _halign_center_change_id;
  unsigned long _valign_top_change_id;
  unsigned long _valign_bottom_change_id;
  unsigned long _valign_center_change_id;
  unsigned long _halign_scale_change_id;
  unsigned long _valign_scale_change_id;
  unsigned long _depth_id;
  unsigned long _vis_id;


  int          _depthval; 
  string       _fontval; 
  int          _fontstyleval;
  float        _halignval;
  float        _valignval;
  int          _sizeval;
  string       _textval;
  bool         _visval;
};

#endif
