// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiLabelWidget.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiPropPane.hh"
#include "ChipsDefinesP.hh"
#include "ChipsGuiPivotPointWidget.hh"

#include <sstream>
#include <cstdlib>

using namespace std;

ChipsGuiLabelWidgetCallback::ChipsGuiLabelWidgetCallback (void (*callback)(ChipsGuiLabelWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;
}

void ChipsGuiLabelWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiLabelWidget *colorWidget = dynamic_cast<ChipsGuiLabelWidget *>(caller);
   _callback (colorWidget, _callbackData);
}



// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiLabelWidget::ChipsGuiLabelWidget():ChipsGuiWidget(),ChipsGuiTooltipObject()
{
   _valid_callback_ids.push_back ("change_color");
   _valid_callback_ids.push_back ("change_angle");
   _valid_callback_ids.push_back ("change_text");
   _valid_callback_ids.push_back ("change_font");
   _valid_callback_ids.push_back ("change_fontstyle");
   _valid_callback_ids.push_back ("change_align");
   _valid_callback_ids.push_back ("change_size");
   _valid_callback_ids.push_back ("change_position");
   _valid_callback_ids.push_back ("change_visible");

   _color = NULL;
   _alignment = NULL;
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiLabelWidget::~ChipsGuiLabelWidget()
{
//  gtk_widget_destroy(_frame);

   if (_color) delete _color;
   if (_alignment) delete _alignment;
}

void ChipsGuiLabelWidget::build_form()
{
   this->build_form(true);
}

void ChipsGuiLabelWidget::build_form(bool include_visible)
{

   GtkWidget*   table;
   GtkWidget*   label;
   GtkWidget*   vis_hbox;

  
   _base_widget = table = gtk_table_new (4, 2, FALSE);

   ChipsGuiPropPane::font_info.initialize ();


   // setup font frame
   {
      GtkWidget*   fontframe;
      GtkWidget*   hbox_font;
      GtkWidget*   hbox1;
      GtkCellRenderer* cell1;
      GtkCellRenderer* cell2;


      fontframe = gtk_frame_new("Font");
      gtk_frame_set_label_align(GTK_FRAME(fontframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), fontframe, 0, 2, 0, 1, 
                                        (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5,5);

      hbox1 = gtk_hbox_new (FALSE, 4);
      gtk_container_add(GTK_CONTAINER(fontframe), hbox1);

      ChipsGuiPropPane::font_info.update_font_list(); 

      _font = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL( ChipsGuiPropPane::font_info.get_font_list ()), 0);
      gtk_box_pack_start (GTK_BOX (hbox1), _font, FALSE, FALSE, 2);

      cell1 = gtk_cell_renderer_text_new();
      gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( _font ), cell1, FALSE );
//      gtk_cell_layout_add_attribute( GTK_CELL_LAYOUT( _font ), cell1, "text", 0);

      _size = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL( ChipsGuiPropPane::font_info.get_size_list ()), 0);
      gtk_widget_set_size_request (_size, 65, -1);
      gtk_box_pack_start (GTK_BOX (hbox1), _size, FALSE, FALSE, 2);

      cell2 = gtk_cell_renderer_text_new();
      gtk_cell_layout_pack_start( GTK_CELL_LAYOUT( _size ), cell2, FALSE );
//      gtk_cell_layout_add_attribute( GTK_CELL_LAYOUT( _size ), cell2, "text", 0 );

      hbox_font = gtk_hbutton_box_new ();
      gtk_button_box_set_layout (GTK_BUTTON_BOX(hbox_font), GTK_BUTTONBOX_CENTER);
      gtk_box_pack_start (GTK_BOX (hbox1), hbox_font, FALSE, FALSE, 2);

      _bold = gtk_toggle_button_new_with_label (GTK_STOCK_BOLD);
      gtk_button_set_use_stock (GTK_BUTTON(_bold), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox1), _bold, FALSE, FALSE, 0);
   

      _italic = gtk_toggle_button_new_with_label (GTK_STOCK_ITALIC);
      gtk_button_set_use_stock (GTK_BUTTON(_italic), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox1), _italic, FALSE, FALSE, 0);

      _color = new ChipsGuiColorWidget();
      _color->build_form ();
      _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_change_callback, this));
      gtk_box_pack_start (GTK_BOX (hbox1), _color->base_widget (), FALSE, FALSE, 5);
   }
   // add text frame
   {
      GtkWidget*   textframe;
      GtkWidget*   hbox1;

      textframe = gtk_frame_new(NULL);
      gtk_frame_set_label_align(GTK_FRAME(textframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), textframe, 0, 2, 1, 2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5, 5);

      hbox1 = gtk_hbox_new (FALSE, 2);
      gtk_container_add(GTK_CONTAINER(textframe), hbox1);
      label = gtk_label_new("Text");
      gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE,5);

      _text = gtk_entry_new ();
      gtk_box_pack_start(GTK_BOX(hbox1), _text, TRUE, TRUE,5);

   }

   // add position and align
   {
      GtkWidget*   table_pos;
      GtkWidget*   hbox3;
      GtkWidget*   hbox4;
      GtkWidget*   hbox5;
      GtkWidget*   positionframe;

      // add position frame
      positionframe = gtk_frame_new("Position");
      gtk_frame_set_label_align(GTK_FRAME(positionframe), .0, .5);
      gtk_table_attach(GTK_TABLE(table), positionframe, 0, 1, 2, 3, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5, 5);

      table_pos = gtk_table_new (4, 2, FALSE);
      gtk_container_add(GTK_CONTAINER(positionframe), table_pos);

      hbox3 = gtk_hbox_new (FALSE, 2);
      gtk_table_attach(GTK_TABLE(table_pos), hbox3, 0,1,1,2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL),5, 5);

      label = gtk_label_new("Position X");
      gtk_box_pack_start(GTK_BOX(hbox3), label, FALSE, FALSE,5);

      _posx = gtk_spin_button_new_with_range (-G_MAXDOUBLE, G_MAXDOUBLE,1.0);
      gtk_box_pack_start(GTK_BOX(hbox3), _posx, FALSE, FALSE,5);
      gtk_spin_button_set_digits (GTK_SPIN_BUTTON(_posx), 5);

      hbox4 = gtk_hbox_new (FALSE, 2);
      gtk_table_attach(GTK_TABLE(table_pos), hbox4, 0,1,2,3, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL),5, 5);

      label = gtk_label_new("Position Y");
      gtk_box_pack_start(GTK_BOX(hbox4), label, FALSE, FALSE,5);

      _posy = gtk_spin_button_new_with_range (-G_MAXDOUBLE, G_MAXDOUBLE,1.0);
      gtk_box_pack_start(GTK_BOX(hbox4), _posy, FALSE, FALSE,5);
      gtk_spin_button_set_digits (GTK_SPIN_BUTTON(_posy), 5);

      hbox5 = gtk_hbox_new (FALSE, 2);
      gtk_table_attach(GTK_TABLE(table_pos), hbox5, 0,1,3,4, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL),5, 5);

      label = gtk_label_new("Angle");
      gtk_box_pack_start(GTK_BOX(hbox5), label, FALSE, FALSE,5);

      _angle = gtk_spin_button_new_with_range (0, 359.99999, 1);
      gtk_box_pack_start(GTK_BOX(hbox5), _angle, FALSE, FALSE,5);
      gtk_spin_button_set_wrap (GTK_SPIN_BUTTON(_angle), TRUE);

      // setup alignment
      _alignment = new ChipsGuiPivotPointWidget();
      _alignment->build_form (false);
      _alignment->add_callback ("change_pivot", new ChipsGuiPivotPointWidgetCallback (align_change_callback, this));

      gtk_table_attach(GTK_TABLE(table_pos), _alignment->base_widget (), 1,2,0,4, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL),5, 5);
   }

   // add visible
   if (include_visible)
   {
      vis_hbox = gtk_hbox_new (FALSE, 2);
      gtk_table_attach(GTK_TABLE(table), vis_hbox, 0, 1, 3, 4,
            GTK_FILL, GTK_SHRINK, 5, 5);
      _visible = gtk_check_button_new_with_label ("Visible");
      gtk_box_pack_start (GTK_BOX (vis_hbox), _visible, FALSE, FALSE, 0);
      _visible_change_id = g_signal_connect(G_OBJECT(_visible), "toggled", G_CALLBACK(visible_change_callback), (void*)this);

   }
   else
      _visible = NULL;


   _text_change_id = g_signal_connect(G_OBJECT(_text), "changed", G_CALLBACK(text_changed_callback), (void *)this);
   _font_change_id = g_signal_connect(G_OBJECT(_font), "changed", G_CALLBACK(font_change_callback), (void*)this);
   _fontstyle_bold_change_id = g_signal_connect(G_OBJECT(_bold), "toggled", G_CALLBACK(fontstyle_change_callback), (void*)this);
   _fontstyle_italic_change_id = g_signal_connect(G_OBJECT(_italic), "toggled", G_CALLBACK(fontstyle_change_callback), (void*)this);
   _size_change_id = g_signal_connect(G_OBJECT(_size), "changed", G_CALLBACK(size_change_callback), (void*)this);
   _angle_change_id = g_signal_connect(G_OBJECT(_angle), "value-changed", G_CALLBACK(angle_change_callback), (void*)this);
   _xposition_changed_id = g_signal_connect(G_OBJECT(_posx), "value-changed", G_CALLBACK(position_change_callback), (void*)this);
   _yposition_changed_id = g_signal_connect(G_OBJECT(_posy), "value-changed", G_CALLBACK(position_change_callback), (void*)this);
   _xposition_focusout_id = g_signal_connect(G_OBJECT(_posx), "focus-out-event", G_CALLBACK(position_focusout_callback), (void*)this);
   _yposition_focusout_id = g_signal_connect(G_OBJECT(_posy), "focus-out-event", G_CALLBACK(position_focusout_callback), (void*)this);

   // add tooltips
   setup_tooltip(_font,"Specifies the name of the font to use for the text"); 
   setup_tooltip(_size,"Specifies the point size to use for the text"); 
   setup_tooltip(_bold,"Specifies whether a bold fontstyle should be used"); 
   setup_tooltip(_italic,"Specifies whether the text should be italicized"); 
   setup_tooltip(_color->base_widget(), "Specifies the color of the label text"); 
   setup_tooltip(_text,"Raw text (includes limited latex support) to use for the label");

}


void ChipsGuiLabelWidget::set_color(const string &color)
{
   _color->set_chips_color (color);
}

void ChipsGuiLabelWidget::set_angle(float val)
{
   _angleval = val;
   g_signal_handler_block(GTK_OBJECT(_angle), _angle_change_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_angle), val);  
   g_signal_handler_unblock(GTK_OBJECT(_angle), _angle_change_id);
}

void ChipsGuiLabelWidget::set_font(const string &val)
{
   GtkTreeIter iter;
   int index = 0;
   const gchar *tmp_font;
   GValue value={0,};
   bool font_set = false;
   GtkListStore *font_list;

   _fontval = val;
   g_signal_handler_block(GTK_OBJECT(_font), _font_change_id);

   ChipsGuiPropPane::font_info.update_font_list(); 

   font_list = ChipsGuiPropPane::font_info.get_font_list ();

   // this doesn't work if the default value is not in the model tree
   // it's not clear from the documentation how to make this work
   gtk_tree_model_get_iter_root (GTK_TREE_MODEL(font_list), &iter);
   do
   {
      gtk_tree_model_get_value (GTK_TREE_MODEL(font_list), &iter,
                                    0, &value);
      tmp_font = g_value_get_string (&value);
      if (chips_strcmp (tmp_font,_fontval.c_str ()) == 0)
      {
         font_set = true;
         break;
      }
      index++;
      g_value_unset (&value);
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(font_list), &iter));

   if (font_set)
      gtk_combo_box_set_active(GTK_COMBO_BOX(_font),index);
   else
      gtk_combo_box_set_active(GTK_COMBO_BOX(_font), -1);

   g_signal_handler_unblock(GTK_OBJECT(_font), _font_change_id);
}

void ChipsGuiLabelWidget::set_fontstyle(int val)
{
   _fontstyleval = (V_FONTSTYLE_TYPE)val;
   g_signal_handler_block(GTK_OBJECT(_bold), _fontstyle_bold_change_id);
   g_signal_handler_block(GTK_OBJECT(_italic), _fontstyle_italic_change_id);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_bold), FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_italic), FALSE);
   if ((_fontstyleval == chips_bold) || (_fontstyleval == chips_bolditalic))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_bold), TRUE);
   if ((_fontstyleval == chips_italic) || (_fontstyleval == chips_bolditalic))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_italic), TRUE);
   g_signal_handler_unblock(GTK_OBJECT(_bold), _fontstyle_bold_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_italic), _fontstyle_italic_change_id);
}
void ChipsGuiLabelWidget::set_align(float hval, float vval)
{
   _alignment->set_pivot_position (hval, vval);
}

/*
void ChipsGuiLabelWidget::set_halign(float val)
{
   _halignval = val;
   g_signal_handler_block(GTK_OBJECT(_halign), _halign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(_halign_left), _halign_left_change_id);
   g_signal_handler_block(GTK_OBJECT(_halign_right), _halign_right_change_id);
   g_signal_handler_block(GTK_OBJECT(_halign_center), _halign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_left),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_right),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_center),FALSE);


   if (flt_equal (_halignval, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_left),TRUE);
   else if (flt_equal (_halignval, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_center),TRUE);
   else if (flt_equal (_halignval, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_halign_right),TRUE);
   gtk_range_set_value(GTK_RANGE(_halign), _halignval);

   g_signal_handler_unblock(GTK_OBJECT(_halign), _halign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_left), _halign_left_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_right), _halign_right_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_halign_center), _halign_center_change_id);

}
*/
/*
void ChipsGuiLabelWidget::set_valign(float val)
{
   _valignval = val;

   g_signal_handler_block(GTK_OBJECT(_valign), _valign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(_valign_top), _valign_top_change_id);
   g_signal_handler_block(GTK_OBJECT(_valign_bottom), _valign_bottom_change_id);
   g_signal_handler_block(GTK_OBJECT(_valign_center), _valign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_top),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_bottom),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_center),FALSE);


   if (flt_equal (_valignval, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_bottom),TRUE);
   else if (flt_equal (_valignval, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_center),TRUE);
   else if (flt_equal (_valignval, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_valign_top),TRUE);
   gtk_range_set_value(GTK_RANGE(_valign), _valignval);

   g_signal_handler_unblock(GTK_OBJECT(_valign), _valign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_top), _valign_top_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_bottom), _valign_bottom_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_valign_center), _valign_center_change_id);

}
*/
void ChipsGuiLabelWidget::set_size(int val)
{
   stringstream sstream;

   _sizeval = val;

   g_signal_handler_block(GTK_COMBO_BOX(_size), _size_change_id);

   sstream << val << ends;
   gtk_entry_set_text (GTK_ENTRY (GTK_BIN (_size)->child), sstream.str ().c_str ());
   g_signal_handler_unblock(GTK_COMBO_BOX(_size), _size_change_id);
}

void ChipsGuiLabelWidget::set_text(const string &val)
{
   _textval = val;

   g_signal_handler_block(GTK_ENTRY(_text), _text_change_id);
   gtk_entry_set_text(GTK_ENTRY(_text),(const gchar *)_textval.c_str());
   g_signal_handler_unblock(GTK_ENTRY(_text), _text_change_id);

}

void ChipsGuiLabelWidget::set_visible(bool val)
{
   if (_visible)
   {
      _visibleval = val;

      g_signal_handler_block(GTK_OBJECT(_visible), _visible_change_id);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_visible), val);
      g_signal_handler_unblock(GTK_OBJECT(_visible), _visible_change_id);
   }

}



void ChipsGuiLabelWidget::set_position(double xpos, double ypos)
{
   _posxval = xpos;
   _posyval = ypos;
   g_signal_handler_block(GTK_SPIN_BUTTON(_posx), _xposition_changed_id);
   g_signal_handler_block(GTK_SPIN_BUTTON(_posy), _yposition_changed_id);
   g_signal_handler_block(GTK_SPIN_BUTTON(_posx), _xposition_focusout_id);
   g_signal_handler_block(GTK_SPIN_BUTTON(_posy), _yposition_focusout_id);

   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_posx), _posxval);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_posy), _posyval);

   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posx), _xposition_changed_id);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posy), _yposition_changed_id);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posx), _xposition_focusout_id);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posy), _yposition_focusout_id);


}

void ChipsGuiLabelWidget::set_position_type(POSITION_TYPE xtype, POSITION_TYPE ytype)
{
   g_signal_handler_block(GTK_SPIN_BUTTON(_posx), _xposition_changed_id);
   g_signal_handler_block(GTK_SPIN_BUTTON(_posy), _yposition_changed_id);

   if (xtype == INT)
   {
      gtk_spin_button_set_digits (GTK_SPIN_BUTTON(_posx), 0);
      gtk_spin_button_set_increments (GTK_SPIN_BUTTON(_posx), 1, 5);
   }
   else
   {
      gtk_spin_button_set_digits (GTK_SPIN_BUTTON(_posx), 2);
      gtk_spin_button_set_increments (GTK_SPIN_BUTTON(_posx), .01, .02);
   }

   if (ytype == INT)
   {
      gtk_spin_button_set_digits (GTK_SPIN_BUTTON(_posy), 0);
      gtk_spin_button_set_increments (GTK_SPIN_BUTTON(_posy), 1, 5);
   }
   else
   {
      gtk_spin_button_set_digits (GTK_SPIN_BUTTON(_posy), 2);
      gtk_spin_button_set_increments (GTK_SPIN_BUTTON(_posy), .01, .02);
   }

   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posx), _xposition_changed_id);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posy), _yposition_changed_id);

      
}

void ChipsGuiLabelWidget::set_position_range(double xrange[], double xstep, 
                                             double yrange[], double ystep)
{
   g_signal_handler_block(GTK_SPIN_BUTTON(_posx), _xposition_changed_id);
   g_signal_handler_block(GTK_SPIN_BUTTON(_posy), _yposition_changed_id);

   gtk_spin_button_set_range (GTK_SPIN_BUTTON (_posx), xrange[0], xrange[1]);
   gtk_spin_button_set_range (GTK_SPIN_BUTTON (_posy), yrange[0], yrange[1]);
   gtk_spin_button_set_increments (GTK_SPIN_BUTTON(_posx), xstep, xstep*2);
   gtk_spin_button_set_increments (GTK_SPIN_BUTTON(_posy), ystep, ystep*2);

   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posx), _xposition_changed_id);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_posy), _yposition_changed_id);

}




void ChipsGuiLabelWidget::color_change_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);

   if (!labelWidget)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
     
   labelWidget->set_color(clr);
   if (labelWidget->_callbacks.count ("change_color") > 0)
      labelWidget->_callbacks["change_color"]->execute (labelWidget, NULL);
}

void ChipsGuiLabelWidget::text_changed_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   const char *val = gtk_entry_get_text (GTK_ENTRY(widget));
   string text;

   if (val)
   {
      text = val;
      labelWidget->set_text(text); 

      if (labelWidget->_callbacks.count ("change_text") > 0)
         labelWidget->_callbacks["change_text"]->execute (labelWidget, NULL);
   }
}

void ChipsGuiLabelWidget::font_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   string font;
   char *str_font = gtk_combo_box_get_active_text(GTK_COMBO_BOX (widget));

   if (str_font)
   {
      font = str_font;
      labelWidget->set_font (font);

      if (labelWidget->_callbacks.count ("change_font") > 0)
         labelWidget->_callbacks["change_font"]->execute (labelWidget, NULL);

      g_free (str_font);
   }

}

void ChipsGuiLabelWidget::fontstyle_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   int set_style = 0;
   V_FONTSTYLE_TYPE style;

   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (labelWidget->_bold)))
      set_style += 1;
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (labelWidget->_italic)))
      set_style += 2;

   if (set_style == 1)
      style = chips_bold;
   else if (set_style == 2)
      style = chips_italic;
   else if (set_style == 3)
      style = chips_bolditalic;
   else
      style = chips_normal;

   labelWidget->set_fontstyle (style);
   if (labelWidget->_callbacks.count ("change_fontstyle") > 0)
      labelWidget->_callbacks["change_fontstyle"]->execute (labelWidget, NULL);


}

void ChipsGuiLabelWidget::size_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);

   char *str_size = gtk_combo_box_get_active_text(GTK_COMBO_BOX (widget));
   int int_size;

   if (str_size)
   {
      int_size = atoi (str_size);
      labelWidget->set_size (int_size);

      if (labelWidget->_callbacks.count ("change_size") > 0)
         labelWidget->_callbacks["change_size"]->execute (labelWidget, NULL);

      g_free (str_size);
   }

}

void ChipsGuiLabelWidget::angle_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   float val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));

   labelWidget->set_angle (val);

   if (labelWidget->_callbacks.count ("change_angle") > 0)
      labelWidget->_callbacks["change_angle"]->execute (labelWidget, NULL);

}

void ChipsGuiLabelWidget::position_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   double xval = gtk_spin_button_get_value(GTK_SPIN_BUTTON(labelWidget->_posx));
   double yval = gtk_spin_button_get_value(GTK_SPIN_BUTTON(labelWidget->_posy));

   labelWidget->set_position (xval, yval);

   if (labelWidget->_callbacks.count ("change_position") > 0)
      labelWidget->_callbacks["change_position"]->execute (labelWidget, NULL);


}

bool ChipsGuiLabelWidget::position_focusout_callback(GtkWidget *widget, GdkEvent *event, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   labelWidget->position_change_callback (widget, data);
   return false;
}
/*
void ChipsGuiLabelWidget::halign_scale_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_left),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_right),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_center),FALSE);

   val = gtk_range_get_value(GTK_RANGE(labelWidget->_halign));

   if (flt_equal (val, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_left),TRUE);
   else if (flt_equal (val, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_center),TRUE);
   else if (flt_equal (val, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_right),TRUE);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   labelWidget->set_halign (val);

   if (labelWidget->_callbacks.count ("change_halign") > 0)
      labelWidget->_callbacks["change_halign"]->execute (labelWidget, NULL);
}

void ChipsGuiLabelWidget::valign_scale_change_callback(GtkWidget *widget, gpointer data)
{

   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_top),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_bottom),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_center),FALSE);


   val = gtk_range_get_value(GTK_RANGE(labelWidget->_valign));

   if (flt_equal (val, 0))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_bottom),TRUE);
   else if (flt_equal (val, 0.5))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_center),TRUE);
   else if (flt_equal (val, 1))
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_top),TRUE);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);


 
   labelWidget->set_valign (val);     

   if (labelWidget->_callbacks.count ("change_valign") > 0)
      labelWidget->_callbacks["change_valign"]->execute (labelWidget, NULL);
}

void ChipsGuiLabelWidget::halign_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign), labelWidget->_halign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_left),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_right),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_halign_center),FALSE);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
   if (widget == labelWidget->_halign_left)
      val = 0;
   else if (widget == labelWidget->_halign_center)
      val = .5;
   else
      val = 1;

   gtk_range_set_value(GTK_RANGE(labelWidget->_halign), val);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign), labelWidget->_halign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_left), labelWidget->_halign_left_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_right), labelWidget->_halign_right_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_halign_center), labelWidget->_halign_center_change_id);


   labelWidget->set_halign (val);

   if (labelWidget->_callbacks.count ("change_halign") > 0)
      labelWidget->_callbacks["change_halign"]->execute (labelWidget, NULL);
   
}

void ChipsGuiLabelWidget::valign_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   float val = 0;

   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign), labelWidget->_valign_scale_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_block(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_bottom),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_top),FALSE);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (labelWidget->_valign_center), FALSE);

   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
   if (widget == labelWidget->_valign_bottom)
      val = 0;
   else if (widget == labelWidget->_valign_center)
      val = .5;
   else
      val = 1;

   gtk_range_set_value(GTK_RANGE(labelWidget->_valign), val);

   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign), labelWidget->_valign_scale_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_top), labelWidget->_valign_top_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_bottom), labelWidget->_valign_bottom_change_id);
   g_signal_handler_unblock(GTK_OBJECT(labelWidget->_valign_center), labelWidget->_valign_center_change_id);


   labelWidget->set_valign (val);

   if (labelWidget->_callbacks.count ("change_valign") > 0)
      labelWidget->_callbacks["change_valign"]->execute (labelWidget, NULL);
}
*/

void ChipsGuiLabelWidget::align_change_callback(ChipsGuiPivotPointWidget *widget, void * data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);

   if (labelWidget->_callbacks.count ("change_align") > 0)
      labelWidget->_callbacks["change_align"]->execute (labelWidget, NULL);
}

void ChipsGuiLabelWidget::visible_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLabelWidget *labelWidget = static_cast <ChipsGuiLabelWidget*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

   labelWidget->set_visible (val);

   if (labelWidget->_callbacks.count ("change_visible") > 0)
      labelWidget->_callbacks["change_visible"]->execute (labelWidget, NULL);

}


float ChipsGuiLabelWidget::get_angle()
{
   return gtk_spin_button_get_value(GTK_SPIN_BUTTON(_angle));
}

string ChipsGuiLabelWidget::get_color()
{
   return _color->get_chips_color ();
}

string ChipsGuiLabelWidget::get_font()
{
   char * font_cstr = gtk_combo_box_get_active_text(GTK_COMBO_BOX (_font));
   string font_str = font_cstr;
   
   g_free (font_cstr);
   return font_str;
}

int ChipsGuiLabelWidget::get_fontstyle()
{
   int style;
   int set_style = 0;

   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (_bold)))
      set_style += 1;
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (_italic)))
      set_style += 2;

   if (set_style == 1)
      style = chips_bold;
   else if (set_style == 2)
      style = chips_italic;
   else if (set_style == 3)
      style = chips_bolditalic;
   else
      style = chips_normal;

   return style;
}

void ChipsGuiLabelWidget::get_align (float &hval, float &vval)
{
   _alignment->get_pivot_position (hval, vval);
}

float ChipsGuiLabelWidget::get_halign()
{
   float xval;
   float yval;

   _alignment->get_pivot_position (xval, yval);
   return xval;
}

float ChipsGuiLabelWidget::get_valign()
{
   float xval;
   float yval;

   _alignment->get_pivot_position (xval, yval);
   return yval;

}

int ChipsGuiLabelWidget::get_size()
{
   char *str_size = gtk_combo_box_get_active_text(GTK_COMBO_BOX (_size));
   int int_size = 0;

   if (str_size)
   {
      int_size = atoi (str_size);
      g_free (str_size);
   }

   return int_size;

}

string ChipsGuiLabelWidget::get_text()
{
   return gtk_entry_get_text (GTK_ENTRY(_text));
}

bool ChipsGuiLabelWidget::get_visible()
{
   if (_visible)
      return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(_visible));
   return false;

}


void ChipsGuiLabelWidget::get_position (float &xpos, float &ypos)
{
   xpos = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_posx));
   ypos = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_posy));
}

void ChipsGuiLabelWidget::enable_angle()
{
   gtk_widget_set_sensitive (_angle, TRUE);
}

void ChipsGuiLabelWidget::enable_color()
{
   _color->enable ();
}

void ChipsGuiLabelWidget::enable_font()
{
   gtk_widget_set_sensitive (_font, TRUE);
}

void ChipsGuiLabelWidget::enable_fontstyle()
{
   gtk_widget_set_sensitive (_font, TRUE);
}

void ChipsGuiLabelWidget::enable_align ()
{
   _alignment->enable ();
}

void ChipsGuiLabelWidget::enable_size()
{
   gtk_widget_set_sensitive (_size, TRUE);
}

void ChipsGuiLabelWidget::enable_text()
{
   gtk_widget_set_sensitive (_text, TRUE);
}

void ChipsGuiLabelWidget::enable_position ()
{
   gtk_widget_set_sensitive (_posx, TRUE);
   gtk_widget_set_sensitive (_posy, TRUE);
}

void ChipsGuiLabelWidget::enable_visible ()
{
   gtk_widget_set_sensitive (_visible, TRUE);
}

void ChipsGuiLabelWidget::disable_angle()
{
   gtk_widget_set_sensitive (_angle, FALSE);
}

void ChipsGuiLabelWidget::disable_color()
{
   _color->disable ();
}

void ChipsGuiLabelWidget::disable_font()
{
   gtk_widget_set_sensitive (_font, FALSE);
}

void ChipsGuiLabelWidget::disable_fontstyle()
{
   gtk_widget_set_sensitive (_font, FALSE);
}

void ChipsGuiLabelWidget::disable_align ()
{
   _alignment->disable ();
}

void ChipsGuiLabelWidget::disable_size()
{
   gtk_widget_set_sensitive (_size, FALSE);
}

void ChipsGuiLabelWidget::disable_text()
{
   gtk_widget_set_sensitive (_text, FALSE);
}

void ChipsGuiLabelWidget::disable_position ()
{
   gtk_widget_set_sensitive (_posx, FALSE);
   gtk_widget_set_sensitive (_posy, FALSE);
}

void ChipsGuiLabelWidget::disable_visible ()
{
   gtk_widget_set_sensitive (_visible, FALSE);
}

void ChipsGuiLabelWidget::show ()
{
   gtk_widget_show (_base_widget);
}

void ChipsGuiLabelWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}


