// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUILABELWIDGET_HH
#define CHIPSGUILABELWIDGET_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiWidget.hh"
#include "ChipsDefines.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

class ChipsLabel;
class ChipsGuiColorWidget;
class ChipsGuiLabelWidget;
class ChipsGuiPivotPointWidget;

// valid callback ids are
//  * change_color
//  * change_angle
//  * change_text
//  * change_font
//  * change_fontstyle
//  * change_halign
//  * change_valign
//  * change_size
//  * change_text
//  * change_position
//  * change_visible

class ChipsGuiLabelWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiLabelWidgetCallback (void (*callback)(ChipsGuiLabelWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiLabelWidget *, void *);
     void *_callbackData;
};


//---------------------------------------------------------------------------

class ChipsGuiLabelWidget : public ChipsGuiWidget, public ChipsGuiTooltipObject
{
public:

  enum POSITION_TYPE {INT, FLOAT};

  ChipsGuiLabelWidget();
  ~ChipsGuiLabelWidget();

  virtual void build_form(); 
  void build_form (bool include_visible);

  void set_position_type (POSITION_TYPE xtype, POSITION_TYPE ytype);
  void set_position_range(double xrange[], double xstep,
                                     double yrange[], double ystep);


  virtual void show ();
  virtual void hide ();
  

  void set_angle(float val); 
  void set_color(const string &color); 
  void set_font(const string &val);
  void set_fontstyle(int val);
  void set_align(float hval, float vval);
  void set_size(int val);
  void set_text(const string &val);
  void set_position (double xpos, double ypos);
  void set_visible (bool val);

  float  get_angle();
  string get_color();
  string get_font();
  int    get_fontstyle();
  void   get_align (float &hval, float &vval);
  float  get_halign();
  float  get_valign();
  int    get_size();
  string get_text();
  void   get_position (float &xpos, float &ypos);
  bool   get_visible ();

  void enable_angle();
  void enable_color();
  void enable_font();
  void enable_fontstyle();
  void enable_align ();
  void enable_halign();
  void enable_valign();
  void enable_size();
  void enable_text();
  void enable_position ();
  void enable_visible ();

  void disable_angle();
  void disable_color();
  void disable_font();
  void disable_fontstyle();
  void disable_align ();
  void disable_halign();
  void disable_valign();
  void disable_size();
  void disable_text();
  void disable_position ();
  void disable_visible ();


private:

   GtkWidget*   _frame;
   GtkWidget*   _text;
   GtkWidget*   _bold;
   GtkWidget*   _italic;
   GtkWidget*   _font;
   GtkWidget*   _size;
   GtkWidget*   _angle;
   GtkWidget*   _posx;
   GtkWidget*   _posy;
   GtkWidget*   _visible;

   ChipsGuiColorWidget*   _color;
   ChipsGuiPivotPointWidget *_alignment;

  static void text_changed_callback(GtkWidget *widget, gpointer data);
  static void font_change_callback(GtkWidget *widget, gpointer data);
  static void fontstyle_change_callback(GtkWidget *widget, gpointer data);
  static void size_change_callback(GtkWidget *widget, gpointer data);
  static void angle_change_callback(GtkWidget *widget, gpointer data);
  static void position_change_callback(GtkWidget *widget, gpointer data);
  static bool position_focusout_callback(GtkWidget *widget, GdkEvent *event, gpointer data);
  static void align_change_callback(ChipsGuiPivotPointWidget *widget, gpointer data);

  static void color_change_callback(ChipsGuiColorWidget *widget, void *data);
  static void visible_change_callback (GtkWidget *widget, gpointer data);


  // signals for callbacks 
  unsigned long _text_change_id;
  unsigned long _font_change_id;
  unsigned long _fontstyle_bold_change_id;
  unsigned long _fontstyle_italic_change_id;
  unsigned long _size_change_id;
  unsigned long _angle_change_id;
  unsigned long _xposition_changed_id;
  unsigned long _yposition_changed_id;
  unsigned long _xposition_focusout_id;
  unsigned long _yposition_focusout_id;
  unsigned long _halign_left_change_id;
  unsigned long _halign_right_change_id;
  unsigned long _halign_center_change_id;
  unsigned long _valign_top_change_id;
  unsigned long _valign_bottom_change_id;
  unsigned long _valign_center_change_id;
  unsigned long _halign_scale_change_id;
  unsigned long _valign_scale_change_id;
  unsigned long _visible_change_id;



  float        _angleval;
  string       _fontval; 
  V_FONTSTYLE_TYPE _fontstyleval;
  float        _halignval;
  float        _valignval;
  int          _sizeval;
  string       _textval;
  double       _posxval;
  double       _posyval;
  bool         _visibleval;

};

#endif
