// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUILEVELSWIDGET_HH
#define CHIPSGUILEVELSWIDGET_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiWidget.hh"

using namespace std;

class ChipsGuiLevelsWidget;
   

// valid callback ids are
//  * change_levels

class ChipsGuiLevelsWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiLevelsWidgetCallback (void (*callback)(ChipsGuiLevelsWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiLevelsWidget *, void *);
     void *_callbackData;
};



//---------------------------------------------------------------------------

class ChipsGuiLevelsWidget : public ChipsGuiWidget
{
public:

  ChipsGuiLevelsWidget(int num_columns, GType type[]);
  ~ChipsGuiLevelsWidget();
  
  virtual void build_form(); 

  void set_column_title (const string &title, unsigned int column);

  static void add_item_callback(GtkWidget *widget, gpointer data);
  static void del_item_callback(GtkWidget *widget, gpointer data);
  static gboolean  del_item_keypress_callback (GtkWidget *widget, 
                                 GdkEvent *event, gpointer data);
  static void edit_item_callback(GtkCellRendererText *cell, 
                                 const gchar *path_string,
                                 const gchar *new_text, gpointer data);

  static gboolean set_cursor(ChipsGuiLevelsWidget* me);

  void initialize_column (double *vals, int cnt, int column);
  void initialize_column (float *vals, int cnt, int column);
  void initialize_column (string *vals, int cnt, int column);
  void initialize_column (int *vals, int cnt, int column);


  void get_column (double **elems, int &out_cnt, int column);
  void get_column (float **elems, int &out_cnt, int column);
  void get_column (string **elems, int &out_cnt, int column);
  void get_column (int **elems, int &out_cnt, int column);


  virtual void show ();
  virtual void hide ();

  void disable_widget ();
  void enable_widget ();

  void set_min_level (int min_level) {this->min_level = min_level;};
  int  get_min_level () {return this->min_level;};


protected:

  template<class T>
  void initialize_column(T* vals, int cnt, unsigned int column);

  template<class T>
  T * get_column(int &out_cnt, int column);


  void add_item();
  void delete_item();
  void edit_item(GtkCellRendererText *cell,const gchar *path_string, const gchar *new_text);

  static void set_gvalue (GValue &value, void *val, GType type);
  static void get_gvalue_data (GValue &value, int &data);
  static void get_gvalue_data (GValue &value, float &data);
  static void get_gvalue_data (GValue &value, double &data);
  static void get_gvalue_data (GValue &value, string &data);

  static bool set_string_to_gvalue (GValue &value, const string &val, GType type);
  static void init_gvalue (GValue &value, GType type);

  
  GtkWidget*   _addBtn;
  GtkWidget*   _delBtn; 

  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *sw;
  GtkTreeView *treeview;
  GtkTreeModel *items_model;
  GtkTreeIter *last_iter;
  GType*       _column_types;
  unsigned int _num_columns;

  string *_column_titles;

  int min_level;

};

template<class T>
T* ChipsGuiLevelsWidget::get_column(int &out_cnt, int column)
{
   GtkTreeIter iter;
   int index = 0;
   GValue value={0,};

   out_cnt = gtk_tree_model_iter_n_children (GTK_TREE_MODEL(items_model), NULL);
   
   T *dd = new T[out_cnt];


   if (gtk_tree_model_get_iter_root (GTK_TREE_MODEL(items_model), &iter))
   {
      do
      {
         gtk_tree_model_get_value (GTK_TREE_MODEL(items_model), &iter,
                                    column, &value);
         get_gvalue_data (value, dd[index]);
         index++;
         g_value_unset (&value);
      } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(items_model), &iter));
   }

   return dd;
}

template<class T>
void ChipsGuiLevelsWidget::initialize_column(T* vals, int cnt, unsigned int column)
{
    GtkTreeIter iter;
    bool iter_exists;
    GValue value = {0,};

    if (vals == NULL)  return;

    if (_num_columns <= column)
       return;

    g_value_init(&value, _column_types[column]);

    if (column == 0)
    {
       // clear existing list and add new rows of a single column 
       gtk_list_store_clear (GTK_LIST_STORE(items_model));

       // load in the new list
       for (int rr = 0; rr < cnt; rr++)
       {
          gtk_list_store_append (GTK_LIST_STORE(items_model), &iter);
   
          ChipsGuiLevelsWidget::set_gvalue (value, (void *)(&vals[rr]), _column_types[column]);
          gtk_list_store_set_value (GTK_LIST_STORE(items_model), &iter, column, &value);
       }
    }
    else 
    {
       // add column of data to existing row of 1 or more cols 
       GtkTreePath *path =  gtk_tree_path_new_first();

       for (int rr = 0; rr < cnt; rr++)
       {
          if (rr == 0)
             gtk_tree_model_get_iter(GTK_TREE_MODEL(items_model), &iter, path);
          else 
             gtk_tree_model_iter_next(GTK_TREE_MODEL(items_model), &iter);
          ChipsGuiLevelsWidget::set_gvalue (value, (void *)(&vals[rr]), _column_types[column]);
          gtk_list_store_set_value (GTK_LIST_STORE(items_model), &iter, column, &value);
       }       
    } 
}


#endif
