// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiLinePrefFrame.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsLine.hh"
#include "ChipsGuiLinePropWidget.hh"
#include "ChipsCmdManager.hh"

#include <algorithm>

using namespace std;

extern ChipsGuiMain* chipsGui;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiLinePrefFrame::ChipsGuiLinePrefFrame():ChipsGuiPrefFrame(),ChipsGuiTooltipObject()
{
   _lineprop = NULL;
   _object_states = STATE_CHNG_PROP_LINE;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiLinePrefFrame::~ChipsGuiLinePrefFrame()
{
   if (_lineprop)
      delete _lineprop;
}

void ChipsGuiLinePrefFrame::build_form()
{
   _frame = gtk_frame_new("Line Properties");

   _lineprop = new ChipsGuiLinePropWidget ();
   _lineprop->build_form ();

   _lineprop->disable_position ();
   _lineprop->disable_coordinate ();
   _lineprop->disable_visibility ();


   gtk_container_add(GTK_CONTAINER(_frame), _lineprop->base_widget ());

   _lineprop->add_callback ("change_color", new ChipsGuiLinePropWidgetCallback (color_callback, this));
/*   _lineprop->add_callback ("change_coordinate", new ChipsGuiLinePropWidgetCallback (coordinate_callback, this));
*/
   _lineprop->add_callback ("change_style", new ChipsGuiLinePropWidgetCallback (style_callback, this));
   _lineprop->add_callback ("change_depth", new ChipsGuiLinePropWidgetCallback (depth_callback, this));
   _lineprop->add_callback ("change_thickness", new ChipsGuiLinePropWidgetCallback (thick_callback, this));
   _lineprop->add_callback ("change_extent", new ChipsGuiLinePropWidgetCallback (extent_callback, this));

}

void ChipsGuiLinePrefFrame::set_color(string color)
{
   stringstream pref;
   pref << "color=" << color;
   set_preference_override(pref.str (), chips_line);
}


/*
void ChipsGuiLinePrefFrame::set_coordinate(int coordinate)
{
   ChipsId cid;
   COORD_SYS coordsys = (COORD_SYS)coordinate;
   string coordid = "";

   _eng->get_current_coord (&cid);

   if ((coordsys == DATA) && (cid.get_coord_sys () != DATA))
      chipsGui->log_error ("Data coordinate system does not exists.");
   else
      coordid = cid.get_coord_id ();

   coordid = ChipsCoordinate::extract_id (coordsys, coordid);

   _eng->change_annotation_coordinate (&cid, chips_line, coordsys, coordid);
}
*/

void ChipsGuiLinePrefFrame::set_thick(float val)
{
   stringstream pref;
   pref << "thickness=" << val;
   set_preference_override(pref.str (), chips_line);
}

void ChipsGuiLinePrefFrame::set_style(int val)
{
   stringstream pref;
   pref << "style=" << val;
   set_preference_override(pref.str (), chips_line);
}

void ChipsGuiLinePrefFrame::set_depth(int val)
{
   stringstream pref;
   pref << "depth=" << val;
   set_preference_override(pref.str (), chips_line);
}

void ChipsGuiLinePrefFrame::set_extent(int val)
{
   stringstream pref;
   pref << "extend=" << val;
   set_preference_override(pref.str (), chips_line);
}

void ChipsGuiLinePrefFrame::color_callback(ChipsGuiLinePropWidget *widget, void *data)
{
   ChipsGuiLinePrefFrame *linFrame = static_cast <ChipsGuiLinePrefFrame*>(data);

   if (!linFrame)  return; //ensure we have needed pointer 
     
   string clr = widget->get_color ();

   linFrame->set_color(clr);
}

/*
void ChipsGuiLinePrefFrame::coordinate_callback(ChipsGuiLinePropWidget *widget, void *data)
{
   ChipsGuiLinePrefFrame *pntFrame = static_cast <ChipsGuiLinePrefFrame*>(data);

   if (!pntFrame)  return; //ensure we have needed pointer
   int coordinate = widget->get_coordinate ();
   pntFrame->set_coordinate (coordinate);
}
*/

void ChipsGuiLinePrefFrame::style_callback(ChipsGuiLinePropWidget *widget, gpointer data)
{
   ChipsGuiLinePrefFrame *linFrame = static_cast <ChipsGuiLinePrefFrame*>(data);
   int val = widget->get_style ();
   if (linFrame)  linFrame->set_style(val); 
}

void ChipsGuiLinePrefFrame::thick_callback(ChipsGuiLinePropWidget *widget, gpointer data)
{
   ChipsGuiLinePrefFrame *linFrame = static_cast <ChipsGuiLinePrefFrame*>(data);
   float fval = widget->get_thick ();
   if (linFrame)  linFrame->set_thick(fval);
}

void ChipsGuiLinePrefFrame::depth_callback(ChipsGuiLinePropWidget *widget, gpointer data)
{
   ChipsGuiLinePrefFrame *linFrame = static_cast <ChipsGuiLinePrefFrame*>(data);
   int val = widget->get_depth ();
   if (linFrame)  linFrame->set_depth(val); 
}

void ChipsGuiLinePrefFrame::extent_callback(ChipsGuiLinePropWidget *widget, gpointer data)
{
   ChipsGuiLinePrefFrame *linFrame = static_cast <ChipsGuiLinePrefFrame*>(data);
   int val = widget->get_extent ();
   if (linFrame)  linFrame->set_extent(val); 
}

void ChipsGuiLinePrefFrame::initialize()
{
   ChipsId cid;
   COORD_SYS coordsys;
   string coordid;
   ChipsVector pos(3);
   string colorval;
   float thickval;
   int depthval;
   V_LINE_EXTENSION_TYPE extentval;
   V_LINE_TYPE styleval;
   char *pref_cstr;

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 


   if ((pref_cstr = _eng->get_preference ("line.color")) != NULL)
   {
      colorval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("line.extend")) != NULL)
   {
      c2_string_to_line_extension (pref_cstr, &extentval, NULL);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("line.thickness")) != NULL)
   {
      stringstream(pref_cstr) >> thickval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("line.depth")) != NULL)
   {
      if (chips_strcmp (pref_cstr, "default") == 0)
      {
         free (pref_cstr);
         if ((pref_cstr = _eng->get_preference ("default.depth")) == NULL)
            depthval = V_DEFAULT_DEPTH;
      }

      if (pref_cstr)
      {
         stringstream(pref_cstr) >> depthval;
         free (pref_cstr);
      }
   }

   if ((pref_cstr = _eng->get_preference ("line.style")) != NULL)
   {
      c2_string_to_line (pref_cstr, &styleval);
      free (pref_cstr);
   }

   _eng->get_object_coordinfo (&cid, chips_line, coordsys, coordid);


   _lineprop->set_color (colorval);
   _lineprop->set_coordinate ((int)coordsys);
   _lineprop->set_extent ((int)extentval);
   _lineprop->set_thick (thickval);
   _lineprop->set_style (styleval);
   _lineprop->set_depth (depthval);
   _lineprop->set_visibility (true);

}

