// 
//  Copyright (C) 2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiLinePropWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiPositionWidget.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;


ChipsGuiLinePropWidgetCallback::ChipsGuiLinePropWidgetCallback (void (*callback)(ChipsGuiLinePropWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;

}

void ChipsGuiLinePropWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiLinePropWidget *lineWidget = dynamic_cast<ChipsGuiLinePropWidget *>(caller);
   _callback (lineWidget, _callbackData);
}



// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiLinePropWidget::ChipsGuiLinePropWidget():ChipsGuiWidget(),ChipsGuiTooltipObject()
{
   _valid_callback_ids.push_back ("change_color");
   _valid_callback_ids.push_back ("change_position");
   _valid_callback_ids.push_back ("change_coordinate");
   _valid_callback_ids.push_back ("change_style");
   _valid_callback_ids.push_back ("change_depth");
   _valid_callback_ids.push_back ("change_thickness");
   _valid_callback_ids.push_back ("change_extent");
   _valid_callback_ids.push_back ("change_visibility");

   _color = NULL;
   _position = NULL;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiLinePropWidget::~ChipsGuiLinePropWidget()
{
   if (_color)
      delete _color;

   if (_position)
      delete _position;
}

void ChipsGuiLinePropWidget::build_form()
{
   GtkWidget* tmplabel;
   GtkWidget* table;

   _base_widget = _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);

   table = gtk_table_new(8,8,FALSE);
   tmplabel  = gtk_label_new("Line");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), table, tmplabel);

   _style = gtk_combo_box_new_text();
   _extent = gtk_combo_box_new_text();
   _thick = gtk_spin_button_new_with_range(0.5, 10, 1);
   _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);

   _color = new ChipsGuiColorWidget();
   _color->build_form ();
   _position = new ChipsGuiPositionWidget();
   _position->build_form (false);

   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_thick), 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);

   gtk_table_attach(GTK_TABLE(table), _color->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(table), _style, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "no line");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "solid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "dot");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "dot longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "dot shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "shortdash longdash");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_style),1);

   gtk_combo_box_append_text(GTK_COMBO_BOX(_extent), "none");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_extent), "start");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_extent), "end");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_extent), "both");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_extent),0);

   tmplabel = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Thickness");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 1, 2, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(table), _thick, 2, 3, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Extent");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 4, 5, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(table), _extent, 5, 6, 3,4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   gtk_table_attach(GTK_TABLE(table), _position->base_widget (), 1, 7, 6, 7,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   tmplabel = gtk_label_new("General");
   GtkWidget* tmptab = gtk_table_new(7,7,FALSE);
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), tmptab, tmplabel);

   tmplabel = gtk_label_new("Depth");
   gtk_table_attach(GTK_TABLE(tmptab), tmplabel, 4, 5, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(tmptab), _depth, 5, 6, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   _vis = gtk_check_button_new();
   tmplabel = gtk_label_new("Visible");
   gtk_table_attach(GTK_TABLE(tmptab), tmplabel, 4, 5, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);
   gtk_table_attach(GTK_TABLE(tmptab), _vis, 5, 6, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);

   _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_callback, this));
   _position->add_callback ("change_position", new ChipsGuiPositionWidgetCallback (position_callback, this));
   _position->add_callback ("change_coordinate", new ChipsGuiPositionWidgetCallback (coordinate_callback, this));
   _style_id = g_signal_connect(G_OBJECT(_style), "changed",
        G_CALLBACK(style_callback), (void*)this);
   _extent_id = g_signal_connect(G_OBJECT(_extent), "changed",
        G_CALLBACK(extent_callback), (void*)this);
   _thick_id = g_signal_connect(G_OBJECT(_thick), "value-changed",
        G_CALLBACK(thick_callback), (void*)this);
   _depth_id = g_signal_connect(G_OBJECT(_depth), "value-changed",
        G_CALLBACK(depth_callback), (void*)this);
   _vis_id = g_signal_connect(G_OBJECT(_vis), "toggled",
      G_CALLBACK(vis_callback), (void*)this);

   setup_tooltip(_color->base_widget(),"Sets the color to use for the line"); 
   setup_tooltip(_style,"Sets the pattern used to display the line"); 
   setup_tooltip(_extent,"Controls whether line extends indefinitely (a line, serment, or ray)"); 
   setup_tooltip(_thick,"Controls how thick the line is drawn"); 
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the line is drawn");
   setup_tooltip(_vis,"Should the line be displayed or hidden");
}


void ChipsGuiLinePropWidget::show ()
{
   gtk_widget_show_all (_base_widget);
}

void ChipsGuiLinePropWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}

void ChipsGuiLinePropWidget::set_position (double x, double y)
{
   if (_position)
      _position->set_position (x,y);

}

void ChipsGuiLinePropWidget::set_color(string color)
{
   if (_color)
      _color->set_chips_color (color);
}

void ChipsGuiLinePropWidget::set_coordinate (int val)
{
   if (_position)
      _position->set_coordinate (val);
}

void ChipsGuiLinePropWidget::set_position_format (int xformat, int yformat)
{
   if (_position)
      _position->set_format (xformat, yformat);
}


void ChipsGuiLinePropWidget::set_thick(float val)
{
   if (val < .75)  _thickval = 0.5;
      else         _thickval = (int)val;  
   g_signal_handler_block(GTK_SPIN_BUTTON(_thick), _thick_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_thick), _thickval);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_thick), _thick_id);
   
}

void ChipsGuiLinePropWidget::set_style(int val)
{
   _styleval = val;
   g_signal_handler_block(GTK_OBJECT(_style), _style_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_style),_styleval);
   g_signal_handler_unblock(GTK_OBJECT(_style), _style_id);
}

void ChipsGuiLinePropWidget::set_depth(int val)
{
   _depthval = val;
   g_signal_handler_block(GTK_OBJECT(_depth), _depth_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depthval);
   g_signal_handler_unblock(GTK_OBJECT(_depth), _depth_id);
}

void ChipsGuiLinePropWidget::set_extent(int val)
{
   _extentval = val;
   g_signal_handler_block(GTK_OBJECT(_extent), _extent_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_extent), _extentval);
   g_signal_handler_unblock(GTK_OBJECT(_extent), _extent_id);
}

void ChipsGuiLinePropWidget::set_visibility(bool val)
{
   _visval = val;
   g_signal_handler_block(GTK_OBJECT(_vis), _vis_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_vis), _visval);
   g_signal_handler_unblock(GTK_OBJECT(_vis), _vis_id);
}

void ChipsGuiLinePropWidget::get_position (double &x, double &y)
{
   _position->get_position (x,y);
}

int ChipsGuiLinePropWidget::get_coordinate ()
{
   return _position->get_coordinate ();
}

void ChipsGuiLinePropWidget::get_position_format (int &xformat, int &yformat)
{
   _position->get_format (xformat, yformat);
}



int ChipsGuiLinePropWidget::get_style()
{
   return _styleval;
}

int ChipsGuiLinePropWidget::get_depth()
{
   return _depthval;
}

string ChipsGuiLinePropWidget::get_color()
{
   return _color->get_chips_color ();
}

float ChipsGuiLinePropWidget::get_thick()
{
   return _thickval;
}

int ChipsGuiLinePropWidget::get_extent()
{
   return _extentval;
}

bool ChipsGuiLinePropWidget::get_visibility()
{
   return _visval;
}

void ChipsGuiLinePropWidget::enable_style()
{
   gtk_widget_set_sensitive (_style, TRUE);
}

void ChipsGuiLinePropWidget::enable_depth()
{
   gtk_widget_set_sensitive (_depth, TRUE);
}

void ChipsGuiLinePropWidget::enable_color()
{
   _color->enable ();
}

void ChipsGuiLinePropWidget::enable_thick()
{
   gtk_widget_set_sensitive (_thick, TRUE);
}

void ChipsGuiLinePropWidget::enable_extent()
{
   gtk_widget_set_sensitive (_extent, TRUE);
}

void ChipsGuiLinePropWidget::enable_visibility()
{
   gtk_widget_set_sensitive (_vis, TRUE);
}

void ChipsGuiLinePropWidget::enable_position ()
{
   _position->enable_position ();
}

void ChipsGuiLinePropWidget::enable_coordinate ()
{
   _position->enable_coordinate ();
}

void ChipsGuiLinePropWidget::enable_data_coordinate ()
{
   _position->enable_data_coordinate ();
}

void ChipsGuiLinePropWidget::disable_style()
{
   gtk_widget_set_sensitive (_style, FALSE);
}

void ChipsGuiLinePropWidget::disable_depth()
{
   gtk_widget_set_sensitive (_depth, FALSE);
}

void ChipsGuiLinePropWidget::disable_color()
{
   _color->disable ();
}

void ChipsGuiLinePropWidget::disable_thick()
{
   gtk_widget_set_sensitive (_thick, FALSE);
}

void ChipsGuiLinePropWidget::disable_extent()
{
   gtk_widget_set_sensitive (_extent, FALSE);
}

void ChipsGuiLinePropWidget::disable_visibility()
{
   gtk_widget_set_sensitive (_vis, FALSE);
}

void ChipsGuiLinePropWidget::disable_position ()
{
   _position->disable_position ();
}

void ChipsGuiLinePropWidget::disable_coordinate ()
{
   _position->disable_coordinate ();
}

void ChipsGuiLinePropWidget::disable_data_coordinate ()
{
   _position->disable_data_coordinate ();
}



void ChipsGuiLinePropWidget::color_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiLinePropWidget *lineWidget= static_cast <ChipsGuiLinePropWidget*>(data);

   if (!lineWidget)  return; //ensure we have needed pointer 
     
   string clr = widget->get_chips_color ();

   if (lineWidget->_callbacks.count ("change_color") > 0)
      lineWidget->_callbacks["change_color"]->execute (lineWidget, NULL);
}

void ChipsGuiLinePropWidget::position_callback(ChipsGuiPositionWidget *widget,
gpointer data)
{
   ChipsGuiLinePropWidget *lineWidget = static_cast <ChipsGuiLinePropWidget*>
(data);
   if (!lineWidget)  return; //ensure we have needed pointer
   double x,y;
   widget->get_position (x,y);

   if (lineWidget->_callbacks.count ("change_position") > 0)
      lineWidget->_callbacks["change_position"]->execute (lineWidget, NULL);

}

void ChipsGuiLinePropWidget::coordinate_callback(ChipsGuiPositionWidget *widget, void *data)
{
   ChipsGuiLinePropWidget *lineWidget = static_cast <ChipsGuiLinePropWidget*>(data);

   if (!lineWidget)  return; //ensure we have needed pointer
   if (lineWidget->_callbacks.count ("change_coordinate") > 0)
      lineWidget->_callbacks["change_coordinate"]->execute (lineWidget, NULL);

}


void ChipsGuiLinePropWidget::style_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLinePropWidget *lineWidget = static_cast <ChipsGuiLinePropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (lineWidget)  lineWidget->set_style(val); 

   if (lineWidget->_callbacks.count ("change_style") > 0)
      lineWidget->_callbacks["change_style"]->execute (lineWidget, NULL);
}

void ChipsGuiLinePropWidget::thick_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLinePropWidget *lineWidget = static_cast <ChipsGuiLinePropWidget*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   float fval = val;
   if (val == 0) fval = 0.5;

   if (lineWidget)  lineWidget->set_thick(fval);

   if (lineWidget->_callbacks.count ("change_thickness") > 0)
      lineWidget->_callbacks["change_thickness"]->execute (lineWidget, NULL);
}

void ChipsGuiLinePropWidget::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLinePropWidget *lineWidget = static_cast <ChipsGuiLinePropWidget*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (lineWidget)  lineWidget->set_depth(val); 

   if (lineWidget->_callbacks.count ("change_depth") > 0)
      lineWidget->_callbacks["change_depth"]->execute (lineWidget, NULL);
}

void ChipsGuiLinePropWidget::extent_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLinePropWidget *lineWidget = static_cast <ChipsGuiLinePropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (lineWidget)  lineWidget->set_extent(val); 

   if (lineWidget->_callbacks.count ("change_extent") > 0)
      lineWidget->_callbacks["change_extent"]->execute (lineWidget, NULL);
}

void ChipsGuiLinePropWidget::vis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiLinePropWidget *lineWidget = static_cast <ChipsGuiLinePropWidget*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (lineWidget)  lineWidget->set_visibility(val);

   if (lineWidget->_callbacks.count ("change_visibility") > 0)
      lineWidget->_callbacks["change_visibility"]->execute (lineWidget, NULL);

}

