// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUILINEPROPFRAME_HH
#define CHIPSGUILINEPROPFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>

#include "ChipsGuiWidget.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

// valid callback ids are
//  * change_style
//  * change_depth
//  * change_thickness
//  * change_visibility
//  * change_extent
//  * change_position
//  * change_coordinate


class ChipsGuiLinePropWidget;
class ChipsGuiPositionWidget;
class ChipsGuiColorWidget;

class ChipsGuiLinePropWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiLinePropWidgetCallback (void (*callback)(ChipsGuiLinePropWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiLinePropWidget *, void *);
     void *_callbackData;
};


//---------------------------------------------------------------------------

class ChipsGuiLinePropWidget: public ChipsGuiWidget, public ChipsGuiTooltipObject
{
public:

  ChipsGuiLinePropWidget();
  virtual ~ChipsGuiLinePropWidget();

  virtual void build_form(); 

  void set_style(int val); 
  void set_depth(int val); 
  void set_color(string color); 
  void set_thick(float val);
  void set_extent(int extent); 
  void set_visibility(bool val);
  void set_position (double xpos, double ypos);
  void set_coordinate (int coordinate);
  void set_position_format (int xformat, int yformat);


  int    get_style();
  int    get_depth();
  string get_color();
  float  get_thick();
  int    get_extent();
  bool   get_visibility();
  void   get_position (double &xpos, double &ypos);
  int    get_coordinate ();
  void   get_position_format (int &xformat, int &yformat);

  void enable_style();
  void enable_depth();
  void enable_color();
  void enable_thick();
  void enable_extent ();
  void enable_visibility();
  void enable_position ();
  void enable_coordinate ();
  void enable_data_coordinate ();

  void disable_style();
  void disable_depth();
  void disable_color();
  void disable_thick();
  void disable_extent();
  void disable_visibility();
  void disable_position ();
  void disable_coordinate ();
  void disable_data_coordinate ();

  virtual void show ();
  virtual void hide ();

protected:

  static void color_callback(GtkWidget *widget, gpointer data);
  static void style_callback(GtkWidget *widget, gpointer data);
  static void thick_callback(GtkWidget *widget, gpointer data);
  static void depth_callback(GtkWidget *widget, gpointer data);
  static void extent_callback(GtkWidget *widget, gpointer data);
  static void vis_callback(GtkWidget *widget, gpointer data);
  static void color_callback(ChipsGuiColorWidget *widget, void *data);
  static void position_callback(ChipsGuiPositionWidget *widget, void *data);
  static void coordinate_callback(ChipsGuiPositionWidget *widget, void *data);

  
  GtkWidget*   _style;
  GtkWidget*   _thick;
  GtkWidget*   _extent;
  GtkWidget*   _depth;
  GtkWidget*   _vis;
  GtkWidget*   _nbook;

  ChipsGuiColorWidget*   _color;
  ChipsGuiPositionWidget* _position;

  // signals for callbacks 
  unsigned long _style_id; 
  unsigned long _thick_id; 
  unsigned long _extent_id; 
  unsigned long _depth_id; 
  unsigned long _vis_id;

  float        _thickval;
  int          _depthval; 
  int          _extentval;
  int          _styleval;  
  bool         _visval;

};

#endif
