// 
//  Copyright (C) 2011,2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiMain.hh"
#include "ChipsGuiMenu.hh"
#include "ChipsGuiHelpMenu.hh"
#include "ChipsGuiEditMenu.hh"
#include "ChipsGuiAnnotateMenu.hh"
#include "ChipsGuiViewMenu.hh"
#include "ChipsGuiNavPane.hh"
#include "ChipsGuiPopUp.hh"
#include "ChipsGuiPropPane.hh"
#include "ChipsGuiInfoPane.hh"
#include "ChipsGuiSettings.hh"
#include "ChipsGuiShellPane.hh"
#include "ChipsGuiToolbar.hh"
#include "ChipsServer.hh"
#include "ChipsEngine.hh"
#include "ChipsCmdManager.hh"
#include "ChipsId.hh"

#include <iostream>
#include <unistd.h>

using namespace std;

extern ChipsCmdManager* cmd_manager;

ChipsGuiSettings* GuiSettings = NULL; 

ChipsGuiMain::ChipsGuiMain(ChipsServer* par, ChipsSystem* systm, string plugs)
{
   GuiSettings = new ChipsGuiSettings(); 
   _def_height = 65;
   _def_width  = 800;
   _timer_id = 0; 
   _timeout=750;
   _parent = par;
   _sys = systm;
   _eng = _sys->get_engine(); 
   _menu  = new ChipsGuiMenu("");
   _visible = false; 
   _server_id = ""; 
   _resetDef = false; 
   _undo_cnt = _redo_cnt = 0; 

   if (plugs.length())
   {
     // parse_custom_file(plugs); 
   }
   
   _window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_position(GTK_WINDOW(_window), GTK_WIN_POS_CENTER);
   gtk_window_set_default_size(GTK_WINDOW(_window), _def_width, _def_height);
   gtk_window_set_title(GTK_WINDOW(_window), "ChIPS GUI Panel");
   _mainvbox = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(_window), _mainvbox);

   gtk_box_pack_start(GTK_BOX(_mainvbox), _menu->base_widget(), 0, 0, 0);

   _tbar = new ChipsGuiToolbar();   
   gtk_box_pack_start(GTK_BOX(_mainvbox), _tbar->base_widget(), FALSE,FALSE, 0);

   _hbox1 = gtk_hbox_new(FALSE, 0);
   _vbox1 = gtk_vbox_new(FALSE, 0);

   gtk_box_pack_start(GTK_BOX(_mainvbox), _hbox1, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(_hbox1), _vbox1, FALSE,TRUE, 0);

   _nav = new ChipsGuiNavPane(); 
   gtk_box_pack_start(GTK_BOX(_vbox1), _nav->base_widget(), TRUE,TRUE, 0);

   _info = new ChipsGuiInfoPane(); 
   gtk_box_pack_start(GTK_BOX(_vbox1), _info->base_widget(), TRUE,TRUE, 0);

   _prop = new ChipsGuiPropPane(); 
   gtk_box_pack_start(GTK_BOX(_hbox1), _prop->base_widget(), TRUE, TRUE, 0);

   _popup = new ChipsGuiPopUp(); 

   if (_parent)  _server_id = _parent->get_server_id(); 
   _shell = new ChipsGuiShellPane( _server_id); 
   gtk_box_pack_start(GTK_BOX(_mainvbox), _shell->base_widget(), TRUE,TRUE, 0);

   g_signal_connect(G_OBJECT(_window), "delete_event",
      G_CALLBACK(cleanup_callback), static_cast<void*> (this));

}



ChipsGuiMain::~ChipsGuiMain()
{
   clear_timer(); 
   if (_tbar)
   {
      delete _tbar;
      _tbar = NULL;
   }

   if (_shell)
   {
      delete _shell;
      _shell = NULL;
   }
   if (_nav)
   {
      delete _nav;
      _nav = NULL;
   }
   if (_info)
   {
      delete _info;
      _info = NULL;
   }
   if (_prop)
   {
      delete _prop;
      _prop = NULL;
   }
   if (_popup)
   {
      delete _popup;
      _popup = NULL;
   }
   if (_menu)
   {
      delete _menu;
      _menu = NULL;
   }
   gtk_widget_destroy(GTK_WIDGET (this->_window));
   this->_window = NULL;
}


void ChipsGuiMain::show_dialog(string wd)
{
   if (wd.length()) _workingdir = wd; 
   gtk_widget_show_all(this->_window);
   _visible = true; 
   update_panels();

   // if update poll was disabled- start it up again
   if (_timer_id == 0)  start_timer(); 
}


void ChipsGuiMain::hide_dialog(bool reset)
{
   _resetDef = reset; 
   gtk_widget_hide(this->_window);
   if (_menu && _menu->get_help_menu())  _menu->get_help_menu()->cleanup();
   _visible = false; 
}

void ChipsGuiMain::update_panels()
{
   gint ww,hh, newht = 27;
   gtk_window_get_size(GTK_WINDOW(_window), &ww, &hh);

   if (_menu == NULL)   return;

   if (_menu->get_view_menu()->display_tbar())
   {
      // if gui was 'hidden'- we use the previous mode but if we 'closed'
      // the gui we want to reset the mode to the default mode
      if (_resetDef)  _tbar->set_mode("");
      _tbar->show();
      newht += _tbar->get_height();

   }
   else
   {
      _tbar->hide();
      hh -= _tbar->get_height(); 
   }

   if (_menu->get_view_menu()->display_tbar())
   {
      // if gui was 'hidden'- we use the previous mode but if we 'closed'
      // the gui we want to reset the mode to the default mode
      newht += _tbar->get_height();
   }


   if (_menu->get_view_menu()->display_property())
   {
      _prop->show();
      newht += _prop->get_height();
      
      if (_menu->get_view_menu()->display_nav())   _nav->show();
         else    _nav->hide();
      if (_menu->get_view_menu()->display_info())  _info->show();
         else    _info->hide();
   }
   else
   {
      _prop->hide();

      if (_menu->get_view_menu()->display_nav())
      {
         _nav->show();
         newht += _nav->get_height();
      }
      else
      {
         _nav->hide();
         hh -= _nav->get_height(); 
      }
      if (_menu->get_view_menu()->display_info())
      {
         _info->show();
         newht += _info->get_height();
      }
      else
      {
         _info->hide();
         hh -= _info->get_height();
      }
   }

   // if both info and coord are hidden- hide vbox so props pane fills gui
   if (_menu->get_view_menu()->display_info())
      gtk_widget_show(_vbox1); 
   else
      gtk_widget_hide(_vbox1); 

   if (_menu->get_view_menu()->display_shell())
   {
      _shell->show();
      newht += _shell->get_height();
   }
   else
   {
      _shell->hide();
      hh -= _shell->get_height();
   }

   gtk_window_resize(GTK_WINDOW(_window), ww, newht);
}



void ChipsGuiMain::update_grids()
{
   // handle window grids 
   if (_eng)
   {
      ChipsId cid; 
      cid.set_window_id("all"); 
      string previd = _eng->current_window(); 
     
      // if no windows to modify, we are done 
      if (previd.length() == 0)  return;  

      cmd_manager->open_block();

      int rv; 
      if (_menu->get_view_menu()->display_grid())
      {
         rv = _eng->display_grid(&cid);
      }
      else 
         rv = _eng->hide_grid(&cid);
      if (rv != ERR_STATUS_GOOD)  log_error();
      if (_eng->current_window() != previd)
      {
         // we changed the current window id while setting grids
         // reset current window id. 
         cid.set_window_id(previd); 
         _eng->set_current(&cid, chips_window, false, false); 
      }

   cmd_manager->clear_hold_buffer();
   cmd_manager->close_block();

   }
}

void ChipsGuiMain::exit_cb()
{
   hide_dialog(true); 
}

int ChipsGuiMain::update_info_panel()
{
   // only need to update if the info panel is visible 
   if (_menu == NULL) return 0; 
   if (_info == NULL) return 0; 
   if (!_visible)  return 0; 
   if (!_menu->get_view_menu()->display_info()) return 0;

   string inf = "";
   char *inf_res = NULL;
   if (_eng)
   {
      ChipsId cid; 
      // call with full=false to avoid getting plot coord raneg
      cid.set_all(); 
      inf_res = _eng->info(&cid, chips_window, true, false);
   }
   if (inf_res)
   {
      inf = inf_res;
      free (inf_res);
   }

   return _info->update_list(inf.c_str());
}

void ChipsGuiMain::update_nav_panel(bool force)
{
   if (_eng && _nav && _visible && cmd_manager)
   {
      if (force)
      {
         _nav->update(); 
      }
      else if (cmd_manager->check_mask(STATE_CHNG_LIMS|STATE_CHNG_CURRENCY))
      {
         _nav->update(); 
         cmd_manager->reset_mask(STATE_CHNG_LIMS|STATE_CHNG_CURRENCY); 
      }
      else if (cmd_manager->check_mask(STATE_CHNG_COORD))
      {
         _nav->update(true);
         cmd_manager->reset_mask(STATE_CHNG_COORD);
      }
   }
}

void ChipsGuiMain::update_prop_panel()
{
   if (_prop && cmd_manager)
      _prop->update ();
}


gboolean ChipsGuiMain::cleanup_callback (GtkWidget *widget, GdkEvent *event, gpointer data)
{
   ChipsGuiMain *guiWin = static_cast <ChipsGuiMain *>(data);
   guiWin->hide_dialog(true);
   return TRUE; 
}

void ChipsGuiMain::clear_timer()
{
   if (_timer_id > 0)  gtk_timeout_remove(_timer_id);
   _timer_id = 0;
}

void ChipsGuiMain::start_timer()
{
   _timer_id = gtk_timeout_add(_timeout, &ChipsGuiMain::update_gui, this);
}



int ChipsGuiMain::update_gui(gpointer data)
{
   ChipsGuiMain *guiWin = static_cast <ChipsGuiMain *>(data);

   if (!guiWin)  return -1; 

   ChipsServer* svr = guiWin->get_parent();
   if (!svr)   return -2;
   if (svr->is_locked()) return -3;

   guiWin->clear_timer();
   svr->lock(-999);
   if (guiWin->get_visible())
   {
      int upd = guiWin->update_info_panel();  // update icon/tree view 
      guiWin->update_nav_panel(upd);  // update nav panel 
      guiWin->update_undo_sensitivity(); 
      guiWin->update_annotate_sensitivity();
      guiWin->update_grids(); 
   }
   guiWin->update_prop_panel(); // update prop panel
   cmd_manager->apply_resets ();  // apply all the resets aquired
   svr->unlock(-999);
   guiWin->start_timer();  // reset timer  

   return 0; 
}

void ChipsGuiMain::gui_prefs_changed()
{
   if (_menu)
   {
      if (_menu->get_view_menu())  _menu->get_view_menu()->update_gui_prefs(); 
   }
   if (_tbar)  _tbar->update_gui_prefs(); 
   update_panels(); 
}

void ChipsGuiMain::undo_command()
{
   if (_eng) 
      if (_eng->undo() != ERR_STATUS_GOOD)  log_error();
}

void ChipsGuiMain::redo_command()
{
   if (_eng) 
      if (_eng->redo() != ERR_STATUS_GOOD)  log_error();
}

void ChipsGuiMain::edit_dialog_command()
{
   if (_prop)  _prop->display_dialog();
}


void ChipsGuiMain::print_command()
{
   if (!_eng)  return;
   ChipsId cid;
   if (_eng->print_window(&cid, _workingdir, "printdialog=true") != ERR_STATUS_GOOD)
      log_error();
}

void ChipsGuiMain::clear_command(bool plot_only)
{
   if (!_eng)  return;
   ChipsId cid;
   if (plot_only) 
   {
      if (_eng->clear_plot(&cid) != ERR_STATUS_GOOD)
         log_error();
   }
   else 
   {
      cid.set_window_id("all"); 
      if (_eng->delete_window(&cid) != ERR_STATUS_GOOD)
         log_error();
   }
}


void ChipsGuiMain::log_error(string err)
{
   if (_shell)    _shell->log_error(err); 
}

void ChipsGuiMain::log_error()
{
   if (_eng && _shell) 
   {
      char* bf = _eng->get_errors();
      if (bf)   _shell->log_error(bf);  
   }
}

void ChipsGuiMain::swap_layout(int val)
{
   GtkWidget *nw = _nav->base_widget(), 
             *iw = _info->base_widget(),
             *pw = _prop->base_widget(); 

   gtk_widget_ref(nw);
   gtk_widget_ref(iw);
   gtk_widget_ref(pw);
   gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(nw)), nw);
   gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(iw)), iw);
   gtk_container_remove(GTK_CONTAINER(gtk_widget_get_parent(pw)), pw);

   if ((val == 0) || (val == -1 && _layout == 1))
   {
      gtk_box_pack_start(GTK_BOX(_vbox1), nw, TRUE,TRUE, 0);
      gtk_box_pack_start(GTK_BOX(_vbox1), iw, TRUE,TRUE, 0);
      gtk_box_pack_start(GTK_BOX(_hbox1), pw, TRUE,TRUE, 0);
      _layout = 0; 
   }
   else 
   {
      gtk_box_pack_start(GTK_BOX(_vbox1), pw, TRUE,TRUE, 0);
      gtk_box_pack_start(GTK_BOX(_vbox1), nw, TRUE,TRUE, 0);
      gtk_box_pack_start(GTK_BOX(_hbox1), iw, TRUE,TRUE, 0);
      _layout = 1; 
   }

   gtk_widget_unref(nw);
   gtk_widget_unref(iw);
   gtk_widget_unref(pw);

}

void ChipsGuiMain::delete_command(ChipsId* cid, int typ)
{
   if (!(cid && _eng)) return; 
   int rval; 

   switch(typ)
   {
      case chips_window:
        rval = _eng->delete_window(cid);  
      break; 
      case chips_frame:
        rval = _eng->delete_frame(cid);
      break;
      case chips_plot:
        rval = _eng->delete_plot(cid);
      break;
      case chips_curve:
        rval = _eng->delete_curve(cid);
      break;
      case chips_image:
        rval = _eng->delete_image(cid);
      break;
      case chips_histogram:
        rval = _eng->delete_histogram(cid);
      break;
      case chips_contour:
        rval = _eng->delete_contour(cid);
      break;
      case chips_axis:
        rval = _eng->delete_axis(cid);
      break;
      case chips_label:
        rval = _eng->delete_label(cid);
      break;
      case chips_line:
        rval = _eng->delete_line(cid);
      break;
      case chips_point:
        rval = _eng->delete_point(cid);
      break;
      case chips_region:
        rval = _eng->delete_region(cid);
      break;
      case chips_colorbar:
        rval = _eng->delete_colorbar(cid);
      break;
      default:
        rval = ERR_BAD_OBJECT;
      break;
   }
   if (rval != ERR_STATUS_GOOD)   log_error(); 
}


void ChipsGuiMain::update_undo_sensitivity()
{
   bool undo, redo; 
   cmd_manager->get_undo_counts(_undo_cnt, _redo_cnt); 
   undo = (_undo_cnt > 0); 
   redo = (_redo_cnt > 0); 
   _tbar->set_undo_redo_sensitivity(undo, redo);
   _menu->get_edit_menu()->set_undo_redo_sensitivity(undo,redo);
}

bool ChipsGuiMain::get_annotate_sensitivity ()
{
   char *inf_res = NULL;
   bool enable = false;
   if (_eng)
   {
      ChipsId cid;
      // call with full=false to avoid getting plot coord raneg
      cid.set_all();
      inf_res = _eng->info(&cid, chips_plot, false, false);
   }

   if (inf_res)
   {
      enable = true;
      free (inf_res);
   }

   return enable;
}

void ChipsGuiMain::update_annotate_sensitivity()
{
   bool enable = this->get_annotate_sensitivity ();
   _menu->get_annotate_menu()->set_sensitivity(enable);
   _popup->enable_annotate_menu (enable);
}


void ChipsGuiMain::save_command(string fname, int mode)
{
   if (fname.length())
   {
      // if file already exists- delete it- this command is only run
      // from the dialog after the user has confirmed the delete request 
      if (access(fname.c_str(), W_OK) == 0)  remove(fname.c_str());  
       
      if (mode == 0)
         _eng->save_state(fname, true, false, mode);
      else 
      {
         // we should delete the prefs file if it exists
         string prefname = fname;
         prefname.append(".prefs");
         if (access(prefname.c_str(), W_OK) == 0)  remove(prefname.c_str());

         // make script- need to shell out to handle this since the
         // python HLUI handles crates i/o
         char namebuf[256];
         int pd = (int)getpid();
         char* lgname = getlogin();
         if (lgname)  sprintf(namebuf,"/tmp/.chips.%s.%1d.savecmd", lgname,pd);
            else      sprintf(namebuf,"/tmp/.chips.%1d.savecmd", pd);
         ofstream out(namebuf);
         out << "import pychips.all as chips" << endl;
         out << "chips.connect(\"" << get_server_id() << "\")" << endl;
         out << "try:"<<endl<<"   chips.make_script(\""<<fname;
         out << "\",True)" << endl<<"finally:"<<endl;
         out << "   import os"<<endl;
         out << "   os.remove(\"" << namebuf << "\")" << endl;
         out.close();
         string cmd = "python ";
         cmd.append(namebuf);
         cmd.append(" &");
         system(cmd.c_str());
      }
      update_recent_filelist(fname);
   }
}

void ChipsGuiMain::update_recent_filelist(string fname)
{
   _refname = fname; 
   _refcnt = 10; 
   g_timeout_add(1000, &update_recent_file, this);
}

int ChipsGuiMain::update_recent_file(gpointer data)
{
   ChipsGuiMain* me = (ChipsGuiMain*)data;
   int rv = 0; 
   if (me)  rv = me->process_reffile();  
   return rv; 
}

int ChipsGuiMain::process_reffile()
{
   if (_refname.length() == 0 || _refcnt < 1)  return 0; 
   _refcnt--;
   string tmpf, tmpf2;  
   if (_refname.find("file:") == 0)  
   {
      tmpf = _refname.substr(5, _refname.length());
      tmpf2 = _refname; 
   }
   else
   {
      tmpf = _refname;
      tmpf2 = "file:"; 
      tmpf2.append(_refname); 
   }

   if (!access(tmpf.c_str(), F_OK)) 
   {
      GtkRecentManager *manager = gtk_recent_manager_get_default();
      gtk_recent_manager_add_item(manager, tmpf2.c_str());
      _refcnt = 0;
      _refname = ""; 
      return 0; 
   }
   return 1; 
}


