// 
//  Copyright (C) 2011,2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSGUIMAIN_HH
#define CHIPSGUIMAIN_HH

#include <gtk/gtk.h>
#include <string>

using namespace std;

class ChipsGuiMenu;
class ChipsGuiToolbar;
class ChipsGuiNavPane;
class ChipsGuiInfoPane;
class ChipsGuiPropPane;
class ChipsGuiShellPane; 
class ChipsGuiPopUp; 

class ChipsServer;
class ChipsSystem;
class ChipsEngine; 
class ChipsId; 

class ChipsGuiMain
{
  public:
    ChipsGuiMain(ChipsServer* par,  ChipsSystem* syst, string plugs="");
    virtual ~ChipsGuiMain();

    void show_dialog(string wd="");
    void hide_dialog(bool reset=false);
    void update_grids();
    void update_panels();
    int  update_info_panel();
    void update_nav_panel(bool force);
    void update_prop_panel ();
    void update_undo_sensitivity();  
    void update_annotate_sensitivity();
    bool get_annotate_sensitivity ();
    
    void gui_prefs_changed();  

    void undo_command();
    void redo_command();
    void save_command(string fname, int mode=0);
    void edit_dialog_command(); 
    void delete_command(ChipsId* cid, int typ); 
    void print_command(); 
    void clear_command(bool plot_only); 
    void swap_layout(int val=-1); 
    void update_recent_filelist(string fname);
    int process_reffile(); 

    void exit_cb();
    static gboolean cleanup_callback (GtkWidget *widget, GdkEvent *event, gpointer data);
    static int  update_gui(gpointer);
    static int  update_recent_file(gpointer data); 

    unsigned long get_timer_id()  { return _timer_id; }
    unsigned long get_timeout()   { return _timeout; }
    void          start_timer(); 
    void          clear_timer(); 
    bool          get_visible()   { return _visible; } 
    int           get_undo_available() { return _undo_cnt; }
    int           get_redo_available() { return _redo_cnt; }
    ChipsEngine*  get_engine()   { return _eng; }
    ChipsServer*  get_parent()   { return _parent; }
    string        get_server_id() { return _server_id; }
    string        get_working_dir()  { return _workingdir; }
    void          set_working_dir(string path) { _workingdir = path; }
    void          log_error(string err); 
    void          log_error(); 
  protected:

  private:
    ChipsGuiMenu* _menu;
    ChipsGuiToolbar* _tbar;
    ChipsGuiNavPane* _nav;
    ChipsGuiInfoPane* _info; 
    ChipsGuiPropPane* _prop;
    ChipsGuiPopUp*  _popup; 
    ChipsGuiShellPane* _shell;  

    GtkWidget* _window;
    GtkWidget* _mainvbox; 
    GtkWidget* _hbox1;       // horizontal split- info | prop  
    GtkWidget* _vbox1;       // vertical split coord/info 

    ChipsServer* _parent;
    ChipsSystem* _sys; 
    ChipsEngine* _eng; 
    int          _layout; 
    int          _def_height;
    int          _def_width; 
    bool         _resetDef;    // reset 'default' mode on show 
    string       _customfile; 
    string       _server_id; 
    string       _workingdir; 
    string       _refname; 
    unsigned long _timer_id;
    unsigned long _timeout;
    int           _undo_cnt; 
    int           _redo_cnt; 
    int           _refcnt; 
    bool       _visible; // is gui visibile? 
};


#endif 
