// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiMenu.hh"
#include "ChipsGuiFileMenu.hh"
#include "ChipsGuiEditMenu.hh"
#include "ChipsGuiViewMenu.hh"
#include "ChipsGuiPlugInMenu.hh"
#include "ChipsGuiHelpMenu.hh"
#include "ChipsGuiAnnotateMenu.hh"

#include <stdlib.h>

#include <atk/atk.h>

using namespace std;

ChipsGuiMenu::ChipsGuiMenu(string filename)
{
   AtkObject *accessible;

   base_w = gtk_menu_bar_new();
   menuFile = new ChipsGuiFileMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuFile->base_widget());

   menuEdit = new ChipsGuiEditMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuEdit->base_widget());

   menuAnnotate = new ChipsGuiAnnotateMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuAnnotate->base_widget());

   menuView = new ChipsGuiViewMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuView->base_widget());

   menuPlug = new ChipsGuiPlugInMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuPlug->base_widget());

   // for now load $HOME/.chips.funcs 
   string tmp;
   char *pth = getenv("HOME");
   if (pth)
   {
       tmp = pth;
       tmp.append("/");
   }
   tmp.append(".chips.funcs"); 
   load_plugin_menu(tmp); 

   menuHelp = new ChipsGuiHelpMenu();
   gtk_menu_item_set_right_justified(GTK_MENU_ITEM(menuHelp->base_widget()),
      TRUE);
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuHelp->base_widget());

   // add accessibility info
   accessible = gtk_widget_get_accessible (base_w);
   atk_object_set_name (accessible, "mainMenu");
   atk_object_set_description (accessible, "Top level menu for the app\n");
}

ChipsGuiMenu::~ChipsGuiMenu()
{
   if (menuFile)
   {
      delete menuFile;
      menuFile = NULL;
   }
   if (menuEdit)
   {
      delete menuEdit;
      menuEdit = NULL;
   }
   if (menuAnnotate)
   {
      delete menuAnnotate;
      menuAnnotate = NULL;
   }
   if (menuView)
   {
      delete menuView;
      menuView = NULL;
   }
   if (menuPlug)
   {
      delete menuPlug;
      menuPlug = NULL;
   }
   if (menuHelp)
   {
      delete menuHelp;
      menuHelp = NULL;
   }
}

int ChipsGuiMenu::show()
{
   menuFile->show();
   menuEdit->show();
   menuAnnotate->show ();
   menuView->show();
   menuPlug->show();
   menuHelp->show();
   gtk_widget_show(base_w);
   return 0; 
}

int ChipsGuiMenu::load_plugin_menu(string fname) 
{
   if (fname.length() == 0)  return -1; 

   if (menuPlug) menuPlug->load_file(fname); 
   return 0;
}


