// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiNavPane.hh"
#include "ChipsGuiSpinWidget.hh"

#define FGINDEX  1
#define BGINDEX  3 

#include "ChipsEngine.hh"
#include "ChipsGuiMain.hh"
#include "ChipsId.hh"

#include <gdk/gdkkeysyms.h>
#include <sstream>
#include <stdio.h>
#include <stdlib.h>

using namespace std; 

extern ChipsGuiMain* chipsGui;
extern ChipsEventManager* event_manager;

int ChipsGuiNavPane::_prevx = -99;
int ChipsGuiNavPane::_prevy = -99;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiNavPane::ChipsGuiNavPane():ChipsEventObserver(),ChipsGuiTooltipObject()
{
   _cids.clear(); 
   _objCnt = 0;
   _idcnt = 0; 
   _nodata = true; 
   _enabled = false; 
   _prev = ""; 
   _frame = gtk_frame_new("Coordinates");
   _table = gtk_table_new (20,20, FALSE);
   gtk_container_add (GTK_CONTAINER(_frame), _table);

   _drawarea = gtk_drawing_area_new ();
   gtk_drawing_area_size(GTK_DRAWING_AREA(_drawarea),100,100);
   _asp = gtk_aspect_frame_new(NULL, .5, .5, 1.0, false); 
   gtk_table_attach(GTK_TABLE (_table), _asp, 11, 19, 1, 9,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_container_add(GTK_CONTAINER(_asp), _drawarea);
   gtk_frame_set_shadow_type(GTK_FRAME(_asp), GTK_SHADOW_NONE); 

   // coord sys info
   _coordsys = gtk_combo_box_new_text();
   gtk_table_attach (GTK_TABLE (_table), _coordsys, 2, 10, 1, 2,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   _active = -1;
   _activeTxt = "";
   _label[2] = gtk_label_new("Data Extent");  
   _label[3] = gtk_label_new("    :    "); 
   _label[4] = gtk_label_new("    :    "); 
   gtk_table_attach (GTK_TABLE (_table), _label[2], 1, 10, 3, 4,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _label[3], 2, 10, 6, 7,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _label[4], 2, 10, 7, 8,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

   // X coords 
   _label[0] = gtk_label_new("X");
   _xmin = new ChipsGuiSpinWidget(0);
   _xmax = new ChipsGuiSpinWidget(0);
   _logx =  gtk_toggle_button_new_with_label("log"); 
   _flipx = gtk_toggle_button_new_with_label("flip"); 
   _autox = gtk_button_new_with_label("auto"); 

   // Y coords  
   _label[1] = gtk_label_new("Y");
   _ymin = new ChipsGuiSpinWidget(1); 
   _ymax = new ChipsGuiSpinWidget(1); 
   _logy =  gtk_toggle_button_new_with_label("log"); 
   _flipy = gtk_toggle_button_new_with_label("flip"); 
   _autoy = gtk_button_new_with_label("auto"); 

   gtk_table_attach (GTK_TABLE (_table), _label[0], 1, 3, 14, 15,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _xmin->base_widget(), 4,7, 14, 15,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _xmax->base_widget(), 8,11, 14, 15,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _logx, 12,14, 14, 15,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _flipx, 14,16, 14, 15,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _autox, 16,18, 14, 15,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

   gtk_table_attach (GTK_TABLE (_table), _label[1], 1, 3, 16, 17,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _ymin->base_widget(), 4,7, 16, 17,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _ymax->base_widget(), 8,11, 16, 17,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _logy, 12,14, 16, 17,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _flipy, 14,16, 16, 17,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
   gtk_table_attach (GTK_TABLE (_table), _autoy, 16,18, 16, 17,
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

   setup_tooltip(_drawarea, "Scroll button zooms in/out\nShows data extents (black rectangle) vs viewable area (blue rectangle)");
   setup_tooltip(_autox, "Set x axis limits to the data range spanned by curves, contours, histograms, and images");
   setup_tooltip(_autoy, "Set y axis limits to the data range spanned by curves, contours, histograms, and images");
   setup_tooltip(_flipx, "Reverse the direction of the x axis");
   setup_tooltip(_flipy, "Reverse the direction of the y axis");
   setup_tooltip(_logx, "Toggle the scaling of the x axis to log/linear");
   setup_tooltip(_logy, "Toggle the scaling of the y axis to log/linear");
   setup_tooltip(_coordsys, "Select which data coordinate system to display");

   // attach signals
   _signals[0] = g_signal_connect(G_OBJECT(_autox), "clicked",
                    G_CALLBACK(autolim_callback), (void*)this);
   _signals[1] = g_signal_connect(G_OBJECT(_autoy), "clicked",
                    G_CALLBACK(autolim_callback), (void*)this);
   _signals[2] = g_signal_connect(G_OBJECT(_logx), "clicked",
                    G_CALLBACK(log_callback), (void*)this);
   _signals[3] = g_signal_connect(G_OBJECT(_logy), "clicked",
                    G_CALLBACK(log_callback), (void*)this);
   _signals[4] = g_signal_connect(G_OBJECT(_flipx), "clicked",
                    G_CALLBACK(flip_callback), (void*)this);
   _signals[5] = g_signal_connect(G_OBJECT(_flipy), "clicked",
                    G_CALLBACK(flip_callback), (void*)this);
   _signals[6] = g_signal_connect ((gpointer) _drawarea, "expose_event",
                     G_CALLBACK (da_expose_cb), (void*)this);

   _xmin->add_callback ("change_value", new ChipsGuiSpinWidgetCallback(spin_changed_cb, this));
   _xmax->add_callback ("change_value", new ChipsGuiSpinWidgetCallback(spin_changed_cb, this));
   _ymin->add_callback ("change_value", new ChipsGuiSpinWidgetCallback(spin_changed_cb, this));
   _ymax->add_callback ("change_value", new ChipsGuiSpinWidgetCallback(spin_changed_cb, this));
   _xmin->add_callback ("change_step", new ChipsGuiSpinWidgetCallback(spin_change_step_cb, this));
   _xmax->add_callback ("change_step", new ChipsGuiSpinWidgetCallback(spin_change_step_cb, this));
   _ymin->add_callback ("change_step", new ChipsGuiSpinWidgetCallback(spin_change_step_cb, this));
   _ymax->add_callback ("change_step", new ChipsGuiSpinWidgetCallback(spin_change_step_cb, this));

   _signals[7] = g_signal_connect ((gpointer) _coordsys, "changed",
                     G_CALLBACK(combo_changed_cb), (void*)this);

   g_signal_connect(_drawarea, "scroll-event",
                    G_CALLBACK(da_scroll_cb), (void*)this);
   g_signal_connect(_drawarea, "motion_notify_event",
                    G_CALLBACK(da_motion_cb), (void*)this);
   g_signal_connect(_drawarea, "button-release-event",
                    G_CALLBACK(da_button_cb), NULL);

   gtk_widget_set_events(_drawarea, gtk_widget_get_events(_drawarea) |
      GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |
      GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_SCROLL_MASK);

   // register w/ event observer to get changes in limits 
   register_event(ChipsEventManager::PanEvent);
   register_event(ChipsEventManager::ZoomEvent);
   register_event(ChipsEventManager::RangeZoomEvent);
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiNavPane::~ChipsGuiNavPane()
{
   // delete all ChipsId maps
   for(cid_map::iterator ii=_cids.begin(); ii!=_cids.end(); ++ii)
   {
      delete((*ii).second);
   }
   _cids.clear();

   gtk_widget_destroy(_asp);
   gtk_widget_destroy(_frame);
}

void ChipsGuiNavPane::da_scroll_cb(GtkWidget* widget, GdkEventScroll *event,
   gpointer data)
{
   // drawing ares scroll events are used to zoom 
   ChipsGuiNavPane *cpane = static_cast <ChipsGuiNavPane *>(data);
   if (!(chipsGui && event))  return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng) return; 

   static ChipsId cid; 
   if (event->direction)  eng->zoom(&cid, .9);
      else                eng->zoom(&cid, 1.1);
   cpane->update();
}

void ChipsGuiNavPane::da_expose_cb(GtkWidget* widget, GdkEventExpose *event,
   gpointer data)
{
   ChipsGuiNavPane *cpane = static_cast <ChipsGuiNavPane *>(data);
   cpane->coord_update();
}

void ChipsGuiNavPane::da_button_cb(GtkWidget* widget, GdkEventButton *event,
   gpointer data)
{
   // drawing area button releases indicate the start/stop of drag to pan 
   if (event->type == GDK_BUTTON_RELEASE)
   {
      _prevx = _prevy = -99;
   } 
}



void ChipsGuiNavPane::da_motion_cb(GtkWidget* widget, GdkEventMotion *event,
   gpointer data)
{
   // drawing area drags are used to pan 
   ChipsGuiNavPane *cpane = static_cast <ChipsGuiNavPane *>(data);

   // Use gdk_window_get_pointer and gtk_widget_set_events so that
   // can handle motion events as we handle the current one- prevents
   // getting flooded with motion events  
   int x, y;
   GdkModifierType state;
   gdk_window_get_pointer (event->window, &x, &y, &state);
   if (state & GDK_BUTTON1_MASK)  
   {
      if (_prevx != -99 && _prevy != -99) 
         cpane->adjust_limits((x - _prevx),(_prevy -y));
      _prevx = x;
      _prevy = y; 
   }
}

void ChipsGuiNavPane::show()
{
   gtk_widget_show(GTK_WIDGET(_frame));
   update(); 
}

void ChipsGuiNavPane::hide()
{
   gtk_widget_hide(GTK_WIDGET(_frame));
}

void ChipsGuiNavPane::coord_update(bool reset)
{
   double minx, maxx, miny, maxy;
   int ww = _drawarea->allocation.width;
   int hh = _drawarea->allocation.height;
   float xoff = 1.0, yoff = 1.0; 
   ChipsEngine* eng = chipsGui->get_engine();

   if (_nodata || (eng == NULL))  return; 

   // convert data range and limits from data coords to pixel
   ChipsVector vin1(_limx1,_limy1), vout1;
   ChipsVector vin2(_limx2,_limy2), vout2;
   ChipsVector vin3(_rngx1,_rngy1), vout3;
   ChipsVector vin4(_rngx2,_rngy2), vout4;
   ChipsId cid1, cid2; 
   cid1.set_coord_sys(DATA); 
   if (_active > -1 && _active < _idcnt) 
   {
      ChipsId* tid = get_active_cid(_active); 
      if (tid)  cid1.set_coord_id(tid->get_coord_id()); 
   }
   cid2.set_coord_sys(PIXEL_REAL); 
   eng->convert_coordinate(vin1, vout1, &cid1, &cid2, XY_AXIS); 
   eng->convert_coordinate(vin2, vout2, &cid1, &cid2, XY_AXIS); 
   if ((eng->convert_coordinate(vin3, vout3, &cid1, &cid2, XY_AXIS)) ==
       ERR_NEG_LOG_VAL)
   {
      vin3[0] = _rngxlog;
      vin3[1] = _rngylog;
      eng->convert_coordinate(vin3, vout3, &cid1, &cid2, XY_AXIS);
   } 
   eng->convert_coordinate(vin4, vout4, &cid1, &cid2, XY_AXIS); 

   // bail if we don't have valid pixel coords 
   if (vout1.get_size() < 2 || vout2.get_size() < 2 || 
       vout3.get_size() < 2 || vout4.get_size() < 2)  return; 

   // find pixel extents 
   if (vout1[0] < vout2[0])
   {
      minx = vout1[0];  maxx = vout2[0]; 
   }
   else 
   {
      minx = vout2[0];  maxx = vout1[0]; 
   }
   if (vout3[0] < minx)   minx = vout3[0]; 
      else if (vout3[0] > maxx)   maxx = vout3[0];
   if (vout4[0] < minx)   minx = vout4[0]; 
      else if (vout4[0] > maxx)   maxx = vout4[0];
   if (vout1[1] < vout2[1])
   {
      miny = vout1[1];  maxy = vout2[1];
   }
   else
   {
      miny = vout2[1];  maxy = vout1[1];
   }
   if (vout3[1] < miny)   miny = vout3[1];
      else if (vout3[1] > maxy)   maxy = vout3[1];
   if (vout4[1] < miny)   miny = vout4[1];
      else if (vout4[1] > maxy)   maxy = vout4[1];

   _pixx1 = vout1[0]; 
   _pixy1 = vout1[1];
   _pixx2 = vout2[0]; 
   _pixy2 = vout2[1];

   // compute unit scaling 
   float dx = maxx - minx;
   float dy = maxy - miny;
   float midpnt, ratio, ux, uy;
   if (dy != 0) ratio = dx/dy; 
      else ratio = 1; 
   if (ratio == 0.0)  ratio = 1.0;
   if (dy == 0.0)  dy = 1.0;  

   // clear old drawing 
   gdk_draw_rectangle(_drawarea->window,
      _drawarea->style->bg_gc[0], TRUE, 0, 0, ww, hh);

   // take aspect ratio of plot into account 
   if (ratio >= 1)
   {
      midpnt = (maxy + miny) / 2.0; 
      maxy = (maxy - midpnt) * ratio + midpnt;
      miny = midpnt - (midpnt - miny) * ratio;
      ux = (ww - 2*xoff)/ dx;
      uy = (hh - 2*yoff)/ (dy*ratio);
   }
   else 
   {
      midpnt = (maxx + minx) / 2.0;
      maxx = midpnt + (maxx - midpnt) / ratio;
      minx = midpnt - (midpnt - minx) / ratio;
      ux = ((ww - 2*xoff)/ dx) * ratio;
      uy = (hh - 2*yoff)/ dy;
   }

   // handle flipped axes  
   double xlocR, xdistR, ylocR, ydistR; 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_flipx)))
   {
      xlocR = vout4[0] - minx;
      xdistR = vout3[0]-vout4[0]; 
   }
   else 
   {
      xlocR = vout3[0] - minx;
      xdistR = vout4[0]-vout3[0]; 
   }
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_flipy)))
   {
      ylocR = maxy-vout3[1]; 
      ydistR = vout3[1]-vout4[1];
   }
   else
   {
      ylocR = maxy-vout4[1]; 
      ydistR = vout4[1]-vout3[1];
   }

   gdk_draw_rectangle(_drawarea->window,     // data range box 
                _drawarea->style->fg_gc[FGINDEX], FALSE,
                xoff+xlocR*ux,yoff+ylocR*uy,xdistR*ux, ydistR*uy);
   gdk_draw_rectangle(_drawarea->window,     // limits box 
                _drawarea->style->bg_gc[BGINDEX], FALSE,
                xoff+(vout1[0] - minx)*ux,yoff+(maxy-vout2[1])*uy,
                (vout2[0]-vout1[0])*ux, (vout2[1]-vout1[1])*uy);
}

int ChipsGuiNavPane::get_height()
{
   int ht = 0;

   if (_frame)   ht = _frame->allocation.height;
   return ht;
}

int ChipsGuiNavPane::update(bool reset)
{
   if (!chipsGui)  return -1;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng)  return -2;
   if (!chipsGui->get_visible()) return 0; 
   ChipsId cid; 
   ChipsId* myid; 
   string coords;
   char cbuf[256];
   int tdim; 
   static int prexmode = -1, preymode = -1; 

   g_signal_handler_block(GTK_COMBO_BOX(_coordsys), _signals[7]); 

   // if a coord sys is not set- override reset 
   if (_active == -1)   reset = true;

   // get all coordinate systems
   char *cptr = eng->info_coordinate(&cid);
   if (cptr)   coords = cptr;
      else     coords = ""; 
   free (cptr);

   //get current coord sys
   eng->get_current_coord(&cid);
   string buf; // Have a buffer string
   string wid, fid, pid, axid, ayid, csys, tmp; 
   size_t pos1, pos2; 

   COORD_ELEM_P_T elem_p; 
   string csid = cid.get_coord_id();

   // do we need to regenerate the combo box entries
   if (coords != _prev)
   {
      istringstream iss(coords); // Insert the string into a stream

      // remove entries from combo box 
      for(int ii = _idcnt; ii--; ) 
      {
         gtk_combo_box_remove_text(GTK_COMBO_BOX(_coordsys), ii);
      }
      // reset mapped coord sys slot indices
      for(cid_map::iterator ii=_cids.begin(); ii!=_cids.end(); ++ii)
         ((*ii).second)->slot = -2;
      _idcnt = 0; 

      while (getline(iss, buf, '\n') )
      {
         pos1 = buf.find("["); 
         pos2 = buf.find("]"); 
         if (buf.find("Window [") != string::npos)
         {
            wid = buf.substr(pos1+1,pos2-pos1-1); 
         }
         else if (buf.find("  Frame [") != string::npos)
         {
            fid = buf.substr(pos1+1,pos2-pos1-1); 
         }
         else if (buf.find("    Plot [") != string::npos)
         {
            pid = buf.substr(pos1+1,pos2-pos1-1); 
            tdim = 0; 
         }
         else if (buf.find("        Coord Sys ID [") != string::npos)
         {
            csys = buf.substr(pos1+1,pos2-pos1-1); 
         }
         else if (buf.find("        X Axis [") != string::npos)
         {
            axid = buf.substr(pos1+1,pos2-pos1-1); 
            if (axid.length())  tdim |= X_AXIS; 
         }
         else if (buf.find("        Y Axis [") != string::npos)
         {
            ayid = buf.substr(pos1+1,pos2-pos1-1); 
            tmp = pid.c_str();
            if (tmp.length()) tmp.append(": ");
            tmp.append(axid);
            if (axid.length() && ayid.length())  tmp.append(" & ");
            tmp.append(ayid);
            if (ayid.length())  tdim |= Y_AXIS; 
            if (_cids.count(csys))
            {
               // coord sys already in map- reindex it 
               _cids[csys]->slot = _idcnt; 
               if (_cids[csys]->title != tmp)  _cids[csys]->title = tmp; 
               if (_cids[csys]->dim != tdim)   _cids[csys]->dim = tdim; 
               if (_cids[csys]->id.get_xaxis_id() != axid)
                  _cids[csys]->id.set_xaxis_id(axid);
               if (_cids[csys]->id.get_yaxis_id() != ayid)
                  _cids[csys]->id.set_yaxis_id(ayid);
            }
            else 
            {
               // new coord sys- add to map 
               COORD_ELEM_P_T elem_p = new COORD_ELEM_T;
               if (elem_p)
               {
                  elem_p->slot = _idcnt; 
                  elem_p->title = tmp;
                  elem_p->dim = tdim; 
                  myid = &elem_p->id;
                  myid->set_window_id(wid);
                  myid->set_frame_id(fid);
                  myid->set_plot_id(pid);
                  myid->set_xaxis_id(axid);
                  myid->set_yaxis_id(ayid);
                  myid->set_coord_id(csys);
                  _cids.insert(cid_map::value_type (csys, elem_p));
               }
            }
            _idcnt++; 
         }
      }
      _nodata = true; 

      // no data coords sys- disable widgets 
      if (_idcnt == 0 || (cid.get_coord_sys() != DATA))  disable();  
         else  enable(); 
   
      if (_idcnt != (int)_cids.size())
      {
         // upmap coordinate systems which are no longer in use
         for(cid_map::iterator ii=_cids.begin(); ii!=_cids.end(); ++ii)
            if (((COORD_ELEM_P_T)((*ii).second))->slot == -2)  
            {
               delete((*ii).second);
               _cids.erase(ii);
            }
      }

      for (int rr = 0; rr < _idcnt; rr++)
      {
         if ((elem_p = get_active_elem(rr)) != NULL) 
            gtk_combo_box_insert_text(GTK_COMBO_BOX(_coordsys), rr, elem_p->title.c_str());
      }
      _prev = coords; 
   }
   else 
   {
      // coord list  hasn't changed- but the current coord sys may have
      if (_idcnt == 0 || (cid.get_coord_sys() != DATA))  disable();
         else  enable();
   }

   if (_cids.count(csid) != 0)
   {
      _active = _cids[csid]->slot; 
   }
   if (_active < 0)  
   {
       _active = 0;  // use first coord sys if active not set
       if ((elem_p = get_active_elem(0)) != NULL) 
          csid = elem_p->id.get_coord_id();
   }

   // handle selected 
   if (_active < _idcnt && _cids.count(csid))  
   {
      double out_val[6];

      myid = get_active_cid();

      _objCnt = eng->get_coord_object_count(myid, chips_curve) +
                eng->get_coord_object_count(myid, chips_contour) + 
                eng->get_coord_object_count(myid, chips_histogram) + 
                eng->get_coord_object_count(myid, chips_image); 
      _nodata = (_objCnt == 0);

      int dim = _cids[csid]->dim; 
      set_axis_sensitivity(X_AXIS, (dim & X_AXIS));
      set_axis_sensitivity(Y_AXIS, (dim & Y_AXIS));

      if ((eng->get_property(myid,(void*)&out_val,GET_PLOT_DATA_RANGE)) == ERR_STATUS_GOOD)
      {
         _rngx1 = out_val[0];
         _rngx2 = out_val[1];
         _rngy1 = out_val[2];
         _rngy2 = out_val[3];
         _rngxlog = out_val[4]; 
         _rngylog = out_val[5]; 
      }
      if ((eng->get_property(myid,(void*)&out_val,GET_PLOT_RANGE)) == ERR_STATUS_GOOD)
      {
         _limx1 = out_val[0];
         _limx2 = out_val[1];
         _limy1 = out_val[2];
         _limy2 = out_val[3];
      }

      // set active coord system 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_coordsys), _active); 

      if (_objCnt)
      {
         string lb1, lb2;

         // get tick formats
         int xmode = 0, ymode = 0;
         string xformat, yformat;
         ChipsId* tmpid = new ChipsId(*(myid));
         tmpid->set_axis_id(myid->get_xaxis_id());
         eng->get_property(tmpid, (void*)&xformat, SET_AXIS_TICKFORMAT);
         eng->format_axis_point(tmpid, _rngx1, lb1);
         eng->format_axis_point(tmpid, _rngx2, lb2);
         sprintf(cbuf, "X %s .. %s", lb1.c_str(), lb2.c_str());
         gtk_label_set_text(GTK_LABEL(_label[3]), cbuf);
         tmpid->set_axis_id(myid->get_yaxis_id());
         eng->get_property(tmpid, (void*)&yformat, SET_AXIS_TICKFORMAT);
         eng->format_axis_point(tmpid, _rngy1, lb1);
         eng->format_axis_point(tmpid, _rngy2, lb2);
         delete tmpid;
         sprintf(cbuf, "Y %s .. %s", lb1.c_str(), lb2.c_str());
         gtk_label_set_text(GTK_LABEL(_label[4]),cbuf);

         if (!xformat.length())
            xformat = "%5.4f";
         else if (xformat.find("ra") != string::npos ||
            xformat.find("RA") != string::npos ||
            xformat.find("dec") != string::npos ||
            xformat.find("DEC") != string::npos)      xmode = 1;
         if (!yformat.length())   
            yformat = "%5.4f";
         else if (yformat.find("ra") != string::npos ||
            yformat.find("RA") != string::npos ||
            yformat.find("dec") != string::npos ||
            yformat.find("DEC") != string::npos)      ymode = 1;
         // handle format changes from numeric to sexagesimal
         if (prexmode != xmode)
         {
            if (_xmin->get_mode() != xmode)   _xmin->set_mode(xmode);
            if (_xmax->get_mode() != xmode)   _xmax->set_mode(xmode);
            prexmode = xmode; 
         }
         if (preymode != ymode)
         {
            if (_ymin->get_mode() != ymode)   _ymin->set_mode(ymode);
            if (_ymax->get_mode() != ymode)   _ymax->set_mode(ymode);
            preymode = ymode; 
         }
         gtk_widget_show(GTK_WIDGET(_drawarea));
         gtk_widget_set_sensitive(GTK_WIDGET(_autox), _enabled); 
         gtk_widget_set_sensitive(GTK_WIDGET(_autoy), _enabled); 
      }
      else 
      {
         gtk_label_set_text(GTK_LABEL(_label[3]),"X undefined");
         gtk_label_set_text(GTK_LABEL(_label[4]),"Y undefined");
         gtk_widget_set_sensitive(GTK_WIDGET(_autox), false); 
         gtk_widget_set_sensitive(GTK_WIDGET(_autoy), false); 
         gtk_widget_hide(GTK_WIDGET(_drawarea));
      }

      _xmin->set(_limx1); 
      _xmax->set(_limx2); 
      _ymin->set(_limy1);
      _ymax->set(_limy2);

      // set toggles for log scale
      ChipsId* tmp = new ChipsId(*(myid)); 
      int ascale; 
      tmp->set_axis_id(myid->get_xaxis_id());
      tmp->set_axis_dim(X_AXIS); 
      if ((eng->get_property(tmp,(void*)&ascale, GET_AXIS_SCALE)) == 
         ERR_STATUS_GOOD)
      {
         g_signal_handler_block(GTK_TOGGLE_BUTTON(_logx), _signals[2]); 
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_logx), ascale); 
         g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_logx), _signals[2]); 
      }
      if ((eng->get_property(tmp,(void*)&ascale, SET_AXIS_REVERSE)) == 
         ERR_STATUS_GOOD)
      {
         g_signal_handler_block(GTK_TOGGLE_BUTTON(_flipx), _signals[4]); 
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_flipx), ascale); 
         g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_flipx), _signals[4]); 
      }
      tmp->set_axis_id(myid->get_yaxis_id());
      tmp->set_axis_dim(Y_AXIS); 
      if ((eng->get_property(tmp,(void*)&ascale, GET_AXIS_SCALE)) ==
          ERR_STATUS_GOOD)
      {
         g_signal_handler_block(GTK_TOGGLE_BUTTON(_logy), _signals[3]); 
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_logy), ascale); 
         g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_logy), _signals[3]); 
      }
      if ((eng->get_property(tmp,(void*)&ascale, SET_AXIS_REVERSE)) == 
         ERR_STATUS_GOOD)
      {
         g_signal_handler_block(GTK_TOGGLE_BUTTON(_flipy), _signals[5]); 
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_flipy), ascale); 
         g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_flipy), _signals[5]); 
      }
      delete tmp; 

      coord_update(); 
   }
   g_signal_handler_unblock(GTK_COMBO_BOX(_coordsys), _signals[7]); 
   return 0; 
}

void ChipsGuiNavPane::adjust_limits(int dx, int dy)
{
   if (!chipsGui)  return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng) return;

   static ChipsId cid1, cid2;

   // scale the draw area's pixel delta x to the plot area size
   double xoff = (_pixx2 - _pixx1)/_drawarea->allocation.width;
   double yoff = (_pixy2 - _pixy1)/_drawarea->allocation.height;
   if (xoff < 0) xoff *= -1;
   if (yoff < 0) yoff *= -1;
   xoff *= dx;
   yoff *= dy; 

   // offset the previous pixel positions by the drag deltas
   ChipsVector vin1(_pixx1 - xoff, _pixy1 - yoff), vout1;
   ChipsVector vin2(_pixx2 - xoff,_pixy2 - yoff), vout2;

   // use coord sys to convert from pixel to data coords 
   cid1.set_coord_sys(PIXEL_REAL);
   cid2.set_coord_sys(DATA);
   if (_active > -1 && _active < _idcnt)
   {
      ChipsId* tid = get_active_cid(_active);
      if (tid)  cid2.set_coord_id(tid->get_coord_id());
   }
   eng->convert_coordinate(vin1, vout1, &cid1, &cid2, XY_AXIS);
   eng->convert_coordinate(vin2, vout2, &cid1, &cid2, XY_AXIS);
   // reset limits 
   eng->limits(&cid2, XY_AXIS, vout1[0], vout2[0], vout1[1], vout2[1]);
   update();
}


void ChipsGuiNavPane::autolim_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiNavPane* nav = (ChipsGuiNavPane*) data;

   int dim;
   if (!nav || !chipsGui)  return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng)  return;
   ChipsId cid;

   if (widget == nav->_autox)  dim = X_AXIS;
      else                     dim = Y_AXIS;
   eng->set_ignore_state(true); 
   eng->limits(&cid, dim, AUTO, AUTO);
   eng->set_ignore_state(false); 

   nav->update(); 
}

void ChipsGuiNavPane::log_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiNavPane* nav = (ChipsGuiNavPane*) data;
   int dim, scale; 
   if (!nav || !chipsGui)  return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng)  return;
   ChipsId cid; 

   if (widget == nav->_logx)  dim = X_AXIS;
      else                    dim = Y_AXIS;

   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
      scale = SET_PLOT_LOGSCALE;
   else 
      scale = SET_PLOT_LINEARSCALE;
 
   eng->set_ignore_state(true); 
   if (eng->set_property(&cid, static_cast<void*>(&dim), scale) !=
       ERR_STATUS_GOOD)  chipsGui->log_error();
   eng->set_ignore_state(false); 

   nav->update();
}

void ChipsGuiNavPane::flip_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiNavPane* nav = (ChipsGuiNavPane*) data;
   int dim;
   if (!nav || !chipsGui)  return;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng)  return;
   ChipsId cid;

   if (widget == nav->_flipx)  dim = X_AXIS;
      else                     dim = Y_AXIS;

   eng->set_ignore_state(true); 
   eng->set_property(&cid, static_cast<void*>(&dim), SET_AXIS_REVERSE);
   eng->set_ignore_state(false); 

   nav->update();
}

void ChipsGuiNavPane::adjust_limits(ChipsGuiSpinWidget *spin)
{
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng)  return;

   double x1 = 0.0, x2 = 0.0, y1 = 0.0, y2 = 0.0 ;
   int ascale;
   ChipsId tmp;
   tmp.set_axis_dim(X_AXIS);

   eng->get_property(&tmp,(void*)&ascale, SET_AXIS_REVERSE);
   if (ascale)
   {
      x2 = _xmin->get_numeric_value(); 
      x1 = _xmax->get_numeric_value(); 
   }
   else 
   {
      x1 = _xmin->get_numeric_value(); 
      x2 = _xmax->get_numeric_value(); 
   }

   y1 = _ymin->get_numeric_value(); 
   y2 = _ymax->get_numeric_value(); 

   ChipsId cid;
   int dim = get_dim(); 

   eng->set_ignore_state(true); 
   if (dim == XY_AXIS) 
   {
      eng->limits(&cid, XY_AXIS, x1, x2, y1, y2);
   }
   else if (dim == X_AXIS)
   {
      eng->limits(&cid, dim, x1, x2);
   }
   else if (dim == Y_AXIS)
      eng->limits(&cid, dim, y1, y2);

   eng->set_ignore_state(false); 

   // now that limits have been adjusted, make sure that our
   // current step size won't cause limits to equal zero
   if (spin)
   { 
      double step = spin->get_stepsize();
      if (spin->get_type() == 0)
      {  
         if (dbl_equal((x2 - x1), step)) reset_spin_step(spin); 
      }
      else 
      {
         if ((y2 - y1) == step) reset_spin_step(spin);
      }
   }

   update();
}


void ChipsGuiNavPane::spin_changed_cb(ChipsGuiSpinWidget *spin,
   gpointer data)
{
   ChipsGuiNavPane* nav = (ChipsGuiNavPane*) data;
   if (nav)  nav->adjust_limits(spin);
}

void ChipsGuiNavPane::spin_change_step_cb(ChipsGuiSpinWidget *spin,
   gpointer data)
{
   ChipsGuiNavPane* nav = (ChipsGuiNavPane*) data;
   if (nav) nav->reset_spin_step(spin);
}

void ChipsGuiNavPane::combo_changed_cb(GtkWidget *widget, gpointer data)
{
   ChipsGuiNavPane* nav = (ChipsGuiNavPane*) data;
   int   snum = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));

   nav->update_coord_sys(snum); 
}

// event observer to handle mode change from event manager
void ChipsGuiNavPane::execute_event (int event, void *data)
{
   // limits have changed - reset display 
   update(); 
}

void ChipsGuiNavPane::update_coord_sys(int val)
{
   if (val < 0 || val >= _idcnt)  return; 

   COORD_ELEM_P_T elem_p = get_active_elem(val); 

   if (elem_p) 
   {
      ChipsId* myid = &elem_p->id; 
      _active = val;
   
      // need to update currency in server so subsequent access such
      // as zoom operates on the correct coord sys- for now do not add
      // the change to the undo/redo stack 
      ChipsId *tmpid = new ChipsId(*myid);
      ChipsEngine* eng = chipsGui->get_engine();
      string xid = myid->get_xaxis_id(); 
      tmpid->set_axis_id(xid); 
      if (eng)  eng->set_current(tmpid, chips_axis,false); 
      xid = myid->get_yaxis_id();
      tmpid->set_axis_id(xid); 
      if (eng)  eng->set_current(tmpid, chips_axis, false); 

      // propogate selection
      if (_idcnt > 1) 
      {
         ChipsEventObjectInfo *sel=new ChipsEventObjectInfo(*tmpid,chips_plot);
         double evtpos[2] = {0.0,0.0};
         event_manager->invoke_event(evtpos, ChipsEventManager::SelectEvent,
               (void*)sel, this);
      }
      delete tmpid; 
      update(false);
   }
}

void ChipsGuiNavPane::set_axis_sensitivity(int dim, bool setting)
{
   setting = setting && _enabled; 
   if (dim & X_AXIS)
   {
      gtk_widget_set_sensitive(GTK_WIDGET(_xmin->base_widget()),  setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_xmax->base_widget()),  setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_logx),  setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_flipx), setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_autox), setting && !_nodata);
   }
   if (dim & Y_AXIS)
   {
      gtk_widget_set_sensitive(GTK_WIDGET(_ymin->base_widget()),  setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_ymax->base_widget()),  setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_logy),  setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_flipy), setting);
      gtk_widget_set_sensitive(GTK_WIDGET(_autoy), setting && !_nodata);
   }
}

void ChipsGuiNavPane::enable()
{
   gtk_widget_set_sensitive(GTK_WIDGET(_coordsys), (_enabled = true)); 
   set_axis_sensitivity(XY_AXIS); 
}

void ChipsGuiNavPane::disable()
{
   gtk_widget_set_sensitive(GTK_WIDGET(_coordsys), (_enabled = false)); 
   set_axis_sensitivity(XY_AXIS); 
   gtk_label_set_text(GTK_LABEL(_label[3]),"X undefined");
   gtk_label_set_text(GTK_LABEL(_label[4]),"Y undefined");
}

ChipsId* ChipsGuiNavPane::get_active_cid(int val)
{
   ChipsId* rv = NULL; 
   if (val == -1)  val = _active; 
   for(cid_map::iterator ii=_cids.begin(); ii!=_cids.end(); ++ii)
   {
      if (((COORD_ELEM_P_T)((*ii).second))->slot == val) 
         rv = &(((COORD_ELEM_P_T)((*ii).second))->id); 
   }
   return rv; 
}

COORD_ELEM_P_T ChipsGuiNavPane::get_active_elem(int val)
{
   COORD_ELEM_P_T rv = NULL;
   if (val == -1)  val = _active;
   for(cid_map::iterator ii=_cids.begin(); ii!=_cids.end(); ++ii)
   {
      if (((COORD_ELEM_P_T)((*ii).second))->slot == val) 
         rv = (*ii).second;
   }
   return rv;
}

int ChipsGuiNavPane::get_dim()
{
   int rv; 
   for(cid_map::iterator ii=_cids.begin(); ii!=_cids.end(); ++ii)
   {
      if (((COORD_ELEM_P_T)((*ii).second))->slot == _active)
         rv = ((COORD_ELEM_P_T)((*ii).second))->dim;
   }
   return rv; 
}

void ChipsGuiNavPane::reset_spin_step(ChipsGuiSpinWidget *spin)
{
   double mn, mx; 
   if (spin == _xmin || spin == _xmax)
   {
      mn = _xmin->get_numeric_value(); 
      mx = _xmax->get_numeric_value(); 
   }
   else 
   {
      mn = _ymin->get_numeric_value(); 
      mx = _ymax->get_numeric_value(); 
   }
   double outmin, outmax, dist, base;

   scale1(mn, mx, 25, &outmin, &outmax, &dist, &base);

   // if our dist becomes zero- spin stops working 
   if (dist == 0) 
   {
      if (mx != 0) 
      {
         dist = mx/100.0; 
      }
      else 
      {
         return;
      }
   }
//   if (dist > 1.0e-14)  spin->set_stepsize(dist);
   if (dist > 1.0e-16)  spin->set_stepsize(dist);
}

void ChipsGuiNavPane::scale1(double inmin, double inmax, int slices,
   double* outmin, double* outmax, double* dist, double *base)
{
   double interval, scaled, fm, baseval;
   int nal, m;

   if (inmax == inmin)  ++inmax;
   if (!slices) slices = 5;     // avoid div by 0- use 5 as desired num ticks

   // calculate the interval scale
   interval = (inmax-inmin)/(double)slices;
   nal = (int) log10(interval);
   if (interval < 1) --nal;

   // scale interval between 1-10 and find value base on the geometric mean
   // if the interval is greater than the smallest degrees use
   // degree interval calculations -- that is intervals greater
   // than 15 should use nice degree intervals
   if (interval <= 15)
   {
      if ((scaled = interval/pow(10.0, nal)) < 1.41421356237) baseval = 1.0;
         else if (scaled < 3.16227766017)                     baseval = 2.0;
            else  if (scaled < 7.07106781187)                 baseval = 5.0;
               else                                           baseval = 10;
   }
   else
  {
      if ((scaled = interval/pow(10.0, nal)) < sqrt(1.5*3)) baseval = 1.5;
         else if (scaled < sqrt(1.5*3.0*4.5))               baseval = 3.0;
            else  if (scaled < sqrt(1.5*3.0*4.5*6))         baseval = 4.5;
               else if (scaled < sqrt(1.5*3.0*4.5*6*9))     baseval = 6.0;
                  else                                      baseval = 9.0;
   }
   if (base)  *base = baseval;
   if ((*dist = copysign(baseval * pow(10.0, nal), (inmax - inmin))) != 0)
   {
      // compute interval size
      fm = inmin/(*dist);
      if (fm < 0.0)   m = (int)fm - 1;
         else   m = (int)fm;
      if (fabs((double)m+1.0-fm) < 0.0000000000000001)  ++m;

      // find new minimum and maximum
      *outmin = (*dist) * (double)m;
      fm = inmax/(*dist);
      if (fm < -1.0)  m = (int)fm;
         else m = (int)fm + 1;
      if (fabs(fm+1.0-(double)m) < 0.0000000000000001)  --m;
      *outmax = (*dist) * (double)m;

      // adjust for roundoff
      if (*outmin > inmin)   *outmin = inmin;
      if (*outmax < inmax)   *outmax = inmax;
   }
   else
   {
      *outmin = inmin;
      *outmax = inmax;
   }
}


