// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUINAVPANE_HH
#define CHIPSGUINAVPANE_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <map>

#include "ChipsId.hh"
#include "ChipsEventObserver.hh"
#include "ChipsGuiTooltipObject.hh"

class ChipsGuiSpinWidget;

typedef struct coord_elem_t
{
   int        slot; 
   int        dim; 
   string     title;
   ChipsId    id; 
} COORD_ELEM_T, *COORD_ELEM_P_T;

typedef map <string, COORD_ELEM_P_T>cid_map;


using namespace std;

class ChipsId;

//---------------------------------------------------------------------------

class ChipsGuiNavPane: public ChipsEventObserver, public ChipsGuiTooltipObject
{
public:

   ChipsGuiNavPane();
   ~ChipsGuiNavPane();

   GtkWidget* base_widget() { return _frame; } 

   void show();
   void hide();
   int  get_height(); 
   int  get_dim();

   void enable(); 
   void disable(); 
   void set_axis_sensitivity(int dim, bool setting=true); 

   void coord_update(bool reset=false);
   void adjust_limits(int dx, int dy);

   int update(bool reset=true); 
   void update_coord_sys(int val); 
   void adjust_limits(ChipsGuiSpinWidget *spin=NULL); 
   void reset_spin_step(ChipsGuiSpinWidget *spin);
   ChipsId* get_active_cid(int val=-1); 
   COORD_ELEM_P_T get_active_elem(int val =-1); 

   static void autolim_callback(GtkWidget *widget, gpointer data);
   static void log_callback(GtkWidget *widget, gpointer data);
   static void flip_callback(GtkWidget *widget, gpointer data);
   static void spin_changed_cb(ChipsGuiSpinWidget *spin, gpointer data);
   static void spin_change_step_cb(ChipsGuiSpinWidget *spin, gpointer data);
   static void da_motion_cb(GtkWidget *widget, GdkEventMotion *event, 
                                  gpointer data);
   static void da_scroll_cb(GtkWidget *widget, GdkEventScroll *event, 
                                  gpointer data);
   static void da_button_cb(GtkWidget *widget, GdkEventButton *event, 
                                  gpointer data);
   static void combo_changed_cb(GtkWidget *widget, gpointer data);

   virtual void execute_event (int event, void *data);
   void scale1(double inmin, double inmax, int slices,
      double* outmin, double* outmax, double* dist, double *base); 

private:
  
   GtkWidget*   _frame; 
   GtkWidget*   _table;
   GtkWidget*   _asp;
   GtkWidget*   _drawarea; 
   GtkWidget*   _coordsys;
   ChipsGuiSpinWidget* _xmin;
   ChipsGuiSpinWidget* _xmax;
   ChipsGuiSpinWidget* _ymin;
   ChipsGuiSpinWidget* _ymax;

   GtkWidget*   _logx;
   GtkWidget*   _logy;
   GtkWidget*   _flipx;
   GtkWidget*   _flipy;
   GtkWidget*   _autox;
   GtkWidget*   _autoy;
   GtkWidget*   _label[5];

   cid_map      _cids;
   ChipsId      _cid[10]; 
   int          _idcnt; 
   int          _signals[8]; 
   bool         _nodata; 
   bool         _enabled;

   double       _rngx1, _rngx2, _rngy1, _rngy2, _rngxlog, _rngylog;
   double       _limx1, _limx2, _limy1, _limy2; 
   double       _pixx1, _pixx2, _pixy1, _pixy2;
   int          _objCnt; 
   int          _active;
   string       _activeTxt;  
   string       _prev; 

   static void da_expose_cb(GtkWidget* widget, GdkEventExpose *event, 
                            gpointer user_data);

   static int   _prevx;
   static int   _prevy; 

};

#endif
