#include "ChipsGuiPivotPointWidget.hh"

#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsId.hh"


#include <iostream>

extern ChipsGuiMain* chipsGui;


ChipsGuiPivotPointWidgetCallback::ChipsGuiPivotPointWidgetCallback (void (*callback)(ChipsGuiPivotPointWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;

}

void ChipsGuiPivotPointWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiPivotPointWidget *pivotWidget = dynamic_cast<ChipsGuiPivotPointWidget *>(caller);
   _callback (pivotWidget, _callbackData);
}



ChipsGuiPivotPointWidget::ChipsGuiPivotPointWidget ()
{
   _valid_callback_ids.push_back ("change_pivot");
   _valid_callback_ids.push_back ("enable_xauto");
   _valid_callback_ids.push_back ("enable_yauto");

   _xpos_val = 0;
   _ypos_val = 0;

   _xauto = NULL;
   _yauto = NULL;
}

ChipsGuiPivotPointWidget::~ChipsGuiPivotPointWidget ()
{
}

void ChipsGuiPivotPointWidget::build_form ()
{
   this->build_form (true);
}

void ChipsGuiPivotPointWidget::build_form (bool include_autos)
{
   GtkWidget *vbox;
   GtkWidget *label;
   _base_widget = vbox = gtk_vbox_new (FALSE, 3);

   
   label = gtk_label_new("Base Point");
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE,0);

   // setup privot frame
   {
      GtkWidget *frame; 
      GtkWidget *table;
      GtkWidget *seperator;

      frame = gtk_frame_new ("");
      gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);

      table = gtk_table_new(5,5, FALSE);
      gtk_container_add(GTK_CONTAINER(frame), table);

      _upper_left = gtk_radio_button_new(NULL);
      _upper_center = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _upper_right = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _lower_left = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _lower_center = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _lower_right = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _center = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _custom = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _center_left = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));
      _center_right = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(_upper_left));



      gtk_table_attach (GTK_TABLE(table), _upper_left, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _upper_center, 2, 3, 0, 1, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _upper_right, 4, 5, 0, 1, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _lower_left, 0, 1, 4, 5, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _lower_center, 2, 3, 4, 5, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _lower_right, 4, 5, 4, 5, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _center, 2, 3, 2, 3, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _center_left, 0, 1, 2, 3, GTK_SHRINK, GTK_SHRINK, 5,5);
      gtk_table_attach (GTK_TABLE(table), _center_right, 4, 5, 2, 3, GTK_SHRINK, GTK_SHRINK, 5,5);

      gtk_table_attach (GTK_TABLE(table), _custom, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 5,5);
      seperator = gtk_hseparator_new ();
      gtk_table_attach (GTK_TABLE(table), seperator, 1,2,0,1, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);
      seperator = gtk_hseparator_new ();
      gtk_table_attach (GTK_TABLE(table), seperator, 3,4,0,1, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);
      seperator = gtk_hseparator_new ();
      gtk_table_attach (GTK_TABLE(table), seperator, 1,2,4,5, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);
      seperator = gtk_hseparator_new ();
      gtk_table_attach (GTK_TABLE(table), seperator, 3,4,4,5, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);
      seperator = gtk_vseparator_new ();
      gtk_table_attach (GTK_TABLE(table), seperator, 0,1,1,2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);
      seperator = gtk_vseparator_new ();
      gtk_table_attach (GTK_TABLE(table),seperator, 4,5,1,2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);
      seperator = gtk_vseparator_new ();
      gtk_table_attach (GTK_TABLE(table), seperator, 0,1,3,4, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);
      seperator = gtk_vseparator_new ();
      gtk_table_attach (GTK_TABLE(table),seperator, 4,5,3,4, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 5,5);


      gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);

      _upper_left_toggle_id = g_signal_connect(G_OBJECT(_upper_left), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _upper_center_toggle_id = g_signal_connect(G_OBJECT(_upper_center), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _upper_right_toggle_id = g_signal_connect(G_OBJECT(_upper_right), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _lower_left_toggle_id = g_signal_connect(G_OBJECT(_lower_left), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _lower_center_toggle_id = g_signal_connect(G_OBJECT(_lower_center), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _lower_right_toggle_id = g_signal_connect(G_OBJECT(_lower_right), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _center_toggle_id = g_signal_connect(G_OBJECT(_center), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _center_left_toggle_id = g_signal_connect(G_OBJECT(_center_left), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);
      _center_right_toggle_id = g_signal_connect(G_OBJECT(_center_right), "toggled", G_CALLBACK(default_position_change_callback), (void *)this);



   }

   // custom spinners
   {
      GtkWidget *hbox1;
      GtkWidget *hbox2;
     
      _xpos = gtk_spin_button_new_with_range (-9.9, 9.9, .1);
      _ypos = gtk_spin_button_new_with_range (-9.9, 9.9, .1);

      hbox1 = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(hbox1), _xpos, TRUE, FALSE,0);
      gtk_box_pack_start(GTK_BOX(hbox1), _ypos, TRUE, FALSE,0);

      gtk_box_pack_start(GTK_BOX(vbox), hbox1, TRUE, TRUE,5);

      _xpos_change_id = g_signal_connect(G_OBJECT(_xpos), "value-changed", G_CALLBACK(custom_position_change_callback), (void *)this);
      _ypos_change_id = g_signal_connect(G_OBJECT(_ypos), "value-changed", G_CALLBACK(custom_position_change_callback), (void *)this);


      gtk_widget_hide (_custom);

      if (include_autos)
      {
         _xauto = gtk_check_button_new_with_label ("X Auto");
         _yauto = gtk_check_button_new_with_label ("Y Auto");

         hbox2 = gtk_hbox_new (FALSE, 2);
         gtk_box_pack_start(GTK_BOX(hbox2), _xauto, TRUE, FALSE,0);
         gtk_box_pack_start(GTK_BOX(hbox2), _yauto, TRUE, FALSE,0);


         gtk_box_pack_start(GTK_BOX(vbox), hbox2, TRUE, TRUE,5);

         _xauto_toggle_id = g_signal_connect(G_OBJECT(_xauto), "toggled", G_CALLBACK(xauto_position_click_callback), (void *)this);
         _yauto_toggle_id = g_signal_connect(G_OBJECT(_yauto), "toggled", G_CALLBACK(yauto_position_click_callback), (void *)this);

      }
   }
}

void ChipsGuiPivotPointWidget::show ()
{
   gtk_widget_show_all (_base_widget);
   gtk_widget_hide (_custom);
}

void ChipsGuiPivotPointWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}


void ChipsGuiPivotPointWidget::default_position_change_callback(GtkWidget *widget, gpointer *data)
{ 
   ChipsGuiPivotPointWidget *pivotWidget = (ChipsGuiPivotPointWidget *)data;
   float xval = .5;
   float yval = .5;

   if (widget == pivotWidget->_lower_left)
   {
      xval = 0;
      yval = 0;
   }
   else if (widget == pivotWidget->_lower_center)
   {
      xval = .5;
      yval = 0;
   }
   else if (widget == pivotWidget->_lower_right)
   {
      xval = 1;
      yval = 0;
   }
   else if (widget == pivotWidget->_upper_left)
   {
      xval = 0;
      yval = 1;
   }
   else if (widget == pivotWidget->_upper_center)
   {
      xval = .5;
      yval = 1;
   }
   else if (widget == pivotWidget->_upper_right)
   {
      xval = 1;
      yval = 1;
   }
   else if (widget == pivotWidget->_center_left)
   {
      xval = 0;
      yval = .5;
   }
   else if (widget == pivotWidget->_center_right)
   {
      xval = 1;
      yval = .5;
   }



   if (flt_equal (xval,pivotWidget->_xpos_val) && flt_equal (yval,pivotWidget->_ypos_val))
      return;
   
   pivotWidget->_xpos_val = xval;
   pivotWidget->_ypos_val = yval;

   pivotWidget->set_pivot_position_widgets (xval, yval);

   if (pivotWidget->_xauto)
   {
      g_signal_handler_block(GTK_OBJECT(pivotWidget->_xauto), pivotWidget->_xauto_toggle_id);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pivotWidget->_xauto), FALSE);
      g_signal_handler_unblock(GTK_OBJECT(pivotWidget->_xauto), pivotWidget->_xauto_toggle_id);
   }

   if (pivotWidget->_yauto)
   {
      g_signal_handler_block(GTK_OBJECT(pivotWidget->_yauto), pivotWidget->_yauto_toggle_id);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pivotWidget->_yauto), FALSE);
      g_signal_handler_unblock(GTK_OBJECT(pivotWidget->_yauto), pivotWidget->_yauto_toggle_id);
   }


   if (pivotWidget->_callbacks.count ("change_pivot") > 0)
      pivotWidget->_callbacks["change_pivot"]->execute (pivotWidget, NULL);

}

void ChipsGuiPivotPointWidget::custom_position_change_callback(GtkWidget *widget, gpointer *data)
{
   ChipsGuiPivotPointWidget *pivotWidget = (ChipsGuiPivotPointWidget *)data;

   float xval;
   float yval;

   xval = gtk_spin_button_get_value (GTK_SPIN_BUTTON(pivotWidget->_xpos));
   yval = gtk_spin_button_get_value (GTK_SPIN_BUTTON(pivotWidget->_ypos));

   if (flt_equal (xval,pivotWidget->_xpos_val) && flt_equal (yval,pivotWidget->_ypos_val))
      return;

   pivotWidget->_xpos_val = xval;
   pivotWidget->_ypos_val = yval;

   pivotWidget->set_pivot_position_widgets (xval, yval);

   if (pivotWidget->_xauto && (widget == pivotWidget->_xpos))
   {
      g_signal_handler_block(GTK_OBJECT(pivotWidget->_xauto), pivotWidget->_xauto_toggle_id);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pivotWidget->_xauto), FALSE);
      g_signal_handler_unblock(GTK_OBJECT(pivotWidget->_xauto), pivotWidget->_xauto_toggle_id);
   }

   if (pivotWidget->_yauto && (widget == pivotWidget->_ypos))
   {
      g_signal_handler_block(GTK_OBJECT(pivotWidget->_yauto), pivotWidget->_yauto_toggle_id);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pivotWidget->_yauto), FALSE);
      g_signal_handler_unblock(GTK_OBJECT(pivotWidget->_yauto), pivotWidget->_yauto_toggle_id);
   }


   if (pivotWidget->_callbacks.count ("change_pivot") > 0)
      pivotWidget->_callbacks["change_pivot"]->execute (pivotWidget, NULL);

}

void ChipsGuiPivotPointWidget::xauto_position_click_callback(GtkWidget *widget, gpointer *data)
{
   ChipsGuiPivotPointWidget *pivotWidget = (ChipsGuiPivotPointWidget *)data;

   bool active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));

   if (active && (pivotWidget->_callbacks.count ("enable_xauto") > 0))
      pivotWidget->_callbacks["enable_xauto"]->execute (pivotWidget, NULL);
}

void ChipsGuiPivotPointWidget::yauto_position_click_callback(GtkWidget *widget, gpointer *data)
{
   ChipsGuiPivotPointWidget *pivotWidget = (ChipsGuiPivotPointWidget *)data;

   bool active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));

   if (active && (pivotWidget->_callbacks.count ("enable_yauto") > 0))
      pivotWidget->_callbacks["enable_yauto"]->execute (pivotWidget, NULL);
}


void ChipsGuiPivotPointWidget::set_pivot_position (float x, float y)
{

   _xpos_val = x;
   _ypos_val = y;

   this->set_pivot_position_widgets(x,y);
}

void ChipsGuiPivotPointWidget::get_pivot_position (float &x, float &y)
{
   x = _xpos_val;
   y = _ypos_val;

}

void ChipsGuiPivotPointWidget::set_auto_x (bool enable)
{
   g_signal_handler_block(GTK_OBJECT(_xauto), _xauto_toggle_id);
   if (enable)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_xauto), TRUE);
   else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_xauto), FALSE);
   g_signal_handler_unblock(GTK_OBJECT(_xauto), _xauto_toggle_id);
}

void ChipsGuiPivotPointWidget::set_auto_y (bool enable)
{
   g_signal_handler_block(GTK_OBJECT(_yauto), _yauto_toggle_id);
   if (enable)
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_yauto), TRUE);
   else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_yauto), FALSE);
   g_signal_handler_unblock(GTK_OBJECT(_yauto), _yauto_toggle_id);
}

bool ChipsGuiPivotPointWidget::get_auto_x ()
{
   return (bool)gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (_xauto));
}

bool ChipsGuiPivotPointWidget::get_auto_y ()
{
   return (bool)gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (_yauto));
}


   
void ChipsGuiPivotPointWidget::set_pivot_position_widgets (float xval, float yval)
{
   g_signal_handler_block(GTK_OBJECT(_upper_left), _upper_left_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_upper_center), _upper_center_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_upper_right), _upper_right_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_lower_left), _lower_left_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_lower_center), _lower_center_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_lower_right), _lower_right_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_center), _center_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_center_left), _center_left_toggle_id);
   g_signal_handler_block(GTK_OBJECT(_center_right), _center_right_toggle_id);

   g_signal_handler_block(GTK_OBJECT(_xpos), _xpos_change_id);
   g_signal_handler_block(GTK_OBJECT(_ypos), _ypos_change_id);

   if (flt_equal (yval, 0.0))
   {
      if (flt_equal (xval, 0.0))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_lower_left), TRUE);
      else if (flt_equal (xval, 0.5))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_lower_center), TRUE);
      else if (flt_equal (xval, 1.0))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_lower_right), TRUE);
      else
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_custom), TRUE);
   }
   else if (flt_equal (yval,1.0))
   {
      if (flt_equal (xval, 0.0))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_upper_left), TRUE);
      else if (flt_equal (xval, .5))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_upper_center), TRUE);
      else if (flt_equal (xval,1.0))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_upper_right), TRUE);
      else
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_custom), TRUE);
   }
   else if (flt_equal (yval, .5))
   {
      if (flt_equal (xval, 0.0))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_center_left), TRUE);
      else if (flt_equal (xval, .5))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_center), TRUE);
      else if (flt_equal (xval,1.0))
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_center_right), TRUE);
      else
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_custom), TRUE);
   }
   else
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (_custom), TRUE);

   gtk_spin_button_set_value (GTK_SPIN_BUTTON(_xpos),xval);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON(_ypos),yval);

   g_signal_handler_unblock(GTK_OBJECT(_upper_left), _upper_left_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_upper_center), _upper_center_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_upper_right), _upper_right_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_lower_left), _lower_left_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_lower_center), _lower_center_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_lower_right), _lower_right_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_center), _center_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_center_left), _center_left_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_center_right), _center_right_toggle_id);
   g_signal_handler_unblock(GTK_OBJECT(_xpos), _xpos_change_id);
   g_signal_handler_unblock(GTK_OBJECT(_ypos), _ypos_change_id);



}

void ChipsGuiPivotPointWidget::enable ()
{
   gtk_widget_set_sensitive (_upper_left, TRUE);
   gtk_widget_set_sensitive (_upper_center, TRUE);
   gtk_widget_set_sensitive (_upper_right, TRUE);
   gtk_widget_set_sensitive (_lower_left, TRUE);
   gtk_widget_set_sensitive (_lower_center, TRUE);
   gtk_widget_set_sensitive (_lower_right, TRUE);
   gtk_widget_set_sensitive (_center, TRUE);
   gtk_widget_set_sensitive (_center_left, TRUE);
   gtk_widget_set_sensitive (_center_right, TRUE);
   gtk_widget_set_sensitive (_custom, TRUE);
   gtk_widget_set_sensitive (_xpos, TRUE);
   gtk_widget_set_sensitive (_ypos, TRUE);
   gtk_widget_set_sensitive (_xauto, TRUE);
   gtk_widget_set_sensitive (_yauto, TRUE);
}

void ChipsGuiPivotPointWidget::disable ()
{
   gtk_widget_set_sensitive (_upper_left, FALSE);
   gtk_widget_set_sensitive (_upper_center, FALSE);
   gtk_widget_set_sensitive (_upper_right, FALSE);
   gtk_widget_set_sensitive (_lower_left, FALSE);
   gtk_widget_set_sensitive (_lower_center, FALSE);
   gtk_widget_set_sensitive (_lower_right, FALSE);
   gtk_widget_set_sensitive (_center, FALSE);
   gtk_widget_set_sensitive (_center_left, FALSE);
   gtk_widget_set_sensitive (_center_right, FALSE);
   gtk_widget_set_sensitive (_custom, FALSE);
   gtk_widget_set_sensitive (_xpos, FALSE);
   gtk_widget_set_sensitive (_ypos, FALSE);
   gtk_widget_set_sensitive (_xauto, FALSE);
   gtk_widget_set_sensitive (_yauto, FALSE);
}



   
