// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIPIVOTPOINTWIDGET_HH
#define CHIPSGUIPIVOTPOINTWIDGET_HH

#include <gtk/gtk.h>
#include <string>
#include <map>
#include <list>

#include "ChipsGuiWidget.hh"

using namespace std;

// valid callback ids are
//  * change_pivot
//  * enable_xauto
//  * enable_yauto

class ChipsGuiPivotPointWidget;

class ChipsGuiPivotPointWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiPivotPointWidgetCallback (void (*callback)(ChipsGuiPivotPointWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiPivotPointWidget *, void *);
     void *_callbackData;     
};

class ChipsGuiPivotPointWidget : public ChipsGuiWidget
{
   public:

      ChipsGuiPivotPointWidget ();
      ~ChipsGuiPivotPointWidget ();

      virtual void build_form ();

      void build_form (bool include_auto);

      void set_pivot_position (float x, float y);
      void get_pivot_position (float &x, float &y);

      void set_auto_x (bool enable);
      void set_auto_y (bool enable);
      bool get_auto_x ();
      bool get_auto_y ();

      void enable ();
      void disable ();

      virtual void show ();
      virtual void hide ();

     
   protected:

      static void default_position_change_callback(GtkWidget *widget, gpointer *data);
      static void custom_position_change_callback(GtkWidget *widget, gpointer *data);
      static void xauto_position_click_callback(GtkWidget *widget, gpointer *data);

      static void yauto_position_click_callback(GtkWidget *widget, gpointer *data);
      void set_pivot_position_widgets (float xval, float yval);

      GtkWidget *_upper_left;
      GtkWidget *_upper_center;
      GtkWidget *_upper_right;
      GtkWidget *_lower_left;
      GtkWidget *_lower_center;
      GtkWidget *_lower_right;
      GtkWidget *_center;
      GtkWidget *_center_left;
      GtkWidget *_center_right;
      GtkWidget *_custom;
      GtkWidget *_xpos;
      GtkWidget *_ypos;
      GtkWidget *_xauto;
      GtkWidget *_yauto;

      unsigned int _upper_left_toggle_id;
      unsigned int _upper_center_toggle_id;
      unsigned int _upper_right_toggle_id;
      unsigned int _lower_left_toggle_id;
      unsigned int _lower_center_toggle_id;
      unsigned int _lower_right_toggle_id;
      unsigned int _center_toggle_id;
      unsigned int _center_left_toggle_id;
      unsigned int _center_right_toggle_id;
      unsigned int _xpos_change_id;
      unsigned int _ypos_change_id;
      unsigned int _xauto_toggle_id;
      unsigned int _yauto_toggle_id;

      float _xpos_val;
      float _ypos_val;

};

#endif

