// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiPlotFrame.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsPlot.hh"
#include "ChipsGuiLabelWidget.hh"
#include "ChipsCmdManager.hh"

using namespace std; 

extern ChipsGuiMain* chipsGui;
extern ChipsCmdManager* cmd_manager;


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiPlotFrame::ChipsGuiPlotFrame():ChipsGuiPropFrame()
{
   _title = NULL;
   _object_states = STATE_CHNG_PROP_PLT;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiPlotFrame::~ChipsGuiPlotFrame()
{
   if (_title)
      delete _title;
}

void ChipsGuiPlotFrame::build_form()
{
   GtkWidget* tmp; 
   GtkWidget *label = gtk_label_new (NULL);
   gtk_label_set_markup (GTK_LABEL(label), "Plot Properties");
   _frame = gtk_frame_new(NULL);
   gtk_frame_set_label_widget (GTK_FRAME(_frame), label);
   _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(_frame), _nbook);

   _table1 = gtk_table_new(7,3,FALSE);
   _tabGen = gtk_table_new(7,5,FALSE);
   _tabMgn = gtk_table_new(6,2,TRUE);
   _tabAsp = gtk_table_new(6,2,FALSE);
   _frmGen = gtk_frame_new("Style");
   gtk_frame_set_label_align(GTK_FRAME(_frmGen), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmGen), _tabGen);
   gtk_table_attach(GTK_TABLE(_table1), _frmGen, 0, 7, 0 , 1,
      GTK_EXPAND, GTK_SHRINK, 10, 10);
   _frmMgn = gtk_frame_new("Margins");
   gtk_frame_set_label_align(GTK_FRAME(_frmMgn), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmMgn), _tabMgn);
   gtk_table_attach(GTK_TABLE(_table1), _frmMgn, 0, 7, 1, 2,
      GTK_FILL, GTK_SHRINK, 5, 5);

   tmp = gtk_label_new("General");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table1, tmp);

   // style frame
   _style = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "open");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "boxed");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "closed");
   _corner = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_corner), "miter");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_corner), "bevel");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_corner), "filled");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_corner), "notch");
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_tabGen), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabGen), _style, 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Corner");
   gtk_table_attach(GTK_TABLE(_tabGen), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabGen), _corner, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   // margin frame 
   _topmgn = gtk_hscale_new_with_range(0,.99,.01);
   gtk_scale_set_digits(GTK_SCALE(_topmgn), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_topmgn), true);
   _bottommgn = gtk_hscale_new_with_range(0,.99,.01);
   gtk_scale_set_digits(GTK_SCALE(_bottommgn), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_bottommgn), true);
   _leftmgn = gtk_hscale_new_with_range(0,.99,.01);
   gtk_scale_set_digits(GTK_SCALE(_leftmgn), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_leftmgn), true);
   _rightmgn = gtk_hscale_new_with_range(0,.99,.01);
   gtk_scale_set_digits(GTK_SCALE(_rightmgn), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_rightmgn), true);

   tmp = gtk_label_new("Top");
   gtk_table_attach(GTK_TABLE(_tabMgn), tmp, 0, 1, 0, 1,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabMgn), _topmgn, 1 ,3, 0, 1,
      GTK_FILL, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Bottom");
   gtk_table_attach(GTK_TABLE(_tabMgn), tmp, 3, 4, 0, 1,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabMgn), _bottommgn, 4, 6, 0, 1,
      GTK_FILL, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Left");
   gtk_table_attach(GTK_TABLE(_tabMgn), tmp, 0, 1, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabMgn), _leftmgn, 1, 3, 1, 2,
      GTK_FILL, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Right");
   gtk_table_attach(GTK_TABLE(_tabMgn), tmp, 3, 4, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabMgn), _rightmgn, 4, 6, 1, 2,
      GTK_FILL, GTK_SHRINK, 5, 5);

  // aspect frame 
   _table2 = gtk_table_new(7,7,FALSE);
   _frmAsp = gtk_frame_new("Aspect");
   gtk_frame_set_label_align(GTK_FRAME(_frmAsp), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmAsp), _tabAsp);
   gtk_table_attach(GTK_TABLE(_table2), _frmAsp, 0, 7, 2, 3,
      GTK_EXPAND, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("Aspect");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table2, tmp);
   _aspmode = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_aspmode), "None");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_aspmode), "Current Dimensions");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_aspmode), "Fit to Object");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_aspmode), "Custom Ratio");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_aspmode), "1:1");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_aspmode), "1:2");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_aspmode), "2:1");
   _aspw = gtk_spin_button_new_with_range(0.01, 100, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_aspw), 2);
   _asph = gtk_spin_button_new_with_range(0.01, 100, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_asph), 2);
   tmp = gtk_label_new("Mode");
   gtk_table_attach(GTK_TABLE(_tabAsp), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabAsp), _aspmode, 2, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("width");
   gtk_table_attach(GTK_TABLE(_tabAsp), tmp, 1, 2, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabAsp), _aspw, 2, 3, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   tmp = gtk_label_new("height");
   gtk_table_attach(GTK_TABLE(_tabAsp), tmp, 3, 4, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 5, 5);
   gtk_table_attach(GTK_TABLE(_tabAsp), _asph, 4, 5, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 5, 5);

   int ndx = 0; 
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_style), "changed",
      G_CALLBACK(style_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_corner), "changed",
      G_CALLBACK(corner_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_topmgn), "value-changed",
      G_CALLBACK(topmargin_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_bottommgn), "value-changed",
      G_CALLBACK(bottommargin_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_leftmgn), "value-changed",
      G_CALLBACK(leftmargin_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_rightmgn), "value-changed",
      G_CALLBACK(rightmargin_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_aspmode), "changed",
      G_CALLBACK(aspect_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_aspw), "value-changed",
      G_CALLBACK(aspect_callback), (void*)this);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_asph), "value-changed",
      G_CALLBACK(aspect_callback), (void*)this);

   // create title
   double title_xrange[] = {0,1};
   double title_yrange[] = {-9999,9999};
   GtkWidget*   vbox = gtk_vbox_new (FALSE, 3); 
   GtkWidget*   hbox = gtk_hbox_new (FALSE, 3); 
   _title = new ChipsGuiLabelWidget();
   _title->build_form (false);
   _title->set_position_type (ChipsGuiLabelWidget::FLOAT,
                              ChipsGuiLabelWidget::INT);
   _title->set_position_range (title_xrange, .01, title_yrange, 1);

   _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);
   _cb_sigs[ndx++] = g_signal_connect(G_OBJECT(_depth), "value-changed",
      G_CALLBACK(depth_callback), (void*)this);

   tmp = gtk_label_new("Depth");
   gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE,5);
   gtk_box_pack_start(GTK_BOX(hbox), _depth, FALSE, FALSE,5);
   gtk_box_pack_start(GTK_BOX(vbox), _title->base_widget(), FALSE, FALSE,5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,5);
   
   tmp = gtk_label_new("Title");
   _title_page_num = gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), vbox, tmp);

   _title->add_callback ("change_color", new ChipsGuiLabelWidgetCallback (color_change_callback, this));
   _title->add_callback ("change_angle", new ChipsGuiLabelWidgetCallback (angle_change_callback, this));
   _title->add_callback ("change_text", new ChipsGuiLabelWidgetCallback (text_change_callback, this));
   _title->add_callback ("change_fontstyle", new ChipsGuiLabelWidgetCallback (fontstyle_change_callback, this));
   _title->add_callback ("change_font", new ChipsGuiLabelWidgetCallback (font_change_callback, this));
   _title->add_callback ("change_align", new ChipsGuiLabelWidgetCallback (align_change_callback, this));
   _title->add_callback ("change_size", new ChipsGuiLabelWidgetCallback (size_change_callback, this));
   _title->add_callback ("change_position", new ChipsGuiLabelWidgetCallback (position_change_callback, this));


   // add tooltips
   setup_tooltip(_style,"Sets the appearance of the plot borders"); 
   setup_tooltip(_corner,"Sets the drawing style for the plot border corners"); 
   setup_tooltip(_topmgn,"Normalized offset from the frame's top edge to the top of plot"); 
   setup_tooltip(_bottommgn,"Normalized distance from frame's bottom edge to the bottom of plot"); 
   setup_tooltip(_leftmgn,"Normalized offset from the frame's left edge to the left edge of the plot"); 
   setup_tooltip(_rightmgn,"Normalized offset from the frame's right edge to the right edge of the plot"); 
   setup_tooltip(_aspmode,"Controls the plot's shape in terms of aspect ratios"); 
   setup_tooltip(_aspw,"Sets the width used by the 'Custom Ratio' (width:height) setting"); 
   setup_tooltip(_asph,"Sets the height used by the 'Custom Ratio' (width:height) setting"); 
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the plot title is drawn");
}

void ChipsGuiPlotFrame::adjust_margin_ranges(int mgn)
{
   // validate data 
   if (mgn == 0)
   {
      if (_top_v < 0)  _top_v = 0;
      else if (_top_v + _bottom_v > 0.99)  _top_v = 0.99 - _bottom_v;  
   }
   else if (mgn == 1) 
   {
      if (_bottom_v < 0)  _bottom_v = 0;
      else if (_bottom_v + _top_v > 0.99)  _bottom_v = 0.99 - _top_v;
   }
   else if (mgn == 2)
   {
      if (_left_v < 0)  _left_v = 0;
      else if (_left_v + _right_v > 0.99)  _left_v = 0.99 - _right_v;  
   }
   else if (mgn == 3) 
   {
      if (_right_v < 0)  _right_v = 0;
      else if (_right_v + _left_v > 0.99)  _right_v = 0.99 - _left_v;
   }

   if (mgn == 0 || mgn == 1) 
   {
      _topmax = .99 - _bottom_v;
      _bottommax = .99 - _top_v;
      if (_topmax < 0) _topmax = 0.001;    // min < max for gtk_range 
      if (_bottommax < 0) _bottommax = 0.001; // min < max for gtk_range
      g_signal_handler_block(GTK_SCALE(_topmgn), _cb_sigs[2]);
      g_signal_handler_block(GTK_SCALE(_bottommgn), _cb_sigs[3]);
      gtk_range_set_range(GTK_RANGE(_topmgn), 0.0, _topmax); 
      gtk_range_set_range(GTK_RANGE(_bottommgn), 0.0, _bottommax); 
      gtk_range_set_value(GTK_RANGE(_topmgn), _top_v);
      gtk_range_set_value(GTK_RANGE(_bottommgn), _bottom_v);
      g_signal_handler_unblock(GTK_SCALE(_topmgn), _cb_sigs[2]);
      g_signal_handler_unblock(GTK_SCALE(_bottommgn), _cb_sigs[3]);
   }
   else 
   {
      _leftmax = .99 - _right_v;
      _rightmax = .99 - _left_v;   
      if (_leftmax < 0)   _leftmax = 0.001;    // min < max for gtk_range 
      if (_rightmax < 0)  _rightmax = 0.001;  // min < max for gtk_range
      g_signal_handler_block(GTK_SCALE(_leftmgn), _cb_sigs[4]);
      g_signal_handler_block(GTK_SCALE(_rightmgn), _cb_sigs[5]);
      gtk_range_set_range(GTK_RANGE(_leftmgn), 0.0, _leftmax); 
      gtk_range_set_range(GTK_RANGE(_rightmgn), 0.0, _rightmax); 
      gtk_range_set_value(GTK_RANGE(_leftmgn), _left_v);
      gtk_range_set_value(GTK_RANGE(_rightmgn), _right_v);
      g_signal_handler_unblock(GTK_SCALE(_leftmgn), _cb_sigs[4]);
      g_signal_handler_unblock(GTK_SCALE(_rightmgn), _cb_sigs[5]);
   }
}


void ChipsGuiPlotFrame::set_margin(int mgn, float val)
{
   if (!_eng)  return; 
   int key;
   switch(mgn)
   {
      case 0:   key = SET_PLOT_TMARGIN; _top_v = val;     break;
      case 1:   key = SET_PLOT_BMARGIN; _bottom_v = val;  break;
      case 2:   key = SET_PLOT_LMARGIN; _left_v = val;    break;
      case 3:   key = SET_PLOT_RMARGIN; _right_v = val;   break;
      default:  key = -99;                                break;
   }

   if (key == -99)  return;   // invalid margin key 

   set_engine_property(static_cast<void*>(&val), key); 
   adjust_margin_ranges(mgn); 
}

void ChipsGuiPlotFrame::set_style(int val)
{
   set_engine_property(static_cast<void*>(&val),
                                 SET_PLOT_AXIS_STYLE);
}

void ChipsGuiPlotFrame::set_corner(int val)
{
   set_engine_property(static_cast<void*>(&val),
                                 SET_PLOT_CORNER);
}

void ChipsGuiPlotFrame::set_aspect_ratio()
{
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(_aspmode));

   // set aspect width/height spin button sensitive only if 'user' mode
   gtk_widget_set_sensitive(GTK_WIDGET(_aspw),  (val == 3)); 
   gtk_widget_set_sensitive(GTK_WIDGET(_asph),  (val == 3)); 

   if (!_eng)  return; 

   string ratio; 
   switch (val)
   {
      case 0:  ratio = "";         break;      
      case 1:  ratio = "current";  break; 
      case 2:  ratio = "fit";      break; 
      case 4:  ratio = "1:1";      break; 
      case 5:  ratio = "1:2";      break; 
      case 6:  ratio = "2:1";      break; 
      case 3:
      {
         float ww, hh;
         char buf[32]; 
         ww = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_aspw));
         hh = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_asph));
         sprintf(buf,"%2.1f:%2.1f",ww,hh); 
         ratio = buf; 
      }
      break; 
      default: ratio = "";         break;
   }
   set_engine_property((void*)ratio.c_str(), SET_PLOT_ASPECT_RATIO, false);
   _ratio_v = ratio; 
}

void ChipsGuiPlotFrame::set_depth(int val)
{
   set_engine_property (static_cast<void*>(&val), SET_PLOT_DEPTH);
}

void ChipsGuiPlotFrame::style_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *crvFrame = static_cast <ChipsGuiPlotFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (crvFrame)  crvFrame->set_style(val); 
}

void ChipsGuiPlotFrame::corner_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *crvFrame = static_cast <ChipsGuiPlotFrame*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (crvFrame)  crvFrame->set_corner(val);
}

int ChipsGuiPlotFrame::topmargin_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *crvFrame = static_cast <ChipsGuiPlotFrame*>(data);
   float val = gtk_range_get_value(GTK_RANGE(widget));
   if (crvFrame)  crvFrame->set_margin(0, val);
   return 0; 
}

void ChipsGuiPlotFrame::bottommargin_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *crvFrame = static_cast <ChipsGuiPlotFrame*>(data);
   float val = gtk_range_get_value(GTK_RANGE(widget));
   if (crvFrame)  crvFrame->set_margin(1, val);
}

void ChipsGuiPlotFrame::leftmargin_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *crvFrame = static_cast <ChipsGuiPlotFrame*>(data);
   float val = gtk_range_get_value(GTK_RANGE(widget));
   if (crvFrame)  crvFrame->set_margin(2, val);
}

void ChipsGuiPlotFrame::rightmargin_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *crvFrame = static_cast <ChipsGuiPlotFrame*>(data);
   float val = gtk_range_get_value(GTK_RANGE(widget));
   if (crvFrame)  crvFrame->set_margin(3, val);
}

void ChipsGuiPlotFrame::aspect_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *crvFrame = static_cast <ChipsGuiPlotFrame*>(data);
   if (crvFrame)  crvFrame->set_aspect_ratio();
}

void ChipsGuiPlotFrame::color_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   string color = widget->get_color ();
   plotFrame->set_engine_property(static_cast<void*>(const_cast<char *>(color.c_str())),
                                 SET_PLOT_TITLE_COLOR);
}

void ChipsGuiPlotFrame::angle_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   float angle = widget->get_angle ();
   plotFrame->set_engine_property(static_cast<void*>(&angle),
                                 SET_PLOT_TITLE_ANGLE);
}

void ChipsGuiPlotFrame::text_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   string text = widget->get_text ();
   plotFrame->set_engine_property(static_cast<void*>(const_cast<char *>(text.c_str())),
                                 SET_PLOT_TITLE);
}

void ChipsGuiPlotFrame::font_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   string font = widget->get_font ();
   plotFrame->set_engine_property(static_cast<void*>(const_cast<char *>(font.c_str())),
                                 SET_PLOT_TITLE_FONT);
}

void ChipsGuiPlotFrame::fontstyle_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   int val = widget->get_fontstyle ();
   const char *fontstyle = v_fontstyle_to_string((V_FONTSTYLE_TYPE)val);
   plotFrame->set_engine_property(static_cast<void*>(const_cast<char *>(fontstyle)),
                                 SET_PLOT_TITLE_FONTSTYLE);

}

void ChipsGuiPlotFrame::align_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   float halign = widget->get_halign ();
   float valign = widget->get_valign ();

   cmd_manager->open_block ();
   plotFrame->set_engine_property(static_cast<void*>(&valign),
                                 SET_PLOT_TITLE_VALIGN);
   plotFrame->set_engine_property(static_cast<void*>(&halign),
                                 SET_PLOT_TITLE_HALIGN);
   cmd_manager->close_block ();

}

void ChipsGuiPlotFrame::size_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   int size = widget->get_size ();
   plotFrame->set_engine_property(static_cast<void*>(&size),
                                 SET_PLOT_TITLE_SIZE);

}

void ChipsGuiPlotFrame::position_change_callback(ChipsGuiLabelWidget *widget, void *data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   float posx, posy;
   widget->get_position (posx, posy);
   plotFrame->set_engine_property(static_cast<void*>(&posx),
                                 SET_PLOT_TITLE_X);
   plotFrame->set_engine_property(static_cast<void*>(&posy),
                                 SET_PLOT_TITLE_Y);
}

void ChipsGuiPlotFrame::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPlotFrame *plotFrame = static_cast <ChipsGuiPlotFrame*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (plotFrame)  plotFrame->set_depth(val);
}


void ChipsGuiPlotFrame::initialize(ChipsId* id)
{
   if (!id)  return; 

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   _cid.reset_window_id(id->get_window_id()); 
   _cid.set_frame_id(id->get_frame_id()); 
   _cid.set_plot_id(id->get_plot_id()); 

   // get values
   string tmpstr;
   V_AXISSTYLE_TYPE tmpa; 
   V_CORNERSTYLE_TYPE tmpc; 
   _eng->get_property(&_cid, (void*)&tmpstr, SET_PLOT_AXIS_STYLE);
   if (string_to_v_axisstyle(tmpstr.c_str(), &tmpa) == 0)  _style_v = tmpa;
      else   _style_v = 2;
   _eng->get_property(&_cid, (void*)&tmpstr, SET_PLOT_AXIS_STYLE);
   if (string_to_v_cornerstyle(tmpstr.c_str(), &tmpc) == 0)  _corner_v = tmpc;
      else _corner_v = 0; 
   _eng->get_property(&_cid, static_cast<void*>(&_top_v), SET_PLOT_TMARGIN);
   _eng->get_property(&_cid, static_cast<void*>(&_bottom_v), SET_PLOT_BMARGIN);
   _eng->get_property(&_cid, static_cast<void*>(&_left_v), SET_PLOT_LMARGIN);
   _eng->get_property(&_cid, static_cast<void*>(&_right_v), SET_PLOT_RMARGIN);
   _eng->get_property(&_cid, static_cast<void*>(&_ratio_v), SET_PLOT_ASPECT_RATIO);
   _eng->get_property(&_cid, static_cast<void*>(&_depth_v), SET_PLOT_DEPTH);

   // line values
   g_signal_handler_block(GTK_COMBO_BOX(_style), _cb_sigs[0]);
   g_signal_handler_block(GTK_COMBO_BOX(_corner), _cb_sigs[1]);
   g_signal_handler_block(GTK_SCALE(_topmgn), _cb_sigs[2]);
   g_signal_handler_block(GTK_SCALE(_bottommgn), _cb_sigs[3]);
   g_signal_handler_block(GTK_SCALE(_leftmgn), _cb_sigs[4]);
   g_signal_handler_block(GTK_SCALE(_rightmgn), _cb_sigs[5]);

   gtk_combo_box_set_active(GTK_COMBO_BOX(_style),(int)_style_v);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_corner),(int)_corner_v);
   _topmax = _bottommax = _leftmax = _rightmax = 0.99; 
   gtk_range_set_value(GTK_RANGE(_topmgn), _top_v);
   gtk_range_set_value(GTK_RANGE(_bottommgn), _bottom_v);
   gtk_range_set_value(GTK_RANGE(_leftmgn), _left_v);
   gtk_range_set_value(GTK_RANGE(_rightmgn), _right_v);
   adjust_margin_ranges(0);    //  vertical adjustments
   adjust_margin_ranges(2);    //  horizontal adjustments

   g_signal_handler_unblock(GTK_COMBO_BOX(_style), _cb_sigs[0]);
   g_signal_handler_unblock(GTK_COMBO_BOX(_corner), _cb_sigs[1]);
   g_signal_handler_unblock(GTK_SCALE(_topmgn), _cb_sigs[2]);
   g_signal_handler_unblock(GTK_SCALE(_bottommgn), _cb_sigs[3]);
   g_signal_handler_unblock(GTK_SCALE(_leftmgn), _cb_sigs[4]);
   g_signal_handler_unblock(GTK_SCALE(_rightmgn), _cb_sigs[5]);

   g_signal_handler_block(GTK_COMBO_BOX(_aspmode), _cb_sigs[6]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_aspw), _cb_sigs[7]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_asph), _cb_sigs[8]);
   g_signal_handler_block(GTK_SPIN_BUTTON(_depth), _cb_sigs[9]);

   gtk_widget_set_sensitive(GTK_WIDGET(_aspw), false);
   gtk_widget_set_sensitive(GTK_WIDGET(_asph), false);
   if (_ratio_v == "")  
      gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 0);
   else if (_ratio_v == "current")  
      gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 1);
   else if (_ratio_v == "fit")  
      gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 2);
   else if (_ratio_v == "1:1")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 4);
   else if (_ratio_v == "1:2")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 5);
   else if (_ratio_v == "2:1")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 6);
   else 
   {
      float hh, ww; 
      if (sscanf(_ratio_v.c_str(), "%f:%f", &ww, &hh) == 2)
      {
         gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 3); 
         gtk_spin_button_set_value(GTK_SPIN_BUTTON(_aspw), ww);  
         gtk_spin_button_set_value(GTK_SPIN_BUTTON(_asph), hh);  
         gtk_widget_set_sensitive(GTK_WIDGET(_aspw), true);
         gtk_widget_set_sensitive(GTK_WIDGET(_asph), true);
      }
      else 
         gtk_combo_box_set_active(GTK_COMBO_BOX(_aspmode), 0);
   }
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depth_v); 
   g_signal_handler_unblock(GTK_COMBO_BOX(_aspmode), _cb_sigs[6]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_aspw), _cb_sigs[7]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_asph), _cb_sigs[8]);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_depth), _cb_sigs[9]);

   // setup the title
   {
      int ival;
      float fval1;
      float fval2;
      string sval; 
      V_FONTSTYLE_TYPE fontstyle;

      _eng->get_property(&_cid, static_cast<void*>(&fval1), SET_PLOT_TITLE_X);
      _eng->get_property(&_cid, static_cast<void*>(&fval2), SET_PLOT_TITLE_Y);
      _title->set_position (fval1, fval2);

      _eng->get_property(&_cid, static_cast<void*>(&sval), SET_PLOT_TITLE_COLOR);
      _title->set_color (sval);

      _eng->get_property(&_cid, static_cast<void*>(&ival), SET_PLOT_TITLE_SIZE);
      _title->set_size (ival);

      _eng->get_property(&_cid, static_cast<void*>(&sval), SET_PLOT_TITLE_FONT);
      _title->set_font (sval);

      _eng->get_property(&_cid, static_cast<void*>(&fval1), SET_PLOT_TITLE_ANGLE);
      _title->set_angle (fval1);

      _eng->get_property(&_cid, static_cast<void*>(&fval1), SET_PLOT_TITLE_HALIGN);
      _eng->get_property(&_cid, static_cast<void*>(&fval2), SET_PLOT_TITLE_VALIGN);

      _title->set_align (fval1, fval2);

      _eng->get_property(&_cid, static_cast<void*>(&sval), SET_PLOT_TITLE_FONTSTYLE);
      (int)string_to_v_fontstyle (sval.c_str (), &fontstyle);
      _title->set_fontstyle (fontstyle);

      _eng->get_property(&_cid, static_cast<void*>(&sval), SET_PLOT_TITLE);
      _title->set_text (sval);

   }

}

void ChipsGuiPlotFrame::display_subobject (V_SUBOBJECT_TYPE subtype)
{
   if (subtype == chips_plot_title)
      gtk_notebook_set_current_page (GTK_NOTEBOOK(_nbook), _title_page_num);
}

