// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIPLOTFRAME_HH
#define CHIPSGUIPLOTFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiPropFrame.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

class ChipsGuiLabelWidget;

//---------------------------------------------------------------------------

class ChipsGuiPlotFrame: public ChipsGuiPropFrame, public ChipsGuiTooltipObject
{
public:
   ChipsGuiPlotFrame();
   virtual ~ChipsGuiPlotFrame();

   static void style_callback(GtkWidget *widget, gpointer data);
   static void corner_callback(GtkWidget *widget, gpointer data);
   static int topmargin_callback(GtkWidget *widget, gpointer data);
   static void bottommargin_callback(GtkWidget *widget, gpointer data);
   static void leftmargin_callback(GtkWidget *widget, gpointer data);
   static void rightmargin_callback(GtkWidget *widget, gpointer data);
   static void aspect_callback(GtkWidget *widget, gpointer data);

   static void color_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void angle_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void text_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void font_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void fontstyle_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void align_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void size_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void position_change_callback(ChipsGuiLabelWidget *widget, void *data);
   static void depth_callback(GtkWidget *widget, gpointer data);

   virtual void initialize(ChipsId* id);
   virtual void build_form(); 

   virtual void display_subobject (V_SUBOBJECT_TYPE subtype);

   void set_margin(int mgn, float val);
   void set_style(int val);
   void set_corner(int val); 
   void adjust_margin_ranges(int mgn); 
   void set_depth(int val);
   void set_aspect_ratio(); 

private:
  
   GtkWidget*   _table1;
   GtkWidget*   _frmGen;
   GtkWidget*   _tabGen;
   GtkWidget*   _frmMgn;
   GtkWidget*   _tabMgn;
   GtkWidget*   _frmAsp;
   GtkWidget*   _tabAsp;
   GtkWidget*   _table2;

   GtkWidget*   _topmgn;
   GtkWidget*   _bottommgn;
   GtkWidget*   _leftmgn;
   GtkWidget*   _rightmgn;
   GtkWidget*   _aspmode;
   GtkWidget*   _aspw;
   GtkWidget*   _asph;
   GtkWidget*   _style;
   GtkWidget*   _corner;
   GtkWidget*   _depth;

   ChipsGuiLabelWidget *_title;

   // signals for callbacks
   unsigned long _cb_sigs[10];

   int          _style_v;
   int          _corner_v;
   float        _top_v;
   float        _bottom_v;
   float        _left_v;
   float        _right_v;

   float        _topmax;
   float        _bottommax;
   float        _leftmax;
   float        _rightmax;
   string       _ratio_v;
   int          _depth_v; 
   int          _title_page_num;

};

#endif
