// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <iostream>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "ChipsGuiPlugInMenu.hh"
#include "ChipsGuiMain.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;
#define BUF_SIZE_PIM   1024

ChipsGuiPlugInMenu::ChipsGuiPlugInMenu()
{
   base_w = gtk_menu_item_new_with_mnemonic("_PlugIn");

   menu = gtk_menu_new();
   items.clear(); 

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
}

ChipsGuiPlugInMenu::~ChipsGuiPlugInMenu()
{
   // delete all menu items
   for(plugin_map::iterator ii=items.begin(); ii!=items.end(); ++ii)
   {
      delete((*ii).second);
   }
   items.clear();
}

void ChipsGuiPlugInMenu::set_sensitive(int val)
{
   if (val == 1 || val == 0) 
      gtk_widget_set_sensitive(GTK_WIDGET(base_w), val);
   else 
      gtk_widget_set_sensitive(GTK_WIDGET(base_w), (items.size()> 0));
}


int ChipsGuiPlugInMenu::show()
{
   gtk_widget_show_all(base_w); 
   return 0; 
}

void ChipsGuiPlugInMenu::run_custom_callback(GtkWidget *widget, gpointer data)
{
   PLUGIN_ELEM_P_T pi = (PLUGIN_ELEM_P_T)data; 

   char namebuf[256];
   int pd = (int)getpid();
   char* lgname = getlogin();
   if (lgname)  sprintf(namebuf,"/tmp/.chips.%s.%1d.callback%1ld", lgname, pd, 
pi->sigid);
      else      sprintf(namebuf,"/tmp/.chips.%1d.callback%1ld", pd, pi->sigid);
   ofstream out(namebuf);
   out << "execfile(\"" << pi->filename << "\")" << endl;
   out << "chips.connect(\"" << chipsGui->get_server_id() << "\")" << endl;
   out << "try:"<<endl<<"   ";
   out << pi->funcname << "()" << endl<<"finally:"<<endl;
   out << "   import os"<<endl;
   out << "   os.remove(\"" << namebuf << "\")" << endl;
   out.close();
   string cmd = "python ";
   cmd.append(namebuf);
   cmd.append(" &");
   system(cmd.c_str()); 
}

int ChipsGuiPlugInMenu::load_file(string fname)
{
   if (fname.length() == 0) 
   {
      set_sensitive(); 
      return -1; 
   }
   char   *start, *pos, *end;
   int    state = 0, rv = 0;
   char   buf[BUF_SIZE_PIM];
   string cback, title; 

   ifstream *infile =  new ifstream(fname.c_str(), ios::in);

   while (infile->getline(buf, BUF_SIZE_PIM))
   {
      // find def line
      start = buf;
      if (((pos = strstr(buf, "def ")) != NULL) &&
          ((end = strstr(buf, "(")) != NULL))
      {
         // new function
         while (isspace(*start)) start++;
         if (pos == start  && (end > pos))
         {
            pos+= 4;   // position to start of callback name
            *end = '\0';
            cback = pos;
            state = 1;
         }
      }
      if (state == 1)
      {
         // got function file menu entry name
         if ((pos = strstr(buf, "menu title =")) != NULL)
         {
            end = &buf[strlen(buf)-1];
       
            // remove any trialing white space or lone quote mark 
            //while(isspace(*end))
            while(isspace(*end) || 
                  (*end == '\'' && strchr(pos+12, '\'') == end) ||
                  (*end == '"' && strchr(pos+12, '"') == end))
            {
               *end = '\0';
               end--;
            }
            title = pos + 12;
            state = 0;
            rv |= add_entry(title, fname, cback);   
            title.clear(); 
            cback.clear(); 
         }
      }
   }  // end while
   delete infile; 

   set_sensitive(); 
   return rv; 
}

int ChipsGuiPlugInMenu::add_entry(string title, string file, string funcname)
{
   if (title.length() == 0)  return -1;   // need title
   PLUGIN_ELEM_P_T ap = NULL; 

   if (items.count(title))
   {
      // replace existing entry 
      items[title]->filename = file;
      items[title]->funcname = funcname;
      ap = items[title]; 
   }
   else
   {
      // add new entry 
      ap = new PLUGIN_ELEM_T;
      ap->title = title; 
      ap->filename = file; 
      ap->funcname = funcname;
      ap->widget = gtk_menu_item_new_with_label(title.c_str());
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), ap->widget);

      items.insert(plugin_map::value_type (ap->title, ap));
      ap->sigid = g_signal_connect(G_OBJECT(ap->widget), "activate",
         G_CALLBACK(ChipsGuiPlugInMenu::run_custom_callback), (void*)ap);
   }
   if (!ap)  return -1; // err encountered  

   return 0; 
}

int ChipsGuiPlugInMenu::remove_entry(string funcname)
{
   plugin_map::iterator pi = items.find(funcname);

   if (pi == items.end())  return -1;   // not found 

   items.erase(pi); 
   return 0; 
}

 



