// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiPointPrefFrame.hh"
#include "ChipsGuiPointPropWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsCoordinate.hh"
#include "ChipsCmdManager.hh"

#include <sstream>

using namespace std;

extern ChipsGuiMain* chipsGui;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiPointPrefFrame::ChipsGuiPointPrefFrame():ChipsGuiPrefFrame(),ChipsGuiTooltipObject()
{
   _pointprop = NULL;
   _object_states = STATE_CHNG_PROP_PNT;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiPointPrefFrame::~ChipsGuiPointPrefFrame()
{
   if (_pointprop)
      delete _pointprop;
}

void ChipsGuiPointPrefFrame::build_form()
{
   _frame = gtk_frame_new("Point Properties");

   _pointprop = new ChipsGuiPointPropWidget ();
   _pointprop->build_form ();

   _pointprop->disable_position ();
   _pointprop->disable_coordinate ();
   _pointprop->disable_visibility ();

   gtk_container_add(GTK_CONTAINER(_frame), _pointprop->base_widget ());

   _pointprop->add_callback ("change_color", new ChipsGuiPointPropWidgetCallback (color_callback, this));
   _pointprop->add_callback ("change_angle", new ChipsGuiPointPropWidgetCallback (angle_callback, this));
/*   _pointprop->add_callback ("change_coordinate", new ChipsGuiPointPropWidgetCallback (coordinate_callback, this));
*/
   _pointprop->add_callback ("change_style", new ChipsGuiPointPropWidgetCallback (style_callback, this));
   _pointprop->add_callback ("change_depth", new ChipsGuiPointPropWidgetCallback (depth_callback, this));
   _pointprop->add_callback ("change_size", new ChipsGuiPointPropWidgetCallback (size_callback, this));
   _pointprop->add_callback ("change_fill", new ChipsGuiPointPropWidgetCallback (fill_callback, this));

}

void ChipsGuiPointPrefFrame::set_color(string color)
{
   stringstream pref;
   pref << "color=" << color;
   set_preference_override(pref.str (), chips_point);
}

/*
void ChipsGuiPointPrefFrame::set_coordinate(int coordinate)
{
   ChipsId cid;
   COORD_SYS coordsys = (COORD_SYS)coordinate;
   string coordid = "";

   _eng->get_current_coord (&cid);

   if ((coordsys == DATA) && (cid.get_coord_sys () != DATA))
      chipsGui->log_error ("Data coordinate system does not exists.");
   else
      coordid = cid.get_coord_id ();

   coordid = ChipsCoordinate::extract_id (coordsys, coordid);

   _eng->change_annotation_coordinate (&cid, chips_point, coordsys, coordid);
}
*/

void ChipsGuiPointPrefFrame::set_size(int val)
{
   stringstream pref;
   pref << "size=" << val;
   set_preference_override(pref.str (), chips_point);

}

void ChipsGuiPointPrefFrame::set_fill(bool fill)
{
   stringstream pref;
   pref << "fill=" << (int)fill;
   set_preference_override(pref.str (), chips_point);
}

void ChipsGuiPointPrefFrame::set_style(int val)
{
   stringstream pref;
   pref << "style=" << val;
   set_preference_override(pref.str (), chips_point);
}

void ChipsGuiPointPrefFrame::set_depth(int val)
{
   stringstream pref;
   pref << "depth=" << val;
   set_preference_override(pref.str (), chips_point);
}

void ChipsGuiPointPrefFrame::set_angle(float val)
{
   stringstream pref;
   pref << "angle=" << val;
   set_preference_override(pref.str (), chips_point);
}

void ChipsGuiPointPrefFrame::color_callback(ChipsGuiPointPropWidget *widget, void *data)
{
   ChipsGuiPointPrefFrame *pntFrame = static_cast <ChipsGuiPointPrefFrame*>(data);

   if (!pntFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_color (); 
   pntFrame->set_color(clr);
}

/*
void ChipsGuiPointPrefFrame::coordinate_callback(ChipsGuiPointPropWidget *widget, void *data)
{
   ChipsGuiPointPrefFrame *pntFrame = static_cast <ChipsGuiPointPrefFrame*>(data);

   if (!pntFrame)  return; //ensure we have needed pointer
   int coordinate = widget->get_coordinate ();
   pntFrame->set_coordinate (coordinate);
}
*/

void ChipsGuiPointPrefFrame::angle_callback(ChipsGuiPointPropWidget *widget, void *data)
{
   ChipsGuiPointPrefFrame *pntFrame = static_cast <ChipsGuiPointPrefFrame*>(data);

   if (!pntFrame)  return; //ensure we have needed pointer
   float angle = widget->get_angle ();
   pntFrame->set_angle (angle);
}



void ChipsGuiPointPrefFrame::style_callback(ChipsGuiPointPropWidget *widget, gpointer data)
{
   ChipsGuiPointPrefFrame *pntFrame = static_cast <ChipsGuiPointPrefFrame*>(data);
   int val = widget->get_style ();
   if (pntFrame)  pntFrame->set_style(val); 
}

void ChipsGuiPointPrefFrame::size_callback(ChipsGuiPointPropWidget *widget, gpointer data)
{
   ChipsGuiPointPrefFrame *pntFrame = static_cast <ChipsGuiPointPrefFrame*>(data);
   int vv = widget->get_size ();
   if (pntFrame)  pntFrame->set_size(vv);
}

void ChipsGuiPointPrefFrame::depth_callback(ChipsGuiPointPropWidget *widget, gpointer data)
{
   ChipsGuiPointPrefFrame *pntFrame = static_cast <ChipsGuiPointPrefFrame*>(data);
   int val = widget->get_depth ();
   if (pntFrame)  pntFrame->set_depth(val); 
}

void ChipsGuiPointPrefFrame::fill_callback(ChipsGuiPointPropWidget *widget, gpointer data)
{
   ChipsGuiPointPrefFrame *pntFrame = static_cast <ChipsGuiPointPrefFrame*>(data);
   bool fill = widget->get_fill ();
   if (pntFrame)  pntFrame->set_fill(fill);
}

void ChipsGuiPointPrefFrame::initialize()
{
   ChipsId cid;
   COORD_SYS coordsys;
   string coordid;
   string colorval;
   int sizeval;
   int depthval;
   bool fillval;
   float angleval;
   V_SYMBOL_TYPE styleval;
   stringstream pref_str_val;
   char *pref_cstr;

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 


   if ((pref_cstr = _eng->get_preference ("point.color")) != NULL)
   {
      colorval = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("point.fill")) != NULL)
   {
      fillval = (chips_strcmp (pref_cstr, "true") == 0) ? true : false;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("point.angle")) != NULL)
   {
      stringstream(pref_cstr) >> angleval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("point.size")) != NULL)
   {
      stringstream(pref_cstr) >> sizeval;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("point.depth")) != NULL)
   {
      if (chips_strcmp (pref_cstr, "default") == 0)
      {
         free (pref_cstr);
         if ((pref_cstr = _eng->get_preference ("default.depth")) == NULL)
            depthval = V_DEFAULT_DEPTH;
      }

      if (pref_cstr)
      {
         stringstream(pref_cstr) >> depthval;
         free (pref_cstr);
      }
   }

   if ((pref_cstr = _eng->get_preference ("point.style")) != NULL)
   {
      c2_string_to_symbol (pref_cstr, &styleval);
      free (pref_cstr);
   }


   _eng->get_object_coordinfo (&cid, chips_point, coordsys, coordid);

   _pointprop->set_color (colorval);
   _pointprop->set_coordinate ((int)coordsys);
   _pointprop->set_angle (angleval);
   _pointprop->set_fill (fillval);
   _pointprop->set_size (sizeval);
   _pointprop->set_style ((int)styleval);
   _pointprop->set_depth (depthval);
   _pointprop->set_visibility (true);
   _pointprop->set_position (0,0);

}


