// 
//  Copyright (C) 2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiPointPropWidget.hh"

#include "ChipsGuiMain.hh"
#include "ChipsDefines.hh"

#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiPositionWidget.hh"

#include <iostream>

extern ChipsGuiMain* chipsGui;


ChipsGuiPointPropWidgetCallback::ChipsGuiPointPropWidgetCallback (void (*callback)(ChipsGuiPointPropWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;

}

void ChipsGuiPointPropWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiPointPropWidget *pointWidget = dynamic_cast<ChipsGuiPointPropWidget *>(caller);
   _callback (pointWidget, _callbackData);
}



ChipsGuiPointPropWidget::ChipsGuiPointPropWidget ()
{
   _valid_callback_ids.push_back ("change_color");
   _valid_callback_ids.push_back ("change_position");
   _valid_callback_ids.push_back ("change_coordinate");
   _valid_callback_ids.push_back ("change_angle");
   _valid_callback_ids.push_back ("change_style");
   _valid_callback_ids.push_back ("change_depth");
   _valid_callback_ids.push_back ("change_size");
   _valid_callback_ids.push_back ("change_fill");
   _valid_callback_ids.push_back ("change_visibility");

   _color = NULL;
   _position = NULL;


}

ChipsGuiPointPropWidget::~ChipsGuiPointPropWidget ()
{
   if (_color)
      delete _color;

   if (_position)
      delete _position;
}

void ChipsGuiPointPropWidget::build_form ()
{
   GtkWidget* tmplabel;
   GtkWidget*   table;

   _base_widget = _nbook = gtk_notebook_new ();

   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);

   table = gtk_table_new(8,7,FALSE);
   tmplabel  = gtk_label_new("Point");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), table, tmplabel);
   _color = new ChipsGuiColorWidget();
   _color->build_form ();
   _position = new ChipsGuiPositionWidget();
   _position->build_form ();

   _style = gtk_combo_box_new_text();
   _size = gtk_spin_button_new_with_range(1, 100, 1);
   _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
   _fill =  gtk_check_button_new();
   _vis = gtk_check_button_new();

   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_size), 0);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);

   gtk_table_attach(GTK_TABLE(table), _color->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(table), _style, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "no symbol");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "cross");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "diamond");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "down triangle");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "circle");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "plus");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "square");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "up triangle");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "point");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_style), "arrow");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_style),1);

   tmplabel = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(table), _size, 2, 3, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(table), _fill, 5, 6, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   tmplabel = gtk_label_new("Size");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 1, 2, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Fill");
   gtk_table_attach(GTK_TABLE(table), tmplabel, 4, 5, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   gtk_table_attach(GTK_TABLE(table), _position->base_widget (), 1, 7, 6, 7,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   tmplabel = gtk_label_new("General");
   GtkWidget* tmptab = gtk_table_new(7,7,FALSE);
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), tmptab, tmplabel);
   tmplabel = gtk_label_new("depth");
   gtk_table_attach(GTK_TABLE(tmptab), tmplabel, 4, 5, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(tmptab), _depth, 5, 6, 5, 6,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   _vis = gtk_check_button_new();
   tmplabel = gtk_label_new("Visible");
   gtk_table_attach(GTK_TABLE(tmptab), tmplabel, 4, 5, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);
   gtk_table_attach(GTK_TABLE(tmptab), _vis, 5, 6, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);

   _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_callback, this));
   _position->add_callback ("change_position", new ChipsGuiPositionWidgetCallback (position_callback, this));
   _position->add_callback ("change_angle", new ChipsGuiPositionWidgetCallback (angle_callback, this));
   _position->add_callback ("change_coordinate", new ChipsGuiPositionWidgetCallback (coordinate_callback, this));

   _style_id = g_signal_connect(G_OBJECT(_style), "changed",
        G_CALLBACK(style_callback), (void*)this);
   _size_id = g_signal_connect(G_OBJECT(_size), "value-changed",
        G_CALLBACK(size_callback), (void*)this);
   _fill_id = g_signal_connect(G_OBJECT(_fill),"toggled",
        G_CALLBACK(fill_callback), (void*)this);
   _depth_id = g_signal_connect(G_OBJECT(_depth), "value-changed",
        G_CALLBACK(depth_callback), (void*)this);
   _vis_id = g_signal_connect(G_OBJECT(_vis), "toggled",
      G_CALLBACK(vis_callback), (void*)this);



   setup_tooltip(_color->base_widget(),"Sets the color to use for the point marker");
   setup_tooltip(_style, "Sets the shape used to display the marker");
   setup_tooltip(_size, "Controls how large the point marker is drawn");
   setup_tooltip(_fill,"Specifies whether the point marker is filled or hollow");
   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the marker is drawn");
   setup_tooltip(_vis,"Should the point marker be displayed or hidden");

}

void ChipsGuiPointPropWidget::show ()
{
   gtk_widget_show_all (_base_widget);
}

void ChipsGuiPointPropWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}

void ChipsGuiPointPropWidget::color_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);

   if (!pointWidget)  return; //ensure we have needed pointer
   string clr = widget->get_chips_color ();

   if (pointWidget->_callbacks.count ("change_color") > 0)
      pointWidget->_callbacks["change_color"]->execute (pointWidget, NULL);

}


void ChipsGuiPointPropWidget::position_callback(ChipsGuiPositionWidget *widget, gpointer data)
{ 
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);
   if (!pointWidget)  return; //ensure we have needed pointer
   double x,y;
   widget->get_position (x,y);

   if (pointWidget->_callbacks.count ("change_position") > 0)
      pointWidget->_callbacks["change_position"]->execute (pointWidget, NULL);

}

void ChipsGuiPointPropWidget::coordinate_callback(ChipsGuiPositionWidget *widget, void *data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);

   if (!pointWidget)  return; //ensure we have needed pointer

   if (pointWidget->_callbacks.count ("change_coordinate") > 0)
      pointWidget->_callbacks["change_coordinate"]->execute (pointWidget, NULL);

}

void ChipsGuiPointPropWidget::angle_callback(ChipsGuiPositionWidget *widget, void *data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);

   if (!pointWidget)  return; //ensure we have needed pointer
   float angle = widget->get_angle ();
   pointWidget->set_angle (angle);

   if (pointWidget->_callbacks.count ("change_angle") > 0)
      pointWidget->_callbacks["change_angle"]->execute (pointWidget, NULL);

}

void ChipsGuiPointPropWidget::style_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (pointWidget)  pointWidget->set_style(val);

   if (pointWidget->_callbacks.count ("change_style") > 0)
      pointWidget->_callbacks["change_style"]->execute (pointWidget, NULL);

}

void ChipsGuiPointPropWidget::size_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);
   int vv = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (pointWidget)  pointWidget->set_size(vv);

   if (pointWidget->_callbacks.count ("change_size") > 0)
      pointWidget->_callbacks["change_size"]->execute (pointWidget, NULL);

}

void ChipsGuiPointPropWidget::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (pointWidget)  pointWidget->set_depth(val);

   if (pointWidget->_callbacks.count ("change_depth") > 0)
      pointWidget->_callbacks["change_depth"]->execute (pointWidget, NULL);

}

void ChipsGuiPointPropWidget::fill_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);
   bool fill = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (pointWidget)  pointWidget->set_fill(fill);

   if (pointWidget->_callbacks.count ("change_fill") > 0)
      pointWidget->_callbacks["change_fill"]->execute (pointWidget, NULL);

}

void ChipsGuiPointPropWidget::vis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPointPropWidget *pointWidget = static_cast <ChipsGuiPointPropWidget*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (pointWidget)  pointWidget->set_visibility(val);

   if (pointWidget->_callbacks.count ("change_visibility") > 0)
      pointWidget->_callbacks["change_visibility"]->execute (pointWidget, NULL);

}


void ChipsGuiPointPropWidget::set_position (double x, double y)
{
   if (_position)
      _position->set_position (x,y);

}

void ChipsGuiPointPropWidget::set_color (const string &color)
{
   if (_color)
      _color->set_chips_color (color);
}

void ChipsGuiPointPropWidget::set_angle(float val)
{
   if (_position)
      _position->set_angle (val);
}


void ChipsGuiPointPropWidget::set_coordinate (int val)
{
   if (_position)
      _position->set_coordinate (val);
}

void ChipsGuiPointPropWidget::set_position_format (int xformat, int yformat)
{
   if (_position)
      _position->set_format (xformat, yformat);
}


void ChipsGuiPointPropWidget::set_size(int val)
{
   _sizeval = val;
   g_signal_handler_block(GTK_OBJECT(_size), _size_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_size), val);
   g_signal_handler_unblock(GTK_OBJECT(_size), _size_id);
}

void ChipsGuiPointPropWidget::set_fill(bool fill)
{
   _fillval = fill;
   g_signal_handler_block(GTK_OBJECT(_fill), _fill_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_fill), _fillval);
   g_signal_handler_unblock(GTK_OBJECT(_fill), _fill_id);
}

void ChipsGuiPointPropWidget::set_style(int val)
{
   _styleval = val;
   g_signal_handler_block(GTK_OBJECT(_style), _style_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_style),_styleval);
   g_signal_handler_unblock(GTK_OBJECT(_style), _style_id);
}

void ChipsGuiPointPropWidget::set_depth(int val)
{
   _depthval = val;
   g_signal_handler_block(GTK_OBJECT(_depth), _depth_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depthval);
   g_signal_handler_unblock(GTK_OBJECT(_depth), _depth_id);
}

void ChipsGuiPointPropWidget::set_visibility(bool val)
{
   _visval = val;
   g_signal_handler_block(GTK_OBJECT(_vis), _vis_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_vis), _visval);
   g_signal_handler_unblock(GTK_OBJECT(_vis), _vis_id);
}


void ChipsGuiPointPropWidget::get_position (double &x, double &y)
{
   _position->get_position (x,y);
}

float ChipsGuiPointPropWidget::get_angle ()
{
   return _position->get_angle ();
}

int ChipsGuiPointPropWidget::get_coordinate ()
{
   return _position->get_coordinate ();
}

void ChipsGuiPointPropWidget::get_position_format (int &xformat, int &yformat)
{
   _position->get_format (xformat, yformat);
}



int ChipsGuiPointPropWidget::get_style()
{
   return _styleval;
}

int ChipsGuiPointPropWidget::get_depth()
{
   return _depthval;
}

string ChipsGuiPointPropWidget::get_color()
{
   return _color->get_chips_color ();
}

int ChipsGuiPointPropWidget::get_size()
{
   return _sizeval;
}

bool ChipsGuiPointPropWidget::get_fill()
{
   return _fillval;
}

bool ChipsGuiPointPropWidget::get_visibility()
{
   return _visval;
}

void ChipsGuiPointPropWidget::enable_angle()
{
   _position->enable_angle ();
}

void ChipsGuiPointPropWidget::enable_style()
{
   gtk_widget_set_sensitive (_style, TRUE);
}

void ChipsGuiPointPropWidget::enable_depth()
{
   gtk_widget_set_sensitive (_depth, TRUE);
}

void ChipsGuiPointPropWidget::enable_color()
{
   _color->enable ();
}

void ChipsGuiPointPropWidget::enable_size()
{
   gtk_widget_set_sensitive (_size, TRUE);
}

void ChipsGuiPointPropWidget::enable_fill()
{
   gtk_widget_set_sensitive (_fill, TRUE);
}

void ChipsGuiPointPropWidget::enable_visibility()
{
   gtk_widget_set_sensitive (_vis, TRUE);
}

void ChipsGuiPointPropWidget::enable_position ()
{
   _position->enable_position ();
}

void ChipsGuiPointPropWidget::enable_coordinate ()
{
   _position->enable_coordinate ();
}

void ChipsGuiPointPropWidget::enable_data_coordinate ()
{
   _position->enable_data_coordinate ();
}


void ChipsGuiPointPropWidget::disable_angle()
{
   _position->disable_angle ();
}

void ChipsGuiPointPropWidget::disable_style()
{
   gtk_widget_set_sensitive (_style, FALSE);
}

void ChipsGuiPointPropWidget::disable_depth()
{
   gtk_widget_set_sensitive (_depth, FALSE);
}

void ChipsGuiPointPropWidget::disable_color()
{
   _color->disable ();
}

void ChipsGuiPointPropWidget::disable_size()
{
   gtk_widget_set_sensitive (_size, FALSE);
}

void ChipsGuiPointPropWidget::disable_fill()
{
   gtk_widget_set_sensitive (_fill, FALSE);
}

void ChipsGuiPointPropWidget::disable_visibility()
{
   gtk_widget_set_sensitive (_vis, FALSE);
}

void ChipsGuiPointPropWidget::disable_position ()
{
   _position->disable_position ();
}

void ChipsGuiPointPropWidget::disable_coordinate ()
{
   _position->disable_coordinate ();
}

void ChipsGuiPointPropWidget::disable_data_coordinate ()
{
   _position->disable_data_coordinate ();
}


   
