// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIPOINTPROPPOINTWIDGET_HH
#define CHIPSGUIPOINTPROPPOINTWIDGET_HH

#include <gtk/gtk.h>
#include <string>

#include "ChipsGuiWidget.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;


//  * change_style
//  * change_depth
//  * change_size
//  * change_fill
//  * change_visibility
//  * change_position
//  * change_coordinate

class ChipsGuiPointPropWidget;
class ChipsGuiPositionWidget;
class ChipsGuiColorWidget;

class ChipsGuiPointPropWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiPointPropWidgetCallback (void (*callback)(ChipsGuiPointPropWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiPointPropWidget *, void *);
     void *_callbackData;     
};

class ChipsGuiPointPropWidget : public ChipsGuiWidget, public ChipsGuiTooltipObject
{
   public:

      ChipsGuiPointPropWidget ();
      ~ChipsGuiPointPropWidget ();

      virtual void build_form ();

      void set_angle(float val);
      void set_style(int val);
      void set_depth(int val);
      void set_color(const string &color);
      void set_size(int val);
      void set_fill(bool fill);
      void set_visibility(bool val);
      void set_position (double xpos, double ypos);
      void set_coordinate (int coordinate);
      void set_position_format (int xformat, int yformat);

      float  get_angle();
      int    get_style();
      int    get_depth();
      string get_color();
      int    get_size();
      bool   get_fill();
      bool   get_visibility();
      void   get_position (double &xpos, double &ypos);
      int    get_coordinate ();
      void   get_position_format (int &xformat, int &yformat);

      void enable_angle();
      void enable_style();
      void enable_depth();
      void enable_color();
      void enable_size();
      void enable_fill();
      void enable_visibility();
      void enable_position ();
      void enable_coordinate ();
      void enable_data_coordinate ();

      void disable_angle();
      void disable_style();
      void disable_depth();
      void disable_color();
      void disable_size();
      void disable_fill();
      void disable_visibility();
      void disable_position ();
      void disable_coordinate ();
      void disable_data_coordinate ();





      virtual void show ();
      virtual void hide ();


     
   protected:

      static void color_callback(ChipsGuiColorWidget *widget, void *data);
      static void style_callback(GtkWidget *widget, gpointer data);
      static void size_callback(GtkWidget *widget, gpointer data);
      static void depth_callback(GtkWidget *widget, gpointer data);
      static void fill_callback(GtkWidget *widget, gpointer data);
      static void vis_callback(GtkWidget *widget, gpointer data);
      static void position_callback(ChipsGuiPositionWidget *widget, void *data);
      static void coordinate_callback(ChipsGuiPositionWidget *widget, void *data);
      static void angle_callback(ChipsGuiPositionWidget *widget, void *data);

      ChipsGuiColorWidget*   _color;
      GtkWidget*   _style;
      GtkWidget*   _size;
      GtkWidget*   _fill;
      GtkWidget*   _depth;
      GtkWidget*   _vis;
      ChipsGuiPositionWidget*  _position;
      GtkWidget*   _nbook;


      // signals for callbacks
      unsigned long _style_id;
      unsigned long _size_id;
      unsigned long _fill_id;
      unsigned long _depth_id;
      unsigned long _vis_id;

      int          _sizeval;
      int          _depthval;
      bool         _fillval;
      bool         _visval;
      int          _styleval;


};

#endif

