// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "ChipsEventManager.hh"
#include "ChipsGuiPopUp.hh"
#include "ChipsGuiMain.hh"
#include "ChipsObject.hh"
#include "ChipsEngine.hh"
#include "ChipsGuiPropDialog.hh"
#include "ChipsGuiSettings.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;
extern ChipsEventManager* event_manager;
extern ChipsGuiSettings* GuiSettings;


ChipsGuiPopUp::ChipsGuiPopUp():ChipsEventObserver(), ChipsGuiTooltipObject()
{
   int pos = 0, spos = 0; 
   obj = NULL; 
 
   menu = gtk_menu_new();

   menuItemUndo = gtk_menu_item_new_with_label(("Undo"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemUndo, pos++);

   menuItemRedo = gtk_menu_item_new_with_label(("Redo"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemRedo, pos++);

   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, pos++);

   menuItemProps = gtk_menu_item_new_with_label(("Edit"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemProps, pos++);

   menuItemDel = gtk_menu_item_new_with_label(("Delete"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemDel, pos++);

   submenu = gtk_menu_new();  // shuffle submenu 
   menuItemShuf = gtk_menu_item_new_with_label(("Shuffle Object"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemShuf, pos++);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(menuItemShuf), submenu);

   menuItemShufU = gtk_menu_item_new_with_label(("Shuffle Forward"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(submenu), menuItemShufU, spos++);

   menuItemShufD = gtk_menu_item_new_with_label(("Shuffle Backward"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(submenu), menuItemShufD, spos++);

   menuItemShufF = gtk_menu_item_new_with_label(("Shuffle Front"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(submenu), menuItemShufF, spos++);

   menuItemShufB = gtk_menu_item_new_with_label(("Shuffle Back"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(submenu), menuItemShufB, spos++);

   sep2 = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep2, pos++);

   add_submenu = gtk_menu_new();  // add submenu
   menuItemAdd = gtk_menu_item_new_with_label("Annotate");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAdd, pos++);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(menuItemAdd), add_submenu);

   spos = 0;
   menuItemAddLabel = gtk_menu_item_new_with_label(("Label"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddLabel, spos++);

   menuItemAddPoint = gtk_menu_item_new_with_label(("Point"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddPoint, spos++);

   menuItemAddLine = gtk_menu_item_new_with_label(("Polyline"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddLine, spos++);

   menuItemAddHLine = gtk_menu_item_new_with_label(("Horizontal Line"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddHLine, spos++);

   menuItemAddVLine = gtk_menu_item_new_with_label(("Vertical Line"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddVLine, spos++);

   menuItemAddPolygon = gtk_menu_item_new_with_label(("Polygon"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddPolygon, spos++);
   menuItemAddEllipse = gtk_menu_item_new_with_label(("Ellipse"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddEllipse, spos++);
   menuItemAddBox = gtk_menu_item_new_with_label(("Box"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddBox, spos++);
   menuItemAddColorBar = gtk_menu_item_new_with_label(("Colorbar"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(add_submenu), menuItemAddColorBar, spos++);

   sep3 = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep3, pos++);

   menuItemPan = gtk_radio_menu_item_new_with_label(NULL, "Pan");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemPan, pos++);

   menuItemRange = gtk_radio_menu_item_new_with_label_from_widget (GTK_RADIO_MENU_ITEM(menuItemPan),"Range");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemRange, pos++);

   menuItemSel = gtk_radio_menu_item_new_with_label_from_widget (GTK_RADIO_MENU_ITEM(menuItemPan),"Select");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemSel, pos++);

   add_submenu = gtk_menu_new();  // add submenu

   sep4 = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep4, pos++);

   menuItemPrint = gtk_menu_item_new_with_label(("Print"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemPrint, pos++);

   sep5 = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep5, pos++);

   menuItemShowGui = gtk_menu_item_new_with_label(("Show Gui"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemShowGui, pos++);

   /* We can attach the Quit menu item to our exit function */
   g_signal_connect(G_OBJECT(menuItemDel), "activate",
                    G_CALLBACK(delete_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemShuf), "activate",
                    G_CALLBACK(shuffle_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemShufU), "activate",
                    G_CALLBACK(shuffle_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemShufD), "activate",
                    G_CALLBACK(shuffle_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemShufF), "activate",
                    G_CALLBACK(shuffle_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemShufB), "activate",
                    G_CALLBACK(shuffle_cb), (gpointer) this);
   menuItemAdd_id = g_signal_connect(G_OBJECT(menuItemAdd), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddLabel), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddPoint), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddLine), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddHLine), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddVLine), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddPolygon), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddEllipse), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddBox), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemAddColorBar), "activate",
                    G_CALLBACK(add_cb), (gpointer) this);
   g_signal_connect_swapped(G_OBJECT(menuItemProps), "activate",
                    G_CALLBACK (props_cb), (gpointer) this);
   menuItemPan_id = g_signal_connect(G_OBJECT(menuItemPan), "activate",
                    G_CALLBACK (pan_cb), 
                    (gpointer) this);
   menuItemRange_id = g_signal_connect(G_OBJECT(menuItemRange), "activate",
                    G_CALLBACK (range_cb), 
                    (gpointer) this);
   menuItemSel_id = g_signal_connect(G_OBJECT(menuItemSel), "activate",
                    G_CALLBACK (select_cb), 
                    (gpointer) this);
   g_signal_connect_swapped(G_OBJECT(menuItemUndo), "activate",
                    G_CALLBACK (undo_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemRedo), "activate",
                    G_CALLBACK (redo_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemShowGui), "activate",
                    G_CALLBACK (showgui_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemPrint), "activate",
                    G_CALLBACK (print_cb), (gpointer) this);

   if (GuiSettings)
   {
      // set the default value
      string mode = GuiSettings->get_defbutton();
      g_signal_handler_block (menuItemPan, menuItemPan_id);
      g_signal_handler_block (menuItemRange, menuItemRange_id);
      g_signal_handler_block (menuItemSel, menuItemSel_id);

      if (mode == "pan")
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemPan),true);
      else if (mode == "range")
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemRange),true);
      else if (mode == "select")
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemSel),true);
      g_signal_handler_unblock (menuItemPan, menuItemPan_id);
      g_signal_handler_unblock (menuItemRange, menuItemRange_id);
      g_signal_handler_unblock (menuItemSel, menuItemSel_id);

   }


   setup_tooltip(menuItemAdd, "Add annotation to the specified plot"); 
   setup_tooltip(menuItemAdd, "Add a point to the specified plot");
   setup_tooltip(menuItemDel, "Remove the selected object from the plot/window"); 
   setup_tooltip(menuItemProps, "Edit properties of the selected object"); 
   setup_tooltip(menuItemShuf, "Adjust the rendering order of the selected object");  
   setup_tooltip(menuItemShufU, "Move the current object forward in the plot"); 
   setup_tooltip(menuItemShufD, "Move the current object backward in the plot"); 
   setup_tooltip(menuItemShufF, "Move the current object to the front of the plot"); 
   setup_tooltip(menuItemShufB, "Move the current object to the back of the plot"); 
   setup_tooltip(menuItemPan, "Change the gui interaction mode to Pan"); 
   setup_tooltip(menuItemRange, "Change the gui interaction mode to Range Zoom"); 
   setup_tooltip(menuItemSel, "Change the gui interaction mode to Select"); 
   setup_tooltip(menuItemUndo, "Undo the last executed command/change"); 
   setup_tooltip(menuItemRedo, "Redo the last undone command/change"); 
   setup_tooltip(menuItemPrint, "Launch dialog with printing options"); 
   setup_tooltip(menuItemShowGui, "Launch the ChIPS gui window"); 

   register_event(ChipsEventManager::RightClickEvent);
   register_event(ChipsEventManager::ChangeStateEvent);


   show(); 
}

ChipsGuiPopUp::~ChipsGuiPopUp()
{
   
}

int ChipsGuiPopUp::show()
{
   gtk_widget_show_all(menu); 
   return 0; 
}

int ChipsGuiPopUp::shuffle_object(int dir)
{
   if (!chipsGui)  return -1;
   if (!obj)  return -2;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng)  return -3;

   ChipsId cid;
   obj->populate_chips_id(cid); 

   return eng->rearrange_depth(&cid, typ, dir); 
}

int ChipsGuiPopUp::add_object(int substate)
{
   if (!chipsGui)  return -1;
   ChipsEngine* eng = chipsGui->get_engine();
   if (!eng)  return -3;

   double pos[2] = {0.0,0.0};
   event_manager->invoke_event(pos, ChipsEventManager::ChangeAddStateEvent,
      (void*)&substate, this);


   return 0;
}


int ChipsGuiPopUp::delete_object()
{
   if (!chipsGui)  return -1;
   if (!obj)  return -2;
   ChipsId cid;
   obj->populate_chips_id(cid); 

   chipsGui->delete_command(&cid, typ); 
   return 0;
}


int ChipsGuiPopUp::add_cb(GtkMenuItem *mitem, gpointer user_data)
{
   ChipsGuiPopUp* pup = (ChipsGuiPopUp*)user_data;
   int rv = -1;
   
   if (mitem == GTK_MENU_ITEM(pup->menuItemAddLabel))
      rv = pup->add_object(ChipsEventManager::LabelAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddPoint))
      rv = pup->add_object(ChipsEventManager::PointAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddLine))
      rv = pup->add_object(ChipsEventManager::PolyLineAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddHLine))
      rv = pup->add_object(ChipsEventManager::HLineAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddVLine))
      rv = pup->add_object(ChipsEventManager::VLineAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddPolygon))
      rv = pup->add_object(ChipsEventManager::PolygonAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddEllipse))
      rv = pup->add_object(ChipsEventManager::EllipseAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddBox))
      rv = pup->add_object(ChipsEventManager::BoxAdd);
   else if (mitem == GTK_MENU_ITEM(pup->menuItemAddColorBar))
      rv = pup->add_object(ChipsEventManager::ColorBarAdd);


   return rv;

}

int ChipsGuiPopUp::delete_cb(GtkMenuItem *mitem, gpointer user_data)
{
   ChipsGuiPopUp* pup = (ChipsGuiPopUp*)user_data; 
   if (pup)  return pup->delete_object(); 
   return -1; 
}

int ChipsGuiPopUp::props_cb(GtkMenuItem *mitem, gpointer user_data)
{
   if (!chipsGui)  return -1;
   chipsGui->edit_dialog_command(); 
   return 0;
}

int ChipsGuiPopUp::print_cb(GtkMenuItem *mitem, gpointer user_data)
{
   if (!chipsGui)  return -1;
   chipsGui->print_command();
   return 0;
}

int ChipsGuiPopUp::pan_cb(GtkMenuItem *mitem, gpointer user_data)
{
   ChipsGuiPopUp* pup = (ChipsGuiPopUp*)user_data;
   if (event_manager)
   {
      int val = ChipsEventManager::PanState;
      double pos[2] = {0.0,0.0};
      event_manager->invoke_event(pos, ChipsEventManager::ChangeStateEvent,
         (void*)&val, pup);
   }
   return 0;
}

int ChipsGuiPopUp::range_cb(GtkMenuItem *mitem, gpointer user_data)
{
   ChipsGuiPopUp* pup = (ChipsGuiPopUp*)user_data;
   if (event_manager)
   {
      int val = ChipsEventManager::RangeZoomState;
      double pos[2] = {0.0,0.0};
      event_manager->invoke_event(pos, ChipsEventManager::ChangeStateEvent,
         (void*)&val, pup);
   }
   return 0;
}


int ChipsGuiPopUp::select_cb(GtkMenuItem *mitem, gpointer user_data)
{
   ChipsGuiPopUp* pup = (ChipsGuiPopUp*)user_data;
   if (event_manager)
   {
      int val = ChipsEventManager::SelectState;
      double pos[2] = {0.0,0.0};
      event_manager->invoke_event(pos, ChipsEventManager::ChangeStateEvent,
         (void*)&val, pup);
   }
   return 0;
}

int ChipsGuiPopUp::shuffle_cb(GtkMenuItem *mitem, gpointer user_data)
{
   ChipsGuiPopUp* pup = (ChipsGuiPopUp*)user_data; 
   int rv = -1; 

   if (mitem == GTK_MENU_ITEM(pup->menuItemShufU))  
      rv = pup->shuffle_object(chips_forward);    
   else if (mitem == GTK_MENU_ITEM(pup->menuItemShufD))
      rv = pup->shuffle_object(chips_backward);    
   else if (mitem == GTK_MENU_ITEM(pup->menuItemShufF))
      rv = pup->shuffle_object(chips_front);    
   else if (mitem == GTK_MENU_ITEM(pup->menuItemShufB))
      rv = pup->shuffle_object(chips_back);    
   return rv;
}

int ChipsGuiPopUp::undo_cb(GtkMenuItem *mitem, gpointer user_data)
{
  if (!chipsGui)  return -1;
  chipsGui->undo_command(); 
  return 0; 
}

int ChipsGuiPopUp::redo_cb(GtkMenuItem *mitem, gpointer user_data)
{
  if (!chipsGui)  return -1;
  chipsGui->redo_command();
  return 0;
}

int ChipsGuiPopUp::showgui_cb(GtkMenuItem *mitem, gpointer user_data)
{
  if (!chipsGui)  return -1;
  chipsGui->show_dialog();
  return 0;
}

// event observer to handle right click (pop up request) from event manager
void ChipsGuiPopUp::execute_event (int event, void *data)
{
   switch (event)
   {
      case ChipsEventManager::RightClickEvent:
      {
         ChipsObject* obj = (ChipsObject*) data;
         popup(obj);
      }
      break;
      case ChipsEventManager::ChangeStateEvent:
      {
         g_signal_handler_block (menuItemPan, menuItemPan_id);
         g_signal_handler_block (menuItemRange, menuItemRange_id);
         g_signal_handler_block (menuItemSel, menuItemSel_id);
         g_signal_handler_block (menuItemAdd, menuItemAdd_id);

         int *states = (int*)data;
         if (states[1] != ChipsEventManager::PickState)
         {
            gtk_widget_set_sensitive (GTK_WIDGET(menuItemPan), TRUE);
            gtk_widget_set_sensitive (GTK_WIDGET(menuItemRange), TRUE);
            gtk_widget_set_sensitive (GTK_WIDGET(menuItemSel), TRUE);
            gtk_widget_set_sensitive (GTK_WIDGET(menuItemAdd), TRUE);
         }

         switch(states[1])
         {
            case ChipsEventManager::PanState:
               gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemPan),true);
            break;
            case ChipsEventManager::RangeZoomState:
               gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemRange),true);
            break;
            case ChipsEventManager::SelectState:
               gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemSel),true);
            break;
            case ChipsEventManager::PickState:
               gtk_widget_set_sensitive (GTK_WIDGET(menuItemPan), FALSE);
               gtk_widget_set_sensitive (GTK_WIDGET(menuItemRange), FALSE);
               gtk_widget_set_sensitive (GTK_WIDGET(menuItemSel), FALSE);
               gtk_widget_set_sensitive (GTK_WIDGET(menuItemAdd), FALSE);
            default:
            break;
         }

         g_signal_handler_unblock (menuItemPan, menuItemPan_id);
         g_signal_handler_unblock (menuItemRange, menuItemRange_id);
         g_signal_handler_unblock (menuItemSel, menuItemSel_id);
         g_signal_handler_unblock (menuItemAdd, menuItemAdd_id);

      }
      break;
      default:
      break;
   }
}

void ChipsGuiPopUp::popup(ChipsObject* object)
{
   string lab, lab2 =""; 
   GtkWidget *menu_label = gtk_bin_get_child(GTK_BIN(menuItemDel));
   if ((obj = object) == NULL)
   {
      typ = chips_not_object; 
      gtk_label_set_text(GTK_LABEL(menu_label), "Delete"); 
      gtk_widget_set_sensitive(GTK_WIDGET(menuItemDel), false);
      menu_label = gtk_bin_get_child(GTK_BIN(menuItemProps));
      gtk_label_set_text(GTK_LABEL(menu_label), "Edit"); 
   }
   else 
   {
      typ = obj->get_object_type(); 

      // set delete entry to specify object type 
      lab = "Delete ";
      const char *ctyp = v_object_to_string(typ); 
      if (ctyp) 
      {
         lab2 = ctyp;
         lab2[0] = toupper(lab2[0]); 
         lab.append(lab2);  
      }
      gtk_label_set_text(GTK_LABEL(menu_label), lab.c_str()); 
      gtk_widget_set_sensitive(GTK_WIDGET(menuItemDel), true);
      lab = "Edit ";
      if (ctyp)
      {
         lab.append(lab2);
      }
      menu_label = gtk_bin_get_child(GTK_BIN(menuItemProps));
      gtk_label_set_text(GTK_LABEL(menu_label), lab.c_str());
   }

   // set shuffle and edit sensitivity based on mode
   bool can_edit = (typ != chips_not_object &&
     event_manager->get_state() == ChipsEventManager::SelectState);
   bool can_shuf = (can_edit && 
      (typ != chips_window && typ != chips_frame && typ != chips_plot));
    
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemShuf), can_shuf); 
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemProps), can_edit); 

   // set undo/redo sensitivity 
   chipsGui->update_undo_sensitivity(); 
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemUndo), 
      chipsGui->get_undo_available()); 
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemRedo), 
      chipsGui->get_redo_available()); 

   // set show gui sensitivity based on whether gui is up 
   gtk_widget_set_sensitive(GTK_WIDGET(menuItemShowGui), 
      (chipsGui->get_visible() == 0));

   // check if the annotation should be displayed
   this->enable_annotate_menu (chipsGui->get_annotate_sensitivity ());

   gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, 
      gtk_get_current_event_time());
}

void ChipsGuiPopUp::enable_annotate_menu (bool enable)
{
   if (enable)
      gtk_widget_set_sensitive (GTK_WIDGET(menuItemAdd), TRUE);
   else
      gtk_widget_set_sensitive (GTK_WIDGET(menuItemAdd), FALSE);
}
