// 
//  Copyright (C) 2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiPositionWidget.hh"

#include "ChipsGuiSpinWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsDefines.hh"


#include <iostream>

extern ChipsGuiMain* chipsGui;


ChipsGuiPositionWidgetCallback::ChipsGuiPositionWidgetCallback (void (*callback)(ChipsGuiPositionWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;
}

void ChipsGuiPositionWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiPositionWidget *positionWidget = dynamic_cast<ChipsGuiPositionWidget *>(caller);
   _callback (positionWidget, _callbackData);
}



ChipsGuiPositionWidget::ChipsGuiPositionWidget ()
{
   _valid_callback_ids.push_back ("change_position");
   _valid_callback_ids.push_back ("change_coordinate");
   _valid_callback_ids.push_back ("change_angle");

   _xpos = NULL;
   _ypos = NULL;
   _coordsys = NULL;
   _angle = NULL;

   _xpos_val = 0;
   _ypos_val = 0;
   _angle = 0;
   _xpos_format = 0;
   _ypos_format = 0;

}

ChipsGuiPositionWidget::~ChipsGuiPositionWidget ()
{
   if (_xpos)
      delete _xpos;
   if (_ypos)
      delete _ypos;
}

void ChipsGuiPositionWidget::build_form ()
{
   this->build_form (true);
}

void ChipsGuiPositionWidget::build_form (bool include_angle)
{
   GtkWidget *table;
   GtkWidget *hbox1;
   GtkWidget *hbox2;
   GtkWidget *hbox3;
   GtkWidget *label;
   _base_widget = table = gtk_table_new (2, 2, FALSE);
   
   label = gtk_label_new("X");
   hbox1 = gtk_hbox_new (FALSE, 2);
   gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE,5);
   _xpos = new ChipsGuiSpinWidget(0);
   gtk_box_pack_start(GTK_BOX(hbox1), _xpos->base_widget (), FALSE, FALSE,5);

   gtk_table_attach(GTK_TABLE(table), hbox1, 0, 1, 0, 1,
                    (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5,5);

   label = gtk_label_new("Y");
   hbox2 = gtk_hbox_new (FALSE, 2);
   gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE,5);
   _ypos = new ChipsGuiSpinWidget(1);
   gtk_box_pack_start(GTK_BOX(hbox2), _ypos->base_widget (), FALSE, FALSE,5);

   gtk_table_attach(GTK_TABLE(table), hbox2, 0, 1, 1, 2,
                    (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5,5);


   enum { COL_TEXT, COL_SENS };
   GtkTreeIter iter;
   GtkCellRenderer *renderer;
   _store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
   gtk_list_store_append(_store, &iter);
   gtk_list_store_set(_store, &iter, COL_TEXT, "Pixel", COL_SENS, TRUE, -1);
   gtk_list_store_append(_store, &iter);
   gtk_list_store_set(_store, &iter, COL_TEXT, "Frame", COL_SENS, TRUE, -1);
   gtk_list_store_append(_store, &iter);
   gtk_list_store_set(_store, &iter, COL_TEXT, "Plot", COL_SENS, TRUE, -1);
   gtk_list_store_append(_store, &iter);
   gtk_list_store_set(_store, &iter, COL_TEXT, "Data", COL_SENS, FALSE, -1);
   _iterD = iter; 
   _coordsys = gtk_combo_box_new_with_model(GTK_TREE_MODEL(_store));
   gtk_table_attach(GTK_TABLE(table), _coordsys, 1, 2, 0, 1,
                    (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5,5);
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(_coordsys), renderer, TRUE);
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(_coordsys), renderer,
                                 "text", COL_TEXT);
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(_coordsys), renderer,
                                 "sensitive", COL_SENS);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_coordsys),0);

   if (include_angle)
   {
      label = gtk_label_new("Angle");
      hbox3 = gtk_hbox_new (FALSE, 2);
      gtk_box_pack_start(GTK_BOX(hbox3), label, FALSE, FALSE,5);

      _angle = gtk_spin_button_new_with_range (0, 359.99999, 1);
      gtk_box_pack_start(GTK_BOX(hbox3), _angle, FALSE, FALSE,5);
      gtk_spin_button_set_wrap (GTK_SPIN_BUTTON(_angle), TRUE);

      gtk_table_attach(GTK_TABLE(table), hbox3, 1, 2, 1, 2,
                    (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_SHRINK,5,5);

      _angle_change_id = g_signal_connect(G_OBJECT(_angle), "value-changed", G_CALLBACK(angle_change_callback), (void*)this);
   }
   else
   {
      _angle = NULL;
      _angle_change_id = -1;
   }
   _xpos->add_callback ("change_value", new ChipsGuiSpinWidgetCallback(position_change_callback, this));
   _ypos->add_callback ("change_value", new ChipsGuiSpinWidgetCallback(position_change_callback, this));
   _coordinate_change_id = g_signal_connect(G_OBJECT(_coordsys), "changed", G_CALLBACK(coordinate_change_callback), (void*)this);

   setup_tooltip(_xpos->base_widget (),"Sets the x location");
   setup_tooltip(_ypos->base_widget (),"Sets the y location");
   setup_tooltip(_angle,"Sets the rotation angle");
   setup_tooltip(_coordsys,"Sets the coordinate system");



}

void ChipsGuiPositionWidget::disable_position ()
{
   _xpos->disable ();
   _ypos->disable ();
}

void ChipsGuiPositionWidget::enable_position ()
{
   _xpos->enable ();
   _ypos->enable ();
}

void ChipsGuiPositionWidget::disable_coordinate ()
{
   gtk_widget_set_sensitive (_coordsys, FALSE);
}

void ChipsGuiPositionWidget::enable_coordinate ()
{
   gtk_widget_set_sensitive (_coordsys, TRUE);
}

void ChipsGuiPositionWidget::disable_data_coordinate ()
{
   gtk_list_store_set(_store, &_iterD, 0, "Data", 1, FALSE, -1);
}

void ChipsGuiPositionWidget::enable_data_coordinate ()
{
   gtk_list_store_set(_store, &_iterD, 0, "Data", 1, TRUE, -1);
}


void ChipsGuiPositionWidget::disable_angle ()
{
   if (_angle)
      gtk_widget_set_sensitive (_angle, FALSE);
}

void ChipsGuiPositionWidget::enable_angle ()
{
   if (_angle)
      gtk_widget_set_sensitive (_angle, TRUE);
}


void ChipsGuiPositionWidget::show ()
{
   gtk_widget_show_all (_base_widget);
}

void ChipsGuiPositionWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}


void ChipsGuiPositionWidget::position_change_callback(ChipsGuiSpinWidget *widget, gpointer data)
{ 
   ChipsGuiPositionWidget *positionWidget = static_cast <ChipsGuiPositionWidget*>(data);
   double xval = positionWidget->_xpos->get_numeric_value ();
   double yval = positionWidget->_ypos->get_numeric_value ();

   positionWidget->set_position (xval, yval);

   
   if (positionWidget->_callbacks.count ("change_position") > 0)
      positionWidget->_callbacks["change_position"]->execute (positionWidget, NULL);

}

void ChipsGuiPositionWidget::angle_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPositionWidget *positionWidget = (ChipsGuiPositionWidget *)data;

   float val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));

   positionWidget->set_angle (val);

   if (positionWidget->_callbacks.count ("change_angle") > 0)
      positionWidget->_callbacks["change_angle"]->execute (positionWidget, NULL);

}

void ChipsGuiPositionWidget::coordinate_change_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiPositionWidget *positionWidget = (ChipsGuiPositionWidget *)data;

   int   combo_num = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   int   coordsys = COORD_NONE;

   switch (combo_num)
   {
      case 0: 
        coordsys = PIXEL;
      break;
      case 1:
        coordsys = FRAME_NORM;
      break;
      case 2:
        coordsys = PLOT_NORM;
      break;
      case 3:
        coordsys = DATA;
      break;
      default:
      break;
   }

   if (coordsys == DATA)
   {
      positionWidget->_xpos->enable_menu ();
      positionWidget->_xpos->set_mode (positionWidget->_xpos_format);
      positionWidget->_ypos->enable_menu ();
      positionWidget->_ypos->set_mode (positionWidget->_ypos_format);
   }
   else
   {
      positionWidget->_xpos_format = positionWidget->_xpos->get_mode ();
      positionWidget->_xpos->set_mode (0);
      positionWidget->_xpos->disable_menu ();
      positionWidget->_ypos_format = positionWidget->_ypos->get_mode ();
      positionWidget->_ypos->set_mode (0);
      positionWidget->_ypos->disable_menu ();
   }
 
   positionWidget->set_coordinate (coordsys);

   if (positionWidget->_callbacks.count ("change_coordinate") > 0)
      positionWidget->_callbacks["change_coordinate"]->execute (positionWidget, NULL);
}

void ChipsGuiPositionWidget::set_position (double x, double y)
{
   _xpos->set (x,-1);
   _ypos->set (y,-1);

}

void ChipsGuiPositionWidget::set_angle(float val)
{
   if (!_angle)
      return;

   _angle_val = val;
   g_signal_handler_block(GTK_OBJECT(_angle), _angle_change_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_angle), val);
   g_signal_handler_unblock(GTK_OBJECT(_angle), _angle_change_id);
}


void ChipsGuiPositionWidget::set_coordinate (int val)
{
   int combo_num = 0;
   _coordsys_val = val;

   switch ((COORD_SYS)val)
   {
      case PIXEL:
        combo_num = 0;
      break;
      case FRAME_NORM:
        combo_num = 1;
      break;
      case PLOT_NORM:
        combo_num = 2;
      break;
      case DATA:
        combo_num = 3;
      break;
      default:
      break;
   }

   g_signal_handler_block(GTK_OBJECT(_coordsys), _coordinate_change_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_coordsys), combo_num);
   g_signal_handler_unblock(GTK_OBJECT(_coordsys), _coordinate_change_id);

}

void ChipsGuiPositionWidget::set_format (int xformat, int yformat)
{
   _xpos_format = xformat;
   _ypos_format = yformat;
   

   if (_coordsys_val == DATA)
   {
      _xpos->set_mode (_xpos_format);
      _ypos->set_mode (_ypos_format);
   }
   else
   {
      _xpos->set_mode (0);
      _ypos->set_mode (0);
   }
   

}


void ChipsGuiPositionWidget::get_position (double &x, double &y)
{
   x = _xpos->get_numeric_value ();
   y = _ypos->get_numeric_value ();

}

float ChipsGuiPositionWidget::get_angle ()
{
   return _angle_val;
}

int ChipsGuiPositionWidget::get_coordinate ()
{
   return _coordsys_val;
}

void ChipsGuiPositionWidget::get_format (int &xformat, int &yformat)
{
   xformat = _xpos_format;
   yformat = _ypos_format;
}


   
