// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSGUIPOSITIONPOINTWIDGET_HH
#define CHIPSGUIPOSITIONPOINTWIDGET_HH

#include <gtk/gtk.h>

#include "ChipsGuiWidget.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

// valid callback ids are
//  * change_position
//  * change_coordinate
//  * change_angle

class ChipsGuiPositionWidget;
class ChipsGuiSpinWidget;

class ChipsGuiPositionWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiPositionWidgetCallback (void (*callback)(ChipsGuiPositionWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiPositionWidget *, void *);
     void *_callbackData;     
};

class ChipsGuiPositionWidget : public ChipsGuiWidget, public ChipsGuiTooltipObject
{
   public:

      ChipsGuiPositionWidget ();
      ~ChipsGuiPositionWidget ();

      virtual void build_form ();

      void build_form (bool include_angle);

      void set_position (double x, double y);
      void get_position (double &x, double &y);

      void set_angle (float angle);
      float get_angle ();

      void set_coordinate (int coordsys);
      int get_coordinate ();

      void set_format (int xformat, int yformat);
      void get_format (int &xformat, int &yformat);

      void disable_position ();
      void enable_position ();

      void disable_coordinate ();
      void enable_coordinate ();

      void disable_data_coordinate ();
      void enable_data_coordinate ();

      void disable_angle ();
      void enable_angle ();

      virtual void show ();
      virtual void hide ();

     
   protected:

      static void coordinate_change_callback(GtkWidget *widget, gpointer data);
      static void angle_change_callback(GtkWidget *widget, gpointer data);
      static void position_change_callback(ChipsGuiSpinWidget *widget, gpointer data);

      ChipsGuiSpinWidget *_xpos;
      ChipsGuiSpinWidget *_ypos;
      GtkWidget *_coordsys;
      GtkWidget *_angle;
      GtkListStore *_store;
      GtkTreeIter _iterD;

      unsigned int _xpos_change_id;
      unsigned int _ypos_change_id;
      unsigned int _xpos_focusout_id;
      unsigned int _ypos_focusout_id;
      unsigned int _coordinate_change_id;
      unsigned int _angle_change_id;

      double _xpos_val;
      double _ypos_val;
      float _angle_val;
      int _coordsys_val;
      int _xpos_format;
      int _ypos_format;

};

#endif

