// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiPrefFrame.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsId.hh"
#include "ChipsCmdManager.hh"

extern ChipsCmdManager *cmd_manager;

using namespace std;

extern ChipsGuiMain* chipsGui;

class ChipsSetEnginePrefCallback : public ChipsGuiPrefFrameCallback
{
   public:
      ChipsSetEnginePrefCallback (int object_type)
      {
         this->object_type = "";
         switch (object_type)
         {
            case chips_window:
               this->object_type = "window";
            break;
            case chips_frame:
               this->object_type = "frame";
            break;
            case chips_plot:
               this->object_type = "plot";
            break;
            case chips_curve:
               this->object_type = "curve";
            break;
            case chips_histogram:
               this->object_type = "histogram";
            break;
            case chips_contour:
               this->object_type = "contour";
            break;
            case chips_image:
               this->object_type = "image";
            break;
            case chips_colorbar:
               this->object_type = "colorbar";
            break;
            case chips_label:
               this->object_type = "label";
            break;
            case chips_axis:
               this->object_type = "axis";
            break;
            case chips_point:
               this->object_type = "point";
            break;
            case chips_line:
               this->object_type = "line";
            break;
            case chips_region:
               this->object_type = "region";
            break;
            default:
            break;
         }
      };
      virtual void execute (ChipsEngine *eng, void *data)
      {
         if ((data != NULL) && eng)
         {
            string preferences = *((string *)data);
            eng->override_preferences(preferences, this->object_type);
         }
      }

   protected:
      string object_type;
};


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiPrefFrame::ChipsGuiPrefFrame()
{
   _eng = NULL; 
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiPrefFrame::~ChipsGuiPrefFrame()
{
//  gtk_widget_destroy(_frame);
}

void ChipsGuiPrefFrame::build_form()
{
}

void ChipsGuiPrefFrame::show()
{
   gtk_widget_show_all(GTK_WIDGET(_frame));
}

void ChipsGuiPrefFrame::hide()
{
   gtk_widget_hide(GTK_WIDGET(_frame));
}

void ChipsGuiPrefFrame::update_property (ChipsGuiPrefFrameCallback *callback, void *data, bool ignore)
{
   if (!callback)
      return;

   bool bb = cmd_manager->get_ignore_state();
   cmd_manager->set_ignore_state(ignore);
   callback->execute (_eng, data);
   cmd_manager->set_ignore_state(bb);
}

void ChipsGuiPrefFrame::set_preference_override (const string &preference, int object_type, bool ignore)
{
   ChipsSetEnginePrefCallback callback (object_type);
   this->update_property (&callback, (void *)(&preference));
}

void ChipsGuiPrefFrame::update  ()
{
   if (cmd_manager->check_mask (_object_states))
   {
      cmd_manager->set_ignore_state (true);
      this->initialize ();
      cmd_manager->set_ignore_state (false);
   }

}

   
