// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiPropDialog.hh"
#include <string>
#include <fstream>
#include <iostream>


using namespace std;


ChipsGuiPropDialog::ChipsGuiPropDialog()
{
   _borrowed = NULL; 
   _oldparent = NULL; 
   generate_form();
}

ChipsGuiPropDialog::~ChipsGuiPropDialog()
{
}

void ChipsGuiPropDialog::generate_form()
{
   // we will actually add the close button separately below so we can
   // hook up our own callback 
   base_w = gtk_dialog_new_with_buttons("ChIPS Edit Properties", NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
   gtk_widget_set_size_request(base_w, 450, 400);
   _contain = GTK_DIALOG(base_w)->vbox; 

   g_signal_connect(G_OBJECT(base_w), "delete_event",
      G_CALLBACK(cleanup_cb), static_cast<void*> (this));

   // want to hide the window (not destroy it)- hook up a callback to capture
   // the [close] button press and use show_all instead of gtk_dialog_run
   GtkWidget *mybtn = gtk_dialog_add_button(GTK_DIALOG(base_w),
      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
   g_signal_connect(mybtn, "button-press-event",
      G_CALLBACK(close_button_cb), (void*)this);
}

int ChipsGuiPropDialog::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int ChipsGuiPropDialog::doit()
{
   gtk_window_set_modal(GTK_WINDOW(base_w), true); 
   return show(); 
}


void  ChipsGuiPropDialog::add_widget(GtkWidget* wid, GtkWidget* oldparent)
{
   if (wid)
   {
      gtk_container_add(GTK_CONTAINER(_contain), wid);
      _borrowed = wid; 

      // set the default dialog size and resize to handle larger widgets
      int wd = 450, ht = 400; 
      gtk_window_resize(GTK_WINDOW(base_w), wd, ht); 
      GtkRequisition req; 
      gtk_widget_size_request(GTK_WIDGET(wid), &req);
      if (req.width > wd)  wd = req.width; 
      if (req.height > ht)  ht = req.width; 
      gtk_widget_set_size_request(base_w, wd, ht);
   }
   if (oldparent) _oldparent = oldparent; 
}


void ChipsGuiPropDialog::remove_dialog()
{
   // reparent 
   if (_oldparent)
   {
      gtk_widget_ref(_borrowed);
      gtk_container_remove(GTK_CONTAINER(_contain), _borrowed);
      // reinsert into gui's table
      gtk_table_attach (GTK_TABLE(_oldparent), _borrowed, 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      gtk_widget_unref(_borrowed);
      _oldparent = NULL;
      _borrowed = NULL; 
   }

   gtk_window_set_modal(GTK_WINDOW(base_w), false); 
   if (base_w)  gtk_widget_hide(base_w);
}

// handle close button press in the dialog 
void ChipsGuiPropDialog::close_button_cb(GtkWidget *wid, GdkEventButton *event,
    gpointer data)
{
   ChipsGuiPropDialog *propDlg = static_cast<ChipsGuiPropDialog *>(data);
   if (propDlg)  propDlg->remove_dialog();    
}

bool ChipsGuiPropDialog::cleanup_cb(GtkWidget *wid, GdkEvent *event, gpointer data)
{
   ChipsGuiPropDialog *propDlg = static_cast<ChipsGuiPropDialog *>(data);
   if (propDlg)  propDlg->remove_dialog();
   return true; 
}

