// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiPropPane.hh"
#include "ChipsGuiPropDialog.hh"
#include "ChipsGuiAxisFrame.hh"
#include "ChipsGuiContourFrame.hh"
#include "ChipsGuiCurveFrame.hh"
#include "ChipsGuiFrameFrame.hh"
#include "ChipsGuiHistogramFrame.hh"
#include "ChipsGuiImageFrame.hh"
#include "ChipsGuiLabelFrame.hh"
#include "ChipsGuiLabelPrefFrame.hh"
#include "ChipsGuiLineFrame.hh"
#include "ChipsGuiLinePrefFrame.hh"
#include "ChipsGuiPlotFrame.hh"
#include "ChipsGuiPointFrame.hh"
#include "ChipsGuiPointPrefFrame.hh"
#include "ChipsGuiRegionFrame.hh"
#include "ChipsGuiRegionPrefFrame.hh"
#include "ChipsGuiWindowFrame.hh"
#include "ChipsGuiColorBarFrame.hh"
#include "ChipsGuiColorBarPrefFrame.hh"
#include "ChipsEventObserver.hh"
#include "ChipsEventManager.hh"
#include "ChipsGuiFontInfo.hh"
#include "ChipsCmdManager.hh"
#include "ChipsGuiMain.hh"


#define FGINDEX  1
#define BGINDEX  3 

extern ChipsCmdManager *cmd_manager;
extern ChipsEventManager* event_manager;
extern ChipsGuiMain* chipsGui; 

ChipsGuiFontInfo ChipsGuiPropPane::font_info;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiPropPane::ChipsGuiPropPane():ChipsEventObserver()
{
   AtkObject *accessible;

   GtkWidget *label = gtk_label_new (NULL);
   gtk_label_set_markup (GTK_LABEL(label), "Properties");
   _framew = gtk_frame_new(NULL);
   gtk_frame_set_label_widget (GTK_FRAME(_framew), label);
   
   _table = gtk_table_new (10,10, FALSE);
   _frames.clear(); 

   font_info.initialize ();

   _display_prefs = false;

   // add object property frames
   ChipsGuiPropFrame *prop = NULL; 
   if ((prop = new ChipsGuiLineFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_line, prop));
   }

   if ((prop = new ChipsGuiCurveFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_curve, prop));
   }
   if ((prop = new ChipsGuiImageFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_image, prop));
   }
   if ((prop = new ChipsGuiContourFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_contour, prop));
   }
   if ((prop = new ChipsGuiFrameFrame()) != NULL) 
   {
      prop->build_form(); 
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_frame, prop));
   }
   if ((prop = new ChipsGuiPlotFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_plot, prop));
   }
   if ((prop = new ChipsGuiHistogramFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_histogram, prop));
   }
   if ((prop = new ChipsGuiPointFrame()) != NULL)
   {
      prop->build_form(); 
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_point, prop));
   }
   if ((prop = new ChipsGuiRegionFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_region, prop));
   }
   if ((prop = new ChipsGuiLabelFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_label, prop));
   }
   if ((prop = new ChipsGuiAxisFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_axis, prop));
   }
   if ((prop = new ChipsGuiColorBarFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_colorbar, prop));
   }
   if ((prop = new ChipsGuiWindowFrame()) != NULL)
   {
      prop->build_form();
      gtk_table_attach (GTK_TABLE(_table), prop->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _frames.insert(frame_map::value_type (chips_window, prop));
   }

   // add object preference frames
   ChipsGuiPrefFrame *pref = NULL;
   if ((pref = new ChipsGuiPointPrefFrame()) != NULL)
   {
      pref->build_form();
      gtk_table_attach (GTK_TABLE(_table), pref->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _pref_frames.insert(pref_frame_map::value_type (chips_point, pref));
   }
   if ((pref = new ChipsGuiLinePrefFrame()) != NULL)
   {
      pref->build_form();
      gtk_table_attach (GTK_TABLE(_table), pref->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _pref_frames.insert(pref_frame_map::value_type (chips_line, pref));
   }
   if ((pref = new ChipsGuiRegionPrefFrame()) != NULL)
   {
      pref->build_form();
      gtk_table_attach (GTK_TABLE(_table), pref->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _pref_frames.insert(pref_frame_map::value_type (chips_region, pref));
   }
   if ((pref = new ChipsGuiLabelPrefFrame()) != NULL)
   {
      pref->build_form();
      gtk_table_attach (GTK_TABLE(_table), pref->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _pref_frames.insert(pref_frame_map::value_type (chips_label, pref));
   }
   if ((pref = new ChipsGuiColorBarPrefFrame()) != NULL)
   {
      pref->build_form();
      gtk_table_attach (GTK_TABLE(_table), pref->base_widget(), 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
      _pref_frames.insert(pref_frame_map::value_type (chips_colorbar, pref));
   }





   if (_framew)
   {
      gtk_table_attach (GTK_TABLE(_table), _framew, 1, 10, 1, 10,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL), 0, 0);
   }


   // register w/ event observer to get changes in selection
   register_event(ChipsEventManager::SelectEvent);
   register_event(ChipsEventManager::ChangeAddStateEvent);
   register_event(ChipsEventManager::ChangeStateEvent);
   set_pane(_type=chips_not_object, false);  
   _dialog = new ChipsGuiPropDialog();

   // add accessibility info
   accessible = gtk_widget_get_accessible (this->base_widget ());
   atk_object_set_name (accessible, "propertyPane");
   atk_object_set_description (accessible, "The base widget for the property pane");
      
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiPropPane::~ChipsGuiPropPane()
{
   gtk_widget_destroy(_framew);
   if (_dialog)
   {
      delete _dialog;
      _dialog = NULL; 
   }
   // delete all frames
   for(frame_map::iterator ii=_frames.begin(); ii!=_frames.end(); ++ii)
   {
      delete((*ii).second);
   }
   _frames.clear();

   for(pref_frame_map::iterator ii=_pref_frames.begin(); ii!=_pref_frames.end(); ++ii)
   {
      delete((*ii).second);
   }
   _pref_frames.clear();


}


void ChipsGuiPropPane::show()
{
   gtk_widget_show(GTK_WIDGET(_table));

   // hide all widgets (we only want to see active one) 
   for(frame_map::iterator ii=_frames.begin(); ii!=_frames.end(); ++ii)
   {
      ((*ii).second)->hide();
   }
   for(pref_frame_map::iterator ii=_pref_frames.begin(); ii!=_pref_frames.end(); ++ii)
   {
      ((*ii).second)->hide();
   }
   gtk_widget_hide (_framew);

   // show active widget 
   set_pane(_type, _display_prefs); 
}

void ChipsGuiPropPane::update ()
{
   if (cmd_manager->check_mask (STATE_CHNG_FONTS))
   {
      font_info.update_font_list ();
      cmd_manager->reset_mask (STATE_CHNG_FONTS);
   }

   if (!_display_prefs && _frames.count(_type))
   {
      cmd_manager->set_ignore_state (true);
      _frames[_type]->update ();
      cmd_manager->set_ignore_state (false);
   }
   else if (_display_prefs && _pref_frames.count(_type))
   {
      cmd_manager->set_ignore_state (true);
      _pref_frames[_type]->update ();
      cmd_manager->set_ignore_state (false);
   }


   // clear property pane if we no longer have the selected object
   if (!_display_prefs && event_manager && (_type != chips_not_object))
   {
      ChipsEventObjectInfo sob; 
      event_manager->get_selected(sob); 
      ChipsEngine* eng = chipsGui->get_engine();
      int cntval;
      // the event manager is setting the selected obj type to not_object
      // during pan/zoom- use our stored _type value to keep the property pane
      // set unless the object is removed. 
      //   eng->get_object_count(&sob.cid, sob.object_type, cntval);
      eng->get_object_count(&sob.cid, (V_OBJECT_TYPE)_type, cntval);
      if (cntval == 0)
      {
         set_pane(_type=chips_not_object, false);
         show(); 
      }
   }


}

void ChipsGuiPropPane::reparent_frame()
{
   GtkWidget* wid = NULL;

   if (_frames.count(_type))
   {
      wid = _frames[_type]->base_widget();
   }
   if (!wid)  return;   // don't have a frame for selected type 

   // reparent from dialog to gui
   GtkWidget* oparent = _dialog->get_container(); 
   gtk_widget_ref(wid);
   gtk_container_remove(GTK_CONTAINER(oparent), wid);
   gtk_container_add(GTK_CONTAINER(_table), wid);
   gtk_widget_unref(wid);
}

void ChipsGuiPropPane::display_dialog()
{
   if (!_dialog)  return; 

   font_info.update_font_list ();

   GtkWidget* wid = NULL;

   if (_frames.count(_type))
   {
      wid = _frames[_type]->base_widget();
   }
   if (!wid)  return;   // don't have a frame for selected type 

   // reparent to dialog
   gtk_widget_ref(wid);
   gtk_container_remove(GTK_CONTAINER(_table), wid);
   _dialog->add_widget(wid, _table);
   gtk_widget_unref(wid);
   if (_dialog)  _dialog->doit();
}

void ChipsGuiPropPane::hide()
{
   gtk_widget_hide(GTK_WIDGET(_table));
}

int ChipsGuiPropPane::get_height()
{
   int ht = 0;

   if (_framew)   ht = _framew->allocation.height;
   return ht;
}

void ChipsGuiPropPane::set_pane(int type, bool show_pref)
{
   // hide previously active frame 
   if (!_display_prefs && _frames.count(_type))
      _frames[_type]->hide();
   else if (_display_prefs && _pref_frames.count(_type))
      _pref_frames[_type]->hide();
   else 
      gtk_widget_hide(_framew);
   // display the new active frame 
   if (!show_pref && _frames.count(type))
      _frames[type]->show();
   else if (show_pref && _pref_frames.count(type))
      _pref_frames[type]->show();
   else
      gtk_widget_show(_framew);

   _display_prefs = show_pref;
   _type = type; 
}

// event observer to handle selection change from event manager
void ChipsGuiPropPane::execute_event (int event, void *data)
{
   int add_substate;
   bool change_state_event = false;

   if (event == ChipsEventManager::ChangeStateEvent)
   {
      int* states = (int*)data;
      if (states[1] == ChipsEventManager::AddState)
      {
         change_state_event = true;
         add_substate = event_manager->get_add_substate ();
      } 
      else if (states[0] == ChipsEventManager::AddState)
      {
         set_pane(chips_not_object, false);
      }
   }

   // Don't update the select during an AddState. This can cause the 
   // the plot to appear during a subselect
   if ((event == ChipsEventManager::SelectEvent) &&
       (event_manager->get_state () != ChipsEventManager::AddState))
   {
      ChipsEventObjectInfo* sel = (ChipsEventObjectInfo*)data;
      if (_frames.count(sel->object_type))
      {
         cmd_manager->set_ignore_state (true);
         _frames[sel->object_type]->initialize(&sel->cid);
     
         if (sel->subobject_type != chips_not_subobject)
            _frames[sel->object_type]->display_subobject (sel->subobject_type);
         set_pane(sel->object_type, false);
         cmd_manager->set_ignore_state (false);

         // since the frame is being initialized any previous
         // changes can be ignored
         if (cmd_manager)
            cmd_manager->reset_mask (_frames[_type]->get_masks ());
      }
      else 
      {
         set_pane(chips_not_object, false); 
      }
   }
   else if ((event == ChipsEventManager::ChangeAddStateEvent) ||
            change_state_event)
   {
      int object_type;
      int state;

      if (change_state_event)
         state = add_substate;
      else
         state = *((int *)data);
      switch (state)
      {
         case ChipsEventManager::LabelAdd:
            object_type = chips_label;
         break;
         case ChipsEventManager::PointAdd:
            object_type = chips_point;
         break;
         case ChipsEventManager::HLineAdd:
         case ChipsEventManager::VLineAdd:
         case ChipsEventManager::PolyLineAdd:
            object_type = chips_line;
         break;
         case ChipsEventManager::PolygonAdd:
         case ChipsEventManager::EllipseAdd:
         case ChipsEventManager::BoxAdd:
            object_type = chips_region;
         break;
         case ChipsEventManager::ColorBarAdd:
            object_type = chips_colorbar;
         break;
         default:
            object_type = chips_not_object;
         break;
      }
      if (_pref_frames.count(object_type))
      {
         cmd_manager->set_ignore_state (true);
         _pref_frames[object_type]->initialize();

         set_pane(object_type, true);
         cmd_manager->set_ignore_state (false);
         if (cmd_manager)
            cmd_manager->reset_mask (_pref_frames[_type]->get_masks ());
      }
      else
      {
         set_pane(chips_not_object, false);
      }
   }
}

