// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiRefMan.hh"
#include <gdk/gdkkeysyms.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <string>
#include <fstream>
#include <iostream>


using namespace std;

#define BUF_SIZE_PARSE 1024

#define SECTION_TAG     0
#define SUBSECTION_TAG  1
#define SUB2SECTION_TAG 2
#define SUB3SECTION_TAG 4


ChipsGuiRefMan::ChipsGuiRefMan()
{
   char *tmpdir = getenv("ASCDS_INSTALL");
   base_w = NULL; 
   pages.clear();
   if (tmpdir)  config_dir = tmpdir;
      else      config_dir  = ""; 
   config_dir.append("/");
   
   parse_document(); 
   hovering = false;
   hand_cursor = gdk_cursor_new (GDK_HAND2);
   regular_cursor = gdk_cursor_new (GDK_XTERM);
   generate_form();
}

ChipsGuiRefMan::~ChipsGuiRefMan()
{
   pages.clear();
}

void ChipsGuiRefMan::generate_form()
{
   // we will actually add the close button separately below so we can
   // hook up our own callback 
   base_w = gtk_dialog_new_with_buttons("ChIPS Gui Reference Manual", NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT, NULL);
   gtk_widget_set_size_request(base_w, 475, 450);
   sw = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);

   view = gtk_text_view_new();
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view), GTK_WRAP_WORD);
   gtk_text_view_set_editable(GTK_TEXT_VIEW (view), false);

   gtk_container_add (GTK_CONTAINER (sw), view);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), sw); 

   g_signal_connect(view, "key-press-event",
                    G_CALLBACK(key_press_event), this);
   g_signal_connect(view, "event-after",
                    G_CALLBACK(event_after), this);
   g_signal_connect(view, "motion-notify-event",
                    G_CALLBACK(motion_notify_event), this);
   g_signal_connect(view, "visibility-notify-event",
                    G_CALLBACK(vis_notify_event), this);
   g_signal_connect(G_OBJECT(base_w), "delete_event",
      G_CALLBACK(cleanup_cb), static_cast<void*> (this));

   display_page(GTK_TEXT_VIEW (view), 1);

   // since we want to run non-modal- hook up a callback to capture
   // the [close] button press and use show_all instead of gtk_dialog_run
   GtkWidget *mybtn = gtk_dialog_add_button(GTK_DIALOG(base_w),
      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
   g_signal_connect(mybtn, "button-press-event",
      G_CALLBACK(close_button_cb), (void*)this);
}

int ChipsGuiRefMan::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int ChipsGuiRefMan::doit()
{
   return show(); 
}

void ChipsGuiRefMan::remove_dialog()
{
   if (base_w)  gtk_widget_hide(base_w);
}

void ChipsGuiRefMan::parse_document()
{
   GtkTextIter toc_iter;
   GtkTextIter iter;
   string reffile = config_dir;
   reffile.append("config/chips/refmanual.tex");
   ifstream *infile =  new ifstream(reffile.c_str(), ios::in);
   char   buf[BUF_SIZE_PARSE];
   char   *bspos, *lbpos, *rbpos;
   string toc_txt, txt, tmp;
   int mode = SECTION_TAG;
   bool first_used = false, last_verb = false, verb = false;
   int pageno = 1;
   GtkTextBuffer *cur_page, *toc_page;

   if (pages.size() == 0)
   {
      toc_page = add_page();
      pages.insert(pages_map::value_type(1, toc_page));
      pageno++;
      cur_page = add_page();
      pages.insert(pages_map::value_type(pageno, cur_page));
   }

   gtk_text_buffer_set_text (cur_page, "", 0);
   gtk_text_buffer_get_iter_at_offset (cur_page, &iter, 0);
   add_controls(cur_page, &iter, pageno, 0);
   gtk_text_buffer_set_text (toc_page, "", 0);
   gtk_text_buffer_get_iter_at_offset (toc_page, &toc_iter, 0);
   add_controls(toc_page, &toc_iter, 1, 0);

   // start table of contents page
   gtk_text_buffer_insert_with_tags_by_name(toc_page, &toc_iter,
      "       Table of Contents\n", -1, "header", NULL);

   while (infile->getline(buf, BUF_SIZE_PARSE))
   {
      if (((bspos = strstr(buf, "\\")) != NULL) &&
          ((lbpos = strstr(buf, "{")) != NULL) &&
          ((rbpos = strstr(buf, "}")) != NULL))
      {
         // latex markup
         if (!strncmp(bspos,"\\section", 8))
         {
            // sections have page breaks
            if (first_used)
            {
               add_controls(cur_page, &iter, pageno++, 1);
               cur_page = add_page();
               pages.insert(pages_map::value_type(pageno, cur_page));
               gtk_text_buffer_set_text (cur_page, "", 0);
               gtk_text_buffer_get_iter_at_offset (cur_page, &iter, 0);
               add_controls(cur_page, &iter, pageno, 0);
            }

            // handle section
            *rbpos = '\0';
            tmp = lbpos +1;
            mode = SECTION_TAG;
            toc_txt = "\n";
            toc_txt.append(tmp);
            tmp.append("\n");
            toc_txt.append("\n\n");
            insert_link (toc_page, &toc_iter, toc_txt.c_str(), "subsection_mgn", pageno);
            gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
               tmp.c_str(), -1, "header", "section_mgn", NULL);
            first_used = true; 
         }
         else if (!strncmp(bspos,"\\subsection", 11))
         {
            // subsections have page breaks
            add_controls(cur_page, &iter, pageno++, 1);
            cur_page = add_page();
            pages.insert(pages_map::value_type(pageno, cur_page));
            gtk_text_buffer_set_text (cur_page, "", 0);
            gtk_text_buffer_get_iter_at_offset (cur_page, &iter, 0);
            add_controls(cur_page, &iter, pageno, 0);

            // handle subsection
            *rbpos = '\0';
            tmp = "\n";
            tmp.append(lbpos +1);
            tmp.append("\n");
            mode = SUBSECTION_TAG;
            toc_txt = lbpos +1;
            toc_txt.append("\n");

            insert_link (toc_page, &toc_iter, toc_txt.c_str(), "sub2section_mgn", pageno);
            gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
               tmp.c_str(), -1, "section", "subsection_mgn", NULL);
            first_used = true; 
         }
         else if (!strncmp(bspos,"\\subsubsection", 14))
         {
            // handle subsubsection
            *rbpos = '\0';
            tmp = "\n\n";
            tmp.append(lbpos +1);
            tmp.append("\n");
            mode = SUB2SECTION_TAG;
            gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
               tmp.c_str(), -1, "bold", "sub2section_mgn", NULL);
            first_used = true; 
         }
         else if (!strncmp(bspos,"\\begin{trivlist}", 16))
         {
            // handle heading for sub subsubsection
            bspos = &buf[17];
            if ((rbpos = strstr(bspos, "\\end{trivlist}")) != NULL)
            {
               *rbpos = '\0';
               tmp = "\n\n";
               if (((lbpos = strstr(bspos, "{")) != NULL) &&
                   ((rbpos = strstr(bspos, "}")) != NULL))
               {
                  *rbpos = '\0';
                  tmp.append(lbpos +1);
               }
               else
                  tmp.append(bspos);
               tmp.append("\n");
               mode = SUB3SECTION_TAG;
               gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
                  tmp.c_str(), -1, "italic", "sub3section_mgn", NULL);
            }
            first_used = true; 
         }
         else if (!strncmp(bspos,"\\begin{verbatim}", 16))
         {
            verb = true;
         }
         else if (!strncmp(bspos,"\\end{verbatim}", 14))
         {
            verb = false;
         }
         else if (!strncmp(bspos,"\\includegraphics", 16))
         {
            // add image to document
            *rbpos = '\0';
            string tname, fname = config_dir;
            fname.append(lbpos+1);
            if (mode == SUB3SECTION_TAG)  tname = "sub3section_mgn";
            else if (mode == SUB2SECTION_TAG)  tname = "sub2section_mgn";
            else if (mode == SUBSECTION_TAG)  tname = "subsection_mgn";
            else if (mode == SECTION_TAG)  tname = "subsection_mgn";
            GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(fname.c_str(),NULL);
            gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
               "\n\n ", -1, tname.c_str(), NULL);
            gtk_text_buffer_insert_pixbuf (cur_page, &iter, pixbuf);
            gtk_text_buffer_insert (cur_page, &iter,"\n",-1);
            first_used = true; 
         }
         else
         {
            // unsupported command- ignore latex line
         }
      }
      else if (buf[0] == '\\')
      {
         // unsupported command- ignore latex line
      }
      else
      {
         // if not verbatim- remove preceeding whitespace
         if (buf[0] != '\0')
         {
            char* bf = buf;
            if (!verb)  while (isspace(*bf))  bf++;
            if (last_verb != verb)  tmp = "\n";
               else                 tmp = "";
            tmp.append(bf);
            if (verb)  tmp.append("\n");
               else if (!isspace(tmp[tmp.length()-1]))  tmp.append(" "); 
            last_verb = verb;
            first_used = true; 
         }
         else 
            tmp = "\n"; 
         
         switch (mode)
         {
            case SECTION_TAG:
               gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
                  tmp.c_str(), -1, "section_mgn", NULL);
            break;
            case SUBSECTION_TAG:
               gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
                  tmp.c_str(), -1, "subsection_mgn", NULL);
            break;
            case SUB2SECTION_TAG:
               gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
                  tmp.c_str(), -1, "sub2section_mgn", NULL);
            break;
            case SUB3SECTION_TAG:
               gtk_text_buffer_insert_with_tags_by_name(cur_page, &iter,
                  tmp.c_str(), -1, "sub3section_mgn", NULL);
            break;
            default:
            break;
         }
      }
   }
   infile->close(); 
   delete infile;

   // add links to bottom of table of contents
   add_controls(toc_page, &toc_iter, 1, 1);
   // add links to bottom of last page
   add_controls(cur_page, &iter, pageno, 1, false);
}

void ChipsGuiRefMan::display_page(GtkTextView* tv, int page)
{
   if (pages.count(page) != 0)
   {
      GtkTextBuffer* buffer = pages[page];
      gtk_text_view_set_buffer(GTK_TEXT_VIEW(tv),
          GTK_TEXT_BUFFER(buffer));
   }
}

GtkTextBuffer* ChipsGuiRefMan::add_page()
{
   GtkTextBuffer* buffer = gtk_text_buffer_new(NULL);

   gtk_text_buffer_create_tag (buffer, "header", "scale", 
      PANGO_SCALE_X_LARGE, NULL);
   gtk_text_buffer_create_tag (buffer, "section", "scale", 
      PANGO_SCALE_LARGE, NULL);
   gtk_text_buffer_create_tag (buffer, "bold", "weight", 
      PANGO_WEIGHT_BOLD, NULL);
   gtk_text_buffer_create_tag (buffer, "italic", "style", 
      PANGO_STYLE_ITALIC, NULL);
   gtk_text_buffer_create_tag (buffer, "section_mgn",
      "left_margin", 10, "right_margin", 10, NULL);
   gtk_text_buffer_create_tag (buffer, "subsection_mgn",
      "left_margin", 30, "right_margin", 10, NULL);
   gtk_text_buffer_create_tag (buffer, "sub2section_mgn",
      "left_margin", 50, "right_margin", 10, NULL);
   gtk_text_buffer_create_tag (buffer, "sub3section_mgn",
      "left_margin", 50, "right_margin", 10, NULL);
   return buffer;
}

void ChipsGuiRefMan::insert_link (GtkTextBuffer *buffer, GtkTextIter *iter,
   const char *text, const char *fmt, int page)
{
   GtkTextTag *tag;
   static int tcnt = 0;

   if (fmt)
   {
      char buf[32];
      sprintf(buf,"link%1d", tcnt++);
      tag = gtk_text_buffer_create_tag (buffer, buf, "foreground", "blue",
         "underline", PANGO_UNDERLINE_SINGLE, NULL);
      g_object_set_data (G_OBJECT (tag), "page", GINT_TO_POINTER (page));
      gtk_text_buffer_insert_with_tags_by_name(buffer, iter, text, -1,
                                               buf, fmt, NULL);
   }
   else
   {
      tag = gtk_text_buffer_create_tag (buffer, NULL, "foreground", "blue",
         "underline", PANGO_UNDERLINE_SINGLE, NULL);
      g_object_set_data (G_OBJECT (tag), "page", GINT_TO_POINTER (page));
      gtk_text_buffer_insert_with_tags (buffer, iter, text, -1, tag, NULL);
   }
}

// add links for  prev - toc - next to page 
// loc = 0 (top), 1 = bottom,  
void ChipsGuiRefMan::add_controls(GtkTextBuffer *buffer, GtkTextIter* iter,
   int page, int loc, bool next)
{
   if (loc > 0)  // at bottom - precede w/ blank lines
   gtk_text_buffer_insert (buffer, iter, "\n\n\n\n\n", -1);

   gtk_text_buffer_insert (buffer, iter, "  ", -1);
   if (page > 1)
   {
      insert_link (buffer, iter, "Previous", NULL, page-1);
      gtk_text_buffer_insert (buffer, iter, "                        ", -1);
      insert_link (buffer, iter, "Table of Contents", NULL, 1);
      gtk_text_buffer_insert (buffer, iter, "                        ", -1);
      if (next)
         insert_link (buffer, iter, "Next", NULL, page+1);
      else      
         gtk_text_buffer_insert (buffer, iter, "Next", -1);
   }
   else 
   {
      gtk_text_buffer_insert (buffer, iter, "Previous                        Table of Contents                        ", -1);
      if (next)
         insert_link (buffer, iter, "Next", NULL, page+1);
      else      
         gtk_text_buffer_insert (buffer, iter, "Next", -1);
   }
   if (loc == 0)   // at top - follow with a blank line
      gtk_text_buffer_insert (buffer, iter, "\n\n", -1);
}


// Looks at all tags covering the position of iter in the text view,
// and if one of them is a link, follow it by showing the page identified
// by the data attached to it.
void ChipsGuiRefMan::follow_if_link(GtkWidget *text_view, GtkTextIter *iter)
{
   GSList *tags, *tagp;

   tags = gtk_text_iter_get_tags(iter);
   for (tagp = tags; tagp != NULL; tagp = tagp->next)
   {
      GtkTextTag *tag = GTK_TEXT_TAG(tagp->data);
      int page = GPOINTER_TO_INT(g_object_get_data(G_OBJECT (tag), "page"));

      if (page != 0)
      {
         display_page(GTK_TEXT_VIEW(text_view), page);
         break;
      }
   }
   if (tags)  g_slist_free (tags);
}

// activate link if enter pressed while over link 
bool ChipsGuiRefMan::key_press_event (GtkWidget *text_view, GdkEventKey *event,
   gpointer data)
{
   ChipsGuiRefMan *refman = static_cast<ChipsGuiRefMan *>(data);

   GtkTextIter iter;
   GtkTextBuffer *buffer;

   switch (event->keyval)
   {
      case GDK_Return:    // fallthrough intended 
      case GDK_KP_Enter:
      {
         buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
         gtk_text_buffer_get_iter_at_mark (buffer, &iter,
            gtk_text_buffer_get_insert (buffer));
         refman->follow_if_link (text_view, &iter);
      }
      break;
      default:
      break;
   }
   return false;
}

// activate link if mouse click on link
bool ChipsGuiRefMan::event_after(GtkWidget *text_view, GdkEventKey *ev,
   gpointer data)
{
   ChipsGuiRefMan *refman = static_cast<ChipsGuiRefMan *>(data);

   GtkTextIter start, end, iter;
   GtkTextBuffer *buffer;
   GdkEventButton *event;
   int x, y;

   if (ev->type != GDK_BUTTON_RELEASE)  return false;

   event = (GdkEventButton *)ev;
   
   if (event->button != 1)  return false;

   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));

   // we shouldn't follow a link if the user has selected something 
   gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
   if (gtk_text_iter_get_offset(&start) != gtk_text_iter_get_offset(&end))
       return false;

   gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW (text_view),
      GTK_TEXT_WINDOW_WIDGET, (int)event->x, (int)event->y, &x, &y);
   gtk_text_view_get_iter_at_location(GTK_TEXT_VIEW(text_view), &iter, x, y);
   refman->follow_if_link (text_view, &iter);

   return false;
}

void ChipsGuiRefMan::motion_notify_event(GtkWidget *text_view, 
   GdkEventMotion *event, gpointer data)
{
   ChipsGuiRefMan *refman = static_cast<ChipsGuiRefMan *>(data);
   int x, y;

   gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view),
                                         GTK_TEXT_WINDOW_WIDGET,
                                         (int)event->x, (int)event->y, &x, &y);
   refman->set_cursor(GTK_TEXT_VIEW(text_view), x, y);
   gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
}

void ChipsGuiRefMan::vis_notify_event (GtkWidget *text_view, 
   GdkEventVisibility *event, gpointer data)
{
   ChipsGuiRefMan *refman = static_cast<ChipsGuiRefMan *>(data);
   int wx, wy, bx, by;

   gdk_window_get_pointer(text_view->window, &wx, &wy, NULL);
   gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW (text_view),
                                         GTK_TEXT_WINDOW_WIDGET,
                                         wx, wy, &bx, &by);
   refman->set_cursor(GTK_TEXT_VIEW(text_view), bx, by);

}

void ChipsGuiRefMan::set_cursor(GtkTextView *text_view, int x, int y)
{
   GSList *tags = NULL, *tagp = NULL;
   GtkTextBuffer *buffer;
   GtkTextIter iter;
   bool hover = false;

   buffer = gtk_text_view_get_buffer(text_view);
   gtk_text_view_get_iter_at_location(text_view, &iter, x, y);

   tags = gtk_text_iter_get_tags (&iter);
   for (tagp = tags;  tagp != NULL;  tagp = tagp->next)
   {
      GtkTextTag *tag = GTK_TEXT_TAG(tagp->data);
      int page = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tag), "page"));

      if (page != 0)
      {
         hover = true;
         break;
      }
   }
   if (hover != hovering)
   {
      if ((hovering = hover) == true)
         gdk_window_set_cursor(gtk_text_view_get_window(text_view,
            GTK_TEXT_WINDOW_TEXT), hand_cursor);
      else
         gdk_window_set_cursor(gtk_text_view_get_window (text_view,
            GTK_TEXT_WINDOW_TEXT), regular_cursor);
   }
   if (tags)    g_slist_free (tags);
}

// handle close button press in the dialog 
void ChipsGuiRefMan::close_button_cb(GtkWidget *wid, GdkEventButton *event,
    gpointer data)
{
   ChipsGuiRefMan *refman = static_cast<ChipsGuiRefMan *>(data);
   if (refman)  refman->remove_dialog();    
}

bool ChipsGuiRefMan::cleanup_cb(GtkWidget *wid, GdkEvent *event, gpointer data)
{
   ChipsGuiRefMan *refman = static_cast<ChipsGuiRefMan *>(data);
   if (refman)  refman->remove_dialog();
   return true; 
}

