// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiRegionFrame.hh"
#include "ChipsGuiRegionPropWidget.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsRegion.hh"
#include "ChipsCmdManager.hh"

#include <algorithm>

using namespace std;

extern ChipsGuiMain* chipsGui;

class ChipsRegionMoveCallback : public ChipsGuiPropFrameCallback
{
   public:
      virtual void execute (ChipsEngine *eng, ChipsId &cid, void *data)
      {
         double *pos = static_cast<double *>(data);
         if ((pos != NULL) && eng)
            eng->move(&cid, chips_region, pos[0], pos[1], V_MOVE_ABSOLUTE);
      }
};

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiRegionFrame::ChipsGuiRegionFrame():ChipsGuiPropFrame(),ChipsGuiTooltipObject()
{
   _regionprop = NULL;
   _object_states = STATE_CHNG_PROP_RGN;
   _object_states |= STATE_CHNG_AXS_FMT;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiRegionFrame::~ChipsGuiRegionFrame()
{
   if (_regionprop)
      delete _regionprop;
}

void ChipsGuiRegionFrame::build_form()
{
   _frame = gtk_frame_new("Region Properties");

   _regionprop = new ChipsGuiRegionPropWidget ();
   _regionprop->build_form ();

   gtk_container_add(GTK_CONTAINER(_frame), _regionprop->base_widget ());

   _regionprop->add_callback ("change_linecolor", new ChipsGuiRegionPropWidgetCallback (linecolor_callback, this));
   _regionprop->add_callback ("change_position", new ChipsGuiRegionPropWidgetCallback (position_callback, this));
/*   _regionprop->add_callback ("change_angle", new ChipsGuiRegionPropWidgetCallback (angle_callback, this));
 */
   _regionprop->add_callback ("change_coordinate", new ChipsGuiRegionPropWidgetCallback (coordinate_callback, this));
   _regionprop->add_callback ("change_linestyle", new ChipsGuiRegionPropWidgetCallback (linestyle_callback, this));
   _regionprop->add_callback ("change_depth", new ChipsGuiRegionPropWidgetCallback (depth_callback, this));
   _regionprop->add_callback ("change_linethickness", new ChipsGuiRegionPropWidgetCallback (linethick_callback, this));
   _regionprop->add_callback ("change_fillstyle", new ChipsGuiRegionPropWidgetCallback (fillstyle_callback, this));
   _regionprop->add_callback ("change_fillcolor", new ChipsGuiRegionPropWidgetCallback (fillcolor_callback, this));
   _regionprop->add_callback ("change_visibility", new ChipsGuiRegionPropWidgetCallback (vis_callback, this));
   _regionprop->add_callback ("change_opacity", new ChipsGuiRegionPropWidgetCallback (opacity_callback, this));

}

void ChipsGuiRegionFrame::set_linecolor(const string &color)
{
   set_engine_property((void*)color.c_str(), SET_REGN_EDGE_COLOR); 
}

void ChipsGuiRegionFrame::set_position(double xpos, double ypos)
{
   ChipsRegionMoveCallback move_callback;
   double pos[2] = {xpos, ypos};
   this->update_property (&move_callback, &pos[0]);
}

void ChipsGuiRegionFrame::set_coordinate(int coordinate)
{
   ChipsId cid;
   COORD_SYS coordsys = (COORD_SYS)coordinate;
   string coordid = "";

   _eng->get_current_coord (&cid);

   if ((coordsys == DATA) && (cid.get_coord_sys () != DATA))
      chipsGui->log_error ("Data coordinate system does not exists.");
   else
      coordid = cid.get_coord_id ();

   coordid = ChipsCoordinate::extract_id (coordsys, coordid);

   _eng->change_annotation_coordinate (&cid, chips_region, coordsys, coordid);
}


void ChipsGuiRegionFrame::set_linethick(float val)
{
   set_engine_property(static_cast<void*>(&val), 
      SET_REGN_EDGE_THICKNESS); 
}

void ChipsGuiRegionFrame::set_linestyle(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_REGN_EDGE_STYLE);
}

void ChipsGuiRegionFrame::set_angle(float val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_REGN_ANGLE);
}

void ChipsGuiRegionFrame::set_visibility(bool val)
{
   int ival = val; 
   set_engine_property(static_cast<void*>(&ival),
      SET_REGN_VISIBLE);
}

void ChipsGuiRegionFrame::set_depth(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_REGN_DEPTH);
}

void ChipsGuiRegionFrame::set_opacity(float val)
{
   double dval = val; 
   set_engine_property(static_cast<void*>(&dval),
      SET_REGN_OPACITY);
}

void ChipsGuiRegionFrame::set_fillcolor(const string &color)
{
   set_engine_property((void*)color.c_str(), SET_REGN_FILL_COLOR); 
}

void ChipsGuiRegionFrame::set_fillstyle(int val)
{
   set_engine_property(static_cast<void*>(&val),
      SET_REGN_FILL_STYLE);
}

void ChipsGuiRegionFrame::linecolor_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);

   if (!crvFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_linecolor ();
   crvFrame->set_linecolor(clr);
}

void ChipsGuiRegionFrame::position_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *regionFrame = static_cast <ChipsGuiRegionFrame*>(data);
   double x,y;

   if (!regionFrame)  return; //ensure we have needed pointer
   widget->get_position (x,y);
   regionFrame->set_position(x,y);
}
void ChipsGuiRegionFrame::coordinate_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *regionFrame = static_cast <ChipsGuiRegionFrame*>(data);

   if (!regionFrame)  return; //ensure we have needed pointer
   int coordinate = widget->get_coordinate ();
   regionFrame->set_coordinate (coordinate);
}

void ChipsGuiRegionFrame::angle_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *regionFrame = static_cast <ChipsGuiRegionFrame*>(data);

   if (!regionFrame)  return; //ensure we have needed pointer
   float angle = widget->get_angle ();
   regionFrame->set_angle (angle);
}

void ChipsGuiRegionFrame::linestyle_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);
   int val = widget->get_linestyle ();
   if (crvFrame)  crvFrame->set_linestyle(val); 
}

void ChipsGuiRegionFrame::linethick_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);
   float fval = widget->get_linethick ();
   if (crvFrame)  crvFrame->set_linethick(fval);
}

void ChipsGuiRegionFrame::vis_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);
   bool val = widget->get_visibility ();
   if (crvFrame)  crvFrame->set_visibility(val);
}

void ChipsGuiRegionFrame::depth_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);
   int val = widget->get_depth ();
   if (crvFrame)  crvFrame->set_depth(val); 
}

void ChipsGuiRegionFrame::fillcolor_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);

   if (!crvFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_fillcolor ();
   crvFrame->set_fillcolor(clr);
}

void ChipsGuiRegionFrame::opacity_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);
   float val = widget->get_opacity ();
   if (crvFrame)  crvFrame->set_opacity(val);
}

void ChipsGuiRegionFrame::fillstyle_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionFrame *crvFrame = static_cast <ChipsGuiRegionFrame*>(data);
   int val = widget->get_fillstyle ();
   if (crvFrame)  crvFrame->set_fillstyle(val);
}

void ChipsGuiRegionFrame::initialize(ChipsId* id)
{
   COORD_SYS coordsys;
   string coordid;
   ChipsVector pos(3);
   string linecolor_val;
   float linethick_val;
   int depth_val;
   V_FILL_TYPE fillstyle_val;
   string fillcolor_val;
   double opacity_val;
   bool vis_val;
   float angle_val;
   V_LINE_TYPE linestyle_val;

   if (!id)  return; 

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   if (&_cid != id)
   {
      _cid.reset_window_id(id->get_window_id()); 
      _cid.set_frame_id(id->get_frame_id()); 
      _cid.set_plot_id(id->get_plot_id()); 
      _cid.set_region_id(id->get_region_id()); 
   }

   // get values
   _eng->get_property(&_cid, (void*)&linecolor_val, SET_REGN_EDGE_COLOR);
   _eng->get_property(&_cid, (void*)&linethick_val, SET_REGN_EDGE_THICKNESS);
   _eng->get_property(&_cid, static_cast<void*>(&linestyle_val), 
      SET_REGN_EDGE_STYLE);
   _eng->get_property(&_cid, (void*)&depth_val, SET_REGN_DEPTH);

   _eng->get_property(&_cid, (void*)&angle_val, SET_REGN_ANGLE);
   _eng->get_property(&_cid, (void*)&vis_val, SET_REGN_VISIBLE);
   _eng->get_property(&_cid, static_cast<void*>(&opacity_val),
      SET_REGN_OPACITY);
   _eng->get_property(&_cid, (void*)&fillcolor_val, SET_REGN_FILL_COLOR);
   _eng->get_property(&_cid, static_cast<void*>(&fillstyle_val),
      SET_REGN_FILL_STYLE);
   _eng->get_object_coordinfo (&_cid, chips_region, coordsys, coordid);
   _eng->get_annotation_position (&_cid, chips_region, pos);


   _regionprop->set_linecolor (linecolor_val);
   _regionprop->set_coordinate ((int)coordsys);
   _regionprop->set_angle (angle_val);
   _regionprop->set_position (pos[0],pos[1]);
   _regionprop->set_fillstyle (fillstyle_val);
   _regionprop->set_fillcolor (fillcolor_val);
   _regionprop->set_linethick (linethick_val);
   _regionprop->set_linestyle (linestyle_val);
   _regionprop->set_opacity (opacity_val);
   _regionprop->set_depth (depth_val);
   _regionprop->set_visibility (vis_val);

   // get tick formats
   int xmode = 0, ymode = 0;
   string xformat, yformat;
   ChipsId tmpid (_cid);
   tmpid.set_axis_id(_cid.get_xaxis_id());
   tmpid.set_axis_dim (X_AXIS);
   _eng->get_property(&tmpid, (void*)&xformat, SET_AXIS_TICKFORMAT);
   tmpid.set_axis_id(_cid.get_yaxis_id());
   tmpid.set_axis_dim (Y_AXIS);
   _eng->get_property(&tmpid, (void*)&yformat, SET_AXIS_TICKFORMAT);

   transform(xformat.begin(), xformat.end(), xformat.begin(), ::tolower);
   transform(yformat.begin(), yformat.end(), yformat.begin(), ::tolower);

   if (xformat.find("ra") != string::npos ||
       xformat.find("dec") != string::npos)
      xmode = 1;
   if (yformat.find("ra") != string::npos ||
       yformat.find("dec") != string::npos)
      ymode = 1;

   _regionprop->set_position_format (xmode, ymode);
   
   // enable/disable the data coord pulldown entry based on existing coords
   ChipsId mcid;
   mcid.set_coord_sys(DATA);
   if (_eng->check_coordinate_exists(&mcid))  
      _regionprop->enable_data_coordinate();
   else 
      _regionprop->disable_data_coordinate();

}

