// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiRegionPrefFrame.hh"
#include "ChipsGuiRegionPropWidget.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsRegion.hh"
#include "ChipsCmdManager.hh"

#include <algorithm>

using namespace std;

extern ChipsGuiMain* chipsGui;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiRegionPrefFrame::ChipsGuiRegionPrefFrame():ChipsGuiPrefFrame(),ChipsGuiTooltipObject()
{
   _regionprop = NULL;
   _object_states = STATE_CHNG_PROP_RGN;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiRegionPrefFrame::~ChipsGuiRegionPrefFrame()
{
   if (_regionprop)
      delete _regionprop;
}

void ChipsGuiRegionPrefFrame::build_form()
{
   _frame = gtk_frame_new("Region Properties");

   _regionprop = new ChipsGuiRegionPropWidget ();
   _regionprop->build_form ();

   gtk_container_add(GTK_CONTAINER(_frame), _regionprop->base_widget ());


   _regionprop->disable_position ();
   _regionprop->disable_coordinate ();
   _regionprop->disable_visibility ();


   _regionprop->add_callback ("change_linecolor", new ChipsGuiRegionPropWidgetCallback (linecolor_callback, this));
/*   _regionprop->add_callback ("change_angle", new ChipsGuiRegionPropWidgetCallback (angle_callback, this));
 */
/*
   _regionprop->add_callback ("change_coordinate", new ChipsGuiRegionPropWidgetCallback (coordinate_callback, this));
*/
   _regionprop->add_callback ("change_linestyle", new ChipsGuiRegionPropWidgetCallback (linestyle_callback, this));
   _regionprop->add_callback ("change_depth", new ChipsGuiRegionPropWidgetCallback (depth_callback, this));
   _regionprop->add_callback ("change_linethickness", new ChipsGuiRegionPropWidgetCallback (linethick_callback, this));
   _regionprop->add_callback ("change_fillstyle", new ChipsGuiRegionPropWidgetCallback (fillstyle_callback, this));
   _regionprop->add_callback ("change_fillcolor", new ChipsGuiRegionPropWidgetCallback (fillcolor_callback, this));
   _regionprop->add_callback ("change_opacity", new ChipsGuiRegionPropWidgetCallback (opacity_callback, this));

}

void ChipsGuiRegionPrefFrame::set_linecolor(const string &color)
{
   stringstream pref;
   pref << "edge.color=" << color;
   set_preference_override(pref.str (), chips_region);

   // Also set the line color since this is currently how 
   // the region is drawn
   pref.str ("");
   pref << "color=" << color;
   set_preference_override(pref.str (), chips_line);
}

/*
void ChipsGuiRegionPrefFrame::set_coordinate(int coordinate)
{
   ChipsId cid;
   COORD_SYS coordsys = (COORD_SYS)coordinate;
   string coordid = "";

   _eng->get_current_coord (&cid);

   if ((coordsys == DATA) && (cid.get_coord_sys () != DATA))
      chipsGui->log_error ("Data coordinate system does not exists.");
   else
      coordid = cid.get_coord_id ();

   coordid = ChipsCoordinate::extract_id (coordsys, coordid);

   _eng->change_annotation_coordinate (&cid, chips_region, coordsys, coordid);
}
*/


void ChipsGuiRegionPrefFrame::set_linethick(float val)
{
   stringstream pref;
   pref << "edge.thickness=" << val;
   set_preference_override(pref.str (), chips_region);

   // Also set the line thickness since this is currently how 
   // the region is drawn
   pref.str ("");
   pref << "thickness=" << val;
   set_preference_override(pref.str (), chips_line);

}

void ChipsGuiRegionPrefFrame::set_linestyle(int val)
{
   stringstream pref;
   pref << "edge.style=" << val;
   set_preference_override(pref.str (), chips_region);

   // Also set the line style since this is currently how 
   // the region is drawn
   pref.str ("");
   pref << "style=" << val;
   set_preference_override(pref.str (), chips_line);

}

void ChipsGuiRegionPrefFrame::set_angle(float val)
{
   stringstream pref;
   pref << "angle=" << val;
   set_preference_override(pref.str (), chips_region);
}

void ChipsGuiRegionPrefFrame::set_depth(int val)
{
   stringstream pref;
   pref << "depth=" << val;
   set_preference_override(pref.str (), chips_region);
}

void ChipsGuiRegionPrefFrame::set_opacity(float val)
{
   stringstream pref;
   pref << "opacity=" << val;
   set_preference_override(pref.str (), chips_region);
}

void ChipsGuiRegionPrefFrame::set_fillcolor(const string &color)
{
   stringstream pref;
   pref << "fill.color=" << color;
   set_preference_override(pref.str (), chips_region);
}

void ChipsGuiRegionPrefFrame::set_fillstyle(int val)
{
   stringstream pref;
   pref << "fill.style=" << val;
   set_preference_override(pref.str (), chips_region);
}

void ChipsGuiRegionPrefFrame::linecolor_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *regionFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);

   if (!regionFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_linecolor ();
   regionFrame->set_linecolor(clr);
}

/*
void ChipsGuiRegionPrefFrame::coordinate_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *pntFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);

   if (!pntFrame)  return; //ensure we have needed pointer
   int coordinate = widget->get_coordinate ();
   pntFrame->set_coordinate (coordinate);
}
*/

void ChipsGuiRegionPrefFrame::angle_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *pntFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);

   if (!pntFrame)  return; //ensure we have needed pointer
   float angle = widget->get_angle ();
   pntFrame->set_angle (angle);
}

void ChipsGuiRegionPrefFrame::linestyle_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *regionFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);
   int val = widget->get_linestyle ();
   if (regionFrame)  regionFrame->set_linestyle(val); 
}

void ChipsGuiRegionPrefFrame::linethick_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *regionFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);
   float fval = widget->get_linethick ();
   if (regionFrame)  regionFrame->set_linethick(fval);
}

void ChipsGuiRegionPrefFrame::depth_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *regionFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);
   int val = widget->get_depth ();
   if (regionFrame)  regionFrame->set_depth(val); 
}

void ChipsGuiRegionPrefFrame::fillcolor_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *regionFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);

   if (!regionFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_fillcolor ();
   regionFrame->set_fillcolor(clr);
}

void ChipsGuiRegionPrefFrame::opacity_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *regionFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);
   float val = widget->get_opacity ();
   if (regionFrame)  regionFrame->set_opacity(val);
}

void ChipsGuiRegionPrefFrame::fillstyle_callback(ChipsGuiRegionPropWidget *widget, void *data)
{
   ChipsGuiRegionPrefFrame *regionFrame = static_cast <ChipsGuiRegionPrefFrame*>(data);
   int val = widget->get_fillstyle ();
   if (regionFrame)  regionFrame->set_fillstyle(val);
}

void ChipsGuiRegionPrefFrame::initialize()
{
   ChipsId cid;
   COORD_SYS coordsys;
   string coordid;
   ChipsVector pos(3);
   string linecolor_val;
   int linethick_val;
   int depth_val;
   V_FILL_TYPE fillstyle_val;
   string fillcolor_val;
   double opacity_val;
   float angle_val;
   V_LINE_TYPE linestyle_val;
   char *pref_cstr;

   if (chipsGui && !_eng) 
      _eng = chipsGui->get_engine(); 

   // get values
   if ((pref_cstr = _eng->get_preference ("region.edge.color")) != NULL)
   {
      linecolor_val = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("region.edge.thickness")) != NULL)
   {
      stringstream(pref_cstr) >> linethick_val;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("region.depth")) != NULL)
   {
      if (chips_strcmp (pref_cstr, "default") == 0)
      {
         free (pref_cstr);
         if ((pref_cstr = _eng->get_preference ("default.depth")) == NULL)
            depth_val = V_DEFAULT_DEPTH;
      }

      if (pref_cstr)
      {
         stringstream(pref_cstr) >> depth_val;
         free (pref_cstr);
      }
   }

   if ((pref_cstr = _eng->get_preference ("region.edge.style")) != NULL)
   {
      c2_string_to_line (pref_cstr, &linestyle_val);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("region.fill.color")) != NULL)
   {
      fillcolor_val = pref_cstr;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("region.fill.style")) != NULL)
   {
      c2_string_to_fill (pref_cstr, &fillstyle_val);
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("region.angle")) != NULL)
   {
      stringstream(pref_cstr) >> angle_val;
      free (pref_cstr);
   }

   if ((pref_cstr = _eng->get_preference ("region.opacity")) != NULL)
   {
      stringstream(pref_cstr) >> opacity_val;
      free (pref_cstr);
   }

   _eng->get_object_coordinfo (&cid, chips_region, coordsys, coordid);

   _regionprop->set_linecolor (linecolor_val);
   _regionprop->set_coordinate ((int)coordsys);
   _regionprop->set_angle (angle_val);
   _regionprop->set_position (pos[0],pos[1]);
   _regionprop->set_fillstyle (fillstyle_val);
   _regionprop->set_fillcolor (fillcolor_val);
   _regionprop->set_linethick (linethick_val);
   _regionprop->set_linestyle (linestyle_val);
   _regionprop->set_opacity (opacity_val);
   _regionprop->set_depth (depth_val);
   _regionprop->set_visibility (true);

}

