// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiRegionPropWidget.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiPositionWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsRegion.hh"
#include "ChipsCmdManager.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;

ChipsGuiRegionPropWidgetCallback::ChipsGuiRegionPropWidgetCallback (void (*callback)(ChipsGuiRegionPropWidget *, void *), void *data)
{
   _callback = callback;
   _callbackData = data;

}

void ChipsGuiRegionPropWidgetCallback::execute (ChipsGuiWidget *caller, void *)
{
   ChipsGuiRegionPropWidget *regionWidget = dynamic_cast<ChipsGuiRegionPropWidget *>(caller);
   _callback (regionWidget, _callbackData);
}



// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiRegionPropWidget::ChipsGuiRegionPropWidget():ChipsGuiWidget(),ChipsGuiTooltipObject()
{

   _valid_callback_ids.push_back ("change_linecolor");
   _valid_callback_ids.push_back ("change_linethickness");
   _valid_callback_ids.push_back ("change_position");
   _valid_callback_ids.push_back ("change_coordinate");
//   _valid_callback_ids.push_back ("change_angle");
   _valid_callback_ids.push_back ("change_linestyle");
   _valid_callback_ids.push_back ("change_depth");
   _valid_callback_ids.push_back ("change_opacity");
   _valid_callback_ids.push_back ("change_fillcolor");
   _valid_callback_ids.push_back ("change_fillstyle");
   _valid_callback_ids.push_back ("change_visibility");

   _linecolor = NULL;
   _fillcolor = NULL;
   _position = NULL;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiRegionPropWidget::~ChipsGuiRegionPropWidget()
{
   if (_linecolor)
      delete _linecolor;
   if (_fillcolor)
      delete _fillcolor;
   if (_position)
      delete _position;
}

void ChipsGuiRegionPropWidget::build_form()
{
   GtkWidget* tmp; 
   GtkWidget* position_frame;
   _base_widget = _nbook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_nbook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_nbook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_nbook), GTK_POLICY_AUTOMATIC);

   _table1 = gtk_table_new(8,7,FALSE);
   _tabline = gtk_table_new(7,7,FALSE);
   _tabfill = gtk_table_new(7,7,FALSE);
   _frmline = gtk_frame_new("Edges");
   gtk_frame_set_label_align(GTK_FRAME(_frmline), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmline), _tabline);
   gtk_table_attach(GTK_TABLE(_table1), _frmline, 1, 7, 1, 3,
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 10, 10);

   _frmfill = gtk_frame_new("Fill");
   gtk_frame_set_label_align(GTK_FRAME(_frmfill), .5, .5);
   gtk_container_add (GTK_CONTAINER (_frmfill), _tabfill);
   gtk_table_attach(GTK_TABLE(_table1), _frmfill, 1, 7, 3, 5,
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 10, 10);
   tmp = gtk_label_new("Edges / Fill");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table1, tmp);

   _table4 = gtk_table_new(7,7,FALSE);
   tmp = gtk_label_new("General");
   gtk_notebook_append_page(GTK_NOTEBOOK(_nbook), _table4, tmp);

   // line frame
   _linecolor = new ChipsGuiColorWidget();
   _linecolor->build_form ();
   _linestyle = gtk_combo_box_new_text();
   _linethick = gtk_spin_button_new_with_range(0.5, 10, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_linethick), 1);
   gtk_table_attach(GTK_TABLE(_tabline), _linecolor->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabline), _linestyle, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "no line");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "solid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "dot");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "dot longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "dot shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "longdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "shortdash");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_linestyle), "shortdash longdash");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_linestyle),1);
   tmp = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_tabline), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_tabline), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabline), _linethick, 2, 3, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Thickness");
   gtk_table_attach(GTK_TABLE(_tabline), tmp, 1, 2, 3, 4,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
 
   // fill frame
   _fillcolor = new ChipsGuiColorWidget();
   _fillcolor->build_form ();
   _fillstyle = gtk_combo_box_new_text();
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "no fill");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "solid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "up diagonal");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "down diagonal");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "horizontal");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "vertical");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "crisscross");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "brick");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "grid");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "hexagon");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "polkadot");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "zigzag");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "wave");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "flower");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "custom 1");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "custom 2");
   gtk_combo_box_append_text(GTK_COMBO_BOX(_fillstyle), "custom 3");

   _opacity = gtk_hscale_new_with_range(0,1,.01);
   gtk_scale_set_digits(GTK_SCALE(_opacity), 2);
   gtk_scale_set_draw_value(GTK_SCALE(_opacity), true);

   tmp = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_tabfill), tmp, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabfill), _fillcolor->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmp = gtk_label_new("Style");
   gtk_table_attach(GTK_TABLE(_tabfill), tmp, 4, 5, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_tabfill), _fillstyle, 5, 6, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   _position = new ChipsGuiPositionWidget();
   _position->build_form (false);

   position_frame = gtk_frame_new("Position");
   gtk_frame_set_label_align(GTK_FRAME(position_frame), .5, .5);
   gtk_container_add (GTK_CONTAINER (position_frame), _position->base_widget ());
   gtk_table_attach(GTK_TABLE(_table1), position_frame, 1, 7, 5, 6,
      (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 10, 10);

   tmp = gtk_label_new("Opacity");
   gtk_table_attach(GTK_TABLE(_table1), tmp, 3, 5, 6, 7,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table1), _opacity, 1, 7, 7, 8,
      GTK_FILL, GTK_FILL, 5, 5);

   // general frame
   tmp = gtk_label_new("Depth");
   gtk_table_attach(GTK_TABLE(_table4), tmp, 4, 5, 2, 3,
      GTK_FILL, GTK_FILL, 10, 10);
   _depth = gtk_spin_button_new_with_range(1, G_MAXINT, 1);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(_depth), 0);
   gtk_table_attach(GTK_TABLE(_table4), _depth, 5, 6, 2, 3,
      GTK_FILL, GTK_FILL, 10, 10);
   _vis = gtk_check_button_new();
   tmp = gtk_label_new("Visible");
   gtk_table_attach(GTK_TABLE(_table4), tmp, 4, 5, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);
   gtk_table_attach(GTK_TABLE(_table4), _vis, 5, 6, 3, 4,
      GTK_FILL, GTK_FILL, 10, 10);

   _linecolor->add_callback ("change_color", new ChipsGuiColorWidgetCallback (linecolor_callback, this));
   _position->add_callback ("change_position", new ChipsGuiPositionWidgetCallback (position_callback, this));
/*   _position->add_callback ("change_angle", new ChipsGuiPositionWidgetCallback (angle_callback, this));
 */
   _position->add_callback ("change_coordinate", new ChipsGuiPositionWidgetCallback (coordinate_callback, this));
   _linestyle_id = g_signal_connect(G_OBJECT(_linestyle), "changed",
        G_CALLBACK(linestyle_callback), (void*)this);
   _linethick_id = g_signal_connect(G_OBJECT(_linethick), "value-changed",
        G_CALLBACK(linethick_callback), (void*)this);

   _vis_id = g_signal_connect(G_OBJECT(_vis), "toggled",
      G_CALLBACK(vis_callback), (void*)this);
   _depth_id = g_signal_connect(G_OBJECT(_depth), "value-changed",
      G_CALLBACK(depth_callback), (void*)this);

  _opacity_id = g_signal_connect(G_OBJECT(_opacity), "value-changed",
        G_CALLBACK(opacity_callback), (void*)this);
  _fillcolor->add_callback ("change_color", new ChipsGuiColorWidgetCallback (fillcolor_callback, this));
  _fillstyle_id = g_signal_connect(G_OBJECT(_fillstyle), "changed",
        G_CALLBACK(fillstyle_callback), (void*)this);
  

   setup_tooltip(_linecolor->base_widget(),"Sets the color to use for the region's exterior edge segments");
   setup_tooltip(_linestyle, "Sets the pattern used to display region edge segments");
   setup_tooltip(_linethick, "Controls how thick the edge segments are drawn");

   setup_tooltip(_fillcolor->base_widget(),"Sets the color to use to fill the region");
   setup_tooltip(_fillstyle, "Sets the fill pattern to use for the region");
   setup_tooltip(_opacity, "Sets the transparency level for the entire region");

   setup_tooltip(_depth,"Specifies the rendering layer (depth) where the region is drawn");
   setup_tooltip(_vis,"Should the region be displayed or hidden");

}

void ChipsGuiRegionPropWidget::show ()
{
   gtk_widget_show_all (_base_widget);
}

void ChipsGuiRegionPropWidget::hide ()
{
   gtk_widget_hide (_base_widget);
}

void ChipsGuiRegionPropWidget::set_position (double x, double y)
{
   if (_position)
      _position->set_position (x,y);
}

void ChipsGuiRegionPropWidget::set_coordinate (int val)
{
   if (_position)
      _position->set_coordinate (val);
}

void ChipsGuiRegionPropWidget::set_position_format (int xformat, int yformat)
{
   if (_position)
      _position->set_format (xformat, yformat);
}

void ChipsGuiRegionPropWidget::set_linecolor(const string &color)
{
   if (_linecolor)
      _linecolor->set_chips_color (color);
}

void ChipsGuiRegionPropWidget::set_linethick(float val)
{
   if (val < .75)  _linethick_v = 0.5;
      else         _linethick_v = (int)val;  
   g_signal_handler_block(GTK_SPIN_BUTTON(_linethick), _linethick_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_linethick), _linethick_v);
   g_signal_handler_unblock(GTK_SPIN_BUTTON(_linethick), _linethick_id);
}

void ChipsGuiRegionPropWidget::set_linestyle(int val)
{
   _linestyle_v = val;
   g_signal_handler_block(GTK_OBJECT(_linestyle), _linestyle_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_linestyle),_linestyle_v);
   g_signal_handler_unblock(GTK_OBJECT(_linestyle), _linestyle_id);

}

void ChipsGuiRegionPropWidget::set_angle(float val)
{
  if (_position)
      _position->set_angle (val);
}

void ChipsGuiRegionPropWidget::set_visibility(bool val)
{
   _vis_v = val;
   g_signal_handler_block(GTK_OBJECT(_vis), _vis_id);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_vis), _vis_v);
   g_signal_handler_unblock(GTK_OBJECT(_vis), _vis_id);
}

void ChipsGuiRegionPropWidget::set_depth(int val)
{
   _depth_v = val;
   g_signal_handler_block(GTK_OBJECT(_depth), _depth_id);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_depth), _depth_v);
   g_signal_handler_unblock(GTK_OBJECT(_depth), _depth_id);

}

void ChipsGuiRegionPropWidget::set_opacity(float val)
{
   _opacity_v = val;
   g_signal_handler_block(GTK_OBJECT(_opacity), _opacity_id);
   gtk_range_set_value(GTK_RANGE(_opacity), _opacity_v);
   g_signal_handler_unblock(GTK_OBJECT(_opacity), _opacity_id);
}

void ChipsGuiRegionPropWidget::set_fillcolor(const string &color)
{
   if (_fillcolor)
      _fillcolor->set_chips_color (color);
}

void ChipsGuiRegionPropWidget::set_fillstyle(int val)
{
   _fillstyle_v = val;
   g_signal_handler_block(GTK_OBJECT(_fillstyle), _fillstyle_id);
   gtk_combo_box_set_active(GTK_COMBO_BOX(_fillstyle),_fillstyle_v);
   g_signal_handler_unblock(GTK_OBJECT(_fillstyle), _fillstyle_id);

}

void ChipsGuiRegionPropWidget::linecolor_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);

   if (!regionWidget)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();

   if (regionWidget->_callbacks.count ("change_linecolor") > 0)
      regionWidget->_callbacks["change_linecolor"]->execute (regionWidget, NULL);
}

void ChipsGuiRegionPropWidget::position_callback(ChipsGuiPositionWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   if (!regionWidget)  return; //ensure we have needed pointer
   double x,y;
   widget->get_position (x,y);

   if (regionWidget->_callbacks.count ("change_position") > 0)
      regionWidget->_callbacks["change_position"]->execute (regionWidget, NULL);

}

void ChipsGuiRegionPropWidget::coordinate_callback(ChipsGuiPositionWidget *widget, void *data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);

   if (!regionWidget)  return; //ensure we have needed pointer

   if (regionWidget->_callbacks.count ("change_coordinate") > 0)
      regionWidget->_callbacks["change_coordinate"]->execute (regionWidget, NULL);

}


void ChipsGuiRegionPropWidget::linestyle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (regionWidget)  regionWidget->set_linestyle(val); 

   if (regionWidget->_callbacks.count ("change_linestyle") > 0)
      regionWidget->_callbacks["change_linestyle"]->execute (regionWidget, NULL);

}

void ChipsGuiRegionPropWidget::linethick_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   int val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   float fval = val;
   if (val == 0) fval = 0.5;

   if (regionWidget)  regionWidget->set_linethick(fval);

   if (regionWidget->_callbacks.count ("change_linethickness") > 0)
      regionWidget->_callbacks["change_linethickness"]->execute (regionWidget, NULL);

}
/*
void ChipsGuiRegionPropWidget::angle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   float val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
   if (regionWidget)  regionWidget->set_angle(val);

   if (regionWidget->_callbacks.count ("change_angle") > 0)
      regionWidget->_callbacks["change_angle"]->execute (regionWidget, NULL);

}
*/

void ChipsGuiRegionPropWidget::vis_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   bool val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (regionWidget)  regionWidget->set_visibility(val);

   if (regionWidget->_callbacks.count ("change_visibility") > 0)
      regionWidget->_callbacks["change_visibility"]->execute (regionWidget, NULL);
}

void ChipsGuiRegionPropWidget::depth_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   int val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   if (regionWidget)  regionWidget->set_depth(val); 

   if (regionWidget->_callbacks.count ("change_depth") > 0)
      regionWidget->_callbacks["change_depth"]->execute (regionWidget, NULL);

}

void ChipsGuiRegionPropWidget::fillcolor_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);

   if (!regionWidget)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();

   if (regionWidget->_callbacks.count ("change_fillcolor") > 0)
      regionWidget->_callbacks["change_fillcolor"]->execute (regionWidget, NULL);

}

void ChipsGuiRegionPropWidget::opacity_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   float val = gtk_range_get_value(GTK_RANGE(widget));
   if (regionWidget)  regionWidget->set_opacity(val);

   if (regionWidget->_callbacks.count ("change_opacity") > 0)
      regionWidget->_callbacks["change_opacity"]->execute (regionWidget, NULL);

}

void ChipsGuiRegionPropWidget::fillstyle_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiRegionPropWidget *regionWidget = static_cast <ChipsGuiRegionPropWidget*>(data);
   int val = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
   if (regionWidget)  regionWidget->set_fillstyle(val);

   if (regionWidget->_callbacks.count ("change_fillstyle") > 0)
      regionWidget->_callbacks["change_fillstyle"]->execute (regionWidget, NULL);

}

void ChipsGuiRegionPropWidget::get_position (double &x, double &y)
{
   _position->get_position (x,y);
}

float ChipsGuiRegionPropWidget::get_angle ()
{
   return _position->get_angle ();
}

int ChipsGuiRegionPropWidget::get_coordinate ()
{
   return _position->get_coordinate ();
}

void ChipsGuiRegionPropWidget::get_position_format (int &xformat, int &yformat)
{
   _position->get_format (xformat, yformat);
}
int ChipsGuiRegionPropWidget::get_linestyle()
{
   return _linestyle_v;
}

int ChipsGuiRegionPropWidget::get_depth()
{
   return _depth_v;
}

string ChipsGuiRegionPropWidget::get_linecolor()
{
   return _linecolor->get_chips_color ();
}

float ChipsGuiRegionPropWidget::get_linethick()
{
   return _linethick_v;
}

int ChipsGuiRegionPropWidget::get_fillstyle()
{
   return _fillstyle_v;
}

string ChipsGuiRegionPropWidget::get_fillcolor()
{
   return _fillcolor->get_chips_color ();
}

bool ChipsGuiRegionPropWidget::get_visibility()
{
   return _vis_v;
}

float ChipsGuiRegionPropWidget::get_opacity()
{
   return _opacity_v;
}


void ChipsGuiRegionPropWidget::enable_angle()
{
   _position->enable_angle ();
}

void ChipsGuiRegionPropWidget::enable_linestyle()
{
   gtk_widget_set_sensitive (_linestyle, TRUE);
}

void ChipsGuiRegionPropWidget::enable_depth()
{
   gtk_widget_set_sensitive (_depth, TRUE);
}

void ChipsGuiRegionPropWidget::enable_linecolor()
{
   _linecolor->enable ();
}

void ChipsGuiRegionPropWidget::enable_linethick()
{
   gtk_widget_set_sensitive (_linethick, TRUE);
}

void ChipsGuiRegionPropWidget::enable_fillstyle()
{
   gtk_widget_set_sensitive (_fillstyle, TRUE);
}

void ChipsGuiRegionPropWidget::enable_fillcolor()
{
   _fillcolor->enable ();
}

void ChipsGuiRegionPropWidget::enable_visibility()
{
   gtk_widget_set_sensitive (_vis, TRUE);
}

void ChipsGuiRegionPropWidget::enable_position ()
{
   _position->enable_position ();
}

void ChipsGuiRegionPropWidget::enable_coordinate ()
{
   _position->enable_coordinate ();
}

void ChipsGuiRegionPropWidget::enable_data_coordinate ()
{
   _position->enable_data_coordinate ();
}

void ChipsGuiRegionPropWidget::enable_opacity()
{
   gtk_widget_set_sensitive (_opacity, TRUE);
}


void ChipsGuiRegionPropWidget::disable_angle()
{
   _position->disable_angle ();
}

void ChipsGuiRegionPropWidget::disable_linestyle()
{
   gtk_widget_set_sensitive (_linestyle, FALSE);
}

void ChipsGuiRegionPropWidget::disable_depth()
{
   gtk_widget_set_sensitive (_depth, FALSE);
}

void ChipsGuiRegionPropWidget::disable_linecolor()
{
   _linecolor->disable ();
}

void ChipsGuiRegionPropWidget::disable_linethick()
{
   gtk_widget_set_sensitive (_linethick, FALSE);
}

void ChipsGuiRegionPropWidget::disable_fillstyle()
{
   gtk_widget_set_sensitive (_fillstyle, FALSE);
}

void ChipsGuiRegionPropWidget::disable_fillcolor()
{
   _fillcolor->disable ();
}

void ChipsGuiRegionPropWidget::disable_visibility()
{
   gtk_widget_set_sensitive (_vis, FALSE);
}

void ChipsGuiRegionPropWidget::disable_opacity()
{
   gtk_widget_set_sensitive (_opacity, FALSE);
}


void ChipsGuiRegionPropWidget::disable_position ()
{
   _position->disable_position ();
}

void ChipsGuiRegionPropWidget::disable_coordinate ()
{
   _position->disable_coordinate ();
}

void ChipsGuiRegionPropWidget::disable_data_coordinate ()
{
   _position->disable_data_coordinate ();
}



