// 
//  Copyright (C) 2011-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUIREGIONWIDGETFRAME_HH
#define CHIPSGUIREGIONWIDGETFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiWidget.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;

//  * change_linestyle
//  * change_linecolor
//  * change_linethickness
//  * change_depth
//  * change_opacity
//  * change_fillcolor
//  * change_fillstyle
//  * change_visibility
//  * change_position
//  * change_coordinate

class ChipsGuiRegionPropWidget;
class ChipsGuiPositionWidget;
class ChipsGuiColorWidget;

class ChipsGuiRegionPropWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiRegionPropWidgetCallback (void (*callback)(ChipsGuiRegionPropWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiRegionPropWidget *, void *);
     void *_callbackData;
};

//---------------------------------------------------------------------------

class ChipsGuiRegionPropWidget: public ChipsGuiWidget, public ChipsGuiTooltipObject
{
public:

   ChipsGuiRegionPropWidget();
   virtual ~ChipsGuiRegionPropWidget();

   virtual void build_form ();

   virtual void show ();
   virtual void hide ();

   void set_linecolor(const string &color); 
   void set_linestyle(int val); 
   void set_linethick(float val);
   void set_angle(float val);
   void set_depth(int val); 
   void set_visibility(bool val); 
   void set_fillcolor(const string &color); 
   void set_fillstyle(int val);
   void set_opacity(float val); 
   void set_position (double xpos, double ypos);
   void set_coordinate (int coordinate);
   void set_position_format (int xformat, int yformat);


   string get_linecolor();
   int    get_linestyle();
   float  get_linethick();
   float  get_angle();
   int    get_depth();
   bool   get_visibility();
   string get_fillcolor();
   int    get_fillstyle();
   float  get_opacity();
   void   get_position (double &xpos, double &ypos);
   int    get_coordinate ();
   void   get_position_format (int &xformat, int &yformat);

   void enable_linecolor();
   void enable_linestyle();
   void enable_linethick();
   void enable_angle();
   void enable_depth();
   void enable_visibility();
   void enable_fillcolor();
   void enable_fillstyle();
   void enable_opacity();
   void enable_position ();
   void enable_coordinate ();
   void enable_data_coordinate ();

   void disable_linecolor();
   void disable_linestyle();
   void disable_linethick();
   void disable_angle();
   void disable_depth();
   void disable_visibility();
   void disable_fillcolor();
   void disable_fillstyle();
   void disable_opacity();
   void disable_position ();
   void disable_coordinate ();
   void disable_data_coordinate ();


private:

   static void linecolor_callback(ChipsGuiColorWidget *widget, void *data);
   static void linestyle_callback(GtkWidget *widget, gpointer data);
   static void linethick_callback(GtkWidget *widget, gpointer data);
   static void vis_callback(GtkWidget *widget, gpointer data);
   static void depth_callback(GtkWidget *widget, gpointer data);
   static void opacity_callback(GtkWidget *widget, gpointer data);
   static void fillcolor_callback(ChipsGuiColorWidget *widget, void *data);
   static void fillstyle_callback(GtkWidget *widget, gpointer data);
   static void position_callback(ChipsGuiPositionWidget *widget, void *data);
   static void coordinate_callback(ChipsGuiPositionWidget *widget, void *data);


   GtkWidget*   _nbook; 
   GtkWidget*   _table1;
   GtkWidget*   _frmline;
   GtkWidget*   _tabline;
   GtkWidget*   _frmfill;
   GtkWidget*   _tabfill;
   GtkWidget*   _table4;

   ChipsGuiColorWidget*   _linecolor;
   GtkWidget*   _linestyle;
   GtkWidget*   _linethick;

   
   ChipsGuiColorWidget*   _fillcolor;
   GtkWidget*   _fillstyle;
   GtkWidget*   _opacity;

   GtkWidget*   _vis; 
   GtkWidget*   _depth;

   ChipsGuiPositionWidget* _position;

   // signals for callbacks 
   unsigned int _linestyle_id;    // style of line
   unsigned int _linethick_id;     // thickness of line
   unsigned int _fillstyle_id;
   unsigned int _opacity_id;
   unsigned int _depth_id;         // depth/layer  
   unsigned int _vis_id;


   int     _linestyle_v;    // style of line
   float   _linethick_v;     // thickness of line
   int     _fillstyle_v;
   float   _opacity_v;
   string  _fillcolor_v; 
   int     _depth_v;         // depth/layer  
   bool    _vis_v; 

};

#endif
