// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiSaveDialog.hh"
#include "ChipsGuiMain.hh"
#include <string>
#include <fstream>
#include <iostream>

extern ChipsGuiMain* chipsGui;

using namespace std;

string ChipsGuiSaveDialog::_path = "";
int ChipsGuiSaveDialog::_mode = 0; 

ChipsGuiSaveDialog::ChipsGuiSaveDialog()
{
}

ChipsGuiSaveDialog::~ChipsGuiSaveDialog()
{
   gtk_widget_destroy(base_w);
}

void ChipsGuiSaveDialog::generate_form()
{
   base_w = gtk_file_chooser_dialog_new ("Save File", NULL,
                                      GTK_FILE_CHOOSER_ACTION_SAVE,
                                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                      NULL);

   gtk_widget_set_size_request(base_w, 450, 400);

   // add radio button to select the output format 
   _contain = gtk_dialog_get_content_area(GTK_DIALOG(base_w));
   GtkWidget* hbox = gtk_hbox_new (FALSE, 2);
   GtkWidget* label = gtk_label_new("Output format:  "); 
   _ascii =  gtk_radio_button_new_with_label_from_widget(NULL, "ascii python script  "); 
   _binary =  gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(_ascii), "binary state file"); 
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
   gtk_box_pack_start (GTK_BOX (hbox), _ascii, FALSE, FALSE, 5);
   gtk_box_pack_start (GTK_BOX (hbox), _binary, FALSE, FALSE, 5);
   gtk_box_pack_start(GTK_BOX(_contain), hbox, FALSE, FALSE, 5);
   gtk_box_reorder_child(GTK_BOX(_contain), hbox, 0);
   show(); 

   // set default path
   if (_path.length() == 0)   _path = chipsGui->get_working_dir(); 
   gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(base_w), _path.c_str());
   // set default mode 
   if (_mode)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_ascii), TRUE);
   else 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_binary), TRUE);

   gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(base_w), TRUE);
}

int ChipsGuiSaveDialog::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int ChipsGuiSaveDialog::doit()
{
   generate_form();
   if (gtk_dialog_run(GTK_DIALOG (base_w)) == GTK_RESPONSE_ACCEPT)
   {
      char *fname;

      if ((fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(base_w))) !=
          NULL)
      {
     
         chipsGui->save_command(fname,
            (_mode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_ascii)))); 
         string tmp = fname; 
         size_t ps; 
         if ((ps = tmp.rfind('/')) != string::npos)
         {
            _path = tmp.substr(0, ps); 
         }

         g_free (fname);
      }
   }
   gtk_widget_destroy(base_w);
   return 0; 
}

