// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSGUISETTINGS_HH
#define CHIPSGUISETTINGS_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>

using namespace std;

class ChipsGuiSettings
{
  public:
    ChipsGuiSettings();
    ~ChipsGuiSettings();

    void initialize(); 
    int load();
    int save();

    int get_tbar_style()  { return tbar_style; }
    int set_tbar_style(int val); 
    int set_tbar_style(string val); 

    bool get_pane_tbar_vis()   { return pane_tbar; }
    bool get_pane_nav_vis()    { return pane_nav; }
    bool get_pane_info_vis()   { return pane_info; }
    bool get_pane_prop_vis()   { return pane_prop; }
    bool get_pane_shell_vis()  { return pane_shell; }
    bool get_wingrids()        { return wingrids; }
    bool get_tooltips()        { return tooltips; }
    bool get_zoombutton()      { return zoombutton; }
#ifdef CHIPS_GUI_ADDDEL
    bool get_delbutton()       { return delbutton; }
#endif 
    string get_defbutton()     { return defbutton; }
    string get_selcolor()      { return selcolor; }
    string get_rangecolor()    { return rangecolor; }

    void set_pane_nav_vis(bool val)    { pane_nav = val; } 
    void set_pane_tbar_vis(bool val)   { pane_tbar = val; } 
    void set_pane_info_vis(bool val)   { pane_info = val; } 
    void set_pane_prop_vis(bool val)   { pane_prop = val; } 
    void set_pane_shell_vis(bool val)  { pane_shell = val; } 
    void set_tooltips(bool val);
    void set_wingrids(bool val);
    void set_zoombutton(bool val)      { zoombutton = val; }
#ifdef CHIPS_GUI_ADDDEL
    void set_delbutton(bool val)       { delbutton = val; }
#endif
    int  set_defbutton(const string &val); 
    int  set_selcolor(const string &val); 
    int  set_rangecolor(const string &val); 

  protected:
    string tbar_style_as_string(); 

    bool pane_tbar;        // show toolbar in gui 
    bool pane_nav;         // show coord/nav panel in gui  
    bool pane_info;        // show icon/treeview panel in gui  
    bool pane_prop;        // show properties panel in gui 
    bool pane_shell;       // show shell notebook panel in gui 
    bool wingrids;         // show window grids
    bool tooltips;         // show tooltips  
    bool zoombutton;       // show zoom button in toolbar 
#ifdef CHIPS_GUI_ADDDEL
    bool delbutton;        // show remove button in toolbar 
#endif
    string defbutton;      // default button for toolbar 
    string selcolor;       // selection bounding box anchor color 
    string rangecolor;     // pick limits bounding box color 
    
    int tbar_style;        // toolbar button style- 2=both, 0=icon 1=label 

};

#endif 
