// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiSettingsDialog.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEventManager.hh"
#include "ChipsGuiSettings.hh"
#include "ChipsGuiColorWidget.hh"

#include <fstream>
#include <iostream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

using namespace std;

extern ChipsGuiSettings* GuiSettings;
extern ChipsGuiMain* chipsGui;
extern ChipsEventManager* event_manager;

ChipsGuiSettingsDialog::ChipsGuiSettingsDialog():ChipsGuiTooltipObject()
{
   _selcolor = NULL;
   _rangecolor = NULL; 
}

ChipsGuiSettingsDialog::~ChipsGuiSettingsDialog()
{
   if (_selcolor)  delete _selcolor;
   if (_rangecolor)  delete _rangecolor;
   _selcolor = NULL;
   _rangecolor = NULL;
}

void ChipsGuiSettingsDialog::generate_form()
{
   string ss; 
   base_w = gtk_dialog_new_with_buttons (_title.c_str(), NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   gtk_widget_set_size_request(base_w, 400, 300);
   _notebook = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(_notebook), GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(_notebook));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(_notebook), GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _notebook);

   _tab_tbar = gtk_table_new(10,14, true);

   _tab_wids[0] = gtk_label_new("button style");
   _tab_wids[1] = gtk_radio_button_new_with_label_from_widget(NULL, "icons"); 
   _tab_wids[2] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(_tab_wids[1]), "labels"); 
   _tab_wids[3] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(_tab_wids[1]), "both"); 
   _tab_wids[4] = gtk_hseparator_new(); 
   _tab_wids[5] = gtk_label_new("default mode");
   _tab_wids[6] = gtk_radio_button_new_with_label_from_widget(NULL, "pan"); 
   _tab_wids[7] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(_tab_wids[6]), "range"); 
   _tab_wids[8] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(_tab_wids[7]), "select"); 
#ifdef CHIPS_GUI_ADDDEL
   _tab_wids[9] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(_tab_wids[8]), "add"); 
#endif 
   _tab_wids[10] = gtk_hseparator_new(); 
   _tab_wids[11] = gtk_check_button_new_with_label("Show Zoom Button"); 
#ifdef CHIPS_GUI_ADDDEL
   _tab_wids[12] = gtk_check_button_new_with_label("Show Delete Button"); 
#endif

   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[5],1,5,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[6],6,9,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[7],10,13,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[8],6,9,3,4);
#ifdef CHIPS_GUI_ADDDEL
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[9],10,13,3,4);
#endif
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[4],1,13,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[0],1,5,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[1],6,8,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[2],8,11,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[3],11,13,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[10],1,13,7,8);
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[11],2,11,8,9);
#ifdef CHIPS_GUI_ADDDEL
   gtk_table_attach_defaults(GTK_TABLE(_tab_tbar), _tab_wids[12],2,11,9,10);
#endif
   _tab_panes = gtk_table_new(10,12, true);
   _pane_tog[0] =  gtk_check_button_new_with_label("Show Toolbar"); 
   _pane_tog[1] =  gtk_check_button_new_with_label("Show Coordinate Frame"); 
   _pane_tog[2] =  gtk_check_button_new_with_label("Show Information Frame");
   _pane_tog[3] =  gtk_check_button_new_with_label("Show Property Frame"); 
   _pane_tog[4] =  gtk_check_button_new_with_label("Show Shell Frame"); 

   gtk_table_attach_defaults(GTK_TABLE(_tab_panes), _pane_tog[0],2,9,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_tab_panes), _pane_tog[1],2,9,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_tab_panes), _pane_tog[2],2,9,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_tab_panes), _pane_tog[3],2,9,7,8);
   gtk_table_attach_defaults(GTK_TABLE(_tab_panes), _pane_tog[4],2,9,9,10);

   _tab_gen = gtk_table_new(10,12, true);
   _gen_wids[0] =  gtk_check_button_new_with_label("Show Tooltips"); 
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _gen_wids[0],2,9,1,2);
   _gen_wids[1] =  gtk_check_button_new_with_label("Highlight on Selection"); 
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _gen_wids[1],2,9,3,4);

   _selcolor = new ChipsGuiColorWidget(); 
   _selcolor->build_form ();
   _selcolor->set_foreground_color (false);
   _rangecolor = new ChipsGuiColorWidget(); 
   _rangecolor->build_form ();
   _rangecolor->set_foreground_color (false);
   _label = gtk_label_new("Selection Color");
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _label, 2,6,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _selcolor->base_widget(),
      7,10,5,6);
   _label = gtk_label_new("Range Color");
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _label, 2,6,7,8);
   gtk_table_attach_defaults(GTK_TABLE(_tab_gen), _rangecolor->base_widget(),
      7,10,7,8);

   _label = gtk_label_new("Panes");
   gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), _tab_panes, _label),
   _label = gtk_label_new("Toolbar");
   gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), _tab_tbar, _label),
   _label = gtk_label_new("General");
   gtk_notebook_append_page(GTK_NOTEBOOK(_notebook), _tab_gen, _label),

   setup_tooltip(_selcolor->base_widget(), "Change the color of the selection anchors");
   setup_tooltip(_rangecolor->base_widget(), "Change the color of the range/zoom bounding box");
   setup_tooltip(_gen_wids[0],"Display usage information on mouse over");
   setup_tooltip(_gen_wids[1],"Blink the object when selected to identify it");
   setup_tooltip(_tab_wids[1],"Display the toolbar buttons as icons");
   setup_tooltip(_tab_wids[2],"Display the toolbar buttons using labels");
   setup_tooltip(_tab_wids[3],"Display the toolbar buttons using icons with labels");
   setup_tooltip(_tab_wids[6],"Set the gui's default interaction mode to Pan");
   setup_tooltip(_tab_wids[7],"Set the gui's default interaction mode to Range/Zoom");
   setup_tooltip(_tab_wids[8],"Set the gui's default interaction mode to Select");
   setup_tooltip(_tab_wids[11],"Display the Range/Zoom button in the toolbar");
#ifdef CHIPS_GUI_ADDDEL
   setup_tooltip(_tab_wids[9],"Set the gui's default interaction mode to Add/Insert");
   setup_tooltip(_tab_wids[12],"Display the Delete button in the toolbar");
#endif
   setup_tooltip(_pane_tog[0], "Display the toolbar in the gui");
   setup_tooltip(_pane_tog[1], "Display the Coordinates frame in the gui");
   setup_tooltip(_pane_tog[2], "Display the Information frame in the gui");
   setup_tooltip(_pane_tog[3], "Display the Property editing frame in the gui");
   setup_tooltip(_pane_tog[4], "Display the Shell (commandline prompt) frame in the gui");

   gtk_widget_show_all(base_w);
}

int ChipsGuiSettingsDialog::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int ChipsGuiSettingsDialog::doit()
{
   generate_form();
   load_preferences(); 
   if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
   {
      set_preferences(); 
   }
   gtk_widget_destroy(base_w);
   return 0; 
}

void ChipsGuiSettingsDialog::load_preferences()
{
   string sval;
   int ival;
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_pane_tog[0]),
      GuiSettings->get_pane_tbar_vis()); 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_pane_tog[1]),
      GuiSettings->get_pane_nav_vis()); 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_pane_tog[2]),
      GuiSettings->get_pane_info_vis()); 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_pane_tog[3]),
      GuiSettings->get_pane_prop_vis()); 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_pane_tog[4]),
      GuiSettings->get_pane_shell_vis()); 

   ival = GuiSettings->get_tbar_style();
   if (ival == 0)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[1]),1);
   else if (ival == 1)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[2]),1);
   else 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[3]),1);

   sval = GuiSettings->get_defbutton();
   if (sval == "pan")
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[6]),1);
   else if (sval == "range")
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[7]),1);
#ifdef CHIPS_GUI_ADDDEL
   else if (sval == "add")
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[9]),1);
#endif
   else  // use select as default if invalid value received  
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[8]),1);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[11]),
      GuiSettings->get_zoombutton()); 
#ifdef CHIPS_GUI_ADDDEL
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tab_wids[12]),
      GuiSettings->get_delbutton()); 
#endif
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_gen_wids[0]),
      GuiSettings->get_tooltips()); 

   string clr = GuiSettings->get_selcolor();
   _selcolor->set_chips_color(clr);    
   clr = GuiSettings->get_rangecolor();
   _rangecolor->set_chips_color(clr);    

   // for now- use value stored in event manager 
   if (event_manager) 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_gen_wids[1]), 
         event_manager->get_highlight_on_select()); 
}

void ChipsGuiSettingsDialog::set_preferences()
{
   // panel visibilty 
   GuiSettings->set_pane_tbar_vis(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_pane_tog[0]))); 
   GuiSettings->set_pane_nav_vis(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_pane_tog[1]))); 
   GuiSettings->set_pane_info_vis(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_pane_tog[2]))); 
   GuiSettings->set_pane_prop_vis(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_pane_tog[3]))); 
   GuiSettings->set_pane_shell_vis(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_pane_tog[4]))); 

  // toolbar style 
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[1])))
     GuiSettings->set_tbar_style(0);
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[2])))
     GuiSettings->set_tbar_style(1); 
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[3])))
     GuiSettings->set_tbar_style(2); 

   // mode buttons 
   GuiSettings->set_zoombutton(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[11]))); 
#ifdef CHIPS_GUI_ADDDEL
   GuiSettings->set_delbutton(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[12]))); 
#endif
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[6])))
      GuiSettings->set_defbutton("pan"); 
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[7])))
      GuiSettings->set_defbutton("range"); 
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[8])))
      GuiSettings->set_defbutton("select"); 
#ifdef CHIPS_GUI_ADDDEL
  else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tab_wids[9])))
      GuiSettings->set_defbutton("add"); 
#endif 
   // general settings 
   GuiSettings->set_tooltips(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_gen_wids[0])));

   // colors
   string clr = _selcolor->get_chips_color();
   GuiSettings->set_selcolor(clr); 
   clr = _rangecolor->get_chips_color(); 
   GuiSettings->set_rangecolor(clr); 

   // save preferences 
   GuiSettings->save(); 

   // update gui to reflect changes 
   if (chipsGui)  chipsGui->gui_prefs_changed(); 

   // for now handle highlight selection separately 
   if (event_manager)   event_manager->set_highlight_on_select(
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_gen_wids[1]))); 
}



