// 
//  Copyright (C) 2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUISPINWIDGET_HH
#define CHIPSGUISPINWIDGET_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsGuiWidget.hh"
#include "ChipsDefines.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;


// valid callback ids are
//  * change_value
//  * auto_inc
//  * auto_dec

class ChipsGuiSpinWidget;

class ChipsGuiSpinWidgetCallback : public ChipsGuiWidgetCallback
{
   public:
     ChipsGuiSpinWidgetCallback (void (*callback)(ChipsGuiSpinWidget *, void *), void *data);
     virtual void execute (ChipsGuiWidget *caller, void *);
   protected:
     void (*_callback)(ChipsGuiSpinWidget *, void *);
     void *_callbackData;
};


//---------------------------------------------------------------------------

class ChipsGuiSpinWidget : public ChipsGuiWidget, public ChipsGuiTooltipObject
{
public:

  ChipsGuiSpinWidget(int type=0);
  ~ChipsGuiSpinWidget();

  GtkWidget*  base_widget()   { return _base_w; }

  virtual void build_form(); 

  virtual void show ();
  virtual void hide ();
  

  void set(double val, int mode=-1); // mode 0=numeric 1=hms -1=existing 
  void set_value(string val); 
  void set_value(double val); 

  void disable ();
  void enable ();

  void disable_menu ();
  void enable_menu ();

  int  validate_text(); 

  void flag_invalid_text(bool set);

  string get_string_value();  
  double get_numeric_value(); 

  void set_mode(int mode);
  int  get_mode()  { return _mode; }
  void set_type(int val)   { _type = val; }
  int  get_type()  { return _type; }
  
  void  set_precision(int digits);
  int  get_precision()  { return _precision; }

  double ra_to_deg(string val); 
  double dec_to_deg(string val); 
  string deg_to_ra(double val);
  string deg_to_dec(double val);

  int ra_to_deg(string val, double &dval); 
  int dec_to_deg(string val, double &dval); 
  int deg_to_ra(double dval, string &sval); 
  int deg_to_dec(double dval, string &sval); 

  void set_stepsize(double val)  { _stepsize = val; }
  double get_stepsize()  { return _stepsize; }
  void reset_scrollsize(); 
  void reset_scrollcnt()   { _scrollcnt = 0; } 
  void reset_scrollcnt(unsigned int timestamp); 
  void process_scroll(int dir=0);   // dir 0 = up  1 = down 
  void run_change_value_cb(); 

private:

  GtkWidget* _base_w;
  GtkObject* _adj;

  int        _mode;   // 0 = numeric, 1 = sexagesimal 
  int        _type;   // 0 = ra, 1 = dec 
  string     _sval;
  double     _dval; 
  int        _precision;  
  double     _stepsize;   // numeric step size 
  unsigned long _scrollcnt;
  unsigned int  _scrolltimestamp; 

  bool       _enable_popup;

  static bool entry_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data);  
  static bool entry_key_press_cb(GtkWidget *widget, GdkEvent *event, gpointer data);  
  static bool entry_key_release_cb(GtkWidget *widget, GdkEvent *event, gpointer data);  
  static bool entry_scroll_cb(GtkWidget *widget, GdkEvent *event, gpointer data);  
  static void value_changed_callback(GtkWidget *widget, gpointer data);
  static bool focus_out_cb(GtkWidget *widget, GdkEvent *event, gpointer data);
  static int  process_output_cb(GtkEntry *wid, gpointer data);


  void popup_menu();
  static void set_numeric_cb(GtkWidget *widget, gpointer data);
  static void set_sexagesimal_cb(GtkWidget *widget, gpointer data);
 
  // signals for callbacks 
  unsigned long _btn_press_id;
  unsigned long _key_press_id;
  unsigned long _key_release_id;
  unsigned long _scroll_id;
  unsigned long _process_output_id;
  unsigned long _focus_out_id;
 
};

#endif
