// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiMain.hh"
#include "ChipsGuiToolbar.hh"
#include "ChipsEventObserver.hh"
#include "ChipsGuiSettings.hh"
#include <stdlib.h>

#include <atk/atk.h>


extern ChipsEventManager* event_manager;
extern ChipsGuiSettings* GuiSettings;
extern ChipsGuiMain* chipsGui;


using namespace std;


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiToolbar::ChipsGuiToolbar():ChipsEventObserver(),ChipsGuiTooltipObject()
{
   int pos = 0;
   GtkWidget *img_icon;
   string dname = "", fname; 
   AtkObject *accessible;

   // create the buttons in toolbar
   _toolbar = gtk_toolbar_new ();

   initialize(); 

   // set path to icons 
   char *tmpdir = getenv("ASCDS_INSTALL");
   if (tmpdir)   dname = tmpdir;
   dname.append("/config/chips/");

   if ((_emptyBtn = gtk_radio_tool_button_new(NULL)) != NULL)
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _emptyBtn, -1);
   if ((_panBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      fname = dname;
      fname.append("icon_pan.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_panBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_panBtn),"pan"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _panBtn, pos++);
      _panBtn_id = g_signal_connect(G_OBJECT(_panBtn), "clicked",
         G_CALLBACK(pan_cb), (void*)this);
   }
   if ((_rangeBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      fname = dname;
      fname.append("icon_range.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_rangeBtn), GTK_WIDGET(img_icon));
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _rangeBtn, pos++);
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_rangeBtn),"range"); 
      _rangeBtn_id = g_signal_connect(G_OBJECT(_rangeBtn), "clicked",
         G_CALLBACK(range_cb), (void*)this);
   }
   if ((_zoomBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      fname = dname;
      fname.append("icon_zoom.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_zoomBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_zoomBtn),"zoom"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _zoomBtn, pos++);
      _zoomBtn_id = g_signal_connect(G_OBJECT(_zoomBtn), "clicked",
         G_CALLBACK(zoom_cb), (void*)this);
   }
   if ((_selectBtn = gtk_radio_tool_button_new_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn))) != NULL)
   {
      fname = dname;
      fname.append("icon_select.png");
      img_icon = gtk_image_new_from_file (fname.c_str());
      gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(_selectBtn), GTK_WIDGET(img_icon));
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_selectBtn),"select"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _selectBtn, pos++);
      _selectBtn_id = g_signal_connect(G_OBJECT(_selectBtn), "clicked",
         G_CALLBACK(select_cb), (void*)this);
   }
#ifdef CHIPS_GUI_ADDDEL
   if ((_delBtn = gtk_radio_tool_button_new_with_stock_from_widget(GTK_RADIO_TOOL_BUTTON(_emptyBtn), GTK_STOCK_REMOVE)) != NULL)
   {
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_delBtn),"delete"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _delBtn, pos++);
      g_signal_connect(G_OBJECT(_delBtn), "clicked",
         G_CALLBACK(delete_cb), (void*)this);
   }
#endif
   _sep = gtk_separator_tool_item_new();
   gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _sep, -1);
   gtk_tool_item_set_expand(GTK_TOOL_ITEM(_sep), TRUE);

#ifdef CHIPS_GUI_ADDDEL
   if ((_loadBtn = gtk_tool_button_new_from_stock(GTK_STOCK_FILE)) != NULL)
   {
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_loadBtn), "load file"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _loadBtn, -1);
      g_signal_connect(G_OBJECT(_loadBtn), "clicked",
         G_CALLBACK(load_cb), (void*)this);
   }
#endif
   if ((_undoBtn = gtk_tool_button_new_from_stock(GTK_STOCK_UNDO)) != NULL)
   {
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_undoBtn), "undo"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _undoBtn, -1);
      g_signal_connect(G_OBJECT(_undoBtn), "clicked",
         G_CALLBACK(undo_cb), (void*)this);
   }
   if ((_redoBtn = gtk_tool_button_new_from_stock(GTK_STOCK_REDO)) != NULL)
   {
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_redoBtn), "redo"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _redoBtn, -1);
      g_signal_connect(G_OBJECT(_redoBtn), "clicked",
         G_CALLBACK(redo_cb), (void*)this);
   }
   if ((_printBtn = gtk_tool_button_new_from_stock(GTK_STOCK_PRINT)) != NULL)
   {
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_printBtn), "print"); 
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _printBtn, -1);
      g_signal_connect(G_OBJECT(_printBtn), "clicked",
         G_CALLBACK(print_cb), (void*)this);
   }
   if ((_clearBtn = gtk_tool_button_new_from_stock(GTK_STOCK_CLEAR)) != NULL)
   {
      gtk_tool_button_set_label(GTK_TOOL_BUTTON(_clearBtn), "clear");
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _clearBtn, -1);
      g_signal_connect(G_OBJECT(_clearBtn), "clicked",
         G_CALLBACK(clear_cb), (void*)this);
   }


#ifdef CHIPS_GUI_ADDDEL
   setup_tooltip(_delBtn, "Delete Mode- remove an object");
   setup_tooltip(_loadBtn, "Load data from a file for curves, contours, histograms, or images");
#endif
   setup_tooltip(GTK_WIDGET(_panBtn), "Pan Mode- left button press and drag to pan ChIPS window");
   setup_tooltip(GTK_WIDGET(_rangeBtn), "Range Mode- left button press, drag, and release to view area in bounding box");
   setup_tooltip(GTK_WIDGET(_zoomBtn), "Zoom Mode- scroll button to zoom in/out in the ChIPS window");
   setup_tooltip(GTK_WIDGET(_selectBtn), "Select Mode- left button click to select an object in the ChIPS window");
   setup_tooltip(GTK_WIDGET(_undoBtn), "Undo last action/command");
   setup_tooltip(GTK_WIDGET(_redoBtn), "Redo last undone action/command");
   setup_tooltip(GTK_WIDGET(_printBtn), "Print options");
   setup_tooltip(GTK_WIDGET(_clearBtn), "Clear Plot - keeps the plot but removes objects contained in it");

   // testing accessiblity
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_panBtn));
   atk_object_set_name (accessible, "panButton");
   atk_object_set_description (accessible, "Chooses the pan mode\n");
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_rangeBtn));
   atk_object_set_name (accessible, "rangeButton");
   atk_object_set_description (accessible, "Chooses the range mode\n");
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_zoomBtn));
   atk_object_set_name (accessible, "zoomButton");
   atk_object_set_description (accessible, "Chooses the zoom mode\n");
   accessible = gtk_widget_get_accessible (GTK_WIDGET(_selectBtn));
   atk_object_set_name (accessible, "selectButton");
   atk_object_set_description (accessible, "Chooses the select mode\n");




   // register w/ event observer to get changes in mode 
   register_event(ChipsEventManager::ChangeStateEvent);

   // set initial mode
   set_mode(); 

   // gray out unimplemented widgets
#ifdef CHIPS_GUI_ADDDEL
   gtk_widget_set_sensitive(GTK_WIDGET(_delBtn), false);  
   gtk_widget_set_sensitive(GTK_WIDGET(_loadBtn), false);  
#endif
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiToolbar::~ChipsGuiToolbar()
{
  gtk_widget_destroy(_toolbar);
}


int ChipsGuiToolbar::pan_cb(GObject *object, gpointer user_data)
{
   ChipsGuiToolbar *tbar = static_cast <ChipsGuiToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
      tbar->invoke_mode_change(ChipsEventManager::PanState); 
   return 0;
}

int ChipsGuiToolbar::range_cb(GObject *object, gpointer user_data)
{
   ChipsGuiToolbar *tbar = static_cast <ChipsGuiToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
      tbar->invoke_mode_change(ChipsEventManager::RangeZoomState); 

   return 0;
}

int ChipsGuiToolbar::zoom_cb(GObject *object, gpointer user_data)
{
   ChipsGuiToolbar *tbar = static_cast <ChipsGuiToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
      tbar->invoke_mode_change(ChipsEventManager::DragZoomState); 

   return 0;
}

int ChipsGuiToolbar::select_cb(GObject *object, gpointer user_data)
{
   ChipsGuiToolbar *tbar = static_cast <ChipsGuiToolbar *>(user_data);
   GtkToggleToolButton *btn = GTK_TOGGLE_TOOL_BUTTON(object);

   if (tbar && (gtk_toggle_tool_button_get_active(btn) == TRUE))
      tbar->invoke_mode_change(ChipsEventManager::SelectState); 

   return 0; 
}

#ifdef CHIPS_GUI_ADDDEL
int ChipsGuiToolbar::delete_cb(GObject *object, gpointer user_data)
{
   ChipsGuiToolbar *tbar = static_cast <ChipsGuiToolbar *>(user_data);

   cout << " delete mode callback"<< endl;

   return 0;
}

void ChipsGuiToolbar::load_cb(GtkWidget *widget, gpointer user_data)
{
   ChipsGuiToolbar *tbar = static_cast <ChipsGuiToolbar *>(user_data);

   cout << " load callback"<< endl;
}
#endif

void ChipsGuiToolbar::undo_cb(GtkWidget *widget, gpointer user_data)
{
   if (chipsGui) chipsGui->undo_command();
}

void ChipsGuiToolbar::redo_cb(GtkWidget *widget, gpointer user_data)
{
   if (chipsGui) chipsGui->redo_command();
}

void ChipsGuiToolbar::print_cb(GtkWidget *widget, gpointer user_data)
{
   if (chipsGui) chipsGui->print_command(); 
}

void ChipsGuiToolbar::clear_cb(GtkWidget *widget, gpointer user_data)
{
   if (chipsGui) chipsGui->clear_command(true);
}


void ChipsGuiToolbar::set_style(int style)
{
   if (style == 0)
      gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_ICONS);    
   else if (style == 1)
      gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_TEXT);    
   else if (style == 2)
      gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_BOTH);    
}

void ChipsGuiToolbar::show()
{
   // set appearance based on loaded prefs 
   update_gui_prefs(); 

   gtk_widget_show(GTK_WIDGET(_toolbar));
   gtk_widget_hide (GTK_WIDGET(_emptyBtn));
}

void ChipsGuiToolbar::hide()
{
   gtk_widget_hide(GTK_WIDGET(_toolbar));
   gtk_widget_hide (GTK_WIDGET(_emptyBtn));
}

int ChipsGuiToolbar::get_height()
{
   int ht = 0; 
   
   if (_toolbar)   ht = _toolbar->allocation.height; 
   return ht; 
}

// event observer to handle mode change from event manager 
void ChipsGuiToolbar::execute_event (int event, void *data)
{
   int* states = (int*)data; 
   g_signal_handler_block (_selectBtn, _selectBtn_id);
   g_signal_handler_block (_rangeBtn, _rangeBtn_id);
   g_signal_handler_block (_panBtn, _panBtn_id);
   g_signal_handler_block (_zoomBtn, _zoomBtn_id);

   this->change_mode(states[1]); 

   g_signal_handler_unblock (_selectBtn, _selectBtn_id);
   g_signal_handler_unblock (_rangeBtn, _rangeBtn_id);
   g_signal_handler_unblock (_panBtn, _panBtn_id);
   g_signal_handler_unblock (_zoomBtn, _zoomBtn_id);

}


// passes mode change event to event manager 
void ChipsGuiToolbar::invoke_mode_change(int val)
{
   if (event_manager) 
   {
      double pos[2] = {0.0,0.0};
      event_manager->invoke_event(pos, ChipsEventManager::ChangeStateEvent,
         (void*)&val, this);
   }
}

// sets appropriate radio button to match mode 
void ChipsGuiToolbar::change_mode(int val)
{
   if (val != ChipsEventManager::PickState)
   {
      gtk_widget_set_sensitive (GTK_WIDGET(_selectBtn), TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET(_rangeBtn), TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET(_panBtn), TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET(_zoomBtn), TRUE);
   }

   switch(val)
   {
      case ChipsEventManager::PanState: 
         gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_panBtn),true); 
      break;
      case ChipsEventManager::RangeZoomState: 
         gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_rangeBtn),true); 
      break;
      case ChipsEventManager::SelectState: 
         gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_selectBtn),true); 
      break;
      case ChipsEventManager::AddState:
         gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_emptyBtn),true);
      break;
      case ChipsEventManager::PickState:
         gtk_widget_set_sensitive (GTK_WIDGET(_selectBtn), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(_rangeBtn), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(_panBtn), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(_zoomBtn), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(_panBtn), FALSE);
      default:
      break; 
   }
}

void ChipsGuiToolbar::initialize()
{
   _showZoom = _showRem = true;
   _defMode = "select";
   _btnStyle = 2; 
   load_prefs();
}

void ChipsGuiToolbar::load_prefs()
{
   if (GuiSettings)
   {
      _showZoom = GuiSettings->get_zoombutton();
#ifdef CHIPS_GUI_ADDDEL
      _showRem = GuiSettings->get_delbutton();
#endif
      _defMode = GuiSettings->get_defbutton(); 
      _btnStyle = GuiSettings->get_tbar_style(); 
   }
}

void ChipsGuiToolbar::update_gui_prefs()
{
   // reload latest gui settings 
   load_prefs(); 

   if (_showZoom)
      gtk_widget_show(GTK_WIDGET(_zoomBtn));
   else 
      gtk_widget_hide(GTK_WIDGET(_zoomBtn));
#ifdef CHIPS_GUI_ADDDEL
   if (_showRem)
      gtk_widget_show(GTK_WIDGET(_delBtn));
   else 
      gtk_widget_hide(GTK_WIDGET(_delBtn));
#endif
   set_style(_btnStyle); 
   
}


void ChipsGuiToolbar::set_mode(string val)
{
   string mode = val;

   if (mode.length() == 0) mode = _defMode; 

   if (mode == "pan") 
      gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_panBtn),true);
   else if (mode == "range") 
      gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_rangeBtn),true);
   else if (mode == "zoom") 
      gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_zoomBtn),true);
   else if (mode == "select")
      gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_selectBtn),true);
#ifdef CHIPS_GUI_ADDDEL
   else if (mode == "delete" || mode == "remove") 
      gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(_delBtn),true);
#endif
}


void ChipsGuiToolbar::set_undo_redo_sensitivity(bool undo, bool redo)
{
   gtk_widget_set_sensitive(GTK_WIDGET(_undoBtn), undo);  
   gtk_widget_set_sensitive(GTK_WIDGET(_redoBtn), redo);  
}

