// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//

#ifndef CHIPSGUITOOLBAR_HH
#define CHIPSGUITOOLBAR_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "ChipsEventObserver.hh"
#include "ChipsGuiTooltipObject.hh"

using namespace std;


//---------------------------------------------------------------------------

class ChipsGuiToolbar: public ChipsEventObserver, public ChipsGuiTooltipObject
{
public:

  ChipsGuiToolbar();
  ~ChipsGuiToolbar();

  GtkWidget* base_widget() { return _toolbar; } 

  void set_style(int style); 
  void invoke_mode_change(int val); 
  void change_mode(int val); 

  virtual void execute_event (int event, void *data); 

  void initialize(); 
  void load_prefs(); 
  void update_gui_prefs();
  void set_undo_redo_sensitivity(bool undo, bool redo);
  void set_mode(string mode=""); 

  void show();
  void hide();
  int  get_height(); 

private:
  
  // mode buttons 
  GtkWidget*   _toolbar; 
  GtkToolItem* _emptyBtn;
  GtkToolItem* _panBtn;
  GtkToolItem* _rangeBtn;
  GtkToolItem* _zoomBtn;
  GtkToolItem* _selectBtn;
#ifdef CHIPS_GUI_ADDDEL
  GtkToolItem* _delBtn; 
#endif 
  GtkToolItem* _sep; 

  // cmd buttons 
#ifdef CHIPS_GUI_ADDDEL
  GtkToolItem* _loadBtn; 
#endif
  GtkToolItem* _undoBtn; 
  GtkToolItem* _redoBtn; 
  GtkToolItem* _printBtn; 
  GtkToolItem* _clearBtn; 

  bool _showZoom;
  bool _showRem; 
  int  _btnStyle; 
  string _defMode; 

  // signals for callbacks
  unsigned long _selectBtn_id;
  unsigned long _rangeBtn_id;
  unsigned long _panBtn_id;
  unsigned long _zoomBtn_id;


  // callbacks 
  static int pan_cb(GObject *object, gpointer user_data);
  static int range_cb(GObject *object, gpointer user_data);
  static int zoom_cb(GObject *object, gpointer user_data);
  static int select_cb(GObject *object, gpointer user_data);
#ifdef CHIPS_GUI_ADDDEL
  static int delete_cb(GObject *object, gpointer user_data);

  static void load_cb(GtkWidget *widget, gpointer data);
#endif
  static void undo_cb(GtkWidget *widget, gpointer data);
  static void redo_cb(GtkWidget *widget, gpointer data);
  static void print_cb(GtkWidget *widget, gpointer data);
  static void clear_cb(GtkWidget *widget, gpointer data);

};

#endif
