// 
//  Copyright (C) 2011,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "ChipsGuiMain.hh"
#include "ChipsGuiSettings.hh"
#include "ChipsGuiViewMenu.hh"
#include "ChipsGuiGridPropDialog.hh"

using namespace std;

extern ChipsGuiMain* chipsGui; 
extern ChipsGuiSettings* GuiSettings;

ChipsGuiViewMenu::ChipsGuiViewMenu() : ChipsGuiTooltipObject()
{
   int pos = 0; 
   base_w = gtk_menu_item_new_with_mnemonic("_View");

   menu = gtk_menu_new();

   menuItemTbar = gtk_check_menu_item_new_with_mnemonic("Show _Toolbar");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemTbar, pos++);

   menuItemProperty = gtk_check_menu_item_new_with_mnemonic("Show _Property Frame");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemProperty, pos++);

   menuItemInfo = gtk_check_menu_item_new_with_mnemonic("Show _Info Frame");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemInfo, pos++);

   menuItemNav = gtk_check_menu_item_new_with_mnemonic("Show _Coordinate Frame");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemNav, pos++);

   menuItemShell = gtk_check_menu_item_new_with_mnemonic("Show _Shell Frame");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemShell, pos++);


   menuItemAlt = gtk_check_menu_item_new_with_mnemonic("_Alternate Layout");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemAlt, pos++);

   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, pos++);

   gridLayout = gtk_check_menu_item_new_with_mnemonic("Show Grid Lines");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), gridLayout, pos++);

   gridProps = gtk_menu_item_new_with_label("Edit Grid Properties");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), gridProps, pos++);

   initialize(); 

   g_signal_connect(G_OBJECT(menuItemTbar), "toggled",
                    G_CALLBACK (toggled_cb), this);
   g_signal_connect(G_OBJECT(menuItemInfo), "toggled",
                    G_CALLBACK (toggled_cb), this);
   g_signal_connect(G_OBJECT(menuItemNav), "toggled",
                    G_CALLBACK (toggled_cb), this);
   g_signal_connect(G_OBJECT(menuItemProperty), "toggled",
                    G_CALLBACK (toggled_cb), this);
   g_signal_connect(G_OBJECT(menuItemShell), "toggled",
                    G_CALLBACK (toggled_cb), this);
   g_signal_connect(G_OBJECT(menuItemAlt), "toggled",
                    G_CALLBACK (alternate_cb), this);
   g_signal_connect(G_OBJECT(gridLayout), "toggled",
                    G_CALLBACK (gridvis_cb), this);
  g_signal_connect_swapped(G_OBJECT(gridProps), "activate",
                    G_CALLBACK (gridprops_cb), (gpointer) NULL);

   setup_tooltip(menuItemTbar, "Show the Toolbar Frame");
   setup_tooltip(menuItemTbar, "Show the Object Toolbar Frame");
   setup_tooltip(menuItemProperty, "Show the Properties Frame");
   setup_tooltip(menuItemInfo, "Show the Info List Frame");
   setup_tooltip(menuItemNav, "Show the Coordinate Navigation Frame");
   setup_tooltip(menuItemShell, "Show the Commandline Shell Frame");
   setup_tooltip(menuItemAlt, "Use alternate layout of Coordinate, Info, and Property Frames");
   setup_tooltip(gridLayout, "Show Layout grid lines in chips window");
   setup_tooltip(gridProps, "Modify the appearance of grid lines");

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
}

ChipsGuiViewMenu::~ChipsGuiViewMenu()
{
   
}

int ChipsGuiViewMenu::show()
{
   gtk_widget_show(menuItemTbar);
   gtk_widget_show(menuItemProperty);
   gtk_widget_show(menuItemInfo);
   gtk_widget_show(menuItemNav);
   gtk_widget_show(menuItemShell);
   gtk_widget_show(gridLayout); 
   gtk_widget_show(base_w); 
   return 0; 
}

bool ChipsGuiViewMenu::display_tbar()
{
   return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuItemTbar));
}

bool ChipsGuiViewMenu::display_info()
{
   return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuItemInfo));
}

bool ChipsGuiViewMenu::display_nav()
{
   return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuItemNav));
}

bool ChipsGuiViewMenu::display_property()
{
   return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuItemProperty));
}

bool ChipsGuiViewMenu::display_shell()
{
   return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuItemShell));
}

bool ChipsGuiViewMenu::display_grid()
{
   return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gridLayout));
}

int ChipsGuiViewMenu::toggled_cb(GObject *object, gpointer user_data)
{
   //ChipsGuiViewMenu* viewmenu = (ChipsGuiViewMenu*)user_data; 

   if (chipsGui)  chipsGui->update_panels(); 

   return 0;
}

int ChipsGuiViewMenu::gridvis_cb(GObject *object, gpointer user_data)
{
   ChipsGuiViewMenu* viewmenu = (ChipsGuiViewMenu*)user_data; 

   if (GuiSettings && viewmenu) GuiSettings->set_wingrids(viewmenu->display_grid()); 
   if (chipsGui)  chipsGui->update_panels(); 

   return 0;
}

int ChipsGuiViewMenu::alternate_cb(GObject *object, gpointer user_data)
{
   //ChipsGuiViewMenu* viewmenu = (ChipsGuiViewMenu*)user_data;

   if (chipsGui)  chipsGui->swap_layout(); 

   return 0;
}



void  ChipsGuiViewMenu::initialize()
{
   if (GuiSettings)
   {
      bool bval = GuiSettings->get_pane_tbar_vis(); 
      if (menuItemTbar)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemTbar), bval);
      bval = GuiSettings->get_pane_nav_vis(); 
      if (menuItemNav)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemNav), bval);
      bval = GuiSettings->get_pane_info_vis(); 
      if (menuItemInfo)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemInfo), bval);
      bval = GuiSettings->get_pane_prop_vis(); 
      if (menuItemProperty)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemProperty), bval);
      bval = GuiSettings->get_pane_shell_vis(); 
      if (menuItemShell)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuItemShell), bval);
      bval = GuiSettings->get_wingrids(); 
      if (gridLayout)
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gridLayout), bval);
   }
}

int  ChipsGuiViewMenu::gridprops_cb(GObject *object, gpointer data)
{
   ChipsGuiGridPropDialog* dialog = new ChipsGuiGridPropDialog(); 
   dialog->doit();
   delete dialog; 
   return 0; 
}

void ChipsGuiViewMenu::update_gui_prefs()
{
   initialize(); // simply need to call initialize to reload prefs  
}

