// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsGuiWindowFrame.hh"
#include "ChipsGuiColorWidget.hh"
#include "ChipsGuiMain.hh"
#include "ChipsEngine.hh"
#include "ChipsFrame.hh"
#include "ChipsCmdManager.hh"

using namespace std;

extern ChipsGuiMain* chipsGui;


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
ChipsGuiWindowFrame::ChipsGuiWindowFrame():ChipsGuiPropFrame(),ChipsGuiTooltipObject()
{
   _color = NULL;
   _object_states = STATE_CHNG_PROP_WIN;
}

// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
ChipsGuiWindowFrame::~ChipsGuiWindowFrame()
{
   if (_color)
      delete _color;
}

void ChipsGuiWindowFrame::build_form()
{
   GtkWidget* tmplabel;
   _frame = gtk_frame_new("Window Properties");
   _table = gtk_table_new(7,7,FALSE);
   gtk_container_add (GTK_CONTAINER (_frame), _table);
   _color = new ChipsGuiColorWidget();
   _color->build_form ();
   _color->set_foreground_color (false);
   _redraw =  gtk_check_button_new();

   tmplabel = gtk_label_new("Color");
   gtk_table_attach(GTK_TABLE(_table), tmplabel, 1, 2, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table), _color->base_widget (), 2, 3, 1, 2,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   tmplabel = gtk_label_new("Display Redraw");
   gtk_table_attach(GTK_TABLE(_table), tmplabel, 1, 2, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 10, 10);
   gtk_table_attach(GTK_TABLE(_table), _redraw, 2, 3, 2, 3,
      GTK_SHRINK, GTK_SHRINK, 10, 10);

   setup_tooltip(_redraw, "Disables displaying of changes made to the window");
   setup_tooltip(_color->base_widget(),"Sets the window's background color");

  _color->add_callback ("change_color", new ChipsGuiColorWidgetCallback (color_callback, this));
  _redraw_id = g_signal_connect(G_OBJECT(_redraw),"toggled",
        G_CALLBACK(redraw_callback), (void*)this);
}

void ChipsGuiWindowFrame::set_color(string color)
{
   set_engine_property((void*)color.c_str(), SET_WINDOW_COLOR); 
}

void ChipsGuiWindowFrame::set_redraw(bool val)
{
   int ival = val; 
   set_engine_property(static_cast<void*>(&ival),
      SET_WINDOW_REDRAW);
   _redrawval = val; 
}

void ChipsGuiWindowFrame::color_callback(ChipsGuiColorWidget *widget, void *data)
{
   ChipsGuiWindowFrame *winFrame = static_cast <ChipsGuiWindowFrame*>(data);

   if (!winFrame)  return; //ensure we have needed pointer 
   string clr = widget->get_chips_color ();
   winFrame->set_color(clr);
     
}


void ChipsGuiWindowFrame::redraw_callback(GtkWidget *widget, gpointer data)
{
   ChipsGuiWindowFrame *winFrame = static_cast <ChipsGuiWindowFrame*>(data);
   bool vis = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
   if (winFrame)  winFrame->set_redraw(vis);
}

void ChipsGuiWindowFrame::initialize(ChipsId* id)
{
   string colorval;
   if (chipsGui && !_eng)
      _eng = chipsGui->get_engine();

   _cid.set_frame_id(id->get_frame_id());

   _eng->get_property(&_cid, (void*)&colorval, SET_WINDOW_COLOR);
   _eng->get_property(&_cid, (void*)&_redrawval, SET_WINDOW_REDRAW);

   g_signal_handler_block(GTK_TOGGLE_BUTTON(_redraw), _redraw_id);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_redraw), _redrawval);
   _color->set_chips_color (colorval);
   g_signal_handler_unblock(GTK_TOGGLE_BUTTON(_redraw), _redraw_id);

}

