%include typemaps.i

%{
#include <numpy/noprefix.h>
%}

%init %{
  import_array();
%}

%{
  static int get_array_type(short* ptr) { return PyArray_SHORT; }
  static int get_array_type(long* ptr) { return PyArray_LONG; }
  static int get_array_type(float* ptr) { return PyArray_FLOAT; }
  static int get_array_type(double* ptr) { return PyArray_DOUBLE; }
%}

%{
/*Define helper function */
PyArrayObject * convert_array (PyObject *input, int arr_type, int dim){
  PyArrayObject *arr1;
  PyArrayObject *arr2;

  /* Check that obj is really an 2D array of bytes */

/*  if (!PyArray_Check(input)) {
    return NULL;
  }
*/

  /* check type (could also use arr->descr->type_num) */

/*  if (PyArray_ObjectType(input,0) != arr_type) {
    PyErr_SetString(PyExc_TypeError, \
                    "Incorrect array type");
    return NULL;
  }
*/
  if (input == Py_None)
     return NULL;

  arr1 = (PyArrayObject *)PyArray_FromAny(input, NULL, 0, 0, CONTIGUOUS, NULL);
  if (arr1 == NULL){
     PyErr_SetString(PyExc_TypeError, \
                    "Python object is not a contiguous array");
     return NULL;
  }

  if (arr1->descr->type_num != arr_type){
     if (PyArray_CanCastSafely (arr1->descr->type_num, arr_type) > 0||
         (PyArray_ISINTEGER(arr1) && PyTypeNum_ISINTEGER(arr_type))){
        arr2 = (PyArrayObject *)PyArray_Cast (arr1, arr_type);
        if (arr2 == NULL){
           PyErr_SetString(PyExc_TypeError, \
                    "Invalid array type");

	   Py_DECREF(arr1);
           return NULL;
	}
        Py_DECREF(arr1);
	arr1 = arr2;
     }
     else {
	PyErr_SetString(PyExc_TypeError, \
                    "Invalid array type");

	Py_DECREF(arr1);
        return NULL;
     }
  }

/*  if (arr1->nd < dim) { // we are really strict ! 
    char msg[80];
    sprintf (msg, "Incorrect number of dims: we want at least %d array", dim);
    PyErr_SetString(PyExc_TypeError, msg);
    return NULL;
  }
*/
  return arr1;
}
%}


%typemap(in) float* {
  PyArrayObject *arr;
  arr = convert_array ($input, PyArray_FLOAT, 1);
  if (arr != NULL){
     int arr_size = 1;
     for (int ii=arr->nd; ii--; arr_size *= arr->dimensions[ii]);
     $1 = new float[arr_size];
         for (int ii=arr_size; ii--;)
            $1[ii] = ((float *)(arr->data))[ii];
     Py_DECREF(arr);  /* Release our local copy of the PyArray */
  }
  else
     goto fail;

}

%typemap(freearg) float* {
   if ($1)
      delete []$1;
}

%typemap(in) float** (int numcol){
  PyArrayObject *arr;
  arr = convert_array ($input, PyArray_DOUBLE, 2);
  if (arr != NULL){
     numcol = arr->dimensions[1];
     $1 = new float*[arr->dimensions[0]];
     for (int ii=0; ii<arr->dimensions[0]; ii++){
       $1[ii] = new float[numcol];
       for (int jj=0; jj<numcol; jj++)
         $1[ii][jj] = ((float *)(arr->data))[ii*numcol+jj];
     }
     Py_DECREF(arr);  /* Release our local copy of the PyArray */
  }
  else
     goto fail;
}

%typemap(freearg) float** {
   if ($1){
      for (int ii=0; ii<numcol$argnum; ii++)
         delete []$1[ii];
      delete []$1;
   }
}


%typemap(in) double* {
  PyArrayObject *arr;
  arr = convert_array ($input, PyArray_DOUBLE, 1);
  if (arr != NULL){
     int arr_size = 1;
     for (int ii=arr->nd; ii--; arr_size *= arr->dimensions[ii]);
     $1 = new double[arr_size];
         for (int ii=arr_size; ii--;)
            $1[ii] = ((double *)(arr->data))[ii];
     Py_DECREF(arr);  /* Release our local copy of the PyArray */
  }
  else
     goto fail;
}

%typemap(freearg) double* {
   if ($1)
      delete []$1;
}

%typemap(in) double** (int numcol){
  PyArrayObject *arr;
  arr = convert_array ($input, PyArray_DOUBLE, 2);
  if (arr != NULL){
     numcol = arr->dimensions[1];
     $1 = new double*[arr->dimensions[0]];
     for (int ii=0; ii<arr->dimensions[0]; ii++){
       $1[ii] = new double[numcol];
       for (int jj=0; jj<numcol; jj++)
         $1[ii][jj] = ((double *)(arr->data))[ii*numcol+jj];
     }
     Py_DECREF(arr);  /* Release our local copy of the PyArray */
  }
  else
     goto fail;
}

%typemap(freearg) double** {
   if ($1){
      for (int ii=0; ii<numcol$argnum; ii++)
         delete []$1[ii];
      delete []$1;
   }
}


%typemap(in) int* {
  PyArrayObject *arr;
  arr = convert_array ($input, PyArray_INT, 1);
  if (arr == NULL){
     int arr_size = 1;
     for (int ii=arr->nd; ii--; arr_size *= arr->dimensions[ii]);
     $1 = new int[arr_size];
         for (int ii=arr_size; ii--;)
            $1[ii] = ((int *)(arr->data))[ii];
     Py_DECREF(arr);  /* Release our local copy of the PyArray */
  }
  else
     goto fail;
}

%typemap(freearg) int* {
   if ($1)
      delete []$1;
}


%typemap(out) float* {
  if (NULL == $1) {
    Py_INCREF(Py_None);
    $result = Py_None;
  } else {
    intp dims[1];
    dims[0] = 1;

    if (NULL == ($result =
                 PyArray_SimpleNewFromData(1, dims, get_array_type($1), $1)))
      return NULL;

    Py_INCREF(obj0);
    PyArray_BASE($result) = obj0;
  }
}

%typemap(out) double* {
  if (NULL == $1) {
    Py_INCREF(Py_None);
    $result = Py_None;
  } else {
    intp dims[1];
    dims[0] = intp(arg1->get_row () * arg1->get_col ());

    if (NULL == ($result =
                 PyArray_SimpleNewFromData(1, dims, get_array_type($1), $1)))
      return NULL;

    Py_INCREF(obj0);
    PyArray_BASE($result) = obj0;
  }
}



