/*                                                                
**  Copyright (C) 2006,2007,2013  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/* wrap these classes */
%include "numpy.i"
%module pychips
%{
  #include "ChipsRemoteMod.hh"     
  #include "ChipsId.hh"
  #include "ChipsDefines.hh"  
  #include "ChipsErrs.hh"

  #define SWIG_PYTHON_CAST_MODE
%}

%{
/*Define helper function */
double *create_1darray_double (PyObject *py_array, int &out_size){
   PyArrayObject *arr;
   double *out_array = NULL;
   out_size = 1;
   arr = convert_array (py_array, PyArray_DOUBLE, 1);
   if (arr) {
      for (int ii=arr->nd; ii--; out_size *= arr->dimensions[ii]);
      out_array = new double[out_size];
      for (int ii=out_size; ii--;)
         out_array[ii] = ((double *)(arr->data))[ii];
      Py_DECREF(arr);
   } else {
      out_array = 0;
      return 0;
   }
   return out_array;
}
%}


%{
/*Define helper function */
float *create_1darray_float (PyObject *py_array, int &out_size){
   PyArrayObject *arr;
   float *out_array = NULL;
   out_size = 1;
   arr = convert_array (py_array, PyArray_DOUBLE, 1);
   if (arr) {
      for (int ii=arr->nd; ii--; out_size *= arr->dimensions[ii]);
      out_array = new float[out_size];
      for (int ii=out_size; ii--;)
         out_array[ii] = (float)((double *)(arr->data))[ii];
      Py_DECREF(arr);
   } else {
      out_array = 0;
      return 0;
   }
   return out_array;
}
%}


%{
/* Define global variable for checking array sizes */
int current_array_size;
bool good_array_size;
%}


/***** BEGIN DOC STRING SECTION *****/
%feature("docstring", "load_font(fontname fontpath [fontstyle])\n\nAdds the specified font to the set of supported fonts. Supported fontsyles are chips_normal|chips_bold|chips_italic|chips_bolditalic.") load_font;
%feature("docstring", "set_preference_autoload(toggle)\n\n Tells the client to load or not to load preferences when initializing a new server. This flag can be overwritten by using create_server manually and setting the load_pref flag.\n") set_preference_autoload;
%feature("docstring", "load_preferences([filename])\n\nSets the internal default preferences to the values specified by filename (or $HOME/.chips.rc if omitted)") load_preferences;
%feature("docstring", "save_preferences([filename])\n\nSaves the internal default preference values to the specified file (or $HOME/.chips.rc if omitted).") save_preferences;
%feature("docstring", "set_preference(attribute, value)\n\nSets the default value of the specified attribute to the supplied value") set_preference;
%feature("docstring", "get_preference(attribute)\n\nReturns the current value of the specified attribute") get_preference;
%feature("docstring", "set_line_stipple(linetype, pattern)\n\nSets the stipple used to represent the specified line type to the supplied pattern") set_line_stipple;
%feature("docstring", "get_line_stipple(linetype)\n\nReturns the stipple pattern used to represent the specified line type") get_line_stipple;
%feature("docstring", "show_gui([path])\n\nOpens the chips gui. Optional path argument specifies the working directory for the gui invoked print dialog.") show_gui;
%feature("docstring", "hide_gui()\n\nCloses the chips gui") hide_gui;
%feature("docstring", "load_state(filename)\n\nLoads the specified state file") load_state;
%feature("docstring", "save_state(filename)\n\nSaves the current state of the chipsServer out to the specified file. The state file is a binary file and contains a copy of all data used to generate the state.") save_state;
%feature("docstring", "undo([tag|depth])\n\nUndoes the last executed command. If a tag is provided, commands upto the tag are undone. If a depth is provided, that many commands are undone.") undo;
%feature("docstring", "redo([tag|depth])\n\nReapplies the last undone command. If a tag is provided, commands upto the tag are reapplied. If a depth is provided, that many commands are redone.") redo;
%feature("docstring", "set_undo_tag(tagname)\n\nSets a tag that can be used by the undo/redo command to iterate to move to this state.") set_undo_tag;
%feature("docstring", "clear_undo_tag(tagname)\n\nErases a tag that was set by the user to mark a location in the undo/redo stack") clear_undo_tag;
%feature("docstring", "open_undo_block()\n\nMarks the beginning of a collection of individual commands which should be treated as a single unit by the undo/redo commands") open_undo_block;
%feature("docstring", "close_undo_block()\n\nMarks the end of a collection of individual commands which should be treated as a single unit by the undo/redo commands") close_undo_block;
%feature("docstring", "clear_errors()\n\nClears out any messages which may exist in the error list.") clear_errors;
%feature("docstring", "set_cascading_property([ChipsId,] start_object, attribute, value)\n\nChanges the specified attribute of all objects in the hierarchy from the start_object down to the provided value.") set_cascading_property;
%feature("docstring", "move([ChipsId,], object, xval, yval, [mode])\n\nMoves the specified object by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).") move;
%feature("docstring", "get_object_count([ChipsId,] object_type)\n\nGets the number of instances of the specified object type in the context specified by ChipsId (current context if omitted)") get_object_count;
%feature("docstring", "add_window([ChipsId,] [attribute_list])\nadd_window([ChipsId,] width, height [,units] [,attribute_list]\n\nAdds a window using user set specified attributes if provided") add_window;
%feature("docstring", "delete_window([ChipsId])\n\nDeletes the window specified by ChipsId (or current window if omitted)") delete_window;
%feature("docstring", "set_current_window(ChipsId)\n\nSets the current window to the one specified by ChipsId") set_current_window;
%feature("docstring", "set_window_title([ChipsId,] title)\n\nRequests the window manager to change the window's title to the provided value") set_window_title;
%feature("docstring", "get_window_title([ChipsId])\n\nIssues a request to the window manager to return the value in the window's title bar") get_window_title;
%feature("docstring", "set_window_bgcolor([ChipsId,] color)\n\nSets the specified window (current if ChipsId is omitted) to the specified color") set_window_bgcolor;
%feature("docstring", "get_window_bgcolor([ChipsId])\n\nGets the background color of the window specified by ChipsId (or current window if omitted") get_window_bgcolor;
%feature("docstring", "set_window_redraw([ChipsId,], toggle)\n\nTurns window redrawing on/off based on the value of toggle") set_window_redraw;
%feature("docstring", "get_window_redraw([ChipsId])\n\nReturns whether or not window redrawing is enabled") get_window_redraw;
%feature("docstring", "iconify_window([ChipsId])\n\nIssues a request to the window manager to iconify the window specified by ChipsId (current window if omitted)") iconify_window;
%feature("docstring", "deiconify_window([ChipsId])\n\nIssues a request to the window manager to raise the window specified by ChipsId (current window if omitted)") deiconify_window;
%feature("docstring", "set_frame_bgcolor([ChipsId,] color)\n\nSets the specified frame (current if ChipsId is omitted) to the specified color") set_frame_bgcolor;
%feature("docstring", "get_frame_bgcolor([ChipsId])\n\nGets the background color of the frame specified by ChipsId (or current frame if omitted") get_frame_bgcolor;
%feature("docstring", "set_frame_border_color([ChipsId,] color)\n\nSets the specified frame's (current if ChipsId is omitted) border color to the specified color") set_frame_border_color;
%feature("docstring", "get_frame_border_color([ChipsId])\n\nGets the border color of the frame specified by ChipsId (or current frame if omitted") get_frame_border_color;
%feature("docstring", "get_frame_border_visible([ChipsId])\n\nReturns whether or not the frame border is being displayed") get_frame_border_visible;
%feature("docstring", "display_frame_border([ChipsId])\n\nSets the visibility of the frame border on") display_frame_border;
%feature("docstring", "hide_frame_border([ChipsId])\n\nSets the visibility of the frame border off") hide_frame_border;
%feature("docstring", "get_frame_visible([ChipsId])\n\nReturns whether or not the frame is being displayed") get_frame_visible;
%feature("docstring", "display_frame([ChipsId])\n\nSets the visibility of the frame on") display_frame;
%feature("docstring", "hide_frame([ChipsId])\n\nSets the visibility of the frame off") hide_frame;
%feature("docstring", "pick_limits([ChipsId,] [dim])\n\nSets limits of a plot to the bounding box defined by left button press and mouse drag. Dim specifies X_AXIS, Y_AXIS, XY_AXIS (default is XY_AXIS)") pick_limits;
%feature("docstring", "pick([ChipsId], [count [, event_mask]])\n\nPrints the coordinate value of count instances of events that are specified in the event maskinitial default of button down clicks if omitted)") pick;
%feature("docstring", "get_pick([ChipsId], [count [, event_mask]])\n\nReturns an array of the coordinate values of count instances of events that are specified in the event_mask (initial default of button down clicks if omitted)") get_pick;
%feature("docstring", "add_frame([ChipsId,] [attribute_list])\nadd_frame([ChipsId,] x1, y1, x2, y2 [,attribute_list])\n\nAdds a frame to the window specified by ChipsId (current window if omitted). Uses normalized coordinates (x1,y1)-(x2,y2) if provided to determine the position of the frame.") add_frame;
%feature("docstring", "delete_frame([ChipsId])\n\nDeletes the frame specified by ChipsId (current frame if omitted)") delete_frame;
%feature("docstring", "set_framemode([ChipsId,] mode)\n\nSets the display mode of frames within the window to single (the window displays one frame), tiled (a user defined grid of frames is displayed), or placed mode (all frames are displayed at their normalized positions)") set_framemode;
%feature("docstring", "tile([ChipsId,] columns, rows)\n\nConfigures the window to display frames using a columns by rows grid") tile;
%feature("docstring", "row_tile([ChipsId,] rows)\n\nConfigures the window to display frames using the specified number of rows") row_tile;
%feature("docstring", "col_tile([ChipsId,] columns)\n\nConfigures the window to display frames using the specified number of columns") col_tile;
%feature("docstring", "prev_frame([ChipsId,])\n\nIf the window specified by ChipsId (or current window if omitted) has its frame mode set to single, cycle to the previous frame in the window's list of frames") prev_frame;
%feature("docstring", "next_frame([ChipsId,])\n\nIf the window specified by ChipsId (or current window if omitted) has its frame mode set to single, cycle to the next frame in the window's list of frames") next_frame;
%feature("docstring", "reposition_frame([ChipsId,] x1, y1, x2, y2)\n\nResets the normalized range of the specifed frame (current frame if ChipsId omitted) to the area defined from (x1,y1) to (x2,y2)") reposition_frame;
%feature("docstring", "view_single_frame([ChipsId])\n\nSets the display mode of the window specified by ChipsId (current if omitted) to single frame mode") view_single_frame;
%feature("docstring", "view_placed_frame([ChipsId])\n\nSets the display mode of the window specified by ChipsId (current if omitted) to placed frame mode where all frames are displayed in their normalized positions") view_placed_frame;
%feature("docstring", "arrange_frames([ChipsId,], id_list)\n\nChanges the order of the rendering of frames to the order specified by the list of frame ids. The list of ids is ordered from back to front for rendering") arrange_frames;
%feature("docstring", "set_frame_transparency([Chipsid,], toggle)\n\nFor the frame specified by ChipsId (current frame if omitted), set whether the frame's background is transparent based on toggle. Frames that are not transparent are displayed in their background color (bgcolor) value") set_frame_transparency;
%feature("docstring", "get_frame_transparency([ChipsId,])\n\nReturns whether the specified frame (current if ChipsId is omitted) has a transparent background") get_frame_transparency;
%feature("docstring", "blink_frames(ChipsId, frame2_id, [duration])\n\nThe window alternates displaying the frame specified by ChipsId and the frame specified by frame2_id at intervals of duration seconds") blink_frames;
%feature("docstring", "stop_blink()\n\nDiscontinues blinking frame mode") stop_blink;
%feature("docstring", "set_current_frame(ChipsId)\n\nSets the current frame to the one specified by ChipsId") set_current_frame;
%feature("docstring", "limits([ChipsId,] dimension, min, max)\nlimits([ChipsId,] object_type, object_id)\nlimits(ChipsId, dimension, object_type, ChipsId2)\n\nSets the limits of the axes specified by dimension (X_AXIS|Y_AXIS|XY_AXIS) in the plot specified by ChipsId (current if omitted). If min or max is set to the string 'AUTO' then that value is automatically set based on the data range. The method taking in an object type and object_id will set the X and Y axes of the current plot to the limits of the object identified by those parameters. The method taking in a ChipsId, a dimension, an object type, and a second ChipsId can be used to specify which dimensions to apply the limits of the object specified by object_type and ChipsId2.") limits;
%feature("docstring", "logarithmic_scale([ChipsId,], [dimension])\n\nSets the axes in the specified dimension (X_AXIS|Y_AXIS|XY_AXIS) to display using log scale. The optional ChipsId allows the user to specify a plot other than the current one. If a dimension is not specified, the change is applied to X and Y.") logarithmic_scale;
%feature("docstring", "log_scale([ChipsId,], [dimension])\n\nSets the axes in the specified dimension (X_AXIS|Y_AXIS|XY_AXIS) to display using log scale. The optional ChipsId allows the user to specify a plot other than the current one. If a dimension is not specified, the change is applied to X and Y.") log_scale;
%feature("docstring", "linear_scale([ChipsId,], [dimension])\n\nSets the axes in the specified dimension (X_AXIS|Y_AXIS|XY_AXIS) to display using linear scale. The optional ChipsId allows the user to specify a plot other than the current one. If a dimension is not specified, the change is applied to X and Y.") linear_scale;
%feature("docstring", "lin_scale([ChipsId,], [dimension])\n\nSets the axes in the specified dimension (X_AXIS|Y_AXIS|XY_AXIS) to display using linear scale. The optional ChipsId allows the user to specify a plot other than the current one. If a dimension is not specified, the change is applied to X and Y.") lin_scale;
%feature("docstring", "reverse_axes([ChipsId,], [dimension])\n\nToggles the display of ranges for the axes in the specified dimension (X_AXIS|Y_AXIS|XY_AXIS) between low-high and high-low. The optional ChipsId allows the user to specify a plot other than the current one. If a dimension is not specified, the change is applied to X and Y.") reverse_axes;
%feature("docstring", "get_label_visible([ChipsId])\n\nReturns whether the label specified by ChipsId (or current if omitted) is displayed or hidden") get_label_visible;
%feature("docstring", "get_line_visible([ChipsId])\n\nReturns whether the line specified by ChipsId (or current if omitted) is displayed or hidden") get_line_visible;
%feature("docstring", "get_point_visible([ChipsId])\n\nReturns whether the point specified by ChipsId (or current if omitted) is displayed or hidden") get_point_visible;
%feature("docstring", "get_region_visible([ChipsId])\n\nReturns whether the region specified by ChipsId (or current if omitted) is displayed or hidden") get_region_visible;
%feature("docstring", "get_plot_visible([ChipsId])\n\nReturns whether the plot specified by ChipsId (or current if omitted) is displayed or hidden") get_plot_visible;
%feature("docstring", "get_axis_visible([ChipsId])\n\nReturns whether the axis specified by ChipsId (or current if omitted) is displayed or hidden") get_axis_visible;
%feature("docstring", "clear_plot([ChipsId])\n\nDeletes all curves, contours, histograms, and annotations associated with the plot specified by ChipsId (or current plot if omitted)") clear_plot;
%feature("docstring", "get_major_ticks_visible([ChipsId])\n\nReturns whether the major ticks of the axis specified by ChipsId (or current if omitted) are displayed or hidden") get_major_tick_visible;
%feature("docstring", "get_minor_ticks_visible([ChipsId])\n\nReturns whether the minor ticks of the axis specified by ChipsId (or current if omitted) are displayed or hidden") get_minor_tick_visible;
%feature("docstring", "get_axis_label_visible([ChipsId])\n\nReturns whether the label of the axis specified by ChipsId (or current if omitted) is displayed or hidden") get_axis_label_visible;
%feature("docstring", "get_axis_ticklabel_visible([ChipsId])\n\nReturns whether the ticklabels of the axis specified by ChipsId (or current if omitted) are displayed or hidden") get_axis_ticklabel_visible;
%feature("docstring", "get_axis_majorgrid_visible([ChipsId])\n\nReturns whether the major grids of the axis specified by ChipsId (or current if omitted) are displayed or hidden") get_axis_majorgrid_visible;
%feature("docstring", "get_axis_minorgrid_visible([ChipsId])\n\nReturns whether the minor grids of the axis specified by ChipsId (or current if omitted) are displayed or hidden") get_axis_minorgrid_visible;
%feature("docstring", "display_plot([ChipsId])\n\nDisplays the plot instance specified by ChipsId (current instance if omitted)") display_plot;
%feature("docstring", "display_label([ChipsId])\n\nDisplays the label instance specified by ChipsId (current instance if omitted)") display_label;
%feature("docstring", "display_line([ChipsId])\n\nDisplays the line instance specified by ChipsId (current instance if omitted)") display_line;
%feature("docstring", "display_point([ChipsId])\n\nDisplays the point instance specified by ChipsId (current instance if omitted)") display_point;
%feature("docstring", "display_region([ChipsId])\n\nDisplays the region instance specified by ChipsId (current instance if omitted)") display_region;
%feature("docstring", "hide_plot([ChipsId])\n\nHides the plot instance specified by ChipsId (current instance if omitted)") hide_plot;
%feature("docstring", "hide_label([ChipsId])\n\nHides the label instance specified by ChipsId (current instance if omitted)") hide_label;
%feature("docstring", "hide_line([ChipsId])\n\nHides the line instance specified by ChipsId (current instance if omitted)") hide_line;
%feature("docstring", "hide_point([ChipsId])\n\nHides the point instance specified by ChipsId (current instance if omitted)") hide_point;
%feature("docstring", "hide_region([ChipsId])\n\nHides the region instance specified by ChipsId (current instance if omitted)") hide_region;
%feature("docstring", "delete_all_windows([ChipsId])\n\nDeletes all window instances in the context specified by ChipsId (current context if omitted)") delete_all_windows;
%feature("docstring", "delete_all_frames([ChipsId])\n\nDeletes all frame instances in the context specified by ChipsId (current context if omitted)") delete_all_frames;
%feature("docstring", "delete_all_plots([ChipsId])\n\nDeletes all plot instances in the context specified by ChipsId (current context if omitted)") delete_all_plots;
%feature("docstring", "delete_plot([ChipsId])\n\nDeletes the plot specified by ChipsId (or current plot if omitted)") delete_plot;
%feature("docstring", "delete_axis([ChipsId])\n\nDeletes the axis specified by ChipsId (or current axis if omitted)") delete_axis;
%feature("docstring", "delete_curve([ChipsId])\n\nDeletes the curve specified by ChipsId (or current curve if omitted)") delete_curve;
%feature("docstring", "delete_contour([ChipsId])\n\nDeletes the contour specified by ChipsId (or current contour if omitted)") delete_contour;
%feature("docstring", "delete_histogram([ChipsId])\n\nDeletes the histogram specified by ChipsId (or current histogram if omitted)") delete_histogram;
%feature("docstring", "delete_line([ChipsId])\n\nDeletes the line specified by ChipsId (or current line if omitted)") delete_line;
%feature("docstring", "delete_image([ChipsId])\n\nDeletes the image specified by ChipsId (or current image if omitted)") delete_image;
%feature("docstring", "delete_colorbar([ChipsId])\n\nDeletes the colorbar specified by ChipsId (or current colorbar if omitted)") delete_colorbar;
%feature("docstring", "delete_label([ChipsId])\n\nDeletes the label specified by ChipsId (or current label if omitted)") delete_label;
%feature("docstring", "delete_point([ChipsId])\n\nDeletes the point specified by ChipsId (or current point if omitted)") delete_point;
%feature("docstring", "delete_region([ChipsId])\n\nDeletes the region specified by ChipsId (or current region if omitted)") delete_region;
%feature("docstring", "set_current_plot(ChipsId)\n\nSets the current plot to the one specified by ChipsId") set_current_plot;
%feature("docstring", "set_current_axis(ChipsId)\n\nSets the current axis to the one specified by ChipsId") set_current_axis;
%feature("docstring", "set_current_curve(ChipsId)\n\nSets the current curve to the one specified by ChipsId") set_current_curve;
%feature("docstring", "set_current_contour(ChipsId)\n\nSets the current contour to the one specified by ChipsId") set_current_contour;
%feature("docstring", "set_current_histogram(ChipsId)\n\nSets the current histogram to the one specified by ChipsId") set_current_histogram;
%feature("docstring", "set_current_label(ChipsId)\n\nSets the current label to the one specified by ChipsId") set_current_label;
%feature("docstring", "set_current_line(ChipsId)\n\nSets the current line to the one specified by ChipsId") set_current_line;
%feature("docstring", "set_current_point(ChipsId)\n\nSets the current point to the one specified by ChipsId") set_current_point;
%feature("docstring", "set_current_region(ChipsId)\n\nSets the current region to the one specified by ChipsId") set_current_region;
%feature("docstring", "set_current_image(ChipsId)\n\nSets the current image to the one specified by ChipsId") set_current_image;
%feature("docstring", "set_current_colorbar(ChipsId)\n\nSets the current colorbar to the one specified by ChipsId") set_current_colorbar;
%feature("docstring", "lock()\n\nLock the chipsServer so that the server will not process commands issued by other clients.") lock;
%feature("docstring", "unlock()\n\nRemove a lock on the chipsServer so that the server can process commands issued by other clients.") unlock;
%feature("docstring", "check_server_exists(server_id)\n\nChecks to see if the the client is able to connect to the specified server.") check_server_exists;
%feature("docstring", "list_servers()\n\nReturns a list of potential servers that the user may be able to access. A call to check_server_exists() is suggested before attempting to connect to a listed candidate. Note that the status of available servers could change between the time this call is made and a connection attempt is made.") list_servers;
%feature("docstring", "connect(server_id)\n\nIf the client is not currently connected to a chipsServer, attempt to attach to the specified server")connect ;
%feature("docstring", "create_server(loadprefs,[serverId])\n\nThis routine allows the user to create a new chips server if the client is currently not connected to a server. The loadprefs parameter determines if the user's .chips.rc file is loaded when the server is instantiated. An optional serverId string may be specified.") create_server;
%feature("docstring", "disconnect()\n\nDetaches this client from its chipsServer. This command may be used to disconnect from the current chipsServer to connect to a different server") disconnect;
%feature("docstring", "get_server_id()\n\nReturns the id of the chipsServer being this chips client is connected to. The id may be used in a connect call by another client to attach to the same chipsServer") get_server_id;
%feature("docstring", "get_server_cwd()\n\nReturns the directory that the chipsServer considers the current working directory.") get_server_cwd;
%feature("docstring", "set_plot_title([ChipsId,] title)\n\nSets the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title;
%feature("docstring", "get_plot_title([ChipsId])\n\nReturns the title of the plot specified by ChipsId (current plot if omitted)") get_plot_title;
%feature("docstring", "set_plot_title_color([ChipsId,] color)\n\nSets the color of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_color;
%feature("docstring", "get_plot_title_color([ChipsId])\n\nReturns the title color of the plot specified by ChipsId (current plot if omitted)") get_plot_title_color;
%feature("docstring", "set_plot_title_angle([ChipsId,] angle)\n\nSets the angle of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_angle;
%feature("docstring", "get_plot_title_angle([ChipsId])\n\nReturns the title angle of the plot specified by ChipsId (current plot if omitted)") get_plot_title_angle;
%feature("docstring", "set_plot_title_halign([ChipsId,] alignment)\n\nSets the horizontal alignment of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_halign;
%feature("docstring", "get_plot_title_halign([ChipsId])\n\nReturns the title horizontal alignment for the plot specified by ChipsId (current plot if omitted)") get_plot_title_halign;
%feature("docstring", "set_plot_title_valign([ChipsId,] alignment)\n\nSets the vertical alignment of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_valign;
%feature("docstring", "get_plot_title_valign([ChipsId])\n\nReturns the title vertical alignment for the plot specified by ChipsId (current plot if omitted)") get_plot_title_valign;
%feature("docstring", "set_plot_title_size([ChipsId,] size)\n\nSets the fontsize of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_size;
%feature("docstring", "get_plot_title_size([ChipsId])\n\nReturns the title fontsize of the plot specified by ChipsId (current plot if omitted)") get_plot_title_size;
%feature("docstring", "set_plot_title_font([ChipsId,] font)\n\nSets the font of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_font;
%feature("docstring", "get_plot_title_font([ChipsId])\n\nReturns the title font of the plot specified by ChipsId (current plot if omitted)") get_plot_title_font;
%feature("docstring", "set_plot_title_fontstyle([ChipsId,] fontstyle)\n\nSets the fontstyle of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_fontstyle;
%feature("docstring", "get_plot_title_fontstyle([ChipsId])\n\nReturns the title fontstyle of the plot specified by ChipsId (current plot if omitted)") get_plot_title_fontstyle;
%feature("docstring", "set_plot_title_xpos([ChipsId,] xpos)\n\nSets the normalized x position of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_xpos;
%feature("docstring", "get_plot_title_xpos([ChipsId])\n\nReturns the normalized x position for the title of the plot specified by ChipsId (current plot if omitted)") get_plot_title_xpos;
%feature("docstring", "set_plot_title_ypos([ChipsId,] ypos)\n\nSets the normalized y position of the title of the specified plot (current if ChipsId is omitted) to the provided value") set_plot_title_ypos;
%feature("docstring", "get_plot_title_ypos([ChipsId])\n\nReturns the normalized y position for the title of the plot specified by ChipsId (current plot if omitted)") get_plot_title_ypos;
%feature("docstring", "set_plot_xlabel([ChipsId,] text)\n\nSets the plot's current X axis label to the the provided value. If a ChipsId is provided, it is used to determine the currency.") set_plot_xlabel;
%feature("docstring", "set_plot_ylabel([ChipsId,] text)\n\nSets the plot's current Y axis label to the the provided value. If a ChipsId is provided, it is used to determine the currency.") set_plot_ylabel;
%feature("docstring", "get_plot_range([ChipsId])\n\nReturns the X and Y ranges of the current axes of the current plot. The values are returned as [xmin, xmax, ymin, ymax].") get_plot_range;
%feature("docstring", "get_plot_xrange([ChipsId])\n\nReturns the X range of the data in the current plot") get_plot_xrange;
%feature("docstring", "get_plot_yrange([ChipsId])\n\nReturns the Y range of the data in the current plot") get_plot_yrange;
%feature("docstring", "get_curve_range([ChipsId])\n\nReturns the X and Y ranges of the specified curve. The values are returned as [xmin, xmax, ymin, ymax].") get_curve_range;
%feature("docstring", "get_curve_xrange([ChipsId])\n\nReturns the X range of the specified curve. The values are returned as [xmin, xmax].") get_curve_xrange;
%feature("docstring", "get_curve_yrange([ChipsId])\n\nReturns the Y range of the specified curve. The values are returned as [ymin, ymax].") get_curve_yrange;
%feature("docstring", "get_contour_range([ChipsId])\n\nReturns the X and Y ranges of the specified contour. The values are returned as [xmin, xmax, ymin, ymax].") get_contour_range;
%feature("docstring", "get_contour_xrange([ChipsId])\n\nReturns the X range of the specified contour. The values are returned as [xmin, xmax].") get_contour_xrange;
%feature("docstring", "get_contour_yrange([ChipsId])\n\nReturns the Y range of the specified contour. The values are returned as [ymin, ymax].") get_contour_yrange;
%feature("docstring", "get_contour_zrange([ChipsId])\n\nReturns the Z range of the specified contour. The values are returned as [zmin, zmax].") get_contour_yrange;
%feature("docstring", "get_histogram_range([ChipsId])\n\nReturns the X and Y ranges of the specified histogram. The values are returned as [xmin, xmax, ymin, ymax].") get_histogram_range;
%feature("docstring", "get_histogram_xrange([ChipsId])\n\nReturns the X range of the specified histogram. The values are returned as [xmin, xmax].") get_histogram_xrange;
%feature("docstring", "get_histogram_yrange([ChipsId])\n\nReturns the Y range of the specified histogram. The values are returned as [ymin, ymax].") get_histogram_yrange;
%feature("docstring", "get_axis_range([ChipsId])\n\nReturns the range of the specified axis. The values are returned as [min, max].") get_axis_range;
%feature("docstring", "set_curve_symbolstyle([ChipsId,], style)\n\nSets the symbol style of the curve specified by ChipsId (or current curve if omitted) to the value specified by style") set_curve_symbolstyle;
%feature("docstring", "set_curve_symbolfill([ChipsId,], fill)\n\nSets the symbol fill of the curve specified by ChipsId (or current curve if omitted) to the value specified by fill") set_curve_symbolfill;
%feature("docstring", "set_curve_symbolsize([ChipsId,], size)\n\nSets the symbol size of the curve specified by ChipsId (or current curve if omitted) to the value specified by size") set_curve_symbolsize;
%feature("docstring", "set_curve_symbolcolor([ChipsId,], color)\n\nSets the symbol color of the curve specified by ChipsId (or current curve if omitted) to the value specified by color") set_curve_symbolcolor;
%feature("docstring", "set_curve_linestyle([ChipsId,], style)\n\nSets the line style of the curve specified by ChipsId (or current curve if omitted) to the value specified by style") set_curve_linestyle;
%feature("docstring", "set_curve_linethickness([ChipsId,], thickness)\n\nSets the line thickness of the curve specified by ChipsId (or current curve if omitted) to the value specified by thickness") set_curve_linethickness;
%feature("docstring", "set_curve_linecolor([ChipsId,], color)\n\nSets the line color of the curve specified by ChipsId (or current curve if omitted) to the value specified by color") set_curve_linecolor;
%feature("docstring", "set_curve_errstyle([ChipsId,], style)\n\nSets the error bar style ['bar'|'cap'] for the curve specified by ChipsId (or current curve if omitted) to the value specified by style") set_curve_errstyle;
%feature("docstring", "set_curve_errcolor([ChipsId,], color)\n\nSets the error bar color for the curve specified by ChipsId (or current curve if omitted) to the value specified by color") set_curve_errcolor;
%feature("docstring", "set_curve_errthickness([ChipsId,], thickness)\n\nSets the error bar thickness for the curve specified by ChipsId (or current curve if omitted) to the value specified by thickness") set_curve_errthickness;
%feature("docstring", "set_curve_errup([ChipsId,], toggle)\n\nControls whether up err bars should be displayed for the curve specified by ChipsId (or current curve if omitted). Only applies to curves containing up error data.") set_curve_errup;
%feature("docstring", "set_curve_errdown([ChipsId,], toggle)\n\nControls whether down err bars should be displayed for the curve specified by ChipsId (or current curve if omitted). Only applies to curves containing down error data.") set_curve_errdown;
%feature("docstring", "set_curve_errleft([ChipsId,], toggle)\n\nControls whether left err bars should be displayed for the curve specified by ChipsId (or current curve if omitted). Only applies to curves containing left error data.") set_curve_errleft;
%feature("docstring", "set_curve_errright([ChipsId,], toggle)\n\nControls whether right err bars should be displayed for the curve specified by ChipsId (or current curve if omitted). Only applies to curves containing right error data.") set_curve_errright;
%feature("docstring", "set_curve_errcaplength([ChipsId,], length)\n\nSets the length in pixels of the error bar caps of the curve specified by ChipsId (or current curve if omitted) to the value specified by length") set_curve_errcaplength;
%feature("docstring", "set_histogram_errcaplength([ChipsId,], length)\n\nSets the length in pixels of the error bar caps of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by length") set_histogram_errcaplength;
%feature("docstring", "set_curve_limitlength([ChipsId,], length)\n\nSets the length of the bar for the upper and lower limits of the curve specified by ChipsId (or the current curve if ommitted) to the value specified by length. The length is in plot normalized coordinates and should be a value between 0 and 1")set_curve_limitlength;
%feature("docstring", "set_curve_limitoverride([ChipsId,], toggle)\n\nControls whether a point on a curve, which has data for both error bars and limits, displays both the error bar and limits. Setting the toggle to true causes only limits to be displayed while setting it false will have both limits and error bars be displayed on an individual point. The curve is specified by the ChipsId (or the current curve if ommited).")set_curve_limitoverride;
%feature("docstring", "set_histogram_symbolstyle([ChipsId,], style)\n\nSets the symbol style of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by style") set_histogram_symbolstyle;
%feature("docstring", "set_histogram_symbolfill([ChipsId,], fill)\n\nSets the symbol fill of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by fill") set_histogram_symbolfill;
%feature("docstring", "set_histogram_symbolsize([ChipsId,], size)\n\nSets the symbol size of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by size") set_histogram_symbolsize;
%feature("docstring", "set_histogram_symbolcolor([ChipsId,], color)\n\nSets the symbol color of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by color") set_histogram_symbolcolor;
%feature("docstring", "set_histogram_linestyle([ChipsId,], style)\n\nSets the line style of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by style") set_histogram_linestyle;
%feature("docstring", "set_histogram_linethickness([ChipsId,], thickness)\n\nSets the line thickness of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by thickness") set_histogram_linethickness;
%feature("docstring", "set_histogram_linecolor([ChipsId,], color)\n\nSets the line color of the histogram specified by ChipsId (or current histogram if omitted) to the value specified by color") set_histogram_linecolor;
%feature("docstring", "set_histogram_errstyle([ChipsId,], style)\n\nSets the error bar style ['bar'|'cap'] for the histogram specified by ChipsId (or current histogram if omitted) to the value specified by style") set_histogram_errstyle;
%feature("docstring", "set_histogram_errcolor([ChipsId,], color)\n\nSets the error bar color for the histogram specified by ChipsId (or current histogram if omitted) to the value specified by color") set_histogram_errcolor;
%feature("docstring", "set_histogram_errthickness([ChipsId,], thickness)\n\nSets the error bar thickness for the histogram specified by ChipsId (or current histogram if omitted) to the value specified by thickness") set_histogram_errthickness;
%feature("docstring", "set_histogram_errup([ChipsId,], toggle)\n\nControls whether up err bars should be displayed for the histogram specified by ChipsId (or current histogram if omitted). Only applies to histograms containing up error data.") set_histogram_errup;
%feature("docstring", "set_histogram_errdown([ChipsId,], toggle)\n\nControls whether down err bars should be displayed for the histogram specified by ChipsId (or current histogram if omitted). Only applies to histograms containing down error data.") set_histogram_errdown;
%feature("docstring", "set_histogram_dropline([ChipsId,], toggle)\n\nControls whether droplines should be displayed at bin boundaries for the histogram specified by ChipsId (or current histogram if omitted).") set_histogram_dropline;
%feature("docstring", "get_histogram_dropline([ChipsId])\n\nReturns whether droplines are displayed at bin boundaries for the histogram specified by ChipsId (or current histogram if omitted).") get_histogram_dropline;
%feature("docstring", "bind_axes(axis1_id, axis2_id)\nbind_axes(ChipsId, plot_id, axis_id)\nbind_axes(plot1_id, axis1_id, plot2_id, axis2_id)\n\nThe bind_axes command synchronizes a pair of axes so that they hace the same limits, tickstyles, and other properties. If both axes are in the current plot the bind command can be utilized with just the two axis ids, otherwise a ChipsId structure can be passed in to set the context of which plots to bind.") bind_axes;
%feature("docstring", "unbind_axes(axis1_id, axis2_id)\nunbind_axes(ChipsId, plot_id, axis_id)\nunbind_axes(plot1_id, axis1_id, plot2_id, axis2_id)\n\nThe unbind_axes command disconnects the specified bound axes so that they are no longer synchronized. If both axes are in the current plot the unbind command can be utilized with just the two axis ids, otherwise a ChipsId structure can be passed in to set the context of which plots to unbind.") unbind_axes;
%feature("docstring", "set_axis_color([ChipsId,], color)\n\nSets the color of the axis specified by ChipsId (or current axis if omitted) to the value specified by color") set_axis_color;
%feature("docstring", "set_axis_major_color([ChipsId,], color)\n\nSets the major tick color of the axis specified by ChipsId (or current axis if omitted) to the value specified by color") set_axis_major_color;
%feature("docstring", "set_axis_minor_color([ChipsId,], color)\n\nSets the minor tick color of the axis specified by ChipsId (or current axis if omitted) to the value specified by color") set_axis_minor_color;
%feature("docstring", "set_label_color([ChipsId,], color)\n\nSets the color of the label specified by ChipsId (or current label if omitted) to the value specified by color") set_label_color;
%feature("docstring", "set_line_color([ChipsId,], color)\n\nSets the color of the line specified by ChipsId (or current line if omitted) to the value specified by color") set_line_color;
%feature("docstring", "set_point_color([ChipsId,], color)\n\nSets the color of the point specified by ChipsId (or current point if omitted) to the value specified by color") set_point_color;
%feature("docstring", "set_region_edgecolor([ChipsId,], color)\n\nSets the edge color of the region specified by ChipsId (or current region if omitted) to the value specified by color") set_region_edgecolor;
%feature("docstring", "set_region_fillcolor([ChipsId,], color)\n\nSets the fill color of the region specified by ChipsId (or current region if omitted) to the value specified by color") set_region_fillcolor;
%feature("docstring", "set_axis_label_color([ChipsId,], color)\n\nSets the label color of the axis specified by ChipsId (or current axis if omitted) to the value specified by color") set_axis_label_color;
%feature("docstring", "set_axis_ticklabel_color([ChipsId,], color)\n\nSets the ticklabel color of the axis specified by ChipsId (or current axis if omitted) to the value specified by color") set_axis_ticklabel_color;
%feature("docstring", "set_axis_majorgrid_color([ChipsId,], color)\n\nSets the majorgrid color of the axis specified by ChipsId (or current axis if omitted) to the value specified by color") set_axis_majorgrid_color;
%feature("docstring", "set_axis_minorgrid_color([ChipsId,], color)\n\nSets the minorgrid color of the axis specified by ChipsId (or current axis if omitted) to the value specified by color") set_axis_minorgrid_color;
%feature("docstring", "set_contour_color([ChipsId,], color)\n\nSets the color of the contour specified by ChipsId (or current contour if omitted) to the value specified by color") set_contour_color;
%feature("docstring", "set_plot_axisstyle([ChipsId,], style)\n\nSets the axis style (chips_open|chips_box|chips_closed) of the plot specified by ChipsId (or current plot if omitted) to the value specified by style") set_plot_axisstyle;
%feature("docstring", "set_plot_cornerstyle([ChipsId,], style)\n\nSets the corner style (chips_miter|chips_bevel|chips_filled|chips_notch) of the plot specified by ChipsId (or current plot if omitted) to the value specified by style") set_plot_cornerstyle;
%feature("docstring", "get_plot_axisstyle([ChipsId,], style)\n\nReturns the axis style of the plot specified by ChipsId (or current plot if omitted).") get_plot_axisstyle;
%feature("docstring", "get_plot_cornerstyle([ChipsId,], style)\n\nReturns the corner style of the plot specified by ChipsId (or current plot if omitted).") get_plot_cornerstyle;
%feature("docstring", "set_axis_major_style([ChipsId,], style)\n\nSets the major tick style (chips_inside|chips_outside|chips_centered) of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_major_style;
%feature("docstring", "set_axis_minor_style([ChipsId,], style)\n\nSets the minor tick style (chips_inside|chips_outside|chips_centered) of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_minor_style;
%feature("docstring", "set_axis_label_font([ChipsId,], style)\n\nSets the font of the axis label specified by ChipsId (or current axis if omitted) to the provided font") set_axis_label_font;
%feature("docstring", "set_axis_label_fontstyle([ChipsId,], style)\n\nSets the label fontstyle ('bold'|'bolditalic'|'italic'|'normal') of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_label_fontstyle;
%feature("docstring", "set_axis_ticklabel_fontstyle([ChipsId,], style)\n\nSets the ticklabel fontstyle ('bold'|'bolditalic'|'italic'|'normal') of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_ticklabel_fontstyle;
%feature("docstring", "set_axis_ticklabel_style([ChipsId,], style)\n\nSets the ticklabel style (chips_inside|chips_outside) of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_ticklabel_style;
%feature("docstring", "set_axis_majorgrid_style([ChipsId,], style)\n\nSets the majorgrid style (chips_solid|chips_dot|chips_longdash|chips_shortdash|chips_dotlongdash|chips_dotshortdash|chips_shortdashlongdash) of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_majorgrid_style;
%feature("docstring", "set_axis_minorgrid_style([ChipsId,], style)\n\nSets the minorgrid style (chips_solid|chips_dot|chips_longdash|chips_shortdash|chips_dotlongdash|chips_dotshortdash|chips_shortdashlongdash) of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_minorgrid_style;
%feature("docstring", "set_contour_style([ChipsId,], style)\n\nSets the style ('solid'|'dot'|'longdash'|'shortdash'|'dotlongdash'|'dotshortdash'|'shortdashlongdash') of the contour specified by ChipsId (or current contour if omitted) to the value specified by style") set_contour_style;
%feature("docstring", "set_axis_tickstyle([ChipsId,], style)\n\nSets the major and minor tickstyle (chips_inside|chips_outside|chips_centered) of the axis specified by ChipsId (or current axis if omitted) to the value specified by style") set_axis_tickstyle;
%feature("docstring", "set_label_fontstyle([ChipsId,], style)\n\nSets the fontstyle ('bold'|'bolditalic'|'italic'|'normal') of the label specified by ChipsId (or current label if omitted) to the value specified by style") set_label_fontstyle;
%feature("docstring", "set_line_style([ChipsId,], style)\n\nSets the style (chips_solid|chips_dot|chips_longdash|chips_shortdash|chips_dotlongdash|chips_dotshortdash|chips_shortdashlongdash) of the line specified by ChipsId (or current line if omitted) to the value specified by style") set_line_style;
%feature("docstring", "set_region_edgestyle([ChipsId,], style)\n\nSets the edge style (chips_solid|chips_dot|chips_longdash|chips_shortdash|chips_dotlongdash|chips_dotshortdash|chips_shortdashlongdash|chips_noline) of the region specified by ChipsId (or current region if omitted) to the value specified by style") set_region_edgestyle;
%feature("docstring", "set_region_fillstyle([ChipsId,], style)\n\nSets the fill style (chips_solidfill|chips_updiagonal|chips_downdiagonal|chips_horizontal|chips_vertical|chips_crisscross|chips_brick|chips_grid|chips_hexagon|chips_polkadot|chips_zigzag|chips_wave|chips_flower|chips_userfill1|chips_userfill2|chips_userfill3) of the region specified by ChipsId (or current region if omitted) to the value specified by style") set_region_fillstyle;
%feature("docstring", "load_fill(filename, [slot])\n\nLoads the specified xbm file into the chips_userfill1, chips_userfill2, or chips_userfill3 slot. If the slot is not provided, chips_userfill1 is used.") load_fill;
%feature("docstring", "load_colormap(r, g, b, [alpha], [slot])\nload_colormap(filename, [slot])\n\nLoads the specified colormap into the chips_usercmap1, chips_usercmap2, or chips_usercmap3 slot. Input can be either a file name or four arrays of red, green, blue, and alpha values. If the slot is not provided, chips_usercmap1 is used. If an alpha is not provided, a constant level of one is used.") load_colormap;
%feature("docstring", "set_histogram_fillstyle([ChipsId,], style)\n\nSets the fill style (chips_solidfill|chips_updiagonal|chips_downdiagonal|chips_horizontal|chips_vertical|chips_crisscross|chips_brick|chips_grid|chips_hexagon|chips_polkadot|chips_zigzag|chips_wave|chips_flower|chips_userfill1|chips_userfill2|chips_userfill3) of the histogram specified by ChipsId (or current region if omitted) to the value specified by style") set_histogram_fillstyle;
%feature("docstring", "set_plot_leftmargin([ChipsId,], size)\n\nSets the left margin size as a normalized value for the plot specified by ChipsId (or current plot if omitted).") set_plot_leftmargin;
%feature("docstring", "set_plot_rightmargin([ChipsId,], size)\n\nSets the right margin size as a normalized value for the plot specified by ChipsId (or current plot if omitted).") set_plot_rightmargin;
%feature("docstring", "set_plot_topmargin([ChipsId,], size)\n\nSets the top margin size as a normalized value for the plot specified by ChipsId (or current plot if omitted).") set_plot_topmargin;
%feature("docstring", "set_plot_bottommargin([ChipsId,], size)\n\nSets the bottom margin size as a normalized value for the plot specified by ChipsId (or current plot if omitted).") set_plot_bottommargin;
%feature("docstring", "set_axis_ticklabel_size([ChipsId,], size)\n\nSets the ticklabel size of the axis specified by ChipsId (or current axis if omitted) to the value specified by size") set_axis_ticklabel_size;
%feature("docstring", "set_axis_label_size([ChipsId,], size)\n\nSets the label size of the axis specified by ChipsId (or current axis if omitted) to the value specified by size") set_axis_label_size;
%feature("docstring", "get_axis_label_size([ChipsId])\n\nReturns the label size of the axis specified by ChipsId (or current axis if omitted). An error is returned if accessing more than one axis.") get_axis_label_size;
%feature("docstring", "set_label_size([ChipsId,], size)\n\nSets the size of the label specified by ChipsId (or current label if omitted) to the value specified by size") set_label_size;
%feature("docstring", "set_point_size([ChipsId,], size)\n\nSets the size of the point specified by ChipsId (or current point if omitted) to the value specified by size") set_point_size;
%feature("docstring", "set_axis_major_thickness([ChipsId,], thickness)\n\nSets the major tick thickness for the axis specified by ChipsId (or current axis if omitted) to the value specified by thickness") set_axis_major_thickness;
%feature("docstring", "set_axis_minor_thickness([ChipsId,], thickness)\n\nSets the minor tick thickness for the axis specified by ChipsId (or current axis if omitted) to the value specified by thickness") set_axis_minor_thickness;
%feature("docstring", "set_axis_majorgrid_thickness([ChipsId,], thickness)\n\nSets the major grid thickness for the axis specified by ChipsId (or current axis if omitted) to the value specified by thickness") set_axis_majorgrid_thickness;
%feature("docstring", "set_axis_minorgrid_thickness([ChipsId,], thickness)\n\nSets the minor grid thickness for the axis specified by ChipsId (or current axis if omitted) to the value specified by thickness") set_axis_minorgrid_thickness;
%feature("docstring", "set_contour_thickness([ChipsId,], thickness)\n\nSets the thickness of the contour specified by ChipsId (or current contour if omitted) to the value specified by thickness") set_contour_thickness;
%feature("docstring", "set_axis_thickness([ChipsId,], thickness)\n\nSets the thickness of the axis specified by ChipsId (or current axis if omitted) to the value specified by thickness") set_axis_thickness;
%feature("docstring", "set_line_thickness([ChipsId,], thickness)\n\nSets the thickness of the line specified by ChipsId (or current line if omitted) to the value specified by thickness") set_line_thickness;
%feature("docstring", "set_region_edgethickness([ChipsId,], thickness)\n\nSets the edge thickness of the region specified by ChipsId (or current region if omitted) to the value specified by thickness") set_region_edgethickness;
%feature("docstring", "set_axis_automin([ChipsId,], toggle)\n\nSets whether the axis specified by ChipsId (or current axis if omitted) should automatically recalculate the minimum limit value when a new curve, contour, or histogram is added to the plot") set_axis_automin;
%feature("docstring", "set_axis_automax([ChipsId,], toggle)\n\nSets whether the axis specified by ChipsId (or current axis if omitted) should automatically recalculate the maximum limit value when a new curve, contour, or histogram is added to the plot") set_axis_automax;
%feature("docstring", "set_arbitrary_tick_positions([ChipsId,], positions, [labels])\n\nSets the location of arbitrary ticks based on the values in the positions array. If a labels array is also provided, the strings in the label array are used at their repsective positions. The axis tick mode must be set to 'arbitrary' to display arbitrary ticks.") set_arbitrary_tick_positions;
%feature("docstring", "reposition_plot([ChipsId,] x1, y1, x2, y2)\n\nReadjusts the plot's margins such that the plot drawing area spans the normalized area from (x1,y1) to (x2,y2).") reposition_plot;
%feature("docstring", "print_window([ChipsId,] [filename [,options]])\n\nDumps the contents of the specified (or current) window to the printer. If a filename is provided, an output file is generated in lieu of sending the data to the printer.") print_window;
%feature("docstring", "adjust_grid_xgap([ChipsId,] gap)\n\nSets the gap between columns of plots in a grid to the specified value. Gap is specified as a percentage of the available frame width.") adjust_grid_xgap;
%feature("docstring", "adjust_grid_ygap([ChipsId,] gap)\n\nSets the gap between rows of plots in a grid to the specified value. Gap is specified as a percentage of the available frame height.") adjust_grid_ygap;
%feature("docstring", "adjust_grid_gaps([ChipsId,] xgap, ygap)\n\nSets the gap between columns in a grid to xgap and the spacing between rows to ygap. Xgap and ygap are specified as a percentage of the available frame height.") adjust_grid_gaps;
%feature("docstring", "adjust_grid_xrelsize([ChipsId,] column, ratio)\n\nModifies the widths of all columns such that the width of the specified column (1-n, where 1=leftmost) is determined by the specified ratio (1=uniform width, 2=twice as wide, etc...).") adjust_grid_xrelsize;
%feature("docstring", "adjust_grid_xrelsizes([ChipsId,] ratios)\n\nModifies the widths of all columns (from left to right) to use the corresponding value in the ratios array. The array must be the same size as the number of columns in the grid.") adjust_grid_xrelsizes;
%feature("docstring", "adjust_grid_yrelsize([ChipsId,] row, ratio)\n\nModifies the height of all rows such that the height of the specified row (1-n, where 1=topmost) is determined by the specified ratio (.5=half height, 1=homogeneous height, etc...).") adjust_grid_yrelsize;
%feature("docstring", "adjust_grid_yrelsizes([ChipsId,] ratios)\n\nModifies the height of each row (from top to bottom) to use the corresponding value in the ratios array. The array must be the same size as the number of rows in the grid.") adjust_grid_yrelsizes;
%feature("docstring", "grid_objects([ChipsId,] columns, rows, xgap, ygap, [usage,] [plots])\n\nComputes the plot locations for a columns by rows grid with xgap spacing between columns and ygap spacing between rows. The usage parameter dictates behavior with regards to plots (0=use existing plots only, 1=use existing plots and add new ones as needed, 2=always create new plots,3=use only existing plots not listed, 4=exclude the listed plots but add new ones as necessary).") grid_objects;
%feature("docstring", "row_grid_objects([ChipsId,] columns, xgap, ygap, [usage,] [plots])\n\nUses an array of column counts to compute plot locations for a grid of plots with xgap spacing between columns and ygap spacing between rows. The usage parameter dictates behavior with regards to plots (0=use existing plots only, 1=use existing plots and add new ones as needed, 2=always create new plots,3=use only existing plots not listed, 4=exclude the listed plots but add new ones as necessary).") row_grid_objects;
%feature("docstring", "col_grid_objects([ChipsId,] rows, xgap, ygap, [usage,] [plots])\n\nUses an array of row counts to compute plot locations for a grid of plots with xgap spacing between columns and ygap spacing between rows. The usage parameter dictates behavior with regards to plots (0=use existing plots only, 1=use existing plots and add new ones as needed, 2=always create new plots,3=use only existing plots not listed, 4=exclude the listed plots but add new ones as necessary).") col_grid_objects;
%feature("docstring", "swap_object_positions(ChipsId, plot2)\n\nSwaps the grid positions of the plots specified by ChipsId.plot and plot2.") swap_object_positions;
%feature("docstring", "add_plot([ChipsId,] [x1, y1, x2, y2] [,attributes])\n\nAdds a plot to the current frame. If x1,y1,x2,y2 are provided, they are used to specify the location of the plot drawing area otherwise default values are used.") add_plot;
%feature("docstring", "get_curve_symbolstyle([ChipsId])\n\nReturns the symbol style of the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_symbolstyle;
%feature("docstring", "get_curve_symbolfill([ChipsId])\n\nReturns the symbol fill value of the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_symbolfill;
%feature("docstring", "get_curve_symbolsize([ChipsId])\n\nReturns the symbol size of the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_symbolsize;
%feature("docstring", "get_curve_symbolcolor([ChipsId])\n\nReturns the symbol color of the curve specified by ChipsId (or current curve if omitted).An error is returned if accessing more than one curve (ie. id='all').") get_curve_symbolcolor;
%feature("docstring", "get_curve_linestyle([ChipsId])\n\nReturns the line style of the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_linestyle;
%feature("docstring", "get_curve_linethickness([ChipsId])\n\nReturns the line thickness of the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_linethickness;
%feature("docstring", "get_curve_linecolor([ChipsId])\n\nReturns the line color of the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_linecolor;
%feature("docstring", "get_curve_errstyle([ChipsId])\n\nReturns the error bar style for the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errstyle;
%feature("docstring", "get_curve_errcolor([ChipsId])\n\nReturns the error bar color for the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errcolor;
%feature("docstring", "get_curve_errthickness([ChipsId])\n\nReturns the error bar thickness for the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errthickness;
%feature("docstring", "get_curve_errup([ChipsId])\n\nReturns whether displaying of up err bars is enabled for the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errup;
%feature("docstring", "get_curve_errdown([ChipsId])\n\nReturns whether displaying of down err bars is enabled for the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errdown;
%feature("docstring", "get_curve_errleft([ChipsId])\n\nReturns whether displaying of left err bars is enabled for the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errleft;
%feature("docstring", "get_curve_errright([ChipsId])\n\nReturns whether displaying of right err bars is enabled for the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errright;
%feature("docstring", "get_curve_errcaplength([ChipsId])\n\nReturns the error cap length of the curve specified by ChipsId (or current curve if omitted). An error is returned if accessing more than one curve (ie. id='all').") get_curve_errcaplength;
%feature("docstring", "get_histogram_errcaplength([ChipsId])\n\nReturns the error cap length of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_errcaplength;
%feature("docstring", "get_histogram_symbolstyle([ChipsId])\n\nReturns the symbol style of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_symbolstyle;
%feature("docstring", "get_histogram_symbolfill([ChipsId])\n\nReturns the symbol fill value of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_symbolfill;
%feature("docstring", "get_histogram_symbolsize([ChipsId])\n\nReturns the symbol size of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_symbolsize;
%feature("docstring", "get_histogram_symbolcolor([ChipsId])\n\nReturns the symbol color of the histogram specified by ChipsId (or current histogram if omitted).An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_symbolcolor;
%feature("docstring", "get_histogram_linestyle([ChipsId])\n\nReturns the line style of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_linestyle;
%feature("docstring", "get_histogram_fillstyle([ChipsId])\n\nReturns the fill style of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_fillstyle;
%feature("docstring", "get_histogram_linethickness([ChipsId])\n\nReturns the line thickness of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_linethickness;
%feature("docstring", "get_histogram_linecolor([ChipsId])\n\nReturns the line color of the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_linecolor;
%feature("docstring", "get_histogram_errstyle([ChipsId])\n\nReturns the error bar style for the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_errstyle;
%feature("docstring", "get_histogram_errcolor([ChipsId])\n\nReturns the error bar color for the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_errcolor;
%feature("docstring", "get_histogram_errthickness([ChipsId])\n\nReturns the error bar thickness for the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_errthickness;
%feature("docstring", "get_histogram_errup([ChipsId])\n\nReturns whether displaying of up err bars is enabled for the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_errup;
%feature("docstring", "get_histogram_errdown([ChipsId])\n\nReturns whether displaying of down err bars is enabled for the histogram specified by ChipsId (or current histogram if omitted). An error is returned if accessing more than one histogram (ie. id='all').") get_histogram_errdown;
%feature("docstring", "get_axis_color([ChipsId])\n\nReturns the color for the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_color;
%feature("docstring", "get_axis_major_color([ChipsId])\n\nReturns the color for the major ticks of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_major_color;
%feature("docstring", "get_axis_minor_color([ChipsId])\n\nReturns the color for the minor ticks of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minor_color;
%feature("docstring", "get_label_color([ChipsId])\n\nReturns the color for the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_color;
%feature("docstring", "get_line_color([ChipsId])\n\nReturns the color for the line specified by ChipsId (or current if omitted). An error is returned if accessing more than one line.") get_line_color;
%feature("docstring", "get_point_color([ChipsId])\n\nReturns the color for the point specified by ChipsId (or current if omitted). An error is returned if accessing more than one point.") get_point_color;
%feature("docstring", "get_region_edgecolor([ChipsId])\n\nReturns the color for the edge of the region specified by ChipsId (or current if omitted). An error is returned if accessing more than one region.") get_region_edgecolor;
%feature("docstring", "get_region_fillcolor([ChipsId])\n\nReturns the fill color for the region specified by ChipsId (or current if omitted). An error is returned if accessing more than one region.") get_region_fillcolor;
%feature("docstring", "get_region_fillstyle([ChipsId])\n\nReturns the fill style of the region specified by ChipsId (or current region if omitted). An error is returned if accessing more than one region (ie. id='all').") get_region_fillstyle;
%feature("docstring", "get_axis_label_color([ChipsId])\n\nReturns the color for the axis label specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_label_color;
%feature("docstring", "get_acis_ticklabel_color([ChipsId])\n\nReturns the color for the tick labels of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_ticklabel_color;
%feature("docstring", "get_axis_majorgrid_color([ChipsId])\n\nReturns the color of the majorgrids for the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_majorgrid_color;
%feature("docstring", "get_axis_minorgrid_color([ChipsId])\n\nReturns the color of the minorgrids for the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minorgrid_color;
%feature("docstring", "get_contour_color([ChipsId])\n\nReturns the color of the contour specified by ChipsId (or current if omitted). An error is returned if accessing more than one contour.") get_contour_color;
%feature("docstring", "set_axis_label_angle([ChipsId], angle)\n\nSets the angle of the axis label specified by ChipsId (or current if omitted) to the provided angle (0=horizontal).") set_axis_label_angle;
%feature("docstring", "get_axis_label_angle([ChipsId])\n\nReturns the angle for the label of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_label_angle;
%feature("docstring", "get_axis_ticklabel_angle([ChipsId])\n\nReturns the angle for the tick labels of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_ticklabel_angle;
%feature("docstring", "get_label_angle([ChipsId])\n\nReturns the angle of the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_angle;
%feature("docstring", "get_region_angle([ChipsId])\n\nReturns the angle used to orient the region specified by ChipsId (or current if omitted). An error is returned if accessing more than one region.") get_region_angle;
%feature("docstring", "get_axis_label_font([ChipsId])\n\nReturns the font for the label of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_label_font;
%feature("docstring", "get_axis_ticklabel_font([ChipsId])\n\nReturns the ticklabel font for the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_ticklabel_font;
%feature("docstring", "get_label_font([ChipsId])\n\nReturns the font for the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_font;
%feature("docstring", "get_axis_label_fontstyle([ChipsId])\n\nReturns the fontstyle for the label of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_label_fontstyle;
%feature("docstring", "get_axis_ticklabel_fontstyle([ChipsId])\n\nReturns the ticklabel fontstyle for the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_ticklabel_fontstyle;
%feature("docstring", "get_label_fontstyle([ChipsId])\n\nReturns the fontstyle for the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_fontstyle;
%feature("docstring", "get_plot_leftmargin([ChipsId])\n\nReturns the left margin size for the plot specified by ChipsId (or current if omitted).") get_plot_leftmargin;
%feature("docstring", "get_plot_rightmargin([ChipsId])\n\nReturns the right margin size for the plot specified by ChipsId (or current if omitted).") get_plot_rightmargin;
%feature("docstring", "get_plot_topmargin([ChipsId])\n\nReturns the top margin size for the plot specified by ChipsId (or current if omitted).") get_plot_topmargin;
%feature("docstring", "get_plot_bottommargin([ChipsId])\n\nReturns the bottom margin size for the plot specified by ChipsId (or current if omitted).") get_plot_bottommargin;
%feature("docstring", "get_axis_automax([ChipsId])\n\nReturns whether the auto limits functionality is enabled for the limits maximum of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_automax;
%feature("docstring", "get_axis_automin([ChipsId])\n\nReturns whether the auto limits functionality is enabled for the limits minimum of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_automin;
%feature("docstring", "get_axis_pad([ChipsId])\n\nReturns the pad factor used by arbitrary, interval, and limits modes when adjusting limits for the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_pad;
%feature("docstring", "display_axis([ChipsId])\n\nSets the visibility of the axis specified by ChipsId (or current if omitted) to on.") display_axis;
%feature("docstring", "hide_axis([ChipsId])\n\nSets the visibility of the axis specified by ChipsId (or current if omitted) to off.") hide_axis;
%feature("docstring", "display_major_ticks([ChipsId])\n\nSets the visibility of the major ticks of the axis specified by ChipsId (or current if omitted) to on.") display_major_ticks;
%feature("docstring", "hide_major_ticks([ChipsId])\n\nSets the visibility of the major ticks of the axis specified by ChipsId (or current if omitted) to off.") hide_major_ticks;
%feature("docstring", "display_minor_ticks([ChipsId])\n\nSets the visibility of the minor ticks of the axis specified by ChipsId (or current if omitted) to on.") display_minor_ticks;
%feature("docstring", "hide_minor_ticks([ChipsId])\n\nSets the visibility of the minor ticks of the axis specified by ChipsId (or current if omitted) to off.") hide_minor_ticks;
%feature("docstring", "get_axis_major_mode([ChipsId])\n\nReturns the mode used to display major tick marks on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_major_mode;
%feature("docstring", "get_axis_minor_mode([ChipsId])\n\nReturns the mode used to display minor tick marks on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minor_mode;
%feature("docstring", "get_axis_major_thickness([ChipsId])\n\nReturns the thickness of the major tick marks on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_major_thickness;
%feature("docstring", "get_axis_minor_thickness([ChipsId])\n\nReturns the thickness of the minor tick marks on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minor_thickness;
%feature("docstring", "get_axis_major_style([ChipsId])\n\nReturns the style of the major tick marks on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_major_style;
%feature("docstring", "get_axis_minor_style([ChipsId])\n\nReturns the style of the minor tick marks on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minor_style;
%feature("docstring", "get_axis_majorgrid_thickness([ChipsId])\n\nReturns the thickness of the major grids on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_majorgrid_thickness;
%feature("docstring", "get_axis_minorgrid_thickness([ChipsId])\n\nReturns the thickness of the minor grids on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minorgrid_thickness;
%feature("docstring", "get_axis_majorgrid_style([ChipsId])\n\nReturns the style of the major grids on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_majorgrid_style;
%feature("docstring", "get_axis_minorgrid_style([ChipsId])\n\nReturns the style of the minor grids on the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minorgrid_style;
%feature("docstring", "get_axis_thickness([ChipsId])\n\nReturns the thickness of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_thickness;
%feature("docstring", "get_axis_tickstyle([ChipsId])\n\nReturns the tickstyle of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_tickstyle;
%feature("docstring", "set_axis_tickformat([ChipsId], format)\n\nSets the tick format of the axis specified by ChipsId (or current if omitted) to the specified print format (ie. '%3.2f', '%g', '%z', etc...).") set_axis_tickformat;
%feature("docstring", "get_axis_tickformat([ChipsId])\n\nReturns the tick format of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_tickformat;
%feature("docstring", "set_axis_text([ChipsId], value)\n\nSets the label of the axis specified by ChipsId (or current if omitted) to the provided value.") set_axis_text;
%feature("docstring", "get_axis_text([ChipsId])\n\nReturns the label of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_text;
%feature("docstring", "set_axis_majorlength([ChipsId], value)\n\nSets the major tick length of the axis specified by ChipsId (or current if omitted) to the provided value.") set_axis_majorlength;
%feature("docstring", "get_axis_majorlength([ChipsId])\n\nReturns the major tick length of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_majorlength;
%feature("docstring", "set_axis_minorlength([ChipsId], value)\n\nSets the minor tick length of the axis specified by ChipsId (or current if omitted) to the provided value.") set_axis_minorlength;
%feature("docstring", "get_axis_minorlength([ChipsId])\n\nReturns the minor tick length of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_minorlength;
%feature("docstring", "set_axis_minor_num([ChipsId], value)\n\nSets the number of minor ticks to display between majorticks when using 'count' mode") set_axis_minor_num;
%feature("docstring", "get_axis_minor_num([ChipsId])\n\nReturns the number of minor ticks displayed between major ticks when using 'count' mode. An error is returned if accessing more than one axis.") get_axis_minor_num;
%feature("docstring", "set_axis_minor_interval([ChipsId], value)\n\nSets the spacing interval for minor ticks when using 'interval' mode") set_axis_minor_interval;
%feature("docstring", "get_axis_minor_interval([ChipsId])\n\nReturns the interval between minor ticks when using 'interval' mode. An error is returned if accessing more than one axis.") get_axis_minor_interval;

%feature("docstring", "set_axis_major_num([ChipsId], value)\n\nSets the number of major ticks to display on the specified axis (or current if omitted) when using 'count' mode.") set_axis_major_num;
%feature("docstring", "get_axis_major_num([ChipsId])\n\nReturns the number of major ticks displayed on the specified axis (or current if omitted) when using 'count' mode. An error is returned if accessing more than one axis.") get_axis_major_num;
%feature("docstring", "set_axis_major_interval([ChipsId], value)\n\nSets the spacing interval of major ticks on the specified axis (or current if omitted) when using 'interval' mode.") set_axis_major_interval;
%feature("docstring", "get_axis_major_interval([ChipsId])\n\nReturns the interval between major ticks when using 'interval' mode. An error is returned if accessing more than one axis.") get_axis_major_interval;

%feature("docstring", "get_line_thickness([ChipsId])\n\nReturns the thickness of the line specified by ChipsId (or current if omitted). An error is returned if accessing more than one line.") get_line_thickness;
%feature("docstring", "get_region_edgethickness([ChipsId])\n\nReturns the thickness of the region edge specified by ChipsId (or current if omitted). An error is returned if accessing more than one region.") get_region_edgethickness;
%feature("docstring", "get_contour_thickness([ChipsId])\n\nReturns the thickness of the contour lines specified by ChipsId (or current if omitted). An error is returned if accessing more than one contour.") get_contour_thickness;
%feature("docstring", "get_label_text([ChipsId])\n\nReturns the text of the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_text;
%feature("docstring", "set_label_text([ChipsId], value)\n\nSets the text of the label specified by ChipsId (or current if omitted) to the provided value.") set_label_text;
%feature("docstring", "get_label_size([ChipsId])\n\nReturns the font size of the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_size;
%feature("docstring", "set_label_halign([ChipsId], value)\n\nSets the horizontal alignment of the label specified by ChipsId (or current if omitted) to the provided value (0.0 = left justified, .5 = centered, 1.0 = right justified).") set_label_halign;
%feature("docstring", "get_label_halign([ChipsId])\n\nReturns the numeric horizontal alignment value of the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_halign;
%feature("docstring", "get_label_valign([ChipsId])\n\nReturns the numeric vertical alignment value of the label specified by ChipsId (or current if omitted). An error is returned if accessing more than one label.") get_label_valign;
%feature("docstring", "set_label_font([ChipsId], value)\n\nSets the font of the label specified by ChipsId (or current if omitted) to the provided value.") set_label_font;
%feature("docstring", "set_label_angle([ChipsId], angle)\n\nSets the angle of the label specified by ChipsId (or current if omitted) to the provided angle (0=horizontal).") set_label_angle;
%feature("docstring", "get_point_size([ChipsId])\n\nReturns the size of the point specified by ChipsId (or current if omitted). An error is returned if accessing more than one point.") get_point_size;
%feature("docstring", "set_point_angle([ChipsId], angle)\n\nSets the angle of the point symbol specified by ChipsId (or current if omitted) to the provided angle.") set_point_angle;
%feature("docstring", "add_curve([ChipsId,] xdata, ydata, [attribute_list])\nadd_curve([ChipsId,] xdata, ydata, yerrs [,attribute_list])\nadd_curve([ChipsId,] xdata, ydata, yup, ydown [,yleft [,yright]] [,attribute_list])\n\nAdds a curve using the provided data, creating window, frame, or plot as needed.") add_curve;
%feature("docstring", "add_histogram([ChipsId,] counts [,attribute_list])\nadd_histogram([ChipsId,] bins, counts [,attribute_list])\nadd_histogram([ChipsId,] bin_lo, bin_hi, counts, [,yerrs] [,attribute_list])\nadd_histogram([ChipsId,] bin_lo, bin_hi, ydown, yup, counts [,attribute_list])\n\nAdds a histogram using the provided data, creating window, frame, or plot as needed.") add_histogram;
%feature("docstring", "add_contour([ChipsId,] image, x-dim, y-dim [,levels] [,transform] [,attribute_list])\nadd_contour([ChipsId,] x-indices, y-indices, image [,attribute_list])\n\nAdds a contour from the image provided. Only regularly gridded images are supported.") add_contour;
%feature("docstring", "add_axis([ChipsId,] dimension, position, min, max [,attribute_list])\n\nAdds an axis at the specified position in the given dimension (X_AXIS|Y_AXIS|XY_AXIS) with a range from min to max. The call creates a window, frame, or plot as needed.") add_axis;
%feature("docstring", "add_label([ChipsId,] xpos, ypos, text [,attribute_list])\n\nAdds a label at the specified (xpos, ypos) containing the provided text (partial latex formatting supported). The call creates a window or frame as needed.") add_label;
%feature("docstring", "add_line([ChipsId,] x1, y1, x2, y2 [,attribute_list])\nadd_line([ChipsId,] xlist, ylist [,attribute_list])\n\nAdds a line from (x1,y1) to (x2,y2) or a poly-line using xlist, ylist pairs. The call creates a window or frame as needed.") add_line;
%feature("docstring", "add_point([ChipsId,] xpos, ypos, [,attribute_list])\nAdds a point at the specified (xpos, ypos). The call creates a window or frame as needed.") add_point;
%feature("docstring", "add_region([ChipsId,] sides, xpos, ypos, radius [,attribute_list])\nadd_region([ChipsId,] xlist, ylist [,attribute_list])\n\nAdds a region as a regular polygon with the specified radius and number of sides at (xpos, ypos) or an irregular polygon generated from using xlist, ylist pairs. The call creates a window or frame as needed. Degenerate (self-intersecting) polygons are not supported.") add_region;
%feature("docstring", "set_point_fill([ChipsId,], fill)\n\nSets the symbol fill of the point specified by ChipsId (or current point if omitted) to the value specified by fill") set_point_fill;
%feature("docstring", "get_point_fill([ChipsId,])\n\nReturns whether the symbol of the point specified by ChipsId (or current point if omitted) is filled or hollow. An error is returned if accessing more than one point.") get_point_fill;
%feature("docstring", "set_region_opacity([ChipsId,], opacity)\n\nSets the opcaity level of the region specified by ChipsId (or current region if omitted). An opacity value of 0 is transparent, a value of 1 is translucent.") set_region_opacity;
%feature("docstring", "get_region_opacity([ChipsId,])\n\nReturns the opacity level of the region specified by ChipsId (or current region if omitted). An error is returned if accessing more than one region.") get_region_opacity;
%feature("docstring", "info_current()\n\nReturns a string containing a listing of what objects are 'current'.") info_current;
%feature("docstring", "info([ChipsId])\n\nReturns a string containing a listing of objects within the context specified by ChipsId (or all objects if omitted).") info;
%feature("docstring", "add_image([ChipsId], filename [, attribute_list])\n\nAdds the specified png image to the plot.") add_image;
%feature("docstring", "info_bound_axes([ChipsId])\n\nReturns a string containing a listing of bound axes specified by plot_id:axis_id.") info_bound_axes;
%feature("docstring", "info_coordinate([ChipsId])\n\nReturns a string containing a listing of available coordinate systems within the context specified by ChipsId (or current if omitted).") info_coordinate;
%feature("docstring", "set_axis_ticklabel_halign([ChipsId], value)\n\nSets the horizontal alignment of the ticklabels of the axis specified by ChipsId (or current if omitted) to the provided value (0.0 = left justified, .5 = centered, 1.0 = right justified).") set_axis_ticklabel_halign;
%feature("docstring", "get_axis_ticklabel_halign([ChipsId])\n\nReturns the numeric horizontal alignment value of the ticklabels of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_ticklabel_halign;
%feature("docstring", "set_axis_ticklabel_valign([ChipsId], value)\n\nSets the vertical alignment of the ticklabels of the axis specified by ChipsId (or current if omitted) to the provided value (0.0 = bottom justified, .5 = centered, 1.0 = top justified).") set_axis_ticklabel_valign;
%feature("docstring", "get_axis_ticklabel_valign([ChipsId])\n\nReturns the numeric vertical alignment value of the ticklabels of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_ticklabel_valign;
%feature("docstring", "set_axis_label_halign([ChipsId], value)\n\nSets the horizontal alignment of the label of the axis specified by ChipsId (or current if omitted) to the provided value (0.0 = left justified, .5 = centered, 1.0 = right justified).") set_axis_label_halign;
%feature("docstring", "get_axis_label_halign([ChipsId])\n\nReturns the numeric horizontal alignment value of the label of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_label_halign;
%feature("docstring", "set_axis_label_valign([ChipsId], value)\n\nSets the vertical alignment of the label of the axis specified by ChipsId (or current if omitted) to the provided value (0.0 = bottom justified, .5 = centered, 1.0 = top justified).") set_axis_label_valign;
%feature("docstring", "get_axis_label_valign([ChipsId])\n\nReturns the numeric vertical alignment value of the label of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_label_valign;
%feature("docstring", "get_axis_parallel_offset([ChipsId])\n\nReturns the parallel offset value of the label of the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_parallel_offset;
%feature("docstring", "get_axis_perpendicular_offset([ChipsId])\n\nReturns the perpendicular offset value (distance between axis and label in pixels) for the axis specified by ChipsId (or current if omitted). An error is returned if accessing more than one axis.") get_axis_perpendicular_offset;
%feature("docstring", "set_axis_perpendicular_offset([ChipsId], pixels)\n\nSets the distance between the axis and its label to the specified number of pixels. Values less than 0 are 'inside' the axis while values greater than 0 are outside.") set_axis_perpendicular_offset;
%feature("docstring", "set_axis_parallel_offset([ChipsId], pixels)\n\nOffsets the location of the axis label. Accepted values are (-.4:.4). A value of 0 is centered on the axis.") set_axis_parallel_offset;
%feature("docstring", "get_curve_baddatamode([ChipsId])\n\nReturns the bad data handling mode for the curve specified by ChipsId (or current curve if omitted). A value of 0 indicates that the bad data point is ignored, while a value of 1 indicates the value is omitted (gapped). An error is returned if accessing more than one curve (ie. id='all').") get_curve_baddatamode;
%feature("docstring", "set_curve_baddatamode([ChipsId], mode)\n\nSets the bad data handling mode for the curve specified by ChipsId (or current curve if omitted) to the specified value. A value of chips_ignore (0) indicates that the bad data point is ignored, while a value of chips_omit (1) indicates a gap should exist where the bad data was encountered.") set_curve_baddatamode;
%feature("docstring", "set_contour_algorithm([ChipsId,], alg)\n\nSets the contouring algorithm of the contour specified by ChipsId (or current contour if omitted) to the value specified by alg (chips_standard|chips_marching)") set_contour_algorithm;
%feature("docstring", "get_contour_algorithm([ChipsId,])\n\nReturns the contouring algorithm used by the contour specified by ChipsId (or current contour if omitted). The numeric value of the algorithm translates as follows: 0 = vtk's internal default algorithm, 1 = marching algorithm.") get_contour_algorithm;

%feature("docstring", "display_depth([ChipsId,] depth)\n\nCauses the specified depth to be visible in the context specified by ChipsId (or current contour if omitted). Items at the specified depth with their visibility set true are displayed.") display_depth;
%feature("docstring", "hide_depth([ChipsId,] depth)\n\nCauses the specified depth to be hidden in the context specified by ChipsId (or current contour if omitted). Any items at the specified depth are not displayed regardless of their individual visibility settings.") hide_depth;
%feature("docstring", "collapse_depths([ChipsId,] depth_list, new_depth)\n\nCauses the objects contained in the depths_list array to be moved into the new_depth in the order they are listed in depth_list.") collapse_depths;
%feature("docstring", "shuffle([ChipsId,] object_type, direction)\n\nAdjusts the position of the current (or ChipsId specified) object such that it's rendering order is within its depth is reordered. The direction parameter specifies whether the current object should be moved forward, backward, to the front, or to the back (chips_forward|chips_backward|chips_front|chips_back) of the depth.") shuffle_forward;
%feature("docstring", "shuffle_forward([ChipsId,] object_type)\n\nAdjusts the position of the current (or ChipsId specified) object such that it's rendering order is one position closer to the front of the display.") shuffle_forward;
%feature("docstring", "shuffle_backward([ChipsId,] object_type)\n\nAdjusts the position of the current (or ChipsId specified) object such that it's rendering order is one position closer to the back of the display.") shuffle_backward;
%feature("docstring", "shuffle_front([ChipsId,] object_type)\n\nAdjusts the position of the current (or ChipsId specified) object such that it's rendering order is at the front of the display.") shuffle_front;
%feature("docstring", "shuffle_back([ChipsId,] object_type)\n\nAdjusts the position of the current (or ChipsId specified) object such that it's rendering order is at the back of the display.") shuffle_back;
%feature("docstring", "set_default_depth([ChipsId,] depth)\n\nAdjusts the 'current' depth value to use for any object created with a depth specified as 'default' (V_USE_DEFAULT_DEPTH). Changes to the default depth only impact subsequently created objects.") set_default_depth;
%feature("docstring", "get_default_depth([ChipsId,])\n\nReturns the value that is utilized for the current 'default' depth.") get_default_depth;
%feature("docstring", "info_depth([ChipsId], [depth])\n\nReturns a string containing a listing of objects at the specified depth (or all depths if omitted) within the context specified by ChipsId. If all depths are listed, the result will contain all accessed depths whether they contain objects or not.") info_depth;
%feature("docstring", "set_axis_depth([ChipsId,] depth)\n\nSets the depth of the specified axis (or current if ChipsId is omitted) to the provided depth.") set_axis_depth;
%feature("docstring", "get_axis_depth([ChipsId,])\n\nReturns the depth of the specified axis (or current if ChipsId is omitted).") get_axis_depth;
%feature("docstring", "set_contour_depth([ChipsId,] depth)\n\nSets the depth of the specified contour (or current if ChipsId is omitted) to the provided depth.") set_contour_depth;
%feature("docstring", "get_contour_depth([ChipsId,])\n\nReturns the depth of the specified contour (or current if ChipsId is omitted).") get_contour_depth;
%feature("docstring", "set_plot_title_depth([ChipsId,] depth)\n\nSets the depth of the title for the specified plot (or current if ChipsId is omitted) to the provided depth.") set_plot_title_depth;
%feature("docstring", "get_plot_title_depth([ChipsId,])\n\nReturns the depth of the title for the specified plot (or current if ChipsId is omitted).") get_plot_title_depth;
%feature("docstring", "set_curve_depth([ChipsId,] depth)\n\nSets the depth of the specified curve (or current if ChipsId is omitted) to the provided depth.") set_curve_depth;
%feature("docstring", "get_curve_depth([ChipsId,])\n\nReturns the depth of the specified curve (or current if ChipsId is omitted).") get_curve_depth;
%feature("docstring", "set_histogram_depth([ChipsId,] depth)\n\nSets the depth of the specified histogram (or current if ChipsId is omitted) to the provided depth.") set_histogram_depth;
%feature("docstring", "get_histogram_depth([ChipsId,])\n\nReturns the depth of the specified histogram (or current if ChipsId is omitted).") get_histogram_depth;
%feature("docstring", "set_label_depth([ChipsId,] depth)\n\nSets the depth of the specified label (or current if ChipsId is omitted) to the provided depth.") set_label_depth;
%feature("docstring", "get_label_depth([ChipsId,])\n\nReturns the depth of the specified label (or current if ChipsId is omitted).") get_label_depth;

%feature("docstring", "set_line_depth([ChipsId,] depth)\n\nSets the depth of the specified line (or current if ChipsId is omitted) to the provided depth.") set_line_depth;
%feature("docstring", "get_line_depth([ChipsId,])\n\nReturns the depth of the specified line (or current if ChipsId is omitted).") get_line_depth;
%feature("docstring", "set_point_depth([ChipsId,] depth)\n\nSets the depth of the specified point (or current if ChipsId is omitted) to the provided depth.") set_point_depth;
%feature("docstring", "get_point_depth([ChipsId,])\n\nReturns the depth of the specified point (or current if ChipsId is omitted).") get_point_depth;
%feature("docstring", "set_region_depth([ChipsId,] depth)\n\nSets the depth of the specified region (or current if ChipsId is omitted) to the provided depth.") set_region_depth;
%feature("docstring", "get_region_depth([ChipsId,])\n\nReturns the depth of the specified region (or current if ChipsId is omitted).") get_region_depth;
%feature("docstring", "set_controur_transform([ChipsId,] Transform)\n\nSet the transform of the specified contour (or current if ChipsId is omitted). The transform will automatically be applied to the data upon setting it.") set_contour_transform;
%feature("docstring", "apply_controur_transform([ChipsId])\n\nApply the specified contour's (or current if ChipsId is omitted) transform to the contour's data.") apply_contour_transform;
%feature("docstring", "apply_controur_transform_inverse([ChipsId])\n\nApply the specified contour's (or current if ChipsId is omitted) transform inverse to the contour's data.") apply_contour_transform_inverse;
%feature("docstring", "get_controur_transform([ChipsId])\n\nReturns the transform of the specified contour (or current if ChipsId is omitted).") get_contour_transform;
%feature("docstring", "display_curve([ChipsId])\n\nSets the visibility of the curve specified by ChipsId (or current if omitted) to on.") display_curve;
%feature("docstring", "hide_curve([ChipsId])\n\nSets the visibility of the curve specified by ChipsId (or current if omitted) to off.") hide_curve;
%feature("docstring", "get_curve_visible([ChipsId])\n\nReturns whether the curve specified by ChipsId (or current if omitted) is displayed or hidden") get_curve_visible;

%feature("docstring", "display_image([ChipsId])\n\nSets the visibility of the image specified by ChipsId (or current if omitted) to on.") display_image;
%feature("docstring", "hide_image([ChipsId])\n\nSets the visibility of the image specified by ChipsId (or current if omitted) to off.") hide_image;
%feature("docstring", "get_image_visible([ChipsId])\n\nReturns whether the image specified by ChipsId (or current if omitted) is displayed or hidden") get_image_visible;
%feature("docstring", "get_data_aspect_ratio([ChipsId])\n\nReturns the data aspect ratio for the data coordinate system specified by ChipsId (or current if omitted). If the ratio is not set an empty string is returned.") get_data_aspect_ratio;
%feature("docstring", "get_plot_aspect_ratio([ChipsId])\n\nReturns the aspect ratio for the plot specified by ChipsId (or current if omitted). If the ratio is not set an empty string is returned.") get_plot_aspect_ratio;
%feature("docstring", "get_plot_aspect_width([ChipsId])\n\nReturns the horizontal component of the aspect ratio for the plot specified by ChipsId (or current if omitted). If the ratio is not set, the value 0.0 is returned.") get_plot_aspect_width;
%feature("docstring", "get_plot_aspect_height([ChipsId])\n\nReturns the vertical component of the aspect ratio for the plot specified by ChipsId (or current if omitted). If the ratio is not set, the value 0.0 is returned.") get_plot_aspect_height;
%feature("docstring", "set_data_aspect_ratio([ChipsId],ratio)\n\nSets the aspect ratio of the data coordinate system specified by ChipsId (or current if omitted) to the specified string ratio 'a:b'. If the ratio string is blank or omitted, the data aspect ratio is disabled.") set_data_aspect_ratio;
%feature("docstring", "set_plot_aspect_ratio([ChipsId],ratio)\n\nSets the aspect ratio of the plot specified by ChipsId (or current if omitted) to the specified string ratio 'a:b', 'current', or 'fit'. If the ratio string is blank or omitted, the plot aspect ratio is disabled.") set_plot_aspect_ratio;

%feature("docstring", "display_contour([ChipsId])\n\nSets the visibility of the contour specified by ChipsId (or current if omitted) to on.") display_contour;
%feature("docstring", "hide_contour([ChipsId])\n\nSets the visibility of the contour specified by ChipsId (or current if omitted) to off.") hide_contour;
%feature("docstring", "get_contour_visible([ChipsId])\n\nReturns whether the contour specified by ChipsId (or current if omitted) is displayed or hidden") get_contour_visible;
%feature("docstring", "display_histogram([ChipsId])\n\nSets the visibility of the histogram specified by ChipsId (or current if omitted) to on.") display_histogram;
%feature("docstring", "hide_histogram([ChipsId])\n\nSets the visibility of the histogram specified by ChipsId (or current if omitted) to off.") hide_histogram;
%feature("docstring", "get_histogram_visible([ChipsId])\n\nReturns whether the histogram specified by ChipsId (or current if omitted) is displayed or hidden") get_histogram_visible;
%feature("docstring", "set_image_colormap([ChipsId,], map)\n\nSets the colormap of the image specified by ChipsId (or current image if omitted) to the value specified by map") set_image_colormap;
%feature("docstring", "get_image_colormap([ChipsId,])\n\nReturns the enumerated value of the colormap of the image specified by ChipsId (or current image if omitted)") get_image_colormap;
%feature("docstring", "set_image_colormap_size([ChipsId,], numelems)\n\nSpecifies the size of the colormap used by the image specified by ChipsId (or current image if omitted)") set_image_colormap_size;
%feature("docstring", "get_image_colormap_size([ChipsId,])\n\nReturns the number of elements in the colormap of the image specified by ChipsId (or current image if omitted)") get_image_colormap_size;
%feature("docstring", "set_image_invert_colormap([ChipsId,], inv)\n\nSpecifies whether the colormap of the image specified by ChipsId (or current image if omitted) should be inverted") set_image_invert_colormap;
%feature("docstring", "get_image_invert_colormap([ChipsId,])\n\nSpecifies whether the colormap of the image specified by ChipsId (or current image if omitted) is inverted") get_image_invert_colormap;
%feature("docstring", "set_image_interpolation([ChipsId,], interp)\n\nSets the interpolation method of the image specified by ChipsId (or current image if omitted) to the value specified by interp") set_image_interpolation;
%feature("docstring", "set_image_colormap_interpolate([ChipsId,], bool)\n\nSpecifies if the colormap should interpolate over the range of values") set_image_colormap_interpolate;
%feature("docstring", "get_image_colormap_interpolate([ChipsId])\n\nReturns whether or not the colormap is interpolating over the range of values") get_image_colormap_interpolate;
%feature("docstring", "get_image_interpolation([ChipsId,])\n\nReturns the interpolation method of the image specified by ChipsId (or current image if omitted)") get_image_interpolation;
%feature("docstring", "set_image_alpha([ChipsId,], min, max)\n\nSets the minimum and maximum bounds for the alpha channel of the image specified by ChipsId (or current image if omitted) to the range defined by min and max") set_image_alpha;
%feature("docstring", "set_image_channel([ChipsId,], vals, xdim, yim, channel)\n\nUses the data specified by values to represent the specified channel of the image specified by ChipsId (or current image if omitted)") set_image_channel;
%feature("docstring", "remove_image_channel([ChipsId,], channel)\n\nRemoves the specified channel from the image specified by ChipsId (or current image if omitted)") remove_image_channel;
%feature("docstring", "set_image_threshold([ChipsId,], min, max)\n\nSets the minimum and maximum thresholding bounds for the image specified by ChipsId (or current image if omitted)") set_image_threshold;
%feature("docstring", "get_image_alpha([ChipsId])\n\nReturns the range used for the alpha channel of the current image") get_image_alpha;
%feature("docstring", "get_image_threshold([ChipsId])\n\nReturns the thresholding range used for the current image") get_image_threshold;
%feature("docstring", "set_image_depth([ChipsId,] depth)\n\nSets the depth of the specified image (or current if ChipsId is omitted) to the provided depth.") set_image_depth;
%feature("docstring", "get_image_depth([ChipsId])\n\nReturns the depth of the specified image (or current if ChipsId is omitted).") get_image_depth;
%feature("docstring", "set_colorbar_depth([ChipsId,] depth)\n\nSets the depth of the specified colorbar (or current if ChipsId is omitted) to the provided depth.") set_colorbar_depth;
%feature("docstring", "get_colorbar_depth([ChipsId])\n\nReturns the depth of the specified colorbar (or current if ChipsId is omitted).") get_colorbar_depth;
%feature("docstring", "display_colorbar([ChipsId])\n\nSets the visibility of the colorbar specified by ChipsId (or current if omitted) to on.") display_colorbar;
%feature("docstring", "hide_colorbar([ChipsId])\n\nSets the visibility of the colorbar specified by ChipsId (or current if omitted) to off.") hide_colorbar;
%feature("docstring", "get_colorbar_visible([ChipsId])\n\nReturns whether the colorbar specified by ChipsId (or current if omitted) is displayed or hidden") get_colorbar_visible;

%feature("docstring", "set_colorbar_tick_visible([ChipsId], value)\n\nSets the visibility of the ticks of the colorbar specified by ChipsId (or current if omitted).") set_colorbar_tick_visible;
%feature("docstring", "get_colorbar_tick_visible([ChipsId])\n\nReturns whether the ticks of the colorbar specified by ChipsId (or current if omitted) are displayed or hidden") get_colorbar_tick_visible;
%feature("docstring", "set_colorbar_label_visible([ChipsId], value)\n\nSets the visibility of the label of the colorbar specified by ChipsId (or current if omitted).") set_colorbar_label_visible;
%feature("docstring", "get_colorbar_label_visible([ChipsId])\n\nReturns whether the label of the colorbar specified by ChipsId (or current if omitted) is displayed or hidden") get_colorbar_label_visible;
%feature("docstring", "set_colorbar_ticklabel_visible([ChipsId], value)\n\nSets the visibility of the tick labels of the colorbar specified by ChipsId (or current if omitted).") set_colorbar_ticklabel_visible;
%feature("docstring", "get_colorbar_ticklabel_visible([ChipsId])\n\nReturns whether the tick labels of the colorbar specified by ChipsId (or current if omitted) are displayed or hidden") get_colorbar_ticklabel_visible;
%feature("docstring", "get_colorbar_border_visible([ChipsId])\n\nReturns whether the border of the colorbar specified by ChipsId (or current if omitted) is displayed or hidden") get_colorbar_border_visible;
%feature("docstring", "set_colorbar_ticklabel_halign([ChipsId], value)\n\nSets the horizontal alignment of the ticklabel of the colorbar specified by ChipsId (or current if omitted) to the provided value (0.0 = left justified, .5 = centered, 1.0 = right justified).") set_colorbar_ticklabel_halign;
%feature("docstring", "get_colorbar_ticklabel_halign([ChipsId])\n\nReturns the numeric horizontal alignment value of the label of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_ticklabel_halign;
%feature("docstring", "set_colorbar_ticklabel_valign([ChipsId], value)\n\nSets the vertical alignment of the ticklabel of the colorbar specified by ChipsId (or current if omitted) to the provided value (0.0 = left justified, .5 = centered, 1.0 = right justified).") set_colorbar_ticklabel_valign;
%feature("docstring", "get_colorbar_ticklabel_valign([ChipsId])\n\nReturns the numeric vertical alignment value of the label of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_ticklabel_valign;
%feature("docstring", "set_colorbar_label_halign([ChipsId], value)\n\nSets the horizontal alignment of the label of the colorbar specified by ChipsId (or current if omitted) to the provided value (0.0 = left justified, .5 = centered, 1.0 = right justified).") set_colorbar_label_halign;
%feature("docstring", "get_colorbar_label_halign([ChipsId])\n\nReturns the numeric horizontal alignment value of the label of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_halign;
%feature("docstring", "set_colorbar_label_valign([ChipsId], value)\n\nSets the vertical alignment of the label of the colorbar specified by ChipsId (or current if omitted) to the provided value (0.0 = bottom justified, .5 = centered, 1.0 = top justified).") set_colorbar_label_valign;
%feature("docstring", "get_colorbar_label_valign([ChipsId])\n\nReturns the numeric vertical alignment value of the label of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_valign;
%feature("docstring", "set_colorbar_halign([ChipsId], value)\n\nSets the horizontal alignment of the colorbar specified by ChipsId (or current if omitted) to the provided value (0.0 = left justified, .5 = centered, 1.0 = right justified).") set_colorbar_halign;
%feature("docstring", "get_colorbar_halign([ChipsId])\n\nReturns the numeric horizontal alignment value of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_halign;
%feature("docstring", "set_colorbar_valign([ChipsId], value)\n\nSets the vertical alignment of the colorbar specified by ChipsId (or current if omitted) to the provided value (0.0 = bottom justified, .5 = centered, 1.0 = top justified).") set_colorbar_valign;
%feature("docstring", "get_colorbar_valign([ChipsId])\n\nReturns the numeric vertical alignment value of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_valign;
%feature("docstring", "set_colorbar_label_angle([ChipsId], angle)\n\nSets the angle of the label of the colorbar specified by ChipsId (or current if omitted) to the provided angle (0=horizontal).") set_colorbar_label_angle;
%feature("docstring", "get_colorbar_label_angle([ChipsId])\n\nReturns the angle for the label of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_angle;
%feature("docstring", "set_colorbar_ticklabel_angle([ChipsId], angle)\n\nSets the angle of the ticklabels of the colorbar specified by ChipsId (or current if omitted) to the provided angle (0=horizontal).") set_colorbar_ticklabel_angle;
%feature("docstring", "get_colorbar_ticklabel_angle([ChipsId])\n\nReturns the angle for the tick labels of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_ticklabel_angle;
%feature("docstring", "set_colorbar_label_color([ChipsId,], color)\n\nSets the label color of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by color") set_colorbar_label_color;
%feature("docstring", "get_colorbar_label_color([ChipsId])\n\nReturns the color of the label of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_color;
%feature("docstring", "set_colorbar_ticklabel_color([ChipsId,], color)\n\nSets the ticklabel color of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by color") set_colorbar_ticklabel_color;
%feature("docstring", "get_colorbar_ticklabel_color([ChipsId])\n\nReturns the color of the ticklabels of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_ticklabel_color;
%feature("docstring", "set_colorbar_tick_color([ChipsId,], color)\n\nSets the tick color of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by color") set_colorbar_tick_color;
%feature("docstring", "get_colorbar_tick_color([ChipsId])\n\nReturns the color of the ticks of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_tick_color;
%feature("docstring", "set_colorbar_label_font([ChipsId,], font)\n\nSets the font of the colorbar label specified by ChipsId (or current colorbar if omitted) to the provided font") set_colorbar_label_font;
%feature("docstring", "set_colorbar_label_fontstyle([ChipsId,], style)\n\nSets the label fontstyle ('bold'|'bolditalic'|'italic'|'normal') of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by style") set_colorbar_label_fontstyle;
%feature("docstring", "set_colorbar_ticklabel_font([ChipsId,], font)\n\nSets the font of ticklabel of the colorbar specified by ChipsId (or current colorbar if omitted) to the provided font") set_colorbar_ticklabel_font;
%feature("docstring", "set_colorbar_ticklabel_fontstyle([ChipsId,], style)\n\nSets the ticklabel fontstyle ('bold'|'bolditalic'|'italic'|'normal') of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by style") set_colorbar_ticklabel_fontstyle;
%feature("docstring", "get_colorbar_label_font([ChipsId])\n\nReturns the label font for the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_font;
%feature("docstring", "get_colorbar_label_fontstyle([ChipsId])\n\nReturns the label fontstyle for the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_fontstyle;
%feature("docstring", "get_colorbar_ticklabel_font([ChipsId])\n\nReturns the ticklabel font for the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_ticklabel_font;
%feature("docstring", "get_colorbar_ticklabel_fontstyle([ChipsId])\n\nReturns the ticklabel fontstyle for the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_ticklabel_fontstyle;
%feature("docstring", "set_colorbar_ticklabel_size([ChipsId,], size)\n\nSets the ticklabel size of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by size") set_colorbar_ticklabel_size;
%feature("docstring", "get_colorbar_ticklabel_size([ChipsId])\n\nReturns the ticklabel size of the colorbar specified by ChipsId (or current colorbar if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_ticklabel_size;
%feature("docstring", "set_colorbar_label_size([ChipsId,], size)\n\nSets the label size of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by size") set_colorbar_label_size;
%feature("docstring", "get_colorbar_label_size([ChipsId])\n\nReturns the label size of the colorbar specified by ChipsId (or current colorbar if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_size;
%feature("docstring", "get_colorbar_label_text([ChipsId])\n\nReturns the text of the label for the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_text;
%feature("docstring", "set_colorbar_label_text([ChipsId], value)\n\nSets the text of the label of the colorbar specified by ChipsId (or current if omitted) to the provided value.") set_colorbar_label_text;
%feature("docstring", "get_colorbar_label_location([ChipsId])\n\nReturns the location specification (inside|outside) of the label for the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_label_location;
%feature("docstring", "set_colorbar_label_location([ChipsId], value)\n\nSets the location (inside|outside) of the label of the colorbar specified by ChipsId (or current if omitted) to the provided value.") set_colorbar_label_location;
%feature("docstring", "set_colorbar_length([ChipsId], value)\n\nSets the length of the colorbar specified by ChipsId (or current if omitted) to the provided value. The value is percentage of the plot size (.5 = half of corresponding plot dimension, 1 equals size of corresponding plot dimension") set_colorbar_length;
%feature("docstring", "get_colorbar_length([ChipsId])\n\nReturns the length of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_length;
%feature("docstring", "get_colorbar_width([ChipsId])\n\nReturns the width (as a percentage of the plot dimension) of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_width;
%feature("docstring", "set_colorbar_width([ChipsId], value)\n\nSets the width of the colorbar specified by ChipsId (or current if omitted) to the provided value. The value is a percentage of the plot size (.1 = a tenth of corresponding plot dimension,  .05 equals a twentieth of corresponding plot dimension") set_colorbar_width;
%feature("docstring", "get_colorbar_orientation([ChipsId])\n\nReturns the orientation (0 = horizontal, 1 = vertical) of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_orientation;
%feature("docstring", "set_colorbar_orientation([ChipsId], value)\n\nSets the orientation of the colorbar specified by ChipsId (or current if omitted) to the provided value. Accepted values are 0 for horizontal and 1 for vertical orientations.") set_colorbar_orientation;
%feature("docstring", "get_colorbar_parallel_offset([ChipsId])\n\nReturns the parallel offset value of the label of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_parallel_offset;
%feature("docstring", "get_colorbar_perpendicular_offset([ChipsId])\n\nReturns the perpendicular offset value (distance between colorbar and label in pixels) for the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_perpendicular_offset;
%feature("docstring", "set_colorbar_perpendicular_offset([ChipsId], pixels)\n\nSets the distance between the colorbar and its label to the specified number of pixels. Values less than 0 move away from the plot and values greater than zero move towards the plot.") set_colorbar_perpendicular_offset;
%feature("docstring", "set_colorbar_parallel_offset([ChipsId], pixels)\n\nOffsets the location of the colorbar label. Accepted values are (-.5:.5). A value of 0 is centered in relation to the colorbar.") set_colorbar_parallel_offset;
%feature("docstring", "set_colorbar_tick_style([ChipsId,], style)\n\nSets the tick style (chips_inside|chips_outside|chips_centered) of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by style") set_colorbar_tick_style;
%feature("docstring", "get_colorbar_tick_style([ChipsId])\n\nReturns the tick style of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_tick_style;
%feature("docstring", "set_colorbar_tick_mode([ChipsId,], mode)\n\nSets the tick mode ('none'|'nice'|'ends') of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by mode") set_colorbar_tick_mode;
%feature("docstring", "get_colorbar_tick_mode([ChipsId])\n\nReturns the tick mode of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_tick_mode;
%feature("docstring", "set_colorbar_tick_thickness([ChipsId,], thickness)\n\nSets the tick and border thickness (0.5-10) of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by thickness") set_colorbar_tick_thickness;
%feature("docstring", "get_colorbar_tick_thickness([ChipsId])\n\nReturns the thickness of the ticks and border of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_tick_thickness;
%feature("docstring", "set_colorbar_tick_location([ChipsId,], location)\n\nSets the tick and ticklabel location of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by location. If the value 'inside' is specified the ticks and ticklabels are drawn on the side of the colorbar that faces the plot. If 'outside' is specified the ticks and ticklabels are drawn on the side of the colorbar that faces away from the plot.") set_colorbar_tick_location;
%feature("docstring", "get_colorbar_tick_location([ChipsId])\n\nReturns the location of the ticks and ticklabels of the colorbar specified by ChipsId (or current if omitted). The location indicates whether ticks are drawn on the side of the colorbar that faces the plot ('inside') or the side that faces away from the center of the plot ('outside'). An error is returned if accessing more than one colorbar.") get_colorbar_tick_location;
%feature("docstring", "set_colorbar_tick_length([ChipsId,], length)\n\nSets the length of the tick marks of the colorbar specified by ChipsId (or current colorbar if omitted) to the value specified by length. Accepted values are 0..100") set_colorbar_tick_length;
%feature("docstring", "get_colorbar_tick_length([ChipsId])\n\nReturns the length of the tick marks of the colorbar specified by ChipsId (or current if omitted). An error is returned if accessing more than one colorbar.") get_colorbar_tick_length;
%feature("docstring", "valid_axisstyle_type(type)\n\nReturns true if the input value is considered a valid axisstyle enumeration such as chips_open or a corresponding numeric value such as 0.") valid_axisstyle_type;
%feature("docstring", "valid_baddatamode_type(type)\n\nReturns true if the input value is considered a valid bad data mode such as chips_ignore or chips_omit or a corresponding numeric value such as 0.") valid_baddatamode_type;
%feature("docstring", "valid_color(string)\n\nReturns true if the input string is considered a valid color (ie. a predefined color such as 'red' or a six digit hex rgb value such as 'ff0000').") valid_color;
%feature("docstring", "valid_colormap_type(type)\n\nReturns true if the input value is considered a valid colormap type such as chips_hsv or a corresponding enumerated value such as 5.") valid_colormap_type;
%feature("docstring", "valid_contour_type(type)\n\nReturns true if the input value is considered a valid contour algorithm such as chips_standard or chips_marching or a corresponding numeric value.") valid_contour_type;
%feature("docstring", "valid_coord_sys_type(type)\n\nReturns true if the input value is considered a valid coordinate system type such as PIXEL, FRAME_NORM, or a corresponding enumerated value such as 2.") valid_coord_sys_type;
%feature("docstring", "valid_cornerstyle_type(type)\n\nReturns true if the input value is considered a valid cornerstyle enumeration such as chips_bevel or a corresponding numeric value such as 1.") valid_cornerstyle_type;
%feature("docstring", "valid_errstyle_type(type)\n\nReturns true if the input value is considered a valid error style enumeration such as chips_bar or a corresponding numeric value such as 0.") valid_errstyle_type;
%feature("docstring", "valid_fill_type(type)\n\nReturns true if the input value is considered a valid fill style enumeration such as chips_brick or a corresponding numeric value such as 7.") valid_fill_type;
%feature("docstring", "valid_font_type(type)\n\nReturns true if the input value is considered a valid font type enumeration such as chips_courier or a corresponding numeric value such as 1.") valid_font_type;
%feature("docstring", "valid_fontstyle_type(type)\n\nReturns true if the input value is considered a valid fontstyle enumeration such as chips_italic or a corresponding numeric value such as 2.") valid_fontstyle_type;
%feature("docstring", "valid_interpolation_type(type)\n\nReturns true if the input value is considered a valid interpolation enumeration such as chips_bilinear or a corresponding numeric value such as 1.") valid_interpolation_type;
%feature("docstring", "valid_line_extension_type(type)\n\nReturns true if the input value is considered a valid line extension enumeration such as chips_extend_both or a corresponding numeric value such as 3.") valid_line_extension_type;
%feature("docstring", "valid_line_type(type)\n\nReturns true if the input value is considered a valid line type enumeration such as chips_solid or a corresponding numeric value such as 0.") valid_line_type;
%feature("docstring", "valid_object_type(type)\n\nReturns true if the input value is considered a valid object type enumeration such as chips_curve or a corresponding numeric value such as 6.") valid_object_type;
%feature("docstring", "valid_printfile_type(type)\n\nReturns true if the input value is considered a valid printing type enumeration such as chips_rgb or a corresponding numeric value such as 1.") valid_printfile_type;
%feature("docstring", "valid_rgba_type(type)\n\nReturns true if the input value is considered a valid rgba channel enumeration such as chips_red or chips_alpha or a corresponding numeric value such as 3.") valid_rgba_type;
%feature("docstring", "valid_shuffle_type(type)\n\nReturns true if the input value is considered a valid shuffle enumeration such as chips_forward or chips_front or a corresponding numeric value such as 2.") valid_shuffle_type;
%feature("docstring", "valid_symbol_type(type)\n\nReturns true if the input value is considered a valid symbol enumeration such as chips_cross or chips_circle or a corresponding numeric value such as 1.") valid_symbol_type;
%feature("docstring", "valid_tickmode_type(mode)\n\nReturns true if the input value is considered a valid tickmode enumeration such as chips_count or a corresponding numeric value such as 0.") valid_tickmode_type;
%feature("docstring", "valid_tickstyle_type(type)\n\nReturns true if the input value is considered a valid tick style enumeration such as chips_inside or chips_centered or a corresponding numeric value such as 2.") valid_tickstyle_type;
%feature("docstring", "valid_units_type(type)\n\nReturns true if the input value is considered a valid units enumeration such as chips_pixels or chips_cm or a corresponding numeric value such as 3.") valid_units_type;
%feature("docstring", "highlight_object([ChipsId|idstr,] object_type)\n\nHighlights (blinks) the instance(s) of object type in the context specified by ChipsId (current context if omitted)") highlight_object;
/***** END DOC STRING SECTION *****/


/* ignore the following values */
%ignore approx_equal (double d1, double d2, double relerr = 0.00001, double maxerr= 0.000000001);
%ignore chips_strcmp(const char *str1, const char *str2);
%ignore set_grid_color(ChipsId* cid, string in_color);
%ignore set_grid_thickness(ChipsId* cid, int in_thickness);
%ignore set_grid_linestyle(ChipsId* cid, V_LINE_TYPE in_style);
%ignore set_grid_scale(ChipsId* cid, int in_scale);
%ignore display_frame_layout(ChipsId* cid);
%ignore hide_frame_layout(ChipsId* cid);
%ignore set_layout_color(ChipsId* cid, string in_color);
%ignore set_layout_thickness(ChipsId* cid, int in_thickness);
%ignore set_layout_linestyle(ChipsId* cid, V_LINE_TYPE in_style);
%ignore serialize(ChipsStateStream &out);
%ignore deserialize(ChipsStateStream &in);
%ignore delete_all_images();
%ignore delete_all_images(ChipsId* cid);
%ignore get_group_id();
%ignore set_group_id(string val);
%ignore get_legend_id();
%ignore set_legend_id(string val);
%ignore hoffset_to_string(float val);
%ignore voffset_to_string(float val);
%ignore V_INCH2PIX_CONV;
%ignore V_CM2PIX_CONV;
%ignore V_MM2PIX_CONV;
%ignore V_DEF_WINDOW_SCALE;
%ignore V_DEF_WINDOW_DIM;
%ignore MAX_NUM_FRAMES;
%ignore MAX_NUM_LAYERS;
%ignore V_MAX_TILE_COLS;
%ignore V_MAX_TILE_ROWS;
%ignore V_MAX_MARGIN_SIZE;
%ignore V_MIN_PAR_OFFSET;
%ignore V_MAX_PAR_OFFSET;
%ignore V_MIN_PERP_OFFSET;
%ignore V_MAX_PERP_OFFSET;
%ignore V_GRID_SCALE_DEFAULT;
%ignore V_GRID_SCALE_QTR_INCH;
%ignore V_GRID_SCALE_HALF_INCH;
%ignore V_GRID_SCALE_CENTIMETER;
%ignore V_GRID_SCALE_NORMALIZED;
%ignore V_SIZE_MIN_VAL;
%ignore V_SIZE_DEF_VAL;
%ignore V_SIZE_MAX_VAL;
%ignore SET_CURVE_SYMBOL_ANGLE;
%ignore SET_AXIS_DEPTH;
%ignore SET_CONTOUR_DEPTH;
%ignore SET_CURVE_DEPTH;
%ignore SET_DEFAULT_DEPTH;
%ignore SET_HISTO_DEPTH;
%ignore SET_LINE_DEPTH;
%ignore SET_PLOT_DEPTH;
%ignore SET_POINT_DEPTH;
%ignore SET_COLORBAR_DEPTH; 
%ignore V_CLEAR_ALL; 
%ignore V_DEFAULT_DEPTH;
%ignore V_DEPTH_BACK; 
%ignore V_DEPTH_BACKWARD; 
%ignore V_DEPTH_FORWARD; 
%ignore V_DEPTH_FRONT; 
%ignore XZ_AXIS; 
%ignore YZ_AXIS; 
%ignore V_PI; 
%ignore SET_HISTO_SYMBOL_ANGLE;
%ignore SET_POINT_SYMBOL_STYLE;
%ignore SET_POINT_SYMBOL_COLOR;
%ignore SET_POINT_SYMBOL_SIZE;
%ignore SET_POINT_SYMBOL_FILL;
%ignore SET_POINT_SYMBOL_ANGLE;
%ignore SET_POINT_VISIBLE;
%ignore SET_CURVE_SYMBOL_STYLE;
%ignore SET_CURVE_SYMBOL_COLOR;
%ignore SET_CURVE_SYMBOL_SIZE;
%ignore SET_CURVE_SYMBOL_FILL;
%ignore SET_CURVE_LINE_STYLE;
%ignore SET_CURVE_LINE_COLOR;
%ignore SET_CURVE_LINE_THICKNESS;
%ignore SET_CURVE_ERR_STYLE;
%ignore SET_CURVE_ERR_COLOR;
%ignore SET_CURVE_ERR_THICKNESS;
%ignore SET_CURVE_ERR_UP;
%ignore SET_CURVE_ERR_DOWN;
%ignore SET_CURVE_ERR_LEFT;
%ignore SET_CURVE_LIMLENGTH;
%ignore SET_CURVE_LIMOVERRIDE;
%ignore SET_CURVE_ERR_RIGHT;
%ignore SET_LINE_STYLE;
%ignore SET_LINE_COLOR;
%ignore SET_LINE_THICKNESS;
%ignore SET_LINE_VISIBLE;
%ignore SET_AXIS_COLOR;
%ignore SET_AXIS_THICKNESS;
%ignore SET_AXIS_TEXT;
%ignore SET_AXIS_MAJ_STYLE;
%ignore SET_AXIS_TICKFORMAT;
%ignore SET_AXIS_MAJOR_NUM;
%ignore SET_AXIS_MINOR_NUM;
%ignore SET_AXIS_MAJOR_COLOR;
%ignore SET_AXIS_MINOR_COLOR;
%ignore SET_AXIS_MAJORINT;
%ignore SET_AXIS_MINORINT;
%ignore SET_AXIS_MAJORLENGTH;
%ignore SET_AXIS_MINORLENGTH;
%ignore SET_AXIS_VISIBLE;
%ignore SET_AXIS_SCALABLE;
%ignore SET_AXIS_MAJ_VIS;
%ignore SET_AXIS_MIN_VIS;
%ignore SET_AXIS_MAJ_THICKNESS;
%ignore SET_AXIS_MIN_THICKNESS;
%ignore SET_AXIS_MIN_STYLE;
%ignore SET_AXIS_MAJ_MODE;
%ignore SET_AXIS_MIN_MODE;
%ignore SET_AXIS_REVERSE;
%ignore SET_AXIS_OFFSET_PAR;
%ignore SET_AXIS_OFFSET_PERP;
%ignore SET_AXIS_LOG_MIN;
%ignore SET_AXIS_TICKLABEL_STYLE;
%ignore SET_AXIS_PAD;
%ignore SET_AXIS_AUTOMIN;
%ignore SET_AXIS_AUTOMAX;
%ignore SET_GRID_THICKNESS;
%ignore SET_GRID_COLOR;
%ignore SET_GRID_LINESTYLE;
%ignore SET_GRID_SCALE;
%ignore GET_AXIS_SCALE;
%ignore SET_LAYOUT_COLOR;
%ignore SET_LAYOUT_LINESTYLE;
%ignore SET_LAYOUT_THICKNESS;
%ignore SET_PLOT_LMARGIN;
%ignore SET_PLOT_RMARGIN;
%ignore SET_PLOT_TMARGIN;
%ignore SET_PLOT_BMARGIN;
%ignore SET_PLOT_LOGSCALE;
%ignore SET_PLOT_LINEARSCALE;
%ignore SET_PLOT_TITLE;
%ignore SET_PLOT_VISIBLE;
%ignore SET_PLOT_XLABEL;
%ignore SET_PLOT_YLABEL;
%ignore SET_PLOT_ZLABEL;
%ignore SET_PLOT_TITLE_COLOR;
%ignore SET_PLOT_TITLE_X;
%ignore SET_PLOT_TITLE_Y;
%ignore SET_PLOT_TITLE_FONT;
%ignore SET_PLOT_TITLE_FONTSTYLE;
%ignore SET_PLOT_TITLE_SIZE;
%ignore SET_PLOT_AXIS_STYLE;
%ignore SET_PLOT_TITLE_ANGLE;
%ignore SET_WINDOW_COLOR;
%ignore SET_WINDOW_TITLE;
%ignore SET_WINDOW_REDRAW;
%ignore SET_WINDOW_VISIBLE;
%ignore SET_FRAME_COLOR;
%ignore SET_FRAME_TRANSPARENT;
%ignore SET_FRAME_SCALE;
%ignore SET_FRAME_BORDER_COLOR;
%ignore SET_FRAME_BORDER_VIS;
%ignore SET_FRAME_VISIBLE;
%ignore SET_LABEL_STYLE;
%ignore SET_LABEL_SIZE;
%ignore SET_LABEL_COLOR;
%ignore SET_LABEL_ANGLE;
%ignore SET_LABEL_VISIBLE;
%ignore SET_LABEL_TEXT;
%ignore SET_LABEL_FONT;
%ignore SET_AXIS_LABEL_ANGLE;
%ignore SET_AXIS_LABEL_FONT;
%ignore SET_AXIS_LABEL_FONTSTYLE;
%ignore SET_AXIS_LABEL_SIZE;
%ignore SET_AXIS_LABEL_COLOR;
%ignore SET_AXIS_TICKLABEL_OFFSET;
%ignore SET_AXIS_TICKLABEL_FONT;
%ignore SET_AXIS_TICKLABEL_FNTSTL;
%ignore SET_AXIS_TICKLABEL_SIZE;
%ignore SET_AXIS_TICKLABEL_COLOR;
%ignore SET_AXIS_TICKLABEL_ANGLE;
%ignore SET_AXIS_MAJGRD_VIS;
%ignore SET_AXIS_MAJGRD_COLOR;
%ignore SET_AXIS_MAJGRD_STYLE;
%ignore SET_AXIS_MAJGRD_THICK;
%ignore SET_AXIS_MINGRD_VIS;
%ignore SET_AXIS_MINGRD_COLOR;
%ignore SET_AXIS_MINGRD_STYLE;
%ignore SET_AXIS_MINGRD_THICK;
%ignore SET_AXIS_TICKLABEL_VIS;
%ignore SET_CONTOUR_COLOR;
%ignore SET_CONTOUR_STYLE;
%ignore SET_CONTOUR_THICKNESS;
%ignore SET_CONTOUR_MODE;
%ignore SET_CONTOUR_TICK_VIS;
%ignore SET_CONTOUR_TICK_COLOR;
%ignore SET_CONTOUR_TICK_THICK;
%ignore SET_CONTOUR_TICK_LENGTH;
%ignore SET_CONTOUR_TICK_DIR;
%ignore SET_CONTOUR_LAB_VIS;
%ignore SET_CONTOUR_LAB_COLOR;
%ignore SET_CONTOUR_LAB_SIZE;
%ignore SET_CONTOUR_LAB_FONT;
%ignore SET_CONTOUR_LAB_FONTSTYLE;
%ignore SET_HISTO_SYMBOL_STYLE;
%ignore SET_HISTO_SYMBOL_COLOR;
%ignore SET_HISTO_SYMBOL_SIZE;
%ignore SET_HISTO_SYMBOL_FILL;
%ignore SET_HISTO_LINE_STYLE;
%ignore SET_HISTO_LINE_COLOR;
%ignore SET_HISTO_LINE_THICKNESS;
%ignore SET_HISTO_ERR_STYLE;
%ignore SET_HISTO_ERR_COLOR;
%ignore SET_HISTO_ERR_THICKNESS;
%ignore SET_HISTO_ERR_UP;
%ignore SET_HISTO_ERR_DOWN;
%ignore SET_HISTO_DROPLINE;
%ignore SET_LEGEND_COLOR;
%ignore SET_LEGEND_THICKNESS;
%ignore SET_LEGEND_TRANSPARENT;
%ignore SET_LEGEND_VISIBLE;
%ignore SET_LEGEND_BORDERS;
%ignore SET_EXPORT_FONTS;
%ignore SET_EXPORT_ORIENT;
%ignore SET_PLOT_TITLE_HALIGN;
%ignore SET_PLOT_TITLE_VALIGN;
%ignore SET_LABEL_HALIGN;
%ignore SET_LABEL_VALIGN;
%ignore SET_LABEL_DEPTH;
%ignore SET_AXIS_LABEL_HALIGN;
%ignore SET_AXIS_LABEL_VALIGN;
%ignore SET_AXIS_TICKLABEL_HALIGN;
%ignore SET_AXIS_TICKLABEL_VALIGN;
%ignore SET_REGN_ANGLE;
%ignore SET_REGN_EDGE_COLOR;
%ignore SET_REGN_EDGE_STYLE;
%ignore SET_REGN_EDGE_THICKNESS;
%ignore SET_REGN_FILL;
%ignore SET_REGN_DEPTH;
%ignore SET_REGN_FILL_COLOR;
%ignore SET_REGN_OPACITY;
%ignore SET_REGN_VISIBLE;
%ignore V_CHECKPOINT_VAL;
%ignore CMD_MNGER_MAX_WEIGHT;
%ignore V_STATE_MAJ_VER;
%ignore V_STATE_MIN_VER;
%ignore GET_AXIS_RANGE;
%ignore GET_CONTOUR_RANGE;
%ignore GET_CONTOUR_XRANGE;
%ignore GET_CONTOUR_YRANGE;
%ignore GET_CONTOUR_ZRANGE;
%ignore GET_HISTO_RANGE;
%ignore GET_HISTO_XRANGE;
%ignore GET_HISTO_YRANGE;
%ignore GET_PLOT_XRANGE;
%ignore GET_PLOT_YRANGE;
%ignore GET_CURVE_RANGE;
%ignore GET_CURVE_XRANGE;
%ignore GET_CURVE_YRANGE;
%ignore SET_AXIS_LABEL_VIS;
%ignore SET_AXIS_LLCORNER;
%ignore SET_AXIS_LLCORNWID;
%ignore SET_AXIS_SCALE;
%ignore SET_AXIS_TICKSTYLE;
%ignore SET_AXIS_URCORNER;
%ignore SET_AXIS_URCORNWID;
%ignore SET_CONTOUR_ALGORITHM;
%ignore SET_CONTOUR_INTERVAL;
%ignore SET_CONTOUR_NUMLEVELS;
%ignore SET_CONTOUR_VISIBLE;
%ignore SET_CURVE_BADDATA;
%ignore SET_CURVE_VISIBLE;
%ignore SET_HISTO_VISIBLE;
%ignore SET_PAGE_BMARGIN;
%ignore SET_PAGE_FIT;
%ignore SET_PAGE_HEIGHT;
%ignore SET_PAGE_KEEP_ASPECT;
%ignore SET_PAGE_LMARGIN;
%ignore SET_PAGE_RMARGIN;
%ignore SET_PAGE_SCALE_HEIGHT;
%ignore SET_PAGE_SCALE_UNITS;
%ignore SET_PAGE_SCALE_WIDTH;
%ignore SET_PAGE_SIZE;
%ignore SET_PAGE_TMARGIN;
%ignore SET_PAGE_UNITS;
%ignore SET_PAGE_WIDTH;
%ignore SET_PLOT_CORNER;
%ignore GET_CURVE_RANGE;
%ignore GET_HISTOGRAM_RANGE;
%ignore GET_PLOT_RANGE;
%ignore Z_AXIS;
%ignore set_region_angle;
%ignore V_THICK_MIN_VAL;
%ignore V_THICK_DEF_VAL;
%ignore V_THICK_MAX_VAL;
%ignore SET_IMAGE_CMAP;
%ignore SET_IMAGE_INV_CMAP;
%ignore SET_IMAGE_INTERPOLATION;
%ignore SET_IMAGE_ALPHA;
%ignore SET_IMAGE_DEPTH;
%ignore SET_IMAGE_THRESHOLD;
%ignore SET_IMAGE_VISIBLE;
%ignore SET_IMAGE_CMAP_SIZE;
%ignore SET_IMAGE_CMAP_INTERPOLATE;
%ignore SET_CBAR_BORDERVIS;
%ignore SET_CBAR_DEPTH;
%ignore SET_CBAR_HALIGN;
%ignore SET_CBAR_HEIGHT;
%ignore SET_CBAR_LABEL_ANGLE;
%ignore SET_CBAR_LABEL_COLOR;
%ignore SET_CBAR_LABEL_FONT;
%ignore SET_CBAR_LABEL_FONTSTYLE;
%ignore SET_CBAR_LABEL_HALIGN;
%ignore SET_CBAR_LABEL_VALIGN;
%ignore SET_CBAR_LABEL_SIZE;
%ignore SET_CBAR_LABEL_VIS; 
%ignore SET_CBAR_ORIENTATION; 
%ignore SET_CBAR_TEXT; 
%ignore SET_CBAR_TICK_THICKNESS; 
%ignore SET_CBAR_TICK_LENGTH; 
%ignore SET_CBAR_TICK_COLOR; 
%ignore SET_CBAR_TICK_MODE; 
%ignore SET_CBAR_TICK_STYLE; 
%ignore SET_CBAR_TICKLABEL_ANGLE; 
%ignore SET_CBAR_TICKLABEL_COLOR; 
%ignore SET_CBAR_TICKLABEL_FONT; 
%ignore SET_CBAR_TICKLABEL_FNTSTL; 
%ignore SET_CBAR_TICKLABEL_STYLE; 
%ignore SET_CBAR_TICKLABEL_OFFSET; 
%ignore SET_CBAR_TICKLABEL_SIZE; 
%ignore SET_CBAR_TICKLABEL_HALIGN; 
%ignore SET_CBAR_TICKLABEL_VALIGN; 
%ignore SET_CBAR_TICKLABEL_VIS; 
%ignore SET_CBAR_VISIBLE; 
%ignore SET_CBAR_VALIGN; 
%ignore SET_CBAR_WIDTH; 
%ignore new_instancemethod;
%ignore get_axis_ref_id;
%ignore get_contour_ref_id;
%ignore get_curve_ref_id;
%ignore get_frame_ref_id;
%ignore get_histogram_ref_id;
%ignore get_image_ref_id;
%ignore get_colorbar_ref_id;
%ignore get_label_ref_id;
%ignore get_legend_ref_id;
%ignore get_line_ref_id;
%ignore get_plot_ref_id;
%ignore get_point_ref_id;
%ignore get_region_ref_id;
%ignore get_window_ref_id;
%ignore get_xaxis_ref_id;
%ignore get_yaxis_ref_id;
%ignore set_axis_ref_id;
%ignore set_contour_ref_id;
%ignore set_curve_ref_id;
%ignore set_frame_ref_id;
%ignore set_histogram_ref_id;
%ignore set_image_ref_id;
%ignore set_colorbar_ref_id;
%ignore set_label_ref_id;
%ignore set_legend_ref_id;
%ignore set_line_ref_id;
%ignore set_plot_ref_id;
%ignore set_point_ref_id;
%ignore set_region_ref_id;
%ignore set_window_ref_id;
%ignore set_xaxis_ref_id;
%ignore set_yaxis_ref_id;
%ignore set_layout_thickness;
%ignore set_object_ref_id;
%ignore set_axis_logminimum;
%ignore get_axis_logminimum;
%ignore set_axis_scalable;
%ignore set_grid_thickness;
%ignore chips_fill_size;
%ignore chips_cmap_size;
%ignore chips_not_subobject;
%ignore chips_plot_title;
%ignore chips_axis_label;
%ignore chips_colorbar_label;

/* Typemap arrays and remove any reference to size of the array */
%typemap(in) (double*){
   int cnt=0;

   if (Py_None != $input){
      $1 = create_1darray_double ($input, cnt);
      if (current_array_size == 0)
         current_array_size = cnt;
      else
         good_array_size &= (cnt == current_array_size);
      if (cnt == 0)
      {
         SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: Zero length arrays are not supported for the requested operation");
         goto fail;
      }
   }
}


%typemap(freearg) (double *) {
   if ($1)
      delete []$1;
}


%typemap(in) (long cnt, double*){
   int cnt=0;

   if (Py_None != $input){
      $2 = create_1darray_double ($input, cnt);
      $1 = static_cast <$1_type> (cnt);
      /*reset the array size for future use. Necessary
        functions like add_contour which had to arrays of
        different size */
      current_array_size = cnt;
      good_array_size = true;

      if ($1 == 0)
      {
         SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: Zero length arrays are not supported for the requested operation");
         goto fail;
      }
   }
}

/* The reason for introducing the check rather than just
   putting the conditional in with the 'double *' typemap
   is that it doesn't restrict the 'double *' typemap as
   much. That is it can be used in other things that are
   unrelated to this condition with out interfering with
   the creation of the array */
%typemap (check) (long cnt, double *){
  if (!good_array_size){
      SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: All arrays must be of equal length");
      goto fail;
  }
}

%typemap(freearg) (long cnt, double *) {
   if ($2)
      delete []$2;
}

%typemap(arginit) (long cnt, double *){
   $1 = 0;
   current_array_size = 0;
   good_array_size=true;
}

%typecheck(SWIG_TYPECHECK_POINTER) (long cnt, double*){
  // This means a string cannot be used as an array of characters
  $1 = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None));
}

%typecheck(SWIG_TYPECHECK_POINTER) (double*){
   // This means a string cannot be used as an array of characters
  $1 = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None));
      
}


%apply (long cnt, double *)  {(int cnt, double *)};
%apply (long cnt, double *)  {(unsigned int arrsize, double*)};
%apply (long cnt, double *)  {(int numlev, double* levels)};
%apply (long cnt, double *)  {(unsigned int numlev, double* levels)};
%apply (long cnt, double *)  {(unsigned int arraysize, double* pos)};


%typemap(in) (double *yy,unsigned int count){
   int cnt = 0;
   if (Py_None != $input){
      $1 = create_1darray_double ($input, cnt);
      $2 = static_cast <$2_type> (cnt);
      if (current_array_size == 0)
         current_array_size = cnt;
      else
         good_array_size &= (cnt == current_array_size);
      if ($2 == 0)
      {
         SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: Zero length arrays are not supported for the requested operation");
         goto fail;
      }
   }
}


%typemap(freearg) (double *yy,unsigned int count){
   if ($1)
      delete []$1;
}

%typemap(arginit) (double *yy,unsigned int count){
   $2 = 0;
   current_array_size = 0;
   good_array_size=true;
}


%typecheck(SWIG_TYPECHECK_POINTER) (double *yy,unsigned int count){
  // This means a string cannot be used as an array of characters
  _v = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None
));
}

/* The reason for introducing the check rather than just
   putting the conditional in with the 'double *' typemap
   is that it doesn't restrict the 'double *' typemap as
   much. That is it can be used in other things that are
   unrelated to this condition with out interfering with
   the creation of the array */
%typemap (check) (double *yy,unsigned int count){
  if (!good_array_size){
      SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: All arrays must be of equal length");
      goto fail;
  }
}




%typemap(in) (int row, int column, double **transform){
  PyArrayObject *arr;
  arr = convert_array ($input, PyArray_DOUBLE, 2);
  if ((arr == NULL) || (arr->nd > 2)){
     $1=0;
     $2=0;
     $3=NULL;
     if (arr->nd > 2)
        Py_DECREF(arr);  /* Release our local copy of the PyArray */
     goto fail;
  }
  else if (arr->nd == 2){
     $1 = arr->dimensions[0];
     $2 = arr->dimensions[1];
     $3 = new double*[$1];
     for (int ii=0; ii<$1; ii++){
       $3[ii] = new double[$2];
       for (int jj=0; jj<$2; jj++)
         $3[ii][jj] = ((double *)(arr->data))[ii*$2+jj];
     }
     Py_DECREF(arr);  /* Release our local copy of the PyArray */
  }
  else {
     double sqr = sqrt ((double)(arr->dimensions[0]));
     if (sqr != (int)sqr){
        $1=0;
        $2=0;
        $3=NULL;
        PyErr_SetString(PyExc_TypeError, "Matrix must be a square matrix\n");
     }
     else{

        $1 = (int)sqr;
        $2 = $1;
        $3 = new double*[$1];
        for (int ii=0; ii<$1; ii++){
          $3[ii] = new double[$2];
          for (int jj=0; jj<$2; jj++)
            $3[ii][jj] = ((double *)(arr->data))[ii*$2+jj];
        }
     }
     Py_DECREF(arr);  /* Release our local copy of the PyArray */
  }
}

%typemap(freearg) (int row, int column, double **transform){
   if ($3){
      for (int ii=0; ii<$1; ii++)
         delete []$3[ii];
      delete []$3;
   }

}

%typecheck(SWIG_TYPECHECK_POINTER) (int row, int column, double **transform){
  // This means a string cannot be used as an array of characters
  _v = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None));
}



%typemap(in) (int cnt, std::string*){
   PyArrayObject *arr;
   arr = convert_array ($input, PyArray_STRING, 1);
   if (arr) {
      int arr_size = 1;
      for (int ii=arr->nd; ii--; arr_size *= arr->dimensions[ii]);
      $2 = new string[arr_size];
      char *tmp_str;
      int pos = 0;
      for (int ii=0; ii<arr_size; ii++){
         tmp_str = new char[arr->strides[0]+1];
         memcpy (tmp_str, &(arr->data)[pos], arr->strides[0]);
         tmp_str[arr->strides[0]] = '\0';
	 pos+=arr->strides[0];
         $2[ii] = tmp_str;
         delete[] tmp_str;
      }
      $1 = ($1_type) arr_size;
      Py_DECREF(arr);
   } else {
      $1 = 0;
      goto fail;
   }
}

%typemap(freearg) (int cnt, std::string*){
   if ($2)
      delete []$2;
}

%typecheck(SWIG_TYPECHECK_POINTER) (int cnt, std::string*){
  // This means a string cannot be used as an array of characters
  _v = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None));
}

%apply (int cnt, std::string*) {(int lcnt, std::string*)};



%typemap(in) (std::string* vals){
   PyArrayObject *arr;
   arr = convert_array ($input, PyArray_STRING, 1);
   if (arr){
      int arr_size = 1;
      for (int ii=arr->nd; ii--; arr_size *= arr->dimensions[ii]);
      $1 = new string[arr_size];
      char *tmp_str;
      int pos = 0;
      for (int ii=0; ii<arr_size; ii++){
         tmp_str = new char[arr->strides[0]+1];
         memcpy (tmp_str, &(arr->data)[pos], arr->strides[0]);
         tmp_str[arr->strides[0]] = '\0';
         pos+=arr->strides[0];
         $1[ii] = tmp_str;
         delete[] tmp_str;
      }
      Py_DECREF(arr);

      if (current_array_size == 0)
         current_array_size = arr_size;
      else
         good_array_size &= (arr_size == current_array_size);

   } else {
      $1 = 0;
      SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: Zero length arrays are not supported for the requested operation");
      goto fail;
   }
}

%typemap(freearg) (std::string* vals){
   if ($1)
      delete []$1;
}

%typecheck(SWIG_TYPECHECK_POINTER) (std::string* vals){
  // This means a string cannot be used as an array of characters
  _v = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None));
}

%typemap(in) (int cnt, int*){
   PyArrayObject *arr;
   arr = convert_array ($input, PyArray_INT, 1);
   if (arr) {
      int arr_size = 1;
      for (int ii=arr->nd; ii--; arr_size *= arr->dimensions[ii]);
      $1 = ($1_type) arr_size;
      $2 = new int[$1];
      for (int ii=$1; ii--;)
         $2[ii] = ((int *)(arr->data))[ii];
   } else {
      $1 = 0;
      goto fail;
   }
}

%typemap(freearg) (int cnt, int*){
   if ($2)
      delete []$2;
}

%typecheck(SWIG_TYPECHECK_POINTER) (int cnt, int*){
  // This means a string cannot be used as an array of characters
  _v = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None));
}

%typecheck(SWIG_TYPECHECK_POINTER) (int fcnt, float*){
  // This means a string cannot be used as an array of characters
  _v = ((PySequence_Check($input) && !PyString_Check ($input)) || ($input==Py_None));
}

%typemap(in) (float*){
   int cnt=0;

   if (Py_None != $input){
      $1 = create_1darray_float ($input, cnt);
      if (current_array_size == 0)
         current_array_size = cnt;
      else
         good_array_size &= (cnt == current_array_size);
      if (cnt == 0)
      {
         SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: Zero length arrays are not supported for the requested operation");
         goto fail;
      }
   }
}


%typemap(freearg) (float *) {
   if ($1)
      delete []$1;
}


%typemap(in) (int fcnt, float*){
   int fcnt=0;

   if (Py_None != $input){
      $2 = create_1darray_float ($input, fcnt);
      $1 = static_cast <$1_type> (fcnt);
      /*reset the array size for future use. Necessary
        functions like add_contour which had to arrays of
        different size */
      current_array_size = fcnt;
      good_array_size = true;

      if ($1 == 0)
      {
         SWIG_exception_fail(SWIG_ArgError(0), "chips ERROR: Zero length arrays are not supported for the requested operation");
         goto fail;
      }
   }
}

%typemap(freearg) (int fcnt, float*){
   if ($2)
      delete []$2;
}


/* typemap reference input arguments as output */

%typemap(in, numinputs=0)(std::string& out_val) (string tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (std::string& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_std_string(static_cast<std::string >(tmp_arg$argnum));
}

%apply (std::string& out_val) {(std::string& out_color)};

%typemap(in, numinputs=0)(int& out_val) (int tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (int& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(COORD_SYS* type) (COORD_SYS tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (COORD_SYS* type)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_LINE_TYPE& out_val) (V_LINE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_LINE_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_LINE_TYPE* out_line) (V_LINE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_LINE_TYPE* out_line)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_FILL_TYPE& out_val) (V_FILL_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_FILL_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_FILL_TYPE* out_line) (V_FILL_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_FILL_TYPE* out_line)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_LINE_EXTENSION_TYPE& out_val) (V_LINE_EXTENSION_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_LINE_EXTENSION_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_LINE_EXTENSION_TYPE* type) (V_LINE_EXTENSION_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_LINE_EXTENSION_TYPE* type)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_SYMBOL_TYPE* out_symbol) (V_SYMBOL_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_SYMBOL_TYPE* out_symbol)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_COLORMAP_TYPE* out_cmap) (V_COLORMAP_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_COLORMAP_TYPE* out_cmap)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_RGBA_TYPE* out_cmap) (V_RGBA_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_RGBA_TYPE* out_cmap)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_INTERPOLATION_TYPE* out_int) (V_INTERPOLATION_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_INTERPOLATION_TYPE* out_int)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_FONT_TYPE* out_font) (V_FONT_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_FONT_TYPE* out_font)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_CONTOUR_TYPE* out_contour) (V_CONTOUR_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_CONTOUR_TYPE* out_contour)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_CONTOUR_TYPE& out_contour) (V_CONTOUR_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_CONTOUR_TYPE& out_contour)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_FONTSTYLE_TYPE* out_fontstyle) (V_FONTSTYLE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_FONTSTYLE_TYPE* out_fontstyle)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_ERRSTYLE_TYPE* out_errstyle) (V_ERRSTYLE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_ERRSTYLE_TYPE* out_errstyle)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_AXISSTYLE_TYPE* out_axisstyle) (V_AXISSTYLE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_AXISSTYLE_TYPE* out_axisstyle)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_BADDATAMODE_TYPE* out_baddatamode) (V_BADDATAMODE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_BADDATAMODE_TYPE* out_baddatamode)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_CORNERSTYLE_TYPE* out_cornerstyle) (V_CORNERSTYLE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_CORNERSTYLE_TYPE* out_cornerstyle)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_TICKSTYLE_TYPE* out_tickstyle) (V_TICKSTYLE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_TICKSTYLE_TYPE* out_tickstyle)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_TICKMODE_TYPE* out_tickmode) (V_TICKMODE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_TICKMODE_TYPE* out_tickmode)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_OBJECT_TYPE* out_objecttype) (V_OBJECT_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_OBJECT_TYPE* out_objecttype)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}



%typemap(in, numinputs=0)(V_SYMBOL_TYPE& out_val) (V_SYMBOL_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_SYMBOL_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_COLORMAP_TYPE& out_val) (V_COLORMAP_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_COLORMAP_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_RGBA_TYPE& out_val) (V_RGBA_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_RGBA_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(V_INTERPOLATION_TYPE& out_val) (V_INTERPOLATION_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_INTERPOLATION_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(V_BADDATAMODE_TYPE& out_val) (V_BADDATAMODE_TYPE tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (V_BADDATAMODE_TYPE& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_From_int(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(double& out_val) (double tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (double& out_val)
{
   if ($result == Py_None) 
      Py_DECREF($result);

   $result = SWIG_From_double(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(float& out_val) (float tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (float& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);

   $result = SWIG_From_double(tmp_arg$argnum);
}


%typemap(in, numinputs=0)(bool& out_val) (bool tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (bool& out_val)
{
   if ($result == Py_None)
      Py_DECREF($result);

   $result = SWIG_From_bool(tmp_arg$argnum);
}

%typemap(in, numinputs=0)(COORD_SYS& out_val, std::string& out_id) (COORD_SYS tmp_sys, string tmp_id)
{
   $1 = &tmp_sys;
   $2 = &tmp_id;
}

%typemap (argout) (COORD_SYS& out_val, std::string& out_id)
{
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_int(tmp_sys$argnum));
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_std_string(tmp_id$argnum));
}


%typemap(in, numinputs=0)(double& out1, double& out2) (double tmp_arg[2])
{
   $1 = &tmp_arg[0];
   $2 = &tmp_arg[1];
}

%typemap (argout) (double& out1, double& out2)
{
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_double(tmp_arg$argnum[0]));
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_double(tmp_arg$argnum[1]));
}

%typemap(in, numinputs=0)(double& outx1, double& outx2, double& outy1, double& outy2) (double tmp_arg[4])
{
   $1 = &tmp_arg[0];
   $2 = &tmp_arg[1];
   $3 = &tmp_arg[2];
   $4 = &tmp_arg[3];
}

%typemap (argout) (double& outx1, double& outx2, double& outy1, double& outy2)
{
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_double(tmp_arg$argnum[0]));
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_double(tmp_arg$argnum[1]));
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_double(tmp_arg$argnum[2]));
   $result = SWIG_Python_AppendOutput ($result, SWIG_From_double(tmp_arg$argnum[3]));
}

%typemap (in, numinputs=1)(double *in_val, double *out_val, int count)
{

   PyArrayObject *arr;
   arr = convert_array ($input, PyArray_DOUBLE, 1);
   if (arr) {
      int arr_size = 1;
      for (int ii=arr->nd; ii--; arr_size *= arr->dimensions[ii]);
      $3 = ($3_type) arr_size;
      $1 = new double[$3];
      for (int ii=$3; ii--;)
         $1[ii] = ((double *)(arr->data))[ii];
   } else {
      $1 = 0;
      goto fail;
   }

   $2 = new double [$3];

}

%typemap (argout) (double *in_val, double *out_val, int count)
{

   PyObject *arr;
   intp dims[1];
   dims[0] = intp($3);
   arr = PyArray_SimpleNewFromData (1, dims, PyArray_DOUBLE, $2);
   if (arr != NULL){
     $result = SWIG_Python_AppendOutput ($result, arr);
   }
}


%typemap(in, numinputs=0)(int& outcount, CHIPS_PICK_EVT_P* picklist) (int tmpcnt, CHIPS_PICK_EVT_P tmp_pick)
{
   $1 = &tmpcnt;
   $2 = &tmp_pick;
}


%typemap(out) (list<std::string>) 
{
   int i =0;
   int sizei = $1.size();
   PyObject* my_list = PyList_New(sizei);
   std::list<std::string>::iterator iter;
   for (iter=$1.begin (); iter!=$1.end (); iter++)
      PyList_SetItem(my_list,i++,PyString_FromString((const char*)(*iter).c_str()));
   $result = (PyObject *)my_list;
}


%typemap (argout) (int& outcount, CHIPS_PICK_EVT_P* picklist)
{

   if (tmpcnt$argnum > 0)
   {
      double *x = new double [tmpcnt$argnum];
      double *y = new double [tmpcnt$argnum];
      unsigned int *event = new unsigned int [tmpcnt$argnum];
      char *key = new char[tmpcnt$argnum];
      bool keys_found = false;

      for (int ii=0; ii<tmpcnt$argnum; ii++){
         x[ii] = tmp_pick$argnum[ii].x;
         y[ii] = tmp_pick$argnum[ii].y;
         event[ii] = tmp_pick$argnum[ii].event;
         if ((event[ii] & KEY_RELEASE) || (event[ii] & KEY_PRESS))
            keys_found = true;
     
         key[ii] = tmp_pick$argnum[ii].key;
      }

      PyObject *arr;
      intp dims[1];
      dims[0] = intp(tmpcnt$argnum);
      arr = PyArray_SimpleNewFromData (1, dims, PyArray_DOUBLE, x);
      if (arr != NULL){
        $result = SWIG_Python_AppendOutput ($result, arr);
      }
      arr = PyArray_SimpleNewFromData (1, dims, PyArray_DOUBLE, y);
      if (arr != NULL){
        $result = SWIG_Python_AppendOutput ($result, arr);
      }
      arr = PyArray_SimpleNewFromData (1, dims, PyArray_UINT, event);
      if (arr != NULL){
        $result = SWIG_Python_AppendOutput ($result, arr);
      }

      if (keys_found){
        arr = PyArray_SimpleNewFromData (1, dims, PyArray_CHAR, key);
        if (arr != NULL){
          $result = SWIG_Python_AppendOutput ($result, arr);
        }
      }

      char *x_str;
      char *y_str;
      int itemsize = 0;
      int totalcnt = 0;
      PyArray_Dims  str_dims = {NULL, 1};
      intp tmp[] = {tmpcnt$argnum};
      str_dims.ptr = &tmp[0];


      // calculate the size of the x data
      for (int ii=0; ii<tmpcnt$argnum; ii++)
      {
        if (itemsize < tmp_pick$argnum[ii].x_str.length ()+1)
          itemsize = tmp_pick$argnum[ii].x_str.length ()+1;
        totalcnt += tmp_pick$argnum[ii].x_str.length ()+1;
      }

      // copy in the x data
      x_str = (char*)calloc (totalcnt, sizeof(char*));
      for (int ii=0; ii<tmpcnt$argnum; ii++)
        strcpy (&x_str[ii*itemsize], tmp_pick$argnum[ii].x_str.c_str());

      arr = PyArray_New(&PyArray_Type, str_dims.len, str_dims.ptr, PyArray_STRING, NULL, x_str, itemsize, 0, NULL );
      if (arr != NULL){
        $result = SWIG_Python_AppendOutput ($result, arr);
      }

      // calculate the size of the y data
      itemsize = 0;
      totalcnt = 0;
      for (int ii=0; ii<tmpcnt$argnum; ii++)
      {
        if (itemsize < tmp_pick$argnum[ii].y_str.length ()+1)
          itemsize = tmp_pick$argnum[ii].y_str.length ()+1;
        totalcnt += tmp_pick$argnum[ii].y_str.length ()+1;
      }

      // copy in the y data
      y_str = (char*)calloc (totalcnt, sizeof(char*));
      for (int ii=0; ii<tmpcnt$argnum; ii++)
        strcpy (&y_str[ii*itemsize], tmp_pick$argnum[ii].y_str.c_str());

      arr = PyArray_New(&PyArray_Type, str_dims.len, str_dims.ptr, PyArray_STRING, NULL, y_str, itemsize, 0, NULL );
      if (arr != NULL){
        $result = SWIG_Python_AppendOutput ($result, arr);
      }


/*      arr = PyArray_SimpleNewFromData (1, dims, PyArray_STRING, x_str);
      if (arr != NULL){
        $result = SWIG_Python_AppendOutput ($result, arr);
      }

      arr = PyArray_SimpleNewFromData (1, dims, PyArray_STRING, y_str);
      if (arr != NULL){
        $result = SWIG_Python_AppendOutput ($result, arr);
      }
*/

      if (tmp_pick$argnum)
         delete[] (tmp_pick$argnum);


   }
   else
      $result = SWIG_Py_Void ();
}


%typemap(in, numinputs=0)(Transform** trans) (Transform* tmp_arg)
{
   $1 = &tmp_arg;
}

%typemap (argout) (Transform** trans)
{
   if ($result == Py_None)
      Py_DECREF($result);
   $result = SWIG_NewPointerObj(SWIG_as_voidptr(tmp_arg$argnum), SWIGTYPE_p_Transform, 0 |  0 );
}


%typemap(in, numinputs=0)(unsigned int &arraysize, double** pos) (unsigned int tmp_size, double* tmp_arg)
{
   $1 = &tmp_size;
   $2 = &tmp_arg;
}

%typemap (argout) (unsigned int &arraysize, double** pos)
{
   if ($result == Py_None)
      Py_DECREF($result);

   for (int ii=0; ii<*($1); ii++)
   {
      $result = SWIG_Python_AppendOutput ($result, SWIG_From_double(tmp_arg$argnum[ii]));
   }
   
/*   intp dims[] = {*($1)};
   
   $result = PyArray_SimpleNewFromData(1, dims, NPY_DOUBLE, tmp_arg$argnum );
   if (NULL == $result) SWIG_fail;
*/
}




%include "std_string.i"

%include "ChipsId.hh"
%include "ChipsDefines.hh"

%typemap(out) int {
   int verb=0, wcnt=0;
   int rval = (arg1)->get_error_verbosity(verb);
   if (!rval && (verb > 0)){
      if ($1 != ERR_STATUS_GOOD)  
      {
        const char* estr = (arg1)->get_errors();
        if (estr && !strlen(estr))
           estr = get_error_message($1);
	 (arg1)->get_error_types(wcnt);
	 if (wcnt != 2)
            SWIG_exception_fail(SWIG_ArgError(0), estr);
         else if (verb > 1)
            printf("%s", (arg1)->get_errors(2));
      }
      else{ 
         (arg1)->get_error_count(2, wcnt);
	 if ((verb >  1) && wcnt)
              printf("%s", (arg1)->get_errors(2));
      }
      $result = SWIG_Py_Void ();
   }
   else if (!rval)
   {
      $result = SWIG_From_int($1);
   }
   else if (rval == ERR_NO_SOCKET)
   {
      const char* estr = (arg1)->get_errors();
      if (verb  && estr && strlen(estr)) 
         SWIG_exception_fail(SWIG_ArgError(0), estr);
      else 
         $result = SWIG_Py_Void ();
   }
   else
      $result = SWIG_Py_Void ();
}

%include "ChipsRemoteMod.hh"

%init %{
   import_array();
%}


