% 
%  Copyright (C) 2008  Smithsonian Astrophysical Observatory
%
%
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License along
%  with this program; if not, write to the Free Software Foundation, Inc.,
%  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
%

require ("chips");
py_import ("pychips.hlui");
private define _chips_push ( arg ) { return arg; }
%Check for return. Push tuples on the list individually
private define check_return ()
{
  variable ret_val=NULL;
  if (_stkdepth () > 0){
     ret_val = ();
     if (py_call(py.type,ret_val) == py.tuple){
        variable len = py_call (py.len (ret_val))-1;
        variable ii;
        _for (0, len, 1){
            ii = ();
           _chips_push (ret_val[ii]);
        }
     }
  }
  return ret_val;
}

%Create valid types for python
private define _chips_check_type (value);
private define _chips_check_type (value)
{
  if (typeof (value) == Char_Type)
     value = int (value);
  return value;
}
%Turn a structure into a list
private define _chips_struct_to_list (s);
private define _chips_struct_to_list (s)
{
  variable sl_list = {};
  variable name, value;
  variable prefix = NULL;
  if (_NARGS > 1)
     prefix = ();
  try{
     %Go through each element in the structure and make it a list
     foreach (get_struct_field_names (s)){
        name = ();
        value = get_struct_field (s, name);
        if (prefix != NULL)
           name = prefix + "." + name;
        %Expand out sub structures
        if (is_struct_type (value)){
           variable tmp_list = _chips_struct_to_list (name,value);
           variable ii;
           %Concatinate the two lists
           _for (length (tmp_list)-1, 0, -1){
              ii = ();
              list_insert (sl_list, tmp_list[ii], 0);
           }
        }
        else{
           value = _chips_check_type (value);
           list_insert (sl_list,value,0);
           list_insert (sl_list,name,0);
        }
     }
  }
  catch RunTimeError:
     %Return NULL if this was not a structure
     return NULL;
  return sl_list;
}
#if (_slang_version >= 20101)
%Return all the qualifiers in a struct
private define _chips_get_qualifiers (kwstruct);
private define _chips_get_qualifiers (kwstruct)
{
  if( kwstruct == NULL )
    return NULL;
  variable ii, name;
  variable kwnames = get_struct_field_names(kwstruct);
  variable kwargs = Struct_Type[2 * length(kwnames)];
  ii = 0;
  foreach (kwnames)
  {
     name = ();
     kwargs[ii] = struct{value=name};
     ii = ii + 1;
     kwargs[ii] = struct{value=get_struct_field(kwstruct, name)};
     ii = ii + 1;
   }
 return kwargs;
}
#endif
%Chips Functions
define ChipsContour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.ChipsContour, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.ChipsContour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.ChipsContour);
       else
         py_call(py.pychips.hlui.crates.ChipsContour, PY_KW,  __push_args(kwargs));
     }
}

define get_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.hlui.crates.get_contour, __push_args (f_args));
        else
         _py_call(py.pychips.hlui.crates.get_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.hlui.crates.get_contour);
       else
         _py_call(py.pychips.hlui.crates.get_contour, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define set_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.set_contour, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.set_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.set_contour);
       else
         py_call(py.pychips.hlui.crates.set_contour, PY_KW,  __push_args(kwargs));
     }
}

define add_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_contour, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.add_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_contour);
       else
         py_call(py.pychips.hlui.crates.add_contour, PY_KW,  __push_args(kwargs));
     }
}

define add_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_curve, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.add_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_curve);
       else
         py_call(py.pychips.hlui.crates.add_curve, PY_KW,  __push_args(kwargs));
     }
}

define add_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_histogram, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.add_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_histogram);
       else
         py_call(py.pychips.hlui.crates.add_histogram, PY_KW,  __push_args(kwargs));
     }
}

define add_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_image, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.add_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.add_image);
       else
         py_call(py.pychips.hlui.crates.add_image, PY_KW,  __push_args(kwargs));
     }
}

define get_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.hlui.crates.get_image, __push_args (f_args));
        else
         _py_call(py.pychips.hlui.crates.get_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.hlui.crates.get_image);
       else
         _py_call(py.pychips.hlui.crates.get_image, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define delete_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.delete_image, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.delete_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.delete_image);
       else
         py_call(py.pychips.hlui.crates.delete_image, PY_KW,  __push_args(kwargs));
     }
}

define set_image_channel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.set_image_channel, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.set_image_channel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.set_image_channel);
       else
         py_call(py.pychips.hlui.crates.set_image_channel, PY_KW,  __push_args(kwargs));
     }
}

define make_figure ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hlui.crates.make_figure, __push_args (f_args));
        else
         py_call(py.pychips.hlui.crates.make_figure, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hlui.crates.make_figure);
       else
         py_call(py.pychips.hlui.crates.make_figure, PY_KW,  __push_args(kwargs));
     }
}

$1 = path_concat (path_dirname (__FILE__), "help/chipsfuns_hlui.hlp");
if (NULL != stat_file ($1))
  add_doc_file ($1);
provide ("chips_hlui");
