// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "VtkAxis.hh"
#include "vtkProperty2D.h"
#include "vtkTextProperty.h"
#include "VtkAxisLabel.hh"
#include "vtkAxisActor2DExtra.hh"
#include "vtkTextActorExtra.hh"
#include <sstream>

#include "ChipsCmdManager.hh"
#include "ChipsAxisAlgorithm.hh"
#include "ChipsDataSystem.hh"

#ifndef HEADLESS
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkChipsPickableWidget.hh"
#include "vtkAxisRepresentation.hh"
#endif

#include "VtkUtils.hh"

extern ChipsCmdManager* cmd_manager;

VtkAxis::VtkAxis(vtkRenderer* inRenderer,ChipsCoordinate *inCoord,
   COORD_SYS coord_sys, double inMinXPos, double inMinYPos,
   double inMaxXPos,double inMaxYPos,int inOrientation,
   ChipsAxisAlgorithmFactory::TransformType type,
   ChipsObject* par, const string& val):ChipsAxis(par,inCoord,coord_sys,inMinXPos,inMinYPos,inMaxXPos,inMaxYPos,inOrientation,type,val)
{
  renderer=inRenderer;

  axis = vtkAxisActor2DExtra::New();
  axis->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
  axis->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
  axis->SetClipData (coord_system == DATA);

#ifndef HEADLESS
  ChipsId cid;
  vtkAxisRepresentation *axis_rep = vtkAxisRepresentation::New ();
  axis_rep->SetAxisActor (axis);

  this->populate_chips_id (cid);
  axis_widget = vtkChipsPickableWidget::New ();
  axis_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  axis_widget->SetRepresentation (axis_rep);
  axis_widget->SetIdInfo (cid, this->get_object_type());
  axis_widget->ResizableOff ();
  axis_widget->RotateableOff ();
  axis_widget->SetDefaultRenderer (renderer);
  axis_widget->EnabledOn ();
  axis_rep->Delete ();
#endif

  if (title_flip) axis->SetTitleFlip(1);

  label = new VtkAxisLabel(axis,renderer,this,static_cast<double>(offset_par),static_cast<double>(offset_perp), "");

  // the label will add an error for not having a coordinate system
  // remove this error
  cmd_manager->remove_last_err ();

  label->set_fontstyle(label_fontstyle);
  label->set_size(label_size);

  label->set_font(label_font);
  label->set_color(label_color);
  label->set_vertalignment(label_vertalignment);
  label->set_horizalignment(label_horizalignment);
  label->set_angle(label_angle);


  change_color();
  change_alignment();
  change_corner();
  change_position();
  change_range();
  change_reverse();
  change_scale();
  change_visible();
  change_thickness();
  change_tick_format();
  change_ticklabel_style();
  change_ticklabel_color();
  change_ticklabel_font();
  change_ticklabel_size();
  change_ticklabel_angle();
  change_ticklabel_offset();
  change_ticklabel_alignment();

  change_major_color();
  change_major_interval();
  change_major_length();
  change_major_mode();
  change_major_num();
  change_major_style();
  change_major_thickness();

  change_minor_color();
  change_minor_interval();
  change_minor_length();
  change_minor_mode();
  change_minor_num();
  change_minor_style();
  change_minor_thickness();
  change_parallel_offset();
  change_perpendicular_offset();


  renderer->AddActor2D(axis);

};




VtkAxis::VtkAxis(vtkRenderer* inRenderer,ChipsCoordinate *inCoord,
   COORD_SYS coord_sys, double inMinXPos, double inMinYPos, 
   double inMaxXPos,double inMaxYPos,int inOrientation,
   double inMin,double inMax, ChipsAxisAlgorithmFactory::TransformType type,
   ChipsObject* par, const string& val):ChipsAxis(par,inCoord,coord_sys,inMinXPos,inMinYPos,inMaxXPos,inMaxYPos,inOrientation,inMin,inMax,type,val)
{
  renderer=inRenderer; 

  axis = vtkAxisActor2DExtra::New();
  axis->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
  axis->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
  axis->SetClipData (coord_system == DATA);

#ifndef HEADLESS
  ChipsId cid;
  vtkAxisRepresentation *axis_rep = vtkAxisRepresentation::New ();
  axis_rep->SetAxisActor (axis);

  this->populate_chips_id (cid);
  axis_widget = vtkChipsPickableWidget::New ();
  axis_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  axis_widget->SetRepresentation (axis_rep);
  axis_widget->SetIdInfo (cid, this->get_object_type());
  axis_widget->ResizableOff ();
  axis_widget->RotateableOff ();
  axis_widget->SetDefaultRenderer (renderer);
  axis_widget->EnabledOn ();
  axis_rep->Delete ();
#endif

  if (title_flip) axis->SetTitleFlip(1);

  label = new VtkAxisLabel(axis,renderer,this,static_cast<double>(offset_par),static_cast<double>(offset_perp), "");


  // the label will add an error for not having a coordinate system
  // remove this error
  cmd_manager->remove_last_err (); 
  
  label->set_fontstyle(label_fontstyle);
  label->set_size(label_size);
  label->set_font(label_font);
  label->set_color(label_color);
  label->set_vertalignment(label_vertalignment);
  label->set_horizalignment(label_horizalignment);
  label->set_angle(label_angle);


  change_color();
  change_alignment();
  change_corner();
  change_position();
  change_range();
  change_reverse();
  change_scale();
  change_visible();
  change_thickness();
  change_tick_format();
  change_ticklabel_style();
  change_ticklabel_color();
  change_ticklabel_font();
  change_ticklabel_size();
  change_ticklabel_angle();
  change_ticklabel_offset();
  change_ticklabel_alignment();

  change_major_color();
  change_major_interval();
  change_major_length();
  change_major_mode();
  change_major_num();
  change_major_style();
  change_major_thickness();
  
  change_minor_color();
  change_minor_interval();
  change_minor_length();
  change_minor_mode();
  change_minor_num();
  change_minor_style();
  change_minor_thickness();
  change_parallel_offset();
  change_perpendicular_offset();


  renderer->AddActor2D(axis);

};

VtkAxis::VtkAxis(vtkRenderer* inRenderer, ChipsObject* par,ChipsCoordinate 
   *inCoord, COORD_SYS coord_sys, ChipsStateStream& in):ChipsAxis(par, 
   inCoord, coord_sys, in)
{
  stringstream tmpid;
  tmpid << ref_cnt;
  ref_id = tmpid.str();

  renderer=inRenderer;
  axis=0;

  axis = vtkAxisActor2DExtra::New();
  axis->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
  axis->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
  axis->SetClipData (coord_system == DATA);

#ifndef HEADLESS
  ChipsId cid;
  vtkAxisRepresentation *axis_rep = vtkAxisRepresentation::New ();
  axis_rep->SetAxisActor (axis);

  this->populate_chips_id (cid);
  axis_widget = vtkChipsPickableWidget::New ();
  axis_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  axis_widget->SetRepresentation (axis_rep);
  axis_widget->SetIdInfo (cid, this->get_object_type());
  axis_widget->ResizableOff ();
  axis_widget->RotateableOff ();
  axis_widget->SetDefaultRenderer (renderer);
  axis_widget->EnabledOn ();
  axis_rep->Delete ();
#endif

  if (title_flip) axis->SetTitleFlip(1);

  if (deserialize_label)
    {
      label = new VtkAxisLabel(axis,renderer,this,in);
      // the label will add an error for not having a coordinate system
      // remove this error
      cmd_manager->remove_last_err ();
    }
  else
    {
      label = new VtkAxisLabel(axis,renderer,(ChipsObject*)this,
         static_cast<double>(offset_par),static_cast<double>(offset_perp),"");

      // the label will add an error for not having a coordinate system
      // remove this error
      cmd_manager->remove_last_err ();

      label->set_fontstyle(label_fontstyle);
      label->set_size(label_size);
      label->set_font(label_font);
      label->set_color(label_color);
      label->set_vertalignment(label_vertalignment);
      label->set_horizalignment(label_horizalignment);
      label->set_angle(label_angle);
    }
  if (label)   set_label_visible(labvis);

  change_color();
  change_alignment();
  change_corner();
  change_position();
  change_range();
  change_reverse();
  change_scale();
  change_visible();
  change_thickness();
  change_tick_format();
  change_ticklabel_style();
  change_ticklabel_color();
  change_ticklabel_font();
  change_ticklabel_size();
  change_ticklabel_angle();
  change_ticklabel_offset();
  change_ticklabel_alignment();

  change_major_color();
  change_major_interval();
  change_major_length();
  change_major_mode();
  change_major_num();
  change_major_style();
  change_major_thickness();

  change_minor_color();
  change_minor_interval();
  change_minor_length();
  change_minor_mode();
  change_minor_num();
  change_minor_style();
  change_minor_thickness();
  change_parallel_offset();
  change_perpendicular_offset();

  renderer->AddActor2D(axis);
};


VtkAxis::~VtkAxis()
{
  if ( label)
    label->object_delete ();

  if ( axis)
    {
      renderer->RemoveActor(axis);
      axis->Delete();
    }
};



int VtkAxis::change_color()
{

  if ( axis)
    {
      axis->GetProperty()->SetColor(color.red(),color.green(),color.blue());
    }
  else
    return ERR_STATUS_BAD;


  return ERR_STATUS_GOOD;
};


int VtkAxis::change_position()
{

  if ( axis)
    {
      double tmp_pnt1[] = {norm_pos[0][0], norm_pos[0][1]};
      double tmp_pnt2[] = {norm_pos[1][0], norm_pos[1][1]};
      VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_pnt1);
      VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_pnt2);
      axis->SetPoint1(tmp_pnt1[0], tmp_pnt1[1]);
      axis->SetPoint2(tmp_pnt2[0], tmp_pnt2[1]);
#ifndef HEADLESS
      axis_widget->UpdateRepresentation ();
#endif

    }

  return ERR_STATUS_GOOD;
};


int VtkAxis::change_range()
{
  unsigned int tickcnt = axis_alg->get_tick_count ();
  double *tickpos = axis_alg->get_tick_positions (false);
  unsigned int min_tickcnt = axis_alg->get_minortick_count ();
  double *min_tickpos = axis_alg->get_minortick_positions ();
  

  vector<double> tmpMajLoc(tickcnt);
  vector<double> tmpMinLoc;
  vector<bool> tmpLab(tickcnt);
  vector<vtkTextActorFormat> tmpFormats;


  if ( tick_text)
    tmpFormats.resize(tickcnt);

  if (min_tickpos)
  {
    tmpMinLoc.resize (min_tickcnt);
    for(unsigned int indx=0;indx<min_tickcnt;indx++)
      tmpMinLoc[indx]=min_tickpos[indx];
  }


  for(unsigned int indx=0;indx<tickcnt;indx++)
    {
      tmpMajLoc[indx]=tickpos[indx];
      tmpLab[indx]=true;
      if ( tick_text )
	CopyFormats(&tmpFormats[indx],tick_text[indx]);
    };

  if ( axis)
    {

      int index = (axis_alg->get_orientation () == RA_AXIS) ? 0 : 1;
      int rangeIndex = (axis_alg->get_orientation () == RA_AXIS) ? 0 : 2;
      ChipsDataSystem *dsCoord = this->get_data_system ();
      ChipsVector min_coord (0,0);
      ChipsVector max_coord (1,1);

      if (dsCoord)
      {
         min_coord = dsCoord->get_lower_bounds ();
         max_coord = dsCoord->get_upper_bounds ();
      }
      min_coord[index] = axis_alg->get_min (false);
      max_coord[index] = axis_alg->get_max (false);


      if (reverse)
        {
          double tmp = min_coord[index];
          min_coord[index] = max_coord[index];
          max_coord[index] = tmp;
        }
      axis->SetRange(min_coord[0], max_coord[0], min_coord[1], max_coord[1], rangeIndex);
      axis->SetMajTicksLocations(tmpMajLoc,tmpLab,tmpFormats);

      if (min_tickpos)
         axis->SetMinTicksLocations(tmpMinLoc);
    }

  return ERR_STATUS_GOOD;

};


int VtkAxis::change_reverse()
{
  if ( axis)
    {
      int index = (axis_alg->get_orientation () == RA_AXIS) ? 0 : 1;
      int rangeIndex = (axis_alg->get_orientation () == RA_AXIS) ? 0 : 2;
      ChipsDataSystem *dsCoord = this->get_data_system ();
      ChipsVector min_coord (0,0);
      ChipsVector max_coord (1,1);

      if (dsCoord)
      {
         min_coord = dsCoord->get_lower_bounds ();
         max_coord = dsCoord->get_upper_bounds ();
      }
      min_coord[index] = axis_alg->get_min (false);
      max_coord[index] = axis_alg->get_max (false);

      if (reverse)
        {
          double tmp = min_coord[index];
          min_coord[index] = max_coord[index];
          max_coord[index] = tmp;
        }
      axis->SetRange(min_coord[0], max_coord[0], min_coord[1], max_coord[1], rangeIndex);
    }

  return ERR_STATUS_GOOD;

};


int VtkAxis::change_scale()
{
  if ( axis)
    {
      int tmpScale;
      switch(axis_alg->get_scaletype ())
        {
        case V_SCALE_LOG:
          tmpScale=1;
          break;
        case V_SCALE_LINEAR:
        default:
          tmpScale=0;
        };

      if (axis_alg->get_orientation () == X_AXIS)
         axis->SetScale(tmpScale, 0);
      else
         axis->SetScale(0, tmpScale);
    }

  return ERR_STATUS_GOOD;


};

int VtkAxis::change_depth()
{
  if (axis)
     axis->SetLayerNumber(depth*10+1);
  return ERR_STATUS_GOOD;
}



int VtkAxis::change_thickness()
{

  if ( axis)
    {
      axis->GetProperty()->SetLineWidth(thickness);
      change_style();
      //      tmpAxis->GetMajTicks()->GetProperty()->SetLineWidth(major_thickness);
      //      tmpAxis->GetMinTicks()->GetProperty()->SetLineWidth(minor_thickness);
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;
  
};


int VtkAxis::change_tick_format()
{
  unsigned int tickcnt = axis_alg->get_tick_count ();
  double *tickpos = axis_alg->get_tick_positions (false);
  unsigned int min_tickcnt = axis_alg->get_minortick_count ();
  double *min_tickpos = axis_alg->get_minortick_positions ();

  vector<double> tmpMajLoc(tickcnt);
  vector<double> tmpMinLoc;
  vector<bool> tmpLab(tickcnt);
  vector<vtkTextActorFormat> tmpFormats;


  if ( tick_text)
    tmpFormats.resize(tickcnt);

  if (min_tickpos)
  {
    tmpMinLoc.resize (min_tickcnt);
    for(unsigned int indx=0;indx<min_tickcnt;indx++)
      tmpMinLoc[indx]=min_tickpos[indx];
  }

  for(unsigned int indx=0;indx<tickcnt;indx++)
    {
      tmpMajLoc[indx]=tickpos[indx];
      tmpLab[indx]=true;
      if ( tick_text )
	CopyFormats(&tmpFormats[indx],tick_text[indx]);
    };
  

  if ( axis)
    {
      axis->SetMajTicksLocations(tmpMajLoc,tmpLab,tmpFormats);
      axis->SetLabelFormat(tickformat.c_str());
      if (min_tickpos)
         axis->SetMinTicksLocations(tmpMinLoc);
    }
  
  return ERR_STATUS_GOOD;

};

int VtkAxis::change_ticklabel_style()
{


  if ( axis)
  {
     axis->SetLabelsFlip(
        ((title_flip && (ticklabstyle == chips_outside)) ||
         (!title_flip && (ticklabstyle == chips_inside))));
  }
  else
     return ERR_STATUS_BAD;

  return change_ticklabel_alignment();

};


int VtkAxis::change_ticklabel_color()
{
  unsigned int tickcnt = axis_alg->get_tick_count ();
  double *tickpos = axis_alg->get_tick_positions (false);

  vector<double> tmpMajLoc(tickcnt);
  vector<bool> tmpLab(tickcnt);
  vector<vtkTextActorFormat> tmpFormats;
  
  
  if ( tick_text)
    tmpFormats.resize(tickcnt);

  
  for(unsigned int indx=0;indx<tickcnt;indx++)
    {
      tmpMajLoc[indx]=tickpos[indx];
      tmpLab[indx]=true;
      if ( tick_text )
	CopyFormats(&tmpFormats[indx],tick_text[indx]);
    };

  
  if ( axis)
    {
      axis->SetMajTicksLocations(tmpMajLoc,tmpLab,tmpFormats);
      axis->GetLabelTextProperty()->SetColor(ticklabcolor.red(),ticklabcolor.green(),ticklabcolor.blue());
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;
  
};

int VtkAxis::change_ticklabel_font()
{  
  unsigned int tickcnt = axis_alg->get_tick_count ();
  double *tickpos = axis_alg->get_tick_positions (false);

  vector<double> tmpMajLoc(tickcnt);
  vector<bool> tmpLab(tickcnt);
  vector<vtkTextActorFormat> tmpFormats;
  
  
  if ( tick_text)
    tmpFormats.resize(tickcnt);
  
  
  for(unsigned int indx=0;indx<tickcnt;indx++)
    {
      tmpMajLoc[indx]=tickpos[indx];
      tmpLab[indx]=true;
      if ( tick_text )
	CopyFormats(&tmpFormats[indx],tick_text[indx]);
    };

  if ( axis)
    {
      axis->SetMajTicksLocations(tmpMajLoc,tmpLab,tmpFormats);

      

      axis->GetLabelTextProperty()->SetFaceFileName(0);
      
      if ( ticklabfontfile == "DEFAULT_HELVETICA" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToHelvetica();
	  axis->GetLabelTextProperty()->BoldOff();
	  axis->GetLabelTextProperty()->ItalicOff();
	}
      else if ( ticklabfontfile == "DEFAULT_HELVETICA_BOLD" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToHelvetica();
	  axis->GetLabelTextProperty()->BoldOn();
	  axis->GetLabelTextProperty()->ItalicOff();
	}
      else if ( ticklabfontfile == "DEFAULT_HELVETICA_ITALIC" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToHelvetica();
	  axis->GetLabelTextProperty()->BoldOff();
	  axis->GetLabelTextProperty()->ItalicOn();
	}
      else if ( ticklabfontfile == "DEFAULT_HELVETICA_BOLDITALIC" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToHelvetica();
	  axis->GetLabelTextProperty()->BoldOn();
	  axis->GetLabelTextProperty()->ItalicOn();
	}
      else if ( ticklabfontfile == "DEFAULT_COURIER" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToCourier();
	  axis->GetLabelTextProperty()->BoldOff();
	  axis->GetLabelTextProperty()->ItalicOff();
	}
      else if ( ticklabfontfile == "DEFAULT_COURIER_BOLD" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToCourier();
	  axis->GetLabelTextProperty()->BoldOn();
	  axis->GetLabelTextProperty()->ItalicOff();
	}
      else if ( ticklabfontfile == "DEFAULT_COURIER_ITALIC" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToCourier();
	  axis->GetLabelTextProperty()->BoldOff();
	  axis->GetLabelTextProperty()->ItalicOn();
	}
      else if ( ticklabfontfile == "DEFAULT_COURIER_BOLDITALIC" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToCourier();
	  axis->GetLabelTextProperty()->BoldOn();
	  axis->GetLabelTextProperty()->ItalicOn();
	}
      else if ( ticklabfontfile == "DEFAULT_TIMES" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToTimes();
	  axis->GetLabelTextProperty()->BoldOff();
	  axis->GetLabelTextProperty()->ItalicOff();
	}
      else if ( ticklabfontfile == "DEFAULT_TIMES_BOLD" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToTimes();
	  axis->GetLabelTextProperty()->BoldOn();
	  axis->GetLabelTextProperty()->ItalicOff();
	}
       else if ( ticklabfontfile == "DEFAULT_TIMES_ITALIC" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToTimes();
	  axis->GetLabelTextProperty()->BoldOff();
	  axis->GetLabelTextProperty()->ItalicOn();
	}
      else if ( ticklabfontfile == "DEFAULT_TIMES_BOLDITALIC" )
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToTimes();
	  axis->GetLabelTextProperty()->BoldOn();
	  axis->GetLabelTextProperty()->ItalicOn();
	}
      else if( ( ticklabfontfile == "DEFAULT_GREEK" ) 
	       ||(ticklabfontfile == "DEFAULT_GREEK_BOLD")
	       ||(ticklabfontfile == "DEFAULT_GREEK_ITALIC")
	       ||(ticklabfontfile == "DEFAULT_GREEK_BOLDITALIC"))
	{
	  axis->GetLabelTextProperty()->SetFontFamilyToGreek();
	  axis->GetLabelTextProperty()->BoldOff();
	  axis->GetLabelTextProperty()->ItalicOff();
	}
      else 
	{
	  axis->GetLabelTextProperty()->SetFaceFileName(ticklabfontfile.c_str());
	}

      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;
};

int VtkAxis::change_scale_factor()
{
   int rval;
   if ((rval = change_ticklabel_offset()) == ERR_STATUS_GOOD)
      rval = change_ticklabel_size();
   if (rval == ERR_STATUS_GOOD)
      rval = change_perpendicular_offset ();
   if (!rval && label)
      rval = label->change_scale_factor ();

   return rval; 
};

int VtkAxis::change_ticklabel_size()
{
  unsigned int tickcnt = axis_alg->get_tick_count ();
  double *tickpos = axis_alg->get_tick_positions (false);
  
  vector<double> tmpMajLoc(tickcnt);
  vector<bool> tmpLab(tickcnt);
  vector<vtkTextActorFormat> tmpFormats;
  
  
  if ( tick_text)
    tmpFormats.resize(tickcnt);
  
  
  for(unsigned int indx=0;indx<tickcnt;indx++)
    {
      tmpMajLoc[indx]=tickpos[indx];
      tmpLab[indx]=true;
      if ( tick_text )
	CopyFormats(&tmpFormats[indx],tick_text[indx]);
    };

  
  if ( axis)
    {
      axis->SetMajTicksLocations(tmpMajLoc,tmpLab,tmpFormats);
      axis->GetLabelTextProperty()->SetFontSize((int)(ticklabsize*get_scale_factor()));
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;
  

};

int VtkAxis::change_ticklabel_angle()
{

  if ( axis)
    {
      axis->GetLabelTextProperty()->SetOrientation(ticklabangle);
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;
  

};

int VtkAxis::change_ticklabel_offset()
{
  
  if ( axis)
    {
      axis->SetLabelOffset((int)(ticklaboffset*get_scale_factor()));
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;


};


int VtkAxis::change_ticklabel_visible()
{
  int tmp_vis = (visible) ? ticklabvis : 0;
  
  if ( axis)
    {
      axis->SetLabelVisibility(static_cast<int>(tmp_vis));
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;


}


int VtkAxis::change_ticklabel_alignment()
{

  if ( axis)
    {
      axis->SetMajTickLabelVertAlignmentOffset(use_valign);
      axis->SetMajTickLabelHorizAlignmentOffset(use_halign);
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;

  
  return ERR_STATUS_GOOD;


}


int VtkAxis::change_major_color()
{

  if ( axis)
    {
      
      axis->GetMajTicks()->GetProperty()->SetColor(major_color.red(),major_color.green(),major_color.blue());
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;
  
};


int VtkAxis::change_major_interval()
{

  return ERR_FEATURE_DNE;

};


int VtkAxis::change_major_length()
{

  if ( axis)
    {
      
      axis->SetMajTicksLength(major_len);
      axis->Modified();

      return change_style();
      
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;

};

int VtkAxis::change_major_mode()
{
  unsigned int tickcnt = axis_alg->get_tick_count ();
  double *tickpos = axis_alg->get_tick_positions (false);

  if ( axis)
    {
      unsigned int min_tickcnt = axis_alg->get_minortick_count ();
      double *min_tickpos = axis_alg->get_minortick_positions ();


      vector<double> tmpMajLoc(tickcnt);
      vector<double> tmpMinLoc;
      vector<bool> tmpLab(tickcnt);
      vector<vtkTextActorFormat> tmpFormats;


      if ( tick_text)
        tmpFormats.resize(tickcnt);

      if (min_tickpos)
      {
        tmpMinLoc.resize (min_tickcnt);
        for(unsigned int indx=0;indx<min_tickcnt;indx++)
          tmpMinLoc[indx]=min_tickpos[indx];
      }

      for(unsigned int indx=0;indx<tickcnt;indx++)
	{
	  tmpMajLoc[indx]=tickpos[indx];
	  tmpLab[indx]=true;
	  if ( tick_text )
	    CopyFormats(&tmpFormats[indx],tick_text[indx]);
	};
      
      axis->SetMajTicksLocations(tmpMajLoc,tmpLab,tmpFormats);
      axis->SetMajTicksMode(2);
     
      if (min_tickpos)
         axis->SetMinTicksLocations (tmpMinLoc);
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;
 
};


int VtkAxis::change_major_num()
{
  if ( axis)
    {
      axis->SetNumberOfMajTicks(axis_alg->get_major_num ());
    }
  else
      return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

};


int VtkAxis::change_major_style()
{
  return change_style();

};


int VtkAxis::change_major_thickness()
{
  
  if ( axis)
    {
      axis->GetMajTicks()->GetProperty()->SetLineWidth(major_thickness);
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

};

int VtkAxis::change_major_visible()
{
  int tmp_vis = (visible) ? major_vis : 0;
  
  if ( axis)
    {
      axis->SetMajTicksVisibility(tmp_vis);
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;



};

int VtkAxis::change_minor_color()
{

  if ( axis)
    {
      
      axis->GetMinTicks()->GetProperty()->SetColor(minor_color.red(),minor_color.green(),minor_color.blue());
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;


};

int VtkAxis::change_minor_interval()
{

  if ( axis)
    {
      
      axis->SetMinTicksInterval(axis_alg->get_minor_interval ());
      axis->Modified();

    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;


};

int VtkAxis::change_minor_length()
{

  if ( axis)
    {
      
      axis->SetMinTicksLength(minor_len);

      return change_style();

    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;


};

int VtkAxis::change_minor_mode()
{

 
  int tmpMode;

  switch(axis_alg->get_minor_mode (true))
    {
    case chips_count:
      tmpMode=0;
      break;
    case chips_interval:
      tmpMode=1;
      break;
    case chips_arbitrary:
      {

        unsigned int min_tickcnt = axis_alg->get_minortick_count ();
        double *min_tickpos = axis_alg->get_minortick_positions ();

        vector<double> tmpMinLoc;


        if (min_tickpos && axis)
        {
          tmpMinLoc.resize (min_tickcnt);
          for(unsigned int indx=0;indx<min_tickcnt;indx++)
            tmpMinLoc[indx]=min_tickpos[indx];

          axis->SetMinTicksLocations (tmpMinLoc);
        }

        tmpMode=2;
      }
      break;
    case chips_nice:
    default:
      tmpMode=3;
    };

  if ( axis)
    {
      
      axis->SetMinTicksMode(tmpMode);
      axis->Modified();      
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;
 

};

int VtkAxis::change_minor_num()
{

  if ( axis)
    {
      axis->SetNumberOfMinTicks(axis_alg->get_minor_num (true));
      axis->Modified();
    }
  
  return ERR_STATUS_GOOD;



};

int VtkAxis::change_minor_style()
{
  return change_style();

};

int VtkAxis::change_minor_thickness()
{
  
  if ( axis)
    {
      axis->GetMinTicks()->GetProperty()->SetLineWidth(minor_thickness);
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;


};

int VtkAxis::change_minor_visible()
{
  int tmp_vis = (is_visible ()) ? minor_vis : 0;
  
  if ( axis)
    {
      axis->SetMinTicksVisibility(tmp_vis);
      axis->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

};







int VtkAxis::change_parallel_offset()
{
  
  if ( axis)
    {
      axis->SetTitleParallelOffset(offset_par); 
      axis->Modified();
      axis->GetTitleActor()->Modified();
     
      VtkAxisLabel *text = dynamic_cast<VtkAxisLabel *>(this->label);
      if (text)
         text->modify ();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

  

};



int VtkAxis::change_perpendicular_offset()
{
  if ( axis)
    {
      float scaled_offset = offset_perp * get_scale_factor();
      axis->SetTitlePerpendicularOffset(scaled_offset);
      axis->Modified();
      axis->GetTitleActor()->Modified();

      VtkAxisLabel *text = dynamic_cast<VtkAxisLabel *>(this->label);
      if (text)
         text->modify ();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

};


int VtkAxis::change_log_minimum()
{
  double log_minimum = get_log_minimum ();
  if ( log_minimum<=0 )
    return ERR_STATUS_BAD;

  if ( axis)
    {
      axis->SetMinLogLimit(log_minimum);
    }

  return ERR_STATUS_GOOD;

};


int VtkAxis::change_title_flip()
{
  if ( axis)
    {
      axis->SetTitleFlip(title_flip);
      axis->Modified();
      axis->GetTitleActor()->Modified();
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

};

int VtkAxis::change_title_visible()
{
  int tmp_vis = (is_visible ()) ? labvis : 0;
  
  if ( axis)
    {
    this->label->change_visible ();
    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;


}



int VtkAxis::change_style()
{
  float tmpOffset;

  if ( axis)
    {

      switch(use_major_tickstyle)
	{
	  
	case chips_inside:   
	  axis->SetMajorTicksFlip(1); 
	  axis->SetMajTickOffset(0);
	  break;
	case chips_outside:   
	  axis->SetMajorTicksFlip(0); 
	  axis->SetMajTickOffset(0);
	  break;
	case chips_centered:
	  {
	    axis->SetMajorTicksFlip(0); 
	    tmpOffset=major_len+thickness;
	    axis->SetMajTickOffset(static_cast <int> (-tmpOffset/2));
	  }
	break;
	default:
	  return ERR_STATUS_BAD;
	  
	};

      switch(use_minor_tickstyle)
	{
	  
	case chips_inside:   
	  axis->SetMinorTicksFlip(1); 
	  axis->SetMinTickOffset(0);
	  break;
	case chips_outside:   
	  axis->SetMinorTicksFlip(0); 
	  axis->SetMinTickOffset(0);
	  break;
	case chips_centered:
	  {
	    axis->SetMinorTicksFlip(0); 
	    tmpOffset=minor_len+thickness;
	    axis->SetMinTickOffset(static_cast<int>(-tmpOffset/2));
	  }
	break;
	default:
	  return ERR_STATUS_BAD;
	  
	  
	};
      axis->Modified();      

    }
  else
    return ERR_STATUS_BAD;

  return ERR_STATUS_GOOD;
}; 

  
int VtkAxis::change_visible()
{
  int tmp_vis = (!get_parent ()->is_visible ()) ? 0 : this->is_visible ();

  if ( axis)
    {
      axis->SetVisibility(tmp_vis);
      change_title_visible();
      if (tmp_vis)
        {
          change_ticklabel_visible(); 
          change_major_visible();
          change_minor_visible();
        }
#ifndef HEADLESS
      axis_widget->GetRepresentation ()->SetVisibility (tmp_vis);
#endif
    }
  else
    return ERR_STATUS_BAD;


  return ERR_STATUS_GOOD;
};

int VtkAxis::change_grid_property()
{
  
  if ( axis)
    {
      double tmp_pnt1[] = {norm_pos[0][0], norm_pos[0][1]};
      double tmp_pnt2[] = {norm_pos[1][0], norm_pos[1][1]};
      VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_pnt1);
      VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_pnt2);

      if (axis_alg->get_orientation () == X_AXIS)
         axis->SetGridPositions (tmp_pnt1[0], grid_start_pos, tmp_pnt2[0], grid_stop_pos);
      else
         axis->SetGridPositions (grid_start_pos, tmp_pnt1[1], grid_stop_pos, tmp_pnt2[1]);
     
      axis->SetMinGridVisibility(grid_min_vis);
      axis->SetMajGridVisibility(grid_maj_vis);
      axis->GetMinGrid()->GetProperty()->SetLineWidth(grid_min_thick);
      axis->GetMajGrid()->GetProperty()->SetLineWidth(grid_maj_thick);
      axis->GetMinGrid()->GetProperty()->SetColor(grid_min_clr.red(),grid_min_clr.green(),grid_min_clr.blue());

      axis->GetMajGrid()->GetProperty()->SetColor(grid_maj_clr.red(),grid_maj_clr.green(),grid_maj_clr.blue());
      
      switch(grid_min_style)
	{
	case chips_noline:
	  axis->SetMinGridVisibility(0);
	  break;
	case chips_dotlongdash: 
	  axis->GetMinGrid()->GetProperty()->SetLineStipplePattern(chips_dotlongdash_pattern);
	  break;
	case chips_dotshortdash: 
	  axis->GetMinGrid()->GetProperty()->SetLineStipplePattern(chips_dotshortdash_pattern);
	  break;
	case chips_longdash: 
	  axis->GetMinGrid()->GetProperty()->SetLineStipplePattern(chips_longdash_pattern);
	  break;
	case chips_shortdash:
	  axis->GetMinGrid()->GetProperty()->SetLineStipplePattern(chips_shortdash_pattern);
	  break;
	case chips_shortdashlongdash:
	  axis->GetMinGrid()->GetProperty()->SetLineStipplePattern(chips_shortdashlongdash_pattern);
	  break;
	case chips_dot:
	  axis->GetMinGrid()->GetProperty()->SetLineStipplePattern(chips_dot_pattern);
	   break;
	case chips_solid:
	default:
	  axis->GetMinGrid()->GetProperty()->SetLineStipplePattern(chips_solid_pattern);
	  
	};
      
      
      switch(grid_maj_style)
	{
	case chips_noline:
	  axis->SetMajGridVisibility(0);
	  break;
	case chips_dotlongdash: 
	  axis->GetMajGrid()->GetProperty()->SetLineStipplePattern(chips_dotlongdash_pattern);
	  break;
	case chips_dotshortdash: 
	  axis->GetMajGrid()->GetProperty()->SetLineStipplePattern(chips_dotshortdash_pattern);
	  break;
	case chips_longdash: 
	  axis->GetMajGrid()->GetProperty()->SetLineStipplePattern(chips_longdash_pattern);
	  break;
	case chips_shortdash:
	  axis->GetMajGrid()->GetProperty()->SetLineStipplePattern(chips_shortdash_pattern);
	  break;
	case chips_shortdashlongdash:
	  axis->GetMajGrid()->GetProperty()->SetLineStipplePattern(chips_shortdashlongdash_pattern);
	  break;
	case chips_dot:
	  axis->GetMajGrid()->GetProperty()->SetLineStipplePattern(chips_dot_pattern);
	  break;
	case chips_solid:
	default:
	  axis->GetMajGrid()->GetProperty()->SetLineStipplePattern(chips_solid_pattern);
	  
	};
      
      
    };
  
  
  return ERR_STATUS_GOOD;

}


int VtkAxis::change_corner()
{
  if ( axis)
    {
      axis->SetFrontFillerLength(ll_corner_width);
      if (ll_corner == chips_fill)
      {
         axis->SetFrontFillerStyle(1);
	 if (alignment == chips_inside)
            axis->SetFrontFillerAlignment(2);
	 else if (alignment == chips_outside) 
            axis->SetFrontFillerAlignment(0);
	 else
            axis->SetFrontFillerAlignment(1);
      }
      else if (ll_corner == chips_notch)
      {
	 if (alignment == chips_outside)
	 {
            axis->SetFrontFillerStyle(1);
            axis->SetFrontFillerLength(ll_corner_width/2.0);
	 }
         else 
            axis->SetFrontFillerStyle(0);
      }
      else if (ll_corner == chips_bevel)
      {
         axis->SetFrontFillerStyle(4);
         axis->SetFrontFillerLength(ll_corner_width/2.0);
	 if (alignment == chips_outside)
            axis->SetFrontFillerAlignment(0);
         else 
            axis->SetFrontFillerAlignment(2);
      }
      else if (title_flip)
      {
         axis->SetFrontFillerStyle(2);
	 if (alignment == chips_inside)
            axis->SetFrontFillerAlignment(2);
	 else if (alignment == chips_outside) 
            axis->SetFrontFillerAlignment(0);
	 else
            axis->SetFrontFillerAlignment(1);
      }
      else 
      {
         axis->SetFrontFillerStyle(3);
	 if (alignment == chips_inside)
            axis->SetFrontFillerAlignment(2);
	 else if (alignment == chips_outside) 
            axis->SetFrontFillerAlignment(0);
	 else
            axis->SetFrontFillerAlignment(1);
      }
      axis->SetBackFillerLength(ur_corner_width);
      if (ur_corner == chips_fill)
      {
         axis->SetBackFillerStyle(1);
	 if (alignment == chips_inside)
            axis->SetBackFillerAlignment(2);
	 else if (alignment == chips_outside) 
            axis->SetBackFillerAlignment(0);
	 else
            axis->SetBackFillerAlignment(1);
      }
      else if (ur_corner == chips_notch)
	 if (alignment == chips_outside)
	 {
            axis->SetBackFillerStyle(1);
            axis->SetBackFillerLength(ur_corner_width/2.0);
	 }
         else 
            axis->SetBackFillerStyle(0);
      else if (ur_corner == chips_bevel)
      {
         axis->SetBackFillerStyle(4);
         axis->SetBackFillerLength(ur_corner_width/2.0);
	 if (alignment == chips_outside)
            axis->SetBackFillerAlignment(0);
         else 
            axis->SetBackFillerAlignment(2);
      }
      else if (title_flip)
      {
         axis->SetBackFillerStyle(2);
	 if (alignment == chips_inside)
            axis->SetBackFillerAlignment(2);
	 else if (alignment == chips_outside) 
            axis->SetBackFillerAlignment(0);
	 else
            axis->SetBackFillerAlignment(1);
      }
      else 
      {
         axis->SetBackFillerStyle(3);
	 if (alignment == chips_inside)
            axis->SetBackFillerAlignment(2);
	 else if (alignment == chips_outside) 
            axis->SetBackFillerAlignment(0);
	 else
            axis->SetBackFillerAlignment(1);
      }
      axis->Modified();
    }
  return ERR_STATUS_GOOD;
}


int VtkAxis::change_alignment()
{
  if ( axis)
    {
      switch(alignment)
	{
	case chips_inside:
         if (!title_flip)
            axis->SetAlignment(0);
	  else
            axis->SetAlignment(2);
          axis->SetFrontFillerAlignment(2);
          axis->SetBackFillerAlignment(2);
	  break;
	case chips_outside:
         if (!title_flip)
	    axis->SetAlignment(2);
	  else
            axis->SetAlignment(0);
          axis->SetFrontFillerAlignment(0);
          axis->SetBackFillerAlignment(0);
	  break;
        case chips_centered:
          axis->SetAlignment(1);
          axis->SetFrontFillerAlignment(1);
          axis->SetBackFillerAlignment(1);
        break;
        default:
          return ERR_STATUS_BAD;
        };
      axis->Modified();
    }
  return ERR_STATUS_GOOD;
}


void VtkAxis::CopyFormats(vtkTextActorFormat* outFormat, ELEM_P_T inFormat)
{
  
  if ((! inFormat) || (!outFormat))
    return ;
  
  if ( outFormat->super )
    delete outFormat->super;
  if ( outFormat->next)
    delete outFormat->next;
  if (outFormat->sub)
    delete outFormat->sub; 
  

  outFormat->text = inFormat->text;


  outFormat->fontFile = "";
  outFormat->fontBold =0;
  outFormat->fontItalic =0;

  if ( inFormat->fontfile == "DEFAULT_HELVETICA" )
    {
      outFormat->fontFamily = VTK_HELVETICA;
    }
  else if ( inFormat->fontfile == "DEFAULT_HELVETICA_BOLD" )
    {
      outFormat->fontFamily = VTK_HELVETICA;
      outFormat->fontBold =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_HELVETICA_ITALIC" )
    {
      outFormat->fontFamily = VTK_HELVETICA;
      outFormat->fontItalic =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_HELVETICA_BOLDITALIC" )
    {
      outFormat->fontFamily = VTK_HELVETICA;
      outFormat->fontBold =1;
      outFormat->fontItalic =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_COURIER" )
    {
      outFormat->fontFamily = VTK_COURIER;
    }
  else if ( inFormat->fontfile == "DEFAULT_COURIER_BOLD" )
    {
      outFormat->fontFamily = VTK_COURIER;
      outFormat->fontBold =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_COURIER_ITALIC" )
    {
      outFormat->fontFamily = VTK_COURIER;
      outFormat->fontItalic =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_COURIER_BOLDITALIC" )
    {
      outFormat->fontFamily = VTK_COURIER; 
      outFormat->fontBold =1;
      outFormat->fontItalic =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_TIMES" )
    {
      outFormat->fontFamily = VTK_TIMES;
      
    }
  else if ( inFormat->fontfile == "DEFAULT_TIMES_BOLD" )
    {
      outFormat->fontFamily = VTK_TIMES;
      outFormat->fontBold =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_TIMES_ITALIC" )
    {
      outFormat->fontFamily = VTK_TIMES;
      outFormat->fontItalic =1;
    }
  else if ( inFormat->fontfile == "DEFAULT_TIMES_BOLDITALIC" )
    {
      outFormat->fontFamily = VTK_TIMES;
      outFormat->fontBold =1;
      outFormat->fontItalic =1;
    }
  else if( ( inFormat->fontfile == "DEFAULT_GREEK" ) 
	   ||(inFormat->fontfile == "DEFAULT_GREEK_BOLD")
	   ||(inFormat->fontfile == "DEFAULT_GREEK_ITALIC")
	   ||(inFormat->fontfile == "DEFAULT_GREEK_BOLDITALIC"))
    {
      outFormat->fontFamily = VTK_GREEK; 
    }
  else 
    {
      outFormat->fontFile=inFormat->fontfile;
    }


  outFormat->size = static_cast <int>(inFormat->size*get_scale_factor());
  outFormat->red= inFormat->color.red();
  outFormat->green= inFormat->color.green();
  outFormat->blue= inFormat->color.blue();
  
  if ( inFormat->super )
    {
      outFormat->super = new vtkTextActorFormat();
      CopyFormats(outFormat->super,inFormat->super);
    }
  else
    outFormat->super=0;

  if ( inFormat->next )
    {
      outFormat->next = new vtkTextActorFormat();
      CopyFormats(outFormat->next,inFormat->next);
    }
  else
    outFormat->next =0;

  if ( inFormat->sub )
    {
      outFormat->sub=new vtkTextActorFormat();
      CopyFormats(outFormat->sub,inFormat->sub);
    }
  else
    outFormat->sub=0;


};

int VtkAxis::set_plot_location (double x1, double y1, double x2, double y2)
{
   if (axis)  axis->SetClipLocations (x1,y1,x2,y2);

   ChipsAxis::set_plot_location (x1, y1, x2, y2);

   return ERR_STATUS_GOOD;
}

int VtkAxis::set_major_grid (double ***grid, int slices, int lines)
{
   axis->SetMajGridValues (grid, slices, lines, false);
   return ERR_STATUS_GOOD;
}

int VtkAxis::set_minor_grid (double ***grid, int slices, int lines)
{
   axis->SetMinGridValues (grid, slices, lines, false);
   return ERR_STATUS_GOOD;
}

void VtkAxis::select ()
{
#ifndef HEADLESS
  if (axis_widget)
    axis_widget->Select (true);
#endif
}

void VtkAxis::unselect ()
{
#ifndef HEADLESS
  if (axis_widget)
    axis_widget->Select (false);
#endif
}


void VtkAxis::disable_interaction ()
{
#ifndef HEADLESS
  vtkAxisRepresentation *rep = reinterpret_cast<vtkAxisRepresentation *> (this->axis_widget->GetRepresentation ());
  rep->DisableBorder ();
  this->axis_widget->EnabledOff ();
 
  this->label->disable_interaction ();
#endif
}

void VtkAxis::enable_interaction ()
{
#ifndef HEADLESS

  if (this->is_interaction_blocked ())
      return;

  vtkAxisRepresentation *rep = reinterpret_cast<vtkAxisRepresentation *> (this->axis_widget->GetRepresentation ());

  this->axis_widget->EnabledOn ();
  rep->EnableBorder ();

  this->label->enable_interaction ();
#endif
}

