// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKAXIS_HH
#define VTKAXIS_HH


#include "vtkRenderer.h"
#include "vtkActor.h"

#include "ChipsAxis.hh"

class vtkXYPlotActorExtra;
class vtkAxisActor2DExtra;
class vtkTextActorFormat;
#ifndef HEADLESS
class vtkChipsPickableWidget;
#endif

class VtkAxis: public ChipsAxis
{
public:

  VtkAxis(vtkRenderer* inRenderer,ChipsCoordinate *inCoord,
   COORD_SYS coord_sys, double inMinXPos, double inMinYPos,
   double inMaxXPos,double inMaxYPos,int inOrientation,
   ChipsAxisAlgorithmFactory::TransformType type,
   ChipsObject* par, const string& val);
  VtkAxis(vtkRenderer* inRenderer,ChipsCoordinate *inCoord, 
   COORD_SYS coord_sys, double inMinXPos, double inMinYPos,
   double inMaxXPos,double inMaxYPos,int inOrientation,
   double inMin,double inMax, ChipsAxisAlgorithmFactory::TransformType type,
   ChipsObject* par, const string& val);
  VtkAxis(vtkRenderer* inRenderer, ChipsObject* par, ChipsCoordinate *inCoord,
          COORD_SYS coord_sys, ChipsStateStream& in);
  virtual ~VtkAxis();


  virtual int change_color();
  virtual int change_position();
  virtual int change_range();
  virtual int change_reverse();
  virtual int change_scale();
  virtual int change_depth();
  virtual int change_thickness();
  virtual int change_scale_factor();
  virtual int change_tick_format();
  virtual int change_ticklabel_style();
  virtual int change_ticklabel_color();
  virtual int change_ticklabel_font();
  virtual int change_ticklabel_size();
  virtual int change_ticklabel_angle();
  virtual int change_ticklabel_offset();
  virtual int change_ticklabel_visible();
  virtual int change_ticklabel_alignment();

  virtual int change_visible();
  virtual int change_major_color();
  virtual int change_major_interval();
  virtual int change_major_length();
  virtual int change_major_mode();
  virtual int change_major_num();
  virtual int change_major_style();
  virtual int change_major_thickness();
  virtual int change_major_visible();
  
  virtual int change_minor_color();
  virtual int change_minor_interval();
  virtual int change_minor_length();
  virtual int change_minor_mode();
  virtual int change_minor_num();
  virtual int change_minor_style();
  virtual int change_minor_thickness();
  virtual int change_minor_visible();
  virtual int change_parallel_offset();
  virtual int change_perpendicular_offset();
  virtual int change_log_minimum();
  virtual int change_title_flip();
  virtual int change_title_visible();
  virtual int change_grid_property();

  virtual int change_corner(); 
  virtual int change_alignment(); 
  virtual int set_plot_location (double x1, double y1, double x2, double y2);

  virtual int set_major_grid (double ***grid, int slices, int lines);
  virtual int set_minor_grid (double ***grid, int slices, int lines);

  virtual void select ();
  virtual void unselect ();

  virtual void disable_interaction ();
  virtual void enable_interaction ();


/*  void update_plot_actor (vtkXYPlotActorExtra *inPlotActor);
*/

protected:
  int change_style();


private:

  VtkAxis(const VtkAxis& inAxis);

  vtkRenderer* renderer;

  vtkAxisActor2DExtra* axis;

  void CopyFormats(vtkTextActorFormat* outFormat, ELEM_P_T inFormat);

  string ref_id;

#ifndef HEADLESS
  vtkChipsPickableWidget *axis_widget;
#endif


};





#endif
