// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "VtkAxisLabel.hh"
#include "VtkFrame.hh"
#include "vtkCoordinate.h"
#include "vtkCamera.h"
#include "vtkTextActorExtra.hh"
#include "ChipsCoordinate.hh"
#include "vtkAxisActor2DExtra.hh"
#include "vtkRenderer.h"


#ifndef HEADLESS
#include "vtkChipsTextWidget.hh"
#include "vtkChipsTextRepresentation.hh"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#endif


VtkAxisLabel::VtkAxisLabel(vtkAxisActor2DExtra* inAxis,vtkRenderer *renderer, ChipsObject* par, double inParallelOffset, double inPerpendicularOffset,const string& val):ChipsLabel(par,"",NULL,COORD_NONE,inParallelOffset,inPerpendicularOffset,val)
{
  axis=inAxis;

  if ( axis)
    {

      AddFormattedText(format,axis->GetTitleActor(), get_scale_factor ());
      vtkText = axis->GetTitleActor();
      this->renderer = renderer;

#ifndef HEADLESS
      ChipsId cid;
      this->populate_chips_id (cid);

      vtkChipsTextRepresentation *label_rep = vtkChipsTextRepresentation::New ();
      label_rep->SetTextActor (vtkText);
      label_rep->SetMaxFontSize (V_SIZE_MAX_VAL);


      this->label_widget = vtkChipsTextWidget::New ();
      this->label_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      this->label_widget->SetRepresentation (label_rep);
      this->label_widget->SetIdInfo (cid, chips_axis, chips_axis_label);
      this->label_widget->SetDefaultRenderer (renderer);
      this->label_widget->EnabledOn ();
      label_rep->Delete ();
#endif


      change_color();
      change_rotation();
      change_alignment();
      axis->Modified();
      axis->GetTitleActor()->Modified();

    };

};

VtkAxisLabel::VtkAxisLabel(vtkAxisActor2DExtra* inAxis,vtkRenderer *renderer,ChipsObject* par, ChipsStateStream& in):ChipsLabel(par, NULL, COORD_NONE, in)
{
  axis=inAxis;

  if ( axis)
    {
      AddFormattedText(format,axis->GetTitleActor(),get_scale_factor ());
      vtkText = axis->GetTitleActor();

      this->renderer = renderer;

#ifndef HEADLESS
      ChipsId cid;
      this->populate_chips_id (cid);

      vtkChipsTextRepresentation *label_rep = vtkChipsTextRepresentation::New ();
      label_rep->SetTextActor (vtkText);
      label_rep->SetMaxFontSize (V_SIZE_MAX_VAL);


      this->label_widget = vtkChipsTextWidget::New ();
      this->label_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      this->label_widget->SetRepresentation (label_rep);
      this->label_widget->SetIdInfo (cid, chips_axis, chips_axis_label);
      this->label_widget->SetDefaultRenderer (renderer);
      this->label_widget->EnabledOn ();
      label_rep->Delete ();
#endif



      change_color();
      change_rotation();
      change_alignment();
      axis->Modified();
      axis->GetTitleActor()->Modified();

    };

};


VtkAxisLabel::~VtkAxisLabel()
{
  if ( axis)
    {
      axis->GetTitleActor()->Clear();
    }
};

int VtkAxisLabel::redraw()
{

  change_color();
  change_rotation();
  change_alignment();
  return ERR_STATUS_GOOD;
};

int VtkAxisLabel::change_text()
{  

  if (vtkText)
    {
      vtkText->Clear();
      AddFormattedText(format,vtkText,get_scale_factor ());
#ifndef HEADLESS
      if (this->label_widget)
        this->label_widget->UpdateRepresentation ();
#endif

    }
  else
    return ERR_STATUS_BAD ;


  return ERR_STATUS_GOOD;
};

int VtkAxisLabel::change_font()
{
  return change_text();

};

int VtkAxisLabel::change_size()
{ 
  return change_text();
  
};


int VtkAxisLabel::change_scale_factor()
{ 
  return change_text();
  
};

int VtkAxisLabel::change_color()
{
  return change_text();
};

int VtkAxisLabel::change_visible ()
{
   int tmp_vis = (!get_parent()->is_visible ()) ? 0 : is_visible();

   axis->SetTitleVisibility(static_cast<int>(tmp_vis));
   axis->Modified();


   if (vtkText)
     vtkText->SetVisibility (tmp_vis);

#ifndef HEADLESS
   if (this->label_widget)
   {
     if (tmp_vis)
     {
       //this->label_widget->EnabledOn ();
       this->label_widget->GetRepresentation ()->SetVisibility (tmp_vis);
     }
     //else
       //this->label_widget->EnabledOff ();
   }
#endif



   return ERR_STATUS_GOOD;

}




int VtkAxisLabel::change_rotation()
{
  if (vtkText)
    {
      vtkText->GetTextProperty()->SetOrientation(angle);
      vtkText->Modified();
#ifndef HEADLESS
      if (this->label_widget)
        this->label_widget->UpdateRepresentation ();
#endif

    }

  if (axis)
    axis->Modified();

  return ERR_STATUS_GOOD;
};


int VtkAxisLabel::change_alignment()
{
  if ( vtkText)
    {
      vtkText->SetVertAlignmentOffset(vert_alignment);
      vtkText->SetHorizAlignmentOffset(horiz_alignment);
      vtkText->Modified();
#ifndef HEADLESS
      if (this->label_widget)
        this->label_widget->UpdateRepresentation ();
#endif

    }
  if ( axis)
    axis->Modified();

  return ERR_STATUS_GOOD;
};

void VtkAxisLabel::select ()
{
#ifndef HEADLESS
  if (this->label_widget)
    this->label_widget->Select (true);
#endif
}

void VtkAxisLabel::unselect ()
{
#ifndef HEADLESS
  if (this->label_widget)
    this->label_widget->Select (false);
#endif
}


int VtkAxisLabel::compute_resize( double bounds[])
{
  // bounds should be in display coordinates

  double outPosition[2];
  ChipsId cid;

  ChipsAxis *axis_parent = dynamic_cast <ChipsAxis *>(this->parent);
  if (!axis_parent)
    return ERR_STATUS_BAD;

  int font = ComputeFontSizeAndPosFromBBox (this->vtkText, renderer, bounds, outPosition);
  int rval = axis_parent->set_property(static_cast<void*>(&font), SET_AXIS_LABEL_SIZE);

  if (!rval)
    axis_parent->move_object (chips_axis_label, PIXEL_REAL, "", outPosition[0], outPosition[1], V_MOVE_ABSOLUTE);

  return rval;

}

void VtkAxisLabel::disable_interaction ()
{
#ifndef HEADLESS
  vtkChipsTextRepresentation *rep = reinterpret_cast<vtkChipsTextRepresentation *> (this->label_widget->GetRepresentation ());
  rep->DisableBorder ();
  this->label_widget->EnabledOff ();
#endif
}

void VtkAxisLabel::enable_interaction ()
{
#ifndef HEADLESS
  if (this->is_interaction_blocked ())
      return;

  vtkChipsTextRepresentation *rep = reinterpret_cast<vtkChipsTextRepresentation *> (this->label_widget->GetRepresentation ());

  this->label_widget->EnabledOn ();
  rep->EnableBorder ();
#endif
}

void VtkAxisLabel::modify ()
{
#ifndef HEADLESS
  if (this->label_widget)
    this->label_widget->UpdateRepresentation ();
#endif
  this->vtkText->Modified ();

}
