// 
//  Copyright (C) 2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "VtkColorBar.hh"
#include "VtkColorMapTable.hh"
#include "VtkColorMap.hh"
#include "VtkLabelInterface.hh"
#include "VtkUtils.hh"

#include "vtkProperty2D.h"
#include "vtkTextProperty.h"
#include "vtkScalarBarActorExtra.hh"
#include "vtkTextActorExtra.hh"
#include "vtkLookupTable.h"
#include <sstream>

#ifndef HEADLESS
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkChipsScalarBarWidget.hh"
#include "vtkChipsScalarBarRepresentation.hh"
#include "vtkChipsTextWidget.hh"
#include "vtkChipsTextRepresentation.hh"
#endif 

#include "ChipsCmdManager.hh"
#include "ChipsColorMapTable.hh"

extern ChipsCmdManager* cmd_manager;
extern ChipsColorMapTable *colormap_table;


VtkColorBar::VtkColorBar(vtkRenderer* inRenderer, ChipsObject* par,
   ChipsCoordinate *inCoord, COORD_SYS coord_sys, double xpos, 
   double ypos, unsigned short cmap_id, string val) : ChipsColorBar (
    par, inCoord, coord_sys, xpos, ypos, cmap_id, val)
{
   double range[2];
   this->renderer=inRenderer; 

   this->scalarBar = vtkScalarBarActorExtra::New();
   this->scalarBar->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();

   this->scalarBar->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
   this->scalarBar->SetClipData (this->coord_system == DATA);
   this->scalarBar->SetLabelFormat ("%g");

   colormap_table->get_colormap_scalar_range (colormap_id, range);
   this->scalarBar->SetScalarRange (range);
   this->scalarBar->SetUseLookupTableRange (false);
   renderer->AddActor2D(scalarBar);

#ifndef HEADLESS
   ChipsId cid;
   vtkChipsScalarBarRepresentation *scalarBarRep = vtkChipsScalarBarRepresentation::New ();
   scalarBarRep->SetScalarBarActor (this->scalarBar);

   this->populate_chips_id (cid);
   this->scalarBarWidget = vtkChipsScalarBarWidget::New ();
   this->scalarBarWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   this->scalarBarWidget->SetRepresentation (scalarBarRep);
   this->scalarBarWidget->SetIdInfo (cid, this->get_object_type());
   this->scalarBarWidget->RotateableOff ();
   this->scalarBarWidget->SetDefaultRenderer (renderer);
   this->scalarBarWidget->EnabledOn ();
   scalarBarRep->Delete ();

   vtkChipsTextRepresentation *labelRep = vtkChipsTextRepresentation::New ();
   labelRep->SetTextActor (this->scalarBar->GetTitleActor ());
   labelRep->SetMaxFontSize (V_SIZE_MAX_VAL);

   this->labelWidget = vtkChipsTextWidget::New ();
   this->labelWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   this->labelWidget->SetRepresentation (labelRep);
   this->labelWidget->SetIdInfo (cid, this->get_object_type(), chips_colorbar_label);
   this->labelWidget->SetDefaultRenderer (renderer);
   this->labelWidget->EnabledOn ();
   labelRep->Delete ();
#endif




/*   if (this->title_flip) this->scalarBar->SetTitleFlip(1);
*/

   change_colormap();
   change_position();
   change_alignment ();
   change_visible();
   change_size ();
   change_orientation ();
 
   change_tick_mode();
   change_tick_style();
   change_tick_color();
   change_tick_thickness();
   change_tick_length();
   change_tick_location();

   change_ticklabel_color();
   change_ticklabel_font();
   change_ticklabel_size();
   change_ticklabel_angle();
   change_ticklabel_offset();
   change_ticklabel_visible();
   change_ticklabel_alignment();

   change_title_text ();
   change_title_color();
   change_title_font();
   change_title_fontstyle();
   change_title_size();
   change_title_angle();
   change_title_alignment();
   change_title_visible ();
   change_title_location ();
   change_parallel_offset();
   change_perpendicular_offset();


   change_bar_flip ();

   change_border_visible ();

};

VtkColorBar::VtkColorBar(vtkRenderer* inRenderer, ChipsObject* par,
   ChipsCoordinate *inCoord, COORD_SYS coord_sys, double xpos,
   double ypos, string val) : ChipsColorBar (
    par, inCoord, coord_sys, xpos, ypos, val)
{
   double range[2];
   this->renderer=inRenderer;

   this->scalarBar = vtkScalarBarActorExtra::New();
   this->scalarBar->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();

   this->scalarBar->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
   this->scalarBar->SetClipData (this->coord_system == DATA);
   this->scalarBar->SetLabelFormat ("%g");

   colormap_table->get_colormap_scalar_range (colormap_id, range);
   this->scalarBar->SetScalarRange (range);
   this->scalarBar->SetUseLookupTableRange (false);
   renderer->AddActor2D(scalarBar);

#ifndef HEADLESS
   ChipsId cid;
   vtkChipsScalarBarRepresentation *scalarBarRep = vtkChipsScalarBarRepresentation::New ();
   scalarBarRep->SetScalarBarActor (this->scalarBar);

   this->populate_chips_id (cid);
   this->scalarBarWidget = vtkChipsScalarBarWidget::New ();
   this->scalarBarWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   this->scalarBarWidget->SetRepresentation (scalarBarRep);
   this->scalarBarWidget->SetIdInfo (cid, this->get_object_type());
   this->scalarBarWidget->RotateableOff ();
   this->scalarBarWidget->SetDefaultRenderer (renderer);
   this->scalarBarWidget->EnabledOn ();
   scalarBarRep->Delete ();

   vtkChipsTextRepresentation *labelRep = vtkChipsTextRepresentation::New ();
   labelRep->SetTextActor (this->scalarBar->GetTitleActor ());
   labelRep->SetMaxFontSize (V_SIZE_MAX_VAL);

   this->labelWidget = vtkChipsTextWidget::New ();
   this->labelWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   this->labelWidget->SetRepresentation (labelRep);
   this->labelWidget->SetIdInfo (cid, this->get_object_type(), chips_colorbar_label);
   this->labelWidget->SetDefaultRenderer (renderer);
   this->labelWidget->EnabledOn ();
   labelRep->Delete ();
#endif




/*   if (this->title_flip) this->scalarBar->SetTitleFlip(1);
*/


   change_colormap();
   change_position();
   change_alignment ();
   change_visible();
   change_size ();
   change_orientation ();

   change_tick_mode();
   change_tick_style();
   change_tick_color();
   change_tick_thickness();
   change_tick_length();
   change_tick_location();

   change_ticklabel_color();
   change_ticklabel_font();
   change_ticklabel_size();
   change_ticklabel_angle();
   change_ticklabel_offset();
   change_ticklabel_visible();
   change_ticklabel_alignment();

   change_title_text ();
   change_title_color();
   change_title_font();
   change_title_fontstyle();
   change_title_size();
   change_title_angle();
   change_title_alignment();
   change_title_visible ();
   change_title_location ();
   change_parallel_offset();
   change_perpendicular_offset();


   change_bar_flip ();

   change_border_visible ();

};


VtkColorBar::VtkColorBar(vtkRenderer* inRenderer, ChipsObject* par,ChipsCoordinate 
   *inCoord, COORD_SYS coord_sys, ChipsStateStream& in):ChipsColorBar(par, 
   inCoord, coord_sys, in)
{
   double range[2];
   stringstream tmpid;
   tmpid << ref_cnt;
   ref_id = tmpid.str();
 
   renderer=inRenderer;
   scalarBar=0;

   scalarBar = vtkScalarBarActorExtra::New();
   scalarBar->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
   scalarBar->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
   scalarBar->SetClipData (coord_system == DATA);
   this->scalarBar->SetLabelFormat ("%g");

   colormap_table->get_colormap_scalar_range (colormap_id, range);
   this->scalarBar->SetScalarRange (range);
   this->scalarBar->SetUseLookupTableRange (false);
   renderer->AddActor2D(scalarBar);

#ifndef HEADLESS
   ChipsId cid;
   vtkChipsScalarBarRepresentation *scalarBarRep = vtkChipsScalarBarRepresentation::New ();
   scalarBarRep->SetScalarBarActor (this->scalarBar);

   this->populate_chips_id (cid);
   this->scalarBarWidget = vtkChipsScalarBarWidget::New ();
   this->scalarBarWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   this->scalarBarWidget->SetRepresentation (scalarBarRep);
   this->scalarBarWidget->SetIdInfo (cid, this->get_object_type());
   this->scalarBarWidget->RotateableOff ();
   this->scalarBarWidget->SetDefaultRenderer (renderer);
   this->scalarBarWidget->EnabledOn ();
   scalarBarRep->Delete ();

   vtkChipsTextRepresentation *labelRep = vtkChipsTextRepresentation::New ();
   labelRep->SetTextActor (this->scalarBar->GetTitleActor ());
   labelRep->SetMaxFontSize (V_SIZE_MAX_VAL);

   this->labelWidget = vtkChipsTextWidget::New ();
   this->labelWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   this->labelWidget->SetRepresentation (labelRep);
   this->labelWidget->SetIdInfo (cid, this->get_object_type(), chips_colorbar_label);
   this->labelWidget->SetDefaultRenderer (renderer);
   this->labelWidget->EnabledOn ();
   labelRep->Delete ();
#endif



   
/*   if (title_flip) scalarBar->SetTitleFlip(1);

   if (deserialize_label)
   {
      label = new VtkLabel(scalarBar,this, NULL, COORD_NONE, in);
      // the label will add an error for not having a coordinate system
      // remove this error
      cmd_manager->remove_last_err ();
   }
  else
    {
      label = new VtkLabel(scalarBar,(ChipsObject*)this,"",NULL,COORD_NONE,
         static_cast<double>(offset_par),static_cast<double>(offset_perp),"");

      // the label will add an error for not having a coordinate system
      // remove this error
      cmd_manager->remove_last_err ();

      label->set_fontstyle(label_fontstyle);
      label->set_size(label_size);
      label->set_font(label_font);
      label->set_color(label_color);
      label->set_vertalignment(label_vertalignment);
      label->set_horizalignment(label_horizalignment);
      label->set_angle(label_angle);
    }
  if (label)   set_label_visible(labvis);
*/

   change_colormap();
   change_position();
   change_alignment ();
   change_visible();
   change_size ();
   change_orientation ();
  
   change_tick_mode();
   change_tick_style();
   change_tick_color();
   change_tick_thickness();
   change_tick_length();
   change_tick_location();

   change_ticklabel_color();
   change_ticklabel_font();
   change_ticklabel_size();
   change_ticklabel_angle();
   change_ticklabel_offset();
   change_ticklabel_visible();
   change_ticklabel_alignment();

   change_title_text ();
   change_title_color();
   change_title_font();
   change_title_fontstyle();
   change_title_size();
   change_title_angle();
   change_title_alignment();
   change_title_visible ();
   change_title_location ();
   change_parallel_offset();
   change_perpendicular_offset();

   change_bar_flip ();
   change_border_visible ();

};


VtkColorBar::~VtkColorBar()
{
/*  if ( label)
    label->object_delete ();
*/

   if ( scalarBar)
   {
      renderer->RemoveActor(scalarBar);
      scalarBar->Delete();
   }
};



int VtkColorBar::change_colormap()
{

   if ( this->scalarBar)
   {
      VtkColorMap *tmpColorMap = dynamic_cast <VtkColorMap *> (colormap_table-> get_colormap (this->colormap_id));

      this->scalarBar->SetLookupTable (static_cast <vtkScalarsToColors *>(tmpColorMap->get_lookup_table ()));
   }
   else
      return ERR_STATUS_BAD;


   return ERR_STATUS_GOOD;
};


int VtkColorBar::change_position()
{

   if ( scalarBar)
   {
      double tmp_pos[] = {norm_pos[0], norm_pos[1]};
      VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_pos);  
      scalarBar->SetOrigin(tmp_pos);
#ifndef HEADLESS
      if (this->scalarBarWidget)
         this->scalarBarWidget->UpdateRepresentation ();
#endif
   }

   return ERR_STATUS_GOOD;
};


int VtkColorBar::change_depth()
{
   if (scalarBar)
      scalarBar->SetLayerNumber(depth*10+1);
   return ERR_STATUS_GOOD;
}

int VtkColorBar::change_tick_visible()
{
   int tmp_vis = (visible && tickmode!=0) ? tickvis : 0;
   if ( scalarBar)
   {
      scalarBar->SetMajorTicksVisibility(static_cast<int>(tmp_vis));
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;


   return ERR_STATUS_GOOD;


}

int VtkColorBar::change_tick_mode()
{
   vector<double> tmpMajLoc(tickcnt);
   vector<bool> tmpLab(tickcnt);
   vector<vtkTextActorFormat> tmpFormats;
  
   if ( tickpos )
      tmpFormats.resize(tickcnt);
  
  
   for(int indx=0;indx<tickcnt;indx++)
      tmpMajLoc[indx]=tickpos[indx];
  

   if ( scalarBar)
   {
      scalarBar->SetMajorTicksLocations(tmpMajLoc);
//      scalarBar->SetLabelFormat(tickformat.c_str());
      change_tick_visible(); 
      change_ticklabel_visible(); 
      scalarBar->Modified();
   }
  
   return ERR_STATUS_GOOD;

};


int VtkColorBar::change_ticklabel_color()
{

   if ( scalarBar)
   {
      scalarBar->GetLabelTextProperty()->SetColor(ticklabcolor.red(),ticklabcolor.green(),ticklabcolor.blue());
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;
  
};

int VtkColorBar::change_ticklabel_font()
{  
   if ( scalarBar)
   {
      scalarBar->GetLabelTextProperty()->SetFaceFileName(0);
      
      if ( ticklabfontfile == "DEFAULT_HELVETICA" )
      {
         scalarBar->GetLabelTextProperty()->SetFontFamilyToHelvetica();
	 scalarBar->GetLabelTextProperty()->BoldOff();
	 scalarBar->GetLabelTextProperty()->ItalicOff();
      }
      else if ( ticklabfontfile == "DEFAULT_HELVETICA_BOLD" )
      {
         scalarBar->GetLabelTextProperty()->SetFontFamilyToHelvetica();
         scalarBar->GetLabelTextProperty()->BoldOn();
         scalarBar->GetLabelTextProperty()->ItalicOff();
      }
      else if ( ticklabfontfile == "DEFAULT_HELVETICA_ITALIC" )
      {
         scalarBar->GetLabelTextProperty()->SetFontFamilyToHelvetica();
	 scalarBar->GetLabelTextProperty()->BoldOff();
	 scalarBar->GetLabelTextProperty()->ItalicOn();
      }
      else if ( ticklabfontfile == "DEFAULT_HELVETICA_BOLDITALIC" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToHelvetica();
	 scalarBar->GetLabelTextProperty()->BoldOn();
	 scalarBar->GetLabelTextProperty()->ItalicOn();
      }
      else if ( ticklabfontfile == "DEFAULT_COURIER" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToCourier();
	 scalarBar->GetLabelTextProperty()->BoldOff();
	 scalarBar->GetLabelTextProperty()->ItalicOff();
      }
      else if ( ticklabfontfile == "DEFAULT_COURIER_BOLD" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToCourier();
	 scalarBar->GetLabelTextProperty()->BoldOn();
	 scalarBar->GetLabelTextProperty()->ItalicOff();
      }
      else if ( ticklabfontfile == "DEFAULT_COURIER_ITALIC" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToCourier();
	 scalarBar->GetLabelTextProperty()->BoldOff();
	 scalarBar->GetLabelTextProperty()->ItalicOn();
      }
      else if ( ticklabfontfile == "DEFAULT_COURIER_BOLDITALIC" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToCourier();
	 scalarBar->GetLabelTextProperty()->BoldOn();
	 scalarBar->GetLabelTextProperty()->ItalicOn();
      }
      else if ( ticklabfontfile == "DEFAULT_TIMES" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToTimes();
	 scalarBar->GetLabelTextProperty()->BoldOff();
	 scalarBar->GetLabelTextProperty()->ItalicOff();
      }
      else if ( ticklabfontfile == "DEFAULT_TIMES_BOLD" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToTimes();
	 scalarBar->GetLabelTextProperty()->BoldOn();
	 scalarBar->GetLabelTextProperty()->ItalicOff();
      }
      else if ( ticklabfontfile == "DEFAULT_TIMES_ITALIC" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToTimes();
	 scalarBar->GetLabelTextProperty()->BoldOff();
	 scalarBar->GetLabelTextProperty()->ItalicOn();
      }
      else if ( ticklabfontfile == "DEFAULT_TIMES_BOLDITALIC" )
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToTimes();
	 scalarBar->GetLabelTextProperty()->BoldOn();
	 scalarBar->GetLabelTextProperty()->ItalicOn();
      }
      else if( ( ticklabfontfile == "DEFAULT_GREEK" ) 
	       ||(ticklabfontfile == "DEFAULT_GREEK_BOLD")
	       ||(ticklabfontfile == "DEFAULT_GREEK_ITALIC")
	       ||(ticklabfontfile == "DEFAULT_GREEK_BOLDITALIC"))
      {
	 scalarBar->GetLabelTextProperty()->SetFontFamilyToGreek();
	 scalarBar->GetLabelTextProperty()->BoldOff();
	 scalarBar->GetLabelTextProperty()->ItalicOff();
      }
      else 
      {
	 scalarBar->GetLabelTextProperty()->SetFaceFileName(ticklabfontfile.c_str());
      }

      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;
};

/*
int VtkColorBar::change_scale_factor()
{
   int rval;
   if ((rval = change_ticklabel_offset()) == ERR_STATUS_GOOD)
      rval = change_ticklabel_size();
   if (rval == ERR_STATUS_GOOD)
      rval = change_perpendicular_offset ();
   if (!rval && label)
      rval = label->change_scale_factor ();

   return rval; 
};
*/

int VtkColorBar::change_ticklabel_size()
{  
   if ( scalarBar)
   {
      scalarBar->GetLabelTextProperty()->SetFontSize((int)(ticklabsize*get_scale_factor()));
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;
  

};

int VtkColorBar::change_ticklabel_angle()
{

   if ( scalarBar)
   {
      scalarBar->GetLabelTextProperty()->SetOrientation(ticklabangle);
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;
  

};

int VtkColorBar::change_ticklabel_offset()
{
  
   if ( scalarBar)
   {
      scalarBar->SetLabelOffset(ticklaboffset*get_scale_factor());
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;


};


int VtkColorBar::change_ticklabel_visible()
{
   int tmp_vis = (visible && tickmode!=0) ? ticklabvis : 0;
  
   if ( scalarBar)
   {
      scalarBar->SetLabelVisibility(static_cast<int>(tmp_vis));
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;


}


int VtkColorBar::change_ticklabel_alignment()
{

   if ( scalarBar)
   {
      scalarBar->SetMajorTickLabelVertAlignmentOffset(use_ticklab_valign);
      scalarBar->SetMajorTickLabelHorizAlignmentOffset(use_ticklab_halign);
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;

  
   return ERR_STATUS_GOOD;


}


int VtkColorBar::change_tick_color()
{

   if ( scalarBar)
   {
      
      scalarBar->GetMajorTicks()->GetProperty()->SetColor(tickcolor.red(),tickcolor.green(),tickcolor.blue());
      scalarBar->GetBoundingBox()->GetProperty()->SetColor(tickcolor.red(),tickcolor.green(),tickcolor.blue());
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;
};



int VtkColorBar::change_tick_length()
{

   if ( scalarBar)
   {
      
      scalarBar->SetMajorTicksLength(ticklength);
      scalarBar->Modified();

//      return change_style();
   }
   else
      return ERR_STATUS_BAD;
  
  
   return ERR_STATUS_GOOD;

};


int VtkColorBar::change_tick_thickness()
{
  
   if ( scalarBar)
   {
      scalarBar->GetMajorTicks()->GetProperty()->SetLineWidth(tickthickness);
      scalarBar->GetBoundingBox()->GetProperty()->SetLineWidth(tickthickness);
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
   return ERR_STATUS_GOOD;

};

int VtkColorBar::change_tick_location()
{
   if ( scalarBar)
   {
      this->scalarBar->SetLabelsFlip(ticklocation == chips_inside);
      this->scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;
};

int VtkColorBar::change_border_visible()
{

   int tmp_vis = (visible) ? this->border_vis : 0;

   if ( scalarBar)
   {
      scalarBar->SetBoundingBoxVisibility(static_cast<int>(tmp_vis));
      scalarBar->Modified();
   }
   else
      return ERR_STATUS_BAD;

  return ERR_STATUS_GOOD;
};


int VtkColorBar::change_parallel_offset()
{
  
  if ( scalarBar)
    {
      scalarBar->SetTitleParallelOffset(offset_par); 
      scalarBar->Modified();
#ifndef HEADLESS
      if (this->labelWidget)
         this->labelWidget->UpdateRepresentation ();
#endif
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

  

};

int VtkColorBar::change_perpendicular_offset()
{
  if ( scalarBar)
    {
      int scaled_offset = static_cast <int>(offset_perp * get_scale_factor() + 0.5);
      scalarBar->SetTitlePerpendicularOffset(static_cast<int>(scaled_offset));
      scalarBar->Modified();
#ifndef HEADLESS
      if (this->labelWidget)
         this->labelWidget->UpdateRepresentation ();
#endif
    }
  else
    return ERR_STATUS_BAD;
  
  return ERR_STATUS_GOOD;

};


int VtkColorBar::change_title_visible()
{
  int tmp_vis = (is_visible ()) ? labvis : 0;
  
  if ( scalarBar)
    {
      scalarBar->SetTitleVisibility(tmp_vis);
      scalarBar->Modified();
#ifndef HEADLESS
      labelWidget->GetRepresentation ()->SetVisibility (tmp_vis);
#endif

    }
  else
    return ERR_STATUS_BAD;
  
  
  return ERR_STATUS_GOOD;


}



int VtkColorBar::change_tick_style()
{
  float tmpOffset;

  if ( scalarBar)
  {

     switch(tickstyle)
     {
	  
        case chips_inside:   
	   scalarBar->SetMajorTicksFlip(1); 
	   scalarBar->SetMajorTickOffset(0);
	break;
	case chips_outside:   
	   scalarBar->SetMajorTicksFlip(0); 
	   scalarBar->SetMajorTickOffset(0);
	break;
	case chips_centered:
	{
	   scalarBar->SetMajorTicksFlip(0); 
	   tmpOffset=ticklength+this->length;
	   scalarBar->SetMajorTickOffset(static_cast <int> (-tmpOffset/2));
	}
	break;
	default:
	  return ERR_STATUS_BAD;
	  
      };

      scalarBar->Modified();      

   }
   else
      return ERR_STATUS_BAD;

  return ERR_STATUS_GOOD;
};


int VtkColorBar::change_visible()
{
   int tmp_vis = (!get_parent ()->is_visible ()) ? 0 : this->is_visible ();

   if ( scalarBar)
   {
      scalarBar->SetVisibility(tmp_vis);
#ifndef HEADLESS
      scalarBarWidget->GetRepresentation ()->SetVisibility (tmp_vis);
#endif
   }
   else
      return ERR_STATUS_BAD;


   return ERR_STATUS_GOOD;
};

int VtkColorBar::change_alignment()
{
   if ( scalarBar)
   {
      scalarBar->SetHorizAlignmentOffset(horiz_alignment);
      scalarBar->SetVertAlignmentOffset(vert_alignment);
      scalarBar->Modified();

#ifndef HEADLESS
      if (this->scalarBarWidget)
         this->scalarBarWidget->UpdateRepresentation ();
#endif
   }
   return ERR_STATUS_GOOD;
}

int VtkColorBar::change_size()
{
   if ( this->scalarBar)
   {
      double tmp_vector1[] = {0,0};
      double tmp_vector2[] = {0,0};
      float pixel_width =0;
      float pixel_length = 0;

      // vtk expects the width and height to be in pixels
      if (orient == 0)
      {
         tmp_vector1[0] = norm_pos[0] - (frame_length * horiz_alignment);
         tmp_vector1[1] = norm_pos[1] - (frame_width * vert_alignment);
         tmp_vector2[0] = tmp_vector1[0] + frame_length;
         tmp_vector2[1] = tmp_vector1[1] + frame_width;
/*
         coord->convert_cs_to_cs (&tmp_vector1, &tmp_vector1, 1, FRAME_NORM,
                            "", PIXEL_REAL, "");
         coord->convert_cs_to_cs (&tmp_vector2, &tmp_vector2, 1, FRAME_NORM,
                            "", PIXEL_REAL, "");
*/
/*
         VtkUtils::CoordSys2Viewport (coord, FRAME_NORM, "", renderer, tmp_vector1);
         VtkUtils::CoordSys2Viewport (coord, FRAME_NORM, "", renderer, tmp_vector2);
*/
         VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_vector1);
         VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_vector2);

         pixel_length = (float)(tmp_vector2[0]-tmp_vector1[0]);
         pixel_width = (float)(tmp_vector2[1]-tmp_vector1[1]);
      }
      else
      {
         tmp_vector1[0] = norm_pos[0] - (frame_width * vert_alignment);
         tmp_vector1[1] = norm_pos[1] - (frame_length * horiz_alignment);
         tmp_vector2[0] = tmp_vector1[0] + frame_width;
         tmp_vector2[1] = tmp_vector1[1] + frame_length;

/*
         coord->convert_cs_to_cs (&tmp_vector1, &tmp_vector1, 1, FRAME_NORM,
                            "", PIXEL_REAL, "");
         coord->convert_cs_to_cs (&tmp_vector2, &tmp_vector2, 1, FRAME_NORM,
                            "", PIXEL_REAL, "");
*/

/*
         VtkUtils::CoordSys2Viewport (coord, FRAME_NORM, "", renderer, tmp_vector1);
         VtkUtils::CoordSys2Viewport (coord, FRAME_NORM, "", renderer, tmp_vector2);
*/
         VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_vector1);
         VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmp_vector2);
         pixel_length = (float)(tmp_vector2[1]-tmp_vector1[1]);
         pixel_width = (float)(tmp_vector2[0]-tmp_vector1[0]);
      }

      this->scalarBar->SetWidth (pixel_width);
      this->scalarBar->SetLength (pixel_length);

      // if we have resized, the label may need to be updated
      this->change_title_alignment();

#ifndef HEADLESS
      if (this->scalarBarWidget)
         this->scalarBarWidget->UpdateRepresentation ();
#endif
   }

   return ERR_STATUS_GOOD;
}

int VtkColorBar::change_orientation()
{
   if ( this->scalarBar)
   {
      if (this->orient == 0)
         this->scalarBar->SetOrientationToHorizontal ();
      else
         this->scalarBar->SetOrientationToVertical ();

#ifndef HEADLESS
      if (this->scalarBarWidget)
         this->scalarBarWidget->UpdateRepresentation ();
#endif
   }

   return ERR_STATUS_GOOD;
}

int VtkColorBar::change_bar_flip()
{
   if ( this->scalarBar)
      this->scalarBar->SetBarFlip (bar_flip);

#ifndef HEADLESS
   if (this->scalarBarWidget)
      this->scalarBarWidget->UpdateRepresentation ();
#endif

   return ERR_STATUS_GOOD;

}

int VtkColorBar::change_title_text ()
{
   if ( this->scalarBar)
   {
      vtkTextActorExtra* title = this->scalarBar->GetTitleActor ();
      if (title)
      {
         title->Clear ();
         VtkLabelInterface::AddFormattedText(label_format,title,get_scale_factor ());
#ifndef HEADLESS
         if (this->labelWidget)
            this->labelWidget->UpdateRepresentation ();
#endif
      }
   }
   return ERR_STATUS_GOOD;
}


int VtkColorBar::change_title_color()
{
   return change_title_text ();
}

int VtkColorBar::change_title_font()
{
   return change_title_text ();
}

int VtkColorBar::change_title_fontstyle()
{
   return change_title_text ();
}

int VtkColorBar::change_title_size()
{
   return change_title_text ();
}

int VtkColorBar::change_title_angle()
{
   if ( this->scalarBar)
   {
      vtkTextActorExtra* title = this->scalarBar->GetTitleActor ();
      if (title)
      {
         title->GetTextProperty()->SetOrientation(label_angle);
         title->Modified ();
      }
   }
   return ERR_STATUS_GOOD;

}

int VtkColorBar::change_title_alignment()
{
   if ( this->scalarBar)
   {
      vtkTextActorExtra* title = this->scalarBar->GetTitleActor ();
      if (title)
      {
         title->SetVertAlignmentOffset(use_label_valign);
         title->SetHorizAlignmentOffset(use_label_halign);
         title->Modified ();

#ifndef HEADLESS
         if (this->labelWidget)
            this->labelWidget->UpdateRepresentation ();
#endif
      }
   }
   return ERR_STATUS_GOOD;

}


int VtkColorBar::change_title_location()
{
   if ( this->scalarBar)
   {
      if (label_location == chips_outside)
         scalarBar->SetTitleFlip(1);
      else
         scalarBar->SetTitleFlip(0);
      scalarBar->Modified();

#ifndef HEADLESS
      if (this->labelWidget)
         this->labelWidget->UpdateRepresentation ();
#endif
   }
   return ERR_STATUS_GOOD;

}


void VtkColorBar::CopyFormats(vtkTextActorFormat* outFormat, ELEM_P_T inFormat)
{
  
   if ((! inFormat) || (!outFormat))
      return ;
  
   if ( outFormat->super )
      delete outFormat->super;
   if ( outFormat->next)
      delete outFormat->next;
   if (outFormat->sub)
      delete outFormat->sub; 
  

   outFormat->text = inFormat->text;


   outFormat->fontFile = "";
   outFormat->fontBold =0;
   outFormat->fontItalic =0;

   if ( inFormat->fontfile == "DEFAULT_HELVETICA" )
   {
      outFormat->fontFamily = VTK_HELVETICA;
   }
   else if ( inFormat->fontfile == "DEFAULT_HELVETICA_BOLD" )
   {
      outFormat->fontFamily = VTK_HELVETICA;
      outFormat->fontBold =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_HELVETICA_ITALIC" )
   {
      outFormat->fontFamily = VTK_HELVETICA;
      outFormat->fontItalic =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_HELVETICA_BOLDITALIC" )
   {
      outFormat->fontFamily = VTK_HELVETICA;
      outFormat->fontBold =1;
      outFormat->fontItalic =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_COURIER" )
   {
      outFormat->fontFamily = VTK_COURIER;
   }
   else if ( inFormat->fontfile == "DEFAULT_COURIER_BOLD" )
   {
      outFormat->fontFamily = VTK_COURIER;
      outFormat->fontBold =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_COURIER_ITALIC" )
   {
      outFormat->fontFamily = VTK_COURIER;
      outFormat->fontItalic =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_COURIER_BOLDITALIC" )
   {
      outFormat->fontFamily = VTK_COURIER; 
      outFormat->fontBold =1;
      outFormat->fontItalic =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_TIMES" )
   {
      outFormat->fontFamily = VTK_TIMES;
      
   }
   else if ( inFormat->fontfile == "DEFAULT_TIMES_BOLD" )
   {
      outFormat->fontFamily = VTK_TIMES;
      outFormat->fontBold =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_TIMES_ITALIC" )
   {
      outFormat->fontFamily = VTK_TIMES;
      outFormat->fontItalic =1;
   }
   else if ( inFormat->fontfile == "DEFAULT_TIMES_BOLDITALIC" )
   {
      outFormat->fontFamily = VTK_TIMES;
      outFormat->fontBold =1;
      outFormat->fontItalic =1;
   }
   else if( ( inFormat->fontfile == "DEFAULT_GREEK" ) 
	   ||(inFormat->fontfile == "DEFAULT_GREEK_BOLD")
	   ||(inFormat->fontfile == "DEFAULT_GREEK_ITALIC")
	   ||(inFormat->fontfile == "DEFAULT_GREEK_BOLDITALIC"))
   {
      outFormat->fontFamily = VTK_GREEK; 
   }
   else 
   {
      outFormat->fontFile=inFormat->fontfile;
   }


   outFormat->size = static_cast <int>(inFormat->size*get_scale_factor());
   outFormat->red= inFormat->color.red();
   outFormat->green= inFormat->color.green();
   outFormat->blue= inFormat->color.blue();
  
   if ( inFormat->super )
   {
      outFormat->super = new vtkTextActorFormat();
      CopyFormats(outFormat->super,inFormat->super);
   }
   else
    outFormat->super=0;

   if ( inFormat->next )
   {
      outFormat->next = new vtkTextActorFormat();
      CopyFormats(outFormat->next,inFormat->next);
   }
   else
      outFormat->next =0;

   if ( inFormat->sub )
   {
      outFormat->sub=new vtkTextActorFormat();
      CopyFormats(outFormat->sub,inFormat->sub);
   }
   else
      outFormat->sub=0;


};

int VtkColorBar::set_clip_location (double x1, double y1, double x2, double y2)
{
   if (scalarBar)  scalarBar->SetClipLocations (x1,y1,x2,y2);

   return ERR_STATUS_GOOD;
}

void VtkColorBar::select ()
{
#ifndef HEADLESS
  if (scalarBarWidget)
    scalarBarWidget->Select (true);
#endif
}

void VtkColorBar::unselect ()
{
#ifndef HEADLESS
  if (scalarBarWidget)
    scalarBarWidget->Select (false);
#endif
}

void VtkColorBar::select_title ()
{
#ifndef HEADLESS
  if (labelWidget)
    labelWidget->Select (true);
#endif
}

void VtkColorBar::unselect_title ()
{
#ifndef HEADLESS
  if (labelWidget)
    labelWidget->Select (false);
#endif
}



void VtkColorBar::disable_interaction ()
{
#ifndef HEADLESS
  vtkChipsScalarBarRepresentation *rep = reinterpret_cast<vtkChipsScalarBarRepresentation *> (this->scalarBarWidget->GetRepresentation ());
  vtkChipsTextRepresentation *titleRep = reinterpret_cast<vtkChipsTextRepresentation *> (this->labelWidget->GetRepresentation ());
  rep->DisableBorder ();
  titleRep->DisableBorder ();

  this->scalarBarWidget->EnabledOff ();
  this->labelWidget->EnabledOff ();
#endif


}

void VtkColorBar::enable_interaction ()
{
#ifndef HEADLESS

  if (this->is_interaction_blocked ())
      return;

  vtkChipsScalarBarRepresentation *rep = reinterpret_cast<vtkChipsScalarBarRepresentation *> (this->scalarBarWidget->GetRepresentation ());
  vtkChipsTextRepresentation *titleRep = reinterpret_cast<vtkChipsTextRepresentation *> (this->labelWidget->GetRepresentation ());

  this->scalarBarWidget->EnabledOn ();
  this->labelWidget->EnabledOn ();
  rep->EnableBorder ();
  titleRep->EnableBorder ();
#endif
}

int VtkColorBar::compute_resize_title (double *bounds)
{
  double outPosition[2];
  vtkTextActorExtra* title = this->scalarBar->GetTitleActor ();
  int font = VtkLabelInterface::ComputeFontSizeAndPosFromBBox (title, renderer, bounds, outPosition);
  int rval = this->set_property(static_cast<void*>(&font), SET_CBAR_LABEL_SIZE);

  COORD_SYS coord_sys = PIXEL_REAL;
  string coord_id = "";

  if (!rval)
     this->move_object (chips_colorbar_label, coord_sys, coord_id, outPosition[0], outPosition[1], V_MOVE_ABSOLUTE);

  return rval;

}


