// 
//  Copyright (C) 2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKCOLORBAR_HH
#define VTKCOLORBAR_HH


#include "vtkRenderer.h"
#include "vtkActor.h"

#include "ChipsColorBar.hh"

class vtkScalarBarActorExtra;
class vtkTextActorFormat;
#ifndef HEADLESS
class vtkChipsScalarBarWidget;
class vtkChipsTextWidget;
#endif

class VtkColorBar : public ChipsColorBar
{
  public:

   VtkColorBar(vtkRenderer* inRenderer, ChipsObject* par, 
                     ChipsCoordinate *inCoord,
                     COORD_SYS coord_sys, double xpos, double ypos,
                     unsigned short cmap_id, string val);
   VtkColorBar(vtkRenderer* inRenderer, ChipsObject* par, 
                     ChipsCoordinate *inCoord,
                     COORD_SYS coord_sys, ChipsStateStream& in);
   VtkColorBar(vtkRenderer* inRenderer, ChipsObject* par, 
                     ChipsCoordinate *inCoord,
                     COORD_SYS coord_sys, double xpos, double ypos,
                     string val);

   virtual ~VtkColorBar();

   virtual int  apply_transform () { return ERR_FEATURE_DNE;};
   virtual int change_depth();
   virtual int change_position();
   virtual int change_colormap();

   virtual int change_ticklabel_color();
   virtual int change_ticklabel_font();
   virtual int change_ticklabel_size();
   virtual int change_ticklabel_angle();
   virtual int change_ticklabel_offset();
   virtual int change_ticklabel_visible();
   virtual int change_ticklabel_alignment();

   virtual int change_tick_style();
   virtual int change_tick_color();
   virtual int change_tick_thickness();
   virtual int change_tick_length();
   virtual int change_tick_mode();
   virtual int change_tick_location();
   virtual int change_tick_visible ();

   virtual int change_title_text ();
   virtual int change_title_color();
   virtual int change_title_font();
   virtual int change_title_fontstyle();
   virtual int change_title_size();
   virtual int change_title_angle();
   virtual int change_title_visible();
   virtual int change_title_alignment();
   virtual int change_title_location();
   virtual int change_parallel_offset();
   virtual int change_perpendicular_offset();
  


   virtual int change_visible();
   virtual int change_size();               // height or length
   virtual int change_alignment();          // halign or valign
   virtual int change_orientation ();
   virtual int change_bar_flip ();

   virtual int change_border_visible();

   virtual int set_clip_location (double x1, double y1, double x2, double y2);

   virtual void select ();
   virtual void unselect ();

   virtual void select_title ();
   virtual void unselect_title ();

   virtual void disable_interaction ();
   virtual void enable_interaction ();

   virtual int  compute_resize_title (double *bounds);



/*  protected:
   int change_style();
*/


  private:

   VtkColorBar(const VtkColorBar& inColorBar);

   vtkRenderer* renderer;

   vtkScalarBarActorExtra* scalarBar;

#ifndef HEADLESS
   vtkChipsScalarBarWidget *scalarBarWidget;
   vtkChipsTextWidget *labelWidget;
#endif

   void CopyFormats(vtkTextActorFormat* outFormat, ELEM_P_T inFormat);

   string ref_id;


};





#endif
