#ifndef VTKCOLORMAP_HH
#define VTKCOLORMAP_HH

#include "ChipsColorMap.hh"
#include "ChipsStateStream.hh"

#include <map>
#include <list>

using namespace std;

class vtkLookupTable;

class VtkColorMap : public ChipsColorMap
{
   public:
      enum TABLETYPE {HSV, RGB, TABLE};

      VtkColorMap ();
      virtual ~VtkColorMap ();

      virtual void set_table_size (int size);
      virtual int  get_table_size ();

      virtual void set_alpha_position (double position, double value);
      virtual void set_alpha_range (double min, double max);
      virtual void set_alpha_range (double alpha_range[]);
      virtual void get_alpha_range (double alpha_range[]);

      virtual void set_color_scale (int scale);
      virtual int  get_color_scale ();

      virtual void set_color_ramp (int ramp);
      virtual int  get_color_ramp ();

      virtual void set_scalar_threshold (double threshold[]);
      virtual void get_scalar_threshold (double threshold[]);

      virtual void set_scalar_range (double range[]);
      virtual void get_scalar_range (double range[]);

      virtual void clear_positions (int position=15);

      virtual void set_blue_position (double position, double value);
      virtual void set_green_position (double position, double value);
      virtual void set_red_position (double position, double value);

      virtual void set_blue_range (double min, double max);
      virtual void set_green_range (double min, double max);
      virtual void set_red_range (double min, double max);

      virtual void set_hue_position (double position, double value);
      virtual void set_saturation_position (double position, double value);
      virtual void set_value_position (double position, double value);

      virtual void set_hue_range (double min, double max);
      virtual void set_value_range (double min, double max);
      virtual void set_saturation_range (double min, double max);

      virtual void set_table (double **table, int table_size);

      virtual void set_data_range (double min, double max);

      virtual void invert_table ();

      virtual void update ();

      virtual void serialize (ChipsStateStream& out);
      virtual void deserialize (ChipsStateStream& in);

      vtkLookupTable* get_lookup_table () { return this->colorTable; };

      virtual unsigned char * get_computed_table (int &out_size);


   protected:
      double linear_interpolate (double position1, double position2,
                                 double value1, double value2, double index);
      void   build_inverted_table ();
      void   set_table_type (TABLETYPE type);

      vtkLookupTable* colorTable;

      TABLETYPE tableType;
      bool tableTypeChanged;

      list <pair <double, double> > rgbaMap[4]; // hsva and rgba position map

      double **dataTable;
      int tableSize;

      bool built;

      int colorRamp;
      int colorScale;
      bool alphaOverride;
      bool thresholdSet;
      double scalarRange[2];
};

#endif
