// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <vector>

#include "VtkContour.hh"
#include "vtkDoubleArray.h"
#include "vtkPoints.h"
#include "vtkPolyData.h"
#include "vtkPointData.h"
#include "vtkGlyphSource2D.h"
#include "vtkProperty2D.h"
#include "vtkDoubleArray.h"
#include "vtkAxisActor2DExtra.hh"
#include "vtkPoints.h"
#include "vtkDoubleArray.h"
#include "vtkChipsTransform.hh"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "VtkUtils.hh"

#ifndef HEADLESS
#include "vtkChipsPickableWidget.hh"
#include "vtkPlotContourRepresentation.hh"
#endif

#include <sstream>

#include "ChipsCmdManager.hh"

extern ChipsCmdManager* cmd_manager;



VtkContour::VtkContour(vtkRenderer* inRenderer,
		   ChipsObject* par, 
		   unsigned long cnt, double* x, double* y,double* z,
		   string transform_id, const string& cid):
                   ChipsContour(par,cnt,x,y,z,transform_id,cid)
{
  if (!inRenderer){
     cmd_manager->add_err (id, 0);
     status = ERR_STATUS_BAD;
     return;
  }

  renderer = inRenderer;

  contourData= vtkPlotContourActor2DExtra::New();
  renderer->AddActor (contourData);

#ifndef HEADLESS
  // setup gui widget
  vtkPlotContourRepresentation *contourDataRep = vtkPlotContourRepresentation::New ();
  ChipsId chipsId;
  contourDataRep->SetPlotContourActor (contourData);

  this->populate_chips_id (chipsId);
  contourDataWidget = vtkChipsPickableWidget::New ();
  contourDataWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  contourDataWidget->SetRepresentation (contourDataRep);
  contourDataWidget->SetIdInfo (chipsId, this->get_object_type());
  contourDataWidget->ResizableOff ();
  contourDataWidget->RotateableOff ();
  contourDataWidget->MoveableOff ();
  contourDataWidget->SetDefaultRenderer (renderer);
  contourDataWidget->EnabledOn ();
  contourDataRep->Delete ();
#endif

  setup_data ();


  change_tick_property();
  change_label_property();
  change_thickness();
  change_style();
  change_color(); 
  set_plot_location (0,0,1,1);
  
};


VtkContour::VtkContour(vtkRenderer *inRenderer, ChipsObject* par, 
		   ChipsStateStream& in):ChipsContour(par,in)
{
  if (!inRenderer){
     status = ERR_STATUS_BAD;
     return;
  }

  renderer = inRenderer;

  contourData= vtkPlotContourActor2DExtra::New();
  renderer->AddActor (contourData);

#ifndef HEADLESS
  // setup gui widget
  vtkPlotContourRepresentation *contourDataRep = vtkPlotContourRepresentation::New ();
  ChipsId chipsId;
  contourDataRep->SetPlotContourActor (contourData);

  this->populate_chips_id (chipsId);
  contourDataWidget = vtkChipsPickableWidget::New ();
  contourDataWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  contourDataWidget->SetRepresentation (contourDataRep);
  contourDataWidget->SetIdInfo (chipsId, this->get_object_type());
  contourDataWidget->ResizableOff ();
  contourDataWidget->RotateableOff ();
  contourDataWidget->MoveableOff ();
  contourDataWidget->SetDefaultRenderer (renderer);
  contourDataWidget->EnabledOn ();
  contourDataRep->Delete ();
#endif

  setup_data ();

  change_tick_property();
  change_label_property();
  change_thickness();
  change_style();
  change_color(); 
  change_visible ();
  set_plot_location (0,0,1,1);

};



VtkContour::VtkContour(vtkRenderer *inRenderer,
		       ChipsObject* par, 
		       unsigned long cnt, double* x, double* y,double* z,
		       unsigned int numlev, double *inLevels, string transform_id,
		       const string& cid):ChipsContour(par,cnt,x,y,z,numlev,inLevels,transform_id,cid)
{   
  if (!inRenderer){
     status = ERR_STATUS_BAD;
     return;
  }

  renderer = inRenderer;

  contourData= vtkPlotContourActor2DExtra::New();
  renderer->AddActor (contourData);

#ifndef HEADLESS
  // setup gui widget
  vtkPlotContourRepresentation *contourDataRep = vtkPlotContourRepresentation::New ();
  ChipsId chipsId;
  contourDataRep->SetPlotContourActor (contourData);

  this->populate_chips_id (chipsId);
  contourDataWidget = vtkChipsPickableWidget::New ();
  contourDataWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  contourDataWidget->SetRepresentation (contourDataRep);
  contourDataWidget->SetIdInfo (chipsId, this->get_object_type());
  contourDataWidget->ResizableOff ();
  contourDataWidget->RotateableOff ();
  contourDataWidget->MoveableOff ();
  contourDataWidget->SetDefaultRenderer (renderer);
  contourDataWidget->EnabledOn ();
  contourDataRep->Delete ();
#endif


  setup_data ();

  change_tick_property();
  change_label_property();
  change_thickness();
  change_style();
  change_color(); 
  set_plot_location (0,0,1,1);


};

VtkContour::~VtkContour()
{
  renderer->RemoveActor (contourData);
  contourData->Delete ();
#ifndef HEADLESS
  contourDataWidget->Delete ();
#endif
};


int VtkContour::change_thickness()
{
  contourData->GetLineActor()->GetProperty()->SetLineWidth(thickness);
  return ERR_STATUS_GOOD;
}; 

int VtkContour::change_style()
{

  contourData->SetVisibility(1);
  switch(style)
    {
    case chips_noline:
      contourData->SetVisibility(0);
      break;
    case chips_dotlongdash: 
      contourData->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dotlongdash_pattern);
      break;
    case chips_dotshortdash: 
      contourData->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dotshortdash_pattern);
      break;
    case chips_longdash: 
      contourData->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_longdash_pattern);
      break;
    case chips_shortdash:
      contourData->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_shortdash_pattern);
      break;
    case chips_shortdashlongdash:
      contourData->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_shortdashlongdash_pattern);
      break;
    case chips_dot:
      contourData->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dot_pattern);
      break;
    case chips_solid:
    default:
      contourData->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_solid_pattern);

    };

  return ERR_STATUS_GOOD;
}; 

int VtkContour::change_color()
{  
  contourData->GetLineActor()->GetProperty()->SetColor(color.red(),color.green(),color.blue());
  
  return ERR_STATUS_GOOD;
}; 

int VtkContour::change_levels()
{
     
  setup_data (); 
  change_tick_property();
  change_label_property();
  change_thickness();
  change_style();
  change_color();

  return ERR_STATUS_GOOD;
}; 


int VtkContour::change_tick_property()
{
  return ERR_STATUS_BAD;
}; 


int VtkContour::change_label_property()
{
  return ERR_STATUS_BAD;
}; 


void VtkContour::create_griddata(vtkStructuredGrid* outData,
				 bool inRegularFlag,
				 unsigned long inXSize,unsigned long inYSize,
				 unsigned long inSize,double* inX,double* inY,double* inZ)
{
  
  //For now assuming regular spaced data only
  
  
  if ( inRegularFlag)
    {
      
      vtkPoints *unstrPts = vtkPoints::New();
      vtkDoubleArray*  scalars = vtkDoubleArray::New();
      
      
      
      for(unsigned long indx=0;indx<(inXSize*inYSize);indx++)
	{	
	  unstrPts->InsertNextPoint(inX[indx],inY[indx],0);
	  scalars->InsertNextValue(inZ[indx]);
	};
      
      
      
      outData->SetDimensions(inXSize,inYSize,1);
      outData->SetPoints(unstrPts);
      outData->GetPointData()->SetScalars(scalars);
      outData->Modified();
      
      unstrPts->Delete();
      scalars->Delete();
    } 
  else // Non- regular data will be handled later
    {
      outData->SetDimensions(0,0,0); 
      outData->SetPoints(0);
      outData->GetPointData()->SetScalars(0);
      outData->Modified();
    }

  

};

int VtkContour::change_depth()
{
   contourData->SetLayerNumber (depth*10+1);
   return ERR_STATUS_GOOD;
}


int VtkContour::change_visible ()
{
   int tmp_vis = (!get_parent()->is_visible ()) ? 0 : is_visible();
   contourData->SetVisibility(tmp_vis);

#ifndef HEADLESS
   contourDataWidget->GetRepresentation ()->SetVisibility (tmp_vis);
#endif


   return ERR_STATUS_GOOD;

}

int VtkContour::update_data ()
{
   if ( contourData)
    {
       setup_data ();
    }
    return ERR_STATUS_GOOD;
};

int VtkContour::change_algorithm()
{
  setup_data ();
  change_tick_property();
  change_label_property();
  change_thickness();
  change_style();
  change_color();

  return ERR_STATUS_GOOD;
};

int VtkContour::set_plot_location (double x1, double y1, double x2, double y2)
{
   double pos1[] = {x1,y1};
   double pos2[] = {x2,y2};
   VtkUtils::FrameNorm2ViewportNorm (coord, renderer, pos1);
   VtkUtils::FrameNorm2ViewportNorm (coord, renderer, pos2);

   contourData->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
   contourData->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();

   contourData->GetPositionCoordinate()->SetValue(pos1[0],pos1[1]);
   contourData->GetPosition2Coordinate()->SetValue(pos2[0], pos2[1]);

   return ERR_STATUS_GOOD;
}

int VtkContour::change_range (const ChipsVector &low_range, const ChipsVector &high_range)
{
  double tmpRangeX[2];
  double tmpRangeY[2];

  tmpRangeX[0] = low_range[0];
  tmpRangeY[0] = low_range[1];
  tmpRangeX[1] = high_range[0];
  tmpRangeY[1] = high_range[1];


  // Vtk does checking to see if the ranges are the same before updating
  // the actors. Actors wont get rerendered if ranges do not change.
  contourData->SetXRange(tmpRangeX);
  contourData->SetYRange(tmpRangeY);

#ifndef HEADLESS
  contourDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;
}

int VtkContour::setup_data ()
{
  vtkStructuredGrid* data= vtkStructuredGrid::New();
  create_griddata(data,is_regular(),get_x_gridsize(),get_y_gridsize(),num_pts,xx,yy,zz);

  vector<double> tmpLevels(num_levels);
  for(unsigned int indx=0;indx<num_levels;indx++)
    tmpLevels[indx]=levels[indx];

  int tmpAlgorithm;
  string tmp_coord_world;

  switch(algorithm)
    {
    case chips_marching :
      tmpAlgorithm=1;
      break;
    case chips_standard :
    default:
      tmpAlgorithm=0;
      break;
    };

  contourData->Generate(data,tmpAlgorithm,tmpLevels);

  xmin = contourData->GetXDataRange ()[0];
  xmax = contourData->GetXDataRange ()[1];
  ymin = contourData->GetYDataRange ()[0];
  ymax = contourData->GetYDataRange ()[1];

#ifndef HEADLESS
  contourDataWidget->UpdateRepresentation ();
#endif

  data->Delete();

  return ERR_STATUS_GOOD;

}

int VtkContour::add_transform (Transform *transform, bool inverse)
{
  if (transform != NULL)
    {
      vtkChipsTransform *vtkTrans = vtkChipsTransform::New ();
      vtkTrans->SetChipsTransform (transform);
      if (inverse)
         vtkTrans->Inverse ();
      contourData->ApplyTransform (vtkTrans);
      vtkTrans->Delete ();
    }

  xmin = contourData->GetXDataRange ()[0];
  xmax = contourData->GetXDataRange ()[1];
  ymin = contourData->GetYDataRange ()[0];
  ymax = contourData->GetYDataRange ()[1];

  return ERR_STATUS_GOOD;
}

int VtkContour::remove_transforms ()
{
  contourData->RemoveAllTransforms ();
  contourData->ResetContourData ();

  xmin = contourData->GetXDataRange ()[0];
  xmax = contourData->GetXDataRange ()[1];
  ymin = contourData->GetYDataRange ()[0];
  ymax = contourData->GetYDataRange ()[1];
  return ERR_STATUS_GOOD;
}

int VtkContour::get_levels (vector <double> &levs)
{
  if (!contourData)
    return ERR_STATUS_BAD;

  contourData->GetLevels (levs);

  return ERR_STATUS_GOOD;

}

void VtkContour::select ()
{
#ifndef HEADLESS
  if (contourDataWidget)
    contourDataWidget->Select (true);
#endif
}

void VtkContour::unselect ()
{
#ifndef HEADLESS
  if (contourDataWidget)
    contourDataWidget->Select (false);
#endif
}

void VtkContour::disable_interaction ()
{
#ifndef HEADLESS
  vtkPlotContourRepresentation *rep = reinterpret_cast<vtkPlotContourRepresentation *> (this->contourDataWidget->GetRepresentation ());
  rep->DisableBorder ();
  this->contourDataWidget->EnabledOff ();
#endif
}

void VtkContour::enable_interaction ()
{
#ifndef HEADLESS
  if (this->is_interaction_blocked ())
    return;

  vtkPlotContourRepresentation *rep = reinterpret_cast<vtkPlotContourRepresentation *> (this->contourDataWidget->GetRepresentation ());

  this->contourDataWidget->EnabledOn ();
  rep->EnableBorder ();
#endif
}


