// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKCONTOUR_HH
#define VTKCONTOUR_HH

#include <string>
#include <vector>

#include "vtkRenderer.h"
#include "vtkActor.h"

#include "ChipsContour.hh"
#include "vtkPlotContourActor2DExtra.hh"
#include "vtkRenderer.h"
#include "vtkStructuredGrid.h"

using namespace std;

#ifndef HEADLESS
class vtkChipsPickableWidget;
#endif

class VtkContour: public ChipsContour
{
public:

  VtkContour(vtkRenderer* inRenderer,
	     ChipsObject* par, 
	     unsigned long cnt, double* x, double* y, double* z,
	     string transform_id, const string& cid=NULL);

  VtkContour(vtkRenderer* inRenderer, ChipsObject* par, ChipsStateStream& in);

  VtkContour(vtkRenderer* inRenderer,
	   ChipsObject* par, 
	   unsigned long cnt, double* x, double* y,double* z,
	   unsigned int numlev, double *inLevels,
	   string transform_id, const string& cid=NULL);

  virtual ~VtkContour();
  
  virtual int change_levels(); 
  virtual int change_tick_property(); 
  virtual int change_label_property(); 
  virtual int change_thickness(); 
  virtual int change_style(); 
  virtual int change_color(); 
  virtual int change_depth(); 
  virtual int change_visible ();
  virtual int change_algorithm();
  virtual int update_data ();
  virtual int set_plot_location (double x1, double y1, double x2, double y2);
  virtual int change_range (const ChipsVector &low_range,const  ChipsVector &high_range);
  virtual int get_levels (vector<double> &levs);

  virtual void select ();
  virtual void unselect ();

  virtual void disable_interaction ();
  virtual void enable_interaction ();


protected:

  virtual int add_transform (Transform *transform, bool inverse=false);
  virtual int remove_transforms ();

private:
  int setup_data ();

  VtkContour(const VtkContour& inContour);


  //
  // DESCRIPTION: Put data into structured grid
  //
  //
  //
  // PRE: outData - initialized structured grid structure
  //      inRegularFlag - is input data regularly spaced
  //      inXSize - x size of structured grid       
  //      inYSize - y size of structured grid
  //      inSize - size of input data 
  //      inX - input data x coordinates vector
  //      inY - input data y coordinates vector
  //      inX - input data z coordinates vector
  //
  // POST: outData contains structured grid of size inXSize x inYSize
  //       that contains points from inX,inY,inZ vectors.
  //       In the future if input data is not regular it will be interpolated
  //       to fit into structured grid.
  //

  void create_griddata(vtkStructuredGrid* outData,
		       bool inRegularFlag,
		       unsigned long inXSize,unsigned long inYSize,
		       unsigned long inSize,double* inX,double* inY,double* inZ);

//  vtkXYPlotActorExtra* plotActor;
  vtkPlotContourActor2DExtra *contourData;
  vtkRenderer *renderer;
  string ref_id;

#ifndef HEADLESS
  vtkChipsPickableWidget *contourDataWidget;
#endif


};





#endif
