// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKCURVE_HH
#define VTKCURVE_HH

#include <string>
#include <list>

#include "vtkRenderer.h"
#include "vtkActor.h"

#include "ChipsCurve.hh"
#include "vtkPlotDataListActor2DExtra.hh"

using namespace std;

#ifndef HEADLESS
class vtkChipsPickableWidget;
#endif

class VtkCurve: public ChipsCurve
{
public:

  VtkCurve(vtkRenderer *inRenderer,
	   ChipsObject* par, 
	   unsigned long cnt, double* x, double* y,
	   const string &transform_id, const string& cid=NULL, 
           bool compute=true);

  VtkCurve(vtkRenderer *inRenderer,
	   ChipsObject* par, 
	   unsigned long cnt, double* x, double* y,
	   double* ydown, double* yup, double* xdown, double* xup,
           double* ylims, double* xlims,
           const string& transform_id,
	   const string& cid=NULL, bool compute=true);

  VtkCurve(vtkRenderer *inRenderer, ChipsObject* par, ChipsStateStream& in);

  virtual ~VtkCurve();

  virtual int render_err();
  virtual int change_linethickness(); 
  virtual int change_linestyle(); 
  virtual int change_linecolor(); 
  virtual int change_symbolsize(); 
  virtual int change_symbolstyle(); 
  virtual int change_symbolcolor();
  virtual int change_symbolfilled(); 
  virtual int change_errcolor(); 
  virtual int change_errstyle(); 
  virtual int change_errup(); 
  virtual int change_errdown(); 
  virtual int change_errleft(); 
  virtual int change_errright(); 
  virtual int change_errthickness(); 
  virtual int change_visible ();
  virtual int change_limitslength();
  virtual int change_limitsoverride();
  virtual int update_data ();
  virtual int set_plot_location (double x1, double y1, double x2, double y2);
  virtual int change_range (const ChipsVector &low_range,const  ChipsVector &high_range);
  virtual int change_scale (int inXScale, int inYScale);
  virtual int change_depth ();

  virtual void select ();
  virtual void unselect ();

  virtual void disable_interaction ();
  virtual void enable_interaction ();

protected:


private:
  VtkCurve(const VtkCurve& inCurve);
  void adjust_plotlimits(const string& inXAxisId,
			 const string& inYAxisId);


  int create_curvedata(long inPlotDataId, 
		       vtkDoubleArray* inUpErr,
		       vtkDoubleArray* inDownErr,
		       vtkDoubleArray* inLeftErr,
		       vtkDoubleArray* inRightErr,
		       vtkDoubleArray* inYLim,
		       vtkDoubleArray* inXLim,
		       CHIPS_RANGE_P inRangeInfo);

  vtkPlotDataListActor2DExtra *plotData;
  vtkRenderer *renderer;

#ifndef HEADLESS
  vtkChipsPickableWidget *plotDataWidget;
#endif


};





#endif
