// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "VtkEngine.hh" 
#include "VtkWindow.hh"
#include "VtkColorMapTable.hh"

#include "vtkGL2PSExporter.h"
#include "vtkFreeTypeUtilitiesExtra.hh"
#include "ChipsCmdManager.hh"

extern ChipsCmdManager* cmd_manager;
extern ChipsColorMapTable* colormap_table;

VtkEngine::VtkEngine(bool load_prefs):ChipsEngine(load_prefs)
{
  vtkGL2PSExporter::SetGlobalLineWidthFactor(1.0);

  if (colormap_table)
     delete colormap_table;

  colormap_table = new VtkColorMapTable ();
}

VtkEngine::~VtkEngine()
{

//  vtkFreeTypeFontCacheExtra::GetInstance()->ReleaseCache();
}

bool VtkEngine::verify_fontfile_exists(string fontpath)
{
   bool result = false; 
   if (fontpath.length())
   {
      result = vtkFreeTypeUtilitiesExtra::TryFont((char*)fontpath.c_str());
   }
   return result; 
}

ChipsWindow* VtkEngine::new_window(string id)
{
   string tmpid;
 
   if (id.length())
      tmpid = id;
   else
      tmpid =  create_window_id();

   return new VtkWindow (this, tmpid);
}


ChipsWindow* VtkEngine::new_window(ChipsStateStream& in)
{
   return new VtkWindow (this, in);
}

ChipsWindow* VtkEngine::new_window(float width, float height,
                                 string format,
                                 string window_id)
{
   string tmpid;
 
   if (window_id.length())
      tmpid = window_id;
   else
      tmpid =  create_window_id();

   return new VtkWindow(this, width, height, format, tmpid);

}

