// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKFRAME_HH
#define VTKFRAME_HH

#include <string>
#include <list>


#include "ChipsFrame.hh" 


using namespace std;

#ifndef HEADLESS
class vtkChipsWidget;
class vtkChipsRepresentation;
#endif 

class vtkRenderWindow;
class vtkRenderer;
class vtkActor2D;
class vtkPolyData;

class VtkFrame: public ChipsFrame
{
  public:

   VtkFrame(vtkRenderWindow* inRenWin, ChipsObject* par, int pos, string val="");
   VtkFrame(vtkRenderWindow* inRenWin, ChipsObject* par, int pos, ChipsStateStream& in);
   VtkFrame(vtkRenderWindow* inRenWin, ChipsObject* par, int pos, float x1, float x2, 
			       float y1, float y2, 
			       string val="");

   virtual ~VtkFrame();

   virtual int set_order (int order_num);
   virtual int change_position();
   virtual int change_color();
   virtual int change_border_color();
   virtual int change_border_visibility();
   virtual int change_transparency();
   virtual int change_visible ();
   virtual int deserialize_depth_table(ChipsStateStream& in);

   virtual int redraw();

   virtual ChipsPlot* new_plot(string plid="");
   virtual ChipsPlot* new_plot(ChipsStateStream& in);
   virtual ChipsPlot* new_plot(float posx1, float posy1, float posx2,
      float posy2, string plid="");

  virtual void select ();
  virtual void unselect ();

  virtual void add_grid (vtkActor2D *grid);

  virtual void disable_interaction ();
  virtual void enable_interaction ();


  protected:

   virtual void change_mode (int mode);

   void initialize (vtkRenderWindow* inRenWin, int pos );
   void create_plane ();
   void create_bounding_box ();
   void insert_renderer (vtkRenderer *new_ren, vtkRenderWindow *renWin, int pos);
   void remove_renderer ();


 
   vtkRenderer *frameRenderer;
   vtkActor2D *planeActor;
   vtkActor2D *boundingBoxActor;

   vtkPolyData *plane_data;
   vtkPolyData *box_data;
   int plane_height;
   int plane_width;

#ifndef HEADLESS
  vtkChipsWidget *frame_widget;
  vtkChipsRepresentation *frame_rep;
#endif


};

#endif 
