// 
//  Copyright (C) 2005-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <sstream>
#include "VtkHistogram.hh"
#include "vtkDoubleArray.h"
#include "vtkPoints.h"
#include "vtkPolyData.h"
#include "vtkPointData.h"
#include "vtkGlyphSource2D.h"
#include "vtkProperty2D.h"
#include "vtkDoubleArray.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkAxisActor2DExtra.hh"
#include "ChipsFillPattern.hh"

#include "VtkUtils.hh"

#ifndef HEADLESS
#include "vtkChipsPickableWidget.hh"
#include "vtkHistogramDataRepresentation.hh"
#endif


extern ChipsFillPattern *fill_patterns;

VtkHistogram::VtkHistogram(vtkRenderer *inRenderer,
			   ChipsObject* par, 
			   unsigned long cnt, 
			   double* bin_lo,double* bin_hi, double* y,
			   const string& cid):ChipsHistogram(par,cnt,bin_lo,bin_hi,y,cid)
{
  long index = 0;
  renderer = inRenderer;
  histogramData = vtkHistogramDataListActor2DExtra::New ();
  if ( renderer)
    { 

      renderer->AddActor (histogramData);

#ifndef HEADLESS
      // setup gui widget
      vtkHistogramDataRepresentation *histogramDataRep = vtkHistogramDataRepresentation::New ();
      ChipsId chipsId;
      histogramDataRep->SetHistogramDataActor (histogramData);

      this->populate_chips_id (chipsId);
      histogramDataWidget = vtkChipsPickableWidget::New ();
      histogramDataWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      histogramDataWidget->SetRepresentation (histogramDataRep);
      histogramDataWidget->SetIdInfo (chipsId, this->get_object_type());
      histogramDataWidget->ResizableOff ();
      histogramDataWidget->RotateableOff ();
      histogramDataWidget->MoveableOff ();
      histogramDataWidget->SetDefaultRenderer (renderer);
      histogramDataWidget->EnabledOn ();
      histogramDataRep->Delete ();
#endif

      
      for (list<CHIPS_RANGE_P>::const_iterator tmpRangePtr=indices.begin(); tmpRangePtr!=indices.end();tmpRangePtr++, index++)
	{

	  if ( create_histogramdata(index,0,0,0,0,*tmpRangePtr) != ERR_STATUS_GOOD)
	    {
	      cerr<<"Error: VtkHistogram::VtkHistogram - AddData Failed"<<endl;
	    }
	};
	  
      change_linethickness();
      change_linestyle();
      change_linecolor(); 
      change_symbolsize(); 
      change_symbolstyle(); 
      change_symbolcolor();
      change_dropline();
      change_errcolor();
      change_errstyle();
      change_errup();
      change_errdown();
      change_errleft();
      change_errright();
      change_errthickness();
      change_visible();
      change_fillopacity ();
      change_fillcolor ();
      change_fillstyle(); 
    }

};


VtkHistogram::VtkHistogram(vtkRenderer *inRenderer, ChipsObject* par,  
			   ChipsStateStream& in): ChipsHistogram(par,in)
{

  renderer = inRenderer;
  histogramData = vtkHistogramDataListActor2DExtra::New ();
  if ( renderer)
    {
      long index = 0;
      vtkDoubleArray* upErr=vtkDoubleArray::New();
      vtkDoubleArray* downErr=vtkDoubleArray::New();

      renderer->AddActor (histogramData);

#ifndef HEADLESS
      // setup gui widget
      vtkHistogramDataRepresentation *histogramDataRep = vtkHistogramDataRepresentation::New ();
      ChipsId chipsId;
      histogramDataRep->SetHistogramDataActor (histogramData);

      this->populate_chips_id (chipsId);
      histogramDataWidget = vtkChipsPickableWidget::New ();
      histogramDataWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      histogramDataWidget->SetRepresentation (histogramDataRep);
      histogramDataWidget->SetIdInfo (chipsId, this->get_object_type());
      histogramDataWidget->ResizableOff ();
      histogramDataWidget->RotateableOff ();
      histogramDataWidget->MoveableOff ();
      histogramDataWidget->SetDefaultRenderer (renderer);
      histogramDataWidget->EnabledOn ();
      histogramDataRep->Delete ();
#endif


      for (list<CHIPS_RANGE_P>::const_iterator tmpRangePtr=indices.begin(); tmpRangePtr!=indices.end();tmpRangePtr++, index++)
	 {
	   if ( create_histogramdata(index,upErr,downErr,0,0,*tmpRangePtr)!= ERR_STATUS_GOOD)
	     {
	       cerr<<"Error: VtkHistogram::VtkHistogram - AddData Failed"<<endl;
	     }
	 };

      change_linethickness();
      change_linestyle();
      change_linecolor(); 
      change_symbolsize(); 
      change_symbolstyle(); 
      change_symbolcolor();
      change_dropline();
      change_errcolor();
      change_errstyle();
      change_errup();
      change_errdown();
      change_errleft();
      change_errright();
      change_errthickness();
      change_visible();
      change_fillopacity ();
      change_fillcolor ();
      change_fillstyle(); 

      upErr->Delete ();
      downErr->Delete ();
      
    };
};



VtkHistogram::VtkHistogram(vtkRenderer *inRenderer,
			   ChipsObject* par, 
			   unsigned long cnt, 
			   double* bin_lo,double* bin_hi, double* y,
			   double* ydown, double* yup,
			   const string& cid):ChipsHistogram(par,cnt,bin_lo,bin_hi,y,ydown,yup,cid)
{   
  renderer = inRenderer;
  histogramData = vtkHistogramDataListActor2DExtra::New ();

  if ( renderer)
    {
      long index=0;
      vtkDoubleArray * upErr=vtkDoubleArray::New();
      vtkDoubleArray * downErr=vtkDoubleArray::New();

      renderer->AddActor (histogramData);

#ifndef HEADLESS
      // setup gui widget
      vtkHistogramDataRepresentation *histogramDataRep = vtkHistogramDataRepresentation::New ();
      ChipsId chipsId;
      histogramDataRep->SetHistogramDataActor (histogramData);

      this->populate_chips_id (chipsId);
      histogramDataWidget = vtkChipsPickableWidget::New ();
      histogramDataWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      histogramDataWidget->SetRepresentation (histogramDataRep);
      histogramDataWidget->SetIdInfo (chipsId, this->get_object_type());
      histogramDataWidget->ResizableOff ();
      histogramDataWidget->RotateableOff ();
      histogramDataWidget->MoveableOff ();
      histogramDataWidget->SetDefaultRenderer (renderer);
      histogramDataWidget->EnabledOn ();
      histogramDataRep->Delete ();
#endif


      for (list<CHIPS_RANGE_P>::const_iterator tmpRangePtr=indices.begin(); tmpRangePtr!=indices.end();tmpRangePtr++, index++)
	{
	  if (create_histogramdata(index,upErr,downErr,0,0,*tmpRangePtr)!= ERR_STATUS_GOOD)
	    {
	       cerr<<"Error: VtkHistogram::VtkHistogram - AddHistogramData Failed"<<endl;
	    };
	};

      change_linethickness();
      change_linestyle();
      change_linecolor(); 
      change_symbolsize(); 
      change_symbolstyle(); 
      change_symbolcolor(); 
      change_dropline();
      change_errcolor(); 
      change_errstyle(); 
      change_errup(); 
      change_errdown(); 
      change_errleft(); 
      change_errright(); 
      change_errthickness(); 
      change_visible();
      change_fillopacity ();
      change_fillcolor ();
      change_fillstyle(); 
      
      upErr->Delete();
      downErr->Delete();
    };
};

VtkHistogram::~VtkHistogram()
{
  if (renderer)
    renderer->RemoveActor (histogramData);
  histogramData->Delete ();
#ifndef HEADLESS
  histogramDataWidget->Delete ();
#endif

};

int VtkHistogram::change_linethickness()
{

  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
   {
     dataPtr->GetLineActor()->GetProperty()->SetLineWidth(linethickness);
   };
  return ERR_STATUS_GOOD;

}; 

int VtkHistogram::change_linestyle()
{

  vtkHistogramDataActor2DExtra *dataPtr = NULL;

  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      switch(linestyle)
        {
        case chips_noline:
          dataPtr->SetLinesVisibility(0);
          break;
        case chips_dotlongdash:
          dataPtr->SetLinesVisibility(1);
          dataPtr->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dotlongdash_pattern);
          break;
        case chips_dotshortdash:
          dataPtr->SetLinesVisibility(1);
          dataPtr->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dotshortdash_pattern);
          break;
        case chips_longdash:
          dataPtr->SetLinesVisibility(1);
          dataPtr->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_longdash_pattern);
          break;
        case chips_shortdash:
          dataPtr->SetLinesVisibility(1);
          dataPtr->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_shortdash_pattern);
          break;
        case chips_shortdashlongdash:
          dataPtr->SetLinesVisibility(1);
          dataPtr->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_shortdashlongdash_pattern);
          break;
        case chips_dot:
          dataPtr->SetLinesVisibility(1);
          dataPtr->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dot_pattern);
          break;
        case chips_solid:
        default:
          dataPtr->SetLinesVisibility(1);
          dataPtr->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_solid_pattern);
        };
   };
   return ERR_STATUS_GOOD;

}; 

int VtkHistogram::change_linecolor()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->GetLineActor()->GetProperty()->SetColor(linecolor.red(),linecolor.green(),linecolor.blue());
    };

  return ERR_STATUS_GOOD;

}; 

int VtkHistogram::change_symbolsize()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->SetGlyphScale(symbolsize);
    };
#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;
  
}; 

int VtkHistogram::change_symbolstyle()
{

  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->SetGlyphAngle(symbolangle);
      switch (symbolstyle)
        {
        case chips_none:
          dataPtr->SetPointsVisibility(0);
          break;
        case chips_circle:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(7);
          break;
        case chips_cross:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(4);
          dataPtr->SetGlyphAngle(symbolangle + 45);
          break;
        case chips_diamond:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(8);
          break;
        case chips_downtriangle:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(5);
          dataPtr->SetGlyphAngle(symbolangle + 180);
          break;
        case chips_plus:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(4);
          break;
        case chips_square:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(6);
          break;
        case chips_uptriangle:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(5);
          break;
        case chips_arrow:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(10);
          break;
        case chips_point_type:
        default:
          dataPtr->SetPointsVisibility(1);
          dataPtr->SetGlyphType(1);
        };


      dataPtr->GetPointActor()->GetProperty()->SetColor(symbolcolor.red(),symbolcolor.green(),symbolcolor.blue());

      if ( symbolfilled)
        {
          dataPtr->SetGlyphFilled(1);
        }
      else
        {
          dataPtr->SetGlyphFilled(0) ;
        };

      dataPtr->SetGlyphScale(symbolsize);
#ifndef HEADLESS
      histogramDataWidget->UpdateRepresentation ();
#endif
    };
  return ERR_STATUS_GOOD;

}; 

int VtkHistogram::change_symbolcolor()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->GetPointActor()->GetProperty()->SetColor(symbolcolor.red(),symbolcolor.green(),symbolcolor.blue());
    };

  return ERR_STATUS_GOOD;

};



int VtkHistogram::change_symbolfilled()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      if ( symbolfilled)
        {
          dataPtr->SetGlyphFilled(1);
        }
      else
        {
          dataPtr->SetGlyphFilled(0) ;
        };
    };

  return ERR_STATUS_GOOD;
 
}; 



int VtkHistogram::change_dropline()
{

  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      if ( drop_line)
        {
          dataPtr->SetStyle(1);
        }
      else
        {
          dataPtr->SetStyle(0) ;
        };
    };

  return ERR_STATUS_GOOD;

};


int VtkHistogram::change_fillcolor()
{

  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->GetFillActor()->GetProperty()->SetColor(fillcolor.red(),fillcolor.green(),fillcolor.blue());
    };

  return ERR_STATUS_GOOD;

};


int VtkHistogram::change_fillstyle()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      switch(fillstyle)
        {
        case chips_nofill:
          dataPtr->SetFillVisibility(0);
          dataPtr->GetFillActor()->GetProperty()->SetUsePolygonStipple(0);
        break;
        case chips_solidfill:
          dataPtr->SetFillVisibility(1);
          dataPtr->GetFillActor()->GetProperty()->SetUsePolygonStipple(0);
        break;
        default:
          dataPtr->SetFillVisibility(1);
          dataPtr->GetFillActor()->GetProperty()->SetUsePolygonStipple(1);
          dataPtr->GetFillActor()->GetProperty()->SetPolygonStipplePattern(fillpattern);
        break;
        }
    }
  return ERR_STATUS_GOOD;
}

int VtkHistogram::change_fillopacity()
{

  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->GetFillActor()->GetProperty()->SetOpacity (fillopacity);
    };

  return ERR_STATUS_GOOD;

};



int VtkHistogram::change_errcolor()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->GetErrBarActor()->GetProperty()->SetColor(errcolor.red(),errcolor.green(),errcolor.blue());
    };

  return ERR_STATUS_BAD;
};
 
int VtkHistogram::change_errstyle()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      switch( errstyle)
        {
        case chips_bar:
          dataPtr->SetErrBarCapVisibility(0);
          break;
        case chips_cap:
        default:
          dataPtr->SetErrBarCapLength(errcaplen);
          dataPtr->SetErrBarCapVisibility(1);
        };
    };
#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;
 
};

int VtkHistogram::change_errup()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {

      if ( errup )
        dataPtr->SetUpErrBarVisibility(1);
      else
        dataPtr->SetUpErrBarVisibility(0);
    };
#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;

}; 

int VtkHistogram::change_errdown()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {

      if ( errdown ){
        dataPtr->SetDownErrBarVisibility(1);
      }
      else{
        dataPtr->SetDownErrBarVisibility(0);
      };
    };
#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;

}; 

int VtkHistogram::change_errleft()
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {

      if ( errleft ){
        dataPtr->SetLeftErrBarVisibility(1);
      }
      else{
        dataPtr->SetLeftErrBarVisibility(0);
      }
    };
#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;

}; 

int VtkHistogram::change_errright()
{
 
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      if ( errright ){
        dataPtr->SetRightErrBarVisibility(1);
      }
      else{
        dataPtr->SetRightErrBarVisibility(0);
      }
    };
#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;
 
}; 

int VtkHistogram::change_errthickness()
{
  if ( errthickness < 0 )
    return ERR_STATUS_BAD;
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      dataPtr->GetErrBarActor()->GetProperty()->SetLineWidth(errthickness);
    };

  return ERR_STATUS_GOOD;
  
};

int VtkHistogram::create_histogramdata(long inHistogramDataId,  
					vtkDoubleArray* inUpErr,
					vtkDoubleArray* inDownErr,
					vtkDoubleArray* inLeftErr,
					vtkDoubleArray* inRightErr,
					CHIPS_RANGE_P inRangeInfo)
{

  if ( inRangeInfo )
    {
      if ((inRangeInfo->lo > inRangeInfo->hi) || (inRangeInfo->lo<0) || (inRangeInfo->hi<0))
	{
	  return ERR_STATUS_BAD;
	}
    };

  vtkDoubleArray* binData=vtkDoubleArray::New();
  vtkPolyData* curveData = vtkPolyData::New ();

  curveData->Initialize();
  binData->Initialize();

  if ( inUpErr)
    {
      inUpErr->Initialize();
    };

  if ( inDownErr)
    {
      inDownErr->Initialize();
    };

  if ( inLeftErr)
    {
      inLeftErr->Initialize();
    };

  if ( inRightErr)
    {
      inRightErr->Initialize();
    };
  

  vtkPoints* points = vtkPoints::New();
  points->SetDataTypeToDouble();  

  int increment;
  int low,high;
  int tmpIndx=0;
  
  if ( inRangeInfo)
    {
      low=inRangeInfo->lo;
      high=inRangeInfo->hi;
      if (inRangeInfo->inclusive )
	{
	  increment=1;
	}
      else
	{
	  if ( inRangeInfo->hi==inRangeInfo->lo)
	    increment=1;
	  else
	    increment=inRangeInfo->hi-inRangeInfo->lo;
	};
    }
  else
    {
      increment=1;
      low=0;
      high=num_pts-1;
    };

  for (int i=low;i <= high; i+=increment) 
    {
      points->InsertPoint(tmpIndx++,xx[i],yy[i],0);


      if ( yup && inUpErr )
	{
	  inUpErr->InsertNextValue(yup[i]);
	}
      
      if ( ydown && inDownErr)
	{
	  inDownErr->InsertNextValue(ydown[i]);
	}
  
      if ( xup && inRightErr )
	{
	  inRightErr->InsertNextValue(xup[i]);
	}
      
      if ( xdown && inLeftErr)
	{
	  inLeftErr->InsertNextValue(xdown[i]);
	};

      if ( binlo && binhi)
        {

          binData->InsertNextValue(binlo[i]);
          binData->InsertNextValue(binhi[i]);
        }
    };
  
  
  
  curveData->SetPoints(points);
  

  // Add data to vtk actor
  histogramData->AddData (inHistogramDataId, curveData, binData, inUpErr, inDownErr, inLeftErr, inRightErr);

#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif


  points->Delete();
  curveData->Delete ();
  binData->Delete ();


  return ERR_STATUS_GOOD;

};

int VtkHistogram::change_depth()
{
  histogramData->SetLayerNumber (depth*10+1);
  return ERR_STATUS_GOOD;
}


int VtkHistogram::change_visible ()
{
  int tmp_vis = (!get_parent()->is_visible ()) ? 0 : is_visible ();

  list<CHIPS_RANGE_P>::const_iterator tmpRangePtr=indices.begin();
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      if ( tmp_vis)
        {
          if (((*tmpRangePtr)->mask & display_mask) == (*tmpRangePtr)->mask)
            {
              dataPtr->SetVisibility(tmp_vis);
            }
          else
            {
              dataPtr->SetVisibility(!tmp_vis);
            };
        }
      else
        {
            dataPtr->SetVisibility(tmp_vis);
        };
        tmpRangePtr++;
     }

#ifndef HEADLESS
   histogramDataWidget->GetRepresentation ()->SetVisibility (tmp_vis);
#endif


   return ERR_STATUS_GOOD;

}

int VtkHistogram::set_plot_location (double x1, double y1, double x2, double y2)
{
   double pos1[] = {x1,y1};
   double pos2[] = {x2,y2};
   histogramData->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
   histogramData->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();

   VtkUtils::FrameNorm2ViewportNorm (coord, renderer, pos1);
   VtkUtils::FrameNorm2ViewportNorm (coord, renderer, pos2);

   histogramData->GetPositionCoordinate()->SetValue(pos1[0],pos1[1]);
   histogramData->GetPosition2Coordinate()->SetValue(pos2[0],pos2[1]);

   histogramData->UpdatePosition ();

#ifndef HEADLESS
   histogramDataWidget->UpdateRepresentation ();
#endif

   return ERR_STATUS_GOOD;
}

int VtkHistogram::change_range (const ChipsVector &low_range, const ChipsVector &high_range)
{
  double tmpRangeX[2];
  double tmpRangeY[2];
  vtkHistogramDataActor2DExtra *dataPtr = NULL;

  tmpRangeX[0] = low_range[0];
  tmpRangeY[0] = low_range[1];
  tmpRangeX[1] = high_range[0];
  tmpRangeY[1] = high_range[1];
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      // Vtk does checking to see if the ranges are the same before updating
      // the actors. Actors wont get rerendered if ranges do not change.
      dataPtr->SetXRange(tmpRangeX);
      dataPtr->SetYRange(tmpRangeY);

    };

#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;
}

int VtkHistogram::change_scale (int inXScale, int inYScale)
{
  vtkHistogramDataActor2DExtra *dataPtr = NULL;
  for (histogramData->InitTraversal (); (dataPtr = histogramData->GetNextHistogramDataActor ());)
    {
      // Vtk does checking to see if the ranges are the same before updating
      // the actors. Actors wont get rerendered if ranges do not change.
      dataPtr->SetXScale(inXScale);
      dataPtr->SetYScale(inYScale);
    };
#ifndef HEADLESS
  histogramDataWidget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;
}


int VtkHistogram::update_data ()
{
  // clear old histogram data
  histogramData->RemoveData ();

  // add new histogram data
  long index = 0;
  vtkDoubleArray* upErr=vtkDoubleArray::New();
  vtkDoubleArray* downErr=vtkDoubleArray::New();

  renderer->AddActor (histogramData);

  for (list<CHIPS_RANGE_P>::const_iterator tmpRangePtr=indices.begin(); tmpRangePtr!=indices.end();tmpRangePtr++, index++)
    {
      if ( create_histogramdata(index,upErr,downErr,0,0,*tmpRangePtr)!= ERR_STATUS_GOOD)
        {
          cerr<<"Error: VtkHistogram::VtkHistogram - AddData Failed"<<endl;
        }
    };

  change_linethickness();
  change_linestyle();
  change_linecolor();
  change_symbolsize();
  change_symbolstyle();
  change_symbolcolor();
  change_dropline();
  change_errcolor();
  change_errstyle();
  change_errup();
  change_errdown();
  change_errleft();
  change_errright();
  change_errthickness();
  change_visible();
  change_fillopacity ();
  change_fillcolor ();
  change_fillstyle();

  upErr->Delete ();
  downErr->Delete ();

  return ERR_STATUS_GOOD;
}

void VtkHistogram::select ()
{
#ifndef HEADLESS
  if (histogramDataWidget)
    histogramDataWidget->Select (true);
#endif
}

void VtkHistogram::unselect ()
{
#ifndef HEADLESS
  if (histogramDataWidget)
    histogramDataWidget->Select (false);
#endif
}

void VtkHistogram::disable_interaction ()
{
#ifndef HEADLESS
  vtkHistogramDataRepresentation *rep = reinterpret_cast<vtkHistogramDataRepresentation *> (this->histogramDataWidget->GetRepresentation ());
  rep->DisableBorder ();
  this->histogramDataWidget->EnabledOff ();
#endif
}

void VtkHistogram::enable_interaction ()
{
#ifndef HEADLESS
  if (this->is_interaction_blocked ())
    return;

  vtkHistogramDataRepresentation *rep = reinterpret_cast<vtkHistogramDataRepresentation *> (this->histogramDataWidget->GetRepresentation ());

  this->histogramDataWidget->EnabledOn ();
  rep->EnableBorder ();
#endif
}

