// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "VtkImage.hh"
#include "VtkColorMapTable.hh"
#include "VtkColorMap.hh"
 

#include "vtkPNGReader.h"
#include "vtkImageData.h"
#include "vtkImageShiftScale.h"
#include "vtkImageMapper.h"

#include "vtkImageReslice.h"
#include "vtkImageChangeInformation.h"
#include "vtkTransform.h"

#include "vtkImageMapToColors.h"
#include "vtkLookupTable.h"

#include "vtkImageAppendComponents.h"
#include "vtkImageMathematics.h"
#include "vtkPointData.h"

#include "vtkJPEGWriter.h"
#include "vtkPNGWriter.h"
#include "vtkImageCast.h"

#ifndef HEADLESS
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkChipsWidget.hh"
#include "vtkChipsRepresentation.hh"
#endif

#include "VtkUtils.hh"


#include <float.h>
#include <config.h>
#ifdef HAVE_IEEEFP_H
#include <ieeefp.h>        // finite defined in ieeefp.h (solaris)
#else
#include <math.h>          // finite() defined here      (linux, OSX)
#endif

// isinf is not definded on solaris
#ifdef HAVE_IEEEFP_H
#  define is_negative_inf(a) (fpclass(a) == FP_NINF)
#  define is_positive_inf(a) (fpclass(a) == FP_PINF)
#  define isinf(a) (is_positive_inf(a) || is_negative_inf(a))
#else 
#  define is_negative_inf(a) (isinf(a) && signbit(a))
#  define is_positive_inf(a) (isinf(a) && !signbit(a))
#endif


extern ChipsColorMapTable *colormap_table;
extern ChipsDefaults* theDefaults; 

VtkImage::VtkImage(vtkRenderer* inRenderer,ChipsObject* par, 
		   string filename, string transform_id,
                   string val):ChipsImage(par,filename,transform_id,val)
{
/*   renderer=inRenderer;

   this->transform = vtkTransform::New ();
   reslice = vtkImageReslice::New ();
   changeInformation = vtkImageChangeInformation::New ();

   LoadImage(filename);
*/

};

VtkImage::VtkImage(vtkRenderer* inRenderer,ChipsObject* par, 
		   string filename,
		   float x1, float x2, float y1, float y2, string transform_id,
		   string val):ChipsImage(par,filename,x1,x2,y1,y2,transform_id,val)
{
/*   renderer=inRenderer;
   this->transform = vtkTransform::New ();
   reslice = vtkImageReslice::New ();
   changeInformation = vtkImageChangeInformation::New ();

   LoadImage(fname);
*/

};

VtkImage::VtkImage(vtkRenderer* inRenderer,ChipsObject* par, 
                   int xaxis, int yaxis, double* rvals, double* gvals, 
                   double *bvals, double *avals, string transform_id,
                   string val):ChipsImage(par, xaxis, yaxis, rvals, gvals, bvals, avals, transform_id, val)
{
   this->renderer=inRenderer;

   double *vals[] = {rvals, gvals, bvals, avals};
   bool fillZero = false;
   bool alpha_used = false;

   
   // create an array to be filled with zeros
   this->zeroData = vtkImageData::New();
   this->zeroDataFilled = false;

   // count the number of images supplied
   this->imageCount = 0;
   this->pseudoColor = false;
   for (int ii=0; ii<4; ii++)
      if (vals[ii] != NULL)
         this->imageCount++;

   // if only one image is set and its the first image
   // then assume the image is pseudoColor
   this->pseudoColor = ((this->imageCount == 1) && (vals[0] != NULL));

   // convert nans to alpha values
   if (!vals[3] && !this->pseudoColor)
   {
      bool set_alpha = true;
      vals[3] = new double[xaxis*yaxis];
      for (int ii=0; ii<xaxis*yaxis; ii++)
      {
         set_alpha = true;
         for (int jj = 0; jj<3; jj++)
         {
            if (vals[jj] == NULL)
               continue;
            set_alpha &= isnan (vals[jj][ii]);
         }

         if (set_alpha)
         {
            alpha_used = true;
            vals[3][ii] = 0;
         }
         else
            vals[3][ii] = 1;
      }

      if (!alpha_used)
      {
         delete [] vals[3];
         vals[3] = NULL;
      }
   }


   // fill the rgba arrays
   for (int ii=0; ii<4; ii++)
   {
      if (vals[ii] != NULL)
         this->imageData[ii] = this->create_image (vals[ii], xaxis, yaxis);
      else if ((ii != 3) && !this->pseudoColor)
      {
         this->imageData[ii] = this->zeroData;
         fillZero = true;
      }
      else
         this->imageData[ii] = NULL;
         
   }  

   if (fillZero)
      this->fill_zero_data (this->dim[0], this->dim[1]);

   if (alpha_used)
   {
      delete [] vals[3];
      vals[3] = NULL;
   }

   this->Initialize ();
  
};


VtkImage::VtkImage(const VtkImage& inImage):ChipsImage(inImage.parent,inImage.fname.c_str(),0,0,0,0,inImage.transform_id.c_str (),inImage.id.c_str())
{

/*   renderer=inImage.renderer;
   this->transform = vtkTransform::New ();
   reslice = vtkImageReslice::New ();
   changeInformation = vtkImageChangeInformation::New ();
 
   LoadImage(fname);
*/
};

VtkImage::VtkImage(vtkRenderer *inRenderer, ChipsObject* par,
                   ChipsStateStream& in): ChipsImage(par, in)
{
   this->renderer=inRenderer;

   this->zeroData = vtkImageData::New();
   this->zeroDataFilled = false;

   this->deserialize (in);
   this->Initialize();
}

VtkImage::~VtkImage()
{
   this->renderer->RemoveActor(vtkImage);
   this->transform->Delete ();
   this->reslice->Delete ();
   this->changeInformation->Delete ();
   this->vtkImage->Delete();

   this->zeroData->Delete ();

   if (this->colorMap)
      this->colorMap->Delete ();

   if (this->appendComponents)
     this->appendComponents->Delete ();

   for (int ii=0; ii<4; ii++)
   {
      if (this->imageData[ii] && imageData[ii] != zeroData)
         this->imageData[ii]->Delete ();
      if (this->scaleFilter[ii])
         this->scaleFilter[ii]->Delete ();
      if (this->offsetFilter[ii])
         this->offsetFilter[ii]->Delete ();

   }

#ifndef HEADLESS
   this->imageWidget->Delete ();
#endif

};

void VtkImage::Initialize ()
{

   // initialize the filters

   if (this->imageCount == 0)
      return;

   this->vtkImage = vtkActor2D::New();

   this->transform = vtkTransform::New ();
   this->reslice = vtkImageReslice::New ();
   this->changeInformation = vtkImageChangeInformation::New ();


   // reslice is used for scaling and rotations
   this->reslice->WrapOff ();
   this->reslice->AutoCropOutputOff ();
   this->reslice->SetResliceTransform (this->transform);

   // this allows all transformations to happen on the center of the image
   this->changeInformation->SetInput (this->imageData[0]);
   this->changeInformation->CenterImageOn ();

   this->reslice->SetInputConnection (changeInformation->GetOutputPort ());

   this->clipImage = false;
 
   // add the image to a graphics mapper
   vtkImageMapper* mapImage = vtkImageMapper::New();
   mapImage->SetInputConnection(this->reslice->GetOutputPort ());
   mapImage->SetColorWindow( 255);
   mapImage->SetColorLevel( 127.5);
   mapImage->UseCustomExtentsOn ();

   this->vtkImage->SetMapper(mapImage);

   this->vtkImage->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
   this->vtkImage->SetPosition(0,0);

   this->renderer->AddActor(vtkImage);

   this->change_interpolation ();

   mapImage->Delete();


   this->appendComponents = NULL;
   this->colorMap = NULL;

   for (int ii=0; ii<4; ii++)
   {
      this->scaleFilter[ii] = NULL;
      this->offsetFilter[ii] = NULL;
   }

   // if there is only one image and its the first image assume that
   // the image is pseudo color
   if (this->pseudoColor)
      this->InitializePseudoColor ();
   else
      this->InitializeTrueColor ();

#ifndef HEADLESS
   // setup gui widget
   vtkChipsRepresentation *imageRep = vtkChipsRepresentation::New ();
   ChipsId chipsId;

   this->populate_chips_id (chipsId);
   imageWidget = vtkChipsWidget::New ();
   imageWidget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   imageWidget->SetRepresentation (imageRep);
   imageWidget->SetIdInfo (chipsId, this->get_object_type());
   imageWidget->ResizableOff ();
   imageWidget->RotateableOff ();
   imageWidget->MoveableOff ();
   imageWidget->SetDefaultRenderer (renderer);
   imageWidget->EnabledOn ();
   imageRep->Delete ();
#endif


}

void VtkImage::InitializeTrueColor ()
{
   this->appendComponents = vtkImageAppendComponents::New();
   for (int ii=0; ii<4; ii++)
   {
      if (this->imageData[ii] == NULL)
         continue;

      // add in filters for normalization
      this->scaleFilter[ii] = vtkImageMathematics::New ();
      this->offsetFilter[ii] = vtkImageMathematics::New ();

      this->scaleFilter[ii]->SetOperationToMultiplyByK ();
      this->offsetFilter[ii]->SetOperationToAddConstant ();

      // dont set the input if the data is null
      // this wont be added to the pipeline
      if (this->imageData[ii] != NULL)
         this->offsetFilter[ii]->SetInput (this->imageData[ii]);

      this->scaleFilter[ii]->SetInputConnection (this->offsetFilter[ii]->GetOutputPort ());
   }

   this->appendComponents->SetInputConnection (this->scaleFilter[0]->GetOutputPort ());
   this->appendComponents->AddInputConnection (this->scaleFilter[1]->GetOutputPort());
   this->appendComponents->AddInputConnection (this->scaleFilter[2]->GetOutputPort());

   // dont add the alpha channel to the pipeline if it isn't set
   if (this->imageData[3] != NULL)
      this->appendComponents->AddInputConnection (this->scaleFilter[3]->GetOutputPort());
   this->changeInformation->SetInputConnection (this->appendComponents->GetOutputPort ());

   // update the normalization filters
   this->change_normalize_channels ();

}

void VtkImage::InitializePseudoColor ()
{
   double *scalarRange = imageData[0]->GetScalarRange ();;
   double currScalarRange[2];

   // add a color map to the image filtering process
   
   this->colorMap = vtkImageMapToColors::New ();
   this->colorMap->SetInputConnection (this->reslice->GetOutputPort ());

   // only set the scalar range if it has not already been set. the
   // range is initialized to -1, -1 which means it hasn't been set
   colormap_table->get_colormap_scalar_range (colormap_id, currScalarRange);
   if (currScalarRange[0] == -1 && currScalarRange[1] == -1)
      colormap_table->set_colormap_scalar_range (colormap_id, scalarRange);

   VtkColorMap *tmpColorMap = dynamic_cast <VtkColorMap *> (colormap_table->
get_colormap (colormap_id));

   this->colorMap->SetLookupTable (tmpColorMap->get_lookup_table ());

   vtkMapper2D* mapImage = this->vtkImage->GetMapper();
   mapImage->SetInputConnection(colorMap->GetOutputPort ());

}

int VtkImage::LoadImage(const string& inFileName)
{
/*
   vtkPNGReader* imageReader=vtkPNGReader::New(); 

   imageReader->SetDataSpacing(0.8,0.8,1.5);
   imageReader->SetFileName(inFileName.c_str());
   imageReader->UpdateWholeExtent ();

   this->Initialize (imageReader->GetOutput ());  
   imageReader->Delete();
*/
   return 1;
  
};

int VtkImage::change_visible ()
{
   if (this->clipImage)
   {
      this->vtkImage->SetVisibility (0);
/*
#ifndef HEADLESS
      this->disable_interaction ();
#endif
*/
   }
   else
   {
      int tmp_vis = (!get_parent()->is_visible ()) ? 0 : visible;
      this->vtkImage->SetVisibility (tmp_vis);

#ifndef HEADLESS
      if (tmp_vis)
      {
         //this->enable_interaction ();
         this->imageWidget->GetRepresentation ()->SetVisibility (tmp_vis);
      }
#endif

   }
   return ERR_STATUS_GOOD;
}


int VtkImage::change_rotation (double angle)
{
   this->transform->RotateZ (angle*180.0/V_PI);
   return ERR_STATUS_GOOD;
}

int VtkImage::change_position (double x, double y)
{
   double pos[] = {x,y};
   VtkUtils::FrameNorm2ViewportNorm (coord, renderer, pos);
   this->vtkImage->SetPosition(pos[0], pos[1]);

 
#ifndef HEADLESS 
   vtkChipsRepresentation *imageRep = reinterpret_cast<vtkChipsRepresentation *>(this->imageWidget->GetRepresentation ());
   imageRep->GetPositionCoordinate ()->SetCoordinateSystemToNormalizedViewport ();
   imageRep->GetPositionCoordinate ()->SetValue (pos[0], pos[1]);

   this->imageWidget->UpdateRepresentation ();
#endif

   return ERR_STATUS_GOOD;
}

int VtkImage::change_origin (double x, double y, double scaleX, double scaleY)
{
   x -= xmin;
   y -= ymin;

   this->reslice->SetOutputOrigin((x-(xmax-xmin)/2.0)*scaleX, 
                                  (y-(ymax-ymin)/2.0)*scaleY, 0);

   return ERR_STATUS_GOOD;
}

int VtkImage::change_scale (double scaleX, double scaleY)
{

   // if no y scale was provided scale both x and y equally
   if (dbl_equal (scaleY,0))
      scaleY = scaleX;

   if (dbl_equal (scaleX,0) || dbl_equal (scaleY,0))
      return ERR_STATUS_BAD;

   // scales are inverted .5 doubles the size while 2 decreases it by half
/*   double invertScaleX = 1/scaleX;
   double invertScaleY = 1/scaleY;

   // scale off the original image and not the current
   // remove the old scale first
   double scale[] = {1,1,1};
   this->transform->GetScale (scale);


   scale[0] = invertScaleX / scale[0];
   scale[1] = invertScaleY / scale[1];

*/
   double scale[3] = {1/scaleX,1/scaleY,1};

   this->transform->Identity ();
   this->transform->Scale (scale);

//   this->reslice->SetOutputSpacing (1,1,1);
   return ERR_STATUS_GOOD;
}


int VtkImage::doprint (string outfile, int fmt)
{
   double scale[3] = {1,1,1};
   string outfile_ext = "";
/*
   int tmp_extent [] =  {0,1024,0,1024,0,1};
   double x = -get_xmin ();
   double y = -get_ymin ();
*/

   // check the extension of the outfile to and use that as the format
   // if it's a proper extensions otherwise refer to the preferences
   int ext_pos = outfile.find_last_of (".", outfile.length ());

   if ((ext_pos != string::npos) && (ext_pos+1 != outfile.length ()))
      outfile_ext = outfile.substr (ext_pos, outfile.length ());


   vtkTransform *tmp_transform = vtkTransform::New ();
   vtkImageReslice *tmp_reslice = vtkImageReslice::New (); 
   vtkImageCast *image_cast = vtkImageCast::New ();

   if ((this->print_width != 0) && (this->print_height != 0))
   {
      scale[0] = this->dim[0]/(float)this->print_width;
      scale[1] = this->dim[1]/(float)this->print_height;
/*      tmp_extent[1] = this->print_width;
      tmp_extent[3] = this->print_height;
*/
   }

   tmp_transform->Identity ();
   tmp_transform->Scale (scale);

   tmp_reslice->WrapOff ();
   tmp_reslice->AutoCropOutputOn ();
   tmp_reslice->SetResliceTransform (tmp_transform);
/*   tmp_reslice->SetOutputOrigin((x-dim[0]/2.0)/scale[0], (y-dim[1]/2.0)/scale[1], 0);
   tmp_reslice->SetOutputExtent (tmp_extent);
*/
   switch (this->interpolation)
   {
      case chips_neighbor:
         tmp_reslice->SetInterpolationModeToNearestNeighbor ();
      break;
      case chips_bilinear:
         tmp_reslice->SetInterpolationModeToLinear ();
      break;
      case chips_bicubic:
         tmp_reslice->SetInterpolationModeToCubic ();
      break;
      default:
      break;
   }

   if (this->pseudoColor)
   {
      this->colorMap->SetInputConnection (this->changeInformation->GetOutputPort ());
      tmp_reslice->SetInputConnection (this->colorMap->GetOutputPort ());
   }
   else
      tmp_reslice->SetInputConnection (this->changeInformation->GetOutputPort ());
   image_cast->SetInputConnection (tmp_reslice->GetOutputPort ());
   image_cast->SetOutputScalarTypeToUnsignedChar ();
   image_cast->Update ();

   if (fmt == 3)   
   {
      vtkPNGWriter *pngw = vtkPNGWriter::New();

      // don't append if the extension already is exists
      if (outfile_ext != ".png")
         outfile.append(".png");
      pngw->SetInput(image_cast->GetOutput ());
      pngw->SetFileName(outfile.c_str());
      pngw->Write();
      pngw->Delete();
   }
   else if ((fmt == 4) || (fmt==5))
   {
      vtkJPEGWriter *jpgw = vtkJPEGWriter::New();

      if ((fmt == 4) && (outfile_ext != ".jpg"))
         outfile.append(".jpg");
      else if ((fmt == 5) && (outfile_ext != ".jpeg"))
         outfile.append(".jpeg");
      jpgw->SetInput(image_cast->GetOutput ());
      jpgw->SetFileName(outfile.c_str());
      jpgw->Write();
      jpgw->Delete();
   }

   // reset the colormap to have the correct input
   if (this->pseudoColor)
      this->colorMap->SetInputConnection (this->reslice->GetOutputPort ());


   tmp_transform->Delete ();
   tmp_reslice->Delete ();
   image_cast->Delete ();
   return ERR_STATUS_GOOD;
}


int VtkImage::change_extent (int lower_x, int lower_y, int upper_x, int upper_y)
{
   int tmp_extent[] = {0,0,0,0,0,1};
   double pos[2];

   vtkImageMapper* mapImage = dynamic_cast <vtkImageMapper*> (this->vtkImage->GetMapper());
#ifndef HEADLESS
   vtkChipsRepresentation *imageRep = reinterpret_cast<vtkChipsRepresentation *>(this->imageWidget->GetRepresentation ());
#endif


   tmp_extent[0] = lower_x;
   tmp_extent[1] = upper_x-1;
   tmp_extent[2] = lower_y;
   tmp_extent[3] = upper_y-1;


   this->reslice->SetOutputExtent (tmp_extent);


   // this is necessary for printing
   if (mapImage)
      mapImage->SetCustomDisplayExtents (tmp_extent);

#ifndef HEADLESS
   pos[0] = tmp_extent[1];
   pos[1] = tmp_extent[3];

   renderer->ViewportToNormalizedViewport(pos[0], pos[1]);

   this->imageWidget->UpdateRepresentation ();

   imageRep->GetPosition2Coordinate ()->SetCoordinateSystemToNormalizedViewport ();
   imageRep->GetPosition2Coordinate ()->SetValue (pos[0], pos[1]);
#endif


   return ERR_STATUS_GOOD;
}

int VtkImage::clip_image (bool clip)
{
   this->clipImage = clip;
   return change_visible ();
}

int VtkImage::flip_axis (int axis)
{
   bool removeMatrix = false;
   vtkMatrix4x4 *matrix = this->reslice->GetResliceAxes ();
   if (matrix == NULL)
   {
      matrix = vtkMatrix4x4::New ();
      matrix->Identity ();
      this->reslice->SetResliceAxes (matrix);
      removeMatrix = true;
   }
     
   switch (axis)
   {
      case X_AXIS:
         matrix->SetElement(0,0, -matrix->GetElement(0, 0));
      break;
      case Y_AXIS:
         matrix->SetElement(1,1, -matrix->GetElement(1, 1));
      break;
      case XY_AXIS:
         matrix->SetElement(0,0, -matrix->GetElement(0, 0));
         matrix->SetElement(1,1, -matrix->GetElement(1, 1));
      break;
   }

   if (removeMatrix)
      matrix->Delete ();

   return ERR_STATUS_GOOD;
}


int VtkImage::change_interpolation ()
{
   switch (interpolation)
   {
      case chips_neighbor:
         this->reslice->SetInterpolationModeToNearestNeighbor ();
      break;
      case chips_bilinear:
         this->reslice->SetInterpolationModeToLinear ();
      break;
      case chips_bicubic:
         this->reslice->SetInterpolationModeToCubic ();
      break;
      default:
      break;
   }

   return ERR_STATUS_GOOD;
}

int VtkImage::set_plot_location (double x1, double y1, double x2, double y2)
{
   return ERR_STATUS_GOOD;
}

int VtkImage::change_depth()
{
  vtkImage->SetLayerNumber (depth*10+1);
  return ERR_STATUS_GOOD;
}

int VtkImage::change_normalize_channels ()
{
   double *scalarRange;
   double totalScalarRange[4][2];
   double tmpScalarRange[] = {FLT_MAX, FLT_MIN};

   for (int ii=0; ii<4; ii++)
   {
      if (this->imageData[ii] == NULL)
         continue;

      scalarRange = this->imageData[ii]->GetScalarRange ();

      // get the scalar range for each of the channels
      // and normalize against it; always process the alpha
      // channel against itself
      if (this->normalize_channels || (ii==3))
      {
         totalScalarRange[ii][0] = scalarRange[0];
         totalScalarRange[ii][1] = scalarRange[1];
      }
      // determine what the minimum and maximum scalar ranges are
      // for the entire data set, minus the alpha channel
      else
      {
         if (tmpScalarRange[0] > scalarRange[0])
            tmpScalarRange[0] = scalarRange[0];
         if (tmpScalarRange[1] < scalarRange[1])
            tmpScalarRange[1] = scalarRange[1];
      }
   }

   // set the scalar range for the full set except for alpha channel
   // that's normalized against itself
   if (!this->normalize_channels)
      for (int ii=0; ii<3; ii++)
      {
         totalScalarRange[ii][0] = tmpScalarRange[0];
         totalScalarRange[ii][1] = tmpScalarRange[1];
      }

   for (int ii=0; ii<4; ii++)
   {
      // skip data channels which don't exists
      if (this->imageData[ii] == NULL)
         continue;

      if ((totalScalarRange[ii][1] - totalScalarRange[ii][0]) == 0)
         this->scaleFilter[ii]->SetConstantK (0);
      else
         this->scaleFilter[ii]->SetConstantK (255/(totalScalarRange[ii][1] - totalScalarRange[ii][0]));
      this->offsetFilter[ii]->SetConstantC (-totalScalarRange[ii][0]);
   }

   return ERR_STATUS_GOOD;
}


int VtkImage::change_mode (int mode)
{
   int rval = ERR_STATUS_GOOD;
   if (this->mode == 0)
   {
      // component mode already set
      if (!this->pseudoColor)
         rval= ERR_STATUS_GOOD;
      else
      {
         this->pseudoColor = false;

         // remove the colormap
         vtkAlgorithmOutput* filter = this->colorMap->GetInputConnection (0,0);
         vtkMapper2D* mapImage = this->vtkImage->GetMapper();
         mapImage->SetInputConnection(filter);

         this->colorMap->Delete ();
         this->colorMap = NULL;

         // set up the true color imaging
         this->InitializeTrueColor ();
      }
   }
   else
   {
      // component mode is not set
      if (this->pseudoColor)
         rval = ERR_STATUS_GOOD;
      else if (this->imageCount > 1)
         rval = ERR_STATUS_BAD;
      else
      {
         this->pseudoColor = false;
         // remove the component filters
         this->changeInformation->RemoveInputConnection (0, this->appendComponents->GetOutputPort ());
         this->appendComponents->Delete ();                                          this->appendComponents = NULL;

         for (int ii=0; ii<4; ii++)
         {
            this->scaleFilter[ii]->Delete ();
            this->scaleFilter[ii] = NULL;
            this->offsetFilter[ii]->Delete ();
            this->offsetFilter[ii] = NULL;
         }

         // move the existing image to the first image
         

         // add back colormap
         this->InitializePseudoColor ();

      }
   }

   return rval;     
}

int VtkImage::add_component (double* vals, int channel)
{

   if (this->pseudoColor)
      return ERR_STATUS_BAD;

   int xx, yy;
   double *scalarRange;
   bool setAlpha = false;

   if (this->pseudoColor)
      return ERR_STATUS_BAD;

   // if the filters have never been created create them
   if (this->scaleFilter[channel] == NULL)
   {
      this->scaleFilter[channel] = vtkImageMathematics::New ();
      this->offsetFilter[channel] = vtkImageMathematics::New ();

      this->scaleFilter[channel]->SetOperationToMultiplyByK ();
      this->offsetFilter[channel]->SetOperationToAddConstant ();

      this->scaleFilter[channel]->SetInputConnection (this->offsetFilter[channel]->GetOutputPort ());
   }


   // create an image if none has been created for this channel yet
   if ((this->imageData[channel] == NULL) || (this->imageData[channel] == this->zeroData))
   {
      this->imageData[channel] = create_image (vals, this->dim[0], this->dim[1]);
      this->offsetFilter[channel]->SetInput (this->imageData[channel]);

      // check to see if the alpha channel needs to be set
      if (channel == 3)
         setAlpha = true;
   }
   else
   {

      // copy data array into vtkImageData scalars
      double *ptr = static_cast<double*>(this->imageData[channel]->GetScalarPointer());
      for(yy = 0; yy < this->dim[1]; ++yy)
         for (xx = 0; xx < this->dim[0]; ++xx)
            *ptr++ = vals[yy*this->dim[0] + xx];

      
      this->imageData[channel]->GetPointData ()->GetScalars ()->Modified ();
   }

   scalarRange = this->imageData[channel]->GetScalarRange ();

   // update the filter information with the correct information
   if ((scalarRange[1] - scalarRange[0]) == 0)
      this->scaleFilter[channel]->SetConstantK (0);
   else
      this->scaleFilter[channel]->SetConstantK (255/(scalarRange[1] - scalarRange[0]));
   this->offsetFilter[channel]->SetConstantK (-scalarRange[0]);

   this->offsetFilter[channel]->SetInput (this->imageData[channel]);


   // if this an alpha channel the filter connection was disabled
   // re-enable it
   if (setAlpha)
      this->appendComponents->AddInputConnection (this->scaleFilter[channel]->GetOutputPort());

   return ERR_STATUS_GOOD;
}

int VtkImage::remove_component (int channel)
{

   if (this->pseudoColor)
      return ERR_STATUS_BAD;

   // the channel does not exist -- just return
   if ((this->imageData[channel] == NULL) || (this->imageData[channel] == this->zeroData))
      return ERR_STATUS_GOOD;

   // delete the image associated with this data
   this->imageData[channel]->Delete ();

   // if its not the alpha channel assign the channel to have zero values
   if (channel != 3)
   {
    
      this->fill_zero_data (this->dim[0], this->dim[1]);
      this->imageData[channel] = this->zeroData;
      this->scaleFilter[channel]->SetConstantK (0);
      this->offsetFilter[channel]->SetConstantC (0);
      this->offsetFilter[channel]->SetInput (this->imageData[channel]);
   }
   else
   {
      this->imageData[channel] = NULL;
      this->appendComponents->RemoveInputConnection (0, this->scaleFilter[3]->GetOutputPort());
   }

   return ERR_STATUS_GOOD;
}


void VtkImage::serialize(ChipsStateStream& out)
{
   int sz;
   bool valid = true;

   // this is a little sloppy but the cleanest approach
   // since the order of serialize matters

   ChipsImage::serialize (out);

   out.write_bool (&this->pseudoColor);
   sz = this->dim[0] * this->dim[1];

   if (sz > 0)
   {

      // write out each channel
      double* dp;
      for (int ii=0; ii<4; ii++)
      {
         // if the channel has real information write it to file
         if ((this->imageData[ii] != NULL) && (this->imageData[ii] != this->zeroData))
         {
            valid = true;
            dp = (double *)(this->imageData[ii]->GetScalarPointer ());
            out.write_bool(&valid);
            out.write_double(dp, sz);
         }
         else
         {
            valid = false;
            out.write_bool(&valid);
         }
      }
   }
   else
   {
      sz = 0;
      out.write_int(&sz);
   }

}

/*
void VtkImage::serialize_ascii(ChipsStateStream& out)
{
   string att = "", tmp;
   char buf[80];
   V_COLORMAP_TYPE ic = colormap_table->get_colormap_type(colormap_id);
   if (ic != theDefaults->get_image_colormap())
   {
      const char *val = v_colormap_to_string(ic);
      if (val)
      {
         att.append("colormap=");
         att.append(val);
      }
   }
   if (interpolation != theDefaults->get_image_interpolation())
   {
      const char * ic = v_interpolation_to_string((V_INTERPOLATION_TYPE)interpolation);
      if (ic)
      {
         if (att.length()) att.append(" ");
         {
            att.append("interpolation=");
            att.append(ic);
         }
      }
   }
   // add image depth if not default
//   int dep = theDefaults->get_image_depth();
//WLM   if (depth != dep &&
//WLM       (dep!=V_USE_DEFAULT_DEPTH || depth!=((VtkPlot*)parent)->get_depth()))
//WLM   {
//WLM      sprintf(buf, "depth=%d", depth);
//WLM      if (att.length()) att.append(" ");
//WLM      att.append(buf);
//WLM   }

   if (colormap_table)
   {
      double thresh[2];
      colormap_table->get_colormap_scalar_threshold (colormap_id, thresh);

      if (thresh[0] != theDefaults->get_image_threshold_lower() ||
          thresh[1] != theDefaults->get_image_threshold_upper())
      {
         if (att.length()) att.append(" ");
         sprintf(buf, "threshold=[%3.2f,%3.2f]", thresh[0],thresh[1]);
         att.append(buf);
      }
   }

   // add image id
   if (att.length()) att.append(" ");
   att.append("id=");
   att.append(id);

   string dfname = out.use_tempname();
   string mfile = "";

   if (meta.length())
   {
      mfile = dfname;
      mfile.append("=\"");
      mfile.append(meta);
      mfile.append("\"");
      out << "add_image("<<dfname;
   }
   else
   {
      string tmpfile = out.get_filename();
      sprintf(buf, "%s.Data%d_image.bin", tmpfile.c_str(), out.use_tempfnum());
      tmpfile = buf;
      mfile = dfname;
      mfile.append(" = DataObj()  # image data- ");
      mfile.append(tmpfile);
      mfile.append("\n");
      mfile.append(dfname);
      mfile.append(".channel0 = #  insert image array here \n");

      // dump image
      ofstream ff(tmpfile.c_str(), ios::out|ios::binary);
      if (ff)
      {
         // image data format:
         //  [type (1 = image)][ num of axis (2)   ]
         //  [axis 1 dim      ][ axis 2 dim        ]
         //  [ double array col1 data .. dim1 * dim 2]
         long tmpax1 = 1, tmpax2 = 2;
         ff.write((char*)&tmpax1, sizeof(long));   // image format (1)
         ff.write((char*)&tmpax2, sizeof(long));   // num axes (2)
         tmpax1 = dim[1];
         ff.write((char*)&tmpax1, sizeof(long));   // x axis dim
         tmpax2 = dim[0];
         ff.write((char*)&tmpax2, sizeof(long));   // y axis dim

         int sz = dim[0] * dim[1];
         if (this->imageData[0] && (this->imageData[0] != this->zeroData))
         {
            double* dp = (double *)(this->imageData[0]->GetScalarPointer());
            ff.write((char*)dp, sizeof(double)*sz);
         }
      }
      out<<"add_image("<<dfname<<".channel0, None, None, None, "<<this->dim[0]<<", "<<this->dim[1];
   }
   if (att.length()) out <<",'"<<att<<"'";
   out <<")"<<endl;
   if (mfile.length())   out.append_to_template(mfile);
}
*/


void VtkImage::deserialize(ChipsStateStream& in)
{
   int len;
   bool valid;
   bool fillZero = false;

   // this is a little sloppy but the cleanest approach
   // since the order of serialize matters
   ChipsImage::deserialize (in);

   this->imageCount = 0;

   in.read_bool (&this->pseudoColor);

   len = this->dim[0] * this->dim[1];
   if (len > 0)
   {

      // read in each channel from file
      double* dp;
      for (int ii=0; ii<4; ii++)
      {

         // if the channel had real values read them in 
         // otherwise if the channel was not an alpha channel 
         // set the values to zero 
         in.read_bool(&valid);
         if (valid)
         {
            dp = new double[len];
            in.read_double(dp, len);
            this->imageCount++;
            this->imageData[ii] = this->create_image (dp, this->dim[0], this->dim[1]);
            delete []dp;
         }
         else if (!this->pseudoColor && (ii!=3))
         {
            this->imageData[ii] = this->zeroData;
            fillZero = true; 
         }
         else
            this->imageData[ii] = NULL;
      }
   }
   else
   {
      for (int ii=0; ii<4; ii++)
         this->imageData[ii] = NULL;
   }

   // fill any empty spots with zeros
   if (fillZero)
      this->fill_zero_data (this->dim[0], this->dim[1]);
}

double *VtkImage::get_data (int channel, bool ignore_zero)
{
   double *data;
   if (this->imageData[channel])
   {
      // if ignore flag is set return NULL otherwise return the
      // actual data being stored
      if (ignore_zero && this->imageData[channel] == this->zeroData)
         data = NULL;
      else
         data = (double *)this->imageData[channel]->GetScalarPointer ();
   }
   else
      data = NULL;

   return data;
      
}

void VtkImage::fill_zero_data (int xdim, int ydim)
{
   if (zeroDataFilled)
      return;

   // create an image filled with zeros
   this->zeroData->SetDimensions(xdim, ydim, 1);
   this->zeroData->SetScalarTypeToDouble();
   this->zeroData->AllocateScalars();

   double *ptr = static_cast<double*>(zeroData->GetScalarPointer());
   for(int yy = 0; yy < ydim; ++yy)
      for (int xx = 0; xx < xdim; ++xx)
         *ptr++ = 0;

   this->zeroDataFilled = true;
}

vtkImageData *VtkImage::create_image (double *data, int xdim, int ydim)
{
   // create an image
   vtkImageData *imageData = vtkImageData::New();
   imageData->SetDimensions(xdim, ydim, 1);
   imageData->SetScalarTypeToDouble();
   imageData->AllocateScalars();

   double dataMin = FLT_MAX;
   double dataMax = -FLT_MAX;

   // run through the data once to pull out the min and max
   for(int yy = 0; yy < ydim; ++yy)
      for (int xx = 0; xx < xdim; ++xx)
      {
         if (!isnan(data[yy*xdim + xx]) && !isinf (data[yy*xdim + xx]))
         {
            if (dataMin > data[yy*xdim + xx])
               dataMin = data[yy*xdim + xx];
            if (dataMax < data[yy*xdim + xx])
               dataMax = data[yy*xdim + xx];
         }
      }

   // copy data array into vtkImageData scalars
   double *ptr = static_cast<double*>(imageData->GetScalarPointer());
   for(int yy = 0; yy < ydim; ++yy)
      for (int xx = 0; xx < xdim; ++xx)
      {
         if (isnan(data[yy*xdim + xx]))
            *ptr = dataMin;
         else if (is_positive_inf(data[yy*xdim + xx]))
            *ptr = dataMax;
         else if (is_negative_inf(data[yy*xdim + xx]))
            *ptr = dataMin;
         else
            *ptr = data[yy*xdim + xx];
         ptr++;
      }

   return imageData;
}

int VtkImage::get_mode ()
{
   // return 0 for pseudo color and 1 for true color
   if (this->pseudoColor)
      return 0;
   return 1;
}

void VtkImage::select ()
{
#ifndef HEADLESS
  if (this->imageWidget)
    this->imageWidget->Select (true);
#endif
}

void VtkImage::unselect ()
{
#ifndef HEADLESS
  if (this->imageWidget)
    this->imageWidget->Select (false);
#endif
}

void VtkImage::disable_interaction ()
{
#ifndef HEADLESS
  vtkChipsRepresentation *rep = reinterpret_cast<vtkChipsRepresentation *> (this->imageWidget->GetRepresentation ());
  rep->DisableBorder ();
  this->imageWidget->EnabledOff ();
#endif
}

void VtkImage::enable_interaction ()
{
#ifndef HEADLESS
  if (this->is_interaction_blocked ())
	return;
  vtkChipsRepresentation *rep = reinterpret_cast<vtkChipsRepresentation *> (this->imageWidget->GetRepresentation ());

  this->imageWidget->EnabledOn ();
  rep->EnableBorder ();
#endif
}

