// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKIMAGE_HH
#define VTKIMAGE_HH


#include "vtkRenderer.h"
#include "vtkActor2D.h"


class vtkTransform;
class vtkImageReslice;
class vtkImageChangeInformation;
class vtkImageMapToColors;
class vtkLookupTable;
class vtkImageAppendComponents;
class vtkImageMathematics;
#ifndef HEADLESS
class vtkChipsWidget;
#endif



#include "ChipsImage.hh"
#include "vtkImageActor.h"

class VtkImage: public ChipsImage
{
public:

  VtkImage(vtkRenderer* inRenderer,ChipsObject* par, string filename, 
	   string transform_id, string val);
  VtkImage(vtkRenderer* inRenderer,ChipsObject* par, string filename,
	   float x1, float x2, float y1, float y2, string transform_id,
           string val);
  VtkImage(vtkRenderer* inRenderer,ChipsObject* par, int xaxis, int yaxis, 
           double* rvals, double* gvals, double *bvals, double *avals,
           string transform_id, string val);
  VtkImage(vtkRenderer *inRenderer, ChipsObject* par, ChipsStateStream& in);
  VtkImage(const VtkImage& inImage);
  virtual ~VtkImage();
  
  virtual int change_visible ();
  virtual int change_rotation (double angle);
  virtual int change_position (double x, double y);
  virtual int change_origin (double x, double y, double scaleX, double scaleY);
  virtual int change_extent (int lower_x, int lower_y, int upper_x, int upper_y);
  virtual int change_scale (double scaleX, double scaleY=0);
  virtual int flip_axis (int axis);
  virtual int change_depth ();

  virtual int change_interpolation ();
  virtual int change_normalize_channels ();
  virtual int change_mode (int use_component);
  virtual int add_component (double* vals, int channel); 
  virtual int remove_component (int channel);

  virtual int clip_image (bool clip);

  virtual int doprint (string output, int fmt);

  virtual int set_plot_location (double x1, double y1, double x2, double y2);

  virtual void serialize(ChipsStateStream& out);
/*  virtual void serialize_ascii(ChipsStateStream& out);*/
  virtual void deserialize(ChipsStateStream& in);

  virtual double *get_data (int channel, bool ignore_zero=false);

  virtual void select ();
  virtual void unselect ();

  virtual void disable_interaction ();
  virtual void enable_interaction ();


protected:

  virtual int     get_mode ();

  void Initialize ();
  void InitializeTrueColor ();
  void InitializePseudoColor ();
  int LoadImage(const string& inFileName);
  void fill_zero_data (int xdim, int ydim);
  vtkImageData *create_image (double *data, int xdim, int ydim);



private:
  vtkRenderer* renderer;
  vtkActor2D* vtkImage;

  vtkTransform *transform;
  vtkImageReslice *reslice;
  vtkImageChangeInformation *changeInformation;
  vtkImageAppendComponents *appendComponents;
 
  vtkImageMathematics *scaleFilter[4];
  vtkImageMathematics *offsetFilter[4];

  vtkImageMapToColors* colorMap;

  vtkImageData* imageData[4];
  vtkImageData* zeroData;

  int imageCount;
  bool zeroDataFilled;
  bool pseudoColor;

  bool clipImage;

#ifndef HEADLESS
  vtkChipsWidget *imageWidget;
#endif
};





#endif
