#include "VtkLabelInterface.hh"

#include "vtkTextActorExtra.hh"
#include "vtkTextProperty.h"
#include "vtkMath.h"
#include "vtkViewport.h"

#include "ChipsDefinesP.hh"

#include <cmath>

void VtkLabelInterface::AddFormattedText(ELEM_P_T inFormat,vtkTextActorExtra* inTextActor, double scale_factor, bool inRootFlag)
{

  if ((inTextActor ) && ( inFormat))
    {
      inTextActor->SetRootFlag(inRootFlag);
      inTextActor->SetInput(inFormat->text.c_str());
      inTextActor->GetTextProperty()->SetFaceFileName(0);
      if ( inFormat->fontfile == "DEFAULT_HELVETICA" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToHelvetica();
          inTextActor->GetTextProperty()->BoldOff();
          inTextActor->GetTextProperty()->ItalicOff();
        }
      else if ( inFormat->fontfile == "DEFAULT_HELVETICA_BOLD" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToHelvetica();
          inTextActor->GetTextProperty()->BoldOn();
          inTextActor->GetTextProperty()->ItalicOff();
        }
      else if ( inFormat->fontfile == "DEFAULT_HELVETICA_ITALIC" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToHelvetica();
          inTextActor->GetTextProperty()->BoldOff();
          inTextActor->GetTextProperty()->ItalicOn();
        }
      else if ( inFormat->fontfile == "DEFAULT_HELVETICA_BOLDITALIC" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToHelvetica();
          inTextActor->GetTextProperty()->BoldOn();
          inTextActor->GetTextProperty()->ItalicOn();
        }
      else if ( inFormat->fontfile == "DEFAULT_COURIER" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToCourier();
          inTextActor->GetTextProperty()->BoldOff();
          inTextActor->GetTextProperty()->ItalicOff();
        }
      else if ( inFormat->fontfile == "DEFAULT_COURIER_BOLD" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToCourier();
          inTextActor->GetTextProperty()->BoldOn();
          inTextActor->GetTextProperty()->ItalicOff();
        }
      else if ( inFormat->fontfile == "DEFAULT_COURIER_ITALIC" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToCourier();
          inTextActor->GetTextProperty()->BoldOff();
          inTextActor->GetTextProperty()->ItalicOn();
        }
      else if ( inFormat->fontfile == "DEFAULT_COURIER_BOLDITALIC" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToCourier();
          inTextActor->GetTextProperty()->BoldOn();
          inTextActor->GetTextProperty()->ItalicOn();
        }
      else if ( inFormat->fontfile == "DEFAULT_TIMES" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToTimes();
          inTextActor->GetTextProperty()->BoldOff();
          inTextActor->GetTextProperty()->ItalicOff();
        }
      else if ( inFormat->fontfile == "DEFAULT_TIMES_BOLD" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToTimes();
          inTextActor->GetTextProperty()->BoldOn();
          inTextActor->GetTextProperty()->ItalicOff();
        }
       else if ( inFormat->fontfile == "DEFAULT_TIMES_ITALIC" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToTimes();
          inTextActor->GetTextProperty()->BoldOff();
          inTextActor->GetTextProperty()->ItalicOn();
        }
      else if ( inFormat->fontfile == "DEFAULT_TIMES_BOLDITALIC" )
        {
          inTextActor->GetTextProperty()->SetFontFamilyToTimes();
          inTextActor->GetTextProperty()->BoldOn();
          inTextActor->GetTextProperty()->ItalicOn();
        }
      else if( ( inFormat->fontfile == "DEFAULT_GREEK" )
               ||(inFormat->fontfile == "DEFAULT_GREEK_BOLD")
               ||(inFormat->fontfile == "DEFAULT_GREEK_ITALIC")
               ||(inFormat->fontfile == "DEFAULT_GREEK_BOLDITALIC"))
        {
          inTextActor->GetTextProperty()->SetFontFamilyToGreek();
          inTextActor->GetTextProperty()->BoldOff();
          inTextActor->GetTextProperty()->ItalicOff();
        }
      else
        {
          inTextActor->GetTextProperty()->SetFaceFileName(inFormat->fontfile.c_str());
        }

      inTextActor->GetTextProperty()->SetFontSize(inFormat->size * scale_factor);
      inTextActor->GetTextProperty()->SetColor(inFormat->color.red(),inFormat->color.green(),inFormat->color.blue() );

      if ( inFormat->super )
        {
          AddFormattedText(inFormat->super,inTextActor->GetSuperTextActorExtra(),scale_factor,false);
        };

      if ( inFormat->sub )
        {
          AddFormattedText(inFormat->sub,inTextActor->GetSubTextActorExtra(),scale_factor,false);
        };

      if ( inFormat->next )
        {
          AddFormattedText(inFormat->next,inTextActor->GetNextTextActorExtra(),scale_factor,false);
        };

      inTextActor->Modified();
    };
};

int VtkLabelInterface::ComputeFontSizeAndPosFromBBox (vtkTextActorExtra* inTextActor,
                                                vtkViewport* viewport,
                                                double bbox[],
                                                double outPosition[])
{

  double xRange[2];
  double yRange[2];
  double tmpCoord[2];
  double angle;
  double currBBox[4];

  int fontSize = inTextActor->GetTextProperty()->GetFontSize();
  int tmpFontSize = fontSize;

  double textHalign = inTextActor->GetHorizAlignmentOffset ();
  double textValign = inTextActor->GetVertAlignmentOffset ();

  double halign = .5 - textHalign;
  double valign = .5 - textValign;

  double *pos = inTextActor->GetPositionCoordinate()->GetComputedDoubleViewportValue(viewport);
  double origPosition[2] = {pos[0], pos[1]};
  double newPosition[2] = {pos[0], pos[1]};

  bool greater=false;
  bool lesser=false;

  vtkCoordinate *refCoordinate = inTextActor->GetPositionCoordinate ()->GetReferenceCoordinate ();

  if (refCoordinate)
  {
     double *refPosition = refCoordinate->GetComputedDoubleViewportValue(viewport);
     newPosition[0] -= refPosition[0];
     newPosition[1] -= refPosition[1];
  }

  inTextActor->GetBBox (viewport, currBBox[0], currBBox[1],
                                    currBBox[2], currBBox[3]);

  // Move the label anchor to the center and reposition it so match the
  // the current position

  inTextActor->SetVertAlignmentOffset(.5);
  inTextActor->SetHorizAlignmentOffset(.5);

  angle=vtkMath::RadiansFromDegrees(inTextActor->GetTextProperty()->GetOrientation());

  inTextActor->GetAlignmentRange(viewport,xRange,yRange);

  tmpCoord[0]=(xRange[1]-xRange[0])*halign;
  tmpCoord[1]=yRange[1]*valign;

  newPosition[0] += tmpCoord[0]*cos(angle) - tmpCoord[1]*sin(angle);
  newPosition[1] += tmpCoord[0]*sin(angle) + tmpCoord[1]*cos(angle);

  inTextActor->GetPositionCoordinate()->SetCoordinateSystemToViewport ();
  inTextActor->GetPositionCoordinate()->SetValue (newPosition);


  // increase the size of the font until its with the bounding box

  greater = ((bbox[0] < currBBox[0]) && (bbox[1] < currBBox[1]) &&
             (bbox[2] > currBBox[2]) && (bbox[3] > currBBox[3]));


  while (greater)
    {
    // assume there's an error
    if (tmpFontSize > V_SIZE_MAX_VAL)
       break;
    inTextActor->GetTextProperty()->SetFontSize(++tmpFontSize);
    inTextActor->GetBBox (viewport, currBBox[0], currBBox[1],
                                            currBBox[0], currBBox[1]);
    greater = ((bbox[0] < currBBox[0]) && (bbox[1] < currBBox[1]) &&
             (bbox[2] > currBBox[2]) && (bbox[3] > currBBox[3]));
    }

  lesser = ((bbox[0] > currBBox[0]) && (bbox[1] > currBBox[1]) &&
             (bbox[2] < currBBox[2]) && (bbox[3] < currBBox[3]));

  while (lesser)
    {
    // there's an error
    if (tmpFontSize <= 0)
       break;

    inTextActor->GetTextProperty()->SetFontSize(--tmpFontSize);
    inTextActor->GetBBox (viewport, currBBox[0], currBBox[1],
                                            currBBox[0], currBBox[1]);
    lesser = ((bbox[0] > currBBox[0]) || (bbox[1] > currBBox[1]) ||
             (bbox[2] < currBBox[2]) || (bbox[3] < currBBox[3]));

    }

  // compute the new position of the label

  halign = -.5 + textHalign;
  valign = -.5 + textValign;

  inTextActor->GetAlignmentRange(viewport,xRange,yRange);

  tmpCoord[0]=(xRange[1]-xRange[0])*halign;
  tmpCoord[1]=yRange[1]*valign;


  outPosition[0] = tmpCoord[0]*cos(angle) - tmpCoord[1]*sin(angle) + newPosition[0];
  outPosition[1] = tmpCoord[0]*sin(angle) + tmpCoord[1]*cos(angle) + newPosition[1];

  if (refCoordinate)
  {
     double *refPosition = refCoordinate->GetComputedDoubleViewportValue(viewport);
     outPosition[0] += refPosition[0];
     outPosition[1] += refPosition[1];
  }



  inTextActor->SetVertAlignmentOffset(textValign);
  inTextActor->SetHorizAlignmentOffset(textHalign);

  inTextActor->GetTextProperty()->SetFontSize(fontSize);
  inTextActor->GetPositionCoordinate ()->SetValue(origPosition[0], origPosition[0]);

  return tmpFontSize;
  
}
