// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "VtkLine.hh"

#include "vtkPoints.h"
#include "vtkCellArray.h"
#include "vtkProperty2D.h"
#include "vtkRenderWindow.h"
#include "vtkPolyData.h"

#include "ChipsFrameSystem.hh"
#include "ChipsDataSystem.hh"

#ifndef HEADLESS
#include "vtkChipsPickableWidget.hh"
#include "vtkPolyLineRepresentation.hh"
#endif

#include "VtkUtils.hh"

VtkLine::VtkLine(vtkRenderer* inRenderer,ChipsObject* par,
        ChipsCoordinate *inCoord, COORD_SYS coord_system, 
	double startx, double starty,
	double endx, double endy, string lnid):ChipsLine(par,inCoord,coord_system,startx,starty,endx,endy,lnid)
{
   ChipsId cid;
   renderer=inRenderer;

   if (!renderer)
      return;

   line_actor = vtkPolyLineActor2DExtra::New ();
   renderer->AddActor2D(line_actor); 

#ifndef HEADLESS
   vtkPolyLineRepresentation *line_rep = vtkPolyLineRepresentation::New ();
   line_rep->SetLineActor (line_actor);

   this->populate_chips_id (cid);
   line_widget = vtkChipsPickableWidget::New ();
   line_widget->SetDefaultRenderer (renderer);
   line_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   line_widget->SetRepresentation (line_rep);
   line_widget->SetIdInfo (cid, this->get_object_type());
   line_widget->ResizableOff ();
   line_widget->RotateableOff ();
   line_widget->SetDefaultRenderer (renderer);
   line_widget->EnabledOn ();
   line_rep->Delete ();
#endif

   create_polyline ();

   change_position();
   change_linestyle();
   change_thickness();
   change_color();
};

VtkLine::VtkLine(vtkRenderer* inRenderer,ChipsObject* par, 
                 ChipsCoordinate *inCoord, COORD_SYS coord_system,
		 double* xcoords, double* ycoords,
		 unsigned int cnt, string lnid):ChipsLine(par,inCoord,coord_system,xcoords,ycoords,cnt,lnid)
{
   ChipsId cid;
   renderer=inRenderer;
   
   if (!renderer)
      return;
   line_actor = vtkPolyLineActor2DExtra::New ();
   renderer->AddActor2D(line_actor);

#ifndef HEADLESS
   vtkPolyLineRepresentation *line_rep = vtkPolyLineRepresentation::New ();
   line_rep->SetLineActor (line_actor);

   this->populate_chips_id (cid);
   line_widget = vtkChipsPickableWidget::New ();
   line_widget->SetDefaultRenderer (renderer);
   line_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   line_widget->SetRepresentation (line_rep);
   line_widget->SetIdInfo (cid, this->get_object_type());
   line_widget->ResizableOff ();
   line_widget->RotateableOff ();
   line_widget->SetEnabled (1);
#endif

   create_polyline ();
   

   change_position();
   change_linestyle();
   change_thickness();
   change_color();
}

VtkLine::VtkLine(vtkRenderer* inRenderer,ChipsObject* par, 
   ChipsCoordinate *inCoord, COORD_SYS coord_sys, ChipsStateStream& in):
   ChipsLine(par, inCoord, coord_sys, in)
{
   ChipsId cid;
   renderer=inRenderer;


   if (!renderer)
      return;
   line_actor = vtkPolyLineActor2DExtra::New ();
   renderer->AddActor2D(line_actor);

#ifndef HEADLESS
   vtkPolyLineRepresentation *line_rep = vtkPolyLineRepresentation::New ();
   line_rep->SetLineActor (line_actor);

   this->populate_chips_id (cid);
   line_widget = vtkChipsPickableWidget::New ();
   line_widget->SetDefaultRenderer (renderer);
   line_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
   line_widget->SetRepresentation (line_rep);
   line_widget->SetIdInfo (cid, this->get_object_type());
   line_widget->ResizableOff ();
   line_widget->RotateableOff ();
   line_widget->SetEnabled (1);
#endif

   create_polyline ();

   change_position();
   change_linestyle();
   change_thickness();
   change_color();
   change_visible();
}


VtkLine::~VtkLine()
{
   renderer->RemoveActor2D (line_actor);
   line_actor->Delete ();
#ifndef HEADLESS
   line_widget->Delete ();
#endif

};

int VtkLine::redraw()
{
  return ERR_FEATURE_DNE;
};

int VtkLine::change_position()
{
   if (line_actor)
     {
       ChipsCoordinate *tmp_coord1 = NULL;
       int tmpFlag[] = {0,0};
       double tmpDataXRange[] = {0.0, 1.0};
       double tmpDataYRange[] = {0.0, 1.0};
       double tmpActorLow[] = {0.0, 0.0};
       double tmpActorHigh[] = {1.0, 1.0};

       switch (coord_system)
         {
         case DATA:
           {
             ChipsDataSystem *tmp_coord = NULL;
             if (coord)
                tmp_coord = dynamic_cast <ChipsDataSystem *>(coord);
             if (tmp_coord != NULL)
               {
                 tmpDataXRange[0]=tmp_coord->get_lower_bounds ()[0];
                 tmpDataXRange[1]=tmp_coord->get_upper_bounds ()[0];
                 tmpDataYRange[0]=tmp_coord->get_lower_bounds ()[1];
                 tmpDataYRange[1]=tmp_coord->get_upper_bounds ()[1];

                 if ( tmp_coord->get_log_flag (X_AXIS) )
                   tmpFlag[0]=1;
                 if ( tmp_coord->get_log_flag (Y_AXIS) )
                   tmpFlag[1]=1;
               }
             else
               return ERR_STATUS_BAD;
           }
         case PLOT_NORM :
           if (coord)
              tmp_coord1 = coord->get_coord (PLOT_NORM, "");

           if (tmp_coord1)
           {
              tmpActorLow[0]= tmp_coord1->get_lower_bounds ()[0];
              tmpActorLow[1]= tmp_coord1->get_lower_bounds ()[1];
              tmpActorHigh[0]= tmp_coord1->get_upper_bounds ()[0];
              tmpActorHigh[1]= tmp_coord1->get_upper_bounds ()[1];
           }

           break;

         case FRAME_NORM:
           break;
         case WINDOW_NORM:
           if (coord)
              tmp_coord1 = coord->get_coord (FRAME_NORM, "");
           if (tmp_coord1 != NULL)
             {
               tmpDataXRange[0]=tmp_coord1->get_lower_bounds ()[0];
               tmpDataXRange[1]=tmp_coord1->get_upper_bounds ()[0];
               tmpDataYRange[0]=tmp_coord1->get_lower_bounds ()[1];
               tmpDataYRange[1]=tmp_coord1->get_upper_bounds ()[1];
             }
           else
             return ERR_STATUS_BAD;
           break;
         case PIXEL:
           if (coord)
              tmp_coord1 = coord->get_coord (PIXEL, "");
           if (tmp_coord1 != NULL)
             {
               tmpDataXRange[0]=tmp_coord1->get_lower_bounds ()[0];
               tmpDataXRange[1]=tmp_coord1->get_upper_bounds ()[0];
               tmpDataYRange[0]=tmp_coord1->get_lower_bounds ()[1];
               tmpDataYRange[1]=tmp_coord1->get_upper_bounds ()[1];
             }
           else
             return ERR_STATUS_BAD;
           break;
         case PIXEL_REAL:
         case COORD_NONE:
         default:
           return ERR_STATUS_BAD;

         };

       VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmpActorLow);
       VtkUtils::FrameNorm2ViewportNorm (coord, renderer, tmpActorHigh);

       line_actor->SetLocation(tmpActorLow[0],tmpActorLow[1],tmpActorHigh[0],tmpActorHigh[1]);
       line_actor->SetXRange(tmpDataXRange);
       line_actor->SetXScale(tmpFlag[0]);
       line_actor->SetYRange(tmpDataYRange);
       line_actor->SetYScale(tmpFlag[1]);

#ifndef HEADLESS
       if (line_widget)
         line_widget->UpdateRepresentation ();
#endif

     };


   return ERR_STATUS_GOOD;
};

int VtkLine::change_linestyle()
{
   if (!line_actor)
      return ERR_STATUS_BAD;

   switch (linestyle){
      case chips_noline:
	 line_actor->SetVisibility(0);
         break;
      case chips_dotlongdash:
         line_actor->SetVisibility(1);
         line_actor->GetProperty()->SetLineStipplePattern(chips_dotlongdash_pattern);
         break;
      case chips_dotshortdash:
         line_actor->SetVisibility(1);
         line_actor->GetProperty()->SetLineStipplePattern(chips_dotshortdash_pattern);
         break;
      case chips_longdash:
         line_actor->SetVisibility(1);
         line_actor->GetProperty()->SetLineStipplePattern(chips_longdash_pattern);
         break;
      case chips_shortdash:
         line_actor->SetVisibility(1);
         line_actor->GetProperty()->SetLineStipplePattern(chips_shortdash_pattern);
         break;
      case chips_shortdashlongdash:
	 line_actor->SetVisibility(1);
         line_actor->GetProperty()->SetLineStipplePattern(chips_shortdashlongdash_pattern);
         break;
      case chips_dot:
         line_actor->SetVisibility(1);
         line_actor->GetProperty()->SetLineStipplePattern(chips_dot_pattern);
         break;
      case chips_solid:
      default:
         line_actor->SetVisibility(1);
         line_actor->GetProperty()->SetLineStipplePattern(chips_solid_pattern);
	 break;

   };

   return ERR_STATUS_GOOD;

};

int VtkLine::change_thickness()
{

   if (line_actor)
     line_actor->GetProperty()->SetLineWidth(thickness);
   return ERR_STATUS_GOOD;
};

int VtkLine::change_color ()
{
   if (line_actor)
     line_actor->GetProperty()->SetColor(color.red(),color.green(),color.blue());
   return ERR_STATUS_GOOD;
};

int VtkLine::change_depth ()
{
  if (line_actor)
     line_actor->SetLayerNumber(depth*10+1);
  return ERR_STATUS_GOOD;
};


int VtkLine::change_visible ()
{
   if (!line_actor)
      return ERR_STATUS_BAD;

   int tmp_vis = (get_parent()->is_visible ()) ? is_visible() : 0;
   if (linestyle == chips_noline) tmp_vis = 0;
   line_actor->SetVisibility(tmp_vis);

#ifndef HEADLESS
   line_widget->GetRepresentation ()->SetVisibility (tmp_vis);
#endif

   return ERR_STATUS_GOOD;
}

int VtkLine::update_data ()
{
   if (line_actor)
     {
       create_polyline();

       change_position();
       change_color();
       change_thickness();
       change_linestyle();
       change_visible();
     };

   return ERR_STATUS_GOOD;
}


void VtkLine::create_polyline ()
{
  vtkPolyData *lineData = vtkPolyData::New ();

  lineData->Initialize ();

  vtkPoints* points = vtkPoints::New();
  points->SetDataTypeToDouble();
  int tmpCount=num_pnts;

  for (int i=0;i <tmpCount; i++)
    {
      points->InsertPoint(i,pnts[i][0],pnts[i][1],0);


    };
  lineData->SetPoints(points);

  line_actor->SetData (lineData);
  line_actor->SetClipData (coord_system == DATA);
//  line_actor->SetClipData (false);

  points->Delete();
  lineData->Delete ();

#ifndef HEADLESS
  if (line_widget)
    line_widget->UpdateRepresentation ();
#endif


};

void VtkLine::select ()
{
#ifndef HEADLESS
  if (line_widget)
    line_widget->Select (true);
#endif
}

void VtkLine::unselect ()
{
#ifndef HEADLESS
  if (line_widget)
    line_widget->Select (false);
#endif
}

void VtkLine::disable_interaction ()
{
#ifndef HEADLESS
  vtkPolyLineRepresentation *rep = reinterpret_cast<vtkPolyLineRepresentation *> (this->line_widget->GetRepresentation ());
  rep->DisableBorder ();
  this->line_widget->EnabledOff ();
#endif
}

void VtkLine::enable_interaction ()
{
#ifndef HEADLESS
  if (this->is_interaction_blocked ())
	return;
  vtkPolyLineRepresentation *rep = reinterpret_cast<vtkPolyLineRepresentation *> (this->line_widget->GetRepresentation ());

  this->line_widget->EnabledOn ();
  rep->EnableBorder ();
#endif
}

int VtkLine::update_clipping ()
{
  line_actor->SetClipData (coord_system == DATA);
  return ERR_STATUS_GOOD;
}
