// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKPLOT_HH
#define VTKPLOT_HH


#include <string>

#include "vtkRenderer.h"
#include "vtkActor.h"

#include "ChipsPlot.hh"
#include "ChipsAxisAlgorithmFactory.hh"

#ifndef HEADLESS
class vtkBoundingBoxWidget;
class vtkAddAnnotationWidget;
class vtkChipsPlotWidget;
class vtkChipsRepresentation;
#endif


using namespace std;

class VtkPlot: public ChipsPlot
{
public:
  VtkPlot(vtkRenderer* inRenderer, ChipsObject* par, const string& val="");
  VtkPlot(vtkRenderer* inRenderer, ChipsObject* par, ChipsStateStream&  in);
  VtkPlot(vtkRenderer* inRenderer, ChipsObject* par, 
	  float x1, float y1, float x2, float y2, 
	  const string& val="");
  virtual ~VtkPlot();
  
  virtual int change_visible ();
  virtual void enable_picklimits (bool enabled);
  virtual void enable_addannotation (bool enabled);
  virtual int recalculate_position ();
  virtual void select ();
  virtual void unselect ();
  virtual void disable_interaction ();
  virtual void enable_interaction ();



  virtual ChipsCurve* new_curve(long cnt, double* x, double* y, double* ydown,
                                double* yup, double* xdown, double* xup,
                                double* ylims, double* xlims,
				const string &transform_id, const string & cvid);

  virtual ChipsHistogram* new_histogram(long cnt, double* lo, double* hi, 
                                        double* bincnts, double* yd, double* yu,
                                        const string& hid);

  virtual ChipsAxis* new_axis(ChipsCoordinate *inCoord, COORD_SYS sys,
                              double inMinX, double inMinY, double inMaxX,
                              double inMaxY,int inOrientation,
                              ChipsAxisAlgorithmFactory::TransformType type,
                              const string& aid);
  virtual ChipsAxis* new_axis(ChipsCoordinate *inCoord, COORD_SYS sys,
			      double inMinX, double inMinY, double inMaxX,
                              double inMaxY,int inOrientation,
			      double inMin,double inMax, 
                              ChipsAxisAlgorithmFactory::TransformType type,
                              const string& aid);

  virtual ChipsLabel* new_label(ChipsCoordinate *inCoord, COORD_SYS coord_sys,
          double xpos, double ypos, string text, string tid="");
  virtual ChipsLabel* new_label(ChipsStateStream& in);
  virtual ChipsLine* new_line(ChipsCoordinate *inCoord, COORD_SYS coord_sys,
     double startx, double starty, double endx, double endy, string lnid="");
  virtual ChipsLine* new_line(ChipsCoordinate *inCoord, COORD_SYS coord_sys,
          double* xx, double* yy, unsigned int count, string lnid="");
  virtual ChipsLine* new_line(ChipsStateStream& in);

  virtual ChipsPoint* new_point(ChipsCoordinate *inCoord, COORD_SYS coord_sys,
        double xx, double yy, string ptid="");
  virtual ChipsPoint* new_point(ChipsCoordinate *inCoord, COORD_SYS coord_sys,
        double xx, double yy, double zz, string ptid="");
  virtual ChipsPoint* new_point(ChipsStateStream& in);

  virtual ChipsImage* new_image(ChipsStateStream& in);
  virtual ChipsImage* new_image(string fname, string transform_id,
                                string iid="");
  virtual ChipsImage* new_image(string fname, float posx1, float posy1,
                                float posx2, float posy2, string transform_id, 
                                string iid="");
  virtual ChipsImage* new_image(double* rvals, double *gvals, double *bvals, 
                                double *avals, int xaxis, int yaxis, 
                                string transform_id, string iid="");

  virtual ChipsRegion* new_polygon(ChipsCoordinate *inCoord, COORD_SYS sys,
                                   double* xx, double* yy, unsigned int count,
                                   string rid);

  virtual ChipsRegion* new_polygon(ChipsCoordinate *inCoord, COORD_SYS sys,
                                   int sides, double xx, double yy,
                                   double radius, string rid);

  virtual ChipsRegion* new_polygon(ChipsStateStream& in);

  virtual ChipsColorBar* new_colorbar(ChipsCoordinate *inCoord, COORD_SYS sys,
      double xpos, double ypos, string tid="");
  virtual ChipsColorBar* new_colorbar(ChipsCoordinate *inCoord, COORD_SYS sys,
      double xpos, double ypos, unsigned short cmap_id, string tid="");
  virtual ChipsColorBar* new_colorbar(ChipsStateStream& in);



  virtual ChipsAxis* new_axis(ChipsStateStream& in);
  virtual ChipsCurve* new_curve(ChipsStateStream& in);
  virtual ChipsContour* new_contour(ChipsStateStream& in);
  virtual ChipsHistogram* new_histogram(ChipsStateStream& in);

  virtual ChipsContour* new_contour(long cnt, double* x, double* y, double* z,
				    unsigned int count, double *levels,
				    string transform_id, const string& ctid);

protected:

  virtual void set_data_aspect_ratio (const ChipsId *cid);

#ifndef HEADLESS
  vtkBoundingBoxWidget *bounding_box;
  vtkAddAnnotationWidget *annotation_widget;
  vtkChipsPlotWidget *plot_widget;
  vtkChipsRepresentation *plot_rep;
#endif

private:
  VtkPlot(const VtkPlot& inPlot);
  vtkRenderer* renderer;


};





#endif
