// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "VtkPoint.hh"
#include "vtkPolyData.h"
#include "vtkPoints.h"
#include "vtkProperty2D.h"
#include "vtkRenderWindow.h"
#include "ChipsFrameSystem.hh"
#include "VtkUtils.hh"

#ifndef HEADLESS
#include "vtkChipsPointRepresentation.hh"
#include "vtkChipsPointWidget.hh"
#endif


VtkPoint::VtkPoint(vtkRenderer* inRenderer,ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, double xx, double yy, string ptid):ChipsPoint(par,inCoord,coord_sys,xx,yy,ptid)
{
  this->renderer=inRenderer;

  vtkPoints *pnts = vtkPoints::New();
  pnts->SetDataTypeToDouble();
  pnts->SetNumberOfPoints (1);
  
  pnts->InsertPoint(0, 0.0, 0.0, 0.0);

  this->point_data = vtkPolyData::New();
  this->point_data->Initialize ();
  this->point_data->SetPoints(pnts);

  this->glyph_data = vtkGlyph2D::New();
  this->glyph_data->SetInput(this->point_data);
  this->glyph_data->ScalingOff();
  this->glyph_data->SetIndexModeToOff();
//  this->glyph_data->SetNumberOfSources(1);

  this->glyph_source = vtkGlyphSource2D::New();
  this->glyph_source->SetRotationAngle(0);
  this->glyph_data->SetSource(this->glyph_source->GetOutput());


  for(int indx=0;indx<4;indx++)
    {
      clipPlanes[indx]=vtkPlane::New();
    }
   

  clipPlanes[0]->SetOrigin(0,0,0);
  clipPlanes[0]->SetNormal(1,0,0);

  clipPlanes[1]->SetOrigin(1,0,0);
  clipPlanes[1]->SetNormal(-1,0,0);

  clipPlanes[2]->SetOrigin(0,0,0);
  clipPlanes[2]->SetNormal(0,1,0);

  clipPlanes[3]->SetOrigin(0,1,0);
  clipPlanes[3]->SetNormal(0,-1,0);


  this->point_mapper = vtkPolyDataMapper2D::New();

// The following will not work because the clipped data screws
// up the gl2ps output. Must use clip filter instead
  this->point_mapper->SetInput(this->glyph_data->GetOutput ());

  if (coord_system == DATA)
  {
    for (int ii=0; ii<4; ii++)
      point_mapper->AddClippingPlane (clipPlanes[ii]);
    this->clip_data = true;
  }
  else
    this->clip_data = false;


  this->point_actor = vtkActor2D::New();
  this->point_actor->SetMapper(this->point_mapper);

  point_actor->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
  point_actor->GetPositionCoordinate()->SetValue(0.0, 0.0);
  point_actor->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
  point_actor->GetPosition2Coordinate()->SetValue(1.0, 1.0);

  renderer->AddActor2D(point_actor);

#ifndef HEADLESS
  // update the gui
  ChipsId cid;
  vtkChipsPointRepresentation *point_rep = vtkChipsPointRepresentation::New ();
  point_rep->SetPickProp (point_actor);
  point_rep->SetGlyphSource (this->glyph_source);

  this->populate_chips_id (cid);
  point_widget = vtkChipsPointWidget::New ();
  point_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  point_widget->SetRepresentation (point_rep);
  point_widget->SetIdInfo (cid, this->get_object_type());
  point_widget->SetDefaultRenderer (renderer);
  point_widget->EnabledOn ();

  point_rep->Delete ();
#endif

  change_position ();
  change_filled();
  change_color();
  change_style();
  change_size();
  pnts->Delete ();

};

VtkPoint::VtkPoint(vtkRenderer* inRenderer,ChipsObject* par,  
   ChipsCoordinate *inCoord, COORD_SYS coord_sys, ChipsStateStream& in):
   ChipsPoint(par, inCoord, coord_sys, in)
{
  this->renderer=inRenderer;


  vtkPoints *pnts = vtkPoints::New();
  pnts->SetDataTypeToDouble();
  pnts->SetNumberOfPoints (1);

  pnts->InsertPoint(0, 0.0, 0.0, 0.0);

  this->point_data = vtkPolyData::New();
  this->point_data->Initialize ();
  this->point_data->SetPoints(pnts);

  this->glyph_data = vtkGlyph2D::New();
  this->glyph_data->SetInput(this->point_data);
  this->glyph_data->ScalingOff();
  this->glyph_data->SetIndexModeToOff();
//  this->glyph_data->SetNumberOfSources(1);

  this->glyph_source = vtkGlyphSource2D::New();
  this->glyph_source->SetRotationAngle(0);
  this->glyph_data->SetSource(this->glyph_source->GetOutput());

  for(int indx=0;indx<4;indx++)
    {
      clipPlanes[indx]=vtkPlane::New();
    }

  clipPlanes[0]->SetOrigin(0,0,0);
  clipPlanes[0]->SetNormal(1,0,0);

  clipPlanes[1]->SetOrigin(1,0,0);
  clipPlanes[1]->SetNormal(-1,0,0);

  clipPlanes[2]->SetOrigin(0,0,0);
  clipPlanes[2]->SetNormal(0,1,0);

  clipPlanes[3]->SetOrigin(0,1,0);
  clipPlanes[3]->SetNormal(0,-1,0);

  this->point_mapper = vtkPolyDataMapper2D::New();

// The following will not work because the clipped data screws
// up the gl2ps output. Must use clip filter instead
 this->point_mapper->SetInput(this->glyph_data->GetOutput ());

  this->point_actor = vtkActor2D::New();
  this->point_actor->SetMapper(this->point_mapper);

  if (coord_system == DATA)
  {
    for (int ii=0; ii<4; ii++)
      point_mapper->AddClippingPlane (clipPlanes[ii]);
    this->clip_data = true;
  }
  else 
    this->clip_data = false;



  this->point_actor = vtkActor2D::New();
  this->point_actor->SetMapper(this->point_mapper);

  point_actor->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();
  point_actor->GetPositionCoordinate()->SetValue(0.0, 0.0);
  point_actor->GetPosition2Coordinate()->SetCoordinateSystemToNormalizedViewport();
  point_actor->GetPosition2Coordinate()->SetValue(1.0, 1.0);

  renderer->AddActor2D(point_actor);

#ifndef HEADLESS
  // update the gui
  ChipsId cid;
  vtkChipsPointRepresentation *point_rep = vtkChipsPointRepresentation::New ();
  point_rep->SetPickProp (point_actor);
  point_rep->SetGlyphSource (this->glyph_source);

  this->populate_chips_id (cid);
  point_widget = vtkChipsPointWidget::New ();
  point_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
  point_widget->SetRepresentation (point_rep);
  point_widget->SetIdInfo (cid, this->get_object_type());
  point_widget->SetDefaultRenderer (renderer);
  point_widget->EnabledOn ();

  point_rep->Delete ();
#endif

  change_position ();
  change_filled();
  change_color();
  change_style();
  change_size();
  change_visible ();

  pnts->Delete ();

};


VtkPoint::VtkPoint(vtkRenderer* inRenderer,ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, double xx, double yy, double zz, string ptid):ChipsPoint(par,inCoord,coord_sys,xx,yy,zz,ptid)
{
  renderer=inRenderer;
};


VtkPoint::~VtkPoint()
{
  renderer->RemoveActor2D (point_actor);
  glyph_source->Delete ();
  point_data->Delete ();
  glyph_data->Delete ();
  point_mapper->Delete ();
  point_actor->Delete ();
  for(int indx=0;indx<4;indx++)
    {
      clipPlanes[indx]->Delete();
    }

#ifndef HEADLESS
  point_widget->Delete ();
#endif

  

};

int VtkPoint::redraw()
{
  change_filled();
  change_position();
  change_color();
  change_style();
  change_size();
  return ERR_STATUS_GOOD;
};

int VtkPoint::change_filled()
{
  if (filled)
     this->glyph_source->FilledOn();
  else
     this->glyph_source->FilledOff();

  this->point_actor->Modified();
  return ERR_STATUS_GOOD;
};

int VtkPoint::change_position()
{
     int rval = ERR_STATUS_GOOD;
     if (point_actor)
     {
       ChipsVector tmp_pnt (0,0);
       ChipsCoordinate *tmp_coord1 = NULL;
       double tmpActorLow[] = {0.0, 0.0};
       double tmpActorHigh[] = {1.0, 1.0};

       this->set_clip_data (coord_system == DATA);

       if (coord)
          rval = coord->convert_cs_to_cs (&pnt, &tmp_pnt, 1, coord_system, data_system_id, FRAME_NORM, "");

       if (!rval)
          point_actor->GetPositionCoordinate()->SetValue(tmp_pnt[0], tmp_pnt[1]);
       else
       {
          tmp_pnt[0] = -1;
          tmp_pnt[1] = -1;
       }
          point_actor->GetPositionCoordinate()->SetValue(-1, -1);

       VtkUtils::FrameNorm2ViewportNorm (coord, renderer, &tmp_pnt[0]);
       point_actor->GetPositionCoordinate ()->SetValue (tmp_pnt[0], tmp_pnt[1]);

       if ((coord_system == DATA) && !rval)
         {
           vtkCoordinate *Location[2];
           double *pos1, *pos2;
           double *tmp_pos1;

           if (coord)
              tmp_coord1 = coord->get_coord (PLOT_NORM, "");

           if (tmp_coord1)
           {
              tmpActorLow[0]= tmp_coord1->get_lower_bounds ()[0];
              tmpActorLow[1]= tmp_coord1->get_lower_bounds ()[1];
              tmpActorHigh[0]= tmp_coord1->get_upper_bounds ()[0];
              tmpActorHigh[1]= tmp_coord1->get_upper_bounds ()[1];
           }
           
           Location[0] = vtkCoordinate::New ();
           Location[1] = vtkCoordinate::New ();
	   Location[0]->SetCoordinateSystemToNormalizedViewport();
           Location[1]->SetCoordinateSystemToNormalizedViewport();


           Location[0]->SetValue(tmpActorLow[0],tmpActorLow[1]);
           Location[1]->SetValue(tmpActorHigh[0],tmpActorHigh[1]);

           pos1 = Location[0]->GetComputedDoubleViewportValue(renderer);
           pos2 = Location[1]->GetComputedDoubleViewportValue(renderer);

	   tmp_pos1 = point_actor->GetPositionCoordinate()->GetComputedDoubleViewportValue(renderer);

           clipPlanes[0]->SetOrigin(pos1[0]-tmp_pos1[0],0,0);
           clipPlanes[1]->SetOrigin(pos2[0]-tmp_pos1[0],0,0);
           clipPlanes[2]->SetOrigin(0,pos1[1]-tmp_pos1[1],0);
           clipPlanes[3]->SetOrigin(0,pos2[1]-tmp_pos1[1],0);

           Location[0]->Delete ();
           Location[1]->Delete ();
         };

      // This error is fine. It just indicates that the point needs
      // to be clipped
      if (rval == ERR_NEG_LOG_VAL)
        rval = ERR_STATUS_GOOD;

    }

  this->glyph_source->Modified ();

#ifndef HEADLESS
  if (point_widget)
    point_widget->UpdateRepresentation ();
#endif


  return rval;
};



int VtkPoint::change_color()
{
  this->point_actor->GetProperty()->SetColor(color.red(),color.green(),color.blue());
  this->point_actor->Modified();
  return ERR_STATUS_GOOD;
};

int VtkPoint::change_style()
{

  if (visible)
     this->show ();
  this->glyph_source->SetRotationAngle(angle);

#ifndef HEADLESS
  vtkChipsPointRepresentation *point_rep;
  point_rep = reinterpret_cast<vtkChipsPointRepresentation *>(point_widget->GetRepresentation ());
  point_rep->SetAngleOffset (0);
#endif
  switch (style)
    {
    case chips_none:
      this->point_actor->SetVisibility (0);
    case chips_circle:
      this->glyph_source->SetGlyphType(7);
      break;
    case chips_cross:
      this->glyph_source->SetGlyphType(4);
      this->glyph_source->SetRotationAngle(angle+45);
#ifndef HEADLESS
      point_rep->SetAngleOffset (45);
#endif
      break;
    case chips_diamond:
      this->glyph_source->SetGlyphType(8);
      break;
    case chips_downtriangle:
      this->glyph_source->SetGlyphType(5);
      this->glyph_source->SetRotationAngle(angle+180);
#ifndef HEADLESS
      point_rep->SetAngleOffset (180);
#endif
      break;
    case chips_plus:
      this->glyph_source->SetGlyphType(4);
      break;
    case chips_square:
      this->glyph_source->SetGlyphType(6);
      break;
    case chips_uptriangle:
      this->glyph_source->SetGlyphType(5);
      break;
    case chips_arrow:
      this->glyph_source->SetGlyphType(10);
      break;
    case chips_point_type:
    default:
      this->glyph_source->SetGlyphType(1);
    }
  this->point_actor->Modified();
  return ERR_STATUS_GOOD;
};

int VtkPoint::change_size ()
{
  this->glyph_source->SetScale(size*2);

#ifndef HEADLESS
  if (point_widget)
    point_widget->UpdateRepresentation ();
#endif

  return ERR_STATUS_GOOD;
}

int VtkPoint::change_rotation ()
{
  switch (style)
    {
    case chips_downtriangle:
      this->glyph_source->SetRotationAngle(angle+180);
      break;
    case chips_cross:
      this->glyph_source->SetRotationAngle(angle+45);
      break;
    default:
      this->glyph_source->SetRotationAngle(angle);
    }
  return ERR_STATUS_GOOD;
}

int VtkPoint::change_visible ()
{
  int tmp_vis = (!get_parent()->is_visible ()) ? 0 : is_visible();
  if (style == chips_none) tmp_vis = 0;
  this->point_actor->SetVisibility (tmp_vis);
  this->point_actor->Modified();

#ifndef HEADLESS
  this->point_widget->GetRepresentation ()->SetVisibility (tmp_vis);
#endif

  return ERR_STATUS_GOOD;

}

int VtkPoint::change_depth ()
{
  if (point_actor)
    point_actor->SetLayerNumber(depth*10+1);
  return ERR_STATUS_GOOD;
};

void VtkPoint::select ()
{
#ifndef HEADLESS
  if (point_widget)
    point_widget->Select (true);
#endif
}

void VtkPoint::unselect ()
{
#ifndef HEADLESS
  if (point_widget)
    point_widget->Select (false);
#endif
}

void VtkPoint::disable_interaction ()
{
#ifndef HEADLESS
  vtkChipsPointRepresentation *rep = reinterpret_cast<vtkChipsPointRepresentation *> (this->point_widget->GetRepresentation ());
  rep->DisableBorder ();
  this->point_widget->EnabledOff ();
#endif
}

void VtkPoint::enable_interaction ()
{
#ifndef HEADLESS
  if (this->is_interaction_blocked ())
	return;
  vtkChipsPointRepresentation *rep = reinterpret_cast<vtkChipsPointRepresentation *> (this->point_widget->GetRepresentation ());

  this->point_widget->EnabledOn ();
  rep->EnableBorder ();
#endif
}


int VtkPoint::compute_resize( double *bounds)

{
  ChipsVector center (pnt);
  double currBBox[4];
  int newSize;
  int rval;
  double dx, dy;

  if (coord)
     coord->convert_cs_to_cs (&pnt, &center, 1, coord_system, data_system_id, PIXEL_REAL, "");
  
  currBBox[0] = center[0] - (size*2);
  currBBox[1] = center[1] - (size*2);
  currBBox[2] = center[0] + (size*2);
  currBBox[3] = center[1] + (size*2);

  // if the bounds are greater than the current bounds
  if ((bounds[0] <= currBBox[0]) && (bounds[1] <= currBBox[1]) &&
             (bounds[2] >= currBBox[2]) && (bounds[3] >= currBBox[3]))
  {
    dx = fabs (bounds[0]-center[0]) > fabs (bounds[2]-center[0]) ? fabs (bounds[0]-center[0]) : fabs (bounds[2]-center[0]);
    dy = fabs (bounds[1]-center[1]) > fabs (bounds[3]-center[1]) ? fabs (bounds[1]-center[1]) : fabs (bounds[3]-center[1]);

    newSize = (dx > dy) ? (int)dx : (int)dy;
    newSize /= 2;
  }
  else
  {
    dx = fabs (bounds[0]-center[0]) < fabs (bounds[2]-center[0]) ? fabs (bounds[0]-center[0]) : fabs (bounds[2]-center[0]);
    dy = fabs (bounds[1]-center[1]) < fabs (bounds[3]-center[1]) ? fabs (bounds[1]-center[1]) : fabs (bounds[3]-center[1]);

    newSize = (dx < dy) ? (int)dx : (int)dy;
    newSize /= 2;

  }

  rval = this->set_property (&newSize, SET_POINT_SYMBOL_SIZE);

  return rval;
}

void VtkPoint::set_clip_data (int flag)
{
  if (this->clip_data == flag)
    return;

  this->clip_data = flag;

  if (!this->clip_data)
     this->point_mapper->RemoveAllClippingPlanes ();
  else
  {
      for (int ii=0; ii<4; ii++)
        this->point_mapper->AddClippingPlane (clipPlanes[ii]);
  }
}

