// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef VTKPOINT_HH
#define VTKPOINT_HH


#include "vtkRenderer.h"
#include "vtkGlyphSource2D.h"
#include "vtkGlyph2D.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkActor2D.h"
#include "vtkPlane.h"
#include "vtkClipPolyData.h"
#include "ChipsPoint.hh"

#ifndef HEADLESS
class vtkChipsPointWidget;
#endif

class VtkPoint: public ChipsPoint
{
public:

  VtkPoint(vtkRenderer* inRenderer,ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, double xx, double yy, string ptid);

  VtkPoint(vtkRenderer* inRenderer,ChipsObject* par, ChipsCoordinate *inCoord,
     COORD_SYS coord_sys, ChipsStateStream& in);

  VtkPoint(vtkRenderer* inRenderer,ChipsObject* par, ChipsCoordinate *inCoord, COORD_SYS coord_sys, double xx, double yy, double zz, string ptid);

/*  VtkPoint(const VtkPoint& inPoint);
*/
  virtual ~VtkPoint();

  virtual int redraw();
  virtual int change_filled();
  virtual int change_position();
  virtual int change_color();
  virtual int change_style();
  virtual int change_size();
  virtual int change_rotation();
  virtual int change_depth();
  virtual int change_visible();

  virtual int  compute_resize (double *bounds);

  virtual void select ();
  virtual void unselect ();

  virtual void disable_interaction ();
  virtual void enable_interaction ();


protected:

  void set_clip_data (int flag);

private:
  vtkRenderer	  	  *renderer;
  vtkGlyphSource2D        *glyph_source;
  vtkGlyph2D              *glyph_data;
  vtkPolyDataMapper2D     *point_mapper;
  vtkActor2D              *point_actor;
  vtkPolyData		  *point_data;
  vtkPlane                *clipPlanes[4];
  vtkClipPolyData         *clipFilter[4];

  bool                    clip_data;

#ifndef HEADLESS
  vtkChipsPointWidget  *point_widget;
#endif
};



#endif
