// 
//  Copyright (C) 2006-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <sstream>
#include "VtkPolygon.hh"
#include "vtkProperty2D.h"
#include "vtkTransform.h"
#include "vtkAxisActor2DExtra.hh"
#include "ChipsMatrix.hh"
#include "ChipsDataSystem.hh"
#include "ChipsFillPattern.hh"

#ifndef HEADLESS
#include "vtkChipsPointWidget.hh"
#include "vtkPolygonRepresentation.hh"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#endif 

#include "vtkDataArray.h"

#include "VtkUtils.hh"


extern ChipsFillPattern *fill_patterns;

VtkPolygon:: VtkPolygon(vtkRenderer* inRenderer,ChipsObject* par, 
		       ChipsCoordinate *inCoord, COORD_SYS coord_sys,
		       double* xcoords, double* ycoords, unsigned int cnt, 
		       string val):ChipsPolygon(par, inCoord, coord_sys,
						xcoords, ycoords, cnt, val)

{

  if ( ( !xcoords) || (!ycoords) || (!cnt))
    {
      renderer=0;
      polygonActor=0;
      return;
    }

  renderer = inRenderer;

  if ( renderer)
    {
      polygonActor = vtkPolygonDataActor2DExtra::New();
      

      renderer->AddActor2D(polygonActor);

      // Turn clipping off for plot norm
      polygonActor->SetClipData ((coord_system == DATA));

#ifndef HEADLESS
      // setup gui widget
      vtkPolygonRepresentation *polygon_rep = vtkPolygonRepresentation::New ();
      ChipsId cid;
      polygon_rep->SetPolygonActor (polygonActor);

      this->populate_chips_id (cid);
      polygon_widget = vtkChipsPointWidget::New ();
      polygon_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      polygon_widget->SetRepresentation (polygon_rep);
      polygon_widget->SetIdInfo (cid, this->get_object_type());
      polygon_widget->ResizableOff ();
      polygon_widget->RotateableOff ();
      polygon_widget->SetDefaultRenderer (renderer);
      polygon_widget->EnabledOn ();
      polygon_rep->Delete ();
#endif
      setup_polygondata();

      change_color();
      change_opacity();
      change_fillcolor();
//      change_rotation(); 
      change_thickness(); 
      change_linestyle(); 
      change_visible(); 
      change_fillstyle(); 

    };

};

VtkPolygon::VtkPolygon(vtkRenderer* inRenderer,ChipsObject* par, 
		      ChipsCoordinate *inCoord, COORD_SYS coord_sys,
		      int sides, double xcoord, double ycoord, double radius, 
		      string val):ChipsPolygon(par, inCoord, coord_sys,
					       sides, xcoord, ycoord, radius, 
					       val)

{
  
  renderer = inRenderer;

  if ( renderer)
    {
      polygonActor = vtkPolygonDataActor2DExtra::New();
      
      renderer->AddActor2D(polygonActor);
     
      // Turn clipping on for DATA
      polygonActor->SetClipData ((coord_system == DATA));


#ifndef HEADLESS
      // setup gui widget
      vtkPolygonRepresentation *polygon_rep = vtkPolygonRepresentation::New ();
      ChipsId cid;
      polygon_rep->SetPolygonActor (polygonActor);

      this->populate_chips_id (cid);
      polygon_widget = vtkChipsPointWidget::New ();
      polygon_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      polygon_widget->SetRepresentation (polygon_rep);
      polygon_widget->SetIdInfo (cid, this->get_object_type());
      polygon_widget->ResizableOff ();
      polygon_widget->RotateableOff ();
      polygon_widget->SetDefaultRenderer (renderer);
      polygon_widget->EnabledOn ();
      polygon_rep->Delete ();
#endif
      setup_polygondata();
      
      change_color();
      change_opacity();
      change_fillcolor();
//      change_rotation(); 
      change_thickness(); 
      change_linestyle(); 
      change_visible(); 
      change_fillstyle(); 
      
    };

};




VtkPolygon::VtkPolygon(vtkRenderer* inRenderer,ChipsObject* par, 
   ChipsCoordinate *inCoord, COORD_SYS coord_sys, ChipsStateStream& in):
   ChipsPolygon(par, inCoord, coord_sys, in)
{   
  renderer = inRenderer;

  if ( renderer)
    {
      polygonActor = vtkPolygonDataActor2DExtra::New();

      renderer->AddActor2D(polygonActor);
     
      // Turn clipping off for plot norm 
      polygonActor->SetClipData ((coord_system == DATA));

#ifndef HEADLESS
      // setup gui widget
      vtkPolygonRepresentation *polygon_rep = vtkPolygonRepresentation::New ();
      ChipsId cid;
      polygon_rep->SetPolygonActor (polygonActor);

      this->populate_chips_id (cid);
      polygon_widget = vtkChipsPointWidget::New ();
      polygon_widget->SetInteractor (renderer->GetRenderWindow ()->GetInteractor ());
      polygon_widget->SetRepresentation (polygon_rep);
      polygon_widget->SetIdInfo (cid, this->get_object_type());
      polygon_widget->ResizableOff ();
      polygon_widget->RotateableOff ();
      polygon_widget->SetDefaultRenderer (renderer);
      polygon_widget->EnabledOn ();
      polygon_rep->Delete ();
#endif

      setup_polygondata();

 
      change_color();
      change_opacity();
      change_fillcolor();
//      change_rotation(); 
      change_thickness(); 
      change_linestyle(); 
      change_visible(); 
      change_fillstyle(); 
      
    };
};

VtkPolygon::~VtkPolygon()
{
  if ( polygonActor)
    {
      renderer->RemoveActor2D(polygonActor);

      polygonActor->Delete();

    };

#ifndef HEADLESS
  polygon_widget->Delete ();
#endif
};

int VtkPolygon::change_depth ()
{
  if ( polygonActor)
    {
      polygonActor->SetLayerNumber (depth*10+1);
    };
  return ERR_STATUS_GOOD;
}

int VtkPolygon::change_thickness()
{
  if ( polygonActor)
    { 
      polygonActor->GetLineActor()->GetProperty()->SetLineWidth(thickness);
    };
  return ERR_STATUS_GOOD;
}; 

int VtkPolygon::change_linestyle()
{

   if ( polygonActor)
     {
       
       switch(linestyle)
	 {
	 case chips_noline:
	   polygonActor->SetLinesVisibility(0);
	   break;
	 case chips_dotlongdash: 
	   polygonActor->SetLinesVisibility(1);
	   polygonActor->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dotlongdash_pattern);
	   break;
	 case chips_dotshortdash: 
	   polygonActor->SetLinesVisibility(1);
	   polygonActor->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dotshortdash_pattern);
	   break;
	 case chips_longdash: 
	   polygonActor->SetLinesVisibility(1);
	   polygonActor->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_longdash_pattern);
	   break;
	 case chips_shortdash:
	   polygonActor->SetLinesVisibility(1);
	   polygonActor->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_shortdash_pattern);
	   break;
	 case chips_shortdashlongdash:
	   polygonActor->SetLinesVisibility(1);
	   polygonActor->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_shortdashlongdash_pattern);
	   break;
	 case chips_dot:
	   polygonActor->SetLinesVisibility(1);
	   polygonActor->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_dot_pattern);
	   break;
	 case chips_solid:
	 default:
	   polygonActor->SetLinesVisibility(1);
	   polygonActor->GetLineActor()->GetProperty()->SetLineStipplePattern(chips_solid_pattern);
	   
	 };
     };       
   return ERR_STATUS_GOOD;
}; 


int VtkPolygon::change_fillstyle()
{
   if ( polygonActor)
     {
       switch(fillstyle)
         {
         case chips_nofill:
           polygonActor->SetPolygonVisibility(0);
           polygonActor->GetPolygonActor()->GetProperty()->SetUsePolygonStipple(0);
         break; 
         case chips_solidfill:
           polygonActor->SetPolygonVisibility(1);
           polygonActor->GetPolygonActor()->GetProperty()->SetUsePolygonStipple(0);
	 break; 
         default:
           polygonActor->SetPolygonVisibility(1);
           polygonActor->GetPolygonActor()->GetProperty()->SetUsePolygonStipple(1);
           polygonActor->GetPolygonActor()->GetProperty()->SetPolygonStipplePattern(fillpattern);
	 break; 
         };
     };
   return ERR_STATUS_GOOD;
};


int VtkPolygon::change_color()
{ 
 if ( polygonActor)
     {
       polygonActor->GetLineActor()->GetProperty()->SetColor(color.red(),color.green(),color.blue());
     };
 
 return ERR_STATUS_GOOD;
}; 


int VtkPolygon::change_opacity()
{ 
  if ( polygonActor)
    {
      polygonActor->GetLineActor()->GetProperty()->SetOpacity(opacity);
      polygonActor->GetPolygonActor()->GetProperty()->SetOpacity(opacity);
    };
  
  return ERR_STATUS_GOOD;
}; 

int VtkPolygon::change_fillcolor()
{ 
  if ( polygonActor)
    {
      polygonActor->GetPolygonActor()->GetProperty()->SetColor(fillcolor.red(),fillcolor.green(),fillcolor.blue());
     };
 
 return ERR_STATUS_GOOD;
}; 

void VtkPolygon::set_clip_bounds ()
{
  ChipsVector lowBounds;
  ChipsVector upBounds;
  COORD_SYS tmpCoordSys;
  double bounds[4];
  string coord_id = "";

  if (!coord)
     return;

  lowBounds = coord->get_lower_bounds ();
  upBounds = coord->get_upper_bounds ();

  tmpCoordSys = coord->get_bounds_coord_sys ();

  if (tmpCoordSys == DATA)
  {
     ChipsDataSystem *ds_coord = dynamic_cast <ChipsDataSystem *>(coord); 
     coord_id = ds_coord->get_id_no_wcs ();
  }


  VtkUtils::CoordSys2ViewportNorm (coord, tmpCoordSys, coord_id, renderer, &lowBounds[0]);

  VtkUtils::CoordSys2ViewportNorm (coord, tmpCoordSys, coord_id, renderer, &upBounds[0]);


  bounds[0] = lowBounds[0];
  bounds[1] = lowBounds[1];
  bounds[2] = upBounds[0];
  bounds[3] = upBounds[1];

  this->polygonActor->SetClipBounds (bounds);
}

int VtkPolygon::calculate_points (vtkDataArray *points)
{
  ChipsVector tmpPnt;
  int rval = ERR_STATUS_BAD;

  if (num_pnts == 0)
     return rval;

  tmpPnt = pnts[0];

  for (int i=0;i <points->GetNumberOfTuples (); i++)
  {
    if ((rval = coord->convert_cs_to_cs (&pnts[i], &tmpPnt, 1, coord_system, data_system_id, FRAME_NORM, "")) != ERR_STATUS_GOOD)
      break;
    VtkUtils::FrameNorm2ViewportNorm (coord, renderer, &tmpPnt[0]);
    points->SetTuple3(i,tmpPnt[0],tmpPnt[1],0);

  };

  return rval;

}

void VtkPolygon::setup_polygondata()
{

  if (!this->polygonActor || !coord)
    return;

  vtkDoubleArray *points = vtkDoubleArray::New ();
  ChipsVector tmpPnt;
  int rval = ERR_STATUS_BAD;
  int tmpCount=num_pnts;
  vtkTransform *transform = polygonActor->GetTransform ();


  ChipsDataSystem *dsCoord = dynamic_cast<ChipsDataSystem *>(coord);

  if (dsCoord)
  {
    scale[0] = dsCoord->get_log_flag (X_AXIS);
    scale[1] = dsCoord->get_log_flag (Y_AXIS);
  }


  transform->Identity ();

  this->lower_bounds_norm = this->lower_bounds;
  this->upper_bounds_norm = this->upper_bounds;

  VtkUtils::CoordSys2ViewportNorm (coord, coord_system, data_system_id, renderer, &this->lower_bounds_norm[0]);
  VtkUtils::CoordSys2ViewportNorm (coord, coord_system, data_system_id, renderer, &this->upper_bounds_norm[0]);


  this->set_clip_bounds ();

  if ( num_pnts )
  {
    if ( dbl_equal (pnts[0][0],pnts[num_pnts-1][0]) && dbl_equal(pnts[0][1],pnts[num_pnts-1][1]))
      tmpCount--;

    points->SetNumberOfComponents(3);
    points->SetNumberOfTuples (tmpCount);

    rval = calculate_points (points);
  }
  
  
  if (rval == ERR_STATUS_GOOD)
  {
    this->polygonActor->GetPositionCoordinate ()->SetValue (this->lower_bounds_norm[0], this->lower_bounds_norm[1]);
    this->polygonActor->GetPosition2Coordinate ()->SetValue (this->upper_bounds_norm[0], this->upper_bounds_norm[1]);

#ifndef HEADLESS
    if (polygon_widget)
    {
      polygon_widget->SetEnabled (1);
      polygon_widget->UpdateRepresentation ();
    }
#endif

    this->polygonActor->Enable ();
    this->polygonActor->LoadData (points);

  }
  else
  {
    this->polygonActor->GetPositionCoordinate ()->SetValue (0, 0);
    this->polygonActor->GetPosition2Coordinate ()->SetValue (1, 1);
    this->polygonActor->Disable ();
#ifndef HEADLESS
    if (this->polygon_widget)
       polygon_widget->SetEnabled (0);
#endif
  }


  points->Delete();



};

int VtkPolygon::change_position()
{
  
  if (!polygonActor || !coord) 
    return ERR_STATUS_BAD;
   

  vtkTransform *transform = polygonActor->GetTransform ();

  ChipsVector lowBounds;
  ChipsVector upBounds;
  bool changed = false;
  double transPos1[3];
  double transPos2[3];

  if (!coord)
     return ERR_STATUS_BAD;

  lowBounds = this->lower_bounds;
  upBounds = this->upper_bounds;

  VtkUtils::CoordSys2ViewportNorm (coord, coord_system, data_system_id, renderer, &lowBounds[0]);
  VtkUtils::CoordSys2ViewportNorm (coord, coord_system, data_system_id, renderer, &upBounds[0]);

  changed = ((lowBounds != this->lower_bounds_norm) || (upBounds != this->upper_bounds_norm));

  transform->Identity();

  if (changed)
  {
    ChipsDataSystem *dsCoord = dynamic_cast<ChipsDataSystem *>(coord);
    bool xScale = false;
    bool yScale = false;

    if (dsCoord)
    {
      xScale = dsCoord->get_log_flag (X_AXIS);
      yScale = dsCoord->get_log_flag (Y_AXIS);
    }

    if ((xScale || yScale) || (xScale != scale[0]) || (yScale != scale[1]))
    {
      vtkDoubleArray *points = reinterpret_cast<vtkDoubleArray *>(this->polygonActor->GetData ());
      int rval = calculate_points (points);

      polygonActor->GetTransform ()->Identity ();
      this->lower_bounds_norm = lowBounds;
      this->upper_bounds_norm = upBounds;

      if (!rval)
      {
        for (int ii=0; ii<lowBounds.get_size (); ii++)
        {
           if (lowBounds[ii] > upBounds[ii])
           {
              double tmp = lowBounds[ii];
              lowBounds[ii] = upBounds[ii];
              upBounds[ii] = tmp;
           }
        }

        this->polygonActor->GetPositionCoordinate ()->SetValue (lowBounds[0], lowBounds[1]);
        this->polygonActor->GetPosition2Coordinate ()->SetValue (upBounds[0], upBounds[1]);
#ifndef HEADLESS
        if (this->polygon_widget)
            polygon_widget->SetEnabled (1);
#endif
        this->polygonActor->Enable ();
      }
      else
      {
         this->polygonActor->GetPositionCoordinate ()->SetValue (0, 0);
         this->polygonActor->GetPosition2Coordinate ()->SetValue (1, 1);
         this->polygonActor->Disable ();
#ifndef HEADLESS
         if (this->polygon_widget)
            polygon_widget->SetEnabled (0);
#endif
      }


      this->scale[0] = xScale;
      this->scale[1] = yScale;
    }
    else
    {
      ChipsVector scale(lowBounds);

      scale[0] = (upBounds[0] - lowBounds[0])/(this->upper_bounds_norm[0] - this->lower_bounds_norm[0]);
      scale[1] = (upBounds[1] - lowBounds[1])/(this->upper_bounds_norm[1] - this->lower_bounds_norm[1]);


      this->set_clip_bounds ();

      transform->Translate (lowBounds[0], lowBounds[1], 0.0);
      transform->Scale(scale[0],scale[1],1);
      transform->Translate(-this->lower_bounds_norm[0], -this->lower_bounds_norm[1], 0.0);
    
      // update the bounding box
      transform->TransformPoint (&this->lower_bounds_norm[0], transPos1);
      transform->TransformPoint (&this->upper_bounds_norm[0], transPos2);

      // order the points
      if (transPos2[0] < transPos1[0])
      {
         double tmp = transPos1[0];
         transPos1[0] = transPos2[0];
         transPos2[0] = tmp;
      }

      if (transPos2[1] < transPos1[1])
      {
         double tmp = transPos1[1];
         transPos1[1] = transPos2[1];
         transPos2[1] = tmp;
      }

      polygonActor->GetPositionCoordinate()->SetValue(transPos1[0], transPos1[1]);
      polygonActor->GetPosition2Coordinate()->SetValue(transPos2[0], transPos2[1]);
    }
  }

#ifndef HEADLESS
  if (polygon_widget)
    polygon_widget->UpdateRepresentation ();
#endif

  polygonActor->Modified ();

  return ERR_STATUS_GOOD;
    
    
};

int VtkPolygon::change_rotation()
{
  return ERR_STATUS_GOOD;
};


int VtkPolygon::change_visible ()
{
   int tmp_vis = (!get_parent()->is_visible ()) ? 0 : is_visible();
   if (polygonActor) 
     {
       polygonActor->SetVisibility(tmp_vis);
	   #ifndef HEADLESS
	   polygon_widget->GetRepresentation ()->SetVisibility (tmp_vis);
	   #endif
     };

   return ERR_STATUS_GOOD;

}


int VtkPolygon::update_data ()
{
   if (!coord)
     return ERR_STATUS_BAD;

   if (polygonActor) 
     {

       vtkDoubleArray *points = reinterpret_cast<vtkDoubleArray *>(this->polygonActor->GetData ());
       int rval = calculate_points (points);

       ChipsDataSystem *dsCoord = dynamic_cast<ChipsDataSystem *>(coord);

       if (dsCoord)
       {
         scale[0] = dsCoord->get_log_flag (X_AXIS);
         scale[1] = dsCoord->get_log_flag (Y_AXIS);
       }

       polygonActor->GetTransform ()->Identity ();
       if (!rval)
       {

         this->lower_bounds_norm = this->lower_bounds;
         this->upper_bounds_norm = this->upper_bounds;
         VtkUtils::CoordSys2ViewportNorm (coord, coord_system, data_system_id, renderer, &lower_bounds_norm[0]);
         VtkUtils::CoordSys2ViewportNorm (coord, coord_system, data_system_id, renderer, &upper_bounds_norm[0]);

         this->polygonActor->GetPositionCoordinate ()->SetValue (this->lower_bounds_norm[0], this->lower_bounds_norm[1]);
         this->polygonActor->GetPosition2Coordinate ()->SetValue (this->upper_bounds_norm[0], this->upper_bounds_norm[1]);

#ifndef HEADLESS
         if (polygon_widget)
         {
           polygon_widget->SetEnabled (1);
           polygon_widget->UpdateRepresentation ();
         }
#endif

         this->polygonActor->Enable ();
       }
       else
       {
         this->polygonActor->GetPositionCoordinate ()->SetValue (0, 0);
         this->polygonActor->GetPosition2Coordinate ()->SetValue (1, 1);
         this->polygonActor->Disable ();
#ifndef HEADLESS
         if (this->polygon_widget)
            polygon_widget->SetEnabled (0);
#endif
       }
       
       change_color();
       change_fillcolor();
//       change_rotation(); 
       change_thickness(); 
       change_linestyle(); 
       change_visible(); 
       change_fillstyle(); 
     };

   return ERR_STATUS_GOOD;

}

void VtkPolygon::select ()
{
#ifndef HEADLESS
  if (polygon_widget)
    polygon_widget->Select (true);
#endif
}

void VtkPolygon::unselect ()
{
#ifndef HEADLESS
  if (polygon_widget)
    polygon_widget->Select (false);
#endif
}

void VtkPolygon::disable_interaction ()
{
#ifndef HEADLESS
  vtkPolygonRepresentation *rep = reinterpret_cast<vtkPolygonRepresentation *> (this->polygon_widget->GetRepresentation ());
  rep->DisableBorder ();
  this->polygon_widget->EnabledOff ();
#endif
}

void VtkPolygon::enable_interaction ()
{
#ifndef HEADLESS
  if (this->is_interaction_blocked ())
	return;
  vtkPolygonRepresentation *rep = reinterpret_cast<vtkPolygonRepresentation *> (this->polygon_widget->GetRepresentation ());

  this->polygon_widget->EnabledOn ();
  rep->EnableBorder ();
#endif
}

int VtkPolygon::update_clipping ()
{
  this->polygonActor->SetClipData (coord_system == DATA);
  return ERR_STATUS_GOOD;
}

