#include "VtkUtils.hh"

#include "ChipsCoordinate.hh"
#include "ChipsMatrix.hh"

#include "vtkViewport.h"
#include <cmath>

void VtkUtils::FrameNorm2ViewportNorm (ChipsCoordinate *coord,
                              vtkViewport *viewport, double inOutPos[2])
{
   VtkUtils::CoordSys2ViewportNorm (coord, FRAME_NORM, "", viewport, inOutPos);
}

void VtkUtils::CoordSys2ViewportNorm (ChipsCoordinate *coord,
                              COORD_SYS coordSys, const string &coordId,
                              vtkViewport *viewport, double inOutPos[2])
{
   if (!coord || !viewport)
      return;

   VtkUtils::CoordSys2Viewport (coord,
                               coordSys, coordId,
                               viewport, inOutPos);

   viewport->ViewportToNormalizedViewport(inOutPos[0], inOutPos[1]);
}

void VtkUtils::CoordSys2Viewport (ChipsCoordinate *coord,
                              COORD_SYS coordSys, const string &coordId,
                              vtkViewport *viewport, double inOutPos[2])
{

   if (!coord || !viewport)
      return;

   ChipsVector pos (inOutPos[0], inOutPos[1]);
   double minx, miny;
   double maxx, maxy;

   coord->convert_cs_to_cs(&pos, &pos, 1, coordSys, coordId, FRAME_NORM, "");

   minx = 0;
   miny = 0;
   maxx = 1;
   maxy = 1;
   viewport->NormalizedViewportToViewport(minx, miny);
   viewport->NormalizedViewportToViewport(maxx, maxy);
   minx+=.5;
   miny+=.5;
   maxx+=.5;
   maxy+=.5;

   pos[0] = (maxx - minx)*pos[0] + minx;
   pos[1] = (maxy - miny)*pos[1] + miny;

/*
   pos[0] = floor(pos[0]) + .5;
   pos[1] = floor(pos[1]) + .5;

   cerr << "POS ";
   pos.print ();

   viewport->DisplayToNormalizedDisplay (pos[0], pos[1]);
   viewport->NormalizedDisplayToViewport(pos[0], pos[1]);
*/

   inOutPos[0] = pos[0];
   inOutPos[1] = pos[1];
}


