// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



#ifndef VTKWINDOW_HH
#define VTKWINDOW_HH

#include <string>

#include "vtkRenderWindow.h"
//#include "vtkXOpenGLOffScreenRenderWindow.h"
#include "vtkRenderer.h"
#include "vtkActor2D.h"

#include <GL/gl.h>
#include <GL/glu.h>


#include "ChipsObject.hh" 
#include "ChipsWindow.hh" 
#include "ChipsFrame.hh" 
#include "VtkFrame.hh"

class VtkEngine;
class vtkPolyData;
class vtkPolyDataMapper2D;

using namespace std;

class VtkWindow: public ChipsWindow
{
  public:

   VtkWindow(ChipsEngine* en, string id);
   VtkWindow(ChipsEngine* en, ChipsStateStream& in);
   VtkWindow(ChipsEngine* en, float width, float height, string format="inch", 
             string id=""); 

   virtual int do_print(string outfile, int fmt=0, int colorSpace=6, bool add_ext=true);
   virtual int redraw(); 
   virtual int render_layout(); 
   virtual int change_layout_color(); 
   virtual int change_layout_thickness(); 
   virtual int render_grids(); 
   virtual int change_grid_color(); 
   virtual int change_grid_thickness(); 
   virtual int change_grid_linestyle ();
   virtual int change_grid_visible ();
   virtual int change_order();
   virtual int change_color();
   virtual int change_size ();
   virtual int change_redraw ();
   virtual int re_order_frames ();
   virtual void remap_hide ();
   virtual void remap_show ();

#ifndef HEADLESS
   virtual ChipsGtkWindow *new_gui_window (string geom, bool focus);
#endif

   virtual ChipsFrame* new_frame(string fid="");
   virtual ChipsFrame* new_frame(ChipsStateStream& in);
   virtual ChipsFrame* new_frame(float xmin, float ymin, float xmax,
      float ymax, string fid="");

  protected:

  virtual ~VtkWindow();

  void initialize ();
  void copy_render_window (vtkRenderWindow *vrw1, vtkRenderWindow *vrw2);
  void move_render_window (vtkRenderWindow *vrw1, vtkRenderWindow *vrw2);
//  static void delete_pixbuf ( guchar *pixels, gpointer data) { delete[] pixels;};

  void draw_backdrop ();

  virtual void       set_window_size();
  virtual void       set_window_size(int inPixelWidth,int inPixelHeight);

   vtkRenderWindow *renWin;
   vtkRenderer *base_renderer;
   vtkRenderer *redraw_renderer;    // Used when redraw off
   vtkActor2D  *backdrop_actor;     // Used when redraw off

   vtkPolyData       *gridData;
   vtkActor2D        *gridActor;
   vtkPolyDataMapper2D *gridMapper;

   bool   rgba_comp;                // flag if the pixbuf has an alpha channel
   bool   remapping;        // flag to indicate if the window is being remapped

};

#endif 
