#include "vtkAddAnnotationRepresentation.hh"
#include "vtkRenderer.h"
#include "vtkWindow.h"
#include "vtkObjectFactory.h"

#include "vtkPoints.h"
#include "vtkCellArray.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkActor2D.h"
#include "vtkProperty2D.h"
#include "vtkMath.h"

vtkStandardNewMacro(vtkAddAnnotationRepresentation);

vtkProperty2D *vtkAddAnnotationRepresentation::GlobalAnnotationProperty = vtkProperty2D::New ();

//-------------------------------------------------------------------------
vtkAddAnnotationRepresentation::vtkAddAnnotationRepresentation()
{

  // Initial positioning information
  this->PositionCoordinate = vtkCoordinate::New();
  this->PositionCoordinate->SetCoordinateSystemToDisplay();
  this->PositionCoordinate->SetValue(0.05, 0.05);
  this->Position2Coordinate = vtkCoordinate::New();
  this->Position2Coordinate->SetCoordinateSystemToDisplay();
  this->Position2Coordinate->SetReferenceCoordinate(this->PositionCoordinate);

  vtkPoints *annotationPoints = vtkPoints::New();
  annotationPoints->SetDataTypeToDouble();
  annotationPoints->Allocate(100);
  for (int ii=0; ii<100; ii++)
     annotationPoints->InsertNextPoint(0.0, 0.0, 0.0);
  vtkCellArray *annotationLine = vtkCellArray::New();

  annotationLine->Allocate (2);
  annotationLine->InsertNextCell(2);
  for (int ii=0; ii<2; ii++)
    annotationLine->InsertCellPoint(ii);

  this->AnnotationPolyData = vtkPolyData::New ();
  this->AnnotationPolyData->SetPoints(annotationPoints);
  this->AnnotationPolyData->SetLines(annotationLine);

  annotationPoints->Delete ();
  annotationLine->Delete();

  this->AnnotationMapper = vtkPolyDataMapper2D::New();
  this->AnnotationMapper->SetInput(this->AnnotationPolyData);
  this->AnnotationActor = vtkActor2D::New();
  this->AnnotationActor->SetMapper(this->AnnotationMapper);

  this->AnnotationProperty = vtkProperty2D::New();
//  this->AnnotationActor->SetProperty(this->AnnotationProperty);
  this->AnnotationActor->SetProperty (GlobalAnnotationProperty);

  this->AnnotationProperty->SetColor (1, 0, 1);
  this->AnnotationProperty->SetLineStipplePattern (0x5555);
  this->GlobalAnnotationProperty->SetLineStipplePattern (0x5555);

  this->AnnotationActor->SetVisibility (0);

  this->UseGlobalProperty = true;

  this->DrawMode = vtkAddAnnotationRepresentation::Line;
  this->InitMode = false;

}

//----------------------------------------------------------------------------
vtkAddAnnotationRepresentation::~vtkAddAnnotationRepresentation()
{
  this->AnnotationPolyData->Delete ();
  this->AnnotationMapper->Delete ();
  this->AnnotationActor->Delete ();
  this->AnnotationProperty->Delete ();

  this->PositionCoordinate->Delete ();
  this->Position2Coordinate->Delete ();

}

//-------------------------------------------------------------------------
void vtkAddAnnotationRepresentation::StartWidgetInteraction(double eventPos[2])
{
  this->StartEventPosition[0] = eventPos[0];
  this->StartEventPosition[1] = eventPos[1];

  this->PositionCoordinate->SetValue(eventPos[0],eventPos[1]);
  this->Position2Coordinate->SetValue(1,1);

  // make sure this is the top of the render list
  this->Renderer->RemoveViewProp (this);
  this->Renderer->AddViewProp (this);

  this->AnnotationActor->SetVisibility (1);

  this->Modified();


}

//-------------------------------------------------------------------------
void vtkAddAnnotationRepresentation::WidgetInteraction(double eventPos[2])
{
/*
  // convert to normalized viewport coordinates
  this->Renderer->DisplayToNormalizedDisplay(XF,YF);
  this->Renderer->NormalizedDisplayToViewport(XF,YF);
  this->Renderer->ViewportToNormalizedViewport(XF,YF);
*/

  // there are four parameters that can be adjusted
  double *fpos1 = this->PositionCoordinate->GetValue();

  this->Position2Coordinate->SetValue(eventPos[0] - fpos1[0], eventPos[1] - fpos1[1]);

  this->Modified();
}

//-------------------------------------------------------------------------
void vtkAddAnnotationRepresentation::EndWidgetInteraction(double eventPos[2])
{
  this->AnnotationActor->SetVisibility (0);
  this->Modified();
}

void vtkAddAnnotationRepresentation::SetDrawMode (vtkAddAnnotationRepresentation::DRAWSTATE mode)
{
  if (mode != this->DrawMode)
    InitMode = true;
  this->DrawMode = mode;
}

void vtkAddAnnotationRepresentation::GetPoints (vtkstd::vector<double> &points)
{
  vtkIdType npts;
  vtkIdType *pts = NULL;
  double x[3];

  vtkCellArray *lines = this->AnnotationPolyData->GetLines ();
  lines->GetCell(0,npts,pts);

  for (int ii=0; ii<npts; ii++)
    {
    this->AnnotationPolyData->GetPoint (pts[ii], x);
    points.push_back (x[0]);
    points.push_back (x[1]);
    }
}

void vtkAddAnnotationRepresentation::DrawLine ()
{
    vtkPoints *annotationPoints = this->AnnotationPolyData->GetPoints ();

    int *fpos1 = this->PositionCoordinate->GetComputedViewportValue(this->Renderer);
    int *fpos2 = this->Position2Coordinate->GetComputedViewportValue(this->Renderer);

    annotationPoints->SetPoint(0, fpos1[0], fpos1[1], 0.0);
    annotationPoints->SetPoint(1, fpos2[0], fpos2[1], 0.0);

    if (this->InitMode)
      {
      vtkCellArray *annotationLine = vtkCellArray::New();

      annotationLine->Allocate (2);
      annotationLine->InsertNextCell(2);
      for (int ii=0; ii<2; ii++)
        annotationLine->InsertCellPoint(ii);

      this->AnnotationPolyData->SetLines(annotationLine);

      annotationLine->Delete();
      this->InitMode = false;

      }
}


void vtkAddAnnotationRepresentation::DrawBox ()
{
    vtkPoints *annotationPoints = this->AnnotationPolyData->GetPoints ();

    int *fpos1 = this->PositionCoordinate->GetComputedViewportValue(this->Renderer);
    int *fpos2 = this->Position2Coordinate->GetComputedViewportValue(this->Renderer);
    
    annotationPoints->SetPoint(0, fpos1[0], fpos1[1], 0.0);
    annotationPoints->SetPoint(1, fpos1[0], fpos2[1], 0.0);
    annotationPoints->SetPoint(2, fpos2[0], fpos2[1], 0.0);
    annotationPoints->SetPoint(3, fpos2[0], fpos1[1], 0.0);



    if (this->InitMode)
      {
      vtkCellArray *annotationLine = vtkCellArray::New();

      annotationLine->Allocate (5);
      annotationLine->InsertNextCell(5);
      for (int ii=0; ii<4; ii++)
        annotationLine->InsertCellPoint(ii);
      annotationLine->InsertCellPoint(0);

      this->AnnotationPolyData->SetLines(annotationLine);

      annotationLine->Delete();
      this->InitMode = false;

      }

}

void vtkAddAnnotationRepresentation::DrawEllipse ()
{
    vtkPoints *annotationPoints = this->AnnotationPolyData->GetPoints ();
  
    double center_x, center_y;
    double px, py;
    double radius1, radius2;
    double theta;

    int *fpos1 = this->PositionCoordinate->GetComputedViewportValue(this->Renderer);
    int *fpos2 = this->Position2Coordinate->GetComputedViewportValue(this->Renderer);


    center_x = (fpos1[0] + fpos2[0])/2.0;
    center_y = (fpos1[1] + fpos2[1])/2.0;

    radius1 = fabs (center_x - fpos1[0]);
    radius2 = fabs (center_y - fpos1[1]);

    for (int ii=0; ii<100; ii++)
      {
      theta = vtkMath::RadiansFromDegrees(ii*3.6);
      px = center_x + radius1 * cos(theta);
      py = center_y + radius2 * sin(theta);
      annotationPoints->SetPoint(ii, px, py, 0.0);
      }

    if (this->InitMode)
      {
      vtkCellArray *annotationLine = vtkCellArray::New();

      annotationLine->Allocate (101);
      annotationLine->InsertNextCell(101);
      for (int ii=0; ii<100; ii++)
        annotationLine->InsertCellPoint(ii);
      annotationLine->InsertCellPoint(0);

      this->AnnotationPolyData->SetLines(annotationLine);

      annotationLine->Delete();
      this->InitMode = false;

      }

}


//-------------------------------------------------------------------------
void vtkAddAnnotationRepresentation::BuildRepresentation()
{
  if ( this->GetMTime() > this->BuildTime ||
       (this->Renderer && this->Renderer->GetVTKWindow() &&
        this->Renderer->GetVTKWindow()->GetMTime() > this->BuildTime) )
    {

    switch (this->DrawMode)
      {
      case vtkAddAnnotationRepresentation::Line:
        this->DrawLine ();
        break;
      case vtkAddAnnotationRepresentation::Box:
        this->DrawBox ();
        break;
      case vtkAddAnnotationRepresentation::Ellipse:
        this->DrawEllipse ();
        break;
      }

    this->BuildTime.Modified();
    }
}

//-------------------------------------------------------------------------
void vtkAddAnnotationRepresentation::ReleaseGraphicsResources(vtkWindow *w)
{
  this->AnnotationActor->ReleaseGraphicsResources(w);
/*  this->AnnotationOutlineActor->ReleaseGraphicsResources(w);
*/
}

//-------------------------------------------------------------------------
int vtkAddAnnotationRepresentation::RenderOverlay(vtkViewport *w)
{
  int count = 0;
  if ( ! this->AnnotationActor->GetVisibility() )
    {
    return 0;
    }
  this->BuildRepresentation();
  count += this->AnnotationActor->RenderOverlay(w);
  return count;
}

//-------------------------------------------------------------------------
int vtkAddAnnotationRepresentation::RenderOpaqueGeometry(vtkViewport *w)
{
  int count = 0;

  if ( ! this->AnnotationActor->GetVisibility() )
    {
    return 0;
    }
  this->BuildRepresentation();
  count += this->AnnotationActor->RenderOpaqueGeometry(w);
  return count;
}

//-----------------------------------------------------------------------------
int vtkAddAnnotationRepresentation::RenderTranslucentPolygonalGeometry(vtkViewport *w)
{
  int count = 0;
  if ( ! this->AnnotationActor->GetVisibility() )
    {
    return 0;
    }
  this->BuildRepresentation();
  count += this->AnnotationActor->RenderTranslucentPolygonalGeometry(w);
  return count;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkAddAnnotationRepresentation::HasTranslucentPolygonalGeometry()
{
  int count = 0;
  if ( ! this->AnnotationActor->GetVisibility() )
    {
    return 0;
    }
  this->BuildRepresentation();
  count += this->AnnotationActor->HasTranslucentPolygonalGeometry();
  return count;
}

//-------------------------------------------------------------------------
void vtkAddAnnotationRepresentation::SetUseGlobalProperty (bool enable)
{
  this->UseGlobalProperty = enable;

  if (enable)
    this->AnnotationActor->SetProperty (GlobalAnnotationProperty);
  else
    this->AnnotationActor->SetProperty (this->AnnotationProperty);


}

//-------------------------------------------------------------------------
void vtkAddAnnotationRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}




