#ifndef __vtkAddAnnotationRepresentation_hh
#define __vtkAddAnnotationRepresentation_hh

#include "vtkWidgetRepresentation.h"
#include "vtkCoordinate.h" //Because of the viewport coordinate macro
#include <vtkstd/vector>

class vtkPolyData;
class vtkPolyDataMapper2D;
class vtkActor2D;
class vtkProperty2D;

class VTK_WIDGETS_EXPORT vtkAddAnnotationRepresentation : public vtkWidgetRepresentation
{
public:

  enum DRAWSTATE
  {
    Line=0,
    Ellipse,
    Box
  };

  // Description:
  // Instantiate this class.
  static vtkAddAnnotationRepresentation *New();

  // Description:
  // Define standard methods.
  vtkTypeMacro(vtkAddAnnotationRepresentation,vtkWidgetRepresentation);
  void PrintSelf(ostream& os, vtkIndent indent);


  // Description:
  // Specify opposite corners of the box defining the boundary of the
  // widget. By default, these coordinates are in the viewport
  // coordinate system, with Position the lower left of the outline, and
  // Position2 relative to Position. Note that using these methods are
  // affected by the ProportionalResize flag. That is, if the aspect ratio of
  // the representation is to be preserved (e.g., ProportionalResize is on),
  // then the rectangle (Position,Position2) is a bounding rectangle. Also,
  vtkViewportCoordinateMacro(Position);
  vtkViewportCoordinateMacro(Position2);

  // Description:
  // Set/Get whether the bounding box should use a specified ratio
  virtual void SetUseGlobalProperty (bool enable);
  vtkGetMacro (UseGlobalProperty, bool);

  // Description:
  // Subclasses should implement these methods. See the superclasses'
  // documentation for more information.
  virtual void BuildRepresentation();
  virtual void StartWidgetInteraction(double eventPos[2]);
  virtual void WidgetInteraction(double eventPos[2]);
  virtual void EndWidgetInteraction (double eventPos[2]);

  // Description:
  // These methods are necessary to make this representation behave as
  // a vtkProp.
  virtual void ReleaseGraphicsResources(vtkWindow*);
  virtual int RenderOverlay(vtkViewport*);
  virtual int RenderOpaqueGeometry(vtkViewport*);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport*);
  virtual int HasTranslucentPolygonalGeometry();

  static vtkProperty2D *GetGlobalAnnotationProperty () {return GlobalAnnotationProperty;};

  vtkGetMacro (DrawMode, vtkAddAnnotationRepresentation::DRAWSTATE);
  void SetDrawMode (vtkAddAnnotationRepresentation::DRAWSTATE mode);

  void GetPoints (vtkstd::vector<double> &points);

protected:
  vtkAddAnnotationRepresentation();
  ~vtkAddAnnotationRepresentation();

  void DrawEllipse ();
  void DrawLine ();
  void DrawBox ();

  // Layout (position of lower left and upper right corners of border)
  vtkCoordinate *PositionCoordinate;
  vtkCoordinate *Position2Coordinate;

  vtkPolyData                *AnnotationPolyData;
  vtkPolyDataMapper2D        *AnnotationMapper;
  vtkActor2D                 *AnnotationActor;
  vtkProperty2D              *AnnotationProperty;

  DRAWSTATE DrawMode;
  bool   UseGlobalProperty;

  static vtkProperty2D       *GlobalAnnotationProperty;

  bool   InitMode;

};

#endif

