// 
//  Copyright (C) 2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __vtkAddAnnotationWidget_h
#define __vtkAddAnnotationWidget_h

#include "vtkAbstractWidget.h"
#include "ChipsId.hh"

#include <vtkstd/vector>

class vtkAddAnnotationRepresentation;
class vtkAddAnnotationEventObserver;

class VTK_WIDGETS_EXPORT vtkAddAnnotationWidget : public vtkAbstractWidget
{
public:
  // Description:
  // Method to instantiate class.
  static vtkAddAnnotationWidget *New();

  // Description;
  // Standard methods for class.
  vtkTypeMacro(vtkAddAnnotationWidget,vtkAbstractWidget);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Specify an instance of vtkWidgetRepresentation used to represent this
  // widget in the scene. Note that the representation is a subclass of vtkProp
  // so it can be added to the renderer independent of the widget.
  void SetRepresentation(vtkAddAnnotationRepresentation *r)
    {this->Superclass::SetWidgetRepresentation(reinterpret_cast<vtkWidgetRepresentation*>(r));}

  // Description:
  // Create the default widget representation if one is not set. 
  virtual void CreateDefaultRepresentation();

  virtual void SetBlock (bool flag);
  inline void Block () {this->SetBlock (true);};
  inline void Unblock () {this->SetBlock (false);};

  friend class vtkAddAnnotationEventObserver;

protected:
  vtkAddAnnotationWidget();
  ~vtkAddAnnotationWidget();

  void EndInteraction ();


  //processes the registered events
  static void OnMouseMove(vtkAbstractWidget* w);
  static void OnLeftButtonRelease(vtkAbstractWidget* w);
  static void OnLeftButtonPress(vtkAbstractWidget* w);
  static void OnKeyPress(vtkAbstractWidget* w);

  virtual int SubOnMouseMove(vtkAbstractWidget* w) {return 0;};
  virtual int SubOnLeftButtonRelease(vtkAbstractWidget* w) {return 0;};
  virtual int SubOnLeftButtonPress(vtkAbstractWidget* w) {return 0;};
  virtual int SubOnKeyPress(vtkAbstractWidget* w) {return 0;};



  enum WIDGETSTATE
  {
    Blocked=0,
    Start,
    Continue,
    DrawEllipse,
    DrawBox,
    DrawText
  };

  int WidgetState;

  
  vtkstd::vector<double> AnnotationPoints;

  vtkAddAnnotationEventObserver *eventObserver;

private:
  vtkAddAnnotationWidget(const vtkAddAnnotationWidget&);  //Not implemented
  void operator=(const vtkAddAnnotationWidget&);  //Not implemented
};

#endif
