#ifndef __vtkBoundingBoxRepresentation_hh
#define __vtkBoundingBoxRepresentation_hh

#include "vtkWidgetRepresentation.h"
#include "vtkCoordinate.h" //Because of the viewport coordinate macro

class vtkTransform;
class vtkTransformPolyDataFilter;
class vtkPolyDataMapper2D;
class vtkActor2D;
class vtkProperty2D;

class VTK_WIDGETS_EXPORT vtkBoundingBoxRepresentation : public vtkWidgetRepresentation
{
public:
  // Description:
  // Instantiate this class.
  static vtkBoundingBoxRepresentation *New();

  // Description:
  // Define standard methods.
  vtkTypeMacro(vtkBoundingBoxRepresentation,vtkWidgetRepresentation);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Set/Get whether the bounding box should use a specified ratio
  vtkSetMacro (UseRatio, bool);
  vtkGetMacro (UseRatio, bool);
  vtkBooleanMacro (UseRatio, bool);

  // Description:
  // Set/Get whether the bounding box should use a specified ratio
  virtual void SetUseGlobalProperty (bool enable);
  vtkGetMacro (UseGlobalProperty, bool);


  // Description:
  // Set/Get bounding box x/y ratio. Must turn on the UseRatio for
  // values to be effective.
  vtkSetVectorMacro (Ratio, float, 2);
  vtkGetVectorMacro (Ratio, float, 2);

  // Description:
  // Set/Get minimum x/y value bounding box can be stretched. Values
  // should be in display coordinates. Values less than 0 will 
  // translated to 0.
  vtkSetVectorMacro (MinBoundPos, double, 2);
  vtkGetVectorMacro (MinBoundPos, double, 2);

  // Description:
  // Set/Get maximum x/y value bounding box can be stretched. Values
  // should be in display coordinates. Values less than 0 will 
  // translated to max display.
  vtkSetVectorMacro (MaxBoundPos, double, 2);
  vtkGetVectorMacro (MaxBoundPos, double, 2);

  // Get the range of the last bounding box;
  void GetBoxRange (double range[]);



  // Description:
  // Specify opposite corners of the box defining the boundary of the
  // widget. By default, these coordinates are in the viewport
  // coordinate system, with Position the lower left of the outline, and
  // Position2 relative to Position. Note that using these methods are
  // affected by the ProportionalResize flag. That is, if the aspect ratio of
  // the representation is to be preserved (e.g., ProportionalResize is on),
  // then the rectangle (Position,Position2) is a bounding rectangle. Also,
  vtkViewportCoordinateMacro(Position);
  vtkViewportCoordinateMacro(Position2);

  // Description:
  // Subclasses should implement these methods. See the superclasses'
  // documentation for more information.
  virtual void BuildRepresentation();
  virtual void StartWidgetInteraction(double eventPos[2]);
  virtual void WidgetInteraction(double eventPos[2]);
  virtual void EndWidgetInteraction (double eventPos[2]);

  // Description:
  // These methods are necessary to make this representation behave as
  // a vtkProp.
  virtual void ReleaseGraphicsResources(vtkWindow*);
  virtual int RenderOverlay(vtkViewport*);
  virtual int RenderOpaqueGeometry(vtkViewport*);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport*);
  virtual int HasTranslucentPolygonalGeometry();

  static vtkProperty2D *GetGlobalBorderProperty () {return GlobalBorderProperty;};

protected:
  vtkBoundingBoxRepresentation();
  ~vtkBoundingBoxRepresentation();

  // Layout (position of lower left and upper right corners of border)
  vtkCoordinate *PositionCoordinate;
  vtkCoordinate *Position2Coordinate;

  vtkTransform               *BorderTransform;
  vtkTransformPolyDataFilter *BorderTransformFilter;
  vtkPolyDataMapper2D        *BorderMapper;
  vtkActor2D                 *BorderActor;
  vtkProperty2D              *BorderProperty;

  bool DragOrigin[2];
  float Ratio[2];
  bool UseRatio;

  double MinBoundPos[2];
  double MaxBoundPos[2];

  double Range[4];

  bool   UseGlobalProperty;

  static vtkProperty2D       *GlobalBorderProperty;


};

#endif

