/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtkChipsCommand.hh

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkChipsCommand - superclass for callback/observer methods
// .SECTION Description
// This file is an extension of vtkCommand. It's primary purpose is to
// add a double click event.
// .SECTION See Also
// vtkObject vtkCallbackCommand vtkOldStyleCallbackCommand
// vtkInteractorObserver vtk3DWidget

#ifndef __vtkChipsCommand_h
#define __vtkChipsCommand_h

#include "vtkCommand.h"

class vtkObject;

// The superclass that all commands should be subclasses of
class VTK_COMMON_EXPORT vtkChipsCommand : public vtkCommand
{
public:
  // Description:
  // Decrease the reference count (release by another object). This has
  // the same effect as invoking Delete() (i.e., it reduces the reference
  // count by 1).
  void UnRegister();
  virtual void UnRegister(vtkObjectBase *) 
    { this->UnRegister(); }
  
//BTX
  // Description:
  // All the currently defined events are listed here.  Developers can
  // use -- vtkChipsCommand::UserEvent + int to specify their own event
  // ids. If this list is adjusted, be sure to adjust
  // vtkChipsCommandEventStrings in vtkChipsCommand.cxx to match.
  enum ChipsEventIds {
    LeftButtonDoubleClickEvent=vtkCommand::UserEvent,
    UserEvent = 2000
  };
//ETX

protected:
//BTX
  vtkChipsCommand(const vtkChipsCommand& c) : vtkCommand(c) {}
  void operator=(const vtkChipsCommand&) {}
//ETX
};

#endif /* __vtkChipsCommand_h */
 
