// 
//  Copyright (C) 2004-2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "vtkChipsPickableRepresentation.hh"
#include "vtkRenderedAreaPicker.h"
#include "vtkObjectFactory.h"
#include "vtkRenderer.h"
#include "vtkAssemblyPath.h"
#include "vtkProp3DCollection.h"



vtkStandardNewMacro(vtkChipsPickableRepresentation);


//-------------------------------------------------------------------------
vtkChipsPickableRepresentation::vtkChipsPickableRepresentation()
{
  this->PickProp = NULL;
  this->PropPicked = false;
  this->PickPath = NULL;
}

//-------------------------------------------------------------------------
vtkChipsPickableRepresentation::~vtkChipsPickableRepresentation()
{
  if (this->PickProp)
    this->PickProp->Delete ();
}

//------------------------------------------------------------------------
void vtkChipsPickableRepresentation::SetPickPath (vtkAssemblyPath* path)
{
  this->PickPath = path;
  this->PropPicked = true;
}


//-------------------------------------------------------------------------
int vtkChipsPickableRepresentation::ComputeInteractionState(int X, int Y, int vtkNotUsed(modify))
{
  this->ComputeBorderInteractionState (X,Y);
  this->OnObject = false;

  if (this->InteractionState != vtkChipsRepresentation::Outside)
    {

    if (this->PropPicked && this->PickPath)
      {
      if (this->PickPath->GetFirstNode()->GetViewProp () == this->PickProp)
        {
        this->OnObject = true;
        if (this->InteractionState == vtkChipsRepresentation::Inside)
           this->InteractionState = vtkChipsRepresentation::On;
        }
      this->PropPicked = false;
      }
    }
  return this->InteractionState; 
}

//-------------------------------------------------------------------------
void vtkChipsPickableRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
  
  os << indent << "Prop Picked: " << (this->PropPicked ? "On\n" : "Off\n");
  
  if (this->PickPath)
     this->PickPath->PrintSelf (os, indent);
  else
     os << indent << "Pick Path: NULL\n";
}
