// 
//  Copyright (C) 2004-2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "vtkChipsPlotWidget.hh"
#include "vtkChipsWidgetEvent.hh"
#include "vtkCommand.h"
#include "vtkWidgetEventTranslator.h"
#include "vtkObjectFactory.h"
#include "vtkAbstractPropPicker.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkGtkRenderWindowInteractor.hh"

#include "vtkKeySym.hh"

#include "vtkChipsRepresentation.hh"
#include "ChipsEventManager.hh"



vtkStandardNewMacro(vtkChipsPlotWidget);

extern ChipsEventManager *event_manager;


//-------------------------------------------------------------------------
vtkChipsPlotWidget::vtkChipsPlotWidget()
{
  vtkWidgetEventTranslator *translator = this->GetEventTranslator ();
  translator->SetTranslation (vtkCommand::EndPickEvent, vtkChipsWidgetEvent::Move);
  translator->RemoveTranslation (vtkCommand::MouseMoveEvent);
}

//-------------------------------------------------------------------------
vtkChipsPlotWidget::~vtkChipsPlotWidget()
{
}

//-------------------------------------------------------------------------
void vtkChipsPlotWidget::SetCursor(int cState)
{
  vtkGtkRenderWindowInteractor *guiInteractor = vtkGtkRenderWindowInteractor::SafeDownCast (this->Interactor);
  vtkChipsRepresentation *chipsRep = reinterpret_cast<vtkChipsRepresentation*>(this->WidgetRep);

  switch (cState)
    {
    case vtkChipsRepresentation::On:
      if (chipsRep->GetMoving () && !this->edgeSelect)
        {
        if (guiInteractor)
          guiInteractor->SetCurrentCursor(VTK_CURSOR_DRAG);
        else
          this->RequestCursorShape(VTK_CURSOR_DEFAULT);
        }
      else
        {
        if ( guiInteractor )
          guiInteractor->SetCurrentCursor(VTK_CURSOR_DEFAULT);
        else
          this->RequestCursorShape(VTK_CURSOR_DEFAULT);
        }
      break;
    default:
      this->Superclass::SetCursor (cState);
    }
}



//-------------------------------------------------------------------------
int vtkChipsPlotWidget::SubclassSelectAction()
{
  vtkChipsRepresentation *rep = reinterpret_cast<vtkChipsRepresentation*>(this->WidgetRep);

  if (( this->WidgetState != vtkChipsWidget::Selected ) &&
      ( !rep->IsInteractionStateOnObject () ))
    return 1;

  // Picked something inside the widget
  int X = this->Interactor->GetEventPosition()[0];
  int Y = this->Interactor->GetEventPosition()[1];

  double eventPos[2];
  eventPos[0] = static_cast<double>(X);
  eventPos[1] = static_cast<double>(Y);

  edgeSelect = false;

  if ( this->WidgetRep->GetInteractionState() == vtkChipsRepresentation::Outside ) 
    {
    this->WidgetState = vtkChipsWidget::Start;
    ChipsEventObjectInfo select_info (*this->objectInfo);
    event_manager->invoke_event (eventPos, ChipsEventManager::UnselectEvent, &select_info);
    }
  else
    {

    bool selected = false;

    this->WidgetRep->StartWidgetInteraction(eventPos);

    if (this->Selectable)
      {
      if ( this->WidgetRep->GetInteractionState() == vtkChipsRepresentation::On )
        selected = event_manager->request_subselect (*(this->objectInfo));
      else
        selected = event_manager->request_select (*(this->objectInfo));
      }

    if (selected &&
        (event_manager->get_state () == ChipsEventManager::SelectState))
      {


      if(this->Resizable &&
         ((rep->GetInteractionState() == vtkChipsRepresentation::RightHandle) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::LeftHandle) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::TopHandle) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::BottomHandle) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::TopRightHandle) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::TopLeftHandle) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::BottomRightHandle) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::BottomLeftHandle)))
        {
        rep->ResizingOn ();
        this->edgeSelect = true;
        }

      else if (this->Moveable &&
         ((rep->GetInteractionState() == vtkChipsRepresentation::RightEdge) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::LeftEdge) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::TopEdge) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::BottomEdge) ||
          (rep->GetInteractionState() == vtkChipsRepresentation::On)))

        {
        rep->MovingOn ();
        if (rep->GetInteractionState() != vtkChipsRepresentation::On)
          this->edgeSelect = true;
        }
      else if (this->Rotateable &&
         (rep->GetInteractionState() == vtkChipsRepresentation::RotateHandle))
        {
        rep->RotatingOn ();
        this->edgeSelect = true;
        }




      this->WidgetState = vtkChipsWidget::Modify;
      }
    }
  this->StartInteraction();
  this->InvokeEvent(vtkCommand::StartInteractionEvent,NULL);
  this->Render ();

  return 1;
}



//-------------------------------------------------------------------------
int vtkChipsPlotWidget::SubclassMoveAction()
{
  vtkChipsRepresentation *chipsRep = reinterpret_cast<vtkChipsRepresentation*>(this->WidgetRep);

  if ( this->WidgetState == vtkChipsWidget::Selected )
     this->SetCursor (this->WidgetRep->GetInteractionState());

  // compute some info we need for all cases
  int X = this->Interactor->GetEventPosition()[0];
  int Y = this->Interactor->GetEventPosition()[1];

  // Update the rep state
  if ( this->WidgetState != vtkChipsWidget::Modify)
    {
    int modify = (this->Interactor->GetControlKey ()) ? vtkKeySym::RightControl : 0;
    chipsRep->ComputeInteractionState(X, Y, modify);
    }
  else
    {

    double newEventPosition[2];
    newEventPosition[0] = static_cast<double>(X);
    newEventPosition[1] = static_cast<double>(Y);

    // handle resizing
    if(chipsRep->GetResizing ())
      {
      // Okay, adjust the representation (the widget is currently selected)

      static double lastx1 = -1.0, lasty1 = -1.0, lastx2 = -1.0, lasty2 = -1.0; 

      chipsRep->WidgetInteraction(newEventPosition);

      vtkCoordinate *pos1 = chipsRep->GetPositionCoordinate ();
      vtkCoordinate *pos2 = chipsRep->GetPosition2Coordinate ();
      double *fpos1 = pos1->GetComputedDoubleDisplayValue (this->CurrentRenderer);
      double *fpos2 = pos2->GetComputedDoubleDisplayValue (this->CurrentRenderer);
      double dtmp; 
      int swap, pad, tmp_w = -1, tmp_h = -1;
      if (this->Interactor)  this->Interactor->GetSize(tmp_w, tmp_h);

      // handle- plot repositions off frame viewport, less than 20 pixels
      // wide, or when bbox LL cross UR 
      if ((swap = (fpos1[0] > fpos2[0])) == 1)  // check x axis
         pad = ((fpos1[0] - fpos2[0] < 20) && (lastx1 != -1));
      else
         pad = ((fpos2[0] - fpos1[0] < 20) && (lastx1 != -1));
      if (pad)
      {
         if (fpos1[0] == lastx1)
            fpos2[0] = fpos1[0] + 20;   // LL hasn't changed
         else
            fpos1[0] = fpos2[0] - 20;   // UR hasn't changed
      }
      else if (swap)
      {
         dtmp = fpos1[0];
         fpos1[0] = fpos2[0];
         fpos2[0] = dtmp;
      }
      if ((swap = (fpos1[1] > fpos2[1])) == 1)  // check y axis
         pad = ((fpos1[1] - fpos2[1] < 20) && (lasty1 != -1));
      else 
         pad = ((fpos2[1] - fpos1[1] < 20) && (lasty1 != -1));
      if (pad)
      {
         if (fpos1[1] == lasty1)
            fpos2[1] = fpos1[1] + 20;   // LL hasn't changed
         else
            fpos1[1] = fpos2[1] - 20;   // UR hasn't changed
      } 
      else if (swap)
      {
         dtmp = fpos1[1];
         fpos1[1] = fpos2[1];
         fpos2[1] = dtmp;
      }

      // handle outside of frame viewport
      if ((fpos1[0] < 0))      fpos1[0] = 0;
      if ((fpos1[1] < 0))      fpos1[1] = 0;
      if (tmp_w > -1 && (fpos2[0] > tmp_w))  fpos2[0] = tmp_w;
      if (tmp_h > -1 && (fpos2[1] > tmp_h))  fpos2[1] = tmp_h;
      
      double boundingBox[4] = {fpos1[0], fpos1[1], fpos2[0], fpos2[1]};

      // store locations to compare during next reposition
      lastx1 = fpos1[0]; 
      lasty1 = fpos1[1]; 
      lastx2 = fpos2[0]; 
      lasty2 = fpos2[1]; 

      event_manager->invoke_event (newEventPosition, ChipsEventManager::ResizeEvent, &boundingBox[0]);

      }
    else if (chipsRep->GetMoving ())
      {

      // Okay, adjust the representation (the widget is currently selected)
      double startPosition[2];
      chipsRep->GetCurrentEventPosition (startPosition);
      chipsRep->WidgetInteraction(newEventPosition);

      if (this->edgeSelect)
        event_manager->invoke_event (newEventPosition, ChipsEventManager::MoveEvent, startPosition);
      else 
        event_manager->invoke_event (newEventPosition, ChipsEventManager::PanEvent, startPosition);

      }

    else if (chipsRep->GetRotating ())
      {

      // Okay, adjust the representation (the widget is currently selected)
      double startPosition[2];
      float angle;
      chipsRep->GetCurrentEventPosition (startPosition);
      chipsRep->WidgetInteraction(newEventPosition);
      angle = chipsRep->GetAngle ();
      event_manager->invoke_event (newEventPosition, ChipsEventManager::RotateEvent, &angle);

      }



    // start a drag
    this->InvokeEvent(vtkCommand::InteractionEvent,NULL);
    this->Render();
    }

    return 1;
}


//-------------------------------------------------------------------------
void vtkChipsPlotWidget::FinalizeSelect ()
{
  vtkChipsRepresentation *chipsRep = reinterpret_cast<vtkChipsRepresentation*>(this->WidgetRep);
  if (!event_manager->is_selected (*this->objectInfo) &&
      !event_manager->is_subselected (*this->objectInfo))
    {
    this->WidgetState = vtkChipsWidget::Start;
    chipsRep->MovingOff ();
    chipsRep->ResizingOff ();
    }
  else
    {
    if ((event_manager->get_state () == ChipsEventManager::SelectState) &&
        (event_manager->is_subselected (*this->objectInfo)))
        
      {
      // verify panning is invalid turn moving off
      if (this->Moveable &&
          (chipsRep->GetInteractionState() == vtkChipsRepresentation::On))
        {
        if (!event_manager->is_valid_state (ChipsEventManager::PanState))
          chipsRep->MovingOff ();
        else
          {
          chipsRep->MovingOn ();
          this->WidgetState = vtkChipsWidget::Modify;
          }
        }
      this->SetCursor(this->WidgetRep->GetInteractionState());
      }
    }
}

//-------------------------------------------------------------------------
int vtkChipsPlotWidget::SubclassEndSelectAction()
{
  vtkChipsRepresentation *chipsRep = reinterpret_cast<vtkChipsRepresentation*>(this->WidgetRep);

  if ( this->WidgetState != vtkChipsWidget::Modify )
    return 1;


  if (chipsRep->GetResizing ())
    chipsRep->ResizingOff ();
  if (chipsRep->GetMoving ())
    chipsRep->MovingOff ();
  if (chipsRep->GetRotating ())
    chipsRep->RotatingOff ();

  double newEventPosition[2] = {this->Interactor->GetEventPosition()[0],
                                  this->Interactor->GetEventPosition()[1]};
  // Return state to not modifying
  if (this->edgeSelect)
    this->WidgetState = vtkChipsWidget::Selected;
  else
    {
    this->WidgetState = vtkChipsWidget::Start;
    this->SetCursor(this->WidgetRep->GetInteractionState());
    }

  chipsRep->EndWidgetInteraction (newEventPosition);


  // stop adjusting
  this->EndInteraction();
  this->InvokeEvent(vtkCommand::EndInteractionEvent,NULL);

  return 1;
}


    

//-------------------------------------------------------------------------
void vtkChipsPlotWidget::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}

