#include "vtkChipsPointRepresentation.hh"

#include "vtkRenderedAreaPicker.h"
#include "vtkObjectFactory.h"
#include "vtkRenderer.h"
#include "vtkTransform.h"
#include "vtkAssemblyPath.h"
#include "vtkProp3DCollection.h"
#include "vtkGlyphSource2D.h"
#include "vtkMath.h"
#include "vtkActor2D.h"

#include <float.h>
#include <cmath>


vtkStandardNewMacro(vtkChipsPointRepresentation);

//-------------------------------------------------------------------------
vtkChipsPointRepresentation::vtkChipsPointRepresentation()
{
  this->GlyphSource = NULL;
  this->AngleOffset = 0;
}

//-------------------------------------------------------------------------
vtkChipsPointRepresentation::~vtkChipsPointRepresentation()
{
  if (this->GlyphSource)
    this->GlyphSource->Delete ();
}


//-------------------------------------------------------------------------
void vtkChipsPointRepresentation::NegotiateLayout()
{
  if (!this->GlyphSource)
    return;

  double x[2] = {0,0};
  double y[2] = {0,0};

  vtkActor2D *pointActor = reinterpret_cast<vtkActor2D *>(this->PickProp);

  if (!pointActor)
    return;

  double *pos = pointActor->GetPositionCoordinate()->GetComputedDoubleViewportValue(this->Renderer);

  int pointScale = this->GlyphSource->GetScale ();

  x[0] = pos[0] - pointScale;
  y[0] = pos[1] - pointScale;
  x[1] = pos[0] + pointScale;
  y[1] = pos[1] + pointScale;

  this->PositionCoordinate->SetCoordinateSystemToViewport();
  this->PositionCoordinate->SetValue(x[0], y[0]);
  this->Position2Coordinate->SetCoordinateSystemToViewport();
  this->Position2Coordinate->SetValue(x[1]-x[0], y[1]-y[0]);

}

//-------------------------------------------------------------------------
void vtkChipsPointRepresentation::WidgetInteraction(double eventPos[2])
{
  double dx = eventPos[0] - this->CurrentEventPosition[0];
  double dy = eventPos[1] - this->CurrentEventPosition[1];

  if (this->Resizing)
    {
    int scale = this->GlyphSource->GetScale ();

    switch (this->InteractionState)
      {
      case vtkChipsRepresentation::RightHandle:
        scale += dx;
      break;
      case vtkChipsRepresentation::LeftHandle:
        scale -= dx;
      break;
      case vtkChipsRepresentation::TopHandle:
        scale += dy;
      break;
      case vtkChipsRepresentation::BottomHandle:
        scale -= dy;
      break;
      case vtkChipsRepresentation::TopRightHandle:
        scale += (dx>dy) ? dx : dy;
      break;
      case vtkChipsRepresentation::TopLeftHandle:
        scale += (dx>dy) ? -dx : dy;
      break;
      case vtkChipsRepresentation::BottomRightHandle:
        scale += (dx>dy) ? dx : -dy;
      break;
      case vtkChipsRepresentation::BottomLeftHandle:
        scale -= (dx>dy) ? dx : dy;
      break;
      default:
      break;
      }
    this->GlyphSource->SetScale (scale);
    this->Update ();

    }
  else if (this->Rotating)
    {
    float angle = 0;
    float startAngle = 0;
    double anchor[2];

    double *fpos1 = this->PositionCoordinate->GetComputedDoubleDisplayValue (this->Renderer);
    double *fpos2 = this->Position2Coordinate->GetComputedDoubleDisplayValue (this->Renderer);


    anchor[0] =  (fpos1[0] + fpos2[0])/2.;
    anchor[1] =  (fpos1[1] + fpos2[1])/2.;

    dx = this->CurrentEventPosition[0] - anchor[0];
    dy = this->CurrentEventPosition[1] - anchor[1];

    if ((dx != 0) && (dy != 0))
      startAngle = atan2 (dy,dx);

    dx = eventPos[0] - anchor[0];
    dy = eventPos[1] - anchor[1];

    if ((dx != 0) && (dy != 0))
      angle = atan2 (dy,dx);

    this->Angle += vtkMath::DegreesFromRadians (angle - startAngle);
    this->GlyphSource->SetRotationAngle (this->Angle);

    }

  this->CurrentEventPosition[0] = eventPos[0];
  this->CurrentEventPosition[1] = eventPos[1];


}

void vtkChipsPointRepresentation::SetRotating (bool rotate)
{
  if (rotate == this->Rotating)
    return;

  this->Rotating = rotate;

  if (!this->GlyphSource)
    return;

  if (this->Rotating)
    this->Angle = this->GlyphSource->GetRotationAngle();
  else
    this->Angle -= this->AngleOffset;
}



void vtkChipsPointRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}


