// 
//  Copyright (C) 2004-2009,2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "vtkChipsRepresentation.hh"
#include "vtkRenderer.h"
#include "vtkRenderWindow.h"
#include "vtkPoints.h"
#include "vtkCellArray.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkActor2D.h"
#include "vtkProperty2D.h"
#include "vtkTransform.h"
#include "vtkTransformPolyDataFilter.h"
#include "vtkWindow.h"
#include "vtkObjectFactory.h"
#include "vtkGlyphSource2D.h"
#include "vtkGlyph2D.h"
#include "vtkKeySym.hh"
#include "vtkMath.h"


vtkStandardNewMacro(vtkChipsRepresentation);

vtkProperty2D *vtkChipsRepresentation::GlobalBorderProperty = vtkProperty2D::New ();

//-------------------------------------------------------------------------
vtkChipsRepresentation::vtkChipsRepresentation()
{
  this->InteractionState = vtkChipsRepresentation::Outside;

//  this->ShowBorder = false;
  this->BorderEnabled = true;
  this->RotationEnabled = true;
  this->ProportionalResize = 0;

  // Initial positioning information
  this->Negotiated = 0;
  this->PositionCoordinate = vtkCoordinate::New();
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(0.0, 0.0);
  this->Position2Coordinate = vtkCoordinate::New();
  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(1.0, 1.0); //may be updated by the subclass
  this->Position2Coordinate->SetReferenceCoordinate(this->PositionCoordinate);

  // Create the geometry in canonical coordinates
  vtkPoints *BorderPoints = vtkPoints::New();
  BorderPoints->SetDataTypeToDouble();
  BorderPoints->SetNumberOfPoints(8);
  BorderPoints->SetPoint(0, 0.0, 0.0, 0.0); //may be updated by the subclass
  BorderPoints->SetPoint(1, 0.5, 0.0, 0.0);
  BorderPoints->SetPoint(2, 1.0, 0.0, 0.0);
  BorderPoints->SetPoint(3, 1.0, 0.5, 0.0);
  BorderPoints->SetPoint(4, 1.0, 1.0, 0.0);
  BorderPoints->SetPoint(5, 0.5, 1.0, 0.0);
  BorderPoints->SetPoint(6, 0.0, 1.0, 0.0);
  BorderPoints->SetPoint(7, 0.0, 0.5, 0.0);

  this->BorderPolyData = vtkPolyData::New();
  this->BorderPolyData->SetPoints(BorderPoints);

  BorderPoints->Delete ();

  this->BorderTransform = vtkTransform::New();
  this->BorderTransformFilter = vtkTransformPolyDataFilter::New();
  this->BorderTransformFilter->SetTransform(this->BorderTransform);
  this->BorderTransformFilter->SetInput(this->BorderPolyData);

  this->BorderGlyphSource = vtkGlyphSource2D::New();
  this->BorderGlyphSource->SetColor(1,1,0);
  this->BorderGlyphSource->SetGlyphType(6);
  this->BorderGlyphSource->SetScale(5);
  this->BorderGlyphSource->SetRotationAngle(0);
  this->BorderGlyphSource->SetFilled(1);

  this->BorderGlyphs = vtkGlyph2D::New();
  this->BorderGlyphs->SetInputConnection(this->BorderTransformFilter->GetOutputPort ());
  this->BorderGlyphs->SetSource(this->BorderGlyphSource->GetOutput());


  this->BorderMapper = vtkPolyDataMapper2D::New();
  this->BorderMapper->SetInputConnection(this->BorderGlyphs->GetOutputPort());
  this->BorderActor = vtkActor2D::New();
  this->BorderActor->SetMapper(this->BorderMapper);
  
  this->BorderProperty = vtkProperty2D::New();
//  this->BorderActor->SetProperty(this->BorderProperty);
  this->BorderActor->SetProperty (GlobalBorderProperty);

  this->BorderProperty->SetColor (0, 1, 1);


  vtkPoints *RotationPoint = vtkPoints::New();
  RotationPoint->SetDataTypeToDouble();
  RotationPoint->SetNumberOfPoints(1);
  RotationPoint->SetPoint(0, 0.5, 1.0, 0.0); //may be updated by the subclass

  this->RotationPolyData = vtkPolyData::New();
  this->RotationPolyData->SetPoints(RotationPoint);

  RotationPoint->Delete ();

  this->RotationTransform = vtkTransform::New();
  this->RotationTransformFilter = vtkTransformPolyDataFilter::New();
  this->RotationTransformFilter->SetTransform(this->RotationTransform);

  this->RotationTransformFilter->SetInput(this->RotationPolyData);

  this->RotationGlyphSource = vtkGlyphSource2D::New();
  this->RotationGlyphSource->SetColor(0,1,0);
  this->RotationGlyphSource->SetGlyphType(7);
  this->RotationGlyphSource->SetScale(4);
  this->RotationGlyphSource->SetFilled(1);

  this->RotationGlyphs = vtkGlyph2D::New();
  this->RotationGlyphs->SetInputConnection(this->RotationTransformFilter->GetOutputPort ());
  this->RotationGlyphs->SetSource(this->RotationGlyphSource->GetOutput());


  this->RotationMapper = vtkPolyDataMapper2D::New();
  this->RotationMapper->SetInputConnection(this->RotationGlyphs->GetOutputPort());
  this->RotationActor = vtkActor2D::New();
  this->RotationActor->SetMapper(this->RotationMapper);

  this->RotationProperty = vtkProperty2D::New();
//  this->RotationActor->SetProperty(this->RotationProperty);
  this->RotationActor->SetProperty (GlobalBorderProperty);

  this->RotationProperty->SetColor (0, 1, 0);

  this->RotationPointOffset = 10;


  this->MinimumSize[0] = 1;
  this->MinimumSize[1] = 1;
  this->MaximumSize[0] = 100000;
  this->MaximumSize[1] = 100000;

  this->Tolerance = 2;
  this->OnObject = false;
  
  this->Angle = 0;

  this->SetShowBorder (false);

  this->Resizing = false;
  this->Rotating = false;
  this->Moving = false;

}

//-------------------------------------------------------------------------
vtkChipsRepresentation::~vtkChipsRepresentation()
{
  this->PositionCoordinate->Delete();
  this->Position2Coordinate->Delete();

  this->BorderTransform->Delete();
  this->BorderTransformFilter->Delete();
  this->BorderPolyData->Delete();
  this->BorderMapper->Delete();
  this->BorderActor->Delete();
  this->BorderProperty->Delete();
  this->BorderGlyphs->Delete();
  this->BorderGlyphSource->Delete(); 
  this->RotationTransform->Delete();
  this->RotationTransformFilter->Delete();
  this->RotationPolyData->Delete();
  this->RotationMapper->Delete();
  this->RotationActor->Delete();
  this->RotationProperty->Delete();
  this->RotationGlyphs->Delete(); 
  this->RotationGlyphSource->Delete();
}

//-------------------------------------------------------------------------
void vtkChipsRepresentation::StartWidgetInteraction(double eventPos[2])
{
  this->CurrentEventPosition[0] = this->StartEventPosition[0] = eventPos[0];
  this->CurrentEventPosition[1] = this->StartEventPosition[1] = eventPos[1];
}

//-------------------------------------------------------------------------
void vtkChipsRepresentation::WidgetInteraction(double eventPos[2])
{
  double dx = eventPos[0] - this->CurrentEventPosition[0];
  double dy = eventPos[1] - this->CurrentEventPosition[1];

  if (this->Resizing)
    {
    double *fpos1 = this->PositionCoordinate->GetComputedDoubleDisplayValue (this->Renderer);
    double *fpos2 = this->Position2Coordinate->GetComputedDoubleDisplayValue (this->Renderer);

    switch (this->InteractionState)
      {
      case vtkChipsRepresentation::RightHandle:
        fpos2[0] += dx;
      break;
      case vtkChipsRepresentation::LeftHandle:
        fpos1[0] += dx;
      break;
      case vtkChipsRepresentation::TopHandle:
        fpos2[1] += dy;
      break;
      case vtkChipsRepresentation::BottomHandle:
        fpos1[1] += dy;
      break;
      case vtkChipsRepresentation::TopRightHandle:
        fpos2[0] += dx;
        fpos2[1] += dy;
      break;
      case vtkChipsRepresentation::TopLeftHandle:
        fpos1[0] += dx;
        fpos2[1] += dy;
      break;
      case vtkChipsRepresentation::BottomRightHandle:
        fpos2[0] += dx;
        fpos1[1] += dy;
      break;
      case vtkChipsRepresentation::BottomLeftHandle:
        fpos1[0] += dx;
        fpos1[1] += dy;
      break;
      default:
      break;
      }

    this->Renderer->DisplayToNormalizedDisplay (fpos1[0], fpos1[1]);
    this->Renderer->NormalizedDisplayToViewport (fpos1[0], fpos1[1]);
    this->Renderer->ViewportToNormalizedViewport (fpos1[0], fpos1[1]);

    this->Renderer->DisplayToNormalizedDisplay (fpos2[0], fpos2[1]);
    this->Renderer->NormalizedDisplayToViewport (fpos2[0], fpos2[1]);
    this->Renderer->ViewportToNormalizedViewport (fpos2[0], fpos2[1]);

    this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
    this->PositionCoordinate->SetValue(fpos1[0], fpos1[1]);
    this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
    this->Position2Coordinate->SetValue(fpos2[0]-fpos1[0], fpos2[1]-fpos1[1]); 

    }
  else if (this->Rotating)
    {
    float angle = 0;
    float startAngle = 0;
    double anchor[2];

    double *fpos1 = this->PositionCoordinate->GetComputedDoubleDisplayValue (this->Renderer);
    double *fpos2 = this->Position2Coordinate->GetComputedDoubleDisplayValue (this->Renderer);


    anchor[0] =  (fpos1[0] + fpos2[0])/2.;
    anchor[1] =  (fpos1[1] + fpos2[1])/2.;

    dx = this->CurrentEventPosition[0] - anchor[0];
    dy = this->CurrentEventPosition[1] - anchor[1];

    if ((dx != 0) && (dy != 0))
      startAngle = atan2 (dy,dx);

    dx = eventPos[0] - anchor[0];
    dy = eventPos[1] - anchor[1];

    if ((dx != 0) && (dy != 0))
      angle = atan2 (dy,dx);

    this->Angle += vtkMath::DegreesFromRadians (angle - startAngle);
    }

  this->CurrentEventPosition[0] = eventPos[0];
  this->CurrentEventPosition[1] = eventPos[1];


}

void vtkChipsRepresentation::EndWidgetInteraction (double eventPos[2])
{
  if (this->Resizing)
     this->ResizingOff ();
  if (this->Moving)
     this->MovingOff ();
  if (this->Rotating)
     this->RotatingOff ();

  this->CurrentEventPosition[0] = eventPos[0];
  this->CurrentEventPosition[1] = eventPos[1];

}

//-------------------------------------------------------------------------
void vtkChipsRepresentation::NegotiateLayout()
{
  double size[2];
  this->GetSize(size);
  
  // Update the initial border geoemtry
  vtkPoints *BorderPoints = this->BorderPolyData->GetPoints ();

  BorderPoints->SetPoint(0, 0.0, 0.0, 0.0); 
  BorderPoints->SetPoint(1, size[0]/2.0, 0.0, 0.0);
  BorderPoints->SetPoint(2, size[0], 0.0, 0.0);
  BorderPoints->SetPoint(3, size[0], size[1]/2.0, 0.0);
  BorderPoints->SetPoint(4, size[0], size[1], 0.0);  
  BorderPoints->SetPoint(5, size[0]/2.0, size[1], 0.0);
  BorderPoints->SetPoint(6, 0.0, size[1], 0.0);
  BorderPoints->SetPoint(7, 0.0, size[1]/2., 0.0);

}


//-------------------------------------------------------------------------
int vtkChipsRepresentation::ComputeBorderInteractionState(int X, int Y, int modify)
{

  int *pos1 = this->PositionCoordinate->
    GetComputedDisplayValue(this->Renderer);
  int *pos2 = this->Position2Coordinate->
    GetComputedDisplayValue(this->Renderer);

  // first check the rotation case since it falls outside the bounding box
  bool rotate = (Y >= (pos2[1]+this->RotationPointOffset/2.) && Y <= (pos2[1]+this->RotationPointOffset + this->Tolerance));

  if (rotate)
    {

    float xcenter = (pos1[0] + pos2[0]) / 2.0;

    if (X >= (xcenter - this->Tolerance) && X <= (xcenter + this->Tolerance))
      this->InteractionState = vtkChipsRepresentation::RotateHandle;
    }
  // Figure out where we are in the widget. Exclude outside case.
  else if ( X < (pos1[0]-this->Tolerance) || (pos2[0]+this->Tolerance) < X || 
       Y < (pos1[1]-this->Tolerance) || (pos2[1]+this->Tolerance) < Y )
    {
    this->InteractionState = vtkChipsRepresentation::Outside;
    }
  else
    {
  
    // Now check for proximinity to edges and points
    bool bottom = (Y >= (pos1[1] - this->Tolerance) && Y <= (pos1[1] + this->Tolerance));
    bool right = (X >= (pos2[0] - this->Tolerance) && X <= (pos2[0] + this->Tolerance));
    bool left = (X >= (pos1[0] - this->Tolerance) && X <= (pos1[0] + this->Tolerance));

    bool top = (Y >= (pos2[1] - this->Tolerance) && Y <= (pos2[1] + this->RotationPointOffset/2.));
    bool rotate = (Y >= (pos2[1]+this->RotationPointOffset/2.) && Y <= (pos2[1]+this->RotationPointOffset + this->Tolerance));

    // Points
    if ( bottom && right )
      {
      this->InteractionState = vtkChipsRepresentation::BottomRightHandle;
      }
    else if ( right && top )
      {
      this->InteractionState = vtkChipsRepresentation::TopRightHandle;
      }
    else if ( top && left )
      {
      this->InteractionState = vtkChipsRepresentation::TopLeftHandle;
      }
    else if ( left && bottom )
      {
      this->InteractionState = vtkChipsRepresentation::BottomLeftHandle;
      }

    // Edges
    else if ( bottom || top || right || left )
      {
      float xcenter = (pos1[0] + pos2[0]) / 2.0;
      float ycenter = (pos1[1] + pos2[1]) / 2.0;
      if ( bottom )
        {
        if (X >= (xcenter - this->Tolerance) && X <= (xcenter + this->Tolerance))
          this->InteractionState = vtkChipsRepresentation::BottomHandle;
        else
          this->InteractionState = vtkChipsRepresentation::BottomEdge;
        }
      else if ( right )
        {
        if (Y >= (ycenter - this->Tolerance) && Y <= (ycenter + this->Tolerance))
          this->InteractionState = vtkChipsRepresentation::RightHandle;
        else
          this->InteractionState = vtkChipsRepresentation::RightEdge;
        }
      else if ( top )
        {
        if (X >= (xcenter - this->Tolerance) && X <= (xcenter + this->Tolerance))      
          this->InteractionState = vtkChipsRepresentation::TopHandle;
        else
          this->InteractionState = vtkChipsRepresentation::TopEdge;
        }
      else if ( left )
        {
        if (Y >= (ycenter - this->Tolerance) && Y <= (ycenter + this->Tolerance))      
          this->InteractionState = vtkChipsRepresentation::LeftHandle;
        else
          this->InteractionState = vtkChipsRepresentation::LeftEdge;
        }
      else if ( rotate )
        {
        if (X >= (xcenter - this->Tolerance) && X <= (xcenter + this->Tolerance))
          this->InteractionState = vtkChipsRepresentation::RotateHandle;
        }

      }

    else // must be interior
      {
        this->InteractionState = vtkChipsRepresentation::Inside;
      }
    }
  
  return this->InteractionState;
}

void vtkChipsRepresentation::SetRotationEnabled (bool flag)
{
  this->RotationEnabled = flag;

  if (!this->BorderEnabled)
    return;

  this->BorderActor->SetVisibility (flag&this->ShowBorder);
}


void vtkChipsRepresentation::SetShowBorder (bool flag)
{
  this->ShowBorder = flag;
  if (!this->BorderEnabled)
    return;

  this->BorderActor->SetVisibility (flag);

  if (this->RotationEnabled)
    this->RotationActor->SetVisibility (flag);
}

void vtkChipsRepresentation::SetBorderEnabled (bool flag)
{
  this->BorderEnabled = flag;
  if (!this->BorderEnabled)
    {
    this->BorderActor->SetVisibility (0);
    this->RotationActor->SetVisibility (0);
    }
  else
    {
    this->BorderActor->SetVisibility (this->ShowBorder);
    if (this->RotationEnabled)
      this->RotationActor->SetVisibility (this->ShowBorder);
    }
}



//-------------------------------------------------------------------------
int vtkChipsRepresentation::ComputeInteractionState(int X, int Y, int modify)
{
  int state = this->ComputeBorderInteractionState (X,Y,modify);

  this->OnObject = false;

  if (state != vtkChipsRepresentation::Outside)
    {
      this->OnObject = true;
      if (state == vtkChipsRepresentation::Inside)
         this->InteractionState = vtkChipsRepresentation::On;
    }
  return this->InteractionState; 
}


//-------------------------------------------------------------------------
void vtkChipsRepresentation::BuildRepresentation()
{
  if ( this->GetMTime() > this->BuildTime || 
       (this->Renderer && this->Renderer->GetVTKWindow() &&
        this->Renderer->GetVTKWindow()->GetMTime() > this->BuildTime) )
    {
    // Negotiate with subclasses
    if ( this->CheckNegotiateLayout () )
      {
      this->NegotiateLayout();
      this->Negotiated = true;
      }

    // Set things up
    double *pos1 = this->PositionCoordinate->
      GetComputedDoubleViewportValue(this->Renderer);
    double *pos2 = this->Position2Coordinate->
      GetComputedDoubleViewportValue(this->Renderer);


    // If the widget's aspect ratio is to be preserved (ProportionalResizeOn),
    // then (pos1,pos2) are a bounding rectangle.
    if ( this->ProportionalResize )
      {
      }


    // Now transform the canonical widget into display coordinates
    double size[2] = {1,1};
    this->GetSize(size);

    double tx = pos1[0];
    double ty = pos1[1];
    double sx = (pos2[0]-pos1[0]) / size[0];
    double sy = (pos2[1]-pos1[1]) / size[1];

    this->BorderTransform->Identity();
    this->BorderTransform->Translate(tx, ty, 0.0);
    this->BorderTransform->Scale(sx,sy,1);

    this->RotationTransform->Identity();
    this->RotationTransform->Translate(tx, ty+this->RotationPointOffset, 0.0);
    this->RotationTransform->Scale(sx,sy,1);



    this->BuildTime.Modified();
    }
}

//-------------------------------------------------------------------------
void vtkChipsRepresentation::Update()
{
  this->Negotiated = false;

  this->Modified();
//  this->BuildRepresentation();

}

//-------------------------------------------------------------------------
void vtkChipsRepresentation::SetUseGlobalProperty (bool enable)
{
  this->UseGlobalProperty = enable;

  if (enable)
  {
    this->BorderActor->SetProperty (GlobalBorderProperty);
    this->RotationActor->SetProperty (GlobalBorderProperty);
  }
  else
  {
    this->BorderActor->SetProperty (this->BorderProperty);
    this->RotationActor->SetProperty (this->BorderProperty);
  }


}



//-------------------------------------------------------------------------
void vtkChipsRepresentation::GetActors2D(vtkPropCollection *pc)
{
/*  pc->AddItem(this->BorderActor);
  pc->AddItem(this->RotationActor);
*/
}

//-------------------------------------------------------------------------
void vtkChipsRepresentation::ReleaseGraphicsResources(vtkWindow *w)
{
  this->BorderActor->ReleaseGraphicsResources(w);
  this->RotationActor->ReleaseGraphicsResources(w);
}

//-------------------------------------------------------------------------
int vtkChipsRepresentation::RenderOverlay(vtkViewport *w)
{
  int count = 0;
  this->BuildRepresentation();
  if ( this->BorderActor->GetVisibility() )
    count += this->BorderActor->RenderOverlay(w);
  if ( this->RotationActor->GetVisibility() )
    count += this->RotationActor->RenderOverlay(w);
  return count;
}

//-------------------------------------------------------------------------
int vtkChipsRepresentation::RenderOpaqueGeometry(vtkViewport *w)
{
  int count = 0;
  this->BuildRepresentation();
  if ( this->BorderActor->GetVisibility() )
    count += this->BorderActor->RenderOpaqueGeometry(w);
  if ( this->RotationActor->GetVisibility() )
    count += this->RotationActor->RenderOpaqueGeometry(w);
  return count;
}

//-----------------------------------------------------------------------------
int vtkChipsRepresentation::RenderTranslucentPolygonalGeometry(vtkViewport *w)
{
  int count = 0;
  this->BuildRepresentation();
  if ( this->BorderActor->GetVisibility() )
    count += this->BorderActor->RenderTranslucentPolygonalGeometry(w);
  if ( this->RotationActor->GetVisibility() )
    count += this->RotationActor->RenderTranslucentPolygonalGeometry(w);
  return count;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkChipsRepresentation::HasTranslucentPolygonalGeometry()
{
  int count = 0;
  this->BuildRepresentation();
  if (  this->BorderActor->GetVisibility() )
    count += this->BorderActor->HasTranslucentPolygonalGeometry();
  if ( this->RotationActor->GetVisibility() )
    count += this->RotationActor->HasTranslucentPolygonalGeometry();
  return count;
}

//-------------------------------------------------------------------------
void vtkChipsRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
  
  os << indent << "Show Border: " << this->ShowBorder << endl;
  os << indent << "Enable Border: " << this->BorderEnabled << endl;
  os << indent << "Enable Rotation: " << this->RotationEnabled << endl;

    
  if ( this->BorderProperty )
    {
    os << indent << "Border Property:\n";
    this->BorderProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "Border Property: (none)\n";
    }

  os << indent << "Proportional Resize: " 
     << (this->ProportionalResize ? "On\n" : "Off\n");
  os << indent << "Minimum Size: " << this->MinimumSize[0] << " " << this->MinimumSize[1] << endl;
  os << indent << "Maximum Size: " << this->MaximumSize[0] << " " << this->MaximumSize[1] << endl;

  os << indent << "Tolerance: " << this->Tolerance << "\n";
  
}
