#include "vtkChipsScalarBarRepresentation.hh"

#include "vtkRenderedAreaPicker.h"
#include "vtkObjectFactory.h"
#include "vtkRenderer.h"
#include "vtkTransform.h"
#include "vtkAssemblyPath.h"
#include "vtkProp3DCollection.h"
#include "vtkMath.h"
#include "vtkActor2D.h"
#include "vtkScalarBarActorExtra.hh"

#include <float.h>
#include <cmath>


vtkStandardNewMacro(vtkChipsScalarBarRepresentation);

//-------------------------------------------------------------------------
vtkChipsScalarBarRepresentation::vtkChipsScalarBarRepresentation()
{
}

//-------------------------------------------------------------------------
vtkChipsScalarBarRepresentation::~vtkChipsScalarBarRepresentation()
{
}


//-------------------------------------------------------------------------
void vtkChipsScalarBarRepresentation::NegotiateLayout()
{
  vtkScalarBarActorExtra* ScalarBarActor = reinterpret_cast<vtkScalarBarActorExtra*>(this->PickProp);
  if (!ScalarBarActor)
    return;

  ScalarBarActor->UpdatePositions (this->Renderer);

  vtkCoordinate *coordinates[2];
  ScalarBarActor->GetLocation (coordinates);
  double *pos1 = coordinates[0]->GetComputedDoubleViewportValue(this->Renderer);
  double *pos2 = coordinates[1]->GetComputedDoubleViewportValue(this->Renderer);


  if (ScalarBarActor->GetClipData ())
    {
    vtkCoordinate *clipCoordinate[2];
    double *clipPos1;
    double *clipPos2;

    ScalarBarActor->GetClipLocation (clipCoordinate);

    clipPos1 = clipCoordinate[0]->GetComputedDoubleViewportValue(this->Renderer);
    clipPos2 = clipCoordinate[1]->GetComputedDoubleViewportValue(this->Renderer);

    if (clipPos1[0] > pos1[0])
      pos1[0] = clipPos1[0];
    if (clipPos1[1] > pos1[1])
      pos1[1] = clipPos1[1];

    if (clipPos1[0] > pos2[0])
      pos2[0] = clipPos1[0];
    if (clipPos1[1] > pos2[1])
      pos2[1] = clipPos1[1];

    if (clipPos2[0] < pos2[0])
      pos2[0] = clipPos2[0];
    if (clipPos2[1] < pos2[1])
      pos2[1] = clipPos2[1];

    if (clipPos2[0] < pos1[0])
      pos1[0] = clipPos2[0];
    }

  this->PositionCoordinate->SetCoordinateSystemToViewport ();
  this->PositionCoordinate->SetValue(pos1[0], pos1[1]);
  this->Position2Coordinate->SetCoordinateSystemToViewport ();
  this->Position2Coordinate->SetValue(pos2[0]-pos1[0], pos2[1]-pos1[1]);

}

//-------------------------------------------------------------------------
void vtkChipsScalarBarRepresentation::WidgetInteraction(double eventPos[2])
{
  double dx = eventPos[0] - this->CurrentEventPosition[0];
  double dy = eventPos[1] - this->CurrentEventPosition[1];

  vtkScalarBarActorExtra* ScalarBarActor = reinterpret_cast<vtkScalarBarActorExtra*>(this->PickProp);
  if (!ScalarBarActor)
    return;

  if (this->Resizing)
    {
    float length = ScalarBarActor->GetLength ();
    float width = ScalarBarActor->GetWidth ();
    int orientation = ScalarBarActor->GetOrientation ();
    double origin[2];
    double vertAlignment = ScalarBarActor->GetVertAlignmentOffset ();
    double horizAlignment = ScalarBarActor->GetHorizAlignmentOffset ();

    vtkCoordinate **location = ScalarBarActor->GetLocation ();
    double *pos1 = location[0]->GetComputedDoubleViewportValue(this->Renderer);
    double *pos2 = location[1]->GetComputedDoubleViewportValue(this->Renderer);


    switch (this->InteractionState)
      {
      case vtkChipsRepresentation::RightHandle:
        pos2[0] += dx;
      break;
      case vtkChipsRepresentation::LeftHandle:
        pos1[0] += dx;
      break;
      case vtkChipsRepresentation::TopHandle:
        pos2[1] += dy;
      break;
      case vtkChipsRepresentation::BottomHandle:
        pos1[1] += dy;
      break;
      case vtkChipsRepresentation::BottomLeftHandle:
        pos1[0] += dx;
        pos1[1] += dy;
      break;
      case vtkChipsRepresentation::TopLeftHandle:
        pos1[0] += dx;
        pos2[1] += dy;
      break;
      case vtkChipsRepresentation::BottomRightHandle:
        pos2[0] += dx;
        pos1[1] += dy;
      break;
      case vtkChipsRepresentation::TopRightHandle:
        pos2[0] += dx;
        pos2[1] += dy;
      break;
      default:
      break;
      }

    this->Renderer->ViewportToNormalizedViewport (pos1[0], pos1[1]);
    this->Renderer->ViewportToNormalizedViewport (pos2[0], pos2[1]);
    if (orientation == VTK_ORIENT_HORIZONTAL)
      {
      length = (pos2[0] - pos1[0]);
      width = (pos2[1] - pos1[1]);
      origin[0] = pos1[0] + length * horizAlignment;
      origin[1] = pos1[1] + width * vertAlignment;
      }
    else
      {
      length = (pos2[1] - pos1[1]);
      width = (pos2[0] - pos1[0]);
      origin[0] = pos1[0] + width * vertAlignment;
      origin[1] = pos1[1] + length * horizAlignment;
      }

    if ((length > 0)  && (width > 0))
      {
//      this->Renderer->ViewportToNormalizedViewport (origin[0], origin[1]);
      ScalarBarActor->SetWidth (width);
      ScalarBarActor->SetLength (length);
      ScalarBarActor->SetOrigin (origin);

      this->Update ();
      }

    }

  this->CurrentEventPosition[0] = eventPos[0];
  this->CurrentEventPosition[1] = eventPos[1];


}

void vtkChipsScalarBarRepresentation::SetScalarBarActor (vtkScalarBarActorExtra *actor)
{
  this->SetPickProp (actor);
}

vtkScalarBarActorExtra * vtkChipsScalarBarRepresentation::GetScalarBarActor ()
{
  return reinterpret_cast<vtkScalarBarActorExtra*>(this->GetPickProp ());
}


void vtkChipsScalarBarRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}


