#include "vtkChipsTextRepresentation.hh"

#include "vtkRenderedAreaPicker.h"
#include "vtkObjectFactory.h"
#include "vtkRenderer.h"
#include "vtkTransform.h"
#include "vtkAssemblyPath.h"
#include "vtkProp3DCollection.h"
#include "vtkGlyphSource2D.h"
#include "vtkTextProperty.h"
#include "vtkMath.h"
#include "vtkWindow.h"


#include <float.h>
#include <cmath>


vtkStandardNewMacro(vtkChipsTextRepresentation);

//-------------------------------------------------------------------------
vtkChipsTextRepresentation::vtkChipsTextRepresentation()
{
  this->TextActor = NULL;
  this->FontSize = 0;
  this->MaxFontSize = 1000;

  this->Resized = false;
  this->Rotated = false;
}

//-------------------------------------------------------------------------
vtkChipsTextRepresentation::~vtkChipsTextRepresentation()
{
  if (this->TextActor)
    this->TextActor->Delete ();
}


//-------------------------------------------------------------------------
void vtkChipsTextRepresentation::NegotiateLayout()
{
  if (!this->TextActor)
    return;

  double x[2] = {0,0};
  double y[2] = {0,0};

  int glyphScale = this->BorderGlyphSource->GetScale ();

  this->TextActor->GetBBox (this->Renderer, x[0], y[0], x[1], y[1]);

  x[0] -= glyphScale;
  y[0] -= glyphScale;
  x[1] += glyphScale;
  y[1] += glyphScale;

  this->PositionCoordinate->SetCoordinateSystemToViewport();
  this->PositionCoordinate->SetValue(x[0], y[0]);
  this->Position2Coordinate->SetCoordinateSystemToViewport();
  this->Position2Coordinate->SetValue(x[1]-x[0], y[1]-y[0]);

}

void vtkChipsTextRepresentation::WidgetInteraction(double eventPos[2])
{
  double lowerBox[2] = {0,0};
  double upperBox[2] = {0,0};
  double newLowerBox[2] = {0,0};
  double newUpperBox[2] = {0,0};

  int glyphScale = this->BorderGlyphSource->GetScale ();

  this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1], 
                                            upperBox[0], upperBox[1]);


  newLowerBox[0] = this->PositionCoordinate->GetValue ()[0];
  newLowerBox[1] = this->PositionCoordinate->GetValue ()[1];
  newUpperBox[0] = newLowerBox[0] + this->Position2Coordinate->GetValue ()[0] - glyphScale;
  newUpperBox[1] = newLowerBox[1] + this->Position2Coordinate->GetValue ()[1] - glyphScale;

  newLowerBox[0] += glyphScale;
  newLowerBox[1] += glyphScale;



  double dx = eventPos[0] - this->CurrentEventPosition[0];
  double dy = eventPos[1] - this->CurrentEventPosition[1];

  if (this->Resizing)
    {
    int localFontSize = this->TextActor->GetTextProperty()->GetFontSize();
    int tmpFontSize = localFontSize;
    int sign;

    switch (this->InteractionState)
      {
      case vtkChipsRepresentation::RightHandle:
        {
        newUpperBox[0] += dx;
        sign = copysign (1.0, dx);
        
        while ((sign > 0) ? (newUpperBox[0] > upperBox[0]) : (newUpperBox[0] < upperBox[0]))
          {
          tmpFontSize += sign;
          if ((tmpFontSize <=  0) || (tmpFontSize == this->MaxFontSize))
             break;

          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }
        }

      break;
      case vtkChipsRepresentation::LeftHandle:
        {
        newLowerBox[0] += dx;
        sign = -copysign (1.0, dx);

        while ((sign > 0) ? (newLowerBox[0] < lowerBox[0]) : (newLowerBox[0] > lowerBox[0]))
          {
          tmpFontSize += sign;
          if ((tmpFontSize <=  0) || (tmpFontSize == this->MaxFontSize))
             break;

          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }

        }

      break;
      case vtkChipsRepresentation::TopHandle:
        {
        newUpperBox[1] += dy;
        sign = copysign (1.0, dy);

        while ((sign > 0) ? (newUpperBox[1] > upperBox[1]) : (newUpperBox[1] < upperBox[1]))
          {
          tmpFontSize += sign;
          if ((tmpFontSize <=  0) || (tmpFontSize == this->MaxFontSize))
             break;

          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }
        }

      break;
      case vtkChipsRepresentation::BottomHandle:
        {
        newLowerBox[1] += dy;
        sign = -copysign (1.0, dy);

        while ((sign > 0) ? (newLowerBox[1] < lowerBox[1]) : (newLowerBox[1] > lowerBox[1]))
          {
          tmpFontSize += sign;
          if ((tmpFontSize <=  0) || (tmpFontSize == this->MaxFontSize))
             break;

          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }
        }

      break;
      case vtkChipsRepresentation::TopRightHandle:
        {
        newUpperBox[0] += dx;
        newUpperBox[1] += dy;

        if ((dx==0) && (dy==0))
          break;

        while (1)
          {
          if (fabs(dy) > fabs(dx))
            {
            sign = copysign (1.0, dy);
            if (((sign > 0) && (newUpperBox[1] <= upperBox[1])) ||
                 ((sign < 0) && (newUpperBox[1] > upperBox[1])))
                break;
            }
          else
            {
            sign = copysign (1.0, dx);
            if (((sign > 0) && (newUpperBox[0] <= upperBox[0])) ||
                 ((sign < 0) && (newUpperBox[0] > upperBox[0])))
                break;
            }

          tmpFontSize += sign;
          if ((tmpFontSize <= 0) || (tmpFontSize == this->MaxFontSize))
             break;

          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }
        }
      break;
      case vtkChipsRepresentation::TopLeftHandle:
        {
        newLowerBox[0] += dx;
        newUpperBox[1] += dy;

        if ((dx==0) && (dy==0))
          break;

        while (1)
          {
          if (fabs(dy) > fabs(dx))
            {
            sign = copysign (1.0, dy);
            if (((sign > 0) && (newUpperBox[1] <= upperBox[1])) ||
                 ((sign < 0) && (newUpperBox[1] > upperBox[1])))
                break;
            }
          else
            {
            sign = -copysign (1.0, dx);
            if (((sign > 0) && (newLowerBox[0] >= lowerBox[0])) ||
                 ((sign < 0) && (newLowerBox[0] < lowerBox[0])))
                break;
            }

          tmpFontSize += sign;
          if ((tmpFontSize <=  0) || (tmpFontSize == this->MaxFontSize))
             break;

          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }
        }

      break;
      case vtkChipsRepresentation::BottomRightHandle:
        {
        newUpperBox[0] += dx;
        newLowerBox[1] += dy;

        if ((dx==0) && (dy==0))
          break;

        while (1)
          {
          if (fabs(dy) > fabs(dx))
            {
            sign = -copysign (1.0, dy);
            if (((sign > 0) && (newLowerBox[1] >= lowerBox[1])) ||
                 ((sign < 0) && (newLowerBox[1] < lowerBox[1])))
                break;
            }
          else
            {
            sign = copysign (1.0, dx);
            if (((sign > 0) && (newUpperBox[0] <= upperBox[0])) ||
                 ((sign < 0) && (newUpperBox[0] > upperBox[0])))
                break;
            }

               
          tmpFontSize += sign;
          if ((tmpFontSize < 0) || (tmpFontSize == this->MaxFontSize))
             break;

          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }
        }
      break;
      case vtkChipsRepresentation::BottomLeftHandle:
        {
        newLowerBox[0] += dx;
        newLowerBox[1] += dy;

        if ((dx==0) && (dy==0))
          break;


        while (1)
          {

          if (fabs(dy) > fabs(dx))
            {
            sign = -copysign (1.0, dy);
            if (((sign > 0) && (newLowerBox[1] >= lowerBox[1])) ||
                 ((sign < 0) && (newLowerBox[1] < lowerBox[1])))
                break;
            }
          else
            {
            sign = -copysign (1.0, dx);
            if (((sign > 0) && (newLowerBox[0] >= lowerBox[0])) ||
                 ((sign < 0) && (newLowerBox[0] < lowerBox[0])))
                break;
            }

          tmpFontSize += sign;
          if ((tmpFontSize < 0) || (tmpFontSize == this->MaxFontSize))
             break;
          this->TextActor->GetTextProperty()->SetFontSize(tmpFontSize);
          this->TextActor->GetBBox (this->Renderer, lowerBox[0], lowerBox[1],
                                            upperBox[0], upperBox[1]);
          }
        }

      break;
      }

    lowerBox[0] -= glyphScale;
    lowerBox[1] -= glyphScale;
    upperBox[0] += glyphScale;
    upperBox[1] += glyphScale;

    newLowerBox[0] -= glyphScale;
    newLowerBox[1] -= glyphScale;
    newUpperBox[0] += glyphScale;
    newUpperBox[1] += glyphScale;


    if (tmpFontSize != localFontSize)
      {
      this->TextActor->Modified ();
      this->PositionCoordinate->SetValue(lowerBox[0], lowerBox[1]);
      this->Position2Coordinate->SetValue(upperBox[0]-lowerBox[0],
                                         upperBox[1]-lowerBox[1]);
      }
/*    else
      {
      this->PositionCoordinate->SetValue(newLowerBox[0], newLowerBox[1]);
      this->Position2Coordinate->SetValue(newUpperBox[0]-newLowerBox[0], 
                                         newUpperBox[1]-newLowerBox[1]);
      }
*/


    }
  else if (this->Rotating)
    {

    float angle = 0;
    float startAngle = 0;

/*    double a_sq = (pow(this->anchor[0]-this->StartEventPosition[0],2)+
                     pow(this->anchor[1]-this->StartEventPosition[1],2));
    double b_sq = (pow(this->anchor[0]-eventPos[0],2)+
                     pow(this->anchor[1]-eventPos[1],2));
    double c_sq = (pow(eventPos[0]-this->StartEventPosition[0],2)+
                     pow(eventPos[1]-this->StartEventPosition[1],2));


    angle = vtkMath::DegreesFromRadians (acos ((-c_sq + a_sq + b_sq)/(2*sqrt(a_sq*b_sq))));
*/
    dx = this->StartEventPosition[0] - this->anchor[0];
    dy = this->StartEventPosition[1] - this->anchor[1];

    if ((dx != 0) && (dy != 0))
      startAngle = atan2 (dy,dx);

    dx = eventPos[0] - this->anchor[0];
    dy = eventPos[1] - this->anchor[1];

    if ((dx != 0) && (dy != 0))
      angle = atan2 (dy,dx);

    angle = vtkMath::DegreesFromRadians (angle - startAngle) + this->Angle;

    this->TextActor->GetTextProperty()->SetOrientation(angle);

    this->Update ();
       
    }
  this->CurrentEventPosition[0] = eventPos[0];
  this->CurrentEventPosition[1] = eventPos[1];

}

void vtkChipsTextRepresentation::SetResizing (bool resize)
{
  if (resize == this->Resizing)
    return;

  this->Resizing = resize;

  if (!this->TextActor)
    return;

  double xRange[2];
  double yRange[2];
  double tmpCoord[2];
  double angle;
  double halign, valign;

  double *pos = this->TextActor->GetPositionCoordinate()->GetComputedDoubleViewportValue(this->Renderer);
  vtkCoordinate *refCoordinate = this->TextActor->GetPositionCoordinate ()->GetReferenceCoordinate ();


  if (refCoordinate)
    {
    double *offsetPosition = refCoordinate->GetComputedDoubleViewportValue(this->Renderer);
    pos[0] -= offsetPosition[0];
    pos[1] -= offsetPosition[1];
    }
   

  if (resize)
    {
    this->FontSize = this->TextActor->GetTextProperty()->GetFontSize();
    this->Resized = false;

    this->textHalign = this->TextActor->GetHorizAlignmentOffset ();
    this->textValign = this->TextActor->GetVertAlignmentOffset ();

    halign = .5 - this->textHalign;
    valign = .5 - this->textValign;

    this->TextActor->SetVertAlignmentOffset(.5);
    this->TextActor->SetHorizAlignmentOffset(.5);

    }
  else
    {

    this->Resized = (this->FontSize != this->TextActor->GetTextProperty()->GetFontSize());
    this->FontSize = this->TextActor->GetTextProperty()->GetFontSize();

    halign = -.5 + this->textHalign;
    valign = -.5 + this->textValign;


    this->TextActor->SetVertAlignmentOffset(this->textValign);
    this->TextActor->SetHorizAlignmentOffset(this->textHalign);

    }

    
  angle=vtkMath::RadiansFromDegrees(this->TextActor->GetTextProperty()->GetOrientation());


  this->TextActor->GetAlignmentRange(this->Renderer,xRange,yRange);

  tmpCoord[0]=(xRange[1]-xRange[0])*halign;
  tmpCoord[1]=yRange[1]*valign;

  pos[0] = tmpCoord[0]*cos(angle) - tmpCoord[1]*sin(angle) + pos[0];
  pos[1] = tmpCoord[0]*sin(angle) + tmpCoord[1]*cos(angle) + pos[1];

  this->TextActor->GetPositionCoordinate()->SetCoordinateSystemToViewport ();
  this->TextActor->GetPositionCoordinate()->SetValue (pos); 


}

void vtkChipsTextRepresentation::SetRotating (bool rotate)
{
  if (rotate == this->Rotating)
    return;

  this->Rotating = rotate;

  if (!this->TextActor)
    return;

  if (this->Rotating)
    {

    double *pos = this->TextActor->GetPositionCoordinate()->GetComputedDoubleViewportValue(this->Renderer);
    double xRange[2];
    double yRange[2];
    double tmpCoord[2];

    double halign = this->TextActor->GetHorizAlignmentOffset ();
    double valign = this->TextActor->GetVertAlignmentOffset ();

    this->Rotated = false;

    this->Angle = this->TextActor->GetTextProperty()->GetOrientation();

    this->TextActor->GetAlignmentRange(this->Renderer,xRange,yRange);

    tmpCoord[0]=(xRange[1]-xRange[0])*halign;
    tmpCoord[1]=yRange[1]*valign;


    this->anchor[0] = tmpCoord[0]*cos(this->Angle) - tmpCoord[1]*sin(this->Angle) + pos[0];
    this->anchor[1] = tmpCoord[0]*sin(this->Angle) + tmpCoord[1]*cos(this->Angle) + pos[1];

    this->Renderer->ViewportToNormalizedDisplay(this->anchor[0], this->anchor[1]);
    this->Renderer->NormalizedDisplayToDisplay(this->anchor[0], this->anchor[1]);
    }
  else
    {

    float angle = this->TextActor->GetTextProperty()->GetOrientation();

    this->Rotated = (fabs (angle - this->Angle) > FLT_EPSILON);
    this->Angle = angle;
    }

}


void vtkChipsTextRepresentation::GetBoundingBox (double bbox[])
{
  this->TextActor->GetBBox (this->Renderer, bbox[0], bbox[1],
                                            bbox[2], bbox[3]);

  this->Renderer->ViewportToNormalizedDisplay(bbox[0], bbox[1]);
  this->Renderer->NormalizedDisplayToDisplay(bbox[0], bbox[1]);

  this->Renderer->ViewportToNormalizedDisplay(bbox[2], bbox[3]);
  this->Renderer->NormalizedDisplayToDisplay(bbox[2], bbox[3]);


}


void vtkChipsTextRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}

