// 
//  Copyright (C) 2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __vtkChipsTextRepresentation_h
#define __vtkChipsTextRepresentation_h

#include "vtkChipsRepresentation.hh"
#include "vtkTextActorExtra.hh"


class VTK_WIDGETS_EXPORT vtkChipsTextRepresentation : public vtkChipsRepresentation 
{
public:

  // Description:
  // Instantiate this class.
  static vtkChipsTextRepresentation *New();

  // Description:
  // Define standard methods.
  vtkTypeMacro(vtkChipsTextRepresentation,vtkChipsRepresentation);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Set/Get Text Actor 
  vtkSetObjectMacro (TextActor, vtkTextActorExtra);
  vtkGetObjectMacro (TextActor, vtkTextActorExtra);

  // Description:
  // Get if the text has been resized
  vtkGetMacro (Resized, bool);

  // Description:
  // Get if the text has been resized
  vtkGetMacro (Rotated, bool);


  // Description:
  // Get the current font size
  vtkGetMacro (FontSize, int);

  // Description:
  // Set/Get the maximum font size
  vtkSetMacro (MaxFontSize, int);
  vtkGetMacro (MaxFontSize, int);


  virtual void WidgetInteraction(double eventPos[2]);

  virtual void NegotiateLayout();

  virtual void SetResizing (bool resize);

  virtual void SetRotating (bool rotate);

  virtual void GetBoundingBox (double bbox[]);


protected:

  vtkChipsTextRepresentation();
  ~vtkChipsTextRepresentation();


private:
  vtkChipsTextRepresentation(const vtkChipsTextRepresentation&);  //Not implemented
  void operator=(const vtkChipsTextRepresentation&);  //Not implemented

  vtkTextActorExtra *TextActor;
  double textHalign;
  double textValign;
  double anchor[2];
  int MaxFontSize;

  int FontSize;

  bool Resized;
  bool Rotated;

};

#endif



