// 
//  Copyright (C) 2004-2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "vtkChipsTextWidget.hh"
#include "vtkChipsWidgetEvent.hh"
#include "vtkCommand.h"
#include "vtkObjectFactory.h"
#include "vtkAbstractPropPicker.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkRenderer.h"
#include "vtkMath.h"

#include "vtkChipsTextRepresentation.hh"

#include "ChipsEventManager.hh"


extern ChipsEventManager *event_manager;


vtkStandardNewMacro(vtkChipsTextWidget);


//-------------------------------------------------------------------------
vtkChipsTextWidget::vtkChipsTextWidget()
{
}

//-------------------------------------------------------------------------
vtkChipsTextWidget::~vtkChipsTextWidget()
{
}

//-------------------------------------------------------------------------
int vtkChipsTextWidget::SubclassMoveAction()
{
  vtkChipsTextRepresentation *chipsRep = reinterpret_cast<vtkChipsTextRepresentation*>(this->WidgetRep);

  if ( this->WidgetState == vtkChipsWidget::Selected )
     this->SetCursor (this->WidgetRep->GetInteractionState());

  // compute some info we need for all cases
  int X = this->Interactor->GetEventPosition()[0];
  int Y = this->Interactor->GetEventPosition()[1];


  if ( this->WidgetState != vtkChipsWidget::Modify)
    {
    chipsRep->ComputeInteractionState(X, Y);
    }
  else
    {
    double newEventPosition[2];
    newEventPosition[0] = static_cast<double>(X);
    newEventPosition[1] = static_cast<double>(Y);

    // handle resizing
    if(chipsRep->GetResizing ())
      {
      // Okay, adjust the representation (the widget is currently selected)
      chipsRep->WidgetInteraction(newEventPosition);

      }
    else if (chipsRep->GetMoving ())
      {

      // Okay, adjust the representation (the widget is currently selected)
      double startPosition[2];
      chipsRep->GetCurrentEventPosition (startPosition);
      chipsRep->WidgetInteraction(newEventPosition);

      event_manager->invoke_event (newEventPosition, ChipsEventManager::MoveEvent, startPosition);


      }
    else if (chipsRep->GetRotating ())
      {
      // Okay, adjust the representation (the widget is currently selected)
      chipsRep->WidgetInteraction(newEventPosition);

      }


    // start a drag
    this->InvokeEvent(vtkCommand::InteractionEvent,NULL);
    this->Render();
    }


  return 1;
}

int vtkChipsTextWidget::SubclassEndSelectAction()
{
  vtkChipsTextRepresentation *chipsRep = reinterpret_cast<vtkChipsTextRepresentation*>(this->WidgetRep);
  double newEventPosition[2] = {this->Interactor->GetEventPosition()[0],
                                  this->Interactor->GetEventPosition()[1]};

  if ( this->WidgetState != vtkChipsWidget::Modify )
    return 1;


  if (chipsRep->GetResizing ())
    {
    double bbox[4];
    chipsRep->GetBoundingBox (bbox);

    chipsRep->ResizingOff ();

    if (chipsRep->GetResized ())
      {
      event_manager->invoke_event (newEventPosition, ChipsEventManager::ResizeEvent, &bbox[0]);
      }
    }
  else if (chipsRep->GetRotating ())
    {
    chipsRep->RotatingOff ();

    if (chipsRep->GetRotated ())
      {

      float angle = chipsRep->GetAngle ();

      event_manager->invoke_event (newEventPosition, ChipsEventManager::RotateEvent, &angle);
      }
    }


  // Return state to not modifying
  this->WidgetState = vtkChipsWidget::Selected;
  chipsRep->EndWidgetInteraction (newEventPosition);


  // stop adjusting
  this->EndInteraction();
  this->InvokeEvent(vtkCommand::EndInteractionEvent,NULL);


  return 1;
}


    

//-------------------------------------------------------------------------
void vtkChipsTextWidget::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}

