// 
//  Copyright (C) 2011,2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// We have to define XTSTRINGDEFINES (used in X11/StringDefs.h and X11/Shell.h)
// otherwise the string are define as char * instead of const char which can
// cause warnings.
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gtk/gtkgl.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/gdk.h>

#include "vtkGtkRenderWindowInteractor.hh"

#include "vtkObjectFactory.h"
#include "vtkCommand.h"
#include "vtkRenderWindow.h"
#include "vtkKeySym.hh"
#include "vtkChipsCommand.hh"

#include <vtkstd/map>

#include <string>

using namespace std; 

vtkCxxRevisionMacro(vtkGtkRenderWindowInteractor, "$Revision: 1.138 $");
vtkStandardNewMacro(vtkGtkRenderWindowInteractor);

// Initialize static members:
GdkGLConfig *vtkGtkRenderWindowInteractor::glconfig = NULL;

class vtkGtkRenderWindowTimerData
{
   public:
      vtkGtkRenderWindowInteractor *interactor;
      unsigned int vtkTimerId;
      unsigned int platformTimerId;
};

//-------------------------------------------------------------------------
vtkGtkRenderWindowInteractor::vtkGtkRenderWindowInteractor()
{
  this->window = NULL;
  this->drawingArea = NULL;
  this->ownWindow = true;
  this->ownDrawingArea = true;
  this->shift = 0;
  this->control = 0;

  string dname = "", fname;
  char *tmpdir = getenv("ASCDS_INSTALL");
  if (tmpdir)   dname = tmpdir;
  dname.append("/config/chips/");
  

  GError *gdkError = NULL;
  fname = dname;
  fname.append("cursor_hand.png");
  this->handPixbuf = gdk_pixbuf_new_from_file(fname.c_str(), &gdkError);
  fname = dname;
  fname.append("cursor_drag.png");
  this->dragPixbuf = gdk_pixbuf_new_from_file(fname.c_str(), &gdkError);

}

//-------------------------------------------------------------------------
vtkGtkRenderWindowInteractor::~vtkGtkRenderWindowInteractor()
{
  this->Disable();

  if(this->ownWindow)
    {
    gtk_widget_destroy(GTK_WIDGET (this->window));
    }
  g_object_unref(this->handPixbuf);
  g_object_unref(this->dragPixbuf);

  this->window = NULL;
  this->drawingArea = NULL;
}

//-------------------------------------------------------------------------
void  vtkGtkRenderWindowInteractor::SetWindow(GtkWidget *window)
{
  this->window = window;
  this->ownWindow = false;
} 

//-------------------------------------------------------------------------
void  vtkGtkRenderWindowInteractor::SetDrawingArea(GtkWidget *drawingArea)
{
  this->drawingArea = drawingArea;
  this->ownDrawingArea = false;
}

//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractor::GetMousePosition(int *x, int *y)
{
  GdkModifierType state;
  gdk_window_get_pointer(this->window->window, x, y, &state);
}



//-------------------------------------------------------------------------
// This will start up the X event loop and never return. If you
// call this method it will loop processing X events until the
// application is exited.
void vtkGtkRenderWindowInteractor::Start()
{
  // Let the compositing handle the event loop if it wants to.
  if (this->HasObserver(vtkCommand::StartEvent) && !this->HandleEventLoop)
    {
    this->InvokeEvent(vtkCommand::StartEvent,NULL);
    return;
    }

  if (!this->Initialized)
    {
    this->Initialize();
    }
}

//-------------------------------------------------------------------------
// Initializes the event handlers
void vtkGtkRenderWindowInteractor::Initialize(GdkGLConfig *glconfig)
{
  vtkGtkRenderWindowInteractor::glconfig = glconfig;

  this->Initialize ();
}


//-------------------------------------------------------------------------
// Initializes the event handlers
void vtkGtkRenderWindowInteractor::Initialize()
{
  if (this->Initialized)
    {
    return;
    }

  // if don't have render window then we can't do anything yet
  if (!this->RenderWindow)
    {
    vtkErrorMacro(<< "vtkGtkRenderWindowInteractor::Initialize has no render window");
    return;
    }

  int *size = this->RenderWindow->GetSize();
  // enable everything and start rendering
  this->Enable();

  // set the size in the render window interactor
  this->Size[0] = size[0];
  this->Size[1] = size[1];
  if (this->Size[0] < 50) Size[0] = 50;
  if (this->Size[1] < 50) Size[1] = 50;

  // this is initialized
  this->Initialized = 1;

  // set up drawing area
  if (!this->drawingArea)
    {
    this->drawingArea = gtk_drawing_area_new ();
    this->ownDrawingArea = true;
    }

  if (vtkGtkRenderWindowInteractor::glconfig == NULL)
    {
    vtkGtkRenderWindowInteractor::glconfig = gdk_gl_config_new_by_mode (
                                        (GdkGLConfigMode)(GDK_GL_MODE_RGB  |
                                        GDK_GL_MODE_DEPTH  |
                                        GDK_GL_MODE_DOUBLE));

    if (vtkGtkRenderWindowInteractor::glconfig == NULL)
      {
      vtkErrorMacro(<< "vtkGtkRenderWindowInteractor::Cannot find the double-buffered visual. Trying single-buffered visual.");
      
      
      // Try single-buffered visual
      vtkGtkRenderWindowInteractor::glconfig = gdk_gl_config_new_by_mode (
                                           (GdkGLConfigMode)(GDK_GL_MODE_RGB |
                                           GDK_GL_MODE_DEPTH));
      if (vtkGtkRenderWindowInteractor::glconfig == NULL)
        {
        vtkErrorMacro(<< "vtkGtkRenderWindowInteractor::Cannot find the double-buffered visual. Trying single-buffered visual.");
        return;
        }
      }
    }

    gtk_widget_set_size_request (this->drawingArea, 50, 50);

  if (this->ownDrawingArea)
    gtk_widget_set_gl_capability (this->drawingArea,
                                 vtkGtkRenderWindowInteractor::glconfig,
                                 NULL,
                                 TRUE,
                                 GDK_GL_RGBA_TYPE);

  gtk_widget_get_can_focus(GTK_WIDGET(this->drawingArea));

  if (this->ownDrawingArea)
    gtk_container_add (GTK_CONTAINER (window), drawingArea);

//  gtk_widget_realize (this->drawingArea);
  gtk_widget_show (drawingArea);
  gtk_widget_show (window);

}

//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractor::Enable()
{
  // avoid cycles of calling Initialize() and Enable()
  if (this->Enabled)
    {
    return;
    }

  if (!this->drawingArea)
    {
    vtkErrorMacro(<< "vtkGtkRenderWindowInteractor::Calling Enable too early, before drawing area is set...");
    return;
    }

  // Add the event handler to the system.
  gtk_widget_add_events (this->drawingArea,
                         GDK_STRUCTURE_MASK |
                         GDK_POINTER_MOTION_MASK    |
                         GDK_POINTER_MOTION_HINT_MASK |
                         GDK_KEY_PRESS_MASK         |
                         GDK_KEY_RELEASE_MASK       |
                         GDK_BUTTON_PRESS_MASK      |
                         GDK_BUTTON_RELEASE_MASK    |
                         GDK_SCROLL                 |
                         GDK_ENTER_NOTIFY_MASK      |
                         GDK_LEAVE_NOTIFY_MASK      |
                         GDK_VISIBILITY_NOTIFY_MASK);

   g_signal_connect_after (G_OBJECT (this->drawingArea), "realize",
                          G_CALLBACK (vtkGtkRenderWindowInteractorRealizeCallback), this);

   g_signal_connect (G_OBJECT (this->drawingArea), "visibility_notify_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "expose_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "button_press_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "button_release_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "motion_notify_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "key_press_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "key_release_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "scroll_event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT(this->drawingArea), "configure_event",
                     G_CALLBACK(vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "enter-notify-event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);
   g_signal_connect (G_OBJECT (this->drawingArea), "leave-notify-event",
                     G_CALLBACK (vtkGtkRenderWindowInteractorCallback), this);



  // Setup for capturing the window deletion and configure
  gtk_widget_add_events (this->window, GDK_DELETE |
                                       GDK_CONFIGURE);

  this->Enabled = 1;

  this->Modified();
}

//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractor::Disable()
{
  if (!this->Enabled)
    {
    return;
    }

  this->Enabled = 0;

  this->Modified();
}

//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractor::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

  if (vtkGtkRenderWindowInteractor::glconfig)
    {
    os << indent << "GLConfig: " << this->glconfig << "\n";
    }
  else
    {
    os << indent << "GLConfig: (none)\n";
    }

  if (this->window)
    {
    os << indent << "window: " << this->window << "\n";
    }
  else
    {
    os << indent << "window: (none)\n";
    }

  if (vtkGtkRenderWindowInteractor::glconfig)
    {
    os << indent << "drawing area: " << this->drawingArea << "\n";
    }
  else
    {
    os << indent << "drawing area: (none)\n";
    }


  os << indent << "ownWindow: " 
     << (this->ownWindow ? "True\n" : "False\n");
  os << indent << "ownDrawingArea: "
     << (this->ownDrawingArea ? "True\n" : "False\n");
  os << indent << "shift: " << this->shift;
  os << indent << "control: " << this->control;

}

//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractor::UpdateSize(int x,int y)
{
  //Fixes initialization race condition where this may be called before on_realize in GtkWindow can set up RenderWindow
  //That set to this call isn't necessary, so it's OK to not update the RenderWindow
  if(!this->RenderWindow)
    return;


  // if the size changed send this on to the RenderWindow
  if ((x != this->Size[0])||(y != this->Size[1]))
    {

    this->Size[0] = x;
    this->Size[1] = y;

    this->RenderWindow->SetSize(x,y);
    }
}

//-------------------------------------------------------------------------
int vtkGtkRenderWindowInteractor::InternalCreateTimer(int timerId, int timerType, unsigned long duration)
{
  vtkGtkRenderWindowTimerData* timerData = new vtkGtkRenderWindowTimerData ();
  int platformId = g_timeout_add( duration, vtkGtkRenderWindowInteractorTimer, (void *)timerData);

  timerData->interactor = this;
  timerData->vtkTimerId = timerId;
  timerData->platformTimerId = platformId;
  timerMap[platformId] = timerData;

  return platformId;
}

//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractor::SetCurrentCursor (int shape)
{

  GdkCursor *cursor;

  switch (shape)
    {
            
    case VTK_CURSOR_SIZESW:
      cursor = gdk_cursor_new(GDK_BOTTOM_LEFT_CORNER);
      break;
    case VTK_CURSOR_SIZESE:
      cursor = gdk_cursor_new(GDK_BOTTOM_RIGHT_CORNER);
      break;
    case VTK_CURSOR_SIZENW:
      cursor = gdk_cursor_new(GDK_TOP_LEFT_CORNER);
      break;
    case VTK_CURSOR_SIZENE:
      cursor = gdk_cursor_new(GDK_TOP_RIGHT_CORNER);
      break;
    case VTK_CURSOR_SIZENS:
      cursor = gdk_cursor_new(GDK_SB_V_DOUBLE_ARROW);
      break;
    case VTK_CURSOR_SIZEWE:
      cursor = gdk_cursor_new(GDK_SB_H_DOUBLE_ARROW);
      break;
    case VTK_CURSOR_HAND:
      {
      GdkDisplay *display = gdk_display_get_default();
      cursor = gdk_cursor_new_from_pixbuf(display, this->handPixbuf, 8, 8);
      }
      break;
    case VTK_CURSOR_DRAG:
      {
      GdkDisplay *display = gdk_display_get_default();
      cursor = gdk_cursor_new_from_pixbuf(display, this->dragPixbuf, 8, 8);
      }
      break;
    case VTK_CURSOR_FLEUR:
      cursor = gdk_cursor_new(GDK_FLEUR);
      break;
    case VTK_CURSOR_ROTATE:
      cursor = gdk_cursor_new(GDK_EXCHANGE);
      break;
    case VTK_CURSOR_CROSSHAIR:
      cursor = gdk_cursor_new(GDK_CROSSHAIR);
      break;
    case VTK_CURSOR_DOUBLEARROW:
      cursor = gdk_cursor_new(GDK_DOUBLE_ARROW);
      break;
    case VTK_CURSOR_PENCIL:
      cursor = gdk_cursor_new(GDK_PENCIL);
      break;
    case VTK_CURSOR_XTERM:
      cursor = gdk_cursor_new(GDK_XTERM);
      break;
    case VTK_CURSOR_DEFAULT:
    default:
      cursor = gdk_cursor_new(GDK_LEFT_PTR);
      break;
    }

   gdk_window_set_cursor(window->window, cursor);

}

//-------------------------------------------------------------------------
int vtkGtkRenderWindowInteractor::InternalDestroyTimer(int platformTimerId) 
{
  g_source_remove (platformTimerId);
  this->InternalDestroyTimerData (platformTimerId);
  return 1;
}

//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractorRealizeCallback(GtkWidget   *widget,
                                                  gpointer     data )
{
   vtkGtkRenderWindowInteractor* local = static_cast <vtkGtkRenderWindowInteractor*>(data);

   if (!local->Enabled)
     return;

/*   gint width, height;
   gtk_widget_get_size_request (widget, &width, &height);
*/
   local->RenderWindow->SetWindowId((void*)GDK_WINDOW_XWINDOW(widget->window));

   // It's not clear that this needs to be here. It can provide results
   // that don't get updated appropriately A better place is in config?
/*   local->UpdateSize (width, height);
*/
}


//-------------------------------------------------------------------------
void vtkGtkRenderWindowInteractorCallback(GtkWidget *w,
                                        GdkEvent *event,
                                        gpointer     data )
{
  vtkGtkRenderWindowInteractor *local;
  int xp, yp;

  local = static_cast<vtkGtkRenderWindowInteractor *>(data);


  if (!local->Enabled)
    return;

  switch (event->type) 
    {
    case GDK_ENTER_NOTIFY:
      {
      local->InvokeEvent(vtkCommand::EnterEvent,NULL);
      local->Render();
      }
      break;
    case GDK_LEAVE_NOTIFY:
      {
      local->InvokeEvent(vtkCommand::LeaveEvent,NULL);
      local->Render();
      }
      break;
    case GDK_CONFIGURE:
      {
      GdkEventConfigure *evt = (GdkEventConfigure*)event;
      local->UpdateSize (evt->width, evt->height);
      }
      break;
    case GDK_EXPOSE:
      {
      local->InvokeEvent(vtkCommand::ExposeEvent,NULL);
      local->Render();
      }
      break;
    case GDK_MOTION_NOTIFY:
      {
      GdkEventMotion *evt = (GdkEventMotion*)event;

      xp = evt->x;
      yp = evt->y;

      int width, height;
      local->GetSize (width, height);

      local->SetEventInformation(xp,height-yp,local->shift,0,0,0);
      local->InvokeEvent(vtkCommand::MouseMoveEvent, NULL);
      }
      break;
    case GDK_VISIBILITY_NOTIFY:
      {
/*      GdkEventVisibility *evt = (GdkEventVisibility*)event;
*/
      }
      break;
    case GDK_SCROLL:
      {
      GdkEventScroll *evt = (GdkEventScroll*)event;
      xp = evt->x;
      yp = evt->y;

      int width, height;
      local->GetSize (width, height);

      local->SetEventInformation(xp,height-yp,local->control,local->shift,0,0,0);

      if (evt->direction == GDK_SCROLL_UP)
        local->InvokeEvent(vtkCommand::MouseWheelForwardEvent, NULL);
      else if (evt->direction == GDK_SCROLL_DOWN)
        local->InvokeEvent(vtkCommand::MouseWheelBackwardEvent, NULL);

      }
      break;
    case GDK_2BUTTON_PRESS:
      {
      GdkEventButton *evt = (GdkEventButton*)event;
      xp = evt->x;
      yp = evt->y;

      switch (evt->button)
        {
        case 1:
          local->InvokeEvent(vtkChipsCommand::LeftButtonDoubleClickEvent, NULL);
          break;
        case 2:
        case 3:
        default:
          vtkErrorWithObjectMacro(local, "vtkGtkRenderWindowInteractor::Unknown button press event");
          break;
        }
      }
      break;
    case GDK_BUTTON_PRESS:
      {
      GdkEventButton *evt = (GdkEventButton*)event;
      xp = evt->x;
      yp = evt->y;

      int width, height;
      local->GetSize (width, height);

      local->SetEventInformation(xp,height-yp,local->control,local->shift,0,0,0);

      switch (evt->button)
        {
        case 1:
          local->InvokeEvent(vtkCommand::LeftButtonPressEvent, NULL);
          break;
        case 2:
          local->InvokeEvent(vtkCommand::MiddleButtonPressEvent, NULL);
          break;
        case 3:
          local->InvokeEvent(vtkCommand::RightButtonPressEvent, NULL);
          break;
        default:
          vtkErrorWithObjectMacro(local, "vtkGtkRenderWindowInteractor::Unknown button press event");
          break;
        }
      }
      break;
    case GDK_BUTTON_RELEASE:
      {
      GdkEventButton *evt = (GdkEventButton*)event;

      xp = evt->x;
      yp = evt->y;

      int width, height;
      local->GetSize (width, height);

      local->SetEventInformation(xp,height-yp,local->control,local->shift,0,0,0);

      switch (evt->button)
        {
        case 1:
          local->InvokeEvent(vtkCommand::LeftButtonReleaseEvent, NULL);
          break;
        case 2:
          local->InvokeEvent(vtkCommand::MiddleButtonReleaseEvent, NULL);
          break;
        case 3:
          local->InvokeEvent(vtkCommand::RightButtonReleaseEvent, NULL);
          break;
        default:
          vtkErrorWithObjectMacro(local, << "vtkGtkRenderWindowInteractor::Unknown button release event");
          break;
        }
      }
      break;
    case GDK_KEY_PRESS:
      {
      GdkEventKey *evt = (GdkEventKey*)event;
      GdkModifierType state;

      switch(evt->keyval)
        {
        case GDK_Shift_L:
        case GDK_Shift_R:
          local->shift = 1;
          break;
        case GDK_Control_L:
        case GDK_Control_R:
          local->control = 1;
          break;
        default:
          break;
        }

      int width, height;
      local->GetSize (width, height);

      gdk_window_get_pointer(evt->window, &xp, &yp, &state);
      local->SetEventInformation(xp,height-yp,local->control,local->shift, vtkGtkRenderWindowInteractor::GtkKeyToVtk(evt->keyval),0,evt->string);
      local->InvokeEvent(vtkCommand::KeyPressEvent, NULL);
      local->InvokeEvent(vtkCommand::CharEvent, NULL);
      }
      break;
    case GDK_KEY_RELEASE:
      {
      GdkEventKey *evt = (GdkEventKey*)event;
      GdkModifierType state;

      switch(evt->keyval)
        {
        case GDK_Shift_L:
        case GDK_Shift_R:
          local->shift = 0;
          break;
        case GDK_Control_L:
        case GDK_Control_R:
          local->control = 0;
          break;
        default:
          break;
        }

      int width, height;
      local->GetSize (width, height);

      gdk_window_get_pointer(evt->window, &xp, &yp, &state);
      local->SetEventInformation(xp,height-yp,local->control,local->shift,vtkGtkRenderWindowInteractor::GtkKeyToVtk(evt->keyval),0,evt->string);
      local->InvokeEvent(vtkCommand::KeyReleaseEvent, NULL);
      }
      break;
    default:
      break;
    }
}

unsigned char vtkGtkRenderWindowInteractor::GtkKeyToVtk (unsigned int key)
{

  char vtkKey;
  switch (key)
    {
    case GDK_F1:
      vtkKey = vtkKeySym::F1;
    break;
    case GDK_F2:
      vtkKey = vtkKeySym::F2;
    break;
    case GDK_F3:
      vtkKey = vtkKeySym::F3;
    break;
    case GDK_F4:
      vtkKey = vtkKeySym::F4;
    break;
    case GDK_F5:
      vtkKey = vtkKeySym::F5;
    break;
    case GDK_F6:
      vtkKey = vtkKeySym::F6;
    break;
    case GDK_F7:
      vtkKey = vtkKeySym::F7;
    break;
    case GDK_F8:
      vtkKey = vtkKeySym::F8;
    break;
    case GDK_F9:
      vtkKey = vtkKeySym::F9;
    break;
    case GDK_F10:
      vtkKey = vtkKeySym::F10;
    break;
    case GDK_F11:
      vtkKey = vtkKeySym::F11;
    break;
    case GDK_F12:
      vtkKey = vtkKeySym::F12;
    break;
    case GDK_KP_Up:
    case GDK_Up:
      vtkKey = vtkKeySym::Up;
    break;
    case GDK_KP_Down:
    case GDK_Down:
      vtkKey = vtkKeySym::Down;
    break;
    case GDK_KP_Left:
    case GDK_Left:
      vtkKey = vtkKeySym::Left;
    break;
    case GDK_KP_Right:
    case GDK_Right:
      vtkKey = vtkKeySym::Right;
    break;
    case GDK_KP_Home:
    case GDK_Home:
      vtkKey = vtkKeySym::Home;
    break;
    case GDK_KP_End:
    case GDK_End:
      vtkKey = vtkKeySym::End;
    break;
    case GDK_KP_Page_Up:
    case GDK_Page_Up:
      vtkKey = vtkKeySym::PageUp;
    break;
    case GDK_KP_Page_Down:
    case GDK_Page_Down:
      vtkKey = vtkKeySym::PageDown;
    break;
    case GDK_Alt_L:
      vtkKey = vtkKeySym::LeftAlt;
    break;
    case GDK_Alt_R:
      vtkKey = vtkKeySym::RightAlt;
    break;
    case GDK_KP_Insert:
    case GDK_Insert:
      vtkKey = vtkKeySym::Insert;
    break;
    case GDK_KP_Tab:
    case GDK_Tab:
      vtkKey = vtkKeySym::Tab;
    break;
    case GDK_BackSpace:
      vtkKey = vtkKeySym::Backspace;
    break;
    case GDK_Escape:
      vtkKey = vtkKeySym::Esc;
    break;
    case GDK_Shift_R:
      vtkKey = vtkKeySym::RightShift;
    break;
    case GDK_Shift_L:
      vtkKey = vtkKeySym::LeftShift;
    break;
    case GDK_Control_R:
      vtkKey = vtkKeySym::RightControl;
    break;
    case GDK_Control_L:
      vtkKey = vtkKeySym::LeftControl;
    break;
    case GDK_KP_Space:
    case GDK_space:
      vtkKey = vtkKeySym::Space;
    break;
    case GDK_exclam:
      vtkKey = vtkKeySym::Exclamation;
    break;
    case GDK_quotedbl:
      vtkKey = vtkKeySym::DoubleQuote;
    break;
    case GDK_numbersign:
      vtkKey = vtkKeySym::NumberSign;
    break;
    case GDK_dollar:
      vtkKey = vtkKeySym::DollarSign;
    break;
    case GDK_percent:
      vtkKey = vtkKeySym::Percent;
    break;
    case GDK_ampersand:
      vtkKey = vtkKeySym::Ampersand;
    break;
    case GDK_apostrophe:
      vtkKey = vtkKeySym::Apostrophe;
    break;
    case GDK_parenleft:
      vtkKey = vtkKeySym::OpenParen;
    break;
    case GDK_parenright:
      vtkKey = vtkKeySym::CloseParen;
    break;
    case GDK_asterisk:
      vtkKey = vtkKeySym::Astrisk;
    break;
    case GDK_plus:
    case GDK_KP_Add:
      vtkKey = vtkKeySym::Plus;
    break;
    case GDK_comma:
      vtkKey = vtkKeySym::Comma;
    break;
    case GDK_minus:
    case GDK_KP_Subtract:
      vtkKey = vtkKeySym::Minus;
    break;
    case GDK_period:
      vtkKey = vtkKeySym::Period;
    break;
    case GDK_slash:
      vtkKey = vtkKeySym::Slash;
    break;
    case GDK_0:
      vtkKey = vtkKeySym::Zero;
    break;
    case GDK_1:
      vtkKey = vtkKeySym::One;
    break;
    case GDK_2:
      vtkKey = vtkKeySym::Two;
    break;
    case GDK_3:
      vtkKey = vtkKeySym::Three;
    break;
    case GDK_4:
      vtkKey = vtkKeySym::Four;
    break;
    case GDK_5:
      vtkKey = vtkKeySym::Five;
    break;
    case GDK_6:
      vtkKey = vtkKeySym::Six;
    break;
    case GDK_7:
      vtkKey = vtkKeySym::Seven;
    break;
    case GDK_8:
      vtkKey = vtkKeySym::Eight;
    break;
    case GDK_9:
      vtkKey = vtkKeySym::Nine;
    break;
    case GDK_colon:
      vtkKey = vtkKeySym::Colon;
    break;
    case GDK_semicolon:
      vtkKey = vtkKeySym::SemiColon;
    break;
    case GDK_less:
      vtkKey = vtkKeySym::LessThan;
    break;
    case GDK_equal:
      vtkKey = vtkKeySym::Equals;
    break;
    case GDK_greater:
      vtkKey = vtkKeySym::GreaterThan;
    break;
    case GDK_question:;
      vtkKey = vtkKeySym::QuestionMark;
    break;
    case GDK_at:
      vtkKey = vtkKeySym::At;
    break;
    case GDK_A:
      vtkKey = vtkKeySym::A;
    break;
    case GDK_B:
      vtkKey = vtkKeySym::B;
    break;
    case GDK_C:
      vtkKey = vtkKeySym::C;
    break;
    case GDK_D:
      vtkKey = vtkKeySym::D;
    break;
    case GDK_E:
      vtkKey = vtkKeySym::E;
    break;
    case GDK_F:
      vtkKey = vtkKeySym::F;
    break;
    case GDK_G:
      vtkKey = vtkKeySym::G;
    break;
    case GDK_H:
      vtkKey = vtkKeySym::H;
    break;
    case GDK_I:
      vtkKey = vtkKeySym::I;
    break;
    case GDK_J:
      vtkKey = vtkKeySym::J;
    break;
    case GDK_K:
      vtkKey = vtkKeySym::K;
    break;
    case GDK_L:
      vtkKey = vtkKeySym::L;
    break;
    case GDK_M:
      vtkKey = vtkKeySym::M;
    break;
    case GDK_N:
      vtkKey = vtkKeySym::N;
    break;
    case GDK_O:
      vtkKey = vtkKeySym::O;
    break;
    case GDK_P:
      vtkKey = vtkKeySym::P;
    break;
    case GDK_Q:
      vtkKey = vtkKeySym::Q;
    break;
    case GDK_R:
      vtkKey = vtkKeySym::R;
    break;
    case GDK_S:
      vtkKey = vtkKeySym::S;
    break;
    case GDK_T:
      vtkKey = vtkKeySym::T;
    break;
    case GDK_U:
      vtkKey = vtkKeySym::U;
    break;
    case GDK_V:
      vtkKey = vtkKeySym::V;
    break;
    case GDK_W:
      vtkKey = vtkKeySym::W;
    break;
    case GDK_X:
      vtkKey = vtkKeySym::X;
    break;
    case GDK_Y:
      vtkKey = vtkKeySym::Y;
    break;
    case GDK_Z:
      vtkKey = vtkKeySym::Z;
    break;
    case GDK_bracketleft:
      vtkKey = vtkKeySym::OpenBracket;
    break;
    case GDK_backslash:
      vtkKey = vtkKeySym::BackSlash;
    break;
    case GDK_bracketright:
      vtkKey = vtkKeySym::CloseBracket;
    break;
    case GDK_asciicircum:
      vtkKey = vtkKeySym::Caret;
    break;
    case GDK_underscore:
      vtkKey = vtkKeySym::Underscore;
    break;
    case GDK_quoteleft:
      vtkKey = vtkKeySym::SingleLeftQuote;
    break;
    case GDK_a:
      vtkKey = vtkKeySym::a;
    break;
    case GDK_b:
      vtkKey = vtkKeySym::b;
    break;
    case GDK_c:
      vtkKey = vtkKeySym::c;
    break;
    case GDK_d:
      vtkKey = vtkKeySym::d;
    break;
    case GDK_e:
      vtkKey = vtkKeySym::e;
    break;
    case GDK_f:
      vtkKey = vtkKeySym::f;
    break;
    case GDK_g:
      vtkKey = vtkKeySym::g;
    break;
    case GDK_h:
      vtkKey = vtkKeySym::h;
    break;
    case GDK_i:
      vtkKey = vtkKeySym::i;
    break;
    case GDK_j:
      vtkKey = vtkKeySym::j;
    break;
    case GDK_k:
      vtkKey = vtkKeySym::k;
    break;
    case GDK_l:
      vtkKey = vtkKeySym::l;   
    break;
    case GDK_m:
      vtkKey = vtkKeySym::m;
    break;
    case GDK_n:
      vtkKey = vtkKeySym::n;
    break;
    case GDK_o:
      vtkKey = vtkKeySym::o;
    break;
    case GDK_p:
      vtkKey = vtkKeySym::p;
    break;
    case GDK_q:
      vtkKey = vtkKeySym::q;
    break;
    case GDK_r:
      vtkKey = vtkKeySym::r;
    break;
    case GDK_s:
      vtkKey = vtkKeySym::s;
    break;
    case GDK_t:
      vtkKey = vtkKeySym::t;
    break;
    case GDK_u:
      vtkKey = vtkKeySym::u;
    break;
    case GDK_v:
      vtkKey = vtkKeySym::v;
    break;
    case GDK_w:
      vtkKey = vtkKeySym::w;
    break;
    case GDK_x:
      vtkKey = vtkKeySym::x;
    break;
    case GDK_y:
      vtkKey = vtkKeySym::y;
    break;
    case GDK_z:
      vtkKey = vtkKeySym::z;
    break;
    case GDK_braceleft:
      vtkKey = vtkKeySym::OpenBrace;
    break;
    case GDK_bar:
      vtkKey = vtkKeySym::Bar;
    break;
    case GDK_braceright:
      vtkKey = vtkKeySym::CloseBrace;
    break;
    case GDK_asciitilde:
      vtkKey = vtkKeySym::Tilde;
    break;
    case GDK_Delete:
    case GDK_KP_Delete:
      vtkKey = vtkKeySym::Delete;
    break;

    }

  return vtkKey;
}

void vtkGtkRenderWindowInteractor::InternalDestroyTimerData (unsigned int platformTimerId)
{
  if (timerMap.count (platformTimerId) )
    {
    delete timerMap[platformTimerId];
    timerMap.erase (platformTimerId);
    }
}


int vtkGtkRenderWindowInteractorTimer (gpointer data)
{
  vtkGtkRenderWindowTimerData* timerData = static_cast <vtkGtkRenderWindowTimerData *>(data);

  if (!timerData->interactor->Enabled)
    {
    return 0;
    }

  timerData->interactor->InvokeEvent(vtkCommand::TimerEvent, &(timerData->vtkTimerId));

  timerData->interactor->InternalDestroyTimerData (timerData->platformTimerId);

  

  return 0;
}

