/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkGtkRenderWindowInteractor.h,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkGtkRenderWindowInteractor - an Gtk event driven interface for a RenderWindow
// .SECTION Description
// vtkGtkRenderWindowInteractor is a convenience object that provides event
// bindings to gtk event handler.
//
//  **** NOTE: This interactor has NOT been fully tested. Only the shell
//   is being used a basic interactor for chips. The file was setup as
//   a potential infrastructure if we wanted go with the full gtk interactor.
//   Undefine USE_GTK_SHELL to turn on the full capability.
//
// .SECTION see also
// vtkRenderWindowInteractor vtkGtkRenderWindow

#ifndef __vtkGtkRenderWindowInteractor_h
#define __vtkGtkRenderWindowInteractor_h

#define USE_GTK_SHELL

//===========================================================
// now we define the C++ class

#include "vtkRenderWindowInteractor.h"
#undef VERSION
#include <gtk/gtk.h>
#include <gtk/gtkgl.h>
#include <gdk/gdkglconfig.h>
#include <gdk/gdkx.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <map>

class vtkCallbackCommand;
class vtkGtkRenderWindowTimerData;


//BTX
// Forward declare internal friend functions.
void vtkGtkRenderWindowInteractorCallback(GtkWidget*, GdkEvent*, gpointer);
void vtkGtkRenderWindowInteractorRealizeCallback(GtkWidget*, gpointer);
int vtkGtkRenderWindowInteractorTimer (gpointer);
//ETX

#define VTK_CURSOR_ROTATE        100
#define VTK_CURSOR_DRAG          101
#define VTK_CURSOR_FLEUR         102
#define VTK_CURSOR_DOUBLEARROW   103
#define VTK_CURSOR_PENCIL        104
#define VTK_CURSOR_XTERM         105

class VTK_RENDERING_EXPORT vtkGtkRenderWindowInteractor : public vtkRenderWindowInteractor
{
public:
  static vtkGtkRenderWindowInteractor *New();
  vtkTypeRevisionMacro(vtkGtkRenderWindowInteractor,vtkRenderWindowInteractor);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Initializes the event handlers without a gtk drawing area. 
  virtual void Initialize();
  void Initialize(GdkGLConfig *glconfig);

  // Description:
  // Call exit on 'q','e' keypress. Want more ???
  void TerminateApp(void) { exit(0); }

  // Description:
  // Enable/Disable interactions.  By default interactors are enabled when
  // initialized.  Initialize() must be called prior to enabling/disabling
  // interaction. 
  virtual void Enable();
  virtual void Disable();


  // Description:
  // This will start up the gtk event loop and never return. If you
  // call this method it will loop processing gtk events until the
  // application is exited.
  virtual void Start();

  // Description:
  // Update the Size data member and set the associated RenderWindow's
  // size.
  virtual void UpdateSize(int,int);

  // Description:
  // Set the gtk window. This removes ownership of the window
  virtual void SetWindow(GtkWidget *);
  GtkWidget *GetWindow() {return this->window;};

  // Description:
  // Set the gtk window. This removes ownership of the window
  virtual void SetDrawingArea(GtkWidget *);
  GtkWidget *GetDrawingArea() {return this->drawingArea;};
  
  // Description:
  // Re-defines virtual function to get mouse position
  virtual void GetMousePosition(int *x, int *y); 

  // Description:
  // Set the current cursor to the specified shape
  virtual void SetCurrentCursor (int shape);

  // Description:
  // Functions that are used internally.
  friend void vtkGtkRenderWindowInteractorCallback(GtkWidget*, GdkEvent*, gpointer);
  friend void vtkGtkRenderWindowInteractorRealizeCallback(GtkWidget*, gpointer);
  friend int vtkGtkRenderWindowInteractorTimer (gpointer);

protected:
  vtkGtkRenderWindowInteractor();
  ~vtkGtkRenderWindowInteractor();

  //Using static here to avoid detroying context when many apps are open:
  static GdkGLConfig *glconfig;

  GtkWidget *window;
  GtkWidget *drawingArea;

  GdkPixbuf *handPixbuf;
  GdkPixbuf *dragPixbuf;


  int shift;
  int control;

  bool ownWindow;
  bool ownDrawingArea;

  // Description: 
  // GTK-specific internal timer methods. See the superclass for detailed
  // documentation.
  virtual int InternalCreateTimer(int timerId, int timerType, unsigned long duration);
  virtual int InternalDestroyTimer(int platformTimerId);

  void InternalDestroyTimerData (unsigned int platformTimerId);


  static unsigned char GtkKeyToVtk (unsigned int key);

  std::map<unsigned int, vtkGtkRenderWindowTimerData*> timerMap;


private:
  vtkGtkRenderWindowInteractor(const vtkGtkRenderWindowInteractor&);  // Not implemented.
  void operator=(const vtkGtkRenderWindowInteractor&);  // Not implemented.
};

#endif
