/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkInteractorStyleChips.h,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
// .NAME vtkInteractorStyleChips - class to set interactive style for chips 
//  plotting package
// .SECTION Description
// The class vtkInteractorStyleChips 
// .SECTION See Also
// vtkInteractorStyleJoystickActor vtkInteractorStyleJoystickCamera
// vtkInteractorStyleTrackballActor vtkInteractorStyleTrackballCamera

#ifndef __vtkInteractorStyleChips_h
#define __vtkInteractorStyleChips_h

#include "vtkInteractorStyle.h"
/*#include <vtkstd/vector>
*/

class vtkAreaPicker;
/*
class vtkPolyData;
class vtkPolyDataMapper2D;
class vtkActor2D;
class vtkProperty2D;
*/

class vtkInteractorEventObserver;
class vtkScrollCountCallback;
class vtkScrollCountResetCallback;


class VTK_RENDERING_EXPORT vtkInteractorStyleChips : public vtkInteractorStyle
{
public:
  static vtkInteractorStyleChips *New();
  vtkTypeRevisionMacro(vtkInteractorStyleChips, vtkInteractorStyle);
  void PrintSelf(ostream& os, vtkIndent indent);

  virtual void OnChar();
  virtual void OnKeyPress();
  virtual void OnKeyRelease();
  virtual void OnEnter();
  virtual void OnLeave();
  virtual void OnLeftButtonDown();
  virtual void OnLeftButtonUp();
  virtual void OnMiddleButtonDown();
  virtual void OnMiddleButtonUp();
  virtual void OnMouseWheelForward ();
  virtual void OnMouseWheelBackward ();
  virtual void OnMouseMove();
  virtual void OnRightButtonDown();
  virtual void OnRightButtonUp();
  virtual void OnUser();

  // Description:
  // Specify the properties of the border.
/*  vtkGetObjectMacro(AnnotationProperty,vtkProperty2D);
*/

  friend class vtkInteractorEventObserver;
  friend class vtkScrollCountCallback;
  friend class vtkScrollCountResetCallback;
 
protected:
  vtkInteractorStyleChips();
  ~vtkInteractorStyleChips();

  int Tolerance;
  bool LeftMousePress;
/*
  vtkPolyData                *AnnotationPolyData;
  vtkPolyDataMapper2D        *AnnotationMapper;
  vtkActor2D                 *AnnotationActor;
  vtkProperty2D              *AnnotationProperty;
*/
  double EventPosition[2];
/*
  vtkstd::vector<double> AnnotationPoints;
*/
  vtkInteractorEventObserver *eventObserver;

  int ScrollCount;
  int MaxScrollCount;
  vtkScrollCountCallback *scrollCallback;
  vtkScrollCountResetCallback *scrollResetCallback;
  bool scrollCallbackAdded;
  bool resetScrollCallback;
  
  

  

private:
  vtkInteractorStyleChips(const vtkInteractorStyleChips&);  // Not implemented.
  void operator=(const vtkInteractorStyleChips&);  // Not implemented.

};

#endif
