#include "vtkPolygonRepresentation.hh"
#include "vtkPolygonDataActor2DExtra.hh"

#include "vtkRenderedAreaPicker.h"
#include "vtkObjectFactory.h"
#include "vtkRenderer.h"
#include "vtkTransform.h"
#include "vtkAssemblyPath.h"
#include "vtkProp3DCollection.h"
#include "vtkMath.h"

#include <float.h>


vtkStandardNewMacro(vtkPolygonRepresentation);

//-------------------------------------------------------------------------
vtkPolygonRepresentation::vtkPolygonRepresentation()
{
}

//-------------------------------------------------------------------------
vtkPolygonRepresentation::~vtkPolygonRepresentation()
{
}


//-------------------------------------------------------------------------
void vtkPolygonRepresentation::NegotiateLayout()
{
  vtkPolygonDataActor2DExtra* PolygonActor = reinterpret_cast<vtkPolygonDataActor2DExtra*>(this->PickProp);
  if (!PolygonActor)
    return;

//  PolygonActor->BuildData (this->Renderer);

  double *pos1 = PolygonActor->GetPositionCoordinate ()->GetComputedDoubleViewportValue(this->Renderer);
  double *pos2 = PolygonActor->GetPosition2Coordinate ()->GetComputedDoubleViewportValue(this->Renderer);
  double clipBounds[4];

  if (PolygonActor->GetClipData ())
    {
    PolygonActor->GetClipBounds (clipBounds);
    vtkCoordinate *clipCoordinate1 = vtkCoordinate::New ();
    vtkCoordinate *clipCoordinate2 = vtkCoordinate::New ();
    double *clipPos1;
    double *clipPos2;

    clipCoordinate1->SetCoordinateSystem (PolygonActor->GetPositionCoordinate ()->GetCoordinateSystem ());
    clipCoordinate2->SetCoordinateSystem (PolygonActor->GetPositionCoordinate ()->GetCoordinateSystem ());
    clipCoordinate1->SetValue (clipBounds[0], clipBounds[1]);
    clipCoordinate2->SetValue (clipBounds[2], clipBounds[3]);

    clipPos1 = clipCoordinate1->GetComputedDoubleViewportValue(this->Renderer);
    clipPos2 = clipCoordinate2->GetComputedDoubleViewportValue(this->Renderer);

    if (clipPos1[0] > pos1[0])
      pos1[0] = clipPos1[0];
    if (clipPos1[1] > pos1[1])
      pos1[1] = clipPos1[1];

    if (clipPos1[0] > pos2[0])
      pos2[0] = clipPos1[0];
    if (clipPos1[1] > pos2[1])
      pos2[1] = clipPos1[1];

    if (clipPos2[0] < pos2[0])
      pos2[0] = clipPos2[0];
    if (clipPos2[1] < pos2[1])
      pos2[1] = clipPos2[1];

    if (clipPos2[0] < pos1[0])
      pos1[0] = clipPos2[0];
    if (clipPos2[1] < pos1[1])
      pos1[1] = clipPos2[1];

    clipCoordinate1->Delete(); 
    clipCoordinate2->Delete(); 
    }

  this->PositionCoordinate->SetCoordinateSystemToViewport ();
  this->PositionCoordinate->SetValue(pos1[0], pos1[1]);
  this->Position2Coordinate->SetCoordinateSystemToViewport ();
  this->Position2Coordinate->SetValue(pos2[0]-pos1[0], pos2[1]-pos1[1]);

}

//-------------------------------------------------------------------------
void vtkPolygonRepresentation::WidgetInteraction(double eventPos[2])
{
  double dx = eventPos[0] - this->CurrentEventPosition[0];
  double dy = eventPos[1] - this->CurrentEventPosition[1];

  if (this->Resizing)
    {
/*
    double *pos1 = PolygonActor->GetPositionCoordinate ()->GetComputedDoubleViewportValue(this->Renderer);
    double *pos2 = PolygonActor->GetPosition2Coordinate ()->GetComputedDoubleViewportValue(this->Renderer);
    switch (this->InteractionState)
      {
      case vtkChipsRepresentation::RightHandle:
        pos1[0] += dx;
      break;
      case vtkChipsRepresentation::LeftHandle:
        pos1[0] -= dx;
      break;
      case vtkChipsRepresentation::TopHandle:
        pos1[0] += dy;
      break;
      case vtkChipsRepresentation::BottomHandle:
        pos1[0] -= dy;
      break;
      case vtkChipsRepresentation::TopRightHandle:
        pos1[0] += (dx>dy) ? dx : dy;
      break;
      case vtkChipsRepresentation::TopLeftHandle:
        pos1[0] += (dx>dy) ? -dx : dy;
      break;
      case vtkChipsRepresentation::BottomRightHandle:
        pos1[0] += (dx>dy) ? dx : -dy;
      break;
      case vtkChipsRepresentation::BottomLeftHandle:
        pos1[0] -= (dx>dy) ? dx : dy;
      break;
      default:
      break;
      }
    this->Update ();
*/
    }
  else if (this->Rotating)
    {
    float angle = 0;
    float startAngle = 0;
    double anchor[2];

    double *fpos1 = this->PositionCoordinate->GetComputedDoubleDisplayValue (this->Renderer);
    double *fpos2 = this->Position2Coordinate->GetComputedDoubleDisplayValue (this->Renderer);


    anchor[0] =  (fpos1[0] + fpos2[0])/2.;
    anchor[1] =  (fpos1[1] + fpos2[1])/2.;

    dx = this->CurrentEventPosition[0] - anchor[0];
    dy = this->CurrentEventPosition[1] - anchor[1];

    if ((dx != 0) && (dy != 0))
      startAngle = atan2 (dy,dx);

    if (startAngle < 0)
      startAngle += vtkMath::DoubleTwoPi();

    dx = eventPos[0] - anchor[0];
    dy = eventPos[1] - anchor[1];

    if ((dx != 0) && (dy != 0))
      angle = atan2 (dy,dx);

    if (angle < 0)
       angle += vtkMath::DoubleTwoPi();

    this->Angle += vtkMath::DegreesFromRadians (angle - startAngle);

    if (this->Angle < 0)
       this->Angle += 360;
    else if (this->Angle >= 360)
       this->Angle -= 360;
    }

  this->CurrentEventPosition[0] = eventPos[0];
  this->CurrentEventPosition[1] = eventPos[1];


}

void vtkPolygonRepresentation::SetPolygonActor (vtkPolygonDataActor2DExtra *actor)
{
  this->SetPickProp (actor);
}

vtkPolygonDataActor2DExtra * vtkPolygonRepresentation::GetPolygonActor ()
{
  return reinterpret_cast<vtkPolygonDataActor2DExtra*>(this->GetPickProp ());
}
  
void vtkPolygonRepresentation::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}


