#include <iostream>
#include <math.h>


#include "freetype/t1tables.h"
#include  "FTFontExtra.hh"
#include  "FTGlyph.h" 



#include  "FTFace.h"
#include  "FTFont.h"
#include  "FTGlyphContainer.h"

#ifdef FTGL_USE_NAMESPACE
namespace ftgl
{
#endif


//using namespace std;

FTFontExtra::FTFontExtra()
{
  angleRad=0;
}


FTFontExtra::~FTFontExtra()
{

}


void FTFontExtra::renderExtra( double inAngleRad,const char* string, 
                     const FTGLRenderContext *context)
{
  const unsigned char* c = (unsigned char*)string;
  pen.x = 0; pen.y = 0;
  angleRad = inAngleRad;
  tmpPen[0]=0;tmpPen[1]=0;
  while( *c)
  {
    doRenderExtra( *c, *(c + 1), context);
    ++c;
  }

  angleRad=0;
}


void FTFontExtra::renderExtra( double inAngleRad,const wchar_t* string, 
                     const FTGLRenderContext *context)
{
  const wchar_t* c = string;
  pen.x = 0; pen.y = 0;
  tmpPen[0]=0;tmpPen[1]=0;
  angleRad = inAngleRad;

  while( *c)
  {
    doRenderExtra( *c, *(c + 1), context);
    ++c;
  }

  angleRad=0;

}


void FTFontExtra::doRenderExtra( const unsigned int chr, 
				 const unsigned int nextChr, 
				 const FTGLRenderContext *context)
{
  FTGlyph* glyph;
  unsigned int g = face.CharIndex( chr);
  glyph = MakeGlyph( g);
  

  //  FT_Vector kernAdvance = glyphList->render( chr, nextChr, pen, context);
  glyph->Render(pen,context);
  FT_GlyphSlot slot = (*face.Face())->glyph;
  tmpPen[0]+=slot->linearHoriAdvance/65536.0*cos(angleRad);
    //slot->advance.x/64.0;
  tmpPen[1]+=slot->linearHoriAdvance/65536.0*sin(angleRad);
    //slot->advance.y/64.0;

  //  pen.x += slot->advance.x >> 6;
  //  pen.y += slot->advance.y >> 6;
  pen.x=tmpPen[0];
  pen.y=tmpPen[1];

  delete glyph;
}


float FTFontExtra::Advance(const char* string)
{

  const unsigned char* c = (unsigned char*)string;
  pen.x = 0; pen.y = 0;
  angleRad = 0;
  float advance=0;
  FT_GlyphSlot slot;
  FTGlyph* glyph;

  while( *c)
  {
    unsigned int g = face.CharIndex( *c);
    glyph = MakeGlyph( g);
    
    
    slot = (*face.Face())->glyph;
    
    advance+= slot->linearHoriAdvance/65536.0;
    //slot->advance.x/64.0;
    ++c;
    delete glyph;
  }

  return advance;
};

float FTFontExtra::Advance(const wchar_t* string)
{

  const wchar_t* c = string;
  pen.x = 0; pen.y = 0;
  angleRad = 0;
  float advance=0;
  FT_GlyphSlot slot;
  FTGlyph* glyph;

  while( *c)
  {
    unsigned int g = face.CharIndex( *c);
    glyph=MakeGlyph( g);
    
    
    slot = (*face.Face())->glyph;
    
    advance+= slot->linearHoriAdvance/65536.0;
    //slot->advance.x/64.0;
    ++c;
    delete glyph;
  }

  return advance;
};



void FTFontExtra::BBox( const char* string,
		   float& llx, float& lly, 
		   float& llz, float& urx, float& ury, float& urz)
{


  const unsigned char* c = (unsigned char*)string;
  llx = lly = llz = urx = ury = urz = 0;
  FTBBox bbox;
  FTGlyph* glyph;
  FT_GlyphSlot slot;
  bool tmpFlag=false;
  
  while( *c)
    {
      unsigned int g = face.CharIndex( static_cast<unsigned int>(*c));
      glyph=MakeGlyph( g);
      slot = (*face.Face())->glyph;

      bbox = glyph->BBox();
      
      if ( !tmpFlag)
	{
	  tmpFlag=true;
	  llx = glyph->BBox().x1;
	};

      // Lower extent
      lly = lly < bbox.y1 ? lly: bbox.y1;
      // Upper extent
      ury = ury > bbox.y2 ? ury: bbox.y2;
      // Depth
      urz = urz < bbox.z2 ? urz: bbox.z2;
      
      // Width
      urx += slot->linearHoriAdvance/65536.0;
	//glyph->Advance();
      ++c;
      delete glyph;
    }

  //Final adjustments
  urx -= slot->linearHoriAdvance/65536.0;
  //glyph->Advance();
  urx += bbox.x2;


};
void FTFontExtra::BBox( const wchar_t* string,
		   float& llx, float& lly, 
		   float& llz, float& urx, float& ury, float& urz)
{


  const  wchar_t* c = ( wchar_t*)string;
  llx = lly = llz = urx = ury = urz = 0;
  FTBBox bbox;
  FTGlyph* glyph; 
  FT_GlyphSlot slot;
  bool tmpFlag=false;
  

  while( *c)
    {
      unsigned int g = face.CharIndex( static_cast<unsigned int>(*c));
      glyph= MakeGlyph( g);
      slot = (*face.Face())->glyph;

      bbox = glyph->BBox();

      if ( !tmpFlag)
	{
	  tmpFlag=true;
	  llx = glyph->BBox().x1;
	};

      // Lower extent
      lly = lly < bbox.y1 ? lly: bbox.y1;
      // Upper extent
      ury = ury > bbox.y2 ? ury: bbox.y2;
      // Depth
      urz = urz < bbox.z2 ? urz: bbox.z2;
      
      // Width
      urx += slot->linearHoriAdvance/65536.0;
      //glyph->Advance();
      ++c;
      delete glyph;
    }

  //Final adjustments
  urx -= slot->linearHoriAdvance/65536.0;
  //glyph->Advance();
  urx += bbox.x2;


};

float FTFontExtra::ItalicAngle()
{
  PS_FontInfoRec tmpInfoRec; 
  if ( ! FT_Get_PS_Font_Info((*face.Face()),&tmpInfoRec))
    {
      return tmpInfoRec.italic_angle;
    };
  
  return 0.0;
};

#ifdef FTGL_USE_NAMESPACE
} // namespace ftgl
#endif

