// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkAxisActor2DExtra.cxx,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#include <float.h>

#include <math.h>
#include <cctype>

#include "vtkAxisActor2DExtra.hh"

#include "vtkCellArray.h"
#include "vtkObjectFactory.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkTextMapper.h"
#include "vtkTextProperty.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkProperty2D.h"
#include "vtkTextActorExtra.hh"


#ifndef PI
#define PI 3.14159265359
#endif

#ifndef MINLOG
#define MINLOG 0.01
#endif


vtkCxxRevisionMacro(vtkAxisActor2DExtra, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkAxisActor2DExtra);

vtkCxxSetObjectMacro(vtkAxisActor2DExtra,LabelTextProperty,vtkTextProperty);


//----------------------------------------------------------------------------
// Instantiate this object.
vtkAxisActor2DExtra::vtkAxisActor2DExtra()
{
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(0.0, 0.0);

  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(0.75, 0.0);
  this->Position2Coordinate->SetReferenceCoordinate(NULL);
  
  this->NumberOfMajTicks = 5;
  this->NumberOfMinTicks = 5;

  this->ShowLimitsTicks=1;
  this->MajTicksLength = 5;
  this->MinTicksLength = 2;
  this->MajTickOffset = 0;
  this->MinTickOffset = 0;

  this->MinLogLimit=MINLOG;
  this->Scale[0] =0;
  this->Scale[1] =0;
  this->TitleFlip=0;
  this->LabelsFlip=0;
  this->MajorTicksFlip=0;
  this->MinorTicksFlip=0;

  this->Range[0] = 0.0;
  this->Range[1] = 1.0;
  this->Range[2] = 0.0;
  this->Range[3] = 1.0;
  this->AdjustedRange[0] = 0.0;
  this->AdjustedRange[1] = 1.0;
  this->AdjustedRange[2] = 0.0;
  this->AdjustedRange[3] = 1.0;
  this->RangeIndex = 0;

  

  this->MajTicksInterval = (this->Range[0]+this->Range[1])/(this->NumberOfMajTicks-1) ;
  this->MinTicksInterval = this->MajTicksInterval/(this->NumberOfMinTicks+1);



  this->LabelOffset=2+this->MajTicksLength+this->MajTickOffset;

  this->LabelTextProperty = vtkTextProperty::New();
  this->LabelTextProperty->SetBold(1);
  this->LabelTextProperty->SetItalic(1);
  this->LabelTextProperty->SetShadow(0);
  this->LabelTextProperty->SetFontFamilyToArial();



  this->LabelFormat = new char[8]; 
  sprintf(this->LabelFormat,"%s","%-#6.3g");

  this->TitleActor = vtkTextActorExtra::New();
  this->TitleActor->GetTextProperty()->ShallowCopy(this->LabelTextProperty);

#ifdef NOCLIP
  this->ClipData = false;
#else
  this->ClipData = true;

  for(int indx=0;indx<4;indx++)
    this->ClipPlanes[indx]=vtkPlane::New();

  this->ClipPlanes[0]->SetOrigin(0,0,0);
  this->ClipPlanes[0]->SetNormal(1,0,0);

  this->ClipPlanes[1]->SetOrigin(1,0,0);
  this->ClipPlanes[1]->SetNormal(-1,0,0);

  this->ClipPlanes[2]->SetOrigin(0,0,0);
  this->ClipPlanes[2]->SetNormal(0,1,0);

  this->ClipPlanes[3]->SetOrigin(0,1,0);
  this->ClipPlanes[3]->SetNormal(0,-1,0);
#endif
  this->ClipLocation[0] = vtkCoordinate::New ();
  this->ClipLocation[1] = vtkCoordinate::New ();

  this->ClipLocation[0]->SetCoordinateSystemToNormalizedViewport();
  this->ClipLocation[1]->SetCoordinateSystemToNormalizedViewport();
  this->ClipLocation[0]->SetValue(0.0,1.0);
  this->ClipLocation[1]->SetValue(0.0,1.0);


  this->Axis = vtkPolyData::New();
  this->AxisMapper = vtkPolyDataMapper2D::New();
  this->AxisMapper->SetInput(this->Axis);
  this->AxisActor = vtkActor2D::New();
  this->AxisActor->SetMapper(this->AxisMapper);

#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->AxisMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif  

  this->MajTicks = vtkPolyData::New();
  this->MajTicksMapper = vtkPolyDataMapper2D::New();
  this->MajTicksMapper->SetInput(this->MajTicks);
  this->MajTicksActor = vtkActor2D::New();
  this->MajTicksActor->SetMapper(this->MajTicksMapper);

#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->MajTicksMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif  

  this->MinTicks = vtkPolyData::New();
  this->MinTicksMapper = vtkPolyDataMapper2D::New();
  this->MinTicksMapper->SetInput(this->MinTicks);
  this->MinTicksActor = vtkActor2D::New();
  this->MinTicksActor->SetMapper(this->MinTicksMapper);

#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->MinTicksMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif

  this->MajTickLabelVertAlignmentOffset=1.0;
  this->MajTickLabelHorizAlignmentOffset=.5;


  this->MajGrid = vtkPolyData::New();
  this->MajGridMapper = vtkPolyDataMapper2D::New();
  this->MajGridMapper->SetInput(this->MajGrid);
  this->MajGridActor = vtkActor2D::New();
  this->MajGridActor->SetMapper(this->MajGridMapper);

#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->MajGridMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif
  

  this->MinGrid = vtkPolyData::New();
  this->MinGridMapper = vtkPolyDataMapper2D::New();
  this->MinGridMapper->SetInput(this->MinGrid);
  this->MinGridActor = vtkActor2D::New();
  this->MinGridActor->SetMapper(this->MinGridMapper);

#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->MinGridMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif
  

  this->AxisVisibility = 1;
  this->MajTicksVisibility = 1;
  this->MinTicksVisibility = 1;
  this->LabelVisibility = 1;
  this->TitleVisibility = 1;
  this->MajGridVisibility = 0;
  this->MinGridVisibility = 0;


  this->LastPosition[0] = this->LastPosition[1] = 0;
  this->LastPosition2[0] = this->LastPosition2[1] = 0;

  
  this->MajTicksLocations.resize(0);
  this->MajTicksLabelLocations.resize(0);
  this->MajTicksLabelFormats.resize(0);

  this->MinTicksLocations.resize(0);

  this->FrontFillerStyle=0;
  this->FrontFillerLength=0;
  this->BackFillerStyle=0;
  this->BackFillerLength=0;

  this->Alignment=1;
  
  this->FrontFillerAlignment=0;
  this->BackFillerAlignment=0;

  this->GridStart = vtkCoordinate::New ();
  this->GridStop = vtkCoordinate::New ();
  this->GridStart->SetCoordinateSystemToNormalizedViewport();
  this->GridStop->SetCoordinateSystemToNormalizedViewport();
  this->GridStart->SetValue(0.0,1.0);
  this->GridStop->SetValue(0.0,1.0);

  this->MajGridValues = NULL;
  this->NumOfMajGridLines = 0;
  this->NumOfMajGridSlices = 0;

  this->MinGridValues = NULL;
  this->NumOfMinGridLines = 0;
  this->NumOfMinGridSlices = 0;


}

//----------------------------------------------------------------------------
vtkAxisActor2DExtra::~vtkAxisActor2DExtra()
{
  if (this->LabelFormat) 
    {
    delete [] this->LabelFormat;
    this->LabelFormat = NULL;
    }
  
  this->TitleActor->Delete();

  while(this->LabelActors.size() > 0)
    {
      this->LabelActors.front()->Delete();
      this->LabelActors.pop_front();
    };


  this->Axis->Delete();
  this->AxisMapper->Delete();
  this->AxisActor->Delete();


  this->MajTicks->Delete();
  this->MajTicksMapper->Delete();
  this->MajTicksActor->Delete();
  

  this->MinTicks->Delete();
  this->MinTicksMapper->Delete();
  this->MinTicksActor->Delete();

  this->MajGrid->Delete();
  this->MajGridMapper->Delete();
  this->MajGridActor->Delete();


  this->MinGrid->Delete();
  this->MinGridMapper->Delete();
  this->MinGridActor->Delete();

#ifndef NOCLIP
  for(int indx=0;indx<4;indx++)
    this->ClipPlanes[indx]->Delete();
#endif

  this->ClipLocation[0]->Delete ();
  this->ClipLocation[1]->Delete ();

  this->GridStart->Delete ();
  this->GridStop->Delete ();

  this->SetMajGridValues (NULL, 0, 0);
  this->SetMinGridValues (NULL, 0, 0);

  
  this->SetLabelTextProperty(NULL);

}

//----------------------------------------------------------------------------
// Build the axis, ticks, title, and labels and render.

int vtkAxisActor2DExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;
  

  if ( this->GetProperty()->GetLineWidth() <= 1 )
    {
      this->BuildLineAxis(viewport);
    }
  else
    {
      this->BuildPolygonAxis(viewport);
    };

  // Everything is built, just have to render
  if ( this->TitleVisibility )
    {
      renderedSomething += this->TitleActor->RenderOpaqueGeometry(viewport);
    }

  if ( this->AxisVisibility)
    {
      renderedSomething += this->AxisActor->RenderOpaqueGeometry(viewport);
      
    }
  
  if ( this->MajTicksVisibility)
    {
      renderedSomething += this->MajTicksActor->RenderOpaqueGeometry(viewport);
    };


  if ( this->MinTicksVisibility)
    {
      renderedSomething += this->MinTicksActor->RenderOpaqueGeometry(viewport);
    };

  if ( this->LabelVisibility )
    {
      for (list<vtkTextActorExtra*>::iterator tmpIter=this->LabelActors.begin();tmpIter!=this->LabelActors.end();tmpIter++)
	{
	  renderedSomething += 
	    (*tmpIter)->RenderOpaqueGeometry(viewport);
	}
    }



  if ( this->MajGridVisibility)
    {
      renderedSomething += this->MajGridActor->RenderOpaqueGeometry(viewport);
    };


  if ( this->MinGridVisibility)
    {
      renderedSomething += this->MinGridActor->RenderOpaqueGeometry(viewport);
    };


  


  return renderedSomething;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkAxisActor2DExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}
   

//----------------------------------------------------------------------------
// Render the axis, ticks, title, and labels.

int vtkAxisActor2DExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  // Everything is built, just have to render
  if ( this->TitleVisibility )
    {
      renderedSomething += this->TitleActor->RenderOverlay(viewport);
    }
  
  if ( this->AxisVisibility )
    {
      renderedSomething += this->AxisActor->RenderOverlay(viewport);

    }




  if ( this->MajTicksVisibility)
    {
      renderedSomething += this->MajTicksActor->RenderOverlay(viewport);
    };


  if ( this->MinTicksVisibility)
    {
      renderedSomething += this->MinTicksActor->RenderOverlay(viewport);
    };
  


  
  if ( this->LabelVisibility )
    {
      for (list<vtkTextActorExtra*>::iterator tmpIter=this->LabelActors.begin();tmpIter!=this->LabelActors.end();tmpIter++)
	{
	  renderedSomething += 
	    (*tmpIter)->RenderOverlay(viewport);
	}
    }



  if ( this->MajGridVisibility)
    {
      renderedSomething += this->MajGridActor->RenderOverlay(viewport);
    };


  if ( this->MinGridVisibility)
    {
      renderedSomething += this->MinGridActor->RenderOverlay(viewport);
    };


  

  return renderedSomething;
}

//----------------------------------------------------------------------------
// Release any graphics resources that are being consumed by this actor.
// The parameter window could be used to determine which graphic
// resources to release.
void vtkAxisActor2DExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  this->TitleActor->ReleaseGraphicsResources(win);
  for (list<vtkTextActorExtra*>::iterator tmpIter=this->LabelActors.begin();tmpIter!=this->LabelActors.end();tmpIter++)
    {
      (*tmpIter)->ReleaseGraphicsResources(win);
    }

  this->AxisActor->ReleaseGraphicsResources(win);

  this->MajTicksActor->ReleaseGraphicsResources(win);
  this->MinTicksActor->ReleaseGraphicsResources(win);


  this->MajGridActor->ReleaseGraphicsResources(win);
  this->MinGridActor->ReleaseGraphicsResources(win);
  
  
}

//----------------------------------------------------------------------------
void vtkAxisActor2DExtra::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

  if (this->LabelTextProperty)
    {
    os << indent << "Label Text Property:\n";
    this->LabelTextProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "Label Text Property: (none)\n";
    }

  os << indent << "Number Of Labels: " << this->NumberOfMajTicks << "\n";
  os << indent << "Range: (" << this->Range[0] 
     << ", " << this->Range[1] << ")\n";

  os << indent << "Label Format: " << this->LabelFormat << "\n";
  os << indent << "Maj Tick Length: " << this->MajTicksLength << "\n";
  os << indent << "Min Tick Length: " << this->MinTicksLength << "\n";
  os << indent << "Tick Offset: " << this->MajTickOffset << "\n";
  os << indent << "Labels Offset: " << this->LabelOffset << "\n";
  os << indent << "Title Parallel Offset: " << this->TitleParallelOffset << "\n";
  os << indent << "Title Perpendicular Offset: " << this->TitlePerpendicularOffset << "\n";
  
  os << indent << "Axis Visibility: " 
     << (this->AxisVisibility ? "On\n" : "Off\n");
  
  os << indent << "Maj Tick Visibility: " 
     << (this->MajTicksVisibility ? "On\n" : "Off\n");
  
  os << indent << "Min Tick Visibility: " 
     << (this->MinTicksVisibility ? "On\n" : "Off\n");

  os << indent << "Label Visibility: " 
     << (this->LabelVisibility ? "On\n" : "Off\n");
  
  os << indent << "Title Visibility: " 
     << (this->TitleVisibility ? "On\n" : "Off\n");

}

//----------------------------------------------------------------------------

void vtkAxisActor2DExtra::SetGridPositions(double inMinX, double inMinY,double inMaxX, double inMaxY)
{
  this->GridStart->SetValue(inMinX,inMinY);
  this->GridStop->SetValue(inMaxX,inMaxY);

  this->Modified();
};

void vtkAxisActor2DExtra::GetGridPositions(double &outMinX, double &outMinY,double &outMaxX, double &outMaxY)
{
  double tmp;
  this->GridStart->GetValue(outMinX,outMinY,tmp);
  this->GridStop->GetValue(outMaxX,outMaxY,tmp);

};


void vtkAxisActor2DExtra::SetMajGridValues (double ***inGrid, int inSlices, int inLines, bool copy)
{
  if (this->MajGridValues)
    {
    for (int ii=0; ii<this->NumOfMajGridLines; ii++)
      {
      for (int jj=0; jj<this->NumOfMajGridSlices; jj++)
        delete []this->MajGridValues[ii][jj];
      delete []this->MajGridValues[ii];
      }

    delete []this->MajGridValues;
    this->MajGridValues = NULL;
    this->NumOfMajGridLines = 0;
    this->NumOfMajGridSlices = 0;
    }

  if (inGrid)
    {
    if ((inSlices <= 0) || (inLines <=0 ))
      {
      vtkErrorMacro(<<"The specified grid has slices, "<<inSlices<<", and lines, "<<inLines<<", which should not be less than or equal to zero");
      return;
      }

    if (copy)
      {
      this->MajGridValues = new double**[inLines];
      for (int ii=0; ii<this->NumOfMajGridLines; ii++)
        {
        this->MajGridValues[ii] = new double*[inSlices];
        for (int jj=0; jj<this->NumOfMajGridSlices; jj++)
          {
          this->MajGridValues[ii][jj] = new double[2];
          this->MajGridValues[ii][jj][0] = inGrid[ii][jj][0];
          this->MajGridValues[ii][jj][1] = inGrid[ii][jj][1];
          this->MajGridValues[ii][jj][2] = 0;
          }
        }
      }
    else
      this->MajGridValues = inGrid;
      
    this->NumOfMajGridSlices = inSlices;
    this->NumOfMajGridLines = inLines;

    }
};

double ***vtkAxisActor2DExtra::GetMajGridValues (int &outSlices, int &outLines)
{
  outSlices = this->NumOfMajGridSlices;
  outLines = this->NumOfMajGridLines;

  return this->MajGridValues;

};


void vtkAxisActor2DExtra::SetMinGridValues (double ***inGrid, int inSlices, int inLines, bool copy)
{
  if (this->MinGridValues)
    {
    for (int ii=0; ii<this->NumOfMinGridLines; ii++)
      {
      for (int jj=0; jj<this->NumOfMinGridSlices; jj++)
        delete []this->MinGridValues[ii][jj];
      delete []this->MinGridValues[ii];
      }

    delete []this->MinGridValues;
    this->MinGridValues = NULL;
    this->NumOfMinGridLines = 0;
    this->NumOfMinGridSlices = 0;
    }

  if (inGrid)
    {
    if ((inSlices <= 0) || (inLines <=0 ))
      {
      vtkErrorMacro(<<"The specified grid has slices, "<<inSlices<<", and lines, "<<inLines<<", which should not be less than or equal to zero");
      return;
      }

    if (copy)
      {
      this->MinGridValues = new double**[inLines];
      for (int ii=0; ii<this->NumOfMinGridLines; ii++)
        {
        this->MinGridValues[ii] = new double*[inSlices];
        for (int jj=0; jj<this->NumOfMinGridSlices; jj++)
          {
          this->MinGridValues[ii][jj] = new double[2];
          this->MinGridValues[ii][jj][0] = inGrid[ii][jj][0];
          this->MinGridValues[ii][jj][1] = inGrid[ii][jj][1];
          }
        }
      }
    else
      this->MinGridValues = inGrid;
    this->NumOfMinGridSlices = inSlices;
    this->NumOfMinGridLines = inLines;

    }
};

double ***vtkAxisActor2DExtra::GetMinGridValues (int &outSlices, int &outLines)
{
  outSlices = this->NumOfMinGridSlices;
  outLines = this->NumOfMinGridLines;

  return this->MinGridValues;

};


void vtkAxisActor2DExtra::SetClipLocations(double inMinX, double inMinY,double inMaxX,double inMaxY)
{
  this->ClipLocation[0]->SetValue(inMinX,inMinY);
  this->ClipLocation[1]->SetValue(inMaxX,inMaxY);

  this->Modified();
};


#define VTK_AA2D_DEBUG 0

void vtkAxisActor2DExtra::BuildPolygonAxis(vtkViewport *viewport)
{

  unsigned int i,positionsHaveChanged;
  vtkIdType ptIds[7];
  double p1[3], p2[3], *x;
  double deltaX, deltaY;
  double xTick[3],xGrid[3];
  double theta, val;
  double stringSize[2];
  char tmpString[512],*chPtr;
  double majRefTick=0;
  double tmpP[3];
  double tmpLineWidth;
  double alignmentOffset=0;
  double ap1[3],ap2[3];
  double *clipPos1, *clipPos2;


  if (this->LabelVisibility && !this->LabelTextProperty)
    {
      vtkErrorMacro(<<"Need label text property to render axis actor");
      return;
    }

  positionsHaveChanged = 0;
  if (viewport->GetMTime() > this->BuildTime || 
      (viewport->GetVTKWindow() && 
       viewport->GetVTKWindow()->GetMTime() > this->BuildTime))
    {
    // Check to see whether we have to rebuild everything
    // Viewport change may not require rebuild
    double *lastPosition = 
      this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
    double *lastPosition2 =
      this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
    if (lastPosition[0] != this->LastPosition[0] ||
        lastPosition[1] != this->LastPosition[1] ||
        lastPosition2[0] != this->LastPosition2[0] ||
        lastPosition2[1] != this->LastPosition2[1] )
      {
	positionsHaveChanged = 1;
      }
    }
  
  if ( (!positionsHaveChanged) && (this->GetMTime() < this->BuildTime)) // && (this->TitleActor->GetMTime()< this->BuildTime))

    {
      return;
    }
  
#if VTK_AA2D_DEBUG
  printf ("vtkAxisActor2DExtra::BuildAxis: Rebuilding axis\n");
#endif

  vtkDebugMacro(<<"Rebuilding axis");

  // Initialize and get important info



  this->Axis->Initialize();
  this->AxisActor->SetProperty(this->GetProperty());

  this->MajTicks->Initialize();
  this->MinTicks->Initialize();
  this->TitleActor->SetProperty(this->GetProperty());

  // Generate the axis and tick marks.
  // We'll do our computation in viewport coordinates. First determine the
  // location of the endpoints.

  x = this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
  p1[0] = x[0]; 
  p1[1] = x[1];
  p1[2] = 0.0;
  this->LastPosition[0] = x[0]; 
  this->LastPosition[1] = x[1];

  x = this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
  p2[0] = x[0]; 
  p2[1] = x[1];
  p2[2] = 0.0;
  this->LastPosition2[0] = x[0]; 
  this->LastPosition2[1] = x[1];

  vtkPoints *pts = vtkPoints::New();
  vtkCellArray *lines = vtkCellArray::New();
  this->Axis->SetPoints(pts);
  this->Axis->SetPolys(lines);
  pts->Delete();
  lines->Delete();

  // Generate point along axis (as well as tick points)

  deltaX = p2[0] - p1[0];
  deltaY = p2[1] - p1[1];

  if (deltaX == 0. && deltaY == 0.)
    {
      theta = 0.;
    }
  else
    {
      theta = atan2(deltaY, deltaX);
    }


  tmpLineWidth =this->GetProperty()->GetLineWidth();


  if ( this->Alignment == 0 )
    {
      alignmentOffset= -tmpLineWidth;
    }
  else if ( this->Alignment == 1 )
    {
      alignmentOffset=-tmpLineWidth/2.0;
    }
  else
    {
      alignmentOffset=0;
    };

  


  if (this->AxisVisibility)
    {

      ap1[2]=0;
      ap2[2]=0;
      tmpP[2]=0;

      double tmpOffset=0;

      if ( this->FrontFillerStyle > 0 )
	{
	  if ( this->FrontFillerAlignment == 1 )
	    tmpOffset=FrontFillerLength/2;
	  else if ( this->FrontFillerAlignment == 2 )
	    tmpOffset=FrontFillerLength;
	};


      ap1[0] = p1[0]+tmpOffset*cos(theta)+ alignmentOffset * sin(theta);
      ap1[1] = p1[1]+tmpOffset*sin(theta)- alignmentOffset * cos(theta);


      tmpOffset=0;
      if ( this->BackFillerStyle > 0 )
	{
	  if ( this->BackFillerAlignment == 1 )
	    tmpOffset=-BackFillerLength/2;
	  else if ( this->BackFillerAlignment == 2 )
	    tmpOffset=-BackFillerLength;
	};

      ap2[0] = p2[0]+tmpOffset*cos(theta)+ alignmentOffset * sin(theta);
      ap2[1] = p2[1]+tmpOffset*sin(theta)- alignmentOffset * cos(theta);

      // Set clip planes
      clipPos1 = ClipLocation[0]->GetComputedDoubleViewportValue(viewport);
      clipPos2 = ClipLocation[1]->GetComputedDoubleViewportValue(viewport);

#ifndef NOCLIP
      this->ClipPlanes[0]->SetOrigin(clipPos1[0],0,0);
      this->ClipPlanes[1]->SetOrigin(clipPos2[0],0,0);
      this->ClipPlanes[2]->SetOrigin(0,clipPos1[1],0);
      this->ClipPlanes[3]->SetOrigin(0,clipPos2[1],0);
#endif

      //Create axis
      
      
      ptIds[0] = pts->InsertNextPoint(ap1);  
      ptIds[1] = pts->InsertNextPoint(ap2); 
      i=2;

      
      
      if ( BackFillerStyle == 0 )
	{
	  tmpP[0] = ap2[0] + tmpLineWidth * sin(theta);
	  tmpP[1] = ap2[1] - tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;
	}
      else  if ( BackFillerStyle == 1 )
	{
	  
	  tmpP[0] = ap2[0]+BackFillerLength*cos(theta);
	  tmpP[1] = ap2[1]+BackFillerLength*sin(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;

	  tmpP[0] = tmpP[0] + tmpLineWidth * sin(theta);
	  tmpP[1] = tmpP[1] - tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;
	}
      else  if ( BackFillerStyle == 2 )
	{
	  tmpP[0] = ap2[0]+BackFillerLength*cos(theta);
	  tmpP[1] = ap2[1]+BackFillerLength*sin(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;

	  tmpP[0] = ap2[0] + tmpLineWidth * sin(theta);
	  tmpP[1] = ap2[1] - tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;

	}
      else  if ( BackFillerStyle == 3 )
	{

	  tmpP[0] = ap2[0]+BackFillerLength*cos(theta) + tmpLineWidth * sin(theta);
	  tmpP[1] = ap2[1]+BackFillerLength*sin(theta) - tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;
	}
      else if ( BackFillerStyle == 4 )
	{

	  tmpP[0] = ap2[0]+BackFillerLength*cos(theta)+ tmpLineWidth/2.0 * sin(theta);
	  tmpP[1] = ap2[1]+BackFillerLength*sin(theta)- tmpLineWidth/2.0 * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;
	  
	  tmpP[0] = ap2[0] + tmpLineWidth * sin(theta);
	  tmpP[1] = ap2[1] - tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;


	};


      if ( FrontFillerStyle == 0 )
	{
	  tmpP[0] = ap1[0] + tmpLineWidth * sin(theta);
	  tmpP[1] = ap1[1] - tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;
	  
	}
      else  if ( FrontFillerStyle == 1 )
	{
	  
	  tmpP[0] = ap1[0]-FrontFillerLength*cos(theta)+ tmpLineWidth * sin(theta);
	  tmpP[1] = ap1[1]-FrontFillerLength*sin(theta)- tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;

	  tmpP[0] = ap1[0]-FrontFillerLength*cos(theta);
	  tmpP[1] = ap1[1]-FrontFillerLength*sin(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);

	  i++;
	  
	}
      else  if ( FrontFillerStyle == 2 )
	{
	  
	  tmpP[0] = ap1[0]+ tmpLineWidth * sin(theta);
	  tmpP[1] = ap1[1]- tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;
	  
	  
	  tmpP[0] = ap1[0]-FrontFillerLength*cos(theta);
	  tmpP[1] = ap1[1]-FrontFillerLength*sin(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);

	  i++;
	  

	}
      else  if ( FrontFillerStyle == 3 )
	{

	  tmpP[0] = ap1[0]-FrontFillerLength*cos(theta)+ tmpLineWidth * sin(theta);
	  tmpP[1] = ap1[1]-FrontFillerLength*sin(theta)- tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;


	}
      else if ( FrontFillerStyle == 4 )
	{
	  tmpP[0] = ap1[0]+ tmpLineWidth * sin(theta);
	  tmpP[1] = ap1[1]- tmpLineWidth * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);
	  i++;
	  
	  
	  tmpP[0] = ap1[0]-FrontFillerLength*cos(theta)+ tmpLineWidth/2.0 * sin(theta);
	  tmpP[1] = ap1[1]-FrontFillerLength*sin(theta)- tmpLineWidth/2.0 * cos(theta);
	  ptIds[i] = pts->InsertNextPoint(tmpP);

	  i++;


	};



      lines->InsertNextCell(i, ptIds);


      // Generate filler pieces 


    };
  
  
  if ( this->Alignment == 0 )
    { 
      if ( MajorTicksFlip )
	{
	  alignmentOffset= tmpLineWidth;
	}
      else
	{
	  alignmentOffset=0;
	}
   }
  else if ( this->Alignment == 1 )
    {
      alignmentOffset=tmpLineWidth/2.0;
    }
  else
    {
      if ( MajorTicksFlip )
	{
	  alignmentOffset=0;
	}
      else
	{
	  alignmentOffset= tmpLineWidth;
	};
    };



  vtkPoints *MajGridPts = vtkPoints::New();
  MajGridPts->SetDataTypeToDouble();
  vtkCellArray *MajGridLines = vtkCellArray::New();
  this->MajGrid->SetPoints(MajGridPts);
  this->MajGrid->SetLines(MajGridLines);
  MajGridPts->Delete();
  MajGridLines->Delete();



  // Create Maj ticks
  vtkPoints *Majpts = vtkPoints::New();
  Majpts->SetDataTypeToDouble();
  vtkCellArray *Majlines = vtkCellArray::New();
  this->MajTicks->SetPoints(Majpts);
  this->MajTicks->SetLines(Majlines);
  Majpts->Delete();
  Majlines->Delete();

  
  xTick[2] = 0.0;
  xGrid[2]=0.0;
  int tmpSign=1;
 
  if ( MajorTicksFlip )
    {
      tmpSign=-1;
    };


  vector<double> tmpMajLocations,tmpMinLocations;
  vector<bool> tmpMajLabelFlags;
  vector<vtkTextActorFormat> tmpMajLabelFormats;

  if ( (this->MajTicksVisibility) || (this->MinTicksVisibility) || (this->LabelVisibility) || (this->MajGridVisibility) || (this->MinGridVisibility))
    {
      switch (this->MajTicksMode)
	{
	case 1:
	  CalculateMajTicksLocationsByInterval(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MajTicksInterval,tmpMajLocations,majRefTick);
	  break;
	case 2:
	  CalculateMajTicksLocationsByLocations(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MajTicksLocations,MajTicksLabelLocations,this->MajTicksLabelFormats,tmpMajLocations,tmpMajLabelFlags,tmpMajLabelFormats);
	  break;
	case 0:
	default:
	  CalculateMajTicksLocationsByCount(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->NumberOfMajTicks,tmpMajLocations);
	  if ( this->AdjustedRange[this->RangeIndex]<=this->AdjustedRange[this->RangeIndex+1])
	    {
	      majRefTick=this->AdjustedRange[this->RangeIndex];
	    }
	  else
	    {
	      majRefTick=this->AdjustedRange[this->RangeIndex+1]; 
	    };
	  break;
	}
    };

  //Map maj ticks to the screen



  if (( this->MajTicksVisibility) || (this->LabelVisibility) || (this->MajGridVisibility) )
    {
      vector<double>::iterator tmpIter=tmpMajLocations.begin();

      while( tmpIter != tmpMajLocations.end())
	{
        if (this->RangeIndex == 0)
          {
	  if (MapScale(&this->AdjustedRange[0],p1[0],p2[0],this->Scale[0],this->MinLogLimit,*tmpIter,true,xTick[0])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point "<<*tmpIter<<endl;
	      tmpMajLocations.erase(tmpIter);
	      continue;
	    };
          xTick[1] = p1[1];
          }
        else
          {
	  if (MapScale(&this->AdjustedRange[2],p1[1],p2[1],this->Scale[1],this->MinLogLimit,*tmpIter,true,xTick[1])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point "<<*tmpIter<<endl;
	      tmpMajLocations.erase(tmpIter);
	      continue;
	    };  
          xTick[0] = p1[0];
          }

          // if the grid is not set calculate it's positions based off of major ticks
	  if (this->MajGridVisibility && !this->MajGridValues)
	    {
              double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
              double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
              double gridOffset[] = {gridStartPos[0] - p1[0], gridStartPos[1] - p1[1]};
              double gridLength[] = {gridStopPos[0] - gridStartPos[0], gridStopPos[1]- gridStartPos[1]};

              xGrid[0] = xTick[0] + (gridOffset[0]) * sin(theta);
              xGrid[1] = xTick[1] + (gridOffset[1]) * cos(theta);

              MajGridPts->InsertNextPoint(xGrid);

              xGrid[0] = xTick[0] + (gridLength[0]+gridOffset[0]) * sin(theta);
              xGrid[1] = xTick[1] + (gridLength[1]+gridOffset[1]) * cos(theta);

	      MajGridPts->InsertNextPoint(xGrid);
	    };

	  if (( this->MajTicksVisibility) || (this->LabelVisibility))
	    {
	      xTick[0] = xTick[0] + tmpSign*(this->MajTickOffset+alignmentOffset) * sin(theta);
	      xTick[1] = xTick[1] - tmpSign*(this->MajTickOffset+alignmentOffset) * cos(theta);
	      Majpts->InsertNextPoint(xTick);
	      

	      
	      
	      xTick[0] = xTick[0] + tmpSign*this->MajTicksLength * sin(theta);
	      xTick[1] = xTick[1] - tmpSign*this->MajTicksLength * cos(theta);
	      Majpts->InsertNextPoint(xTick);
	    };
	  tmpIter++;

	};

      if (this->MajGridVisibility)
	{
        if (!this->MajGridValues)
          {
	  for (i = 0; i < tmpMajLocations.size(); i++)
	    {
	      ptIds[0] = 2*i;
	      ptIds[1] = 2*i + 1;
	      MajGridLines->InsertNextCell(2, ptIds);
	    }
          }
        else
          {
           vtkIdType *gridPtIds = new vtkIdType[this->NumOfMajGridSlices];
           double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
           double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
           for (int ii=0; ii<this->NumOfMajGridLines; ii++)
             {
             for (int jj=0; jj<this->NumOfMajGridSlices; jj++)
               {
               MapScale(&this->AdjustedRange[0],gridStartPos[0],gridStopPos[0],this->Scale[0],this->MinLogLimit,this->MajGridValues[ii][jj][0],false,xGrid[0]);
               MapScale(&this->AdjustedRange[2],gridStartPos[1],gridStopPos[1],this->Scale[1],this->MinLogLimit,this->MajGridValues[ii][jj][1],false,xGrid[1]);

               gridPtIds[jj] = MajGridPts->InsertNextPoint(xGrid);
               }
             MajGridLines->InsertNextCell (this->NumOfMajGridSlices, gridPtIds);
             }
             delete []gridPtIds;
           }

/*          vtkIdType *gridPtIds = new vtkIdType[this->NumOfMajGridSlices];
          for (int ii=0; ii<this->NumOfMajGridLines; ii++)
            {
            for (int jj=0; jj<this->NumOfMajGridSlices; jj++)
              {
              gridPtIds[jj] = MajGridPts->InsertNextPoint(this->MajGridValues[ii][jj]);
              }
            MajGridLines->InsertNextCell (this->NumOfMajGridSlices, gridPtIds);
            }
            delete []gridPtIds;
          }
*/

	}; 

      if (( this->MajTicksVisibility) || (this->LabelVisibility))
	{
	  for (i = 0; i < tmpMajLocations.size(); i++)
	    {
	      ptIds[0] = 2*i;
	      ptIds[1] = 2*i + 1;
	      Majlines->InsertNextCell(2, ptIds);
	    }
	};


      
    }


  // Build the labels

  if (this->LabelVisibility)
    {

      if ( this->LabelsFlip )
	{
	  tmpSign=-1;
	}
      else
	{
	  tmpSign=1;
	};
      


      if ((( this->MajorTicksFlip ) && (this->LabelsFlip))
	  ||
	  (( !this->MajorTicksFlip ) && (!this->LabelsFlip)))
	{
	  alignmentOffset=0;
	}
      else
	{
	  alignmentOffset=tmpLineWidth;
	};



      if ( this->MajTicksMode ==2)
	{
	  BuildLabels(this->LabelActors,tmpMajLabelFlags);
	}
      else
	{
	  BuildLabels(this->LabelActors,tmpMajLocations.size());
	};

      list<vtkTextActorExtra*>::iterator tmpActIter=this->LabelActors.begin();
      bool intFormatFlag=IsIntFormat(this->LabelFormat);

      for(i=0;i<tmpMajLocations.size();i++)
	{
	  (*tmpActIter)->SetProperty(this->GetProperty());
	  (*tmpActIter)->GetTextProperty()->ShallowCopy(this->LabelTextProperty);

	  if ( ( this->MajTicksMode != 2) || (tmpMajLabelFlags[i]))
	    {
	      if ( !tmpMajLabelFormats.size())
		{
		  val = tmpMajLocations[i];
		  if (intFormatFlag)
		    {
		      sprintf(tmpString, this->LabelFormat, static_cast<int>(val));
		    }
		  else
		    {
		      sprintf(tmpString, this->LabelFormat, val);
		    };
		  chPtr=tmpString;
		  while( isspace(*chPtr) && (*chPtr!=0))
		    chPtr++;
		  
		  (*tmpActIter)->SetInput(chPtr);
		}
	      else
		{
		  (*tmpActIter)->SetInput(&tmpMajLabelFormats[i]); 
		};

	      
	      Majpts->GetPoint(2 * i , xTick);


	      xTick[0] = xTick[0] - tmpSign*(this->MajTickOffset) * sin(theta);
	      xTick[1] = xTick[1] + tmpSign*(this->MajTickOffset) * cos(theta);

	      //(*tmpActIter)->GetSize(viewport, stringSize);
	      
	      (*tmpActIter)->SetVertAlignmentOffset(this->MajTickLabelVertAlignmentOffset);
	      (*tmpActIter)->SetHorizAlignmentOffset(this->MajTickLabelHorizAlignmentOffset);
	      


	      /*
	      (*tmpActIter)->GetBBox(viewport,llx,lly,urx,ury);
	      stringSize[0]=urx-llx;
	      stringSize[1]=ury-lly;
	      */
	      this->SetOffsetPosition(xTick, 
				      theta, 
				      0,
				      -(this->LabelOffset+alignmentOffset), 
				      this->LabelsFlip,
				      (*tmpActIter));
	      
	      tmpActIter++;
	    };
	}

      // Check if labels can be clipped. Since there is currently no
      // good way to clip a partial label, clip the entire label if it
      // crosses the clipping plane.
      if (this->ClipData)
        {
	  double *lblPos;
          tmpActIter=this->LabelActors.begin();
          while (tmpActIter!=this->LabelActors.end())
            {
              lblPos = (*tmpActIter)->GetPosition();

              if ((clipPos1[0] > lblPos[0] + FLT_EPSILON) || (clipPos2[0] < lblPos[0] - FLT_EPSILON) ||
                  (clipPos1[1] > lblPos[1] + FLT_EPSILON) || (clipPos2[1] < lblPos[1] - FLT_EPSILON))
                tmpActIter = this->LabelActors.erase (tmpActIter);
              else
                tmpActIter++;

            }
        }


    } // If labels visible






  if ( this->Alignment == 0 )
    {
      if ( MinorTicksFlip )
	{
	  alignmentOffset= tmpLineWidth;
	}
      else
	{
	  alignmentOffset=0;
	}
    }
  else if ( this->Alignment == 1 )
    {
      alignmentOffset=tmpLineWidth/2.0;
    }
  else
    {
      if ( MinorTicksFlip )
	{
	  alignmentOffset=0;
	}
      else
	{
	  alignmentOffset= tmpLineWidth;
	};
    };

  

  vtkPoints *MinGridPts = vtkPoints::New();
  MinGridPts->SetDataTypeToDouble();
  vtkCellArray *MinGridLines = vtkCellArray::New();
  this->MinGrid->SetPoints(MinGridPts);
  this->MinGrid->SetLines(MinGridLines);
  MinGridPts->Delete();
  MinGridLines->Delete();


  

  //Create Min ticks

  vtkPoints *Minpts = vtkPoints::New();
  Minpts->SetDataTypeToDouble();
  vtkCellArray *Minlines = vtkCellArray::New();
  this->MinTicks->SetPoints(Minpts);
  this->MinTicks->SetLines(Minlines);
  Minpts->Delete();
  Minlines->Delete();

  tmpSign=1;


 
  if ( MinorTicksFlip )
    {
      tmpSign=-1;
    };


  if ( (this->MinTicksVisibility) || (this->MinGridVisibility) )
    {

      switch (this->MinTicksMode)
	{
	case 1:
	  if ( this->MajTicksMode !=2)
	    {
	      CalculateMinTicksLocationsByInterval(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MinTicksInterval,majRefTick,tmpMajLocations,tmpMinLocations);
	    }
	  else
	    {
	      CalculateMinTicksLocationsByInterval(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MinTicksInterval,this->MajTicksLocations,tmpMinLocations); 
	    }
	  break;
        case 2:
          CalculateTicksLocationsByLocations(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MinTicksLocations,tmpMinLocations);
          break;
	case 0:
	default:
	  if ( this->MajTicksMode !=2)
	    {
	      CalculateMinTicksLocationsByCount(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->NumberOfMinTicks,majRefTick,tmpMajLocations,tmpMinLocations);
	    }
	  else
	    {
	      CalculateMinTicksLocationsByCount(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->NumberOfMinTicks,this->MajTicksLocations,tmpMinLocations);
	    };
	  break;
	}

      
      vector<double>::iterator tmpIter=tmpMinLocations.begin();


      //Map min ticks to the screen

      while( tmpIter != tmpMinLocations.end())
	{
        if (this->RangeIndex == 0)
          {
	  if (MapScale(&this->AdjustedRange[0],p1[0],p2[0],this->Scale[0],this->MinLogLimit,*tmpIter,true,xTick[0])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point"<<endl;
	      tmpMinLocations.erase(tmpIter);
	      continue;
	    };
          xTick[1] = p1[1];
          }
        else
          {
	  if (
	      MapScale(&this->AdjustedRange[2],p1[1],p2[1],this->Scale[1],this->MinLogLimit,*tmpIter,true,xTick[1])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point"<<endl;
	      tmpMinLocations.erase(tmpIter);

	      continue;
	    };
          xTick[0] = p1[0];
          }


	  if (this->MinGridVisibility && !this->MinGridValues)
	    {
              double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
              double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
              double gridOffset[] = {gridStartPos[0] - p1[0], gridStartPos[1] - p1[1]};
              double gridLength[] = {gridStopPos[0] - gridStartPos[0], gridStopPos[1]- gridStartPos[1]};

              xGrid[0] = xTick[0] + (gridOffset[0]) * sin(theta);
              xGrid[1] = xTick[1] + (gridOffset[1]) * cos(theta);

              MinGridPts->InsertNextPoint(xGrid);

              xGrid[0] = xTick[0] + (gridLength[0]+gridOffset[0]) * sin(theta);
              xGrid[1] = xTick[1] + (gridLength[1]+gridOffset[1]) * cos(theta);

	      MinGridPts->InsertNextPoint(xGrid);
	    };
	  


	  if  (this->MinTicksVisibility)
	    {
	      xTick[0] += tmpSign*(this->MinTickOffset+alignmentOffset) * sin(theta);
	      xTick[1] -= tmpSign*(this->MinTickOffset+alignmentOffset) * cos(theta);
	      
	      Minpts->InsertNextPoint(xTick);
	      
	      
	      xTick[0] = xTick[0] + tmpSign*this->MinTicksLength * sin(theta);
	      xTick[1] = xTick[1] - tmpSign*this->MinTicksLength * cos(theta);
	      Minpts->InsertNextPoint(xTick);
	    };

	  tmpIter++;

	};

      if (this->MinGridVisibility)
	{
        if (!this->MinGridValues)
          {
	  for (i = 0; i < tmpMinLocations.size(); i++)
	    {
	      ptIds[0] = 2*i;
	      ptIds[1] = 2*i+ 1;
	      MinGridLines->InsertNextCell(2, ptIds);
	    }
          }
        else
          {
/*
          vtkIdType *gridPtIds = new vtkIdType[this->NumOfMinGridSlices];
          for (int ii=0; ii<this->NumOfMinGridLines; ii++)
            {
            for (int jj=0; jj<this->NumOfMajGridSlices; jj++)
              {
              gridPtIds[jj] = MinGridPts->InsertNextPoint(this->MinGridValues[ii][jj]);
              }
            MinGridLines->InsertNextCell (this->NumOfMinGridSlices, gridPtIds);
            }
            delete []gridPtIds;
*/

          vtkIdType *gridPtIds = new vtkIdType[this->NumOfMinGridSlices];
          double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
          double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
          for (int ii=0; ii<this->NumOfMinGridLines; ii++)
            {
            for (int jj=0; jj<this->NumOfMinGridSlices; jj++)
              {
              MapScale(&this->AdjustedRange[0],gridStartPos[0],gridStopPos[0],this->Scale[0],this->MinLogLimit,this->MinGridValues[ii][jj][0],false,xGrid[0]);
              MapScale(&this->AdjustedRange[2],gridStartPos[1],gridStopPos[1],this->Scale[1],this->MinLogLimit,this->MinGridValues[ii][jj][1],false,xGrid[1]);

              gridPtIds[jj] = MinGridPts->InsertNextPoint(xGrid);
              }
            MinGridLines->InsertNextCell (this->NumOfMinGridSlices, gridPtIds);
            }
            delete []gridPtIds;
          }

	};
      
      if ( this->MinTicksVisibility)
	{
	  for (i = 0; i < tmpMinLocations.size(); i++)
	    {
	      ptIds[0] = 2*i;
	      ptIds[1] = 2*i+ 1;
	      Minlines->InsertNextCell(2, ptIds);
	    }
	};
    };








  
  // Now build the title

  if (this->TitleVisibility)
    {
      if ( this->Alignment == 0 )
	{ 
	  if ( this->TitleFlip )
	    {
	      alignmentOffset= tmpLineWidth;
	    }
	  else
	    {
	      alignmentOffset=0;
	    }
	}
      else if ( this->Alignment == 1 )
	{
	  alignmentOffset=tmpLineWidth/2.0;
	}
      else
	{
	  if ( this->TitleFlip )
	    {
	      alignmentOffset=0;
	    }
	  else
	    {
	      alignmentOffset= tmpLineWidth;
	    };
	};
      
      double llx,lly,urx,ury;
      this->TitleActor->GetBBox(viewport,llx,lly,urx,ury);
      stringSize[0]=urx-llx;
      stringSize[1]=ury-lly;

      
      //      pts->GetPoint(0 , p1);
      //      pts->GetPoint(1 , p2);
      xTick[0] = p1[0] + (p2[0] - p1[0]) / 2.0;
      xTick[1] = p1[1] + (p2[1] - p1[1]) / 2.0;
      double actualParallelOffset = sqrt(deltaX*deltaX+deltaY*deltaY)*this->TitleParallelOffset;

      
      this->SetOffsetPosition(xTick, 
			      theta, 
			      -actualParallelOffset,
			      -(this->TitlePerpendicularOffset+alignmentOffset),
			      this->TitleFlip,
			      this->TitleActor);

       if (this->ClipData)
        {
          double *lblPos;
          lblPos = this->TitleActor->GetPosition();

          if ((clipPos1[0] > lblPos[0] + FLT_EPSILON) || (clipPos2[0] < lblPos[0] - FLT_EPSILON) ||
              (clipPos1[1] > lblPos[1] + FLT_EPSILON) || (clipPos2[1] < lblPos[1] - FLT_EPSILON))
            this->TitleActor->SetPosition (-1, -1);
        };
    } // If title visible
  
  this->BuildTime.Modified();
}



void vtkAxisActor2DExtra::BuildLineAxis(vtkViewport *viewport)
{

  unsigned int i,positionsHaveChanged;
  vtkIdType ptIds[2];
  double p1[3], p2[3], *x;
  double deltaX, deltaY;
  double xTick[3],xGrid[3];
  double theta, val;
  double stringSize[2];
  char tmpString[512],*chPtr;
  double majRefTick=0;
  double *clipPos1, *clipPos2;


  if (this->LabelVisibility && !this->LabelTextProperty)
    {
    vtkErrorMacro(<<"Need label text property to render axis actor");
    return;
    }

  positionsHaveChanged = 0;
  if (viewport->GetMTime() > this->BuildTime || 
      (viewport->GetVTKWindow() && 
       viewport->GetVTKWindow()->GetMTime() > this->BuildTime))
    {
    // Check to see whether we have to rebuild everything
    // Viewport change may not require rebuild
    double *lastPosition = 
      this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
    double *lastPosition2 =
      this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
    if (lastPosition[0] != this->LastPosition[0] ||
        lastPosition[1] != this->LastPosition[1] ||
        lastPosition2[0] != this->LastPosition2[0] ||
        lastPosition2[1] != this->LastPosition2[1] )
      {
      positionsHaveChanged = 1;
      }
    }
  
  if ( (!positionsHaveChanged) && (this->GetMTime() < this->BuildTime) ) // && (this->TitleActor->GetMTime()< this->BuildTime))

    {
      return;
    }
  
#if VTK_AA2D_DEBUG
  printf ("vtkAxisActor2DExtra::BuildAxis: Rebuilding axis\n");
#endif

  vtkDebugMacro(<<"Rebuilding axis");

  // Initialize and get important info



  this->Axis->Initialize();
  this->AxisActor->SetProperty(this->GetProperty());

  this->MajTicks->Initialize();
  this->MinTicks->Initialize();
  this->TitleActor->SetProperty(this->GetProperty());

  // Generate the axis and tick marks.
  // We'll do our computation in viewport coordinates. First determine the
  // location of the endpoints.

  x = this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
  p1[0] = x[0]; 
  p1[1] = x[1];
  p1[2] = 0.0;
  this->LastPosition[0] = x[0]; 
  this->LastPosition[1] = x[1];

  x = this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
  p2[0] = x[0]; 
  p2[1] = x[1];
  p2[2] = 0.0;
  this->LastPosition2[0] = x[0]; 
  this->LastPosition2[1] = x[1];

  vtkPoints *pts = vtkPoints::New();
  vtkCellArray *lines = vtkCellArray::New();
  this->Axis->SetPoints(pts);
  this->Axis->SetLines(lines);
  pts->Delete();
  lines->Delete();

  // Generate point along axis (as well as tick points)

  deltaX = p2[0] - p1[0];
  deltaY = p2[1] - p1[1];

  if (deltaX == 0. && deltaY == 0.)
    {
    theta = 0.;
    }
  else
    {
    theta = atan2(deltaY, deltaX);
    }

  // Set clip planes
  clipPos1 = this->ClipLocation[0]->GetComputedDoubleViewportValue(viewport);
  clipPos2 = this->ClipLocation[1]->GetComputedDoubleViewportValue(viewport);

#ifndef NOCLIP
  this->ClipPlanes[0]->SetOrigin(clipPos1[0],0,0);
  this->ClipPlanes[1]->SetOrigin(clipPos2[0],0,0);
  this->ClipPlanes[2]->SetOrigin(0,clipPos1[1],0);
  this->ClipPlanes[3]->SetOrigin(0,clipPos2[1],0);
#endif


  //Create axis


  ptIds[0] = pts->InsertNextPoint(p1);  
  ptIds[1] = pts->InsertNextPoint(p2); 

  if (this->AxisVisibility)
    {
      lines->InsertNextCell(2, ptIds);
    }

  vtkPoints *MajGridPts = vtkPoints::New();
  MajGridPts->SetDataTypeToDouble();
  vtkCellArray *MajGridLines = vtkCellArray::New();
  this->MajGrid->SetPoints(MajGridPts);
  this->MajGrid->SetLines(MajGridLines);
  MajGridPts->Delete();
  MajGridLines->Delete();



  // Create Maj ticks
  vtkPoints *Majpts = vtkPoints::New();
  Majpts->SetDataTypeToDouble();
  vtkCellArray *Majlines = vtkCellArray::New();
  this->MajTicks->SetPoints(Majpts);
  this->MajTicks->SetLines(Majlines);
  Majpts->Delete();
  Majlines->Delete();


  
  xTick[2] = 0.0;
  xGrid[2]= 0.0;
  int tmpSign=1;
  double tmpLineWidth;

  if (this->AxisVisibility)
    {
      tmpLineWidth =this->GetProperty()->GetLineWidth()/2.0;
    }
  else
    {
      tmpLineWidth=0; 
    }
 
  if ( MajorTicksFlip )
    {
      tmpSign=-1;
    };


  vector<double> tmpMajLocations,tmpMinLocations;
  vector<bool> tmpMajLabelFlags;
  vector<vtkTextActorFormat> tmpMajLabelFormats;

  if ( (this->MajTicksVisibility) || (this->MinTicksVisibility) || (this->LabelVisibility)|| (this->MajGridVisibility) || (this->MinGridVisibility) )
    {
      switch (this->MajTicksMode)
	{
	case 1:
	  CalculateMajTicksLocationsByInterval(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MajTicksInterval,tmpMajLocations,majRefTick);
	  break;
	case 2:
	  CalculateMajTicksLocationsByLocations(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MajTicksLocations,MajTicksLabelLocations,this->MajTicksLabelFormats,tmpMajLocations,tmpMajLabelFlags,tmpMajLabelFormats);
	  break;
	case 0:
	default:
	  CalculateMajTicksLocationsByCount(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->NumberOfMajTicks,tmpMajLocations);
	  if ( this->AdjustedRange[this->RangeIndex]<=this->AdjustedRange[this->RangeIndex+1])
	    {
	      majRefTick=this->AdjustedRange[this->RangeIndex];
	    }
	  else
	    {
	      majRefTick=this->AdjustedRange[this->RangeIndex+1]; 
	    };
	  break;
	}
    };

  //Map maj ticks to the screen



  if (( this->MajTicksVisibility) || (this->LabelVisibility) )
    {
      vector<double>::iterator tmpIter=tmpMajLocations.begin();

      while( tmpIter != tmpMajLocations.end())
	{
        if (this->RangeIndex == 0)
          {
	  if (MapScale(&this->AdjustedRange[0],p1[0],p2[0],this->Scale[0],this->MinLogLimit,*tmpIter,true,xTick[0])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point "<<*tmpIter<<endl;
	      tmpMajLocations.erase(tmpIter);
	      continue;
	    };
	  xTick[1] = p1[1];
          }
        else
          {
	  if (MapScale(&this->AdjustedRange[2],p1[1],p2[1],this->Scale[1],this->MinLogLimit,*tmpIter,true,xTick[1])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point "<<*tmpIter<<endl;
	      tmpMajLocations.erase(tmpIter);
	      continue;
	    };  
          xTick[0] = p1[0];
          }
	  
	  if (this->MajGridVisibility && !this->MajGridValues)
	    {
              double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
              double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
              double gridOffset[] = {gridStartPos[0] - p1[0], gridStartPos[1] - p1[1]};
              double gridLength[] = {gridStopPos[0] - gridStartPos[0], gridStopPos[1] - gridStartPos[1]};

              xGrid[0] = xTick[0] + (gridOffset[0]) * sin(theta);
              xGrid[1] = xTick[1] + (gridOffset[1]) * cos(theta);

	      MajGridPts->InsertNextPoint(xGrid);

              xGrid[0] = xTick[0] + (gridLength[0]+gridOffset[0]) * sin(theta);
              xGrid[1] = xTick[1] + (gridLength[1]+gridOffset[1]) * cos(theta);
	      
	      MajGridPts->InsertNextPoint(xGrid);
	    };

	  if (( this->MajTicksVisibility) || (this->LabelVisibility))
	    {
	      xTick[0] = xTick[0] + tmpSign*(this->MajTickOffset+tmpLineWidth) * sin(theta);
	      xTick[1] = xTick[1] - tmpSign*(this->MajTickOffset+tmpLineWidth) * cos(theta);
	      Majpts->InsertNextPoint(xTick);
	      
	      
	      xTick[0] = xTick[0] + tmpSign*this->MajTicksLength * sin(theta);
	      xTick[1] = xTick[1] - tmpSign*this->MajTicksLength * cos(theta);
	      Majpts->InsertNextPoint(xTick);
	    };

	  tmpIter++;

	};  
      
       if (this->MajGridVisibility)
         {
         if (!this->MajGridValues)
           {
	   for (i = 0; i < tmpMajLocations.size(); i++)
    	     {
	       ptIds[0] = 2*i;
  	       ptIds[1] = 2*i + 1;
	       MajGridLines->InsertNextCell(2, ptIds);
	     }
           }
         else
           {
           vtkIdType *gridPtIds = new vtkIdType[this->NumOfMajGridSlices];
           double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
           double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
           for (int ii=0; ii<this->NumOfMajGridLines; ii++)
             {
             for (int jj=0; jj<this->NumOfMajGridSlices; jj++)
               {
	       MapScale(&this->AdjustedRange[0],gridStartPos[0],gridStopPos[0],this->Scale[0],this->MinLogLimit,this->MajGridValues[ii][jj][0],false,xGrid[0]);
               MapScale(&this->AdjustedRange[2],gridStartPos[1],gridStopPos[1],this->Scale[1],this->MinLogLimit,this->MajGridValues[ii][jj][1],false,xGrid[1]);

               gridPtIds[jj] = MajGridPts->InsertNextPoint(xGrid);
               }
             MajGridLines->InsertNextCell (this->NumOfMajGridSlices, gridPtIds);
             }
             delete []gridPtIds;
           }
 	}; 

      if (( this->MajTicksVisibility) || (this->LabelVisibility))
	{
	  for (i = 0; i < tmpMajLocations.size(); i++)
	    {
	      ptIds[0] = 2*i;
	      ptIds[1] = 2*i + 1;
	      Majlines->InsertNextCell(2, ptIds);
	    }
	};
    }



  // Build the labels

  if (this->LabelVisibility)
    {

      if ( this->MajTicksMode ==2)
	{
	  BuildLabels(this->LabelActors,tmpMajLabelFlags);
	}
      else
	{
	  BuildLabels(this->LabelActors,tmpMajLocations.size());
	};

      list<vtkTextActorExtra*>::iterator tmpActIter=this->LabelActors.begin();
      bool intFormatFlag=IsIntFormat(this->LabelFormat);

      for(i=0;i<tmpMajLocations.size();i++)
	{
	  (*tmpActIter)->SetProperty(this->GetProperty());
	  (*tmpActIter)->GetTextProperty()->ShallowCopy(this->LabelTextProperty);

	  if ( ( this->MajTicksMode != 2) || (tmpMajLabelFlags[i]))
	    {
	      if ( !tmpMajLabelFormats.size())
		{
		  val = tmpMajLocations[i];
		  if (intFormatFlag)
		    {
		      sprintf(tmpString, this->LabelFormat, static_cast<int>(val));
		    }
		  else
		    {
		      sprintf(tmpString, this->LabelFormat, val);
		    };
		  chPtr=tmpString;
		  while( isspace(*chPtr) && (*chPtr!=0))
		    chPtr++;
		  
		  (*tmpActIter)->SetInput(chPtr);
		}
	      else
		{
		  (*tmpActIter)->SetInput(&tmpMajLabelFormats[i]); 
		};

	      
	      Majpts->GetPoint(2 * i , xTick);
	      xTick[0]-= tmpSign*(this->MajTickOffset+tmpLineWidth) * sin(theta);
	      xTick[1]+= tmpSign*(this->MajTickOffset+tmpLineWidth) * cos(theta);
	      //(*tmpActIter)->GetSize(viewport, stringSize);
	      
	      (*tmpActIter)->SetVertAlignmentOffset(this->MajTickLabelVertAlignmentOffset);
	      (*tmpActIter)->SetHorizAlignmentOffset(this->MajTickLabelHorizAlignmentOffset);
	      


	      /*
	      (*tmpActIter)->GetBBox(viewport,llx,lly,urx,ury);
	      stringSize[0]=urx-llx;
	      stringSize[1]=ury-lly;
	      */
	      this->SetOffsetPosition(xTick, 
				      theta, 
				      0,
				      -(this->LabelOffset+tmpLineWidth), 
				      this->LabelsFlip,
				      (*tmpActIter));
	      
	      tmpActIter++;
	    };
	}

      // Check if labels can be clipped. Since there is currently no
      // good way to clip a partial label, clip the entire label if it
      // crosses the clipping plane.
      if (this->ClipData)
        {
	  double *lblPos;
          tmpActIter=this->LabelActors.begin();
          while (tmpActIter!=this->LabelActors.end())
            {
              lblPos = (*tmpActIter)->GetPosition();

              if ((clipPos1[0] > lblPos[0] + FLT_EPSILON) || (clipPos2[0] < lblPos[0] -FLT_EPSILON) ||
                  (clipPos1[1] > lblPos[1] + FLT_EPSILON) || (clipPos2[1] < lblPos[1] -FLT_EPSILON))
                tmpActIter = this->LabelActors.erase (tmpActIter);
              else
                tmpActIter++;

            }
        };


    } // If labels visible







  vtkPoints *MinGridPts = vtkPoints::New();
  MinGridPts->SetDataTypeToDouble();
  vtkCellArray *MinGridLines = vtkCellArray::New();
  this->MinGrid->SetPoints(MinGridPts);
  this->MinGrid->SetLines(MinGridLines);
  MinGridPts->Delete();
  MinGridLines->Delete();


  



  

  //Create Min ticks

  vtkPoints *Minpts = vtkPoints::New();
  Minpts->SetDataTypeToDouble();
  vtkCellArray *Minlines = vtkCellArray::New();
  this->MinTicks->SetPoints(Minpts);
  this->MinTicks->SetLines(Minlines);
  Minpts->Delete();
  Minlines->Delete();

  tmpSign=1;


 
  if ( MinorTicksFlip )
    {
      tmpSign=-1;
    };


  if ((this->MinTicksVisibility) || (this->MinGridVisibility))
    {

      switch (this->MinTicksMode)
	{
	case 1:
	  if ( this->MajTicksMode !=2)
	    {
	      CalculateMinTicksLocationsByInterval(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MinTicksInterval,majRefTick,tmpMajLocations,tmpMinLocations);
	    }
	  else
	    {
	      CalculateMinTicksLocationsByInterval(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MinTicksInterval,this->MajTicksLocations,tmpMinLocations); 
	    }
	  break;
        case 2:
          CalculateTicksLocationsByLocations(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->MinTicksLocations,tmpMinLocations);
          break;
	case 0:
	default:
	  if ( this->MajTicksMode !=2)
	    {
	      CalculateMinTicksLocationsByCount(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->NumberOfMinTicks,majRefTick,tmpMajLocations,tmpMinLocations);
	    }
	  else
	    {
	      CalculateMinTicksLocationsByCount(this->AdjustedRange[this->RangeIndex],this->AdjustedRange[this->RangeIndex+1],this->NumberOfMinTicks,this->MajTicksLocations,tmpMinLocations);
	    };
	  break;
	}

      
      vector<double>::iterator tmpIter=tmpMinLocations.begin();


      //Map min ticks to the screen

      while( tmpIter != tmpMinLocations.end())
	{
        if (this->RangeIndex == 0)
          {
	  if ( MapScale(&this->AdjustedRange[0],p1[0],p2[0],this->Scale[0],this->MinLogLimit,*tmpIter,true,xTick[0])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point"<<endl;
	      tmpMinLocations.erase(tmpIter);
	      continue;
	    };
          xTick[1] = p1[1];
          }
        else
          {
	  if ( MapScale(&this->AdjustedRange[2],p1[1],p2[1],this->Scale[1],this->MinLogLimit,*tmpIter,true,xTick[1])<0)
	    {
	      //	      cerr<<"vtkAxisActor2DExtra::BuildAxis: Ignoring invalid Point"<<endl;
	      tmpMinLocations.erase(tmpIter);

	      continue;
	    };
          xTick[0] = p1[0];
          }


	    if (this->MinGridVisibility && !this->MinGridValues)
	      {
                double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
                double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
                double gridOffset[] = {gridStartPos[0] - p1[0], gridStartPos[1] - p1[1]};
                double gridLength[] = {gridStopPos[0] - gridStartPos[0], gridStopPos[1] - gridStartPos[1]};

                xGrid[0] = xTick[0] + (gridOffset[0]) * sin(theta);
                xGrid[1] = xTick[1] + (gridOffset[1]) * cos(theta);

                MinGridPts->InsertNextPoint(xGrid);

                xGrid[0] = xTick[0] + (gridLength[0]+gridOffset[0]) * sin(theta);
                xGrid[1] = xTick[1] + (gridLength[1]+gridOffset[1]) * cos(theta);

                MinGridPts->InsertNextPoint(xGrid);

	      };
	    
	    
	    if  (this->MinTicksVisibility)
	      {
		xTick[0] += tmpSign*(this->MinTickOffset+tmpLineWidth) * sin(theta);
		xTick[1] -= tmpSign*(this->MinTickOffset+tmpLineWidth) * cos(theta);
		
		Minpts->InsertNextPoint(xTick);
		
		
		xTick[0] = xTick[0] + tmpSign*this->MinTicksLength * sin(theta);
		xTick[1] = xTick[1] - tmpSign*this->MinTicksLength * cos(theta);
		Minpts->InsertNextPoint(xTick);
	      };
	    
	  tmpIter++;

	};

      if (this->MinGridVisibility)
	{
        if (!this->MinGridValues)
          {
	  for (i = 0; i < tmpMinLocations.size(); i++)
	    {
	      ptIds[0] = 2*i;
	      ptIds[1] = 2*i+ 1;
	      MinGridLines->InsertNextCell(2, ptIds);
	    }
          }
        else
          {
/*          vtkIdType *gridPtIds = new vtkIdType[this->NumOfMinGridSlices];
          for (int ii=0; ii<this->NumOfMinGridLines; ii++)
            {
            for (int jj=0; jj<this->NumOfMinGridSlices; jj++)
              {
              gridPtIds[jj] = MinGridPts->InsertNextPoint(this->MinGridValues[ii][jj]);
              }
            MinGridLines->InsertNextCell (this->NumOfMinGridSlices, gridPtIds);
            }
            delete []gridPtIds;
          }
*/
           vtkIdType *gridPtIds = new vtkIdType[this->NumOfMinGridSlices];
           double *gridStartPos = this->GridStart->GetComputedDoubleViewportValue(viewport);
           double *gridStopPos = this->GridStop->GetComputedDoubleViewportValue(viewport);
           for (int ii=0; ii<this->NumOfMinGridLines; ii++)
             {
             for (int jj=0; jj<this->NumOfMinGridSlices; jj++)
               {
               MapScale(&this->AdjustedRange[0],gridStartPos[0],gridStopPos[0],this->Scale[0],this->MinLogLimit,this->MinGridValues[ii][jj][0],false,xGrid[0]);
               MapScale(&this->AdjustedRange[2],gridStartPos[1],gridStopPos[1],this->Scale[1],this->MinLogLimit,this->MinGridValues[ii][jj][1],false,xGrid[1]);

               gridPtIds[jj] = MinGridPts->InsertNextPoint(xGrid);
               }
             MinGridLines->InsertNextCell (this->NumOfMinGridSlices, gridPtIds);
             }
             delete []gridPtIds;
           }
	};
      
      if ( this->MinTicksVisibility)
	{
	  for (i = 0; i < tmpMinLocations.size(); i++)
	    {
	      ptIds[0] = 2*i;
	      ptIds[1] = 2*i+ 1;
	      Minlines->InsertNextCell(2, ptIds);
	    }
	};
    }
  








  
  // Now build the title

  if (this->TitleVisibility)
    {

      double llx,lly,urx,ury;
      this->TitleActor->GetBBox(viewport,llx,lly,urx,ury);
      stringSize[0]=urx-llx;
      stringSize[1]=ury-lly;

      
      pts->GetPoint(0 , p1);
      pts->GetPoint(1 , p2);
      xTick[0] = p1[0] + (p2[0] - p1[0]) / 2.0;
      xTick[1] = p1[1] + (p2[1] - p1[1]) / 2.0;
      double actualParallelOffset = sqrt(deltaX*deltaX+deltaY*deltaY)*this->TitleParallelOffset;

      
      this->SetOffsetPosition(xTick, 
			      theta, 
			      -actualParallelOffset,
			      -(this->TitlePerpendicularOffset+tmpLineWidth),
			      this->TitleFlip,
			      this->TitleActor);

      if (this->ClipData)
        {
          double *lblPos;
          lblPos = this->TitleActor->GetPosition();

          if ((clipPos1[0] > lblPos[0] + FLT_EPSILON) || (clipPos2[0] < lblPos[0] - FLT_EPSILON) ||
              (clipPos1[1] > lblPos[1] + FLT_EPSILON) || (clipPos2[1] < lblPos[1] - FLT_EPSILON))
            this->TitleActor->SetPosition (-1, -1);
        };


      
    } // If title visible
  
  this->BuildTime.Modified();
}






//----------------------------------------------------------------------------
#define VTK_NUM_DIVS 11
void vtkAxisActor2DExtra::ComputeRange(double inRange[2], 
				       double outRange[2],
				       int inNumTicks, 
				       int &numTicks, 
				       double &interval,
				       int inScale,
				       double inMinLogLimit)
{
  static double divs[VTK_NUM_DIVS] = {10.0, 8.0, 5.0, 4.0, 2.0, 1.0,
                                     0.5, 0.25, 0.20, 0.125, 0.10};

  if (inScale == 1 )
    {
      if (inRange[0] <=0 )
	inRange[0]=inMinLogLimit;
      if ( inRange[1] <=0 )
	inRange[1]=inMinLogLimit;
    }


  double range = fabs(inRange[1]-inRange[0]), sRange[2];
  double logFactor = (double)pow((double)(10.0), (double)(floor(log10(range))));
  double lastInterval;
  int j;

  // Handle the range
  if ( inRange[0] < inRange[1] )
    {
    sRange[0] = inRange[0];
    sRange[1] = inRange[1];
    }
  else if ( inRange[0] > inRange[1] )
    {
    sRange[1] = inRange[0];
    sRange[0] = inRange[1];
    }
  else // they're equal, so perturb them by 1 percent
    {
      double perturb = 100.;
      if (inRange[0] == 0.0)
	{ // if they are both zero, then just perturb about zero
	  sRange[0] = -1/perturb;
	  sRange[1] = 1/perturb;
	}
      else
	{
	  sRange[0] = inRange[0] - inRange[0]/perturb;
	  sRange[1] = inRange[0] + inRange[0]/perturb;
	}

      //recompute range
      range = fabs(sRange[1]-sRange[0]);
      //recompute logfactor
      logFactor = (double)pow((double)(10.0), (double)(floor(log10(range))));
      //recompute inRange so things get flipped correctly at the end
      inRange[0] = sRange[0];
      inRange[1] = sRange[1];
    }

  // let's figure out the interval. We'll loop until we find something with the
  // right number of tick marks.
  lastInterval = logFactor * divs[0];
  for ( j=1; j < VTK_NUM_DIVS; j++ )
    {
    interval = logFactor * divs[j];
    if ( ((inNumTicks-1)*interval) < range )
      {
      break;
      }
    lastInterval = interval;
    }
  interval = lastInterval;

  numTicks = (int) ceil(range/interval) + 1; //trim number of extra ticks

  // Get the start value of the range
  for ( j=0; j < VTK_NUM_DIVS; j++ )
    {
    outRange[0] = ((int)floor(sRange[0]/(logFactor*divs[j]))) * logFactor*divs[j];

    if (inScale == 1 )
      {
	if (outRange[0] <=0 )
	  outRange[0]=inMinLogLimit;
      }
    
    

    if ( (outRange[0] + (numTicks-1)*interval) >= sRange[1] )
      {
      break;
      }
    }

  // Get the end value of the range
  outRange[1] = outRange[0] + (numTicks-1)*interval;

  // Adust if necessary
  if ( inRange[0] > inRange[1] )
    {
    sRange[0] = outRange[1];
    outRange[1] = outRange[0];
    outRange[0] = sRange[0];
    interval = -interval;
    }

}
#undef VTK_NUM_DIVS

//----------------------------------------------------------------------------
// Position text with respect to a point (xTick) where the angle of the line
// from the point to the center of the text is given by theta. The offset
// is the spacing between ticks and labels.
void vtkAxisActor2DExtra::SetOffsetPosition(double xTick[3], double theta, 
					    double inXOffset,double inYOffset,
					    int inAnnotationsFlip,
					    vtkActor2D *actor) 
{
  double pos[2];
  double radAngle=fmod(theta,2*PI);
  
  if (radAngle< 0)
    radAngle = 2*PI +radAngle;

  if ( inAnnotationsFlip)
    inYOffset =-inYOffset;


  pos[0] = xTick[0] + inXOffset*cos(theta)- inYOffset*sin(theta);
  pos[1] = xTick[1] + inXOffset*sin(theta)+ inYOffset*cos(theta);

  /*
  if ( stringHeight > 2 )
    stringHeight -= 1;


  if ((( radAngle>= 0 && radAngle <= (PI/4.0) ))
      ||
      (  ( radAngle> (7.0*PI/4.0) && radAngle <= (2.0*PI))))
    {

      pos[0] -= stringWidth/2;
      if ( ! inAnnotationsFlip)
	{
	  pos[1] -=stringHeight;
	};
    }
  else if (( radAngle> (PI/4.0)) && (radAngle <= (3.0*PI/4.0) ))
    {
      pos[1] -=stringHeight/2;
      if ( inAnnotationsFlip )
	{
	  pos[0] -= stringWidth;
	};
    }
  else if ( ( radAngle> (3.0*PI/4.0)) && (radAngle <= (5.0*PI/4.0) ))
    {
      pos[0] -=stringWidth/2;
      if ( inAnnotationsFlip )
	{
	  pos[1] -=stringHeight;
	};
    }
  else 
    {
      if ( inAnnotationsFlip)
	{
	  pos[0] -=stringWidth;
	};
      pos[1] -=stringHeight/2; 
    }
  */
  actor->SetPosition(pos[0], pos[1]);

}

//----------------------------------------------------------------------------
void vtkAxisActor2DExtra::ShallowCopy(vtkProp *prop)
{
  vtkAxisActor2DExtra *a = vtkAxisActor2DExtra::SafeDownCast(prop);
  if ( a != NULL )
    {
      this->SetRange(a->GetRange(), a->GetRangeIndex ());
      this->SetNumberOfMajTicks(a->GetNumberOfMajTicks());
      this->SetNumberOfMinTicks(a->GetNumberOfMinTicks());
      this->SetLabelFormat(a->GetLabelFormat());
      this->SetMajTicksLength(a->GetMajTicksLength());
      this->SetMinTicksLength(a->GetMinTicksLength());
      this->SetMajTickOffset(a->GetMajTickOffset());
      this->SetMinTickOffset(a->GetMinTickOffset());
      this->SetLabelsFlip(a->GetLabelsFlip());
      this->SetTitleFlip(a->GetTitleFlip());
      this->SetMajorTicksFlip(a->GetMajorTicksFlip());
      this->SetMinorTicksFlip(a->GetMinorTicksFlip());
      this->SetLabelOffset(a->GetLabelOffset());
      this->SetTitleParallelOffset(a->GetTitleParallelOffset());
      this->SetTitlePerpendicularOffset(a->GetTitlePerpendicularOffset());
      this->SetAxisVisibility(a->GetAxisVisibility());
      this->SetMajTicksVisibility(a->GetMajTicksVisibility());
      this->SetMinTicksVisibility(a->GetMinTicksVisibility());
      this->SetLabelVisibility(a->GetLabelVisibility());
      this->SetTitleVisibility(a->GetTitleVisibility());
      //      this->SetTitleFontSize(a->GetTitleFontSize());
      //      this->SetLabelFontSize(a->GetLabelFontSize());
      this->SetLabelTextProperty(a->GetLabelTextProperty());
    }

  // Now do superclass
  this->vtkActor2D::ShallowCopy(prop);
}

int vtkAxisActor2DExtra::MapScale(double* inDataRange, 
				  double inPixelRangeMin, 
				  double inPixelRangeMax,
				  int inScale,
				  double inMinLogLimit,
				  double inValue,
				  bool clip,
				  double& outValue)
{

  if (clip &&
      (( (inDataRange[0] < inDataRange[1]) && (inValue< inDataRange[0] || inValue>inDataRange[1]))
      ||
      ( (inDataRange[0] > inDataRange[1]) && (inValue> inDataRange[0] || inValue<inDataRange[1] )))
      /*      ||
      (inPixelRangeMin <0)
      ||
      (inPixelRangeMax <0)
      */
      ||
      ( ( inDataRange[0] == inDataRange[1]) && (inPixelRangeMin != inPixelRangeMax))
      )
    {
      return -1;
    };
  
  if (inDataRange[0] == inDataRange[1])
    {
      outValue=inPixelRangeMin;
      return 1;
    };

  switch(inScale)
    {
    case 1: // log scale
      if (inValue <=0 )
	return -1;
      double tmpDataRange[2];
      tmpDataRange[0]=inDataRange[0];
      tmpDataRange[1]=inDataRange[1];
      if ( tmpDataRange[0] <=0)
	{
	  if ( inValue > inMinLogLimit)
	    tmpDataRange[0]=inMinLogLimit;
	  else
	    tmpDataRange[0]=inValue;
	};

      if (tmpDataRange[1] <=0 )
	{
	  tmpDataRange[1]=inMinLogLimit;
	  if ( inValue > inMinLogLimit )
	    tmpDataRange[1]=inMinLogLimit;
	  else
	    tmpDataRange[1]=inValue;
	};
	
      outValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(log10(inValue)-log10(tmpDataRange[0]))/(log10(tmpDataRange[1])-log10(tmpDataRange[0]));
      break;
    case 0: // linear scale
    default:
      outValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(inValue-inDataRange[0])/(inDataRange[1]-inDataRange[0]);
      break;
    };

  return 1;

};

int vtkAxisActor2DExtra::SetRange(double* inRange, int index)
{
  if(inRange) 
    {
      if (( inRange[0] != inRange[1]) && ( inRange[2] != inRange[3]))
	{
	  Range[0]=inRange[0]; 
	  Range[1]=inRange[1];
          Range[2]=inRange[2];
          Range[3]=inRange[3];
	  AdjustRange();

          this->RangeIndex = index;

	  this->Modified();
	  return 1;
	};
    };
  return 0;
};
int vtkAxisActor2DExtra::SetRange(double inMinX, double inMaxX, double inMinY, double inMaxY, int index)
{
  if (( inMinX != inMaxX) && ( inMinY != inMaxY))
    {
      Range[0]=inMinX;
      Range[1]=inMaxX;
      Range[2]=inMinY;
      Range[3]=inMaxY;
      AdjustRange();

      this->RangeIndex = index;
      this->Modified();
      return 1;
    };

  return 0;
};



void vtkAxisActor2DExtra::CalculateMajTicksLocationsByInterval(double inMinRange, double inMaxRange,double inInterval,vector<double>& outValues,double& outMajReferenceTick )
{

  outValues.clear();
  
  if (inInterval<=0)
    return;


  // Create tick locations
  int    count;
  double tmpVal;
      
  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };
  
  
  count=static_cast<int>(inMinRange/inInterval);
  
  tmpVal = count * inInterval;;

  outMajReferenceTick=tmpVal;

  if  ( tmpVal < inMinRange) 
    {
      tmpVal+=inInterval;
    }
  else
    {
      outMajReferenceTick -=inInterval;
    };
  
  while(tmpVal<=inMaxRange)
    {
      outValues.push_back(tmpVal);
      tmpVal += inInterval;
    };
  
};



void vtkAxisActor2DExtra::CalculateMajTicksLocationsByCount(double inMinRange, double inMaxRange,int inCount,vector<double>& outValues)
{

  outValues.clear();
  
  if (inCount<=0)
    return;


  // Create tick locations
  
  double tmpVal;

  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };
  
  if ( inCount > 1 )
    {
      tmpVal = (inMaxRange - inMinRange)/(inCount -1 );
    };
  
  
  outValues.resize(inCount);
  
  for(int indx=0;indx<(inCount-1);indx++)
    outValues[indx]=inMinRange+indx*tmpVal;
  
  outValues[inCount-1]=inMaxRange;


};



void vtkAxisActor2DExtra::CalculateMinTicksLocationsByInterval(double inMinRange, double inMaxRange,double inInterval,double inMajReferenceTick,const vector<double>& inMajValues,vector<double>& outValues)
{ 
  outValues.clear();
  
  if (inInterval<=0) 
    return;

  // Create tick locations

  double tmpVal;
  double tmpMax;
  int curMajTick=0;

  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };
  



  tmpVal=inMajReferenceTick+inInterval;

  while(tmpVal < inMinRange)
    tmpVal +=inInterval;

  if ( inMajValues.size() > 0 )
    {
      tmpMax=inMajValues[0];
    }
  else 
    {
      tmpMax=inMaxRange;
    };


  while(tmpVal<=inMaxRange)
    {
      if (( tmpVal <tmpMax) || (( tmpVal == tmpMax) && (curMajTick>=(static_cast<long>(inMajValues.size())-1))))
	{
	  outValues.push_back(tmpVal);
	}
      else 
	{
	  if ( curMajTick >= (static_cast<long>(inMajValues.size())-1))
	    {
	      tmpMax=inMaxRange;
	    }
	  else 
	    {
	      tmpVal =inMajValues[curMajTick];
	      tmpMax =inMajValues[++curMajTick];
	    };
	};
      tmpVal +=inInterval;

    };


};






void vtkAxisActor2DExtra::CalculateMinTicksLocationsByCount(double inMinRange, double inMaxRange,double inCount,double inMajReferenceTick,const vector<double>& inMajValues,vector<double>& outValues)
{ 
  outValues.clear();
  
  if ((inCount<=0) || ( inMajValues.size() < 2 ) )
    return;

  // Create tick locations

  int count=0;
  double tmpVal;
  double tmpMax;
  int curMajTick=0;
  double tmpInterval;


  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };
  
  
  tmpInterval = (inMajValues[1]-inMajValues[0])/(inCount+1);


  tmpVal=inMajReferenceTick+tmpInterval;

  while(tmpVal < inMinRange)
    tmpVal +=tmpInterval;

  if ( inMajValues.size() > 0 )
    {
      tmpMax=inMajValues[0];
    }
  else 
    {
      tmpMax=inMaxRange;
    };

  
  while(tmpVal<=inMaxRange)
    {
      if ( (( tmpVal <tmpMax) || (( tmpVal == tmpMax) && (curMajTick>=(static_cast<long>(inMajValues.size())-1)))) && (count < inCount))
	{
	  outValues.push_back(tmpVal);
	  count++;
	}
      else 
	{
	  count =0;
	  if ( curMajTick >= (static_cast<long>(inMajValues.size())-1))
	    {
	      tmpMax=inMaxRange;
	    }
	  else 
	    {
	      tmpVal =inMajValues[curMajTick];
	      tmpMax =inMajValues[++curMajTick];
	    };
	};
      tmpVal +=tmpInterval;

    };


};







void vtkAxisActor2DExtra::CalculateMinTicksLocationsByCount(double inMinRange, double inMaxRange,
							    int inCount,const vector<double>& inMajValues,vector<double>& outValues)
{ 
  outValues.clear();
  
  if (inCount<=0)
    return;

  // Create tick locations

  double tmpVal,tmpInt;
  double tmpMin,tmpMax;
  
  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };
  
  for(int indx = 0 ;indx<(static_cast<long>(inMajValues.size())-1);indx++)
    {
      tmpMin=inMajValues[indx];
      tmpMax=inMajValues[indx+1];
      tmpInt=(tmpMax-tmpMin)/(inCount+1);


      for(int indx1 = 1 ;indx1<=inCount;indx1++)
	{
	  tmpVal=tmpMin+indx1*tmpInt;

	  if (( tmpVal >= inMinRange) && (tmpVal <=inMaxRange))
	    {
	      outValues.push_back(tmpVal);
	    };
	};
    };

};



void vtkAxisActor2DExtra::BuildLabels(list<vtkTextActorExtra*>& inLabelActors, unsigned int inCount)
{
  if ( inCount < inLabelActors.size() )
    {
      while(inLabelActors.size() > inCount)
	{
	  inLabelActors.front()->Delete();
	  inLabelActors.pop_front();
	};
    }
  else if ( inCount > inLabelActors.size() )
    {
      vtkTextActorExtra* tmpAct;

      while( inLabelActors.size() < inCount)
	{
	  tmpAct = vtkTextActorExtra::New();
	  inLabelActors.push_front(tmpAct);
	}
    };


  for (list<vtkTextActorExtra*>::iterator tmpIter=inLabelActors.begin();tmpIter!=inLabelActors.end();tmpIter++)
    {
	(*tmpIter)->Clear();
    }

};


void vtkAxisActor2DExtra::AdjustRange()
{
  
  switch(this->Scale[0])
    {
    case 1:
      if (this->Range[0] > 0 )
	this->AdjustedRange[0]=this->Range[0];
      else
	{
	  this->AdjustedRange[0]=this->MinLogLimit;
	};
      if (this->Range[1] > 0 )
	this->AdjustedRange[1]=this->Range[1];
      else
	{
	  this->AdjustedRange[1]=this->MinLogLimit;
	};
      break;
    case 0:
    default:
      this->AdjustedRange[0]=this->Range[0];
      this->AdjustedRange[1]=this->Range[1];      
      break;
    };
  switch(this->Scale[1])
    {
    case 1:
      if (this->Range[2] > 0 )
        this->AdjustedRange[2]=this->Range[2];
      else
        {
          this->AdjustedRange[2]=this->MinLogLimit;
        };
      if (this->Range[3] > 0 )
        this->AdjustedRange[3]=this->Range[3];
      else
        {
          this->AdjustedRange[3]=this->MinLogLimit;
        };
      break;
    case 0:
    default:
      this->AdjustedRange[2]=this->Range[2];
      this->AdjustedRange[3]=this->Range[3];
      break;
    };

  
  if ( this->AdjustedRange[0] == this->AdjustedRange[1] )
    {
      if ( this->AdjustedRange[0] == 0 )
	{
	  this->AdjustedRange[1]=1.0;
	}
      else
	{
	  this->AdjustedRange[1]*=1.1; 
	}
      
    };

  if ( this->AdjustedRange[2] == this->AdjustedRange[3] )
    {
      if ( this->AdjustedRange[2] == 0 )
        {
          this->AdjustedRange[3]=1.0;
        }
      else
        {
          this->AdjustedRange[3]*=1.1;
        }

    };


};

void vtkAxisActor2DExtra::SetScale(int inScaleX, int inScaleY)
{
  this->Scale[0]=inScaleX;
  this->Scale[1]=inScaleY;

  AdjustRange();

  this->Modified();
};

void vtkAxisActor2DExtra::GetScale(int &outScaleX, int &outScaleY)
{
  outScaleX = this->Scale[0];
  outScaleY = this->Scale[1];

  AdjustRange();

  this->Modified();
};



int  vtkAxisActor2DExtra::SetMajTicksLocations(const vector<double>& inMajLocations,const vector<bool>& inLabelLocations, const vector<vtkTextActorFormat>& inLabelFormats )
{


  this->MajTicksLocations.resize(inMajLocations.size());
  this->MajTicksLabelLocations.resize(inLabelLocations.size());
  this->MajTicksLabelFormats.resize(inLabelFormats.size());

  for(unsigned int indx=0;indx<inMajLocations.size();indx++)
    {
      this->MajTicksLocations[indx]=inMajLocations[indx];

      if ( indx <inLabelLocations.size())
	{
	  this->MajTicksLabelLocations[indx]=inLabelLocations[indx];
	};

      if ( indx < inLabelFormats.size())
	{
	  this->MajTicksLabelFormats[indx]=inLabelFormats[indx];
	};
    }

  
  this->Modified();
  return 1;
};

int vtkAxisActor2DExtra::SetMinTicksLocations(const vector<double>& inMinLocations)
{


  this->MinTicksLocations.resize(inMinLocations.size());

  for(unsigned int indx=0;indx<inMinLocations.size();indx++)
    {
      this->MinTicksLocations[indx]=inMinLocations[indx];
    }

  this->Modified();
  return 1;
};


void vtkAxisActor2DExtra::CalculateMajTicksLocationsByLocations(double inMinRange, double inMaxRange,const vector<double>& inTickLocations,const vector<bool>& inLabelLocations,const vector<vtkTextActorFormat>& inLabelFormats,vector<double>& outTickLocations,vector<bool>& outLabelLocations,vector<vtkTextActorFormat>& outLabelFormats)
{
  double tmpVal;
  int tickCount=0;
  
  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };
  
  outTickLocations.resize(inTickLocations.size());
  outLabelLocations.resize(inTickLocations.size());

  if ( inLabelFormats.size())
    {
      outLabelFormats.resize(inTickLocations.size());
    }
  else
    {
      outLabelFormats.resize(0) ;
    };

  for(unsigned int indx=0;indx<inTickLocations.size();indx++)
    {
      if (( inTickLocations[indx] >= inMinRange) && ( inTickLocations[indx] <= inMaxRange) )
	{
	  outTickLocations[tickCount]=inTickLocations[indx];
	  if (indx<inLabelLocations.size())
	    {
	      outLabelLocations[tickCount]=inLabelLocations[indx];
	    }
	  else
	    {
	      outLabelLocations[tickCount]=false;
	    }

	  
	  if (indx<inLabelFormats.size())
	    {
	      outLabelFormats[tickCount]=inLabelFormats[indx];
	    }
	  else if ( inLabelFormats.size() )
	    {
	      vtkTextActorFormat tmpFormat;
	      outLabelFormats[tickCount]=tmpFormat;
	    };

	  tickCount++;
	};
    };

  outTickLocations.resize(tickCount);
  outLabelLocations.resize(tickCount);
  if ( inLabelFormats.size())
    {
      outLabelFormats.resize(tickCount);
    };
};







void vtkAxisActor2DExtra::CalculateTicksLocationsByLocations(double inMinRange, double inMaxRange,const vector<double>& inTickLocations,vector<double>& outTickLocations)
{
  double tmpVal;
  int tickCount=0;
  
  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };
  
  outTickLocations.resize(inTickLocations.size());


  for(unsigned int indx=0;indx<inTickLocations.size();indx++)
    {
      if (( inTickLocations[indx] >= inMinRange) && ( inTickLocations[indx] <= inMaxRange) )
	{
	  outTickLocations[tickCount]=inTickLocations[indx];
	  tickCount++;
	};
    };

  outTickLocations.resize(tickCount);

};



void vtkAxisActor2DExtra::BuildLabels(list<vtkTextActorExtra*>& inLabelActors, const vector<bool>& inLabelFlags)
{

  int tmpCount=0;

  for(unsigned int indx=0;indx<inLabelFlags.size();indx++)
    { 
      if ( inLabelFlags[indx])
	tmpCount++;
    };

  BuildLabels(inLabelActors,tmpCount);

};

void vtkAxisActor2DExtra::CalculateMinTicksLocationsByInterval(double inMinRange, double inMaxRange,
								double inInterval,
								const vector<double>& inMajValues,
								vector<double>& outValues)
{

  outValues.clear();
  
  if (inMajValues.size() <2)
    return;

  // Create tick locations

  double tmpMin,tmpMax;
  double tmpVal;


  if ( inMinRange> inMaxRange)
    {
      tmpVal=inMinRange;
      inMinRange=inMaxRange;
      inMaxRange=tmpVal;
    };


  for(int indx = 0 ;indx<(static_cast<long>(inMajValues.size())-1);indx++)
    {
      tmpMin=inMajValues[indx];
      tmpMax=inMajValues[indx+1];
      tmpVal = tmpMin+inInterval;
      while( tmpVal<tmpMax)
	{
	  if (( tmpVal >= inMinRange) && (tmpVal <=inMaxRange))
	    {
	      outValues.push_back(tmpVal);
	    };
	  tmpVal+=inInterval;
	};
    };
};


bool vtkAxisActor2DExtra::IsIntFormat(char * inFormatString)
{

  char * tmpP=inFormatString;
  bool retVal=false;

  if (!tmpP)
    return false;

  while ( *tmpP && *tmpP != '%' )
    tmpP++;
    
  if ( *tmpP == '%' )
    {
    tmpP++;
    while(*tmpP)
      {
      if (( *tmpP == 'd') || (*tmpP == 'i') || (*tmpP == 'o') || (*tmpP == 'x') || (*tmpP == 'u') || (*tmpP == 'c') || (*tmpP == 'X') )
	{
	retVal=true;
	break;
	}
      else if (( *tmpP == 'e') || (*tmpP == 'E') || (*tmpP == 'f') || (*tmpP == 'F') || (*tmpP == 'g') || (*tmpP == 'G') || (*tmpP == 'a') || (*tmpP == 'A')|| (*tmpP == 'c') || (*tmpP == 's')|| (*tmpP == 'C') || (*tmpP == 'S') || (*tmpP == 'p') || (*tmpP == 'n')|| (*tmpP == 'z') || (*tmpP == '%') )
        {
	break;
	};
      tmpP++;
      };
    };

  return retVal;

}

void vtkAxisActor2DExtra::SetMajTickLabelVertAlignmentOffset(double inAlignment)
{
  if (( inAlignment >= 0.0 ) && (inAlignment<=1.0 ))
    {    
      this->MajTickLabelVertAlignmentOffset=inAlignment;
      
      
      this->Modified();
    };
}

void vtkAxisActor2DExtra::SetMajTickLabelHorizAlignmentOffset(double inAlignment)
{
  if (( inAlignment >= 0.0 ) && (inAlignment<=1.0 ))
    {    
      this->MajTickLabelHorizAlignmentOffset=inAlignment;
      
      
      this->Modified();
    };
}

void vtkAxisActor2DExtra::SetClipData (int flag)
{
#ifndef NOCLIP

  if (ClipData == flag)
     return;

  ClipData = flag;

  if (!this->ClipData)
    {
      this->AxisMapper->RemoveAllClippingPlanes ();
      this->MajTicksMapper->RemoveAllClippingPlanes ();
      this->MinTicksMapper->RemoveAllClippingPlanes ();
/*      this->MajGridMapper->RemoveAllClippingPlanes ();
      this->MinGridMapper->RemoveAllClippingPlanes ();
*/
    }
  else
    {
      for (int ii=0; ii<4; ii++)
        {
          this->AxisMapper->AddClippingPlane (this->ClipPlanes[ii]);
          this->MajTicksMapper->AddClippingPlane (this->ClipPlanes[ii]);
          this->MinTicksMapper->AddClippingPlane (this->ClipPlanes[ii]);
/*          this->MajGridMapper->AddClippingPlane (this->ClipPlanes[ii]);
          this->MinGridMapper->AddClippingPlane (this->ClipPlanes[ii]);
*/
        }
    }

  this->Modified ();
#endif
}



