// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkChipsTransform.h,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#ifndef __vtkChipsTransform_hh
#define __vtkChipsTransform_hh

#include "vtkWarpTransform.h"
#include "Transform.hh"

class VTK_COMMON_EXPORT vtkChipsTransform : public vtkWarpTransform
{
public:
  static vtkChipsTransform *New();
  vtkTypeRevisionMacro(vtkChipsTransform,vtkWarpTransform);
  virtual void PrintSelf(ostream& os, vtkIndent indent);

  void SetChipsTransform (Transform *trans);
  Transform * GetChipsTransform () {return chipsTransform;} ;


  // Description:
  // Make another transform of the same type.
  vtkAbstractTransform *MakeTransform();

protected:
  vtkChipsTransform();
  ~vtkChipsTransform();

  // Description:
  // Copy this transform from another of the same type.
  void InternalDeepCopy(vtkAbstractTransform *transform);

  // Description:
  // Internal functions for calculating the transformation.
  void ForwardTransformPoint(const float in[3], float out[3]);
  void ForwardTransformPoint(const double in[3], double out[3]);

  void ForwardTransformDerivative(const float in[3], float out[3],
                                  float derivative[3][3]);
  void ForwardTransformDerivative(const double in[3], double out[3],
                                  double derivative[3][3]);

  void InverseTransformPoint(const float in[3], float out[3]);
  void InverseTransformPoint(const double in[3], double out[3]);

  void InverseTransformDerivative(const float in[3], float out[3],
                                  float derivative[3][3]);
  void InverseTransformDerivative(const double in[3], double out[3],
                                  double derivative[3][3]);

  dmDataType GetTransformDataType (double *) {return dmDOUBLE;};
  dmDataType GetTransformDataType (float *) {return dmFLOAT;};

private:

  template<class T>
  void vtkApplyTransform(const T inPoint[3], T outPoint[3],
                                        T derivative[3][3]);
  template<class T>
  void vtkApplyInverse(const T inPoint[3], T outPoint[3],
                                        T derivative[3][3]);

  vtkChipsTransform(const vtkChipsTransform&); // Not implemented.
  void operator=(const vtkChipsTransform&); // Not implemented.

  Transform *chipsTransform;

};

#endif

