// 
//  Copyright (C) 2005-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <math.h>
#include <bitset>

#include "vtkPlanes.h"
#include "vtkPlane.h"
#include "vtkAppendPolyData.h"
#include "vtkGlyph2D.h"
#include "vtkGlyphSource2D.h"
#include "vtkIntArray.h"
#include "vtkCellArray.h"
#include "vtkObjectFactory.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkDoubleArray.h"
#include "vtkPlanes.h"
#include "vtkProperty2D.h"
#include "vtkTriangleFilter.h"
#include "vtkHistogramDataActor2DExtra.hh"

#include "vtkMath.h"

#ifndef MINLOG
#define MINLOG 0.01
#endif


vtkCxxRevisionMacro(vtkHistogramDataActor2DExtra, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkHistogramDataActor2DExtra);



//----------------------------------------------------------------------------
// Instantiate this object.
vtkHistogramDataActor2DExtra::vtkHistogramDataActor2DExtra()
{
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(0.0, 0.0);

  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(1, 1);
  this->Position2Coordinate->SetReferenceCoordinate(NULL);
  
  this->MinLogXLimit=MINLOG;
  this->MinLogYLimit=MINLOG;

  this->XScale =0;
  this->YScale =0;
  this->DataFlip=0;


  this->XRange[0] = 0.0;
  this->XRange[1] = 1.0;

  this->YRange[0] = 0.0;
  this->YRange[1] = 1.0;

#ifndef NOCLIP
  // Create data clippers
  for(int indx=0;indx<4;indx++)
    {
      ClipPlanes[indx]=vtkPlane::New();
    };

  // initialize clipping plane locations
  this->ClipPlanes[0]->SetOrigin(0,0,0);
  this->ClipPlanes[0]->SetNormal(1,0,0);
  this->ClipPlanes[1]->SetOrigin(1,0,0);
  this->ClipPlanes[1]->SetNormal(-1,0,0);
  this->ClipPlanes[2]->SetOrigin(0,0,0);
  this->ClipPlanes[2]->SetNormal(0,1,0);
  this->ClipPlanes[3]->SetOrigin(0,1,0);
  this->ClipPlanes[3]->SetNormal(0,-1,0);
#endif

  this->PlotLineData = vtkPolyData::New();

  this->PlotLineDataMapper = vtkPolyDataMapper2D::New();
  this->PlotLineDataMapper->SetInput (this->PlotLineData);

#ifndef NOCLIP
  // Using mapper AddClippingPlane function will not work because clipped
  // data gets messed up in the gl2ps output. Must use clip filter instead
  
  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[3]);
  
#endif

  this->PlotLineDataActor = vtkActor2D::New();
  this->PlotLineDataActor->SetMapper(this->PlotLineDataMapper);



  this->PlotFillData = vtkPolyData::New();

  this->triangleFilter = vtkTriangleFilter::New();
  this->triangleFilter->SetInput(this->PlotFillData);
  this->triangleFilter->ReleaseDataFlagOn ();

  this->PlotFillDataMapper = vtkPolyDataMapper2D::New();
  this->PlotFillDataMapper->SetInputConnection (this->triangleFilter->GetOutputPort());

#ifndef NOCLIP

  this->PlotFillDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->PlotFillDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->PlotFillDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->PlotFillDataMapper->AddClippingPlane (ClipPlanes[3]);
#endif
  this->PlotFillDataActor = vtkActor2D::New();
  this->PlotFillDataActor->SetMapper(this->PlotFillDataMapper);


  this->PlotPointDataSet = vtkPolyData::New();

  this->PlotPointData = vtkGlyph2D::New();
  this->PlotPointData->SetInput(this->PlotPointDataSet);

  this->PlotPointDataMapper = vtkPolyDataMapper2D::New();
  this->PlotPointDataMapper->SetInputConnection(this->PlotPointData->GetOutputPort ());

#ifndef NOCLIP

  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[3]);

#endif
  this->PlotPointDataActor = vtkActor2D::New();
  this->PlotPointDataActor->SetMapper(this->PlotPointDataMapper);

  this->PlotPointData->ScalingOff();
  this->PlotPointData->SetIndexModeToOff();
//  this->PlotPointData->SetNumberOfSources(1);
  this->GlyphSource = vtkGlyphSource2D::New();
  this->GlyphSource->SetColor(1,1,0);
  this->GlyphSource->SetGlyphType(1);
  this->GlyphSource->SetScale(10);
  this->GlyphSource->SetRotationAngle(0);
  this->GlyphSource->SetFilled(1);

  this->PlotPointData->SetSource(this->GlyphSource->GetOutput());
  this->PlotPointDataActor->GetProperty()->SetColor(1,0,0);

  this->ErrData = vtkPolyData::New();

  this->ErrDataMapper = vtkPolyDataMapper2D::New();
  this->ErrDataMapper->SetInput(this->ErrData);

#ifndef NOCLIP

  this->ErrDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->ErrDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->ErrDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->ErrDataMapper->AddClippingPlane (ClipPlanes[3]);

#endif
  this->ErrDataActor = vtkActor2D::New();
  this->ErrDataActor->SetMapper(this->ErrDataMapper);

  this->LinesVisibility=1;
  this->FillVisibility=1;
  this->PointsVisibility=1;
  this->UpErrBarVisibility=1;
  this->DownErrBarVisibility=1;
  this->LeftErrBarVisibility=1;
  this->RightErrBarVisibility=1;
  this->ErrBarCapVisibility=1;

  this->ErrBarCapLength=10;


  this->LastPosition[0] = this->LastPosition[1] = 0;
  this->LastPosition2[0] = this->LastPosition2[1] = 0;

  this->DataSet=vtkPolyData::New();
  this->DataSet->Initialize();
  
  this->BinsDataSet=vtkDoubleArray::New();
  this->BinsDataSet->Initialize();


  this->UpDataSet=vtkDoubleArray::New();
  this->UpDataSet->Initialize();

  this->DownDataSet=vtkDoubleArray::New();
  this->DownDataSet->Initialize();

  this->LeftDataSet=vtkDoubleArray::New();
  this->LeftDataSet->Initialize();

  this->RightDataSet=vtkDoubleArray::New();
  this->RightDataSet->Initialize();

  this->Style=0;
}

//----------------------------------------------------------------------------
vtkHistogramDataActor2DExtra::~vtkHistogramDataActor2DExtra()
{

  this->PlotLineData->Delete();
  this->PlotLineDataMapper->Delete();
  this->PlotLineDataActor->Delete();

  this->PlotFillData->Delete();
  this->PlotFillDataMapper->Delete();
  this->PlotFillDataActor->Delete();
  this->triangleFilter->Delete ();
  
  this->PlotPointDataSet->Delete();

  this->PlotPointData->Delete();
  this->PlotPointDataMapper->Delete();
  this->PlotPointDataActor->Delete();

  this->GlyphSource->Delete();

  this->ErrData->Delete();
  this->ErrDataMapper->Delete();
  this->ErrDataActor->Delete();
  
  this->DataSet->Delete();
  this->BinsDataSet->Delete();

  this->UpDataSet->Delete();
  this->DownDataSet->Delete();
  this->LeftDataSet->Delete();
  this->RightDataSet->Delete();

#ifndef NOCLIP
  for(int indx=0;indx<4;indx++)
    {
      ClipPlanes[indx]->Delete();
    };
#endif
}

//----------------------------------------------------------------------------
// Build the axis, ticks, title, and labels and render.

int vtkHistogramDataActor2DExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  this->BuildHistogram(viewport);
 
  if ( this->FillVisibility )
    renderedSomething += this->PlotFillDataActor->RenderOpaqueGeometry(viewport);

  if ( this->LinesVisibility )
    renderedSomething += this->PlotLineDataActor->RenderOpaqueGeometry(viewport);

  if ( this->PointsVisibility )
    renderedSomething += this->PlotPointDataActor->RenderOpaqueGeometry(viewport);

  
  if ( ( this->UpErrBarVisibility) || ( this->DownErrBarVisibility) || ( this->LeftErrBarVisibility) || ( this->RightErrBarVisibility) )
    {
      renderedSomething += this->ErrDataActor->RenderOpaqueGeometry(viewport);
    };

  
  return renderedSomething;
}


int vtkHistogramDataActor2DExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  if ( this->FillVisibility )
    renderedSomething += this->PlotFillDataActor->RenderOverlay(viewport);

  if ( this->LinesVisibility )
    renderedSomething += this->PlotLineDataActor->RenderOverlay(viewport);

  if ( this->PointsVisibility )
    renderedSomething += this->PlotPointDataActor->RenderOverlay(viewport);

  
  if ( ( this->UpErrBarVisibility) || ( this->DownErrBarVisibility) || ( this->LeftErrBarVisibility) || ( this->RightErrBarVisibility) )
    {
      renderedSomething += this->ErrDataActor->RenderOverlay(viewport);
    };
  

  return renderedSomething;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkHistogramDataActor2DExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}


void vtkHistogramDataActor2DExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  this->PlotFillDataActor->ReleaseGraphicsResources(win);
  this->PlotLineDataActor->ReleaseGraphicsResources(win);
  this->PlotPointDataActor->ReleaseGraphicsResources(win);
  this->ErrDataActor->ReleaseGraphicsResources(win);

}


void vtkHistogramDataActor2DExtra::BuildHistogram(vtkViewport *viewport)
{

  if (! this->DataSet->GetNumberOfPoints())
    return;

  int positionsHaveChanged;
  double point[3]; point[2] = 0.0;
  vtkIdType numPts, ptId;
  double tmpPoint[3];
  vtkPoints *pts;


  double *pos1 = this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
  double *pos2 = this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
  


  positionsHaveChanged = 0;
  if (viewport->GetMTime() > this->BuildTime || 
      (viewport->GetVTKWindow() && 
       viewport->GetVTKWindow()->GetMTime() > this->BuildTime))
    {
    // Check to see whether we have to rebuild everything
    // Viewport change may not require rebuild
    double *lastPosition = 
      this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
    double *lastPosition2 =
      this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
    if (lastPosition[0] != this->LastPosition[0] ||
        lastPosition[1] != this->LastPosition[1] ||
        lastPosition2[0] != this->LastPosition2[0] ||
        lastPosition2[1] != this->LastPosition2[1] )
      {
      positionsHaveChanged = 1;

      this->LastPosition[0] = lastPosition[0];
      this->LastPosition[1] = lastPosition[1];
      this->LastPosition2[0] = lastPosition2[0];
      this->LastPosition2[1] = lastPosition2[1];

      }
    }
  
  if (!positionsHaveChanged &&
      this->GetMTime() < this->BuildTime )
    {
      return;
    }
  
  // Initialize and get important info


  double tmpXRange[2];
  double tmpYRange[2];

  switch(this->XScale)
    {
    case 1:
      if (this->XRange[0] > 0 )
	tmpXRange[0]=log10(this->XRange[0]);
      else
	{
	  tmpXRange[0]=log10(this->MinLogXLimit);
	};
      if (this->XRange[1] > 0 )
	tmpXRange[1]=log10(this->XRange[1]);
      else
	{
	  tmpXRange[1]=log10(this->MinLogXLimit);
	};
      break;
    case 0:
    default:
      tmpXRange[0]=this->XRange[0];
      tmpXRange[1]=this->XRange[1];      
      break;
    };


  if ( tmpXRange[0]==tmpXRange[1] )
    {
      if ( tmpXRange[0]==0.0)
	{
	  tmpXRange[1]=1.0;
	}
      else
	{
	  tmpXRange[1]*=1.1;
	};
    };

  
  switch(this->YScale)
    {
    case 1:
      if (this->YRange[0] > 0 )
	tmpYRange[0]=log10(this->YRange[0]);
      else
	{
	  tmpYRange[0]=log10(this->MinLogYLimit);
	};
      if (this->YRange[1] > 0 )
	tmpYRange[1]=log10(this->YRange[1]);
      else
	{
	  tmpYRange[1]=log10(this->MinLogYLimit);
	};
      break;
    case 0:
    default:
      tmpYRange[0]=this->YRange[0];
      tmpYRange[1]=this->YRange[1];      
      break;
    };
  
  if ( tmpYRange[0]==tmpYRange[1] )
    {
      if ( tmpYRange[0]==0.0)
	{
	  tmpYRange[1]=1.0;
	}
      else
	{
	  tmpYRange[1]*=1.1;
	};
    };



  this->PlotLineData->Initialize();
  this->PlotFillData->Initialize();

  if ( this->LinesVisibility || this->FillVisibility )
    {
      GenerateLines(this->DataSet,this->BinsDataSet,tmpXRange,this->XScale,tmpYRange,this->YScale,this->Style,this->PlotLineData,this->PlotFillData);
    }

  this->PlotPointDataSet->Initialize();

  if ( this->PointsVisibility)
    {
      GeneratePoints(this->DataSet,tmpXRange,this->XScale,tmpYRange,this->YScale,this->PlotPointDataSet); 
    };


  numPts = 0;
  if (this->LinesVisibility)
  {
     numPts = this->PlotLineData->GetNumberOfPoints();
     pts=this->PlotLineData->GetPoints();
  }
  else if (this->FillVisibility)
  {
     numPts = this->PlotFillData->GetNumberOfPoints();
     pts=this->PlotFillData->GetPoints();
  }

  
  for ( ptId=0; ptId < numPts; ptId++ )
    {

      pts->GetPoint(ptId,tmpPoint);

      if (
	  MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
	{
	  //	  cerr<<"vtkHistogramDataActor2DExtra::BuildHistogram: Ignoring invalid Point"<<endl;
	  continue;
	};
      
      if (
	  MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	{
	  //	  cerr<<"vtkHistogramDataActor2DExtra::BuildHistogram: Ignoring invalid Point "<<endl;
	  continue;
	};

      pts->SetPoint(ptId,tmpPoint);
    }

  // trim fill data after scaling
  if (this->FillVisibility)
    {
    double yZero=0;

    if (this->YScale == 1)
      {
        yZero=tmpYRange[0];
      };

    MapData(tmpYRange,pos1[1],pos2[1],yZero);

    GenerateFill (this->PlotFillData, yZero);
    }



  numPts = this->PlotPointDataSet->GetNumberOfPoints();
  pts=this->PlotPointDataSet->GetPoints();


  for ( ptId=0; ptId < numPts; ptId++ )
    {

      pts->GetPoint(ptId,tmpPoint);

      if (
	  MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
	{
	  //	  cerr<<"vtkHistogramDataActor2DExtra::BuildHistogram: Ignoring invalid Point"<<endl;
	  continue;
	};
      
      if (
	  MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	{
	  //	  cerr<<"vtkHistogramDataActor2DExtra::BuildHistogram: Ignoring invalid Point "<<endl;
	  continue;
	};

      pts->SetPoint(ptId,tmpPoint);
    }


  // Handle Error bars


  this->ErrData->Initialize();
  
  vtkDoubleArray* tmpUp=0,*tmpDown=0,*tmpLeft=0,*tmpRight=0;
  double tmpCapLength=0;
  
  if ( this->UpDataSet && this->UpErrBarVisibility )
    tmpUp= this->UpDataSet;
  
  if ( this->DownDataSet && this->DownErrBarVisibility )
    tmpDown=this->DownDataSet;
  
  if ( this->LeftDataSet && this->LeftErrBarVisibility )
    tmpLeft=this->LeftDataSet;
  
  if ( this->RightDataSet && this->RightErrBarVisibility )
    tmpRight =this->RightDataSet;
  
  if ( this->ErrBarCapVisibility )
    tmpCapLength=this->ErrBarCapLength;
  
  GenerateErrorBars(this->DataSet,tmpUp,tmpDown,tmpLeft,tmpRight,tmpCapLength,tmpXRange,this->XScale,tmpYRange,this->YScale,pos1,pos2,this->ErrData);
  
  
  
  
  
  numPts = this->ErrData->GetNumberOfPoints();
  pts=this->ErrData->GetPoints();
  
  for ( ptId=0; ptId < numPts; ptId++ )
    {
      
      pts->GetPoint(ptId,tmpPoint);
      
      
      if (
	  MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
	{
	  continue;
	};
      
      if (
	  MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	{
	  continue;
	};
      
      pts->SetPoint(ptId,tmpPoint);
    }
  
#ifndef NOCLIP  
    ClipPlanes[0]->SetOrigin(pos1[0],0,0);
    ClipPlanes[0]->SetNormal(1,0,0);

    ClipPlanes[1]->SetOrigin(pos2[0],0,0);
    ClipPlanes[1]->SetNormal(-1,0,0);

    ClipPlanes[2]->SetOrigin(0,pos1[1],0);
    ClipPlanes[2]->SetNormal(0,1,0);

    ClipPlanes[3]->SetOrigin(0,pos2[1],0);
    ClipPlanes[3]->SetNormal(0,-1,0);
#endif
 
    this->BuildTime.Modified();
}

//----------------------------------------------------------------------------
void vtkHistogramDataActor2DExtra::ShallowCopy(vtkProp *prop)
{
  /*  vtkHistogramDataActor2DExtra *a = vtkHistogramDataActor2DExtra::SafeDownCast(prop);
  if ( a != NULL )
    {
      this->SetRange(a->GetRange());
      this->SetNumberOfMajTicks(a->GetNumberOfMajTicks());
      this->SetNumberOfMinTicks(a->GetNumberOfMinTicks());
      this->SetLabelFormat(a->GetLabelFormat());
      this->SetAdjustTicks(a->GetAdjustTicks());
      this->SetMajTicksLength(a->GetMajTicksLength());
      this->SetMinTicksLength(a->GetMinTicksLength());
      this->SetTickOffset(a->GetTickOffset());
      
      this->SetLabelOffset(a->GetLabelOffset());

      this->SetTitlePerpendicularOffset(a->GetTitlePerpendicularOffset());
      this->SetAxisVisibility(a->GetAxisVisibility());
      this->SetMajTicksVisibility(a->GetMajTicksVisibility());
      this->SetMinTicksVisibility(a->GetMinTicksVisibility());
      this->SetLabelVisibility(a->GetLabelVisibility());
      this->SetTitleVisibility(a->GetTitleVisibility());
      this->SetFontFactor(a->GetFontFactor());
      this->SetLabelFactor(a->GetLabelFactor());
      this->SetLabelTextProperty(a->GetLabelTextProperty());
      this->SetTitleTextProperty(a->GetTitleTextProperty());
    }
    */  
  // Now do superclass
  this->vtkActor2D::ShallowCopy(prop);
};

int vtkHistogramDataActor2DExtra::MapData(double* inDataRange, 
				  double inPixelRangeMin, 
				  double inPixelRangeMax,
				  double& inOutValue)
{

  if ( ( inDataRange[0] == inDataRange[1]) && (inPixelRangeMin != inPixelRangeMax) )
    {
      return -1;
    };
  
  if (inDataRange[0] == inDataRange[1])
    return static_cast<int>(inPixelRangeMin);

  inOutValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(inOutValue-inDataRange[0])/(inDataRange[1]-inDataRange[0]);


  if (inOutValue <0 )
    inOutValue=0;
  
  return 1;

};


void vtkHistogramDataActor2DExtra::GenerateErrorBars(vtkPolyData* inData,
						     vtkDoubleArray* inUpDataSet,
						     vtkDoubleArray* inDownDataSet,
						     vtkDoubleArray* inLeftDataSet,
						     vtkDoubleArray* inRightDataSet,
						     double inErrBarCapLength,
						     double inXRange[2], int inXScale, 
						     double inYRange[2],int inYScale,
						     double *inPos1,
						     double *inPos2,
						     vtkPolyData* outLineData)
{
  if ((!inData) || (!outLineData) )
    return;

  vtkPoints *pts;
  vtkPoints *newPoints;
  vtkCellArray *newLines;
  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType npts=0,nsegpts=0;
  vtkIdType newPtsId,*oldPtsId;
  vtkIdType ptsId=0;
  double x1[3], x2[3], t,oldx2[3];
  double tmpPoint[3],tmpPoint2[3],tmpPoint3[3];
  double tmpXRange[2],tmpYRange[2];


  oldx2[2]=0;

  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];


  outLineData->Initialize();



  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }
  



  vtkPoints *errPts= vtkPoints::New();
  errPts->SetDataTypeToDouble();
  
  
  vtkCellArray *errLines= vtkCellArray::New();
  vtkIdType errPtsId[2],errBarPtsId[2];
  long tmpSize=0;
  
  int deltap[2];
  
  deltap[0] = (int)(inPos2[0]-inPos1[0]);
  deltap[1] = (int)(inPos2[1]-inPos1[1]);
  
  
  double deltaXRange;
  double deltaYRange;
  
  deltaXRange= tmpXRange[1]-tmpXRange[0];
  deltaYRange= tmpYRange[1]-tmpYRange[0];
  
  double verCapLength=0,horCapLength=0;
  
  if (( ( deltap[1]) != 0 ) && (inYScale != 1))
    {
      verCapLength = (inErrBarCapLength*(tmpYRange[1]-tmpYRange[0]))/(inPos2[1] -inPos1[1]);
    };
  
  if (( ( deltap[0]) != 0 ) && (inXScale !=1))
    {
      horCapLength = (inErrBarCapLength*(tmpXRange[1]-tmpXRange[0]))/(inPos2[0] -inPos1[0]);
    };
  
  numPts=inData->GetNumberOfPoints();
  pts=inData->GetPoints();
  
  if (inUpDataSet)
    {
      if ( inUpDataSet->GetNumberOfTuples() )
	{
	  tmpSize = numPts;
	};
    };


  if ( inDownDataSet )
    {
      if ( inDownDataSet->GetNumberOfTuples() )
	{
	  tmpSize += numPts;
	};
    };

  
  if ( inLeftDataSet )
    {
      if ( inLeftDataSet->GetNumberOfTuples() )
	{
	  tmpSize += numPts;
	};
    };
  
  
  if ( inRightDataSet)
    {
      if ( inRightDataSet->GetNumberOfTuples() )
	{
	  tmpSize += numPts; 
	};
    };
  
  if ( inErrBarCapLength )
    {
      tmpSize *= 2;
    };
  
  
  pts=inData->GetPoints();
  errPts->Allocate(tmpSize+numPts);  
  errLines->Allocate(tmpSize);
  
  tmpPoint2[2]=0;      
  tmpPoint3[2]=0;
  for ( ptsId=0; ptsId < numPts; ptsId++ )
    {
      
      pts->GetPoint(ptsId,tmpPoint);
      errPtsId[0] = errPts->InsertNextPoint(tmpPoint); 
      
      
      if ( inUpDataSet)
	{
	  if ( inUpDataSet->GetNumberOfTuples() )
	    {
	      tmpPoint2[0]=tmpPoint[0];
	      tmpPoint2[1] = tmpPoint[1]+inUpDataSet->GetValue(ptsId);
	      errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	      errLines->InsertNextCell(2,errPtsId);
	      
	      if ( inErrBarCapLength )
		{
		  tmpPoint3[1]=tmpPoint2[1];
		  if ( inXScale == 1 )
		    {
		      tmpPoint3[0] = tmpPoint2[0]*pow(10.0,-inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		    }
		  else
		    {
		      tmpPoint3[0] = tmpPoint2[0] - horCapLength/2;
		    };
		  errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		  if ( inXScale == 1 )
		    {
		      tmpPoint3[0] = tmpPoint2[0]*pow(10.0,inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		    }
		  else
		    {
		      tmpPoint3[0] =tmpPoint2[0]+horCapLength/2;
		    }
		  errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		  errLines->InsertNextCell(2,errBarPtsId);
		};	
	      
	    };
      
	};


      if ( inDownDataSet )
	{
	  if ( inDownDataSet->GetNumberOfTuples())
	    {
	      
	      tmpPoint2[0]=tmpPoint[0];
	      tmpPoint2[1] = tmpPoint[1]-inDownDataSet->GetValue(ptsId);
	      errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	      errLines->InsertNextCell(2,errPtsId);
	      
	      if ( inErrBarCapLength )
		{
		  tmpPoint3[1]=tmpPoint2[1];
		  if ( inXScale == 1 )
		    {
		      tmpPoint3[0] =tmpPoint2[0]*pow(10.0,-inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		    }
		  else
		    {
		      tmpPoint3[0] =tmpPoint2[0] - horCapLength/2;
		    };
		  errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		  if ( inXScale == 1 )
		    {
		      tmpPoint3[0] = tmpPoint2[0]*pow(10.0,inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		    }
		  else
		    {
		      tmpPoint3[0] = tmpPoint2[0] + horCapLength/2;
		    };
		  errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		  errLines->InsertNextCell(2,errBarPtsId); 
		};	 
	    };
	};


      if ( inLeftDataSet)
	{
	  if ( inLeftDataSet->GetNumberOfTuples())
	    {
	      tmpPoint2[1]=tmpPoint[1];
	      tmpPoint2[0] = tmpPoint[0]-inLeftDataSet->GetValue(ptsId);
	      errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	      errLines->InsertNextCell(2,errPtsId);
	      
	      if ( inErrBarCapLength )
		{
		  tmpPoint3[0]=tmpPoint2[0];
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,-inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] = tmpPoint2[1] - verCapLength/2;
		    };
		  errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] = tmpPoint2[1] + verCapLength/2;
		    };
		  errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		  errLines->InsertNextCell(2,errBarPtsId); 
		  
		};	
	    };
	};
      
      
      if ( inRightDataSet )
	{
	  if ( inRightDataSet->GetNumberOfTuples() )
	    { 
	      tmpPoint2[1]=tmpPoint[1];
	      tmpPoint2[0] = tmpPoint[0]+inRightDataSet->GetValue(ptsId);
	      errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	      errLines->InsertNextCell(2,errPtsId);
	  
	      if ( inErrBarCapLength )
		{
		  tmpPoint3[0]=tmpPoint2[0];
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,-inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] = tmpPoint2[1] - verCapLength/2;
		    };
		  errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] =tmpPoint2[1] + verCapLength/2;
		    };
		  errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		  errLines->InsertNextCell(2,errBarPtsId); 
		};	
	    };
	  
	};

    };  

  
  
  newPoints = vtkPoints::New();
  newPoints->SetDataTypeToDouble();
  newPoints->Allocate(numPts);

  newLines = vtkCellArray::New();
  newLines->Allocate(1);

  
  for( errLines->InitTraversal();errLines->GetNextCell(npts,oldPtsId);)
    {
      for (ptsId=0; ptsId<(npts-1); ptsId++)
	{
	  errPts->GetPoint(oldPtsId[ptsId], x1);
	  errPts->GetPoint(oldPtsId[ptsId+1], x2);

	  if ( inXScale== 1)
	    {
	      
	      if ( (x1[0] <= 0.0 ) && (x2[0] <= 0.0 ))
		continue;
	      
	      if ( x1[0] <= 0 )
		{
		  x1[0] = tmpXRange[0];
		}
	      else
		{
		  x1[0]=log10(x1[0]);
		};


	       if ( x2[0] <= 0 )
		{
		  x2[0] = tmpXRange[0];
		}
	      else
		{
		  x2[0]=log10(x2[0]);
		};

	    };




	  if ( inYScale== 1)
	    {

	      if ( (x1[1] <= 0.0 ) && (x2[1] <= 0.0 ))
		continue;
	      
	      if ( x1[1] <= 0 )
		{
		  x1[1] = tmpYRange[0];
		}
	      else
		{
		  x1[1]=log10(x1[1]);
		};


	       if ( x2[1] <= 0 )
		{
		  x2[1] = tmpYRange[0];
		}
	      else
		{
		  x2[1]=log10(x2[1]);
		};

	    };

	  if ( ( ptsId == 0 ) || ( x1[0] != oldx2[0] ) || (x1[1] != oldx2[1]))
	    {
	      nsegpts=2;	      
	      newLines->InsertNextCell(nsegpts);
	      newPtsId = newPoints->InsertNextPoint(x1);
	      newLines->InsertCellPoint(newPtsId);
	      newPtsId = newPoints->InsertNextPoint(x2);
	      newLines->InsertCellPoint(newPtsId);
	    }
	  else if (( x1[0] ==  oldx2[0] ) && (x1[1] == oldx2[1]))
	    {
	      nsegpts++;
	      
	      newPtsId = newPoints->InsertNextPoint(x2);
	      newLines->InsertCellPoint(newPtsId);
	      newLines->UpdateCellCount(nsegpts); 
	    };
	  
	  oldx2[0]=x2[0];
	  oldx2[1]=x2[1];
	  
	};
      
    };
  
  
  //Update the lines
  outLineData->SetPoints(newPoints);
  outLineData->SetVerts(0);
  outLineData->SetLines(newLines);

  newPoints->Delete();
  newLines->Delete();
  errPts->Delete();
  errLines->Delete();  
  

};


void vtkHistogramDataActor2DExtra::GeneratePoints(vtkPolyData* inData,double inXRange[2], int inXScale, double inYRange[2],int inYScale,vtkPolyData* outPointData)
{
  if ((!inData) || (!outPointData))
    return;

  vtkPoints *points=inData->GetPoints();
  vtkPoints *newPoints;
  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType ptsId=0;
  double x1[3],t;

  double tmpXRange[2],tmpYRange[2];

  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];

  outPointData->Initialize();

  // make sure the limits are in order


  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }
  


  newPoints = vtkPoints::New();
  newPoints->SetDataTypeToDouble();
  newPoints->Allocate(numPts);



  for (ptsId=0; ptsId<numPts; ptsId++)
    {
      points->GetPoint(ptsId, x1);

      if ( inXScale==1)
        {
	if ( x1[0] <= 0.0 )
	  continue;
	else
	  x1[0]=log10(x1[0]);
        }
      if ( inYScale==1)
        {
	if ( x1[1] <=0.0)
	  continue;
	else
	  x1[1]=log10(x1[1]);
        }
      //Check if the point is in the range
      
      if (( x1[0] >= tmpXRange[0] ) && (x1[0]<=tmpXRange[1]) && ( x1[1] >= tmpYRange[0] ) && (x1[1]<=tmpYRange[1]))
	{
	  newPoints->InsertNextPoint(x1);
	};
    };
  
	  
  outPointData->SetPoints(newPoints);
  outPointData->SetVerts(0);
  outPointData->SetLines(0);


  newPoints->Delete();

};



void vtkHistogramDataActor2DExtra::GenerateLines(vtkPolyData* inData,
						 vtkDoubleArray* inBinsDataSet,
						 double inXRange[2], int inXScale,
						 double inYRange[2], int inYScale,
						 int inStyle,vtkPolyData* outLineData,
						 vtkPolyData* outFillData)
{

  if ((!inData) || (!outLineData && !outFillData))
    return;

  vtkPoints *points=inData->GetPoints();
  vtkPoints *newPoints;
  vtkCellArray *newLines;
  vtkCellArray *newDropLines;
/*  vtkCellArray *newFill;
*/

  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType npts = 0,nsegpts=0;
  vtkIdType newPtsId;
  vtkIdType ptsId=0;
  double x0[3],x1[3], x2[3], tmpx1[3],tmpx2[3], tmpx3[3],t,oldx2[3];
  bitset<4> location1,location2,tmpBSet;
  bool breakFlag=false;
  bool dropLineFlag=false;
  bool addToLineCell=true;
  double lastVal;
  double nextLow;
  
  bool orderFlag[2];
  bool firstSet = false;

  double tmpXRange[2],tmpYRange[2];

  orderFlag[0]=false;
  orderFlag[1]=false;


  oldx2[0] = oldx2[1] = oldx2[2]=0;
  tmpx1[2]=0;
  tmpx2[2]=0;
  tmpx3[2]=0;
  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];

  outLineData->Initialize();


  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }
  


  newPoints = vtkPoints::New();
  newPoints->SetDataTypeToDouble();
  newPoints->Initialize ();
//  newPoints->Allocate(numPts);

  newLines = vtkCellArray::New();
  newLines->Initialize ();
//  newLines->Allocate(numPts-1);

  newDropLines = vtkCellArray::New();
  newDropLines->Initialize ();
//  newDropLines->Allocate(numPts-1);

/*  newFill = vtkCellArray::New();
  newFill->Allocate (numPts);
*/

  
  lastVal=0;
  

  
  for (ptsId=0; ptsId<numPts; ptsId++)
    {
      points->GetPoint(ptsId, x0);

      if ( ptsId < (numPts-1))
	nextLow=inBinsDataSet->GetValue((ptsId+1)*2);


      breakFlag=false;
      dropLineFlag=false;

      
      for(int indx=0;(indx<3 ) && (!breakFlag);indx++)
	{
	  
	  switch (indx)
	    {
	    case 0:
	      x1[0]= inBinsDataSet->GetValue(ptsId*2);
	      x1[1]= lastVal;
	      x2[0]= x1[0];
	      x2[1]= x0[1];
	      break;
	    case 1:
	      x1[0]= inBinsDataSet->GetValue(ptsId*2);
	      x1[1]= x0[1];
	      x2[0]= inBinsDataSet->GetValue(ptsId*2+1);
	      x2[1]= x0[1];
	      lastVal=x0[1];
	      break;
	    case 2:
	      if ( ( ptsId == (numPts-1)) || (nextLow!=inBinsDataSet->GetValue(ptsId*2+1)))
	      {
		x1[0]= x2[0];
		x1[1]= lastVal;
		x2[0]= x2[0];
		x2[1]= 0;
		lastVal=0;
	      }
	      else if ( inStyle == 1 )
		{

		  
		  points->GetPoint(ptsId+1, x0);
		  x1[0]= x2[0];
		  x1[1]= 0;

		  if ( x0[1] > lastVal )
		    {
		      x2[1]= lastVal;
		    }
		  else if ( x0[1] < lastVal )
		    {
		      if (( x0[1] > 0 && lastVal > 0) ||
                         ( x0[1] < 0 && lastVal < 0))
			x2[1]= x0[1];
		      else
			x2[1]=0;
		    };

		  dropLineFlag=true;
		}
	      else
		{
		  breakFlag=true;
		};
	      break;
	    };

	  if ( !breakFlag)
	    {

	      tmpx1[0]=x1[0];
	      tmpx1[1]=x1[1];
	      tmpx2[0]=x2[0];
	      tmpx2[1]=x2[1];


	      if ( inXScale== 1)
		{
		  
		  if ( (tmpx1[0] <= 0.0 ) && (tmpx2[0] <= 0.0 ))
		    addToLineCell = false;
		  
		  if ( tmpx1[0] <= 0 )
		    {
		      tmpx1[0] = tmpXRange[0];
		    }
		  else
		    {
		      tmpx1[0]=log10(tmpx1[0]);
		    };
		  
		  
		  if ( tmpx2[0] <= 0 )
		    {
		      tmpx2[0] = tmpXRange[0];
		    }
		  else
		    {
		      tmpx2[0]=log10(tmpx2[0]);
		    };
		  
		};
	      
	      
	      
	      
	      if ( inYScale== 1)
		{
		  
		  if ( (tmpx1[1] <= 0.0 ) && (tmpx2[1] <= 0.0 ))
		    addToLineCell = false;
		  
		  if ( tmpx1[1] <= 0 )
		    {
		      tmpx1[1] = tmpYRange[0];
		    }
		  else
		    {
		      tmpx1[1]=log10(tmpx1[1]);
		    };
		  
		  
		  if ( tmpx2[1] <= 0 )
		    {
		      tmpx2[1] = tmpYRange[0];
		    }
		  else
		    {
		      tmpx2[1]=log10(tmpx2[1]);
		    };
		  
		};

	      if ( dropLineFlag )
		{
                  if (addToLineCell)
		     newDropLines->InsertNextCell(2);
		  newPtsId = newPoints->InsertNextPoint(tmpx1);

                  if (addToLineCell)
		     newDropLines->InsertCellPoint(newPtsId);
		  newPtsId = newPoints->InsertNextPoint(tmpx2);

                  if (addToLineCell)
		     newDropLines->InsertCellPoint(newPtsId);

                  // dont include drop line points in fill data
                  // these are redundant points as far as fill is concern
		}
	      else
		{
		  if ( !firstSet || ( x1[0] != oldx2[0] ) || (x1[1] != oldx2[1]))
		    {

                      if (addToLineCell)
                      { 
                         firstSet = true;
                         nsegpts=2;
		         newLines->InsertNextCell(nsegpts);
                      }
		      newPtsId = newPoints->InsertNextPoint(tmpx1);

                      if (addToLineCell)
		         newLines->InsertCellPoint(newPtsId);

		      newPtsId = newPoints->InsertNextPoint(tmpx2);
                      if (addToLineCell)
                         newLines->InsertCellPoint(newPtsId);


		    }
		  else if (( x1[0] ==  oldx2[0] ) && (x1[1] == oldx2[1]))
		    {

		      if (tmpx1[1] != tmpx2[1])
			{
			  if (inYScale== 1)
			    {
			      tmpx3[1]=tmpYRange[0];
			    }
			  else
			    {
			      tmpx3[1]=0;
			    };
			  tmpx3[0]=tmpx1[0];
			  newPoints->InsertNextPoint(tmpx3);
			}


		      newPtsId = newPoints->InsertNextPoint(tmpx2);
                      if (addToLineCell)
                         {
                         nsegpts++;
		         newLines->InsertCellPoint(newPtsId);
                         newLines->UpdateCellCount(nsegpts);
                         }



		    };
		  
                  if (addToLineCell)
                    {
		    oldx2[0]=x2[0];
		    oldx2[1]=x2[1];
                    }
		  addToLineCell = true;
		};
	    };
	};
    };
      
  if (outFillData)
  {
/*    vtkIdList    *newFillPointsIdList;
    newFillPointsIdList = vtkIdList::New ();
    newFillPointsIdList->Initialize ();
    double yZero=0;

    if (inYScale== 1)
      {
	yZero=tmpYRange[0];
      };
    

    for(int indx=0;indx<newPoints->GetNumberOfPoints();indx++)
      {
	newPoints->GetPoint(indx, x0);

	if ( ( x0[0] != oldx2[0]) || (x0[1]!=oldx2[1] ) || (indx==0))
	  {
	    newFillPointsIdList->InsertNextId (indx);
	  };


	if ( x0[1] == yZero )
	  {
	    
	    if ( newFillPointsIdList->GetNumberOfIds() >=4 )
	      newFill->InsertNextCell (newFillPointsIdList);
	    
	    newFillPointsIdList->Reset();
	    newFillPointsIdList->InsertNextId (indx);
	  };

	
	oldx2[0]=x0[0];
	oldx2[1]=x0[1];


      };
    

     newFill->Squeeze ();
*/

     outFillData->SetPoints(newPoints);
     outFillData->SetVerts(0);
     outFillData->SetPolys(0);

/*     newFillPointsIdList->Delete();
*/
  }


  
  vtkIdType* tmpPtsId;
  

  for( newDropLines->InitTraversal();newDropLines->GetNextCell(npts,tmpPtsId);)
    {
      newLines->InsertNextCell(2,tmpPtsId);
    };
	  
  //Update the lines
  if (outLineData)
  {
     outLineData->SetPoints(newPoints);
     outLineData->SetVerts(0);
     outLineData->SetLines(newLines);

  }
  
  
  newPoints->Delete();
  newLines->Delete();
  newDropLines->Delete();
/*  newFill->Delete ();
*/


};


void vtkHistogramDataActor2DExtra::SetData(vtkPolyData *inData,
					   vtkDoubleArray* inBinsData, 
					   vtkDoubleArray* inUpData, 
					   vtkDoubleArray* inDownData, 
					   vtkDoubleArray* inLeftData,
					   vtkDoubleArray* inRightData)
{


  vtkIdType ptId,id;
  double tmpPoint[3];
  vtkPoints *pts;
  int pointCount=0;
  double tmpVal;

  // initialize curve data
  
  tmpPoint[2]=0;

  this->DataSet->Initialize();
  this->BinsDataSet->Initialize();

  // put data into local vtk structures

  if (inData)
    {
      pts = vtkPoints::New();
      pts->SetDataTypeToDouble();
      pts->Allocate(10,10);
      pointCount=inData->GetNumberOfPoints();
      for ( ptId=0;ptId<pointCount;ptId++)
      {
	inData->GetPoint(ptId, tmpPoint);
	tmpPoint[0]= tmpPoint[0];
	tmpPoint[1]= tmpPoint[1];
	tmpPoint[2]=0;
	id = pts->InsertNextPoint(tmpPoint);
      }

      this->DataSet->SetPoints(pts);
      this->DataSet->SetVerts(0);
      this->DataSet->SetLines(0);
      
      pts->Delete();

    };





  this->DataSet->ComputeBounds();
  double* bounds = this->DataSet->GetBounds();
  this->XDataRange[0]=bounds[0];
  this->XDataRange[1]=bounds[1];
  this->YDataRange[0]=bounds[2];
  this->YDataRange[1]=bounds[3];



  if (inBinsData)
    {
      for(vtkIdType indx=0;indx<pointCount*2;indx++)
	{
	  if ( indx >= inBinsData->GetNumberOfTuples())
	    { 
	      this->DataSet->GetPoint(indx/2, tmpPoint);
	      tmpVal=tmpPoint[0];
	    }
	  else
	    {
	      tmpVal=inBinsData->GetValue(indx);
	    };

	  this->BinsDataSet->InsertNextValue(tmpVal);
	  if ( tmpVal < this->XDataRange[0] )
	    this->XDataRange[0]=tmpVal;
	  else if  ( tmpVal > this->XDataRange[1] )
	    this->XDataRange[1]=tmpVal;
	};
    };


  this->UpDataSet->Initialize();
  this->DownDataSet->Initialize();
  this->LeftDataSet->Initialize();
  this->RightDataSet->Initialize();



  for(vtkIdType indx=0;indx<pointCount;indx++)
    {

      this->DataSet->GetPoint(indx, tmpPoint);
      tmpPoint[0]=(tmpPoint[0]+tmpPoint[1])/2.0;
      tmpPoint[1]= tmpPoint[2];
      tmpPoint[2]=0;

      if ( inUpData)
	{
	  if ( indx < inUpData->GetNumberOfTuples())
	    {
	      tmpVal=inUpData->GetValue(indx);
	      this->UpDataSet->InsertNextValue(tmpVal);
	      tmpVal+=tmpPoint[1];
		
	      if ( tmpVal > this->YDataRange[1] )
		{
		  this->YDataRange[1] = tmpVal;
		}
	    }
	  else
	    {
	      this->UpDataSet->InsertNextValue(0.0);
	    };
	};


      if ( inDownData)
	{

	  if ( indx < inDownData->GetNumberOfTuples())
	    {
	      tmpVal=inDownData->GetValue(indx);
	      this->DownDataSet->InsertNextValue(tmpVal);
	      tmpVal=tmpPoint[1]-tmpVal;
	      
	      if ( tmpVal < this->YDataRange[0] )
		{
		  this->YDataRange[0] = tmpVal;
		}
	    }
	  else
	    {
	      this->DownDataSet->InsertNextValue(0.0);
	    };
	};
      

      
      
      
      if ( inLeftData)
	{
	  if ( indx < inLeftData->GetNumberOfTuples())
	    {
	      tmpVal=inLeftData->GetValue(indx);
	      this->LeftDataSet->InsertNextValue(tmpVal);
	      tmpVal=tmpPoint[0]-tmpVal;
	      
	      if ( tmpVal < this->XDataRange[0] )
		{
		  this->XDataRange[0] = tmpVal;
		}
	    }
	  else
	    {
	      this->LeftDataSet->InsertNextValue(0.0);
	    };
	};
	  
      
      
      if ( inRightData)
	{
	  if ( indx < inRightData->GetNumberOfTuples())
	    {
	      tmpVal=inRightData->GetValue(indx);
	      this->RightDataSet->InsertNextValue(tmpVal);
	      tmpVal+=tmpPoint[0];
	      
	      if ( tmpVal > this->XDataRange[1] )
		{
		  this->XDataRange[1] = tmpVal;
		}
	    }
	  else
	    {
	      this->RightDataSet->InsertNextValue(0.0);
	    };
	};
    }
  
  
  
  
  this->Modified();
  
};


void vtkHistogramDataActor2DExtra::GenerateFill (vtkPolyData* inOutFillData,
                     double yZero)
{
  int numPts = inOutFillData->GetNumberOfPoints();
  double oldTmpPoint[] = {0,0,0};
  vtkIdList *newPtsList = vtkIdList::New ();
  vtkCellArray *newFill = vtkCellArray::New ();

  vtkPoints *pts = inOutFillData->GetPoints();
  vtkIdType ptId;
  double tmpPoint[3];


  newPtsList->Initialize ();
  newFill->Allocate (numPts);

  for ( ptId=0; ptId < numPts; ptId++ )
    {

    pts->GetPoint(ptId,tmpPoint);

    if (( vtkMath::Distance2BetweenPoints(tmpPoint,oldTmpPoint) > 1.0e-06) ||
           (ptId==0))
      newPtsList->InsertNextId (ptId);


    if ( tmpPoint[1] == yZero )
      {

      if ( newPtsList->GetNumberOfIds() >=4 )
        newFill->InsertNextCell (newPtsList);

      newPtsList->Reset();
      newPtsList->InsertNextId (ptId);
      }

    oldTmpPoint[0]=tmpPoint[0];
    oldTmpPoint[1]=tmpPoint[1];
    oldTmpPoint[2]=tmpPoint[2];

    }

  inOutFillData->SetPolys(newFill);

  newFill->Delete ();
  newPtsList->Delete();
}

void vtkHistogramDataActor2DExtra::GetClipPlanes (int &inOutXMin, int &inOutXMax, int &inOutYMin, int &inOutYMax)
{
#ifndef NOCLIP
  inOutXMin = this->ClipPlanes[0]->GetOrigin()[0];
  inOutXMax = this->ClipPlanes[1]->GetOrigin()[0];
  inOutYMin = this->ClipPlanes[2]->GetOrigin()[1];
  inOutYMax = this->ClipPlanes[3]->GetOrigin()[1];
#endif

}
    
void vtkHistogramDataActor2DExtra::GetBounds (vtkViewport *viewport, double bounds[4])
{

  this->BuildHistogram (viewport);

  int numPlotLineDataPnts = this->PlotLineData->GetNumberOfPoints ();
  int numPlotPointDataSetPnts = this->PlotPointDataSet->GetNumberOfPoints ();
  int numErrDataPnts = this->ErrData->GetNumberOfPoints ();

  double *ptsBounds;

  if (!this->LinesVisibility)
    numPlotLineDataPnts = (this->FillVisibility) ? this->PlotFillData->GetNumberOfPoints () : 0;
  if (!this->PointsVisibility)
    numPlotPointDataSetPnts = 0;
  if (!this->UpErrBarVisibility && !this->DownErrBarVisibility &&
      !this->LeftErrBarVisibility && !this->RightErrBarVisibility)
    numErrDataPnts = 0;

  if (numPlotLineDataPnts == 0)
    {
    bounds[0] = 0;
    bounds[1] = 0;
    bounds[2] = 0;
    bounds[3] = 0;
    }
  else
    {
    if (numPlotLineDataPnts > 0)
      {
      if (this->LinesVisibility)
        ptsBounds = this->PlotLineData->GetPoints ()->GetBounds ();
      else
        ptsBounds = this->PlotFillData->GetPoints ()->GetBounds ();

      bounds[0] = ptsBounds[0];
      bounds[1] = ptsBounds[1];
      bounds[2] = ptsBounds[2];
      bounds[3] = ptsBounds[3];
      }
    if (numPlotPointDataSetPnts > 0)
      {
      int glyphSize = this->GlyphSource->GetScale ();
      ptsBounds = this->PlotPointDataSet->GetPoints ()->GetBounds ();

      if (bounds[0] > ptsBounds[0]-glyphSize)
        bounds[0] = ptsBounds[0]-glyphSize;
      if (bounds[1] < ptsBounds[1]+glyphSize)
        bounds[1] = ptsBounds[1]+glyphSize;
      if (bounds[2] > ptsBounds[2]-glyphSize)
        bounds[2] = ptsBounds[2]-glyphSize;
      if (bounds[3] < ptsBounds[3]+glyphSize)
        bounds[3] = ptsBounds[3]+glyphSize;
      }
    if (numErrDataPnts > 0)
      {
      ptsBounds = this->ErrData->GetPoints ()->GetBounds ();

      if (bounds[0] > ptsBounds[0])
        bounds[0] = ptsBounds[0];
      if (bounds[1] < ptsBounds[1])
        bounds[1] = ptsBounds[1];
      if (bounds[2] > ptsBounds[2])
        bounds[2] = ptsBounds[2];
      if (bounds[3] < ptsBounds[3])
        bounds[3] = ptsBounds[3];
      }

    }
}





