// 
//  Copyright (C) 2005,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



#ifndef __vtkHistogramDataActor2DExtra_hh
#define __vtkHistogramDataActor2DExtra_hh

#include <string>
#include "vtkActor2D.h"
#include "vtkGlyphSource2D.h"
#include "vtkProperty2D.h"
#include "vtkDoubleArray.h"
#include "vtkPlane.h"
#include "vtkTriangleFilter.h"


using namespace std;

class vtkPolyDataMapper2D;
class vtkPolyData;
class vtkPlanes;
class vtkGlyph2D;
class vtkAppendPolyData;
class vtkPlanes;
class vtkGlyphSource2D;



class vtkHistogramDataActor2DExtra : public vtkActor2D
{
public:
  vtkTypeRevisionMacro(vtkHistogramDataActor2DExtra,vtkActor2D);

  // Description:
  // Instantiate object.
  static vtkHistogramDataActor2DExtra *New();


  // Description:
  // Set actual data
  void SetData(vtkPolyData *inData, 
	       vtkDoubleArray* inBinsData=0, 
	       vtkDoubleArray* inUpData=0, 
	       vtkDoubleArray* inDownData=0, 
	       vtkDoubleArray* inLeftData=0,
	       vtkDoubleArray* inRightData=0);



  // Description:
  // Specify the (min,max) x data range.
  vtkSetVector2Macro(XRange,double);
  vtkGetVectorMacro(XRange,double,2);


  
  // Description:
  // Specify the (min,max) y data range.
  vtkSetVector2Macro(YRange,double);
  vtkGetVectorMacro(YRange,double,2);

  // Description:
  // Get the actor bounds in viewport coordinates
  void GetBounds (vtkViewport *viewport, double bounds[4]);



  // Description:
  // Set/Get the min value when converting to log

  void SetMinLogXLimit( double inMinLogLimit)
  { if ( inMinLogLimit> 0)
    { this->MinLogXLimit=inMinLogLimit;this->Modified();}
  ;};
  double  GetMinLogXLimit(){return this->MinLogXLimit;};
  
  void SetMinLogYLimit( double inMinLogLimit)
  { if ( inMinLogLimit> 0)
    { this->MinLogYLimit=inMinLogLimit;this->Modified();}
  ;};
  double  GetMinLogYLimit(){return this->MinLogYLimit;};


  // Description:
  // Set/Get visibility of the lines.
  vtkSetMacro(LinesVisibility, int);
  vtkGetMacro(LinesVisibility, int);
  vtkBooleanMacro(LinesVisibility, int);

  vtkSetMacro(FillVisibility, int);
  vtkGetMacro(FillVisibility, int);
  vtkBooleanMacro(FillVisibility, int);

  vtkSetMacro(PointsVisibility, int);
  vtkGetMacro(PointsVisibility, int);
  vtkBooleanMacro(PointsVisibility, int);


  vtkSetMacro(UpErrBarVisibility, int);
  vtkGetMacro(UpErrBarVisibility, int);
  vtkBooleanMacro(UpErrBarVisibility, int);

  vtkSetMacro(DownErrBarVisibility, int);
  vtkGetMacro(DownErrBarVisibility, int);
  vtkBooleanMacro(DownErrBarVisibility, int);

  vtkSetMacro(LeftErrBarVisibility, int);
  vtkGetMacro(LeftErrBarVisibility, int);
  vtkBooleanMacro(LeftErrBarVisibility, int);

  vtkSetMacro(RightErrBarVisibility, int);
  vtkGetMacro(RightErrBarVisibility, int);
  vtkBooleanMacro(RightErrBarVisibility, int);

  vtkSetMacro(ErrBarCapVisibility, int);
  vtkGetMacro(ErrBarCapVisibility, int);
  vtkBooleanMacro(ErrBarCapVisibility, int);
  
  vtkSetMacro(ErrBarCapLength, int);
  vtkGetMacro(ErrBarCapLength, int);


  // Description:
  // Set/Get scale of X data (linear,log...)
  vtkSetClampMacro(Style, int, 0,2);
  vtkGetMacro(Style, int);

  // Description:
  // Set/Get scale of X data (linear,log...)
  vtkSetClampMacro(XScale, int, 0,10);
  vtkGetMacro(XScale, int);


  // Description:
  // Set/Get scale of Y data (linear,log...)
  vtkSetClampMacro(YScale, int, 0,10);
  vtkGetMacro(YScale, int);


  // Description:
  // Set/Get Flipping of X and Y data
  vtkSetMacro(DataFlip, int);
  vtkGetMacro(DataFlip, int);
  vtkBooleanMacro(DataFlip, int);


  // Description:
  // Draw the axis. 
  int RenderOverlay(vtkViewport* viewport);
  int RenderOpaqueGeometry(vtkViewport* viewport);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport *) {return 0;}

  // Description:
  // Does this prop have some translucent polygonal geometry?
  virtual int HasTranslucentPolygonalGeometry();

  // Description:
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  void ReleaseGraphicsResources(vtkWindow *);


  // Description:
  // Shallow copy of an data actor. Overloads the virtual vtkProp method.
  void ShallowCopy(vtkProp *prop);

  // Get the clipping planes in pixels coordinates
  void GetClipPlanes (int &inOutXMin, int &inOutXMax, int &inOutYMin, int &inOutYMax);



  // Glyph manipulation functions

  void SetGlyphType(int inType) {this->GlyphSource->SetGlyphType(inType);this->Modified();} ;
  void SetGlyphCenter(double inX,double inY, double inZ){this->GlyphSource->SetCenter(inX,inY,inZ);this->Modified();} ;
  void SetGlyphColor(double inR,double inG, double inB){this->GlyphSource->SetColor(inR,inG,inB);this->PlotPointDataActor->GetProperty()->SetColor(inR,inG,inB);this->Modified();} ;
  void SetGlyphScale(double inScale){this->GlyphSource->SetScale(inScale);this->Modified();};
  void SetGlyphScale2(double inScale){this->GlyphSource->SetScale2(inScale);this->Modified();};
  void SetGlyphFilled(int inFilledFlag){this->GlyphSource->SetFilled(inFilledFlag);this->Modified();};
  void SetGlyphCross(int inCrossFlag){this->GlyphSource->SetCross(inCrossFlag);this->Modified();};
  void SetGlyphDash(int inDashFlag){this->GlyphSource->SetDash(inDashFlag);this->Modified();};
  void SetGlyphAngle( double inAngle){this->GlyphSource->SetRotationAngle(inAngle);this->Modified();};


  // Accessors for lines and points 

  vtkActor2D* GetLineActor() { return this->PlotLineDataActor;};
  vtkActor2D* GetFillActor() { return this->PlotFillDataActor;};
  vtkActor2D* GetPointActor() { return this->PlotPointDataActor;};
  vtkActor2D* GetErrBarActor() { return this->ErrDataActor;};

protected:
  vtkHistogramDataActor2DExtra();
  ~vtkHistogramDataActor2DExtra();


  double XRange[2];
  double YRange[2];


  double XDataRange[2];
  double YDataRange[2];


  int   XScale;
  int   YScale;

  double MinLogXLimit;
  double MinLogYLimit;

  int   DataFlip;


  int   LinesVisibility;
  int   PointsVisibility;
  int   FillVisibility;

  int   Style;

  int   UpErrBarVisibility;
  int   DownErrBarVisibility;
  int   LeftErrBarVisibility;
  int   RightErrBarVisibility;
  int   ErrBarCapVisibility;
  
  int   ErrBarCapLength;


  double   LastPosition[2];
  double   LastPosition2[2];


  // Description:
  // Map data into different scales

  static int MapData(double* inDataRange, 
		     double inPixelRangeMin, 
		     double inPixelRangeMax,
		     double& inOutValue);
  


  //Description:
  // Calculate limits of data

  void CalculateDataRanges(vtkPolyData* inData, double* outXRange, double* outYRange);


  //Description:
  // Generate Error Bars

  void GenerateErrorBars(vtkPolyData* inData,
			 vtkDoubleArray* inUpDataSet,
			 vtkDoubleArray* inDownDataSet,
			 vtkDoubleArray* inLeftDataSet,
			 vtkDoubleArray* inRightDataSet,
			 double inErrBarCapLength,
			 double inXRange[2], int inXScale,
			 double inYRange[2],int inYScale,
			 double * inPos1,
			 double * inPos2,
			 vtkPolyData* outLineData);

  //Description:
  // Clip points in data

  void GeneratePoints(vtkPolyData* inData,
		      double inXRange[2], int inXScale, 
		      double inYRange[2], int inYScale,
		      vtkPolyData* outPointData);

  //Description:
  // Generate bars for histogram
  void GenerateLines(vtkPolyData* inData,
		     vtkDoubleArray* inBinsDataSet,
		     double inXRange[2], int inXScale, 
		     double inYRange[2], int inYScale,
		     int inStyle,vtkPolyData* outLineData, 
                     vtkPolyData* outFillData);

  //Description:
  // Generate fill for histograms - this assumes that the points
  // have already been created and that only the fill needs to be generated
  void GenerateFill (vtkPolyData* inOutFillData, double yZero);

  //Description:
  // build data ready to display
  virtual void BuildHistogram(vtkViewport *viewport);




  vtkPolyData             *DataSet;
  vtkDoubleArray          *BinsDataSet;

  vtkPolyData             *PlotLineData;
  vtkPolyDataMapper2D     *PlotLineDataMapper;
  vtkActor2D              *PlotLineDataActor;

  vtkPolyData             *PlotPointDataSet;
  vtkGlyphSource2D        *GlyphSource;
  vtkGlyph2D              *PlotPointData;
  vtkPolyDataMapper2D     *PlotPointDataMapper;
  vtkActor2D              *PlotPointDataActor;

  vtkPolyData             *PlotFillData;
  vtkPolyDataMapper2D     *PlotFillDataMapper;
  vtkActor2D              *PlotFillDataActor;
  vtkTriangleFilter       *triangleFilter;
  
  vtkDoubleArray          *UpDataSet;
  vtkDoubleArray          *DownDataSet;
  vtkDoubleArray          *LeftDataSet;
  vtkDoubleArray          *RightDataSet;

  vtkPolyData             *ErrData;
  vtkPolyDataMapper2D     *ErrDataMapper;
  vtkActor2D              *ErrDataActor;
  vtkPlane                *ClipPlanes[4];



  vtkTimeStamp  BuildTime;

private:
  vtkHistogramDataActor2DExtra(const vtkHistogramDataActor2DExtra&);  // Not implemented.
  void operator=(const vtkHistogramDataActor2DExtra&);  // Not implemented.

};


#endif
