// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkXYPlotActor.cxx,v $
  Language:  C++
  Date:      $Date: 2003/11/07 18:32:16 $
  Version:   $Revision: 1.46 $

  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notice for more information.

     This file has been derived from version of vtkXYPlotActor.cc 
     Distribution and modifications to this file are governed by the above
     copyright.

=========================================================================*/
#include "vtkHistogramDataListActor2DExtra.hh"
#include "vtkObjectFactory.h"
#include "vtkViewport.h"



vtkCxxRevisionMacro(vtkHistogramDataListActor2DExtra, "$Revision: 1.46 $");
vtkStandardNewMacro(vtkHistogramDataListActor2DExtra);


//----------------------------------------------------------------------------
// Instantiate object
vtkHistogramDataListActor2DExtra::vtkHistogramDataListActor2DExtra()
{
  this->InitTraversal ();
}

vtkHistogramDataListActor2DExtra::~vtkHistogramDataListActor2DExtra()
{
  
  for(  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.begin();tmpIter!=this->Data.end();tmpIter++)
    {
      tmpIter->second->Delete();
    };
  
}



//----------------------------------------------------------------------------
// Plot scalar data for each input dataset.
int vtkHistogramDataListActor2DExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;
  
  int renderedSomething=0;
  
  for(  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.begin();tmpIter!=this->Data.end();tmpIter++)
    {
      renderedSomething += tmpIter->second->RenderOverlay(viewport);
    };


  return renderedSomething;
}

//----------------------------------------------------------------------------
// Plot scalar data for each input dataset.
int vtkHistogramDataListActor2DExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  // Check modified time to see whether we have to rebuild.
  // Pay attention that GetMTime() has been redefined (see below)

  if ( this->GetMTime() > this->BuildTime )
    {
      this->UpdatePosition();

      this->BuildTime.Modified();

    };


  vtkDebugMacro(<<"Rendering HistogramData");

  for(  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.begin();tmpIter!=this->Data.end();tmpIter++)
    {
      renderedSomething += tmpIter->second->RenderOpaqueGeometry(viewport);
    };

  return renderedSomething;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkHistogramDataListActor2DExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}


//----------------------------------------------------------------------------
// Release any graphics resources that are being consumed by this actor.
// The parameter window could be used to determine which graphic
// resources to release.
void vtkHistogramDataListActor2DExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  for(  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.begin();tmpIter!=this->Data.end();tmpIter++)
    {
      tmpIter->second->ReleaseGraphicsResources(win);
    }; 

}

//----------------------------------------------------------------------------
unsigned long vtkHistogramDataListActor2DExtra::GetMTime()
{
  unsigned long mtime;
  mtime = this->vtkActor2D::GetMTime();

  return mtime;
}



void vtkHistogramDataListActor2DExtra::UpdatePosition()
{
  double * p1=this->GetPosition();
  double * p2=this->GetPosition2();


  // Render all the curves 
  
  for(  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.begin();tmpIter!=this->Data.end();tmpIter++)
    { 

      tmpIter->second->SetPosition(p1);
      tmpIter->second->SetPosition2(p2);

    };
};


int vtkHistogramDataListActor2DExtra::AddData(long inDataId,vtkPolyData *inData,
				 vtkDoubleArray* inBinData,
				 vtkDoubleArray *inUpErrData,
				 vtkDoubleArray *inDownErrData,
				 vtkDoubleArray *inLeftErrData,
				 vtkDoubleArray *inRightErrData)
{
  
  if (! inData )
    {
      return 0;
    };


  // Check if id in use

  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.find(inDataId);
  
  if ( tmpIter != this->Data.end())
    {  
      return 0;
    };

  // add curve and associate it with axis



  vtkHistogramDataActor2DExtra* tmpData= vtkHistogramDataActor2DExtra::New();

  tmpData->SetData(inData,inBinData,inUpErrData,inDownErrData,inLeftErrData,inRightErrData);

  this->Data[inDataId]=tmpData;

  
  this->Modified();


  return 1;

};

int vtkHistogramDataListActor2DExtra::RemoveData(long inDataId)
{
  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.find(inDataId);
  
  if ( tmpIter != this->Data.end())
    {
       tmpIter->second->Delete();
       this->Data.erase(inDataId);
    };
  
  return 0;

};

int vtkHistogramDataListActor2DExtra::RemoveData()
{

  for(  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.begin();tmpIter!=this->Data.end();tmpIter++)
    { 
      tmpIter->second->Delete();
    };

  Data.clear();
  return 0;
};

vtkHistogramDataActor2DExtra* vtkHistogramDataListActor2DExtra::GetHistogramDataActor(long inDataId)
{
  map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.find(inDataId);
  
  if ( tmpIter != this->Data.end())
    {
      return tmpIter->second;
    };
  
  return 0;
};

vtkHistogramDataActor2DExtra* vtkHistogramDataListActor2DExtra::GetNextHistogramDataActor ()
{
  vtkHistogramDataActor2DExtra *histogramData = NULL;
  if (data_iter == Data.end ())
     return NULL;

  histogramData = (data_iter->second);
  data_iter++;

  return histogramData;
}

void vtkHistogramDataListActor2DExtra::GetClipPlanes (int &inOutXMin, int &inOutXMax, int &inOutYMin, int &inOutYMax)
{
  if (this->Data.empty ())
    {
    inOutXMin = 0;
    inOutXMax = 0;
    inOutYMin = 0;
    inOutYMax = 0;
    }
  else
    {
    vtkHistogramDataActor2DExtra* histogramData = (this->Data.begin ())->second;
    histogramData->GetClipPlanes (inOutXMin, inOutXMax, inOutYMin, inOutYMax);
    }
}


void vtkHistogramDataListActor2DExtra::GetBounds (vtkViewport *viewport, double bounds[4])
{
  if (this->Data.empty ())
    {
    bounds[0] = 0;
    bounds[1] = 0;
    bounds[2] = 0;
    bounds[3] = 0;
    }
  else
    {
    double tmpBounds[4];
    vtkHistogramDataActor2DExtra* histogramData = (this->Data.begin ())->second;
    map<long,vtkHistogramDataActor2DExtra*>::iterator tmpIter=this->Data.begin();
    histogramData->GetBounds (viewport, bounds);

    tmpIter++;
    for(;tmpIter!=this->Data.end();tmpIter++)
      {
      histogramData = tmpIter->second;
      tmpIter->second->GetBounds (viewport, tmpBounds);

      if (tmpBounds[0] < bounds[0])
        bounds[0] = tmpBounds[0];
      if (tmpBounds[1] > bounds[1])
        bounds[1] = tmpBounds[1];
      if (tmpBounds[2] < bounds[2])
        bounds[2] = tmpBounds[2];
      if (tmpBounds[3] > bounds[3])
        bounds[3] = tmpBounds[3];
      };
    }
}

