// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkXYPlotActorExtra.h,v $
  Language:  C++
  Date:      $Date: 2002/11/11 21:41:29 $
  Version:   $Revision: 1.23 $

  Copyright (c) 1993-2002 Ken Martin, Will Schroeder, Bill Lorensen 
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notice for more information.


  This file has been derived from version of vtkXYPlotActor.hh 
  Distribution and modifications to this file are governed by the above
  copyright.


=========================================================================*/

#ifndef __vtkHistogramDataListActor2DExtra_hh
#define __vtkHistogramDataListActor2DExtra_hh

#include <map>
#include <vector>


#include "vtkHistogramDataActor2DExtra.hh"

using namespace std;


class vtkHistogramDataListActor2DExtra : public vtkActor2D
{
public:
  vtkTypeRevisionMacro(vtkHistogramDataListActor2DExtra,vtkActor2D);

  static vtkHistogramDataListActor2DExtra *New();

  // Description:
  // Take into account the modified time of internal helper classes.
  unsigned long GetMTime();
  
//BTX  
  // Description:
  // WARNING: INTERNAL METHOD - NOT INTENDED FOR GENERAL USE
  // DO NOT USE THIS METHOD OUTSIDE OF THE RENDERING PROCESS.
  // Draw the x-y plot.
  int RenderOpaqueGeometry(vtkViewport*);
  int RenderOverlay(vtkViewport*);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport *) {return 0;}

  // Description:
  // Does this prop have some translucent polygonal geometry?
  virtual int HasTranslucentPolygonalGeometry();

  // Description:
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  void ReleaseGraphicsResources(vtkWindow *);
//ETX  



  ///////////////////////// CURVE DATA METHODS


  int AddData(long inDataId,vtkPolyData *inData,
	      vtkDoubleArray *BinData,
	      vtkDoubleArray *inUpErrData,
	      vtkDoubleArray *inDownErrData,
	      vtkDoubleArray *inLeftErrData,
	      vtkDoubleArray *inRightErrData);

  int RemoveData(long inDataId);
  int RemoveData();

  

  // Retrieve an actor from the list of curve segments
  vtkHistogramDataActor2DExtra* GetHistogramDataActor(long inDataId);

  // Returns the actor and steps to the next segment in the map
  // This is similar to VtkCollection
  vtkHistogramDataActor2DExtra* GetNextHistogramDataActor ();
  void InitTraversal () {data_iter = Data.begin ();};

  // Get the clipping planes in pixels coordinates
  void GetClipPlanes (int &inOutXMin, int &inOutXMax, int &inOutYMin, int &inOutYMax);

  // Description:
  // Get the actor bounds in viewport coordinates
  void GetBounds (vtkViewport *viewport, double bounds[]);


  
  // Description:
  // Update the sub curve positions
  void UpdatePosition();


protected:
  vtkHistogramDataListActor2DExtra ();
  ~vtkHistogramDataListActor2DExtra ();

  map<long,vtkHistogramDataActor2DExtra*>  Data;

  vtkTimeStamp  BuildTime;

private:
  vtkHistogramDataListActor2DExtra(const vtkHistogramDataListActor2DExtra&);  // Not implemented.
  void operator=(const vtkHistogramDataListActor2DExtra&);  // Not implemented.

  map<long,vtkHistogramDataActor2DExtra*>::iterator data_iter;


};


#endif

