// 
//  Copyright (C) 2005,2007,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkOpenGLFreeTypeTextMapperExtra.cc,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

     This file has been derived from version of vtkOpenGLFreeTypeTextMapper.cxx
     Distribution and modifications to this file are governed by the above
     copyright.
=========================================================================*/
#include <iostream>
#include <math.h>
#include "vtkOpenGLFreeTypeTextMapperExtra.hh"

#include "vtkActor2D.h"
#include "vtkObjectFactory.h"
#include "vtkProperty2D.h"
#include "vtkTextProperty.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkToolkits.h"  // for VTK_USE_GL2PS

#include "vtkFreeTypeUtilitiesExtra.hh"
#include "vtkftglConfig.h"

#include "vtkgluPickMatrix.h"

#ifdef FTGL_USE_NAMESPACE
using namespace ftgl;
#endif

#include "FTFontExtra.hh"
#include "vtkMath.h"

#include "vtkstd/string"
#include "freetype/freetype.h"

#include <vector>

#ifdef VTK_USE_GL2PS
#include "gl2ps.h"
#endif // VTK_USE_GL2PS

using namespace std;

//----------------------------------------------------------------------------
// Print debug info

#define VTK_FTTM_DEBUG 0
#define VTK_FTTM_DEBUG_CD 0


unsigned int SymbolFontWCharmap[256]={0,1,2,3,4,5,6,7,8,9,10,
				      11,12,13,14,15,16,17,18,19,20,
				      21,22,23,24,25,26,27,28,29,30,
				      31,32,33,34,35,36,37,38,39,40,
				      41,42,43,44,45,46,47,48,49,50,
				      51,52,53,54,55,56,57,58,59,60,
				      61,62,63,64,65,66,67,68,69,70,
				      71,72,73,74,75,76,77,78,79,80,
				      81,82,83,84,85,86,87,88,89,90,
				      91,92,93,94,95,96,97,98,99,100,
				      101,102,103,104,105,106,107,108,109,110,
				      111,112,113,114,115,116,117,118,119,120,
				      121,122,123,124,125,126,127,128,129,130,
				      131,132,133,134,135,136,137,138,139,140,
				      141,142,143,144,145,146,147,148,149,150,
				      151,152,153,154,155,156,157,158,159,160,
				      161,162,163,164,165,166,167,168,169,170,
				      171,172,173,174,175,176,177,178,179,180,
				      181,182,183,184,185,186,187,188,189,190,
				      191,192,193,194,195,196,197,198,199,200,
				      201,202,203,204,205,206,207,208,209,210,
				      211,212,213,214,215,216,217,218,219,220,
				      221,222,223,224,225,226,227,228,229,230,
				      231,232,233,234,235,236,237,238,239,240,
				      241,242,243,244,245,246,247,248,249,250,
				      251,252,253,254,255 };


//----------------------------------------------------------------------------
// GL2PS related internal helper functions.

#ifdef VTK_USE_GL2PS
static char* 
vtkOpenGLFreeTypeTextMapperExtra_GetGL2PSFontName(vtkTextProperty *tprop,
                                             unsigned char* fontData,
                                             int fontDataSize)
{
  char *ps_font = NULL;
 // For speed we use ARIAL == 0, COURIER == 1, TIMES == 2 , GREEK=3, HELVETICA=4
  static char const *normal1[] = {
    "Helvetica", 
    "NimbusMonL-Regu-ISOLatin1", 
    "NimbusRomNo9L-Regu-ISOLatin1",
    "ChipsSymbol",
    "NimbusSanL-Regu-ISOLatin1"
  };
  
  static char const *italic1[] = {
    "Helvetica-Oblique", 
    "NimbusMonL-ReguObli-ISOLatin1", 
    "NimbusRomNo9L-ReguItal-ISOLatin1",
    "ChipsSymbol",
    "NimbusSanL-ReguItal-ISOLatin1"
  };
  
  static char const *bold1[] = {
    "Helvetica-Bold", 
    "NimbusMonL-Bold-ISOLatin1", 
    "NimbusRomNo9L-Medi-ISOLatin1",
    "ChipsSymbol",
    "NimbusSanL-Bold-ISOLatin1"
  };
  
  static char const *bolditalic1[] = {
    "Helvetica-BoldOblique", 
    "NimbusMonL-BoldObli-ISOLatin1", 
    "NimbusRomNo9L-MediItal-ISOLatin1",
    "ChipsSymbol",
	"NimbusSanL-BoldItal-ISOLatin1"
  };
  
  
  static char const *normal2[] = {
    "Helvetica", 
    "Courier",
    "Times-Roman",
    "Symbol",
    "Helvetica"
  };
  
  static char const *italic2[] = {
    "Helvetica-Oblique", 
    "Courier-Oblique",
    "Times-Italic",
    "Symbol",
    "Helvetica-Oblique"
  };
  
  static char const *bold2[] = { 
    "Helvetica-Bold", 
    "Courier-Bold",
    "Times-Bold",
    "Symbol",
    "Helvetica-Bold"
  };
  
  static char const *bolditalic2[] = {
    "Helvetica-BoldOblique", 
    "Courier-BoldOblique",
    "Times-BoldItalic",
    "Symbol",
    "Helvetica-BoldOblique"
  };
  
  
  int font = tprop->GetFontFamily();

  // extract the font file name from user defined fonts
  if ( tprop->GetFaceFileName() && fontData )
    {
    for (int ii=0; ii<fontDataSize; ii++)
      {
      if (strncmp ((char *)(&fontData[ii]), "/FontName", 9) == 0)
        {
        int start = ii+9;
        int end = ii+9;
        while ((fontData[end] == ' ') && (end < fontDataSize))
          {
          end++;
          start++;
          }
        while ((fontData[end] != ' ') && (end < fontDataSize)) end++;
        if (end < fontDataSize)
          {
          if (fontData[start] == '/')
            start++;
          ps_font = new char[end-start+1];
          strncpy (ps_font, (char *)(&fontData[start]), end-start);
          ps_font[end-start] = '\0';
          }
        break;
        }
      }
    }

  else if ( gl2psGetFontDataImport() )
    {

    ps_font = new char[64];
    if ( (tprop->GetBold()) && (tprop->GetItalic()))
      strcpy(ps_font,bolditalic1[font] );
    else if (tprop->GetBold())
      strcpy(ps_font,bold1[font] );
    else if (tprop->GetItalic())
      strcpy(ps_font,italic1[font] );
    else
      strcpy(ps_font,normal1[font] );
    }
  else
    {
    ps_font = new char[64];
    if ( (tprop->GetBold()) && (tprop->GetItalic()))
      strcpy(ps_font,bolditalic2[font] );
    else if (tprop->GetBold())
      strcpy(ps_font,bold2[font] );
    else if (tprop->GetItalic())
      strcpy(ps_font,italic2[font] );
    else
      strcpy(ps_font,normal2[font] );
    }
    
  return ps_font;  
}

// Load GL2PS font data structure 
static void 
vtkOpenGLFreeTypeTextMapperExtra_LoadGL2PSFontData (vtkFreeTypeUtilitiesExtra::Entry *entry,
                                                    GL2PSfontDataElem &fontData)
{
  if (!entry)
    return;

  fontData.data = entry->fontData;
  fontData.dataSize=entry->fontDataSize;

  fontData.charCodeSize = entry->charCodeSize;
  fontData.charCodes = entry->charCodes;
  fontData.widths = entry->widths;

  for (int ii=0; ii<4; ii++)
    fontData.descriptor.bbox[ii] = entry->bbox[ii];

  fontData.descriptor.ascent = entry->ascent;
  fontData.descriptor.descent = entry->descent;
  fontData.descriptor.capheight = entry->capheight;
  fontData.descriptor.vstemthickness = entry->vstemthickness;
  fontData.descriptor.avgwidth = entry->avgwidth;
  fontData.descriptor.italicangle = entry->Font->ItalicAngle ();
  fontData.descriptor.symbol = (int)entry->symbol;
  fontData.descriptor.fixedpitch = (int)entry->fixedpitch;



  // get the type -- note despite the namespace of X11 this should be
  // a platform independent function
  if ((entry->type == "Type 1") || (entry->type == "PCF"))
    fontData.type = GL2PS_TYPE1;
  else if (entry->type == "TrueType")
    fontData.type = GL2PS_TRUETYPE;


}
#endif

//----------------------------------------------------------------------------
#ifndef VTK_IMPLEMENT_MESA_CXX
vtkCxxRevisionMacro(vtkOpenGLFreeTypeTextMapperExtra, "$Revision: 1.39 $");
vtkStandardNewMacro(vtkOpenGLFreeTypeTextMapperExtra);
#endif

//----------------------------------------------------------------------------
vtkOpenGLFreeTypeTextMapperExtra::vtkOpenGLFreeTypeTextMapperExtra()
{
  this->LastSize[0] = 0;
  this->LastSize[1] = 0;
}

//----------------------------------------------------------------------------
vtkOpenGLFreeTypeTextMapperExtra::~vtkOpenGLFreeTypeTextMapperExtra()
{
  if (this->LastWindow)
    {
    this->ReleaseGraphicsResources(this->LastWindow);
    }  
}

//----------------------------------------------------------------------------
void vtkOpenGLFreeTypeTextMapperExtra::ReleaseGraphicsResources(vtkWindow *vtkNotUsed(win))
{
#if VTK_FTTM_DEBUG
    printf("vtkOpenGLFreeTypeTextMapperExtra::ReleaseGraphicsResources\n");
#endif
  
  this->LastWindow = NULL;
  
  // Very important
  // the release of graphics resources indicates that significant changes have
  // occurred. Old fonts, cached sizes etc are all no longer valid, so we send
  // ourselves a general modified message.

  // this->Modified();
}

//----------------------------------------------------------------------------
void vtkOpenGLFreeTypeTextMapperExtra::GetSize(vtkViewport* viewport, double size[2])
{

  size[0] = size[1] = 0;
  
  
  // Check for multiline
  
  /*  if (this->NumberOfLines > 1)
    {
      this->GetMultiLineSize(viewport, size);
      return;
    }
  */
  // Check for input
  
  if (this->Input == NULL || this->Input[0] == '\0') 
    {
      return;
    }
  
  vtkTextProperty *tprop = this->GetTextProperty();
  if (!tprop)
    {
      vtkErrorMacro(<< "Need a text property to get size");
      return;
    }
  
  // Check to see whether we have to rebuild anything
  
  if (this->GetMTime() < this->SizeBuildTime &&
      tprop->GetMTime() < this->SizeBuildTime)
    {
#if VTK_FTTM_DEBUG
      printf("vtkOpenGLFreeTypeTextMapperExtra::GetSize: In cache!\n");
#endif
      
      size[0] = this->LastSize[0];
      size[1] = this->LastSize[1];
      return;
      
    }
  
  // Check for font and try to set the size
  
  
  vtkFreeTypeUtilitiesExtra::Entry *entry = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop);
  FTFontExtra *font;
  
  if ( entry)
    {
      font = entry->Font;
      
      if (!font) 
	{
	  vtkErrorMacro(<< "Render - No font");
	  return;
	}
    }
  else
    {
      return;
    };
  
  // The font global ascender and descender might just be too high
  // for given a face. Let's get a compromise by computing these values
  // from some usual ascii chars.
  
  /*  if (entry->LargestAscender < 0 || entry->LargestDescender < 0)
    {
    float llx, lly, llz, urx, ury, urz;
    font->BBox("_/7Agfy", llx, lly, llz, urx, ury, urz);
    entry->LargestAscender = ury;
    entry->LargestDescender = lly;
    }
  this->LastSize[0] = size[0] = static_cast<int>(font->Advance(this->Input));
  this->LastSize[1] = size[1] =
  (int)(entry->LargestAscender - entry->LargestDescender);
  this->LastLargestDescender = static_cast<int>(entry->LargestDescender);
  */

  if (( tprop->GetFontFamily() == VTK_GREEK) && (!tprop->GetFaceFileName()))
    {
      wchar_t* tmpBuf= new wchar_t[strlen(this->Input)+1];
      MapCharToWChar(SymbolFontWCharmap,this->Input,tmpBuf);
      this->LastSize[0] = size[0] = (int)font->Advance(tmpBuf);
      delete[] tmpBuf;
    }
  else
    {
      this->LastSize[0] = size[0] = (int)font->Advance(this->Input);
    };
  
  this->LastSize[1] = size[1] = (int)(entry->LargestAscender - entry->LargestDescender);
  this->SizeBuildTime.Modified();
}

//----------------------------------------------------------------------------
void vtkOpenGLFreeTypeTextMapperExtra::RenderOverlay(vtkViewport* viewport, 
                                                vtkActor2D* actor)
{

  vtkDebugMacro (<< "RenderOverlay");

  // Check for input

  if (this->Input == NULL || this->Input[0] == '\0') 
    {
    return;
    }

  // Check for multi-lines

  if (this->NumberOfLines > 1)
    {
    this->RenderOverlayMultipleLines(viewport, actor);
    return;
    }

  // Get text property

  vtkTextProperty *tprop = this->GetTextProperty();
  if (!tprop)
    {
    vtkErrorMacro(<< "Need a text property to render mapper");
    return;
    }

  // Get the window information for display

  vtkWindow* window = viewport->GetVTKWindow();
  if (this->LastWindow && this->LastWindow != window)
    {
    this->ReleaseGraphicsResources(this->LastWindow);
    }
  this->LastWindow = window;

  // Get size of text

  double  size[2];
  this->GetSize(viewport, size);
  // Get the position of the text actor

  double* actorPos;
  actorPos= 
    actor->GetActualPositionCoordinate()->GetComputedDoubleViewportValue(viewport);
  
  // Define bounding rectangle

  double pos[2];
  pos[0] = actorPos[0];
  pos[1] = static_cast <double>(actorPos[1] - tprop->GetLineOffset());

  switch (tprop->GetJustification())
    {
    case VTK_TEXT_LEFT: 
      break;
    case VTK_TEXT_CENTERED:
      pos[0] = pos[0] - size[0] / 2;
      break;
    case VTK_TEXT_RIGHT: 
      pos[0] = pos[0] - size[0];
      break;
    }

  switch (tprop->GetVerticalJustification())
    {
    case VTK_TEXT_TOP: 
      pos[1] = pos[1] - size[1] - this->LastLargestDescender;
      break;
    case VTK_TEXT_CENTERED:
      pos[1] = pos[1] - size[1] / 2 - this->LastLargestDescender / 2;
      break;
    case VTK_TEXT_BOTTOM: 
      break;
    }


  double angleRad= vtkMath::RadiansFromDegrees (tprop->GetOrientation());

  
  // Push a 2D matrix on the stack

  int *vsize = viewport->GetSize();
  double *vport = viewport->GetViewport();
  double *tileViewport = viewport->GetVTKWindow()->GetTileViewport();
  double visVP[4];

  visVP[0] = (vport[0] >= tileViewport[0]) ? vport[0] : tileViewport[0];
  visVP[1] = (vport[1] >= tileViewport[1]) ? vport[1] : tileViewport[1];
  visVP[2] = (vport[2] <= tileViewport[2]) ? vport[2] : tileViewport[2];
  visVP[3] = (vport[3] <= tileViewport[3]) ? vport[3] : tileViewport[3];

  if (visVP[0] == visVP[2] || visVP[1] == visVP[3])
    {
    return;
    }

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();

  if(viewport->GetIsPicking())
    {
    vtkgluPickMatrix(viewport->GetPickX(), viewport->GetPickY(),
                     viewport->GetPickWidth(),
                     viewport->GetPickHeight(),
                     viewport->GetOrigin(), viewport->GetSize());
    }
  
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glDisable(GL_LIGHTING);
  glDepthFunc(GL_ALWAYS);

  if (actor->GetProperty()->GetDisplayLocation() == VTK_FOREGROUND_LOCATION)
    {
    glOrtho(0, vsize[0] - 1, 0, vsize[1] - 1, 0, 1);
    }
  else
    {
    glOrtho(0, vsize[0] - 1, 0, vsize[1] - 1, -1, 0);
    }

  int *winSize = viewport->GetVTKWindow()->GetSize();

  double xoff = (pos[0] - winSize[0] * (visVP[0] - vport[0]));
  double yoff = (pos[1] - winSize[1] * (visVP[1] - vport[1]));

  // When picking draw the bounds of the text as a rectangle,
  // as text only picks when the pick point is exactly on the
  // origin of the text 

  if(viewport->GetIsPicking())
    {
    float x1 = (2.0 * actorPos[0]) / vsize[0] - 1.0;
    float y1 = 2.0 * (actorPos[1] - tprop->GetLineOffset())/vsize[1] - 1.0;
    float width = (2.0 * size[0]) / vsize[0];
    float height = (2.0 * size[1]) / vsize[1];
    glRectf(x1, y1, x1 + width, y1 + height);

    // Clean up and return after drawing the rectangle

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glEnable(GL_LIGHTING);
    glDepthFunc(GL_LEQUAL);
    
    return;
    }

  double* tprop_color = tprop->GetColor();
  double tprop_opacity = tprop->GetOpacity();

  // Get the font
  vtkFreeTypeUtilitiesExtra::Entry *entry = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop, tprop_color);

    FTFontExtra *font;
#ifdef VTK_USE_GL2PS
    GL2PSfontDataElem fontData;
    fontData.fontName = NULL;
    fontData.data = NULL;
    fontData.dataSize = 0;
    fontData.type = GL2PS_TYPE1;
    fontData.charCodes = NULL;
    fontData.charCodeSize = 0;
    fontData.widths = NULL;
    fontData.descriptor.symbol = 0;
    fontData.descriptor.fixedpitch = 0;
    fontData.descriptor.bbox[0] = 0;
    fontData.descriptor.bbox[1] = 0;
    fontData.descriptor.bbox[2] = 0;
    fontData.descriptor.bbox[3] = 0;
    fontData.descriptor.ascent = 0;
    fontData.descriptor.descent = 0;
    fontData.descriptor.vstemthickness = 0;
    fontData.descriptor.avgwidth = 0;
    fontData.next = NULL;


#endif

  if ( entry )
    {
      font = entry->Font;

      if (!font) 
	{
	  vtkErrorMacro(<< "Render - No font");
	  return;
	};
#ifdef VTK_USE_GL2PS
      vtkOpenGLFreeTypeTextMapperExtra_LoadGL2PSFontData (entry, fontData);
#endif
    }
  else
    {
      return;
    }

  struct FTGLRenderContext *ftgl_context = 0;

#ifdef VTK_IMPLEMENT_MESA_CXX
  // If we support Mangle Mesa, VTK_IMPLEMENT_MESA_CXX will be defined to
  // compile this unit as a Mesa text mapper. In that case, provide a
  // context to FTGL to switch dynamically to Mangle Mesa rendering.
  struct FTGLRenderContext ftgl_context_mesa;
  ftgl_context_mesa.UseMangleMesa = 1;
  ftgl_context = &ftgl_context_mesa;
#endif

  // Setup the fonts for GL2PS output.

  wchar_t* tmpBuf;
  if ((tprop->GetFontFamily() == VTK_GREEK) && (!tprop->GetFaceFileName()))
    {
      tmpBuf= new wchar_t[strlen(this->Input)+1];
      MapCharToWChar(SymbolFontWCharmap,this->Input,tmpBuf);
    };
  
#ifdef VTK_USE_GL2PS
  fontData.fontName = vtkOpenGLFreeTypeTextMapperExtra_GetGL2PSFontName(tprop, fontData.data, fontData.dataSize);
  if (!fontData.fontName)
    {
    fontData.fontName = new char[1];
    fontData.fontName[0] = '\0';
    }

#endif // VTK_USE_GL2PS
  
  // Set up the shadow color
  if (tprop->GetShadow())
    {
    double shadow_color[3], rgb;
    rgb = ((tprop_color[0] + tprop_color[1] + tprop_color[2]) / 3.0
> 0.5)
      ? 0.0 : 1.0;
    shadow_color[0] = shadow_color[1] = shadow_color[2] = rgb;

    // Get the shadow font

    vtkFreeTypeUtilitiesExtra::Entry *shadow_entry =
      vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop, shadow_color);
    FTFont *shadow_font = shadow_entry ? shadow_entry->Font : NULL;
    if (!shadow_font)
      {
      vtkErrorMacro(<< "Render - No shadow font");
      return;
      }

    // Set the color here since load/render glyphs is done
    // on demand and this color has to be consistent for a given font entry.

    glColor4ub(static_cast<unsigned char>(shadow_color[0] * 255.0),
               static_cast<unsigned char>(shadow_color[1] * 255.0),
               static_cast<unsigned char>(shadow_color[2] * 255.0),
               static_cast<unsigned char>(tprop_opacity * 255.0));

    // Required for clipping to work correctly

    glRasterPos2i(0, 0);
    glBitmap(0, 0, 0, 0,
             xoff + tprop->GetShadowOffset()[0],
             yoff + tprop->GetShadowOffset()[1], NULL);

    // Draw the shadow text

    shadow_font->render(this->Input, ftgl_context);

    // Get the font again, Duh, since it may have been freed from the
    // cache by the shadow font

    font = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop, tprop_color)->Font;
    if (!font)
      {
      vtkErrorMacro(<< "Render - No font");
      return;
      }

    // Shadow text for GL2PS.

#ifdef VTK_USE_GL2PS

    vtkOpenGLFreeTypeTextMapperExtra_LoadGL2PSFontData (shadow_entry, fontData);

      
      /* 
	 Check for white space and do not render it, 
	 otherwise it confuses gl2ps 
      */

      bool tmpFlag=false;
      int tmpIndx=0;

      while(this->Input[tmpIndx])
	{
	  if ( ! vtk_isspace(this->Input[tmpIndx]))
	    {
	      tmpFlag=true;
	      break;
	    };
	  tmpIndx++;
	};
      
      if (tmpFlag)
	{
	  float text_pos[2];
	  text_pos[0] = (vport[0] > tileViewport[0]) ? xoff+winSize[0]*(vport[0] - tileViewport[0])+1 : xoff+1;
	  text_pos[1] = (vport[1] > tileViewport[1]) ? yoff+winSize[1]*(vport[1] - tileViewport[1])-1 : yoff-1;
	  /* Use gl2psTextExtra because of angle support */
	  gl2psTextExtra(this->Input, tprop->GetFontSize(), tprop->GetOrientation(),text_pos,2,&fontData);
	};

#endif // VTK_USE_GL2PS
      
#if VTK_FTFC_CACHE_BY_RGBA
      
      if ((tprop->GetFontFamily() == VTK_GREEK) && (!tprop->GetFaceFileName()))
	{
	  font->renderExtra(angleRad,tmpBuf,ftgl_context);
	}
      else
	{
	  shadow_font->renderExtra(angleRad,this->Input, ftgl_context);
	};
      
      // Get the font again, Duh, since it may have been freed from the 
    // cache by the shadow font
      
      if (antialiasing_requested)
	{
	  entry = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop, 1, red, green, blue);
	  if ( entry)
	    {
	      font = entry->Font;
	      
	      if (!font) 
		{
		  vtkErrorMacro(<< "Render - No font");
		  return;
		}
#ifdef VTK_USE_GL2PS
              vtkOpenGLFreeTypeTextMapperExtra_LoadGL2PSFontData (entry, fontData);
#endif
	    }
	  else
	    return;
	}
#else
      if ((tprop->GetFontFamily() == VTK_GREEK) && (!tprop->GetFaceFileName()))
	{
	  font->renderExtra(angleRad,tmpBuf,ftgl_context);
	}
      else
      {
	font->renderExtra(angleRad,this->Input, ftgl_context);
      };
#endif
    }
  
  // Set the color here since load/render glyphs is done
  // on demand and this color has to be consistent for a given font entry.
  
  glColor4ub(static_cast<unsigned char>(tprop_color[0] * 255.0),
             static_cast<unsigned char>(tprop_color[1] * 255.0),
             static_cast<unsigned char>(tprop_color[2] * 255.0),
             static_cast<unsigned char>(tprop_opacity * 255.0));
  
/* Required for clipping to work correctly */
/* Using just glRasterPos will cause the entire text to be clipped
     if its outside of the clip bounds */
  glRasterPos2i(0, 0);
  glBitmap(0, 0, 0, 0, xoff, yoff, NULL);

  // Normal text for GL2PS. Must occur before rendering
  
#ifdef VTK_USE_GL2PS


 /* 
	 Check for white space and do not render it, 
	 otherwise it confuses gl2ps 
      */

      bool tmpFlag=false;
      int tmpIndx=0;

      while(this->Input[tmpIndx])
	{
	  if ( ! vtk_isspace(this->Input[tmpIndx]))
	    {
	      tmpFlag=true;
	      break;
	    };
	  tmpIndx++;
	};
      
      if (tmpFlag)
	{
	  float text_pos[2];
	  /* Adjust for changes in the location of the viewport. Text_pos should be in window coordinates */
	  text_pos[0] = (vport[0] > tileViewport[0]) ? xoff+winSize[0]*(vport[0] - tileViewport[0]) : xoff;
	  text_pos[1] = (vport[1] > tileViewport[1]) ? yoff+winSize[1]*(vport[1] - tileViewport[1]) : yoff;

	  /* Use gl2psTextExtra because of angle support */
	  gl2psTextExtra(this->Input, tprop->GetFontSize(), tprop->GetOrientation(), text_pos, 2, &fontData);
/*
const char *c = this->Input;
while (*c)
{
  unsigned int g = entry->Font->Face()->CharIndex( (unsigned int)(*c));
  FT_Glyph glyph = *(entry->Font->Face()->Glyph( g, FT_LOAD_NO_SCALE));
  FT_Face face = *(entry->Font->Face()->Face ());
  FT_GlyphSlot slot = face->glyph;
  cerr << *c << " " << (unsigned int)(*c) <<  " " << slot->metrics.horiAdvance*1000/face->units_per_EM << endl; //slot->metrics.width << " " ;
  c++;
}
*/
	}
#endif // VTK_USE_GL2PS
  
  // Display a string
  
  if ((tprop->GetFontFamily() == VTK_GREEK) && (!tprop->GetFaceFileName()))
    {
      font->renderExtra(angleRad,tmpBuf,ftgl_context);
      delete[] tmpBuf;
    }
  else
    {
      font->renderExtra(angleRad,this->Input, ftgl_context);
    };
  

  glFlush();
  
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
  glEnable(GL_LIGHTING);
  glDepthFunc(GL_LEQUAL);

#ifdef VTK_USE_GL2PS
  if (fontData.fontName)
    delete []fontData.fontName;
#endif 

}

void vtkOpenGLFreeTypeTextMapperExtra::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);
}

void vtkOpenGLFreeTypeTextMapperExtra::GetFontExtents(double& outLargestAscender,double& outLargestDescender)
{ 
  outLargestAscender=0;
  outLargestDescender=0;
  vtkTextProperty *tprop = this->GetTextProperty();
  if (!tprop)
    {
      vtkErrorMacro(<< "Need a text property to get size");
      return;
    }
  vtkFreeTypeUtilitiesExtra::Entry *entry = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop);
  
  if ( entry)
    {
      outLargestAscender=entry->LargestAscender;
      outLargestDescender=entry->LargestDescender;
    }
  else
    {
      return;
    };

};
  
void vtkOpenGLFreeTypeTextMapperExtra::GetBBox(double& llx, double& lly,double& urx,double& ury)
{

  llx=lly=urx=ury=0;
  
  
  // Check for input
  
  if (this->Input == NULL || this->Input[0] == '\0') 
    {
      return;
    }
  
  vtkTextProperty *tprop = this->GetTextProperty();
  if (!tprop)
    {
      vtkErrorMacro(<< "Need a text property to get size");
      return;
    }
  
  
  // Check for font and try to set the size
  
  
  vtkFreeTypeUtilitiesExtra::Entry *entry = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop);
  FTFontExtra *font;
  
  if ( entry)
    {
      font = entry->Font;
      
      if (!font) 
	{
	  vtkErrorMacro(<< "Render - No font");
	  return;
	}
    }
  else
    {
      return;
    };
  
  float tmpllx,tmplly,tmpurx,tmpury,tmpllz, tmpurz;
      
  if ((tprop->GetFontFamily() == VTK_GREEK) && (!tprop->GetFaceFileName()))
    {
      wchar_t* tmpBuf= new wchar_t[strlen(this->Input)+1];
      MapCharToWChar(SymbolFontWCharmap,this->Input,tmpBuf);
      font->BBox(tmpBuf, tmpllx, tmplly, tmpllz, tmpurx, tmpury, tmpurz);
      delete[] tmpBuf;
    }
  else
    {
      font->BBox(this->Input, tmpllx, tmplly, tmpllz, tmpurx, tmpury, tmpurz);
    };

  llx=tmpllx;
  lly=tmplly;
  urx=tmpurx;
  ury=tmpury;

};

void vtkOpenGLFreeTypeTextMapperExtra::GetApproximateBBox(double& llx, double& lly,double& urx,double& ury)
{

  llx=lly=urx=ury=0;
  
  
  // Check for input
  
  if (this->Input == NULL || this->Input[0] == '\0') 
    {
      return;
    }
  
  vtkTextProperty *tprop = this->GetTextProperty();
  if (!tprop)
    {
      vtkErrorMacro(<< "Need a text property to get size");
      return;
    }
  
  
  // Check for font and try to set the size
  
  
  vtkFreeTypeUtilitiesExtra::Entry *entry = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop);
  FTFontExtra *font;
  
  if ( entry)
    {
      font = entry->Font;
      
      if (!font) 
	{
	  vtkErrorMacro(<< "Render - No font");
	  return;
	}
    }
  else
    {
      return;
    };
  
  float tmpllx,tmplly,tmpurx,tmpury,tmpllz, tmpurz;
      
  // we need to include 'spaces' when we get the bounding box size
  // but preceeding/trailing (or all) spaces are not counted- so we
  // sub them out (w/ commas) to compute the bbox size
  long buflen = strlen(this->Input); 
  if (buflen > 0)
  {
  char* subBuf = new char[buflen+1];
  subBuf[buflen] = '\0'; 
  for (int rr = buflen;rr--;)
     if (this->Input[rr] == ' ')  subBuf[rr] = ',';
        else                      subBuf[rr] = this->Input[rr];

  if ((tprop->GetFontFamily() == VTK_GREEK) && (!tprop->GetFaceFileName()))
    {
      wchar_t* tmpBuf= new wchar_t[buflen+1];
      MapCharToWChar(SymbolFontWCharmap,subBuf,tmpBuf);
      font->BBox(tmpBuf, tmpllx, tmplly, tmpllz, tmpurx, tmpury, tmpurz);
      delete[] tmpBuf;
    }
  else
    {
      font->BBox(subBuf, tmpllx, tmplly, tmpllz, tmpurx, tmpury, tmpurz);
    };
  delete[] subBuf;
  }

  llx=tmpllx;
  lly=tmplly;
  urx=tmpurx;
  ury=tmpury;

};

void vtkOpenGLFreeTypeTextMapperExtra::MapCharToWChar(unsigned int* inMap,char* inStr,wchar_t* outStr)
{
  int indx=0;
  unsigned char c;

  while(inStr[indx])
    {
      c=inStr[indx];
      outStr[indx]=inMap[(int)c];
      indx++;
    };
  
  outStr[indx]=0;

};

void  vtkOpenGLFreeTypeTextMapperExtra::GetItalicAngle(double& outItalicAngle)
{
   
  vtkTextProperty *tprop = this->GetTextProperty();
  if (!tprop)
    {
      vtkErrorMacro(<< "Need a text property to get italic angle");
      return;
    }
  
  
  // Check for font and try to set the size
  
  
  vtkFreeTypeUtilitiesExtra::Entry *entry = vtkFreeTypeUtilitiesExtra::GetInstance()->GetFont(tprop);
  FTFontExtra *font;
  
  if ( entry)
    {
      font = entry->Font;
      
      if (!font) 
	{
	  vtkErrorMacro(<< "ItalicAngle - No font");
	  return;
	}
    }
  else
    {
      return;
    };
   
  outItalicAngle = font->ItalicAngle();

};

/* Regular isspace is not working in slang. */
int vtkOpenGLFreeTypeTextMapperExtra::vtk_isspace (char c)
{
  return ((c==' ') || (c=='\n') || (c=='\t') || (c=='\r') || (c=='\v') || (c=='\f'));
};

