// 
//  Copyright (C) 2005,2007,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkOpenGLFreeTypeTextMapperExtra.hh,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

     This file has been derived from version of vtkOpenGLFreeTypeTextMapper.h
     Distribution and modifications to this file are governed by the above
     copyright.

=========================================================================*/
// .NAME vtkOpenGLFreeTypeTextMapperExtra - 2D Text annotation support
// .SECTION Description
// vtkOpenGLFreeTypeTextMapperExtra provides 2D text annotation support for VTK
// using the FreeType and FTGL libraries. Normally the user should use 
// vtktextMapper which in turn will use this class.

// .SECTION See Also
// vtkTextMapper

#ifndef __vtkOpenGLFreeTypeTextMapperExtra_h
#define __vtkOpenGLFreeTypeTextMapperExtra_h

#include <cwchar>
#include "vtkTextMapper.h"

//BTX

#ifdef FTGL_USE_NAMESPACE
namespace ftgl
{
#endif
class FTFontExtra;
#ifdef FTGL_USE_NAMESPACE
}
#endif

//ETX

class VTK_RENDERING_EXPORT vtkOpenGLFreeTypeTextMapperExtra : public vtkTextMapper
{
public:
  vtkTypeRevisionMacro(vtkOpenGLFreeTypeTextMapperExtra,vtkTextMapper);
  static vtkOpenGLFreeTypeTextMapperExtra *New();
  virtual void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Actally draw the text.
  void RenderOverlay(vtkViewport* viewport, vtkActor2D* actor);

  // Description:
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  virtual void ReleaseGraphicsResources(vtkWindow *);

  // Description:
  // What is the size of the rectangle required to draw this
  // mapper ?
  virtual void GetSize(vtkViewport* viewport, double size[2]);
  void GetFontExtents(double& outLargestAscender,double& outLargestDescender);
  void GetBBox(double& llx, double& lly,double& urx,double& ury);
  void GetApproximateBBox(double& llx, double& lly,double& urx,double& ury);
  void GetItalicAngle(double& outItalicAngle);

protected:
  vtkOpenGLFreeTypeTextMapperExtra();
  ~vtkOpenGLFreeTypeTextMapperExtra();

  vtkTimeStamp  SizeBuildTime;
  double LastSize[2];
  double LastLargestDescender;

private:
  vtkOpenGLFreeTypeTextMapperExtra(const vtkOpenGLFreeTypeTextMapperExtra&);  // Not implemented.
  void operator=(const vtkOpenGLFreeTypeTextMapperExtra&);  // Not implemented.

  void MapCharToWChar(unsigned int* inMap,char* inStr,wchar_t* outStr);
  int vtk_isspace (char c);

};


#endif

