// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <math.h>
#include <bitset>
#include <vector>
#include <map>

#include "vtkPlanes.h"
#include "vtkPlane.h"
#include "vtkAppendPolyData.h"
#include "vtkGlyph2D.h"
#include "vtkGlyphSource2D.h"
#include "vtkIntArray.h"
#include "vtkCellArray.h"
#include "vtkObjectFactory.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkDoubleArray.h"
#include "vtkPlanes.h"
#include "vtkProperty2D.h"
#include "vtkContourFilter.h"
#include "vtkMarchingContourFilter.h"
#include "vtkMarchingSquares.h"
#include "vtkImageShiftScale.h"
#include "vtkDataSet.h"
#include "vtkDataSet.h"
#include "vtkPolyDataAlgorithm.h"
#include "vtkImageData.h"
#include "vtkBandedPolyDataContourFilter.h"
#include "vtkDataObjectToDataSetFilter.h"
#include "vtkStructuredGrid.h"
#include "vtkPointData.h"
#include "vtkPlotContourActor2DExtra.hh"

#include "vtkTransform.h"
#include "vtkTransformPolyDataFilter.h"

#include "vtkChipsTransform.hh"
#include <algorithm>

#ifndef MINLOG
#define MINLOG 0.01
#endif

/* TEST */
#include "vtkScalarTree.h"
#include "vtkDataArray.h"
#include "vtkCellData.h"
/* END TEST */


vtkCxxRevisionMacro(vtkPlotContourActor2DExtra, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkPlotContourActor2DExtra);



//----------------------------------------------------------------------------
// Instantiate this object.
vtkPlotContourActor2DExtra::vtkPlotContourActor2DExtra()
{
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(0.0, 0.0);

  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(1, 1);
  this->Position2Coordinate->SetReferenceCoordinate(NULL);
  
  this->MinLogXLimit=MINLOG;
  this->MinLogYLimit=MINLOG;

  this->XScale =0;
  this->YScale =0;



  this->XRange[0] = 0.0;
  this->XRange[1] = 1.0;

  this->YRange[0] = 0.0;
  this->YRange[1] = 1.0;

#ifndef NOCLIP
  for(int indx=0;indx<4;indx++)
    {
      ClipPlanes[indx]=vtkPlane::New();
    };

    // initialize clipping plane locations
  this->ClipPlanes[0]->SetOrigin(0,0,0);
  this->ClipPlanes[0]->SetNormal(1,0,0);
  this->ClipPlanes[1]->SetOrigin(1,0,0);
  this->ClipPlanes[1]->SetNormal(-1,0,0);
  this->ClipPlanes[2]->SetOrigin(0,0,0);
  this->ClipPlanes[2]->SetNormal(0,1,0);
  this->ClipPlanes[3]->SetOrigin(0,1,0);
  this->ClipPlanes[3]->SetNormal(0,-1,0);
#endif

  this->ContourLineData = vtkPolyData::New();
  this->ContourLineDataMapper = vtkPolyDataMapper2D::New();
  this->ContourLineDataMapper->SetInput(this->ContourLineData);

#ifndef NOCLIP
  // Using mapper AddClippingPlane function will not work because clipped
  // data gets messed up in the gl2ps output. Must use clip filter instead

  this->ContourLineDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->ContourLineDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->ContourLineDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->ContourLineDataMapper->AddClippingPlane (ClipPlanes[3]);

#endif

  this->ContourLineDataActor = vtkActor2D::New();
  this->ContourLineDataActor->SetMapper(this->ContourLineDataMapper);
  /*  this->ContourLineDataMapper->CreateDefaultLookupTable ();
  this->ContourLineDataMapper->SetColorModeToMapScalars ();
  this->ContourLineDataMapper->SetScalarRange(0,256);
  this->ContourLineDataMapper->UseLookupTableScalarRangeOn();
  */
  this->TicksVisibility=1;
  this->LabelsVisibility=1;
/*  this->LevelCount=0;
  this->Levels = static_cast<double *>(0);
*/  


  this->LastPosition[0] = this->LastPosition[1] = 0;
  this->LastPosition2[0] = this->LastPosition2[1] = 0;

  this->DataSet=vtkPolyData::New();
  this->DataSet->Initialize();
  this->DataSet->SetReleaseDataFlag (1);

  this->ContDataSet=vtkPolyData::New();
  this->ContDataSet->Initialize();
  this->ContDataSet->SetReleaseDataFlag (1);


  if (this->DataSet->GetPoints ())
     this->DataSet->GetPoints ()->SetDataTypeToDouble();

  this->Algorithm=0;

  this->transformFilters.clear ();

}

//----------------------------------------------------------------------------
vtkPlotContourActor2DExtra::~vtkPlotContourActor2DExtra()
{

  this->ContourLineData->Delete();
  this->ContourLineDataMapper->Delete();
  this->ContourLineDataActor->Delete();

#ifndef NOCLIP
  for(int indx=0;indx<4;indx++)
    {
      ClipPlanes[indx]->Delete();
    };
#endif

  this->DataSet->Delete();
  this->ContDataSet->Delete();


  for (unsigned int ii=0; ii<transformFilters.size (); ii++)
     transformFilters[ii]->Delete ();
  transformFilters.clear ();

/*  if (this->Levels)
     delete []this->Levels;
*/

}

//----------------------------------------------------------------------------
// Build the axis, ticks, title, and labels and render.

int vtkPlotContourActor2DExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  this->BuildData(viewport);
  

  renderedSomething += this->ContourLineDataActor->RenderOpaqueGeometry(viewport);

  return renderedSomething;
}


int vtkPlotContourActor2DExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;


  renderedSomething += this->ContourLineDataActor->RenderOverlay(viewport);
  

  return renderedSomething;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkPlotContourActor2DExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}


void vtkPlotContourActor2DExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  this->ContourLineDataActor->ReleaseGraphicsResources(win);
}


void vtkPlotContourActor2DExtra::BuildData(vtkViewport *viewport)
{

  if (! this->DataSet->GetNumberOfPoints())
    return;

  int positionsHaveChanged;
  double point[3]; point[2] = 0.0;
  vtkIdType numPts, ptId;
  double tmpPoint[3];
  vtkPoints *pts;



  int *pos1 = this->PositionCoordinate->GetComputedViewportValue(viewport);
  int *pos2 = this->Position2Coordinate->GetComputedViewportValue(viewport);



  positionsHaveChanged = 0;
  if (viewport->GetMTime() > this->BuildTime || 
      (viewport->GetVTKWindow() && 
       viewport->GetVTKWindow()->GetMTime() > this->BuildTime))
    {
    // Check to see whether we have to rebuild everything
    // Viewport change may not require rebuild
    int *lastPosition = 
      this->PositionCoordinate->GetComputedViewportValue(viewport);
    int *lastPosition2 =
      this->Position2Coordinate->GetComputedViewportValue(viewport);
    if (lastPosition[0] != this->LastPosition[0] ||
        lastPosition[1] != this->LastPosition[1] ||
        lastPosition2[0] != this->LastPosition2[0] ||
        lastPosition2[1] != this->LastPosition2[1] )
      {
      positionsHaveChanged = 1;
      this->LastPosition[0] = lastPosition[0];
      this->LastPosition[1] = lastPosition[1];
      this->LastPosition2[0] = lastPosition2[0];
      this->LastPosition2[1] = lastPosition2[1];

      }
    }
  
  if (!positionsHaveChanged &&
      this->GetMTime() < this->BuildTime )
    {
      return;
    }
  
  // Initialize and get important info

  //this->ContourLineData->Initialize();
  //  this->ContourLineDataActor->SetProperty(this->GetProperty());


  double tmpXRange[2];
  double tmpYRange[2];

  switch(this->XScale)
    {
    case 1:
      if (this->XRange[0] > 0 )
	tmpXRange[0]=log10(this->XRange[0]);
      else
	{
	  tmpXRange[0]=log10(this->MinLogXLimit);
	};
      if (this->XRange[1] > 0 )
	tmpXRange[1]=log10(this->XRange[1]);
      else
	{
	  tmpXRange[1]=log10(this->MinLogXLimit);
	};
      break;
    case 0:
    default:
      tmpXRange[0]=this->XRange[0];
      tmpXRange[1]=this->XRange[1];      
      break;
    };


  if ( tmpXRange[0]==tmpXRange[1] )
    {
      if ( tmpXRange[0]==0.0)
	{
	  tmpXRange[1]=1.0;
	}
      else
	{
	  tmpXRange[1]*=1.1;
	};
    };

  
  switch(this->YScale)
    {
    case 1:
      if (this->YRange[0] > 0 )
	tmpYRange[0]=log10(this->YRange[0]);
      else
	{
	  tmpYRange[0]=log10(this->MinLogYLimit);
	};
      if (this->YRange[1] > 0 )
	tmpYRange[1]=log10(this->YRange[1]);
      else
	{
	  tmpYRange[1]=log10(this->MinLogYLimit);
	};
      break;
    case 0:
    default:
      tmpYRange[0]=this->YRange[0];
      tmpYRange[1]=this->YRange[1];      
      break;
    };
  
  if ( tmpYRange[0]==tmpYRange[1] )
    {
      if ( tmpYRange[0]==0.0)
	{
	  tmpYRange[1]=1.0;
	}
      else
	{
	  tmpYRange[1]*=1.1;
	};
    };



  this->ContourLineData->Initialize();

  
  GenerateLines(this->DataSet,tmpXRange,this->XScale,tmpYRange,this->YScale,this->ContourLineData);



  numPts = this->ContourLineData->GetNumberOfPoints();
  pts=this->ContourLineData->GetPoints();
  

  for ( ptId=0; ptId < numPts; ptId++ )
    {

      pts->GetPoint(ptId,tmpPoint);

      if (
	  MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
	{
	  //	  cerr<<"vtkPlotContourActor2DExtra::BuildData: Ignoring invalid Point"<<endl;
	  continue;
	};
      
      if (
	  MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	{
	  //	  cerr<<"vtkPlotContourActor2DExtra::BuildData: Ignoring invalid Point "<<endl;
	  continue;
	};

      pts->SetPoint(ptId,tmpPoint);
    }

#ifndef NOCLIP  
    ClipPlanes[0]->SetOrigin(pos1[0],0,0);
    ClipPlanes[0]->SetNormal(1,0,0);

    ClipPlanes[1]->SetOrigin(pos2[0],0,0);
    ClipPlanes[1]->SetNormal(-1,0,0);

    ClipPlanes[2]->SetOrigin(0,pos1[1],0);
    ClipPlanes[2]->SetNormal(0,1,0);

    ClipPlanes[3]->SetOrigin(0,pos2[1],0);
    ClipPlanes[3]->SetNormal(0,-1,0);
#endif

    this->BuildTime.Modified();

  
}

//----------------------------------------------------------------------------
void vtkPlotContourActor2DExtra::ShallowCopy(vtkProp *prop)
{
  /*  vtkPlotContourActor2DExtra *a = vtkPlotContourActor2DExtra::SafeDownCast(prop);
  if ( a != NULL )
    {
      this->SetRange(a->GetRange());
      this->SetNumberOfMajTicks(a->GetNumberOfMajTicks());
      this->SetNumberOfMinTicks(a->GetNumberOfMinTicks());
      this->SetLabelFormat(a->GetLabelFormat());
      this->SetAdjustTicks(a->GetAdjustTicks());
      this->SetMajTicksLength(a->GetMajTicksLength());
      this->SetMinTicksLength(a->GetMinTicksLength());
      this->SetTickOffset(a->GetTickOffset());
      
      this->SetLabelOffset(a->GetLabelOffset());

      this->SetTitlePerpendicularOffset(a->GetTitlePerpendicularOffset());
      this->SetAxisVisibility(a->GetAxisVisibility());
      this->SetMajTicksVisibility(a->GetMajTicksVisibility());
      this->SetMinTicksVisibility(a->GetMinTicksVisibility());
      this->SetLabelVisibility(a->GetLabelVisibility());
      this->SetTitleVisibility(a->GetTitleVisibility());
      this->SetFontFactor(a->GetFontFactor());
      this->SetLabelFactor(a->GetLabelFactor());
      this->SetLabelTextProperty(a->GetLabelTextProperty());
      this->SetTitleTextProperty(a->GetTitleTextProperty());
    }
    */  
  // Now do superclass
  this->vtkActor2D::ShallowCopy(prop);
};

int vtkPlotContourActor2DExtra::MapData(double* inDataRange, 
				  double inPixelRangeMin, 
				  double inPixelRangeMax,
				  double& inOutValue)
{

  if  ( ( inDataRange[0] == inDataRange[1]) && (inPixelRangeMin != inPixelRangeMax))
    {
      return -1;
    };
  
  if (inDataRange[0] == inDataRange[1])
    return static_cast<int>(inPixelRangeMin);


      inOutValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(inOutValue-inDataRange[0])/(inDataRange[1]-inDataRange[0]);


  return 1;

};

void vtkPlotContourActor2DExtra::SetLocation(double inMinX, double inMinY,double inMaxX,double inMaxY)
{
  this->PositionCoordinate->SetValue(inMinX,inMinY);
  this->Position2Coordinate->SetValue(inMaxX,inMaxY);

  this->Modified();
};




void vtkPlotContourActor2DExtra::GenerateLines(vtkPolyData* inData,double inXRange[2], int inXScale,double inYRange[2],int inYScale,vtkPolyData* outLineData)
{
  if ((!inData) || (!outLineData) )
    return;

  vtkPoints *points=inData->GetPoints();
  vtkPoints *newPoints;
  vtkCellArray *newLines;
  vtkCellArray *oldLines = inData->GetLines();
  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType npts = 0;
  vtkIdType newPtsId[2];
  vtkIdType* oldPtsId;
  double x1[3], x2[3], t;
  double tmpXRange[2],tmpYRange[2];

  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];

  outLineData->Initialize();


  // Use cohen-sutherland line clipping algorithm


  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }
  


  newPoints = vtkPoints::New();
  newPoints->SetDataTypeToDouble();
  newPoints->Allocate(numPts);

  newLines = vtkCellArray::New();
  newLines->Allocate(numPts-1);

  
  for( oldLines->InitTraversal();oldLines->GetNextCell(npts,oldPtsId);)
    {
      
      points->GetPoint(oldPtsId[0], x1);
      points->GetPoint(oldPtsId[1], x2);

      if ( inXScale== 1)
	{
	  
	  if ( (x1[0] <= 0.0 ) || (x2[0] <= 0.0 ))
	    continue;
	  else
	    {
	      x1[0]=log10(x1[0]);
	      x2[0]=log10(x2[0]);
	    };
	  
	};
      
      
      
      
      if ( inYScale== 1)
	{
	  
	  if ( (x1[1] <= 0.0 ) || (x2[1] <= 0.0 ))
	    continue;
	  else
	    {
	      x1[1]=log10(x1[1]);
	      x2[1]=log10(x2[1]);
	    };

	};
      

      newPtsId[0] = newPoints->InsertNextPoint(x1);
      newPtsId[1] = newPoints->InsertNextPoint(x2);
      newLines->InsertNextCell(2,newPtsId);


    };  
	  
  //Update the lines
  outLineData->SetPoints(newPoints);
  outLineData->SetVerts(0);
  outLineData->SetLines(newLines);
  
  newPoints->Delete();
  newLines->Delete();
  
  

};




void vtkPlotContourActor2DExtra::Generate(vtkDataSet *inData,
					  int inAlgorithm,
					  int inLevelCount,
					  double inMin,double inMax)
{



  // initialize contour data
  
  this->DataSet->Initialize();
  this->ContDataSet->Initialize();


  // put data into local vtk structures

  if (inData)
    {
      double *scalarRange = static_cast<double *>(0);
      double *tmpLevels = static_cast<double *>(0);
      int tmpLevelCount = 0;

      this->Algorithm=inAlgorithm;
      switch ( this->Algorithm )
	{
	case 1: 
	  {
	  vtkMarchingContourFilter * contFilterMF = vtkMarchingContourFilter::New();

	  vtkImageData * tmpImageData = vtkImageData::New();
	  int * tmpDims = ((vtkStructuredGrid*)inData)->GetDimensions();
	  tmpImageData->SetDimensions(tmpDims);

	  tmpImageData->GetPointData()->SetScalars(inData->GetPointData()->GetScalars());
	  if ( inData->GetNumberOfPoints() >1)
	    {

	      double tmpPtr[3];
	      double tmpPtr1[3];
	      inData->GetPoint(0,tmpPtr);
	      inData->GetPoint(1,tmpPtr1);
	      tmpImageData->SetOrigin(tmpPtr[0],tmpPtr[1],0);
	      double xSpacing=tmpPtr1[0] -tmpPtr[0];
	      inData->GetPoint(tmpDims[0],tmpPtr1);
	      double ySpacing=tmpPtr1[1] -tmpPtr[1];
	      tmpImageData->SetSpacing(xSpacing,ySpacing,0);


	    };


	  contFilterMF->GenerateValues(inLevelCount,inMin,inMax);
	  

	  contFilterMF->SetInput(tmpImageData);
	  
          if (transformFilters.size () > 0)
            {
            transformFilters[0]->SetInputConnection (contFilterMF->GetOutputPort ());
            for (unsigned int ii=1; ii<transformFilters.size (); ii++)
              { 
              transformFilters[ii]->SetInputConnection (transformFilters[ii-1]->GetOutputPort ());
              }
            transformFilters.back ()->Update ();
            this->DataSet->ShallowCopy( transformFilters.back ()->GetOutput());
            this->ContDataSet->ShallowCopy ( contFilterMF->GetOutput());
            }
          else
            {
               contFilterMF->Update();
	       this->DataSet->ShallowCopy( contFilterMF->GetOutput());
               this->ContDataSet->ShallowCopy ( contFilterMF->GetOutput());
            }

          // save off the levels
          tmpLevelCount = contFilterMF->GetNumberOfContours ();
          tmpLevels = contFilterMF->GetValues ();

	  
	  tmpImageData->Delete();
	  contFilterMF->Delete();
          }
	  
	  break;

	  /*	case 1:
	  vtkMarchingContourFilter* contFilterMF =vtkMarchingContourFilter::New();
	  contFilterMF->SetInput(inData);
	  contFilterMF->GenerateValues(inLevelCount,inMin,inMax);
	  contFilterMF->Update();
	  this->DataSet->ShallowCopy( contFilterMF->GetOutput());
      
	  contFilterMF->Delete();

	  break;

	  
	    case 3:
	  vtkBandedPolyDataContourFilter * contFilterB = vtkBandedPolyDataContourFilter::New();
	  
	  vtkPolyData * tmpPolyData= vtkPolyData::New();
	  vtkPoints *tmpPts = vtkPoints::New();
	  tmpPts->SetDataTypeToDouble();
	  tmpPts->Allocate(inData->GetNumberOfPoints());
	  
	  
	  double* tmpPtr;

	  for(int indx=0;indx<inData->GetNumberOfPoints();indx++)
	    {
	      tmpPtr=inData->GetPoint(indx);
	      tmpPts->InsertNextPoint(tmpPtr[0],tmpPtr[1],0);
	    }


	  tmpPolyData->SetPoints(tmpPts);
	  tmpPolyData->GetPointData()->SetScalars(inData->GetPointData()->GetScalars());

	  tmpPts->Delete();


	  contFilterB->SetInput(tmpPolyData);
	  
	  contFilterB->GenerateValues(inLevelCount,inMin,inMax);
	  contFilterB->Update();
	  this->DataSet->ShallowCopy( contFilterB->GetOutput());
	  
	  tmpPolyData->Delete();
	  contFilterB->Delete();

	  break;
	  */
	case 0:	
	default:
          {
	  vtkContourFilter* contFilter =vtkContourFilter::New();
	  contFilter->SetInput(inData);
	  contFilter->GenerateValues(inLevelCount,inMin,inMax);
          if (transformFilters.size () > 0)
            {
            transformFilters[0]->SetInputConnection (contFilter->GetOutputPort ());
            for (unsigned int ii=1; ii<transformFilters.size (); ii++)
              {
              transformFilters[ii]->SetInputConnection (transformFilters[ii-1]->GetOutputPort ());
              }
            transformFilters.back ()->Update ();
            this->DataSet->ShallowCopy( transformFilters.back ()->GetOutput());
            this->ContDataSet->ShallowCopy ( contFilter->GetOutput());
            }
          else
            {
  	      contFilter->Update();
	      this->DataSet->ShallowCopy( contFilter->GetOutput());
              this->ContDataSet->ShallowCopy ( contFilter->GetOutput());
            }
 
          // save off the levels
          tmpLevelCount = contFilter->GetNumberOfContours ();
          tmpLevels = contFilter->GetValues ();
	  
	  contFilter->Delete();
        }
	break;
	  
      };

    // save off the levels
    scalarRange = this->DataSet->GetScalarRange ();
    this->Levels.clear ();
    for (int ii=0; ii<tmpLevelCount; ii++)
      {
      if ((tmpLevels[ii] >= scalarRange[0]) &&
          (tmpLevels[ii] <= scalarRange[1]))
        this->Levels.push_back(tmpLevels[ii]);
      }

    };

  this->DataSet->ComputeBounds();
  double* bounds = this->DataSet->GetBounds();
  this->XDataRange[0]=bounds[0];
  this->XDataRange[1]=bounds[1];
  this->YDataRange[0]=bounds[2];
  this->YDataRange[1]=bounds[3];


  this->Modified();
  
};


void vtkPlotContourActor2DExtra::Generate(vtkDataSet *inData,
					  int inAlgorithm,
					  const vector<double>&  inLevels)
{



  // initialize contour data
  
  this->DataSet->Initialize();
  this->DataSet->Initialize();


  // put data into local vtk structures

  if (inData)
    {
    map<double, int>tmpLevels;


      this->Algorithm=inAlgorithm;
      switch ( this->Algorithm )
	{
	case 1:
	 { 

	  vtkMarchingContourFilter * contFilterMF = vtkMarchingContourFilter::New();

	  vtkImageData * tmpImageData = vtkImageData::New();
	  int * tmpDims = ((vtkStructuredGrid*)inData)->GetDimensions();
	  tmpImageData->SetDimensions(tmpDims);

	  tmpImageData->GetPointData()->SetScalars(inData->GetPointData()->GetScalars());
	  if ( inData->GetNumberOfPoints() >1)
	    {

	      double tmpPtr[3];
	      double tmpPtr1[3];
	      inData->GetPoint(0,tmpPtr);
	      inData->GetPoint(1,tmpPtr1);
	      tmpImageData->SetOrigin(tmpPtr[0],tmpPtr[1],0);
	      double xSpacing=tmpPtr1[0] -tmpPtr[0];
	      inData->GetPoint(tmpDims[0],tmpPtr1);
	      double ySpacing=tmpPtr1[1] -tmpPtr[1];
	      tmpImageData->SetSpacing(xSpacing,ySpacing,0);

	    };


	  for(unsigned int indx=0;indx<inLevels.size();indx++)
	    {
	      contFilterMF->SetValue(indx,inLevels[indx]);
	    }

          contFilterMF->SetInput(tmpImageData);


          if (transformFilters.size () > 0)
            {
            transformFilters[0]->SetInputConnection (contFilterMF->GetOutputPort ());
            for (unsigned int ii=1; ii<transformFilters.size (); ii++)
              {
              transformFilters[ii]->SetInputConnection (transformFilters[ii-1]->GetOutputPort ());
              }
            transformFilters.back ()->Update ();
            this->DataSet->ShallowCopy( transformFilters.back ()->GetOutput ());
            this->ContDataSet->ShallowCopy ( contFilterMF->GetOutput());
            }
          else
            {
            contFilterMF->Update();
            this->DataSet->ShallowCopy( contFilterMF->GetOutput());
            this->ContDataSet->ShallowCopy ( contFilterMF->GetOutput());
            }

	  tmpImageData->Delete();
	  contFilterMF->Delete();
	}

	break;
	  /*
	  case 3:
	  vtkBandedPolyDataContourFilter * contFilterB = vtkBandedPolyDataContourFilter::New();
	  
	  vtkPolyData * tmpPolyData= vtkPolyData::New();
	  vtkPoints *tmpPts = vtkPoints::New();
	  tmpPts->SetDataTypeToDouble();
	  tmpPts->Allocate(inData->GetNumberOfPoints());
	  
	  double* tmpPtr;

	  for(int indx=0;indx<inData->GetNumberOfPoints();indx++)
	    {
	      tmpPtr=inData->GetPoint(indx);
	      tmpPts->InsertNextPoint(tmpPtr[0],tmpPtr[1],0);
	    }


	  tmpPolyData->SetPoints(tmpPts);

	  tmpPolyData->GetPointData()->SetScalars(inData->GetPointData()->GetScalars());

	  tmpPts->Delete();


	  contFilterB->SetInput(tmpPolyData);

	  for(int indx=0;indx<inLevels.size();indx++)
	    {
	      contFilterB->SetValue(indx,inLevels[indx]);
	    }

	  contFilterB->Update();

	  this->DataSet->ShallowCopy( contFilterMS->GetOutput());
	  
	  tmpPolyData->Delete();
	  contFilterB->Delete();




	  break;
	  */
	case 0:
	default:	
         {
	  vtkContourFilter* contFilter =vtkContourFilter::New();

	  for(unsigned int indx=0;indx<inLevels.size();indx++)
	    {
	      contFilter->SetValue(indx,inLevels[indx]);
	    }

          contFilter->SetInput(inData);

          if (transformFilters.size () > 0)
            {
            transformFilters[0]->SetInputConnection (contFilter->GetOutputPort ());
            for (unsigned int ii=1; ii<transformFilters.size (); ii++)
              {
              transformFilters[ii]->SetInputConnection (transformFilters[ii-1]->GetOutputPort ());
              }
            transformFilters.back ()->Update ();
            this->DataSet->ShallowCopy( transformFilters.back ()->GetOutput());
            this->ContDataSet->ShallowCopy ( contFilter->GetOutput());
            }
          else
            {
               contFilter->Update();
               this->DataSet->ShallowCopy( contFilter->GetOutput());
               this->ContDataSet->ShallowCopy ( contFilter->GetOutput());
            }


	  contFilter->Delete();
         }
	  
	  break;
	};

    // save off the exact created levels

    // get all the scalars 
    vtkDataArray *scalars = NULL;
    this->Levels.clear ();
    if (this->DataSet->GetPointData () && this->DataSet->GetPointData ()->GetScalars ())
      {
      scalars = this->DataSet->GetPointData ()->GetScalars ();
      for (int ii=0; ii<scalars->GetNumberOfTuples (); ii++)
        {
        // count the number of times a scalar exists in the list
        if (tmpLevels.count (scalars->GetComponent (ii, 0)) == 0)
           tmpLevels[scalars->GetComponent (ii, 0)] = 1;
        else
           tmpLevels[scalars->GetComponent (ii, 0)]++;
        }

      for (map<double, int>::iterator iter = tmpLevels.begin (); iter != tmpLevels.end (); iter++)
        {
        // only consider levels that have more than one point. Single points will not
        // actually have a contour
        if (iter->second > 1)
          this->Levels.push_back(iter->first);
        }
      // sort the levels
      sort (this->Levels.begin(), this->Levels.end ());
      }
    };

  this->DataSet->ComputeBounds();
  double* bounds = this->DataSet->GetBounds();
  this->XDataRange[0]=bounds[0];
  this->XDataRange[1]=bounds[1];
  this->YDataRange[0]=bounds[2];
  this->YDataRange[1]=bounds[3];


  this->Modified();

};

void vtkPlotContourActor2DExtra::AddTransform (vtkAbstractTransform *transform)
{
  vtkTransformPolyDataFilter *filter = vtkTransformPolyDataFilter::New ();
  filter->ReleaseDataFlagOn ();
  filter->SetTransform (transform);
  transformFilters.push_back (filter);
}

void vtkPlotContourActor2DExtra::RemoveTransform (int index)
{
  if (((unsigned int)index >= transformFilters.size ()) || (index < 0))
    return ;

  vector <vtkTransformPolyDataFilter *>::iterator iter;
  iter = this->transformFilters.begin ();
  for (int ii=0; ii<index; ii++, iter++);

  
  (*iter)->Delete ();
  transformFilters.erase (iter);
}

void vtkPlotContourActor2DExtra::RemoveAllTransforms ()
{
  for (unsigned int ii=0; ii<transformFilters.size (); ii++)
    transformFilters[ii]->Delete ();
  transformFilters.clear ();

}


void vtkPlotContourActor2DExtra::ApplyTransform (vtkAbstractTransform *transform)
{
  if (transform)
    {
      AddTransform (transform);
    }

  if (transformFilters.size () > 0)
    {

    transformFilters.back ()->SetInput (this->DataSet);
    transformFilters.back ()->Update ();

    this->DataSet->Initialize();
    this->DataSet->ShallowCopy( transformFilters.back ()->GetOutput());

    this->DataSet->ComputeBounds();

    double* bounds = this->DataSet->GetBounds();
    this->XDataRange[0]=bounds[0];
    this->XDataRange[1]=bounds[1];
    this->YDataRange[0]=bounds[2];
    this->YDataRange[1]=bounds[3];

    this->Modified();
    }

}

void vtkPlotContourActor2DExtra::ApplyTransformInverse ()
{
  if (transformFilters.empty ())
     return;

  // turn inverse on
  transformFilters.back ()->GetTransform ()->Inverse ();

  this->ApplyTransform ();

  // turn inverse off
  transformFilters.back ()->GetTransform ()->Inverse ();

  this->Modified();
}

void vtkPlotContourActor2DExtra::ResetContourData ()
{
  this->DataSet->Initialize ();
  this->DataSet->ShallowCopy(ContDataSet);
  this->DataSet->ComputeBounds();

  double* bounds = this->DataSet->GetBounds();
  this->XDataRange[0]=bounds[0];
  this->XDataRange[1]=bounds[1];
  this->YDataRange[0]=bounds[2];
  this->YDataRange[1]=bounds[3];

  this->Modified();
}

vector<double> vtkPlotContourActor2DExtra::GetLevels ()
{
   return this->Levels;
}

void vtkPlotContourActor2DExtra::GetLevels (vector<double> &outLevels)
{
   outLevels = this->Levels;
}

void vtkPlotContourActor2DExtra::GetClipPlanes (int &inOutXMin, int &inOutXMax, int &inOutYMin, int &inOutYMax)
{
#ifndef NOCLIP
  inOutXMin = this->ClipPlanes[0]->GetOrigin()[0];
  inOutXMax = this->ClipPlanes[1]->GetOrigin()[0];
  inOutYMin = this->ClipPlanes[2]->GetOrigin()[1];
  inOutYMax = this->ClipPlanes[3]->GetOrigin()[1];
#endif

}

void vtkPlotContourActor2DExtra::GetBounds (vtkViewport *viewport, double bounds[4])
{
  this->BuildData (viewport);

  vtkPoints *pts = this->ContourLineData->GetPoints();

  if (!pts)
    {
    bounds[0] = 0;
    bounds[1] = 0;
    bounds[2] = 0;
    bounds[3] = 0;
    }
  else
    {
    double *ptsBounds = pts->GetBounds ();
 
    bounds[0] = ptsBounds[0];
    bounds[1] = ptsBounds[1];
    bounds[2] = ptsBounds[2];
    bounds[3] = ptsBounds[3];
    }
}
