// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



#ifndef __vtkPlotContourActor2DExtra_hh
#define __vtkPlotContourActor2DExtra_hh

#include <string>
#include <vector>
#include "vtkActor2D.h"
#include "vtkGlyphSource2D.h"
#include "vtkProperty2D.h"
#include "vtkDoubleArray.h"
#include "vtkPlane.h"
#include "vtkAbstractTransform.h"


using namespace std;

//class vector;

class vtkPolyDataMapper2D;
class vtkPolyData;
class vtkPlanes;
class vtkGlyph2D;
class vtkAppendPolyData;
class vtkPlanes;
class vtkGlyphSource2D;
class vtkDataSet;
class vtkTransformPolyDataFilter;

class vtkPlotContourActor2DExtra : public vtkActor2D
{
public:
  vtkTypeRevisionMacro(vtkPlotContourActor2DExtra,vtkActor2D);

  // Description:
  // Instantiate object.
  static vtkPlotContourActor2DExtra *New();


  // Description:
  // Set actual data
  //                algorithm is on of 
  //               (0 - vtk's , 1 marching sqares, 2 marching filter )


  void Generate(vtkDataSet *inData,
		int inAlgorithm,
		int inLevelCount,
		double inMin,double inMax);
  
  void Generate(vtkDataSet *inData,
		int inAlgorithm,
		const vector<double>&  inLevels);
  


  void SetLocation(double inMinX, double inMinY,double inMaxX,double inMaxY);


  // Description:
  // Specify the (min,max) x data range.
  vtkSetVector2Macro(XRange,double);
  vtkGetVectorMacro(XRange,double,2);


  
  // Description:
  // Specify the (min,max) y data range.
  vtkSetVector2Macro(YRange,double);
  vtkGetVectorMacro(YRange,double,2);

  // Description:
  // Specify the (min,max) x data range.
  vtkSetVector2Macro(XDataRange,double);
  vtkGetVectorMacro(XDataRange,double,2);



  // Description:
  // Specify the (min,max) y data range.
  vtkSetVector2Macro(YDataRange,double);
  vtkGetVectorMacro(YDataRange,double,2);

  // Description: 
  // Get the contour levels
  void GetLevels (vector<double> &outLevels);
  vector<double> GetLevels();
  



  // Description:
  // Set/Get the min value when converting to log

  void SetMinLogXLimit( double inMinLogLimit)
  { if ( inMinLogLimit> 0)
    { this->MinLogXLimit=inMinLogLimit;this->Modified();}
  ;};
  double  GetMinLogXLimit(){return this->MinLogXLimit;};
  
  void SetMinLogYLimit( double inMinLogLimit)
  { if ( inMinLogLimit> 0)
    { this->MinLogYLimit=inMinLogLimit;this->Modified();}
  ;};
  double  GetMinLogYLimit(){return this->MinLogYLimit;};


  vtkSetMacro(LabelsVisibility, int);
  vtkGetMacro(LabelsVisibility, int);
  vtkBooleanMacro(LabelsVisibility, int);


  vtkSetMacro(TicksVisibility, int);
  vtkGetMacro(TicksVisibility, int);
  vtkBooleanMacro(TicksVisibility, int);

  vtkSetMacro(TicksLength, int);
  vtkGetMacro(TicksLength, int);




  // Description:
  // Set/Get scale of X data (linear,log...)
  vtkSetClampMacro(XScale, int, 0,10);
  vtkGetMacro(XScale, int);


  // Description:
  // Set/Get scale of Y data (linear,log...)
  vtkSetClampMacro(YScale, int, 0,10);
  vtkGetMacro(YScale, int);


  // Description:
  // Set/Get contouring algorithm 
  //               (0 - vtk's , 1 marching sqares, 2 marching filter )
  vtkGetMacro(Algorithm, int);




  // Description:
  // Draw the axis. 
  int RenderOverlay(vtkViewport* viewport);
  int RenderOpaqueGeometry(vtkViewport* viewport);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport *) {return 0;}

  // Description:
  // Does this prop have some translucent polygonal geometry?
  virtual int HasTranslucentPolygonalGeometry();

  // Description:
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  void ReleaseGraphicsResources(vtkWindow *);


  // Description:
  // Shallow copy of a data actor. Overloads the virtual vtkProp method.
  void ShallowCopy(vtkProp *prop);

  // Add Transform
  void AddTransform (vtkAbstractTransform *transform);
  void RemoveTransform (int index);
  void RemoveAllTransforms ();
  void ApplyTransform (vtkAbstractTransform *transform=static_cast<vtkAbstractTransform *>(0));
  void ApplyTransformInverse ();
  void ResetContourData ();

  // Accessors for lines 

  vtkActor2D* GetLineActor(){return this->ContourLineDataActor;};

  // Get the bounds of the contour
  void GetBounds (vtkViewport *viewport, double bounds[4]);

  // Get the clipping planes in pixels coordinates
  void GetClipPlanes (int &inOutXMin, int &inOutXMax, int &inOutYMin, int &inOutYMax);


protected:
  vtkPlotContourActor2DExtra();
  ~vtkPlotContourActor2DExtra();


  double XRange[2];
  double YRange[2];


  double XDataRange[2];
  double YDataRange[2];


  int   XScale;
  int   YScale;

  double MinLogXLimit;
  double MinLogYLimit;


  int   TicksVisibility;
  int   LabelsVisibility;
  
  int   TicksLength;

  vector<double> Levels;



  int   LastPosition[2];
  int   LastPosition2[2];
  

  //Description:
  // build data ready to display

  virtual void BuildData(vtkViewport *viewport);


  // Description:
  // Map data into different scales

  static int MapData(double* inDataRange, 
		      double inPixelRangeMin, 
		      double inPixelRangeMax,
		      double& inOutValue);


  
  //Description:
  // Clip points and generate lines
  void GenerateLines(vtkPolyData* inData,double inXRange[2], int inXScale, double inYRange[2], int inYScale,vtkPolyData* outLineData);


  vtkPolyData             *DataSet;
  vtkPolyData             *ContourLineData;
  vtkPlane                *ClipPlanes[4];
  vtkPolyDataMapper2D     *ContourLineDataMapper;
  vtkActor2D              *ContourLineDataActor;

  vector <vtkTransformPolyDataFilter *>transformFilters;
  vtkPolyData             *ContDataSet;

  int                     Algorithm;


  vtkTimeStamp  BuildTime;

private:
  vtkPlotContourActor2DExtra(const vtkPlotContourActor2DExtra&);  // Not implemented.
  void operator=(const vtkPlotContourActor2DExtra&);  // Not implemented.


};


#endif
