// 
//  Copyright (C) 2004-2007,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <math.h>
#include <float.h>
#include <bitset>

#include "vtkPlanes.h"
#include "vtkPlane.h"
#include "vtkAppendPolyData.h"
#include "vtkGlyph2D.h"
#include "vtkGlyphSource2D.h"
#include "vtkIntArray.h"
#include "vtkCellArray.h"
#include "vtkObjectFactory.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkDoubleArray.h"
#include "vtkPlanes.h"
#include "vtkProperty2D.h"
#include "vtkPlotDataActor2DExtra.hh"
#include "vtkMath.h"



#ifndef MINLOG
#define MINLOG 0.01
#endif


vtkCxxRevisionMacro(vtkPlotDataActor2DExtra, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkPlotDataActor2DExtra);



//----------------------------------------------------------------------------
// Instantiate this object.
vtkPlotDataActor2DExtra::vtkPlotDataActor2DExtra()
{
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(0.0, 0.0);

  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(1, 1);
  this->Position2Coordinate->SetReferenceCoordinate(NULL);
  
  this->MinLogXLimit=MINLOG;
  this->MinLogYLimit=MINLOG;

  this->XScale =0;
  this->YScale =0;
  this->DataFlip=0;


  this->XRange[0] = 0.0;
  this->XRange[1] = 1.0;

  this->YRange[0] = 0.0;
  this->YRange[1] = 1.0;

#ifndef NOCLIP
  // Create data clippers
  for(int indx=0;indx<4;indx++)
    ClipPlanes[indx]=vtkPlane::New();

  // initialize clipping plane locations
  this->ClipPlanes[0]->SetOrigin(0,0,0);
  this->ClipPlanes[0]->SetNormal(1,0,0);
  this->ClipPlanes[1]->SetOrigin(1,0,0);
  this->ClipPlanes[1]->SetNormal(-1,0,0);
  this->ClipPlanes[2]->SetOrigin(0,0,0);
  this->ClipPlanes[2]->SetNormal(0,1,0);
  this->ClipPlanes[3]->SetOrigin(0,1,0);
  this->ClipPlanes[3]->SetNormal(0,-1,0);
#endif


  this->PlotLineData = vtkPolyData::New();
  this->PlotLineDataMapper = vtkPolyDataMapper2D::New();
  this->PlotLineDataMapper->SetInput(this->PlotLineData);

#ifndef NOCLIP
  // Using mapper AddClippingPlane function will not work because clipped
  // data gets messed up in the gl2ps output. Must use clip filter instead

  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[3]);

#endif

  this->PlotLineDataActor = vtkActor2D::New();
  this->PlotLineDataActor->SetMapper(this->PlotLineDataMapper);



  this->PlotPointDataSet = vtkPolyData::New();

  this->PlotPointData = vtkGlyph2D::New();
  this->PlotPointData->SetInput(this->PlotPointDataSet);

  this->PlotPointDataMapper = vtkPolyDataMapper2D::New();
  this->PlotPointDataMapper->SetInputConnection(this->PlotPointData->GetOutputPort ());

#ifndef NOCLIP

  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->PlotPointDataMapper->AddClippingPlane (ClipPlanes[3]);

#endif
  this->PlotPointDataActor = vtkActor2D::New();
  this->PlotPointDataActor->SetMapper(this->PlotPointDataMapper);

  this->PlotPointData->ScalingOff();
  this->PlotPointData->SetIndexModeToOff();
//  this->PlotPointData->SetNumberOfSources(1);
  this->GlyphSource = vtkGlyphSource2D::New();
  this->GlyphSource->SetColor(1,1,0);
  this->GlyphSource->SetGlyphType(1);
  this->GlyphSource->SetScale(10);
  this->GlyphSource->SetRotationAngle(0);
  this->GlyphSource->SetFilled(1);

  this->PlotPointData->SetSource(this->GlyphSource->GetOutput());
  this->PlotPointDataActor->GetProperty()->SetColor(1,0,0);

  

  this->ErrData = vtkPolyData::New();

  this->ErrDataMapper = vtkPolyDataMapper2D::New();
  this->ErrDataMapper->SetInput(this->ErrData);

#ifndef NOCLIP

  this->ErrDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->ErrDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->ErrDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->ErrDataMapper->AddClippingPlane (ClipPlanes[3]);

#endif
  this->ErrDataActor = vtkActor2D::New();
  this->ErrDataActor->SetMapper(this->ErrDataMapper);


  this->DataLimits = vtkPolyData::New();

  this->DataLimitsMapper = vtkPolyDataMapper2D::New();
  this->DataLimitsMapper->SetInput(this->DataLimits);

#ifndef NOCLIP

  this->DataLimitsMapper->AddClippingPlane (ClipPlanes[0]);
  this->DataLimitsMapper->AddClippingPlane (ClipPlanes[1]);
  this->DataLimitsMapper->AddClippingPlane (ClipPlanes[2]);
  this->DataLimitsMapper->AddClippingPlane (ClipPlanes[3]);

#endif
  this->DataLimitsActor = vtkActor2D::New();
  this->DataLimitsActor->SetMapper(this->DataLimitsMapper);

  this->LinesVisibility=1;
  this->PointsVisibility=1;
  this->UpErrBarVisibility=1;
  this->DownErrBarVisibility=1;
  this->LeftErrBarVisibility=1;
  this->RightErrBarVisibility=1;
  this->ErrBarCapVisibility=1;
  this->XLimitsVisibility=1;
  this->YLimitsVisibility=1;
  this->DataLimitsCapVisibility=1;

  this->ErrBarCapLength=10;
  this->DataLimitsBarLength=.1;
  this->DataLimitsOverride=0;
  this->DataLimitsCapLength=10;


  this->LastPosition[0] = this->LastPosition[1] = 0;
  this->LastPosition2[0] = this->LastPosition2[1] = 0;

  this->DataSet=vtkPolyData::New();
  this->DataSet->Initialize();

  this->UpDataSet=vtkDoubleArray::New();
  this->UpDataSet->Initialize();

  this->DownDataSet=vtkDoubleArray::New();
  this->DownDataSet->Initialize();

  this->LeftDataSet=vtkDoubleArray::New();
  this->LeftDataSet->Initialize();

  this->RightDataSet=vtkDoubleArray::New();
  this->RightDataSet->Initialize();

  this->XLimitsSet=vtkIntArray::New();
  this->XLimitsSet->Initialize();

  this->YLimitsSet=vtkIntArray::New();
  this->YLimitsSet->Initialize();

}

//----------------------------------------------------------------------------
vtkPlotDataActor2DExtra::~vtkPlotDataActor2DExtra()
{

  this->PlotLineData->Delete();
  this->PlotLineDataMapper->Delete();
  this->PlotLineDataActor->Delete();

  
  this->PlotPointDataSet->Delete();

  this->PlotPointData->Delete();
  this->PlotPointDataMapper->Delete();
  this->PlotPointDataActor->Delete();

  this->GlyphSource->Delete();

  this->ErrData->Delete();
  this->ErrDataMapper->Delete();
  this->ErrDataActor->Delete();

  this->DataLimits->Delete();
  this->DataLimitsMapper->Delete();
  this->DataLimitsActor->Delete();
  
  this->DataSet->Delete();
  this->UpDataSet->Delete();
  this->DownDataSet->Delete();
  this->LeftDataSet->Delete();
  this->RightDataSet->Delete();

  this->XLimitsSet->Delete();
  this->YLimitsSet->Delete();

#ifndef NOCLIP
  for(int indx=0;indx<4;indx++)
    ClipPlanes[indx]->Delete();
#endif

}

//----------------------------------------------------------------------------
// Build the axis, ticks, title, and labels and render.

int vtkPlotDataActor2DExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  this->BuildData(viewport);
  

  renderedSomething += this->PlotLineDataActor->RenderOpaqueGeometry(viewport);

  renderedSomething += this->PlotPointDataActor->RenderOpaqueGeometry(viewport);

  
  if ( ( this->UpErrBarVisibility) || ( this->DownErrBarVisibility) || ( this->LeftErrBarVisibility) || ( this->RightErrBarVisibility) )
    {
      renderedSomething += this->ErrDataActor->RenderOpaqueGeometry(viewport);
    };

  if ( this->XLimitsVisibility ||  this->YLimitsVisibility )
    {
      renderedSomething += this->DataLimitsActor->RenderOpaqueGeometry(viewport);
    };

  
  return renderedSomething;
}


int vtkPlotDataActor2DExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;


  renderedSomething += this->PlotLineDataActor->RenderOverlay(viewport);
  

  renderedSomething += this->PlotPointDataActor->RenderOverlay(viewport);

  
  if ( ( this->UpErrBarVisibility) || ( this->DownErrBarVisibility) || ( this->LeftErrBarVisibility) || ( this->RightErrBarVisibility) )
    {
      renderedSomething += this->ErrDataActor->RenderOverlay(viewport);
    };

  if ( this->XLimitsVisibility ||  this->YLimitsVisibility )
    {
      renderedSomething += this->DataLimitsActor->RenderOverlay(viewport);
    };


  return renderedSomething;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkPlotDataActor2DExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}


void vtkPlotDataActor2DExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  this->PlotLineDataActor->ReleaseGraphicsResources(win);
  this->PlotPointDataActor->ReleaseGraphicsResources(win);
  this->ErrDataActor->ReleaseGraphicsResources(win);

}


void vtkPlotDataActor2DExtra::BuildData(vtkViewport *viewport)
{

  if (! this->DataSet->GetNumberOfPoints())
    return;

  int  positionsHaveChanged;
  double point[3]; point[2] = 0.0;
  vtkIdType numPts, ptId;
  double tmpPoint[3];
  vtkPoints *pts;



  double *pos1 = this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
  double *pos2 = this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
  

  positionsHaveChanged = 0;
  if (viewport->GetMTime() > this->BuildTime || 
      (viewport->GetVTKWindow() && 
       viewport->GetVTKWindow()->GetMTime() > this->BuildTime))
    {
    // Check to see whether we have to rebuild everything
    // Viewport change may not require rebuild
    double *lastPosition = 
      this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
    double *lastPosition2 =
      this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
    if (fabs(lastPosition[0] - this->LastPosition[0]) > FLT_EPSILON ||
        fabs(lastPosition[1] - this->LastPosition[1]) > FLT_EPSILON ||
        fabs(lastPosition2[0] - this->LastPosition2[0]) > FLT_EPSILON ||
        fabs(lastPosition2[1] - this->LastPosition2[1]) > FLT_EPSILON )
      {
      positionsHaveChanged = 1;
      this->LastPosition[0] = lastPosition[0];
      this->LastPosition[1] = lastPosition[1];
      this->LastPosition2[0] = lastPosition2[0];
      this->LastPosition2[1] = lastPosition2[1];
      }
    }
  
  if (!positionsHaveChanged &&
      this->GetMTime() < this->BuildTime )
    {
      return;
    }

  // Initialize and get important info

  //this->PlotLineData->Initialize();
  //  this->PlotLineDataActor->SetProperty(this->GetProperty());

  double tmpXRange[2];
  double tmpYRange[2];

  switch(this->XScale)
    {
    case 1:
      if (this->XRange[0] > 0 )
	tmpXRange[0]=log10(this->XRange[0]);
      else
	{
	  tmpXRange[0]=log10(this->MinLogXLimit);
	};
      if (this->XRange[1] > 0 )
	tmpXRange[1]=log10(this->XRange[1]);
      else
	{
	  tmpXRange[1]=log10(this->MinLogXLimit);
	};
      break;
    case 0:
    default:
      tmpXRange[0]=this->XRange[0];
      tmpXRange[1]=this->XRange[1];      
      break;
    };


  if ( tmpXRange[0]==tmpXRange[1] )
    {
      if ( tmpXRange[0]==0.0)
	{
	  tmpXRange[1]=1.0;
	}
      else
	{
	  tmpXRange[1]*=1.1;
	};
    };

  
  switch(this->YScale)
    {
    case 1:
      if (this->YRange[0] > 0 )
	tmpYRange[0]=log10(this->YRange[0]);
      else
	{
	  tmpYRange[0]=log10(this->MinLogYLimit);
	};
      if (this->YRange[1] > 0 )
	tmpYRange[1]=log10(this->YRange[1]);
      else
	{
	  tmpYRange[1]=log10(this->MinLogYLimit);
	};
      break;
    case 0:
    default:
      tmpYRange[0]=this->YRange[0];
      tmpYRange[1]=this->YRange[1];      
      break;
    };
  
  if ( tmpYRange[0]==tmpYRange[1] )
    {
      if ( tmpYRange[0]==0.0)
	{
	  tmpYRange[1]=1.0;
	}
      else
	{
	  tmpYRange[1]*=1.1;
	};
    };



  this->PlotLineData->Initialize();

  if ( this->LinesVisibility )
    {
      GenerateLines(this->DataSet,tmpXRange,this->XScale,tmpYRange,this->YScale,this->PlotLineData);
    };

  this->PlotPointDataSet->Initialize();

  if ( this->PointsVisibility)
    {
    if (this->LinesVisibility && (this->DataSet->GetNumberOfPoints() > 1) &&
        !(this->YScale || this->XScale))
      this->PlotPointDataSet->SetPoints (this->PlotLineData->GetPoints ());
    else
      GeneratePoints(this->DataSet,tmpXRange,this->XScale,tmpYRange,this->YScale,this->PlotPointDataSet); 
    };



  numPts = this->PlotLineData->GetNumberOfPoints();
  pts=this->PlotLineData->GetPoints();

  
  for ( ptId=0; ptId < numPts; ptId++ )
    {

      pts->GetPoint(ptId,tmpPoint);

      if ( MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
	{
	  //	  cerr<<"vtkPlotDataActor2DExtra::BuildData: Ignoring invalid Point"<<endl;
	  continue;
	};
      
      if (MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	{
	  //	  cerr<<"vtkPlotDataActor2DExtra::BuildData: Ignoring invalid Point "<<endl;
	  continue;
	};

      pts->SetPoint(ptId,tmpPoint);
    }

  if (this->PlotPointDataSet->GetPoints() != this->PlotLineData->GetPoints())
    {
    numPts = this->PlotPointDataSet->GetNumberOfPoints();
    pts=this->PlotPointDataSet->GetPoints();



    for ( ptId=0; ptId < numPts; ptId++ )
      {

        pts->GetPoint(ptId,tmpPoint);

        if (MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
          continue;
      
        if (MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	  continue;

        pts->SetPoint(ptId,tmpPoint);
      }
    }


  // Handle Error bars

  this->ErrData->Initialize();
  
  vtkDoubleArray* tmpUp=0,*tmpDown=0,*tmpLeft=0,*tmpRight=0;
  double tmpCapLength=0;

  vtkIntArray* tmpYLimit=0,*tmpXLimit=0;
  if ( this->XLimitsSet && (this->XLimitsSet->GetNumberOfTuples () > 0 )  &&
       this->XLimitsVisibility) 
    tmpXLimit = this->XLimitsSet;
  if ( this->YLimitsSet && (this->YLimitsSet->GetNumberOfTuples () > 0 )  &&
       this->YLimitsVisibility )
    tmpYLimit = this->YLimitsSet;

  
  if ( this->UpDataSet && this->UpErrBarVisibility )
    tmpUp= this->UpDataSet;
  
  if ( this->DownDataSet && this->DownErrBarVisibility )
    tmpDown=this->DownDataSet;
  
  if ( this->LeftDataSet && this->LeftErrBarVisibility )
    tmpLeft=this->LeftDataSet;
  
  if ( this->RightDataSet && this->RightErrBarVisibility )
    tmpRight =this->RightDataSet;
  
  if ( this->ErrBarCapVisibility )
    tmpCapLength=this->ErrBarCapLength;
  
  GenerateErrorBars(this->DataSet,tmpUp,tmpDown,tmpLeft,tmpRight,tmpXLimit,tmpYLimit,tmpCapLength,this->DataLimitsOverride,tmpXRange,this->XScale,tmpYRange,this->YScale,pos1,pos2,this->ErrData);

  
  numPts = this->ErrData->GetNumberOfPoints();
  pts=this->ErrData->GetPoints();
  
  for ( ptId=0; ptId < numPts; ptId++ )
    {
      
      pts->GetPoint(ptId,tmpPoint);
      
      
      if (
	  MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
	{
	  continue;
	};
      
      if (
	  MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	{
	  continue;
	};
      
      pts->SetPoint(ptId,tmpPoint);
    }

  tmpCapLength=0;
  if ( this->DataLimitsCapVisibility )
    tmpCapLength=this->DataLimitsCapLength;

  GenerateDataLimits(this->DataSet, tmpXLimit, tmpYLimit, this->DataLimitsBarLength, tmpCapLength, tmpXRange, this->XScale, tmpYRange, this->YScale, pos1,pos2,this->DataLimits);

 
#ifndef NOCLIP
    ClipPlanes[0]->SetOrigin(pos1[0],0,0);
    ClipPlanes[0]->SetNormal(1,0,0);

    ClipPlanes[1]->SetOrigin(pos2[0],0,0);
    ClipPlanes[1]->SetNormal(-1,0,0);

    ClipPlanes[2]->SetOrigin(0,pos1[1],0);
    ClipPlanes[2]->SetNormal(0,1,0);

    ClipPlanes[3]->SetOrigin(0,pos2[1],0);
    ClipPlanes[3]->SetNormal(0,-1,0);
#endif
 
  
    this->BuildTime.Modified();
}

//----------------------------------------------------------------------------
void vtkPlotDataActor2DExtra::ShallowCopy(vtkProp *prop)
{
  /*  vtkPlotDataActor2DExtra *a = vtkPlotDataActor2DExtra::SafeDownCast(prop);
  if ( a != NULL )
    {
      this->SetRange(a->GetRange());
      this->SetNumberOfMajTicks(a->GetNumberOfMajTicks());
      this->SetNumberOfMinTicks(a->GetNumberOfMinTicks());
      this->SetLabelFormat(a->GetLabelFormat());
      this->SetAdjustTicks(a->GetAdjustTicks());
      this->SetMajTicksLength(a->GetMajTicksLength());
      this->SetMinTicksLength(a->GetMinTicksLength());
      this->SetTickOffset(a->GetTickOffset());
      
      this->SetLabelOffset(a->GetLabelOffset());

      this->SetTitlePerpendicularOffset(a->GetTitlePerpendicularOffset());
      this->SetAxisVisibility(a->GetAxisVisibility());
      this->SetMajTicksVisibility(a->GetMajTicksVisibility());
      this->SetMinTicksVisibility(a->GetMinTicksVisibility());
      this->SetLabelVisibility(a->GetLabelVisibility());
      this->SetTitleVisibility(a->GetTitleVisibility());
      this->SetFontFactor(a->GetFontFactor());
      this->SetLabelFactor(a->GetLabelFactor());
      this->SetLabelTextProperty(a->GetLabelTextProperty());
      this->SetTitleTextProperty(a->GetTitleTextProperty());
    }
    */  
  // Now do superclass
  this->vtkActor2D::ShallowCopy(prop);
};


int vtkPlotDataActor2DExtra::MapData(double* inDataRange, 
				     double inPixelRangeMin, 
				     double inPixelRangeMax,
				     double& inOutValue)
{
  if ( ( inDataRange[0] == inDataRange[1]) && (inPixelRangeMin != inPixelRangeMax))
    {
      return -1;
    };
  
  if (inDataRange[0] == inDataRange[1])
    return static_cast<int>(inPixelRangeMin);


  inOutValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(inOutValue-inDataRange[0])/(inDataRange[1]-inDataRange[0]);

  return 1;

};


bool vtkPlotDataActor2DExtra::IsInRange(double inVal, double inRange[2])
{
  double tmpRange[2];


  if (inRange[0] > inRange[1])
    {
      tmpRange[0] = inRange[1];
      tmpRange[1] = inRange[0];
    }
  else
    {
      tmpRange[0] = inRange[0];
      tmpRange[1] = inRange[1];
    };


  if (( inVal <tmpRange[0] ) || (inVal>tmpRange[1]))
    return false;

  return true;

};


void vtkPlotDataActor2DExtra::CalculateDataRanges(vtkPolyData* inData, double* outXRange, double* outYRange)
{




  vtkIdType numPts,ptId;
  double x[3];


  if ((! inData) || (!outXRange) || (!outYRange))
    {
      outXRange[0]=0;
      outXRange[1]=0;
      outYRange[0]=0;
      outYRange[1]=0;
      return;
    };

  numPts = inData->GetNumberOfPoints();

  if (! numPts )
    {
      outXRange[0]=0;
      outXRange[1]=0;
      outYRange[0]=0;
      outYRange[1]=0;
      return;
    };

  
  ptId=0;

  inData->GetPoint(ptId, x);

  outXRange[0]=x[0];
  outXRange[1]=x[0];
  outYRange[0]=x[1];
  outYRange[1]=x[1];

  ptId++;
  for (; ptId < numPts; ptId++ )
    {
      inData->GetPoint(ptId, x);
      if ( x[0]<outXRange[0])
	{
	  outXRange[0]=x[0];
	}
      else if (x[0] > outXRange[1])
	{
	  outXRange[1]=x[0];
	};

      if ( x[1]<outYRange[0])
	{
	  outYRange[0]=x[1];
	}
      else if (x[1] > outYRange[1])
	{
	  outYRange[1]=x[1];
	};
    };
};

void vtkPlotDataActor2DExtra::GenerateErrorBars(vtkPolyData* inData,
						     vtkDoubleArray* inUpDataSet,
						     vtkDoubleArray* inDownDataSet,
						     vtkDoubleArray* inLeftDataSet,
						     vtkDoubleArray* inRightDataSet,
						     vtkIntArray*    inXLimitsSet,
						     vtkIntArray*    inYLimitsSet,
						     double inErrBarCapLength,
						     int    inLimitOverride,
						     double inXRange[2], int inXScale, 
						     double inYRange[2],int inYScale,
						     double *inPos1,
						     double *inPos2,
						     vtkPolyData* outLineData)
{
  if ((!inData) || (!outLineData) )
    return;

  // Return if no errors are set
  if (!inUpDataSet && !inDownDataSet && !inLeftDataSet && !inRightDataSet)
    return;

  vtkPoints *pts;
  vtkPoints *newPoints;
  vtkCellArray *newLines;
  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType totNumPts = 0;
  vtkIdType npts=0,nsegpts=0;
  vtkIdType newPtsId,*oldPtsId;
  vtkIdType ptsId=0;
  double x1[3], x2[3], t,oldx2[3];
  double tmpPoint[3],tmpPoint2[3],tmpPoint3[3];
  double tmpXRange[2],tmpYRange[2];


  oldx2[2]=0;

  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];


  outLineData->Initialize();



  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }
  



  vtkPoints *errPts= vtkPoints::New();
  errPts->SetDataTypeToDouble();

  
  
  vtkCellArray *errLines= vtkCellArray::New();
  vtkIdType errPtsId[2],errBarPtsId[2];
  long tmpSize=0;
  
  int deltap[2];
  
  deltap[0] = inPos2[0]-inPos1[0];
  deltap[1] = inPos2[1]-inPos1[1];
  
  
  double deltaXRange;
  double deltaYRange;
  
  deltaXRange= tmpXRange[1]-tmpXRange[0];
  deltaYRange= tmpYRange[1]-tmpYRange[0];
  
  double verCapLength=0,horCapLength=0;
  bool createErrorBarHorz[] = {false,false};
  bool createErrorBarVert[] = {false,false};
  bool xLimitsExist = false;
  bool yLimitsExist = false;
  int xLimitIndex=0;
  int yLimitIndex=0;
  
  if (( ( deltap[1]) != 0 ) && (inYScale != 1))
    {
      verCapLength = (inErrBarCapLength*(tmpYRange[1]-tmpYRange[0]))/(inPos2[1] -inPos1[1]);
    };
  
  if (( ( deltap[0]) != 0 ) && (inXScale !=1))
    {
      horCapLength = (inErrBarCapLength*(tmpXRange[1]-tmpXRange[0]))/(inPos2[0] -inPos1[0]);
    };
  
  numPts=inData->GetNumberOfPoints();
  pts=inData->GetPoints();
  
  if (inUpDataSet)
    {
      if ( inUpDataSet->GetNumberOfTuples() )
	{
	  tmpSize = numPts;
//	  tmpSize = numVerticalErrs;
	};
    };


  if ( inDownDataSet )
    {
      if ( inDownDataSet->GetNumberOfTuples() )
	{
	  tmpSize += numPts;
//	  tmpSize += numVerticalErrs;
	};
    };

  
  if ( inLeftDataSet )
    {
      if ( inLeftDataSet->GetNumberOfTuples() )
	{
	  tmpSize += numPts;
//	  tmpSize += numHorizontalErrs;
	};
    };
  
  
  if ( inRightDataSet)
    {
      if ( inRightDataSet->GetNumberOfTuples() )
	{
	  tmpSize += numPts; 
//          tmpSize += numHorizontalErrs;
	};
    };
  
  if ( inErrBarCapLength )
    {
      tmpSize += (tmpSize * 2);
    };
  
  
  pts=inData->GetPoints();
  errPts->Allocate(tmpSize+numPts);  
  errLines->Allocate(tmpSize);
  
  tmpPoint2[2]=0;      
  tmpPoint3[2]=0;

  xLimitsExist = (inXLimitsSet && (inXLimitsSet->GetNumberOfTuples () > 0));
  yLimitsExist = (inYLimitsSet && (inYLimitsSet->GetNumberOfTuples () > 0));


  for ( ptsId=0; ptsId < numPts; ptsId++ )
    {

      // Dont create an error bar when there's a limit
      do{
         if ( yLimitsExist && ((abs ((int)(inYLimitsSet->GetValue (yLimitIndex)))-1) == ptsId) &&
              (inUpDataSet || inDownDataSet)){ 
            int limitVal = inYLimitsSet->GetValue (yLimitIndex);
	    // Turn off the error bar creation in the direction of the limit.
            // The opposite direction can be turned on through the override
            // Positive is up and Negative is down
	    if (limitVal > 0){
               createErrorBarVert[0] = false;
	       createErrorBarVert[1] = !((bool)inLimitOverride);
            }
            else if (limitVal < 0){
               createErrorBarVert[1] = false;
               createErrorBarVert[0] = !((bool)inLimitOverride);
            }
	    
            yLimitIndex++;
         }
	 else if (inUpDataSet || inDownDataSet){
            createErrorBarVert[0] = true;
            createErrorBarVert[1] = true;
         }


         if ( xLimitsExist && ((abs ((int)(inXLimitsSet->GetValue (xLimitIndex)))-1) == ptsId) &&
	      (inLeftDataSet || inRightDataSet)){
            int limitVal = inXLimitsSet->GetValue (xLimitIndex);
	    // Turn off the error bar creation in the direction of the limit.
            // The opposite direction can be turned on through the override
            // Positive is right and Negative is left 
            if (limitVal > 0){
               createErrorBarHorz[0] = false;
               createErrorBarHorz[1] = !((bool)inLimitOverride);
            }
            else if (limitVal < 0){
               createErrorBarHorz[1] = false;
               createErrorBarHorz[0] = !((bool)inLimitOverride);
            }

            xLimitIndex++;
         }
	 else if (inLeftDataSet || inRightDataSet){
            createErrorBarHorz[0] = true;
            createErrorBarHorz[1] = true;
         }

         
         // If there is no horizontal error bar or vertical error bar skip
         // getting this point
	 if (!createErrorBarHorz[0] && !createErrorBarHorz[1] &&
             !createErrorBarVert[0] && !createErrorBarVert[1])
	    ptsId++;
      } while (!createErrorBarHorz[0] && !createErrorBarHorz[1] &&
             !createErrorBarVert[0] && !createErrorBarVert[1]);

      // If the ptsId has exceed the total number of points 
      // leave the for loop 
      if (ptsId >= numPts)
         break;

       // Track the total number of points created
       totNumPts++;

       pts->GetPoint(ptsId,tmpPoint);
       errPtsId[0] = errPts->InsertNextPoint(tmpPoint); 
   
       if ( inUpDataSet && createErrorBarVert[0] )
	 {
	   if ( inUpDataSet->GetNumberOfTuples() )
	     {
	       totNumPts++;
	       tmpPoint2[0]=tmpPoint[0];
	       tmpPoint2[1] = tmpPoint[1]+inUpDataSet->GetValue(ptsId);
	       errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	       errLines->InsertNextCell(2,errPtsId);

	       if ( inErrBarCapLength )
		 {
	           totNumPts+=2;
		   tmpPoint3[1]=tmpPoint2[1];
		   if ( inXScale == 1 )
		     {
		       tmpPoint3[0] = tmpPoint2[0]*pow(10.0,-inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		     }
		   else
		     {
		       tmpPoint3[0] = tmpPoint2[0] - horCapLength/2;
		     };
		   errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		   if ( inXScale == 1 )
		     {
		       tmpPoint3[0] = tmpPoint2[0]*pow(10.0,inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		     }
		   else
		     {
		       tmpPoint3[0] =tmpPoint2[0]+horCapLength/2;
		     }
		   errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		   errLines->InsertNextCell(2,errBarPtsId);
		 };	
	      
	     };
      
	 };


       if ( inDownDataSet && createErrorBarVert[1] )
	 {
	   if ( inDownDataSet->GetNumberOfTuples())
	     {
	       totNumPts++;
	       tmpPoint2[0]=tmpPoint[0];
	       tmpPoint2[1] = tmpPoint[1]-inDownDataSet->GetValue(ptsId);
	       errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	       errLines->InsertNextCell(2,errPtsId);
	   
	       if ( inErrBarCapLength )
		 {
	           totNumPts+=2;
		   tmpPoint3[1]=tmpPoint2[1];
		   if ( inXScale == 1 )
		     {
		       tmpPoint3[0] =tmpPoint2[0]*pow(10.0,-inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		     }
		   else
		     {
		       tmpPoint3[0] =tmpPoint2[0] - horCapLength/2;
		     };
		   errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		   if ( inXScale == 1 )
		     {
		       tmpPoint3[0] = tmpPoint2[0]*pow(10.0,inErrBarCapLength*deltaXRange/(2.0*deltap[0]));
		     }
		   else
		     {
		       tmpPoint3[0] = tmpPoint2[0] + horCapLength/2;
		     };
		   errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		   errLines->InsertNextCell(2,errBarPtsId); 
		 };	 
	     };
	 };

      if ( inLeftDataSet && createErrorBarHorz [1])
	{
	  if ( inLeftDataSet->GetNumberOfTuples())
	    {
	      totNumPts++;
	      tmpPoint2[1]=tmpPoint[1];
	      tmpPoint2[0] = tmpPoint[0]-inLeftDataSet->GetValue(ptsId);
	      errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	      errLines->InsertNextCell(2,errPtsId);
	      
	      if ( inErrBarCapLength )
		{
	          totNumPts+=2;
		  tmpPoint3[0]=tmpPoint2[0];
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,-inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] = tmpPoint2[1] - verCapLength/2;
		    };
		  errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] = tmpPoint2[1] + verCapLength/2;
		    };
		  errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		  errLines->InsertNextCell(2,errBarPtsId); 
		  
		};	
	    };
	};
      
      
      if ( inRightDataSet && createErrorBarHorz[0])
	{
	  if ( inRightDataSet->GetNumberOfTuples() )
	    { 
	      totNumPts++;
	      tmpPoint2[1]=tmpPoint[1];
	      tmpPoint2[0] = tmpPoint[0]+inRightDataSet->GetValue(ptsId);
	      errPtsId[1] = errPts->InsertNextPoint(tmpPoint2);
	      errLines->InsertNextCell(2,errPtsId);
	  
	      if ( inErrBarCapLength )
		{
	          totNumPts+=2;
		  tmpPoint3[0]=tmpPoint2[0];
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,-inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] = tmpPoint2[1] - verCapLength/2;
		    };
		  errBarPtsId[0]= errPts->InsertNextPoint(tmpPoint3);
		  if ( inYScale == 1 )
		    {
		      tmpPoint3[1] = tmpPoint2[1]*pow(10.0,inErrBarCapLength*deltaYRange/(2.0*deltap[1]));
		    }
		  else
		    {
		      tmpPoint3[1] =tmpPoint2[1] + verCapLength/2;
		    };
		  errBarPtsId[1]= errPts->InsertNextPoint(tmpPoint3);
		  errLines->InsertNextCell(2,errBarPtsId); 
		};	
	    };
	  
	};

    };  

  
  
  newPoints = vtkPoints::New();
  newPoints->SetDataTypeToDouble();
  newPoints->Allocate(totNumPts);

  newLines = vtkCellArray::New();
  newLines->Allocate(1);

  
  for( errLines->InitTraversal();errLines->GetNextCell(npts,oldPtsId);)
    {
      for (ptsId=0; ptsId<(npts-1); ptsId++)
	{
	  errPts->GetPoint(oldPtsId[ptsId], x1);
	  errPts->GetPoint(oldPtsId[ptsId+1], x2);

	  if ( inXScale== 1)
	    {
	      
	      if ( (x1[0] <= 0.0 ) && (x2[0] <= 0.0 ))
		continue;
	      
	      if ( x1[0] <= 0 )
		{
		  x1[0] = tmpXRange[0];
		}
	      else
		{
		  x1[0]=log10(x1[0]);
		};


	       if ( x2[0] <= 0 )
		{
		  x2[0] = tmpXRange[0];
		}
	      else
		{
		  x2[0]=log10(x2[0]);
		};

	    };




	  if ( inYScale== 1)
	    {

	      if ( (x1[1] <= 0.0 ) && (x2[1] <= 0.0 ))
		continue;
	      
	      if ( x1[1] <= 0 )
		{
		  x1[1] = tmpYRange[0];
		}
	      else
		{
		  x1[1]=log10(x1[1]);
		};


	       if ( x2[1] <= 0 )
		{
		  x2[1] = tmpYRange[0];
		}
	      else
		{
		  x2[1]=log10(x2[1]);
		};

	    };

	  if ( ( ptsId == 0 ) || ( x1[0] != oldx2[0] ) || (x1[1] != oldx2[1]))
	    {
	      nsegpts=2;	      
	      newLines->InsertNextCell(nsegpts);
	      newPtsId = newPoints->InsertNextPoint(x1);
	      newLines->InsertCellPoint(newPtsId);
	      newPtsId = newPoints->InsertNextPoint(x2);
	      newLines->InsertCellPoint(newPtsId);
	    }
	  else if (( x1[0] ==  oldx2[0] ) && (x1[1] == oldx2[1]))
	    {
	      nsegpts++;
	      
	      newPtsId = newPoints->InsertNextPoint(x2);
	      newLines->InsertCellPoint(newPtsId);
	      newLines->UpdateCellCount(nsegpts); 
	    };
	  
	  oldx2[0]=x2[0];
	  oldx2[1]=x2[1];
	  
	};
      
    };
  
  
  //Update the lines
  outLineData->SetPoints(newPoints);
  outLineData->SetVerts(0);
  outLineData->SetLines(newLines);

  newPoints->Delete();
  newLines->Delete();
  errPts->Delete();
  errLines->Delete();  
  

};

void vtkPlotDataActor2DExtra::GenerateDataLimits(vtkPolyData* inData,
                                                     vtkIntArray*    inXLimitsSet,
                                                     vtkIntArray*    inYLimitsSet,
                                                     double inLimitBarLength,
						     int inDataLimitsCapLength,
                                                     double inXRange[2],
						     int inXScale,
                                                     double inYRange[2],
                                                     int inYScale,
                                                     double *inPos1,
                                                     double *inPos2,
                                                     vtkPolyData* outLineData)
{
  if ((!inData) || (!outLineData) )
    return;

  if (!inXLimitsSet && !inYLimitsSet)
    return;

  vtkPoints *pts;
  vtkPoints *newPoints = vtkPoints::New ();
  vtkCellArray *newLines = vtkCellArray::New ();
  vtkIdType numPts=0;
  vtkIdType ptsId=0;
  vtkIdType dataPtsId=0;
  vtkIdType newPtsId=0;
  double pointPosition1[3];
  double pointPosition2[3];
  double pointPosition3[3];
  double tmpXRange[2],tmpYRange[2];
  double val;

  double vertBarLength=0;
  double horzBarLength=0;
  double headAngle = 30;
  double headLengthPercentage = .25;
  double headLength=0;
  double headWidth=0;

  bool positiveLimits = true;

  int deltap[2];

  deltap[0] = inPos2[0]-inPos1[0];
  deltap[1] = inPos2[1]-inPos1[1];

  // inLimitBarLength is a percentage of the plot put it into pixels
  vertBarLength = inLimitBarLength * deltap[1];
  horzBarLength = inLimitBarLength * deltap[0];



  // Check that limits list is smaller than or equal to the 
  // size of the data set and that last value in the limits
  // list is smaller than or equal to the size of the data
  numPts = inData->GetNumberOfPoints ();
  if (inXLimitsSet){
     val = abs ((int)(inXLimitsSet->GetValue (inXLimitsSet->GetNumberOfTuples () - 1)));
     if ((inXLimitsSet->GetNumberOfTuples () > numPts) || (val > numPts))
        return;
  }

  if (inYLimitsSet){
     val = abs ((int)(inYLimitsSet->GetValue (inYLimitsSet->GetNumberOfTuples () - 1)));
     if ((inYLimitsSet->GetNumberOfTuples () > numPts) || (val > numPts))
        return;
  }

  
  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];


  outLineData->Initialize();

  // Make sure ranges run in ascending order
/*  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }

  deltaXRange= tmpXRange[1]-tmpXRange[0];
  deltaYRange= tmpYRange[1]-tmpYRange[0];

  deltap[0] = inPos2[0]-inPos1[0];
  deltap[1] = inPos2[1]-inPos1[1];


  // Convert from pixel length to data positiond
  if (( ( deltap[1]) != 0 ) && (inYScale != 1))
      vertBarLength = (inBarLength*(deltaYRange))/(deltap[1]);
  if (( ( deltap[0]) != 0 ) && (inXScale != 1))
      horzBarLength = (inBarLength*(deltaXRange))/(deltap[0]);

*/
  pts = inData->GetPoints ();
  numPts = 0;

  if (inXLimitsSet)
     numPts += inXLimitsSet->GetNumberOfTuples ();
  if (inYLimitsSet)
     numPts += inYLimitsSet->GetNumberOfTuples ();

  // Currently there are 4 points per limit 
  newPoints->Allocate (numPts*4);
 

  if (inXLimitsSet){
     numPts = inXLimitsSet->GetNumberOfTuples ();
     headLength = horzBarLength*headLengthPercentage;
     headWidth = horzBarLength*headLengthPercentage*tan (vtkMath::RadiansFromDegrees (headAngle));
  }
  else
     numPts = 0;
  
  for ( ptsId=0; ptsId < numPts; ptsId++ )
    {

       dataPtsId = inXLimitsSet->GetValue (ptsId);
       positiveLimits = (dataPtsId > 0);

       // Reverse the direction of the limits if the axes are reversed
       if (tmpXRange[1] < tmpXRange[0])
          positiveLimits = !positiveLimits;

       // Make this permanently positive and subtract 1 so that it's zero based
       dataPtsId = abs ((int)dataPtsId) - 1;

       pts->GetPoint (abs ((int)dataPtsId), pointPosition1);


       // Convert data coordinates to pixel coordinates

       // First handle log scale
       if (inXScale)
 	  pointPosition1[0] = log10 (pointPosition1[0]);         
       if (inYScale)
          pointPosition1[1] = log10 (pointPosition1[1]);
       MapData(inXRange,inPos1[0],inPos2[0],pointPosition1[0]);
       MapData(inYRange,inPos1[1],inPos2[1],pointPosition1[1]);

       if (inDataLimitsCapLength)
         {
           pointPosition2[0] = pointPosition1[0];
           pointPosition2[1] = pointPosition1[1] + inDataLimitsCapLength/2.0;
           newPoints->InsertPoint (newPtsId++, pointPosition2);
           pointPosition2[1] = pointPosition1[1] - inDataLimitsCapLength/2.0;
           newPoints->InsertPoint (newPtsId++, pointPosition2);
         }

	newPoints->InsertPoint (newPtsId++, pointPosition1);

          

       // If the data point is positive create limit pointing to
       // the left
       if (positiveLimits)
         {
	   pointPosition1[0] += horzBarLength;
	   pointPosition2[0] = pointPosition1[0] - headLength;
	   pointPosition3[0] = pointPosition2[0];
	   pointPosition2[1] = pointPosition1[1] + headWidth;
	   pointPosition3[1] = pointPosition1[1] - headWidth;
         }
       else
         {
	   pointPosition1[0] -= horzBarLength;
           pointPosition2[0] = pointPosition1[0] + headLength;
           pointPosition3[0] = pointPosition2[0];
           pointPosition2[1] = pointPosition1[1] + headWidth;
           pointPosition3[1] = pointPosition1[1] - headWidth;
         }

/*       if (positiveLimits)
         {
	   if (inXScale != 1)
             {
               pointPosition1[0] += horzBarLength;
               pointPosition2[0] = pointPosition1[0] - horzBarLength*headLengthPercentage;
               pointPosition3[0] = pointPosition2[0];
	       if (inYScale != 1)
                 {
	           pointPosition2[1] = pointPosition1[1] + horzBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[1] = pointPosition1[1] - horzBarLength*headLengthPercentage*tan (headAngle);
	         }
               else
                 {
	           pointPosition2[1] = pointPosition1[1]*pow (10.0, inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                   pointPosition3[1] = pointPosition1[1]*pow (10.0, -inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                 }
	           
             }

	   else
             {
               pointPosition1[0] = pointPosition1[0]*pow(10.0,inBarLength*deltaXRange/deltap[0]);
	       pointPosition2[0] = pointPosition1[0]*pow(10.0, -inBarLength*headLengthPercentageinBarLength*deltaXRange/deltap[0]);
               pointPosition3[0] = pointPosition2[0];
	       if (inYScale != 1)
                 {
                   pointPosition2[1] = pointPosition1[1] + horzBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[1] = pointPosition1[1] - horzBarLength*headLengthPercentage*tan (headAngle);
                 }
               else
                 {
                   pointPosition2[1] = pointPosition1[1]*pow (10.0, inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                   pointPosition3[1] = pointPosition1[1]*pow (10.0, -inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                 }

             }
         }
       else 
         {
           if (inXScale != 1)
             {
               pointPosition1[0] -= horzBarLength;
               pointPosition2[0] = pointPosition1[0] + horzBarLength*headLengthPercentage;
               pointPosition3[0] = pointPosition2[0];
               if (inYScale != 1)
                 {
                   pointPosition2[1] = pointPosition1[1] + horzBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[1] = pointPosition1[1] - horzBarLength*headLengthPercentage*tan (headAngle);
                 }
               else
                 {
                   pointPosition2[1] = pointPosition1[1]*pow (10.0, inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                   pointPosition3[1] = pointPosition1[1]*pow (10.0, -inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                 }

             }

           else
             {
               pointPosition1[0] = pointPosition1[0]*pow(10.0,-inBarLength*deltaXRange/deltap[0]);
               pointPosition2[0] = pointPosition1[0]*pow(10.0,inBarLength*headLengthPercentageinBarLength*deltaXRange/deltap[0]);
               pointPosition3[0] = pointPosition2[0];
               if (inYScale != 1)
                 {
                   pointPosition2[1] = pointPosition1[1] + horzBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[1] = pointPosition1[1] - horzBarLength*headLengthPercentage*tan (headAngle);
                 }
               else
                 {
                   pointPosition2[1] = pointPosition1[1]*pow (10.0, inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                   pointPosition3[1] = pointPosition1[1]*pow (10.0, -inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                 }

             }
         }
*/

	 newPoints->InsertPoint (newPtsId++, pointPosition1);
         newPoints->InsertPoint (newPtsId++, pointPosition2);
         newPoints->InsertPoint (newPtsId++, pointPosition3);

    }
  
  if (inYLimitsSet){   
     numPts = inYLimitsSet->GetNumberOfTuples ();
     headLength = vertBarLength*headLengthPercentage;
     headWidth = vertBarLength*headLengthPercentage*tan (vtkMath::RadiansFromDegrees (headAngle));
  }
  else
     numPts = 0;
  pts = inData->GetPoints ();
  for ( ptsId=0; ptsId < numPts; ptsId++ )
    {

       dataPtsId = inYLimitsSet->GetValue (ptsId);
       positiveLimits = (dataPtsId > 0);

       // Reverse the direction of the limits if the axes are reversed
       if (tmpYRange[1] < tmpYRange[0])
          positiveLimits = !positiveLimits;

       // Make this permanently positive and subtract 1 so that it's zero based
       dataPtsId = abs ((int)dataPtsId) - 1;

       pts->GetPoint (dataPtsId, pointPosition1);

       // Convert data coordinates to pixel coordinates
       // First handle log scale
       if (inXScale)
          pointPosition1[0] = log10 (pointPosition1[0]);
       if (inYScale)
          pointPosition1[1] = log10 (pointPosition1[1]);
       MapData(inXRange,inPos1[0],inPos2[0],pointPosition1[0]);
       MapData(inYRange,inPos1[1],inPos2[1],pointPosition1[1]);

       if (inDataLimitsCapLength)
         {
           pointPosition2[1] = pointPosition1[1];
           pointPosition2[0] = pointPosition1[0] + inDataLimitsCapLength/2.0;
           newPoints->InsertPoint (newPtsId++, pointPosition2);
           pointPosition2[0] = pointPosition1[0] - inDataLimitsCapLength/2.0;
           newPoints->InsertPoint (newPtsId++, pointPosition2);
         }

       newPoints->InsertPoint (newPtsId++, pointPosition1);


       // If the data point is positive create limit pointing to
       // the up
       if (positiveLimits)
         {
	   pointPosition1[1] += vertBarLength;
           pointPosition2[1] = pointPosition1[1] - headLength;
           pointPosition3[1] = pointPosition2[1];
           pointPosition2[0] = pointPosition1[0] + headWidth;
           pointPosition3[0] = pointPosition1[0] - headWidth;
         }
       else
         {
           pointPosition1[1] -= vertBarLength;
           pointPosition2[1] = pointPosition1[1] + headLength;
           pointPosition3[1] = pointPosition2[1];
           pointPosition2[0] = pointPosition1[0] + headWidth;
           pointPosition3[0] = pointPosition1[0] - headWidth;
         }

/*       if (positiveLimits)
         {
           if (inYScale != 1)
             {
               pointPosition1[1] += vertBarLength;
               pointPosition2[1] = pointPosition1[1] - vertBarLength*headLengthPercentage;
               pointPosition3[1] = pointPosition2[1];
               if (inXScale != 1)
                 {
                   pointPosition2[0] = pointPosition1[0] + vertBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[0] = pointPosition1[0] - vertBarLength*headLengthPercentage*tan (headAngle);
                 }
               else
                 {
                   pointPosition2[0] = pointPosition1[0]*pow (10.0, inBarLength*deltaYRange/deltap[1]*headLengthPercentage*tan (headAngle));
                   pointPosition3[0] = pointPosition1[0]*pow (10.0, -inBarLength*deltaYRange/deltap[1]*headLengthPercentage*tan (headAngle));
                 }

             }

           else
             {
               pointPosition1[1] = pointPosition1[1]*pow(10.0,inBarLength*deltaYRange/deltap[1]);
               pointPosition2[1] = pointPosition1[1]*pow(10.0, -inBarLength*headLengthPercentageinBarLength*deltaYRange/deltap[1]);
               pointPosition3[1] = pointPosition2[1];
               if (inYScale != 1)
                 {
                   pointPosition2[0] = pointPosition1[0] + vertBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[0] = pointPosition1[0] - vertBarLength*headLengthPercentage*tan (headAngle);
                 }
               else
                 {
                   pointPosition2[0] = pointPosition1[0]*pow (10.0, inBarLength*deltaYRange/deltap[1]*headLengthPercentage*tan (headAngle));
                   pointPosition3[0] = pointPosition1[0]*pow (10.0, -inBarLength*deltaYRange/deltap[1]*headLengthPercentage*tan (headAngle));
                 }
             }
         }
         else
         {
           if (inXScale != 1)
             {
               pointPosition1[1] -= vertBarLength;
               pointPosition2[1] = pointPosition1[1] + vertBarLength*headLengthPercentage;
               pointPosition3[1] = pointPosition2[1];
               if (inYScale != 1)
                 {
                   pointPosition2[0] = pointPosition1[0] + vertBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[0] = pointPosition1[0] - vertBarLength*headLengthPercentage*tan (headAngle);
                 }
               else
                 {
                   pointPosition2[0] = pointPosition1[0]*pow (10.0, inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                   pointPosition3[0] = pointPosition1[0]*pow (10.0, -inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                 }

             }

           else
             {
               pointPosition1[1] = pointPosition1[1]*pow(10.0,-inBarLength*deltaXRange/deltap[0]);
               pointPosition2[1] = pointPosition1[1]*pow(10.0,inBarLength*headLengthPercentageinBarLength*deltaXRange/deltap[0]);
               pointPosition3[1] = pointPosition2[1];
               if (inYScale != 1)
                 {
                   pointPosition2[0] = pointPosition1[0] + vertBarLength*headLengthPercentage*tan (headAngle);
                   pointPosition3[0] = pointPosition1[0] - vertBarLength*headLengthPercentage*tan (headAngle);
                 }
               else
                 {
                   pointPosition2[0] = pointPosition1[0]*pow (10.0, inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                   pointPosition3[0] = pointPosition1[0]*pow (10.0, -inBarLength*deltaXRange/deltap[0]*headLengthPercentage*tan (headAngle));
                 }
             }
         }
*/

         newPoints->InsertPoint (newPtsId++, pointPosition1);
         newPoints->InsertPoint (newPtsId++, pointPosition2);
         newPoints->InsertPoint (newPtsId++, pointPosition3);

    }

  // Setup cells
  vtkIdType numCells=0;
  vtkIdType cellId;

  // Each limits consits of two cells. Ones the bar and the other is the cap. 
  // Current the bar has two point and one line segment (one cell) and the
  // Arrow head has 3 points and 2 line segments (2nd cell). If there's a
  // cap than the second cell becomes the cap and the arrow head becomes the
  // third cell.
  if (inXLimitsSet)
     numCells += inXLimitsSet->GetNumberOfTuples ();
  if (inYLimitsSet)
     numCells += inYLimitsSet->GetNumberOfTuples ();
  newLines->Allocate (numCells*(2+inDataLimitsCapLength));
  ptsId = 0;
  for (cellId=0; cellId < numCells; cellId++)
    {
       // Insert caps if they exist
       if (inDataLimitsCapLength)
         {
           newLines->InsertNextCell (2);
           newLines->InsertCellPoint (ptsId++);
           newLines->InsertCellPoint (ptsId++);
         }

       // Insert the bar
       newLines->InsertNextCell (2);
       newLines->InsertCellPoint (ptsId++);  //point 0
       newLines->InsertCellPoint (ptsId++); //point 1

       // Insert the arrow head
       newLines->InsertNextCell (3);
       newLines->InsertCellPoint (ptsId++);  //point 2
       newLines->InsertCellPoint (ptsId-2); //point 1
       newLines->InsertCellPoint (ptsId++); //point 3
    }
       
       

  //Update the lines
  outLineData->SetPoints(newPoints);
  outLineData->SetVerts(0);
  outLineData->SetLines(newLines);

  newPoints->Delete();
  newLines->Delete();

}




void vtkPlotDataActor2DExtra::GeneratePoints(vtkPolyData* inData,double inXRange[2],int inXScale,double inYRange[2], int inYScale,vtkPolyData* outPointData)
{
  if ((!inData) || (!outPointData))
    return;

  vtkPoints *points=inData->GetPoints();
  vtkPoints *newPoints;
  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType ptsId=0;
  double x1[3],t;


  double tmpXRange[2],tmpYRange[2];
  
  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];

  outPointData->Initialize();

  // make sure the limits are in order


  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }
  


  newPoints = vtkPoints::New();
  newPoints->SetDataTypeToDouble();
  newPoints->Allocate(numPts);



  for (ptsId=0; ptsId<numPts; ptsId++)
    {
      points->GetPoint(ptsId, x1);

      if ( inXScale==1)
      {
	if ( x1[0] <= 0.0 )
	  continue;
	else
	  x1[0]=log10(x1[0]);
      }
      
      if ( inYScale==1)
      {
	if ( x1[1] <=0.0)
	  continue;
	else
	  x1[1]=log10(x1[1]);
      }

      //Check if the point is in the range
      
      if (( x1[0] >= tmpXRange[0] ) && (x1[0]<=tmpXRange[1]) && ( x1[1] >= tmpYRange[0] ) && (x1[1]<=tmpYRange[1]))
	{
	  newPoints->InsertNextPoint(x1);
	};
    };
  
	  
  outPointData->SetPoints(newPoints);
  outPointData->SetVerts(0);
  outPointData->SetLines(0);


  newPoints->Delete();

};



void vtkPlotDataActor2DExtra::GenerateLines(vtkPolyData* inData,double inXRange[2], int inXScale, double inYRange[2],int inYScale,vtkPolyData* outLineData)
{

  if ((!inData) || (!outLineData) )
    return;

  vtkPoints *points=inData->GetPoints();
  vtkPoints *newPoints;
  vtkCellArray *newLines;
  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType nsegpts=0;
  vtkIdType newPtsId;
  vtkIdType ptsId=0;
  double x1[3], x2[3], oldx2[3],t;



  double tmpXRange[2],tmpYRange[2];


  oldx2[2]=0;

  tmpXRange[0]=inXRange[0];
  tmpXRange[1]=inXRange[1];
  tmpYRange[0]=inYRange[0];
  tmpYRange[1]=inYRange[1];

  outLineData->Initialize();


  // Use cohen-sutherland line clipping algorithm


  if ( tmpXRange[1] < tmpXRange[0])
    {
      t = tmpXRange[1];
      tmpXRange[1] = tmpXRange[0];
      tmpXRange[0]=t;
    }


  if ( tmpYRange[1] < tmpYRange[0])
    {
      t = tmpYRange[1];
      tmpYRange[1] = tmpYRange[0];
      tmpYRange[0]=t;
    }
  




  newPoints = vtkPoints::New();
  newPoints->SetDataTypeToDouble();
  newPoints->Allocate(numPts);

  newLines = vtkCellArray::New();
  newLines->Allocate(1);
  
  
  for (ptsId=0; ptsId<(numPts-1); ptsId++)
    {
      points->GetPoint(ptsId, x1);
      points->GetPoint(ptsId+1, x2);

      if ( inXScale == 1 )
	{
	  if (( x1[0] <=0.0 ) || (x2[0]<=0.0))
	    continue;
	  else
	    {
	      x1[0]=log10(x1[0]);
	      x2[0]=log10(x2[0]);
	    };
	};

      if ( inYScale == 1)
	{
	  if (( x1[1] <=0.0 ) || (x2[1]<=0.0))
	    continue; 
	  else
	    {
	      x1[1]=log10(x1[1]);
	      x2[1]=log10(x2[1]);
	    };
	};

      if ( ( ptsId == 0 ) || ( x1[0] != oldx2[0] ) || (x1[1] != oldx2[1]))
	{
	  nsegpts=2;	      
	  newLines->InsertNextCell(nsegpts);
	  newPtsId = newPoints->InsertNextPoint(x1);
	  newLines->InsertCellPoint(newPtsId);
	  newPtsId = newPoints->InsertNextPoint(x2);
	  newLines->InsertCellPoint(newPtsId);
	}
      else if (( x1[0] ==  oldx2[0] ) && (x1[1] == oldx2[1]))
	{
	  nsegpts++;
	  
	  newPtsId = newPoints->InsertNextPoint(x2);
	  newLines->InsertCellPoint(newPtsId);
	  newLines->UpdateCellCount(nsegpts); 
	};
      
       oldx2[0]=x2[0];
       oldx2[1]=x2[1];


    };


  //Update the lines
  outLineData->SetPoints(newPoints);
  outLineData->SetVerts(0);
  outLineData->SetLines(newLines);

  
  newPoints->Delete();
  newLines->Delete();


};




void vtkPlotDataActor2DExtra::SetData(vtkPolyData *inData, 
				      vtkDoubleArray* inUpData, 
				      vtkDoubleArray* inDownData, 
				      vtkDoubleArray* inLeftData,
				      vtkDoubleArray* inRightData,
				      vtkDoubleArray* inXLimitsSet,
                                      vtkDoubleArray* inYLimitsSet)
{


  vtkIdType ptId,id;
  double tmpPoint[3];
  vtkPoints *pts;
  int pointCount=0;
  double tmpVal;

  // initialize curve data
  
  tmpPoint[2]=0;

  this->DataSet->Initialize();


  // put data into local vtk structures

  if (inData)
    {
      pts = vtkPoints::New();
      pts->SetDataTypeToDouble();
      pts->Allocate(10,10);
      pointCount=inData->GetNumberOfPoints();
      for ( ptId=0;ptId<pointCount;ptId++)
      {
	inData->GetPoint(ptId, tmpPoint);
	id = pts->InsertNextPoint(tmpPoint);
      }

      this->DataSet->SetPoints(pts);
      this->DataSet->SetVerts(0);
      this->DataSet->SetLines(0);
      
      pts->Delete();

    };





  this->DataSet->ComputeBounds();
  double* bounds = this->DataSet->GetBounds();
  this->XDataRange[0]=bounds[0];
  this->XDataRange[1]=bounds[1];
  this->YDataRange[0]=bounds[2];
  this->YDataRange[1]=bounds[3];


  this->UpDataSet->Initialize();
  this->DownDataSet->Initialize();
  this->LeftDataSet->Initialize();
  this->RightDataSet->Initialize();
  this->XLimitsSet->Initialize();
  this->YLimitsSet->Initialize();


  for(vtkIdType indx=0;indx<pointCount;indx++)
    {

      this->DataSet->GetPoint(indx, tmpPoint);

      // Create an array of upper and lower limits for the y direction
      // The data array is actually a list of point indices indicating
      // the point that limit should be drawn. A positive or negative
      // index is used to determine whether its an upper or lower limit.
      // The index is 1 based instead of zero based so that the zeroth
      // position in data array can carry a positive or negative value
      if (inYLimitsSet)
        {
          if ( indx < inYLimitsSet->GetNumberOfTuples())
            {
	      tmpVal=inYLimitsSet->GetValue(indx);
              if (tmpVal > 0)
                 this->YLimitsSet->InsertNextValue((indx+1));
              else if (tmpVal < 0)
		 this->YLimitsSet->InsertNextValue(-(indx+1));
/*              tmpVal=tmpPoint[1]+DataLimitsBarLength;

              if ( tmpVal > this->YDataRange[1] )
                {
                  this->YDataRange[1] = tmpVal;
                }
*/

            }
        }


      if ( inUpData)
	{
	  if ( indx < inUpData->GetNumberOfTuples())
	    {
	      tmpVal=inUpData->GetValue(indx);
	      this->UpDataSet->InsertNextValue(tmpVal);
	      tmpVal+=tmpPoint[1];
		
	      if ( tmpVal > this->YDataRange[1] )
		{
		  this->YDataRange[1] = tmpVal;
		}
	    }
	  else
	    {
	      this->UpDataSet->InsertNextValue(0.0);
	    };
	};


      if ( inDownData)
	{

	  if ( indx < inDownData->GetNumberOfTuples())
	    {
	      tmpVal=inDownData->GetValue(indx);
	      this->DownDataSet->InsertNextValue(tmpVal);
	      tmpVal=tmpPoint[1]-tmpVal;
	      
	      if ( tmpVal < this->YDataRange[0] )
		{
		  this->YDataRange[0] = tmpVal;
		}
	    }
	  else
	    {
	      this->DownDataSet->InsertNextValue(0.0);
	    };
	};
      

      // Create an array of upper and lower limits for the y direction
      // The data array is actually a list of point indices indicating
      // the point that limit should be drawn. A positive or negative
      // index is used to determine whether its an upper or lower limit.
      // The index is 1 based instead of zero based so that the zeroth
      // position in data array can carry a positive or negative value
      if (inXLimitsSet)
        {
          if ( indx < inXLimitsSet->GetNumberOfTuples())
            {
              tmpVal=inXLimitsSet->GetValue(indx);
              if (tmpVal > 0)
                 this->XLimitsSet->InsertNextValue(indx+1);
              else if (tmpVal < 0)
                 this->XLimitsSet->InsertNextValue(-(indx+1));
/*              tmpVal=tmpPoint[1]+DataLimitsBarLength;

              if ( tmpVal > this->XDataRange[1] )
                {
                  this->XDataRange[1] = tmpVal;
                }
*/
            }
        }
      
      if ( inLeftData)
	{
	  if ( indx < inLeftData->GetNumberOfTuples())
	    {
	      tmpVal=inLeftData->GetValue(indx);
	      this->LeftDataSet->InsertNextValue(tmpVal);
	      tmpVal=tmpPoint[0]-tmpVal;
	      
	      if ( tmpVal < this->XDataRange[0] )
		{
		  this->XDataRange[0] = tmpVal;
		}
	    }
	  else
	    {
	      this->LeftDataSet->InsertNextValue(0.0);
	    };
	};
	  
      
      
      if ( inRightData)
	{
	  if ( indx < inRightData->GetNumberOfTuples())
	    {
	      tmpVal=inRightData->GetValue(indx);
	      this->RightDataSet->InsertNextValue(tmpVal);
	      tmpVal+=tmpPoint[0];
	      
	      if ( tmpVal > this->XDataRange[1] )
		{
		  this->XDataRange[1] = tmpVal;
		}
	    }
	  else
	    {
	      this->RightDataSet->InsertNextValue(0.0);
	    };
	};
    }
  
  
  
  
  this->Modified();
  
};

void vtkPlotDataActor2DExtra::GetClipPlanes (int &inOutXMin, int &inOutXMax, int &inOutYMin, int &inOutYMax)
{
#ifndef NOCLIP
  inOutXMin = this->ClipPlanes[0]->GetOrigin()[0];
  inOutXMax = this->ClipPlanes[1]->GetOrigin()[0];
  inOutYMin = this->ClipPlanes[2]->GetOrigin()[1];
  inOutYMax = this->ClipPlanes[3]->GetOrigin()[1];
#endif

}

void vtkPlotDataActor2DExtra::GetBounds (vtkViewport *viewport, double bounds[4])
{
  this->BuildData (viewport);

  int numPlotLineDataPnts = this->PlotLineData->GetNumberOfPoints ();
  int numPlotPointDataSetPnts = this->PlotPointDataSet->GetNumberOfPoints ();
  int numErrDataPnts = this->ErrData->GetNumberOfPoints ();
  int numDataLimitsPnts = this->DataLimits->GetNumberOfPoints ();

  double *ptsBounds;

  if (!this->LinesVisibility)
    numPlotLineDataPnts = 0;
  if (!this->PointsVisibility)
    numPlotPointDataSetPnts = 0;
  if (!this->UpErrBarVisibility && !this->DownErrBarVisibility &&
      !this->LeftErrBarVisibility && !this->RightErrBarVisibility)
    numErrDataPnts = 0;
  if (!this->XLimitsVisibility && !this->YLimitsVisibility)
    numDataLimitsPnts = 0;


  if ((numPlotLineDataPnts == 0) && (numPlotPointDataSetPnts == 0))
    {
    bounds[0] = 0;
    bounds[1] = 0;
    bounds[2] = 0;
    bounds[3] = 0;
    }
  else
    {
    if (numPlotLineDataPnts > 0)
      {
      ptsBounds = this->PlotLineData->GetPoints ()->GetBounds ();

      bounds[0] = ptsBounds[0];
      bounds[1] = ptsBounds[1];
      bounds[2] = ptsBounds[2];
      bounds[3] = ptsBounds[3];
      }
    if (numPlotPointDataSetPnts > 0)
      {
      int glyphSize = this->GlyphSource->GetScale();
      if (this->PlotLineData->GetPoints () ==
          this->PlotPointDataSet->GetPoints ())
        {
        bounds[0] -= glyphSize;
        bounds[1] += glyphSize;
        bounds[2] -= glyphSize;
        bounds[3] += glyphSize;

        }
      else
        {
        ptsBounds = this->PlotPointDataSet->GetPoints ()->GetBounds ();

        bounds[0] = ptsBounds[0] - glyphSize;
        bounds[1] = ptsBounds[1] + glyphSize;
        bounds[2] = ptsBounds[2] - glyphSize;
        bounds[3] = ptsBounds[3] + glyphSize;

        }
      }
    if (numErrDataPnts > 0)
      {
      ptsBounds = this->ErrData->GetPoints ()->GetBounds ();

      if (bounds[0] > ptsBounds[0])
        bounds[0] = ptsBounds[0];
      if (bounds[1] < ptsBounds[1])
        bounds[1] = ptsBounds[1];
      if (bounds[2] > ptsBounds[2])
        bounds[2] = ptsBounds[2];
      if (bounds[3] < ptsBounds[3])
        bounds[3] = ptsBounds[3];
      }
    if (numDataLimitsPnts > 0)
      {
      ptsBounds = this->DataLimits->GetPoints ()->GetBounds ();

      if (bounds[0] > ptsBounds[0])
        bounds[0] = ptsBounds[0];
      if (bounds[1] < ptsBounds[1])
        bounds[1] = ptsBounds[1];
      if (bounds[2] > ptsBounds[2])
        bounds[2] = ptsBounds[2];
      if (bounds[3] < ptsBounds[3])
        bounds[3] = ptsBounds[3];
      }
    }
}

