// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <math.h>
#include <float.h>
#include <bitset>
#include <list>

#include "vtkIntArray.h"
#include "vtkCellArray.h"
#include "vtkObjectFactory.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkViewport.h"
#include "vtkDoubleArray.h"
#include "vtkProperty2D.h"
#include "vtkPolyLineActor2DExtra.hh"
#include "vtkPlane.h"
#include "vtkWindow.h"
#include "vtkCoordinate.h"
#include "vtkTransform.h"
#include "vtkTransformPolyDataFilter.h"


#ifndef MINLOG
#define MINLOG 0.01
#endif



vtkCxxRevisionMacro(vtkPolyLineActor2DExtra, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkPolyLineActor2DExtra);



//----------------------------------------------------------------------------
// Instantiate this object.
vtkPolyLineActor2DExtra::vtkPolyLineActor2DExtra()
{
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(0.0, 0.0);

  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(1, 1);
  this->Position2Coordinate->SetReferenceCoordinate(NULL);

  this->Location[0] = vtkCoordinate::New ();
  this->Location[1] = vtkCoordinate::New ();

  this->Location[0]->SetCoordinateSystemToNormalizedViewport();
  this->Location[0]->SetValue(0.0, 0.0);

  this->Location[1]->SetCoordinateSystemToNormalizedViewport();
  this->Location[1]->SetValue(1, 1);

  
  this->MinLogXLimit=MINLOG;
  this->MinLogYLimit=MINLOG;

  this->XScale =0;
  this->YScale =0;

  this->XRange[0] = 0.0;
  this->XRange[1] = 1.0;

  this->YRange[0] = 0.0;
  this->YRange[1] = 1.0;

  this->XClippedRange[0] = 0.0;
  this->XClippedRange[1] = 1.0;

  this->YClippedRange[0] = 0.0;
  this->YClippedRange[1] = 1.0;


  this->ClipData = true;


  for(int indx=0;indx<4;indx++)
    this->ClipPlanes[indx]=vtkPlane::New();
  


  this->ClipPlanes[0]->SetOrigin(0,0,0);
  this->ClipPlanes[0]->SetNormal(1,0,0);

  this->ClipPlanes[1]->SetOrigin(1,0,0);
  this->ClipPlanes[1]->SetNormal(-1,0,0);

  this->ClipPlanes[2]->SetOrigin(0,0,0);
  this->ClipPlanes[2]->SetNormal(0,1,0);

  this->ClipPlanes[3]->SetOrigin(0,1,0);
  this->ClipPlanes[3]->SetNormal(0,-1,0);
  


  this->Data = vtkPolyData::New();
  this->PolyLine = vtkPolyData::New();

  this->PolyLineTransform = vtkTransform::New();
  this->PolyLineTransform->Identity ();
  this->PolyLineTransformFilter = vtkTransformPolyDataFilter::New();
  this->PolyLineTransformFilter->SetTransform(this->PolyLineTransform);
  this->PolyLineTransformFilter->SetInput(this->PolyLine);


  this->PolyLineMapper = vtkPolyDataMapper2D::New();

// The following will not work because the clipped data screws
// up the gl2ps output. Must use clip filter instead
  this->PolyLineMapper->SetInputConnection(PolyLineTransformFilter->GetOutputPort());
  for (int ii=0; ii<4; ii++)
    this->PolyLineMapper->AddClippingPlane (this->ClipPlanes[ii]);


  this->SetMapper(this->PolyLineMapper);


  this->LastPosition[0] = this->LastPosition[1] = 0;
  this->LastPosition2[0] = this->LastPosition2[1] = 0;

}

//----------------------------------------------------------------------------
vtkPolyLineActor2DExtra::~vtkPolyLineActor2DExtra()
{

  this->Data->Delete();
  this->PolyLine->Delete();
  this->PolyLineMapper->Delete();
/*  this->PolyLineActor->Delete();
*/

  this->PolyLineTransformFilter->Delete ();
  this->PolyLineTransform->Delete ();


  
  for(int indx=0;indx<4;indx++)
    this->ClipPlanes[indx]->Delete();

  this->Location[0]->Delete ();
  this->Location[1]->Delete ();

};

//----------------------------------------------------------------------------
// Build the axis, ticks, title, and labels and render.

int vtkPolyLineActor2DExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  this->BuildData(viewport);

  
/*  renderedSomething += this->PolyLineActor->RenderOpaqueGeometry(viewport);
*/
  renderedSomething += vtkActor2D::RenderOpaqueGeometry(viewport);

  return renderedSomething;
}


int vtkPolyLineActor2DExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  renderedSomething += vtkActor2D::RenderOverlay(viewport);

  return renderedSomething;
}

/*
void vtkPolyLineActor2DExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  this->PolyLineActor->ReleaseGraphicsResources(win);

}
*/


void vtkPolyLineActor2DExtra::BuildData(vtkViewport *viewport)
{

  if (! this->Data->GetNumberOfPoints())
    return;

  int positionsHaveChanged;
  vtkIdType numPts, ptId;
  double tmpPoint[3];
  vtkPoints *pts;

/*  double *pos1 = this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
  double *pos2 = this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
*/
  double *pos1 = this->Location[0]->GetComputedDoubleViewportValue(viewport);
  double *pos2 = this->Location[1]->GetComputedDoubleViewportValue(viewport);


  if (( fabs(pos2[0]-pos1[0])<=1)
      || (fabs(pos2[1]-pos1[1])<=1))
    return;

  positionsHaveChanged = 0;
  if ((viewport->GetMTime() > this->BuildTime) || 
      (viewport->GetVTKWindow() && 
       viewport->GetVTKWindow()->GetMTime() > this->BuildTime)) 
    {
    // Check to see whether we have to rebuild everything
    // Viewport change may not require rebuild
/*    double *lastPosition = 
      this->PositionCoordinate->GetComputedDoubleViewportValue(viewport);
    double *lastPosition2 =
      this->Position2Coordinate->GetComputedDoubleViewportValue(viewport);
*/

    double *lastPosition = pos1;
    double *lastPosition2 = pos2;
    if ((fabs(lastPosition[0] - this->LastPosition[0]) < FLT_EPSILON) ||
        (fabs(lastPosition[1] - this->LastPosition[1]) < FLT_EPSILON) ||
        (fabs(lastPosition2[0] - this->LastPosition2[0]) < FLT_EPSILON) ||
        (fabs(lastPosition2[1] - this->LastPosition2[1]) < FLT_EPSILON) )
      {
      positionsHaveChanged = 1;
      }
    }
  
  if (!positionsHaveChanged &&
      this->GetMTime() < this->BuildTime )
    {
      return;
    }

 
  // Initialize and get important info

  double tmpXRange[2];
  double tmpYRange[2];

  switch(this->XScale)
    {
    case 1:
      if (this->XRange[0] > 0 )
	tmpXRange[0]=log10(this->XRange[0]);
      else
	{
	  tmpXRange[0]=log10(this->MinLogXLimit);
	};
      if (this->XRange[1] > 0 )
	tmpXRange[1]=log10(this->XRange[1]);
      else
	{
	  tmpXRange[1]=log10(this->MinLogXLimit);
	};
      break;
    case 0:
    default:
      tmpXRange[0]=this->XRange[0];
      tmpXRange[1]=this->XRange[1];      
      break;
    };


  if ( tmpXRange[0]==tmpXRange[1] )
    {
      if ( tmpXRange[0]==0.0)
	{
	  tmpXRange[1]=1.0;
	}
      else
	{
	  tmpXRange[1]*=1.1;
	};
    };

  
  switch(this->YScale)
    {
    case 1:
      if (this->YRange[0] > 0 )
	tmpYRange[0]=log10(this->YRange[0]);
      else
	{
	  tmpYRange[0]=log10(this->MinLogYLimit);
	};
      if (this->YRange[1] > 0 )
	tmpYRange[1]=log10(this->YRange[1]);
      else
	{
	  tmpYRange[1]=log10(this->MinLogYLimit);
	};
      break;
    case 0:
    default:
      tmpYRange[0]=this->YRange[0];
      tmpYRange[1]=this->YRange[1];      
      break;
    };
  
  if ( tmpYRange[0]==tmpYRange[1] )
    {
      if ( tmpYRange[0]==0.0)
	{
	  tmpYRange[1]=1.0;
	}
      else
	{
	  tmpYRange[1]*=1.1;
	};
    };



  if ( this->Visibility )
    {
      
      GeneratePolyLine (this->Data,this->XScale,this->YScale,this->PolyLine);

    };

  
  if ( pos1[0] < 0 )
    pos1[0]=0;
  if ( pos1[1] < 0 )
    pos1[1] = 0;
  if ( pos2[0] < 0 )
    pos2[0]=0;
  if ( pos2[1] < 0 )
    pos2[1] = 0;




  numPts = this->PolyLine->GetNumberOfPoints();
  pts = this->PolyLine->GetPoints ();

  for ( ptId=0; ptId < numPts; ptId++ )
    {

      pts->GetPoint(ptId,tmpPoint);

      if (
		  MapData(tmpXRange,pos1[0],pos2[0],tmpPoint[0])<0)
		{
		  //	  cerr<<"vtkPolyLineActor2DExtra::BuildData: Ignoring invalid Point"<<endl;
		  continue;
		};
	      
	      if (
		  MapData(tmpYRange,pos1[1],pos2[1],tmpPoint[1])<0)
	{
	  //	  cerr<<"vtkPolyLineActor2DExtra::BuildData: Ignoring invalid Point "<<endl;
	  continue;
	};

      pts->SetPoint(ptId,tmpPoint);
    }

  this->PolyLine->ComputeBounds();
  double* bounds = this->PolyLine->GetBounds();
  this->XClippedRange[0] = this->XDataRange[0]=bounds[0];
  this->XClippedRange[1] = this->XDataRange[1]=bounds[1];
  this->YClippedRange[0] = this->YDataRange[0]=bounds[2];
  this->YClippedRange[1] = this->YDataRange[1]=bounds[3];


  this->ClipPlanes[0]->SetOrigin(pos1[0],0,0);
  this->ClipPlanes[1]->SetOrigin(pos2[0],0,0);
  this->ClipPlanes[2]->SetOrigin(0,pos1[1],0);
  this->ClipPlanes[3]->SetOrigin(0,pos2[1],0);

  if (this->ClipData)
    {
    if (this->XClippedRange[0] < pos1[0])
      this->XClippedRange[0] = pos1[0];
    if (this->YClippedRange[0] < pos1[1])
      this->YClippedRange[0] = pos1[1];

    if (this->XClippedRange[1] < pos1[0])
      this->XClippedRange[1] = pos1[0];
    if (this->YClippedRange[1] < pos1[1])
      this->YClippedRange[1] = pos1[1];


    if (this->XClippedRange[1] > pos2[0])
      this->XClippedRange[1] = pos2[0];
    if (this->YClippedRange[1] > pos2[1])
      this->YClippedRange[1] = pos2[1];

    if (this->XClippedRange[0] > pos2[0])
      this->XClippedRange[0] = pos2[0];
    if (this->YClippedRange[0] > pos2[1])
      this->YClippedRange[0] = pos2[1];

    }


  this->BuildTime.Modified();

  
}

//----------------------------------------------------------------------------
void vtkPolyLineActor2DExtra::ShallowCopy(vtkProp *prop)
{
  // Now do superclass
  this->vtkActor2D::ShallowCopy(prop);
};

int vtkPolyLineActor2DExtra::MapData(double* inDataRange, 
				  double inPixelRangeMin, 
				  double inPixelRangeMax,
				  double& inOutValue)
{

  if ( (inPixelRangeMin <0)
      ||
      (inPixelRangeMax <0)
      ||
      ( ( inDataRange[0] == inDataRange[1]) && (inPixelRangeMin != inPixelRangeMax))
      )
    {
      return -1;
    };
  
  if (inDataRange[0] == inDataRange[1])
    return static_cast<int>(inPixelRangeMin);


  inOutValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(inOutValue-inDataRange[0])/(inDataRange[1]-inDataRange[0]);


  return 1;

};

void vtkPolyLineActor2DExtra::SetLocation(double inMinX, double inMinY,double inMaxX,double inMaxY)
{
/*  this->PositionCoordinate->SetValue(inMinX,inMinY);
  this->Position2Coordinate->SetValue(inMaxX,inMaxY);
*/
  this->Location[0]->SetValue(inMinX,inMinY);
  this->Location[1]->SetValue(inMaxX,inMaxY);

  this->Modified();
};


void vtkPolyLineActor2DExtra::GeneratePolyLine(vtkPolyData* inData,int inXScale,int inYScale, vtkPolyData* outData)
{

  if (!inData || !outData) 
    return;

  vtkPoints *points=inData->GetPoints();
  vtkPoints *newPoints;
  vtkCellArray *newCells;
  vtkIdType numPts=inData->GetNumberOfPoints();
  vtkIdType ptsId=0, newPtsId, oldPtsId;
  int numCellPnts = 0;
  bool logSafe = true;
  bool cellsCreated = false;

  double x1[3];


  outData->Initialize ();

  if (numPts <2 )
    return;


  newPoints = vtkPoints::New ();
  newPoints->SetDataTypeToDouble();
  newPoints->Allocate(numPts);

  newCells = vtkCellArray::New();
  // Give enough room for every other point being cut do to log scale
  // Use squeeze at the end to reclaim memory
  newCells->Allocate(1,numPts/2+1);

  newCells->InsertNextCell(numPts);
  for (ptsId=0; ptsId<numPts; ptsId++)
    {
      
      points->GetPoint(ptsId, x1);
      if ( inXScale == 1 )
        {
          if (x1[0] > 0)
            x1[0] = log10(x1[0]);
          else
            logSafe = false;
        }

      if ( inYScale == 1 )
        {
          if (x1[1] > 0)
	    x1[1] = log10(x1[1]); 
          else 
            logSafe = false;
        }

      newPtsId = newPoints->InsertNextPoint(x1);
      if (logSafe)
        {
          numCellPnts++;
          newCells->InsertCellPoint(newPtsId);
          cellsCreated = true;
        }
      else if (numCellPnts > 0)
        {
          if (numCellPnts < 2)
          {
             newCells->InsertCellPoint(oldPtsId);
             numCellPnts = 2;
          }
          newCells->UpdateCellCount (numCellPnts);
        
          // if this is not the last point in the list insert another cell
          if (ptsId + 1 < numPts)
          {
             if (numPts-(ptsId+1) ==  1)
                newCells->InsertNextCell(2);
             else
                newCells->InsertNextCell(numPts-(ptsId+1));
             numCellPnts = 0;
          }
        }
      logSafe = true;
      oldPtsId = newPtsId;

    };
  if (numCellPnts == 1)
  {
     newCells->InsertCellPoint(oldPtsId);
     numCellPnts = 2;
  }

  newCells->UpdateCellCount (numCellPnts);
 
  // Reclaim any memory lost do to log scale adjustments 
  newCells->Squeeze ();   
  outData->SetPoints(newPoints);

  if (cellsCreated)
     outData->SetLines(newCells);

  newPoints->Delete (); 
  newCells->Delete();
  
  return;
  
};


void vtkPolyLineActor2DExtra::SetData(vtkPolyData *inData)
{

  // initialize polyline data
  
  this->Data->Initialize();

  // reset the transform
  this->PolyLineTransform->Identity ();

  // put data into local vtk structures

  if (inData)
    {
      this->Data->ShallowCopy (inData);
    };


  this->Modified();
  
};

void vtkPolyLineActor2DExtra::SetClipData (int flag)
{
  if (ClipData == flag)
    return;

  ClipData = flag;

  if (!this->ClipData)
     this->PolyLineMapper->RemoveAllClippingPlanes ();
  else
    {
      for (int ii=0; ii<4; ii++)
        PolyLineMapper->AddClippingPlane (this->ClipPlanes[ii]);
    }


  this->Modified ();
}

