// 
//  Copyright (C) 2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



#ifndef __vtkPolyLineActor2DExtra_hh
#define __vtkPolyLineActor2DExtra_hh

#include <string>
#include "vtkActor2D.h"
#include "vtkProperty2D.h"
#include "vtkDoubleArray.h"
#include "vtkClipPolyData.h"

using namespace std;

class vtkPolyDataMapper2D;
class vtkPolyData;
class vtkPlane;
class vtkTransform;
class vtkTransformPolyDataFilter;

class vtkPolyLineActor2DExtra : public vtkActor2D
{
public:
  vtkTypeRevisionMacro(vtkPolyLineActor2DExtra,vtkActor2D);

  // Description:
  // Instantiate object.
  static vtkPolyLineActor2DExtra *New();


  // Description:
  // Set actual data
  void SetData(vtkPolyData *inData);

  void SetLocation(double inMinX, double inMinY,double inMaxX,double inMaxY);

  // Description:
  // Get the poly line transform
  vtkGetObjectMacro(PolyLineTransform,vtkTransform);


  // Description:
  // Specify the (min,max) x data range.
  vtkSetVector2Macro(XRange,double);
  vtkGetVectorMacro(XRange,double,2);


  
  // Description:
  // Specify the (min,max) y data range.
  vtkSetVector2Macro(YRange,double);
  vtkGetVectorMacro(YRange,double,2);

  // Description:
  // Get the (min,max) x data range.
  vtkGetVectorMacro(XDataRange,double,2);


  // Description:
  // Get the (min,max) y data range.
  vtkGetVectorMacro(YDataRange,double,2);

  // Description:
  // Get the (min,max) x clipped data range.
  vtkGetVectorMacro(XClippedRange,double,2);


  // Description:
  // Get the (min,max) y clipped data range.
  vtkGetVectorMacro(YClippedRange,double,2);




  // Description:
  // Set/Get the min value when converting to log

  void SetMinLogXLimit( double inMinLogLimit)
  { if ( inMinLogLimit> 0)
    { this->MinLogXLimit=inMinLogLimit;this->Modified();}
  ;};
  double  GetMinLogXLimit(){return this->MinLogXLimit;};
  
  void SetMinLogYLimit( double inMinLogLimit)
  { if ( inMinLogLimit> 0)
    { this->MinLogYLimit=inMinLogLimit;this->Modified();}
  ;};
  double  GetMinLogYLimit(){return this->MinLogYLimit;};


  // Description:
  // Set/Get scale of X data (linear,log...)
  vtkSetClampMacro(XScale, int, 0,10);
  vtkGetMacro(XScale, int);


  // Description:
  // Set/Get scale of Y data (linear,log...)
  vtkSetClampMacro(YScale, int, 0,10);
  vtkGetMacro(YScale, int);

  // Clip data
  void SetClipData(int flag);
  int GetClipData() {return ClipData;};


  // Description:
  // Draw the axis. 
  int RenderOverlay(vtkViewport* viewport);
  int RenderOpaqueGeometry(vtkViewport* viewport);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport *) {return 0;}

/*
  // Description:
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  void ReleaseGraphicsResources(vtkWindow *);
*/

  //Description:
  // build data ready to display

  virtual void BuildData(vtkViewport *viewport);



  // Description:
  // Shallow copy of an data actor. Overloads the virtual vtkProp method.
  void ShallowCopy(vtkProp *prop);



protected:
  vtkPolyLineActor2DExtra();
  ~vtkPolyLineActor2DExtra();


  double XRange[2];
  double YRange[2];


  double XDataRange[2];
  double YDataRange[2];

  double XClippedRange[2];
  double YClippedRange[2];


  int   XScale;
  int   YScale;

  double MinLogXLimit;
  double MinLogYLimit;


  double   LastPosition[2];
  double   LastPosition2[2];



  // Description:
  // Map data into screen coords

  static int MapData(double* inDataRange, 
	       double inPixelRangeMin, 
	       double inPixelRangeMax,
	       double& inOutValue);



  //Description:
  // Generate polyline
  void GeneratePolyLine(vtkPolyData* inData,int inXScale,int inYScale, vtkPolyData *outData);



  vtkPolyData             *Data;
  vtkPolyData             *PolyLine;
  vtkPolyDataMapper2D     *PolyLineMapper;
  vtkPlane                *ClipPlanes[4];
  vtkCoordinate           *Location[2];
  vtkTransform            *PolyLineTransform;
  vtkTransformPolyDataFilter *PolyLineTransformFilter;

  vtkTimeStamp  BuildTime;


  bool  ClipData;

  private:
  vtkPolyLineActor2DExtra(const vtkPolyLineActor2DExtra&);  // Not implemented.
  void operator=(const vtkPolyLineActor2DExtra&);  // Not implemented.


};


#endif
