// 
//  Copyright (C) 2006,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <math.h>
#include <bitset>
#include <list>

#include "vtkPlanes.h"
#include "vtkPlane.h"
#include "vtkAppendPolyData.h"
#include "vtkGlyph2D.h"
#include "vtkGlyphSource2D.h"
#include "vtkIntArray.h"
#include "vtkCellArray.h"
#include "vtkObjectFactory.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkDoubleArray.h"
#include "vtkPlanes.h"
#include "vtkProperty2D.h"
#include "vtkPolygonDataActor2DExtra.hh"
#include "vtkTriangleFilter.h"
#include "vtkDelaunay2D.h"
#include "vtkPlane.h"
#include "vtkTransform.h"
#include "vtkTransformPolyDataFilter.h"
#include "vtkPoints.h"


vtkCxxRevisionMacro(vtkPolygonDataActor2DExtra, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkPolygonDataActor2DExtra);



//----------------------------------------------------------------------------
// Instantiate this object.
vtkPolygonDataActor2DExtra::vtkPolygonDataActor2DExtra()
{
  this->PositionCoordinate = vtkCoordinate::New ();
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(0.0, 0.0);

  this->Position2Coordinate = vtkCoordinate::New ();
  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(1, 1);
  this->Position2Coordinate->SetReferenceCoordinate(NULL);
  
  this->ClipData = true;

  for(int indx=0;indx<4;indx++)
    {
      ClipPlanes[indx]=vtkPlane::New();

    };

  this->ClipBounds[0] = 0;
  this->ClipBounds[1] = 0;
  this->ClipBounds[2] = 1;
  this->ClipBounds[3] = 1;


  ClipPlanes[0]->SetOrigin(0,0,0);
  ClipPlanes[0]->SetNormal(1,0,0);

  ClipPlanes[1]->SetOrigin(1,0,0);
  ClipPlanes[1]->SetNormal(-1,0,0);

  ClipPlanes[2]->SetOrigin(0,0,0);
  ClipPlanes[2]->SetNormal(0,1,0);

  ClipPlanes[3]->SetOrigin(0,1,0);
  ClipPlanes[3]->SetNormal(0,-1,0);
  
  vtkCoordinate *coordinateSystem = vtkCoordinate::New();
  coordinateSystem->SetCoordinateSystemToNormalizedViewport ();

  vtkPoints *Points = vtkPoints::New ();
  vtkCellArray *Cells = vtkCellArray::New ();

  Points->Initialize ();

  this->PolyLineDataSet = vtkPolyData::New ();
  this->PolyLineDataSet->Initialize();
  this->PolyLineDataSet->SetPoints (Points);
  this->PolyLineDataSet->SetLines (Cells);



  this->Transform = vtkTransform::New();

  this->PolyLineTransformFilter = vtkTransformPolyDataFilter::New();
  this->PolyLineTransformFilter->SetTransform(this->Transform);
  this->PolyLineTransformFilter->SetInput(this->PolyLineDataSet);

  this->PlotLineDataMapper = vtkPolyDataMapper2D::New();
  this->PlotLineDataMapper->SetInputConnection(this->PolyLineTransformFilter->GetOutputPort ());
  this->PlotLineDataMapper->SetTransformCoordinate (coordinateSystem);

  this->PlotLineDataMapper->AddClippingPlane (this->ClipPlanes[0]);
  this->PlotLineDataMapper->AddClippingPlane (this->ClipPlanes[1]);
  this->PlotLineDataMapper->AddClippingPlane (this->ClipPlanes[2]);
  this->PlotLineDataMapper->AddClippingPlane (this->ClipPlanes[3]);

  this->PlotLineDataActor = vtkActor2D::New();
  this->PlotLineDataActor->SetMapper(this->PlotLineDataMapper);


  this->PolygonDataSet = vtkPolyData::New ();
  this->PolygonDataSet->Initialize();
  this->PolygonDataSet->SetPoints (Points);
  this->PolygonDataSet->SetPolys (Cells);


  this->PolygonTransformFilter = vtkTransformPolyDataFilter::New();
  this->PolygonTransformFilter->SetTransform(this->Transform);
  this->PolygonTransformFilter->SetInput(this->PolygonDataSet);


  this->PlotPolygonDataMapper = vtkPolyDataMapper2D::New();

  this->triangleFilter = vtkTriangleFilter::New();
  this->triangleFilter->SetInputConnection(this->PolygonTransformFilter->GetOutputPort ());
  this->triangleFilter->ReleaseDataFlagOn ();

  this->PlotPolygonDataMapper->SetInputConnection(this->triangleFilter->GetOutputPort());
  this->PlotPolygonDataMapper->SetTransformCoordinate (coordinateSystem);

  this->PlotPolygonDataMapper->AddClippingPlane (ClipPlanes[0]);
  this->PlotPolygonDataMapper->AddClippingPlane (ClipPlanes[1]);
  this->PlotPolygonDataMapper->AddClippingPlane (ClipPlanes[2]);
  this->PlotPolygonDataMapper->AddClippingPlane (ClipPlanes[3]);

  this->PlotPolygonDataActor = vtkActor2D::New();
  this->PlotPolygonDataActor->SetMapper(this->PlotPolygonDataMapper);


  this->LinesVisibility=1;
  this->PolygonVisibility=1;

  this->Enabled = true;


  Points->Delete ();
  Cells->Delete ();

}

//----------------------------------------------------------------------------
vtkPolygonDataActor2DExtra::~vtkPolygonDataActor2DExtra()
{

  this->PlotLineDataMapper->Delete();
  this->PlotLineDataActor->Delete();

  
  this->PlotPolygonDataMapper->Delete();
  this->PlotPolygonDataActor->Delete();

  this->triangleFilter->Delete();


  for(int indx=0;indx<4;indx++)
    {
      ClipPlanes[indx]->Delete();
    };

  this->PolyLineDataSet->Delete();
  this->PolygonDataSet->Delete ();

  this->PolyLineTransformFilter->Delete ();
  this->PolygonTransformFilter->Delete ();
  this->Transform->Delete (); 

};

//----------------------------------------------------------------------------
// Build the axis, ticks, title, and labels and render.

int vtkPolygonDataActor2DExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility || !this->Enabled)
    return 0;

  int renderedSomething=0;

  if (this->ClipData)
    this->CalculateClipBounds (viewport);

  if ( this->PolygonVisibility )
    renderedSomething += this->PlotPolygonDataActor->RenderOpaqueGeometry(viewport);

  if ( this->LinesVisibility)
    renderedSomething += this->PlotLineDataActor->RenderOpaqueGeometry(viewport);

  
  return renderedSomething;
}


int vtkPolygonDataActor2DExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility || !this->Enabled)
    return 0;

  int renderedSomething=0;

  if (this->ClipData)
    this->CalculateClipBounds (viewport);

  if ( this->PolygonVisibility )
    renderedSomething += this->PlotPolygonDataActor->RenderOverlay(viewport);

  if ( this->LinesVisibility)
    renderedSomething += this->PlotLineDataActor->RenderOverlay(viewport);
 
  

  
  return renderedSomething;
}


//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkPolygonDataActor2DExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}


void vtkPolygonDataActor2DExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  this->PlotLineDataActor->ReleaseGraphicsResources(win);
  this->PlotPolygonDataActor->ReleaseGraphicsResources(win);


}


void vtkPolygonDataActor2DExtra::SetClipData (int flag)
{
  if (this->ClipData == flag)
    return;

  this->ClipData = flag;

  if (!this->ClipData)
    {
    this->PlotLineDataMapper->RemoveAllClippingPlanes ();
    this->PlotPolygonDataMapper->RemoveAllClippingPlanes ();
    }
  else
    {
      for (int ii=0; ii<4; ii++)
        {
        this->PlotLineDataMapper->AddClippingPlane (ClipPlanes[ii]);
        this->PlotPolygonDataMapper->AddClippingPlane (ClipPlanes[ii]);
        }
    }

  this->Modified ();
}

void vtkPolygonDataActor2DExtra::SetClipBounds (double *bounds)
{
  if (!bounds)
    return;

  for (int ii=0; ii<4; ii++)
    this->ClipBounds[ii] = bounds[ii];
}

void vtkPolygonDataActor2DExtra::CalculateClipBounds (vtkViewport *viewport)

{
  vtkCoordinate *coordinate =   this->PlotPolygonDataMapper->GetTransformCoordinate ();

  if (coordinate)
    {
    double *pos;

    coordinate->SetValue (this->ClipBounds[0], this->ClipBounds[1]);
    pos = coordinate->GetComputedDoubleViewportValue (viewport);

    this->ClipPlanes[0]->SetOrigin(pos[0],0,0);
    this->ClipPlanes[2]->SetOrigin(0,pos[1],0);

    coordinate->SetValue (this->ClipBounds[2], this->ClipBounds[3]);
    pos = coordinate->GetComputedDoubleViewportValue (viewport);

    this->ClipPlanes[1]->SetOrigin(pos[0],0,0);
    this->ClipPlanes[3]->SetOrigin(0,pos[1],0);

    }
  else
    {
    this->ClipPlanes[0]->SetOrigin(this->ClipBounds[0],0,0);
    this->ClipPlanes[1]->SetOrigin(this->ClipBounds[2],0,0);
    this->ClipPlanes[2]->SetOrigin(0,this->ClipBounds[1],0);
    this->ClipPlanes[3]->SetOrigin(0,this->ClipBounds[3],0);
    }
}

void vtkPolygonDataActor2DExtra::GetClipBounds (double bounds[4])
{
  for (int ii=0; ii<4; ii++)
    bounds[ii] = this->ClipBounds[ii];
}

void vtkPolygonDataActor2DExtra::LoadData (vtkDataArray *data)
{
  vtkPoints *points =  vtkPoints::New ();
  vtkCellArray *cells = vtkCellArray::New ();
  int numPnts;

  if (data)
    points->SetData (data);
  else
    points->Allocate(0);

  numPnts = points->GetNumberOfPoints ();
  cells->InsertNextCell(numPnts+1);

  for (int ptsId=0; ptsId<numPnts; ptsId++)
    cells->InsertCellPoint(ptsId);

  cells->InsertCellPoint(0);

  this->PolyLineDataSet->SetPoints (points);
  this->PolyLineDataSet->SetLines (cells);
  this->PolygonDataSet->SetPoints (points);
  this->PolygonDataSet->SetPolys (cells);

  cells->Delete ();
  points->Delete ();

}

vtkDataArray *vtkPolygonDataActor2DExtra::GetData ()
{
  return this->PolyLineDataSet->GetPoints ()->GetData ();
}


