// 
//  Copyright (C) 2006,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//




#ifndef __vtkPolygonDataActor2DExtra_hh
#define __vtkPolygonDataActor2DExtra_hh

#include <string>
#include "vtkActor2D.h"
#include "vtkProperty2D.h"
#include "vtkDoubleArray.h"
#include "vtkPolyData.h"
#include "vtkTransform.h"

#include "vtkCoordinate.h" //Because of the viewport coordinate macro

using namespace std;

class vtkPolyDataMapper2D;
class vtkAppendPolyData;
class vtkTriangleFilter;
class vtkDelaunay2D;
class vtkPoints;
class vtkCellArray;
class vtkPlane;
class vtkTransformPolyDataFilter;

class vtkPolygonDataActor2DExtra : public vtkActor2D
{
public:
  vtkTypeRevisionMacro(vtkPolygonDataActor2DExtra,vtkActor2D);

  // Description:
  // Instantiate object.
  static vtkPolygonDataActor2DExtra *New();


  // Description:
  // Set/Get actual data
  virtual void LoadData (vtkDataArray *data);
  vtkDataArray *GetData ();

  // Description:
  // Get Transform 
  vtkGetObjectMacro(Transform, vtkTransform);



  // Description:
  // Specify opposite corners of the box defining the boundary of the
  // widget. By default, these coordinates are in the normalized viewport
  // coordinate system, with Position the lower left of the outline, and
  // Position2 relative to Position. Note that using these methods are
  // affected by the ProportionalResize flag. That is, if the aspect ratio of
  // the representation is to be preserved (e.g., ProportionalResize is on),
  // then the rectangle (Position,Position2) is a bounding rectangle. Also,
  vtkViewportCoordinateMacro(Position);
  vtkViewportCoordinateMacro(Position2);


  // Description:
  // Set/Get visibility of the lines.
  vtkSetMacro(LinesVisibility, int);
  vtkGetMacro(LinesVisibility, int);
  vtkBooleanMacro(LinesVisibility, int);

  vtkSetMacro(PolygonVisibility, int);
  vtkGetMacro(PolygonVisibility, int);
  vtkBooleanMacro(PolygonVisibility, int);

  // Clip data
  void SetClipData(int flag);
  int GetClipData() {return ClipData;};

  void SetClipBounds (double *bounds);
  void GetClipBounds (double bounds[4]);

  void Enable () {this->Enabled=true;};
  void Disable () {this->Enabled=false;};


  // Description:
  // Draw the axis. 
  int RenderOverlay(vtkViewport* viewport);
  int RenderOpaqueGeometry(vtkViewport* viewport);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport *) {return 0;}

  // Description:
  // Does this prop have some translucent polygonal geometry?
  virtual int HasTranslucentPolygonalGeometry();

  // Description:
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  void ReleaseGraphicsResources(vtkWindow *);


  // Accessors for border and actual polygon

  vtkActor2D* GetLineActor() { return this->PlotLineDataActor;};
  vtkActor2D* GetPolygonActor() { return this->PlotPolygonDataActor;};

  


protected:
  vtkPolygonDataActor2DExtra();
  ~vtkPolygonDataActor2DExtra();

  void CalculateClipBounds (vtkViewport *viewport);


  int   LinesVisibility;
  int   PolygonVisibility;

  vtkPolyData             *PolyLineDataSet;
  vtkPolyDataMapper2D     *PlotLineDataMapper;
  vtkActor2D              *PlotLineDataActor;

  vtkPolyData             *PolygonDataSet;
  vtkPolyDataMapper2D     *PlotPolygonDataMapper;
  vtkActor2D              *PlotPolygonDataActor;
  vtkTriangleFilter       *triangleFilter;
  vtkDelaunay2D           *delaunayFilter;
  vtkPlane*               ClipPlanes[4];

  vtkTransform            *Transform;
  vtkTransformPolyDataFilter *PolyLineTransformFilter;
  vtkTransformPolyDataFilter *PolygonTransformFilter;


  // Layout (position of lower left and upper right corners of border)
  vtkCoordinate *PositionCoordinate;
  vtkCoordinate *Position2Coordinate;

  bool  ClipData;
  bool  Enabled;
  double ClipBounds[4];

  private:
  vtkPolygonDataActor2DExtra(const vtkPolygonDataActor2DExtra&);  // Not implemented.
  void operator=(const vtkPolygonDataActor2DExtra&);  // Not implemented.


};


#endif
