// 
//  Copyright (C) 2004-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkScalarBarActorExtra.cxx,v $

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/

#include <cmath>
#include <cctype>

#include "vtkScalarBarActorExtra.hh"

#include "vtkCellArray.h"
#include "vtkCellData.h"
#include "vtkObjectFactory.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"
#include "vtkTextMapper.h"
#include "vtkTextProperty.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkProperty2D.h"
#include "vtkTextActorExtra.hh"
#include "vtkScalarsToColors.h"
#include "vtkLookupTable.h"
#include "vtkPlane.h"
#include "vtkDoubleArray.h"

#define NOCLIP 1

#ifndef PI
#define PI 3.14159265359
#endif


vtkCxxRevisionMacro(vtkScalarBarActorExtra, "$Revision: 1.38 $");
vtkStandardNewMacro(vtkScalarBarActorExtra);

vtkCxxSetObjectMacro(vtkScalarBarActorExtra,LabelTextProperty,vtkTextProperty);


//----------------------------------------------------------------------------
// Instantiate this object.
vtkScalarBarActorExtra::vtkScalarBarActorExtra()
{
  double pos1[] = {0,0};
  double pos2[] = {1, 1};
  this->PositionCoordinate->SetCoordinateSystemToNormalizedViewport();
  this->PositionCoordinate->SetValue(pos1);

  this->Position2Coordinate->SetCoordinateSystemToNormalizedViewport();
  this->Position2Coordinate->SetValue(pos2);
  this->Position2Coordinate->SetReferenceCoordinate(NULL);

  this->Location[0] = vtkCoordinate::New ();
  this->Location[0]->SetCoordinateSystemToNormalizedViewport ();
  this->Location[0]->SetValue (pos1);
  this->Location[1] = vtkCoordinate::New ();
  this->Location[1]->SetCoordinateSystemToNormalizedViewport ();
  this->Location[1]->SetValue (pos1);
  
  this->MajorTicksLength = 5;
  this->MajorTickOffset = 0;

  this->TitleFlip=0;
  this->LabelsFlip=0;
  this->MajorTicksFlip=0;
  this->BarFlip=0;

  this->LabelOffset=2+this->MajorTicksLength+this->MajorTickOffset;

  this->LabelTextProperty = vtkTextProperty::New();
  this->LabelTextProperty->SetBold(1);
  this->LabelTextProperty->SetItalic(1);
  this->LabelTextProperty->SetShadow(0);
  this->LabelTextProperty->SetFontFamilyToArial();


  this->LabelFormat = new char[8]; 
  sprintf(this->LabelFormat,"%s","%-#6.2g");

  this->TitleActor = vtkTextActorExtra::New();
  this->TitleActor->GetTextProperty()->ShallowCopy(this->LabelTextProperty);
  this->TitleActor->GetPositionCoordinate()->
                    SetReferenceCoordinate(this->PositionCoordinate);

#ifdef NOCLIP
  this->ClipData = false;
#else
  this->ClipData = true;

  for(int indx=0;indx<4;indx++)
    this->ClipPlanes[indx]=vtkPlane::New();

  this->ClipPlanes[0]->SetOrigin(0,0,0);
  this->ClipPlanes[0]->SetNormal(1,0,0);

  this->ClipPlanes[1]->SetOrigin(1,0,0);
  this->ClipPlanes[1]->SetNormal(-1,0,0);

  this->ClipPlanes[2]->SetOrigin(0,0,0);
  this->ClipPlanes[2]->SetNormal(0,1,0);

  this->ClipPlanes[3]->SetOrigin(0,1,0);
  this->ClipPlanes[3]->SetNormal(0,-1,0);
#endif
  this->ClipLocation[0] = vtkCoordinate::New ();
  this->ClipLocation[1] = vtkCoordinate::New ();

  this->ClipLocation[0]->SetCoordinateSystemToNormalizedViewport();
  this->ClipLocation[1]->SetCoordinateSystemToNormalizedViewport();
  this->ClipLocation[0]->SetValue(0.0,1.0);
  this->ClipLocation[1]->SetValue(0.0,1.0);


  this->ScalarBar = vtkPolyData::New();
  this->ScalarBarMapper = vtkPolyDataMapper2D::New();
  this->ScalarBarMapper->SetInput(this->ScalarBar);
  this->ScalarBarMapper->SetColorModeToMapScalars ();
  this->ScalarBarMapper->SetUseLookupTableScalarRange (1);
  this->ScalarBarActor = vtkActor2D::New();
  this->ScalarBarActor->SetMapper(this->ScalarBarMapper);
  this->ScalarBarActor->GetPositionCoordinate()->
    SetReferenceCoordinate(this->PositionCoordinate);


#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->ScalarBarMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif  

  this->BoundingBox = vtkPolyData::New();
  this->BoundingBoxMapper = vtkPolyDataMapper2D::New();
  this->BoundingBoxMapper->SetInput(this->BoundingBox);
  this->BoundingBoxActor = vtkActor2D::New();
  this->BoundingBoxActor->SetMapper(this->BoundingBoxMapper);
  this->BoundingBoxActor->GetPositionCoordinate()->
    SetReferenceCoordinate(this->PositionCoordinate);


#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->BoundingBoxMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif


  this->MajorTicks = vtkPolyData::New();
  this->MajorTicksMapper = vtkPolyDataMapper2D::New();
  this->MajorTicksMapper->SetInput(this->MajorTicks);
  this->MajorTicksActor = vtkActor2D::New();
  this->MajorTicksActor->SetMapper(this->MajorTicksMapper);
  this->MajorTicksActor->GetPositionCoordinate()->
    SetReferenceCoordinate(this->PositionCoordinate);

#ifndef NOCLIP
  for (int ii=0; ii<4; ii++)
    this->MajorTicksMapper->AddClippingPlane (this->ClipPlanes[ii]);
#endif  

  this->MajorTickLabelVertAlignmentOffset=0.5;
  this->MajorTickLabelHorizAlignmentOffset=1;

  this->ScalarBarVisibility = 1;
  this->BoundingBoxVisibility = 1;
  this->MajorTicksVisibility = 1;
  this->LabelVisibility = 1;
  this->TitleVisibility = 0;

  this->LastPosition[0] = this->LastPosition[1] = 0;
  this->LastPosition2[0] = this->LastPosition2[1] = 0;

  
  this->MajorTicksLocations.resize(0);

  this->LookupTable = NULL; 

  this->updatePositions = false;

  this->ScalarBarVertAlignmentOffset = 0;
  this->ScalarBarHorizAlignmentOffset = .5;

  this->ScalarBarOrigin[0] = 0;
  this->ScalarBarOrigin[1] = 0;
  this->ScalarBarWidth = 0;
  this->ScalarBarLength = 0;

  this->Orientation = VTK_ORIENT_HORIZONTAL;
  
  this->UseLookupTableRange = false;

  this->ScalarRange[0] = 0;
  this->ScalarRange[1] = 1;
}

//----------------------------------------------------------------------------
vtkScalarBarActorExtra::~vtkScalarBarActorExtra()
{
  if (this->LabelFormat) 
    {
    delete [] this->LabelFormat;
    this->LabelFormat = NULL;
    }
  
  this->TitleActor->Delete();

  while(this->LabelActors.size() > 0)
    {
      this->LabelActors.front()->Delete();
      this->LabelActors.pop_front();
    };

  this->Location[0]->Delete ();
  this->Location[1]->Delete ();

  this->ScalarBar->Delete();
  this->ScalarBarMapper->Delete();
  this->ScalarBarActor->Delete();

  this->BoundingBox->Delete();
  this->BoundingBoxMapper->Delete();
  this->BoundingBoxActor->Delete();

  this->MajorTicks->Delete();
  this->MajorTicksMapper->Delete();
  this->MajorTicksActor->Delete();
  

#ifndef NOCLIP
  for(int indx=0;indx<4;indx++)
    this->ClipPlanes[indx]->Delete();
#endif

  this->ClipLocation[0]->Delete ();
  this->ClipLocation[1]->Delete ();

  this->SetLabelTextProperty(NULL);

  if (this->LookupTable)
    this->LookupTable = NULL;
}

//----------------------------------------------------------------------------
// Build the axis, ticks, title, and labels and render.

int vtkScalarBarActorExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;
  

  this->BuildScalarBar(viewport);

  // Everything is built, just have to render
  if ( this->ScalarBarVisibility)
    {
    renderedSomething += this->ScalarBarActor->RenderOpaqueGeometry(viewport);
      
    }

  if ( this->BoundingBoxVisibility)
    {
    renderedSomething += this->BoundingBoxActor->RenderOpaqueGeometry(viewport);
    }

  
  if ( this->MajorTicksVisibility)
    {
    renderedSomething += this->MajorTicksActor->RenderOpaqueGeometry(viewport);
    };


  if ( this->LabelVisibility )
    {
    for (list<vtkTextActorExtra*>::iterator tmpIter=this->LabelActors.begin();tmpIter!=this->LabelActors.end();tmpIter++)
      {
      renderedSomething += 
	    (*tmpIter)->RenderOpaqueGeometry(viewport);
      }
    }

  if ( this->TitleVisibility )
    {
    renderedSomething += this->TitleActor->RenderOpaqueGeometry(viewport);
    }

  return renderedSomething;
}

//----------------------------------------------------------------------------
// Render the axis, ticks, title, and labels.

int vtkScalarBarActorExtra::RenderOverlay(vtkViewport *viewport)
{
  if ( !this->Visibility)
    return 0;

  int renderedSomething=0;

  // Everything is built, just have to render
  if ( this->ScalarBarVisibility )
    {
    renderedSomething += this->ScalarBarActor->RenderOverlay(viewport);
    }

  if ( this->BoundingBoxVisibility )
    {
    renderedSomething += this->BoundingBoxActor->RenderOverlay(viewport);
    }


  if ( this->MajorTicksVisibility)
    {
      renderedSomething += this->MajorTicksActor->RenderOverlay(viewport);
    };


  if ( this->LabelVisibility )
    {
      for (list<vtkTextActorExtra*>::iterator tmpIter=this->LabelActors.begin();tmpIter!=this->LabelActors.end();tmpIter++)
	{
	  renderedSomething += 
	    (*tmpIter)->RenderOverlay(viewport);
	}
    }

  if ( this->TitleVisibility )
    {
    renderedSomething += this->TitleActor->RenderOverlay(viewport);
    }

  return renderedSomething;
}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkScalarBarActorExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}

//----------------------------------------------------------------------------
// Release any graphics resources that are being consumed by this actor.
// The parameter window could be used to determine which graphic
// resources to release.
void vtkScalarBarActorExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  this->TitleActor->ReleaseGraphicsResources(win);
  for (list<vtkTextActorExtra*>::iterator tmpIter=this->LabelActors.begin();tmpIter!=this->LabelActors.end();tmpIter++)
    {
      (*tmpIter)->ReleaseGraphicsResources(win);
    }

  this->ScalarBarActor->ReleaseGraphicsResources(win);

  this->MajorTicksActor->ReleaseGraphicsResources(win);

}

//----------------------------------------------------------------------------
void vtkScalarBarActorExtra::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

  if (this->LabelTextProperty)
    {
    os << indent << "Label Text Property:\n";
    this->LabelTextProperty->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "Label Text Property: (none)\n";
    }

  os << indent << "Label Format: " << this->LabelFormat << "\n";
  os << indent << "Major Tick Length: " << this->MajorTicksLength << "\n";
  os << indent << "Tick Offset: " << this->MajorTickOffset << "\n";
  os << indent << "Labels Offset: " << this->LabelOffset << "\n";
  os << indent << "Title Parallel Offset: " << this->TitleParallelOffset << "\n";
  os << indent << "Title Perpendicular Offset: " << this->TitlePerpendicularOffset << "\n";
  
  os << indent << "Scalar Bar Visibility: " 
     << (this->ScalarBarVisibility ? "On\n" : "Off\n");
  
  os << indent << "Major Tick Visibility: " 
     << (this->MajorTicksVisibility ? "On\n" : "Off\n");
  
  os << indent << "Label Visibility: " 
     << (this->LabelVisibility ? "On\n" : "Off\n");
  
  os << indent << "Title Visibility: " 
     << (this->TitleVisibility ? "On\n" : "Off\n");

}

//----------------------------------------------------------------------------

void vtkScalarBarActorExtra::SetClipLocations(double inMinX, double inMinY,double inMaxX,double inMaxY)
{
  this->ClipLocation[0]->SetValue(inMinX,inMinY);
  this->ClipLocation[1]->SetValue(inMaxX,inMaxY);

  this->Modified();
};

void vtkScalarBarActorExtra::SetLookupTable(vtkScalarsToColors* lookupTable)
{
   this->LookupTable = lookupTable;
   this->ScalarBarMapper->SetLookupTable (this->LookupTable);
}


#define VTK_AA2D_DEBUG 0

void vtkScalarBarActorExtra::BuildScalarBar(vtkViewport *viewport)
{

  unsigned int i,positionsHaveChanged;
  vtkIdType ptIds[2];
  double p1[3], p2[3], *x;
  double xTick[3];
  double theta, val;
  double stringSize[2];
  char tmpString[512],*chPtr;
  double *clipPos1, *clipPos2;

  vtkScalarsToColors *lut = this->LookupTable;
  double range[2];
  double barWidth = 0;
  double barLength = 0;

  if (this->UseLookupTableRange)
    {
    double *tmp_range = lut->GetRange();
    range[0] = tmp_range[0];
    range[1] = tmp_range[1];
    }
  else
    {
    range[0] = this->ScalarRange[0];
    range[1] = this->ScalarRange[1];
    }


  if (this->LabelVisibility && !this->LabelTextProperty)
    {
    vtkErrorMacro(<<"Need label text property to render scalar bar actor");
    return;
    }

  if ((this->ScalarBarLength == 0) || (this->ScalarBarWidth == 0))
    {
    vtkErrorMacro(<<"The scalar bar length and width need to be greater than zero");
    return;
    }

  this->UpdatePositions (viewport);

  positionsHaveChanged = 0;
  if (viewport->GetMTime() > this->BuildTime || 
      (viewport->GetVTKWindow() && 
       viewport->GetVTKWindow()->GetMTime() > this->BuildTime))
    {

    // Check to see whether we have to rebuild everything
    // Viewport change may not require rebuild
    double *lastPosition = 
      this->Location[0]->GetComputedDoubleViewportValue(viewport);
    double *lastPosition2 =
      this->Location[1]->GetComputedDoubleViewportValue(viewport);
    if (lastPosition[0] != this->LastPosition[0] ||
        lastPosition[1] != this->LastPosition[1] ||
        lastPosition2[0] != this->LastPosition2[0] ||
        lastPosition2[1] != this->LastPosition2[1] )
      {
      positionsHaveChanged = 1;
      }
    }
  
  // if the positions have not changed return
  if ( (!positionsHaveChanged) && (this->GetMTime() < this->BuildTime)) // && (this->TitleActor->GetMTime()< this->BuildTime))

    {
    return;
    }

#if VTK_AA2D_DEBUG
  printf ("vtkScalarBarActorExtra::BuildScalarBar: Rebuilding scalar bar\n");
#endif

  vtkDebugMacro(<<"Rebuilding scalar bar");

  // Initialize and get important info


  this->MajorTicks->Initialize();
  this->TitleActor->SetProperty(this->GetProperty());

  // Generate the axis and tick marks.
  // We'll do our computation in viewport coordinates. First determine the
  // location of the endpoints.

  x = this->Location[0]->GetComputedDoubleViewportValue(viewport);
  p1[0] = x[0]; 
  p1[1] = x[1];
  p1[2] = 0.0;
  this->LastPosition[0] = x[0]; 
  this->LastPosition[1] = x[1];

  x = this->Location[1]->GetComputedDoubleViewportValue(viewport);
  p2[0] = x[0]; 
  p2[1] = x[1];
  p2[2] = 0.0;
  this->LastPosition2[0] = x[0]; 
  this->LastPosition2[1] = x[1];

  if (this->Orientation == VTK_ORIENT_HORIZONTAL)
    {
    barWidth = p2[1] - p1[1];
    barLength = p2[0] - p1[0];
    }
  else
    {
    barWidth = p2[0] - p1[0];
    barLength = p2[1] - p1[1];
    }


  // create and fill the scalar bar
  this->GenerateScalarBar (p1, barWidth, barLength);

  // create bounding box
  if (this->BoundingBoxVisibility)
    {

    double BBpoint[] = {0,0,0};
    this->BoundingBox->Initialize();
//    this->BoundingBoxActor->SetProperty(this->GetProperty());

    vtkPoints *BBpts = vtkPoints::New();
    BBpts->SetDataTypeToDouble();
    vtkCellArray *BBlines = vtkCellArray::New();
    this->BoundingBox->SetPoints(BBpts);
    this->BoundingBox->SetLines(BBlines);

    BBpoint[0] = p1[0];
    BBpoint[1] = p1[1];
    BBpts->InsertNextPoint (BBpoint);
    BBpoint[0] = p1[0];
    BBpoint[1] = p2[1];
    BBpts->InsertNextPoint (BBpoint);
    BBpoint[0] = p2[0];
    BBpoint[1] = p2[1];
    BBpts->InsertNextPoint (BBpoint);
    BBpoint[0] = p2[0];
    BBpoint[1] = p1[1];
    BBpts->InsertNextPoint (BBpoint);

    vtkIdType BBptIds[] = {0, 1, 2, 3, 0};
    BBlines->InsertNextCell(5, BBptIds);

    BBpts->Delete();
    BBlines->Delete();

    }

  // Generate point along axis (as well as tick points)

  if (this->Orientation == VTK_ORIENT_HORIZONTAL)
    {
    theta = 0.;
    }
  else
    {
    theta = 3*PI/2.0;
    }

  // Set clip planes
  clipPos1 = this->ClipLocation[0]->GetComputedDoubleViewportValue(viewport);
  clipPos2 = this->ClipLocation[1]->GetComputedDoubleViewportValue(viewport);

#ifndef NOCLIP
  this->ClipPlanes[0]->SetOrigin(clipPos1[0],0,0);
  this->ClipPlanes[1]->SetOrigin(clipPos2[0],0,0);
  this->ClipPlanes[2]->SetOrigin(0,clipPos1[1],0);
  this->ClipPlanes[3]->SetOrigin(0,clipPos2[1],0);
#endif


  // Create Maj ticks
  vtkPoints *Majpts = vtkPoints::New();
  Majpts->SetDataTypeToDouble();
  vtkCellArray *Majlines = vtkCellArray::New();
  this->MajorTicks->SetPoints(Majpts);
  this->MajorTicks->SetLines(Majlines);
  Majpts->Delete();
  Majlines->Delete();


  
  xTick[2] = 0.0;
  int tickDirSign = 1;
  int barFlipSign = 1;
  int ticksFlipSign = 1;

  bool ticksFlip = !(this->BarFlip == this->LabelsFlip);
 
  if ( this->MajorTicksFlip )
    tickDirSign = -1;
  if ( this->BarFlip )
    ticksFlipSign = barFlipSign = -1;
  if ( this->LabelsFlip )
    ticksFlipSign = -barFlipSign;


  vtkstd::vector<double> tmpMajLocations(this->MajorTicksLocations.size());
  //Map maj ticks to the screen

  if (( this->MajorTicksVisibility) || (this->LabelVisibility) )
    {
    vector<double>::iterator tmpIter=this->MajorTicksLocations.begin();
    unsigned int count = 0;
    int scale = this->GetLogScale ();

    if (! this->ScalarBarVisibility )
       barWidth = 0;


    while( tmpIter != this->MajorTicksLocations.end())
      {
      // map the scalar value to the x pixel coordinate
      if (this->Orientation == VTK_ORIENT_HORIZONTAL)
        {
        if (MapScale(range,p1[0],p2[0],scale,.01,*tmpIter,xTick[0])<0)
          {
          tmpIter++;
	  continue;
          }
        // the y value will always be the lower pixel value
        xTick[1] = p1[1];
        }
      else
        {
        if (MapScale(range,p1[1],p2[1],scale,.01,*tmpIter,xTick[1])<0)
          {
          tmpIter++;
          continue;
          }
        // the y value will always be the lower pixel value
        xTick[0] = p1[0];
        }

      tmpMajLocations[count] = *tmpIter;

      xTick[0] = xTick[0] + (ticksFlipSign*(this->MajorTickOffset + ticksFlip*barWidth)) * sin(theta);
      xTick[1] = xTick[1] - (ticksFlipSign*(this->MajorTickOffset + ticksFlip*barWidth)) * cos(theta);

      Majpts->InsertNextPoint(xTick);

	      
      xTick[0] = xTick[0] + ticksFlipSign*tickDirSign*this->MajorTicksLength * sin(theta);
      xTick[1] = xTick[1] - ticksFlipSign*tickDirSign*this->MajorTicksLength * cos(theta);

      Majpts->InsertNextPoint(xTick);


      tmpIter++;
      count++;

      };  
  
    if (count != tmpMajLocations.size ())
      {
      tmpMajLocations.resize (count);
      }

    for (i = 0; i < tmpMajLocations.size(); i++)
      {
      ptIds[0] = 2*i;
      ptIds[1] = 2*i + 1;
      Majlines->InsertNextCell(2, ptIds);
      };
    }



  // Build the labels

  if (this->LabelVisibility)
    {

    BuildLabels(this->LabelActors,tmpMajLocations.size());

    vtkstd::list<vtkTextActorExtra*>::iterator tmpActIter=this->LabelActors.begin();
    bool intFormatFlag=IsIntFormat(this->LabelFormat);

    for(i=0;i<tmpMajLocations.size();i++)
      {
      (*tmpActIter)->SetProperty(this->GetProperty());
      (*tmpActIter)->GetTextProperty()->ShallowCopy(this->LabelTextProperty);


      val = tmpMajLocations[i];
      if (intFormatFlag)
        {
        sprintf(tmpString, this->LabelFormat, static_cast<int>(val));
        }
      else
        {
	sprintf(tmpString, this->LabelFormat, val);
	};
      chPtr=tmpString;
      while( isspace(*chPtr) && (*chPtr!=0))
	chPtr++;
		  
      (*tmpActIter)->SetInput(chPtr);

	      
      Majpts->GetPoint(2 * i , xTick);
      xTick[0]-= ticksFlipSign*tickDirSign*(this->MajorTickOffset) * sin(theta);
      xTick[1]+= ticksFlipSign*tickDirSign*(this->MajorTickOffset) * cos(theta);

      (*tmpActIter)->SetVertAlignmentOffset(this->MajorTickLabelVertAlignmentOffset);
      (*tmpActIter)->SetHorizAlignmentOffset(this->MajorTickLabelHorizAlignmentOffset);
	      


      this->SetOffsetPosition(xTick, 
			      theta, 
			      0,
                              -this->LabelOffset,
//			      -(this->LabelOffset+(this->BarFlip*this->ScalarBarWidth)), 
                              ticksFlip,
			      (*tmpActIter));
	      
      tmpActIter++;
      };

    // Check if labels can be clipped. Since there is currently no
    // good way to clip a partial label, clip the entire label if it
    // crosses the clipping plane.
    if (this->ClipData)
      {
      double *lblPos;
      tmpActIter=this->LabelActors.begin();
      while (tmpActIter!=this->LabelActors.end())
        {
        lblPos = (*tmpActIter)->GetPosition();

        if ((clipPos1[0] > lblPos[0]) || (clipPos2[0] < lblPos[0]) ||
            (clipPos1[1] > lblPos[1]) || (clipPos2[1] < lblPos[1]))
          tmpActIter = this->LabelActors.erase (tmpActIter);
        else
          tmpActIter++;

        }
      };
    } // If labels visible



  // Now build the title

  if (this->TitleVisibility)
    {

    double llx,lly,urx,ury;
    this->TitleActor->GetBBox(viewport,llx,lly,urx,ury);
    stringSize[0]=urx-llx;
    stringSize[1]=ury-lly;

     
    // title is opposite of ticks by default
    // centered in the bar

    // find the base location for the label
    if (this->Orientation == VTK_ORIENT_HORIZONTAL)
      {
      xTick[0] = p1[0] + (barLength / 2.0);
      xTick[1] = (this->BarFlip == this->TitleFlip) ? p2[1] : p1[1];
      }
    else
      {
      xTick[1] = p1[1] + (barLength / 2.0);
      xTick[0] = (this->BarFlip == this->TitleFlip) ? p2[0] : p1[0];
      }

    double actualParallelOffset = barLength * this->TitleParallelOffset;
 
    this->SetOffsetPosition(xTick, 
			      theta, 
			      actualParallelOffset,
			      this->TitlePerpendicularOffset,
			      (this->BarFlip) ? !this->TitleFlip : this->TitleFlip,
			      this->TitleActor);


    if (this->ClipData)
      {
      double *lblPos;
      lblPos = this->TitleActor->GetPosition();

      if ((clipPos1[0] > lblPos[0]) || (clipPos2[0] < lblPos[0]) ||
          (clipPos1[1] > lblPos[1]) || (clipPos2[1] < lblPos[1]))
        this->TitleActor->SetPosition (-1, -1);

      };

    } // If title visible
  
  this->BuildTime.Modified();
}






//----------------------------------------------------------------------------
// Position text with respect to a point (xTick) where the angle of the line
// from the point to the center of the text is given by theta. The offset
// is the spacing between ticks and labels.
void vtkScalarBarActorExtra::SetOffsetPosition(double xTick[3], double theta, 
					    double inXOffset,double inYOffset,
					    int inAnnotationsFlip,
					    vtkActor2D *actor) 
{
  double pos[2];
  double radAngle=fmod(theta,2*PI);
  
  if (radAngle< 0)
    radAngle = 2*PI +radAngle;

  if ( inAnnotationsFlip)
    inYOffset =-inYOffset;


  pos[0] = xTick[0] + inXOffset*cos(theta)- inYOffset*sin(theta);
  pos[1] = xTick[1] + inXOffset*sin(theta)+ inYOffset*cos(theta);

  actor->SetPosition(pos[0], pos[1]);

}

//----------------------------------------------------------------------------
void vtkScalarBarActorExtra::ShallowCopy(vtkProp *prop)
{
  vtkScalarBarActorExtra *a = vtkScalarBarActorExtra::SafeDownCast(prop);
  if ( a != NULL )
    {
      this->SetLookupTable(a->GetLookupTable());
      this->SetLabelFormat(a->GetLabelFormat());
      this->SetMajorTicksLength(a->GetMajorTicksLength());
      this->SetMajorTickOffset(a->GetMajorTickOffset());
      this->SetLabelsFlip(a->GetLabelsFlip());
      this->SetTitleFlip(a->GetTitleFlip());
      this->SetMajorTicksFlip(a->GetMajorTicksFlip());
      this->SetBarFlip(a->GetBarFlip());
      this->SetLabelOffset(a->GetLabelOffset());
      this->SetTitleParallelOffset(a->GetTitleParallelOffset());
      this->SetTitlePerpendicularOffset(a->GetTitlePerpendicularOffset());
      this->SetScalarBarVisibility(a->GetScalarBarVisibility());
      this->SetMajorTicksVisibility(a->GetMajorTicksVisibility());
      this->SetLabelVisibility(a->GetLabelVisibility());
      this->SetTitleVisibility(a->GetTitleVisibility());
      //      this->SetTitleFontSize(a->GetTitleFontSize());
      //      this->SetLabelFontSize(a->GetLabelFontSize());
      this->SetLabelTextProperty(a->GetLabelTextProperty());
    }

  // Now do superclass
  this->vtkActor2D::ShallowCopy(prop);
}

int vtkScalarBarActorExtra::MapScale(double* inDataRange, 
				  double inPixelRangeMin, 
				  double inPixelRangeMax,
				  int inScale,
				  double inMinLogLimit,
				  double inValue,
				  double& outValue)
{

  if (
      ( (inDataRange[0] < inDataRange[1]) && (inValue< inDataRange[0] || inValue>inDataRange[1]))
      ||
      ( (inDataRange[0] > inDataRange[1]) && (inValue> inDataRange[0] || inValue<inDataRange[1] ))
      /*      ||
      (inPixelRangeMin <0)
      ||
      (inPixelRangeMax <0)
      */
      ||
      ( ( inDataRange[0] == inDataRange[1]) && (inPixelRangeMin != inPixelRangeMax))
      )
    {
    return -1;
    };
  
  if (inDataRange[0] == inDataRange[1])
    {
    outValue=inPixelRangeMin;
    return 1;
    };

  switch(inScale)
    {
    case 1: // log scale
      if (inValue <=0 )
	return -1;
      double tmpDataRange[2];
      tmpDataRange[0]=inDataRange[0];
      tmpDataRange[1]=inDataRange[1];
      if ( tmpDataRange[0] <=0)
	{
        if ( inValue > inMinLogLimit)
	  tmpDataRange[0]=inMinLogLimit;
	else
	  tmpDataRange[0]=inValue;
	};

      if (tmpDataRange[1] <=0 )
	{
	tmpDataRange[1]=inMinLogLimit;
	if ( inValue > inMinLogLimit )
	  tmpDataRange[1]=inMinLogLimit;
	else
	  tmpDataRange[1]=inValue;
	};

  
//	outValue = (inPixelRangeMax-inPixelRangeMin)*(log10(inValue)-log10(tmpDataRange[0]))/(log10(tmpDataRange[1])-log10(tmpDataRange[0]));
      outValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(log10(inValue)-log10(tmpDataRange[0]))/(log10(tmpDataRange[1])-log10(tmpDataRange[0]));
      break;
    case 0: // linear scale
    default:
//        outValue = (inPixelRangeMax-inPixelRangeMin)*(inValue-inDataRange[0])/(inDataRange[1]-inDataRange[0]);
      outValue = inPixelRangeMin + (inPixelRangeMax-inPixelRangeMin)*(inValue-inDataRange[0])/(inDataRange[1]-inDataRange[0]);
      break;
    };

  return 1;

};


void vtkScalarBarActorExtra::BuildLabels(list<vtkTextActorExtra*>& inLabelActors, unsigned int inCount)
{
  if ( inCount < inLabelActors.size() )
    {
      while(inLabelActors.size() > inCount)
	{
	  inLabelActors.front()->Delete();
	  inLabelActors.pop_front();
	};
    }
  else if ( inCount > inLabelActors.size() )
    {
      vtkTextActorExtra* tmpAct;

      while( inLabelActors.size() < inCount)
	{
	  tmpAct = vtkTextActorExtra::New();
          tmpAct->GetPositionCoordinate()->
                    SetReferenceCoordinate(this->PositionCoordinate);
	  inLabelActors.push_front(tmpAct);
	}
    };


  for (list<vtkTextActorExtra*>::iterator tmpIter=inLabelActors.begin();tmpIter!=inLabelActors.end();tmpIter++)
    {
	(*tmpIter)->Clear();
    }

};


int  vtkScalarBarActorExtra::SetMajorTicksLocations(const vtkstd::vector<double>& inMajLocations)
{
  this->MajorTicksLocations.resize(inMajLocations.size());
  for(unsigned int indx=0;indx<inMajLocations.size();indx++)
    {
      this->MajorTicksLocations[indx]=inMajLocations[indx];
    }

  
  this->Modified();
  return 1;
};


bool vtkScalarBarActorExtra::IsIntFormat(char * inFormatString)
{

  char * tmpP=inFormatString;
  bool retVal=false;

  while ( *tmpP )
    {
      if ( *tmpP == '%' )
	{
	  tmpP++;
	  while(*tmpP)
	    {
	      if (( *tmpP == 'd') || (*tmpP == 'i') || (*tmpP == 'o') || (*tmpP == 'x') || (*tmpP == 'u') || (*tmpP == 'c') || (*tmpP == 'X') )
		{
		  retVal=true;
		  break;
		}
	      else if (( *tmpP == 'e') || (*tmpP == 'E') || (*tmpP == 'f') || (*tmpP == 'F') || (*tmpP == 'g') || (*tmpP == 'G') || (*tmpP == 'a') || (*tmpP == 'A')|| (*tmpP == 'c') || (*tmpP == 's')|| (*tmpP == 'C') || (*tmpP == 'S') || (*tmpP == 'p') || (*tmpP == 'n')|| (*tmpP == 'z') || (*tmpP == '%') )
		{
		  break;
		};
	      tmpP++;
	    };

	};
      tmpP++;
    };

  return retVal;

}

void vtkScalarBarActorExtra::SetMajorTickLabelVertAlignmentOffset(double inAlignment)
{
  if (( inAlignment >= 0.0 ) && (inAlignment<=1.0 ))
    {    
      this->MajorTickLabelVertAlignmentOffset=inAlignment;
      
      
      this->Modified();
    };
}

void vtkScalarBarActorExtra::SetMajorTickLabelHorizAlignmentOffset(double inAlignment)
{
  if (( inAlignment >= 0.0 ) && (inAlignment<=1.0 ))
    {    
      this->MajorTickLabelHorizAlignmentOffset=inAlignment;
      
      
      this->Modified();
    };
}

void vtkScalarBarActorExtra::SetClipData (int flag)
{
#ifndef NOCLIP

  if (ClipData == flag)
     return;

  ClipData = flag;

  if (!this->ClipData)
    {
      this->ScalarBarMapper->RemoveAllClippingPlanes ();
      this->MajorTicksMapper->RemoveAllClippingPlanes ();
    }
  else
    {
      for (int ii=0; ii<4; ii++)
        {
          this->ScalarBarMapper->AddClippingPlane (this->ClipPlanes[ii]);
          this->MajorTicksMapper->AddClippingPlane (this->ClipPlanes[ii]);
        }
    }

  this->Modified ();
#endif
}

void vtkScalarBarActorExtra::SetOrientation (int orientation)
{
  this->Orientation = orientation;
  this->updatePositions = true;
  this->Modified ();
}

void vtkScalarBarActorExtra::SetOrigin (double origin[])
{
  this->ScalarBarOrigin[0] = origin[0];
  this->ScalarBarOrigin[1] = origin[1];
  this->updatePositions = true;
  this->Modified ();
}

void vtkScalarBarActorExtra::SetLength (float length)
{
  this->ScalarBarLength = length;
  this->updatePositions = true;
  this->Modified ();
}

void vtkScalarBarActorExtra::SetWidth (float width)
{
  this->ScalarBarWidth = width;
  this->updatePositions = true;
  this->Modified ();
}

void vtkScalarBarActorExtra::SetVertAlignmentOffset (double offset)
{
  this->ScalarBarVertAlignmentOffset = offset;
  this->updatePositions = true;
  this->Modified ();
}

void vtkScalarBarActorExtra::SetHorizAlignmentOffset (double offset)
{
  this->ScalarBarHorizAlignmentOffset = offset;
  this->updatePositions = true;
  this->Modified ();
}


void vtkScalarBarActorExtra::GenerateScalarBar (double offset[2], double width, double length)
{
  vtkScalarsToColors *lut = this->LookupTable;
  double range[2];
  int numColors = 256;
  bool isLogTable = false;
  int numPts = 0;

  if (!this->LookupTable)
     return;


  // Build scalar bar object; determine its type
  //
  // is this a vtkLookupTable or a subclass of vtkLookupTable
  // with its scale set to log
  // NOTE: it's possible we could to without the 'lut' variable
  // later in the code, but if the vtkLookupTableSafeDownCast operation
  // fails for some reason, this code will break in new ways. So, the 'LUT'
  // variable is used for this operation only
  vtkLookupTable *LUT = vtkLookupTable::SafeDownCast( this->LookupTable );
  isLogTable = this->GetLogScale ();
  if ( LUT )
    {
    numColors = LUT->GetNumberOfColors  ();
    }

  if (this->UseLookupTableRange)
    {
    double *tmp_range = lut->GetRange();
    range[0] = tmp_range[0];
    range[1] = tmp_range[1];
    }
  else
    {
    range[0] = this->ScalarRange[0];
    range[1] = this->ScalarRange[1];
    }

  numPts = 2*(numColors + 1);
  vtkPoints *pts = vtkPoints::New();
  pts->SetNumberOfPoints(numPts);
  vtkCellArray *polys = vtkCellArray::New();
  polys->Allocate(polys->EstimateSize(numColors,4));
  vtkDoubleArray *colors = vtkDoubleArray::New();
  colors->SetNumberOfComponents(1);
  colors->SetNumberOfTuples(numColors);

  this->ScalarBarActor->SetProperty(this->GetProperty());
  this->ScalarBar->Initialize();

  // generate points
  double x[3]; x[2] = 0.0;
  double delta;
  if ( this->Orientation == VTK_ORIENT_VERTICAL )
    {
    delta=(double)length/numColors;
    for (int i=0; i<numPts/2; i++)
      {
      x[0] = offset[0];
      x[1] = i*delta+offset[1];
      pts->SetPoint(2*i,x);
      x[0] = width+offset[0];
      pts->SetPoint(2*i+1,x);
      }
    }
  else
    {
    delta=(double)length/numColors;
    for (int i=0; i<numPts/2; i++)
      {
      x[0] = i*delta+offset[0];
      x[1] = width+offset[1];
      pts->SetPoint(2*i,x);
      x[1] = offset[1];
      pts->SetPoint(2*i+1,x);
      }
    }

  //polygons & cell colors
  double *intensity;
  vtkIdType ptIds[4];
  for (int i=0; i<numColors; i++)
    {
    ptIds[0] = 2*i;
    ptIds[1] = ptIds[0] + 1;
    ptIds[2] = ptIds[1] + 2;
    ptIds[3] = ptIds[0] + 2;
    polys->InsertNextCell(4,ptIds);

    intensity = colors->GetPointer(i); //write into array directly
    intensity[0] = range[0] + (range[1] - range[0])*
                           ((double)i /(numColors-1.0));
    }

  this->ScalarBar->SetPoints(pts);
  this->ScalarBar->SetPolys(polys);
  this->ScalarBar->GetCellData()->SetScalars(colors);

  pts->Delete();
  colors->Delete();
  polys->Delete();

}

void vtkScalarBarActorExtra::UpdatePositions (vtkViewport *viewport)
{
  double pos1[] = {0, 0, 0};
  double pos2[] = {0, 0, 0};

  // if the positions haven't been modified return
  if (! this->updatePositions)
    return;

  if ( !viewport )
    return;

  this->updatePositions = false;
  double origin[2] = {this->ScalarBarOrigin[0], this->ScalarBarOrigin[1]};

//  viewport->NormalizedViewportToViewport (origin[0], origin[1]);
  if ( this->Orientation == VTK_ORIENT_VERTICAL )
    {
    pos1[0] = origin[0] - (this->ScalarBarWidth * this->ScalarBarVertAlignmentOffset);
    pos2[0] = pos1[0] + this->ScalarBarWidth;
    pos1[1] = origin[1] - (this->ScalarBarLength * this->ScalarBarHorizAlignmentOffset);
    pos2[1] = pos1[1] + this->ScalarBarLength;
    }
  else
    {
    pos1[0] = origin[0] - (this->ScalarBarLength * this->ScalarBarHorizAlignmentOffset);
    pos2[0] = pos1[0] + this->ScalarBarLength;
    pos1[1] = origin[1] - (this->ScalarBarWidth * this->ScalarBarVertAlignmentOffset);
    pos2[1] = pos1[1] + this->ScalarBarWidth;
    }

  
/*  this->PositionCoordinate->SetCoordinateSystemToViewport();
  this->Position2Coordinate->SetCoordinateSystemToViewport();

  this->PositionCoordinate->SetValue(pos1);
  this->Position2Coordinate->SetValue(pos2);
*/
  this->Location[0]->SetCoordinateSystemToNormalizedViewport();
  this->Location[1]->SetCoordinateSystemToNormalizedViewport();

  this->Location[0]->SetValue(pos1);
  this->Location[1]->SetValue(pos2);
}

bool vtkScalarBarActorExtra::GetLogScale ()
{
  vtkLookupTable *LUT = vtkLookupTable::SafeDownCast( this->LookupTable );
  bool isLogTable = false;
  if ( LUT )
    {
    if ( LUT->GetScale() == VTK_SCALE_LOG10 )
      {
      isLogTable = true;
      }
    }

  return isLogTable;
}


