// 
//  Copyright (C) 2005-2007,2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <math.h>

#include "vtkMath.h"
#include "vtkObjectFactory.h"
#include "vtkTextMapper.h"
#include "vtkTextProperty.h"
#include "vtkViewport.h"
#include "vtkWindow.h"
#include "vtkProperty2D.h"
#include "vtkCellArray.h"


#include "vtkTextActorExtra.hh"
#include "vtkOpenGLFreeTypeTextMapperExtra.hh"
#include "vtkMath.h"



vtkTextActorFormat::vtkTextActorFormat()
{
  super=0;
  next=0;
  sub=0;
  size=0;
  red=0;
  green=0;
  blue=0;
  fontFamily=VTK_TIMES;
  fontBold=0;
  fontItalic=0;
  fontFile="";

};
 
vtkTextActorFormat::vtkTextActorFormat(const vtkTextActorFormat& inFormat)
{
  text = inFormat.text;
  fontFamily=inFormat.fontFamily;
  fontBold=inFormat.fontBold;
  fontItalic=inFormat.fontItalic;
  fontFile = inFormat.fontFile;
  size = inFormat.size;
  red= inFormat.red;
  green= inFormat.green;
  blue= inFormat.blue;
  
  if ( inFormat.super )
    super = new vtkTextActorFormat(*(inFormat.super));
  else
    super=0;

  if ( inFormat.next )
    next = new vtkTextActorFormat(*(inFormat.next));
  else
    next =0;

  if ( inFormat.sub )
    sub=new vtkTextActorFormat(*(inFormat.sub));
  else
    sub=0;
};

vtkTextActorFormat::~vtkTextActorFormat()
{
  if ( super )
    delete super;
  if ( next)
    delete next;
  if (sub)
    delete sub;

};

vtkTextActorFormat& vtkTextActorFormat::operator = (const vtkTextActorFormat& inFormat)
{
  if ( &inFormat == this)
    return *this;

  if ( super )
    delete super;
  if ( next)
    delete next;
  if (sub)
    delete sub; 
  

  text = inFormat.text;
  fontFamily=inFormat.fontFamily;
  fontBold=inFormat.fontBold;
  fontItalic=inFormat.fontItalic;
  fontFile = inFormat.fontFile;
  size = inFormat.size;
  red= inFormat.red;
  green= inFormat.green;
  blue= inFormat.blue;
  
  if ( inFormat.super )
    super = new vtkTextActorFormat(*(inFormat.super));
  else
    super=0;

  if ( inFormat.next )
    next = new vtkTextActorFormat(*(inFormat.next));
  else
    next =0;

  if ( inFormat.sub )
    sub=new vtkTextActorFormat(*(inFormat.sub));
  else
    sub=0;


  return *this;
};





vtkCxxRevisionMacro(vtkTextActorExtra, "$Revision: 1.22 $");
vtkStandardNewMacro(vtkTextActorExtra);



// ----------------------------------------------------------------------------
vtkTextActorExtra::vtkTextActorExtra()
{
  this->PositionCoordinate->SetCoordinateSystemToViewport();
  this->Position2Coordinate->SetCoordinateSystemToViewport();


  this->LastOrigin[0]     = 0;
  this->LastOrigin[1]     = 0;

  this->LastSize[0]       = 0;
  this->LastSize[1]       = 0;

  this->textActor = vtkActor2D::New();
  this->textActor->GetPositionCoordinate()->SetCoordinateSystemToViewport();
  this->textActor->GetPosition2Coordinate()->SetCoordinateSystemToViewport();
  this->textMapper = vtkOpenGLFreeTypeTextMapperExtra::New();
  this->textActor->SetMapper(this->textMapper);

  this->superText=0;
  this->subText=0;
  this->nextText=0;

  this->superscriptSize=0.6;
  this->superscriptOffset=.6;
  this->subscriptSize=0.6;
  this->subscriptOffset=-.2;

  
  this->vertAlignmentOffset=0;
  this->horizAlignmentOffset=0;

  this->drawBBox=0;

  this->rootFlag=true;

  this->boxData = vtkPolyData::New();
  this->boxDataMapper = vtkPolyDataMapper2D::New();
  this->boxDataMapper->SetInput(this->boxData);
  this->boxDataActor = vtkActor2D::New();
  this->boxDataActor->SetMapper(this->boxDataMapper);
  this->boxDataActor->GetProperty()->SetColor(1,0,0);

  this->BuildTime.Modified();
}

// ----------------------------------------------------------------------------
vtkTextActorExtra::~vtkTextActorExtra()
{

  this->textMapper->Delete();
  this->textActor->Delete();
  if ( this->superText)
    this->superText->Delete();
  if ( this->subText)
    this->subText->Delete();
  if ( this->nextText)
    this->nextText->Delete();

  this->boxData->Delete();
  this->boxDataMapper->Delete();
  this->boxDataActor->Delete();
}




void vtkTextActorExtra::SetInput(const char* inText)
{
  this->textMapper->SetInput(inText);
  this->Modified();

};

const char* vtkTextActorExtra::GetInput()
{
  return this->textMapper->GetInput();

};



void vtkTextActorExtra::SetInput(vtkTextActorFormat* inFormat)
{
  AddFormattedText(inFormat,this);
  
};


vtkTextActorExtra* vtkTextActorExtra::GetSuperTextActorExtra()
{
  if (! this->superText)
    {
      this->superText = vtkTextActorExtra::New();
      this->superText->GetTextProperty()->ShallowCopy(this->textMapper->GetTextProperty());
      this->superText->SetSubscriptSize(this->subscriptSize);
      this->superText->SetSubscriptOffset(this->subscriptOffset);
      this->superText->SetSuperscriptSize(this->superscriptSize);
      this->superText->SetSuperscriptOffset(this->superscriptOffset);
    };

  return this->superText;
};




vtkTextActorExtra* vtkTextActorExtra::GetSubTextActorExtra()
{
  if (! this->subText)
    {
      this->subText = vtkTextActorExtra::New();
      this->subText->GetTextProperty()->ShallowCopy(this->textMapper->GetTextProperty());
      this->subText->SetSubscriptSize(this->subscriptSize);
      this->subText->SetSubscriptOffset(this->subscriptOffset);
      this->subText->SetSuperscriptSize(this->superscriptSize);
      this->subText->SetSuperscriptOffset(this->superscriptOffset);

    };

  return this->subText;
};


vtkTextActorExtra* vtkTextActorExtra::GetNextTextActorExtra()
{
  if (! this->nextText)
    {
      this->nextText = vtkTextActorExtra::New();
      this->nextText->GetTextProperty()->ShallowCopy(this->textMapper->GetTextProperty());
      this->nextText->SetSubscriptSize(this->subscriptSize);
      this->nextText->SetSubscriptOffset(this->subscriptOffset);
      this->nextText->SetSuperscriptSize(this->superscriptSize);
      this->nextText->SetSuperscriptOffset(this->superscriptOffset);
    };

  return this->nextText;
};


// ----------------------------------------------------------------------------
void vtkTextActorExtra::ShallowCopy(vtkProp *prop)
{
  vtkTextActorExtra *a = vtkTextActorExtra::SafeDownCast(prop);
  if ( a != NULL )
    {
    this->SetPosition2(a->GetPosition2());
    }
  // Now do superclass (mapper is handled by it as well).
  this->vtkActor2D::ShallowCopy(prop);
}

// ----------------------------------------------------------------------------
// Release any graphics resources that are being consumed by this actor.
// The parameter window could be used to determine which graphic
// resources to release.
void vtkTextActorExtra::ReleaseGraphicsResources(vtkWindow *win)
{
  
  this->textActor->ReleaseGraphicsResources(win);
  if ( this->superText)
    this->superText->ReleaseGraphicsResources(win);
  if ( this->subText)
    this->subText->ReleaseGraphicsResources(win);
  if ( this->nextText)
    this->nextText->ReleaseGraphicsResources(win);
  this->boxDataActor->ReleaseGraphicsResources(win);
}

// ----------------------------------------------------------------------------
int vtkTextActorExtra::RenderOverlay(vtkViewport *viewport)
{  

  int renderedSomething=0;

  if (this->GetVisibility () == 0)
    return renderedSomething;
  
  renderedSomething  +=this->textActor->RenderOverlay(viewport);
  if ( this->superText)
    renderedSomething  +=this->superText->RenderOverlay(viewport);
  if ( this->subText)
    renderedSomething  +=this->subText->RenderOverlay(viewport);
  if ( this->nextText)
    renderedSomething  +=this->nextText->RenderOverlay(viewport);
  if ( this->drawBBox)
    renderedSomething  +=this->boxDataActor->RenderOverlay(viewport);
  return renderedSomething;

}

// ----------------------------------------------------------------------------
int vtkTextActorExtra::RenderOpaqueGeometry(vtkViewport *viewport)
{



  int renderedSomething=0;

  if (this->GetVisibility () == 0)
    return renderedSomething;


  if ( this->rootFlag)
    {
    double * tmpP=  this->GetPositionCoordinate()->GetComputedDoubleViewportValue(viewport);
    double  position[2];
    bool positionsHaveChanged = false;

    position[0]=tmpP[0];
    position[1]=tmpP[1];

      
      
    if (viewport->GetMTime() > this->BuildTime ||
	   (viewport->GetVTKWindow() &&
	    viewport->GetVTKWindow()->GetMTime() > this->BuildTime))
      {

      if ((fabs (this->LastOrigin[0] - position[0]) > VTK_DBL_EPSILON) || 
          (fabs (this->LastOrigin[1] - position[1]) > VTK_DBL_EPSILON))
        positionsHaveChanged = true;
      }

    if (positionsHaveChanged || (this->GetMTime() > this->BuildTime ) ||
        (this->GetTextProperty ()->GetMTime () > this->BuildTime))
      {
	  

      double tmpCoord[2] = {0.0,0.0};
      double tmpXRange[2]={0.0,0.0},tmpYRange[2]={0.0,0.0};
      double tmpAngleRad=vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation());

      this->LastOrigin[0]=position[0];
      this->LastOrigin[1]=position[1];

      this->GetAlignmentRange(viewport,tmpXRange,tmpYRange);
	  
      tmpCoord[0]=-(tmpXRange[1]-tmpXRange[0])*this->horizAlignmentOffset;
      tmpCoord[1]=-tmpYRange[1]*this->vertAlignmentOffset;

      position[0] = tmpCoord[0]*cos(tmpAngleRad) - tmpCoord[1]*sin(tmpAngleRad) + position[0];
      position[1] = tmpCoord[0]*sin(tmpAngleRad) + tmpCoord[1]*cos(tmpAngleRad) + position[1];

      BuildText(viewport,position[0],position[1]);
      };
    };
      
  renderedSomething  +=this->textActor->RenderOpaqueGeometry(viewport);
  
  if ( this->superText)
    {
      renderedSomething  +=this->superText->RenderOpaqueGeometry(viewport);
    };
  
  if ( this->subText)
    {
      renderedSomething  +=this->subText->RenderOpaqueGeometry(viewport);
    };
  
  if ( this->nextText)
    {
      renderedSomething  +=this->nextText->RenderOpaqueGeometry(viewport);
    };


  if ( this->drawBBox)
    {
      renderedSomething  +=this->boxDataActor->RenderOpaqueGeometry(viewport);
    };

  return renderedSomething;
      


}

//-----------------------------------------------------------------------------
// Description:
// Does this prop have some translucent polygonal geometry?
int vtkTextActorExtra::HasTranslucentPolygonalGeometry()
{
  return 0;
}


// ----------------------------------------------------------------------------

void vtkTextActorExtra::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os,indent);

}

vtkTextProperty* vtkTextActorExtra::GetTextProperty()
{
  return this->textMapper->GetTextProperty();
}



void vtkTextActorExtra::GetSize(vtkViewport* viewport,double outSize[2])
{

  double sizeXRange[2],sizeYRange[2],bboxXRange[2],bboxYRange[2];

  sizeXRange[0]=sizeYRange[0]=0;
  sizeXRange[1]=sizeYRange[1]=0;
  bboxXRange[0]=bboxYRange[0]=0;
  bboxXRange[1]=bboxYRange[1]=0;

  GetSizeRange(viewport,0,0,sizeXRange,sizeYRange,bboxXRange,bboxYRange);

  outSize[0] = sizeXRange[1]-sizeXRange[0];
  if ( outSize[0] == 0 )
    outSize[1]=0;
  else
    outSize[1] = sizeYRange[1]-sizeYRange[0];
};

void vtkTextActorExtra::GetSizeRange(vtkViewport* viewport,double inXPos,double inYPos,double outXRange[2],double outYRange[2],double outBBoxXRange[2],double outBBoxYRange[2])
{

  double tmpSize[2];
  double tmpVal;
  double maxAsc,maxDesc;

  Order(outXRange);
  Order(outYRange);

  this->textMapper->GetSize(viewport,tmpSize);
  this->textMapper->GetFontExtents(maxAsc,maxDesc);
  

  tmpVal=inXPos+tmpSize[0];
  if (tmpVal > outXRange[1] )
    outXRange[1]=tmpVal;
  else if (tmpVal < outXRange[0])
    outXRange[0]=tmpVal;


  tmpVal=inYPos+maxAsc;
  if (tmpVal > outYRange[1] )
    outYRange[1]=tmpVal;
  else if (tmpVal < outYRange[0])
    outYRange[0]=tmpVal;

  
  tmpVal=inYPos+maxDesc;
  if (tmpVal > outYRange[1] )
    outYRange[1]=tmpVal;
  else if (tmpVal < outYRange[0])
    outYRange[0]=tmpVal;
      

  double tmpllx,tmplly,tmpurx,tmpury;

  this->textMapper->GetBBox(tmpllx,tmplly,tmpurx,tmpury);
  
  tmpllx+=inXPos;
  tmplly+=inYPos;
  tmpurx+=inXPos;
  tmpury+=inYPos;

  if (tmpllx < outBBoxXRange[0])
    outBBoxXRange[0]=tmpllx;
  else if ( tmpllx > outBBoxXRange[1])
    outBBoxXRange[1] =tmpllx;

  if ( tmpurx > outBBoxXRange[1] )
    outBBoxXRange[1]=tmpurx;
  else if ( tmpurx < outBBoxXRange[0] )
    outBBoxXRange[0] = tmpurx;

  if (tmplly < outBBoxYRange[0])
    outBBoxYRange[0]=tmplly;
  else if ( tmplly > outBBoxYRange[1])
    outBBoxYRange[1] =tmplly;

  if ( tmpury > outBBoxYRange[1] )
    outBBoxYRange[1]=tmpury;
  else if ( tmpury < outBBoxYRange[0] )
    outBBoxYRange[0] = tmpury;



  if ( this->superText)
    {
      this->superText->GetTextProperty()->SetFontSize(this->textMapper->GetTextProperty()->GetFontSize()*this->superscriptSize);
      this->superText->GetSizeRange(viewport,inXPos+tmpSize[0],inYPos+maxAsc*this->superscriptOffset,outXRange,outYRange,outBBoxXRange, outBBoxYRange);
    };
  
  if ( this->subText)
    {
      this->subText->GetTextProperty()->SetFontSize(this->textMapper->GetTextProperty()->GetFontSize()*this->subscriptSize);
      this->subText->GetSizeRange(viewport,inXPos+tmpSize[0],inYPos+maxAsc*this->subscriptOffset,outXRange,outYRange,outBBoxXRange, outBBoxYRange);
    };


  if ( this->nextText)
    {
      this->nextText->GetTextProperty()->SetFontSize(this->textMapper->GetTextProperty()->GetFontSize());
      this->nextText->GetSizeRange(viewport,outXRange[1],inYPos,outXRange,outYRange,outBBoxXRange, outBBoxYRange);
    };
  
};


void vtkTextActorExtra::Clear()
{

  this->textMapper->SetInput("");
  
  if ( this->superText)
    {
      this->superText->Delete();
      this->superText=0;
    };
  
  if ( this->subText)
    {
      this->subText->Delete();
      this->subText=0;
    };
  
  if ( this->nextText)
    {
      this->nextText->Delete();
      this->nextText=0;
    };

  this->Modified();
};

void vtkTextActorExtra::SetSuperscriptOffset(double inOffset)
{

  if ( inOffset < 0 )
    return;
  
  this->superscriptOffset=inOffset;
  this->Modified();

   if ( this->superText)
    {
      this->superText->SetSuperscriptOffset(inOffset);
    };
  
  if ( this->subText)
    {
      this->subText->SetSuperscriptOffset(inOffset);
    };

  if ( this->nextText)
    {
      this->nextText->SetSuperscriptOffset(inOffset);
    };

};

void vtkTextActorExtra::SetSuperscriptSize(double inSize)
{
  if ( inSize <= 0 )
    return;
  
  this->superscriptSize=inSize;
  this->Modified();

  if ( this->superText)
    {
      this->superText->SetSuperscriptSize(inSize);
    };
  
  if ( this->subText)
    {
      this->subText->SetSuperscriptSize(inSize);
    };
  
  if ( this->nextText)
    {
      this->nextText->SetSuperscriptSize(inSize);
    };

};

void vtkTextActorExtra::SetSubscriptOffset(double inOffset)
{
  this->subscriptOffset=inOffset;
  this->Modified();
  
  if ( this->superText)
    {
      this->superText->SetSubscriptOffset(inOffset);
    };
  
  if ( this->subText)
    {
      this->subText->SetSubscriptOffset(inOffset);
    };
  
  if ( this->nextText)
    {
      this->nextText->SetSubscriptOffset(inOffset);
    };
};

void vtkTextActorExtra::SetSubscriptSize(double inSize)
{
  if ( inSize < 0 )
    return;
  
  this->subscriptSize=inSize;
  this->Modified();

  if ( this->superText)
    {
      this->superText->SetSubscriptSize(inSize);
    };
  
  if ( this->subText)
    {
      this->subText->SetSubscriptSize(inSize);
    };
  
  if ( this->nextText)
    {
      this->nextText->SetSubscriptSize(inSize);
    };

};


void vtkTextActorExtra::Order(double inTab[2])
{
  double tmpVal;

  if ( inTab[0] > inTab[1])
    {
      tmpVal=inTab[0];
      inTab[0]=inTab[1];
      inTab[1]=tmpVal;
    };
};

void vtkTextActorExtra::RotatePosition(double inRotationOrigin[2],double inAngleRad,double inPosition[2],double outPosition[2])
{
  
  double tmpLoc[2];

  tmpLoc[0] = inPosition[0]-inRotationOrigin[0];
  tmpLoc[1] = inPosition[1]-inRotationOrigin[1];

  outPosition[0] = tmpLoc[0]*cos(inAngleRad) - tmpLoc[1]*sin(inAngleRad) + inRotationOrigin[0];
  outPosition[1] = tmpLoc[0]*sin(inAngleRad) + tmpLoc[1]*cos(inAngleRad) + inRotationOrigin[1];

};

void vtkTextActorExtra::BBoxRecursive(double& llx,double& lly,double& urx, double& ury)
{
  double tmp_llx, tmp_lly, tmp_urx, tmp_ury, tmp;
  double ua = 0.0, ub = 0.0,uc = 0.0,ud = 0.0, aa = 0.0, ab = 0.0, ac = 0.0, 
         ad = 0.0, na = 0.0, nb = 0.0, nc = 0.0, nd = 0.0, dx = 0.0, dy = 0.0;

  this->textMapper->GetApproximateBBox(tmp_llx,tmp_lly,tmp_urx,tmp_ury);

  if (this->subText)  this->subText->BBoxRecursive(ua, ub, uc, ud);
  if (this->superText)  this->superText->BBoxRecursive(aa, ab, ac, ad);
  if (this->nextText)  this->nextText->BBoxRecursive (na, nb, nc, nd);

  // adjust by all horizontal segment lengths
  dx = (nc - na);

  // do we need to account for subscript/superscript size?
//WLM  if ((ac - aa) > (uc - ua))  dx = dx + (ac - aa) * superscriptSize;
//WLM     else                 dx = dx + (uc - ua) * subscriptSize;
  if ((ac - aa) > (uc - ua))  dx = dx + (ac - aa);
     else                     dx = dx + (uc - ua); 

  if (nd - nb > tmp_ury - tmp_lly)  dy = nd - nb - (tmp_ury - tmp_lly); 
 
  //tmp = (ad - ab)*superscriptSize;
  tmp = (ad - ab);
  if (tmp > 0)  dy += tmp;

  llx = tmp_llx; 
  lly = tmp_lly;
  if (ud - ub)  lly  += subscriptOffset*(tmp_ury - tmp_lly);  
  urx = tmp_urx + dx;
  ury = tmp_ury + dy; 
}

void vtkTextActorExtra::GetBBox(vtkViewport* viewport,double& llx,double& lly,double& urx, double& ury)
{
  double *tmpCoord = this->GetPositionCoordinate()->GetComputedDoubleViewportValue(viewport);
  double tmpCoord1[2] = {0,0};
  double tmpXRange[2] = {0,0};
  double tmpYRange[2] = {0,0};
  double tmp_llx;
  double tmp_lly;
  double tmp_urx;
  double tmp_ury;

  BBoxRecursive(tmp_llx,tmp_lly,tmp_urx,tmp_ury);

  this->GetAlignmentRange(viewport,tmpXRange,tmpYRange);

  tmpCoord1[0] = tmpCoord[0] - ((tmpXRange[1]-tmpXRange[0])*this->horizAlignmentOffset) + tmp_llx;
  tmpCoord1[1] = tmpCoord[1] - (tmpYRange[1]*this->vertAlignmentOffset)+tmp_lly;

  vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpCoord1,tmpCoord1);

  llx = tmpCoord1[0];
  lly = tmpCoord1[1];

  tmpCoord1[0] = tmpCoord[0] - ((tmpXRange[1]-tmpXRange[0])*this->horizAlignmentOffset) + tmp_urx;
  tmpCoord1[1] = tmpCoord[1] - (tmpYRange[1]*this->vertAlignmentOffset)+tmp_ury;

  vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpCoord1,tmpCoord1);


  urx = tmpCoord1[0];
  ury = tmpCoord1[1];

  tmpCoord1[0] = llx;
  tmpCoord1[1] = urx;

  Order (tmpCoord1);

  llx = tmpCoord1[0];
  urx = tmpCoord1[1];

  tmpCoord1[0] = lly;
  tmpCoord1[1] = ury;

  Order (tmpCoord1);

  lly = tmpCoord1[0];
  ury = tmpCoord1[1];

  tmpCoord1[0] = tmpCoord[0] - ((tmpXRange[1]-tmpXRange[0])*this->horizAlignmentOffset) + tmp_llx;
  tmpCoord1[1] = tmpCoord[1] - (tmpYRange[1]*this->vertAlignmentOffset)+tmp_ury;

  vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpCoord1,tmpCoord1);

  if (tmpCoord1[0] < llx)
    llx = tmpCoord1[0];
  else if (tmpCoord1[0] > urx)
    urx = tmpCoord1[0];

  if (tmpCoord1[1] < lly)
    lly = tmpCoord1[1];
  else if (tmpCoord1[1] > ury)
    ury = tmpCoord1[1];

  tmpCoord1[0] = tmpCoord[0] - ((tmpXRange[1]-tmpXRange[0])*this->horizAlignmentOffset) + tmp_urx;
  tmpCoord1[1] = tmpCoord[1] - (tmpYRange[1]*this->vertAlignmentOffset)+tmp_lly;

  vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpCoord1,tmpCoord1);

  if (tmpCoord1[0] < llx)
    llx = tmpCoord1[0];
  else if (tmpCoord1[0] > urx)
    urx = tmpCoord1[0];

  if (tmpCoord1[1] < lly)
    lly = tmpCoord1[1];
  else if (tmpCoord1[1] > ury)
    ury = tmpCoord1[1];
}

void vtkTextActorExtra::BuildText(vtkViewport* viewport,double inXPos,double inYPos)
{

  double tmpCoord[2],tmpCoord1[2],tmpPosition[2];
  double  tmpSize[2],supSize[2],subSize[2];
  double tmpFontSize=0;
  double maxDesc;
  double italicAngle=0;
  double tmpAngle=0;
  
  tmpCoord1[0]=tmpCoord[0]= inXPos;
  tmpCoord1[1]=tmpCoord[1]= inYPos;

  supSize[0]=0;
  supSize[1]=0;
  subSize[0]=0;
  subSize[1]=0;

  this->textActor->SetPosition(tmpCoord);

  this->textMapper->GetSize(viewport, tmpSize);
  this->textMapper->GetFontExtents(tmpSize[1],maxDesc);
  this->textMapper->GetItalicAngle(italicAngle);
  
  if ( this->drawBBox)
    {

      double llx,lly,urx, ury;
      vtkIdType ptIds[2];
      vtkPoints *tmpPts = vtkPoints::New();
      tmpPts->SetDataTypeToDouble();
      vtkCellArray *tmpLines = vtkCellArray::New();
      
      // add place holder bounding box around text (including no text)
      BBoxRecursive(llx,lly,urx,ury);
      if (llx == urx && lly == ury)
      {
        double mAsc,mDesc;
        this->textMapper->GetFontExtents(mAsc,mDesc);
        urx += 3;
        ury += mAsc; 
      }
      llx-=2;
      lly-=2;
      urx+=2;
      ury+=2;
      
      tmpPosition[0]=tmpCoord[0]+llx;
      tmpPosition[1]=tmpCoord[1]+lly;

      vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpPosition,tmpCoord1);
      tmpPts->InsertNextPoint(tmpCoord1);

      tmpPosition[0]=tmpCoord[0]+urx;
      vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpPosition,tmpCoord1);
      tmpPts->InsertNextPoint(tmpCoord1);


      tmpPosition[1]=tmpCoord[1]+ury;
      vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpPosition,tmpCoord1);
      tmpPts->InsertNextPoint(tmpCoord1);

      tmpPosition[0]=tmpCoord[0]+llx;
      vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->textMapper->GetTextProperty()->GetOrientation()),tmpPosition,tmpCoord1);
      tmpPts->InsertNextPoint(tmpCoord1);

      
      this->boxData->Initialize();
      this->boxData->SetPoints(tmpPts);
      this->boxData->SetLines(tmpLines);
      tmpPts->Delete();
      tmpLines->Delete();
      
      ptIds[0] = 0;
      ptIds[1] = 1;
      tmpLines->InsertNextCell(2, ptIds);
      ptIds[0] = 1;
      ptIds[1] = 2;
      tmpLines->InsertNextCell(2, ptIds);
      ptIds[0] = 2;
      ptIds[1] = 3;
      tmpLines->InsertNextCell(2, ptIds);
      ptIds[0] = 3;
      ptIds[1] = 0;
      tmpLines->InsertNextCell(2, ptIds);
    };

      
  //  tmpCoord1[0] +=tmpSize[0];
  
  
  if ( this->superText)
    {
      
      tmpFontSize=this->textMapper->GetTextProperty()->GetFontSize()*this->superscriptSize;
      
      tmpCoord1[1] = inYPos+tmpSize[1]*this->superscriptOffset;
      tmpCoord1[0] = inXPos+tmpSize[0];

      tmpAngle = this->superText->GetItalicAngle();
	  
      if ((tmpAngle != italicAngle) && (italicAngle != 0.0))
	{
	  
	  tmpCoord1[0] -= 2*tmpSize[1]*tan(vtkMath::RadiansFromDegrees(italicAngle));
	};

      this->superText->GetTextProperty()->SetOrientation(this->textMapper->GetTextProperty()->GetOrientation());
      this->superText->GetTextProperty()->SetFontSize(tmpFontSize);
      //      this->superText->GetTextProperty()->SetColor(this->textActor->GetTextProperty()->GetColor());
      
      vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->superText->GetTextProperty()->GetOrientation()),tmpCoord1,tmpPosition);
      this->superText->BuildText(viewport,tmpPosition[0],tmpPosition[1]);
      
      this->superText->GetSize(viewport, supSize);
      
      if ( supSize[0] > 0.0)
	supSize[0]=tmpCoord1[0]+supSize[0];

    };
  
  
  
  
  
  
  if ( this->subText)
    {
      tmpFontSize=this->textMapper->GetTextProperty()->GetFontSize()*this->subscriptSize;

      tmpCoord1[0] = inXPos+tmpSize[0];

      tmpCoord1[1] = inYPos+tmpSize[1]*this->subscriptOffset;

      tmpAngle = this->subText->GetItalicAngle();
	  
      if ((tmpAngle != italicAngle) && (italicAngle != 0.0))
	{
	  
	  tmpCoord1[0] -= tmpSize[1]*tan(vtkMath::RadiansFromDegrees(italicAngle));
	};

      this->subText->GetTextProperty()->SetOrientation(this->textMapper->GetTextProperty()->GetOrientation());
      this->subText->GetTextProperty()->SetFontSize(tmpFontSize);
      //      this->subText->GetTextProperty()->SetColor(this->textActor->GetTextProperty()->GetColor());
      
      vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->subText->GetTextProperty()->GetOrientation()),tmpCoord1,tmpPosition);	  
      this->subText->BuildText(viewport,tmpPosition[0],tmpPosition[1]);
      
      this->subText->GetSize(viewport, subSize);  

      if ( subSize[0] > 0.0)
	subSize[0]=tmpCoord1[0]+subSize[0];
    };
  

  if ( this->nextText)
    {
      tmpCoord1[1]=tmpCoord[1];
      
      if (( supSize[0] > 0.0 ) || (subSize[0]>0.0))
	{
	  
	  if ( supSize[0] > subSize[0])
	    {
	      tmpAngle = this->superText->GetItalicAngle();
	      tmpCoord1[0] = supSize[0]-supSize[1]*tan(vtkMath::RadiansFromDegrees(tmpAngle));

	    }
	  else
	    {
	      tmpAngle = this->subText->GetItalicAngle();
	      tmpCoord1[0] = subSize[0]-subSize[1]*tan(vtkMath::RadiansFromDegrees(tmpAngle));
	    };
	}
      else
	{

	  tmpAngle = this->nextText->GetItalicAngle();
	  
	  tmpCoord1[0] = inXPos+tmpSize[0];
	  
	  if ((tmpAngle != italicAngle) && (italicAngle != 0.0))
	    {
	      
	      tmpCoord1[0] -= tmpSize[1]*tan(vtkMath::RadiansFromDegrees(italicAngle));
	    }
	}
      
      this->nextText->GetTextProperty()->SetFontSize(this->textMapper->GetTextProperty()->GetFontSize());
      this->nextText->GetTextProperty()->SetOrientation(this->textMapper->GetTextProperty()->GetOrientation());
      //      this->nextText->GetTextProperty()->SetColor(this->textActor->GetTextProperty()->GetColor());
      
      vtkTextActorExtra::RotatePosition(tmpCoord,vtkMath::RadiansFromDegrees(this->nextText->GetTextProperty()->GetOrientation()),tmpCoord1,tmpPosition);	  
      
      this->nextText->BuildText(viewport,tmpPosition[0],tmpPosition[1]);
    };
  
  this->BuildTime.Modified();

}

void vtkTextActorExtra::SetVertAlignmentOffset(double inAlignment)
{

  this->vertAlignmentOffset=inAlignment;
      
  this->Modified();      

}

void vtkTextActorExtra::SetHorizAlignmentOffset(double inAlignment)
{

  this->horizAlignmentOffset=inAlignment;
      
  this->Modified();

}


void vtkTextActorExtra::GetAlignmentRange(vtkViewport* viewport,double outXRange[2],double outYRange[2])
{
  double maxAsc,maxDesc;
  

  this->textMapper->GetFontExtents(maxAsc,maxDesc);
  
  outXRange[0]=0;
  outXRange[1]=0;
  outYRange[0]=0;
  outYRange[1]=0;
  

  outYRange[0]=0;
  outYRange[1]=maxAsc;
      

  double tmpYRange[2]={0.0,0.0};
  double tmpBBoxXRange[2]={0.0,0.0},tmpBBoxYRange[2]={0.0,0.0};

  this->GetSizeRange(viewport,0,0,outXRange,tmpYRange,tmpBBoxXRange,tmpBBoxYRange);


}

double vtkTextActorExtra::GetItalicAngle()
{
  double tmpItalicAngle=0;
  this->textMapper->GetItalicAngle(tmpItalicAngle);

  return tmpItalicAngle;
};


void vtkTextActorExtra::AddFormattedText(vtkTextActorFormat* inFormat,vtkTextActorExtra* inTextActor,bool inRootFlag)
{


  if ((inTextActor ) && ( inFormat))
    {
      inTextActor->SetRootFlag(inRootFlag);
      inTextActor->SetInput(inFormat->text.c_str());
      inTextActor->GetTextProperty()->SetFaceFileName(0);

      if ( !inFormat->fontFile.empty())
	{
	  inTextActor->GetTextProperty()->SetFaceFileName(inFormat->fontFile.c_str());
	}
      else if ( inFormat->fontFamily == VTK_HELVETICA )
	{
	  inTextActor->GetTextProperty()->SetFontFamilyToHelvetica();
	  if ( inFormat->fontBold )
	    inTextActor->GetTextProperty()->BoldOn();
	  else
	    inTextActor->GetTextProperty()->BoldOff();
	  if ( inFormat->fontItalic )
	    inTextActor->GetTextProperty()->ItalicOn();
	  else
	    inTextActor->GetTextProperty()->ItalicOff();
	}
      else if ( inFormat->fontFamily == VTK_COURIER )
	{
	  inTextActor->GetTextProperty()->SetFontFamilyToCourier();
	  if ( inFormat->fontBold )
	    inTextActor->GetTextProperty()->BoldOn();
	  else
	    inTextActor->GetTextProperty()->BoldOff();
	  if ( inFormat->fontItalic )
	    inTextActor->GetTextProperty()->ItalicOn();
	  else
	    inTextActor->GetTextProperty()->ItalicOff();


	}
      else if ( inFormat->fontFamily == VTK_TIMES )
	{
	  inTextActor->GetTextProperty()->SetFontFamilyToTimes();
	  if ( inFormat->fontBold )
	    inTextActor->GetTextProperty()->BoldOn();
	  else
	    inTextActor->GetTextProperty()->BoldOff();
	  if ( inFormat->fontItalic )
	    inTextActor->GetTextProperty()->ItalicOn();
	  else
	    inTextActor->GetTextProperty()->ItalicOff();

	}
      else if(  inFormat->fontFamily == VTK_GREEK)
	{
	  inTextActor->GetTextProperty()->SetFontFamilyToGreek();
	  inTextActor->GetTextProperty()->BoldOff();
	  inTextActor->GetTextProperty()->ItalicOff();
	}

      inTextActor->GetTextProperty()->SetFontSize(inFormat->size);
      inTextActor->GetTextProperty()->SetColor(inFormat->red,inFormat->green,inFormat->blue );

      
      if ( inFormat->super )
	{
	  AddFormattedText(inFormat->super,inTextActor->GetSuperTextActorExtra(),false);
	};

      if ( inFormat->sub )
	{
	  AddFormattedText(inFormat->sub,inTextActor->GetSubTextActorExtra(),false);
	};
      
      if ( inFormat->next )
	{
	  AddFormattedText(inFormat->next,inTextActor->GetNextTextActorExtra(),false);
	};
      
      inTextActor->Modified();
    };
};

void vtkTextActorExtra::setBBoxColor(double red, double green, double blue)
{
   if (this->boxDataActor)
      this->boxDataActor->GetProperty()->SetColor(red,green,blue);
};

