// 
//  Copyright (C) 2005-2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef __vtkTextActorExtra_hh
#define __vtkTextActorExtra_hh

#include <list>


#include "vtkActor2D.h"
#include "vtkPolyData.h"
#include "vtkPolyDataMapper2D.h"

using namespace std;

// We need to include vtkTextMapper here otherwise we have an ambiguous
// case of vtkMapper2D or vtkTextMapper in SetMapper(vtkTextMapper *mapper);
// - two members with identical prototypes!
class vtkTextProperty;
class vtkOpenGLFreeTypeTextMapperExtra;


class vtkTextActorFormat
{

public:
  vtkTextActorFormat();
  vtkTextActorFormat(const vtkTextActorFormat& inFormat);
  virtual ~vtkTextActorFormat();
  vtkTextActorFormat& operator = (const vtkTextActorFormat& inFormat);


  string text;
  int  fontFamily;
  int fontBold;
  int fontItalic;
  string fontFile;
  int size;
  double red;
  double green;
  double blue;
  vtkTextActorFormat* super;
  vtkTextActorFormat* next;
  vtkTextActorFormat* sub;
};



class vtkTextActorExtra : public vtkActor2D
{
public:

  vtkTypeRevisionMacro(vtkTextActorExtra,vtkActor2D);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Instantiate object with a rectangle in normaled view coordinates
  // of (0.2,0.85, 0.8, 0.95).
  static vtkTextActorExtra *New();

  // Description:
  // Shallow copy of this text actor. Overloads the virtual
  // vtkProp method.
  void ShallowCopy(vtkProp *prop);


  // Description:
  // Set the text string to be displayed. "\n" is recognized
  // as a carriage return/linefeed (line separator).
  // Convenience method to the underlying mapper

  void SetInput(const char* inText);
  void SetInput(vtkTextActorFormat* inFormat);
  const char* GetInput ();
  vtkTextProperty* GetTextProperty();

  vtkTextActorExtra* GetSuperTextActorExtra();
  vtkTextActorExtra* GetSubTextActorExtra();
  vtkTextActorExtra* GetNextTextActorExtra();


  void GetSize(vtkViewport* viewport,double outSize[2]);
  void GetBBox(vtkViewport* viewport,double& llx,double& lly,double& urx, double& ury);
  void BBoxRecursive(double& llx,double& lly,double& urx, double& ury);
  void GetAlignmentRange(vtkViewport *inViewport,double outXRange[2],double outYRange[2]);

  
  double GetItalicAngle();

  void Clear();
  void SetSuperscriptOffset(double inOffset);
  void SetSuperscriptSize(double inSize);
  void SetSubscriptOffset(double inOffset);
  void SetSubscriptSize(double inSize);

  void setBBoxColor(double red, double green, double blue);
  void SetdrawBBox(unsigned int inVal){this->drawBBox=inVal; this->Modified();};
  unsigned int GetdrawBBox(){ return this->drawBBox;};

  void SetRootFlag(bool inFlag){this->rootFlag=inFlag;};

//BTX
  // Description:
  // WARNING: INTERNAL METHOD - NOT INTENDED FOR GENERAL USE
  // DO NOT USE THIS METHOD OUTSIDE OF THE RENDERING PROCESS.
  // Release any graphics resources that are being consumed by this actor.
  // The parameter window could be used to determine which graphic
  // resources to release.
  virtual void ReleaseGraphicsResources(vtkWindow *);

  // Description:
  // WARNING: INTERNAL METHOD - NOT INTENDED FOR GENERAL USE
  // DO NOT USE THIS METHOD OUTSIDE OF THE RENDERING PROCESS.
  // Draw the text actor to the screen.
  int RenderOpaqueGeometry(vtkViewport* viewport);
  int RenderOverlay(vtkViewport* viewport);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport *) {return 0;}

  // Description:
  // Does this prop have some translucent polygonal geometry?
  virtual int HasTranslucentPolygonalGeometry();

//ETX

  void SetVertAlignmentOffset(double inAlignment);
  void SetHorizAlignmentOffset(double inAlignment);
  double GetVertAlignmentOffset(){  return this->vertAlignmentOffset;};
  double GetHorizAlignmentOffset(){  return this->horizAlignmentOffset;};

protected:
  // Description:
  // Hide access methods which use superclass vtkMapper2D and not vtkTextMapper

   vtkTextActorExtra();
  ~vtkTextActorExtra();
  void BuildText(vtkViewport* viewport,double inXPos,double inYPos);


  vtkActor2D* textActor;
  vtkOpenGLFreeTypeTextMapperExtra* textMapper;
  vtkTextActorExtra* superText;
  vtkTextActorExtra* subText;
  vtkTextActorExtra* nextText;

  double superscriptSize;
  double superscriptOffset;
  double subscriptSize;
  double subscriptOffset;

  vtkTimeStamp  BuildTime;
  double LastSize[2];
  double LastOrigin[2];

  unsigned int drawBBox;

  vtkPolyData* boxData;
  vtkPolyDataMapper2D* boxDataMapper;
  vtkActor2D* boxDataActor;

  double vertAlignmentOffset;
  double horizAlignmentOffset;

  bool rootFlag;

  double plotPosition[2];

private:
  vtkTextActorExtra(const vtkTextActorExtra&);  // Not implemented.
  void operator=(const vtkTextActorExtra&);  // Not implemented.

  void GetSizeRange(vtkViewport *inViewport,double inXPos,double inYPos,double outXSize[2],double outYRange[2],double outBBoxXRange[2],double outBBoxYRange[2]);

  static void Order(double inTab[2]);
  static void RotatePosition(double inRotationOrigin[2],double inAngleRad,double inPosition[2],double outPosition[2]);

  void AddFormattedText(vtkTextActorFormat* inFormat,vtkTextActorExtra* inTextActor,bool inRootFlag=true);


  
};


#endif

