// 
//  Copyright (C) 2004-2009,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ChipsDefaults.hh"
#include "ChipsCmdManager.hh"
#include "ChipsPrefCommand.hh"

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

extern ChipsCmdManager* cmd_manager;

ChipsDefaults::ChipsDefaults(bool load)
{
   version_exclude = 0; 
   curr = &defvals; 
   obj_p = buf; 

   load_overrides = true;

   // initialize default values 
   defvals.windowdisplay = true;
   defvals.windowsmoothing = false;
   defvals.windowheight = V_DEF_WINDOW_DIM;
   defvals.windowwidth = V_DEF_WINDOW_DIM;
   defvals.windowscaleheight = V_DEF_WINDOW_SCALE;
   defvals.windowscalewidth = V_DEF_WINDOW_SCALE;
   defvals.windowunits = chips_pixels;
   defvals.windowgeom = ""; 
   defvals.windowstem = "win"; 
   defvals.windowprefix = "chips - "; 
   defvals.windowid = ""; 
   defvals.windowcolor = "default"; 
   defvals.printmethod = "framebuffer";
   defvals.gridscale = V_GRID_SCALE_QTR_INCH; 
   defvals.gridcolor = "default";
   defvals.gridlinestyle = chips_longdash;
   defvals.showlayout = defvals.showgrid = false;
   defvals.windowfocus = false;
   defvals.layoutthickness = defvals.gridthickness = 1.0; 
   defvals.layoutcolor = "default"; 
   defvals.layoutlinestyle = chips_shortdash;
   defvals.frameaspect = true;
   defvals.framescale = true;
   defvals.framemode = V_FRAMEMODE_PLACED; 
   defvals.framestem = "frm"; 
   defvals.frameid = ""; 
   defvals.framecolor = "default"; 
   defvals.framebordercolor = "default"; 
   defvals.framebordervis = false;
   defvals.frametransparent = false;
   defvals.plotstem = "plot"; 
   defvals.plotid = ""; 
   defvals.plotlmargin = 0.15; 
   defvals.plotrmargin = 0.1; 
   defvals.plottmargin = 0.1; 
   defvals.plotbmargin = 0.15; 
   defvals.titledepth = V_USE_DEFAULT_DEPTH;
   defvals.titlexpos = 0.5;
   defvals.titleypos = 15;
   defvals.titlecolor = "default";  
   defvals.titleangle = 0.0; 
   defvals.titlehalign = AUTO_ALIGN;
   defvals.titlevalign = AUTO_ALIGN; 
   defvals.titlesize = 16; 
   defvals.titlefont = "helvetica";
   defvals.titlelabel = ""; 
   defvals.titlefontstyle = chips_normal; 
   defvals.plotaspectw = defvals.plotaspectw = 0;
   defvals.plotratio = "";
   defvals.axisxstem = "ax"; 
   defvals.axisystem = "ay"; 
   defvals.axisxid = ""; 
   defvals.axisyid = ""; 
   defvals.axisxlabel = ""; 
   defvals.axisylabel = ""; 
   defvals.axisdepth = V_USE_DEFAULT_DEPTH;
   defvals.axisstyle = chips_closed;
   defvals.cornerstyle = chips_miter;
   defvals.axiscolor = "default"; 
   defvals.axis_automin = true;
   defvals.axis_automax = true;
   defvals.axis_pad = 0.05; 
   defvals.axisthickness = 1.0; 
   defvals.axis_offsetpar = 0; 
   defvals.axis_offsetperp = 40; 
   defvals.axlabhalign = 0.5; 
   defvals.axlabvalign = 0.5; 
   defvals.axlabxangle = 0.0;
   defvals.axlabyangle = 90.0;
   defvals.axlabcolor = "default";  
   defvals.axlabsize = 14; 
   defvals.axlabfont = "helvetica";
   defvals.axlabfontstyle = chips_normal; 
   defvals.majorgridvisible = false;
   defvals.majorgridcolor = "default";
   defvals.majorgridstyle = chips_shortdash;
   defvals.majorgridthickness = 1.0; 
   defvals.minorgridvisible = false;
   defvals.minorgridcolor = "default";
   defvals.minorgridstyle = chips_dot;
   defvals.minorgridthickness = 1.0; 
   defvals.tickformat = "%g"; 
   defvals.ticklabstyle = chips_outside; 
   defvals.ticklabcolor = "default";  
   defvals.ticklabangle = 0.0; 
   defvals.ticklabhalign = AUTO_ALIGN; 
   defvals.ticklabvalign = AUTO_ALIGN; 
   defvals.ticklabsize = 12; 
   defvals.ticklaboffset = 6; 
   defvals.ticklabfont = "helvetica";
   defvals.ticklabfontstyle = chips_normal; 
   defvals.majortickmode = "limits";
   defvals.majortickinterval = 10; 
   defvals.majortickcount = 6; 
   defvals.majortickthickness = 1.0;
   defvals.majorticklength = 4;
   defvals.majortickcolor = "default";  
   defvals.majortickstyle = chips_inside;
   defvals.minortickmode = "nice";
   defvals.minortickinterval = 5; 
   defvals.minortickcount = 4; 
   defvals.minortickthickness = 1.0;
   defvals.minorticklength = 2;
   defvals.minortickcolor = "default";  
   defvals.minortickstyle = chips_inside;
   defvals.majortickvisible = true; 
   defvals.minortickvisible = true; 
   defvals.pointstem = "pnt"; 
   defvals.pointid = ""; 
   defvals.pointangle = 0.0; 
   defvals.pointstyle = chips_cross;
   defvals.pointcolor = "default"; 
   defvals.pointfill = true;
   defvals.pointsize = V_SIZE_DEF_VAL;
   defvals.pointdepth = V_USE_DEFAULT_DEPTH;
   defvals.imagestem = "image"; 
   defvals.imageid = ""; 
   defvals.imagedepth = V_USE_DEFAULT_DEPTH; 
   defvals.imagecmap = ((V_COLORMAP_TYPE)chips_grayscale); 
   defvals.image_interp = chips_neighbor;
   defvals.imagealpha[0] = defvals.imagealpha[1] = -1.0;
   defvals.imagethreshold[0] = defvals.imagethreshold[1] = 0.0;
   defvals.imageinvertcmap = false;
   defvals.imagecmapsize = 256;
   defvals.imagecmapinterp = true;
   defvals.imagescalechannels = true;
   
   defvals.linestem = "line"; 
   defvals.lineid = ""; 
   defvals.linestyle = chips_solid;
   defvals.linecolor = "default";
   defvals.linethickness = 1.0; 
   defvals.linedepth = V_USE_DEFAULT_DEPTH;
   defvals.lineextend = chips_extend_none;
   defvals.curvestem = "crv"; 
   defvals.curveid  = ""; 
   defvals.curvesymbolstyle = chips_cross;
   defvals.curvesymbolcolor = "default";
   defvals.curvesymbolfill = true;
   defvals.curvesymbolsize = V_SIZE_DEF_VAL;
   defvals.curvesymbolangle = 0.0;
   defvals.curvelinestyle = chips_solid;
   defvals.curvelinecolor = "default";
   defvals.curvelinethickness = 1.0;
   defvals.curvebaddata = chips_omit;
   defvals.errup = defvals.errdown = defvals.errleft = defvals.errright = true; 
   defvals.errstyle = chips_bar; 
   defvals.errcolor = "default"; 
   defvals.errcaplen = 10;
   defvals.errthickness = 1.0;
   defvals.limitlength = 0.05;
   defvals.limitoverride = true;
   defvals.curvedepth = V_USE_DEFAULT_DEPTH;
   defvals.histstem = "hist";
   defvals.histid = ""; 
   defvals.histsymbolstyle = chips_none;
   defvals.histsymbolcolor = "default";
   defvals.histsymbolfill = false;
   defvals.histsymbolsize = V_SIZE_DEF_VAL;
   defvals.histsymbolangle = 0.0;
   defvals.histlinestyle = chips_solid;
   defvals.histlinecolor = "default";
   defvals.histlinethickness = 1.0;
   defvals.histerrup = defvals.histerrdown = true;
   defvals.histdropline = false; 
   defvals.histerrstyle = chips_bar;
   defvals.histerrcolor = "default";
   defvals.histerrcaplen = 10;
   defvals.histerrthickness = 1.0;
   defvals.histdepth = V_USE_DEFAULT_DEPTH;
   defvals.histfillcolor = "default";
   defvals.histfillopacity = 1.0;
   defvals.histfillstyle = chips_nofill;

   defvals.contourstem = "ctr";
   defvals.contourid = ""; 
   defvals.contourcolor = "default";
   defvals.contourstyle = chips_solid;
   defvals.contourthickness = 1.0;
   defvals.contourlevels.clear ();
   defvals.contournumlevels = 5; 
   defvals.contouralgorithm = chips_marching;
   defvals.contourmode = chips_nice;
   defvals.contourinterval = 10; 
   defvals.contourdepth = V_USE_DEFAULT_DEPTH;

   defvals.cbarstem = "cbar";
   defvals.cbarid = "";
   defvals.cbarwidth = .05;
   defvals.cbarlength = 1;
   defvals.cbarbordervis = true;
   defvals.cbardepth = V_USE_DEFAULT_DEPTH;
   defvals.cbarhalign = 0.5;
   defvals.cbarvalign = 0.5;
   defvals.cbarorient = 0;   // 0 = horizontal 1 = vertical  
   defvals.cbartickmode = 2;  // 0 = none, 1 = ends, 2 = nice 
   defvals.cbartickstyle = chips_inside;
   defvals.cbartickcolor = "default"; 
   defvals.cbarticklength = 4;
   defvals.cbartickthickness = 1.0;
   defvals.cbartickloc = chips_outside; 
   defvals.cbarlabloc = chips_inside; 
   defvals.cbartickvisible = true;
   defvals.cbarticklabcolor = "default";
   defvals.cbarticklabangle = 0.0;
   defvals.cbarticklabhalign = AUTO_ALIGN;
   defvals.cbarticklabvalign = AUTO_ALIGN;
   defvals.cbarticklabsize = 12;
   defvals.cbarticklabvisible = true;
   defvals.cbarticklaboffset = 6;
   defvals.cbarticklabfont = "helvetica";
   defvals.cbarticklabfontstyle = chips_normal;
   defvals.cbarlabcolor = "default";
   defvals.cbarlabangle = 0.0;
   defvals.cbarlabhalign = AUTO_ALIGN;
   defvals.cbarlabvalign = AUTO_ALIGN;
   defvals.cbarlabsize = 12;
   defvals.cbarlabtext = "";
   defvals.cbarlabvisible = true;
   defvals.cbarlabfont = "helvetica";
   defvals.cbarlabfontstyle = chips_normal;
   defvals.cbaroffsetpar = 0;
   defvals.cbaroffsetperp = 5;

   defvals.labelstem = "lbl";
   defvals.labelid = "";
   defvals.labelcolor = "default";
   defvals.labelsize  = 12;
   defvals.labelfont  = "helvetica";
   defvals.labelstyle = chips_normal; 
   defvals.labelangle = 0.0; 
   defvals.labelhalign = 0.0; 
   defvals.labelvalign = 0.0; 
   defvals.labeldepth = V_USE_DEFAULT_DEPTH;

   defvals.legendtransparent = true;
   defvals.legendborders = true;
   defvals.legendcolor = "default";
   defvals.legendthickness = 1.0;
   defvals.legendstem = "key"; 
   defvals.legenddepth = V_USE_DEFAULT_DEPTH;

   defvals.regionstem = "reg";
   defvals.regionid = "";
   defvals.regionthickness = 1.0;
   defvals.regionstyle = chips_solid;
   defvals.regionfillstyle = chips_solidfill;
   defvals.regioncolor = "green";
   defvals.regionfillcolor = "green";
   defvals.regionangle = 0;
   defvals.regionopacity = 0.5; 
   defvals.regiondepth = V_USE_DEFAULT_DEPTH;

   defvals.exportclobber = false;
   defvals.exportfonts = true;
   defvals.exportorient = 0;
   defvals.exportformat = "ps"; 
   defvals.exportcolorscheme = chips_rgb; 
   defvals.exportdpi = 150;
   defvals.pagelmargin = 0.2;
   defvals.pagermargin = 0.2;
   defvals.pagetmargin = 0.2;
   defvals.pagebmargin = 0.2;
   defvals.pagewidth = 8.5;
   defvals.pageheight = 11;
   defvals.pageunits = chips_inch;
   defvals.pagescaleheight = 0;
   defvals.pagescalewidth = 0;
   defvals.fittopage = false;
   defvals.pageaspect = true;
   defvals.printdialog = false; 
   defvals.meta1 = "";

   defvals.toolbarvis = true; 
   defvals.navpanevis = true;
   defvals.infpanevis = true;
   defvals.proppanevis = true;
   defvals.shellpanevis = true;
   defvals.tbarmode = "pan";
   defvals.tbarstyle = "both";
   defvals.zoombtn = true;
   defvals.delbtn = true;
   defvals.guitips = true; 

   defaultfgps = "black";
   defaultbgps = "white";
   defaultfgdisp = "white";
   defaultbgdisp = "black";
   defaultdepth = V_DEFAULT_DEPTH; 
   defaultverb = 2;

   pickmask = KEY_PRESS|BTNS_DOWN; 
   defvals.pickcolor = "gold"; 
   infocurrent = false;
   infodepth = false; 
   infocoord = false; 
   min_version = CHIPS_VERSION; 

   allpropmask = 0;
   allbgcolor = ""; 
   allcolor = ""; 
   alllinestyle = chips_solid;
   allthickness = 1.0;
   alllength = 4; 
   allcount = 4; 
   allfont = "";
   allfontstyle = chips_normal;
   allfill = allvisible = allaspect = true; 
   allangle = 0.0; 
   alldepth = V_USE_DEFAULT_DEPTH;
   if (load)  load_file("", false);
}

ChipsDefaults::ChipsDefaults(ChipsStateStream& in)
{
   deserialize(in);
}

ChipsDefaults::~ChipsDefaults()
{

}

int ChipsDefaults::load_file(string in_fname, bool rec)
{
   int rval = ERR_STATUS_GOOD;
   char   linebuffer[PREF_MAX_LEN];

   if (in_fname.length())
      filename = in_fname;
   else
   {
      filename = getenv("HOME");
      if (filename.length()) filename.append("/");
      filename.append(".chips.rc");
      file_version = get_version_from_file(filename); 
      version_exclude = 1; 
   }

   ifstream *infile =  new ifstream(filename.c_str(), ios::in);
   if (infile->good())
   {
      int errs_found = 0;

      // treat entire pref file as a single undo
      if (rec && cmd_manager)   cmd_manager->open_block();

      // treat any error which occurred in the resource file
      // as a warning- we'll discard those settings and only set the
      // valid ones. 
      if (cmd_manager)  cmd_manager->set_default_err_type(1);

      while (infile->getline(linebuffer, PREF_MAX_LEN))
         if (parseLine(linebuffer, ':', rec) != 0) errs_found++;
      
      if (errs_found)
      {
         rval = ERR_IGNORE_RC_LINE;
	 if (cmd_manager)  cmd_manager->add_err(ERR_IGNORE_RC_LINE,"",1);
      }
      // reset default error type to 'error'
      if (cmd_manager)  cmd_manager->set_default_err_type();
      delete infile; 

      if (rec && cmd_manager)   cmd_manager->close_block();
   }
   else
   {
      string errstr = "Unable to load preferences from file: ";
      errstr.append(in_fname);
      if (cmd_manager)  cmd_manager->add_err(errstr);
      rval = ERR_PREF_LOAD;
   }
   version_exclude = 0; 
   return rval;
}

int ChipsDefaults::processLine(bool ignore_checks)
{
   int rval = ERR_STATUS_GOOD;

   if (!curr) 
      rval = ERR_STATUS_BAD; 
   else if (obj_p && prop_p && value_p)
   {
      istringstream value(value_p);
      double        dval = 0.0; 
      float         fval = 0.0; 
      int           ival = 0; 
      int           vlen = strlen(value_p); 
      V_LINE_TYPE   line;
      V_FILL_TYPE   fill; 
      V_AXISSTYLE_TYPE astyle; 
      V_SYMBOL_TYPE sym;
      V_UNITS_TYPE  units;
      V_ERRSTYLE_TYPE errs; 
      V_TICKSTYLE_TYPE tstyle; 
      V_TICKMODE_TYPE  tmode; 
      V_COLORMAP_TYPE cmap;
      V_INTERPOLATION_TYPE interp;

      // validate value based on property type
      // since we converted the string to lowercase, use standard strcmp
      if (!strcmp(prop_p, "color"))
      {
         if (c2_valid_color(value_p) || (strcmp(value_p, "default") == 0) ||
            (ignore_checks))   
         {
            if (!strcmp(obj_p, "window.grid")) 
               curr->gridcolor = value_p;
            else if (!strcmp(obj_p,"frame.border")||(!strcmp(obj_p, "frame.*")))
               curr->framebordercolor = value_p;
            else if (!strcmp(obj_p, "window.layout")) 
               curr->layoutcolor = value_p;
            else if (!strcmp(obj_p, "axis")) 
               curr->axiscolor = value_p;
            else if (!strcmp(obj_p, "line") || (!strcmp(obj_p, "line.*"))) 
               curr->linecolor = value_p;
            else if (!strcmp(obj_p, "point")|| (!strcmp(obj_p, "point.*"))) 
               curr->pointcolor = value_p;
            else if (!strcmp(obj_p, "curve.line")) 
               curr->curvelinecolor = value_p;
            else if (!strcmp(obj_p, "histogram.line")) 
               curr->histlinecolor = value_p;
            else if (!strcmp(obj_p, "contour") || (!strcmp(obj_p, "contour.*")))
               curr->contourcolor = value_p;
            else if (!strcmp(obj_p, "curve.symbol")) 
               curr->curvesymbolcolor = value_p;
            else if (!strcmp(obj_p, "histogram.symbol")) 
               curr->histsymbolcolor = value_p;
            else if (!strcmp(obj_p, "label") || (!strcmp(obj_p, "label.*"))) 
               curr->labelcolor = value_p;
            else if (!strcmp(obj_p, "legend") || (!strcmp(obj_p, "legend.*"))) 
               curr->legendcolor = value_p;
            else if (!strcmp(obj_p, "axis.majortick"))
               curr->majortickcolor = value_p;
            else if (!strcmp(obj_p, "axis.minortick"))
               curr->minortickcolor = value_p;
            else if (!strcmp(obj_p, "curve.err"))
               curr->errcolor = value_p;
            else if (!strcmp(obj_p, "histogram.err"))
               curr->histerrcolor = value_p;
            else if (!strcmp(obj_p, "plot.title") || (!strcmp(obj_p, "plot.*")))
               curr->titlecolor = value_p;
            else if (!strcmp(obj_p, "axis.label"))
               curr->axlabcolor = value_p;
            else if (!strcmp(obj_p, "axis.majorgrid"))
               curr->majorgridcolor = value_p;
            else if (!strcmp(obj_p, "axis.minorgrid"))
               curr->minorgridcolor = value_p;
            else if (!strcmp(obj_p, "axis.ticklabel"))
               curr->ticklabcolor = value_p;
            else if (!strcmp(obj_p, "region.edge"))
               curr->regioncolor = value_p;
            else if (!strcmp(obj_p, "region.fill"))
               curr->regionfillcolor = value_p;
            else if (!strcmp(obj_p, "histogram.fill"))
               curr->histfillcolor = value_p;
            else if (!strcmp(obj_p, "pick"))
               curr->pickcolor = value_p;
            else if (!strcmp(obj_p, "colorbar.label"))
               curr->cbarlabcolor = value_p;
            else if (!strcmp(obj_p, "colorbar.tick"))
               curr->cbartickcolor = value_p;
            else if (!strcmp(obj_p, "colorbar.ticklabel"))
               curr->cbarticklabcolor = value_p;
            else if (!strcmp(obj_p, "curve.*"))
               curr->curvesymbolcolor = curr->curvelinecolor = curr->errcolor = 
                  value_p;
            else if (!strcmp(obj_p, "axis.*"))
               curr->axiscolor =  curr->majortickcolor = curr->minortickcolor =
               curr->axlabcolor = curr->majorgridcolor = curr->minorgridcolor = 
               curr->ticklabcolor = value_p;
            else if  (!strcmp(obj_p, "histogram.*"))
               curr->histlinecolor = curr->histsymbolcolor = 
               curr->histerrcolor = curr->histfillcolor = value_p;
            else if (!strcmp(obj_p, "region.*"))
               curr->regioncolor = curr->regionfillcolor = value_p;
            else if (!strcmp(obj_p, "colorbar.*"))
               curr->cbarlabcolor = curr->cbarticklabcolor = 
               curr->cbartickcolor = value_p; 
            else if (!strcmp(obj_p, "*.fill"))
               curr->regionfillcolor = curr->histfillcolor = value_p; 
            else if (!strcmp(obj_p, "*.line"))
               curr->linecolor = curr->curvelinecolor = 
               curr->histlinecolor = value_p;
            else if (!strcmp(obj_p, "*.symbol"))
               curr->curvesymbolcolor = curr->histsymbolcolor = value_p;
            else if ((strcmp(obj_p, "*.*") == 0) || (strcmp(obj_p, "*") == 0))
            {
               curr->gridcolor = curr->framebordercolor = curr->layoutcolor = 
               curr->cbarlabcolor = curr->linecolor = curr->pointcolor = 
               curr->curvelinecolor = curr->histlinecolor = curr->contourcolor =
               curr->axiscolor = curr->histfillcolor = curr->histsymbolcolor =
               curr->labelcolor = curr->legendcolor = curr->majortickcolor = 
               curr->minortickcolor = curr->errcolor = curr->histerrcolor = 
               curr->titlecolor = curr->axlabcolor = curr->majorgridcolor =
               curr->minorgridcolor = curr->ticklabcolor = curr->regioncolor =
               curr->regionfillcolor = curr->curvesymbolcolor =
               curr->cbartickcolor = curr->cbarticklabcolor = value_p;
            }
            else if (!strcmp(obj_p, "all"))
            {
               allcolor = value_p;
               allpropmask = PREF_ALL_COLOR; 
            }
            else
	       rval = unknown_property_error();
         }
         else  rval = ERR_UNKNOWN_COLOR; // valid_color adds to err list 
      }
      else if (!strcmp(prop_p, "bgcolor"))
      {
         if (c2_valid_color(value_p) || (strcmp(value_p, "default") == 0) ||
            (ignore_checks))   
         {
            if (!strcmp(obj_p, "window") || !strcmp(obj_p, "window.*")) 
               curr->windowcolor = value_p;
            else if (!strcmp(obj_p, "frame") || !strcmp(obj_p, "frame.*")) 
               curr->framecolor = value_p;
            else if ((strcmp(obj_p, "*.*") == 0) || (strcmp(obj_p, "*") == 0))
               curr->windowcolor = curr->framecolor = value_p;
            else if (!strcmp(obj_p, "all"))
            {
               allbgcolor = value_p;
               allpropmask = PREF_ALL_BGCOLOR;
            }
	    else
	       rval = unknown_property_error();
         }
         else  rval = ERR_UNKNOWN_COLOR; // valid_color adds to err list
      }
      else if (!strcmp(prop_p, "thickness"))
      {
         if (vlen > 0)
         {
            value >> fval;
            if (value.fail())
	    {
	       rval = ERR_DBL_MIN;
	       string est = "The ";
	       est.append(obj_p);
	       est.append(".");
	       est.append(prop_p);
	       est.append(" value (");
	       est.append(value_p);
	       est.append(") must be a numeric value.");
	       if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
	    }
            else if (!strcmp(obj_p, "window.grid"))
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->gridthickness = fval;
	    }
            else if (!strcmp(obj_p, "window.layout")) 
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->layoutthickness = fval;
	    }
            else if (!strcmp(obj_p, "axis")) 
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->axisthickness = fval;
	    }
            else if (!strcmp(obj_p, "curve.line")) 
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->curvelinethickness = fval;
	    }
            else if (!strcmp(obj_p, "histogram.line")) 
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->histlinethickness = fval;
	    }
            else if (!strcmp(obj_p, "line") || !strcmp(obj_p, "line.*")) 
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->linethickness = fval;
	    }
            else if (!strcmp(obj_p, "contour") || !strcmp(obj_p, "contour.*")) 
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->contourthickness = fval;
	    }
            else if (!strcmp(obj_p, "legend") || !strcmp(obj_p, "legend.*")) 
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->legendthickness = fval;
	    }
            else if (!strcmp(obj_p, "axis.majortick"))
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->majortickthickness = fval;
	    }
            else if (!strcmp(obj_p, "axis.minortick"))
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->minortickthickness = fval;
	    }
            else if (!strcmp(obj_p, "curve.err"))
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->errthickness = fval;
	    }
            else if (!strcmp(obj_p, "histogram.err"))
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->histerrthickness = fval;
	    }
            else if (!strcmp(obj_p, "axis.majorgrid"))
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->majorgridthickness = fval;
	    }
            else if (!strcmp(obj_p, "axis.minorgrid"))
	    {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
	       else
                  curr->minorgridthickness = fval;
	    }
            else if (!strcmp(obj_p, "region.edge") ||
                     !strcmp(obj_p, "region.*"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
               else
                  curr->regionthickness = fval;
            }
            else if (!strcmp(obj_p, "colorbar.tick") ||
                     !strcmp(obj_p, "colorbar.*"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
               else
                  curr->cbartickthickness = fval;
            }
            else if (!strcmp(obj_p, "all"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
               else
               {
                  allthickness = fval;
                  allpropmask = PREF_ALL_THICKNESS; 
               }
            }
            else if (!strcmp(obj_p, "axis.*"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                   rval = thickness_value_error(fval);
               else
               {
                  curr->axisthickness = curr->majortickthickness =  
                  curr->minortickthickness = curr->majorgridthickness = 
                  curr->minorgridthickness = fval;   
               }
            }
            else if (!strcmp(obj_p, "curve.*"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                   rval = thickness_value_error(fval);
               else
                  curr->curvelinethickness = curr->errthickness = fval; 
            }
            else if (!strcmp(obj_p, "window.*"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                   rval = thickness_value_error(fval);
               else
                  curr->gridthickness = curr->layoutthickness = fval; 
            }
            else if (!strcmp(obj_p, "histogram.*"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                   rval = thickness_value_error(fval);
               else
                  curr->histlinethickness = curr->histerrthickness = fval; 
            }
            else if (!strcmp(obj_p, "*.line"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
               else
                  curr->curvelinethickness = curr->histlinethickness = 
                  curr->linethickness = fval;
            }
            else if (!strcmp(obj_p, "*.err"))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                  rval = thickness_value_error(fval);
               else
                  curr->errthickness = curr->histerrthickness = fval;
            }
            else if ((strcmp(obj_p, "*.*") == 0) || (strcmp(obj_p, "*") == 0))
            {
               if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
                   rval = thickness_value_error(fval);
               else
               {
                  curr->gridthickness = curr->layoutthickness =
                  curr->axisthickness = curr->majortickthickness =  
                  curr->minortickthickness = curr->majorgridthickness = 
                  curr->minorgridthickness =   
                  curr->curvelinethickness = curr->errthickness =  
                  curr->histlinethickness = curr->histerrthickness =  
                  curr->linethickness = curr->contourthickness = 
                  curr->contourthickness = curr->legendthickness = 
                  curr->regionthickness = curr->cbartickthickness = fval; 
               }
            }
            else  rval = unknown_property_error(); 
         }
         else   rval = no_property_value_error();
      }
      else if (!strcmp(prop_p, "size"))
      {
         if (vlen > 0)
         {
            value >> ival;
            if (value.fail())
	    {
	       rval = ERR_INT_MIN;
	       string est = "The ";
	       est.append(obj_p);
	       est.append(".");
	       est.append(prop_p);
	       est.append(" value (");
	       est.append(value_p);
	       est.append(") must be an integer.");
	       if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
	    }
            else if (!strcmp(obj_p, "plot.title") || !strcmp(obj_p, "plot.*"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else           curr->titlesize = ival;
            }
            else if (!strcmp(obj_p, "axis.label"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else           curr->axlabsize = ival;
            }
            else if (!strcmp(obj_p, "label") || !strcmp(obj_p, "label.*"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else           curr->labelsize = ival;
            }
            else if (!strcmp(obj_p, "axis.ticklabel"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else           curr->ticklabsize = ival;
            }
            else if (!strcmp(obj_p, "curve.symbol") || !strcmp(obj_p, "curve.*"))
	    {
               if ((ival < V_SIZE_MIN_VAL) || (ival > V_SIZE_MAX_VAL)) 
                  rval = data_range_error(ival, V_SIZE_MIN_VAL, V_SIZE_MAX_VAL);
	       else 
                  curr->curvesymbolsize = ival;
	    }
            else if (!strcmp(obj_p, "histogram.symbol") || 
                     !strcmp(obj_p, "histogram.*"))
	    {
               if ((ival < V_SIZE_MIN_VAL) || (ival > V_SIZE_MAX_VAL)) 
                  rval = data_range_error(ival, V_SIZE_MIN_VAL, V_SIZE_MAX_VAL);
	       else 
                  curr->histsymbolsize = ival;
	    }
            else if (!strcmp(obj_p, "point") || !strcmp(obj_p, "point.*"))
	    {
               if ((ival < V_SIZE_MIN_VAL) || (ival > V_SIZE_MAX_VAL)) 
                  rval = data_range_error(ival, V_SIZE_MIN_VAL, V_SIZE_MAX_VAL);
	       else 
                  curr->pointsize = ival;
	    }
            else if (!strcmp(obj_p, "colorbar.ticklabel"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else           curr->cbarticklabsize = ival;
            }
            else if (!strcmp(obj_p, "colorbar.label"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else           curr->cbarlabsize = ival;
            }
            else if (!strcmp(obj_p, "colorbar.*"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else    curr->cbarticklabsize = curr->cbarlabsize = ival;
            }
            else if (!strcmp(obj_p, "axis.*"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else  curr->axlabsize = curr->ticklabsize = ival;
            }
            else if ((strcmp(obj_p, "*.*") == 0) || (strcmp(obj_p, "*") == 0))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else    curr->cbarticklabsize = curr->cbarlabsize = ival;
            }
            else if (!strcmp(obj_p, "*.symbol"))
            {
               if ((ival < V_SIZE_MIN_VAL) || (ival > V_SIZE_MAX_VAL))
                  rval = data_range_error(ival, V_SIZE_MIN_VAL, V_SIZE_MAX_VAL);
               else  curr->curvesymbolsize = curr->histsymbolsize = ival;
            }
            else if (!strcmp(obj_p, "*.label"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else   curr->axlabsize=curr->labelsize=curr->cbarlabsize = ival;
            }
            else if (!strcmp(obj_p, "*.ticklabel"))
            {
               if (ival < 0)  rval = negative_value_error(ival);
               else  curr->ticklabsize = curr->cbarticklabsize = ival;
            }
            else  rval = unknown_property_error(); 
         }
         else   rval = no_property_value_error();
      }
      else if (!strcmp(prop_p, "style"))
      {
         if (!strcmp(obj_p, "line"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "line.style"))
                == ERR_STATUS_GOOD)
               curr->linestyle = line; 
         }
         else if (!strcmp(obj_p, "window.grid"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "window.grid.style"))
                 == ERR_STATUS_GOOD)
               curr->gridlinestyle = line; 
         }
         else if (!strcmp(obj_p, "window.layout"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "window.layout.line"))
                == ERR_STATUS_GOOD)
               curr->layoutlinestyle = line; 
         }
         else if (!strcmp(obj_p, "curve.line"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "curve.line.style"))
                == ERR_STATUS_GOOD)
               curr->curvelinestyle = line; 
         }
         else if (!strcmp(obj_p, "histogram.line"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "histogram.line.style"))
                == ERR_STATUS_GOOD)
               curr->histlinestyle = line; 
         }
         else if (!strcmp(obj_p, "contour"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "contour.line.style"))
                == ERR_STATUS_GOOD)
               curr->contourstyle = line; 
         }
         else if (!strcmp(obj_p, "curve.symbol"))
         {
            if ((rval = c2_string_to_symbol(value_p, &sym, "curve.symbol.style"))
                 == ERR_STATUS_GOOD)
               curr->curvesymbolstyle = sym; 
         }
         else if (!strcmp(obj_p, "histogram.symbol"))
         {
            if ((rval = c2_string_to_symbol(value_p, &sym, "histogram.symbol.style"))
                 == ERR_STATUS_GOOD)
               curr->histsymbolstyle = sym; 
         }
         else if (!strcmp(obj_p, "curve.err"))
         {
            if ((rval= c2_string_to_errstyle(value_p, &errs, "curve.err.style"))
                == ERR_STATUS_GOOD)
               curr->errstyle = errs; 
         }
         else if (!strcmp(obj_p, "histogram.err"))
         {
            if ((rval=string_to_v_errstyle(value_p, &errs)) == ERR_STATUS_GOOD)
               curr->histerrstyle = errs; 
         }
         else if (!strcmp(obj_p, "histogram.fill"))
         {
            if ((rval = c2_string_to_fill(value_p, &fill, "histogram.fill.style"))
                == ERR_STATUS_GOOD)
               curr->histfillstyle = fill;
         }
         else if (!strcmp(obj_p, "point"))
         {
            if ((rval = c2_string_to_symbol(value_p, &sym, "point.style")) ==
                ERR_STATUS_GOOD)
               curr->pointstyle = sym; 
         }
         else if (!strcmp(obj_p, "plot"))
         {
            if ((rval = c2_string_to_axisstyle(value_p, &astyle, "plot.style"))
                == ERR_STATUS_GOOD)
               curr->axisstyle = astyle; 
         }
         else if (!strcmp(obj_p, "plot.corner"))
         {
            V_CORNERSTYLE_TYPE cornstyle; 
            
            if ((rval = string_to_v_cornerstyle(value_p, &cornstyle)) ==
                ERR_STATUS_GOOD)
               curr->cornerstyle = cornstyle; 
            else if (rval == ERR_BAD_CORNERSTYLE)
               data_subset_error(value_p, "[miter|bevel|fill|notch]");
         }
         else if (!strcmp(obj_p, "axis.majortick"))
         {
            if ((rval = c2_string_to_tickstyle(value_p, &tstyle, 
                "axis.majortick.style")) == ERR_STATUS_GOOD)
               curr->majortickstyle = tstyle;
         }
         else if (!strcmp(obj_p, "axis.minortick"))
         {
            if ((rval = c2_string_to_tickstyle(value_p, &tstyle, 
                "axis.minortick.style")) == ERR_STATUS_GOOD)
               curr->minortickstyle = tstyle;
         }
         else if (!strcmp(obj_p, "axis.majorgrid"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "axis.majorgrid.style"))
                == ERR_STATUS_GOOD)
               curr->majorgridstyle = line;
         }
         else if (!strcmp(obj_p, "axis.minorgrid"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "axis.minorgrid.style"))
                == ERR_STATUS_GOOD)
               curr->minorgridstyle = line;
         }
         else if (!strcmp(obj_p, "axis.ticklabel"))
         {
            if ((rval = string_to_v_tickstyle(value_p, &tstyle)) == ERR_STATUS_GOOD)
            {
               if ((tstyle == chips_inside) || (tstyle == chips_outside))
                  curr->ticklabstyle = tstyle;
               else
               {
                  rval = data_subset_error(value_p, "[inside|outside]");
               }
            }
         }
         else if (!strcmp(obj_p, "region.edge"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "region.edge.style"))
                == ERR_STATUS_GOOD)
               curr->regionstyle = line;
         }
         else if (!strcmp(obj_p, "region.fill"))
         {
            if ((rval = c2_string_to_fill(value_p, &fill, "region.fill.style"))
                == ERR_STATUS_GOOD)
               curr->regionfillstyle = fill;
         }
         else if (!strcmp(obj_p, "colorbar.tick"))
         {
            if ((rval = string_to_v_tickstyle(value_p, &tstyle)) == ERR_STATUS_GOOD)
            {
               if ((tstyle == chips_inside) || (tstyle == chips_outside))
                  curr->cbartickstyle = tstyle;
               else
               {
                  rval = data_subset_error(value_p, "[inside|outside]");
               }
            }
         }
         else if (!strcmp(obj_p, "gui.toolbar"))
         {
            if (!chips_strcmp(value_p, "icons"))
               curr->tbarstyle = "icons";
            else if (!chips_strcmp(value_p, "labels"))
               curr->tbarstyle = "labels";
            else if (!chips_strcmp(value_p, "both"))
               curr->tbarstyle = "both";
            else
            {
               string est = "The gui.toolbar.style value (";
               est.append(value_p);
               est.append(") must be one of [icons|labels|both]");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval=ERR_DATA_SUBSET,"",-1);
            }
         }
         else if (!strcmp(obj_p, "*.line"))
         {
            if ((rval = c2_string_to_line(value_p, &line, "*.line.style"))
               == ERR_STATUS_GOOD)
            {
               curr->linestyle = line;
               curr->curvelinestyle = line;
               curr->histlinestyle = line;
               curr->contourstyle = line;
            }
         }
         else if (!strcmp(obj_p, "*.fill"))
         {
            if ((rval = c2_string_to_fill(value_p, &fill, "*.fill.style"))
                == ERR_STATUS_GOOD)
            {
               curr->histfillstyle = fill;
               curr->regionfillstyle = fill;
            }
         }
         else if (!strcmp(obj_p, "*.symbol"))
         {
            if ((rval = c2_string_to_symbol(value_p, &sym, "*.symbol.style"))
                 == ERR_STATUS_GOOD)
            {
               curr->curvesymbolstyle = sym;
               curr->histsymbolstyle = sym;
            }
         }
         else  rval = unknown_property_error(); 

         if (rval == ERR_RETURN_PTR_NULL)
            if (cmd_manager)  cmd_manager->add_err(ERR_RETURN_PTR_NULL);
      }
      else if (!strcmp(prop_p, "depth"))
      {
         if (vlen > 0)
         {
            if (!strcmp(obj_p, "info"))
               rval = set_boolean_value(infodepth, value_p);
	    else
	    {
               if (chips_strcmp(value_p, "default") == 0)
                  ival = V_USE_DEFAULT_DEPTH;
               else 
                  value >> ival;
	       if (value.fail())
	       {
	          rval = ERR_STR_SUBSET;  
                  if (cmd_manager)  cmd_manager->add_err(rval, "", -1, "depth", 
	             value_p, "'default'|0|<positive integer>");
	       }
               else if (!strcmp(obj_p, "plot.title") || 
                        !strcmp(obj_p, "plot.*")||!strcmp(obj_p, "*.title"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->titledepth = ival;
               else if (!strcmp(obj_p, "curve"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->curvedepth = ival;
               else if (!strcmp(obj_p, "image"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->imagedepth = ival;
               else if (!strcmp(obj_p, "contour"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->contourdepth = ival;
               else if (!strcmp(obj_p, "histogram"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->histdepth = ival;
               else if (!strcmp(obj_p, "axis"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->axisdepth = ival;
               else if (!strcmp(obj_p, "line"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->linedepth = ival;
               else if (!strcmp(obj_p, "point"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->pointdepth = ival;
               else if (!strcmp(obj_p, "label"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->labeldepth = ival;
               else if (!strcmp(obj_p, "legend"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->legenddepth = ival;
               else if (!strcmp(obj_p, "region"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->regiondepth = ival;
               else if (!strcmp(obj_p, "colorbar"))
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                     curr->cbardepth = ival;
               else if (!strcmp(obj_p, "all"))
               {
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                  {
                     alldepth = ival;
                     allpropmask = PREF_ALL_DEPTH;
                  }
               }
               else if (!strcmp(obj_p, "default"))
               {
                  if (ival < 0) 
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else if (ival == V_USE_DEFAULT_DEPTH)
                  {
                     rval = ERR_DEF_DEPTH;
                     if (cmd_manager) cmd_manager->add_err(ERR_DEF_DEPTH); 
                  }
                  else
                     defaultdepth = ival;
               }
               else if (!strcmp(obj_p, "*"))
               {
                  if (ival < 0)
                     rval = negative_value_error(ival, ERR_NEG_DEPTH);
                  else
                  {
                     curr->titledepth = curr->curvedepth = curr->imagedepth =
                     curr->contourdepth = curr->labeldepth = curr->pointdepth =
                     curr->legenddepth = curr->regiondepth = curr->cbardepth =
                     curr->linedepth = curr->axisdepth = curr->histdepth = ival;
                  }
               }
               else  rval = unknown_property_error(); 
	    }
         }
         else   rval = no_property_value_error();
      }
      else if (!strcmp(prop_p, "units"))
      {
         if (!strcmp(obj_p, "window"))
         {
            if ((rval = c2_string_to_units(value_p, &units, "window.units")) 
                == ERR_STATUS_GOOD)
               curr->windowunits = units;
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(prop_p, "fill"))
      {
         if (!strcmp(obj_p, "point"))
         {
            rval = set_boolean_value(curr->pointfill, value_p);
         }
         else if (!strcmp(obj_p, "curve.symbol"))
         {
            rval = set_boolean_value(curr->curvesymbolfill, value_p);
         }
         else if (!strcmp(obj_p, "histogram.symbol"))
         {
            rval = set_boolean_value(curr->histsymbolfill, value_p);
         }
         else if (!strcmp(obj_p, "*.symbol"))
         {
            if ((rval = set_boolean_value(curr->histsymbolfill, value_p)) ==
                ERR_STATUS_GOOD)
               curr->curvesymbolfill = curr->histsymbolfill;
         }
         else if (!strcmp(obj_p, "*.("))
         {
            if ((rval = set_boolean_value(curr->histsymbolfill, value_p)) ==
                ERR_STATUS_GOOD)
               curr->pointfill = curr->curvesymbolfill = curr->histsymbolfill;
         }
         else if (!strcmp(obj_p, "all"))
         {
            if ((rval = set_boolean_value(allfill, value_p)) == ERR_STATUS_GOOD)
               allpropmask = PREF_ALL_FILL; 
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(prop_p, "id"))
      {
         if (chips_strcmp(value_p, "all") == 0) 
         {
            rval = ERR_USED_ALL_AS_ID; 
            if (cmd_manager)  cmd_manager->add_err(rval);
         }
         else if (!strcmp(obj_p, "window"))
            curr->windowid = value_p;
         else if (!strcmp(obj_p, "frame"))
            curr->frameid = value_p;
         else if (!strcmp(obj_p, "plot"))
            curr->plotid = value_p;
         else if (!strcmp(obj_p, "axis.x"))
            curr->axisxid = value_p;
         else if (!strcmp(obj_p, "axis.y"))
            curr->axisyid = value_p;
         else if (!strcmp(obj_p, "axis"))
         {
            if (curr->axisxid.length() == 0)  curr->axisxid = value_p;
            if (curr->axisyid.length() == 0)  curr->axisyid = value_p;
         }
         else if (!strcmp(obj_p, "curve"))
            curr->curveid = value_p;
         else if (!strcmp(obj_p, "contour"))
            curr->contourid = value_p;
         else if (!strcmp(obj_p, "region"))
            curr->regionid = value_p;
         else if (!strcmp(obj_p, "histogram"))
            curr->histid = value_p;
         else if (!strcmp(obj_p, "label"))
            curr->labelid = value_p;
         else if (!strcmp(obj_p, "line"))
            curr->lineid = value_p;
	 else if (!strcmp(obj_p, "point"))
	    curr->pointid = value_p;
         else if (!strcmp(obj_p, "image"))
            curr->imageid = value_p;
         else if (!strcmp(obj_p, "colorbar"))
            curr->cbarid = value_p;
         else  rval = unknown_property_error();
      }
      else if (!strcmp(prop_p, "meta"))
         curr->meta1 = value_p;
      else if (!strcmp(obj_p, "window"))
      {
         if (!strcmp(prop_p, "stem"))
            curr->windowstem = value_p;
         else if (!strcmp(prop_p, "prefix"))
            curr->windowprefix = value_p;
         else if (!strcmp(prop_p, "geom") || !strcmp(prop_p, "geometry"))
         {
            char c1[10], c2[10];
            int i1 = 0, i2= 0;
   
            if (strlen(value_p) == 0)
               curr->windowgeom = ""; 
            else if (sscanf(value_p,"%1[+-]%d%1[+-]%d",c1,&i1,c2,&i2) == 4)
               curr->windowgeom = value_p;
            else 
            {
               rval = ERR_STR_MINMAX;
               string est = "The window.geom value (";
               est.append(value_p);
               est.append(") must be an emtpy string or '[+|-]x[+|-]y' such as -0+100"); 
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
         }
         else if (!strcmp(prop_p, "display"))
         {
            rval = set_boolean_value(curr->windowdisplay, value_p);
         }
         else if (!strcmp(prop_p, "focus"))
         {
            rval = set_boolean_value(curr->windowfocus, value_p);
         }
         else if (!strcmp(prop_p, "smoothing"))
         {
            rval = set_boolean_value(curr->windowsmoothing, value_p);
         }
         else if (!strcmp(prop_p, "height"))
         {
            if (vlen > 0)
            {
               value >> fval;
	       if (value.fail())
	       {
	          rval = ERR_DBL_GTE;
		  string est = "The window.height value (";
		  est.append(value_p);
		  est.append(") must be a numeric value greater than 0"); 
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }   
               else if (fval > 0.0)
                  curr->windowheight = fval; 
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "width"))
         {
            if (vlen > 0)
            {
               value >> fval;
	       if (value.fail())
	       {
	          rval = ERR_DBL_GTE;
		  string est = "The window.width value (";
		  est.append(value_p);
		  est.append(") must be a numeric value greater than 0"); 
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }   
               else if (fval > 0.0)
                  curr->windowwidth = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
	 else if (!strcmp(prop_p, "scaleheight"))
         {
            if (vlen > 0)
            {
               value >> fval;
	       if (value.fail())
	       {
	          rval = ERR_DBL_GTE;
		  string est = "The window.scaleheight value (";
		  est.append(value_p);
		  est.append(") must be a numeric value greater than 0"); 
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }   
               else if (fval > 0.0)
                  curr->windowscaleheight = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
	 else if (!strcmp(prop_p, "scalewidth"))
         {
            if (vlen > 0)
            {
               value >> fval;
	       if (value.fail())
	       {
	          rval = ERR_DBL_GTE;
		  string est = "The window.scalewidth value (";
		  est.append(value_p);
		  est.append(") must be a numeric value greater than 0"); 
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }   
               else if (fval > 0.0)
                  curr->windowscalewidth = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else  rval = unknown_property_error();
      }   
      else if (!strcmp(obj_p, "window.grid"))
      {
         if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->showgrid, value_p);
         }
         else if (!strcmp(prop_p, "scale"))
         { 
            int tmpval;
            if ((rval = c2_string_to_gridscale(value_p, &tmpval,
                "window.grid.scale")) == ERR_STATUS_GOOD)
               curr->gridscale = tmpval;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "window.layout"))
      {
         if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->showlayout, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "frame"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->framestem = value_p;
         }
         else if (!strcmp(prop_p, "layout"))
         {
            rval = set_boolean_value(curr->showlayout, value_p);
         }
         else if (!strcmp(prop_p, "aspect"))
         {
            rval = set_boolean_value(curr->frameaspect, value_p);
         }
         else if (!strcmp(prop_p, "scale"))
         {
            rval = set_boolean_value(curr->framescale, value_p);
         }
         else if (!strcmp(prop_p, "mode"))
         {
            rval = set_framemode(value_p);
         }
         else if (!strcmp(prop_p, "transparency"))
         {
            rval = set_boolean_value(curr->frametransparent, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "frame.border"))
      {
         if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->framebordervis, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "plot.title"))
      {
         if (!strcmp(prop_p, "xpos"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_GTE;
                  string est = "The plot.title.xpos value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->titlexpos = fval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "ypos"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_GTE;
                  string est = "The plot.title.ypos value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->titleypos = fval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_GTE;
                  string est = "The plot.title.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else  curr->titleangle = fval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "halign"))
         {
            float ff; 
            if ((rval = c2_string_to_hoffset(value_p, &ff, "plot.title.halign"))
                == ERR_STATUS_GOOD)
               curr->titlehalign = ff;
         }
         else if (!strcmp(prop_p, "valign"))
         {
            float ff; 
            if ((rval = c2_string_to_voffset(value_p, &ff, "plot.title.valign"))
                 == ERR_STATUS_GOOD)
               curr->titlevalign = ff;
         }
         else if (!strcmp(prop_p, "font"))
         {
            curr->titlefont = value_p;
         } 
         else if (!strcmp(prop_p, "fontstyle"))
         {
            V_FONTSTYLE_TYPE fntstyle; 
            if ((rval = c2_string_to_fontstyle(value_p, &fntstyle,
	        "plot.title.fontstyle")) == ERR_STATUS_GOOD) 
               curr->titlefontstyle = fntstyle;
         } 
         else if (!strcmp(prop_p, "label"))
         {
            curr->titlelabel = value_p;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "plot.aspect"))
      {
         if (!strcmp(prop_p, "ratio"))
         {
	    rval = set_aspect_ratio(&curr, value_p); 
         }
         else   rval = no_property_value_error();
      } 
      else if (!strcmp(obj_p, "plot"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->plotstem = value_p;
         }
         else if (!strcmp(prop_p, "leftmargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_GTE;
                  string est = "The plot.leftmargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval >= 0) && (fval <= V_MAX_MARGIN_SIZE)) 
                  curr->plotlmargin = fval;
               else   rval = data_range_error(fval, 0.0, V_MAX_MARGIN_SIZE);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "rightmargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_GTE;
                  string est = "The plot.rightmargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval >= 0) && (fval <= V_MAX_MARGIN_SIZE)) 
                  curr->plotrmargin = fval;
               else   rval = data_range_error(fval, 0.0, V_MAX_MARGIN_SIZE);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "bottommargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_GTE;
                  string est = "The plot.bottommargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval >= 0) && (fval <= V_MAX_MARGIN_SIZE)) 
                  curr->plotbmargin = fval;
               else   rval = data_range_error(fval, 0.0, V_MAX_MARGIN_SIZE);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "topmargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_GTE;
                  string est = "The plot.topmargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval >= 0) && (fval <= V_MAX_MARGIN_SIZE)) 
                  curr->plottmargin = fval;
               else   rval = data_range_error(fval, 0.0, V_MAX_MARGIN_SIZE);
            }
            else   rval = no_property_value_error();
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "image"))
      {
         if (!strcmp(prop_p, "stem"))
            curr->imagestem = value_p;
	 else if (!strcmp(prop_p, "colormap"))
	 {
            if ((rval = c2_string_to_colormap(value_p, &cmap, "image.colormap"))
	        == ERR_STATUS_GOOD)
	    curr->imagecmap = cmap;
	 }
	 else if (!strcmp(prop_p, "interpolation"))
	 {
            if ((rval = c2_string_to_interpolation(value_p, &interp, "image.interpolation"))
	        == ERR_STATUS_GOOD)
	    curr->image_interp = interp;
	 }
	 else if (!strcmp(prop_p, "alpha"))
	 {
	    vector<double> alphat; 
	    alphat.clear();
	    if ((rval = this->string_to_array<double>(alphat)) == ERR_STATUS_GOOD)
            {
	       if (alphat.size() == 2)
               {
	          curr->imagealpha[0] = alphat[0]; 
	          curr->imagealpha[1] = alphat[1]; 
	       } 
	       else
                  rval = ERR_NO_VALS_SUPPLIED;
	    }
	 }
         else if (!strcmp(prop_p, "threshold"))
         {
            if ((version_exclude == 0) || (file_version >= 40201))
            {
               vector<double> thresht;
               thresht.clear();
               if ((rval = this->string_to_array<double>(thresht)) == 
                   ERR_STATUS_GOOD)
               {
                  if (thresht.size() == 2)
                  {
                     if (thresht[0] == thresht[1] && thresht[0] != 0)
                     {
                        ostringstream errmsg;
                        errmsg << "The threshold value must be specified as [min,max] where min (" << thresht[0] <<") is less than max (" << thresht[1] << ") or min and max are both zero." << ends;
                        rval = ERR_DMIN_GT_DMAX;
                        if (cmd_manager) 
                           cmd_manager->add_err(errmsg.str(),rval,"",-1);
                     }
                     else if (thresht[0] > thresht[1])
                     {
                        rval = ERR_DMIN_GT_DMAX;
                        if (cmd_manager)
                          cmd_manager->add_err(rval,"",0,thresht[0],thresht[1]);
                     }
                     else 
                     {
                        curr->imagethreshold[0] = thresht[0];
                        curr->imagethreshold[1] = thresht[1];
                     }
                  }
                  else
                     rval = ERR_NO_VALS_SUPPLIED;
               }
            }
         }
         else if (!strcmp(prop_p, "invert_colormap"))
         {
            rval = set_boolean_value(curr->imageinvertcmap, value_p);
         }
         else if (!strcmp(prop_p, "colormap_size"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_GTE;
                  string est = "The image.colormap_size value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (ival >= 0)
                  curr->imagecmapsize = ival;
               else   rval = negative_value_error(ival);
            }
            else   rval = no_property_value_error(); 
         }
         else if (!strcmp(prop_p, "colormap_interpolate"))
         {
            rval = set_boolean_value(curr->imagecmapinterp, value_p);
         }
         else if (!strcmp(prop_p, "scale_channels"))
         {
            rval = set_boolean_value(curr->imagescalechannels, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "axis.label"))
      {
         if (!strcmp(prop_p, "font"))
         {
            curr->axlabfont = value_p;
         }
         else if (!strcmp(prop_p, "halign"))
         {
            float ff; 
            if ((rval = c2_string_to_hoffset(value_p, &ff, "axis.label.halign"))
                 == ERR_STATUS_GOOD)
               curr->axlabhalign = ff;
         }
         else if (!strcmp(prop_p, "valign"))
         {
            float ff; 
            if ((rval = c2_string_to_voffset(value_p, &ff,"axis.label.valign"))
                == ERR_STATUS_GOOD)
               curr->axlabvalign = ff;
         }
         else if (!strcmp(prop_p, "fontstyle"))
         {
            V_FONTSTYLE_TYPE fntstyle;
            if ((rval = c2_string_to_fontstyle(value_p, &fntstyle, 
	        "axis.label.fontstyle")) == ERR_STATUS_GOOD)
               curr->axlabfontstyle = fntstyle;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "axis.ticklabel"))
      {
         if (!strcmp(prop_p, "font"))
         {
            curr->ticklabfont = value_p;
         }
         else if (!strcmp(prop_p, "offset"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The axis.ticklabel.offset value (";
                  est.append(value_p);
                  est.append(") must be an integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->ticklaboffset = ival; 
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.ticklabel.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->ticklabangle = fval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "halign"))
         {
            float ff; 
            if ((rval = c2_string_to_hoffset(value_p, &ff, 
                "axis.ticklabel.halign")) == ERR_STATUS_GOOD)
               curr->ticklabhalign = ff;
         }
         else if (!strcmp(prop_p, "valign"))
         {
            float ff; 
            if ((rval = c2_string_to_voffset(value_p, &ff, 
                "axis.ticklabel.valign")) == ERR_STATUS_GOOD)
               curr->ticklabvalign = ff;
         }
         else if (!strcmp(prop_p, "fontstyle"))
         {
            V_FONTSTYLE_TYPE fntstyle;
            if ((rval = c2_string_to_fontstyle(value_p, &fntstyle,
	         "axis.ticklabel.fontstyle")) == ERR_STATUS_GOOD)
               curr->ticklabfontstyle = fntstyle;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "axis"))
      {
         if (!strcmp(prop_p, "tickformat"))
         {
            curr->tickformat = value_p;
         }
         else if (!strcmp(prop_p, "automin"))
         {
            rval = set_boolean_value(curr->axis_automin, value_p);
         }
         else if (!strcmp(prop_p, "automax"))
         {
            rval = set_boolean_value(curr->axis_automax, value_p);
         }
         else if (!strcmp(prop_p, "pad"))
         {
            if (vlen > 0)
            {
               value >> dval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.pad value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (dval < 0.0) 
                  rval = negative_value_error((float)dval);
               else 
                  curr->axis_pad = dval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "logminimum"))
         {
            rval = ERR_DEPRECATED;
            if (cmd_manager)  cmd_manager->add_err(rval, "", -1, "logminimum");
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "axis.x"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            if (strcmp(value_p, curr->axisystem.c_str()) != 0)
               curr->axisxstem = value_p;
            else   
            {
               rval = ERR_AXIS_XY_ID;
               if (cmd_manager)  cmd_manager->add_err(rval);
            }
         }
         else if (!strcmp(prop_p, "label"))
         {
            curr->axisxlabel = value_p;
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "axis.x.label"))
      {
         if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.x.label.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->axlabxangle = fval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "text"))
         {
            curr->axisxlabel = value_p;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "axis.y"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            if (strcmp(value_p, curr->axisxstem.c_str()) != 0)
               curr->axisystem = value_p;
            else 
            {
               rval = ERR_AXIS_XY_ID;
               if (cmd_manager)  cmd_manager->add_err(rval);
            }
         }
         else if (!strcmp(prop_p, "label"))
         {
            curr->axisylabel = value_p;
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "axis.y.label"))
      {
         if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.y.label.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->axlabyangle = fval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "text"))
         {
            curr->axisylabel = value_p;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "axis.majorgrid"))
      {
         if (!strcmp(prop_p, "visible"))
            rval = set_boolean_value(curr->majorgridvisible, value_p);
      }
      else if (!strcmp(obj_p, "axis.minorgrid"))
      {
         if (!strcmp(prop_p, "visible"))
            rval = set_boolean_value(curr->minorgridvisible, value_p);
      }
      else if (!strcmp(obj_p, "axis.offset"))
      {
         if (!strcmp(prop_p, "parallel"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.offset.parallel value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval>=V_MIN_PAR_OFFSET) && (fval<=V_MAX_PAR_OFFSET))
                  curr->axis_offsetpar = fval; 
               else 
                  rval = data_range_error(fval, V_MIN_PAR_OFFSET, V_MAX_PAR_OFFSET);
            }
            else  rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "perpendicular"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.offset.perpendicular value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval>=V_MIN_PERP_OFFSET) && (fval<=V_MAX_PERP_OFFSET))
                  curr->axis_offsetperp = fval; 
               else
                  rval = data_range_error(fval, (float)V_MIN_PERP_OFFSET,
                                          (float)V_MAX_PERP_OFFSET);
            }
            else   rval = no_property_value_error();
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "axis.majortick"))
      {
         if (!strcmp(prop_p, "mode"))
         {
            if ((rval = c2_string_to_tickmode(value_p, &tmode, 
	        "axis.majortick.mode")) == ERR_STATUS_GOOD) 
               curr->majortickmode = value_p;
         }
         else if (!strcmp(prop_p, "interval"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.majortick.interval value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval > 0)  
                  curr->majortickinterval = fval; 
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "count"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The axis.majortick.count value (";
                  est.append(value_p);
                  est.append(") must be a positive integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (ival > 0)
                  curr->majortickcount = ival; 
               else   rval = negative_value_error(ival);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "length"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The axis.majortick.length value (";
                  est.append(value_p);
                  est.append(") must be a positive integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (ival < 0)
                  rval = negative_value_error(ival);
               else
                  curr->majorticklength = ival;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->majortickvisible, value_p);
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "axis.minortick"))
      {
         if (!strcmp(prop_p, "mode"))
         {
            if ((version_exclude == 0) || (file_version >= 40201)) 
            {
               if ((rval = c2_string_to_tickmode(value_p, &tmode, 
	           "axis.minortick.mode")) == ERR_STATUS_GOOD) 
                  curr->minortickmode = value_p;
            }
         }
         else if (!strcmp(prop_p, "interval"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The axis.minortick.interval value (";
                  est.append(value_p);
                  est.append(") must be a positive numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval > 0) 
                  curr->minortickinterval = fval;
               else rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "count"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The axis.minortick.count value (";
                  est.append(value_p);
                  est.append(") must be a positive integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (ival > 0)  
                  curr->minortickcount = ival;
               else  rval = negative_value_error(ival);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "length"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The axis.minortick.length value (";
                  est.append(value_p);
                  est.append(") must be an integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (ival < 0)
                  rval = negative_value_error(ival);
               else
                  curr->minorticklength = ival;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->minortickvisible, value_p);
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "curve"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->curvestem = value_p;
         }
         else if (!strcmp(prop_p, "baddata"))
         {
	    V_BADDATAMODE_TYPE bmode;
	    if ((rval = string_to_v_baddatamode(value_p, &bmode)) ==
	        ERR_STATUS_GOOD)
               curr->curvebaddata = bmode;
            else 
               data_subset_error(value_p, "ignore|omit");
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "histogram"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->histstem = value_p;
         }
         else if (!strcmp(prop_p, "dropline"))
         {
            rval = set_boolean_value(curr->histdropline, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "histogram.fill"))
      {
         if (!strcmp(prop_p, "opacity"))
         {
            if (vlen > 0)
            {
               value >> dval;
               if (value.fail())
               {
                  rval = ERR_DBL_MINMAX;
                  string est = "The histogram.fill.opacity value (";
                  est.append(value_p);
                  est.append(") must be between 0.000 and 1.000");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((dval < 0.0) || (dval > 1.0))
                  rval = data_range_error((float)dval, 0.0, 1.0);
               else
                  curr->histfillopacity = dval;
            }
            else  rval = no_property_value_error();
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "contour"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->contourstem = value_p;
         }
         else if (!strcmp(prop_p, "levels"))
         {
            curr->contourlevels.clear ();
            rval = this->string_to_array<double> (curr->contourlevels);
         
            if (rval != ERR_STATUS_GOOD)
               curr->contourlevels.clear ();
         }
         else if (!strcmp(prop_p, "numlevels"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The contour.levels value (";
                  est.append(value_p);
                  est.append(") must be a positive integer");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (ival > 0)
                  curr->contournumlevels = ival;
               else 
                  rval = negative_value_error(ival);
            }
            else  rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "algorithm"))
         {
            V_CONTOUR_TYPE calg; 
            if ((rval = string_to_v_contour(value_p, &calg)) == ERR_STATUS_GOOD)
               curr->contouralgorithm = calg; 
            else
               data_subset_error(value_p, "standard|marching");
         }
         else if (!strcmp(prop_p, "mode"))
         {
            V_TICKMODE_TYPE cmod; 
            if ((rval = string_to_v_tickmode(value_p, &cmod)) == ERR_STATUS_GOOD)
               curr->contourmode = cmod; 
         }
         else if (!strcmp(prop_p, "interval"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The contour.interval value (";
                  est.append(value_p);
                  est.append(") must be a positive numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval > 0)
                  curr->contourinterval = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "region"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->regionstem = value_p;
         }
         else if (!strcmp(prop_p, "opacity"))
         {
            if (vlen > 0)
            {
               value >> dval;
               if (value.fail())
               {
                  rval = ERR_DBL_MINMAX;
                  string est = "The region.opacity value (";
                  est.append(value_p);
                  est.append(") must be between 0.000 and 1.000");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((dval < 0.0) || (dval > 1.0))
                  rval = data_range_error((float)dval, 0.0, 1.0);
               else
                  curr->regionopacity = dval;
            }
            else  rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The region.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->regionangle = fval;
            }
            else   rval = no_property_value_error();
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "curve.err"))
      {
         if (!strcmp(prop_p, "up"))
         {
            rval = set_boolean_value(curr->errup, value_p);
         }
         else if (!strcmp(prop_p, "down"))
         {
            rval = set_boolean_value(curr->errdown, value_p);
         }
         else if (!strcmp(prop_p, "left"))
         {
            rval = set_boolean_value(curr->errleft, value_p);
         }
         else if (!strcmp(prop_p, "right"))
         {
            rval = set_boolean_value(curr->errright, value_p);
         }
         else if (!strcmp(prop_p, "*") && 
            (rval=set_boolean_value(curr->errup, value_p)) == ERR_STATUS_GOOD)
         {
            curr->errright = curr->errdown = curr->errleft = curr->errup;
         }
         else if (!strcmp(prop_p, "caplength"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The curve.err.caplength value (";
                  est.append(value_p);
                  est.append(") must be an integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((ival < V_SIZE_MIN_VAL) || (ival > V_SIZE_MAX_VAL))
                  rval = data_range_error(ival, V_SIZE_MIN_VAL, V_SIZE_MAX_VAL);
               else
                  curr->errcaplen = ival;
            }
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "curve.symbol"))
      {
         if (!strcmp(prop_p, "angle") && (vlen > 0))
         {
            value >> fval;
            if (value.fail())
            {
               rval = ERR_DBL_MIN;
               string est = "The curve.symbol.angle value (";
               est.append(value_p);
               est.append(") must be a numeric value");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
            else   curr->curvesymbolangle = fval;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "curve.limit"))
      {
         if (!strcmp(prop_p, "length") && (vlen > 0))
         {
            value >> fval;
            if (value.fail())
            {
               rval = ERR_DBL_MIN;
               string est = "The curve.limit.length value (";
               est.append(value_p);
               est.append(") must be a numeric value");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
            else   curr->limitlength = fval;
         }
         else if (!strcmp(prop_p, "override"))
         {
            rval = set_boolean_value(curr->limitoverride, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "histogram.symbol"))
      {
         if (!strcmp(prop_p, "angle") && (vlen > 0))
         {
            value >> fval;
            if (value.fail())
            {
               rval = ERR_DBL_MIN;
               string est = "The histogram.symbol.angle value (";
               est.append(value_p);
               est.append(") must be a numeric value");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
            else   curr->histsymbolangle = fval;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "histogram.err"))
      {
         if (!strcmp(prop_p, "up"))
         {
            rval = set_boolean_value(curr->histerrup, value_p);
         }
         else if (!strcmp(prop_p, "down"))
         {
            rval = set_boolean_value(curr->histerrdown, value_p);
         }
         else if (!strcmp(prop_p, "*") && 
            (rval=set_boolean_value(curr->histerrup, value_p))==ERR_STATUS_GOOD)
         {
            curr->histerrdown = curr->histerrup;
         }
         else if (!strcmp(prop_p, "caplength"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The histogram.err.caplength value (";
                  est.append(value_p);
                  est.append(") must be an integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((ival < V_SIZE_MIN_VAL) || (ival > V_SIZE_MAX_VAL))
                  rval = data_range_error(ival, V_SIZE_MIN_VAL, V_SIZE_MAX_VAL);
               else
                  curr->histerrcaplen = ival;
            }
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "point"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->pointstem = value_p;
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The point.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->pointangle = fval;
            }
            else   rval = no_property_value_error();
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "line"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->linestem = value_p;
         }
         else if (!strcmp(prop_p, "extend"))
         {
            V_LINE_EXTENSION_TYPE ln_extend;
            if ((rval = c2_string_to_line_extension(value_p, &ln_extend, "line.extend")) ==
                ERR_STATUS_GOOD)
               curr->lineextend = ln_extend;
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "label"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->labelstem = value_p;
         }
         else if (!strcmp(prop_p, "font"))
         {
            curr->labelfont = value_p;
         }
         else if (!strcmp(prop_p, "fontstyle"))
         {
            V_FONTSTYLE_TYPE fntstyle;
            if ((rval = c2_string_to_fontstyle(value_p, &fntstyle,
	        "label.fontstyle")) == ERR_STATUS_GOOD)
               curr->labelstyle = fntstyle;
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The label.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else  curr->labelangle = fval;
            }
            else  rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "halign"))
         {
            float ff; 
            if ((rval = c2_string_to_hoffset(value_p, &ff, "label.halign"))
                == ERR_STATUS_GOOD)
               curr->labelhalign = ff;
         }
         else if (!strcmp(prop_p, "valign"))
         {
            float ff; 
            if ((rval = c2_string_to_voffset(value_p, &ff, "label.valign"))
                == ERR_STATUS_GOOD)
               curr->labelvalign = ff;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "colorbar.label"))
      {
         if (!strcmp(prop_p, "font"))
         {
            curr->cbarlabfont = value_p;
         }
         else if (!strcmp(prop_p, "fontstyle"))
         {
            V_FONTSTYLE_TYPE fntstyle;
            if ((rval = c2_string_to_fontstyle(value_p, &fntstyle,
                "colorbar.label.fontstyle")) == ERR_STATUS_GOOD)
               curr->cbarlabfontstyle = fntstyle;
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The colorbar.label.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else  curr->cbarlabangle = fval;
            }
            else  rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "halign"))
         {
            float ff;
            if ((rval = c2_string_to_hoffset(value_p, &ff, "colorbar.label.halign"))
                == ERR_STATUS_GOOD)
               curr->cbarlabhalign = ff;
         }
         else if (!strcmp(prop_p, "valign"))
         {
            float ff;
            if ((rval = c2_string_to_voffset(value_p, &ff, "colorbar.label.valign"))
                == ERR_STATUS_GOOD)
               curr->cbarlabvalign = ff;
         }
         else if (!strcmp(prop_p, "location"))
         {
            if ((rval = string_to_v_tickstyle(value_p, &tstyle)) == ERR_STATUS_GOOD)
            {
               if ((tstyle == chips_inside) || (tstyle == chips_outside))
                  curr->cbarlabloc = tstyle;
               else
               {
                  rval = data_subset_error(value_p, "[inside|outside]");
               }
            }
         }
         else if (!strcmp(prop_p, "text"))
         {
            curr->cbarlabtext = value_p;
         }
         else if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->cbarlabvisible, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "colorbar.offset"))
      {
         if (!strcmp(prop_p, "parallel"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The colorbar.offset.parallel value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval>=V_MIN_PAR_OFFSET) && (fval<=V_MAX_PAR_OFFSET))
                  curr->cbaroffsetpar = fval;
               else
                  rval = data_range_error(fval, V_MIN_PAR_OFFSET, V_MAX_PAR_OFFSET);
            }
            else  rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "perpendicular"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The colorbar.offset.perpendicular value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if ((fval>=V_MIN_PERP_OFFSET) && (fval<=V_MAX_PERP_OFFSET))
                  curr->cbaroffsetperp = fval;
               else
                  rval = data_range_error(fval, (float)V_MIN_PERP_OFFSET,
                                          (float)V_MAX_PERP_OFFSET);
            }
            else   rval = no_property_value_error();
         }
         else  rval = no_property_value_error();
      }
      else if (!strcmp(obj_p, "colorbar.ticklabel"))
      {
         if (!strcmp(prop_p, "font"))
         {
            curr->cbarticklabfont = value_p;
         }
         else if (!strcmp(prop_p, "offset"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The colorbar.ticklabel.offset value (";
                  est.append(value_p);
                  est.append(") must be an integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->cbarticklaboffset = ival;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The colorbar.ticklabel.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else   curr->cbarticklabangle = fval;
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "halign"))
         {
            float ff;
            if ((rval = c2_string_to_hoffset(value_p, &ff,
                "colorbar.ticklabel.halign")) == ERR_STATUS_GOOD)
               curr->cbarticklabhalign = ff;
         }
         else if (!strcmp(prop_p, "valign"))
         {
            float ff;
            if ((rval = c2_string_to_voffset(value_p, &ff,
                "colorbar.ticklabel.valign")) == ERR_STATUS_GOOD)
               curr->cbarticklabvalign = ff;
         }
         else if (!strcmp(prop_p, "fontstyle"))
         {
            V_FONTSTYLE_TYPE fntstyle;
            if ((rval = c2_string_to_fontstyle(value_p, &fntstyle,
                 "colorbar.ticklabel.fontstyle")) == ERR_STATUS_GOOD)
               curr->cbarticklabfontstyle = fntstyle;
         }
         else if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->cbarticklabvisible, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "colorbar.tick"))
      {
         if (!strcmp(prop_p, "mode"))
         {
            int tmpmode; 
            if ((rval = c2_string_to_cbarmode(value_p, &tmpmode,
                "colorbar.tick.mode")) == ERR_STATUS_GOOD)
               curr->cbartickmode = tmpmode;
         }
         else  if (!strcmp(prop_p, "location"))
         {
            if ((rval = string_to_v_tickstyle(value_p, &tstyle)) == ERR_STATUS_GOOD)
            {
               if ((tstyle == chips_inside) || (tstyle == chips_outside))
                  curr->cbartickloc = tstyle;
               else
               {
                  rval = data_subset_error(value_p, "[inside|outside]");
               }
            }
         }
         else if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->cbartickvisible, value_p);
         }
         else if (!strcmp(prop_p, "length"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The colorbar.tick.length value (";
                  est.append(value_p);
                  est.append(") must be a positive integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (ival < 0)
                  rval = negative_value_error(ival);
               else
                  curr->cbarticklength = ival;
            }
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "colorbar"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->cbarstem = value_p;
         }
         else if (!strcmp(prop_p, "width"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The colorbar.width value (";
                  est.append(value_p);
                  est.append(") must be a positive value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval > 0.0)
                  curr->cbarwidth = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "length"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The colorbar.length value (";
                  est.append(value_p);
                  est.append(") must be a positive integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval > 0.0)
                  curr->cbarlength = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "halign"))
         {
            float ff;
            if ((rval = c2_string_to_hoffset(value_p, &ff, "colorbar.halign"))
                == ERR_STATUS_GOOD)
               curr->cbarhalign = ff;
         }
         else if (!strcmp(prop_p, "valign"))
         {
            float ff;
            if ((rval = c2_string_to_voffset(value_p, &ff, "colorbar.valign"))
                 == ERR_STATUS_GOOD)
               curr->cbarvalign = ff;
         }
         else if (!strcmp(prop_p, "orientation"))
         {
            if (!chips_strcmp(value_p,"horizontal")||!chips_strcmp(value_p,"0"))
               curr->cbarorient = 0; 
            else if (!chips_strcmp(value_p,"vertical")||!chips_strcmp(value_p,"1")) 
               curr->cbarorient = 1;
            else 
            {
               string est = "The colorbar.orientation value (";
               est.append(value_p);
               est.append(") must be one of [horizontal|vertical|0|1]");
               if (cmd_manager)  
                  cmd_manager->add_err(est.c_str(),rval=ERR_DATA_SUBSET,"",-1);
            }
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "colorbar.border"))
      {
        if (!strcmp(prop_p, "visible"))
         {
            rval = set_boolean_value(curr->cbarbordervis, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "legend"))
      {
         if (!strcmp(prop_p, "stem"))
         {
            curr->legendstem = value_p;
         }
         else if (!strcmp(prop_p, "transparency"))
         {
            rval = set_boolean_value(curr->legendtransparent, value_p);
         }
         else if (!strcmp(prop_p, "borders"))
         {
            rval = set_boolean_value(curr->legendborders, value_p);
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "foreground"))
      {
         if (!strcmp(prop_p, "file"))
         {
            defaultfgps = value_p;
         }
         else if (!strcmp(prop_p, "display"))
         {
            defaultfgdisp = value_p;
         }
         else  rval = unknown_property_error(); 
      }
      else if (!strcmp(obj_p, "background"))
      {
         if (!strcmp(prop_p, "file"))
         {
            defaultbgps = value_p;
         }
         else if (!strcmp(prop_p, "display"))
         {
            defaultbgdisp = value_p;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "export"))
      {
         if (!strcmp(prop_p, "fonts"))
         {
            rval = set_boolean_value(curr->exportfonts, value_p);
         }
         else if (!strcmp(prop_p, "clobber"))
         {
            rval = set_boolean_value(curr->exportclobber, value_p);
         }
         else if (!strcmp(prop_p, "format"))
         {
            if (chips_strcmp(value_p, "ps") == 0) 
               curr->exportformat = "ps"; 
            else if (chips_strcmp(value_p, "eps") == 0) 
               curr->exportformat = "eps"; 
            else if (chips_strcmp(value_p, "png") == 0) 
               curr->exportformat = "png"; 
            else if (chips_strcmp(value_p, "pdf") == 0) 
               curr->exportformat = "pdf"; 
            else if (chips_strcmp(value_p, "jpeg") == 0) 
               curr->exportformat = "jpeg"; 
            else if (chips_strcmp(value_p, "jpg") == 0) 
               curr->exportformat = "jpg"; 
            else  
               rval = data_subset_error(value_p, "ps|eps|png|pdf|jpg");
         }
         else if (!strcmp(prop_p, "orientation"))
         {
            if (chips_strcmp(value_p, "portrait") == 0) 
               curr->exportorient = 0;
            else if (chips_strcmp(value_p, "landscape") == 0) 
               curr->exportorient = 1;
            else   
            {
               rval = ERR_EXPORT_ORIENT;
               if (cmd_manager)  cmd_manager->add_err(rval, "", 0, 
                  "export.orientation", "portrait|landscape");
            }
         }
         else if (!strcmp(prop_p, "colorscheme"))
         {
            V_PRINTFILE_TYPE scheme; 
            if ((rval = string_to_v_printfile(value_p, &scheme)) == 
                 ERR_STATUS_GOOD)
               curr->exportcolorscheme = scheme;
	    else if (cmd_manager)  cmd_manager->add_err(ERR_STR_SUBSET, "", 0,
	       "export.colorscheme", value_p, "rgb|cmyk|grayscale|cmyksep");
         }
         else if (!strcmp(prop_p, "dpi"))
         {
            if (vlen > 0)
            {
               value >> ival;
               if (value.fail())
               {
                  rval = ERR_INT_MIN;
                  string est = "The export.dpi value (";
                  est.append(value_p);
                  est.append(") must be a positive integer value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1)
;
               }
               else if (ival < 0)
                  rval = negative_value_error(ival);
               else
                  curr->exportdpi = ival;
            }

         }
         else if (!strcmp(prop_p, "pageunits"))
         {
            if ((rval = c2_string_to_units(value_p, &units)) == ERR_STATUS_GOOD)
               curr->pageunits = units;
         }
         else if (!strcmp(prop_p, "pagesize"))
         {
            rval= string_to_pagesize(value_p, curr->pagewidth, 
                                     curr->pageheight, curr->pageunits);
            if ((rval != ERR_STATUS_GOOD) && cmd_manager)
               cmd_manager->add_err(rval);
         }
         else if (!strcmp(prop_p, "leftmargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.leftmargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pagelmargin = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error(); 
         }
         else if (!strcmp(prop_p, "rightmargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.rightmargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pagermargin = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error(); 
         }
         else if (!strcmp(prop_p, "bottommargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.bottommargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pagebmargin = fval;
               else   rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error(); 
         }
         else if (!strcmp(prop_p, "topmargin"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.topmargin value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pagetmargin = fval;
               else  rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "pagewidth"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.pagewidth value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pagewidth = fval;
               else  rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "pageheight"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.pageheight value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pageheight = fval;
               else  rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "scalewidth"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.scalewidth value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pagescalewidth = fval;
               else  rval = negative_value_error(fval);
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "scaleheight"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The export.scaleheight value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else if (fval >= 0.0)
                  curr->pagescaleheight = fval;
               else  rval = negative_value_error(fval); 
            }
            else   rval = no_property_value_error();
         }
         else if (!strcmp(prop_p, "keepaspect"))
         {
            rval = set_boolean_value(curr->pageaspect, value_p);
         }
         else if (!strcmp(prop_p, "printdialog"))
         {
            rval = set_boolean_value(curr->printdialog, value_p);
         }
	 else if (!strcmp (prop_p, "fittopage"))
         {
            rval = set_boolean_value(curr->fittopage, value_p);
         }
         else if (!strcmp(prop_p, "printmethod"))
         {
            if (chips_strcmp(value_p, "framebuffer") == 0)
               curr->printmethod = "framebuffer";
            else if (chips_strcmp(value_p, "xoffscreen") == 0)
               curr->printmethod = "xoffscreen";
            else if (chips_strcmp(value_p, "none") == 0)
               curr->printmethod = "none";
            else
               rval = data_subset_error(value_p, "framebuffer|xoffscreen|none");

         }
	 else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "gui.view"))
      {
         if (!strcmp(prop_p, "toolbar"))
         {
            rval = set_boolean_value(curr->toolbarvis, value_p);
         }
         else if (!strcmp(prop_p, "navpane") || !strcmp(prop_p, "navframe"))
         {
            rval = set_boolean_value(curr->navpanevis, value_p);
         }
         else if (!strcmp(prop_p, "infopane") || !strcmp(prop_p, "infoframe"))
         {
            rval = set_boolean_value(curr->infpanevis, value_p);
         }
         else if (!strcmp(prop_p, "propertypane") || !strcmp(prop_p, "propertyframe") )
         {
            rval = set_boolean_value(curr->proppanevis, value_p);
         }
         else if (!strcmp(prop_p, "shellpane") || !strcmp(prop_p, "shellframe"))
         {
            rval = set_boolean_value(curr->shellpanevis, value_p);
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "gui.toolbar"))
      {
         if (!strcmp(prop_p, "mode"))
         {
            if (!chips_strcmp(value_p, "pan"))
               curr->tbarmode = "pan";
            else if (!chips_strcmp(value_p, "range"))
               curr->tbarmode = "range";
            else if (!chips_strcmp(value_p, "select"))
               curr->tbarmode = "select";
#ifdef CHIPS_GUI_ADDDEL
            else if (!chips_strcmp(value_p, "add"))
               curr->tbarmode = "add";
#endif
            else
            {
               string est = "The gui.toolbar.mode value (";
               est.append(value_p);
#ifdef CHIPS_GUI_ADDDEL
               est.append(") must be one of [pan|range|select|add]");
#else 
               est.append(") must be one of [pan|range|select]");
#endif 
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval=ERR_DATA_SUBSET,"",-1);
            }
         }
         else if (!strcmp(prop_p, "zoom"))
         {
            rval = set_boolean_value(curr->zoombtn, value_p);
         }
         else if (!strcmp(prop_p, "delete"))
         {
            rval = set_boolean_value(curr->delbtn, value_p);
         }
         else   rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "gui"))
      {
         if (!strcmp(prop_p, "tooltips"))
         {
            rval = set_boolean_value(curr->guitips, value_p);
         }
         else   rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "pick"))
      {
         if (!strcmp(prop_p, "mask"))
         {
            if ((rval = string_to_pickmask(value_p, &pickmask)) != ERR_STATUS_GOOD)
               if (cmd_manager) cmd_manager->add_err(rval);
         }
         else   rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "chips"))
      {
         if (!strcmp(prop_p, "version"))
         {
            value >> ival;
            if (value.fail())
            {
               rval = ERR_INT_MIN;
               string est = "The file.version value (";
               est.append(value_p);
               est.append(") should be an integer representing the package version such as 40201");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
            else
            { 
               file_version = ival; 
            }
         }
         else   rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "all"))
      {
         if (!strcmp(prop_p, "font"))
         {
            allfont = value_p;
            allpropmask = PREF_ALL_FONT; 
         }
         else if (!strcmp(prop_p, "fontstyle"))
         {
            V_FONTSTYLE_TYPE fntstyle;
            if ((rval = c2_string_to_fontstyle(value_p, &fntstyle,
                "all.fontstyle")) == ERR_STATUS_GOOD)
            {
               allfontstyle = fntstyle;
               allpropmask = PREF_ALL_FONTSTYLE; 
            }
         }
         else if (!strcmp(prop_p, "count"))
         {
            value >> ival;
            if (value.fail())
            {
               rval = ERR_INT_MIN;
               string est = "The all.count value (";
               est.append(value_p);
               est.append(") must be an integer value");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
            else if (ival > 0)
            {
               allcount = ival;
               allpropmask = PREF_ALL_COUNT;
            }
            else  rval = negative_value_error(ival);
         }
         else if (!strcmp(prop_p, "length"))
         {
            value >> ival;
            if (value.fail())
            {
               rval = ERR_INT_MIN;
               string est = "The all.length value (";
               est.append(value_p);
               est.append(") must be an integer value");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
            else if (ival < 0)
               rval = negative_value_error(ival);
            else
            {
               alllength = ival; 
               allpropmask = PREF_ALL_LENGTH;
            }
         }
         else if (!strcmp(prop_p, "aspect"))
         {
            if ((rval = set_boolean_value(allaspect, value_p)) == ERR_STATUS_GOOD)
               allpropmask = PREF_ALL_ASPECT;
         }
         else if (!strcmp(prop_p, "visible"))
         {
            if ((rval = set_boolean_value(allvisible, value_p)) == ERR_STATUS_GOOD)
               allpropmask = PREF_ALL_VISIBLE;
         }
         else if (!strcmp(prop_p, "depth"))
         {
            value >> ival;
            if (value.fail())
            {
	       rval = ERR_STR_SUBSET;  
               if (cmd_manager)  cmd_manager->add_err(rval, "", -1, "all.depth",
	          value_p, "'default'|0|<positive integer>");
            }
            else if (ival < 0)
               rval = negative_value_error(ival);
            else
            {
               alldepth = ival; 
               allpropmask = PREF_ALL_DEPTH;
            }
         }
         else if (!strcmp(prop_p, "angle"))
         {
            if (vlen > 0)
            {
               value >> fval;
               if (value.fail())
               {
                  rval = ERR_DBL_MIN;
                  string est = "The all.angle value (";
                  est.append(value_p);
                  est.append(") must be a numeric value");
                  if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
               }
               else
               {
                  allangle = fval;
                  allpropmask = PREF_ALL_ANGLE;
               }
            }
            else  rval = no_property_value_error();
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "default"))
      {
         if (!strcmp(prop_p, "verbosity"))
         {
            value >> ival;
            if (value.fail())
            {
               rval = ERR_INT_MIN;
               string est = "The default.verbosity value (";
               est.append(value_p);
               est.append(") must be an integer value");
               if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval,"",-1);
            }
            else if (ival < 0)
               rval = negative_value_error(ival);
            else
               defaultverb = ival;
         }
         else  rval = unknown_property_error();
      }
      else if (!strcmp(obj_p, "info"))
      {
         if (!strcmp(prop_p, "current"))
            rval = set_boolean_value(infocurrent, value_p);
         else if (!strcmp(prop_p, "coord"))
            rval = set_boolean_value(infocoord, value_p);
         else 
            rval = unknown_property_error();
      }
      else  rval = unknown_property_error(); 
   }
   return rval;
}

int ChipsDefaults::parseLine(const char *in_txt, char del, bool rec)
{
   int rr, inlen, rval = ERR_STATUS_GOOD;
   char *cp = buf;

   prop_p = value_p = NULL;

   if (in_txt && ((inlen = strlen(in_txt)) < PREF_MAX_LEN))
   {
      bool convert =true;  
      bool inquote = false; 
      bool inbracket = false;

      // remove all whitespace not enclosed in quotes or brackets
      for (rr = 0; rr < inlen; rr++)
      {
         if (in_txt[rr] == '[')
            inbracket = true;
         else if ((in_txt[rr] == ']') && (rr != 0) && (in_txt[rr-1] != '\\'))
            inbracket = false;

         // keep the quote if inbracket
         if ((in_txt[rr] == '"') && !inbracket)
	 {
	    inquote = !inquote;
	 }
	 else if (inbracket || inquote || (isspace(in_txt[rr]) == 0))
         {
            // characters upto the delimiter should be converted to lowercase
            if (convert)
               *cp = tolower(in_txt[rr]);
            else  
               *cp = in_txt[rr];

            if (in_txt[rr] == del)
            {
               if (!value_p) value_p = cp;
               convert = false; 
            }
            cp++;
         }
      }
      *cp = '\0';

      if (strlen(obj_p)== 0)
      {
         // blank line- ignore
      }
      else if (*obj_p == '#')
      {
         // comment line- ignore
      }
      else if (value_p == NULL)
      {
         rval = unknown_property_error();   // err - delimeter missing
      }
      else if (obj_p[0] == del)
      {
         rval = ERR_BLANK_ID;   // err - no variable name
      }
      else
      {
         // separate the value portion of the string now so that we can
         // use the last '.' as the delimiter for the object and property
         // without having to worry about decimals in the value string
         *value_p = '\0';
         value_p++;
         prop_p = strrchr(obj_p, '.');

         if (prop_p)
         {
	    string pnm, nvl, ov;
            if (rec)
	    {
	       char* ovc;
               pnm = obj_p;
               nvl = value_p;
               ovc = get_preference(pnm);
	       // get_preference may return a NULL
	       if (ovc)
               {
                  ov = ovc;
                  free (ovc);
               }
	       else ov = "";
	    }
            *prop_p = '\0';
            prop_p++;

            // by passing in record we're ignoring all validation of 
            // preference values. This should be ok since we should
            // be in an undo/redo sequence and all values should be valid
            rval = processLine(!rec);
            if (rval == ERR_STATUS_GOOD && rec && cmd_manager)
            {
               ChipsPrefCommand* cmd = new ChipsPrefCommand(pnm, ov, nvl);
               cmd_manager->add_cmd(cmd);
            }
         }
         else  rval = unknown_property_error();  // no . to delimit object/property
      }
   }
   else  rval = ERR_OBJ_DNE;   // NULL string err

   return rval;
}


void ChipsDefaults::get_window_prefs(string &pref)
{
   stringstream out;
   const char* temp; 

   out << "window.display         : ";
   if (curr->windowdisplay) out << "true" << endl; else out << "false" << endl;
   out << "window.smoothing       : ";
   if (curr->windowsmoothing) out<< "true" << endl; else out << "false" << endl;
   out << "window.height          : " << curr->windowheight << endl;
   out << "window.width           : " << curr->windowwidth << endl;
   out << "window.scaleheight     : " << curr->windowscaleheight << endl;
   out << "window.scalewidth      : " << curr->windowscalewidth << endl;
   out << "window.units           : ";
   temp = v_units_to_string(curr->windowunits);
   if (temp)  out << temp;
   out << endl;
   out << "window.stem            : " << curr->windowstem << endl;
   out << "window.prefix          : \"" << curr->windowprefix << "\"" <<endl;
   out << "window.geom            : " << curr->windowgeom << endl;
   out << "window.bgcolor         : " << curr->windowcolor << endl;
   out << "window.grid.visible    : ";
   if (curr->showgrid) out << "true" << endl; else out << "false" << endl;
   out << "window.grid.scale      : ";
   temp = c2_gridscale_to_string(curr->gridscale);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "window.grid.color      : " << curr->gridcolor << endl;
   out << "window.grid.thickness  : " << curr->gridthickness << endl;
   out << "window.grid.style      : ";
   temp = v_line_to_string(curr->gridlinestyle);
   if (temp)  out << temp;
   out <<  " " << endl;
#ifdef WLM_WINLAYOUT_SUPPORT
   out << "window.layout.visible  : ";
   if (curr->showlayout) out << "true" << endl; else out << "false" << endl;
   out << "window.layout.color    : " << curr->layoutcolor << endl;
   out << "window.layout.thickness: " << curr->layoutthickness << endl;
   out << "window.layout.style    : ";
   temp = v_line_to_string(curr->layoutlinestyle);
   if (temp)  out << temp;
#endif
   pref = out.str ();
}

void ChipsDefaults::get_frame_prefs(string &pref)
{
   stringstream out;

   out << "frame.stem             : " << curr->framestem << endl;
   out << "frame.bgcolor          : " << curr->framecolor << endl;
   out << "frame.border.color     : " << curr->framebordercolor << endl;
   out << "frame.border.visible   : ";
   if (curr->framebordervis) out << "true" << endl; else out << "false" << endl;
   out << "frame.aspect           : ";
   if (curr->frameaspect) out << "true" << endl; else out << "false" << endl;
   out << "frame.scale            : ";
   if (curr->framescale) out << "true" << endl; else out << "false" << endl;
   out << "frame.mode             : ";
   if (curr->framemode == V_FRAMEMODE_TILED)   out << "tiled" << endl;
      if (curr->framemode == V_FRAMEMODE_SINGLE)  out << "single" << endl;
         else out << "placed" << endl; 
   out << "frame.transparency     : ";
   if (curr->frametransparent) 
      out <<"true" << endl; 
   else  
      out << "false" << endl;
   pref = out.str ();
}

void ChipsDefaults::get_devicecolor_prefs(string &pref)
{
   stringstream out;

   out << "foreground.file        : " << defaultfgps << endl;
   out << "background.file        : " << defaultbgps << endl;
   out << "foreground.display     : " << defaultfgdisp << endl;
   out << "background.display     : " << defaultbgdisp << endl;
   pref =  out.str();
}


void ChipsDefaults::get_export_prefs(string &pref)
{
   stringstream out;
   const char* temp; 

   out << "export.fonts           : ";
   if (curr->exportfonts) out << "true" << endl; else out << "false" << endl;
   out << "export.clobber         : ";
   if (curr->exportclobber) out << "true" << endl; else out << "false" << endl;
   out << "export.format          : " << curr->exportformat << endl;
   out << "export.orientation     : ";
   if (curr->exportorient == 0) out << "portrait" << endl; 
      else out << "landscape" << endl;
   out << "export.colorscheme     : ";
   temp = v_printfile_to_string(curr->exportcolorscheme);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "export.leftmargin      : " << curr->pagelmargin << endl;
   out << "export.rightmargin     : " << curr->pagermargin << endl;
   out << "export.topmargin       : " << curr->pagetmargin << endl;
   out << "export.bottommargin    : " << curr->pagebmargin << endl;
   out << "export.pagewidth       : " << curr->pagewidth << endl;
   out << "export.pageheight      : " << curr->pageheight << endl;
   temp = v_pagesize_to_string (curr->pagewidth, curr->pageheight, curr->pageunits);
   if (temp)   out << "export.pagesize        : " << temp << endl;
   temp = v_units_to_string(curr->pageunits);
   if (temp)   out << "export.pageunits       : " << temp << endl;
   out << "export.scaleheight     : " << curr->pagescaleheight << endl;
   out << "export.scalewidth      : " << curr->pagescalewidth << endl;
   out << "export.keepaspect      : ";
   if (curr->pageaspect) out << "true" << endl; else out << "false" << endl;
   out << "export.printdialog     : ";
   if (curr->printdialog) out << "true" << endl; else out << "false" << endl;
   out << "export.fittopage       : ";
   if (curr->fittopage) out << "true" << endl; else out << "false" << endl;
   out << "export.dpi             : " << curr->exportdpi << endl;
   out << "export.printmethod     : " << curr->printmethod << endl;
   out <<  " " << endl;
   pref = out.str();
}

void ChipsDefaults::get_gui_prefs(string &pref)
{
   stringstream out;

   out << "gui.view.toolbar       : ";
   if (curr->toolbarvis) out << "true" << endl; else out << "false" << endl;
   out << "gui.view.propertyframe : ";
   if (curr->proppanevis) out << "true" << endl; else out << "false" << endl;
   out << "gui.view.navframe      : ";
   if (curr->navpanevis) out << "true" << endl; else out << "false" << endl;
   out << "gui.view.infoframe     : ";
   if (curr->infpanevis) out << "true" << endl; else out << "false" << endl;
   out << "gui.view.shellframe    : ";
   if (curr->shellpanevis) out << "true" << endl; else out << "false" << endl;

   out << "gui.toolbar.mode       : " << curr->tbarmode << endl;
   out << "gui.toolbar.style      : " << curr->tbarstyle << endl;  
   out << "gui.toolbar.zoom       : ";
   if (curr->zoombtn) out << "true" << endl; else out << "false" << endl;
#ifdef CHIPS_GUI_ADDDEL
   out << "gui.toolbar.delete     : ";
   if (curr->delbtn) out << "true" << endl; else out << "false" << endl;
#endif
   out << "gui.tooltips           : ";
   if (curr->guitips) out << "true" << endl; else out << "false" << endl;

   pref =  out.str();
}


void ChipsDefaults::get_pick_prefs(string &pref)
{
   stringstream out;
   const char* temp; 
   out << "pick.mask              : ";
   temp = pickmask_to_string(pickmask);
   if (temp) out << temp;
   out << endl;
   out << "pick.color             : " << curr->pickcolor << endl;
   pref = out.str();
}


void ChipsDefaults::get_info_prefs(string &pref)
{
   stringstream out;
   out << "info.coord             : ";
   if (infocoord) out << "true" << endl; else out << "false" << endl;
   out << "info.current           : ";
   if (infocurrent) out << "true" << endl; else out << "false" << endl;
   out << "info.depth             : ";
   if (infodepth) out << "true" << endl; else out << "false" << endl;
   pref = out.str();
}

void ChipsDefaults::get_default_prefs(string &pref)
{
   stringstream out;
   out << "default.depth          : " << defaultdepth <<endl;
   out << "default.verbosity      : " << defaultverb <<endl;
   pref = out.str();
}


void ChipsDefaults::get_plot_prefs(string &pref)
{
   stringstream out;
   const char* temp; 
   out << "plot.stem              : " << curr->plotstem << endl;
   out << "plot.style             : ";
   temp = v_axisstyle_to_string(curr->axisstyle);
   if (temp) out << temp;
   out << " " << endl;
   out << "plot.corner.style      : ";
   temp = v_cornerstyle_to_string(curr->cornerstyle);
   if (temp) out << temp;
   out << " " << endl;
   out << "plot.leftmargin        : " << curr->plotlmargin << endl;
   out << "plot.rightmargin       : " << curr->plotrmargin << endl;
   out << "plot.topmargin         : " << curr->plottmargin << endl;
   out << "plot.bottommargin      : " << curr->plotbmargin << endl;
   out << "plot.title.xpos        : " << curr->titlexpos << endl;
   out << "plot.title.ypos        : " << curr->titleypos << endl;
   out << "plot.title.depth       : ";
   if (curr->titledepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else 
      out  << curr->titledepth << endl;
   out << "plot.title.angle       : " << curr->titleangle << endl;
   out << "plot.title.halign      : ";
   if ((temp = hoffset_to_string(curr->titlehalign)) != NULL)
      out << temp << endl;
   else 
      out << curr->titlehalign << endl;
   out << "plot.title.valign      : ";
   if ((temp = voffset_to_string(curr->titlevalign)) != NULL)
      out << temp << endl;
   else
      out << curr->titlevalign << endl;
   out << "plot.title.color       : " << curr->titlecolor << endl;
   out << "plot.title.size        : " << curr->titlesize << endl;
   out << "plot.title.font        : " << curr->titlefont << endl;
   out << "plot.title.fontstyle   : ";
   temp = v_fontstyle_to_string(curr->titlefontstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "plot.aspect.ratio      : " << curr->plotratio << endl;
   pref =  out.str(); 
}

void ChipsDefaults::get_axis_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "axis.x.stem            : " << curr->axisxstem << endl;
   out << "axis.y.stem            : " << curr->axisystem << endl;
   out << "axis.x.label.text      : " << curr->axisxlabel << endl;
   out << "axis.y.label.text      : " << curr->axisylabel << endl;
   out << "axis.x.label.angle     : " << curr->axlabxangle << endl;
   out << "axis.y.label.angle     : " << curr->axlabyangle << endl;
   out << "axis.color             : " << curr->axiscolor << endl;
   out << "axis.thickness         : " << curr->axisthickness << endl;
   out << "axis.depth             : ";
   if (curr->axisdepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else 
      out  << curr->axisdepth << endl;
   out << "axis.automin           : ";
   if (curr->axis_automin) out << "true" << endl; else out << "false" << endl;
   out << "axis.automax           : ";
   if (curr->axis_automax) out << "true" << endl; else out << "false" << endl;
   out << "axis.pad               : " << curr->axis_pad << endl;
   out << "axis.offset.parallel   : " << curr->axis_offsetpar << endl;
   out << "axis.offset.perpendicular : " << curr->axis_offsetperp << endl;
   out << "axis.tickformat        : " << curr->tickformat << endl;
   out << "axis.ticklabel.style   : ";
   temp = v_tickstyle_to_string(curr->ticklabstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "axis.ticklabel.color   : " << curr->ticklabcolor << endl;
   out << "axis.ticklabel.size    : " << curr->ticklabsize << endl;
   out << "axis.ticklabel.angle   : " << curr->ticklabangle << endl;
   out << "axis.ticklabel.halign  : ";
   if ((temp = hoffset_to_string(curr->ticklabhalign)) != NULL)
      out << temp << endl;
   else 
      out << curr->ticklabhalign << endl;
   out << "axis.ticklabel.valign  : ";
   if ((temp = voffset_to_string(curr->ticklabvalign)) != NULL)
      out << temp << endl;
   else 
      out << curr->ticklabvalign << endl;
   out << "axis.ticklabel.font    : " << curr->ticklabfont << endl;
   out << "axis.ticklabel.fontstyle : ";
   temp = v_fontstyle_to_string(curr->ticklabfontstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "axis.ticklabel.offset  : " << curr->ticklaboffset << endl;
   out << "axis.label.color       : " << curr->axlabcolor << endl;
   out << "axis.label.size        : " << curr->axlabsize << endl;
   out << "axis.label.font        : " << curr->axlabfont << endl;
   out << "axis.label.fontstyle   : ";
   temp = v_fontstyle_to_string(curr->axlabfontstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "axis.label.halign      : ";
   if ((temp = hoffset_to_string(curr->axlabhalign)) != NULL)
      out << temp << endl;
   else
      out << curr->axlabhalign << endl;
   out << "axis.label.valign      : ";
   if ((temp = voffset_to_string(curr->axlabvalign)) != NULL)
      out << temp << endl;
   else
      out << curr->axlabvalign << endl;
   out << "axis.majorgrid.visible : ";
   if (curr->majorgridvisible)
      out <<"true" << endl;
   else
      out << "false" << endl;
   out << "axis.majorgrid.color   : " << curr->majorgridcolor << endl;
   out << "axis.majorgrid.style   : ";
   temp = v_line_to_string(curr->majorgridstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "axis.majorgrid.thickness : " << curr->majorgridthickness << endl;
   out << "axis.minorgrid.visible : ";
   if (curr->minorgridvisible)
      out <<"true" << endl;
   else
      out << "false" << endl;
   out << "axis.minorgrid.color   : " << curr->minorgridcolor << endl;
   out << "axis.minorgrid.style   : ";
   temp = v_line_to_string(curr->minorgridstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "axis.minorgrid.thickness : " << curr->minorgridthickness << endl;
   out << "axis.majortick.mode    : " << curr->majortickmode << endl;
   out << "axis.majortick.interval: " << curr->majortickinterval << endl;
   out << "axis.majortick.count   : " << curr->majortickcount << endl;
   out << "axis.majortick.thickness : " << curr->majortickthickness << endl;
   out << "axis.majortick.length  : " << curr->majorticklength << endl;
   out << "axis.majortick.color   : " << curr->majortickcolor << endl;
   out << "axis.majortick.style   : ";
   temp = v_tickstyle_to_string(curr->majortickstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "axis.majortick.visible : ";
   if (curr->majortickvisible)
      out <<"true" << endl;
   else
      out << "false" << endl;
   out << "axis.minortick.mode    : " << curr->minortickmode << endl;
   out << "axis.minortick.interval: " << curr->minortickinterval << endl;
   out << "axis.minortick.count   : " << curr->minortickcount << endl;
   out << "axis.minortick.thickness : " << curr->minortickthickness << endl;
   out << "axis.minortick.length  : " << curr->minorticklength << endl;
   out << "axis.minortick.color   : " << curr->minortickcolor << endl;
   out << "axis.minortick.style   : ";
   temp = v_tickstyle_to_string(curr->minortickstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "axis.minortick.visible : ";
   if (curr->minortickvisible)
      out <<"true" << endl;
   else
      out << "false" << endl;
   pref =  out.str(); 
}

void ChipsDefaults::get_curve_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "curve.stem             : " << curr->curvestem << endl;
   out << "curve.baddata          : ";
   temp = v_baddatamode_to_string(curr->curvebaddata);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "curve.depth            : ";
   if (curr->curvedepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->curvedepth << endl;
   out << "curve.line.color       : " << curr->curvelinecolor << endl;
   out << "curve.line.thickness   : " << curr->curvelinethickness << endl;
   out << "curve.line.style       : ";
   temp = v_line_to_string(curr->curvelinestyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "curve.symbol.color     : " << curr->curvesymbolcolor << endl;
   out << "curve.symbol.style     : ";
   temp =  v_symbol_to_string(curr->curvesymbolstyle);
   if (temp)  out << temp;
   out << endl;
   out << "curve.symbol.size      : " << curr->curvesymbolsize << endl;
   out << "curve.symbol.angle     : " << curr->curvesymbolangle << endl;
   out << "curve.symbol.fill      : ";
   if (curr->curvesymbolfill) out <<"true" << endl; else out << "false" << endl;
   out << "curve.err.color        : " << curr->errcolor << endl;
   out << "curve.err.thickness    : " << curr->errthickness << endl;
   out << "curve.err.style        : ";
   temp = v_errstyle_to_string(curr->errstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "curve.err.up           : ";
   if (curr->errup) out <<"on" << endl; else out << "off" << endl;
   out << "curve.err.down         : ";
   if (curr->errdown) out <<"on" << endl; else out << "off" << endl;
   out << "curve.err.left         : ";
   if (curr->errleft) out <<"on" << endl; else out << "off" << endl;
   out << "curve.err.right        : ";
   if (curr->errright) out <<"on" << endl; else out << "off" << endl;
   out << "curve.err.caplength    : " << curr->errcaplen << endl; 
   out << "curve.limit.length     : " << curr->limitlength << endl;
   out << "curve.limit.override   : ";
   if (curr->limitoverride) out <<"true" << endl; else out << "false" << endl;
   pref = out.str(); 
}

void ChipsDefaults::get_histogram_prefs(string &pref)
{
   stringstream out;
   const char* temp; 
   out << "histogram.stem         : " << curr->histstem << endl;
   out << "histogram.depth        : ";
   if (curr->histdepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->histdepth << endl;
   out << "histogram.line.color   : " << curr->histlinecolor << endl;
   out << "histogram.line.thickness: " << curr->histlinethickness << endl;
   out << "histogram.line.style   : ";
   temp = v_line_to_string(curr->histlinestyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "histogram.symbol.color : " << curr->histsymbolcolor << endl;
   out << "histogram.symbol.style : ";
   temp =  v_symbol_to_string(curr->histsymbolstyle);
   if (temp)  out << temp;
   out << endl;
   out << "histogram.symbol.size  : " << curr->histsymbolsize << endl;
   out << "histogram.symbol.angle : " << curr->histsymbolangle << endl;
   out << "histogram.symbol.fill  : ";
   if (curr->histsymbolfill) out <<"true" << endl; else out << "false" << endl;
   out << "histogram.err.color    : " << curr->histerrcolor << endl;
   out << "histogram.err.thickness: " << curr->histerrthickness << endl;
   out << "histogram.err.style    : ";
   temp = v_errstyle_to_string(curr->histerrstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "histogram.err.up       : ";
   if (curr->histerrup) out <<"on" << endl; else out << "off" << endl;
   out << "histogram.err.down     : ";
   if (curr->histerrdown) out <<"on" << endl; else out << "off" << endl;
   out << "histogram.err.caplength: " << curr->histerrcaplen << endl; 
   out << "histogram.dropline     : ";
   if (curr->histdropline) out <<"on" << endl; else out << "off" << endl;
   out << "histogram.fill.color   : " << curr->histfillcolor << endl;
   out << "histogram.fill.opacity : " << curr->histfillopacity << endl;
   out << "histogram.fill.style   : ";
   temp = v_fill_to_string(curr->histfillstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   pref = out.str(); 
}

void ChipsDefaults::get_contour_prefs(string &pref)
{
   stringstream out;
   const char* temp; 
   out << "contour.stem           : " << curr->contourstem << endl;
   out << "contour.color          : " << curr->contourcolor << endl;
   out << "contour.thickness      : " << curr->contourthickness << endl;
   out << "contour.levels         : [";
   for (unsigned int ii=0; ii<curr->contourlevels.size (); ii++)
   {
      out << curr->contourlevels[ii];
      if (ii != curr->contourlevels.size ()-1)
         out << ", ";
   }
   out << "]" << endl;
   out << "contour.numlevels      : " << curr->contournumlevels << endl;
   out << "contour.style          : ";
   temp = v_line_to_string(curr->contourstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "contour.algorithm      : ";
   temp = v_contour_to_string(curr->contouralgorithm);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "contour.mode           : ";
   temp = v_tickmode_to_string(curr->contourmode);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "contour.interval       : " << curr->contourinterval << endl;
   out << "contour.depth          : ";
   if (curr->contourdepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->contourdepth << endl;
   pref = out.str(); 
}

void ChipsDefaults::get_region_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "region.stem            : " << curr->regionstem << endl;
   out << "region.fill.color      : " << curr->regionfillcolor << endl;
   out << "region.edge.thickness  : " << curr->regionthickness << endl;
   out << "region.edge.style      : ";
   temp = v_line_to_string(curr->regionstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "region.fill.style      : ";
   temp = v_fill_to_string(curr->regionfillstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "region.edge.color      : " << curr->regioncolor << endl;
   out << "region.angle           : " << curr->regionangle << endl;
   out << "region.opacity         : " << curr->regionopacity << endl;
   out << "region.depth           : ";
   if (curr->regiondepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->regiondepth << endl;
   pref = out.str();
}

void ChipsDefaults::get_label_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "label.stem             : " << curr->labelstem << endl;
   out << "label.font             : " << curr->labelfont << endl;
   out << "label.color            : " << curr->labelcolor << endl;
   out << "label.size             : " << curr->labelsize << endl;
   out << "label.fontstyle        : ";
   temp = v_fontstyle_to_string(curr->labelstyle);
   if (temp)  out << temp;
   out << " " << endl; 
   out << "label.angle            : " << curr->labelangle << endl;
   out << "label.halign           : ";
   if ((temp = hoffset_to_string(curr->labelhalign)) != NULL)
      out << temp << endl;
   else
      out << curr->labelhalign << endl;
   out << "label.valign           : ";
   if ((temp = voffset_to_string(curr->labelvalign)) != NULL)
      out << temp << endl;
   else
      out << curr->labelvalign << endl;
   out << "label.depth            : ";
   if (curr->labeldepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->labeldepth << endl;
   pref = out.str();
}


void ChipsDefaults::get_line_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "line.stem              : " << curr->linestem << endl;
   out << "line.color             : " << curr->linecolor << endl;
   out << "line.thickness         : " << curr->linethickness << endl;
   out << "line.depth             : ";
   if (curr->linedepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->linedepth << endl;
   out << "line.style             : ";
   temp =  v_line_to_string(curr->linestyle);
   if (temp)  out << temp;
   out << endl;
   out << "line.extend            : ";
   temp =  v_line_extension_to_string(curr->lineextend);
   if (temp)  out << temp;
   out << endl;
   pref = out.str();
}


void ChipsDefaults::get_point_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "point.stem             : " << curr->pointstem << endl;
   out << "point.color            : " << curr->pointcolor << endl;
   out << "point.style            : " ;
   temp =  v_symbol_to_string(curr->pointstyle);
   if (temp)  out << temp;
   out << endl;
   out << "point.size             : " << curr->pointsize << endl;
   out << "point.fill             : ";
   if (curr->pointfill) out <<"true" << endl; else out << "false" << endl;
   out << "point.angle            : " << curr->pointangle << endl;
   out << "point.depth            : ";
   if (curr->pointdepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->pointdepth << endl;
   pref = out.str();
}

void ChipsDefaults::get_image_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "image.stem             : " << curr->imagestem << endl;
   out << "image.colormap         : ";
   temp =  v_colormap_to_string(curr->imagecmap);
   if (temp)  out << temp;
   out << endl;
   out << "image.interpolation    : ";
   temp =  v_interpolation_to_string(curr->image_interp);
   if (temp)  out << temp;
   out << endl;
   out << "image.depth            : ";
   if (curr->imagedepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->imagedepth << endl;
   out << "image.alpha            : [" << curr->imagealpha[0]<<","<< 
      curr->imagealpha[1]<<"]"<<endl;
   out << "image.threshold        : [" << curr->imagethreshold[0]<<","<<
      curr->imagethreshold[1]<<"]"<<endl;
   out << "image.invert_colormap  : ";
   if (curr->imageinvertcmap) out <<"true" << endl; else out << "false" << endl;
   out << "image.colormap_size    : " << curr->imagecmapsize << endl;
   out << "image.colormap_interpolate : ";
   if (curr->imagecmapinterp) out <<"true" << endl; else out << "false" << endl;
   out << "image.scale_channels : ";
   if (curr->imagescalechannels) out <<"true" << endl; else out << "false" << endl;
   
   pref = out.str();
}

void ChipsDefaults::get_colorbar_prefs(string &pref)
{
   stringstream out;
   const char* temp;
   out << "colorbar.stem          : " << curr->cbarstem << endl;
   out << "colorbar.width: " << curr->cbarwidth << endl;
   out << "colorbar.length: " << curr->cbarlength << endl;
   out << "colorbar.border.visible: ";
   if (curr->cbarbordervis) out << "true" << endl; else out << "false" << endl;
   out << "colorbar.depth         : ";
   if (curr->cbardepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->imagedepth << endl;
   out << "colorbar.halign        : ";
   if ((temp = hoffset_to_string(curr->cbarhalign)) != NULL)
      out << temp << endl;
   else
      out << curr->cbarhalign << endl;
   out << "colorbar.valign        : ";
   if ((temp = voffset_to_string(curr->cbarvalign)) != NULL)
      out << temp << endl;
   else
      out << curr->cbarvalign << endl;
   out << "colorbar.orientation   : ";
   if (curr->cbarorient == 1)  
      out  << "vertical" << endl;
   else
      out  << "horizontal" << endl;
   out << "colorbar.tick.mode     : ";
   if (curr->cbartickmode == 0)  
      out  << "none" << endl;
   else if (curr->cbartickmode == 2)  
      out  << "nice" << endl;
   else
      out  << "ends" << endl;
   out << "colorbar.tick.location : ";
   temp = v_tickstyle_to_string(curr->cbartickloc);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "colorbar.tick.thickness: " << curr->cbartickthickness << endl;
   out << "colorbar.tick.length   : " << curr->cbarticklength << endl;
   out << "colorbar.tick.color    : " << curr->cbartickcolor << endl;
   out << "colorbar.tick.visible  : ";
   if (curr->cbartickvisible)
      out <<"true" << endl;
   else
      out << "false" << endl;
   out << "colorbar.tick.style    : ";
   temp = v_tickstyle_to_string(curr->cbartickstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out <<  " " << endl;
   out << "colorbar.ticklabel.color: " << curr->cbarticklabcolor << endl;
   out << "colorbar.ticklabel.size: " << curr->cbarticklabsize << endl;
   out << "colorbar.ticklabel.angle: " << curr->cbarticklabangle << endl;
   out << "colorbar.ticklabel.halign: ";
   if ((temp = hoffset_to_string(curr->cbarticklabhalign)) != NULL)
      out << temp << endl;
   else
      out << curr->ticklabhalign << endl;
   out << "colorbar.ticklabel.valign: ";
   if ((temp = voffset_to_string(curr->cbarticklabvalign)) != NULL)
      out << temp << endl;
   else
      out << curr->ticklabvalign << endl;
   out << "colorbar.ticklabel.font: " << curr->ticklabfont << endl;
   out << "colorbar.ticklabel.fontstyle: ";
   temp = v_fontstyle_to_string(curr->cbarticklabfontstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "colorbar.ticklabel.visible: ";
   if (curr->cbarticklabvisible)
      out <<"true" << endl;
   else
      out << "false" << endl;
   out << "colorbar.ticklabel.offset: " << curr->cbarticklaboffset << endl;
   out << "colorbar.label.text    : " << curr->cbarlabtext << endl;
   out << "colorbar.label.angle   : " << curr->cbarlabangle << endl;
   out << "colorbar.label.halign  : ";
   if ((temp = hoffset_to_string(curr->cbarlabhalign)) != NULL)
      out << temp << endl;
   else
      out << curr->cbarlabhalign << endl;
   out << "colorbar.label.valign  : ";
   if ((temp = voffset_to_string(curr->cbarlabvalign)) != NULL)
      out << temp << endl;
   else
      out << curr->cbarlabvalign << endl;
   out << "colorbar.label.location: ";
   temp = v_tickstyle_to_string(curr->cbarlabloc);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "colorbar.label.color   : " << curr->cbarlabcolor << endl;
   out << "colorbar.label.size    : " << curr->cbarlabsize << endl;
   out << "colorbar.label.font    : " << curr->cbarlabfont << endl;
   out << "colorbar.label.fontstyle: ";
   temp = v_fontstyle_to_string(curr->cbarlabfontstyle);
   if (temp)  out << temp;
   out <<  " " << endl;
   out << "colorbar.label.visible : ";
   if (curr->cbarlabvisible)
      out <<"true" << endl;
   else
      out << "false" << endl;
   out << "colorbar.offset.parallel   : " << curr->cbaroffsetpar << endl;
   out << "colorbar.offset.perpendicular : " << curr->cbaroffsetperp << endl;
   out <<  " " << endl;
   pref = out.str();
}

void ChipsDefaults::get_legend_prefs(string &pref)
{
   stringstream out;
#ifdef WLM_LEGEND_SUPPORT
   out << "legend.stem            : " << curr->legendstem << endl;
   out << "legend.color           : " << curr->legendcolor << endl;
   out << "legend.thickness       : " << curr->legendthickness << endl;
   out << "legend.depth           : ";
   if (curr->legenddepth == V_USE_DEFAULT_DEPTH)
      out  << "default" << endl;
   else
      out  << curr->legenddepth << endl;
   out << "legend.transparency    : ";
   if (curr->legendtransparent)
      out <<"true" << endl;
   else 
      out << "false" << endl;
   out << "legend.borders         : ";
   if (curr->legendborders)
      out <<"true" << endl;
   else 
      out << "false" << endl;
#endif
   pref = out.str();
}

int ChipsDefaults::save_file(string in_fname, bool override)
{

   int rval = ERR_STATUS_GOOD;
   string fname;

   // set file name  
   if (in_fname.length() == 0)
      fname = filename;    // no filename provided, use stored filename 
   else 
      fname = in_fname;    // use provided filename

   // determine access status 
   if (fname.length() == 0) 
      rval = ERR_FILE_NOT_SUPPLIED;  
   else if (!access(fname.c_str(), F_OK))
   {
      if ((!curr || (curr->exportclobber == false)) && override == 0)
         rval = ERR_FILE_CLOBBER_OFF;
      else if (access(fname.c_str(), W_OK) != 0)
         rval = ERR_ACCESS_PERMISSION; 
   }

   if (rval == ERR_STATUS_GOOD) 
   {
      ofstream out(fname.c_str());
      if (out)
      {
         string settings;
         get_window_prefs(settings);
         if (settings.length())   out << settings << endl;

         get_frame_prefs(settings);
         if (settings.length())   out << settings << endl;

         get_plot_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_axis_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_curve_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_histogram_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_contour_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_image_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_label_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_line_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_point_prefs(settings);
         if (settings.length())   out << settings << endl;
         
#ifdef WLM_LEGEND_SUPPORT
         get_legend_prefs(settings);
         if (settings.length())   out << settings << endl;
#endif
         get_region_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_colorbar_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_devicecolor_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_export_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_pick_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_info_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         get_default_prefs(settings);
         if (settings.length())   out << settings << endl;

         get_gui_prefs(settings);
         if (settings.length())   out << settings << endl;
         
         out << "chips.version          : " << min_version << endl; 
         out.close();
      }
      else
      {
         rval = ERR_FILE_CREATE_FAIL;    
         if (cmd_manager)
            cmd_manager->add_err(ERR_FILE_CREATE_FAIL,"",0, fname.c_str());
      }
   }
   else if (cmd_manager)   cmd_manager->add_err(rval,"",0,fname.c_str());
   return rval; 
}

int ChipsDefaults::set_preference(string pref, string value)
{
   int rval = ERR_STATUS_GOOD;

   string request = pref;
   request.append(":");
   request.append(value);

   rval = parseLine(request.c_str()); 

   return rval; 
}

char* ChipsDefaults::get_preference(string pref)
{
   char* retbuf = NULL;
   int len; 

   if (pref.length() == 0)
   {
      // get all preference values 
      ostringstream out;
      string everything;
      string settings;
      get_window_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_frame_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_plot_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_axis_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_curve_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_histogram_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_contour_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_image_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_label_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_line_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_point_prefs(settings);
      if (settings.length())   everything.append(settings);
#ifdef WLM_LEGEND_SUPPORT
      get_legend_prefs(settings);
      if (settings.length())   everything.append(settings);
#endif
      get_region_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_colorbar_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_devicecolor_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_export_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_pick_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_info_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_gui_prefs(settings);
      if (settings.length())   everything.append(settings);
      get_default_prefs(settings);
      if (settings.length())   everything.append(settings);

      if ((len = everything.length()) != 0)
      {
         retbuf = (char*) malloc((len+1)*sizeof(char));
         strcpy(retbuf, everything.c_str()); 
      }
   }
   else if (pref == "window")
   {
      string settings;
      get_window_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "frame")
   {
      string settings;
      get_frame_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "plot")
   {
      string settings;
      get_plot_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "axis")
   {
      string settings;
      get_axis_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "curve")
   {
      string settings; 
      get_curve_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "histogram")
   {
      string settings;  
      get_histogram_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "contour")
   {
      string settings; 
      get_contour_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "image")
   {
      string settings; 
      get_image_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str());
      }
   }
   else if (pref == "label")
   {
      string settings;
      get_label_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "line")
   {
      string settings; 
      get_line_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "point")
   {
      string settings;
      get_point_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "legend")
   {
      string settings; 
      get_legend_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "region")
   {
      string settings; 
      get_region_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "colorbar")
   {
      string settings; 
      get_colorbar_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "device")
   {
      string settings;
      get_devicecolor_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str ());
      }
   }
   else if (pref == "export")
   {
      string settings;
      get_export_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str());
      }
   }
   else if (pref == "gui")
   {
      string settings;
      get_gui_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str());
      }
   }
   else if (pref == "info")
   {
      string settings;
      get_info_prefs(settings);
      if (settings.length ())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str());
      }
   }
   else if (pref == "default")
   {
      string settings;
      get_default_prefs(settings);
      if (settings.length())
      {
         retbuf = (char*) malloc((settings.length ()+1)*sizeof(char));
         strcpy(retbuf, settings.c_str());
      }
   }
   else
   {
      ostringstream oss;

      if (pref == "window.stem")
         oss << curr->windowstem << ends; 
      else if (pref == "window.prefix")
         oss << curr->windowprefix << ends; 
      else if (pref == "window.height")
         oss << curr->windowheight << ends; 
      else if (pref == "window.width")
         oss << curr->windowwidth << ends; 
      else if (pref == "window.scaleheight")
         oss << curr->windowscaleheight << ends;
      else if (pref == "window.scalewidth")
         oss << curr->windowscalewidth << ends;
      else if (pref == "window.units")
         oss << v_units_to_string(curr->windowunits) << ends; 
      else if (pref == "window.geom")
         oss << curr->windowgeom << ends; 
      else if (pref == "window.display")
         if (curr->windowdisplay)  oss << "true" << ends;
         else           oss << "false" << ends;
      else if (pref == "window.smoothing")
         if (curr->windowsmoothing)  oss << "true" << ends;
         else           oss << "false" << ends;
      else if (pref == "window.grid.visible")
         if (curr->showgrid)  oss << "true" << ends;
         else           oss << "false" << ends;
      else if (pref == "window.grid.scale")
      {
         const char* wg = c2_gridscale_to_string(curr->gridscale);
         if (wg)  oss << wg << ends;
            else  oss << curr->gridscale << ends;
      }
      else if (pref == "window.bgcolor")
         oss <<  curr->windowcolor << ends; 
      else if (pref == "window.grid.color")
         oss << curr->gridcolor << ends; 
      else if (pref == "window.grid.thickness")
         oss << curr->gridthickness << ends; 
      else if (pref == "window.grid.style")
         oss << v_line_to_string(curr->gridlinestyle) << ends; 
      else if (pref == "window.layout.visible")
         if (curr->showlayout)  oss << "true" << ends;
         else             oss << "false" << ends;
      else if (pref == "window.layout.color")
         oss << curr->layoutcolor << ends; 
      else if (pref == "window.layout.thickness")
         oss << curr->layoutthickness << ends; 
      else if (pref == "window.layout.style")
         oss << v_line_to_string(curr->layoutlinestyle) << ends; 
      else if (pref == "frame.stem")
         oss << curr->framestem << ends; 
      else if (pref == "frame.bgcolor")
         oss << curr->framecolor << ends; 
      else if (pref == "frame.border.color")
         oss << curr->framebordercolor << ends; 
      else if (pref == "frame.border.visible")
         if (curr->framebordervis)  oss << "true" << ends;
         else                 oss <<  "false" << ends;
      else if (pref == "frame.mode")
      {
         if (curr->framemode == V_FRAMEMODE_TILED)  oss << "tiled" << ends;
         else if (curr->framemode == V_FRAMEMODE_SINGLE)  oss << "single" << ends;
         else oss << "placed" << ends;
      }
      else if (pref == "frame.aspect")
         if (curr->frameaspect)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "frame.scale")
         if (curr->framescale)   oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "frame.transparency")
         if (curr->frametransparent)  oss << "true" << ends;
         else             oss << "false" << ends;
      else if (pref == "plot.stem")
         oss << curr->plotstem << ends; 
      else if (pref == "plot.leftmargin")
         oss << curr->plotlmargin << ends; 
      else if (pref == "plot.rightmargin")
         oss << curr->plotrmargin << ends; 
      else if (pref == "plot.topmargin")
         oss << curr->plottmargin << ends; 
      else if (pref == "plot.bottommargin")
         oss << curr->plotbmargin << ends; 
      else if (pref == "plot.title.depth")
         if (curr->titledepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends; 
         else
            oss << curr->titledepth << ends; 
      else if (pref == "plot.title.xpos")
         oss << curr->titlexpos << ends; 
      else if (pref == "plot.title.ypos")
         oss << curr->titleypos << ends; 
      else if (pref == "plot.title.angle")
         oss << curr->titleangle << ends; 
      else if (pref == "plot.title.halign")
      {
         const char* tval = hoffset_to_string(curr->titlehalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->titlehalign << ends;
      }
      else if (pref == "plot.title.valign")
      {
         const char* tval = voffset_to_string(curr->titlevalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->titlevalign << ends;
      }
      else if (pref == "plot.title.color")
         oss << curr->titlecolor << ends; 
      else if (pref == "plot.title.size")
         oss << curr->titlesize << ends; 
      else if (pref == "plot.title.font")
         oss << curr->titlefont << ends;
      else if (pref == "plot.title.fontstyle")
         oss << v_fontstyle_to_string(curr->titlefontstyle) << ends;
      else if (pref == "axis.x.stem")
         oss << curr->axisxstem << ends; 
      else if (pref == "axis.y.stem")
         oss << curr->axisystem << ends; 
      else if (pref == "axis.color")
         oss << curr->axiscolor << ends; 
      else if (pref == "plot.style")
         oss << v_axisstyle_to_string(curr->axisstyle) << ends;
      else if (pref == "plot.corner.style")
         oss << v_cornerstyle_to_string(curr->cornerstyle) << ends;
      else if (pref == "axis.automin")
         if (curr->axis_automin)  oss << "true" << ends;
         else               oss << "false" << ends;
      else if (pref == "axis.automax")
         if (curr->axis_automax)  oss << "true" << ends;
         else               oss << "false" << ends;
      else if (pref == "axis.pad")
         oss << curr->axis_pad << ends; 
      else if (pref == "axis.thickness")
         oss << curr->axisthickness << ends; 
      else if (pref == "axis.depth")
         if (curr->axisdepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->axisdepth << ends;
      else if (pref == "axis.offset.parallel")
         oss << curr->axis_offsetpar << ends; 
      else if (pref == "axis.offset.perpendicular")
         oss << curr->axis_offsetperp << ends; 
      else if (pref == "axis.tickformat")
         oss << curr->tickformat << ends; 
      else if (pref == "axis.ticklabel.style")
         oss << v_tickstyle_to_string(curr->ticklabstyle) << ends;
      else if (pref == "axis.ticklabel.color")
         oss << curr->ticklabcolor << ends;
      else if (pref == "axis.ticklabel.size")
         oss << curr->ticklabsize << ends;
      else if (pref == "axis.ticklabel.angle")
         oss << curr->ticklabangle << ends;
      else if (pref == "axis.ticklabel.halign")
      {
         const char* tval = hoffset_to_string(curr->ticklabhalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->ticklabhalign << ends;
      }
      else if (pref == "axis.ticklabel.valign")
      {
         const char* tval = voffset_to_string(curr->ticklabvalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->ticklabvalign << ends;
      }
      else if (pref == "axis.ticklabel.font")
         oss << curr->ticklabfont << ends;
      else if (pref == "axis.ticklabel.fontstyle")
         oss << v_fontstyle_to_string(curr->ticklabfontstyle) << ends;
      else if (pref == "axis.ticklabel.offset")
         oss << curr->ticklaboffset << ends;
      else if (pref == "axis.label.color")
         oss << curr->axlabcolor << ends; 
      else if (pref == "axis.label.size")
         oss << curr->axlabsize << ends; 
      else if (pref == "axis.label.halign")
      {
         const char* tval = hoffset_to_string(curr->axlabhalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->axlabhalign << ends;
      }
      else if (pref == "axis.label.valign")
      {
         const char* tval = voffset_to_string(curr->axlabvalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->axlabvalign << ends;
      }
      else if (pref == "axis.x.label.angle")
         oss << curr->axlabxangle << ends;
      else if (pref == "axis.y.label.angle")
         oss << curr->axlabyangle << ends;
      else if (pref == "axis.x.label.text")
         oss << curr->axisxlabel << ends;
      else if (pref == "axis.y.label.text")
         oss << curr->axisylabel << ends;
      else if (pref == "axis.label.font")
         oss << curr->axlabfont << ends;
      else if (pref == "axis.label.fontstyle")
         oss << v_fontstyle_to_string(curr->axlabfontstyle) << ends;
      else if (pref == "axis.majorgrid.visible")
         if (curr->majorgridvisible)  oss << "true" << ends;
         else                   oss << "false" << ends;
      else if (pref == "axis.majorgrid.color")
         oss << curr->majorgridcolor << ends;
      else if (pref == "axis.majorgrid.style")
         oss << v_line_to_string(curr->majorgridstyle) << ends;
      else if (pref == "axis.majorgrid.thickness")
         oss << curr->majorgridthickness << ends;
      else if (pref == "axis.minorgrid.visible")
         if (curr->minorgridvisible)  oss << "true" << ends;
         else                   oss << "false" << ends;
      else if (pref == "axis.minorgrid.color")
         oss << curr->minorgridcolor << ends; 
      else if (pref == "axis.minorgrid.style")
         oss << v_line_to_string(curr->minorgridstyle) << ends;
      else if (pref == "axis.minorgrid.thickness")
         oss << curr->minorgridthickness << ends;
      else if (pref == "axis.majortick.mode")
         oss << curr->majortickmode << ends; 
      else if (pref == "axis.majortick.interval")
         oss << curr->majortickinterval << ends; 
      else if (pref == "axis.majortick.count")
         oss << curr->majortickcount << ends; 
      else if (pref == "axis.majortick.thickness")
         oss << curr->majortickthickness << ends; 
      else if (pref == "axis.majortick.length")
         oss << curr->majorticklength << ends; 
      else if (pref == "axis.majortick.color")
         oss << curr->majortickcolor << ends; 
      else if (pref == "axis.majortick.style")
         oss << v_tickstyle_to_string(curr->majortickstyle) << ends;
      else if (pref == "axis.majortick.visible")
         if (curr->majortickvisible)  oss << "true" << ends;
         else                   oss << "false" << ends;
      else if (pref == "axis.minortick.mode")
         oss << curr->minortickmode << ends;
      else if (pref == "axis.minortick.interval")
         oss << curr->minortickinterval << ends;
      else if (pref == "axis.minortick.count")
         oss << curr->minortickcount << ends;
      else if (pref == "axis.minortick.thickness")
         oss << curr->minortickthickness << ends;
      else if (pref == "axis.minortick.length")
         oss << curr->minorticklength << ends;
      else if (pref == "axis.minortick.color")
         oss << curr->minortickcolor << ends;
      else if (pref == "axis.minortick.style")
         oss << v_tickstyle_to_string(curr->minortickstyle) << ends;
      else if (pref == "axis.minortick.visible")
         if (curr->minortickvisible)  oss << "true" << ends;
         else                   oss << "false" << ends;
      else if (pref == "curve.stem")
         oss << curr->curvestem << ends; 
      else if (pref == "curve.baddata")
         oss << v_baddatamode_to_string(curr->curvebaddata) << ends;
      else if (pref == "curve.depth")
         if (curr->curvedepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->curvedepth << ends;
      else if (pref == "curve.line.color")
         oss << curr->curvelinecolor << ends; 
      else if (pref == "curve.line.style")
         oss << v_line_to_string(curr->curvelinestyle) << ends; 
      else if (pref == "curve.line.thickness")
         oss << curr->curvelinethickness << ends; 
      else if (pref == "curve.symbol.color")
         oss << curr->curvesymbolcolor << ends; 
      else if (pref == "curve.symbol.style")
         oss << v_symbol_to_string(curr->curvesymbolstyle) << ends; 
      else if (pref == "curve.symbol.size")
         oss << curr->curvesymbolsize << ends; 
      else if (pref == "curve.symbol.angle")
         oss << curr->curvesymbolangle << ends; 
      else if (pref == "curve.symbol.fill")
         if (curr->curvesymbolfill)  oss << "true" << ends;
         else                  oss << "false" << ends;
      else if (pref == "curve.err.color")
         oss << curr->errcolor << ends;
      else if (pref == "curve.err.style")
         oss << v_errstyle_to_string(curr->errstyle) << ends;
      else if (pref == "curve.err.up")
         if (curr->errup)           oss << "on" << ends;
         else                 oss << "off" << ends;
      else if (pref == "curve.err.down")
         if (curr->errdown)         oss << "on" << ends;
         else                 oss << "off" << ends;
      else if (pref == "curve.err.left")
         if (curr->errleft)         oss << "on" << ends;
         else                 oss << "off" << ends;
      else if (pref == "curve.err.right")
         if (curr->errright)        oss << "on" << ends;
         else                 oss << "off" << ends;
      else if (pref == "curve.err.thickness")
         oss << curr->errthickness << ends;
      else if (pref == "curve.err.caplength")
         oss << curr->errcaplen << ends;
      else if (pref == "curve.limit.length")
         oss << curr->limitlength << ends;
      else if (pref == "curve.limit.override")
         if (curr->limitoverride)  oss << "true" << ends;
         else                  oss << "false" << ends;
      else if (pref == "histogram.stem")
         oss << curr->histstem << ends;
      else if (pref == "histogram.depth")
         if (curr->histdepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->histdepth << ends;
      else if (pref == "histogram.line.color")
         oss << curr->histlinecolor << ends;
      else if (pref == "histogram.line.style")
         oss << v_line_to_string(curr->histlinestyle) << ends;
      else if (pref == "histogram.line.thickness")
         oss << curr->histlinethickness << ends;
      else if (pref == "histogram.symbol.color")
         oss << curr->histsymbolcolor << ends;
      else if (pref == "histogram.symbol.style")
         oss << v_symbol_to_string(curr->histsymbolstyle) << ends;
      else if (pref == "histogram.symbol.size")
         oss << curr->histsymbolsize << ends;
      else if (pref == "histogram.symbol.angle")
         oss << curr->histsymbolangle << ends; 
      else if (pref == "histogram.symbol.fill")
         if (curr->histsymbolfill)  oss << "true" << ends;
         else                  oss << "false" << ends;
      else if (pref == "histogram.err.color")
         oss << curr->histerrcolor << ends;
      else if (pref == "histogram.err.style")
         oss << v_errstyle_to_string(curr->histerrstyle) << ends;
      else if (pref == "histogram.err.up")
         if (curr->histerrup)           oss << "on" << ends;
         else                 oss << "off" << ends;
      else if (pref == "histogram.err.down")
         if (curr->histerrdown)         oss << "on" << ends;
         else                 oss << "off" << ends;
      else if (pref == "histogram.dropline")
         if (curr->histdropline)        oss << "on" << ends;
         else                 oss << "off" << ends;
      else if (pref == "histogram.err.thickness")
         oss << curr->histerrthickness << ends;
      else if (pref == "histogram.err.caplength")
         oss << curr->histerrcaplen << ends;
      else if (pref == "histogram.fill.color")
         oss << curr->histfillcolor << ends;
      else if (pref == "histogram.fill.opacity")
         oss << curr->histfillopacity << ends;
      else if (pref == "histogram.fill.style")
         oss << v_fill_to_string(curr->histfillstyle) << ends; 
      else if (pref == "contour.stem")
         oss << curr->contourstem << ends;
      else if (pref == "contour.depth")
         if (curr->contourdepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->contourdepth << ends;
      else if (pref == "contour.color")
         oss << curr->contourcolor << ends;
      else if (pref == "contour.thickness")
         oss << curr->contourthickness << ends;
      else if (pref == "contour.levels")
      {
         oss << "[";
         for (unsigned int ii=0; ii<curr->contourlevels.size (); ii++)
         {
            oss << curr->contourlevels[ii];
            if (ii != curr->contourlevels.size ()-1)
               oss << ", ";
         }
         oss << "]" << ends;
      }
      else if (pref == "contour.numlevels")
         oss << curr->contournumlevels << ends;
      else if (pref == "contour.style")
         oss << v_line_to_string(curr->contourstyle) << ends;
      else if (pref == "contour.algorithm")
         oss << v_contour_to_string(curr->contouralgorithm) << ends;
      else if (pref == "contour.mode")
         oss << v_tickmode_to_string(curr->contourmode) << ends;
      else if (pref == "contour.interval")
         oss << curr->contourinterval << ends;
      else if (pref == "label.stem")
         oss << curr->labelstem << ends;
      else if (pref == "label.color")
         oss << curr->labelcolor << ends;
      else if (pref == "label.angle")
         oss << curr->labelangle << ends;
      else if (pref == "label.size")
         oss << curr->labelsize << ends;
      else if (pref == "label.halign")
      {
         const char* tval = hoffset_to_string(curr->labelhalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->labelhalign << ends;
      }
      else if (pref == "label.valign")
      {
         const char* tval = voffset_to_string(curr->labelvalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->labelvalign << ends;
      }
      else if (pref == "label.depth")
         if (curr->labeldepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->labeldepth << ends;
      else if (pref == "label.font")
         oss << curr->labelfont << ends;
      else if (pref == "label.fontstyle")
         oss << v_fontstyle_to_string(curr->labelstyle) << ends;
      else if (pref == "line.stem")
         oss << curr->linestem << ends; 
      else if (pref == "line.color")
         oss << curr->linecolor << ends; 
      else if (pref == "line.thickness")
         oss << curr->linethickness << ends; 
      else if (pref == "line.depth")
         if (curr->linedepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->linedepth << ends;
      else if (pref == "line.style")
         oss << v_line_to_string(curr->linestyle) << ends; 
      else if (pref == "line.extend")
         oss << v_line_extension_to_string (curr->lineextend) << ends;
      else if (pref == "point.stem")
         oss << curr->pointstem << ends; 
      else if (pref == "point.color")
         oss << curr->pointcolor << ends; 
      else if (pref == "point.style")
         oss << v_symbol_to_string(curr->pointstyle) << ends; 
      else if (pref == "point.size")
         oss << curr->pointsize << ends; 
      else if (pref == "point.fill")
        if (curr->pointfill)  oss << "true" << ends;
        else            oss << "false" << ends;
      else if (pref == "point.angle")
         oss << curr->pointangle << ends;
      else if (pref == "point.depth")
         if (curr->pointdepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->pointdepth << ends;
      else if (pref == "image.stem")
         oss << curr->imagestem << ends; 
      else if (pref == "image.depth")
         if (curr->imagedepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->imagedepth << ends;
      else if (pref == "image.colormap")
         oss << v_colormap_to_string(curr->imagecmap) << ends;
      else if (pref == "image.interpolation")
         oss << v_interpolation_to_string(curr->image_interp) << ends;
      else if (pref == "image.alpha")
         oss << "["<<curr->imagealpha[0]<<","<<curr->imagealpha[1]<<"]"<<ends; 
      else if (pref == "image.threshold")
         oss << "["<<curr->imagethreshold[0]<<","<<curr->imagethreshold[1]<<"]"<<ends;
      else if (pref == "image.invert_colormap")
         if (curr->imageinvertcmap)
            oss << "true" << ends;
         else
            oss << "false" << ends;
      else if (pref == "image.colormap_size")
         oss << curr->imagecmapsize << ends;
      else if (pref == "image.colormap_interpolate")
         if (curr->imagecmapinterp)
            oss << "true" << ends;
         else
            oss << "false" << ends;
      else if (pref == "image.scale_channels")
         if (curr->imagescalechannels)
            oss << "true" << ends;
         else
            oss << "false" << ends;
      else if (pref == "legend.stem")
         oss << curr->legendstem << ends; 
      else if (pref == "legend.color")
         oss << curr->legendcolor << ends; 
      else if (pref == "legend.thickness")
         oss << curr->legendthickness << ends; 
      else if (pref == "legend.transparency")
        if (curr->legendtransparent)  oss << "true" << ends;
        else                    oss << "false" << ends;
      else if (pref == "legend.borders")
        if (curr->legendborders)  oss << "true" << ends;
        else                oss << "false" << ends;
      else if (pref == "legend.depth")
         if (curr->legenddepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->legenddepth << ends;
      else if (pref == "region.stem")
         oss << curr->regionstem << ends;
      else if (pref == "region.fill.color")
         oss << curr->regionfillcolor << ends;
      else if (pref == "region.edge.thickness")
         oss << curr->regionthickness << ends; 
      else if (pref == "region.edge.style")
         oss << v_line_to_string(curr->regionstyle) << ends; 
      else if (pref == "region.fill.style")
         oss << v_fill_to_string(curr->regionfillstyle) << ends; 
      else if (pref == "region.edge.color")
         oss << curr->regioncolor << ends;
      else if (pref == "region.angle")
         oss << curr->regionangle << ends;
      else if (pref == "region.opacity")
         oss << curr->regionopacity << ends;
      else if (pref == "region.depth")
         if (curr->regiondepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->regiondepth << ends;
      else if (pref == "colorbar.stem")
         oss << curr->cbarstem << ends;
      else if (pref == "colorbar.width")
         oss << curr->cbarwidth << ends;
      else if (pref == "colorbar.length")
         oss << curr->cbarlength << ends;
      else if (pref == "colorbar.depth")
         if (curr->cbardepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << curr->cbardepth << ends;
      else if (pref == "colorbar.border.visible")
      {
         if (curr->cbarbordervis)
            oss << "true" << ends;
         else
            oss << "false" << ends;
      }
      else if (pref == "colorbar.halign")
      {
         const char* tval = hoffset_to_string(curr->cbarhalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->cbarhalign << ends;
      }
      else if (pref == "colorbar.valign")
      {
         const char* tval = voffset_to_string(curr->cbarvalign);
         if (tval)  oss << tval << ends;
         else       oss << curr->cbarvalign << ends;
      }
      else if (pref == "colorbar.orientation")
      {
         if (curr->cbarorient == 1)  oss << "vertical" << ends;
         else                        oss << "horizontal" << ends;
      }
      else if (pref == "colorbar.tick.mode")
      {
         if (curr->cbartickmode == 0)  oss << "none" << ends;
         else if (curr->cbartickmode == 2)  oss << "nice" << ends;
         else                          oss << "ends" << ends;
      }
      else if (pref == "colorbar.tick.style")
         oss << v_tickstyle_to_string(curr->cbartickstyle) << ends;
      else if (pref == "colorbar.tick.color")
         oss << curr->cbartickcolor << ends;
      else if (pref == "colorbar.length")
         oss << curr->cbarticklength << ends;
      else if (pref == "colorbar.tick.thickness")
         oss << curr->cbartickthickness << ends;
      else if (pref == "colorbar.tick.location")
         oss << v_tickstyle_to_string(curr->cbartickloc) << ends;
      else if (pref == "colorbar.tick.visible")
         if (curr->cbartickvisible)  oss << "true" << ends;
         else                   oss << "false" << ends;
      else if (pref == "colorbar.tick.length")
         oss << curr->cbarticklength << ends;
      else if (pref == "colorbar.tick.color")
         oss << curr->cbartickcolor << ends;
      else if (pref == "colorbar.tick.visible")
         oss << curr->cbartickvisible << ends;
      else if (pref == "colorbar.tick.style")
         oss << curr->cbartickstyle << ends;
      else if (pref == "colorbar.ticklabel.color")
         oss << curr->cbarticklabcolor << ends;
      else if (pref == "colorbar.ticklabel.angle")
         oss << curr->cbarticklabangle << ends;
      else if (pref == "colorbar.ticklabel.halign")
         oss << curr->cbarticklabhalign << ends;
      else if (pref == "colorbar.ticklabel.valign")
         oss << curr->cbarticklabvalign << ends;
      else if (pref == "colorbar.ticklabel.size")
         oss << curr->cbarticklabsize << ends;
      else if (pref == "colorbar.ticklabel.offset")
         oss << curr->cbarticklaboffset << ends;
      else if (pref == "colorbar.ticklabel.font")
         oss << curr->cbarticklabfont << ends;
      else if (pref == "colorbar.ticklabel.fontstyle")
         oss << curr->cbarticklabfontstyle << ends;
      else if (pref == "colorbar.ticklabel.visible")
         if (curr->cbarticklabvisible)  oss << "true" << ends;
         else                   oss << "false" << ends;
      else if (pref == "colorbar.label.text")
         oss << curr->cbarlabtext << ends;
      else if (pref == "colorbar.label.size")
         oss << curr->cbarlabsize << ends;
      else if (pref == "colorbar.label.angle")
         oss << curr->cbarlabangle << ends;
      else if (pref == "colorbar.label.color")
         oss << curr->cbarlabcolor << ends;
      else if (pref == "colorbar.label.font")
         oss << curr->cbarlabfont << ends;
      else if (pref == "colorbar.label.fontstyle")
         oss << curr->cbarlabfontstyle << ends;
      else if (pref == "colorbar.label.visible")
         if (curr->cbarlabvisible)  oss << "true" << ends;
         else                   oss << "false" << ends;
      else if (pref == "colorbar.label.halign")
         oss << curr->cbarlabhalign << ends;
      else if (pref == "colorbar.label.valign")
         oss << curr->cbarlabvalign << ends;
      else if (pref == "colorbar.label.location")
         oss << v_tickstyle_to_string(curr->cbarlabloc) << ends;
      else if (pref == "colorbar.offset.parallel")
         oss << curr->cbaroffsetpar << ends;
      else if (pref == "colorbar.offset.perpendicular")
         oss << curr->cbaroffsetperp << ends;
      else if (pref == "default.depth")
         oss << defaultdepth << ends; 
      else if (pref == "default.verbosity")
         oss << defaultverb << ends; 
      else if (pref == "foreground.file")
         oss << defaultfgps << ends; 
      else if (pref == "background.file")
         oss << defaultbgps << ends; 
      else if (pref == "foreground.display")
         oss << defaultfgdisp << ends; 
      else if (pref == "background.display")
         oss << defaultbgdisp << ends; 
      else if (pref == "info.coord")
        if (infocoord)  oss << "true" << ends;
        else              oss << "false" << ends;
      else if (pref == "info.current")
        if (infocurrent)  oss << "true" << ends;
        else              oss << "false" << ends;
      else if (pref == "info.depth")
        if (infodepth)  oss << "true" << ends;
        else              oss << "false" << ends;
      else if (pref == "export.fonts")
        if (curr->exportfonts)  oss << "true" << ends;
        else              oss << "false" << ends;
      else if (pref == "export.clobber")
        if (curr->exportclobber)  oss << "true" << ends;
        else              oss << "false" << ends;
      else if (pref == "export.orientation")
        if (curr->exportorient == 0)  oss << "portrait" << ends;
        else                    oss << "landscape" << ends;
      else if (pref == "export.format")
         oss << curr->exportformat << ends; 
      else if (pref == "export.colorscheme")
         oss << v_printfile_to_string(curr->exportcolorscheme) << ends; 
      else if (pref == "export.leftmargin")
         oss << curr->pagelmargin << ends;
      else if (pref == "export.rightmargin")
         oss << curr->pagermargin << ends;
      else if (pref == "export.topmargin")
         oss << curr->pagetmargin << ends;
      else if (pref == "export.bottommargin")
         oss << curr->pagebmargin << ends;
      else if (pref == "export.pagesize"){
	 const char *tmp_str = v_pagesize_to_string(curr->pagewidth, curr->pageheight, curr->pageunits);
	 if (tmp_str)
            oss << tmp_str << ends;
      }
      else if (pref == "export.pagewidth")
         oss << curr->pagewidth << ends;
      else if (pref == "export.pageheight")
         oss << curr->pageheight << ends;
      else if (pref == "export.pageunits"){
	 const char *tmp_str = v_units_to_string(curr->pageunits);
	 if (tmp_str)
            oss << tmp_str << ends;
      }
      else if (pref == "export.scalewidth")
         oss << curr->pagescalewidth << ends;
      else if (pref == "export.scaleheight")
         oss << curr->pagescaleheight << ends;
      else if (pref == "export.keepaspect")
         if (curr->pageaspect)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "export.printdialog")
         if (curr->printdialog)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "export.fittopage")
         if (curr->fittopage)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "export.printmethod")
         oss << curr->printmethod << ends;
      else if (pref == "export.dpi")
         oss << curr->exportdpi << ends;
      else if (pref == "gui.view.toolbar")
         if (curr->toolbarvis)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "gui.view.propertypane" || pref == "gui.view.propertyframe")
         if (curr->proppanevis)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "gui.view.infopane" || pref == "gui.view.infoframe")
         if (curr->infpanevis)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "gui.view.navpane" || pref == "gui.view.navframe")
         if (curr->navpanevis)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "gui.view.shellpane" || pref == "gui.view.shellframe")
         if (curr->shellpanevis)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "gui.toolbar.mode")
         oss << curr->tbarmode << ends; 
      else if (pref == "gui.toolbar.style")
         oss << curr->tbarstyle << ends; 
      else if (pref == "gui.toolbar.zoom")
         if (curr->zoombtn)  oss << "true" << ends;
         else              oss << "false" << ends;
#ifdef CHIPS_GUI_ADDDEL
      else if (pref == "gui.toolbar.delete")
         if (curr->delbtn)  oss << "true" << ends;
         else              oss << "false" << ends;
#endif
      else if (pref == "gui.tooltips")
         if (curr->guitips)  oss << "true" << ends;
         else              oss << "false" << ends;
      else if (pref == "pick.mask")
          oss << pickmask_to_string(pickmask) << ends; 
      else if (pref == "pick.color")
          oss << curr->pickcolor << ends;
      else if (pref == "info.current")
         if (infocurrent)  oss << "true" << ends;
         else                    oss << "false" << ends;
      else if (pref == "info.depth")
         if (infodepth)  oss << "true" << ends;
         else                  oss << "false" << ends;
      else if (pref == "info.coord")
         if (infocoord)  oss << "true" << ends;
         else                  oss << "false" << ends;
      else if (pref == "all.depth") 
         if (alldepth == V_USE_DEFAULT_DEPTH)
            oss << "default" << ends;
         else
            oss << alldepth << ends;
      else if (pref == "all.angle")
         oss << allangle << ends;
      else if (pref == "all.color")
         oss << allcolor << ends;
      else if (pref == "all.bgcolor")
         oss << allbgcolor << ends;
      else if (pref == "all.thickness")
         oss << allthickness << ends;
      else if (pref == "all.linestyle")
         oss << alllinestyle << ends;
      else if (pref == "all.length")
         oss << alllength << ends;
      else if (pref == "all.fill")
         oss << allfill << ends;
      else if (pref == "all.count")
         oss << allcount << ends;
      else if (pref == "all.font")
         oss << allfont << ends;
      else if (pref == "all.fontstyle")
         oss << allfontstyle << ends;
      else if (pref == "all.visible")
         oss << allvisible << ends;
      else if (pref == "all.aspect")
         oss << allaspect << ends;
         
#ifdef WLM_ATT_WARNINGS
      // the following code is set up to add warnings for attributes which
      // do not exist. Currently they are ignored. If this code is included,
      // we need to add a switch to disable the warnings when called internally
      else if (cmd_manager)
         cmd_manager->add_err(ERR_ATTRIBUTE_DNE,"",1,pref.c_str());
#endif
      else if  (pref == "chips.version")
         oss << file_version << ends; 
      
      if ((len = oss.str().length()) > 0)
      {
         retbuf = (char*) malloc((len+1)*sizeof(char));
         strcpy(retbuf, oss.str().c_str ());
      }
   }
   return retbuf; 
}

int ChipsDefaults::set_gridscale(int val)
{
   int rval = ERR_STATUS_GOOD;
   switch(val)
   {
      case V_GRID_SCALE_DEFAULT:      // fallthrough intended
      case V_GRID_SCALE_QTR_INCH:     // fallthrough intended
      case V_GRID_SCALE_HALF_INCH:    // fallthrough intended
      case V_GRID_SCALE_CENTIMETER:   // fallthrough intended
      case V_GRID_SCALE_NORMALIZED:
         curr->gridscale = val;
      break;
      default:
         rval = ERR_DATA_RANGE;
      break;
   }
   return rval;
}

int ChipsDefaults::set_gridcolor(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (c2_valid_color(val.c_str()))
      curr->gridcolor = val;
   else
      rval = ERR_UNKNOWN_COLOR;
   return rval; 
}

int ChipsDefaults::set_gridlinestyle(string val)
{
   return string_to_v_line(val.c_str(), &curr->gridlinestyle);
}

int ChipsDefaults::set_gridthickness(float fval)
{
   int rval = ERR_STATUS_GOOD;
   if ((fval < V_THICK_MIN_VAL) || (fval > V_THICK_MAX_VAL))
      rval = thickness_value_error(fval);
   else
      curr->gridthickness = fval;
   return rval;
}

int ChipsDefaults::set_framemode(int val)
{
   int rval = ERR_STATUS_GOOD;

   switch(val)
   {
      case V_FRAMEMODE_PLACED:      // fallthrough intended 
      case V_FRAMEMODE_SINGLE:      // fallthrough intended 
      case V_FRAMEMODE_TILED:   
         curr->framemode = val;
      break;
      default:
         rval = ERR_DATA_RANGE;  
      break; 
   }
   return rval; 
}

int ChipsDefaults::set_framemode(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (!chips_strcmp(val.c_str(), "tiled"))
      curr->framemode = V_FRAMEMODE_TILED; 
   else if (!chips_strcmp(val.c_str(), "placed"))
      curr->framemode = V_FRAMEMODE_PLACED; 
   else if (!chips_strcmp(val.c_str(), "single"))
      curr->framemode = V_FRAMEMODE_SINGLE; 
   else
      rval = data_subset_error(val.c_str(), "[tiled|placed|single]");
   return rval;
}


int ChipsDefaults::set_framestem(string val)
{
   curr->framestem = val;
   return ERR_STATUS_GOOD;
}


int ChipsDefaults::set_plotstem(string val)
{
   curr->plotstem = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_imagestem(string val)
{
   curr->imagestem = val;
   return ERR_STATUS_GOOD;
}


int ChipsDefaults::set_axisxstem(string val)
{
   curr->axisxstem = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_axisystem(string val)
{
   curr->axisystem = val;
   return ERR_STATUS_GOOD;
}


int ChipsDefaults::set_curvestem(string val)
{
   curr->curvestem = val;
   return ERR_STATUS_GOOD;
}


int ChipsDefaults::set_linestem(string val)
{
   curr->linestem = val;
   return ERR_STATUS_GOOD;
}


int ChipsDefaults::set_pointstem(string val)
{
   curr->pointstem = val;
   return ERR_STATUS_GOOD;
}



int ChipsDefaults::set_pointstyle(string val)
{
   return string_to_v_symbol(val.c_str(), &curr->pointstyle);
}

int ChipsDefaults::set_curvesymbolstyle(string val)
{
   return string_to_v_symbol(val.c_str(), &curr->curvesymbolstyle);
}



int ChipsDefaults::set_pointstyle(V_SYMBOL_TYPE val)
{
   int rval = ERR_STATUS_GOOD;

   if (valid_symbol_type(val))
      curr->pointstyle = val;
   else
      rval = ERR_BAD_SYMBOL;
   return rval;
}


int ChipsDefaults::set_curvesymbolstyle(V_SYMBOL_TYPE val)
{
   int rval = ERR_STATUS_GOOD;

   if (valid_symbol_type(val))
      curr->curvesymbolstyle = val; 
   else
      rval = ERR_BAD_SYMBOL;
   return rval;
}


int ChipsDefaults::set_pointcolor(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (c2_valid_color(val.c_str()))
      curr->pointcolor = val;
   else
      rval = ERR_UNKNOWN_COLOR;
   return rval; 
}

int ChipsDefaults::set_curvesymbolcolor(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (c2_valid_color(val.c_str()))
      curr->curvesymbolcolor = val;
   else
      rval = ERR_UNKNOWN_COLOR;
   return rval;
}

int ChipsDefaults::set_linestyle(string val)
{
   return string_to_v_line(val.c_str(), &curr->linestyle);
}

int ChipsDefaults::set_curvelinestyle(string val)
{
   return string_to_v_line(val.c_str(), &curr->curvelinestyle);
}

int ChipsDefaults::set_linestyle(V_LINE_TYPE val)
{
   int rval = ERR_STATUS_GOOD;

   if (valid_line_type(val))
      curr->linestyle = val;
   else
      rval = ERR_BAD_LINESTYLE;
   return rval;
}

int ChipsDefaults::set_curvelinestyle(V_LINE_TYPE val)
{
   int rval = ERR_STATUS_GOOD;

   if (valid_line_type(val))
      curr->curvelinestyle = val;
   else
      rval = ERR_BAD_LINESTYLE;
   return rval;
}


int ChipsDefaults::set_linecolor(string val)
{
   int rval = ERR_STATUS_GOOD;

   if (c2_valid_color(val.c_str()))
      curr->linecolor = val; 
   else
      rval = ERR_UNKNOWN_COLOR;
   return rval;
}


int ChipsDefaults::set_curvelinecolor(string val)
{
   int rval = ERR_STATUS_GOOD;
   if (c2_valid_color(val.c_str()))
      curr->curvelinecolor = val;
   else
      rval = ERR_UNKNOWN_COLOR;
   return rval;
}

int ChipsDefaults::set_pageleftmargin(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pagelmargin = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pagerightmargin(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pagermargin = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pagetopmargin(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pagetmargin = val;
   return ERR_STATUS_GOOD;

}

int ChipsDefaults::set_pagebottommargin(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pagebmargin = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pagewidth(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pagewidth = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pageheight(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pageheight = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pagescalewidth(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pagescalewidth = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pagescaleheight(float val)
{
   if (val < 0)
      return negative_value_error(val);
   curr->pagescaleheight = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pageunits(string val)
{
   int rval = ERR_STATUS_GOOD;
   rval = string_to_v_units(val.c_str(), &curr->pageunits);
   return rval;
}


int ChipsDefaults::set_pageunits(V_UNITS_TYPE in_val)
{
   int rval = ERR_STATUS_GOOD;

   if (valid_units_type(in_val))
      curr->pageunits = in_val;
   else
      rval = ERR_BAD_UNITS;   // err - invalid value specified
   return rval;
}


int ChipsDefaults::set_fittopage(bool val)
{
   curr->fittopage = val;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::set_pageaspect(bool val)
{
   curr->pageaspect = val;
   return ERR_STATUS_GOOD;
}


int ChipsDefaults::set_overrides(string inlist, string defobj)
{
   int rval = ERR_STATUS_GOOD;
   string::size_type sz; 

   if ((sz = inlist.length()) != 0)
   {
      curr = &overrides;
      
      // initialize the override lits
      if (this->load_overrides)
         default_load_overrides();

      this->load_overrides = false;

      string entry; 

      // need to parse string into lines to call parse line
      char* entries = new char[2*sz];
      strcpy(entries, inlist.c_str());
    
      string::size_type cpos = 0, epos = 0, sqpos, dqpos, bpos, spos;

      while (epos < sz)
      {
         sqpos = inlist.find("'", cpos);   // check single quotes 
         dqpos = inlist.find("\"", cpos);  // check double quotes 
         bpos = inlist.find("[", cpos);    // check bracket lists  
         spos = inlist.find(" ", cpos);    // check spaces  
      
         if (sqpos != string::npos && 
             (sqpos< dqpos || dqpos == string::npos) &&
             (sqpos< bpos || bpos == string::npos) && 
             (sqpos< spos || spos == string::npos))
         {
            epos = inlist.find("'", sqpos+1)+1;
         }
         else if (dqpos != string::npos && 
             (dqpos < sqpos || sqpos == string::npos) &&
             (dqpos < bpos || bpos == string::npos) &&
             (dqpos < spos || spos == string::npos))
         {
            epos = inlist.find("\"", dqpos+1)+1;
         }
         else if (bpos != string::npos &&
             (bpos < sqpos || sqpos == string::npos) &&
             (bpos < dqpos || dqpos == string::npos) &&
             (bpos < spos || spos == string::npos))
         {
            epos = bpos; 
            unsigned int rr = bpos+1, cnt = 1; 
            // handle nested brackets
            while (rr < sz && cnt != 0)
            { 
               if (entries[rr] == '[')  cnt++; 
                  else if (entries[rr] == ']') cnt--;
               rr++; 
            }
            if (cnt == 0 && entries[rr-1] == ']')  epos = rr;
         }
         else if (spos != string::npos)
         {
            epos = spos; 
         }
         else
            epos = sz;

         if (epos > cpos)
         {
            entry = inlist.substr(cpos, epos-cpos);

            string guess = "";
            if (defobj.length())
            {
               guess = defobj;
               guess.append(".");
            }
            guess.append(entry);

            if ((rval = parseLine(guess.c_str(), '=', false)) == ERR_UNKNOWN_PROPERTY)
	    {
	       // retry- using the entry from the input line, erase the last
	       // stored error since the retry will either succeed or set the
	       // correct error
	       cmd_manager->remove_last_err(); 
	       rval = parseLine(entry.c_str(), '=', false);
	    } 
         }
         epos++;  
         cpos = epos;
      }

      delete []entries;
   }
   else
      curr = &defvals; 
   // if we encountered an error- reset to default values
   if (rval !=  ERR_STATUS_GOOD)    curr = &defvals;
   return cmd_manager->get_err_cnt(1); 
}

int ChipsDefaults::reset_defaults()
{
   curr = &defvals;
   this->load_overrides = true;
   return ERR_STATUS_GOOD;
}

int ChipsDefaults::default_load_overrides()
{
   overrides.windowdisplay     = defvals.windowdisplay;     
   overrides.windowsmoothing   = defvals.windowsmoothing;     
   overrides.windowheight      = defvals.windowheight;        
   overrides.windowwidth       = defvals.windowwidth;         
   overrides.windowscaleheight = defvals.windowscaleheight;        
   overrides.windowscalewidth  = defvals.windowscalewidth;         
   overrides.windowunits       = defvals.windowunits;       
   overrides.windowstem        = defvals.windowstem;          
   overrides.windowprefix      = defvals.windowprefix;          
   overrides.windowgeom        = defvals.windowgeom;          
   overrides.windowid          = defvals.windowid;          
   overrides.windowcolor       = defvals.windowcolor;         
   overrides.windowfocus       = defvals.windowfocus;            
   overrides.showgrid          = defvals.showgrid;            
   overrides.gridscale         = defvals.gridscale;           
   overrides.gridthickness     = defvals.gridthickness;       
   overrides.gridcolor         = defvals.gridcolor;           
   overrides.gridlinestyle     = defvals.gridlinestyle;      
   overrides.showlayout        = defvals.showlayout;          
   overrides.layoutthickness   = defvals.layoutthickness;     
   overrides.layoutcolor       = defvals.layoutcolor;         
   overrides.layoutlinestyle   = defvals.layoutlinestyle;; 
   overrides.printmethod       = defvals.printmethod;
   overrides.frameaspect       = defvals.frameaspect;          
   overrides.framescale        = defvals.framescale;          
   overrides.framebordervis    = defvals.framebordervis;      
   overrides.framemode         = defvals.framemode;           
   overrides.framestem         = defvals.framestem;           
   overrides.frameid           = defvals.frameid;          
   overrides.framecolor        = defvals.framecolor;          
   overrides.framebordercolor  = defvals.framebordercolor;    
   overrides.frametransparent  = defvals.frametransparent; 
   overrides.plotstem          = defvals.plotstem;            
   overrides.plotid            = defvals.plotid;            
   overrides.plotlmargin       = defvals.plotlmargin;         
   overrides.plotrmargin       = defvals.plotrmargin;         
   overrides.plottmargin       = defvals.plottmargin;         
   overrides.plotbmargin       = defvals.plotbmargin;         
   overrides.titlexpos         = defvals.titlexpos;           
   overrides.titleypos         = defvals.titleypos;           
   overrides.titleangle        = defvals.titleangle;           
   overrides.titledepth        = defvals.titledepth;
   overrides.titlehalign       = defvals.titlehalign;         
   overrides.titlevalign       = defvals.titlevalign;         
   overrides.titlecolor        = defvals.titlecolor;          
   overrides.titlesize         = defvals.titlesize;           
   overrides.titlefont         = defvals.titlefont;; 
   overrides.titlelabel        = defvals.titlelabel;            
   overrides.titlefontstyle    = defvals.titlefontstyle;; 
   overrides.imagestem         = defvals.imagestem; 
   overrides.imageid           = defvals.imageid; 
   overrides.axisxstem         = defvals.axisxstem;            
   overrides.axisystem         = defvals.axisystem;            
   overrides.axisdepth         = defvals.axisdepth;
   overrides.axisxid           = defvals.axisxid;            
   overrides.axisyid           = defvals.axisyid;            
   overrides.axisxlabel        = defvals.axisxlabel;            
   overrides.axisylabel        = defvals.axisylabel;
   overrides.axisstyle         = defvals.axisstyle;     
   overrides.cornerstyle       = defvals.cornerstyle;     
   overrides.axiscolor         = defvals.axiscolor;           
   overrides.tickformat        = defvals.tickformat;          
   overrides.ticklabstyle      = defvals.ticklabstyle; 
   overrides.ticklabcolor      = defvals.ticklabcolor;        
   overrides.ticklabangle      = defvals.ticklabangle;        
   overrides.ticklabhalign     = defvals.ticklabhalign;       
   overrides.ticklabvalign     = defvals.ticklabvalign;       
   overrides.ticklabsize       = defvals.ticklabsize;         
   overrides.ticklaboffset     = defvals.ticklaboffset;       
   overrides.ticklabfont       = defvals.ticklabfont;         
   overrides.ticklabfontstyle  = defvals.ticklabfontstyle;   
   overrides.axis_automin      = defvals.axis_automin;        
   overrides.axis_automax      = defvals.axis_automax;        
   overrides.axis_pad          = defvals.axis_pad;            
   overrides.axis_offsetperp   = defvals.axis_offsetperp;     
   overrides.axis_offsetpar    = defvals.axis_offsetpar;      
   overrides.axisthickness     = defvals.axisthickness;       
   overrides.majortickmode     = defvals.majortickmode;       
   overrides.majortickinterval = defvals.majortickinterval;   
   overrides.majortickcount    = defvals.majortickcount;    
   overrides.majortickthickness= defvals.majortickthickness;   
   overrides.majorticklength   = defvals.majorticklength;     
   overrides.majortickcolor    = defvals.majortickcolor;      
   overrides.majortickstyle    = defvals.majortickstyle;      
   overrides.minortickmode     = defvals.minortickmode;       
   overrides.minortickinterval = defvals.minortickinterval;   
   overrides.minortickcount    = defvals.minortickcount;    
   overrides.minortickthickness= defvals.minortickthickness;   
   overrides.minorticklength   = defvals.minorticklength;     
   overrides.minortickcolor    = defvals.minortickcolor;      
   overrides.minortickstyle    = defvals.minortickstyle;      
   overrides.majortickvisible  = defvals.majortickvisible;    
   overrides.minortickvisible  = defvals.minortickvisible;    
   overrides.axlabcolor        = defvals.axlabcolor;          
   overrides.axlabsize         = defvals.axlabsize;           
   overrides.axlabfont         = defvals.axlabfont;           
   overrides.axlabhalign       = defvals.axlabhalign;         
   overrides.axlabvalign       = defvals.axlabvalign;         
   overrides.axlabxangle       = defvals.axlabxangle;
   overrides.axlabyangle       = defvals.axlabyangle;
   overrides.axlabfontstyle    = defvals.axlabfontstyle;    
   overrides.majorgridvisible  = defvals.majorgridvisible;    
   overrides.majorgridcolor    = defvals.majorgridcolor;      
   overrides.majorgridthickness= defvals.majorgridthickness; 
   overrides.majorgridstyle    = defvals.majorgridstyle;     
   overrides.minorgridvisible  = defvals.minorgridvisible;    
   overrides.minorgridcolor    = defvals.minorgridcolor;      
   overrides.minorgridthickness= defvals.minorgridthickness; 
   overrides.minorgridstyle    = defvals.minorgridstyle; 
   overrides.pointstyle        = defvals.pointstyle;       
   overrides.pointcolor        = defvals.pointcolor;          
   overrides.pointstem         = defvals.pointstem;           
   overrides.pointid           = defvals.pointid;           
   overrides.pointfill         = defvals.pointfill;           
   overrides.pointsize         = defvals.pointsize;           
   overrides.pointangle        = defvals.pointangle;           
   overrides.pointdepth        = defvals.pointdepth;
   overrides.linestyle         = defvals.linestyle;          
   overrides.linecolor         = defvals.linecolor;           
   overrides.linethickness     = defvals.linethickness;       
   overrides.linestem          = defvals.linestem;           
   overrides.lineid            = defvals.lineid;           
   overrides.linedepth         = defvals.linedepth;
   overrides.lineextend        = defvals.lineextend;
   overrides.curvestem         = defvals.curvestem;           
   overrides.curvebaddata      = defvals.curvebaddata;           
   overrides.curveid           = defvals.curveid;           
   overrides.curvelinecolor    = defvals.curvelinecolor;      
   overrides.curvelinestyle    = defvals.curvelinestyle;     
   overrides.curvelinethickness= defvals.curvelinethickness; 
   overrides.curvesymbolstyle  = defvals.curvesymbolstyle;   
   overrides.curvesymbolcolor  = defvals.curvesymbolcolor;    
   overrides.curvesymbolfill   = defvals.curvesymbolfill;     
   overrides.curvesymbolsize   = defvals.curvesymbolsize;     
   overrides.curvesymbolangle  = defvals.curvesymbolangle;     
   overrides.curvedepth        = defvals.curvedepth;
   overrides.errup             = defvals.errup;               
   overrides.errdown           = defvals.errdown;             
   overrides.errleft           = defvals.errleft;             
   overrides.errright          = defvals.errright;            
   overrides.errstyle          = defvals.errstyle;       
   overrides.errcolor          = defvals.errcolor;            
   overrides.errthickness      = defvals.errthickness;        
   overrides.errcaplen         = defvals.errcaplen;        
   overrides.limitlength       = defvals.limitlength;
   overrides.limitoverride     = defvals.limitoverride;
   overrides.histstem          = defvals.histstem;         
   overrides.histid            = defvals.histid;         
   overrides.histsymbolstyle   = defvals.histsymbolstyle;    
   overrides.histsymbolcolor   = defvals.histsymbolcolor;     
   overrides.histsymbolfill    = defvals.histsymbolfill;      
   overrides.histsymbolsize    = defvals.histsymbolsize;      
   overrides.histsymbolangle   = defvals.histsymbolangle;     
   overrides.histlinestyle     = defvals.histlinestyle;      
   overrides.histlinecolor     = defvals.histlinecolor;       
   overrides.histlinethickness = defvals.histlinethickness;   
   overrides.histerrup         = defvals.histerrup;           
   overrides.histerrdown       = defvals.histerrdown;         
   overrides.histdropline      = defvals.histdropline;        
   overrides.histerrstyle      = defvals.histerrstyle;   
   overrides.histerrcolor      = defvals.histerrcolor;        
   overrides.histerrthickness  = defvals.histerrthickness;    
   overrides.histerrcaplen     = defvals.histerrcaplen;    
   overrides.histdepth         = defvals.histdepth;
   overrides.histfillcolor     = defvals.histfillcolor;
   overrides.histfillopacity   = defvals.histfillopacity;
   overrides.histfillstyle     = defvals.histfillstyle;
   overrides.contourstem       = defvals.contourstem;         
   overrides.contourid         = defvals.contourid;         
   overrides.contourcolor      = defvals.contourcolor;        
   overrides.contourstyle      = defvals.contourstyle;       
   overrides.contourthickness  = defvals.contourthickness;    
   overrides.contourlevels     = defvals.contourlevels;
   overrides.contournumlevels  = defvals.contournumlevels;     
   overrides.contouralgorithm  = defvals.contouralgorithm;     
   overrides.contourmode       = defvals.contourmode;
   overrides.contourinterval   = defvals.contourinterval; 
   overrides.contourdepth      = defvals.contourdepth;
   overrides.imagestem         = defvals.imagestem;
   overrides.imagedepth        = defvals.imagedepth;
   overrides.imagecmap         = defvals.imagecmap;
   overrides.image_interp      = defvals.image_interp;
   overrides.imagealpha[0]     = defvals.imagealpha[0];
   overrides.imagealpha[1]     = defvals.imagealpha[1];
   overrides.imagethreshold[0] = defvals.imagethreshold[0];
   overrides.imagethreshold[1] = defvals.imagethreshold[1];
   overrides.imageinvertcmap   = defvals.imageinvertcmap;
   overrides.imagecmapsize     = defvals.imagecmapsize;
   overrides.imagecmapinterp   = defvals.imagecmapinterp;
   overrides.imagescalechannels = defvals.imagescalechannels;
   overrides.labelstem         = defvals.labelstem;           
   overrides.labelid           = defvals.labelid;           
   overrides.labelfont         = defvals.labelfont; 
   overrides.labelstyle        = defvals.labelstyle;    
   overrides.labelsize         = defvals.labelsize;           
   overrides.labelcolor        = defvals.labelcolor;          
   overrides.labelangle        = defvals.labelangle;
   overrides.labelhalign       = defvals.labelhalign;         
   overrides.labelvalign       = defvals.labelvalign;         
   overrides.labeldepth        = defvals.labeldepth;
   overrides.legendstem        = defvals.legendstem;          
   overrides.legendcolor       = defvals.legendcolor;        
   overrides.legendtransparent = defvals.legendtransparent; 
   overrides.legendborders     = defvals.legendborders;      
   overrides.legendthickness   = defvals.legendthickness; 
   overrides.legenddepth       = defvals.legenddepth;
   overrides.regionstem        = defvals.regionstem;         
   overrides.regionid          = defvals.regionid;         
   overrides.regionthickness   = defvals.regionthickness;
   overrides.regionstyle       = defvals.regionstyle;
   overrides.regionfillstyle   = defvals.regionfillstyle;
   overrides.regioncolor       = defvals.regioncolor;
   overrides.regionfillcolor   = defvals.regionfillcolor;
   overrides.regiondepth       = defvals.regiondepth;
   overrides.regionangle       = defvals.regionangle; 
   overrides.regionopacity     = defvals.regionopacity; 
   overrides.cbarstem          = defvals.cbarstem;
   overrides.cbarid            = defvals.cbarid;
   overrides.cbarwidth        = defvals.cbarwidth;
   overrides.cbarlength         = defvals.cbarlength;
   overrides.cbarbordervis     = defvals.cbarbordervis; 
   overrides.cbardepth         = defvals.cbardepth ; 
   overrides.cbarhalign        = defvals.cbarhalign ; 
   overrides.cbarvalign        = defvals.cbarvalign ; 
   overrides.cbarorient        = defvals.cbarorient ; 
   overrides.cbartickmode      = defvals.cbartickmode ; 
   overrides.cbartickstyle     = defvals.cbartickstyle ; 
   overrides.cbartickcolor     = defvals.cbartickcolor ; 
   overrides.cbarticklength    = defvals.cbarticklength ; 
   overrides.cbartickthickness = defvals.cbartickthickness ; 
   overrides.cbartickloc       = defvals.cbartickloc ; 
   overrides.cbartickvisible   = defvals.cbartickvisible;
   overrides.cbarlabloc        = defvals.cbarlabloc ; 
   overrides.cbarlabvisible    = defvals.cbarlabvisible;
   overrides.cbarlabtext       = defvals.cbarlabtext ;
   overrides.cbarticklabcolor  = defvals.cbarticklabcolor ; 
   overrides.cbarticklabangle  = defvals.cbarticklabangle ; 
   overrides.cbarticklabhalign = defvals.cbarticklabhalign ; 
   overrides.cbarticklabvalign = defvals.cbarticklabvalign ; 
   overrides.cbarticklabvisible= defvals.cbarticklabvisible;
   overrides.cbarticklabsize   = defvals.cbarticklabsize ; 
   overrides.cbarticklaboffset = defvals.cbarticklaboffset ; 
   overrides.cbarticklabfont   = defvals.cbarticklabfont ; 
   overrides.cbarticklabfontstyle = defvals.cbarticklabfontstyle ; 
   overrides.cbarlabsize       = defvals.cbarlabsize ; 
   overrides.cbarlabangle      = defvals.cbarlabangle ; 
   overrides.cbarlabcolor      = defvals.cbarlabcolor ; 
   overrides.cbarlabfont       = defvals.cbarlabfont ; 
   overrides.cbarlabfontstyle  = defvals.cbarlabfontstyle ; 
   overrides.cbarlabhalign     = defvals.cbarlabhalign ; 
   overrides.cbarlabvalign     = defvals.cbarlabvalign ; 
   overrides.cbaroffsetpar     = defvals.cbaroffsetpar;
   overrides.cbaroffsetperp    = defvals.cbaroffsetperp;
   overrides.exportfonts       = defvals.exportfonts;
   overrides.exportclobber     = defvals.exportclobber;
   overrides.exportorient      = defvals.exportorient;
   overrides.exportformat      = defvals.exportformat;
   overrides.exportcolorscheme = defvals.exportcolorscheme;
   overrides.exportdpi         = defvals.exportdpi;
   overrides.pagelmargin       = defvals.pagelmargin;
   overrides.pagermargin       = defvals.pagermargin;
   overrides.pagetmargin       = defvals.pagetmargin;
   overrides.pagebmargin       = defvals.pagebmargin;
   overrides.pagewidth         = defvals.pagewidth;
   overrides.pageheight        = defvals.pageheight;
   overrides.pageunits         = defvals.pageunits;
   overrides.pagescaleheight   = defvals.pagescaleheight;
   overrides.pagescalewidth    = defvals.pagescalewidth;
   overrides.fittopage         = defvals.fittopage;
   overrides.pageaspect        = defvals.pageaspect;
   overrides.printdialog       = defvals.printdialog;
   overrides.toolbarvis        = defvals.toolbarvis;
   overrides.navpanevis        = defvals.navpanevis;
   overrides.infpanevis        = defvals.infpanevis;
   overrides.proppanevis       = defvals.proppanevis;
   overrides.shellpanevis      = defvals.shellpanevis;
   overrides.meta1             = defvals.meta1;
   overrides.pickcolor         = defvals.pickcolor; 
   return ERR_STATUS_GOOD;  
}


int ChipsDefaults::set_boolean_value(bool& bv, char* val)
{
   int err = ERR_STATUS_GOOD; 
   if (!val)    return ERR_STATUS_BAD;


   if ((chips_strcmp(val, "false") == 0) || (strcmp(val, "0") == 0) ||
       (chips_strcmp(val, "no") == 0) || (chips_strcmp(val, "off") == 0) ||
       (chips_strcmp(val, "f") == 0))
      bv = false;
   else if ((chips_strcmp(val, "true") == 0) || (strcmp(val, "1") == 0) ||
       (chips_strcmp(val, "yes") == 0) || (chips_strcmp(val, "on") == 0) ||
       (chips_strcmp(val, "t") == 0))
      bv = true;
   else 
      err = data_subset_error(val, "[t|f|true|false|on|off|1|0|yes|no]");
   return err; 
}

int ChipsDefaults::set_aspect_ratio(DEFVALS_P_T* cp, char* val)
{
   if (!cp) return ERR_STATUS_BAD; 
   if (!val || (val[0] == '\0'))  // no value specified- not supporting an aspect ratio
   {
      (*cp)->plotaspectw = (*cp)->plotaspecth = 0;
      (*cp)->plotratio = ""; 
      return ERR_STATUS_GOOD; 
   }
   int rval; 
   string::size_type pos; 
   string vval = val; 
   if ((pos = vval.find(":")) == string::npos)  return ERR_STATUS_BAD; 
   if (pos == 0)
   {
      string est = "The plot.aspect.ratio value (";
      est.append(val);
      est.append(") must be specified as width:height (ie. 4:3)."); 
      if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval=ERR_STR_SUBSET,"",-1);
      return rval;
   }
   float tmp1, tmp2;
   istringstream v1(vval.substr(0,pos));
   istringstream v2(vval.substr(pos+1));
   v1 >> tmp1;
   v2 >> tmp2;
   if (tmp1 <= 0 || v1.fail())
   {
      string est = "The plot.aspect.ratio first value (";
      est.append(vval.substr(0,pos));
      est.append(") must be a numeric value greater than 0"); 
      if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval=ERR_DBL_GTE,"",-1);
      return rval; 
   }
   else if (tmp2 <= 0 || v2.fail())
   {
      string est = "The plot.aspect.ratio second value (";
      est.append(vval.substr(pos+1));
      est.append(") must be a numeric value greater than 0"); 
      if (cmd_manager)  cmd_manager->add_err(est.c_str(),rval=ERR_DBL_GTE,"",-1);
      return rval; 
   }
   (*cp)->plotaspectw = tmp1;
   (*cp)->plotaspecth = tmp2;
   (*cp)->plotratio = vval; 
   return ERR_STATUS_GOOD; 
}



int ChipsDefaults::unknown_property_error()
{
   if (cmd_manager)
   {
      string errstr= "Unknown or unsupported <obj>.<attribute> entry: ";
      if (obj_p)  errstr.append(obj_p);
      if (prop_p)
      {
         errstr.append(".");
         errstr.append(prop_p);
      }
      errstr.append("\n");
      cmd_manager->add_err(errstr, ERR_UNKNOWN_PROPERTY);
   }
   return ERR_UNKNOWN_PROPERTY;
}

int ChipsDefaults::thickness_value_error(float val)
{
   if (cmd_manager)
   {
      sprintf(errbuf, "The %s.%s value (%2.1f) must be between %2.1f and %2.1f", 
         obj_p, prop_p, val, (float)V_THICK_MIN_VAL, (float)V_THICK_MAX_VAL);
      string errstr = errbuf; 
      cmd_manager->add_err(errstr, ERR_THICKNESS_VAL);
   }
   return ERR_THICKNESS_VAL;
}

int ChipsDefaults::no_property_value_error()
{
   if (cmd_manager)
   {
      sprintf(errbuf, "The %s.%s value was not specified", obj_p, 
         prop_p);
      string errstr = errbuf; 
      cmd_manager->add_err(errstr, ERR_NO_PROP_VALUE);
   }
   return ERR_NO_PROP_VALUE;
}

int ChipsDefaults::data_range_error(float val, float min, float max)
{
   if (cmd_manager)
   {
      sprintf(errbuf, "The %s.%s value (%2.1f) must be between %2.1f and %2.1f",
         obj_p, prop_p, val, min, max);
      string errstr = errbuf;
      cmd_manager->add_err(errstr, ERR_DATA_RANGE);
   }
   return ERR_DATA_RANGE;
}

int ChipsDefaults::data_range_error(int val, int min, int max)
{
   if (cmd_manager)
   {
      sprintf(errbuf, "The %s.%s value (%1d) must be between %1d and %1d",
         obj_p, prop_p, val, min, max);
      string errstr = errbuf;
      cmd_manager->add_err(errstr, ERR_DATA_RANGE);
   }
   return ERR_DATA_RANGE;
}

int ChipsDefaults::negative_value_error(int val, int eval)
{
   if (cmd_manager)
   {
      sprintf(errbuf, "The %s.%s value (%1d) can not be negative",
         obj_p, prop_p, val);
      string errstr = errbuf;
      cmd_manager->add_err(errstr, eval);
   }
   return eval;
}

int ChipsDefaults::negative_value_error(float val, int eval)
{
   if (cmd_manager)
   {
      sprintf(errbuf, "The %s.%s value (%2.1f) can not be negative",
         obj_p, prop_p, val);
      string errstr = errbuf;
      cmd_manager->add_err(errstr, eval);
   }
   return eval;
}

int ChipsDefaults::data_subset_error(const char* val, const char* subset, int eval)
{
   if (cmd_manager && val && subset)
   {
      sprintf(errbuf, "The %s.%s value (%s) must be one of %s",
         obj_p, prop_p, val, subset);
      string errstr = errbuf;
      cmd_manager->add_err(errstr, eval);
   }
   return eval;
}



void ChipsDefaults::serialize(ChipsStateStream& out)
{
   int sz; 
   float ff;
   out.write_bool(&defvals.windowdisplay);    // always display window
   out.write_bool(&defvals.windowsmoothing);  // smooth to hide aliasing effects
   out.write_bool(&defvals.showgrid);         // should window grids be displayed?
   out.write_bool(&defvals.showlayout);       // display frame boundaries/layouts?
   out.write_bool(&defvals.frameaspect);      // keep frame aspect when tiling
   out.write_bool(&defvals.framescale);       // scale labels when tiling
   out.write_bool(&defvals.framebordervis);   // frame border visible
   out.write_bool(&defvals.frametransparent); // is frame transparent?
   out.write_bool(&defvals.axis_automin);     // auto minimum enabled
   out.write_bool(&defvals.axis_automax);     // auto maximum enabled
   out.write_bool(&defvals.majortickvisible); // display or hide major ticks
   out.write_bool(&defvals.minortickvisible); // display or hide minor ticks
   out.write_bool(&defvals.majorgridvisible); // display or hide major grid
   out.write_bool(&defvals.minorgridvisible); // display or hide minor grid
   out.write_bool(&defvals.pointfill);        // filled or open symbol
   out.write_bool(&defvals.curvesymbolfill);  // filled or open symbol
   out.write_bool(&defvals.errup);            // up errors on/off
   out.write_bool(&defvals.errdown);          // down errors on/off
   out.write_bool(&defvals.errleft);          // left errors on/off
   out.write_bool(&defvals.errright);         // right errors on/off
   out.write_bool(&defvals.limitoverride);    // upper/lower limits overrides up & down errs
   out.write_bool(&defvals.histsymbolfill);   // histogram symbol fill
   out.write_bool(&defvals.histerrup);        // histogram error up
   out.write_bool(&defvals.histerrdown);      // histogram error down
   out.write_bool(&defvals.histdropline);     // histogram drop lines (bar graph)
   out.write_bool(&defvals.legendtransparent);// is legend transparent
   out.write_bool(&defvals.legendborders);    // does legend display borders
   out.write_bool(&defvals.exportclobber);    // overwrite file if it exists
   out.write_bool(&defvals.exportfonts);      // export fonts to ps file
   out.write_bool(&defvals.fittopage);        // fit the window to the page
   out.write_bool(&defvals.pageaspect);       // keep window aspect when fitting to page
   out.write_bool(&defvals.printdialog);      // display print dialog window 
   out.write_bool(&allvisible);        // all visibility settings
   out.write_bool(&allfill);           // all symbol fills
   out.write_bool(&allaspect);         // all aspect settings 
   out.write_bool(&infocurrent);       // add reverse video for currency to info
   out.write_bool(&infocoord);         // add coord system to info
   out.write_bool(&infodepth);         // add depth values to info

   out.write_float(&defvals.layoutthickness);  // how thick to draw the frame borders/layout
   out.write_float(&defvals.windowheight);     // height of window
   out.write_float(&defvals.windowwidth);      // width of window
   out.write_float(&defvals.windowscaleheight);     // height of window
   out.write_float(&defvals.windowscalewidth);      // width of window
   out.write_float(&defvals.gridthickness);    // how thick to draw the grids
   out.write_float(&defvals.plotlmargin);      // left margin of plot
   out.write_float(&defvals.plotrmargin);      // right margin of plot
   out.write_float(&defvals.plottmargin);      // top margin of plot
   out.write_float(&defvals.plotbmargin);      // bottom margin of plot
   out.write_float(&defvals.titlexpos);        // plot title default x normalized pos
   out.write_float(&defvals.titleangle);       // plot title default angle
   out.write_float(&defvals.titlehalign);      // plot title default horizontal alignment
   out.write_float(&defvals.titlevalign);      // plot title default vertical alignment
   out.write_float(&defvals.titleypos);        // plot title default x normalized pos
   out.write_float(&defvals.ticklabangle);     // angle of tick label
   out.write_float(&defvals.ticklabhalign);    // horizontal alignment of tick label
   out.write_float(&defvals.ticklabvalign);    // vertical alignment of tick label
   out.write_double(&defvals.axis_pad);// pad factor % for each side of axis
   out.write_float(&defvals.axis_offsetperp);  // axis label perpendicular offset
   out.write_float(&defvals.axis_offsetpar);   // axis label parallel offset
   out.write_float(&defvals.axisthickness);    // size of axis
   out.write_float(&defvals.majortickinterval);// interval spacing of ticks
   out.write_float(&defvals.majortickthickness);// thickness of major tick marks
   out.write_float(&defvals.minortickinterval);// interval spacing of minor ticks
   out.write_float(&defvals.minortickthickness);// thickness of minor tick marks
   out.write_float(&defvals.axlabhalign);      // horizontal alignment of axis label
   out.write_float(&defvals.axlabvalign);      // vertical alignment of axis label
   if (out.get_minor_version() > 17)
   {
      out.write_float(&defvals.axlabxangle);   // x axis label angle
      out.write_float(&defvals.axlabyangle);   // y axia label angle
   }
   out.write_float(&defvals.majorgridthickness);// thickness of major grid
   out.write_float(&defvals.minorgridthickness);// thickness of minor grid
   out.write_float(&defvals.pointangle);       // point default angle
   out.write_float(&defvals.linethickness);    // thickness of line
   out.write_float(&defvals.curvelinethickness);// thickness of line
   out.write_float(&defvals.curvesymbolangle); // symbol default angle
   out.write_float(&defvals.errthickness);     // thickness of curve errs
   if (out.get_minor_version() > 2)
      out.write_int(&defvals.errcaplen);       // curve err cap length
   out.write_float(&defvals.limitlength);      // upper/lower limits length 
   out.write_float(&defvals.histsymbolangle);  // histogram symbol angle
   out.write_float(&defvals.histlinethickness);// histogram line thickness
   out.write_float(&defvals.histerrthickness); // histogram error thickness
   if (out.get_minor_version() > 2)
      out.write_int(&defvals.histerrcaplen);   // histogram err cap length
   ff = defvals.histfillopacity; out.write_float(&ff); // histogram fill opacity
   sz = (int)defvals.histfillstyle;  
   out.write_int(&sz); 
   out.write_float(&defvals.contourthickness); // thickness of contour line
   sz = defvals.contourlevels.size();    
   out.write_int(&sz);
   for (int ii=0; ii<sz; ii++)
      out.write_double(&(defvals.contourlevels[ii]));
   out.write_float(&defvals.contourinterval);  // interval for contour levels 
   out.write_float(&defvals.labelangle);       // angle of tick label
   out.write_float(&defvals.labelhalign);      // horizontal alignment of label
   out.write_float(&defvals.labelvalign);      // vertical alignment of label
   out.write_float(&defvals.legendthickness);  // thickness of legend border
   out.write_float(&defvals.regionthickness);  // thickness of region edges
   out.write_float(&defvals.regionangle);      // region angle
   ff = defvals.regionopacity; out.write_float(&ff);    // region opacity
   out.write_float(&defvals.pagelmargin);      // page bottom margin
   out.write_float(&defvals.pagermargin);      // page bottom margin
   out.write_float(&defvals.pagetmargin);      // page bottom margin
   out.write_float(&defvals.pagebmargin);      // page bottom margin
   out.write_float(&defvals.pageheight);       // height of page
   out.write_float(&defvals.pagewidth);        // width of page
   out.write_float(&defvals.pagescaleheight);  // scale page height
   out.write_float(&defvals.pagescalewidth);   // scale page width
   out.write_float(&allthickness);      // all line thickess 
   out.write_float(&allangle);          // set all label angles 

   out.write_int(&defvals.gridscale);        // 1 = 1/4in, 2=1/2in, 3=cm, 4=norm
   out.write_int(&defvals.framemode);        // placed = 0, single = 1, tiled = 2
   out.write_int(&defvals.titlesize);        // size of plot title
   out.write_int(&defvals.titledepth);       // depth for plot 
   out.write_int(&defvals.ticklabsize);      // size of tick label
   out.write_int(&defvals.ticklaboffset);    // offset of tick label
   out.write_int(&defvals.axisdepth);        // depth level
   sz = defvals.majortickcount;
   out.write_int(&sz); 
   out.write_int(&defvals.majorticklength);  // length of major tick marks
   sz = defvals.minortickcount; 
   out.write_int(&sz); 
   out.write_int(&defvals.minorticklength);  // length of minor tick marks
   out.write_int(&defvals.axlabsize);        // size of axis label
   out.write_int(&defvals.pointsize);        // size of symbol
   out.write_int(&defvals.pointdepth);        // depth level
   out.write_int(&defvals.linedepth);        // depth level
   out.write_int(&defvals.curvesymbolsize);  // size of symbol
   out.write_int(&defvals.curvedepth);        // depth level
   out.write_int(&defvals.histsymbolsize);   // histogram symbol size
   out.write_int(&defvals.histdepth);        // depth level
   sz = defvals.contournumlevels; 
   out.write_int(&sz);
   out.write_int(&defvals.contourdepth);     // depth level
   out.write_int(&defvals.labelsize);        // size of label
   out.write_int(&defvals.labeldepth);       // depth level
   out.write_int(&defvals.legenddepth);      // depth level
   out.write_int(&defvals.regiondepth);      // depth level
   out.write_int(&defvals.exportorient);     // portrait/landscape
   out.write_int(&defaultdepth);
   out.write_int(&defaultverb);
   out.write_int(&allpropmask);       // mask of all.properties to set
   out.write_int(&allcount);          // all tick count 
   out.write_int(&alllength);         // all tick length
   out.write_int(&alldepth);          // all depth values
   sz = pickmask;
   out.write_int(&sz);      // mask of events for pickpoints

   out.write_string(&defvals.windowstem);       // stem for default window ids
   if (out.get_minor_version() > 2)
   {
      out.write_string(&defvals.windowgeom);    // window geometry hint
      out.write_string(&defvals.windowprefix);  // window titlebar prefix
   }
   out.write_string(&defvals.windowid);         // window id if specified
   out.write_string(&defvals.windowcolor);      // color of window background
   out.write_string(&defvals.gridcolor);        // color of grid
   out.write_string(&defvals.layoutcolor);      // color of layout/frame borders
   out.write_string(&defvals.framestem);        // stem for default frame ids
   out.write_string(&defvals.frameid);          // frame id if specified
   out.write_string(&defvals.framecolor);       // color of frame
   out.write_string(&defvals.framebordercolor); // color of frame border
   out.write_string(&defvals.plotstem);         // stem for default plot ids
   out.write_string(&defvals.plotid);           // plot id if specified
   out.write_string(&defvals.titlecolor);       // color of plot title string
   out.write_string(&defvals.titlefont);        // font of plot title
   out.write_string(&defvals.titlelabel);       // plot title label if provided
   out.write_string(&defvals.imagestem);        // stem for default image ids
   out.write_string(&defvals.imageid);          // image id if specified
   out.write_string(&defvals.axisxstem);         // stem for default x axis ids
   out.write_string(&defvals.axisxid);           // x axis id if provided
   out.write_string(&defvals.axisxlabel);        // x axis label if provided
   out.write_string(&defvals.axisystem);         // stem for default y axis ids
   out.write_string(&defvals.axisyid);           // y axis id if provided
   out.write_string(&defvals.axisylabel);        // y axis label if provided
   out.write_string(&defvals.axiscolor);        // color of axis
   out.write_string(&defvals.tickformat);       // tick format string (ie. "%2d")
   out.write_string(&defvals.ticklabcolor);     // color of tick label string
   out.write_string(&defvals.ticklabfont);      // font of tick label
   out.write_string(&defvals.majortickmode);    // count or interval based tick marks
   out.write_string(&defvals.majortickcolor);   // color of major ticks
   out.write_string(&defvals.minortickmode);    // count or interval based tick marks
   out.write_string(&defvals.minortickcolor);   // color of minor ticks
   out.write_string(&defvals.axlabcolor);       // color of axis label string
   out.write_string(&defvals.axlabfont);        // font of axis label
   out.write_string(&defvals.majorgridcolor);   // color of major grid
   out.write_string(&defvals.minorgridcolor);   // color of minor grid
   out.write_string(&defvals.pointcolor);       // color of symbol
   out.write_string(&defvals.pointstem);        // stem for default curve ids
   out.write_string(&defvals.pointid);          // point id if provided
   out.write_string(&defvals.linecolor);        // color of line
   out.write_string(&defvals.linestem);         // stem for default curve ids
   out.write_string(&defvals.lineid);           // line id if provided
   out.write_string(&defvals.curvestem);        // stem for default curve ids
   out.write_string(&defvals.curveid);          // curve id if provided
   out.write_string(&defvals.curvelinecolor);   // color of curve line
   out.write_string(&defvals.curvesymbolcolor); // color of curve symbol
   out.write_string(&defvals.histstem);         // stem for default histogram ids
   out.write_string(&defvals.histid);           // histogram id if provided
   out.write_string(&defvals.histsymbolcolor);  // histogram symbol color
   out.write_string(&defvals.errcolor);         // curve error color
   out.write_string(&defvals.histlinecolor);    // histogram line color
   out.write_string(&defvals.histerrcolor);     // histogram error color
   out.write_string(&defvals.histfillcolor);    // histogram fill color
   out.write_string(&defvals.contourstem);      // stem for default contour ids
   out.write_string(&defvals.contourid);        // contour id if provided 
   out.write_string(&defvals.contourcolor);     // default contour color
   out.write_string(&defvals.labelstem);        // stem for default labels
   out.write_string(&defvals.labelid);          // label id if provided 
   out.write_string(&defvals.labelfont);        // font for labels
   out.write_string(&defvals.labelcolor);       // color of label
   out.write_string(&defvals.legendstem);       // stem for default legend ids
   out.write_string(&defvals.legendcolor);      // default legend color
   out.write_string(&defvals.regionstem);       // stem for default region ids
   out.write_string(&defvals.regionid);         // region id if provided 
   out.write_string(&defvals.regioncolor);      // region edge color
   out.write_string(&defvals.regionfillcolor);  // region fill color 
   out.write_string(&defvals.exportformat);     // export format- ps, eps, png, pdf, jpg
   out.write_string(&defvals.pickcolor);        // color for pick limits bounding box
   out.write_string(&filename);         // name of parameter file to use
   out.write_string(&defaultfgps);
   out.write_string(&defaultbgps);
   out.write_string(&defaultfgdisp);
   out.write_string(&defaultbgdisp);
   if (out.get_minor_version() < 7)
   {
      string tmp_str("python");
      out.write_string(&tmp_str);
   }
   out.write_string(&allbgcolor);        // all background color 
   out.write_string(&allcolor);          // all foreground color 
   out.write_string(&allfont);           // all label font
   sz = (int)defvals.axisstyle; out.write_int(&sz); // type of axis
   sz = (int)defvals.curvebaddata;  out.write_int(&sz); // how to handle bad data in curve
   sz = (int)defvals.contouralgorithm;  out.write_int(&sz);// contour algorithm to use
   sz = (int)defvals.cornerstyle;  out.write_int(&sz); // type of axis corner joint
   sz = (int)defvals.errstyle;  out.write_int(&sz);    // curve error style
   sz = (int)defvals.histerrstyle;  out.write_int(&sz);// histogram error style
   sz = (int)defvals.titlefontstyle;  out.write_int(&sz); // font style- italic, bold,etc...
   sz = (int)defvals.ticklabfontstyle;  out.write_int(&sz);// tick label fontstyle
   sz = (int)defvals.axlabfontstyle;  out.write_int(&sz); // font style- italic, bold,etc...
   sz = (int)defvals.labelstyle;  out.write_int(&sz); // bold, italic, normal, bolditalic
   sz = (int)allfontstyle;  out.write_int(&sz);// all font style- italic, bold,etc...
   sz = (int)defvals.lineextend;  out.write_int(&sz);// extend line endpoints
   sz = (int)defvals.gridlinestyle;  out.write_int(&sz);   // type of line for grids
   sz = (int)defvals.layoutlinestyle;  out.write_int(&sz); // type of line for layout
   sz = (int)defvals.majorgridstyle;  out.write_int(&sz);  // type of line for major grid
   sz = (int)defvals.minorgridstyle;  out.write_int(&sz);  // type of line for minor grid
   sz = (int)defvals.linestyle;  out.write_int(&sz);       // type of line
   sz = (int)defvals.curvelinestyle;  out.write_int(&sz);  // type of line
   sz = (int)defvals.histlinestyle;  out.write_int(&sz);   // histogram line style
   sz = (int)defvals.contourstyle;  out.write_int(&sz);    // line style for contours
   sz = (int)defvals.regionstyle;  out.write_int(&sz);     // edge style of region 
   sz = (int)defvals.regionfillstyle; out.write_int(&sz); // edge style of region
   sz = (int)alllinestyle;  out.write_int(&sz);    // all linestyle
   sz = (int)defvals.exportcolorscheme;  out.write_int(&sz); 
   sz = (int)defvals.curvesymbolstyle;  out.write_int(&sz);// type of symbol
   sz = (int)defvals.pointstyle;  out.write_int(&sz);    // type of symbol
   sz = (int)defvals.histsymbolstyle;  out.write_int(&sz); // histogram symbol style
   sz = (int)defvals.contourmode;  out.write_int(&sz); // mode for determining contour levels
   sz = (int)defvals.ticklabstyle;  out.write_int(&sz);// inside or outside
   sz = (int)defvals.majortickstyle;  out.write_int(&sz);   // inside, outside, or centered
   sz = (int)defvals.minortickstyle;  out.write_int(&sz);   // inside, outside, or centered
   sz = (int)defvals.windowunits;  out.write_int(&sz);    // units of height/width 
   sz = (int)defvals.pageunits;  out.write_int(&sz);      // units of the page height/width

   if (out.get_minor_version() > 0)
   {
      // printing
      if (out.get_minor_version() > 1)
         out.write_int (&defvals.exportdpi);

      // images
      sz = (int)defvals.imagecmap; out.write_int(&sz);
      sz = (int)defvals.image_interp; out.write_int(&sz);
      out.write_int(&defvals.imagedepth);  
      out.write_double(defvals.imagealpha,2);  
      out.write_double(defvals.imagethreshold,2);
      out.write_bool(&defvals.imageinvertcmap);
      out.write_int(&defvals.imagecmapsize);
      out.write_bool(&defvals.imagecmapinterp);
      out.write_bool(&defvals.imagescalechannels);

      // colorbar 
      out.write_string(&defvals.cbarstem);
      out.write_string(&defvals.cbarid);
      out.write_string(&defvals.cbartickcolor);
      out.write_string(&defvals.cbarticklabcolor);
      out.write_string(&defvals.cbarticklabfont);
      out.write_string(&defvals.cbarlabcolor);
      out.write_string(&defvals.cbarlabfont);
      out.write_float (&defvals.cbarwidth);
      out.write_float (&defvals.cbarlength);
      out.write_bool(&defvals.cbarbordervis);
      out.write_float(&defvals.cbarhalign);
      out.write_float(&defvals.cbarvalign);
      out.write_float(&defvals.cbartickthickness);
      out.write_float(&defvals.cbarticklabangle);
      out.write_float(&defvals.cbarticklabhalign);
      out.write_float(&defvals.cbarticklabvalign);
      out.write_float(&defvals.cbarlabhalign);
      out.write_float(&defvals.cbarlabvalign);
      out.write_float(&defvals.cbarlabangle);
      out.write_int(&defvals.cbardepth);
      out.write_int(&defvals.cbarorient);
      out.write_int(&defvals.cbarlabsize);
      out.write_int(&defvals.cbartickmode);
      out.write_int(&defvals.cbarticklength);
      out.write_int(&defvals.cbarticklabsize);
      out.write_int(&defvals.cbarticklaboffset);
      sz = (int)defvals.cbartickstyle;  out.write_int(&sz);  
      sz = (int)defvals.cbartickloc;  out.write_int(&sz);  
      sz = (int)defvals.cbarlabloc;  out.write_int(&sz);  
      sz = (int)defvals.cbarticklabfontstyle;  out.write_int(&sz);
      sz = (int)defvals.cbarlabfontstyle;  out.write_int(&sz);
      out.write_float(&defvals.cbaroffsetperp);
      out.write_float(&defvals.cbaroffsetpar);
      if (out.get_minor_version() > 1)
      {
         out.write_string (&defvals.cbarlabtext);
         out.write_bool (&defvals.cbarlabvisible);
         out.write_bool (&defvals.cbartickvisible);
         out.write_bool (&defvals.cbarticklabvisible);
      }
   }

   if (out.get_minor_version() > 12)
   {
      out.write_bool (&defvals.toolbarvis);
      out.write_bool (&defvals.navpanevis);
      out.write_bool (&defvals.infpanevis);
      out.write_bool (&defvals.proppanevis);
      out.write_bool (&defvals.shellpanevis);
   }

   // do not save or load printmethod. This is a system
   // dependent preference rather than a user dependent preference

}


void ChipsDefaults::deserialize(ChipsStateStream& in)
{
   int sz; 
   float ff;
   double dd; 

   in.read_bool(&defvals.windowdisplay);    // always display window
   in.read_bool(&defvals.windowsmoothing);  // smooth to hide aliasing effects
   in.read_bool(&defvals.showgrid);         // should window grids be displayed?
   in.read_bool(&defvals.showlayout);       // display frame boundaries/layouts?
   in.read_bool(&defvals.frameaspect);      // keep frame aspect when tiling
   in.read_bool(&defvals.framescale);       // scale labels when tiling
   in.read_bool(&defvals.framebordervis);   // frame border visible
   in.read_bool(&defvals.frametransparent); // is frame transparent?
   in.read_bool(&defvals.axis_automin);     // auto minimum enabled
   in.read_bool(&defvals.axis_automax);     // auto maximum enabled
   in.read_bool(&defvals.majortickvisible); // display or hide major ticks
   in.read_bool(&defvals.minortickvisible); // display or hide minor ticks
   in.read_bool(&defvals.majorgridvisible); // display or hide major grid
   in.read_bool(&defvals.minorgridvisible); // display or hide minor grid
   in.read_bool(&defvals.pointfill);        // filled or open symbol
   in.read_bool(&defvals.curvesymbolfill);  // filled or open symbol
   in.read_bool(&defvals.errup);            // up errors on/off
   in.read_bool(&defvals.errdown);          // down errors on/off
   in.read_bool(&defvals.errleft);          // left errors on/off
   in.read_bool(&defvals.errright);         // right errors on/off
   in.read_bool(&defvals.limitoverride);    // upper/lower limits overrides up & down errs
   in.read_bool(&defvals.histsymbolfill);   // histogram symbol fill
   in.read_bool(&defvals.histerrup);        // histogram error up
   in.read_bool(&defvals.histerrdown);      // histogram error down
   in.read_bool(&defvals.histdropline);     // histogram drop lines (bar graph)
   in.read_bool(&defvals.legendtransparent);// is legend transparent
   in.read_bool(&defvals.legendborders);    // does legend display borders
   in.read_bool(&defvals.exportclobber);    // overwrite file if it exists
   in.read_bool(&defvals.exportfonts);      // export fonts to ps file
   in.read_bool(&defvals.fittopage);        // fit the window to the page
   in.read_bool(&defvals.pageaspect);       // keep window aspect when fitting to page
   in.read_bool(&defvals.printdialog);      // display print dialog window 
   in.read_bool(&allvisible);        // all visibility settings
   in.read_bool(&allfill);           // all symbol fills
   in.read_bool(&allaspect);         // all aspect settings 
   in.read_bool(&infocurrent);       // add reverse video for currency to info
   in.read_bool(&infocoord);         // add coord system to info
   in.read_bool(&infodepth);         // add depth values to info

   in.read_float(&defvals.layoutthickness);  // how thick to draw the frame borders/layout
   in.read_float(&defvals.windowheight);     // height of window
   in.read_float(&defvals.windowwidth);      // width of window
   in.read_float(&defvals.windowscaleheight);     // height of window
   in.read_float(&defvals.windowscalewidth);      // width of window
   in.read_float(&defvals.gridthickness);    // how thick to draw the grids
   in.read_float(&defvals.plotlmargin);      // left margin of plot
   in.read_float(&defvals.plotrmargin);      // right margin of plot
   in.read_float(&defvals.plottmargin);      // top margin of plot
   in.read_float(&defvals.plotbmargin);      // bottom margin of plot
   in.read_float(&defvals.titlexpos);        // plot title default x normalized pos
   in.read_float(&defvals.titleangle);       // plot title default angle
   in.read_float(&defvals.titlehalign);      // plot title default horizontal alignment
   in.read_float(&defvals.titlevalign);      // plot title default vertical alignment
   in.read_float(&defvals.titleypos);        // plot title default x normalized pos
   in.read_float(&defvals.ticklabangle);     // angle of tick label
   in.read_float(&defvals.ticklabhalign);    // horizontal alignment of tick label
   in.read_float(&defvals.ticklabvalign);    // vertical alignment of tick label
   in.read_double(&defvals.axis_pad);// pad factor % for each side of axis
   in.read_float(&defvals.axis_offsetperp);  // axis label perpendicular offset
   in.read_float(&defvals.axis_offsetpar);   // axis label parallel offset
   in.read_float(&defvals.axisthickness);    // size of axis
   in.read_float(&defvals.majortickinterval);// interval spacing of ticks
   in.read_float(&defvals.majortickthickness);// thickness of major tick marks
   in.read_float(&defvals.minortickinterval);// interval spacing of minor ticks
   in.read_float(&defvals.minortickthickness);// thickness of minor tick marks
   in.read_float(&defvals.axlabhalign);      // horizontal alignment of axis label
   in.read_float(&defvals.axlabvalign);      // vertical alignment of axis label
   if (in.get_minor_version() > 17)
   {
      in.read_float(&defvals.axlabxangle);   // x axis label angle
      in.read_float(&defvals.axlabyangle);   // y axis label angle
   }
   in.read_float(&defvals.majorgridthickness);// thickness of major grid
   in.read_float(&defvals.minorgridthickness);// thickness of minor grid
   in.read_float(&defvals.pointangle);       // point default angle
   in.read_float(&defvals.linethickness);    // thickness of line
   in.read_float(&defvals.curvelinethickness);// thickness of line
   in.read_float(&defvals.curvesymbolangle); // symbol default angle
   in.read_float(&defvals.errthickness);     // thickness of curve errs
   if (in.get_minor_version() > 2)
      in.read_int(&defvals.errcaplen);       // curve err cap length
   in.read_float(&defvals.limitlength);      // upper/lower limits length (normalized)
   in.read_float(&defvals.histsymbolangle);  // histogram symbol angle
   in.read_float(&defvals.histlinethickness);// histogram line thickness
   in.read_float(&defvals.histerrthickness); // histogram error thickness
   if (in.get_minor_version() > 2)
      in.read_int(&defvals.histerrcaplen);   // histogram err cap length
   in.read_float(&ff);                       // histogram fill opacity
   defvals.histfillopacity = ff; 
   in.read_int(&sz);                         // edge style of region 
   defvals.histfillstyle = (V_FILL_TYPE)sz;
   in.read_float(&defvals.contourthickness); // thickness of contour line
   in.read_int (&sz);     // contour level vector
   for (int ii=0; ii<sz; ii++)
   {
      in.read_double(&dd);
      defvals.contourlevels.push_back(dd);
   }
   in.read_float(&defvals.contourinterval);  // interval for contour levels (interval mode)
   in.read_float(&defvals.labelangle);       // angle of tick label
   in.read_float(&defvals.labelhalign);      // horizontal alignment of label
   in.read_float(&defvals.labelvalign);      // vertical alignment of label
   in.read_float(&defvals.legendthickness);  // thickness of legend border
   in.read_float(&defvals.regionthickness);  // thickness of region edges
   in.read_float(&defvals.regionangle);      // region angle
   in.read_float(&ff);                       // region opacity
   defvals.regionopacity = ff; 
   in.read_float(&defvals.pagelmargin);      // page bottom margin
   in.read_float(&defvals.pagermargin);      // page bottom margin
   in.read_float(&defvals.pagetmargin);      // page bottom margin
   in.read_float(&defvals.pagebmargin);      // page bottom margin
   in.read_float(&defvals.pageheight);       // height of page
   in.read_float(&defvals.pagewidth);        // width of page
   in.read_float(&defvals.pagescaleheight);  // scale page height
   in.read_float(&defvals.pagescalewidth);   // scale page width
   in.read_float(&allthickness);      // all line thickess 
   in.read_float(&allangle);          // set all label angles 
   in.read_int(&defvals.gridscale);        // 1 = 1/4in, 2=1/2in, 3=cm, 4=normalized
   in.read_int(&defvals.framemode);        // placed = 0, single = 1, tiled = 2
   in.read_int(&defvals.titlesize);        // size of plot title
   in.read_int(&defvals.titledepth);       // depth for plot 
   in.read_int(&defvals.ticklabsize);      // size of tick label
   in.read_int(&defvals.ticklaboffset);    // offset of tick label
   in.read_int(&defvals.axisdepth);        // depth level
   in.read_int(&sz);
   defvals.majortickcount = (unsigned int)sz;
   in.read_int(&sz);
   defvals.majorticklength = (unsigned int)sz; 
   in.read_int(&sz);
   defvals.minortickcount = (unsigned int)sz;
   in.read_int(&defvals.minorticklength);  // length of minor tick marks
   in.read_int(&defvals.axlabsize);        // size of axis label
   in.read_int(&defvals.pointsize);        // size of symbol
   in.read_int(&defvals.pointdepth);        // depth level
   in.read_int(&defvals.linedepth);        // depth level
   in.read_int(&defvals.curvesymbolsize);  // size of symbol
   in.read_int(&defvals.curvedepth);        // depth level
   in.read_int(&defvals.histsymbolsize);   // histogram symbol size
   in.read_int(&defvals.histdepth);        // depth level
   in.read_int(&sz);
   defvals.contournumlevels = (unsigned int)sz;  
   in.read_int(&defvals.contourdepth);     // depth level
   in.read_int(&defvals.labelsize);        // size of label
   in.read_int(&defvals.labeldepth);       // depth level
   in.read_int(&defvals.legenddepth);      // depth level
   in.read_int(&defvals.regiondepth);      // depth level
   in.read_int(&defvals.exportorient);     // portrait/landscape
   in.read_int(&defaultdepth);
   in.read_int(&defaultverb);
   in.read_int(&allpropmask);       // mask of all.properties to set
   in.read_int(&allcount);          // all tick count 
   in.read_int(&alllength);         // all tick length
   in.read_int(&alldepth);          // all depth values
   in.read_int(&sz);      // mask of events for pickpoints
   pickmask = (unsigned int)sz; 
   in.read_string(&defvals.windowstem);       // stem for default window ids
   if (in.get_minor_version() > 2)
   {
      in.read_string(&defvals.windowgeom);    // window geometry hint
      in.read_string(&defvals.windowprefix);  // window title prefix
   }
   in.read_string(&defvals.windowid);         // window id if specified
   in.read_string(&defvals.windowcolor);      // color of window background
   in.read_string(&defvals.gridcolor);        // color of grid
   in.read_string(&defvals.layoutcolor);      // color of layout/frame borders
   in.read_string(&defvals.framestem);        // stem for default frame ids
   in.read_string(&defvals.frameid);          // frame id if specified
   in.read_string(&defvals.framecolor);       // color of frame
   in.read_string(&defvals.framebordercolor); // color of frame border
   in.read_string(&defvals.plotstem);         // stem for default plot ids
   in.read_string(&defvals.plotid);           // plot id if specified
   in.read_string(&defvals.titlecolor);       // color of plot title string
   in.read_string(&defvals.titlefont);        // font of plot title
   in.read_string(&defvals.titlelabel);       // plot title label if provided
   in.read_string(&defvals.imagestem);        // stem for default image ids
   in.read_string(&defvals.imageid);          // image id if specified
   in.read_string(&defvals.axisxstem);         // stem for default x axis ids
   in.read_string(&defvals.axisxid);           // x axis id if provided
   in.read_string(&defvals.axisxlabel);        // x axis label if provided
   in.read_string(&defvals.axisystem);         // stem for default y axis ids
   in.read_string(&defvals.axisyid);           // y axis id if provided
   in.read_string(&defvals.axisylabel);        // y axis label if provided
   in.read_string(&defvals.axiscolor);        // color of axis
   in.read_string(&defvals.tickformat);       // tick format string (ie. "%2d")
   in.read_string(&defvals.ticklabcolor);     // color of tick label string
   in.read_string(&defvals.ticklabfont);      // font of tick label
   in.read_string(&defvals.majortickmode);    // count or interval based tick marks
   in.read_string(&defvals.majortickcolor);   // color of major ticks
   in.read_string(&defvals.minortickmode);    // count or interval based tick marks
   in.read_string(&defvals.minortickcolor);   // color of minor ticks
   in.read_string(&defvals.axlabcolor);       // color of axis label string
   in.read_string(&defvals.axlabfont);        // font of axis label
   in.read_string(&defvals.majorgridcolor);   // color of major grid
   in.read_string(&defvals.minorgridcolor);   // color of minor grid
   in.read_string(&defvals.pointcolor);       // color of symbol
   in.read_string(&defvals.pointstem);        // stem for default curve ids
   in.read_string(&defvals.pointid);          // point id if provided
   in.read_string(&defvals.linecolor);        // color of line
   in.read_string(&defvals.linestem);         // stem for default curve ids
   in.read_string(&defvals.lineid);           // line id if provided
   in.read_string(&defvals.curvestem);        // stem for default curve ids
   in.read_string(&defvals.curveid);          // curve id if provided
   in.read_string(&defvals.curvelinecolor);   // color of curve line
   in.read_string(&defvals.curvesymbolcolor); // color of curve symbol
   in.read_string(&defvals.histstem);         // stem for default histogram ids
   in.read_string(&defvals.histid);           // histogram id if provided
   in.read_string(&defvals.histsymbolcolor);  // histogram symbol color
   in.read_string(&defvals.errcolor);         // curve error color
   in.read_string(&defvals.histlinecolor);    // histogram line color
   in.read_string(&defvals.histerrcolor);     // histogram error color
   in.read_string(&defvals.histfillcolor);    // histogram fill color
   in.read_string(&defvals.contourstem);      // stem for default contour ids
   in.read_string(&defvals.contourid);        // contour id if provided 
   in.read_string(&defvals.contourcolor);     // default contour color
   in.read_string(&defvals.labelstem);        // stem for default labels
   in.read_string(&defvals.labelid);          // label id if provided 
   in.read_string(&defvals.labelfont);        // font for labels
   in.read_string(&defvals.labelcolor);       // color of label
   in.read_string(&defvals.legendstem);       // stem for default legend ids
   in.read_string(&defvals.legendcolor);      // default legend color
   in.read_string(&defvals.regionstem);       // stem for default region ids
   in.read_string(&defvals.regionid);         // region id if provided 
   in.read_string(&defvals.regioncolor);      // region edge color
   in.read_string(&defvals.regionfillcolor);  // region fill color 
   in.read_string(&defvals.exportformat);     // export format- ps, eps, png, pdf, jpg
   in.read_string(&defvals.pickcolor);        // color for pick limits bounding box
   in.read_string(&filename);         // name of parameter file to use
   in.read_string(&defaultfgps);
   in.read_string(&defaultbgps);
   in.read_string(&defaultfgdisp);
   in.read_string(&defaultbgdisp);
   if (in.get_minor_version() < 7)
   {
      string tmp_str;
      in.read_string(&tmp_str);
   }
   in.read_string(&allbgcolor);        // all background color 
   in.read_string(&allcolor);          // all foreground color 
   in.read_string(&allfont);           // all label font

   in.read_int(&sz); // type of axis
   defvals.axisstyle = (V_AXISSTYLE_TYPE)sz;
   in.read_int(&sz); // how to handle bad data in curve
   defvals.curvebaddata = (V_BADDATAMODE_TYPE)sz; 
   in.read_int(&sz);// contour algorithm to use
   defvals.contouralgorithm = (V_CONTOUR_TYPE)sz;
   in.read_int(&sz); // type of axis corner joint
   defvals.cornerstyle = (V_CORNERSTYLE_TYPE)sz;
   in.read_int(&sz);    // curve error style
   defvals.errstyle = (V_ERRSTYLE_TYPE)sz;
   in.read_int(&sz);// histogram error style
   defvals.histerrstyle = (V_ERRSTYLE_TYPE)sz;
   in.read_int(&sz); // font style- italic, bold,etc...
   defvals.titlefontstyle = (V_FONTSTYLE_TYPE)sz;
   in.read_int(&sz);// tick label fontstyle
   defvals.ticklabfontstyle = (V_FONTSTYLE_TYPE)sz;
   in.read_int(&sz); // font style- italic, bold,etc...
   defvals.axlabfontstyle = (V_FONTSTYLE_TYPE)sz;
   in.read_int(&sz); // bold, italic, normal, bolditalic
   defvals.labelstyle = (V_FONTSTYLE_TYPE)sz;
   in.read_int(&sz);// all font style- italic, bold,etc...
   allfontstyle = (V_FONTSTYLE_TYPE)sz;
   in.read_int(&sz);// extend line endpoints
   defvals.lineextend = (V_LINE_EXTENSION_TYPE)sz;
   in.read_int(&sz);   // type of line for grids
   defvals.gridlinestyle = (V_LINE_TYPE)sz;
   in.read_int(&sz); // type of line for layout
   defvals.layoutlinestyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);  // type of line for major grid
   defvals.majorgridstyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);  // type of line for minor grid
   defvals.minorgridstyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);       // type of line
   defvals.linestyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);  // type of line
   defvals.curvelinestyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);   // histogram line style
   defvals.histlinestyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);    // line style for contours
   defvals.contourstyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);     // edge style of region 
   defvals.regionstyle = (V_LINE_TYPE)sz;
   in.read_int(&sz);     // edge style of region 
   defvals.regionfillstyle = (V_FILL_TYPE)sz;
   in.read_int(&sz);    // all linestyle
   alllinestyle = (V_LINE_TYPE)sz;
   in.read_int(&sz); 
   defvals.exportcolorscheme = (V_PRINTFILE_TYPE)sz;
   in.read_int(&sz);// type of symbol
   defvals.curvesymbolstyle = (V_SYMBOL_TYPE)sz;
   in.read_int(&sz);    // type of symbol
   defvals.pointstyle = (V_SYMBOL_TYPE)sz;
   in.read_int(&sz); // histogram symbol style
   defvals.histsymbolstyle = (V_SYMBOL_TYPE)sz;
   in.read_int(&sz); // mode for determining contour levels
   defvals.contourmode = (V_TICKMODE_TYPE)sz;
   in.read_int(&sz);// inside or outside
   defvals.ticklabstyle = (V_TICKSTYLE_TYPE)sz;
   in.read_int(&sz);   // inside, outside, or centered
   defvals.majortickstyle = (V_TICKSTYLE_TYPE)sz;
   in.read_int(&sz);   // inside, outside, or centered
   defvals.minortickstyle = (V_TICKSTYLE_TYPE)sz;
   in.read_int(&sz);    // units of height/width 
   defvals.windowunits = (V_UNITS_TYPE)sz;
   in.read_int(&sz);      // units of the page height/width
   defvals.pageunits = (V_UNITS_TYPE)sz;

   if (in.get_minor_version() > 0)
   {
      // printing
      if (in.get_minor_version() > 1)
         in.read_int (&defvals.exportdpi);

      // images
      in.read_int(&sz); defvals.imagecmap = (V_COLORMAP_TYPE)sz;
      in.read_int(&sz); defvals.image_interp = (V_INTERPOLATION_TYPE)sz;
      in.read_int(&defvals.imagedepth);   
      in.read_double(defvals.imagealpha,2);  
      in.read_double(defvals.imagethreshold,2);
      in.read_bool(&defvals.imageinvertcmap);
      in.read_int(&defvals.imagecmapsize);
      in.read_bool(&defvals.imagecmapinterp);
      in.read_bool(&defvals.imagescalechannels);

      // colorbar 
      in.read_string(&defvals.cbarstem);
      in.read_string(&defvals.cbarid);
      in.read_string(&defvals.cbartickcolor);
      in.read_string(&defvals.cbarticklabcolor);
      in.read_string(&defvals.cbarticklabfont);
      in.read_string(&defvals.cbarlabcolor);
      in.read_string(&defvals.cbarlabfont);
      in.read_float (&defvals.cbarwidth);
      in.read_float (&defvals.cbarlength);
      in.read_bool(&defvals.cbarbordervis);
      in.read_float(&defvals.cbarhalign);
      in.read_float(&defvals.cbarvalign);
      in.read_float(&defvals.cbartickthickness);
      in.read_float(&defvals.cbarticklabangle);
      in.read_float(&defvals.cbarticklabhalign);
      in.read_float(&defvals.cbarticklabvalign);
      in.read_float(&defvals.cbarlabhalign);
      in.read_float(&defvals.cbarlabvalign);
      in.read_float(&defvals.cbarlabangle);
      in.read_int(&defvals.cbardepth);
      in.read_int(&defvals.cbarorient);
      in.read_int(&defvals.cbarlabsize);
      in.read_int(&defvals.cbartickmode);
      in.read_int(&defvals.cbarticklength);
      in.read_int(&defvals.cbarticklabsize);
      in.read_int(&defvals.cbarticklaboffset);
      in.read_int(&sz);
      defvals.cbartickstyle = (V_TICKSTYLE_TYPE)sz; 
      in.read_int(&sz);
      defvals.cbartickloc = (V_TICKSTYLE_TYPE)sz; 
      in.read_int(&sz);
      defvals.cbarlabloc = (V_TICKSTYLE_TYPE)sz; 
      in.read_int(&sz);
      defvals.cbarticklabfontstyle = (V_FONTSTYLE_TYPE)sz; 
      in.read_int(&sz);
      defvals.cbarlabfontstyle = (V_FONTSTYLE_TYPE)sz; 
      in.read_float(&defvals.cbaroffsetperp);
      in.read_float(&defvals.cbaroffsetpar);
      if (in.get_minor_version() > 1)
      {
         in.read_string (&defvals.cbarlabtext);
         in.read_bool (&defvals.cbarlabvisible);
         in.read_bool (&defvals.cbartickvisible);
         in.read_bool (&defvals.cbarticklabvisible);
      }
   }
   if (in.get_minor_version() > 12)
   {
      in.read_bool(&defvals.toolbarvis);
      in.read_bool(&defvals.navpanevis);
      in.read_bool(&defvals.infpanevis);
      in.read_bool(&defvals.proppanevis);
      in.read_bool(&defvals.shellpanevis);
   }

   // do not save or load printmethod. This is a system
   // dependent preference rather than a user dependent preference
}


// return last chips.version number from file or 0
int ChipsDefaults::get_version_from_file(string fname)
{
   int vers = 0, pos, endpos, rr;
   char   linebuf[PREF_MAX_LEN];

   ifstream *infile =  new ifstream(fname.c_str(), ios::in);
   if (infile->good())
   {
      while (infile->getline(linebuf, PREF_MAX_LEN))
      {
         if ((endpos = infile->gcount()) > 18)
         {
            pos = 0; 
            // ignore preceeding white space 
            while (pos < endpos && isspace(linebuf[pos])) pos++;
            if (pos < endpos && (linebuf[pos] == 'c' || linebuf[pos] == 'C'))
            {
               // line starts w/ white space and c- check for chips.version
               for (rr = pos; rr < pos + 13; rr++)  
                  linebuf[rr] = tolower(linebuf[rr]); 
               char* start;
               if ((start=strstr(&linebuf[pos],"chips.version"))==&linebuf[pos])
               {
                  start+=13;
                  // allow only white space til : 
                  while(isspace(*start)) start++;  
                  if (*start == ':')
                  {
                     start++; 
                     istringstream value(start);
                     value >> rr;
                     if (!value.fail())  vers = rr;
                  }
               }
            }
         }
      }  // end while 
   }
   delete infile; 
   return vers; 
}
