// 
//  Copyright (C) 2007-2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CHIPSREMOTEMOD_HH
#define CHIPSREMOTEMOD_HH

#include <string>
#include <list>
#include <iostream>
#include <stdio.h>

#include "ChipsDefines.hh"
#include "ChipsId.hh"
#include "ChipsRemote.hh"

using namespace std;


class ChipsRemoteMod: public ChipsRemote
{
  public:

   ChipsRemoteMod(bool use_server, bool load_prefs);
   ChipsRemoteMod(const string& inServerId, bool load_prefs=true);
   ChipsRemoteMod(bool load_prefs=true);
   virtual ~ChipsRemoteMod();

   // preferences
   int     set_preference_autoload (bool toggle_flag)
           {return ChipsRemote::set_preference_autoload(toggle_flag);};
   int     load_preferences(string in_file="")
	   {return ChipsRemote::load_preferences (in_file);};
   int     save_preferences(string in_file="")
           {return ChipsRemote::save_preferences (in_file);};
   int     set_preference(string pref, string value)
           {return ChipsRemote::set_preference (pref, value);};
   char*   get_preference(string pref)
           {return ChipsRemote::get_preference (pref);};
   int     set_line_stipple(V_LINE_TYPE line, int val)
           {return ChipsRemote::set_line_stipple (line, val);};
   int     get_line_stipple(V_LINE_TYPE line, int &out_val)
           {return ChipsRemote::get_line_stipple (line, out_val);};
   int     set_offscreen_mode (bool flag)
           {return ChipsRemote::set_offscreen_mode(flag);};
   int     load_fill(string in_file, int slot=chips_userfill1)
	   {return ChipsRemote::load_fill(in_file, slot);};
   int     load_colormap(string in_file, int slot=chips_usercmap1)
           {return ChipsRemote::load_colormap(in_file, slot);};
   int     load_colormap(long len, double* r, double* g, 
			 double* b, double* alpha, int slot=chips_usercmap1)
           {return ChipsRemote::load_colormap(len, r, g, b, alpha, slot);};
   int     load_state(string fname)
           {return ChipsRemote::load_state (fname);}; 
   int     save_state(string fname, string mode="binary")
           {return ChipsRemote::save_state(fname, mode);};
   int     undo(string tag)
           {return ChipsRemote::undo (tag);};
   int     undo(int depth=1)
           {return ChipsRemote::undo (depth);};
   int     redo(string tag)
           {return ChipsRemote::redo (tag);};
   int     redo(int depth=1)
           {return ChipsRemote::redo (depth);};
   int     set_undo_tag(string tag)
           {return ChipsRemote::set_undo_tag (tag);};
   int     clear_undo_tag(string tag)
           {return ChipsRemote::clear_undo_tag (tag);};
   int     open_undo_block()
           { return ChipsRemote::open_undo_block(); }
   int     close_undo_block()
           { return ChipsRemote::close_undo_block(); }
   int     open_undo_buffer()
           { return ChipsRemote::open_undo_buffer(); }
   int     close_undo_buffer()
           { return ChipsRemote::close_undo_buffer(); }
   int     discard_undo_buffer()
           { return ChipsRemote::discard_undo_buffer(); }
   int     load_font(string fontname, string fontpath,
                        V_FONTSTYLE_TYPE style = chips_normal)
           { return ChipsRemote::load_font(fontname, fontpath, style); }

   int     set_cascading_property(V_OBJECT_TYPE start, string property,
                                  string value)
           { return ChipsRemote::set_cascading_property(NULL, start, 
                                                        property, value); }  
   int     set_cascading_property(ChipsId* cid, V_OBJECT_TYPE start, 
                                  string property, string value)
           { return ChipsRemote::set_cascading_property(cid, start, 
                                                        property, value); }  

    int    move(ChipsId* cid, V_OBJECT_TYPE obj, double xx, double yy, 
               int mode=V_MOVE_ABSOLUTE)
	   { return ChipsRemote::move(cid, obj, xx, yy, mode); }
    int    move(V_OBJECT_TYPE obj, double xx, double yy, 
               int mode=V_MOVE_ABSOLUTE)
	   { return ChipsRemote::move(NULL, obj, xx, yy, mode); }

   int     get_object_count(V_OBJECT_TYPE obj, int& out_val)
           {  return ChipsRemote::get_object_count(NULL, obj, out_val); }

   int     get_object_count(ChipsId* cid, V_OBJECT_TYPE obj, int& out_val)
           {  return ChipsRemote::get_object_count(cid, obj, out_val); }
   int     get_object_coordinfo(ChipsId* cid, V_OBJECT_TYPE obj,
                                COORD_SYS& out_val, string& out_id)
           {  return ChipsRemote::get_object_coordinfo(cid, obj, out_val, 
                                                       out_id); }
   int     get_object_coordinfo(V_OBJECT_TYPE obj, COORD_SYS& out_val,
                                string& out_id)
           {  return ChipsRemote::get_object_coordinfo(NULL, obj, out_val,  
                                                       out_id); }
  int      highlight_object(V_OBJECT_TYPE obj)
           {  return ChipsRemote::highlight_object(NULL, obj); } 
  int      highlight_object(ChipsId* cid, V_OBJECT_TYPE obj)
           {  return ChipsRemote::highlight_object(cid, obj); } 

   // window operations 
   int     add_window(string attlist = "")
           { return ChipsRemote::add_window((ChipsId*)NULL, attlist); }
   int     add_window(float ww, float hh, string format="", string attlist="")
           { return ChipsRemote::add_window(NULL, ww, hh, format, attlist); }
   int     add_window(ChipsId* cid, string attlist = "")
           { return ChipsRemote::add_window(cid, attlist); }
   int     add_window(ChipsId* cid, float ww, float hh, string format="",
                      string attlist="")
           { return ChipsRemote::add_window(cid, ww, hh, format, attlist); }
   int     delete_window()
           { return ChipsRemote::delete_window(NULL); }
   int     delete_window(ChipsId* cid)
           { return ChipsRemote::delete_window(cid); }
   int     set_current_window(ChipsId* cid)
           { return ChipsRemote::set_current_window(cid); } 
   int     set_window_title(string title) 
           { return ChipsRemote::set_window_title(NULL, title); }
   int     set_window_title(ChipsId* cid, string title) 
           { return ChipsRemote::set_window_title(cid, title); }
   int     get_window_title(string& out_val) 
           { return ChipsRemote::get_window_title(NULL, out_val); } 
   int     get_window_title(ChipsId* cid, string& out_val) 
           { return ChipsRemote::get_window_title(cid, out_val); } 
   int     set_window_bgcolor(string in_color) 
           { return ChipsRemote::set_window_bgcolor(NULL, in_color); }
   int     set_window_bgcolor(ChipsId* cid, string in_color) 
           { return ChipsRemote::set_window_bgcolor(cid, in_color); }
   int     get_window_bgcolor(string& out_val) 
           { return ChipsRemote::get_window_bgcolor(NULL, out_val); } 
   int     get_window_bgcolor(ChipsId* cid, string& out_val) 
           { return ChipsRemote::get_window_bgcolor(cid, out_val); } 
   int     set_window_redraw(bool draw) 
           { return ChipsRemote::set_window_redraw(NULL, draw); } 
   int     set_window_redraw(ChipsId* cid, bool draw) 
           { return ChipsRemote::set_window_redraw(cid, draw); } 
   int     get_window_redraw(bool& out_val)
           { return ChipsRemote::get_window_redraw(NULL, out_val); }
   int     get_window_redraw(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_window_redraw(cid, out_val); }
   int     iconify_window()
           { return ChipsRemote::iconify_window(NULL); }
   int     iconify_window(ChipsId* cid)
           { return ChipsRemote::iconify_window(cid); }
   int     deiconify_window()
           { return ChipsRemote::deiconify_window(NULL); }
   int     deiconify_window(ChipsId* cid)
           { return ChipsRemote::deiconify_window(cid); }

   int     set_frame_bgcolor(string in_color)
           { return ChipsRemote::set_frame_bgcolor(NULL, in_color); }
   int     set_frame_bgcolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_frame_bgcolor(cid, in_color); }
   int     get_frame_bgcolor(string& out_val) 
           { return ChipsRemote::get_frame_bgcolor(NULL, out_val); }
   int     get_frame_bgcolor(ChipsId* cid, string& out_val) 
           { return ChipsRemote::get_frame_bgcolor(cid, out_val); }
   int     set_frame_border_color(string in_color) 
           { return ChipsRemote::set_frame_border_color(NULL, in_color); }
   int     set_frame_border_color(ChipsId* cid, string in_color) 
           { return ChipsRemote::set_frame_border_color(cid, in_color); }
   int     get_frame_border_color(string& out_val)
           { return ChipsRemote::get_frame_border_color(NULL, out_val); }
   int     get_frame_border_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_frame_border_color(cid, out_val); }
   int     get_frame_border_visible(bool& out_val)
           { return ChipsRemote::get_frame_border_visible(NULL, out_val); }
   int     get_frame_border_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_frame_border_visible(cid, out_val); }
   int     display_frame_border()
           { return ChipsRemote::display_frame_border(NULL); }
   int     display_frame_border(ChipsId* cid)
           { return ChipsRemote::display_frame_border(cid); }
   int     hide_frame_border()
           { return ChipsRemote::hide_frame_border(NULL); }
   int     hide_frame_border(ChipsId* cid)
           { return ChipsRemote::hide_frame_border(cid); }
   int     get_frame_visible(bool& out_val)
           { return ChipsRemote::get_frame_visible(NULL, out_val); }
   int     get_frame_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_frame_visible(cid, out_val); }
   int     display_frame()
           { return ChipsRemote::display_frame(NULL); }
   int     display_frame(ChipsId* cid)
           { return ChipsRemote::display_frame(cid); }
   int     hide_frame()
           { return ChipsRemote::hide_frame(NULL); }
   int     hide_frame(ChipsId* cid)
           { return ChipsRemote::hide_frame(cid); }

   int     display_frame_layout(ChipsId* cid)
	   { return ChipsRemote::display_frame_layout (cid); };
   int     hide_frame_layout(ChipsId* cid)
           { return ChipsRemote::hide_frame_layout (cid); };
   int     set_layout_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_layout_color (cid, in_color); }; 
   int     set_layout_thickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_layout_thickness (cid, in_thickness); };
   int     set_layout_linestyle(ChipsId* cid, V_LINE_TYPE in_style)
           { return ChipsRemote::set_layout_linestyle (cid, in_style); };
   int     set_grid_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_grid_color (cid, in_color); }; 
   int     set_grid_thickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_grid_thickness (cid, in_thickness); };
   int     set_grid_linestyle(ChipsId* cid, V_LINE_TYPE in_style)
           { return ChipsRemote::set_grid_linestyle (cid, in_style); };
   int     set_grid_scale(ChipsId* cid, int in_scale)
           { return ChipsRemote::set_grid_scale (cid, in_scale); };

   int     pick(int count = 1, unsigned int mask = 0)
           { return ChipsRemote::pick(NULL, count, mask); }
   int     pick(ChipsId* cid, int count = 1, unsigned int mask = 0)
           { return ChipsRemote::pick(cid, count, mask); }
   int     get_pick(int& outcount, CHIPS_PICK_EVT_P* picklist,
                   int count = 1, unsigned int mask = 0)
           { return ChipsRemote::get_pick(NULL,outcount, picklist, count, mask);}
   int     get_pick(ChipsId* cid, int& outcount, CHIPS_PICK_EVT_P* picklist,
                    int count = 1, unsigned int mask = 0)
           { return ChipsRemote::get_pick(cid, outcount, picklist, count, mask);}


   // frame operations
   int     add_frame(string attlist = "")
           { return ChipsRemote::add_frame(NULL, attlist); }
   int     add_frame(float x1, float y1, float x2, float y2, 
                     string attlist = "")
           { return ChipsRemote::add_frame(NULL, x1, y1, x2, y2, attlist); }
   int     add_frame(ChipsId* cid, string attlist = "")
           { return ChipsRemote::add_frame(cid, attlist); }
   int     add_frame(ChipsId* cid, float x1, float y1, float x2, float y2, 
                     string attlist = "")
           { return ChipsRemote::add_frame(cid, x1, y1, x2, y2, attlist); }
   int     delete_frame()
           { return ChipsRemote::delete_frame(NULL); }
   int     delete_frame(ChipsId* cid)
           { return ChipsRemote::delete_frame(cid); }
   int     display_depth(int depth)
           { return ChipsRemote::display_depth(NULL, depth); }
   int     display_depth(ChipsId* cid, int depth)
           { return ChipsRemote::display_depth(cid, depth); }
   int     hide_depth(int depth)
           { return ChipsRemote::hide_depth(NULL, depth); }
   int     hide_depth(ChipsId* cid, int depth)
           { return ChipsRemote::hide_depth(cid, depth); }
   int     collapse_depths(int cnt, int* vals, int dest)
           { return ChipsRemote::collapse_depths(NULL, cnt, vals, dest); }
   int     collapse_depths(ChipsId* cid, int cnt, int* vals, int dest)
           { return ChipsRemote::collapse_depths(cid, cnt, vals, dest); }
   int     shuffle(ChipsId* cid, V_OBJECT_TYPE obj, V_SHUFFLE_TYPE way)
           { return ChipsRemote::shuffle(cid, obj, way); }
   int     shuffle(V_OBJECT_TYPE obj, V_SHUFFLE_TYPE way)
           { return ChipsRemote::shuffle(NULL, obj, way); }
   int     shuffle_forward(ChipsId* cid, V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_forward(cid, obj); }
   int     shuffle_forward(V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_forward(NULL, obj); }
   int     shuffle_backward(ChipsId* cid, V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_backward(cid, obj); }
   int     shuffle_backward(V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_backward(NULL, obj); }
   int     shuffle_front(ChipsId* cid, V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_front(cid, obj); }
   int     shuffle_front(V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_front(NULL, obj); }
   int     shuffle_back(ChipsId* cid, V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_back(cid, obj); }
   int     shuffle_back(V_OBJECT_TYPE obj)
           { return ChipsRemote::shuffle_back(NULL, obj); }
   int     set_default_depth(int sz)
           { return ChipsRemote::set_default_depth(NULL, sz); }
   int     set_default_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_default_depth(cid, sz); }
   int     get_default_depth(int& out_val)
           { return ChipsRemote::get_default_depth(NULL, out_val); }
   int     get_default_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_default_depth(cid, out_val); }
   int     set_framemode(int mode)
           { return ChipsRemote::set_framemode(NULL, mode); }
   int     set_framemode(ChipsId* cid, int mode)
           { return ChipsRemote::set_framemode(cid, mode); }
   int     tile(unsigned int cols, unsigned int rows)
           { return ChipsRemote::tile(NULL, cols, rows); } 
   int     tile(ChipsId* cid, unsigned int cols, unsigned int rows)
           { return ChipsRemote::tile(cid, cols, rows); } 
   int     row_tile(unsigned int rows)
           { return ChipsRemote::row_tile(NULL, rows); }
   int     row_tile(ChipsId* cid, unsigned int rows)
           { return ChipsRemote::row_tile(cid, rows); }
   int     col_tile(unsigned int cols)
           { return ChipsRemote::col_tile(NULL, cols); }
   int     col_tile(ChipsId* cid, unsigned int cols)
           { return ChipsRemote::col_tile(cid, cols); }
   int     prev_frame()
           { return ChipsRemote::prev_frame(NULL); }
   int     prev_frame(ChipsId* cid)
           { return ChipsRemote::prev_frame(cid); }
   int     next_frame()
           { return ChipsRemote::next_frame(NULL); }
   int     next_frame(ChipsId* cid)
           { return ChipsRemote::next_frame(cid); }
   int     reposition_frame(float x1, float y1, float x2, float y2) 
           { return ChipsRemote::reposition_frame(NULL, x1, y1, x2, y2); }
   int     reposition_frame(ChipsId* cid,float x1, float y1, float x2, float y2)
           { return ChipsRemote::reposition_frame(cid, x1, y1, x2, y2); }
   int     view_single_frame()
           { return ChipsRemote::view_single_frame(NULL); }
   int     view_single_frame(ChipsId* cid)
           { return ChipsRemote::view_single_frame(cid); }
   int     view_placed_frame()
           { return ChipsRemote::view_placed_frame(NULL); }
   int     view_placed_frame(ChipsId* cid)
           { return ChipsRemote::view_placed_frame(cid); }
   int     arrange_frames(int cnt, string* fids)
           { return ChipsRemote::arrange_frames(NULL, cnt, fids); }
   int     arrange_frames(ChipsId* cid, int cnt, string* fids)
           { return ChipsRemote::arrange_frames(cid, cnt, fids); }
   int     set_frame_transparency(bool seethrough) 
           { return ChipsRemote::set_frame_transparency(NULL, seethrough); }
   int     set_frame_transparency(ChipsId* cid, bool seethrough)
           { return ChipsRemote::set_frame_transparency(cid, seethrough); }
   int     get_frame_transparency(bool& out_val)
           { return ChipsRemote::get_frame_transparency(NULL, out_val); }
   int     get_frame_transparency(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_frame_transparency(cid, out_val); }
   int     set_frame_scale(bool val) 
           { return ChipsRemote::set_frame_scale(NULL, val); }
   int     set_frame_scale(ChipsId* cid, bool val) 
           { return ChipsRemote::set_frame_scale(cid, val); }
   int     get_frame_scale(bool& out_val)
           { return ChipsRemote::get_frame_scale(NULL, out_val); }
   int     get_frame_scale(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_frame_scale(cid, out_val); }
   int     blink_frames(ChipsId* cid, string fid2, float duration=3.0)
           { return ChipsRemote::blink_frames(cid, fid2, duration); }
   int     stop_blink()
           { return ChipsRemote::stop_blink(NULL); } 
   int     stop_blink(ChipsId* cid)
           { return ChipsRemote::stop_blink(cid); } 
   int     set_current_frame(ChipsId* cid)
           { return ChipsRemote::set_current_frame(cid); } 

   int     adjust_grid_xgap(float xgap)
           { return ChipsRemote::adjust_grid_xgap(NULL, xgap); } 
   int     adjust_grid_xgap(ChipsId* cid, float xgap)
           { return ChipsRemote::adjust_grid_xgap(cid, xgap); } 
   int     adjust_grid_ygap(float ygap)
           { return ChipsRemote::adjust_grid_ygap(NULL, ygap); } 
   int     adjust_grid_ygap(ChipsId* cid, float ygap)
           { return ChipsRemote::adjust_grid_ygap(cid, ygap); } 
   int     adjust_grid_gaps(float xgap, float ygap)
           { return ChipsRemote::adjust_grid_gaps(NULL, xgap, ygap); } 
   int     adjust_grid_gaps(ChipsId* cid, float xgap, float ygap)
           { return ChipsRemote::adjust_grid_gaps(cid, xgap, ygap); } 
   int     adjust_grid_xrelsize(int col, float ratio)
           { return ChipsRemote::adjust_grid_xrelsize(NULL, col, ratio); } 
   int     adjust_grid_xrelsize(ChipsId* cid, int col, float ratio)
           { return ChipsRemote::adjust_grid_xrelsize(cid, col, ratio); } 
   int     adjust_grid_yrelsize(int row, float ratio)
           { return ChipsRemote::adjust_grid_yrelsize(NULL, row, ratio); } 
   int     adjust_grid_yrelsize(ChipsId* cid, int row, float ratio)
           { return ChipsRemote::adjust_grid_yrelsize(cid, row, ratio); } 
   int     adjust_grid_xrelsizes(int fcnt, float* sizes)
           { return ChipsRemote::adjust_grid_xrelsizes(NULL, fcnt, sizes); }
   int     adjust_grid_xrelsizes(ChipsId* cid, int fcnt, float* sizes)
           { return ChipsRemote::adjust_grid_xrelsizes(cid, fcnt, sizes); }
   int     adjust_grid_yrelsizes(int fcnt, float* sizes)
           { return ChipsRemote::adjust_grid_yrelsizes(NULL, fcnt, sizes); }
   int     adjust_grid_yrelsizes(ChipsId* cid, int fcnt, float* sizes)
           { return ChipsRemote::adjust_grid_yrelsizes(cid, fcnt, sizes); }
   int     grid_objects(int cols, int rows, float xgap, 
                        float ygap, int usage = 0, int lcnt=0, 
                        string* lids = NULL)
           { return ChipsRemote::grid_objects(NULL, cols, rows, xgap, ygap,
                                              usage, lcnt, lids); }
   int     grid_objects(ChipsId* cid, int cols, int rows, float xgap, 
                        float ygap, int usage = 0, int lcnt=0, 
                        string* lids = NULL)
           { return ChipsRemote::grid_objects(cid, cols, rows, xgap, ygap,
                                              usage, lcnt, lids); }
   int     row_grid_objects(int cnt, int* vals, float xg, float yg, 
                            int usage = 0, int lcnt = 0, string* lids = NULL)
           { return ChipsRemote::row_grid_objects(NULL, cnt, vals, xg, yg,
                                                  usage, lcnt, lids); }
   int     row_grid_objects(ChipsId* cid, int cnt, int* vals,
                        float xg, float yg, int usage = 0, int lcnt = 0,
                        string* lids = NULL)
           { return ChipsRemote::row_grid_objects(cid, cnt, vals, xg, yg,
                                                  usage, lcnt, lids); }
   int     col_grid_objects(int cnt, int* vals, float xg, float yg,
                       int usage = 0, int lcnt = 0, string* lids = NULL)
           { return ChipsRemote::col_grid_objects(NULL, cnt, vals, xg, yg,
                                                  usage, lcnt, lids); }

   int     col_grid_objects(ChipsId* cid, int cnt, int* vals,
                        float xg, float yg, int usage = 0, int lcnt = 0,
                        string* lids = NULL)
           { return ChipsRemote::col_grid_objects(cid, cnt, vals, xg, yg,
                                                  usage, lcnt, lids); }
   int     swap_object_positions(string lid2)
           { return ChipsRemote::swap_object_positions(NULL, lid2); }
   int     swap_object_positions(ChipsId* cid, string lid2)
           { return ChipsRemote::swap_object_positions(cid, lid2); }

   // plot operations
   int     add_plot(string attlist = "")
           { return ChipsRemote::add_plot(NULL, attlist); }
   int     add_plot(ChipsId* cid, string attlist = "")
           { return ChipsRemote::add_plot(cid, attlist); }
   int     add_plot(float px1, float py1, float px2, float py2, 
                    string attlist = "")
           { return ChipsRemote::add_plot(NULL, px1, py1, px2, py2, attlist); }
   int     add_plot(ChipsId* cid, float px1, float py1, float px2, float py2, 
                    string attlist = "")
           { return ChipsRemote::add_plot(cid, px1, py1, px2, py2, attlist); }
   int     delete_plot()
           { return ChipsRemote::delete_plot(NULL); } 
   int     delete_plot(ChipsId* cid)
           { return ChipsRemote::delete_plot(cid); } 
   int     clear_plot()
           { return ChipsRemote::clear_plot(NULL); } 
   int     clear_plot(ChipsId* cid)
           { return ChipsRemote::clear_plot(cid); } 
   int     reposition_plot(float x1, float y1, float x2, float y2)
           { return ChipsRemote::reposition_plot(NULL, x1, y1, x2, y2); } 
   int     reposition_plot(ChipsId* cid, float x1, float y1, float x2, float y2)
           { return ChipsRemote::reposition_plot(cid, x1, y1, x2, y2); } 

   int     limits(double xmin, double xmax, double ymin, double ymax)
           { return ChipsRemote::limits(NULL, xmin, xmax, ymin, ymax); }
   int     limits(ChipsId* cid, double xmin, double xmax, double ymin, double ymax)
           { return ChipsRemote::limits(cid, xmin, xmax, ymin, ymax); }

   int     limits(int dim, double min, double max)
           { return ChipsRemote::limits(NULL, dim, min, max); } 
   int     limits(ChipsId* cid, int dim, double min, double max)
           { return ChipsRemote::limits(cid, dim, min, max); } 
   int     limits(int dim, V_AUTOLIMIT_TYPE automin, 
                  V_AUTOLIMIT_TYPE automax)
           { return ChipsRemote::limits(NULL, dim, automin, automax); }
   int     limits(ChipsId* cid, int dim, V_AUTOLIMIT_TYPE automin, 
                  V_AUTOLIMIT_TYPE automax)
           { return ChipsRemote::limits(cid, dim, automin, automax); } 
   int     limits(int dim, double min, V_AUTOLIMIT_TYPE automax)
           { return ChipsRemote::limits(NULL, dim, min, automax); } 
   int     limits(ChipsId* cid, int dim, double min, V_AUTOLIMIT_TYPE automax)
           { return ChipsRemote::limits(cid, dim, min, automax); } 
   int     limits(int dim, V_AUTOLIMIT_TYPE automin, double max) 
           { return ChipsRemote::limits(NULL, dim, automin, max); } 
   int     limits(ChipsId* cid, int dim, V_AUTOLIMIT_TYPE automin, double max) 
           { return ChipsRemote::limits(cid, dim, automin, max); } 
   int     limits(ChipsId* pcid, int dim, V_OBJECT_TYPE obj, ChipsId* ccid)
           { return ChipsRemote::limits(pcid, dim, obj, ccid); }
   int     limits(int dim, V_OBJECT_TYPE obj, ChipsId* ccid)
           { return ChipsRemote::limits(NULL, dim, obj, ccid); }
   // to avoid issues w/ overloading ambiguity in modules, remove the
   // dim paramerter from the limits object id calls- alternatively we
   // could rename the command limits_from_object 
   int     limits(ChipsId* pcid, V_OBJECT_TYPE obj, string oid)
           { return ChipsRemote::limits(pcid, XY_AXIS, obj, oid); }
   int     limits(V_OBJECT_TYPE obj, string oid)
           { return ChipsRemote::limits(NULL, XY_AXIS, obj, oid); }
   int     zoom()
           { return ChipsRemote::zoom(NULL); }
   int     zoom(ChipsId* cid)
           { return ChipsRemote::zoom(cid); }
   int     zoom(float scale)
           { return ChipsRemote::zoom(NULL, scale); }
   int     zoom(ChipsId* cid, double scale)
           { return ChipsRemote::zoom(cid, scale); }
   int     pan()
           { return ChipsRemote::pan(NULL); }
   int     pan(ChipsId* cid)
           { return ChipsRemote::pan(cid); }
   int     panto(double x, double y)
           { return ChipsRemote::panto(NULL, x, y); }
   int     panto(ChipsId* cid, double x, double y)
           { return ChipsRemote::panto(cid, x, y); }
   int     pick_limits()
           { return ChipsRemote::pick_limits(NULL, XY_AXIS); } 
   int     pick_limits(ChipsId* cid)
           { return ChipsRemote::pick_limits(cid, XY_AXIS); } 
   int     pick_limits(int dim)
           { return ChipsRemote::pick_limits(NULL, dim); } 
   int     pick_limits(ChipsId* cid, int dim)
           { return ChipsRemote::pick_limits(cid, dim); } 
   int     log_scale()
           { return ChipsRemote::log_scale(NULL, XY_AXIS); } 
   int     log_scale(ChipsId* cid)
           { return ChipsRemote::log_scale(cid, XY_AXIS); } 
   int     log_scale(int dim)
           { return ChipsRemote::log_scale(NULL, dim); } 
   int     log_scale(ChipsId* cid, int dim)
           { return ChipsRemote::log_scale(cid, dim); } 
   int     logarithmic_scale()
           { return ChipsRemote::log_scale(NULL, XY_AXIS); }
   int     logarithmic_scale(ChipsId* cid)
           { return ChipsRemote::log_scale(cid, XY_AXIS); }
   int     logarithmic_scale(int dim)
           { return ChipsRemote::log_scale(NULL, dim); }
   int     logarithmic_scale(ChipsId* cid, int dim)
           { return ChipsRemote::log_scale(cid, dim); }
   int     linear_scale()
           { return ChipsRemote::linear_scale(NULL, XY_AXIS); } 
   int     linear_scale(ChipsId* cid)
           { return ChipsRemote::linear_scale(cid, XY_AXIS); } 
   int     linear_scale(int dim)
           { return ChipsRemote::linear_scale(NULL, dim); } 
   int     linear_scale(ChipsId* cid, int dim)
           { return ChipsRemote::linear_scale(cid, dim); } 
   int     lin_scale()
           { return ChipsRemote::linear_scale(NULL, XY_AXIS); }
   int     lin_scale(ChipsId* cid)
           { return ChipsRemote::linear_scale(cid, XY_AXIS); }
   int     lin_scale(int dim)
           { return ChipsRemote::linear_scale(NULL, dim); }
   int     lin_scale(ChipsId* cid, int dim)
           { return ChipsRemote::linear_scale(cid, dim); }
   int     set_plot_title(string in_title)
           { return ChipsRemote::set_plot_title(NULL, in_title); }
   int     set_plot_title(ChipsId* cid, string in_title)
           { return ChipsRemote::set_plot_title(cid, in_title); }
   int     get_plot_title(string& out_val)
           { return ChipsRemote::get_plot_title(NULL, out_val); }
   int     get_plot_title(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_plot_title(cid, out_val); }
   int     set_plot_title_color(string in_color)
           { return ChipsRemote::set_plot_title_color(NULL, in_color); }
   int     set_plot_title_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_plot_title_color(cid, in_color); }
   int     get_plot_title_color(string& out_val)
           { return ChipsRemote::get_plot_title_color(NULL, out_val); }
   int     get_plot_title_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_plot_title_color(cid, out_val); }
   int     set_plot_title_angle(float in_val)
           { return ChipsRemote::set_plot_title_angle(NULL, in_val); }
   int     set_plot_title_angle(ChipsId* cid, float in_val)
           { return ChipsRemote::set_plot_title_angle(cid, in_val); }
   int     get_plot_title_angle(float& out_val)
           { return ChipsRemote::get_plot_title_angle(NULL, out_val); }
   int     get_plot_title_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_plot_title_angle(cid, out_val); }
   int     set_plot_title_halign(float in_val)
           { return ChipsRemote::set_plot_title_halign(NULL, in_val); }
   int     set_plot_title_halign(ChipsId* cid, float in_val)
           { return ChipsRemote::set_plot_title_halign(cid, in_val); }
   int     get_plot_title_halign(float& out_val)
           { return ChipsRemote::get_plot_title_halign(NULL, out_val); }
   int     get_plot_title_halign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_plot_title_halign(cid, out_val); }
   int     set_plot_title_valign(float in_val)
           { return ChipsRemote::set_plot_title_valign(NULL, in_val); }
   int     set_plot_title_valign(ChipsId* cid, float in_val)
           { return ChipsRemote::set_plot_title_valign(cid, in_val); }
   int     get_plot_title_valign(float& out_val)
           { return ChipsRemote::get_plot_title_valign(NULL, out_val); }
   int     get_plot_title_valign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_plot_title_valign(cid, out_val); }
   int     set_plot_title_size(int sz)
           { return ChipsRemote::set_plot_title_size(NULL, sz); }
   int     set_plot_title_size(ChipsId* cid, int sz)
           { return ChipsRemote::set_plot_title_size(cid, sz); }
   int     get_plot_title_size(int& out_val)
           { return ChipsRemote::get_plot_title_size(NULL, out_val); }
   int     get_plot_title_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_plot_title_size(cid, out_val); }
   int     set_plot_title_font(string font)
           { return ChipsRemote::set_plot_title_font(NULL, font); }
   int     set_plot_title_font(ChipsId* cid, string font)
           { return ChipsRemote::set_plot_title_font(cid, font); }
   int     get_plot_title_font(string &out_val)
           { return ChipsRemote::get_plot_title_font(NULL, out_val); }
   int     get_plot_title_font(ChipsId* cid, string &out_val)
           { return ChipsRemote::get_plot_title_font(cid, out_val); }
   int     set_plot_title_fontstyle(string style)
           { return ChipsRemote::set_plot_title_fontstyle(NULL, style); }
   int     set_plot_title_fontstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_plot_title_fontstyle(cid, style); }
   int     get_plot_title_fontstyle(string &out_val)
           { return ChipsRemote::get_plot_title_fontstyle(NULL, out_val); }
   int     get_plot_title_fontstyle(ChipsId* cid, string &out_val)
           { return ChipsRemote::get_plot_title_fontstyle(cid, out_val); }
   int     set_plot_title_xpos(float in_norm)
           { return ChipsRemote::set_plot_title_xpos(NULL, in_norm); }
   int     set_plot_title_xpos(ChipsId* cid, float in_norm)
           { return ChipsRemote::set_plot_title_xpos(cid, in_norm); }
   int     get_plot_title_xpos(float& out_val)
           { return ChipsRemote::get_plot_title_xpos(NULL, out_val); }
   int     get_plot_title_xpos(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_plot_title_xpos(cid, out_val); }
   int     set_plot_title_ypos(float in_norm)
           { return ChipsRemote::set_plot_title_ypos(NULL, in_norm); }
   int     set_plot_title_ypos(ChipsId* cid, float in_norm)
           { return ChipsRemote::set_plot_title_ypos(cid, in_norm); }
   int     get_plot_title_ypos(float& out_val)
           { return ChipsRemote::get_plot_title_ypos(NULL, out_val); }
   int     get_plot_title_ypos(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_plot_title_ypos(cid, out_val); }
   int     get_plot_range(double& outx1, double& outx2,
                          double& outy1, double& outy2)
           { return ChipsRemote::get_plot_range(NULL,outx1,outx2,outy1,outy2); }
   int     get_plot_range(ChipsId* cid, double& outx1, double& outx2,
                          double& outy1, double& outy2)
           { return ChipsRemote::get_plot_range(cid,outx1,outx2,outy1,outy2); }
   int     get_plot_xrange(double& out1, double& out2)
           { return ChipsRemote::get_plot_xrange(NULL, out1, out2); }
   int     get_plot_xrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_plot_xrange(cid, out1, out2); }
   int     get_plot_yrange(double& out1, double& out2)
           { return ChipsRemote::get_plot_yrange(NULL, out1, out2); }
   int     get_plot_yrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_plot_yrange(cid, out1, out2); }
   int     set_plot_xlabel(string in_lab)
           { return ChipsRemote::set_plot_xlabel(NULL, in_lab); }
   int     set_plot_xlabel(ChipsId* cid, string in_lab)
           { return ChipsRemote::set_plot_xlabel(cid, in_lab); }
   int     set_plot_ylabel(string in_lab)
           { return ChipsRemote::set_plot_ylabel(NULL, in_lab); }
   int     set_plot_ylabel(ChipsId* cid, string in_lab)
           { return ChipsRemote::set_plot_ylabel(cid, in_lab); }
   int     set_plot_axisstyle(string style) 
           { return ChipsRemote::set_plot_axisstyle(NULL, style); }
   int     set_plot_axisstyle(ChipsId* cid, string style) 
           { return ChipsRemote::set_plot_axisstyle(cid, style); }
   int     set_plot_axisstyle(V_AXISSTYLE_TYPE style)
           { return ChipsRemote::set_plot_axisstyle(NULL, style); }
   int     set_plot_axisstyle(ChipsId* cid, V_AXISSTYLE_TYPE style)
           { return ChipsRemote::set_plot_axisstyle(cid, style); }
   int     get_plot_axisstyle(string& out_val)
           { return ChipsRemote::get_plot_axisstyle(NULL, out_val); }
   int     get_plot_axisstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_plot_axisstyle(cid, out_val); }
   int     set_plot_cornerstyle(string style)
           { return ChipsRemote::set_plot_cornerstyle(NULL, style); }
   int     set_plot_cornerstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_plot_cornerstyle(cid, style); }
   int     set_plot_cornerstyle(V_CORNERSTYLE_TYPE style)
           { return ChipsRemote::set_plot_cornerstyle(NULL, style); }
   int     set_plot_cornerstyle(ChipsId* cid, V_CORNERSTYLE_TYPE style)
           { return ChipsRemote::set_plot_cornerstyle(cid, style); }
   int     get_plot_cornerstyle(string& out_val)
           { return ChipsRemote::get_plot_cornerstyle(NULL, out_val); }
   int     get_plot_cornerstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_plot_cornerstyle(cid, out_val); }
   int     hide_plot()
           { return ChipsRemote::hide_plot(NULL); }
   int     hide_plot(ChipsId* cid)
           { return ChipsRemote::hide_plot(cid); }
   int     display_plot()
           { return ChipsRemote::display_plot(NULL); }
   int     display_plot(ChipsId* cid)
           { return ChipsRemote::display_plot(cid); }
   int     get_plot_visible(bool& out_val)
           { return ChipsRemote::get_plot_visible(NULL, out_val); }
   int     get_plot_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_plot_visible(cid, out_val); }
   int     set_data_aspect_ratio(string val="")
           { return ChipsRemote::set_data_aspect_ratio(NULL, val); }
   int     set_data_aspect_ratio(ChipsId* cid, string val="")
           { return ChipsRemote::set_data_aspect_ratio(cid, val); }
   int     get_data_aspect_ratio(string& out_val)
           { return ChipsRemote::get_data_aspect_ratio(NULL, out_val); }
   int     get_data_aspect_ratio(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_data_aspect_ratio(cid, out_val); }
   int     set_plot_aspect_ratio(string val="")
           { return ChipsRemote::set_plot_aspect_ratio(NULL, val); }
   int     set_plot_aspect_ratio(ChipsId* cid, string val="")
           { return ChipsRemote::set_plot_aspect_ratio(cid, val); }
   int     get_plot_aspect_ratio(string& out_val)
           { return ChipsRemote::get_plot_aspect_ratio(NULL, out_val); }
   int     get_plot_aspect_ratio(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_plot_aspect_ratio(cid, out_val); }
   int     get_plot_aspect_width(float& out_val)
           { return ChipsRemote::get_plot_aspect_width(NULL, out_val); }
   int     get_plot_aspect_width(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_plot_aspect_width(cid, out_val); }
   int     get_plot_aspect_height(float& out_val)
           { return ChipsRemote::get_plot_aspect_height(NULL, out_val); }
   int     get_plot_aspect_height(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_plot_aspect_height(cid, out_val); }
   int     display_axis()
           { return ChipsRemote::display_axis(NULL); }
   int     display_axis(ChipsId* cid)
           { return ChipsRemote::display_axis(cid); }
   int     hide_axis()
           { return ChipsRemote::hide_axis(NULL); }
   int     hide_axis(ChipsId* cid)
           { return ChipsRemote::hide_axis(cid); }
   int     get_axis_visible(bool& out_val)
           { return ChipsRemote::get_axis_visible(NULL, out_val); }
   int     get_axis_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_axis_visible(cid, out_val); }
   int     set_plot_leftmargin(float margin_sz) 
           { return ChipsRemote::set_plot_leftmargin(NULL, margin_sz); }
   int     set_plot_leftmargin(ChipsId* cid, float margin_sz) 
           { return ChipsRemote::set_plot_leftmargin(cid, margin_sz); }
   int     get_plot_leftmargin(float& out_val) 
           { return ChipsRemote::get_plot_leftmargin(NULL, out_val); }
   int     get_plot_leftmargin(ChipsId* cid, float& out_val) 
           { return ChipsRemote::get_plot_leftmargin(cid, out_val); }
   int     set_plot_rightmargin(float margin_sz) 
           { return ChipsRemote::set_plot_rightmargin(NULL, margin_sz); }
   int     set_plot_rightmargin(ChipsId* cid, float margin_sz) 
           { return ChipsRemote::set_plot_rightmargin(cid, margin_sz); }
   int     get_plot_rightmargin(float& out_val) 
           { return ChipsRemote::get_plot_rightmargin(NULL, out_val); }
   int     get_plot_rightmargin(ChipsId* cid, float& out_val) 
           { return ChipsRemote::get_plot_rightmargin(cid, out_val); }
   int     set_plot_topmargin(float margin_sz) 
           { return ChipsRemote::set_plot_topmargin(NULL, margin_sz); }
   int     set_plot_topmargin(ChipsId* cid, float margin_sz) 
           { return ChipsRemote::set_plot_topmargin(cid, margin_sz); }
   int     get_plot_topmargin(float& out_val) 
           { return ChipsRemote::get_plot_topmargin(NULL, out_val); }
   int     get_plot_topmargin(ChipsId* cid, float& out_val) 
           { return ChipsRemote::get_plot_topmargin(cid, out_val); }
   int     set_plot_bottommargin(float margin_sz) 
           { return ChipsRemote::set_plot_bottommargin(NULL, margin_sz); }
   int     set_plot_bottommargin(ChipsId* cid, float margin_sz) 
           { return ChipsRemote::set_plot_bottommargin(cid, margin_sz); }
   int     get_plot_bottommargin(float& out_val) 
           { return ChipsRemote::get_plot_bottommargin(NULL, out_val); }
   int     get_plot_bottommargin(ChipsId* cid, float& out_val) 
           { return ChipsRemote::get_plot_bottommargin(cid, out_val); }
   int     set_plot_title_depth(int sz)
           { return ChipsRemote::set_plot_title_depth(NULL, sz); }
   int     set_plot_title_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_plot_title_depth(cid, sz); }
   int     get_plot_title_depth(int& out_val)
           { return ChipsRemote::get_plot_title_depth(NULL, out_val); }
   int     get_plot_title_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_plot_title_depth(cid, out_val); }
   int     reverse_axes()
           { return ChipsRemote::reverse_axes(NULL, XY_AXIS); } 
   int     reverse_axes(ChipsId* cid)
           { return ChipsRemote::reverse_axes(cid, XY_AXIS); } 
   int     reverse_axes(int dim)
           { return ChipsRemote::reverse_axes(NULL, dim); }
   int     reverse_axes(ChipsId* cid, int dim)
           { return ChipsRemote::reverse_axes(cid, dim); }
   int     set_axis_scalable(bool scalable) 
           { return ChipsRemote::set_axis_scalable(NULL, scalable); }
   int     set_axis_scalable(ChipsId* cid, bool scalable) 
           { return ChipsRemote::set_axis_scalable(cid, scalable); }
   int     display_major_ticks()
           { return ChipsRemote::display_major_ticks(NULL); }
   int     display_major_ticks(ChipsId* cid)
           { return ChipsRemote::display_major_ticks(cid); }
   int     hide_major_ticks()
           { return ChipsRemote::hide_major_ticks(NULL); }
   int     hide_major_ticks(ChipsId* cid)
           { return ChipsRemote::hide_major_ticks(cid); }
   int     get_major_ticks_visible(bool& out_val)
           { return ChipsRemote::get_major_ticks_visible(NULL, out_val); }
   int     get_major_ticks_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_major_ticks_visible(cid, out_val); }
   int     display_minor_ticks()
           { return ChipsRemote::display_minor_ticks(NULL); }
   int     display_minor_ticks(ChipsId* cid)
           { return ChipsRemote::display_minor_ticks(cid); }
   int     hide_minor_ticks()
           { return ChipsRemote::hide_minor_ticks(NULL); }
   int     hide_minor_ticks(ChipsId* cid)
           { return ChipsRemote::hide_minor_ticks(cid); }
   int     get_minor_ticks_visible(bool& out_val)
           { return ChipsRemote::get_minor_ticks_visible(NULL, out_val); }
   int     get_minor_ticks_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_minor_ticks_visible(cid, out_val); }
   int     set_current_plot(ChipsId* cid)
           { return ChipsRemote::set_current_plot(cid); } 
   int     set_arbitrary_tick_positions(unsigned int arraysize, double* pos,
                                        string* vals=NULL)
           { return ChipsRemote::set_arbitrary_tick_positions(NULL, arraysize,
                                                              pos,vals); }
   int     set_arbitrary_tick_positions(ChipsId* cid, unsigned int arraysize, 
                               double* pos, string* vals=NULL)
           { return ChipsRemote::set_arbitrary_tick_positions(cid, arraysize,
                                                              pos, vals); }
   int     set_axis_major_mode(string in_mode) 
           { return ChipsRemote::set_axis_major_mode(NULL, in_mode); }
   int     set_axis_major_mode(ChipsId* cid, string in_mode) 
           { return ChipsRemote::set_axis_major_mode(cid, in_mode); }
   int     get_axis_major_mode(string& out_val) 
           { return ChipsRemote::get_axis_major_mode(NULL, out_val); }
   int     get_axis_major_mode(ChipsId* cid, string& out_val) 
           { return ChipsRemote::get_axis_major_mode(cid, out_val); }
   int     set_axis_major_thickness(float in_thickness)
           { return ChipsRemote::set_axis_major_thickness(NULL, in_thickness); }
   int     set_axis_major_thickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_axis_major_thickness(cid, in_thickness); }
   int     get_axis_major_thickness(float& out_val)
           { return ChipsRemote::get_axis_major_thickness(NULL, out_val); }
   int     get_axis_major_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_major_thickness(cid, out_val); }
   int     set_axis_major_style(string in_style)
           { return ChipsRemote::set_axis_major_style(NULL, in_style); }
   int     set_axis_major_style(ChipsId* cid, string in_style)
           { return ChipsRemote::set_axis_major_style(cid, in_style); }
   int     set_axis_major_style(int in_style)
           { return ChipsRemote::set_axis_major_style(NULL, in_style); }
   int     set_axis_major_style(ChipsId* cid, int in_style)
           { return ChipsRemote::set_axis_major_style(cid, in_style); }
   int     get_axis_major_style(string& out_val)
           { return ChipsRemote::get_axis_major_style(NULL, out_val); }
   int     get_axis_major_style(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_major_style(cid, out_val); }
   int     set_axis_minor_mode(string in_mode)
           { return ChipsRemote::set_axis_minor_mode(NULL, in_mode); }
   int     set_axis_minor_mode(ChipsId* cid, string in_mode)
           { return ChipsRemote::set_axis_minor_mode(cid, in_mode); }
   int     get_axis_minor_mode(string& out_val)
           { return ChipsRemote::get_axis_minor_mode(NULL, out_val); }
   int     get_axis_minor_mode(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_minor_mode(cid, out_val); }
   int     set_axis_minor_thickness(float in_thickness)
           { return ChipsRemote::set_axis_minor_thickness(NULL, in_thickness); }
   int     set_axis_minor_thickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_axis_minor_thickness(cid, in_thickness); }
   int     get_axis_minor_thickness(float& out_val)
           { return ChipsRemote::get_axis_minor_thickness(NULL, out_val); }
   int     get_axis_minor_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_minor_thickness(cid, out_val); }
   int     set_axis_minor_style(string in_style)
           { return ChipsRemote::set_axis_minor_style(NULL, in_style); }
   int     set_axis_minor_style(ChipsId* cid, string in_style)
           { return ChipsRemote::set_axis_minor_style(cid, in_style); }
   int     set_axis_minor_style(int in_style)
           { return ChipsRemote::set_axis_minor_style(NULL, in_style); }
   int     set_axis_minor_style(ChipsId* cid, int in_style)
           { return ChipsRemote::set_axis_minor_style(cid, in_style); }
   int     get_axis_minor_style(string& out_val)
           { return ChipsRemote::get_axis_minor_style(NULL, out_val); }
   int     get_axis_minor_style(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_minor_style(cid, out_val); }
   int     set_axis_parallel_offset(float in_val)
           { return ChipsRemote::set_axis_parallel_offset(NULL, in_val); }
   int     set_axis_parallel_offset(ChipsId* cid, float in_val)
           { return ChipsRemote::set_axis_parallel_offset(cid, in_val); }
   int     get_axis_parallel_offset(float& out_val)
           { return ChipsRemote::get_axis_parallel_offset(NULL, out_val); }
   int     get_axis_parallel_offset(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_parallel_offset(cid, out_val); }
   int     set_axis_perpendicular_offset(float in_val)
           { return ChipsRemote::set_axis_perpendicular_offset(NULL, in_val); }
   int     set_axis_perpendicular_offset(ChipsId* cid, float in_val)
           { return ChipsRemote::set_axis_perpendicular_offset(cid, in_val); }
   int     get_axis_perpendicular_offset(float& out_val)
           { return ChipsRemote::get_axis_perpendicular_offset(NULL, out_val); }
   int     get_axis_perpendicular_offset(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_perpendicular_offset(cid, out_val); }
   int     set_axis_automin(bool on) 
           { return ChipsRemote::set_axis_automin(NULL, on); }
   int     set_axis_automin(ChipsId* cid, bool on) 
           { return ChipsRemote::set_axis_automin(cid, on); }
   int     get_axis_automin(bool& out_val) 
           { return ChipsRemote::get_axis_automin(NULL, out_val); }
   int     get_axis_automin(ChipsId* cid, bool& out_val) 
           { return ChipsRemote::get_axis_automin(cid, out_val); }
   int     set_axis_automax(bool on) 
           { return ChipsRemote::set_axis_automax(NULL, on); }
   int     set_axis_automax(ChipsId* cid, bool on) 
           { return ChipsRemote::set_axis_automax(cid, on); }
   int     get_axis_automax(bool& out_val) 
           { return ChipsRemote::get_axis_automax(NULL, out_val); }
   int     get_axis_automax(ChipsId* cid, bool& out_val) 
           { return ChipsRemote::get_axis_automax(cid, out_val); }
   int     set_axis_depth(int sz)
           { return ChipsRemote::set_axis_depth(NULL, sz); }
   int     set_axis_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_axis_depth(cid, sz); }
   int     get_axis_depth(int& out_val)
           { return ChipsRemote::get_axis_depth(NULL, out_val); }
   int     get_axis_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_depth(cid, out_val); }
   int     set_axis_pad(double in_val)
           { return ChipsRemote::set_axis_pad(NULL, in_val); }
   int     set_axis_pad(ChipsId* cid, double in_val)
           { return ChipsRemote::set_axis_pad(cid, in_val); }
   int     get_axis_pad(double& out_val)
           { return ChipsRemote::get_axis_pad(NULL, out_val); }
   int     get_axis_pad(ChipsId* cid, double& out_val)
           { return ChipsRemote::get_axis_pad(cid, out_val); }
   int     get_axis_scale(int& out_val)
           { return ChipsRemote::get_axis_scale(NULL, out_val); }
   int     get_axis_scale(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_scale(cid, out_val); }

   int     set_axis_label_color(string in_color)
           { return ChipsRemote::set_axis_label_color(NULL, in_color); }
   int     set_axis_label_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_axis_label_color(cid, in_color); }
   int     get_axis_label_color(string& out_val)
           { return ChipsRemote::get_axis_label_color(NULL, out_val); }
   int     get_axis_label_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_label_color(cid, out_val); }
   int     set_axis_label_size(int sz)
           { return ChipsRemote::set_axis_label_size(NULL, sz); }
   int     set_axis_label_size(ChipsId* cid, int sz)
           { return ChipsRemote::set_axis_label_size(cid, sz); }
   int     get_axis_label_size(int& out_val)
           { return ChipsRemote::get_axis_label_size(NULL, out_val); }
   int     get_axis_label_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_label_size(cid, out_val); }
   int     set_axis_label_angle(float sz)
           { return ChipsRemote::set_axis_label_angle(NULL, sz); }
   int     set_axis_label_angle(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_label_angle(cid, sz); }
   int     get_axis_label_angle(float& out_val)
           { return ChipsRemote::get_axis_label_angle(NULL, out_val); }
   int     get_axis_label_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_label_angle(cid, out_val); }
   int     set_axis_xlabel_angle(float sz)
           { return ChipsRemote::set_axis_xlabel_angle(NULL, sz); }
   int     set_axis_xlabel_angle(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_xlabel_angle(cid, sz); }
   int     get_axis_xlabel_angle(float& out_val)
           { return ChipsRemote::get_axis_xlabel_angle(NULL, out_val); }
   int     get_axis_xlabel_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_xlabel_angle(cid, out_val); }
   int     set_axis_ylabel_angle(float sz)
           { return ChipsRemote::set_axis_ylabel_angle(NULL, sz); }
   int     set_axis_ylabel_angle(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_ylabel_angle(cid, sz); }
   int     get_axis_ylabel_angle(float& out_val)
           { return ChipsRemote::get_axis_ylabel_angle(NULL, out_val); }
   int     get_axis_ylabel_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_ylabel_angle(cid, out_val); }
   int     set_axis_label_halign(float sz)
           { return ChipsRemote::set_axis_label_halign(NULL, sz); }
   int     set_axis_label_halign(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_label_halign(cid, sz); }
   int     get_axis_label_halign(float& out_val)
           { return ChipsRemote::get_axis_label_halign(NULL, out_val); }
   int     get_axis_label_halign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_label_halign(cid, out_val); }
   int     set_axis_label_valign(float sz)
           { return ChipsRemote::set_axis_label_valign(NULL, sz); }
   int     set_axis_label_valign(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_label_valign(cid, sz); }
   int     get_axis_label_valign(float& out_val)
           { return ChipsRemote::get_axis_label_valign(NULL, out_val); }
   int     get_axis_label_valign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_label_valign(cid, out_val); }
   int     set_axis_label_font(string font)
           { return ChipsRemote::set_axis_label_font(NULL, font); }
   int     set_axis_label_font(ChipsId* cid, string font)
           { return ChipsRemote::set_axis_label_font(cid, font); }
   int     get_axis_label_font(string& out_val)
           { return ChipsRemote::get_axis_label_font(NULL, out_val); }
   int     get_axis_label_font(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_label_font(cid, out_val); }
   int     set_axis_label_fontstyle(string style)
           { return ChipsRemote::set_axis_label_fontstyle(NULL, style); }
   int     set_axis_label_fontstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_axis_label_fontstyle(cid, style); }
   int     get_axis_label_fontstyle(string& out_val)
           { return ChipsRemote::get_axis_label_fontstyle(NULL, out_val); }
   int     get_axis_label_fontstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_label_fontstyle(cid, out_val); }
   int     set_axis_label_visible(bool vis)
           { return ChipsRemote::set_axis_label_visible(NULL, vis); }
   int     set_axis_label_visible(ChipsId* cid, bool vis)
           { return ChipsRemote::set_axis_label_visible(cid, vis); }
   int     get_axis_label_visible(bool& out_val)
           { return ChipsRemote::get_axis_label_visible(NULL, out_val); }
   int     get_axis_label_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_axis_label_visible(cid, out_val); }
   int     set_axis_ticklabel_style(string style)
           { return ChipsRemote::set_axis_ticklabel_style(NULL, style); }
   int     set_axis_ticklabel_style(ChipsId* cid, string style)
           { return ChipsRemote::set_axis_ticklabel_style(cid, style); }
   int     set_axis_ticklabel_style(int in_style)
           { return ChipsRemote::set_axis_ticklabel_style(NULL, in_style); }
   int     set_axis_ticklabel_style(ChipsId* cid, int in_style)
           { return ChipsRemote::set_axis_ticklabel_style(cid, in_style); }
   int     get_axis_ticklabel_style(string& out_val)
           { return ChipsRemote::get_axis_ticklabel_style(NULL, out_val); }
   int     get_axis_ticklabel_style(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_ticklabel_style(cid, out_val); }
   int     set_axis_ticklabel_color(string in_color)
           { return ChipsRemote::set_axis_ticklabel_color(NULL, in_color); }
   int     set_axis_ticklabel_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_axis_ticklabel_color(cid, in_color); }
   int     get_axis_ticklabel_color(string& out_val)
           { return ChipsRemote::get_axis_ticklabel_color(NULL, out_val); }
   int     get_axis_ticklabel_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_ticklabel_color(cid, out_val); }
   int     set_axis_ticklabel_size(int sz)
           { return ChipsRemote::set_axis_ticklabel_size(NULL, sz); }
   int     set_axis_ticklabel_size(ChipsId* cid, int sz)
           { return ChipsRemote::set_axis_ticklabel_size(cid, sz); }
   int     get_axis_ticklabel_size(int& out_val)
           { return ChipsRemote::get_axis_ticklabel_size(NULL, out_val); }
   int     get_axis_ticklabel_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_ticklabel_size(cid, out_val); }
   int     set_axis_ticklabel_angle(float sz)
           { return ChipsRemote::set_axis_ticklabel_angle(NULL, sz); }
   int     set_axis_ticklabel_angle(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_ticklabel_angle(cid, sz); }
   int     get_axis_ticklabel_angle(float& out_val)
           { return ChipsRemote::get_axis_ticklabel_angle(NULL, out_val); }
   int     get_axis_ticklabel_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_ticklabel_angle(cid, out_val); }
   int     set_axis_ticklabel_halign(float sz)
           { return ChipsRemote::set_axis_ticklabel_halign(NULL, sz); }
   int     set_axis_ticklabel_halign(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_ticklabel_halign(cid, sz); }
   int     get_axis_ticklabel_halign(float& out_val)
           { return ChipsRemote::get_axis_ticklabel_halign(NULL, out_val); }
   int     get_axis_ticklabel_halign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_ticklabel_halign(cid, out_val); }
   int     set_axis_ticklabel_valign(float sz)
           { return ChipsRemote::set_axis_ticklabel_valign(NULL, sz); }
   int     set_axis_ticklabel_valign(ChipsId* cid, float sz)
           { return ChipsRemote::set_axis_ticklabel_valign(cid, sz); }
   int     get_axis_ticklabel_valign(float& out_val)
           { return ChipsRemote::get_axis_ticklabel_valign(NULL, out_val); }
   int     get_axis_ticklabel_valign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_ticklabel_valign(cid, out_val); }
   int     set_axis_ticklabel_font(string font)
           { return ChipsRemote::set_axis_ticklabel_font(NULL, font); }
   int     set_axis_ticklabel_font(ChipsId* cid, string font)
           { return ChipsRemote::set_axis_ticklabel_font(cid, font); }
   int     get_axis_ticklabel_font(string& out_val)
           { return ChipsRemote::get_axis_ticklabel_font(NULL, out_val); }
   int     get_axis_ticklabel_font(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_ticklabel_font(cid, out_val); }
   int     set_axis_ticklabel_fontstyle(string style)
           { return ChipsRemote::set_axis_ticklabel_fontstyle(NULL, style); }
   int     set_axis_ticklabel_fontstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_axis_ticklabel_fontstyle(cid, style); }
   int     get_axis_ticklabel_fontstyle(string& out_val)
           { return ChipsRemote::get_axis_ticklabel_fontstyle(NULL, out_val); }
   int     get_axis_ticklabel_fontstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_ticklabel_fontstyle(cid, out_val); }
   int     set_axis_ticklabel_offset(int in_val)
           { return ChipsRemote::set_axis_ticklabel_offset(NULL, in_val); }
   int     set_axis_ticklabel_offset(ChipsId* cid, int in_val)
           { return ChipsRemote::set_axis_ticklabel_offset(cid, in_val); }
   int     get_axis_ticklabel_offset(int& out_val)
           { return ChipsRemote::get_axis_ticklabel_offset(NULL, out_val); }
   int     get_axis_ticklabel_offset(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_ticklabel_offset(cid, out_val); }
   int     set_axis_ticklabel_visible(bool vis)
           { return ChipsRemote::set_axis_ticklabel_visible(NULL, vis); }
   int     set_axis_ticklabel_visible(ChipsId* cid, bool vis)
           { return ChipsRemote::set_axis_ticklabel_visible(cid, vis); }
   int     get_axis_ticklabel_visible(bool& out_val)
           { return ChipsRemote::get_axis_ticklabel_visible(NULL, out_val); }
   int     get_axis_ticklabel_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_axis_ticklabel_visible(cid, out_val); }

   int     set_axis_majorgrid_visible(bool vis)
           { return ChipsRemote::set_axis_majorgrid_visible(NULL, vis); }
   int     set_axis_majorgrid_visible(ChipsId* cid, bool vis)
           { return ChipsRemote::set_axis_majorgrid_visible(cid, vis); }
   int     get_axis_majorgrid_visible(bool& out_val)
           { return ChipsRemote::get_axis_majorgrid_visible(NULL, out_val); }
   int     get_axis_majorgrid_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_axis_majorgrid_visible(cid, out_val); }
   int     set_axis_majorgrid_thickness(float in_size)
           { return ChipsRemote::set_axis_majorgrid_thickness(NULL, in_size); }
   int     set_axis_majorgrid_thickness(ChipsId* cid, float in_size)
           { return ChipsRemote::set_axis_majorgrid_thickness(cid, in_size); }
   int     get_axis_majorgrid_thickness(float& out_val)
           { return ChipsRemote::get_axis_majorgrid_thickness(NULL, out_val); }
   int     get_axis_majorgrid_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_majorgrid_thickness(cid, out_val); }
   int     set_axis_majorgrid_style(V_LINE_TYPE in_style)
           { return ChipsRemote::set_axis_majorgrid_style(NULL, in_style); }
   int     set_axis_majorgrid_style(ChipsId* cid, V_LINE_TYPE in_style)
           { return ChipsRemote::set_axis_majorgrid_style(cid, in_style); }
   int     get_axis_majorgrid_style(V_LINE_TYPE& out_val)
           { return ChipsRemote::get_axis_majorgrid_style(NULL, out_val); }
   int     get_axis_majorgrid_style(ChipsId* cid, V_LINE_TYPE& out_val)
           { return ChipsRemote::get_axis_majorgrid_style(cid, out_val); }
   int     set_axis_majorgrid_color(string in_color)
           { return ChipsRemote::set_axis_majorgrid_color(NULL, in_color); }
   int     set_axis_majorgrid_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_axis_majorgrid_color(cid, in_color); }
   int     get_axis_majorgrid_color(string& out_val)
           { return ChipsRemote::get_axis_majorgrid_color(NULL, out_val); }
   int     get_axis_majorgrid_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_majorgrid_color(cid, out_val); }

   int     set_axis_minorgrid_visible(bool vis)
           { return ChipsRemote::set_axis_minorgrid_visible(NULL, vis); }
   int     set_axis_minorgrid_visible(ChipsId* cid, bool vis)
           { return ChipsRemote::set_axis_minorgrid_visible(cid, vis); }
   int     get_axis_minorgrid_visible(bool& out_val)
           { return ChipsRemote::get_axis_minorgrid_visible(NULL, out_val); }
   int     get_axis_minorgrid_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_axis_minorgrid_visible(cid, out_val); }
   int     set_axis_minorgrid_thickness(float in_size)
           { return ChipsRemote::set_axis_minorgrid_thickness(NULL, in_size); }
   int     set_axis_minorgrid_thickness(ChipsId* cid, float in_size)
           { return ChipsRemote::set_axis_minorgrid_thickness(cid, in_size); }
   int     get_axis_minorgrid_thickness(float& out_val)
           { return ChipsRemote::get_axis_minorgrid_thickness(NULL, out_val); }
   int     get_axis_minorgrid_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_minorgrid_thickness(cid, out_val); }
   int     set_axis_minorgrid_style(V_LINE_TYPE in_style)
           { return ChipsRemote::set_axis_minorgrid_style(NULL, in_style); }
   int     set_axis_minorgrid_style(ChipsId* cid, V_LINE_TYPE in_style)
           { return ChipsRemote::set_axis_minorgrid_style(cid, in_style); }
   int     get_axis_minorgrid_style(V_LINE_TYPE& out_val)
           { return ChipsRemote::get_axis_minorgrid_style(NULL, out_val); }
   int     get_axis_minorgrid_style(ChipsId* cid, V_LINE_TYPE& out_val)
           { return ChipsRemote::get_axis_minorgrid_style(cid, out_val); }
   int     set_axis_minorgrid_color(string in_color)
           { return ChipsRemote::set_axis_minorgrid_color(NULL, in_color); }
   int     set_axis_minorgrid_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_axis_minorgrid_color(cid, in_color); }
   int     get_axis_minorgrid_color(string& out_val)
           { return ChipsRemote::get_axis_minorgrid_color(NULL, out_val); }
   int     get_axis_minorgrid_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_minorgrid_color(cid, out_val); }

   // curve operations 
   int     add_curve(long cnt, double* x, double* y, Transform *transform,
                      string attlist = "")
           { return ChipsRemote::add_curve(NULL, cnt, x,y, transform, attlist); }
   int     add_curve(long cnt, double* x, double* y, double* ydown, 
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(NULL, cnt, x, y, ydown, NULL, NULL,
                                           NULL, NULL, NULL, transform, attlist); }
   int     add_curve(long cnt, double* x, double* y, double* ydown,
                     double* yup, Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(NULL, cnt, x, y, ydown, yup,NULL,
                                           NULL, NULL, NULL, transform, attlist); }
   int     add_curve(long cnt, double* x, double* y, double* ydown,
                     double* yup, double* xdown,
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(NULL, cnt, x, y, ydown, yup,xdown,
                                           NULL, NULL, NULL, transform, attlist); }
   int     add_curve(long cnt, double* x, double* y, double* ydown,
                     double* yup, double* xdown, double* xup,  
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(NULL, cnt, x, y, ydown, yup,xdown,
                                           xup, NULL, NULL, transform, attlist); }
   int     add_curve(long cnt, double* x, double* y, double* ydown,
                     double* yup, double* xdown, double* xup,
                     double* ylims, Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(NULL, cnt, x, y, ydown, yup,xdown,
                                           xup, ylims, NULL, transform, attlist); }
   int     add_curve(long cnt, double* x, double* y, double* ydown,
                     double* yup, double* xdown, double* xup,
                     double* ylims, double* xlims, 
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(NULL, cnt, x, y, ydown, yup,xdown,
                                           xup, ylims, xlims, transform, attlist); }

   int     add_curve(ChipsId* cid, long cnt, double* x, double* y, 
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(cid, cnt, x,y, transform, attlist); }
   int     add_curve(ChipsId* cid, long cnt, double* x, double* y, 
                     double* ydown, Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(cid, cnt, x, y, ydown, NULL, NULL,
                                           NULL, NULL, NULL, transform, attlist); }
   int     add_curve(ChipsId* cid, long cnt, double* x, double* y,
                     double* ydown, double* yup,
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(cid, cnt, x, y, ydown, yup, NULL,
                                           NULL, NULL, NULL, transform, attlist); }
   int     add_curve(ChipsId* cid, long cnt, double* x, double* y,
                     double* ydown, double* yup,
                     double* xdown, Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(cid, cnt, x, y, ydown, yup, xdown,
                                           NULL, NULL, NULL, transform, attlist); }
   int     add_curve(ChipsId* cid, long cnt, double* x, double* y,
                     double* ydown, double* yup,
                     double* xdown, double* xup,
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(cid, cnt, x, y, ydown, yup, xdown,
                                           xup, NULL, NULL, transform, attlist); }
   int     add_curve(ChipsId* cid, long cnt, double* x, double* y,
                     double* ydown, double* yup,
                     double* xdown, double* xup,
                     double* ylims, Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(cid, cnt, x, y, ydown, yup, xdown,
                                           xup, ylims, NULL, transform, attlist); }
   int     add_curve(ChipsId* cid, long cnt, double* x, double* y,
                     double* ydown, double* yup,
                     double* xdown, double* xup,
                     double* ylims, double* xlims,
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_curve(cid, cnt, x, y, ydown, yup, xdown,
                                           xup, ylims, xlims, transform, attlist); }
   int     delete_curve()
           { return ChipsRemote::delete_curve(NULL); }
   int     delete_curve(ChipsId* cid)
           { return ChipsRemote::delete_curve(cid); }
   int     display_curve()
           { return ChipsRemote::display_curve(NULL); }
   int     display_curve(ChipsId* cid)
           { return ChipsRemote::display_curve(cid); }
   int     hide_curve()
           { return ChipsRemote::hide_curve(NULL); }
   int     hide_curve(ChipsId* cid)
           { return ChipsRemote::hide_curve(cid); }
   int     get_curve_visible(bool& out_val)
           { return ChipsRemote::get_curve_visible(NULL, out_val); }
   int     get_curve_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_curve_visible(cid, out_val); }
   int     set_curve_depth(int sz)
           { return ChipsRemote::set_curve_depth(NULL, sz); }
   int     set_curve_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_curve_depth(cid, sz); }
   int     get_curve_depth(int& out_val)
           { return ChipsRemote::get_curve_depth(NULL, out_val); }
   int     get_curve_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_curve_depth(cid, out_val); }
   int     set_curve_symbolstyle(V_SYMBOL_TYPE val)
           { return ChipsRemote::set_curve_symboltype(NULL, val); }
   int     set_curve_symbolstyle(ChipsId* cid, V_SYMBOL_TYPE val)
           { return ChipsRemote::set_curve_symboltype(cid, val); }
   int     get_curve_symbolstyle(V_SYMBOL_TYPE& out_val)
           { return ChipsRemote::get_curve_symboltype(NULL, out_val); }
   int     get_curve_symbolstyle(ChipsId* cid, V_SYMBOL_TYPE& out_val)
           { return ChipsRemote::get_curve_symboltype(cid, out_val); }
   int     set_curve_symbolfill(bool in_fill)
           { return ChipsRemote::set_curve_symbolfill(NULL, in_fill); }
   int     set_curve_symbolfill(ChipsId* cid, bool in_fill)
           { return ChipsRemote::set_curve_symbolfill(cid, in_fill); }
   int     get_curve_symbolfill(bool& out_val)
           { return ChipsRemote::get_curve_symbolfill(NULL, out_val); }
   int     get_curve_symbolfill(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_curve_symbolfill(cid, out_val); }
   int     set_curve_symbolsize(int in_size)
           { return ChipsRemote::set_curve_symbolsize(NULL, in_size); }
   int     set_curve_symbolsize(ChipsId* cid, int in_size)
           { return ChipsRemote::set_curve_symbolsize(cid, in_size); }
   int     get_curve_symbolsize(int& out_val)
           { return ChipsRemote::get_curve_symbolsize(NULL, out_val); }
   int     get_curve_symbolsize(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_curve_symbolsize(cid, out_val); }
   int     set_curve_symbolangle(float in_val)
           { return ChipsRemote::set_curve_symbolangle(NULL, in_val); }
   int     set_curve_symbolangle(ChipsId* cid, float in_val)
           { return ChipsRemote::set_curve_symbolangle(cid, in_val); }
   int     get_curve_symbolangle(float& out_val)
           { return ChipsRemote::get_curve_symbolangle(NULL, out_val); }
   int     get_curve_symbolangle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_curve_symbolangle(cid, out_val); }
   int     set_curve_symbolcolor(string in_color)
           { return ChipsRemote::set_curve_symbolcolor(NULL, in_color); }
   int     set_curve_symbolcolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_curve_symbolcolor(cid, in_color); }
   int     get_curve_symbolcolor(string& out_color)
           { return ChipsRemote::get_curve_symbolcolor(NULL, out_color); }
   int     get_curve_symbolcolor(ChipsId* cid, string& out_color)
           { return ChipsRemote::get_curve_symbolcolor(cid, out_color); }
   int     set_curve_linestyle(V_LINE_TYPE val)
           { return ChipsRemote::set_curve_linestyle(NULL, val); }
   int     set_curve_linestyle(ChipsId* cid, V_LINE_TYPE val)
           { return ChipsRemote::set_curve_linestyle(cid, val); }
   int     get_curve_linestyle(V_LINE_TYPE& out_val)
           { return ChipsRemote::get_curve_linestyle(NULL, out_val); }
   int     get_curve_linestyle(ChipsId* cid, V_LINE_TYPE& out_val)
           { return ChipsRemote::get_curve_linestyle(cid, out_val); }
   int     set_curve_linethickness(float in_size)
           { return ChipsRemote::set_curve_linethickness(NULL, in_size); }
   int     set_curve_linethickness(ChipsId* cid, float in_size)
           { return ChipsRemote::set_curve_linethickness(cid, in_size); }
   int     get_curve_linethickness(float& out_val)
           { return ChipsRemote::get_curve_linethickness(NULL, out_val); }
   int     get_curve_linethickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_curve_linethickness(cid, out_val); }
   int     set_curve_linecolor(string in_color)
           { return ChipsRemote::set_curve_linecolor(NULL, in_color); }
   int     set_curve_linecolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_curve_linecolor(cid, in_color); }
   int     get_curve_linecolor(string& out_val)
           { return ChipsRemote::get_curve_linecolor(NULL, out_val); }
   int     get_curve_linecolor(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_curve_linecolor(cid, out_val); }
   int     set_curve_baddatamode(V_BADDATAMODE_TYPE val)
           { return ChipsRemote::set_curve_baddatamode(NULL, val); }
   int     set_curve_baddatamode(ChipsId* cid, V_BADDATAMODE_TYPE val)
           { return ChipsRemote::set_curve_baddatamode(cid, val); }
   int     get_curve_baddatamode(V_BADDATAMODE_TYPE& out_val)
           { return ChipsRemote::get_curve_baddatamode(NULL, out_val); }
   int     get_curve_baddatamode(ChipsId* cid, V_BADDATAMODE_TYPE& out_val)
           { return ChipsRemote::get_curve_baddatamode(cid, out_val); }
   int     set_curve_errstyle(string style)
           { return ChipsRemote::set_curve_errstyle(NULL, style); }
   int     set_curve_errstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_curve_errstyle(cid, style); }
   int     get_curve_errstyle(string& out_val)
           { return ChipsRemote::get_curve_errstyle(NULL, out_val); }
   int     get_curve_errstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_curve_errstyle(cid, out_val); }
   int     set_curve_errup(bool val)
           { return ChipsRemote::set_curve_errup(NULL, val); }
   int     set_curve_errup(ChipsId* cid, bool val)
           { return ChipsRemote::set_curve_errup(cid, val); }
   int     get_curve_errup(bool& out_val)
           { return ChipsRemote::get_curve_errup(NULL, out_val); }
   int     get_curve_errup(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_curve_errup(cid, out_val); }
   int     set_curve_errdown(bool val)
           { return ChipsRemote::set_curve_errdown(NULL, val); }
   int     set_curve_errdown(ChipsId* cid, bool val)
           { return ChipsRemote::set_curve_errdown(cid, val); }
   int     get_curve_errdown(bool& out_val)
           { return ChipsRemote::get_curve_errdown(NULL, out_val); }
   int     get_curve_errdown(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_curve_errdown(cid, out_val); }
   int     set_curve_errleft(bool val)
           { return ChipsRemote::set_curve_errleft(NULL, val); }
   int     set_curve_errleft(ChipsId* cid, bool val)
           { return ChipsRemote::set_curve_errleft(cid, val); }
   int     get_curve_errleft(bool& out_val)
           { return ChipsRemote::get_curve_errleft(NULL, out_val); }
   int     get_curve_errleft(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_curve_errleft(cid, out_val); }
   int     set_curve_errright(bool val)
           { return ChipsRemote::set_curve_errright(NULL, val); }
   int     set_curve_errright(ChipsId* cid, bool val)
           { return ChipsRemote::set_curve_errright(cid, val); }
   int     get_curve_errright(bool& out_val)
           { return ChipsRemote::get_curve_errright(NULL, out_val); }
   int     get_curve_errright(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_curve_errright(cid, out_val); }
   int     set_curve_errcaplength(int in_size)
           { return ChipsRemote::set_curve_errcaplength(NULL, in_size); }
   int     set_curve_errcaplength(ChipsId* cid, int in_size)
           { return ChipsRemote::set_curve_errcaplength(cid, in_size); }
   int     get_curve_errcaplength(int& out_val)
           { return ChipsRemote::get_curve_errcaplength(NULL, out_val); }
   int     get_curve_errcaplength(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_curve_errcaplength(cid, out_val); }
   int     set_curve_errthickness(float in_thickness)
           { return ChipsRemote::set_curve_errthickness(NULL, in_thickness); }
   int     set_curve_errthickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_curve_errthickness(cid, in_thickness); }
   int     get_curve_errthickness(float& out_val)
           { return ChipsRemote::get_curve_errthickness(NULL, out_val); }
   int     get_curve_errthickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_curve_errthickness(cid, out_val); }
   int     set_curve_errcolor(string in_color)
           { return ChipsRemote::set_curve_errcolor(NULL, in_color); }
   int     set_curve_errcolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_curve_errcolor(cid, in_color); }
   int     get_curve_errcolor(string& out_val)
           { return ChipsRemote::get_curve_errcolor(NULL, out_val); }
   int     get_curve_errcolor(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_curve_errcolor(cid, out_val); }
   int     set_curve_limitoverride(bool val)
           { return ChipsRemote::set_curve_limitoverride(NULL, val); }
   int     set_curve_limitoverride(ChipsId* cid, bool val)
           { return ChipsRemote::set_curve_limitoverride(cid, val); }
   int     get_curve_limitoverride(bool& out_val)
           { return ChipsRemote::get_curve_limitoverride(NULL, out_val); }
   int     get_curve_limitoverride(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_curve_limitoverride(cid, out_val); }
   int     set_curve_limitlength(double in_val)
           { return ChipsRemote::set_curve_limitlength(NULL, in_val); }
   int     set_curve_limitlength(ChipsId* cid, double in_val)
           { return ChipsRemote::set_curve_limitlength(cid, in_val); }
   int     get_curve_limitlength(double& out_val)
           { return ChipsRemote::get_curve_limitlength(NULL, out_val); }
   int     get_curve_limitlength(ChipsId* cid, double& out_val)
           { return ChipsRemote::get_curve_limitlength(cid, out_val); }

   int     set_current_curve(ChipsId* cid)
           { return ChipsRemote::set_current_curve(cid); } 
   int     get_curve_range(double& outx1, double& outx2,
                          double& outy1, double& outy2)
           { return ChipsRemote::get_curve_range(NULL,outx1,outx2,outy1,outy2);}
   int     get_curve_range(ChipsId* cid, double& outx1, double& outx2,
                          double& outy1, double& outy2)
           { return ChipsRemote::get_curve_range(cid,outx1,outx2,outy1,outy2); }
   int     get_curve_xrange(double& out1, double& out2)
           { return ChipsRemote::get_curve_xrange(NULL, out1, out2); }
   int     get_curve_xrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_curve_xrange(cid, out1, out2); }
   int     get_curve_yrange(double& out1, double& out2)
           { return ChipsRemote::get_curve_yrange(NULL, out1, out2); }
   int     get_curve_yrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_curve_yrange(cid, out1, out2); }

   // contour operations 
   int     add_contour(ChipsId* cid, double* dat, long xdim, long ydim,
		       Transform *trans, string attlist = "")
           {  return ChipsRemote::add_contour(cid, dat, xdim, ydim,
                                              0, NULL, trans, attlist); }
   int     add_contour(ChipsId* cid, double* dat, long xdim, long ydim,
                       unsigned int numlev, double* levels,
                       Transform *trans, string attlist = "")
           {  return ChipsRemote::add_contour(cid, dat, xdim, ydim,
                                              numlev, levels, trans, attlist);}
 
   int     add_contour(double* dat, long xdim, long ydim,
                       Transform *trans, string attlist = "")
           {  return ChipsRemote::add_contour(NULL, dat, xdim, ydim,
                                              0, NULL, trans, attlist); } 
   int     add_contour(double* dat, long xdim, long ydim,
                       unsigned int numlev,
                       double* levels, Transform *trans, string attlist = "")
           {  return ChipsRemote::add_contour(NULL, dat, xdim, ydim,
                                              numlev, levels, trans, attlist);}

   int     add_contour(long cnt, double* x, double* y, double* z, 
                       Transform *trans, string attlist = "") 
           { return ChipsRemote::add_contour(NULL, cnt, x, y, z, 0, NULL,
                                            trans, attlist); } 
   int     add_contour(long cnt, double* x, double* y, double* z,
                       unsigned int numlev, double* levels,
                       Transform *trans, string attlist = "")
           { return ChipsRemote::add_contour(NULL, cnt, x, y, z, numlev, levels,
                                            trans, attlist); }

   int     add_contour(ChipsId* cid, long cnt, double* x, double* y,
                       double* z, Transform *trans, string attlist = "")
           { return ChipsRemote::add_contour(cid, cnt, x, y, z, 0, NULL,
                                            trans, attlist); } 
   int     add_contour(ChipsId* cid, long cnt, double* x, double* y,
                       double* z, unsigned int numlev,
                       double* levels, Transform *trans, string attlist = "")
           { return ChipsRemote::add_contour(cid, cnt, x, y, z, numlev,levels,
                                            trans, attlist); }

   int     add_contour(ChipsId* cid, double* dat, long xdim, long ydim,
                       string attlist = "")
           {  return ChipsRemote::add_contour(cid, dat, xdim, ydim,
                                              0, NULL, NULL, attlist); }
   int     add_contour(ChipsId* cid, double* dat, long xdim, long ydim,
                       unsigned int numlev, double* levels,
                       string attlist = "")
           {  return ChipsRemote::add_contour(cid, dat, xdim, ydim,
                                              numlev, levels, NULL, attlist);}

   int     add_contour(double* dat, long xdim, long ydim,
                       string attlist = "")
           {  return ChipsRemote::add_contour(NULL, dat, xdim, ydim,
                                              0, NULL, NULL, attlist); }
   int     add_contour(double* dat, long xdim, long ydim,
                       unsigned int numlev,
                       double* levels, string attlist = "")
           {  return ChipsRemote::add_contour(NULL, dat, xdim, ydim,
                                              numlev, levels, NULL, attlist);}

   int     add_contour(long cnt, double* x, double* y, double* z,
                       string attlist = "")
           { return ChipsRemote::add_contour(NULL, cnt, x, y, z, 0, NULL,
                                            NULL, attlist); }
   int     add_contour(long cnt, double* x, double* y, double* z,
                       unsigned int numlev, double* levels,
                       string attlist = "")
           { return ChipsRemote::add_contour(NULL, cnt, x, y, z, numlev, levels,
                                             NULL,attlist); }

   int     add_contour(ChipsId* cid, long cnt, double* x, double* y,
                       double* z, string attlist = "")
           { return ChipsRemote::add_contour(cid, cnt, x, y, z, 0, NULL,
                                             NULL,attlist); }
   int     add_contour(ChipsId* cid, long cnt, double* x, double* y,
                       double* z, unsigned int numlev,
                       double* levels, string attlist = "")
           { return ChipsRemote::add_contour(cid, cnt, x, y, z, numlev,levels,
                                             NULL,attlist); }


   int     delete_contour()
           { return ChipsRemote::delete_contour(NULL); }
   int     delete_contour(ChipsId* cid)
           { return ChipsRemote::delete_contour(cid); }
   int     display_contour()
           { return ChipsRemote::display_contour(NULL); }
   int     display_contour(ChipsId* cid)
           { return ChipsRemote::display_contour(cid); }
   int     hide_contour()
           { return ChipsRemote::hide_contour(NULL); }
   int     hide_contour(ChipsId* cid)
           { return ChipsRemote::hide_contour(cid); }
   int     get_contour_visible(bool& out_val)
           { return ChipsRemote::get_contour_visible(NULL, out_val); }
   int     get_contour_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_contour_visible(cid, out_val); }
   int     set_contour_mode(string mode) 
           { return ChipsRemote::set_contour_mode(NULL, mode); }
   int     set_contour_mode(ChipsId* cid, string mode) 
           { return ChipsRemote::set_contour_mode(cid, mode); }
   int     get_contour_mode(string& out_val) 
           { return ChipsRemote::get_contour_mode(NULL, out_val); }
   int     get_contour_mode(ChipsId* cid, string& out_val) 
           { return ChipsRemote::get_contour_mode(cid, out_val); }
   int     set_contour_style(string style) 
           { return ChipsRemote::set_contour_style(NULL, style); }
   int     set_contour_style(ChipsId* cid, string style) 
           { return ChipsRemote::set_contour_style(cid, style); }
   int     get_contour_style(string& out_val) 
           { return ChipsRemote::get_contour_style(NULL, out_val); }
   int     get_contour_style(ChipsId* cid, string& out_val) 
           { return ChipsRemote::get_contour_style(cid, out_val); }
   int     set_contour_color(string in_color) 
           { return ChipsRemote::set_contour_color(NULL, in_color); }
   int     set_contour_color(ChipsId* cid, string in_color) 
           { return ChipsRemote::set_contour_color(cid, in_color); }
   int     get_contour_color(string& out_val) 
           { return ChipsRemote::get_contour_color(NULL, out_val); }
   int     get_contour_color(ChipsId* cid, string& out_val) 
           { return ChipsRemote::get_contour_color(cid, out_val); }
   int     set_contour_numlevels(ChipsId* cid, int in_val)
           { return ChipsRemote::set_contour_numlevels(cid, in_val); }
   int     set_contour_numlevels(int in_val)
           { return ChipsRemote::set_contour_numlevels(NULL, in_val); }
   int     get_contour_numlevels(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_contour_numlevels(cid, out_val); }
   int     get_contour_numlevels(int& out_val)
           { return ChipsRemote::get_contour_numlevels(NULL, out_val); }
   int     set_contour_interval(ChipsId* cid, double in_val)
           { return ChipsRemote::set_contour_interval(cid, in_val); }
   int     set_contour_interval(double in_val)
           { return ChipsRemote::set_contour_interval(NULL, in_val); }
   int     get_contour_interval(ChipsId* cid, double& out_val)
           { return ChipsRemote::get_contour_interval(cid, out_val); }
   int     get_contour_interval(double& out_val)
           { return ChipsRemote::get_contour_interval(NULL, out_val); }
   int     set_contour_levels(unsigned int arraysize, double* pos) 
           { return ChipsRemote::set_contour_levels(NULL, arraysize, pos); }
   int     set_contour_levels(ChipsId* cid, unsigned int arraysize, 
                              double* pos)
           { return ChipsRemote::set_contour_levels(cid, arraysize, pos); }
   int     get_contour_levels(unsigned int &arraysize, double** pos);
   int     get_contour_levels(ChipsId* cid, unsigned int &arraysize,
                              double** pos);
   int     set_contour_thickness(float in_size)
           { return ChipsRemote::set_contour_thickness(NULL, in_size); }
   int     set_contour_thickness(ChipsId* cid, float in_size)
           { return ChipsRemote::set_contour_thickness(cid, in_size); }
   int     get_contour_thickness(float& out_val)
           { return ChipsRemote::get_contour_thickness(NULL, out_val); }
   int     get_contour_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_contour_thickness(cid, out_val); }
   int     set_current_contour(ChipsId* cid)
           { return ChipsRemote::set_current_contour(cid); }
   int     get_contour_range(double& outx1, double& outx2,
                             double& outy1, double& outy2)
           { return ChipsRemote::get_contour_range(NULL,outx1,outx2,outy1,outy2); }
   int     get_contour_range(ChipsId* cid, double& outx1, double& outx2,
                             double& outy1, double& outy2)
           { return ChipsRemote::get_contour_range(cid,outx1,outx2,outy1,outy2); }
   int     get_contour_xrange(double& out1, double& out2)
           { return ChipsRemote::get_contour_xrange(NULL, out1, out2); }
   int     get_contour_xrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_contour_xrange(cid, out1, out2); }
   int     get_contour_yrange(double& out1, double& out2)
           { return ChipsRemote::get_contour_yrange(NULL, out1, out2); }
   int     get_contour_yrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_contour_yrange(cid, out1, out2); }
   int     get_contour_zrange(double& out1, double& out2)
           { return ChipsRemote::get_contour_zrange(NULL, out1, out2); }
   int     get_contour_zrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_contour_zrange(cid, out1, out2); }
   int     set_contour_algorithm(V_CONTOUR_TYPE val)
           { return ChipsRemote::set_contour_algorithm(NULL, val); }
   int     set_contour_algorithm(ChipsId* cid, V_CONTOUR_TYPE val)
           { return ChipsRemote::set_contour_algorithm(cid, val); }
   int     get_contour_algorithm(V_CONTOUR_TYPE& out_contour)
           { return ChipsRemote::get_contour_algorithm(NULL, out_contour); }
   int     get_contour_algorithm(ChipsId* cid, V_CONTOUR_TYPE& out_contour)
           { return ChipsRemote::get_contour_algorithm(cid, out_contour); }
   int     set_contour_depth(int sz)
           { return ChipsRemote::set_contour_depth(NULL, sz); }
   int     set_contour_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_contour_depth(cid, sz); }
   int     get_contour_depth(int& out_val)
           { return ChipsRemote::get_contour_depth(NULL, out_val); }
   int     get_contour_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_contour_depth(cid, out_val); }
/*
DEPRECATED
   int     set_contour_transform(Transform *trans)
           { return ChipsRemote::set_contour_transform(NULL, trans); }
   int     set_contour_transform(ChipsId* cid, Transform *trans)
           { return ChipsRemote::set_contour_transform(cid, trans); }
   int     get_contour_transform (Transform **trans)
           { return ChipsRemote::get_contour_transform(NULL, trans); }
   int     get_contour_transform(ChipsId* cid, Transform **trans)
           { return ChipsRemote::get_contour_transform(cid, trans); }
   int     apply_contour_transform()
           { return ChipsRemote::apply_contour_transform(NULL); }
   int     apply_contour_transform(ChipsId* cid)
           { return ChipsRemote::apply_contour_transform(cid); }
   int     apply_contour_transform_inverse()
           { return ChipsRemote::apply_contour_transform_inverse(NULL); }
   int     apply_contour_transform_inverse(ChipsId* cid)
           { return ChipsRemote::apply_contour_transform_inverse(cid); }
*/


   // histogram operations
   int     add_histogram(long cnt, double* counts, string attlist = "")
           { return ChipsRemote::add_histogram(NULL, cnt, NULL, NULL, counts,
                                               NULL, NULL, attlist); }
   int     add_histogram(long cnt, double* bin_low, double* counts,
                          string attlist = "")
           { return ChipsRemote::add_histogram(NULL, cnt, bin_low, NULL, counts,
                                               NULL, NULL, attlist); }
   int     add_histogram(long cnt, double* bin_low, double* bin_high,
                         double* counts, string attlist = "")
           { return ChipsRemote::add_histogram(NULL, cnt, bin_low, bin_high,
                                               counts, NULL, NULL, attlist); }
   int     add_histogram(long cnt, double* bin_low, double* bin_high,
                         double* counts, double* yd, string attlist = "")
           { return ChipsRemote::add_histogram(NULL, cnt, bin_low, bin_high,
                                               counts, yd, NULL, attlist); }
   int     add_histogram(long cnt, double* bin_low, double* bin_high,
                         double* counts, double* yd, double* yu,
                         string attlist = "")
           { return ChipsRemote::add_histogram(NULL, cnt, bin_low, bin_high,
                                               counts, yd, yu, attlist); }

   int     add_histogram(ChipsId* cid, long cnt, double* counts, 
                         string attlist = "")
           { return ChipsRemote::add_histogram(cid, cnt, NULL, NULL, counts,
                                               NULL, NULL, attlist); }
   int     add_histogram(ChipsId* cid, long cnt, double* bin_low,
                         double* counts, string attlist = "")
           { return ChipsRemote::add_histogram(cid, cnt, bin_low, NULL, counts,
                                               NULL, NULL, attlist); }
   int     add_histogram(ChipsId* cid,long cnt,double* bin_low,double* bin_high,
                         double* counts, string attlist = "")
           { return ChipsRemote::add_histogram(cid, cnt, bin_low, bin_high,
                                               counts, NULL, NULL, attlist); }
   int     add_histogram(ChipsId* cid, long cnt, double* bin_low,
                         double* bin_high, double* counts, double* yd, 
                         string attlist = "")
           { return ChipsRemote::add_histogram(cid, cnt, bin_low, bin_high,
                                               counts, yd, NULL, attlist); }
   int     add_histogram(ChipsId* cid, long cnt, double* bin_low, 
                         double* bin_high, double* counts, double* yd, 
                         double* yu, string attlist = "")
           { return ChipsRemote::add_histogram(cid, cnt, bin_low, bin_high,
                                               counts, yd, yu, attlist); }

   int     delete_histogram()
           { return ChipsRemote::delete_histogram(NULL); }
   int     delete_histogram(ChipsId* cid)
           { return ChipsRemote::delete_histogram(cid); }
   int     display_histogram()
           { return ChipsRemote::display_histogram(NULL); }
   int     display_histogram(ChipsId* cid)
           { return ChipsRemote::display_histogram(cid); }
   int     hide_histogram()
           { return ChipsRemote::hide_histogram(NULL); }
   int     hide_histogram(ChipsId* cid)
           { return ChipsRemote::hide_histogram(cid); }
   int     get_histogram_visible(bool& out_val)
           { return ChipsRemote::get_histogram_visible(NULL, out_val); }
   int     get_histogram_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_histogram_visible(cid, out_val); }
   int     set_histogram_symbolstyle(V_SYMBOL_TYPE val)
           { return ChipsRemote::set_histogram_symboltype(NULL, val); }
   int     set_histogram_symbolstyle(ChipsId* cid, V_SYMBOL_TYPE val)
           { return ChipsRemote::set_histogram_symboltype(cid, val); }
   int     get_histogram_symbolstyle(V_SYMBOL_TYPE& out_val)
           { return ChipsRemote::get_histogram_symboltype(NULL, out_val); }
   int     get_histogram_symbolstyle(ChipsId* cid, V_SYMBOL_TYPE& out_val)
           { return ChipsRemote::get_histogram_symboltype(cid, out_val); }
   int     set_histogram_symbolfill(bool in_fill)
           { return ChipsRemote::set_histogram_symbolfill(NULL, in_fill); }
   int     set_histogram_symbolfill(ChipsId* cid, bool in_fill)
           { return ChipsRemote::set_histogram_symbolfill(cid, in_fill); }
   int     get_histogram_symbolfill(bool& out_val)
           { return ChipsRemote::get_histogram_symbolfill(NULL, out_val); }
   int     get_histogram_symbolfill(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_histogram_symbolfill(cid, out_val); }
   int     set_histogram_symbolsize(int in_size)
           { return ChipsRemote::set_histogram_symbolsize(NULL, in_size); }
   int     set_histogram_symbolsize(ChipsId* cid, int in_size)
           { return ChipsRemote::set_histogram_symbolsize(cid, in_size); }
   int     get_histogram_symbolsize(int& out_val)
           { return ChipsRemote::get_histogram_symbolsize(NULL, out_val); }
   int     get_histogram_symbolsize(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_histogram_symbolsize(cid, out_val); }
   int     set_histogram_symbolangle(float in_val)
           { return ChipsRemote::set_histogram_symbolangle(NULL, in_val); }
   int     set_histogram_symbolangle(ChipsId* cid, float in_val)
           { return ChipsRemote::set_histogram_symbolangle(cid, in_val); }
   int     get_histogram_symbolangle(float& out_val)
           { return ChipsRemote::get_histogram_symbolangle(NULL, out_val); }
   int     get_histogram_symbolangle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_histogram_symbolangle(cid, out_val); }
   int     set_histogram_symbolcolor(string in_color)
           { return ChipsRemote::set_histogram_symbolcolor(NULL, in_color); }
   int     set_histogram_symbolcolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_histogram_symbolcolor(cid, in_color); }
   int     get_histogram_symbolcolor(string& out_color)
           { return ChipsRemote::get_histogram_symbolcolor(NULL, out_color); }
   int     get_histogram_symbolcolor(ChipsId* cid, string& out_color)
           { return ChipsRemote::get_histogram_symbolcolor(cid, out_color); }

    int     set_histogram_linestyle(V_LINE_TYPE val)
           { return ChipsRemote::set_histogram_linestyle(NULL, val); }
   int     set_histogram_linestyle(ChipsId* cid, V_LINE_TYPE val)
           { return ChipsRemote::set_histogram_linestyle(cid, val); }
   int     get_histogram_linestyle(V_LINE_TYPE& out_val)
           { return ChipsRemote::get_histogram_linestyle(NULL, out_val); }
   int     get_histogram_linestyle(ChipsId* cid, V_LINE_TYPE& out_val)
           { return ChipsRemote::get_histogram_linestyle(cid, out_val); }
   int     set_histogram_linethickness(float in_size)
           { return ChipsRemote::set_histogram_linethickness(NULL, in_size); }
   int     set_histogram_linethickness(ChipsId* cid, float in_size)
           { return ChipsRemote::set_histogram_linethickness(cid, in_size); }
   int     get_histogram_linethickness(float& out_val)
           { return ChipsRemote::get_histogram_linethickness(NULL, out_val); }
   int     get_histogram_linethickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_histogram_linethickness(cid, out_val); }
   int     set_histogram_linecolor(string in_color)
           { return ChipsRemote::set_histogram_linecolor(NULL, in_color); }
   int     set_histogram_linecolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_histogram_linecolor(cid, in_color); }
   int     get_histogram_linecolor(string& out_val)
           { return ChipsRemote::get_histogram_linecolor(NULL, out_val); }
   int     get_histogram_linecolor(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_histogram_linecolor(cid, out_val); }
   int     set_histogram_errstyle(string style)
           { return ChipsRemote::set_histogram_errstyle(NULL, style); }
   int     set_histogram_errstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_histogram_errstyle(cid, style); }
   int     get_histogram_errstyle(string& out_val)
           { return ChipsRemote::get_histogram_errstyle(NULL, out_val); }
   int     get_histogram_errstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_histogram_errstyle(cid, out_val); }
   int     set_histogram_errup(bool val)
           { return ChipsRemote::set_histogram_errup(NULL, val); }
   int     set_histogram_errup(ChipsId* cid, bool val)
           { return ChipsRemote::set_histogram_errup(cid, val); }
   int     get_histogram_errup(bool& out_val)
           { return ChipsRemote::get_histogram_errup(NULL, out_val); }
   int     get_histogram_errup(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_histogram_errup(cid, out_val); }
   int     set_histogram_errdown(bool val)
           { return ChipsRemote::set_histogram_errdown(NULL, val); }
   int     set_histogram_errdown(ChipsId* cid, bool val)
           { return ChipsRemote::set_histogram_errdown(cid, val); }
   int     get_histogram_errdown(bool& out_val)
           { return ChipsRemote::get_histogram_errdown(NULL, out_val); }
   int     get_histogram_errdown(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_histogram_errdown(cid, out_val); }
   int     set_histogram_errcaplength(int in_size)
           { return ChipsRemote::set_histogram_errcaplength(NULL, in_size); }
   int     set_histogram_errcaplength(ChipsId* cid, int in_size)
           { return ChipsRemote::set_histogram_errcaplength(cid, in_size); }
   int     get_histogram_errcaplength(int& out_val)
           { return ChipsRemote::get_histogram_errcaplength(NULL, out_val); }
   int     get_histogram_errcaplength(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_histogram_errcaplength(cid, out_val); }
   int     set_histogram_dropline(bool val)
           { return ChipsRemote::set_histogram_dropline(NULL, val); }
   int     set_histogram_dropline(ChipsId* cid, bool val)
           { return ChipsRemote::set_histogram_dropline(cid, val); }
   int     get_histogram_dropline(bool& out_val)
           { return ChipsRemote::get_histogram_dropline(NULL, out_val); }
   int     get_histogram_dropline(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_histogram_dropline(cid, out_val); }

   int     set_histogram_errthickness(float in_thickness)
           { return ChipsRemote::set_histogram_errthickness(NULL,in_thickness);}
   int     set_histogram_errthickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_histogram_errthickness(cid, in_thickness);}
   int     get_histogram_errthickness(float& out_val)
           { return ChipsRemote::get_histogram_errthickness(NULL, out_val); }
   int     get_histogram_errthickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_histogram_errthickness(cid, out_val); }
   int     set_histogram_errcolor(string in_color)
           { return ChipsRemote::set_histogram_errcolor(NULL, in_color); }
   int     set_histogram_errcolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_histogram_errcolor(cid, in_color); }
   int     get_histogram_errcolor(string& out_val)
           { return ChipsRemote::get_histogram_errcolor(NULL, out_val); }
   int     get_histogram_errcolor(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_histogram_errcolor(cid, out_val); }
   int     set_histogram_fillopacity(double in_opacity)
           { return ChipsRemote::set_histogram_fillopacity(NULL,in_opacity);}
   int     set_histogram_fillopacity(ChipsId* cid, double in_opacity)
           { return ChipsRemote::set_histogram_fillopacity(cid, in_opacity);}
   int     get_histogram_fillopacity(double& out_val)
           { return ChipsRemote::get_histogram_fillopacity(NULL, out_val); }
   int     get_histogram_fillopacity(ChipsId* cid, double& out_val)
           { return ChipsRemote::get_histogram_fillopacity(cid, out_val); }
   int     set_histogram_fillcolor(string in_color)
           { return ChipsRemote::set_histogram_fillcolor(NULL, in_color); }
   int     set_histogram_fillcolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_histogram_fillcolor(cid, in_color); }
   int     get_histogram_fillcolor(string& out_val)
           { return ChipsRemote::get_histogram_fillcolor(NULL, out_val); }
   int     get_histogram_fillcolor(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_histogram_fillcolor(cid, out_val); }
   int     set_histogram_fillstyle(V_FILL_TYPE in_style)
           { return ChipsRemote::set_histogram_fillstyle(NULL, in_style); }
   int     set_histogram_fillstyle(ChipsId* cid, V_FILL_TYPE in_style)
           { return ChipsRemote::set_histogram_fillstyle(cid, in_style); }
   int     get_histogram_fillstyle(V_FILL_TYPE& out_val)
           { return ChipsRemote::get_histogram_fillstyle(NULL, out_val); }
   int     get_histogram_fillstyle(ChipsId* cid, V_FILL_TYPE& out_val)
           { return ChipsRemote::get_histogram_fillstyle(cid, out_val); }
   int     set_current_histogram(ChipsId* cid)
           { return ChipsRemote::set_current_histogram(cid); }
   int     get_histogram_range(double& outx1, double& outx2,
                               double& outy1, double& outy2)
           { return ChipsRemote::get_histogram_range(NULL,outx1,outx2,outy1,outy2); }
   int     get_histogram_range(ChipsId* cid, double& outx1, double& outx2,
                               double& outy1, double& outy2)
           { return ChipsRemote::get_histogram_range(cid,outx1,outx2,outy1,outy2); }
   int     get_histogram_xrange(double& out1, double& out2)
           { return ChipsRemote::get_histogram_xrange(NULL, out1, out2); }
   int     get_histogram_xrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_histogram_xrange(cid, out1, out2); }
   int     get_histogram_yrange(double& out1, double& out2)
           { return ChipsRemote::get_histogram_yrange(NULL, out1, out2); }
   int     get_histogram_yrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_histogram_yrange(cid, out1, out2); }
   int     set_histogram_depth(int sz)
           { return ChipsRemote::set_histogram_depth(NULL, sz); }
   int     set_histogram_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_histogram_depth(cid, sz); }
   int     get_histogram_depth(int& out_val)
           { return ChipsRemote::get_histogram_depth(NULL, out_val); }
   int     get_histogram_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_histogram_depth(cid, out_val); }

   // axis  operations 
   int     add_axis(int dim, float pos, float minx, float maxx, float miny, float maxy, Transform *transform, string aid="") 
           { return ChipsRemote::add_axis(NULL, dim, pos, minx, maxx, miny, maxy, transform, aid); }
   int     add_axis(ChipsId* cid, int dim, float pos, float minx,
                    float maxx, float miny, float maxy, Transform *transform, string aid="")
           { return ChipsRemote::add_axis(cid, dim, pos, minx, maxx, miny, maxy, transform, aid); }
   int     delete_axis()
           { return ChipsRemote::delete_axis(NULL); }
   int     delete_axis(ChipsId* cid)
           { return ChipsRemote::delete_axis(cid); }
   int     bind_axes(string lid1, string aid1, string lid2, string aid2);
   int     bind_axes(string aid1, string aid2);
   int     bind_axes(ChipsId* cid, string lid, string aid)
           { return ChipsRemote::bind_axes(cid, lid, aid); }
   int     unbind_axes(string lid1, string aid1, string lid2, string aid2);
   int     unbind_axes(string aid1, string aid2);
   int     unbind_axes(ChipsId* cid, string lid, string aid)
           { return ChipsRemote::unbind_axes(cid, lid, aid); }
   int     set_axis_color(string in_color)
           { return ChipsRemote::set_axis_color(NULL, in_color); }
   int     set_axis_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_axis_color(cid, in_color); }
   int     get_axis_color(string& out_val)
           { return ChipsRemote::get_axis_color(NULL, out_val); }
   int     get_axis_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_color(cid, out_val); }
   int     set_axis_major_color(string in_color)
           { return ChipsRemote::set_axis_major_color(NULL, in_color); }
   int     set_axis_major_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_axis_major_color(cid, in_color); }
   int     get_axis_major_color(string& out_val)
           { return ChipsRemote::get_axis_major_color(NULL, out_val); }
   int     get_axis_major_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_major_color(cid, out_val); }
   int     set_axis_minor_color(string in_color)
           { return ChipsRemote::set_axis_minor_color(NULL, in_color); }
   int     set_axis_minor_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_axis_minor_color(cid, in_color); }
   int     get_axis_minor_color(string& out_val)
           { return ChipsRemote::get_axis_minor_color(NULL, out_val); }
   int     get_axis_minor_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_minor_color(cid, out_val); }
   int     set_axis_text(string in_text)
           { return ChipsRemote::set_axis_text(NULL, in_text); }
   int     set_axis_text(ChipsId* cid, string in_text)
           { return ChipsRemote::set_axis_text(cid, in_text); }
   int     get_axis_text(string& out_val)
           { return ChipsRemote::get_axis_text(NULL, out_val); }
   int     get_axis_text(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_text(cid, out_val); }
   int     set_axis_xlabel_text(string in_text)
           { return ChipsRemote::set_axis_xlabel_text(NULL, in_text); }
   int     set_axis_xlabel_text(ChipsId* cid, string in_text)
           { return ChipsRemote::set_axis_xlabel_text(cid, in_text); }
   int     get_axis_xlabel_text(string& out_val)
           { return ChipsRemote::get_axis_xlabel_text(NULL, out_val); }
   int     get_axis_xlabel_text(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_xlabel_text(cid, out_val); }
   int     set_axis_ylabel_text(string in_text)
           { return ChipsRemote::set_axis_ylabel_text(NULL, in_text); }
   int     set_axis_ylabel_text(ChipsId* cid, string in_text)
           { return ChipsRemote::set_axis_ylabel_text(cid, in_text); }
   int     get_axis_ylabel_text(string& out_val)
           { return ChipsRemote::get_axis_ylabel_text(NULL, out_val); }
   int     get_axis_ylabel_text(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_ylabel_text(cid, out_val); }
   int     set_axis_tickformat(string in_format)
           { return ChipsRemote::set_axis_tickformat(NULL, in_format); }
   int     set_axis_tickformat(ChipsId* cid, string in_format)
           { return ChipsRemote::set_axis_tickformat(cid, in_format); }
   int     get_axis_tickformat(string& out_val)
           { return ChipsRemote::get_axis_tickformat(NULL, out_val); }
   int     get_axis_tickformat(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_tickformat(cid, out_val); }
   int     set_axis_thickness(float in_thickness)
           { return ChipsRemote::set_axis_thickness(NULL, in_thickness); }
   int     set_axis_thickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_axis_thickness(cid, in_thickness); }
   int     get_axis_thickness(float& out_val)
           { return ChipsRemote::get_axis_thickness(NULL, out_val); }
   int     get_axis_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_axis_thickness(cid, out_val); }
   int     set_axis_tickstyle(int in_style)
           { return ChipsRemote::set_axis_tickstyle(NULL, in_style); }
   int     set_axis_tickstyle(ChipsId* cid, int in_style)
           { return ChipsRemote::set_axis_tickstyle(cid, in_style); }
   int     set_axis_tickstyle(string in_style)
           { return ChipsRemote::set_axis_tickstyle(NULL, in_style); }
   int     set_axis_tickstyle(ChipsId* cid, string in_style)
           { return ChipsRemote::set_axis_tickstyle(cid, in_style); }
   int     get_axis_tickstyle(string& out_val)
           { return ChipsRemote::get_axis_tickstyle(NULL, out_val); }
   int     get_axis_tickstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_axis_tickstyle(cid, out_val); }
   int     set_axis_major_num(int in_num)
           { return ChipsRemote::set_axis_major_num(NULL, in_num); }
   int     set_axis_major_num(ChipsId* cid, int in_num)
           { return ChipsRemote::set_axis_major_num(cid, in_num); }
   int     get_axis_major_num(int& out_val)
           { return ChipsRemote::get_axis_major_num(NULL, out_val); }
   int     get_axis_major_num(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_major_num(cid, out_val); }
   int     set_axis_minor_num(int in_num)
           { return ChipsRemote::set_axis_minor_num(NULL, in_num); }
   int     set_axis_minor_num(ChipsId* cid, int in_num)
           { return ChipsRemote::set_axis_minor_num(cid, in_num); }
   int     get_axis_minor_num(int& out_val)
           { return ChipsRemote::get_axis_minor_num(NULL, out_val); }
   int     get_axis_minor_num(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_minor_num(cid, out_val); }
   int     set_axis_major_length(int in_len)
           { return ChipsRemote::set_axis_major_length(NULL, in_len); }
   int     set_axis_major_length(ChipsId* cid, int in_len)
           { return ChipsRemote::set_axis_major_length(cid, in_len); }
   int     get_axis_major_length(int& out_val)
           { return ChipsRemote::get_axis_major_length(NULL, out_val); }
   int     get_axis_major_length(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_major_length(cid, out_val); }
   int     set_axis_minor_length(int in_len)
           { return ChipsRemote::set_axis_minor_length(NULL, in_len); }
   int     set_axis_minor_length(ChipsId* cid, int in_len)
           { return ChipsRemote::set_axis_minor_length(cid, in_len); }
   int     get_axis_minor_length(int& out_val)
           { return ChipsRemote::get_axis_minor_length(NULL, out_val); }
   int     get_axis_minor_length(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_axis_minor_length(cid, out_val); }
   int     set_axis_major_interval(double in_len)
           { return ChipsRemote::set_axis_major_interval(NULL, in_len); }
   int     set_axis_major_interval(ChipsId* cid, double in_len)
           { return ChipsRemote::set_axis_major_interval(cid, in_len); }
   int     get_axis_major_interval(double& out_val)
           { return ChipsRemote::get_axis_major_interval(NULL, out_val); }
   int     get_axis_major_interval(ChipsId* cid, double& out_val)
           { return ChipsRemote::get_axis_major_interval(cid, out_val); }

   int     set_axis_minor_interval(double in_len)
           { return ChipsRemote::set_axis_minor_interval(NULL, in_len); }
   int     set_axis_minor_interval(ChipsId* cid, double in_len)
           { return ChipsRemote::set_axis_minor_interval(cid, in_len); }
   int     get_axis_minor_interval(double& out_val)
           { return ChipsRemote::get_axis_minor_interval(NULL, out_val); }
   int     get_axis_minor_interval(ChipsId* cid, double& out_val)
           { return ChipsRemote::get_axis_minor_interval(cid, out_val); }
   int     set_current_axis(ChipsId* cid)
           { return ChipsRemote::set_current_axis(cid); }
   int     get_axis_range(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_axis_range(cid, out1, out2); }

   // image operations
/*   int     add_image(string filename, Transform *transform, string attlist="")
           { return ChipsRemote::add_image(NULL, filename, transform attlist); }
   int     add_image(ChipsId* cid, string filename, Transform *transform, string attlist="")
           { return ChipsRemote::add_image(cid, filename, ransform attlist); }
   int     add_image(string filename, float ix1, float iy1,
                     float ix2, float iy2, Transform *transform, string attlist = "")
           { return ChipsRemote::add_image(NULL, filename, ix1, iy1, ix2, iy2,
                                           transform, attlist); }
   int     add_image(ChipsId* cid, string filename, float ix1, float iy1,
                     float ix2, float iy2, Transform *transform, string attlist = "")
           { return ChipsRemote::add_image(cid, filename, ix1, iy1, ix2, iy2,
                                           transform, attlist); }
*/
   int     add_image(double* rvals, double *gvals,
                     double *bvals, double *avals, int xaxis, int yaxis, 
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_image(NULL, rvals, gvals, bvals, avals,xaxis, yaxis, transform, attlist); }
   int     add_image(ChipsId* cid, double* rvals, double *gvals, 
                     double *bvals, double *avals, int xaxis, int yaxis, 
                     Transform *transform, string attlist = "")
           { return ChipsRemote::add_image(cid, rvals, gvals, bvals, avals, xaxis, yaxis, transform, attlist); }
   int     delete_image()
           { return ChipsRemote::delete_image(NULL); }
   int     delete_image(ChipsId* cid)
           { return ChipsRemote::delete_image(cid); }
   int     set_image_channel(double* vals, int xaxis, int yaxis, int channel)
           { return ChipsRemote::set_image_channel(NULL, vals, xaxis, yaxis, channel); }
   int     set_image_channel(ChipsId* cid, double* vals, int xaxis, int yaxis, int channel)
           { return ChipsRemote::set_image_channel(cid, vals, xaxis, yaxis, channel); }
   int     remove_image_channel(int channel)
           { return ChipsRemote::remove_image_channel(NULL, channel); }
   int     remove_image_channel(ChipsId* cid, int channel)
           { return ChipsRemote::remove_image_channel(cid, channel); }
   int     display_image()
           { return ChipsRemote::display_image(NULL); }
   int     display_image(ChipsId* cid)
           { return ChipsRemote::display_image(cid); }
   int     hide_image()
           { return ChipsRemote::hide_image(NULL); }
   int     hide_image(ChipsId* cid)
           { return ChipsRemote::hide_image(cid); }
   int     get_image_visible(bool& out_val)
           { return ChipsRemote::get_image_visible(NULL, out_val); }
   int     get_image_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_image_visible(cid, out_val); }
   int     set_current_image(ChipsId* cid)
           { return ChipsRemote::set_current_image(cid); }
   int     set_image_invert_colormap(bool val)
           { return ChipsRemote::set_image_invert_colormap(NULL, val); }
   int     set_image_invert_colormap(ChipsId* cid, bool val)
           { return ChipsRemote::set_image_invert_colormap(cid, val); }
   int     set_image_colormap_interpolate(bool val)
           { return ChipsRemote::set_image_colormap_interpolate(NULL, val); }
   int     set_image_colormap_interpolate(ChipsId* cid, bool val)
           { return ChipsRemote::set_image_colormap_interpolate(cid, val); }
   int     set_image_scale_channels(bool val)
           { return ChipsRemote::set_image_scale_channels(NULL, val); }
   int     set_image_scale_channels(ChipsId* cid, bool val)
           { return ChipsRemote::set_image_scale_channels(cid, val); }
   int     set_image_colormap_size(int val)
           { return ChipsRemote::set_image_colormap_size(NULL, val); }
   int     set_image_colormap_size(ChipsId* cid, int val)
           { return ChipsRemote::set_image_colormap_size(cid, val); }

   int     get_image_invert_colormap(bool& out_val)
           { return ChipsRemote::get_image_invert_colormap(NULL, out_val); }
   int     get_image_invert_colormap(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_image_invert_colormap(cid, out_val); }
   int     get_image_colormap_interpolate(bool& out_val)
           { return ChipsRemote::get_image_colormap_interpolate(NULL, out_val); }
   int     get_image_colormap_interpolate(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_image_colormap_interpolate(cid, out_val); }
   int     get_image_scale_channels(bool& out_val)
           { return ChipsRemote::get_image_scale_channels(NULL, out_val); }
   int     get_image_scale_channels(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_image_scale_channels(cid, out_val); }
   int     get_image_colormap_size(int& out_val)
           { return ChipsRemote::get_image_colormap_size(NULL, out_val); }
   int     get_image_colormap_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_image_colormap_size(cid, out_val); }
   int     set_image_colormap(V_COLORMAP_TYPE val)
           { return ChipsRemote::set_image_colormap(NULL, val); }
   int     set_image_colormap(ChipsId* cid, V_COLORMAP_TYPE val)
           { return ChipsRemote::set_image_colormap(cid, val); }
   int     get_image_colormap(V_COLORMAP_TYPE& out_val)
           { return ChipsRemote::get_image_colormap(NULL, out_val); }
   int     get_image_colormap(ChipsId* cid, V_COLORMAP_TYPE& out_val)
           { return ChipsRemote::get_image_colormap(cid, out_val); }
   int     set_image_interpolation(V_INTERPOLATION_TYPE val)
           { return ChipsRemote::set_image_interpolation(NULL, val); }
   int     set_image_interpolation(ChipsId* cid, V_INTERPOLATION_TYPE val)
           { return ChipsRemote::set_image_interpolation(cid, val); }
   int     get_image_interpolation(V_INTERPOLATION_TYPE& out_val)
           { return ChipsRemote::get_image_interpolation(NULL, out_val); }
   int     get_image_interpolation(ChipsId* cid, V_INTERPOLATION_TYPE& out_val)
           { return ChipsRemote::get_image_interpolation(cid, out_val); }
   int     set_image_alpha(double min, double max)
           { return ChipsRemote::set_image_alpha(NULL, min, max); }
   int     set_image_alpha(ChipsId* cid, double min, double max)
           { return ChipsRemote::set_image_alpha(cid, min, max); }
   int     get_image_alpha(double& out1, double& out2)
           { return ChipsRemote::get_image_alpha(NULL, out1, out2); }
   int     get_image_alpha(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_image_alpha(cid, out1, out2); }
   int     set_image_threshold(double min, double max)
           { return ChipsRemote::set_image_threshold(NULL, min, max); }
   int     set_image_threshold(ChipsId* cid, double min, double max)
           { return ChipsRemote::set_image_threshold(cid, min, max); }
   int     get_image_threshold(double& out1, double& out2)
           { return ChipsRemote::get_image_threshold(NULL, out1, out2); }
   int     get_image_threshold(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_image_threshold(cid, out1, out2); }
   int     set_image_depth(int sz)
           { return ChipsRemote::set_image_depth(NULL, sz); }
   int     set_image_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_image_depth(cid, sz); }
   int     get_image_depth(int& out_val)
           { return ChipsRemote::get_image_depth(NULL, out_val); }
   int     get_image_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_image_depth(cid, out_val); }
   int     get_image_range(double& outx1, double& outx2,
                          double& outy1, double& outy2)
           { return ChipsRemote::get_image_range(NULL,outx1,outx2,outy1,outy2);}
   int     get_image_range(ChipsId* cid, double& outx1, double& outx2,
                          double& outy1, double& outy2)
           { return ChipsRemote::get_image_range(cid,outx1,outx2,outy1,outy2); }
   int     get_image_xrange(double& out1, double& out2)
           { return ChipsRemote::get_image_xrange(NULL, out1, out2); }
   int     get_image_xrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_image_xrange(cid, out1, out2); }
   int     get_image_yrange(double& out1, double& out2)
           { return ChipsRemote::get_image_yrange(NULL, out1, out2); }
   int     get_image_yrange(ChipsId* cid, double& out1, double& out2)
           { return ChipsRemote::get_image_yrange(cid, out1, out2); }

   // colorbar operations 
   int     add_colorbar(double xpos,double ypos, string attlist="")
           { return ChipsRemote::add_colorbar(NULL,xpos,ypos,"",chips_not_object,attlist);}
   int     add_colorbar(ChipsId* cid,double xpos,double ypos, string attlist="")
           { return ChipsRemote::add_colorbar(cid,xpos,ypos,"",chips_not_object,attlist);}
   int     add_colorbar(double xpos,double ypos,string obj_id, V_OBJECT_TYPE obj, string attlist="")
           { return ChipsRemote::add_colorbar(NULL,xpos,ypos,obj_id,obj,attlist);}
   int     add_colorbar(ChipsId* cid,double xpos,double ypos,string obj_id, V_OBJECT_TYPE obj,string attlist="")
           { return ChipsRemote::add_colorbar(cid,xpos,ypos,obj_id,obj,attlist);}
   int     delete_colorbar()
           { return ChipsRemote::delete_colorbar(NULL); }
   int     delete_colorbar(ChipsId* cid)
           { return ChipsRemote::delete_colorbar(cid); }
   int     set_current_colorbar(ChipsId* cid)
           { return ChipsRemote::set_current_colorbar(cid); }
   int     set_colorbar_depth(int sz)
           { return ChipsRemote::set_colorbar_depth(NULL, sz); }
   int     set_colorbar_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_colorbar_depth(cid, sz); }
   int     get_colorbar_depth(int& out_val)
           { return ChipsRemote::get_colorbar_depth(NULL, out_val); }
   int     get_colorbar_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_colorbar_depth(cid, out_val); }
   int     display_colorbar()
           { return ChipsRemote::display_colorbar(NULL); }
   int     display_colorbar(ChipsId* cid)
           { return ChipsRemote::display_colorbar(cid); }
   int     hide_colorbar()
           { return ChipsRemote::hide_colorbar(NULL); }
   int     hide_colorbar(ChipsId* cid)
           { return ChipsRemote::hide_colorbar(cid); }
   int     get_colorbar_visible(bool& out_val)
           { return ChipsRemote::get_colorbar_visible(NULL, out_val); }
   int     get_colorbar_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_colorbar_visible(cid, out_val); }

   int     set_colorbar_width(ChipsId* cid, float in_val)
           { return ChipsRemote::set_colorbar_width(cid, in_val); }
   int     set_colorbar_width(float in_val)
           { return ChipsRemote::set_colorbar_width(NULL, in_val); }
   int     get_colorbar_width(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_width (cid, out_val); }
   int     get_colorbar_width(float& out_val)
           { return ChipsRemote::get_colorbar_width (NULL, out_val); }
   int     set_colorbar_length(ChipsId* cid, float in_val)
           { return ChipsRemote::set_colorbar_length (cid, in_val); }
   int     set_colorbar_length(float in_val)
           { return ChipsRemote::set_colorbar_length (NULL, in_val); }
   int     get_colorbar_length(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_length(cid, out_val); }
   int     get_colorbar_length(float& out_val)
           { return ChipsRemote::get_colorbar_length(NULL, out_val); }
   int     hide_colorbar_border(ChipsId* cid)
           { return ChipsRemote::hide_colorbar_border(cid); }
   int     hide_colorbar_border()
           { return ChipsRemote::hide_colorbar_border(NULL); }
   int     display_colorbar_border(ChipsId* cid)
           { return ChipsRemote::display_colorbar_border(cid); }
   int     display_colorbar_border()
           { return ChipsRemote::display_colorbar_border(NULL); }
   int     get_colorbar_border_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_colorbar_border_visible(cid, out_val); }
   int     get_colorbar_border_visible(bool& out_val)
           { return ChipsRemote::get_colorbar_border_visible(NULL, out_val); } 
   int     set_colorbar_halign(ChipsId* cid, float sz)
           { return ChipsRemote::set_colorbar_halign(cid, sz); }
   int     set_colorbar_halign(float sz)
           { return ChipsRemote::set_colorbar_halign(NULL, sz); }
   int     get_colorbar_halign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_halign (cid, out_val); } 
   int     get_colorbar_halign(float& out_val)
           { return ChipsRemote::get_colorbar_halign (NULL, out_val); }
   int     set_colorbar_valign(ChipsId* cid, float sz)
           { return ChipsRemote::set_colorbar_valign (cid, sz); }
   int     set_colorbar_valign(float sz)
           { return ChipsRemote::set_colorbar_valign (NULL, sz); }
   int     get_colorbar_valign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_valign (cid, out_val); }
   int     get_colorbar_valign(float& out_val)
           { return ChipsRemote::get_colorbar_valign (NULL, out_val); }
   int     set_colorbar_label_angle(ChipsId* cid, float sz)
           { return ChipsRemote::set_colorbar_label_angle (cid, sz); }
   int     set_colorbar_label_angle(float sz)
           { return ChipsRemote::set_colorbar_label_angle (NULL, sz); }
   int     get_colorbar_label_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_label_angle (cid, out_val); }
   int     get_colorbar_label_angle(float& out_val)
           { return ChipsRemote::get_colorbar_label_angle (NULL, out_val); }
   int     set_colorbar_label_halign(ChipsId* cid, float sz)
           { return ChipsRemote::set_colorbar_label_halign (cid, sz); }
   int     set_colorbar_label_halign(float sz)
           { return ChipsRemote::set_colorbar_label_halign (NULL, sz); }
   int     get_colorbar_label_halign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_label_halign (cid, out_val); }
   int     get_colorbar_label_halign(float& out_val) 
           { return ChipsRemote::get_colorbar_label_halign (NULL, out_val); }
   int     set_colorbar_label_valign(ChipsId* cid, float sz)
           { return ChipsRemote::set_colorbar_label_valign (cid, sz); }
   int     set_colorbar_label_valign(float sz)
           { return ChipsRemote::set_colorbar_label_valign (NULL, sz); }
   int     get_colorbar_label_valign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_label_valign (cid, out_val); }
   int     get_colorbar_label_valign(float& out_val)
           { return ChipsRemote::get_colorbar_label_valign (NULL, out_val); }
   int     set_colorbar_orientation(ChipsId* cid, int inval)
           { return ChipsRemote::set_colorbar_orientation (cid, inval); }
   int     set_colorbar_orientation(int inval)
           { return ChipsRemote::set_colorbar_orientation (NULL, inval); }
   int     get_colorbar_orientation(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_colorbar_orientation (cid, out_val); }
   int     get_colorbar_orientation(int& out_val)
           { return ChipsRemote::get_colorbar_orientation (NULL, out_val); }
   int     set_colorbar_label_text(ChipsId* cid, string in_color)
           { return ChipsRemote::set_colorbar_label_text (cid, in_color); }
   int     set_colorbar_label_text(string in_color)
           { return ChipsRemote::set_colorbar_label_text (NULL, in_color); }
   int     get_colorbar_label_text(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_label_text (cid, out_val); }
   int     get_colorbar_label_text(string& out_val)
           { return ChipsRemote::get_colorbar_label_text (NULL, out_val); }
   int     set_colorbar_label_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_colorbar_label_color (cid, in_color); }
   int     set_colorbar_label_color(string in_color)
           { return ChipsRemote::set_colorbar_label_color (NULL, in_color); }
   int     get_colorbar_label_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_label_color (cid, out_val); }
   int     get_colorbar_label_color(string& out_val)
           { return ChipsRemote::get_colorbar_label_color (NULL, out_val); }
   int     set_colorbar_label_size(ChipsId* cid, int sz)
           { return ChipsRemote::set_colorbar_label_size (cid, sz); }
   int     set_colorbar_label_size(int sz)
           { return ChipsRemote::set_colorbar_label_size (NULL, sz); }
   int     get_colorbar_label_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_colorbar_label_size (cid, out_val); }
   int     get_colorbar_label_size(int& out_val)
           { return ChipsRemote::get_colorbar_label_size (NULL, out_val); }
   int     set_colorbar_label_visible(ChipsId* cid, bool vis)
           { return ChipsRemote::set_colorbar_label_visible (cid, vis); }
   int     set_colorbar_label_visible(bool vis)
           { return ChipsRemote::set_colorbar_label_visible (NULL, vis); }
   int     get_colorbar_label_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_colorbar_label_visible (cid, out_val); }
   int     get_colorbar_label_visible(bool& out_val)
           { return ChipsRemote::get_colorbar_label_visible (NULL, out_val); }
   int     set_colorbar_label_font(ChipsId* cid, string font)
           { return ChipsRemote::set_colorbar_label_font (cid, font); }
   int     set_colorbar_label_font(string font)
           { return ChipsRemote::set_colorbar_label_font (NULL, font); }
   int     get_colorbar_label_font(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_label_font (cid, out_val); }
   int     get_colorbar_label_font(string& out_val)
           { return ChipsRemote::get_colorbar_label_font (NULL, out_val); }
   int     set_colorbar_label_fontstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_colorbar_label_fontstyle (cid, style); }
   int     set_colorbar_label_fontstyle(string style)
           { return ChipsRemote::set_colorbar_label_fontstyle (NULL, style); }
   int     get_colorbar_label_fontstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_label_fontstyle (cid, out_val); }
   int     get_colorbar_label_fontstyle(string& out_val)
           { return ChipsRemote::get_colorbar_label_fontstyle (NULL, out_val); }
   int     set_colorbar_ticklabel_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_colorbar_ticklabel_color (cid, in_color); }
   int     set_colorbar_ticklabel_color(string in_color)
           { return ChipsRemote::set_colorbar_ticklabel_color (NULL, in_color); }
   int     get_colorbar_ticklabel_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_color (cid, out_val); }
   int     get_colorbar_ticklabel_color(string& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_color (NULL, out_val); }
   int     set_colorbar_ticklabel_size(ChipsId* cid, int in_val)
           { return ChipsRemote::set_colorbar_ticklabel_size (cid, in_val); }
   int     set_colorbar_ticklabel_size(int in_val)
           { return ChipsRemote::set_colorbar_ticklabel_size (NULL, in_val); }
   int     get_colorbar_ticklabel_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_size (cid, out_val); }
   int     get_colorbar_ticklabel_size(int& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_size (NULL, out_val); }
   int     set_colorbar_ticklabel_font(ChipsId* cid, string in_font)
           { return ChipsRemote::set_colorbar_ticklabel_font (cid, in_font); }
   int     set_colorbar_ticklabel_font(string in_font)
           { return ChipsRemote::set_colorbar_ticklabel_font (NULL, in_font); }
   int     get_colorbar_ticklabel_font(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_font (cid, out_val); }
   int     get_colorbar_ticklabel_font(string& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_font (NULL, out_val); }
   int     set_colorbar_ticklabel_fontstyle(ChipsId* cid, string style)
           { return ChipsRemote::set_colorbar_ticklabel_fontstyle (cid, style); }
   int     set_colorbar_ticklabel_fontstyle(string style)
           { return ChipsRemote::set_colorbar_ticklabel_fontstyle (NULL, style); }
   int     get_colorbar_ticklabel_fontstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_fontstyle (cid, out_val); }
   int     get_colorbar_ticklabel_fontstyle(string& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_fontstyle (NULL, out_val); }
   int     set_colorbar_ticklabel_visible(ChipsId* cid, bool vis)
           { return ChipsRemote::set_colorbar_ticklabel_visible (cid, vis); }
   int     set_colorbar_ticklabel_visible(bool vis)
           { return ChipsRemote::set_colorbar_ticklabel_visible (NULL, vis); }
   int     get_colorbar_ticklabel_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_visible (cid, out_val); }
   int     get_colorbar_ticklabel_visible(bool& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_visible (NULL, out_val); }
   int     set_colorbar_ticklabel_angle(ChipsId* cid, float in_val)
           { return ChipsRemote::set_colorbar_ticklabel_angle (cid, in_val); }
   int     set_colorbar_ticklabel_angle(float in_val)
           { return ChipsRemote::set_colorbar_ticklabel_angle (NULL, in_val); }
   int     get_colorbar_ticklabel_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_angle (cid, out_val); }
   int     get_colorbar_ticklabel_angle(float& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_angle (NULL, out_val); }
   int     set_colorbar_ticklabel_halign(ChipsId* cid, float in_val)
           { return ChipsRemote::set_colorbar_ticklabel_halign (cid, in_val); }
   int     set_colorbar_ticklabel_halign(float in_val)
           { return ChipsRemote::set_colorbar_ticklabel_halign (NULL, in_val); }
   int     get_colorbar_ticklabel_halign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_halign (cid, out_val); }
   int     get_colorbar_ticklabel_halign(float& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_halign (NULL, out_val); }
   int     set_colorbar_ticklabel_valign(ChipsId* cid, float in_val)
           { return ChipsRemote::set_colorbar_ticklabel_valign (cid, in_val); }
   int     set_colorbar_ticklabel_valign(float in_val)
           { return ChipsRemote::set_colorbar_ticklabel_valign (NULL, in_val); }
   int     get_colorbar_ticklabel_valign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_valign (cid, out_val); }
   int     get_colorbar_ticklabel_valign(float& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_valign (NULL, out_val); }
   int     set_colorbar_ticklabel_offset(ChipsId* cid, int in_val)
           { return ChipsRemote::set_colorbar_ticklabel_offset (cid, in_val); }
   int     set_colorbar_ticklabel_offset(int in_val)
           { return ChipsRemote::set_colorbar_ticklabel_offset (NULL, in_val); }
   int     get_colorbar_ticklabel_offset(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_offset (cid, out_val); }
   int     get_colorbar_ticklabel_offset(int& out_val)
           { return ChipsRemote::get_colorbar_ticklabel_offset (NULL, out_val); }
   int     set_colorbar_label_location(ChipsId* cid, string style)
           { return ChipsRemote::set_colorbar_label_location (cid, style); }
   int     set_colorbar_label_location(string style)
           { return ChipsRemote::set_colorbar_label_location (NULL, style); }
   int     set_colorbar_label_location(ChipsId* cid, int in_style)
           { return ChipsRemote::set_colorbar_label_location (cid, in_style); }
   int     set_colorbar_label_location(int in_style)
           { return ChipsRemote::set_colorbar_label_location (NULL, in_style); }
   int     get_colorbar_label_location(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_label_location (cid, out_val); }
   int     get_colorbar_label_location(string& out_val)
           { return ChipsRemote::get_colorbar_label_location (NULL, out_val); }
   int     set_colorbar_tick_style(ChipsId* cid, string in_style)
           { return ChipsRemote::set_colorbar_tick_style (cid, in_style); }
   int     set_colorbar_tick_style(string in_style)
           { return ChipsRemote::set_colorbar_tick_style (NULL, in_style); }
   int     set_colorbar_tick_style(ChipsId* cid, int in_style)
           { return ChipsRemote::set_colorbar_tick_style (cid, in_style); }
   int     set_colorbar_tick_style(int in_style)
           { return ChipsRemote::set_colorbar_tick_style (NULL, in_style); }
   int     get_colorbar_tick_style(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_tick_style (cid, out_val); }
   int     get_colorbar_tick_style(string& out_val)
           { return ChipsRemote::get_colorbar_tick_style (NULL, out_val); }
   int     set_colorbar_tick_thickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_colorbar_tick_thickness (cid, in_thickness); }
   int     set_colorbar_tick_thickness(float in_thickness)
           { return ChipsRemote::set_colorbar_tick_thickness (NULL, in_thickness); }
   int     get_colorbar_tick_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_colorbar_tick_thickness (cid, out_val); }
   int     get_colorbar_tick_thickness(float& out_val)
           { return ChipsRemote::get_colorbar_tick_thickness (NULL, out_val); }
   int     set_colorbar_tick_mode(ChipsId* cid, string in_mode)
           { return ChipsRemote::set_colorbar_tick_mode (cid, in_mode); }
   int     set_colorbar_tick_mode(string in_mode)
           { return ChipsRemote::set_colorbar_tick_mode (NULL, in_mode); }
   int     get_colorbar_tick_mode(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_tick_mode (cid, out_val); }
   int     get_colorbar_tick_mode(string& out_val)
           { return ChipsRemote::get_colorbar_tick_mode (NULL, out_val); }
   int     set_colorbar_tick_location(ChipsId* cid, string in_style)
           { return ChipsRemote::set_colorbar_tick_location (cid, in_style); }
   int     set_colorbar_tick_location(string in_style)
           { return ChipsRemote::set_colorbar_tick_location (NULL, in_style); }
   int     set_colorbar_tick_location(ChipsId* cid, int in_style)
           { return ChipsRemote::set_colorbar_tick_location (cid, in_style); }
   int     set_colorbar_tick_location(int in_style)
           { return ChipsRemote::set_colorbar_tick_location (NULL, in_style); }
   int     get_colorbar_tick_location(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_colorbar_tick_location (cid, out_val); }
   int     get_colorbar_tick_location(string& out_val)
           { return ChipsRemote::get_colorbar_tick_location (NULL, out_val); }
   int     set_colorbar_tick_visible(ChipsId* cid, bool vis)
           { return ChipsRemote::set_colorbar_tick_visible (cid, vis); }
   int     set_colorbar_tick_visible(bool vis)
           { return ChipsRemote::set_colorbar_tick_visible (NULL, vis); }
   int     get_colorbar_tick_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_colorbar_tick_visible (cid, out_val); }
   int     get_colorbar_tick_visible(bool& out_val)
           { return ChipsRemote::get_colorbar_tick_visible (NULL, out_val); }
   int     set_colorbar_tick_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_colorbar_tick_color (cid, in_color); }
   int     set_colorbar_tick_color(string in_color)
           { return ChipsRemote::set_colorbar_tick_color (NULL, in_color); }
   int     get_colorbar_tick_color(ChipsId* cid, string& out_color)
           {return ChipsRemote::get_colorbar_tick_color (cid, out_color); }
   int     get_colorbar_tick_color(string& out_color)
           {return ChipsRemote::get_colorbar_tick_color (NULL, out_color); }
   int     set_colorbar_tick_length(ChipsId* cid, int in_length)
           {return ChipsRemote::set_colorbar_tick_length (cid, in_length); }
   int     set_colorbar_tick_length(int in_length)
           {return ChipsRemote::set_colorbar_tick_length (NULL, in_length); }
   int     get_colorbar_tick_length(ChipsId* cid, int& out_val)
           {return ChipsRemote::get_colorbar_tick_length (cid, out_val); }
   int     get_colorbar_tick_length(int& out_val)
           {return ChipsRemote::get_colorbar_tick_length (NULL, out_val); }
   int     set_colorbar_parallel_offset(ChipsId* cid, float in_val)
           {return ChipsRemote::set_colorbar_parallel_offset (cid, in_val); }
   int     set_colorbar_parallel_offset(float in_val)
           {return ChipsRemote::set_colorbar_parallel_offset (NULL, in_val); }
   int     get_colorbar_parallel_offset(ChipsId* cid, float& out_val)
           {return ChipsRemote::get_colorbar_parallel_offset (cid, out_val); }
   int     get_colorbar_parallel_offset(float& out_val)
           {return ChipsRemote::get_colorbar_parallel_offset (NULL, out_val); }
   int     set_colorbar_perpendicular_offset(ChipsId* cid, float in_val)
           {return ChipsRemote::set_colorbar_perpendicular_offset (cid, in_val); }
   int     set_colorbar_perpendicular_offset(float in_val)
           {return ChipsRemote::set_colorbar_perpendicular_offset (NULL, in_val); }
   int     get_colorbar_perpendicular_offset(ChipsId* cid, float& out_val)
           {return ChipsRemote::get_colorbar_perpendicular_offset (cid, out_val); }
   int     get_colorbar_perpendicular_offset(float& out_val)
           {return ChipsRemote::get_colorbar_perpendicular_offset (NULL, out_val); }

   // label operations
   int     add_label(double xpos, double ypos, string text, string attlist = "")
           { return ChipsRemote::add_label(NULL, xpos, ypos, text, attlist); }
   int     add_label(ChipsId* cid, double xpos, double ypos, 
                     string text, string attlist = "")
           { return ChipsRemote::add_label(cid, xpos, ypos, text, attlist); }
   int     delete_label()
           { return ChipsRemote::delete_label(NULL); }
   int     delete_label(ChipsId* cid)
           { return ChipsRemote::delete_label(cid); }
   int     set_label_color(string in_color)
           { return ChipsRemote::set_label_color(NULL, in_color); }
   int     set_label_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_label_color(cid, in_color); }
   int     get_label_color(string& out_val)
           { return ChipsRemote::get_label_color(NULL, out_val); }
   int     get_label_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_label_color(cid, out_val); }
   int     set_label_text(string in_text)
           { return ChipsRemote::set_label_text(NULL, in_text); }
   int     set_label_text(ChipsId* cid, string in_text)
           { return ChipsRemote::set_label_text(cid, in_text); }
   int     get_label_text(string& out_val)
           { return ChipsRemote::get_label_text(NULL, out_val); }
   int     get_label_text(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_label_text(cid, out_val); }
   int     set_label_size(int sz)
           { return ChipsRemote::set_label_size(NULL, sz); }
   int     set_label_size(ChipsId* cid, int sz)
           { return ChipsRemote::set_label_size(cid, sz); }
   int     get_label_size(int& out_val)
           { return ChipsRemote::get_label_size(NULL, out_val); }
   int     get_label_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_label_size(cid, out_val); }
   int     set_label_depth(int sz)
           { return ChipsRemote::set_label_depth(NULL, sz); }
   int     set_label_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_label_depth(cid, sz); }
   int     get_label_depth(int& out_val)
           { return ChipsRemote::get_label_depth(NULL, out_val); }
   int     get_label_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_label_depth(cid, out_val); }
   int     set_label_angle(float sz)
           { return ChipsRemote::set_label_angle(NULL, sz); }
   int     set_label_angle(ChipsId* cid, float sz)
           { return ChipsRemote::set_label_angle(cid, sz); }
   int     get_label_angle(float& out_val)
           { return ChipsRemote::get_label_angle(NULL, out_val); }
   int     get_label_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_label_angle(cid, out_val); }
   int     set_label_halign(float sz)
           { return ChipsRemote::set_label_halign(NULL, sz); }
   int     set_label_halign(ChipsId* cid, float sz)
           { return ChipsRemote::set_label_halign(cid, sz); }
   int     get_label_halign(float& out_val)
           { return ChipsRemote::get_label_halign(NULL, out_val); }
   int     get_label_halign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_label_halign(cid, out_val); }
   int     set_label_valign(float sz)
           { return ChipsRemote::set_label_valign(NULL, sz); }
   int     set_label_valign(ChipsId* cid, float sz)
           { return ChipsRemote::set_label_valign(cid, sz); }
   int     get_label_valign(float& out_val)
           { return ChipsRemote::get_label_valign(NULL, out_val); }
   int     get_label_valign(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_label_valign(cid, out_val); }
   int     set_label_font(string in_font)
           { return ChipsRemote::set_label_font(NULL, in_font); }
   int     set_label_font(ChipsId* cid, string in_font)
           { return ChipsRemote::set_label_font(cid, in_font); }
   int     get_label_font(string& out_val)
           { return ChipsRemote::get_label_font(NULL, out_val); }
   int     get_label_font(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_label_font(cid, out_val); }
   int     set_label_fontstyle(string in_style)
           { return ChipsRemote::set_label_fontstyle(NULL, in_style); }
   int     set_label_fontstyle(ChipsId* cid, string in_style)
           { return ChipsRemote::set_label_fontstyle(cid, in_style); }
   int     get_label_fontstyle(string& out_val)
           { return ChipsRemote::get_label_fontstyle(NULL, out_val); }
   int     get_label_fontstyle(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_label_fontstyle(cid, out_val); }
   int     hide_label()
           { return ChipsRemote::hide_label(NULL); }
   int     hide_label(ChipsId* cid)
           { return ChipsRemote::hide_label(cid); }
   int     display_label()
           { return ChipsRemote::display_label(NULL); }
   int     display_label(ChipsId* cid)
           { return ChipsRemote::display_label(cid); }
   int     get_label_visible(bool& out_val)
           { return ChipsRemote::get_label_visible(NULL, out_val); }
   int     get_label_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_label_visible(cid, out_val); }
   int     set_current_label(ChipsId* cid)
           { return ChipsRemote::set_current_label(cid); }

   // line operations 
   int     add_line(double startx, double starty, double endx, double endy,
                    string attlist = "")
           { return ChipsRemote::add_line(NULL, startx, starty, endx, endy,
                                          attlist); } 
   int     add_line(ChipsId* cid, double startx, double starty, double endx,
                    double endy, string attlist = "")
           { return ChipsRemote::add_line(cid, startx, starty, endx, endy,
                                          attlist); } 
   int     add_line(double* xx, double* yy, unsigned int count, 
                    string attlist = "")
           { return ChipsRemote::add_line(NULL, xx, yy, count, attlist); }
   int     add_line(ChipsId* cid, double* xx, double* yy, unsigned int count, 
                    string attlist = "")
           { return ChipsRemote::add_line(cid, xx, yy, count, attlist); }
   int     delete_line()
           { return ChipsRemote::delete_line(NULL); }
   int     delete_line(ChipsId* cid)
           { return ChipsRemote::delete_line(cid); }
   int     set_line_color(string in_color)
           { return ChipsRemote::set_line_color(NULL, in_color); }
   int     set_line_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_line_color(cid, in_color); }
   int     get_line_color(string& out_val)
           { return ChipsRemote::get_line_color(NULL, out_val); }
   int     get_line_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_line_color(cid, out_val); }
   int     set_line_thickness(float in_thickness)
           { return ChipsRemote::set_line_thickness(NULL, in_thickness); }
   int     set_line_thickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_line_thickness(cid, in_thickness); }
   int     get_line_thickness(float& out_val)
           { return ChipsRemote::get_line_thickness(NULL, out_val); }
   int     get_line_thickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_line_thickness(cid, out_val); }
   int     set_line_style(V_LINE_TYPE in_style)
           { return ChipsRemote::set_line_style(NULL, in_style); }
   int     set_line_style(ChipsId* cid, V_LINE_TYPE in_style)
           { return ChipsRemote::set_line_style(cid, in_style); }
   int     get_line_style(V_LINE_TYPE& out_val)
           { return ChipsRemote::get_line_style(NULL, out_val); }
   int     get_line_style(ChipsId* cid, V_LINE_TYPE& out_val)
           { return ChipsRemote::get_line_style(cid, out_val); }
   int     set_line_extend(V_LINE_EXTENSION_TYPE val)
           { return ChipsRemote::set_line_extend(NULL, val); }
   int     set_line_extend(ChipsId* cid, V_LINE_EXTENSION_TYPE val)
           { return ChipsRemote::set_line_extend(cid, val); }
   int     get_line_extend(V_LINE_EXTENSION_TYPE &out_val)
           { return ChipsRemote::get_line_extend(NULL, out_val); }
   int     get_line_extend(ChipsId* cid, V_LINE_EXTENSION_TYPE &out_val)
           { return ChipsRemote::get_line_extend(cid, out_val); }
   int     set_line_depth(int sz)
           { return ChipsRemote::set_line_depth(NULL, sz); }
   int     set_line_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_line_depth(cid, sz); }
   int     get_line_depth(int& out_val)
           { return ChipsRemote::get_line_depth(NULL, out_val); }
   int     get_line_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_line_depth(cid, out_val); }
   int     hide_line()
           { return ChipsRemote::hide_line(NULL); }
   int     hide_line(ChipsId* cid)
           { return ChipsRemote::hide_line(cid); }
   int     display_line()
           { return ChipsRemote::display_line(NULL); }
   int     display_line(ChipsId* cid)
           { return ChipsRemote::display_line(cid); }
   int     get_line_visible(bool& out_val)
           { return ChipsRemote::get_line_visible(NULL, out_val); }
   int     get_line_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_line_visible(cid, out_val); }
   int     set_current_line(ChipsId* cid)
           { return ChipsRemote::set_current_line(cid); }

   // point operations
   int     add_point( double xx, double yy,  string attlist = "")
           { return ChipsRemote::add_point(NULL, xx, yy, attlist); }
   int     add_point(ChipsId* cid,  double xx, double yy,  string attlist = "")
           { return ChipsRemote::add_point(cid, xx, yy, attlist); }
   int     delete_point()
           { return ChipsRemote::delete_point(NULL); }
   int     delete_point(ChipsId* cid)
           { return ChipsRemote::delete_point(cid); }
   int     set_point_color(string in_color)
           { return ChipsRemote::set_point_color(NULL, in_color); }
   int     set_point_color(ChipsId* cid, string in_color)
           { return ChipsRemote::set_point_color(cid, in_color); }
   int     get_point_color(string& out_val)
           { return ChipsRemote::get_point_color(NULL, out_val); }
   int     get_point_color(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_point_color(cid, out_val); }
   int     set_point_size(int in_size)
           { return ChipsRemote::set_point_size(NULL, in_size); }
   int     set_point_size(ChipsId* cid, int in_size)
           { return ChipsRemote::set_point_size(cid, in_size); }
   int     get_point_size(int& out_val)
           { return ChipsRemote::get_point_size(NULL, out_val); }
   int     get_point_size(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_point_size(cid, out_val); }
   int     set_point_angle(float in_val)
           { return ChipsRemote::set_point_angle(NULL, in_val); }
   int     set_point_angle(ChipsId* cid, float in_val)
           { return ChipsRemote::set_point_angle(cid, in_val); }
   int     get_point_angle(float& out_val)
           { return ChipsRemote::get_point_angle(NULL, out_val); }
   int     get_point_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_point_angle(cid, out_val); }
   int     set_point_fill(bool in_fill)
           { return ChipsRemote::set_point_fill(NULL, in_fill); }
   int     set_point_fill(ChipsId* cid, bool in_fill)
           { return ChipsRemote::set_point_fill(cid, in_fill); }
   int     get_point_fill(bool& out_val)
           { return ChipsRemote::get_point_fill(NULL, out_val); }
   int     get_point_fill(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_point_fill(cid, out_val); }
   int     set_point_depth(int sz)
           { return ChipsRemote::set_point_depth(NULL, sz); }
   int     set_point_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_point_depth(cid, sz); }
   int     get_point_depth(int& out_val)
           { return ChipsRemote::get_point_depth(NULL, out_val); }
   int     get_point_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_point_depth(cid, out_val); }
   int     set_point_style(V_SYMBOL_TYPE in_symbol)
           { return ChipsRemote::set_point_symbol(NULL, in_symbol); }
   int     set_point_style(ChipsId* cid, V_SYMBOL_TYPE in_symbol)
           { return ChipsRemote::set_point_symbol(cid, in_symbol); }
   int     get_point_style(V_SYMBOL_TYPE& out_val)
           { return ChipsRemote::get_point_symbol(NULL, out_val); }
   int     get_point_style(ChipsId* cid, V_SYMBOL_TYPE& out_val)
           { return ChipsRemote::get_point_symbol(cid, out_val); }
   int     hide_point()
           { return ChipsRemote::hide_point(NULL); }
   int     hide_point(ChipsId* cid)
           { return ChipsRemote::hide_point(cid); }
   int     display_point()
           { return ChipsRemote::display_point(NULL); }
   int     display_point(ChipsId* cid)
           { return ChipsRemote::display_point(cid); }
   int     get_point_visible(bool& out_val)
           { return ChipsRemote::get_point_visible(NULL, out_val); }
   int     get_point_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_point_visible(cid, out_val); }
   int     set_current_point(ChipsId* cid)
           { return ChipsRemote::set_current_point(cid); }

#ifdef WLM_DO_THIS
   // legend operations
//WLM   int     add_legend(ChipsId* cid, const char** curve_ids, 
//WLM                      const char** curve_labels, string lgid = "");
//WLM   int     delete_legend(ChipsId* cid);
//WLM   int     hide_legend(ChipsId* cid);
//WLM   int     display_legend(ChipsId* cid);
//WLM   int     set_legend_color(ChipsId* cid, string in_color);
//WLM   int     set_legend_borders(ChipsId* cid, bool showborders);
//WLM   int     set_legend_thickness(ChipsId* cid, int in_thickness);
//WLM   int     set_legend_transparency(ChipsId* cid, bool seethrough); 
//WLM   int     set_current_legend(ChipsId* cid);
//WLM   int     print_frame(ChipsId* cid, string outfile="");
#endif

   int     add_region(double* xx, double* yy, 
                    unsigned int count, string attlist = "")
           { return ChipsRemote::add_region(NULL, xx, yy, count, attlist); }
   int     add_region(ChipsId* cid, double* xx, double* yy, 
                    unsigned int count, string attlist = "")
           { return ChipsRemote::add_region(cid, xx, yy, count, attlist); }
   int     add_region(int sides, double xx, double yy, 
                      double radius, string attlist = "")
           { return ChipsRemote::add_region(NULL, sides, xx, yy, radius, attlist); }
   int     add_region(ChipsId* cid, int sides, double xx, double yy, 
                      double radius, string attlist = "")
           { return ChipsRemote::add_region(cid, sides, xx, yy, radius, attlist); }
   int     delete_region()
           { return ChipsRemote::delete_region(NULL); }
   int     delete_region(ChipsId* cid)
           { return ChipsRemote::delete_region(cid); }
   int     hide_region()
           { return ChipsRemote::hide_region(NULL); }
   int     hide_region(ChipsId* cid)
           { return ChipsRemote::hide_region(cid); }
   int     display_region()
           { return ChipsRemote::display_region(NULL); }
   int     display_region(ChipsId* cid)
           { return ChipsRemote::display_region(cid); }
   int     get_region_visible(bool& out_val)
           { return ChipsRemote::get_region_visible(NULL, out_val); }
   int     get_region_visible(ChipsId* cid, bool& out_val)
           { return ChipsRemote::get_region_visible(cid, out_val); }
   int     set_region_depth(int sz)
           { return ChipsRemote::set_region_depth(NULL, sz); }
   int     set_region_depth(ChipsId* cid, int sz)
           { return ChipsRemote::set_region_depth(cid, sz); }
   int     get_region_depth(int& out_val)
           { return ChipsRemote::get_region_depth(NULL, out_val); }
   int     get_region_depth(ChipsId* cid, int& out_val)
           { return ChipsRemote::get_region_depth(cid, out_val); }
   int     set_region_angle(float in_val)
           { return ChipsRemote::set_region_angle(NULL, in_val); }
   int     set_region_angle(ChipsId* cid, float in_val)
           { return ChipsRemote::set_region_angle(cid, in_val); }
   int     get_region_angle(float& out_val)
           { return ChipsRemote::get_region_angle(NULL, out_val); }
   int     get_region_angle(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_region_angle(cid, out_val); }
   int     set_region_edgethickness(float in_thickness)
           { return ChipsRemote::set_region_edgethickness(NULL, in_thickness); }
   int     set_region_edgethickness(ChipsId* cid, float in_thickness)
           { return ChipsRemote::set_region_edgethickness(cid, in_thickness); }
   int     get_region_edgethickness(float& out_val)
           { return ChipsRemote::get_region_edgethickness(NULL, out_val); }
   int     get_region_edgethickness(ChipsId* cid, float& out_val)
           { return ChipsRemote::get_region_edgethickness(cid, out_val); }
   int     set_region_edgestyle(V_LINE_TYPE in_style)
           { return ChipsRemote::set_region_edgestyle(NULL, in_style); }
   int     set_region_edgestyle(ChipsId* cid, V_LINE_TYPE in_style)
           { return ChipsRemote::set_region_edgestyle(cid, in_style); }
   int     get_region_edgestyle(V_LINE_TYPE& out_val)
           { return ChipsRemote::get_region_edgestyle(NULL, out_val); }
   int     get_region_edgestyle(ChipsId* cid, V_LINE_TYPE& out_val)
           { return ChipsRemote::get_region_edgestyle(cid, out_val); }
   int     set_region_edgecolor(string in_color)
           { return ChipsRemote::set_region_edgecolor(NULL, in_color); }
   int     set_region_edgecolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_region_edgecolor(cid, in_color); }
   int     get_region_edgecolor(string& out_val)
           { return ChipsRemote::get_region_edgecolor(NULL, out_val); }
   int     get_region_edgecolor(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_region_edgecolor(cid, out_val); }
   int     set_region_fillcolor(string in_color)
           { return ChipsRemote::set_region_fillcolor(NULL, in_color); }
   int     set_region_fillcolor(ChipsId* cid, string in_color)
           { return ChipsRemote::set_region_fillcolor(cid, in_color); }
   int     get_region_fillcolor(string& out_val)
           { return ChipsRemote::get_region_fillcolor(NULL, out_val); }
   int     get_region_fillcolor(ChipsId* cid, string& out_val)
           { return ChipsRemote::get_region_fillcolor(cid, out_val); }
   int     set_region_fillstyle(V_FILL_TYPE in_style)
           { return ChipsRemote::set_region_fillstyle(NULL, in_style); }
   int     set_region_fillstyle(ChipsId* cid, V_FILL_TYPE in_style)
           { return ChipsRemote::set_region_fillstyle(cid, in_style); }
   int     get_region_fillstyle(V_FILL_TYPE& out_val)
           { return ChipsRemote::get_region_fillstyle(NULL, out_val); }
   int     get_region_fillstyle(ChipsId* cid, V_FILL_TYPE& out_val)
           { return ChipsRemote::get_region_fillstyle(cid, out_val); }
   int     set_region_opacity(ChipsId* cid, double in_val)
              { return ChipsRemote::set_region_opacity(cid, in_val); }
   int     set_region_opacity(double in_val)
              { return ChipsRemote::set_region_opacity(NULL, in_val); }
   int     get_region_opacity(ChipsId* cid, double& out_val)
           { return ChipsRemote::get_region_opacity(cid, out_val); }
   int     get_region_opacity(double& out_val)
           { return ChipsRemote::get_region_opacity(NULL, out_val); }
   int     set_current_region(ChipsId* cid)
           { return ChipsRemote::set_current_region(cid); }

   // string instead of cid wrappers
   int     set_cascading_property(string idstr, V_OBJECT_TYPE start, string property, string value); 
   int     set_current_window(string idstr);
   int     set_window_title(string idstr, string title); 
   int     set_window_bgcolor(string idstr, string in_color); 
   int     set_window_redraw(string idstr, bool draw); 
   int     set_frame_bgcolor(string idstr, string in_color);
   int     set_frame_border_color(string idstr, string in_color); 
   int     set_default_depth(string idstr, int sz);
   int     set_framemode(string idstr, int mode);
   int     set_frame_transparency(string idstr, bool seethrough);
   int     set_frame_scale(string idstr, bool val); 
   int     display_frame(string idstr);
   int     hide_frame(string idstr);
   int     display_depth(string idstr, int depth);
   int     hide_depth(string idstr, int depth);
   int     set_current_frame(string idstr);
   int     set_plot_title(string idstr, string in_title);
   int     set_plot_title_color(string idstr, string in_color);
   int     set_plot_title_angle(string idstr, float in_val);
   int     set_plot_title_halign(string idstr, float in_val);
   int     set_plot_title_valign(string idstr, float in_val);
   int     set_plot_title_size(string idstr, int sz);
   int     set_plot_title_font(string idstr, string font);
   int     set_plot_title_fontstyle(string idstr, string style);
   int     set_plot_title_xpos(string idstr, float in_norm);
   int     set_plot_title_ypos(string idstr, float in_norm);
   int     set_plot_xlabel(string idstr, string in_lab);
   int     set_plot_ylabel(string idstr, string in_lab);
   int     set_plot_axisstyle(string idstr, string style); 
   int     set_plot_axisstyle(string idstr, V_AXISSTYLE_TYPE style);
   int     set_plot_cornerstyle(string idstr, string style);
   int     set_plot_cornerstyle(string idstr, V_CORNERSTYLE_TYPE style);
   int     set_plot_leftmargin(string idstr, float margin_sz); 
   int     set_plot_rightmargin(string idstr, float margin_sz); 
   int     set_plot_topmargin(string idstr, float margin_sz); 
   int     set_plot_bottommargin(string idstr, float margin_sz); 
   int     set_plot_title_depth(string idstr, int sz);
   int     set_axis_scalable(string idstr, bool scalable); 
   int     hide_plot(string idstr);
   int     display_plot(string idstr);
   int     set_current_plot(string idstr);
   int     set_arbitrary_tick_positions(string idstr, unsigned int arraysize, double* pos, string* vals=NULL); 
   int     set_axis_major_mode(string idstr, string in_mode); 
   int     set_axis_major_thickness(string idstr, float in_thickness);
   int     set_axis_major_style(string idstr, string in_style);
   int     set_axis_major_style(string idstr, int in_style);
   int     set_axis_minor_mode(string idstr, string in_mode);
   int     set_axis_minor_thickness(string idstr, float in_thickness);
   int     set_axis_minor_style(string idstr, string in_style);
   int     set_axis_minor_style(string idstr, int in_style);
   int     set_axis_parallel_offset(string idstr, float in_val);
   int     get_axis_parallel_offset(string idstr, float& out_val);
   int     set_axis_perpendicular_offset(string idstr, float in_val);
   int     get_axis_perpendicular_offset(string idstr, float& out_val);
   int     set_axis_automin(string idstr, bool on); 
   int     set_axis_automax(string idstr, bool on); 
   int     set_axis_depth(string idstr, int sz);
   int     set_axis_pad(string idstr, double in_val);
   int     set_axis_label_color(string idstr, string in_color);
   int     set_axis_label_size(string idstr, int sz);
   int     set_axis_label_angle(string idstr, float sz);
   int     set_axis_xlabel_angle(string idstr, float sz);
   int     set_axis_ylabel_angle(string idstr, float sz);
   int     set_axis_label_halign(string idstr, float sz);
   int     set_axis_label_valign(string idstr, float sz);
   int     set_axis_label_font(string idstr, string font);
   int     set_axis_label_fontstyle(string idstr, string style);
   int     set_axis_label_visible(string idstr, bool vis);
   int     set_axis_ticklabel_style(string idstr, string style);
   int     set_axis_ticklabel_style(string idstr, int in_style);
   int     set_axis_ticklabel_color(string idstr, string in_color);
   int     set_axis_ticklabel_size(string idstr, int sz);
   int     set_axis_ticklabel_angle(string idstr, float sz);
   int     set_axis_ticklabel_halign(string idstr, float sz);
   int     set_axis_ticklabel_valign(string idstr, float sz);
   int     set_axis_ticklabel_font(string idstr, string font);
   int     set_axis_ticklabel_fontstyle(string idstr, string style);
   int     set_axis_ticklabel_offset(string idstr, int in_val);
   int     get_axis_ticklabel_offset(string idstr, int& out_val);
   int     set_axis_ticklabel_visible(string idstr, bool vis);
   int     set_axis_majorgrid_visible(string idstr, bool vis);
   int     set_axis_majorgrid_thickness(string idstr, float in_size);
   int     set_axis_majorgrid_style(string idstr, V_LINE_TYPE in_style);
   int     set_axis_majorgrid_color(string idstr, string in_color);
   int     set_axis_minorgrid_visible(string idstr, bool vis);
   int     set_axis_minorgrid_thickness(string idstr, float in_size);
   int     set_axis_minorgrid_style(string idstr, V_LINE_TYPE in_style);
   int     set_axis_minorgrid_color(string idstr, string in_color);
   int     display_axis(string idstr);
   int     hide_axis(string idstr);
   int     reverse_axes(string idstr, int dim);
   int     display_major_ticks(string idstr);
   int     hide_major_ticks(string idstr);
   int     display_minor_ticks(string idstr);
   int     hide_minor_ticks(string idstr);
   int     set_curve_depth(string idstr, int sz);
   int     set_curve_symbolstyle(string idstr, V_SYMBOL_TYPE val);
   int     set_curve_symbolfill(string idstr, bool in_fill);
   int     set_curve_symbolsize(string idstr, int in_size);
   int     set_curve_symbolangle(string idstr, float in_val);
   int     set_curve_symbolcolor(string idstr, string in_color);
   int     set_curve_linestyle(string idstr, V_LINE_TYPE val);
   int     set_curve_linethickness(string idstr, float in_size);
   int     set_curve_linecolor(string idstr, string in_color);
   int     set_curve_baddatamode(string idstr, V_BADDATAMODE_TYPE val);
   int     set_curve_errstyle(string idstr, string style);
   int     set_curve_errup(string idstr, bool val);
   int     set_curve_errdown(string idstr, bool val);
   int     set_curve_errleft(string idstr, bool val);
   int     set_curve_errright(string idstr, bool val);
   int     set_curve_errthickness(string idstr, float in_thickness);
   int     set_curve_errcolor(string idstr, string in_color);
   int     set_curve_errcaplength(string idstr, int in_size);
   int     set_curve_limitoverride(string idstr, bool val);
   int     set_curve_limitlength(string idstr, double in_val);
   int     hide_curve(string idstr);
   int     display_curve(string idstr);
   int     set_current_curve(string idstr);

   int     set_image_colormap(string idstr, V_COLORMAP_TYPE val);
   int     set_image_invert_colormap(string idstr, bool val);
   int     set_image_colormap_interpolate(string idstr, bool val);
   int     set_image_colormap_size(string idstr, int val);
   int     set_image_interpolation(string idstr, V_INTERPOLATION_TYPE val);
   int     set_image_alpha(string idstr, double min, double max);
   int     set_image_threshold(string idstr, double min, double max);
   int     set_image_depth(string idstr, int sz);
   int     set_image_scale_channels (string idstr, bool val);
   int     hide_image(string idstr);
   int     display_image(string idstr);

   int     set_colorbar_depth(string idstr, int sz);
   int     hide_colorbar(string idstr);
   int     display_colorbar(string idstr);
   int     get_colorbar_depth(string idstr, int& out_val);
   int     set_colorbar_width(string idstr, float in_val);
   int     get_colorbar_width(string idstr, float& out_val);
   int     set_colorbar_length(string idstr, float in_val);
   int     get_colorbar_length(string idstr, float& out_val);
   int     hide_colorbar_border(string idstr);
   int     display_colorbar_border(string idstr);
   int     get_colorbar_border_visible(string idstr, bool& out_val);
   int     set_colorbar_halign(string idstr, float sz);
   int     get_colorbar_halign(string idstr, float& out_val);
   int     set_colorbar_valign(string idstr, float sz);
   int     get_colorbar_valign(string idstr, float& out_val);
   int     set_colorbar_label_angle(string idstr, float sz);
   int     get_colorbar_label_angle(string idstr, float& out_val);
   int     set_colorbar_label_halign(string idstr, float sz);
   int     get_colorbar_label_halign(string idstr, float& out_val);
   int     set_colorbar_label_valign(string idstr, float sz);
   int     get_colorbar_label_valign(string idstr, float& out_val);
   int     set_colorbar_orientation(string idstr, int inval);
   int     get_colorbar_orientation(string idstr, int& out_val);
   int     set_colorbar_label_text(string idstr, string in_color);
   int     get_colorbar_label_text(string idstr, string& out_val);
   int     set_colorbar_label_color(string idstr, string in_color);
   int     get_colorbar_label_color(string idstr, string& out_val);
   int     set_colorbar_label_size(string idstr, int sz);
   int     get_colorbar_label_size(string idstr, int& out_val);
   int     set_colorbar_label_visible(string idstr, bool vis);
   int     get_colorbar_label_visible(string idstr, bool& out_val);
   int     set_colorbar_label_font(string idstr, string font);
   int     get_colorbar_label_font(string idstr, string& out_val);
   int     set_colorbar_label_fontstyle(string idstr, string style);
   int     get_colorbar_label_fontstyle(string idstr, string& out_val);
   int     set_colorbar_ticklabel_color(string idstr, string in_color);
   int     get_colorbar_ticklabel_color(string idstr, string& out_val);
   int     set_colorbar_ticklabel_size(string idstr, int in_val);
   int     get_colorbar_ticklabel_size(string idstr, int& out_val);
   int     set_colorbar_ticklabel_font(string idstr, string in_font);
   int     get_colorbar_ticklabel_font(string idstr, string& out_val);
   int     set_colorbar_ticklabel_fontstyle(string idstr, string style);
   int     get_colorbar_ticklabel_fontstyle(string idstr, string& out_val);
   int     set_colorbar_ticklabel_visible(string idstr, bool vis);
   int     get_colorbar_ticklabel_visible(string idstr, bool& out_val);
   int     set_colorbar_ticklabel_angle(string idstr, float in_val);
   int     get_colorbar_ticklabel_angle(string idstr, float& out_val);
   int     set_colorbar_ticklabel_halign(string idstr, float in_val);
   int     get_colorbar_ticklabel_halign(string idstr, float& out_val);
   int     set_colorbar_ticklabel_valign(string idstr, float in_val);
   int     get_colorbar_ticklabel_valign(string idstr, float& out_val);
   int     set_colorbar_ticklabel_offset(string idstr, int in_val);
   int     get_colorbar_ticklabel_offset(string idstr, int& out_val);
   int     set_colorbar_label_location(string idstr, string style);
   int     set_colorbar_label_location(string idstr, int in_style);
   int     get_colorbar_label_location(string idstr, string& out_val);
   int     set_colorbar_tick_style(string idstr, string in_style);
   int     set_colorbar_tick_style(string idstr, int in_style);
   int     get_colorbar_tick_style(string idstr, string& out_val);
   int     set_colorbar_tick_thickness(string idstr, float in_thickness);
   int     get_colorbar_tick_thickness(string idstr, float& out_val);
   int     set_colorbar_tick_mode(string idstr, string in_mode);
   int     get_colorbar_tick_mode(string idstr, string& out_val);
   int     set_colorbar_tick_location(string idstr, string in_style);
   int     set_colorbar_tick_location(string idstr, int in_style);
   int     get_colorbar_tick_location(string idstr, string& out_val);
   int     set_colorbar_tick_visible(string idstr, bool vis);
   int     get_colorbar_tick_visible(string idstr, bool& out_val);
   int     set_colorbar_tick_color(string idstr, string in_color);
   int     get_colorbar_tick_color(string idstr, string& out_color);
   int     set_colorbar_tick_length(string idstr, int in_length);
   int     get_colorbar_tick_length(string idstr, int& out_val);
   int     set_colorbar_parallel_offset(string idstr, float in_val);
   int     get_colorbar_parallel_offset(string idstr, float& out_val);
   int     set_colorbar_perpendicular_offset(string idstr, float in_val);
   int     get_colorbar_perpendicular_offset(string idstr, float& out_val);


   int     set_contour_mode(string idstr, string mode); 
   int     set_contour_style(string idstr, string style); 
   int     set_contour_color(string idstr, string in_color); 
   int     set_contour_interval(string idstr, double in_val);
   int     set_contour_numlevels(string idstr, int in_val);
   int     set_contour_levels(string idstr, unsigned int arraysize,double* pos);
   int     set_contour_thickness(string idstr, float in_size);
   int     set_current_contour(string idstr);
   int     set_contour_algorithm(string idstr, V_CONTOUR_TYPE val);
   int     set_contour_depth(string idstr, int sz);
/*
DEPRECATED
   int     set_contour_transform(string idstr, Transform *trans);
   int     get_contour_transform (string idstr, Transform **trans);
   int     apply_contour_transform(string idstr);
   int     apply_contour_transform_inverse(string idstr);
*/
   int     hide_contour(string idstr);
   int     display_contour(string idstr);
   int     set_histogram_symbolstyle(string idstr, V_SYMBOL_TYPE val);
   int     set_histogram_symbolfill(string idstr, bool in_fill);
   int     set_histogram_symbolsize(string idstr, int in_size);
   int     set_histogram_symbolangle(string idstr, float in_val);
   int     set_histogram_symbolcolor(string idstr, string in_color);
   int     set_histogram_linestyle(string idstr, V_LINE_TYPE val);
   int     set_histogram_linethickness(string idstr, float in_size);
   int     set_histogram_linecolor(string idstr, string in_color);
   int     set_histogram_errstyle(string idstr, string style);
   int     set_histogram_errup(string idstr, bool val);
   int     set_histogram_errdown(string idstr, bool val);
   int     set_histogram_dropline(string idstr, bool val);
   int     set_histogram_errthickness(string idstr, float in_thickness);
   int     set_histogram_errcolor(string idstr, string in_color);
   int     set_histogram_errcaplength(string idstr, int in_size);
   int     set_histogram_fillopacity(string idstr, double in_opacity);
   int     set_histogram_fillcolor(string idstr, string in_color);
   int     set_histogram_fillstyle(string idstr, V_FILL_TYPE in_style);
   int     hide_histogram(string idstr);
   int     display_histogram(string idstr);
   int     set_current_histogram(string idstr);
   int     set_histogram_depth(string idstr, int sz);
   int     set_axis_color(string idstr, string in_color);
   int     set_axis_major_color(string idstr, string in_color);
   int     set_axis_minor_color(string idstr, string in_color);
   int     set_axis_text(string idstr, string in_text);
   int     set_axis_xlabel_text(string idstr, string in_text);
   int     set_axis_ylabel_text(string idstr, string in_text);
   int     set_axis_tickformat(string idstr, string in_format);
   int     set_axis_thickness(string idstr, float in_thickness);
   int     set_axis_tickstyle(string idstr, int in_style);
   int     set_axis_tickstyle(string idstr, string in_style);
   int     set_axis_major_num(string idstr, int in_num);
   int     set_axis_minor_num(string idstr, int in_num);
   int     set_axis_major_length(string idstr, int in_len);
   int     set_axis_minor_length(string idstr, int in_len);
   int     set_axis_major_interval(string idstr, double in_len);
   int     set_axis_minor_interval(string idstr, double in_len);
   int     set_current_axis(string idstr);
   int     set_current_image(string idstr);
   int     set_current_colorbar(string idstr);
   int     set_label_color(string idstr, string in_color);
   int     set_label_text(string idstr, string in_text);
   int     set_label_size(string idstr, int sz);
   int     set_label_depth(string idstr, int sz);
   int     set_label_angle(string idstr, float sz);
   int     set_label_halign(string idstr, float sz);
   int     set_label_valign(string idstr, float sz);
   int     set_label_font(string idstr, string in_font);
   int     set_label_fontstyle(string idstr, string in_style);
   int     hide_label(string idstr);
   int     display_label(string idstr);
   int     set_current_label(string idstr);
   int     set_line_color(string idstr, string in_color);
   int     set_line_thickness(string idstr, float in_thickness);
   int     set_line_style(string idstr, V_LINE_TYPE in_style);
   int     set_line_extend(string idstr, V_LINE_EXTENSION_TYPE in_extend);
   int     set_line_depth(string idstr, int sz);
   int     hide_line(string idstr);
   int     display_line(string idstr);
   int     set_current_line(string idstr);
   int     set_point_color(string idstr, string in_color);
   int     set_point_size(string idstr, int in_size);
   int     set_point_angle(string idstr, float in_val);
   int     set_point_fill(string idstr, bool in_fill);
   int     set_point_depth(string idstr, int sz);
   int     set_point_style(string idstr, V_SYMBOL_TYPE in_symbol);
   int     hide_point(string idstr);
   int     display_point(string idstr);
   int     set_current_point(string idstr);
   int     set_region_depth(string idstr, int sz);
   int     set_region_angle(string idstr, float in_val);
   int     set_region_edgethickness(string idstr, float in_thickness);
   int     set_region_edgestyle(string idstr, V_LINE_TYPE in_style);
   int     set_region_fillstyle(string idstr, V_FILL_TYPE in_style);
   int     set_region_edgecolor(string idstr, string in_color);
   int     set_region_fillcolor(string idstr, string in_color);
   int     set_region_opacity(string idstr, double in_val);
   int     hide_region(string idstr);
   int     display_region(string idstr);
   int     set_current_region(string idstr);
   int     set_plot_aspect_ratio(string idstr, string in_val);
   int     set_data_aspect_ratio(string idstr, string in_val);
   int     highlight_object(string idstr, V_OBJECT_TYPE obj); 
   int     get_object_count(string idstr, V_OBJECT_TYPE obj, int& out_val);
   int     get_object_coordinfo(string idstr, V_OBJECT_TYPE obj, COORD_SYS& out_val, string& out_id);
   int     get_window_title(string idstr, string& out_val); 
   int     get_window_bgcolor(string idstr, string& out_val); 
   int     get_window_redraw(string idstr, bool& out_val);
   int     get_frame_bgcolor(string idstr, string& out_val); 
   int     get_frame_border_color(string idstr, string& out_val);
   int     get_frame_border_visible(string idstr, bool& out_val);
   int     get_default_depth(string idstr, int& out_val);
   int     get_frame_transparency(string idstr, bool& out_val);
   int     get_frame_scale(string idstr, bool& out_val);
   int     get_plot_title(string idstr, string& out_val);
   int     get_plot_title_color(string idstr, string& out_val);
   int     get_plot_title_angle(string idstr, float& out_val);
   int     get_plot_title_halign(string idstr, float& out_val);
   int     get_plot_title_valign(string idstr, float& out_val);
   int     get_plot_title_size(string idstr, int& out_val);
   int     get_plot_title_font(string idstr, string &out_val);
   int     get_plot_title_fontstyle(string idstr, string &out_val);
   int     get_plot_title_xpos(string idstr, float& out_val);
   int     get_plot_title_ypos(string idstr, float& out_val);
   int     get_plot_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2);
   int     get_plot_xrange(string idstr, double& out1, double& out2);
   int     get_plot_yrange(string idstr, double& out1, double& out2);
   int     get_plot_axisstyle(string idstr, string& out_val);
   int     get_plot_cornerstyle(string idstr, string& out_val);
   int     get_plot_visible(string idstr, bool& out_val);
   int     get_axis_visible(string idstr, bool& out_val);
   int     get_contour_visible(string idstr, bool& out_val);
   int     get_curve_visible(string idstr, bool& out_val);
   int     get_image_visible(string idstr, bool& out_val);
   int     get_colorbar_visible(string idstr, bool& out_val);
   int     get_histogram_visible(string idstr, bool& out_val);
   int     get_plot_leftmargin(string idstr, float& out_val); 
   int     get_plot_rightmargin(string idstr, float& out_val); 
   int     get_plot_topmargin(string idstr, float& out_val); 
   int     get_plot_bottommargin(string idstr, float& out_val); 
   int     get_plot_title_depth(string idstr, int& out_val);
   int     get_major_ticks_visible(string idstr, bool& out_val);
   int     get_minor_ticks_visible(string idstr, bool& out_val);
   int     get_axis_major_mode(string idstr, string& out_val); 
   int     get_axis_major_thickness(string idstr, float& out_val);
   int     get_axis_major_style(string idstr, string& out_val);
   int     get_axis_minor_mode(string idstr, string& out_val);
   int     get_axis_minor_thickness(string idstr, float& out_val);
   int     get_axis_minor_style(string idstr, string& out_val);
   int     get_axis_automin(string idstr, bool& out_val); 
   int     get_axis_automax(string idstr, bool& out_val); 
   int     get_axis_depth(string idstr, int& out_val);
   int     get_axis_pad(string idstr, double& out_val);
   int     get_axis_scale(string idstr, int& out_val);
   int     get_axis_label_color(string idstr, string& out_val);
   int     get_axis_label_size(string idstr, int& out_val);
   int     get_axis_label_angle(string idstr, float& out_val);
   int     get_axis_xlabel_angle(string idstr, float& out_val);
   int     get_axis_ylabel_angle(string idstr, float& out_val);
   int     get_axis_label_halign(string idstr, float& out_val);
   int     get_axis_label_valign(string idstr, float& out_val);
   int     get_axis_label_font(string idstr, string& out_val);
   int     get_axis_label_fontstyle(string idstr, string& out_val);
   int     get_axis_label_visible(string idstr, bool& out_val);
   int     get_axis_ticklabel_style(string idstr, string& out_val);
   int     get_axis_ticklabel_color(string idstr, string& out_val);
   int     get_axis_ticklabel_size(string idstr, int& out_val);
   int     get_axis_ticklabel_angle(string idstr, float& out_val);
   int     get_axis_ticklabel_halign(string idstr, float& out_val);
   int     get_axis_ticklabel_valign(string idstr, float& out_val);
   int     get_axis_ticklabel_font(string idstr, string& out_val);
   int     get_axis_ticklabel_fontstyle(string idstr, string& out_val);
   int     get_axis_ticklabel_visible(string idstr, bool& out_val);
   int     get_axis_majorgrid_visible(string idstr, bool& out_val);
   int     get_axis_majorgrid_thickness(string idstr, float& out_val);
   int     get_axis_majorgrid_style(string idstr, V_LINE_TYPE& out_val);
   int     get_axis_majorgrid_color(string idstr, string& out_val);
   int     get_axis_minorgrid_visible(string idstr, bool& out_val);
   int     get_axis_minorgrid_thickness(string idstr, float& out_val);
   int     get_axis_minorgrid_style(string idstr, V_LINE_TYPE& out_val);
   int     get_axis_minorgrid_color(string idstr, string& out_val);
   int     get_curve_depth(string idstr, int& out_val);
   int     get_curve_symbolstyle(string idstr, V_SYMBOL_TYPE& out_val);
   int     get_curve_symbolfill(string idstr, bool& out_val);
   int     get_curve_symbolsize(string idstr, int& out_val);
   int     get_curve_symbolangle(string idstr, float& out_val);
   int     get_curve_symbolcolor(string idstr, string& out_color);
   int     get_curve_linestyle(string idstr, V_LINE_TYPE& out_val);
   int     get_curve_linethickness(string idstr, float& out_val);
   int     get_curve_linecolor(string idstr, string& out_val);
   int     get_curve_baddatamode(string idstr, V_BADDATAMODE_TYPE& out_val);
   int     get_curve_errstyle(string idstr, string& out_val);
   int     get_curve_errup(string idstr, bool& out_val);
   int     get_curve_errdown(string idstr, bool& out_val);
   int     get_curve_errleft(string idstr, bool& out_val);
   int     get_curve_errright(string idstr, bool& out_val);
   int     get_curve_errcaplength(string idstr, int& out_val);
   int     get_curve_errthickness(string idstr, float& out_val);
   int     get_curve_errcolor(string idstr, string& out_val);
   int     get_curve_limitoverride(string idstr, bool& out_val);
   int     get_curve_limitlength(string idstr, double& out_val);
   int     get_curve_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2);
   int     get_curve_xrange(string idstr, double& out1, double& out2);
   int     get_curve_yrange(string idstr, double& out1, double& out2);
   int     get_image_colormap(string idstr, V_COLORMAP_TYPE& out_val);
   int     get_image_interpolation(string idstr, V_INTERPOLATION_TYPE& out_val);
   int     get_data_aspect_ratio(string idstr, string& out_val);
   int     get_plot_aspect_ratio(string idstr, string& out_val);
   int     get_plot_aspect_height(string idstr, float& out_val);
   int     get_plot_aspect_width(string idstr, float& out_val);
   int     get_image_invert_colormap(string idstr, bool& out_val);
   int     get_image_colormap_interpolate(string idstr, bool& out_val);
   int     get_image_scale_channels(string idstr, bool& out_val);
   int     get_image_colormap_size(string idstr, int& out_val);
   int     get_image_alpha(string idstr, double& out1, double& out2);
   int     get_image_threshold(string idstr, double& out1, double& out2);
   int     get_image_depth(string idstr, int& out_val);
   int     get_image_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2);
   int     get_image_xrange(string idstr, double& out1, double& out2);
   int     get_image_yrange(string idstr, double& out1, double& out2);
   int     get_contour_mode(string idstr, string& out_val); 
   int     get_contour_style(string idstr, string& out_val); 
   int     get_contour_color(string idstr, string& out_val); 
   int     get_contour_thickness(string idstr, float& out_val);
   int     get_contour_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2);
   int     get_contour_xrange(string idstr, double& out1, double& out2);
   int     get_contour_yrange(string idstr, double& out1, double& out2);
   int     get_contour_zrange(string idstr, double& out1, double& out2);
   int     get_contour_algorithm(string idstr, V_CONTOUR_TYPE& out_contour);
   int     get_contour_depth(string idstr, int& out_val);
   int     get_contour_levels(string idstr, unsigned int &arraysize,double** pos);
   int     get_histogram_symbolstyle(string idstr, V_SYMBOL_TYPE& out_val);
   int     get_histogram_symbolfill(string idstr, bool& out_val);
   int     get_histogram_symbolsize(string idstr, int& out_val);
   int     get_histogram_symbolangle(string idstr, float& out_val);
   int     get_histogram_symbolcolor(string idstr, string& out_color);
   int     get_histogram_linestyle(string idstr, V_LINE_TYPE& out_val);
   int     get_histogram_linethickness(string idstr, float& out_val);
   int     get_histogram_linecolor(string idstr, string& out_val);
   int     get_histogram_errstyle(string idstr, string& out_val);
   int     get_histogram_errup(string idstr, bool& out_val);
   int     get_histogram_errdown(string idstr, bool& out_val);
   int     get_histogram_dropline(string idstr, bool& out_val);
   int     get_histogram_errthickness(string idstr, float& out_val);
   int     get_histogram_errcolor(string idstr, string& out_val);
   int     get_histogram_errcaplength(string idstr, int& out_val);
   int     get_histogram_fillopacity(string idstr, double& out_val);
   int     get_histogram_fillcolor(string idstr, string& out_val);
   int     get_histogram_fillstyle(string idstr, V_FILL_TYPE& out_val);
   int     get_histogram_range(string idstr, double& outx1, double& outx2, double& outy1, double& outy2);
   int     get_histogram_xrange(string idstr, double& out1, double& out2);
   int     get_histogram_yrange(string idstr, double& out1, double& out2);
   int     get_histogram_depth(string idstr, int& out_val);
   int     get_axis_color(string idstr, string& out_val);
   int     get_axis_major_color(string idstr, string& out_val);
   int     get_axis_minor_color(string idstr, string& out_val);
   int     get_axis_text(string idstr, string& out_val);
   int     get_axis_xlabel_text(string idstr, string& out_val);
   int     get_axis_ylabel_text(string idstr, string& out_val);
   int     get_axis_tickformat(string idstr, string& out_val);
   int     get_axis_thickness(string idstr, float& out_val);
   int     get_axis_tickstyle(string idstr, string& out_val);
   int     get_axis_major_num(string idstr, int& out_val);
   int     get_axis_minor_num(string idstr, int& out_val);
   int     get_axis_major_length(string idstr, int& out_val);
   int     get_axis_minor_length(string idstr, int& out_val);
   int     get_axis_major_interval(string idstr, double& out_val);
   int     get_axis_minor_interval(string idstr, double& out_val);
   int     get_axis_range(string idstr, double& out1, double& out2);
   int     get_label_color(string idstr, string& out_val);
   int     get_label_text(string idstr, string& out_val);
   int     get_label_size(string idstr, int& out_val);
   int     get_label_depth(string idstr, int& out_val);
   int     get_label_angle(string idstr, float& out_val);
   int     get_label_halign(string idstr, float& out_val);
   int     get_label_valign(string idstr, float& out_val);
   int     get_label_font(string idstr, string& out_val);
   int     get_label_fontstyle(string idstr, string& out_val);
   int     get_label_visible(string idstr, bool& out_val);
   int     get_line_color(string idstr, string& out_val);
   int     get_line_thickness(string idstr, float& out_val);
   int     get_line_style(string idstr, V_LINE_TYPE& out_val);
   int     get_line_depth(string idstr, int& out_val);
   int     get_line_visible(string idstr, bool& out_val);
   int     get_point_color(string idstr, string& out_val);
   int     get_point_size(string idstr, int& out_val);
   int     get_point_angle(string idstr, float& out_val);
   int     get_point_fill(string idstr, bool& out_val);
   int     get_point_depth(string idstr, int& out_val);
   int     get_point_style(string idstr, V_SYMBOL_TYPE& out_val);
   int     get_point_visible(string idstr, bool& out_val);
   int     get_region_visible(string idstr, bool& out_val);
   int     get_region_depth(string idstr, int& out_val);
   int     get_region_angle(string idstr, float& out_val);
   int     get_region_edgethickness(string idstr, float& out_val);
   int     get_region_edgestyle(string idstr, V_LINE_TYPE& out_val);
   int     get_region_edgecolor(string idstr, string& out_val);
   int     get_region_fillcolor(string idstr, string& out_val);
   int     get_region_fillstyle(string idstr, V_FILL_TYPE& out_val);
   int     get_region_opacity(string idstr, double& out_val);
   int     delete_window(string idstr);
   int     delete_frame(string idstr);
   int     delete_plot(string idstr);
   int     clear_plot(string idstr);
   int     delete_curve(string idstr);
   int     delete_contour(string idstr);
   int     delete_histogram(string idstr);
   int     delete_axis(string idstr);
   int     delete_image(string idstr);
   int     delete_colorbar(string idstr);
   int     set_image_channel (string idstr, double* vals, int xaxis, int yaxis, int channel);
   int     remove_image_channel(string idstr, int channel);
   int     delete_label(string idstr);
   int     delete_line(string idstr);
   int     delete_point(string idstr);
   int     delete_region(string idstr);
   int     shuffle(string idstr, V_OBJECT_TYPE obj, V_SHUFFLE_TYPE way);
   int     move(string idstr, V_OBJECT_TYPE obj, double xx, double yy, 
               int mode=V_MOVE_ABSOLUTE);
   int     log_scale(string idstr, int dim=XY_AXIS);
   int     logarithmic_scale(string idstr, int dim=XY_AXIS);
   int     lin_scale(string idstr, int dim=XY_AXIS);
   int     linear_scale(string idstr, int dim=XY_AXIS);

   // print operations
   int     print_window(string outfile="", string prefs="")
           { return ChipsRemote::print_window(NULL, outfile, prefs); }
   int     print_window(ChipsId* cid, string outfile="", string prefs="")
           { return ChipsRemote::print_window(cid, outfile, prefs); }
   int     print_image(string outfile="", string prefs="")
           { return ChipsRemote::print_image(NULL, outfile, prefs); }
   int     print_image(ChipsId* cid, string outfile="", string prefs="")
           { return ChipsRemote::print_image(cid, outfile, prefs); }

   char*   info(bool descend=true)
           { return ChipsRemote::info(NULL, chips_window, descend); }
   char*   info(ChipsId* cid, bool descend=true)
           { return ChipsRemote::info(cid, chips_window, descend); }
   char*   info(V_OBJECT_TYPE obj, bool descend=true)
           { return ChipsRemote::info(NULL, obj, descend); }
   char*   info(ChipsId* cid, V_OBJECT_TYPE obj, bool descend=true)
           { return ChipsRemote::info(cid, obj, descend); }
   char*   info_coordinate()
           { return ChipsRemote::info_coordinate(NULL); }
   char*   info_coordinate(ChipsId* cid)
           { return ChipsRemote::info_coordinate(cid); }
   char*   info_depth(int depth=0)
           { return ChipsRemote::info_depth(NULL, depth); }
   char*   info_depth(ChipsId* cid, int depth=0)
           { return ChipsRemote::info_depth(cid, depth); }
   char*   info_bound_axes()
           { return ChipsRemote::info_bound_axes(NULL); }
   char*   info_bound_axes(ChipsId* cid)
           { return ChipsRemote::info_bound_axes(cid); }
   char*   info_current()
           { return ChipsRemote::info_current(); }
   const char*   get_errors(int mode=0)
           { return ChipsRemote::get_errors(mode); }
   int     clear_errors()
           { return ChipsRemote::clear_errors(); }
   int     set_error_verbosity(int val)
           { return ChipsRemote::set_error_verbosity(val); }
   int     get_error_verbosity(int &out_val)
           { return ChipsRemote::get_error_verbosity(out_val); }
   int     get_error_count(int mode, int &out_val)
           {return ChipsRemote::get_error_count(mode, out_val);};
   int     get_error_types(int &out_val)
           { return ChipsRemote::get_error_types(out_val); }

#ifdef WLM_DO_THIS
//WLM   char*   get_current_frame(ChipsId* cid);
#endif

   // 'delete all' operations 
   int     delete_all_windows()
           { return ChipsRemote::delete_all_windows(NULL); }
   int     delete_all_windows(ChipsId* cid)
           { return ChipsRemote::delete_all_windows(cid); }
   int     delete_all_frames()
           { return ChipsRemote::delete_all_frames(NULL); }
   int     delete_all_frames(ChipsId* cid)
           { return ChipsRemote::delete_all_frames(cid); }
   int     delete_all_plots()
           { return ChipsRemote::delete_all_plots(NULL); }
   int     delete_all_plots(ChipsId* cid)
           { return ChipsRemote::delete_all_plots(cid); }
   int     delete_all_images()
           { return ChipsRemote::delete_all_images(NULL); }
   int     delete_all_images(ChipsId* cid)
           { return ChipsRemote::delete_all_images(cid); }

   int     convert_coordinate (double *in_val, double *out_val, int count,
                               ChipsId *cid1, ChipsId *cid2, int axis=XY_AXIS)
           {return ChipsRemote::convert_coordinate (in_val, out_val, count,
                                                     cid1, cid2, axis); };
   int     convert_coordinate (double in_val, double &out_val,
                               ChipsId *cid1, ChipsId *cid2, int axis)
           {return ChipsRemote::convert_coordinate (in_val, out_val,
                                                     cid1, cid2, axis);};



   bool    check_server_exists(const string& inServerId)
	   { return ChipsRemote::check_server_exists(inServerId); }
   int     lock()
	   { return ChipsRemote::lock(); }
   int     unlock()
	   { return ChipsRemote::unlock(); }
   int     connect(const string& inServerId, string myid="")
	   { return ChipsRemote::connect(inServerId, myid); }
   int     disconnect()
	   { return ChipsRemote::disconnect(); }
   int     create_server(string id="") 
           { return ChipsRemote::create_server(id); }
   int     create_server(bool use_prefs, string id="")
	   { return ChipsRemote::create_server(use_prefs, id); }
   int     get_server_cwd(string& out_val)
           { return ChipsRemote::get_server_cwd(out_val); }
   int     get_server_id(string& out_val)
	   { return ChipsRemote::get_server_id(out_val); }
   list <string> list_servers()
           { return ChipsRemote::list_servers(); } 
   int     show_gui(string opts="")
           { return ChipsRemote::show_gui(opts); }
   int     hide_gui()
           { return ChipsRemote::hide_gui(); }

};

#endif 
