% 
%  Copyright (C) 2008  Smithsonian Astrophysical Observatory
%
%
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License along
%  with this program; if not, write to the Free Software Foundation, Inc.,
%  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
%

require ("pysl");
py_import ("pychips");
private define _chips_push ( arg ) { return arg; }
%Check for return. Push tuples on the list individually
private define check_return ()
{
  variable ret_val=NULL;
  if (_stkdepth () > 0){
     ret_val = ();
     if (py_call(py.type,ret_val) == py.tuple){
        variable len = py_call (py.len (ret_val))-1;
        variable ii;
        _for (0, len, 1){
            ii = ();
           _chips_push (ret_val[ii]);
        }
     }
  }
  return ret_val;
}

%Create valid types for python
private define _chips_check_type (value);
private define _chips_check_type (value)
{
  if (typeof (value) == Char_Type)
     value = int (value);
  return value;
}
%Turn a structure into a list
private define _chips_struct_to_list (s);
private define _chips_struct_to_list (s)
{
  variable sl_list = {};
  variable name, value;
  variable prefix = NULL;
  if (_NARGS > 1)
     prefix = ();
  try{
     %Go through each element in the structure and make it a list
     foreach (get_struct_field_names (s)){
        name = ();
        value = get_struct_field (s, name);
        if (prefix != NULL)
           name = prefix + "." + name;
        %Expand out sub structures
        if (is_struct_type (value)){
           variable tmp_list = _chips_struct_to_list (name,value);
           variable ii;
           %Concatinate the two lists
           _for (length (tmp_list)-1, 0, -1){
              ii = ();
              list_insert (sl_list, tmp_list[ii], 0);
           }
        }
        else{
           value = _chips_check_type (value);
           list_insert (sl_list,value,0);
           list_insert (sl_list,name,0);
        }
     }
  }
  catch RunTimeError:
     %Return NULL if this was not a structure
     return NULL;
  return sl_list;
}
#if (_slang_version >= 20101)
%Return all the qualifiers in a struct
private define _chips_get_qualifiers (kwstruct);
private define _chips_get_qualifiers (kwstruct)
{
  if( kwstruct == NULL )
    return NULL;
  variable ii, name;
  variable kwnames = get_struct_field_names(kwstruct);
  variable kwargs = Struct_Type[2 * length(kwnames)];
  ii = 0;
  foreach (kwnames)
  {
     name = ();
     kwargs[ii] = struct{value=name};
     ii = ii + 1;
     kwargs[ii] = struct{value=get_struct_field(kwstruct, name)};
     ii = ii + 1;
   }
 return kwargs;
}
#endif
%Chips Functions
define clear ()
{
  if (_NARGS == 0)
     usage ("clear (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.clear, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.clear);
  }
}

define get_axis_dim ()
{
  if (_NARGS == 0)
     usage ("get_axis_dim (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_axis_dim, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_axis_dim);
  }
}

define get_axis_id ()
{
  if (_NARGS == 0)
     usage ("get_axis_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_axis_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_axis_id);
  }
}

define get_colorbar_id ()
{
  if (_NARGS == 0)
     usage ("get_colorbar_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_colorbar_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_colorbar_id);
  }
}

define get_contour_id ()
{
  if (_NARGS == 0)
     usage ("get_contour_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_contour_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_contour_id);
  }
}

define get_coord_id ()
{
  if (_NARGS == 0)
     usage ("get_coord_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_coord_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_coord_id);
  }
}

define get_coord_sys ()
{
  if (_NARGS == 0)
     usage ("get_coord_sys (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_coord_sys, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_coord_sys);
  }
}

define get_curve_id ()
{
  if (_NARGS == 0)
     usage ("get_curve_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_curve_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_curve_id);
  }
}

define get_frame_id ()
{
  if (_NARGS == 0)
     usage ("get_frame_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_frame_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_frame_id);
  }
}

define get_histogram_id ()
{
  if (_NARGS == 0)
     usage ("get_histogram_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_histogram_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_histogram_id);
  }
}

define get_image_id ()
{
  if (_NARGS == 0)
     usage ("get_image_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_image_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_image_id);
  }
}

define get_label_id ()
{
  if (_NARGS == 0)
     usage ("get_label_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_label_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_label_id);
  }
}

define get_line_id ()
{
  if (_NARGS == 0)
     usage ("get_line_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_line_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_line_id);
  }
}

define get_plot_id ()
{
  if (_NARGS == 0)
     usage ("get_plot_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_plot_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_plot_id);
  }
}

define get_point_id ()
{
  if (_NARGS == 0)
     usage ("get_point_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_point_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_point_id);
  }
}

define get_region_id ()
{
  if (_NARGS == 0)
     usage ("get_region_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_region_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_region_id);
  }
}

define get_window_id ()
{
  if (_NARGS == 0)
     usage ("get_window_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_window_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_window_id);
  }
}

define get_xaxis_id ()
{
  if (_NARGS == 0)
     usage ("get_xaxis_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_xaxis_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_xaxis_id);
  }
}

define get_yaxis_id ()
{
  if (_NARGS == 0)
     usage ("get_yaxis_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.get_yaxis_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.get_yaxis_id);
  }
}

define reset_frame_id ()
{
  if (_NARGS == 0)
     usage ("reset_frame_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.reset_frame_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.reset_frame_id);
  }
}

define reset_plot_id ()
{
  if (_NARGS == 0)
     usage ("reset_plot_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.reset_plot_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.reset_plot_id);
  }
}

define reset_window_id ()
{
  if (_NARGS == 0)
     usage ("reset_window_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.reset_window_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.reset_window_id);
  }
}

define set_all ()
{
  if (_NARGS == 0)
     usage ("set_all (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_all, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_all);
  }
}

define set_axis_dim ()
{
  if (_NARGS == 0)
     usage ("set_axis_dim (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_axis_dim, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_axis_dim);
  }
}

define set_axis_id ()
{
  if (_NARGS == 0)
     usage ("set_axis_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_axis_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_axis_id);
  }
}

define set_colorbar_id ()
{
  if (_NARGS == 0)
     usage ("set_colorbar_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_colorbar_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_colorbar_id);
  }
}

define set_contour_id ()
{
  if (_NARGS == 0)
     usage ("set_contour_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_contour_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_contour_id);
  }
}

define set_coord_id ()
{
  if (_NARGS == 0)
     usage ("set_coord_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_coord_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_coord_id);
  }
}

define set_coord_sys ()
{
  if (_NARGS == 0)
     usage ("set_coord_sys (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_coord_sys, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_coord_sys);
  }
}

define set_curve_id ()
{
  if (_NARGS == 0)
     usage ("set_curve_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_curve_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_curve_id);
  }
}

define set_frame_id ()
{
  if (_NARGS == 0)
     usage ("set_frame_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_frame_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_frame_id);
  }
}

define set_histogram_id ()
{
  if (_NARGS == 0)
     usage ("set_histogram_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_histogram_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_histogram_id);
  }
}

define set_image_id ()
{
  if (_NARGS == 0)
     usage ("set_image_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_image_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_image_id);
  }
}

define set_label_id ()
{
  if (_NARGS == 0)
     usage ("set_label_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_label_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_label_id);
  }
}

define set_line_id ()
{
  if (_NARGS == 0)
     usage ("set_line_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_line_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_line_id);
  }
}

define set_object_id ()
{
  if (_NARGS == 0)
     usage ("set_object_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_object_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_object_id);
  }
}

define set_plot_id ()
{
  if (_NARGS == 0)
     usage ("set_plot_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_plot_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_plot_id);
  }
}

define set_point_id ()
{
  if (_NARGS == 0)
     usage ("set_point_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_point_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_point_id);
  }
}

define set_region_id ()
{
  if (_NARGS == 0)
     usage ("set_region_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_region_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_region_id);
  }
}

define set_window_id ()
{
  if (_NARGS == 0)
     usage ("set_window_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_window_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_window_id);
  }
}

define set_xaxis_id ()
{
  if (_NARGS == 0)
     usage ("set_xaxis_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_xaxis_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_xaxis_id);
  }
}

define set_yaxis_id ()
{
  if (_NARGS == 0)
     usage ("set_yaxis_id (id,...)");
  else{
     variable f_args = __pop_args (_NARGS);
     if (_NARGS > 1)
        py_call (f_args[0].value.set_yaxis_id, __push_args (f_args[[1:length(f_args)-1]]));
     else
        py_call (f_args[0].value.set_yaxis_id);
  }
}

try
{ require ("chips_offscreen.sl"); }
catch:
{}
define AUTO () { return py.pychips.AUTO; }
define AUTO_ALIGN () { return py.pychips.AUTO_ALIGN; }
define BTN1_DOWN () { return py.pychips.BTN1_DOWN; }
define BTN1_UP () { return py.pychips.BTN1_UP; }
define BTN2_DOWN () { return py.pychips.BTN2_DOWN; }
define BTN2_UP () { return py.pychips.BTN2_UP; }
define BTN3_DOWN () { return py.pychips.BTN3_DOWN; }
define BTN3_UP () { return py.pychips.BTN3_UP; }
define BTNS_DOWN () { return py.pychips.BTNS_DOWN; }
define BTNS_UP () { return py.pychips.BTNS_UP; }
define CHIPS_PICK_EVT ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.CHIPS_PICK_EVT, __push_args (f_args));
        else
         py_call(py.pychips.CHIPS_PICK_EVT, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.CHIPS_PICK_EVT);
       else
         py_call(py.pychips.CHIPS_PICK_EVT, PY_KW,  __push_args(kwargs));
     }
}

define CHIPS_PICK_EVT_swigregister ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.CHIPS_PICK_EVT_swigregister, __push_args (f_args));
        else
         py_call(py.pychips.CHIPS_PICK_EVT_swigregister, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.CHIPS_PICK_EVT_swigregister);
       else
         py_call(py.pychips.CHIPS_PICK_EVT_swigregister, PY_KW,  __push_args(kwargs));
     }
}

define COORD_NONE () { return py.pychips.COORD_NONE; }
define ChipsAttributeObject ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsAttributeObject, __push_args (f_args));
        else
         py_call(py.pychips.ChipsAttributeObject, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsAttributeObject);
       else
         py_call(py.pychips.ChipsAttributeObject, PY_KW,  __push_args(kwargs));
     }
}

define ChipsAxis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsAxis, __push_args (f_args));
        else
         py_call(py.pychips.ChipsAxis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsAxis);
       else
         py_call(py.pychips.ChipsAxis, PY_KW,  __push_args(kwargs));
     }
}

define ChipsAxisLabel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsAxisLabel, __push_args (f_args));
        else
         py_call(py.pychips.ChipsAxisLabel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsAxisLabel);
       else
         py_call(py.pychips.ChipsAxisLabel, PY_KW,  __push_args(kwargs));
     }
}

define ChipsAxisOffset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsAxisOffset, __push_args (f_args));
        else
         py_call(py.pychips.ChipsAxisOffset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsAxisOffset);
       else
         py_call(py.pychips.ChipsAxisOffset, PY_KW,  __push_args(kwargs));
     }
}

define ChipsAxisTicklabel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsAxisTicklabel, __push_args (f_args));
        else
         py_call(py.pychips.ChipsAxisTicklabel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsAxisTicklabel);
       else
         py_call(py.pychips.ChipsAxisTicklabel, PY_KW,  __push_args(kwargs));
     }
}

define ChipsAxisXBase ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsAxisXBase, __push_args (f_args));
        else
         py_call(py.pychips.ChipsAxisXBase, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsAxisXBase);
       else
         py_call(py.pychips.ChipsAxisXBase, PY_KW,  __push_args(kwargs));
     }
}

define ChipsAxisYBase ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsAxisYBase, __push_args (f_args));
        else
         py_call(py.pychips.ChipsAxisYBase, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsAxisYBase);
       else
         py_call(py.pychips.ChipsAxisYBase, PY_KW,  __push_args(kwargs));
     }
}

define ChipsBackground ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsBackground, __push_args (f_args));
        else
         py_call(py.pychips.ChipsBackground, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsBackground);
       else
         py_call(py.pychips.ChipsBackground, PY_KW,  __push_args(kwargs));
     }
}

define ChipsBorder ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsBorder, __push_args (f_args));
        else
         py_call(py.pychips.ChipsBorder, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsBorder);
       else
         py_call(py.pychips.ChipsBorder, PY_KW,  __push_args(kwargs));
     }
}

define ChipsColorBar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsColorBar, __push_args (f_args));
        else
         py_call(py.pychips.ChipsColorBar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsColorBar);
       else
         py_call(py.pychips.ChipsColorBar, PY_KW,  __push_args(kwargs));
     }
}

define ChipsColorBarBorder ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarBorder, __push_args (f_args));
        else
         py_call(py.pychips.ChipsColorBarBorder, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarBorder);
       else
         py_call(py.pychips.ChipsColorBarBorder, PY_KW,  __push_args(kwargs));
     }
}

define ChipsColorBarLabel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarLabel, __push_args (f_args));
        else
         py_call(py.pychips.ChipsColorBarLabel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarLabel);
       else
         py_call(py.pychips.ChipsColorBarLabel, PY_KW,  __push_args(kwargs));
     }
}

define ChipsColorBarTick ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarTick, __push_args (f_args));
        else
         py_call(py.pychips.ChipsColorBarTick, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarTick);
       else
         py_call(py.pychips.ChipsColorBarTick, PY_KW,  __push_args(kwargs));
     }
}

define ChipsColorBarTicklabel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarTicklabel, __push_args (f_args));
        else
         py_call(py.pychips.ChipsColorBarTicklabel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsColorBarTicklabel);
       else
         py_call(py.pychips.ChipsColorBarTicklabel, PY_KW,  __push_args(kwargs));
     }
}

define ChipsContour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsContour, __push_args (f_args));
        else
         py_call(py.pychips.ChipsContour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsContour);
       else
         py_call(py.pychips.ChipsContour, PY_KW,  __push_args(kwargs));
     }
}

define ChipsCurve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsCurve, __push_args (f_args));
        else
         py_call(py.pychips.ChipsCurve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsCurve);
       else
         py_call(py.pychips.ChipsCurve, PY_KW,  __push_args(kwargs));
     }
}

define ChipsCurveLimit ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsCurveLimit, __push_args (f_args));
        else
         py_call(py.pychips.ChipsCurveLimit, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsCurveLimit);
       else
         py_call(py.pychips.ChipsCurveLimit, PY_KW,  __push_args(kwargs));
     }
}

define ChipsCurveLine ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsCurveLine, __push_args (f_args));
        else
         py_call(py.pychips.ChipsCurveLine, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsCurveLine);
       else
         py_call(py.pychips.ChipsCurveLine, PY_KW,  __push_args(kwargs));
     }
}

define ChipsDefaultPrefs ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsDefaultPrefs, __push_args (f_args));
        else
         py_call(py.pychips.ChipsDefaultPrefs, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsDefaultPrefs);
       else
         py_call(py.pychips.ChipsDefaultPrefs, PY_KW,  __push_args(kwargs));
     }
}

define ChipsErr ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsErr, __push_args (f_args));
        else
         py_call(py.pychips.ChipsErr, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsErr);
       else
         py_call(py.pychips.ChipsErr, PY_KW,  __push_args(kwargs));
     }
}

define ChipsExport ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsExport, __push_args (f_args));
        else
         py_call(py.pychips.ChipsExport, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsExport);
       else
         py_call(py.pychips.ChipsExport, PY_KW,  __push_args(kwargs));
     }
}

define ChipsForeground ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsForeground, __push_args (f_args));
        else
         py_call(py.pychips.ChipsForeground, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsForeground);
       else
         py_call(py.pychips.ChipsForeground, PY_KW,  __push_args(kwargs));
     }
}

define ChipsFrame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsFrame, __push_args (f_args));
        else
         py_call(py.pychips.ChipsFrame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsFrame);
       else
         py_call(py.pychips.ChipsFrame, PY_KW,  __push_args(kwargs));
     }
}

define ChipsHistogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsHistogram, __push_args (f_args));
        else
         py_call(py.pychips.ChipsHistogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsHistogram);
       else
         py_call(py.pychips.ChipsHistogram, PY_KW,  __push_args(kwargs));
     }
}

define ChipsHistogramErr ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsHistogramErr, __push_args (f_args));
        else
         py_call(py.pychips.ChipsHistogramErr, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsHistogramErr);
       else
         py_call(py.pychips.ChipsHistogramErr, PY_KW,  __push_args(kwargs));
     }
}

define ChipsHistogramFill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsHistogramFill, __push_args (f_args));
        else
         py_call(py.pychips.ChipsHistogramFill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsHistogramFill);
       else
         py_call(py.pychips.ChipsHistogramFill, PY_KW,  __push_args(kwargs));
     }
}

define ChipsId ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsId, __push_args (f_args));
        else
         py_call(py.pychips.ChipsId, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsId);
       else
         py_call(py.pychips.ChipsId, PY_KW,  __push_args(kwargs));
     }
}

define ChipsId_swigregister ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsId_swigregister, __push_args (f_args));
        else
         py_call(py.pychips.ChipsId_swigregister, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsId_swigregister);
       else
         py_call(py.pychips.ChipsId_swigregister, PY_KW,  __push_args(kwargs));
     }
}

define ChipsImage ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsImage, __push_args (f_args));
        else
         py_call(py.pychips.ChipsImage, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsImage);
       else
         py_call(py.pychips.ChipsImage, PY_KW,  __push_args(kwargs));
     }
}

define ChipsInfo ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsInfo, __push_args (f_args));
        else
         py_call(py.pychips.ChipsInfo, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsInfo);
       else
         py_call(py.pychips.ChipsInfo, PY_KW,  __push_args(kwargs));
     }
}

define ChipsLabel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsLabel, __push_args (f_args));
        else
         py_call(py.pychips.ChipsLabel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsLabel);
       else
         py_call(py.pychips.ChipsLabel, PY_KW,  __push_args(kwargs));
     }
}

define ChipsLine ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsLine, __push_args (f_args));
        else
         py_call(py.pychips.ChipsLine, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsLine);
       else
         py_call(py.pychips.ChipsLine, PY_KW,  __push_args(kwargs));
     }
}

define ChipsMajorGrid ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsMajorGrid, __push_args (f_args));
        else
         py_call(py.pychips.ChipsMajorGrid, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsMajorGrid);
       else
         py_call(py.pychips.ChipsMajorGrid, PY_KW,  __push_args(kwargs));
     }
}

define ChipsMajorTick ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsMajorTick, __push_args (f_args));
        else
         py_call(py.pychips.ChipsMajorTick, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsMajorTick);
       else
         py_call(py.pychips.ChipsMajorTick, PY_KW,  __push_args(kwargs));
     }
}

define ChipsMinorGrid ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsMinorGrid, __push_args (f_args));
        else
         py_call(py.pychips.ChipsMinorGrid, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsMinorGrid);
       else
         py_call(py.pychips.ChipsMinorGrid, PY_KW,  __push_args(kwargs));
     }
}

define ChipsMinorTick ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsMinorTick, __push_args (f_args));
        else
         py_call(py.pychips.ChipsMinorTick, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsMinorTick);
       else
         py_call(py.pychips.ChipsMinorTick, PY_KW,  __push_args(kwargs));
     }
}

define ChipsPick ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsPick, __push_args (f_args));
        else
         py_call(py.pychips.ChipsPick, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsPick);
       else
         py_call(py.pychips.ChipsPick, PY_KW,  __push_args(kwargs));
     }
}

define ChipsPlot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsPlot, __push_args (f_args));
        else
         py_call(py.pychips.ChipsPlot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsPlot);
       else
         py_call(py.pychips.ChipsPlot, PY_KW,  __push_args(kwargs));
     }
}

define ChipsPlotAspect ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsPlotAspect, __push_args (f_args));
        else
         py_call(py.pychips.ChipsPlotAspect, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsPlotAspect);
       else
         py_call(py.pychips.ChipsPlotAspect, PY_KW,  __push_args(kwargs));
     }
}

define ChipsPlotCorner ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsPlotCorner, __push_args (f_args));
        else
         py_call(py.pychips.ChipsPlotCorner, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsPlotCorner);
       else
         py_call(py.pychips.ChipsPlotCorner, PY_KW,  __push_args(kwargs));
     }
}

define ChipsPoint ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsPoint, __push_args (f_args));
        else
         py_call(py.pychips.ChipsPoint, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsPoint);
       else
         py_call(py.pychips.ChipsPoint, PY_KW,  __push_args(kwargs));
     }
}

define ChipsPreferences ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsPreferences, __push_args (f_args));
        else
         py_call(py.pychips.ChipsPreferences, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsPreferences);
       else
         py_call(py.pychips.ChipsPreferences, PY_KW,  __push_args(kwargs));
     }
}

define ChipsRegion ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsRegion, __push_args (f_args));
        else
         py_call(py.pychips.ChipsRegion, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsRegion);
       else
         py_call(py.pychips.ChipsRegion, PY_KW,  __push_args(kwargs));
     }
}

define ChipsRegionEdge ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsRegionEdge, __push_args (f_args));
        else
         py_call(py.pychips.ChipsRegionEdge, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsRegionEdge);
       else
         py_call(py.pychips.ChipsRegionEdge, PY_KW,  __push_args(kwargs));
     }
}

define ChipsRegionFill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsRegionFill, __push_args (f_args));
        else
         py_call(py.pychips.ChipsRegionFill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsRegionFill);
       else
         py_call(py.pychips.ChipsRegionFill, PY_KW,  __push_args(kwargs));
     }
}

define ChipsRemoteMod ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsRemoteMod, __push_args (f_args));
        else
         py_call(py.pychips.ChipsRemoteMod, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsRemoteMod);
       else
         py_call(py.pychips.ChipsRemoteMod, PY_KW,  __push_args(kwargs));
     }
}

define ChipsRemoteMod_swigregister ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsRemoteMod_swigregister, __push_args (f_args));
        else
         py_call(py.pychips.ChipsRemoteMod_swigregister, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsRemoteMod_swigregister);
       else
         py_call(py.pychips.ChipsRemoteMod_swigregister, PY_KW,  __push_args(kwargs));
     }
}

define ChipsSymbol ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsSymbol, __push_args (f_args));
        else
         py_call(py.pychips.ChipsSymbol, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsSymbol);
       else
         py_call(py.pychips.ChipsSymbol, PY_KW,  __push_args(kwargs));
     }
}

define ChipsTitle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsTitle, __push_args (f_args));
        else
         py_call(py.pychips.ChipsTitle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsTitle);
       else
         py_call(py.pychips.ChipsTitle, PY_KW,  __push_args(kwargs));
     }
}

define ChipsWildCard ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsWildCard, __push_args (f_args));
        else
         py_call(py.pychips.ChipsWildCard, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsWildCard);
       else
         py_call(py.pychips.ChipsWildCard, PY_KW,  __push_args(kwargs));
     }
}

define ChipsWindow ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.ChipsWindow, __push_args (f_args));
        else
         py_call(py.pychips.ChipsWindow, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.ChipsWindow);
       else
         py_call(py.pychips.ChipsWindow, PY_KW,  __push_args(kwargs));
     }
}

define DATA () { return py.pychips.DATA; }
define DEC_AXIS () { return py.pychips.DEC_AXIS; }
define FRAME_NORM () { return py.pychips.FRAME_NORM; }
define KEY_PRESS () { return py.pychips.KEY_PRESS; }
define KEY_RELEASE () { return py.pychips.KEY_RELEASE; }
define PIXEL () { return py.pychips.PIXEL; }
define PIXEL_REAL () { return py.pychips.PIXEL_REAL; }
define PLOT_NORM () { return py.pychips.PLOT_NORM; }
define RADEC_AXIS () { return py.pychips.RADEC_AXIS; }
define RA_AXIS () { return py.pychips.RA_AXIS; }
define V_FRAMEMODE_PLACED () { return py.pychips.V_FRAMEMODE_PLACED; }
define V_FRAMEMODE_SINGLE () { return py.pychips.V_FRAMEMODE_SINGLE; }
define V_FRAMEMODE_TILED () { return py.pychips.V_FRAMEMODE_TILED; }
define V_LOAD_ALL () { return py.pychips.V_LOAD_ALL; }
define V_LOAD_COMMANDS () { return py.pychips.V_LOAD_COMMANDS; }
define V_LOAD_STATE () { return py.pychips.V_LOAD_STATE; }
define V_MOVE_ABSOLUTE () { return py.pychips.V_MOVE_ABSOLUTE; }
define V_MOVE_DELTA () { return py.pychips.V_MOVE_DELTA; }
define V_SCALE_LINEAR () { return py.pychips.V_SCALE_LINEAR; }
define V_SCALE_LOG () { return py.pychips.V_SCALE_LOG; }
define V_USE_DEFAULT_DEPTH () { return py.pychips.V_USE_DEFAULT_DEPTH; }
define WINDOW_NORM () { return py.pychips.WINDOW_NORM; }
define XY_AXIS () { return py.pychips.XY_AXIS; }
define X_AXIS () { return py.pychips.X_AXIS; }
define Y_AXIS () { return py.pychips.Y_AXIS; }
define add_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_axis, __push_args (f_args));
        else
         py_call(py.pychips.add_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_axis);
       else
         py_call(py.pychips.add_axis, PY_KW,  __push_args(kwargs));
     }
}

define add_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.add_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_colorbar);
       else
         py_call(py.pychips.add_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define add_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_contour, __push_args (f_args));
        else
         py_call(py.pychips.add_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_contour);
       else
         py_call(py.pychips.add_contour, PY_KW,  __push_args(kwargs));
     }
}

define add_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_curve, __push_args (f_args));
        else
         py_call(py.pychips.add_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_curve);
       else
         py_call(py.pychips.add_curve, PY_KW,  __push_args(kwargs));
     }
}

define add_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_frame, __push_args (f_args));
        else
         py_call(py.pychips.add_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_frame);
       else
         py_call(py.pychips.add_frame, PY_KW,  __push_args(kwargs));
     }
}

define add_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_histogram, __push_args (f_args));
        else
         py_call(py.pychips.add_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_histogram);
       else
         py_call(py.pychips.add_histogram, PY_KW,  __push_args(kwargs));
     }
}

define add_hline ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_hline, __push_args (f_args));
        else
         py_call(py.pychips.add_hline, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_hline);
       else
         py_call(py.pychips.add_hline, PY_KW,  __push_args(kwargs));
     }
}

define add_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_image, __push_args (f_args));
        else
         py_call(py.pychips.add_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_image);
       else
         py_call(py.pychips.add_image, PY_KW,  __push_args(kwargs));
     }
}

define add_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_label, __push_args (f_args));
        else
         py_call(py.pychips.add_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_label);
       else
         py_call(py.pychips.add_label, PY_KW,  __push_args(kwargs));
     }
}

define add_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_line, __push_args (f_args));
        else
         py_call(py.pychips.add_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_line);
       else
         py_call(py.pychips.add_line, PY_KW,  __push_args(kwargs));
     }
}

define add_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_plot, __push_args (f_args));
        else
         py_call(py.pychips.add_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_plot);
       else
         py_call(py.pychips.add_plot, PY_KW,  __push_args(kwargs));
     }
}

define add_plotarea ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_plotarea, __push_args (f_args));
        else
         py_call(py.pychips.add_plotarea, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_plotarea);
       else
         py_call(py.pychips.add_plotarea, PY_KW,  __push_args(kwargs));
     }
}

define add_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_point, __push_args (f_args));
        else
         py_call(py.pychips.add_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_point);
       else
         py_call(py.pychips.add_point, PY_KW,  __push_args(kwargs));
     }
}

define add_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_region, __push_args (f_args));
        else
         py_call(py.pychips.add_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_region);
       else
         py_call(py.pychips.add_region, PY_KW,  __push_args(kwargs));
     }
}

define add_vline ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_vline, __push_args (f_args));
        else
         py_call(py.pychips.add_vline, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_vline);
       else
         py_call(py.pychips.add_vline, PY_KW,  __push_args(kwargs));
     }
}

define add_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.add_window, __push_args (f_args));
        else
         py_call(py.pychips.add_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.add_window);
       else
         py_call(py.pychips.add_window, PY_KW,  __push_args(kwargs));
     }
}

define adjust_grid_gaps ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.adjust_grid_gaps, __push_args (f_args));
        else
         py_call(py.pychips.adjust_grid_gaps, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.adjust_grid_gaps);
       else
         py_call(py.pychips.adjust_grid_gaps, PY_KW,  __push_args(kwargs));
     }
}

define adjust_grid_xgap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.adjust_grid_xgap, __push_args (f_args));
        else
         py_call(py.pychips.adjust_grid_xgap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.adjust_grid_xgap);
       else
         py_call(py.pychips.adjust_grid_xgap, PY_KW,  __push_args(kwargs));
     }
}

define adjust_grid_xrelsize ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.adjust_grid_xrelsize, __push_args (f_args));
        else
         py_call(py.pychips.adjust_grid_xrelsize, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.adjust_grid_xrelsize);
       else
         py_call(py.pychips.adjust_grid_xrelsize, PY_KW,  __push_args(kwargs));
     }
}

define adjust_grid_xrelsizes ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.adjust_grid_xrelsizes, __push_args (f_args));
        else
         py_call(py.pychips.adjust_grid_xrelsizes, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.adjust_grid_xrelsizes);
       else
         py_call(py.pychips.adjust_grid_xrelsizes, PY_KW,  __push_args(kwargs));
     }
}

define adjust_grid_ygap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.adjust_grid_ygap, __push_args (f_args));
        else
         py_call(py.pychips.adjust_grid_ygap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.adjust_grid_ygap);
       else
         py_call(py.pychips.adjust_grid_ygap, PY_KW,  __push_args(kwargs));
     }
}

define adjust_grid_yrelsize ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.adjust_grid_yrelsize, __push_args (f_args));
        else
         py_call(py.pychips.adjust_grid_yrelsize, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.adjust_grid_yrelsize);
       else
         py_call(py.pychips.adjust_grid_yrelsize, PY_KW,  __push_args(kwargs));
     }
}

define adjust_grid_yrelsizes ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.adjust_grid_yrelsizes, __push_args (f_args));
        else
         py_call(py.pychips.adjust_grid_yrelsizes, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.adjust_grid_yrelsizes);
       else
         py_call(py.pychips.adjust_grid_yrelsizes, PY_KW,  __push_args(kwargs));
     }
}

define arrange_frames ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.arrange_frames, __push_args (f_args));
        else
         py_call(py.pychips.arrange_frames, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.arrange_frames);
       else
         py_call(py.pychips.arrange_frames, PY_KW,  __push_args(kwargs));
     }
}

define bind_axes ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.bind_axes, __push_args (f_args));
        else
         py_call(py.pychips.bind_axes, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.bind_axes);
       else
         py_call(py.pychips.bind_axes, PY_KW,  __push_args(kwargs));
     }
}

define check_server_exists ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.check_server_exists, __push_args (f_args));
        else
         py_call(py.pychips.check_server_exists, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.check_server_exists);
       else
         py_call(py.pychips.check_server_exists, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - check_server_exists (server_id)", err.object;
   };
}

define chips_alpha () { return py.pychips.chips_alpha; }
define chips_arbitrary () { return py.pychips.chips_arbitrary; }
define chips_arrow () { return py.pychips.chips_arrow; }
define chips_axis () { return py.pychips.chips_axis; }
define chips_back () { return py.pychips.chips_back; }
define chips_backward () { return py.pychips.chips_backward; }
define chips_bar () { return py.pychips.chips_bar; }
define chips_bevel () { return py.pychips.chips_bevel; }
define chips_bicubic () { return py.pychips.chips_bicubic; }
define chips_bilinear () { return py.pychips.chips_bilinear; }
define chips_blue () { return py.pychips.chips_blue; }
define chips_bold () { return py.pychips.chips_bold; }
define chips_bolditalic () { return py.pychips.chips_bolditalic; }
define chips_box () { return py.pychips.chips_box; }
define chips_brick () { return py.pychips.chips_brick; }
define chips_cap () { return py.pychips.chips_cap; }
define chips_centered () { return py.pychips.chips_centered; }
define chips_circle () { return py.pychips.chips_circle; }
define chips_closed () { return py.pychips.chips_closed; }
define chips_cm () { return py.pychips.chips_cm; }
define chips_cmyk () { return py.pychips.chips_cmyk; }
define chips_cmyksep () { return py.pychips.chips_cmyksep; }
define chips_colorbar () { return py.pychips.chips_colorbar; }
define chips_contour () { return py.pychips.chips_contour; }
define chips_cool () { return py.pychips.chips_cool; }
define chips_coordinate_system () { return py.pychips.chips_coordinate_system; }
define chips_count () { return py.pychips.chips_count; }
define chips_courier () { return py.pychips.chips_courier; }
define chips_crisscross () { return py.pychips.chips_crisscross; }
define chips_cross () { return py.pychips.chips_cross; }
define chips_curve () { return py.pychips.chips_curve; }
define chips_diamond () { return py.pychips.chips_diamond; }
define chips_dot () { return py.pychips.chips_dot; }
define chips_dotlongdash () { return py.pychips.chips_dotlongdash; }
define chips_dotshortdash () { return py.pychips.chips_dotshortdash; }
define chips_downdiagonal () { return py.pychips.chips_downdiagonal; }
define chips_downtriangle () { return py.pychips.chips_downtriangle; }
define chips_extend_both () { return py.pychips.chips_extend_both; }
define chips_extend_end () { return py.pychips.chips_extend_end; }
define chips_extend_none () { return py.pychips.chips_extend_none; }
define chips_extend_start () { return py.pychips.chips_extend_start; }
define chips_fill () { return py.pychips.chips_fill; }
define chips_flower () { return py.pychips.chips_flower; }
define chips_forward () { return py.pychips.chips_forward; }
define chips_frame () { return py.pychips.chips_frame; }
define chips_front () { return py.pychips.chips_front; }
define chips_grayscale () { return py.pychips.chips_grayscale; }
define chips_greek () { return py.pychips.chips_greek; }
define chips_green () { return py.pychips.chips_green; }
define chips_grid () { return py.pychips.chips_grid; }
define chips_heat () { return py.pychips.chips_heat; }
define chips_helvetica () { return py.pychips.chips_helvetica; }
define chips_hexagon () { return py.pychips.chips_hexagon; }
define chips_histogram () { return py.pychips.chips_histogram; }
define chips_horizontal () { return py.pychips.chips_horizontal; }
define chips_hsv () { return py.pychips.chips_hsv; }
define chips_ignore () { return py.pychips.chips_ignore; }
define chips_image () { return py.pychips.chips_image; }
define chips_inch () { return py.pychips.chips_inch; }
define chips_inside () { return py.pychips.chips_inside; }
define chips_interval () { return py.pychips.chips_interval; }
define chips_italic () { return py.pychips.chips_italic; }
define chips_label () { return py.pychips.chips_label; }
define chips_legend () { return py.pychips.chips_legend; }
define chips_limits () { return py.pychips.chips_limits; }
define chips_line () { return py.pychips.chips_line; }
define chips_longdash () { return py.pychips.chips_longdash; }
define chips_marching () { return py.pychips.chips_marching; }
define chips_miter () { return py.pychips.chips_miter; }
define chips_mm () { return py.pychips.chips_mm; }
define chips_neighbor () { return py.pychips.chips_neighbor; }
define chips_nice () { return py.pychips.chips_nice; }
define chips_nofiletype () { return py.pychips.chips_nofiletype; }
define chips_nofill () { return py.pychips.chips_nofill; }
define chips_noline () { return py.pychips.chips_noline; }
define chips_none () { return py.pychips.chips_none; }
define chips_normal () { return py.pychips.chips_normal; }
define chips_not_object () { return py.pychips.chips_not_object; }
define chips_notch () { return py.pychips.chips_notch; }
define chips_nounit () { return py.pychips.chips_nounit; }
define chips_object () { return py.pychips.chips_object; }
define chips_omit () { return py.pychips.chips_omit; }
define chips_open () { return py.pychips.chips_open; }
define chips_outside () { return py.pychips.chips_outside; }
define chips_pixels () { return py.pychips.chips_pixels; }
define chips_plot () { return py.pychips.chips_plot; }
define chips_plus () { return py.pychips.chips_plus; }
define chips_point () { return py.pychips.chips_point; }
define chips_point_type () { return py.pychips.chips_point_type; }
define chips_polkadot () { return py.pychips.chips_polkadot; }
define chips_rainbow () { return py.pychips.chips_rainbow; }
define chips_red () { return py.pychips.chips_red; }
define chips_region () { return py.pychips.chips_region; }
define chips_rgb () { return py.pychips.chips_rgb; }
define chips_shortdash () { return py.pychips.chips_shortdash; }
define chips_shortdashlongdash () { return py.pychips.chips_shortdashlongdash; }
define chips_solid () { return py.pychips.chips_solid; }
define chips_solidfill () { return py.pychips.chips_solidfill; }
define chips_square () { return py.pychips.chips_square; }
define chips_standard () { return py.pychips.chips_standard; }
define chips_times () { return py.pychips.chips_times; }
define chips_updiagonal () { return py.pychips.chips_updiagonal; }
define chips_uptriangle () { return py.pychips.chips_uptriangle; }
define chips_usercmap1 () { return py.pychips.chips_usercmap1; }
define chips_usercmap2 () { return py.pychips.chips_usercmap2; }
define chips_usercmap3 () { return py.pychips.chips_usercmap3; }
define chips_userfill1 () { return py.pychips.chips_userfill1; }
define chips_userfill2 () { return py.pychips.chips_userfill2; }
define chips_userfill3 () { return py.pychips.chips_userfill3; }
define chips_vertical () { return py.pychips.chips_vertical; }
define chips_wave () { return py.pychips.chips_wave; }
define chips_window () { return py.pychips.chips_window; }
define chips_zigzag () { return py.pychips.chips_zigzag; }
define clear ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.clear, __push_args (f_args));
        else
         py_call(py.pychips.clear, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.clear);
       else
         py_call(py.pychips.clear, PY_KW,  __push_args(kwargs));
     }
}

define clear_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.clear_plot, __push_args (f_args));
        else
         py_call(py.pychips.clear_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.clear_plot);
       else
         py_call(py.pychips.clear_plot, PY_KW,  __push_args(kwargs));
     }
}

define clear_undo_tag ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.clear_undo_tag, __push_args (f_args));
        else
         py_call(py.pychips.clear_undo_tag, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.clear_undo_tag);
       else
         py_call(py.pychips.clear_undo_tag, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - clear_undo_tag (tagname)", err.object;
   };
}

define col_grid_objects ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.col_grid_objects, __push_args (f_args));
        else
         py_call(py.pychips.col_grid_objects, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.col_grid_objects);
       else
         py_call(py.pychips.col_grid_objects, PY_KW,  __push_args(kwargs));
     }
}

define col_tile ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.col_tile, __push_args (f_args));
        else
         py_call(py.pychips.col_tile, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.col_tile);
       else
         py_call(py.pychips.col_tile, PY_KW,  __push_args(kwargs));
     }
}

define collapse_depths ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.collapse_depths, __push_args (f_args));
        else
         py_call(py.pychips.collapse_depths, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.collapse_depths);
       else
         py_call(py.pychips.collapse_depths, PY_KW,  __push_args(kwargs));
     }
}

define connect ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.connect, __push_args (f_args));
        else
         py_call(py.pychips.connect, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.connect);
       else
         py_call(py.pychips.connect, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - connect (server_id)", err.object;
   };
}

define convert_coordinate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.convert_coordinate, __push_args (f_args));
        else
         py_call(py.pychips.convert_coordinate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.convert_coordinate);
       else
         py_call(py.pychips.convert_coordinate, PY_KW,  __push_args(kwargs));
     }
}

define create_server ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.create_server, __push_args (f_args));
        else
         py_call(py.pychips.create_server, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.create_server);
       else
         py_call(py.pychips.create_server, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - create_server (loadprefs)", err.object;
   };
}

define current_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_axis, __push_args (f_args));
        else
         py_call(py.pychips.current_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_axis);
       else
         py_call(py.pychips.current_axis, PY_KW,  __push_args(kwargs));
     }
}

define current_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.current_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_colorbar);
       else
         py_call(py.pychips.current_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define current_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_contour, __push_args (f_args));
        else
         py_call(py.pychips.current_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_contour);
       else
         py_call(py.pychips.current_contour, PY_KW,  __push_args(kwargs));
     }
}

define current_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_curve, __push_args (f_args));
        else
         py_call(py.pychips.current_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_curve);
       else
         py_call(py.pychips.current_curve, PY_KW,  __push_args(kwargs));
     }
}

define current_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_frame, __push_args (f_args));
        else
         py_call(py.pychips.current_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_frame);
       else
         py_call(py.pychips.current_frame, PY_KW,  __push_args(kwargs));
     }
}

define current_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_histogram, __push_args (f_args));
        else
         py_call(py.pychips.current_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_histogram);
       else
         py_call(py.pychips.current_histogram, PY_KW,  __push_args(kwargs));
     }
}

define current_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_image, __push_args (f_args));
        else
         py_call(py.pychips.current_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_image);
       else
         py_call(py.pychips.current_image, PY_KW,  __push_args(kwargs));
     }
}

define current_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_label, __push_args (f_args));
        else
         py_call(py.pychips.current_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_label);
       else
         py_call(py.pychips.current_label, PY_KW,  __push_args(kwargs));
     }
}

define current_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_line, __push_args (f_args));
        else
         py_call(py.pychips.current_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_line);
       else
         py_call(py.pychips.current_line, PY_KW,  __push_args(kwargs));
     }
}

define current_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_plot, __push_args (f_args));
        else
         py_call(py.pychips.current_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_plot);
       else
         py_call(py.pychips.current_plot, PY_KW,  __push_args(kwargs));
     }
}

define current_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_point, __push_args (f_args));
        else
         py_call(py.pychips.current_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_point);
       else
         py_call(py.pychips.current_point, PY_KW,  __push_args(kwargs));
     }
}

define current_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_region, __push_args (f_args));
        else
         py_call(py.pychips.current_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_region);
       else
         py_call(py.pychips.current_region, PY_KW,  __push_args(kwargs));
     }
}

define current_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.current_window, __push_args (f_args));
        else
         py_call(py.pychips.current_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.current_window);
       else
         py_call(py.pychips.current_window, PY_KW,  __push_args(kwargs));
     }
}

define deiconify_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.deiconify_window, __push_args (f_args));
        else
         py_call(py.pychips.deiconify_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.deiconify_window);
       else
         py_call(py.pychips.deiconify_window, PY_KW,  __push_args(kwargs));
     }
}

define delete_all_frames ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_all_frames, __push_args (f_args));
        else
         py_call(py.pychips.delete_all_frames, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_all_frames);
       else
         py_call(py.pychips.delete_all_frames, PY_KW,  __push_args(kwargs));
     }
}

define delete_all_plots ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_all_plots, __push_args (f_args));
        else
         py_call(py.pychips.delete_all_plots, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_all_plots);
       else
         py_call(py.pychips.delete_all_plots, PY_KW,  __push_args(kwargs));
     }
}

define delete_all_windows ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_all_windows, __push_args (f_args));
        else
         py_call(py.pychips.delete_all_windows, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_all_windows);
       else
         py_call(py.pychips.delete_all_windows, PY_KW,  __push_args(kwargs));
     }
}

define delete_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_axis, __push_args (f_args));
        else
         py_call(py.pychips.delete_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_axis);
       else
         py_call(py.pychips.delete_axis, PY_KW,  __push_args(kwargs));
     }
}

define delete_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.delete_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_colorbar);
       else
         py_call(py.pychips.delete_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define delete_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_contour, __push_args (f_args));
        else
         py_call(py.pychips.delete_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_contour);
       else
         py_call(py.pychips.delete_contour, PY_KW,  __push_args(kwargs));
     }
}

define delete_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_curve, __push_args (f_args));
        else
         py_call(py.pychips.delete_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_curve);
       else
         py_call(py.pychips.delete_curve, PY_KW,  __push_args(kwargs));
     }
}

define delete_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_frame, __push_args (f_args));
        else
         py_call(py.pychips.delete_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_frame);
       else
         py_call(py.pychips.delete_frame, PY_KW,  __push_args(kwargs));
     }
}

define delete_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_histogram, __push_args (f_args));
        else
         py_call(py.pychips.delete_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_histogram);
       else
         py_call(py.pychips.delete_histogram, PY_KW,  __push_args(kwargs));
     }
}

define delete_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_image, __push_args (f_args));
        else
         py_call(py.pychips.delete_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_image);
       else
         py_call(py.pychips.delete_image, PY_KW,  __push_args(kwargs));
     }
}

define delete_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_label, __push_args (f_args));
        else
         py_call(py.pychips.delete_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_label);
       else
         py_call(py.pychips.delete_label, PY_KW,  __push_args(kwargs));
     }
}

define delete_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_line, __push_args (f_args));
        else
         py_call(py.pychips.delete_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_line);
       else
         py_call(py.pychips.delete_line, PY_KW,  __push_args(kwargs));
     }
}

define delete_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_plot, __push_args (f_args));
        else
         py_call(py.pychips.delete_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_plot);
       else
         py_call(py.pychips.delete_plot, PY_KW,  __push_args(kwargs));
     }
}

define delete_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_point, __push_args (f_args));
        else
         py_call(py.pychips.delete_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_point);
       else
         py_call(py.pychips.delete_point, PY_KW,  __push_args(kwargs));
     }
}

define delete_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_region, __push_args (f_args));
        else
         py_call(py.pychips.delete_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_region);
       else
         py_call(py.pychips.delete_region, PY_KW,  __push_args(kwargs));
     }
}

define delete_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.delete_window, __push_args (f_args));
        else
         py_call(py.pychips.delete_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.delete_window);
       else
         py_call(py.pychips.delete_window, PY_KW,  __push_args(kwargs));
     }
}

define disconnect ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.disconnect, __push_args (f_args));
        else
         py_call(py.pychips.disconnect, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.disconnect);
       else
         py_call(py.pychips.disconnect, PY_KW,  __push_args(kwargs));
     }
}

define display_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_axis, __push_args (f_args));
        else
         py_call(py.pychips.display_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_axis);
       else
         py_call(py.pychips.display_axis, PY_KW,  __push_args(kwargs));
     }
}

define display_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.display_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_colorbar);
       else
         py_call(py.pychips.display_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define display_colorbar_border ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_colorbar_border, __push_args (f_args));
        else
         py_call(py.pychips.display_colorbar_border, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_colorbar_border);
       else
         py_call(py.pychips.display_colorbar_border, PY_KW,  __push_args(kwargs));
     }
}

define display_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_contour, __push_args (f_args));
        else
         py_call(py.pychips.display_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_contour);
       else
         py_call(py.pychips.display_contour, PY_KW,  __push_args(kwargs));
     }
}

define display_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_curve, __push_args (f_args));
        else
         py_call(py.pychips.display_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_curve);
       else
         py_call(py.pychips.display_curve, PY_KW,  __push_args(kwargs));
     }
}

define display_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_depth, __push_args (f_args));
        else
         py_call(py.pychips.display_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_depth);
       else
         py_call(py.pychips.display_depth, PY_KW,  __push_args(kwargs));
     }
}

define display_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_frame, __push_args (f_args));
        else
         py_call(py.pychips.display_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_frame);
       else
         py_call(py.pychips.display_frame, PY_KW,  __push_args(kwargs));
     }
}

define display_frame_border ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_frame_border, __push_args (f_args));
        else
         py_call(py.pychips.display_frame_border, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_frame_border);
       else
         py_call(py.pychips.display_frame_border, PY_KW,  __push_args(kwargs));
     }
}

define display_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_histogram, __push_args (f_args));
        else
         py_call(py.pychips.display_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_histogram);
       else
         py_call(py.pychips.display_histogram, PY_KW,  __push_args(kwargs));
     }
}

define display_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_image, __push_args (f_args));
        else
         py_call(py.pychips.display_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_image);
       else
         py_call(py.pychips.display_image, PY_KW,  __push_args(kwargs));
     }
}

define display_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_label, __push_args (f_args));
        else
         py_call(py.pychips.display_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_label);
       else
         py_call(py.pychips.display_label, PY_KW,  __push_args(kwargs));
     }
}

define display_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_line, __push_args (f_args));
        else
         py_call(py.pychips.display_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_line);
       else
         py_call(py.pychips.display_line, PY_KW,  __push_args(kwargs));
     }
}

define display_major_ticks ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_major_ticks, __push_args (f_args));
        else
         py_call(py.pychips.display_major_ticks, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_major_ticks);
       else
         py_call(py.pychips.display_major_ticks, PY_KW,  __push_args(kwargs));
     }
}

define display_minor_ticks ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_minor_ticks, __push_args (f_args));
        else
         py_call(py.pychips.display_minor_ticks, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_minor_ticks);
       else
         py_call(py.pychips.display_minor_ticks, PY_KW,  __push_args(kwargs));
     }
}

define display_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_plot, __push_args (f_args));
        else
         py_call(py.pychips.display_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_plot);
       else
         py_call(py.pychips.display_plot, PY_KW,  __push_args(kwargs));
     }
}

define display_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_point, __push_args (f_args));
        else
         py_call(py.pychips.display_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_point);
       else
         py_call(py.pychips.display_point, PY_KW,  __push_args(kwargs));
     }
}

define display_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.display_region, __push_args (f_args));
        else
         py_call(py.pychips.display_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.display_region);
       else
         py_call(py.pychips.display_region, PY_KW,  __push_args(kwargs));
     }
}

define erase ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.erase, __push_args (f_args));
        else
         py_call(py.pychips.erase, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.erase);
       else
         py_call(py.pychips.erase, PY_KW,  __push_args(kwargs));
     }
}

define get_attribute ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_attribute, __push_args (f_args));
        else
         _py_call(py.pychips.get_attribute, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_attribute);
       else
         _py_call(py.pychips.get_attribute, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_axis, __push_args (f_args));
        else
         _py_call(py.pychips.get_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_axis);
       else
         _py_call(py.pychips.get_axis, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_range ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_axis_range, __push_args (f_args));
        else
         _py_call(py.pychips.get_axis_range, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_axis_range);
       else
         _py_call(py.pychips.get_axis_range, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_scale ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_axis_scale, __push_args (f_args));
        else
         _py_call(py.pychips.get_axis_scale, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_axis_scale);
       else
         _py_call(py.pychips.get_axis_scale, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_text ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_axis_text, __push_args (f_args));
        else
         _py_call(py.pychips.get_axis_text, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_axis_text);
       else
         _py_call(py.pychips.get_axis_text, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_axis_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_axis_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_axis_visible);
       else
         _py_call(py.pychips.get_axis_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_colorbar, __push_args (f_args));
        else
         _py_call(py.pychips.get_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_colorbar);
       else
         _py_call(py.pychips.get_colorbar, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_border_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_colorbar_border_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_colorbar_border_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_colorbar_border_visible);
       else
         _py_call(py.pychips.get_colorbar_border_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_colorbar_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_colorbar_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_colorbar_visible);
       else
         _py_call(py.pychips.get_colorbar_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_contour, __push_args (f_args));
        else
         _py_call(py.pychips.get_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_contour);
       else
         _py_call(py.pychips.get_contour, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_levels ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_contour_levels, __push_args (f_args));
        else
         _py_call(py.pychips.get_contour_levels, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_contour_levels);
       else
         _py_call(py.pychips.get_contour_levels, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_range ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_contour_range, __push_args (f_args));
        else
         _py_call(py.pychips.get_contour_range, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_contour_range);
       else
         _py_call(py.pychips.get_contour_range, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_contour_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_contour_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_contour_visible);
       else
         _py_call(py.pychips.get_contour_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_xrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_contour_xrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_contour_xrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_contour_xrange);
       else
         _py_call(py.pychips.get_contour_xrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_yrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_contour_yrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_contour_yrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_contour_yrange);
       else
         _py_call(py.pychips.get_contour_yrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_zrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_contour_zrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_contour_zrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_contour_zrange);
       else
         _py_call(py.pychips.get_contour_zrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_curve, __push_args (f_args));
        else
         _py_call(py.pychips.get_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_curve);
       else
         _py_call(py.pychips.get_curve, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_range ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_curve_range, __push_args (f_args));
        else
         _py_call(py.pychips.get_curve_range, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_curve_range);
       else
         _py_call(py.pychips.get_curve_range, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_curve_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_curve_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_curve_visible);
       else
         _py_call(py.pychips.get_curve_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_xrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_curve_xrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_curve_xrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_curve_xrange);
       else
         _py_call(py.pychips.get_curve_xrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_yrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_curve_yrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_curve_yrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_curve_yrange);
       else
         _py_call(py.pychips.get_curve_yrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_data_aspect_ratio ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_data_aspect_ratio, __push_args (f_args));
        else
         _py_call(py.pychips.get_data_aspect_ratio, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_data_aspect_ratio);
       else
         _py_call(py.pychips.get_data_aspect_ratio, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_default_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_default_depth, __push_args (f_args));
        else
         _py_call(py.pychips.get_default_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_default_depth);
       else
         _py_call(py.pychips.get_default_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_frame, __push_args (f_args));
        else
         _py_call(py.pychips.get_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_frame);
       else
         _py_call(py.pychips.get_frame, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_frame_border_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_frame_border_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_frame_border_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_frame_border_visible);
       else
         _py_call(py.pychips.get_frame_border_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_frame_scale ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_frame_scale, __push_args (f_args));
        else
         _py_call(py.pychips.get_frame_scale, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_frame_scale);
       else
         _py_call(py.pychips.get_frame_scale, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_frame_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_frame_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_frame_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_frame_visible);
       else
         _py_call(py.pychips.get_frame_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_histogram, __push_args (f_args));
        else
         _py_call(py.pychips.get_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_histogram);
       else
         _py_call(py.pychips.get_histogram, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_range ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_histogram_range, __push_args (f_args));
        else
         _py_call(py.pychips.get_histogram_range, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_histogram_range);
       else
         _py_call(py.pychips.get_histogram_range, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_xrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_histogram_xrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_histogram_xrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_histogram_xrange);
       else
         _py_call(py.pychips.get_histogram_xrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_yrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_histogram_yrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_histogram_yrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_histogram_yrange);
       else
         _py_call(py.pychips.get_histogram_yrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_image, __push_args (f_args));
        else
         _py_call(py.pychips.get_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_image);
       else
         _py_call(py.pychips.get_image, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_range ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_image_range, __push_args (f_args));
        else
         _py_call(py.pychips.get_image_range, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_image_range);
       else
         _py_call(py.pychips.get_image_range, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_image_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_image_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_image_visible);
       else
         _py_call(py.pychips.get_image_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_xrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_image_xrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_image_xrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_image_xrange);
       else
         _py_call(py.pychips.get_image_xrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_yrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_image_yrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_image_yrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_image_yrange);
       else
         _py_call(py.pychips.get_image_yrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_label, __push_args (f_args));
        else
         _py_call(py.pychips.get_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_label);
       else
         _py_call(py.pychips.get_label, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_text ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_label_text, __push_args (f_args));
        else
         _py_call(py.pychips.get_label_text, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_label_text);
       else
         _py_call(py.pychips.get_label_text, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_line, __push_args (f_args));
        else
         _py_call(py.pychips.get_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_line);
       else
         _py_call(py.pychips.get_line, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_object_coordinfo ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_object_coordinfo, __push_args (f_args));
        else
         _py_call(py.pychips.get_object_coordinfo, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_object_coordinfo);
       else
         _py_call(py.pychips.get_object_coordinfo, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_object_count ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_object_count, __push_args (f_args));
        else
         _py_call(py.pychips.get_object_count, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_object_count);
       else
         _py_call(py.pychips.get_object_count, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_pick ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_pick, __push_args (f_args));
        else
         _py_call(py.pychips.get_pick, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_pick);
       else
         _py_call(py.pychips.get_pick, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot);
       else
         _py_call(py.pychips.get_plot, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_aspect_height ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_aspect_height, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_aspect_height, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_aspect_height);
       else
         _py_call(py.pychips.get_plot_aspect_height, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_aspect_ratio ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_aspect_ratio, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_aspect_ratio, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_aspect_ratio);
       else
         _py_call(py.pychips.get_plot_aspect_ratio, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_aspect_width ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_aspect_width, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_aspect_width, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_aspect_width);
       else
         _py_call(py.pychips.get_plot_aspect_width, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_range ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_range, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_range, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_range);
       else
         _py_call(py.pychips.get_plot_range, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_title, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_title, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_title);
       else
         _py_call(py.pychips.get_plot_title, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_visible);
       else
         _py_call(py.pychips.get_plot_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_xrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_xrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_xrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_xrange);
       else
         _py_call(py.pychips.get_plot_xrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_yrange ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_plot_yrange, __push_args (f_args));
        else
         _py_call(py.pychips.get_plot_yrange, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_plot_yrange);
       else
         _py_call(py.pychips.get_plot_yrange, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_point, __push_args (f_args));
        else
         _py_call(py.pychips.get_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_point);
       else
         _py_call(py.pychips.get_point, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_point_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_point_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_point_visible);
       else
         _py_call(py.pychips.get_point_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_preference ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_preference, __push_args (f_args));
        else
         _py_call(py.pychips.get_preference, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_preference);
       else
         _py_call(py.pychips.get_preference, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - get_preference (attribute)", err.object;
   };
}

define get_preferences ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_preferences, __push_args (f_args));
        else
         _py_call(py.pychips.get_preferences, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_preferences);
       else
         _py_call(py.pychips.get_preferences, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_region, __push_args (f_args));
        else
         _py_call(py.pychips.get_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_region);
       else
         _py_call(py.pychips.get_region, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_region_visible, __push_args (f_args));
        else
         _py_call(py.pychips.get_region_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_region_visible);
       else
         _py_call(py.pychips.get_region_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_server_id ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_server_id, __push_args (f_args));
        else
         _py_call(py.pychips.get_server_id, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_server_id);
       else
         _py_call(py.pychips.get_server_id, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_window, __push_args (f_args));
        else
         _py_call(py.pychips.get_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_window);
       else
         _py_call(py.pychips.get_window, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_window_title ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_window_title, __push_args (f_args));
        else
         _py_call(py.pychips.get_window_title, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_window_title);
       else
         _py_call(py.pychips.get_window_title, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_xaxis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_xaxis, __push_args (f_args));
        else
         _py_call(py.pychips.get_xaxis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_xaxis);
       else
         _py_call(py.pychips.get_xaxis, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_yaxis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.get_yaxis, __push_args (f_args));
        else
         _py_call(py.pychips.get_yaxis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.get_yaxis);
       else
         _py_call(py.pychips.get_yaxis, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define grid_objects ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.grid_objects, __push_args (f_args));
        else
         py_call(py.pychips.grid_objects, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.grid_objects);
       else
         py_call(py.pychips.grid_objects, PY_KW,  __push_args(kwargs));
     }
}

define hide_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_axis, __push_args (f_args));
        else
         py_call(py.pychips.hide_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_axis);
       else
         py_call(py.pychips.hide_axis, PY_KW,  __push_args(kwargs));
     }
}

define hide_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.hide_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_colorbar);
       else
         py_call(py.pychips.hide_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define hide_colorbar_border ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_colorbar_border, __push_args (f_args));
        else
         py_call(py.pychips.hide_colorbar_border, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_colorbar_border);
       else
         py_call(py.pychips.hide_colorbar_border, PY_KW,  __push_args(kwargs));
     }
}

define hide_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_contour, __push_args (f_args));
        else
         py_call(py.pychips.hide_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_contour);
       else
         py_call(py.pychips.hide_contour, PY_KW,  __push_args(kwargs));
     }
}

define hide_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_curve, __push_args (f_args));
        else
         py_call(py.pychips.hide_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_curve);
       else
         py_call(py.pychips.hide_curve, PY_KW,  __push_args(kwargs));
     }
}

define hide_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_depth, __push_args (f_args));
        else
         py_call(py.pychips.hide_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_depth);
       else
         py_call(py.pychips.hide_depth, PY_KW,  __push_args(kwargs));
     }
}

define hide_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_frame, __push_args (f_args));
        else
         py_call(py.pychips.hide_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_frame);
       else
         py_call(py.pychips.hide_frame, PY_KW,  __push_args(kwargs));
     }
}

define hide_frame_border ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_frame_border, __push_args (f_args));
        else
         py_call(py.pychips.hide_frame_border, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_frame_border);
       else
         py_call(py.pychips.hide_frame_border, PY_KW,  __push_args(kwargs));
     }
}

define hide_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_histogram, __push_args (f_args));
        else
         py_call(py.pychips.hide_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_histogram);
       else
         py_call(py.pychips.hide_histogram, PY_KW,  __push_args(kwargs));
     }
}

define hide_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_image, __push_args (f_args));
        else
         py_call(py.pychips.hide_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_image);
       else
         py_call(py.pychips.hide_image, PY_KW,  __push_args(kwargs));
     }
}

define hide_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_label, __push_args (f_args));
        else
         py_call(py.pychips.hide_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_label);
       else
         py_call(py.pychips.hide_label, PY_KW,  __push_args(kwargs));
     }
}

define hide_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_line, __push_args (f_args));
        else
         py_call(py.pychips.hide_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_line);
       else
         py_call(py.pychips.hide_line, PY_KW,  __push_args(kwargs));
     }
}

define hide_major_ticks ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_major_ticks, __push_args (f_args));
        else
         py_call(py.pychips.hide_major_ticks, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_major_ticks);
       else
         py_call(py.pychips.hide_major_ticks, PY_KW,  __push_args(kwargs));
     }
}

define hide_minor_ticks ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_minor_ticks, __push_args (f_args));
        else
         py_call(py.pychips.hide_minor_ticks, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_minor_ticks);
       else
         py_call(py.pychips.hide_minor_ticks, PY_KW,  __push_args(kwargs));
     }
}

define hide_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_plot, __push_args (f_args));
        else
         py_call(py.pychips.hide_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_plot);
       else
         py_call(py.pychips.hide_plot, PY_KW,  __push_args(kwargs));
     }
}

define hide_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_point, __push_args (f_args));
        else
         py_call(py.pychips.hide_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_point);
       else
         py_call(py.pychips.hide_point, PY_KW,  __push_args(kwargs));
     }
}

define hide_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.hide_region, __push_args (f_args));
        else
         py_call(py.pychips.hide_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.hide_region);
       else
         py_call(py.pychips.hide_region, PY_KW,  __push_args(kwargs));
     }
}

define iconify_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.iconify_window, __push_args (f_args));
        else
         py_call(py.pychips.iconify_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.iconify_window);
       else
         py_call(py.pychips.iconify_window, PY_KW,  __push_args(kwargs));
     }
}

define info ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.info, __push_args (f_args));
        else
         _py_call(py.pychips.info, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.info);
       else
         _py_call(py.pychips.info, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define info_bound_axes ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.info_bound_axes, __push_args (f_args));
        else
         _py_call(py.pychips.info_bound_axes, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.info_bound_axes);
       else
         _py_call(py.pychips.info_bound_axes, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define info_coordinate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.info_coordinate, __push_args (f_args));
        else
         _py_call(py.pychips.info_coordinate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.info_coordinate);
       else
         _py_call(py.pychips.info_coordinate, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define info_current ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.info_current, __push_args (f_args));
        else
         _py_call(py.pychips.info_current, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.info_current);
       else
         _py_call(py.pychips.info_current, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define info_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.info_depth, __push_args (f_args));
        else
         _py_call(py.pychips.info_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.info_depth);
       else
         _py_call(py.pychips.info_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define limits ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.limits, __push_args (f_args));
        else
         py_call(py.pychips.limits, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.limits);
       else
         py_call(py.pychips.limits, PY_KW,  __push_args(kwargs));
     }
}

define lin_scale ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.lin_scale, __push_args (f_args));
        else
         py_call(py.pychips.lin_scale, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.lin_scale);
       else
         py_call(py.pychips.lin_scale, PY_KW,  __push_args(kwargs));
     }
}

define linear_scale ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.linear_scale, __push_args (f_args));
        else
         py_call(py.pychips.linear_scale, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.linear_scale);
       else
         py_call(py.pychips.linear_scale, PY_KW,  __push_args(kwargs));
     }
}

define list_servers ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.list_servers, __push_args (f_args));
        else
         py_call(py.pychips.list_servers, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.list_servers);
       else
         py_call(py.pychips.list_servers, PY_KW,  __push_args(kwargs));
     }
}

define load_colormap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.load_colormap, __push_args (f_args));
        else
         py_call(py.pychips.load_colormap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.load_colormap);
       else
         py_call(py.pychips.load_colormap, PY_KW,  __push_args(kwargs));
     }
}

define load_preferences ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.load_preferences, __push_args (f_args));
        else
         py_call(py.pychips.load_preferences, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.load_preferences);
       else
         py_call(py.pychips.load_preferences, PY_KW,  __push_args(kwargs));
     }
}

define load_state ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.load_state, __push_args (f_args));
        else
         py_call(py.pychips.load_state, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.load_state);
       else
         py_call(py.pychips.load_state, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - load_state (filename)", err.object;
   };
}

define lock ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.lock, __push_args (f_args));
        else
         py_call(py.pychips.lock, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.lock);
       else
         py_call(py.pychips.lock, PY_KW,  __push_args(kwargs));
     }
}

define log_scale ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.log_scale, __push_args (f_args));
        else
         py_call(py.pychips.log_scale, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.log_scale);
       else
         py_call(py.pychips.log_scale, PY_KW,  __push_args(kwargs));
     }
}

define logarithmic_scale ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.logarithmic_scale, __push_args (f_args));
        else
         py_call(py.pychips.logarithmic_scale, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.logarithmic_scale);
       else
         py_call(py.pychips.logarithmic_scale, PY_KW,  __push_args(kwargs));
     }
}

define move ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move, __push_args (f_args));
        else
         py_call(py.pychips.move, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move);
       else
         py_call(py.pychips.move, PY_KW,  __push_args(kwargs));
     }
}

define move_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_axis, __push_args (f_args));
        else
         py_call(py.pychips.move_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_axis);
       else
         py_call(py.pychips.move_axis, PY_KW,  __push_args(kwargs));
     }
}

define move_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.move_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_colorbar);
       else
         py_call(py.pychips.move_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define move_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_frame, __push_args (f_args));
        else
         py_call(py.pychips.move_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_frame);
       else
         py_call(py.pychips.move_frame, PY_KW,  __push_args(kwargs));
     }
}

define move_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_label, __push_args (f_args));
        else
         py_call(py.pychips.move_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_label);
       else
         py_call(py.pychips.move_label, PY_KW,  __push_args(kwargs));
     }
}

define move_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_line, __push_args (f_args));
        else
         py_call(py.pychips.move_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_line);
       else
         py_call(py.pychips.move_line, PY_KW,  __push_args(kwargs));
     }
}

define move_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_plot, __push_args (f_args));
        else
         py_call(py.pychips.move_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_plot);
       else
         py_call(py.pychips.move_plot, PY_KW,  __push_args(kwargs));
     }
}

define move_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_point, __push_args (f_args));
        else
         py_call(py.pychips.move_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_point);
       else
         py_call(py.pychips.move_point, PY_KW,  __push_args(kwargs));
     }
}

define move_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.move_region, __push_args (f_args));
        else
         py_call(py.pychips.move_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.move_region);
       else
         py_call(py.pychips.move_region, PY_KW,  __push_args(kwargs));
     }
}

define next_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.next_frame, __push_args (f_args));
        else
         py_call(py.pychips.next_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.next_frame);
       else
         py_call(py.pychips.next_frame, PY_KW,  __push_args(kwargs));
     }
}

define pick ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.pick, __push_args (f_args));
        else
         py_call(py.pychips.pick, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.pick);
       else
         py_call(py.pychips.pick, PY_KW,  __push_args(kwargs));
     }
}

define pick_limits ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.pick_limits, __push_args (f_args));
        else
         py_call(py.pychips.pick_limits, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.pick_limits);
       else
         py_call(py.pychips.pick_limits, PY_KW,  __push_args(kwargs));
     }
}

define prev_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.prev_frame, __push_args (f_args));
        else
         py_call(py.pychips.prev_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.prev_frame);
       else
         py_call(py.pychips.prev_frame, PY_KW,  __push_args(kwargs));
     }
}

define print_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.print_image, __push_args (f_args));
        else
         py_call(py.pychips.print_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.print_image);
       else
         py_call(py.pychips.print_image, PY_KW,  __push_args(kwargs));
     }
}

define print_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.print_window, __push_args (f_args));
        else
         py_call(py.pychips.print_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.print_window);
       else
         py_call(py.pychips.print_window, PY_KW,  __push_args(kwargs));
     }
}

define redo ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.redo, __push_args (f_args));
        else
         py_call(py.pychips.redo, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.redo);
       else
         py_call(py.pychips.redo, PY_KW,  __push_args(kwargs));
     }
}

define reindex_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.reindex_depth, __push_args (f_args));
        else
         py_call(py.pychips.reindex_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.reindex_depth);
       else
         py_call(py.pychips.reindex_depth, PY_KW,  __push_args(kwargs));
     }
}

define remove_image_channel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.remove_image_channel, __push_args (f_args));
        else
         py_call(py.pychips.remove_image_channel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.remove_image_channel);
       else
         py_call(py.pychips.remove_image_channel, PY_KW,  __push_args(kwargs));
     }
}

define reposition_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.reposition_frame, __push_args (f_args));
        else
         py_call(py.pychips.reposition_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.reposition_frame);
       else
         py_call(py.pychips.reposition_frame, PY_KW,  __push_args(kwargs));
     }
}

define reposition_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.reposition_plot, __push_args (f_args));
        else
         py_call(py.pychips.reposition_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.reposition_plot);
       else
         py_call(py.pychips.reposition_plot, PY_KW,  __push_args(kwargs));
     }
}

define reverse_axes ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.reverse_axes, __push_args (f_args));
        else
         py_call(py.pychips.reverse_axes, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.reverse_axes);
       else
         py_call(py.pychips.reverse_axes, PY_KW,  __push_args(kwargs));
     }
}

define row_grid_objects ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.row_grid_objects, __push_args (f_args));
        else
         py_call(py.pychips.row_grid_objects, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.row_grid_objects);
       else
         py_call(py.pychips.row_grid_objects, PY_KW,  __push_args(kwargs));
     }
}

define row_tile ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.row_tile, __push_args (f_args));
        else
         py_call(py.pychips.row_tile, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.row_tile);
       else
         py_call(py.pychips.row_tile, PY_KW,  __push_args(kwargs));
     }
}

define save_preferences ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.save_preferences, __push_args (f_args));
        else
         py_call(py.pychips.save_preferences, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.save_preferences);
       else
         py_call(py.pychips.save_preferences, PY_KW,  __push_args(kwargs));
     }
}

define save_state ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.save_state, __push_args (f_args));
        else
         py_call(py.pychips.save_state, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.save_state);
       else
         py_call(py.pychips.save_state, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - save_state (filename)", err.object;
   };
}

define set_arbitrary_tick_positions ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_arbitrary_tick_positions, __push_args (f_args));
        else
         py_call(py.pychips.set_arbitrary_tick_positions, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_arbitrary_tick_positions);
       else
         py_call(py.pychips.set_arbitrary_tick_positions, PY_KW,  __push_args(kwargs));
     }
}

define set_attribute ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_attribute, __push_args (f_args));
        else
         py_call(py.pychips.set_attribute, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_attribute);
       else
         py_call(py.pychips.set_attribute, PY_KW,  __push_args(kwargs));
     }
}

define set_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_axis, __push_args (f_args));
        else
         py_call(py.pychips.set_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_axis);
       else
         py_call(py.pychips.set_axis, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_text ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_axis_text, __push_args (f_args));
        else
         py_call(py.pychips.set_axis_text, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_axis_text);
       else
         py_call(py.pychips.set_axis_text, PY_KW,  __push_args(kwargs));
     }
}

define set_cascading_property ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_cascading_property, __push_args (f_args));
        else
         py_call(py.pychips.set_cascading_property, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_cascading_property);
       else
         py_call(py.pychips.set_cascading_property, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.set_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_colorbar);
       else
         py_call(py.pychips.set_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define set_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_contour, __push_args (f_args));
        else
         py_call(py.pychips.set_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_contour);
       else
         py_call(py.pychips.set_contour, PY_KW,  __push_args(kwargs));
     }
}

define set_current ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current, __push_args (f_args));
        else
         py_call(py.pychips.set_current, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current);
       else
         py_call(py.pychips.set_current, PY_KW,  __push_args(kwargs));
     }
}

define set_current_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_axis, __push_args (f_args));
        else
         py_call(py.pychips.set_current_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_axis);
       else
         py_call(py.pychips.set_current_axis, PY_KW,  __push_args(kwargs));
     }
}

define set_current_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.set_current_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_colorbar);
       else
         py_call(py.pychips.set_current_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define set_current_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_contour, __push_args (f_args));
        else
         py_call(py.pychips.set_current_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_contour);
       else
         py_call(py.pychips.set_current_contour, PY_KW,  __push_args(kwargs));
     }
}

define set_current_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_curve, __push_args (f_args));
        else
         py_call(py.pychips.set_current_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_curve);
       else
         py_call(py.pychips.set_current_curve, PY_KW,  __push_args(kwargs));
     }
}

define set_current_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_frame, __push_args (f_args));
        else
         py_call(py.pychips.set_current_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_frame);
       else
         py_call(py.pychips.set_current_frame, PY_KW,  __push_args(kwargs));
     }
}

define set_current_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_histogram, __push_args (f_args));
        else
         py_call(py.pychips.set_current_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_histogram);
       else
         py_call(py.pychips.set_current_histogram, PY_KW,  __push_args(kwargs));
     }
}

define set_current_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_image, __push_args (f_args));
        else
         py_call(py.pychips.set_current_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_image);
       else
         py_call(py.pychips.set_current_image, PY_KW,  __push_args(kwargs));
     }
}

define set_current_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_label, __push_args (f_args));
        else
         py_call(py.pychips.set_current_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_label);
       else
         py_call(py.pychips.set_current_label, PY_KW,  __push_args(kwargs));
     }
}

define set_current_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_line, __push_args (f_args));
        else
         py_call(py.pychips.set_current_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_line);
       else
         py_call(py.pychips.set_current_line, PY_KW,  __push_args(kwargs));
     }
}

define set_current_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_plot, __push_args (f_args));
        else
         py_call(py.pychips.set_current_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_plot);
       else
         py_call(py.pychips.set_current_plot, PY_KW,  __push_args(kwargs));
     }
}

define set_current_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_point, __push_args (f_args));
        else
         py_call(py.pychips.set_current_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_point);
       else
         py_call(py.pychips.set_current_point, PY_KW,  __push_args(kwargs));
     }
}

define set_current_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_region, __push_args (f_args));
        else
         py_call(py.pychips.set_current_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_region);
       else
         py_call(py.pychips.set_current_region, PY_KW,  __push_args(kwargs));
     }
}

define set_current_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_current_window, __push_args (f_args));
        else
         py_call(py.pychips.set_current_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_current_window);
       else
         py_call(py.pychips.set_current_window, PY_KW,  __push_args(kwargs));
     }
}

define set_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_curve, __push_args (f_args));
        else
         py_call(py.pychips.set_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_curve);
       else
         py_call(py.pychips.set_curve, PY_KW,  __push_args(kwargs));
     }
}

define set_data_aspect_ratio ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_data_aspect_ratio, __push_args (f_args));
        else
         py_call(py.pychips.set_data_aspect_ratio, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_data_aspect_ratio);
       else
         py_call(py.pychips.set_data_aspect_ratio, PY_KW,  __push_args(kwargs));
     }
}

define set_default_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_default_depth, __push_args (f_args));
        else
         py_call(py.pychips.set_default_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_default_depth);
       else
         py_call(py.pychips.set_default_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_frame, __push_args (f_args));
        else
         py_call(py.pychips.set_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_frame);
       else
         py_call(py.pychips.set_frame, PY_KW,  __push_args(kwargs));
     }
}

define set_framemode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_framemode, __push_args (f_args));
        else
         py_call(py.pychips.set_framemode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_framemode);
       else
         py_call(py.pychips.set_framemode, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_histogram, __push_args (f_args));
        else
         py_call(py.pychips.set_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_histogram);
       else
         py_call(py.pychips.set_histogram, PY_KW,  __push_args(kwargs));
     }
}

define set_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_image, __push_args (f_args));
        else
         py_call(py.pychips.set_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_image);
       else
         py_call(py.pychips.set_image, PY_KW,  __push_args(kwargs));
     }
}

define set_image_channel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_image_channel, __push_args (f_args));
        else
         py_call(py.pychips.set_image_channel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_image_channel);
       else
         py_call(py.pychips.set_image_channel, PY_KW,  __push_args(kwargs));
     }
}

define set_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_label, __push_args (f_args));
        else
         py_call(py.pychips.set_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_label);
       else
         py_call(py.pychips.set_label, PY_KW,  __push_args(kwargs));
     }
}

define set_label_text ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_label_text, __push_args (f_args));
        else
         py_call(py.pychips.set_label_text, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_label_text);
       else
         py_call(py.pychips.set_label_text, PY_KW,  __push_args(kwargs));
     }
}

define set_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_line, __push_args (f_args));
        else
         py_call(py.pychips.set_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_line);
       else
         py_call(py.pychips.set_line, PY_KW,  __push_args(kwargs));
     }
}

define set_plot ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_plot, __push_args (f_args));
        else
         py_call(py.pychips.set_plot, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_plot);
       else
         py_call(py.pychips.set_plot, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_aspect_ratio ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_plot_aspect_ratio, __push_args (f_args));
        else
         py_call(py.pychips.set_plot_aspect_ratio, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_plot_aspect_ratio);
       else
         py_call(py.pychips.set_plot_aspect_ratio, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_plot_title, __push_args (f_args));
        else
         py_call(py.pychips.set_plot_title, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_plot_title);
       else
         py_call(py.pychips.set_plot_title, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_xlabel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_plot_xlabel, __push_args (f_args));
        else
         py_call(py.pychips.set_plot_xlabel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_plot_xlabel);
       else
         py_call(py.pychips.set_plot_xlabel, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_ylabel ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_plot_ylabel, __push_args (f_args));
        else
         py_call(py.pychips.set_plot_ylabel, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_plot_ylabel);
       else
         py_call(py.pychips.set_plot_ylabel, PY_KW,  __push_args(kwargs));
     }
}

define set_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_point, __push_args (f_args));
        else
         py_call(py.pychips.set_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_point);
       else
         py_call(py.pychips.set_point, PY_KW,  __push_args(kwargs));
     }
}

define set_preference ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_preference, __push_args (f_args));
        else
         py_call(py.pychips.set_preference, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_preference);
       else
         py_call(py.pychips.set_preference, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - set_preference (attribute, value)", err.object;
   };
}

define set_preference_autoload ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_preference_autoload, __push_args (f_args));
        else
         py_call(py.pychips.set_preference_autoload, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_preference_autoload);
       else
         py_call(py.pychips.set_preference_autoload, PY_KW,  __push_args(kwargs));
     }
}

define set_preferences ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_preferences, __push_args (f_args));
        else
         py_call(py.pychips.set_preferences, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_preferences);
       else
         py_call(py.pychips.set_preferences, PY_KW,  __push_args(kwargs));
     }
}

define set_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_region, __push_args (f_args));
        else
         py_call(py.pychips.set_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_region);
       else
         py_call(py.pychips.set_region, PY_KW,  __push_args(kwargs));
     }
}

define set_undo_tag ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_undo_tag, __push_args (f_args));
        else
         py_call(py.pychips.set_undo_tag, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_undo_tag);
       else
         py_call(py.pychips.set_undo_tag, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - set_undo_tag (tagname)", err.object;
   };
}

define set_window ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_window, __push_args (f_args));
        else
         py_call(py.pychips.set_window, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_window);
       else
         py_call(py.pychips.set_window, PY_KW,  __push_args(kwargs));
     }
}

define set_window_title ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_window_title, __push_args (f_args));
        else
         py_call(py.pychips.set_window_title, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_window_title);
       else
         py_call(py.pychips.set_window_title, PY_KW,  __push_args(kwargs));
     }
}

define set_xaxis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_xaxis, __push_args (f_args));
        else
         py_call(py.pychips.set_xaxis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_xaxis);
       else
         py_call(py.pychips.set_xaxis, PY_KW,  __push_args(kwargs));
     }
}

define set_yaxis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.set_yaxis, __push_args (f_args));
        else
         py_call(py.pychips.set_yaxis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.set_yaxis);
       else
         py_call(py.pychips.set_yaxis, PY_KW,  __push_args(kwargs));
     }
}

define shuffle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle, __push_args (f_args));
        else
         py_call(py.pychips.shuffle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle);
       else
         py_call(py.pychips.shuffle, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_axis ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_axis, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_axis, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_axis);
       else
         py_call(py.pychips.shuffle_axis, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_back ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_back, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_back, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_back);
       else
         py_call(py.pychips.shuffle_back, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_backward ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_backward, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_backward, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_backward);
       else
         py_call(py.pychips.shuffle_backward, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_colorbar ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_colorbar, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_colorbar, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_colorbar);
       else
         py_call(py.pychips.shuffle_colorbar, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_contour ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_contour, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_contour, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_contour);
       else
         py_call(py.pychips.shuffle_contour, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_curve ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_curve, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_curve, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_curve);
       else
         py_call(py.pychips.shuffle_curve, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_forward ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_forward, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_forward, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_forward);
       else
         py_call(py.pychips.shuffle_forward, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_front ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_front, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_front, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_front);
       else
         py_call(py.pychips.shuffle_front, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_histogram ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_histogram, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_histogram, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_histogram);
       else
         py_call(py.pychips.shuffle_histogram, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_image ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_image, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_image, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_image);
       else
         py_call(py.pychips.shuffle_image, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_label ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_label, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_label, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_label);
       else
         py_call(py.pychips.shuffle_label, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_line ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_line, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_line, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_line);
       else
         py_call(py.pychips.shuffle_line, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_point ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_point, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_point, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_point);
       else
         py_call(py.pychips.shuffle_point, PY_KW,  __push_args(kwargs));
     }
}

define shuffle_region ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.shuffle_region, __push_args (f_args));
        else
         py_call(py.pychips.shuffle_region, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.shuffle_region);
       else
         py_call(py.pychips.shuffle_region, PY_KW,  __push_args(kwargs));
     }
}

define split ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.split, __push_args (f_args));
        else
         py_call(py.pychips.split, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.split);
       else
         py_call(py.pychips.split, PY_KW,  __push_args(kwargs));
     }
}

define strip_chart ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.strip_chart, __push_args (f_args));
        else
         py_call(py.pychips.strip_chart, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.strip_chart);
       else
         py_call(py.pychips.strip_chart, PY_KW,  __push_args(kwargs));
     }
}

define swap_object_positions ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.swap_object_positions, __push_args (f_args));
        else
         py_call(py.pychips.swap_object_positions, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.swap_object_positions);
       else
         py_call(py.pychips.swap_object_positions, PY_KW,  __push_args(kwargs));
     }
}

define tile ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.tile, __push_args (f_args));
        else
         py_call(py.pychips.tile, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.tile);
       else
         py_call(py.pychips.tile, PY_KW,  __push_args(kwargs));
     }
}

define unbind_axes ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.unbind_axes, __push_args (f_args));
        else
         py_call(py.pychips.unbind_axes, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.unbind_axes);
       else
         py_call(py.pychips.unbind_axes, PY_KW,  __push_args(kwargs));
     }
}

define undo ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.undo, __push_args (f_args));
        else
         py_call(py.pychips.undo, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.undo);
       else
         py_call(py.pychips.undo, PY_KW,  __push_args(kwargs));
     }
}

define unlock ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.unlock, __push_args (f_args));
        else
         py_call(py.pychips.unlock, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.unlock);
       else
         py_call(py.pychips.unlock, PY_KW,  __push_args(kwargs));
     }
}

define view_placed_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.view_placed_frame, __push_args (f_args));
        else
         py_call(py.pychips.view_placed_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.view_placed_frame);
       else
         py_call(py.pychips.view_placed_frame, PY_KW,  __push_args(kwargs));
     }
}

define view_single_frame ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.view_single_frame, __push_args (f_args));
        else
         py_call(py.pychips.view_single_frame, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.view_single_frame);
       else
         py_call(py.pychips.view_single_frame, PY_KW,  __push_args(kwargs));
     }
}

$1 = path_concat (path_dirname (__FILE__), "help/chipsfuns.hlp");
if (NULL != stat_file ($1))
  add_doc_file ($1);
provide ("chips");
