% 
%  Copyright (C) 2008  Smithsonian Astrophysical Observatory
%
%
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 3 of the License, or
%  (at your option) any later version.
%
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%
%  You should have received a copy of the GNU General Public License along
%  with this program; if not, write to the Free Software Foundation, Inc.,
%  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
%

require ("chips");
py_import ("pychips.advanced");
private define _chips_push ( arg ) { return arg; }
%Check for return. Push tuples on the list individually
private define check_return ()
{
  variable ret_val=NULL;
  if (_stkdepth () > 0){
     ret_val = ();
     if (py_call(py.type,ret_val) == py.tuple){
        variable len = py_call (py.len (ret_val))-1;
        variable ii;
        _for (0, len, 1){
            ii = ();
           _chips_push (ret_val[ii]);
        }
     }
  }
  return ret_val;
}

%Create valid types for python
private define _chips_check_type (value);
private define _chips_check_type (value)
{
  if (typeof (value) == Char_Type)
     value = int (value);
  return value;
}
%Turn a structure into a list
private define _chips_struct_to_list (s);
private define _chips_struct_to_list (s)
{
  variable sl_list = {};
  variable name, value;
  variable prefix = NULL;
  if (_NARGS > 1)
     prefix = ();
  try{
     %Go through each element in the structure and make it a list
     foreach (get_struct_field_names (s)){
        name = ();
        value = get_struct_field (s, name);
        if (prefix != NULL)
           name = prefix + "." + name;
        %Expand out sub structures
        if (is_struct_type (value)){
           variable tmp_list = _chips_struct_to_list (name,value);
           variable ii;
           %Concatinate the two lists
           _for (length (tmp_list)-1, 0, -1){
              ii = ();
              list_insert (sl_list, tmp_list[ii], 0);
           }
        }
        else{
           value = _chips_check_type (value);
           list_insert (sl_list,value,0);
           list_insert (sl_list,name,0);
        }
     }
  }
  catch RunTimeError:
     %Return NULL if this was not a structure
     return NULL;
  return sl_list;
}
#if (_slang_version >= 20101)
%Return all the qualifiers in a struct
private define _chips_get_qualifiers (kwstruct);
private define _chips_get_qualifiers (kwstruct)
{
  if( kwstruct == NULL )
    return NULL;
  variable ii, name;
  variable kwnames = get_struct_field_names(kwstruct);
  variable kwargs = Struct_Type[2 * length(kwnames)];
  ii = 0;
  foreach (kwnames)
  {
     name = ();
     kwargs[ii] = struct{value=name};
     ii = ii + 1;
     kwargs[ii] = struct{value=get_struct_field(kwstruct, name)};
     ii = ii + 1;
   }
 return kwargs;
}
#endif
%Chips Functions
define blink_frames ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.blink_frames, __push_args (f_args));
        else
         py_call(py.pychips.advanced.blink_frames, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.blink_frames);
       else
         py_call(py.pychips.advanced.blink_frames, PY_KW,  __push_args(kwargs));
     }
}

define clear_errors ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.clear_errors, __push_args (f_args));
        else
         py_call(py.pychips.advanced.clear_errors, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.clear_errors);
       else
         py_call(py.pychips.advanced.clear_errors, PY_KW,  __push_args(kwargs));
     }
}

define close_undo_block ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.close_undo_block, __push_args (f_args));
        else
         py_call(py.pychips.advanced.close_undo_block, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.close_undo_block);
       else
         py_call(py.pychips.advanced.close_undo_block, PY_KW,  __push_args(kwargs));
     }
}

define close_undo_buffer ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.close_undo_buffer, __push_args (f_args));
        else
         py_call(py.pychips.advanced.close_undo_buffer, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.close_undo_buffer);
       else
         py_call(py.pychips.advanced.close_undo_buffer, PY_KW,  __push_args(kwargs));
     }
}

define discard_undo_buffer ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.discard_undo_buffer, __push_args (f_args));
        else
         py_call(py.pychips.advanced.discard_undo_buffer, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.discard_undo_buffer);
       else
         py_call(py.pychips.advanced.discard_undo_buffer, PY_KW,  __push_args(kwargs));
     }
}

define get_axis_automax ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_automax, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_automax, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_automax);
       else
         _py_call(py.pychips.advanced.get_axis_automax, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_automin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_automin, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_automin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_automin);
       else
         _py_call(py.pychips.advanced.get_axis_automin, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_color);
       else
         _py_call(py.pychips.advanced.get_axis_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_depth);
       else
         _py_call(py.pychips.advanced.get_axis_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_angle);
       else
         _py_call(py.pychips.advanced.get_axis_label_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_color);
       else
         _py_call(py.pychips.advanced.get_axis_label_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_font, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_font);
       else
         _py_call(py.pychips.advanced.get_axis_label_font, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_fontstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_fontstyle);
       else
         _py_call(py.pychips.advanced.get_axis_label_fontstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_halign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_halign);
       else
         _py_call(py.pychips.advanced.get_axis_label_halign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_size);
       else
         _py_call(py.pychips.advanced.get_axis_label_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_valign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_valign);
       else
         _py_call(py.pychips.advanced.get_axis_label_valign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_label_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_label_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_label_visible);
       else
         _py_call(py.pychips.advanced.get_axis_label_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_major_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_major_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_color);
       else
         _py_call(py.pychips.advanced.get_axis_major_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_major_interval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_interval, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_major_interval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_interval);
       else
         _py_call(py.pychips.advanced.get_axis_major_interval, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_major_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_length, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_major_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_length);
       else
         _py_call(py.pychips.advanced.get_axis_major_length, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_major_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_mode, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_major_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_mode);
       else
         _py_call(py.pychips.advanced.get_axis_major_mode, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_major_num ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_num, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_major_num, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_num);
       else
         _py_call(py.pychips.advanced.get_axis_major_num, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_major_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_major_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_style);
       else
         _py_call(py.pychips.advanced.get_axis_major_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_major_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_major_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_major_thickness);
       else
         _py_call(py.pychips.advanced.get_axis_major_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_majorgrid_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_majorgrid_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_color);
       else
         _py_call(py.pychips.advanced.get_axis_majorgrid_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_majorgrid_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_majorgrid_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_style);
       else
         _py_call(py.pychips.advanced.get_axis_majorgrid_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_majorgrid_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_majorgrid_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_thickness);
       else
         _py_call(py.pychips.advanced.get_axis_majorgrid_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_majorgrid_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_majorgrid_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_majorgrid_visible);
       else
         _py_call(py.pychips.advanced.get_axis_majorgrid_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minor_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minor_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_color);
       else
         _py_call(py.pychips.advanced.get_axis_minor_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minor_interval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_interval, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minor_interval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_interval);
       else
         _py_call(py.pychips.advanced.get_axis_minor_interval, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minor_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_length, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minor_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_length);
       else
         _py_call(py.pychips.advanced.get_axis_minor_length, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minor_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_mode, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minor_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_mode);
       else
         _py_call(py.pychips.advanced.get_axis_minor_mode, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minor_num ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_num, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minor_num, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_num);
       else
         _py_call(py.pychips.advanced.get_axis_minor_num, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minor_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minor_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_style);
       else
         _py_call(py.pychips.advanced.get_axis_minor_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minor_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minor_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minor_thickness);
       else
         _py_call(py.pychips.advanced.get_axis_minor_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minorgrid_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minorgrid_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_color);
       else
         _py_call(py.pychips.advanced.get_axis_minorgrid_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minorgrid_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minorgrid_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_style);
       else
         _py_call(py.pychips.advanced.get_axis_minorgrid_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minorgrid_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minorgrid_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_thickness);
       else
         _py_call(py.pychips.advanced.get_axis_minorgrid_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_minorgrid_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_minorgrid_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_minorgrid_visible);
       else
         _py_call(py.pychips.advanced.get_axis_minorgrid_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_pad ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_pad, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_pad, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_pad);
       else
         _py_call(py.pychips.advanced.get_axis_pad, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_parallel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_parallel_offset, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_parallel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_parallel_offset);
       else
         _py_call(py.pychips.advanced.get_axis_parallel_offset, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_perpendicular_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_perpendicular_offset, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_perpendicular_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_perpendicular_offset);
       else
         _py_call(py.pychips.advanced.get_axis_perpendicular_offset, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_thickness);
       else
         _py_call(py.pychips.advanced.get_axis_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_tickformat ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_tickformat, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_tickformat, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_tickformat);
       else
         _py_call(py.pychips.advanced.get_axis_tickformat, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_angle);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_color);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_font, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_font);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_font, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_fontstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_fontstyle);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_fontstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_halign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_halign);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_halign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_offset, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_offset);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_offset, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_size);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_style);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_valign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_valign);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_valign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_ticklabel_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_ticklabel_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_ticklabel_visible);
       else
         _py_call(py.pychips.advanced.get_axis_ticklabel_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_axis_tickstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_tickstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_axis_tickstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_axis_tickstyle);
       else
         _py_call(py.pychips.advanced.get_axis_tickstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_algorithm ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_algorithm, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_algorithm, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_algorithm);
       else
         _py_call(py.pychips.advanced.get_contour_algorithm, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_color);
       else
         _py_call(py.pychips.advanced.get_contour_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_depth);
       else
         _py_call(py.pychips.advanced.get_contour_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_interval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_interval, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_interval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_interval);
       else
         _py_call(py.pychips.advanced.get_contour_interval, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_mode, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_mode);
       else
         _py_call(py.pychips.advanced.get_contour_mode, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_numlevels ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_numlevels, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_numlevels, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_numlevels);
       else
         _py_call(py.pychips.advanced.get_contour_numlevels, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_style);
       else
         _py_call(py.pychips.advanced.get_contour_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_contour_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_thickness);
       else
         _py_call(py.pychips.advanced.get_contour_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_baddatamode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_baddatamode, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_baddatamode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_baddatamode);
       else
         _py_call(py.pychips.advanced.get_curve_baddatamode, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_depth);
       else
         _py_call(py.pychips.advanced.get_curve_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_errcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errcolor);
       else
         _py_call(py.pychips.advanced.get_curve_errcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_errdown ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errdown, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errdown, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errdown);
       else
         _py_call(py.pychips.advanced.get_curve_errdown, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_errleft ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errleft, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errleft, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errleft);
       else
         _py_call(py.pychips.advanced.get_curve_errleft, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_errright ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errright, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errright, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errright);
       else
         _py_call(py.pychips.advanced.get_curve_errright, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_errstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errstyle);
       else
         _py_call(py.pychips.advanced.get_curve_errstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_errthickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errthickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errthickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errthickness);
       else
         _py_call(py.pychips.advanced.get_curve_errthickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_errup ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errup, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errup, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errup);
       else
         _py_call(py.pychips.advanced.get_curve_errup, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_limitlength ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_limitlength, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_limitlength, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_limitlength);
       else
         _py_call(py.pychips.advanced.get_curve_limitlength, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_limitoverride ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_limitoverride, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_limitoverride, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_limitoverride);
       else
         _py_call(py.pychips.advanced.get_curve_limitoverride, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_linecolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_linecolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_linecolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_linecolor);
       else
         _py_call(py.pychips.advanced.get_curve_linecolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_linestyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_linestyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_linestyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_linestyle);
       else
         _py_call(py.pychips.advanced.get_curve_linestyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_linethickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_linethickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_linethickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_linethickness);
       else
         _py_call(py.pychips.advanced.get_curve_linethickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_symbolangle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolangle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_symbolangle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolangle);
       else
         _py_call(py.pychips.advanced.get_curve_symbolangle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_symbolcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_symbolcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolcolor);
       else
         _py_call(py.pychips.advanced.get_curve_symbolcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_symbolfill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolfill, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_symbolfill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolfill);
       else
         _py_call(py.pychips.advanced.get_curve_symbolfill, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_symbolsize ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolsize, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_symbolsize, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolsize);
       else
         _py_call(py.pychips.advanced.get_curve_symbolsize, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_curve_symbolstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_symbolstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_symbolstyle);
       else
         _py_call(py.pychips.advanced.get_curve_symbolstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_error_count ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_error_count, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_error_count, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_error_count);
       else
         _py_call(py.pychips.advanced.get_error_count, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_error_types ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_error_types, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_error_types, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_error_types);
       else
         _py_call(py.pychips.advanced.get_error_types, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_error_verbosity ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_error_verbosity, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_error_verbosity, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_error_verbosity);
       else
         _py_call(py.pychips.advanced.get_error_verbosity, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_errors ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_errors, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_errors, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_errors);
       else
         _py_call(py.pychips.advanced.get_errors, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_frame_bgcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_frame_bgcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_frame_bgcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_frame_bgcolor);
       else
         _py_call(py.pychips.advanced.get_frame_bgcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_frame_border_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_frame_border_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_frame_border_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_frame_border_color);
       else
         _py_call(py.pychips.advanced.get_frame_border_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_frame_transparency ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_frame_transparency, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_frame_transparency, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_frame_transparency);
       else
         _py_call(py.pychips.advanced.get_frame_transparency, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_depth);
       else
         _py_call(py.pychips.advanced.get_histogram_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_dropline ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_dropline, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_dropline, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_dropline);
       else
         _py_call(py.pychips.advanced.get_histogram_dropline, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_errcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_errcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errcolor);
       else
         _py_call(py.pychips.advanced.get_histogram_errcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_errdown ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errdown, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_errdown, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errdown);
       else
         _py_call(py.pychips.advanced.get_histogram_errdown, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_errstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_errstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errstyle);
       else
         _py_call(py.pychips.advanced.get_histogram_errstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_errthickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errthickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_errthickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errthickness);
       else
         _py_call(py.pychips.advanced.get_histogram_errthickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_errup ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errup, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_errup, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errup);
       else
         _py_call(py.pychips.advanced.get_histogram_errup, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_fillstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_fillstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_fillstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_fillstyle);
       else
         _py_call(py.pychips.advanced.get_histogram_fillstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_fillcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_fillcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_fillcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_fillcolor);
       else
         _py_call(py.pychips.advanced.get_histogram_fillcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_fillopacity ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_fillopacity, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_fillopacity, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_fillopacity);
       else
         _py_call(py.pychips.advanced.get_histogram_fillopacity, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_linecolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_linecolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_linecolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_linecolor);
       else
         _py_call(py.pychips.advanced.get_histogram_linecolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_linestyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_linestyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_linestyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_linestyle);
       else
         _py_call(py.pychips.advanced.get_histogram_linestyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_linethickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_linethickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_linethickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_linethickness);
       else
         _py_call(py.pychips.advanced.get_histogram_linethickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_symbolangle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolangle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_symbolangle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolangle);
       else
         _py_call(py.pychips.advanced.get_histogram_symbolangle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_symbolcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_symbolcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolcolor);
       else
         _py_call(py.pychips.advanced.get_histogram_symbolcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_symbolfill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolfill, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_symbolfill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolfill);
       else
         _py_call(py.pychips.advanced.get_histogram_symbolfill, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_symbolsize ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolsize, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_symbolsize, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolsize);
       else
         _py_call(py.pychips.advanced.get_histogram_symbolsize, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_symbolstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_symbolstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_symbolstyle);
       else
         _py_call(py.pychips.advanced.get_histogram_symbolstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_visible);
       else
         _py_call(py.pychips.advanced.get_histogram_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_angle);
       else
         _py_call(py.pychips.advanced.get_label_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_color);
       else
         _py_call(py.pychips.advanced.get_label_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_depth);
       else
         _py_call(py.pychips.advanced.get_label_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_font, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_font);
       else
         _py_call(py.pychips.advanced.get_label_font, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_fontstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_fontstyle);
       else
         _py_call(py.pychips.advanced.get_label_fontstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_halign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_halign);
       else
         _py_call(py.pychips.advanced.get_label_halign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_size);
       else
         _py_call(py.pychips.advanced.get_label_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_valign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_valign);
       else
         _py_call(py.pychips.advanced.get_label_valign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_label_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_label_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_label_visible);
       else
         _py_call(py.pychips.advanced.get_label_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_line_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_line_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_color);
       else
         _py_call(py.pychips.advanced.get_line_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_line_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_line_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_depth);
       else
         _py_call(py.pychips.advanced.get_line_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_line_extend ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_extend, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_line_extend, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_extend);
       else
         _py_call(py.pychips.advanced.get_line_extend, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_line_stipple ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_stipple, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_line_stipple, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_stipple);
       else
         _py_call(py.pychips.advanced.get_line_stipple, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - get_line_stipple (linetype)", err.object;
   };
}

define get_line_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_line_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_style);
       else
         _py_call(py.pychips.advanced.get_line_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_line_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_line_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_thickness);
       else
         _py_call(py.pychips.advanced.get_line_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_line_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_line_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_line_visible);
       else
         _py_call(py.pychips.advanced.get_line_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_major_ticks_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_major_ticks_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_major_ticks_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_major_ticks_visible);
       else
         _py_call(py.pychips.advanced.get_major_ticks_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_minor_ticks_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_minor_ticks_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_minor_ticks_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_minor_ticks_visible);
       else
         _py_call(py.pychips.advanced.get_minor_ticks_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_axisstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_axisstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_axisstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_axisstyle);
       else
         _py_call(py.pychips.advanced.get_plot_axisstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_bottommargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_bottommargin, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_bottommargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_bottommargin);
       else
         _py_call(py.pychips.advanced.get_plot_bottommargin, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_cornerstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_cornerstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_cornerstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_cornerstyle);
       else
         _py_call(py.pychips.advanced.get_plot_cornerstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_leftmargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_leftmargin, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_leftmargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_leftmargin);
       else
         _py_call(py.pychips.advanced.get_plot_leftmargin, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_rightmargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_rightmargin, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_rightmargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_rightmargin);
       else
         _py_call(py.pychips.advanced.get_plot_rightmargin, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_angle);
       else
         _py_call(py.pychips.advanced.get_plot_title_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_color);
       else
         _py_call(py.pychips.advanced.get_plot_title_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_depth);
       else
         _py_call(py.pychips.advanced.get_plot_title_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_font, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_font);
       else
         _py_call(py.pychips.advanced.get_plot_title_font, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_fontstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_fontstyle);
       else
         _py_call(py.pychips.advanced.get_plot_title_fontstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_halign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_halign);
       else
         _py_call(py.pychips.advanced.get_plot_title_halign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_size);
       else
         _py_call(py.pychips.advanced.get_plot_title_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_valign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_valign);
       else
         _py_call(py.pychips.advanced.get_plot_title_valign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_xpos ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_xpos, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_xpos, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_xpos);
       else
         _py_call(py.pychips.advanced.get_plot_title_xpos, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_title_ypos ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_ypos, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_title_ypos, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_title_ypos);
       else
         _py_call(py.pychips.advanced.get_plot_title_ypos, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_plot_topmargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_topmargin, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_plot_topmargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_plot_topmargin);
       else
         _py_call(py.pychips.advanced.get_plot_topmargin, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_point_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_angle);
       else
         _py_call(py.pychips.advanced.get_point_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_point_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_color);
       else
         _py_call(py.pychips.advanced.get_point_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_point_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_depth);
       else
         _py_call(py.pychips.advanced.get_point_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point_fill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_fill, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_point_fill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_fill);
       else
         _py_call(py.pychips.advanced.get_point_fill, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_point_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_size);
       else
         _py_call(py.pychips.advanced.get_point_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_point_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_point_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_point_style);
       else
         _py_call(py.pychips.advanced.get_point_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_angle);
       else
         _py_call(py.pychips.advanced.get_region_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_depth);
       else
         _py_call(py.pychips.advanced.get_region_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_edgecolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_edgecolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_edgecolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_edgecolor);
       else
         _py_call(py.pychips.advanced.get_region_edgecolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_edgestyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_edgestyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_edgestyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_edgestyle);
       else
         _py_call(py.pychips.advanced.get_region_edgestyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_edgethickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_edgethickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_edgethickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_edgethickness);
       else
         _py_call(py.pychips.advanced.get_region_edgethickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_fillstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_fillstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_fillstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_fillstyle);
       else
         _py_call(py.pychips.advanced.get_region_fillstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_fillcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_fillcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_fillcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_fillcolor);
       else
         _py_call(py.pychips.advanced.get_region_fillcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_region_opacity ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_opacity, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_region_opacity, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_region_opacity);
       else
         _py_call(py.pychips.advanced.get_region_opacity, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_window_bgcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_window_bgcolor, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_window_bgcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_window_bgcolor);
       else
         _py_call(py.pychips.advanced.get_window_bgcolor, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_window_redraw ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_window_redraw, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_window_redraw, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_window_redraw);
       else
         _py_call(py.pychips.advanced.get_window_redraw, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define load_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.load_font, __push_args (f_args));
        else
         py_call(py.pychips.advanced.load_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.load_font);
       else
         py_call(py.pychips.advanced.load_font, PY_KW,  __push_args(kwargs));
     }
}

define load_fill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.load_fill, __push_args (f_args));
        else
         py_call(py.pychips.advanced.load_fill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.load_fill);
       else
         py_call(py.pychips.advanced.load_fill, PY_KW,  __push_args(kwargs));
     }
}

define open_undo_block ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.open_undo_block, __push_args (f_args));
        else
         py_call(py.pychips.advanced.open_undo_block, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.open_undo_block);
       else
         py_call(py.pychips.advanced.open_undo_block, PY_KW,  __push_args(kwargs));
     }
}

define open_undo_buffer ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.open_undo_buffer, __push_args (f_args));
        else
         py_call(py.pychips.advanced.open_undo_buffer, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.open_undo_buffer);
       else
         py_call(py.pychips.advanced.open_undo_buffer, PY_KW,  __push_args(kwargs));
     }
}

define pickmask_to_string ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.pickmask_to_string, __push_args (f_args));
        else
         py_call(py.pychips.advanced.pickmask_to_string, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.pickmask_to_string);
       else
         py_call(py.pychips.advanced.pickmask_to_string, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_automax ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_automax, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_automax, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_automax);
       else
         py_call(py.pychips.advanced.set_axis_automax, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_automin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_automin, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_automin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_automin);
       else
         py_call(py.pychips.advanced.set_axis_automin, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_color);
       else
         py_call(py.pychips.advanced.set_axis_color, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_depth);
       else
         py_call(py.pychips.advanced.set_axis_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_angle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_angle);
       else
         py_call(py.pychips.advanced.set_axis_label_angle, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_color);
       else
         py_call(py.pychips.advanced.set_axis_label_color, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_font, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_font);
       else
         py_call(py.pychips.advanced.set_axis_label_font, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_fontstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_fontstyle);
       else
         py_call(py.pychips.advanced.set_axis_label_fontstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_halign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_halign);
       else
         py_call(py.pychips.advanced.set_axis_label_halign, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_size);
       else
         py_call(py.pychips.advanced.set_axis_label_size, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_valign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_valign);
       else
         py_call(py.pychips.advanced.set_axis_label_valign, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_label_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_visible, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_label_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_label_visible);
       else
         py_call(py.pychips.advanced.set_axis_label_visible, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_major_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_major_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_color);
       else
         py_call(py.pychips.advanced.set_axis_major_color, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_major_interval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_interval, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_major_interval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_interval);
       else
         py_call(py.pychips.advanced.set_axis_major_interval, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_major_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_length, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_major_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_length);
       else
         py_call(py.pychips.advanced.set_axis_major_length, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_major_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_mode, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_major_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_mode);
       else
         py_call(py.pychips.advanced.set_axis_major_mode, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_major_num ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_num, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_major_num, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_num);
       else
         py_call(py.pychips.advanced.set_axis_major_num, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_major_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_major_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_style);
       else
         py_call(py.pychips.advanced.set_axis_major_style, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_major_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_major_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_major_thickness);
       else
         py_call(py.pychips.advanced.set_axis_major_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_majorgrid_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_majorgrid_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_color);
       else
         py_call(py.pychips.advanced.set_axis_majorgrid_color, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_majorgrid_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_majorgrid_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_style);
       else
         py_call(py.pychips.advanced.set_axis_majorgrid_style, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_majorgrid_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_majorgrid_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_thickness);
       else
         py_call(py.pychips.advanced.set_axis_majorgrid_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_majorgrid_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_visible, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_majorgrid_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_majorgrid_visible);
       else
         py_call(py.pychips.advanced.set_axis_majorgrid_visible, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minor_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minor_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_color);
       else
         py_call(py.pychips.advanced.set_axis_minor_color, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minor_interval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_interval, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minor_interval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_interval);
       else
         py_call(py.pychips.advanced.set_axis_minor_interval, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minor_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_length, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minor_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_length);
       else
         py_call(py.pychips.advanced.set_axis_minor_length, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minor_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_mode, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minor_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_mode);
       else
         py_call(py.pychips.advanced.set_axis_minor_mode, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minor_num ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_num, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minor_num, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_num);
       else
         py_call(py.pychips.advanced.set_axis_minor_num, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minor_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minor_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_style);
       else
         py_call(py.pychips.advanced.set_axis_minor_style, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minor_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minor_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minor_thickness);
       else
         py_call(py.pychips.advanced.set_axis_minor_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minorgrid_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minorgrid_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_color);
       else
         py_call(py.pychips.advanced.set_axis_minorgrid_color, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minorgrid_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minorgrid_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_style);
       else
         py_call(py.pychips.advanced.set_axis_minorgrid_style, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minorgrid_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minorgrid_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_thickness);
       else
         py_call(py.pychips.advanced.set_axis_minorgrid_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_minorgrid_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_visible, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_minorgrid_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_minorgrid_visible);
       else
         py_call(py.pychips.advanced.set_axis_minorgrid_visible, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_pad ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_pad, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_pad, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_pad);
       else
         py_call(py.pychips.advanced.set_axis_pad, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_parallel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_parallel_offset, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_parallel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_parallel_offset);
       else
         py_call(py.pychips.advanced.set_axis_parallel_offset, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_perpendicular_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_perpendicular_offset, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_perpendicular_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_perpendicular_offset);
       else
         py_call(py.pychips.advanced.set_axis_perpendicular_offset, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_thickness);
       else
         py_call(py.pychips.advanced.set_axis_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_tickformat ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_tickformat, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_tickformat, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_tickformat);
       else
         py_call(py.pychips.advanced.set_axis_tickformat, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_angle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_angle);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_angle, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_color);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_color, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_font, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_font);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_font, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_fontstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_fontstyle);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_fontstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_halign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_halign);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_halign, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_offset, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_offset);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_offset, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_size);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_size, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_style);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_style, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_valign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_valign);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_valign, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_ticklabel_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_visible, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_ticklabel_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_ticklabel_visible);
       else
         py_call(py.pychips.advanced.set_axis_ticklabel_visible, PY_KW,  __push_args(kwargs));
     }
}

define set_axis_tickstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_tickstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_axis_tickstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_axis_tickstyle);
       else
         py_call(py.pychips.advanced.set_axis_tickstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_algorithm ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_algorithm, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_algorithm, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_algorithm);
       else
         py_call(py.pychips.advanced.set_contour_algorithm, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_color);
       else
         py_call(py.pychips.advanced.set_contour_color, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_depth);
       else
         py_call(py.pychips.advanced.set_contour_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_interval ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_interval, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_interval, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_interval);
       else
         py_call(py.pychips.advanced.set_contour_interval, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_levels ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_levels, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_levels, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_levels);
       else
         py_call(py.pychips.advanced.set_contour_levels, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_mode, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_mode);
       else
         py_call(py.pychips.advanced.set_contour_mode, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_numlevels ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_numlevels, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_numlevels, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_numlevels);
       else
         py_call(py.pychips.advanced.set_contour_numlevels, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_style);
       else
         py_call(py.pychips.advanced.set_contour_style, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_thickness);
       else
         py_call(py.pychips.advanced.set_contour_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_contour_transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_transform, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_contour_transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_contour_transform);
       else
         py_call(py.pychips.advanced.set_contour_transform, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_baddatamode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_baddatamode, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_baddatamode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_baddatamode);
       else
         py_call(py.pychips.advanced.set_curve_baddatamode, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_depth);
       else
         py_call(py.pychips.advanced.set_curve_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_errcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errcolor);
       else
         py_call(py.pychips.advanced.set_curve_errcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_errdown ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errdown, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errdown, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errdown);
       else
         py_call(py.pychips.advanced.set_curve_errdown, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_errleft ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errleft, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errleft, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errleft);
       else
         py_call(py.pychips.advanced.set_curve_errleft, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_errright ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errright, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errright, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errright);
       else
         py_call(py.pychips.advanced.set_curve_errright, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_errstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errstyle);
       else
         py_call(py.pychips.advanced.set_curve_errstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_errthickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errthickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errthickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errthickness);
       else
         py_call(py.pychips.advanced.set_curve_errthickness, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_errup ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errup, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errup, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errup);
       else
         py_call(py.pychips.advanced.set_curve_errup, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_limitlength ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_limitlength, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_limitlength, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_limitlength);
       else
         py_call(py.pychips.advanced.set_curve_limitlength, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_limitoverride ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_limitoverride, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_limitoverride, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_limitoverride);
       else
         py_call(py.pychips.advanced.set_curve_limitoverride, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_linecolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_linecolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_linecolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_linecolor);
       else
         py_call(py.pychips.advanced.set_curve_linecolor, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_linestyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_linestyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_linestyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_linestyle);
       else
         py_call(py.pychips.advanced.set_curve_linestyle, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_linethickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_linethickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_linethickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_linethickness);
       else
         py_call(py.pychips.advanced.set_curve_linethickness, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_symbolangle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolangle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_symbolangle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolangle);
       else
         py_call(py.pychips.advanced.set_curve_symbolangle, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_symbolcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_symbolcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolcolor);
       else
         py_call(py.pychips.advanced.set_curve_symbolcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_symbolfill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolfill, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_symbolfill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolfill);
       else
         py_call(py.pychips.advanced.set_curve_symbolfill, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_symbolsize ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolsize, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_symbolsize, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolsize);
       else
         py_call(py.pychips.advanced.set_curve_symbolsize, PY_KW,  __push_args(kwargs));
     }
}

define set_curve_symbolstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_symbolstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_symbolstyle);
       else
         py_call(py.pychips.advanced.set_curve_symbolstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_error_verbosity ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_error_verbosity, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_error_verbosity, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_error_verbosity);
       else
         py_call(py.pychips.advanced.set_error_verbosity, PY_KW,  __push_args(kwargs));
     }
}

define set_frame_bgcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_bgcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_frame_bgcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_bgcolor);
       else
         py_call(py.pychips.advanced.set_frame_bgcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_frame_border_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_border_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_frame_border_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_border_color);
       else
         py_call(py.pychips.advanced.set_frame_border_color, PY_KW,  __push_args(kwargs));
     }
}

define set_frame_scale ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_scale, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_frame_scale, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_scale);
       else
         py_call(py.pychips.advanced.set_frame_scale, PY_KW,  __push_args(kwargs));
     }
}

define set_frame_transparency ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_transparency, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_frame_transparency, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_frame_transparency);
       else
         py_call(py.pychips.advanced.set_frame_transparency, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_depth);
       else
         py_call(py.pychips.advanced.set_histogram_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_dropline ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_dropline, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_dropline, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_dropline);
       else
         py_call(py.pychips.advanced.set_histogram_dropline, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_errcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_errcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errcolor);
       else
         py_call(py.pychips.advanced.set_histogram_errcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_errdown ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errdown, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_errdown, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errdown);
       else
         py_call(py.pychips.advanced.set_histogram_errdown, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_errstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_errstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errstyle);
       else
         py_call(py.pychips.advanced.set_histogram_errstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_errthickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errthickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_errthickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errthickness);
       else
         py_call(py.pychips.advanced.set_histogram_errthickness, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_errup ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errup, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_errup, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errup);
       else
         py_call(py.pychips.advanced.set_histogram_errup, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_fillstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_fillstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_fillstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_fillstyle);
       else
         py_call(py.pychips.advanced.set_histogram_fillstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_fillcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_fillcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_fillcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_fillcolor);
       else
         py_call(py.pychips.advanced.set_histogram_fillcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_fillopacity ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_fillopacity, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_fillopacity, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_fillopacity);
       else
         py_call(py.pychips.advanced.set_histogram_fillopacity, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_linecolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_linecolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_linecolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_linecolor);
       else
         py_call(py.pychips.advanced.set_histogram_linecolor, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_linestyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_linestyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_linestyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_linestyle);
       else
         py_call(py.pychips.advanced.set_histogram_linestyle, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_linethickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_linethickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_linethickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_linethickness);
       else
         py_call(py.pychips.advanced.set_histogram_linethickness, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_symbolangle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolangle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_symbolangle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolangle);
       else
         py_call(py.pychips.advanced.set_histogram_symbolangle, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_symbolcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_symbolcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolcolor);
       else
         py_call(py.pychips.advanced.set_histogram_symbolcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_symbolfill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolfill, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_symbolfill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolfill);
       else
         py_call(py.pychips.advanced.set_histogram_symbolfill, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_symbolsize ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolsize, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_symbolsize, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolsize);
       else
         py_call(py.pychips.advanced.set_histogram_symbolsize, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_symbolstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_symbolstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_symbolstyle);
       else
         py_call(py.pychips.advanced.set_histogram_symbolstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_label_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_angle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_angle);
       else
         py_call(py.pychips.advanced.set_label_angle, PY_KW,  __push_args(kwargs));
     }
}

define set_label_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_color);
       else
         py_call(py.pychips.advanced.set_label_color, PY_KW,  __push_args(kwargs));
     }
}

define set_label_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_depth);
       else
         py_call(py.pychips.advanced.set_label_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_label_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_font, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_font);
       else
         py_call(py.pychips.advanced.set_label_font, PY_KW,  __push_args(kwargs));
     }
}

define set_label_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_fontstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_fontstyle);
       else
         py_call(py.pychips.advanced.set_label_fontstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_label_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_halign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_halign);
       else
         py_call(py.pychips.advanced.set_label_halign, PY_KW,  __push_args(kwargs));
     }
}

define set_label_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_size);
       else
         py_call(py.pychips.advanced.set_label_size, PY_KW,  __push_args(kwargs));
     }
}

define set_label_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_valign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_label_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_label_valign);
       else
         py_call(py.pychips.advanced.set_label_valign, PY_KW,  __push_args(kwargs));
     }
}

define set_line_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_line_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_color);
       else
         py_call(py.pychips.advanced.set_line_color, PY_KW,  __push_args(kwargs));
     }
}

define set_line_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_line_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_depth);
       else
         py_call(py.pychips.advanced.set_line_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_line_extend ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_extend, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_line_extend, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_extend);
       else
         py_call(py.pychips.advanced.set_line_extend, PY_KW,  __push_args(kwargs));
     }
}

define set_line_stipple ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
  variable err;
  try (err){
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_stipple, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_line_stipple, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_stipple);
       else
         py_call(py.pychips.advanced.set_line_stipple, PY_KW,  __push_args(kwargs));
     }
   }catch TypeMismatchError:
   {
      throw TypeMismatchError, "Incorrect usage. Try - set_line_stipple (linetype, pattern)", err.object;
   };
}

define set_line_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_line_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_style);
       else
         py_call(py.pychips.advanced.set_line_style, PY_KW,  __push_args(kwargs));
     }
}

define set_line_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_line_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_line_thickness);
       else
         py_call(py.pychips.advanced.set_line_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_axisstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_axisstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_axisstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_axisstyle);
       else
         py_call(py.pychips.advanced.set_plot_axisstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_bottommargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_bottommargin, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_bottommargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_bottommargin);
       else
         py_call(py.pychips.advanced.set_plot_bottommargin, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_cornerstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_cornerstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_cornerstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_cornerstyle);
       else
         py_call(py.pychips.advanced.set_plot_cornerstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_leftmargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_leftmargin, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_leftmargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_leftmargin);
       else
         py_call(py.pychips.advanced.set_plot_leftmargin, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_rightmargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_rightmargin, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_rightmargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_rightmargin);
       else
         py_call(py.pychips.advanced.set_plot_rightmargin, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_angle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_angle);
       else
         py_call(py.pychips.advanced.set_plot_title_angle, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_color);
       else
         py_call(py.pychips.advanced.set_plot_title_color, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_depth);
       else
         py_call(py.pychips.advanced.set_plot_title_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_font, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_font);
       else
         py_call(py.pychips.advanced.set_plot_title_font, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_fontstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_fontstyle);
       else
         py_call(py.pychips.advanced.set_plot_title_fontstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_halign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_halign);
       else
         py_call(py.pychips.advanced.set_plot_title_halign, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_size);
       else
         py_call(py.pychips.advanced.set_plot_title_size, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_valign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_valign);
       else
         py_call(py.pychips.advanced.set_plot_title_valign, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_xpos ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_xpos, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_xpos, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_xpos);
       else
         py_call(py.pychips.advanced.set_plot_title_xpos, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_title_ypos ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_ypos, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_title_ypos, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_title_ypos);
       else
         py_call(py.pychips.advanced.set_plot_title_ypos, PY_KW,  __push_args(kwargs));
     }
}

define set_plot_topmargin ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_topmargin, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_plot_topmargin, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_plot_topmargin);
       else
         py_call(py.pychips.advanced.set_plot_topmargin, PY_KW,  __push_args(kwargs));
     }
}

define set_point_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_angle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_point_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_angle);
       else
         py_call(py.pychips.advanced.set_point_angle, PY_KW,  __push_args(kwargs));
     }
}

define set_point_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_point_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_color);
       else
         py_call(py.pychips.advanced.set_point_color, PY_KW,  __push_args(kwargs));
     }
}

define set_point_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_point_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_depth);
       else
         py_call(py.pychips.advanced.set_point_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_point_fill ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_fill, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_point_fill, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_fill);
       else
         py_call(py.pychips.advanced.set_point_fill, PY_KW,  __push_args(kwargs));
     }
}

define set_point_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_point_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_size);
       else
         py_call(py.pychips.advanced.set_point_size, PY_KW,  __push_args(kwargs));
     }
}

define set_point_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_point_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_point_style);
       else
         py_call(py.pychips.advanced.set_point_style, PY_KW,  __push_args(kwargs));
     }
}

define set_region_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_region_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_depth);
       else
         py_call(py.pychips.advanced.set_region_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_region_edgecolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_edgecolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_region_edgecolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_edgecolor);
       else
         py_call(py.pychips.advanced.set_region_edgecolor, PY_KW,  __push_args(kwargs));
     }
}

define set_region_edgestyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_edgestyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_region_edgestyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_edgestyle);
       else
         py_call(py.pychips.advanced.set_region_edgestyle, PY_KW,  __push_args(kwargs));
     }
}

define set_region_edgethickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.set_region_edgethickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.set_region_edgethickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.set_region_edgethickness);
       else
         _py_call(py.pychips.advanced.set_region_edgethickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define set_region_fillstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_fillstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_region_fillstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_fillstyle);
       else
         py_call(py.pychips.advanced.set_region_fillstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_region_fillcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_fillcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_region_fillcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_fillcolor);
       else
         py_call(py.pychips.advanced.set_region_fillcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_region_opacity ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_opacity, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_region_opacity, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_region_opacity);
       else
         py_call(py.pychips.advanced.set_region_opacity, PY_KW,  __push_args(kwargs));
     }
}

define set_window_bgcolor ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_window_bgcolor, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_window_bgcolor, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_window_bgcolor);
       else
         py_call(py.pychips.advanced.set_window_bgcolor, PY_KW,  __push_args(kwargs));
     }
}

define set_window_redraw ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_window_redraw, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_window_redraw, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_window_redraw);
       else
         py_call(py.pychips.advanced.set_window_redraw, PY_KW,  __push_args(kwargs));
     }
}

define stop_blink ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.stop_blink, __push_args (f_args));
        else
         py_call(py.pychips.advanced.stop_blink, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.stop_blink);
       else
         py_call(py.pychips.advanced.stop_blink, PY_KW,  __push_args(kwargs));
     }
}

define apply_contour_transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.apply_contour_transform, __push_args (f_args));
        else
         py_call(py.pychips.advanced.apply_contour_transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.apply_contour_transform);
       else
         py_call(py.pychips.advanced.apply_contour_transform, PY_KW,  __push_args(kwargs));
     }
}

define apply_contour_transform_inverse ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.apply_contour_transform_inverse, __push_args (f_args));
        else
         py_call(py.pychips.advanced.apply_contour_transform_inverse, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.apply_contour_transform_inverse);
       else
         py_call(py.pychips.advanced.apply_contour_transform_inverse, PY_KW,  __push_args(kwargs));
     }
}

define get_contour_transform ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_transform, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_contour_transform, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_contour_transform);
       else
         _py_call(py.pychips.advanced.get_contour_transform, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define valid_axisstyle_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_axisstyle_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_axisstyle_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_axisstyle_type);
       else
         py_call(py.pychips.advanced.valid_axisstyle_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_baddatamode_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_baddatamode_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_baddatamode_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_baddatamode_type);
       else
         py_call(py.pychips.advanced.valid_baddatamode_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_color);
       else
         py_call(py.pychips.advanced.valid_color, PY_KW,  __push_args(kwargs));
     }
}

define valid_colormap_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_colormap_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_colormap_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_colormap_type);
       else
         py_call(py.pychips.advanced.valid_colormap_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_contour_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_contour_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_contour_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_contour_type);
       else
         py_call(py.pychips.advanced.valid_contour_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_coord_sys_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_coord_sys_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_coord_sys_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_coord_sys_type);
       else
         py_call(py.pychips.advanced.valid_coord_sys_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_cornerstyle_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_cornerstyle_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_cornerstyle_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_cornerstyle_type);
       else
         py_call(py.pychips.advanced.valid_cornerstyle_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_errstyle_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_errstyle_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_errstyle_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_errstyle_type);
       else
         py_call(py.pychips.advanced.valid_errstyle_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_fill_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_fill_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_fill_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_fill_type);
       else
         py_call(py.pychips.advanced.valid_fill_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_font_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_font_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_font_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_font_type);
       else
         py_call(py.pychips.advanced.valid_font_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_fontstyle_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_fontstyle_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_fontstyle_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_fontstyle_type);
       else
         py_call(py.pychips.advanced.valid_fontstyle_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_interpolation_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_interpolation_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_interpolation_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_interpolation_type);
       else
         py_call(py.pychips.advanced.valid_interpolation_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_line_extension_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_line_extension_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_line_extension_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_line_extension_type);
       else
         py_call(py.pychips.advanced.valid_line_extension_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_line_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_line_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_line_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_line_type);
       else
         py_call(py.pychips.advanced.valid_line_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_object_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_object_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_object_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_object_type);
       else
         py_call(py.pychips.advanced.valid_object_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_printfile_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_printfile_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_printfile_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_printfile_type);
       else
         py_call(py.pychips.advanced.valid_printfile_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_rgba_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_rgba_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_rgba_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_rgba_type);
       else
         py_call(py.pychips.advanced.valid_rgba_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_shuffle_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_shuffle_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_shuffle_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_shuffle_type);
       else
         py_call(py.pychips.advanced.valid_shuffle_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_symbol_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_symbol_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_symbol_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_symbol_type);
       else
         py_call(py.pychips.advanced.valid_symbol_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_tickmode_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_tickmode_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_tickmode_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_tickmode_type);
       else
         py_call(py.pychips.advanced.valid_tickmode_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_tickstyle_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_tickstyle_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_tickstyle_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_tickstyle_type);
       else
         py_call(py.pychips.advanced.valid_tickstyle_type, PY_KW,  __push_args(kwargs));
     }
}

define valid_units_type ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.valid_units_type, __push_args (f_args));
        else
         py_call(py.pychips.advanced.valid_units_type, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.valid_units_type);
       else
         py_call(py.pychips.advanced.valid_units_type, PY_KW,  __push_args(kwargs));
     }
}

define set_image_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_depth);
       else
         py_call(py.pychips.advanced.set_image_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_image_alpha ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_alpha, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_alpha, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_alpha);
       else
         py_call(py.pychips.advanced.set_image_alpha, PY_KW,  __push_args(kwargs));
     }
}

define set_image_interpolation ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_interpolation, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_interpolation, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_interpolation);
       else
         py_call(py.pychips.advanced.set_image_interpolation, PY_KW,  __push_args(kwargs));
     }
}

define set_image_invert_colormap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_invert_colormap, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_invert_colormap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_invert_colormap);
       else
         py_call(py.pychips.advanced.set_image_invert_colormap, PY_KW,  __push_args(kwargs));
     }
}

define set_image_colormap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_colormap, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_colormap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_colormap);
       else
         py_call(py.pychips.advanced.set_image_colormap, PY_KW,  __push_args(kwargs));
     }
}

define set_image_scale_channels ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_scale_channels, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_scale_channels, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_scale_channels);
       else
         py_call(py.pychips.advanced.set_image_scale_channels, PY_KW,  __push_args(kwargs));
     }
}

define set_image_colormap_interpolate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_colormap_interpolate, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_colormap_interpolate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_colormap_interpolate);
       else
         py_call(py.pychips.advanced.set_image_colormap_interpolate, PY_KW,  __push_args(kwargs));
     }
}

define set_image_threshold ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_threshold, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_threshold, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_threshold);
       else
         py_call(py.pychips.advanced.set_image_threshold, PY_KW,  __push_args(kwargs));
     }
}

define set_image_colormap_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_colormap_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_image_colormap_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_image_colormap_size);
       else
         py_call(py.pychips.advanced.set_image_colormap_size, PY_KW,  __push_args(kwargs));
     }
}

define get_image_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_depth);
       else
         _py_call(py.pychips.advanced.get_image_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_alpha ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_alpha, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_alpha, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_alpha);
       else
         _py_call(py.pychips.advanced.get_image_alpha, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_interpolation ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_interpolation, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_interpolation, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_interpolation);
       else
         _py_call(py.pychips.advanced.get_image_interpolation, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_invert_colormap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_invert_colormap, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_invert_colormap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_invert_colormap);
       else
         _py_call(py.pychips.advanced.get_image_invert_colormap, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_colormap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_colormap, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_colormap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_colormap);
       else
         _py_call(py.pychips.advanced.get_image_colormap, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_scale_channels ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_scale_channels, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_scale_channels, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_scale_channels);
       else
         _py_call(py.pychips.advanced.get_image_scale_channels, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_colormap_interpolate ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_colormap_interpolate, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_colormap_interpolate, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_colormap_interpolate);
       else
         _py_call(py.pychips.advanced.get_image_colormap_interpolate, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_threshold ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_threshold, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_threshold, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_threshold);
       else
         _py_call(py.pychips.advanced.get_image_threshold, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_image_colormap_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_colormap_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_image_colormap_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_image_colormap_size);
       else
         _py_call(py.pychips.advanced.get_image_colormap_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define set_colorbar_tick_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_length, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_tick_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_length);
       else
         py_call(py.pychips.advanced.set_colorbar_tick_length, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_depth, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_depth);
       else
         py_call(py.pychips.advanced.set_colorbar_depth, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_tick_location ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_location, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_tick_location, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_location);
       else
         py_call(py.pychips.advanced.set_colorbar_tick_location, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_halign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_halign);
       else
         py_call(py.pychips.advanced.set_colorbar_halign, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_tick_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_mode, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_tick_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_mode);
       else
         py_call(py.pychips.advanced.set_colorbar_tick_mode, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_angle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_angle);
       else
         py_call(py.pychips.advanced.set_colorbar_label_angle, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_tick_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_style, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_tick_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_style);
       else
         py_call(py.pychips.advanced.set_colorbar_tick_style, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_color);
       else
         py_call(py.pychips.advanced.set_colorbar_label_color, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_tick_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_thickness, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_tick_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_thickness);
       else
         py_call(py.pychips.advanced.set_colorbar_tick_thickness, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_font, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_font);
       else
         py_call(py.pychips.advanced.set_colorbar_label_font, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_tick_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_visible, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_tick_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_visible);
       else
         py_call(py.pychips.advanced.set_colorbar_tick_visible, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_fontstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_fontstyle);
       else
         py_call(py.pychips.advanced.set_colorbar_label_fontstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_angle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_angle);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_angle, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_halign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_halign);
       else
         py_call(py.pychips.advanced.set_colorbar_label_halign, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_color);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_color, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_location ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_location, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_location, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_location);
       else
         py_call(py.pychips.advanced.set_colorbar_label_location, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_font, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_font);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_font, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_size);
       else
         py_call(py.pychips.advanced.set_colorbar_label_size, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_fontstyle, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_fontstyle);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_fontstyle, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_text ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_text, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_text, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_text);
       else
         py_call(py.pychips.advanced.set_colorbar_label_text, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_halign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_halign);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_halign, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_valign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_valign);
       else
         py_call(py.pychips.advanced.set_colorbar_label_valign, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_offset, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_offset);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_offset, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_label_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_visible, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_label_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_label_visible);
       else
         py_call(py.pychips.advanced.set_colorbar_label_visible, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_size, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_size);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_size, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_length, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_length);
       else
         py_call(py.pychips.advanced.set_colorbar_length, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_valign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_valign);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_valign, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_orientation ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_orientation, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_orientation, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_orientation);
       else
         py_call(py.pychips.advanced.set_colorbar_orientation, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_ticklabel_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_visible, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_ticklabel_visible);
       else
         py_call(py.pychips.advanced.set_colorbar_ticklabel_visible, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_parallel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_parallel_offset, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_parallel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_parallel_offset);
       else
         py_call(py.pychips.advanced.set_colorbar_parallel_offset, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_valign, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_valign);
       else
         py_call(py.pychips.advanced.set_colorbar_valign, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_perpendicular_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_perpendicular_offset, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_perpendicular_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_perpendicular_offset);
       else
         py_call(py.pychips.advanced.set_colorbar_perpendicular_offset, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_width ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_width, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_width, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_width);
       else
         py_call(py.pychips.advanced.set_colorbar_width, PY_KW,  __push_args(kwargs));
     }
}

define set_colorbar_tick_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_color, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_colorbar_tick_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_colorbar_tick_color);
       else
         py_call(py.pychips.advanced.set_colorbar_tick_color, PY_KW,  __push_args(kwargs));
     }
}

define get_colorbar_tick_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_length, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_tick_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_length);
       else
         _py_call(py.pychips.advanced.get_colorbar_tick_length, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_depth ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_depth, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_depth, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_depth);
       else
         _py_call(py.pychips.advanced.get_colorbar_depth, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_tick_location ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_location, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_tick_location, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_location);
       else
         _py_call(py.pychips.advanced.get_colorbar_tick_location, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_halign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_halign);
       else
         _py_call(py.pychips.advanced.get_colorbar_halign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_tick_mode ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_mode, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_tick_mode, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_mode);
       else
         _py_call(py.pychips.advanced.get_colorbar_tick_mode, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_angle);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_tick_style ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_style, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_tick_style, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_style);
       else
         _py_call(py.pychips.advanced.get_colorbar_tick_style, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_color);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_tick_thickness ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_thickness, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_tick_thickness, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_thickness);
       else
         _py_call(py.pychips.advanced.get_colorbar_tick_thickness, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_font, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_font);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_font, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_tick_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_tick_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_visible);
       else
         _py_call(py.pychips.advanced.get_colorbar_tick_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_fontstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_fontstyle);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_fontstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_angle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_angle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_angle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_angle);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_angle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_halign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_halign);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_halign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_color);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_location ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_location, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_location, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_location);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_location, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_font ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_font, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_font, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_font);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_font, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_size);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_fontstyle ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_fontstyle, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_fontstyle, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_fontstyle);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_fontstyle, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_text ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_text, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_text, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_text);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_text, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_halign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_halign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_halign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_halign);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_halign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_valign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_valign);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_valign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_offset, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_offset);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_offset, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_label_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_label_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_label_visible);
       else
         _py_call(py.pychips.advanced.get_colorbar_label_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_size ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_size, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_size, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_size);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_size, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_length ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_length, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_length, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_length);
       else
         _py_call(py.pychips.advanced.get_colorbar_length, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_valign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_valign);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_valign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_orientation ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_orientation, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_orientation, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_orientation);
       else
         _py_call(py.pychips.advanced.get_colorbar_orientation, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_ticklabel_visible ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_visible, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_visible, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_visible);
       else
         _py_call(py.pychips.advanced.get_colorbar_ticklabel_visible, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_parallel_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_parallel_offset, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_parallel_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_parallel_offset);
       else
         _py_call(py.pychips.advanced.get_colorbar_parallel_offset, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_valign ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_valign, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_valign, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_valign);
       else
         _py_call(py.pychips.advanced.get_colorbar_valign, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_perpendicular_offset ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_perpendicular_offset, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_perpendicular_offset, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_perpendicular_offset);
       else
         _py_call(py.pychips.advanced.get_colorbar_perpendicular_offset, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_width ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_width, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_width, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_width);
       else
         _py_call(py.pychips.advanced.get_colorbar_width, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_colorbar_tick_color ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_color, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_colorbar_tick_color, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_colorbar_tick_color);
       else
         _py_call(py.pychips.advanced.get_colorbar_tick_color, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define set_curve_errcaplength ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errcaplength, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_curve_errcaplength, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_curve_errcaplength);
       else
         py_call(py.pychips.advanced.set_curve_errcaplength, PY_KW,  __push_args(kwargs));
     }
}

define set_histogram_errcaplength ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errcaplength, __push_args (f_args));
        else
         py_call(py.pychips.advanced.set_histogram_errcaplength, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.set_histogram_errcaplength);
       else
         py_call(py.pychips.advanced.set_histogram_errcaplength, PY_KW,  __push_args(kwargs));
     }
}

define get_curve_errcaplength ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errcaplength, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_curve_errcaplength, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_curve_errcaplength);
       else
         _py_call(py.pychips.advanced.get_curve_errcaplength, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define get_histogram_errcaplength ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errcaplength, __push_args (f_args));
        else
         _py_call(py.pychips.advanced.get_histogram_errcaplength, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         _py_call(py.pychips.advanced.get_histogram_errcaplength);
       else
         _py_call(py.pychips.advanced.get_histogram_errcaplength, PY_KW,  __push_args(kwargs));
     }
     return check_return ();
}

define string_to_ratio_values ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_ratio_values, __push_args (f_args));
        else
         py_call(py.pychips.advanced.string_to_ratio_values, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_ratio_values);
       else
         py_call(py.pychips.advanced.string_to_ratio_values, PY_KW,  __push_args(kwargs));
     }
}

define string_to_v_interpolation ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_v_interpolation, __push_args (f_args));
        else
         py_call(py.pychips.advanced.string_to_v_interpolation, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_v_interpolation);
       else
         py_call(py.pychips.advanced.string_to_v_interpolation, PY_KW,  __push_args(kwargs));
     }
}

define string_to_v_colormap ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_v_colormap, __push_args (f_args));
        else
         py_call(py.pychips.advanced.string_to_v_colormap, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_v_colormap);
       else
         py_call(py.pychips.advanced.string_to_v_colormap, PY_KW,  __push_args(kwargs));
     }
}

define string_to_v_rgba ()
{
#if (_slang_version >= 20101)
    variable kwargs = _chips_get_qualifiers(__qualifiers());
#else
    variable kwargs = NULL;
#endif
     if (_NARGS != 0){
        variable f_args = __pop_args (_NARGS);
        variable ii, list_conv;
        _for (0, length (f_args)-1,1){
           ii = ();
           list_conv = _chips_struct_to_list (f_args[ii].value);
           if (list_conv != NULL)
              f_args[ii].value = list_conv;
           else
              f_args[ii].value = _chips_check_type (f_args[ii].value);
        }
        if( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_v_rgba, __push_args (f_args));
        else
         py_call(py.pychips.advanced.string_to_v_rgba, __push_args (f_args), PY_KW,  __push_args(kwargs));
     }
     else{
       if ( kwargs == NULL )
         py_call(py.pychips.advanced.string_to_v_rgba);
       else
         py_call(py.pychips.advanced.string_to_v_rgba, PY_KW,  __push_args(kwargs));
     }
}

$1 = path_concat (path_dirname (__FILE__), "help/chipsfuns_advanced.hlp");
if (NULL != stat_file ($1))
  add_doc_file ($1);
provide ("chips_advanced");