# 
#  Copyright (C) 2008,2013  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


import pychips_base as __pychips_base__

import utils as __utils__
from attribute_objects import *
import os
import pytransform
import numpy


__all__ = ["get_attribute", "set_attribute", "get_window", "set_window",
           "get_frame", "set_frame", "get_plot", "set_plot", "get_axis",
           "get_xaxis", "get_yaxis", "set_axis", "set_xaxis", "set_yaxis",
           "get_curve", "set_curve", "get_histogram", "set_histogram",
           "get_contour", "set_contour", "get_label", "set_label",
           "get_line", "set_line", "get_point", "set_point", "get_region",
           "set_region", "get_image", "set_image", "get_preferences",
           "get_colorbar", "set_colorbar", "current_colorbar", "shuffle_colorbar",
           "set_preferences", "set_current", "current_window", "current_frame",
           "current_plot", "current_axis", "current_curve",  "current_histogram",
           "current_contour", "current_label", "current_line", "current_point",
           "current_region", "current_image", "clear", "erase", "strip_chart",
           "move_frame", "move_plot", "move_axis", "move_label", "move_line",
           "move_point", "move_region", "shuffle_axis", "move_colorbar", "shuffle_curve",
           "shuffle_contour", "shuffle_histogram", "shuffle_label", "shuffle_image",
           "shuffle_line", "shuffle_point", "shuffle_region", "split",
           "reindex_depth", "add_hline", "add_vline", "add_window",
           "add_frame", "add_label", "add_line", "add_point", "add_plotarea",
           "add_plot", "add_curve", "add_contour", "add_histogram", "add_colorbar",
           "add_axis", "add_region", "add_image", "print_window", "ChipsId", 
           "print_image", "convert_coordinate", "limits", "connect", "info",
           "info_coordinate", "info_depth", "info_bound_axes", "make_script", 
           "info_current", "set_image_channel", "remove_image_channel", 
           "load_ascii_transform", "set_arbitrary_tick_positions",
           "load_colormap"]


#####################################################
#
#  new set/get chips attribute functions
#
#####################################################

def get_attribute(id=None, object_type=None, property=None):
   'Returns an object containing attribute values of the specified chips object'
   result = None
   obj = object_type
   prop = property

   functions = {__pychips_base__.chips_window: get_window,
                __pychips_base__.chips_frame: get_frame,
                __pychips_base__.chips_plot: get_plot,
                __pychips_base__.chips_axis: get_axis,
                __pychips_base__.chips_curve: get_curve,
                __pychips_base__.chips_histogram: get_histogram,
                __pychips_base__.chips_contour: get_contour,
                __pychips_base__.chips_image: get_image,
                __pychips_base__.chips_colorbar: get_colorbar,
                __pychips_base__.chips_label: get_label,
                __pychips_base__.chips_line: get_line,
                __pychips_base__.chips_point: get_point,
                __pychips_base__.chips_region: get_region}
 
   if isinstance (id,str):
      cid = __pychips_base__.ChipsId()
      if object_type == None:
         raise AttributeError("chips ERROR: id specified without specifying the object type")
      else:
         cid.set_object_id(obj, id)
   elif isinstance(id, __pychips_base__.ChipsId):
      cid = id
      if object_type == None:
         raise AttributeError("chips ERROR: id specified without specifying the object type")
   elif id == None:
      if not (object_type in functions):
         raise AttributeError("chips ERROR: Invalid object type specified")
      cid = __pychips_base__.ChipsId()
      obj = object_type
      prop = property
   elif id in functions and property == None:
      cid = None
      obj = id
      prop = object_type
   else:
      raise AttributeError("chips ERROR: First parameter must be a ChipsId, id string or object_type")

   if not (obj in functions):
      try:
         raise AttributeError("chips ERROR: Unknown object type (%d) specified in get_attribute" % obj)
      except:
         raise AttributeError("chips ERROR: Invalid object type specified. Exepected an integer or chips_object.")
   result = functions[obj] (cid, prop)
   return result


def set_attribute(cid=None, object_type=None, values=None):
   'Sets the attributes of the specified chips object with the provided values'

   if values == None:  # handle values without ids
      if cid == None or object_type == None:
         raise AttributeError("chips ERROR: Must specify an object type and value")
      oid = None
      obj= cid
      vals = object_type
   else:
      oid = cid
      obj = object_type
      vals = values 
   result = None
   if obj == __pychips_base__.chips_window:
      result = set_window(oid, vals)
   elif obj == __pychips_base__.chips_frame:
      result = set_frame(oid, vals)
   elif obj == __pychips_base__.chips_plot:
      result = set_plot(oid, vals)
   elif obj == __pychips_base__.chips_axis:
      result = set_axis(oid, vals)
   elif obj == __pychips_base__.chips_curve:
      result = set_curve(oid, vals)
   elif obj == __pychips_base__.chips_histogram:
      result = set_histogram(oid, vals)
   elif obj == __pychips_base__.chips_contour:
      result = set_contour(oid, vals)
   elif obj == __pychips_base__.chips_image:
      result = set_image(oid, vals)
   elif obj == __pychips_base__.chips_colorbar:
      result = set_colorbar(oid, vals)
   elif obj == __pychips_base__.chips_label:
      result = set_label(oid, vals)
   elif obj == __pychips_base__.chips_line:
      result = set_line(oid, vals)
   elif obj == __pychips_base__.chips_point:
      result = set_point(oid, vals)
   elif obj == __pychips_base__.chips_region:
      result = set_region(oid, vals)
   else:
      raise AttributeError("chips ERROR: Unknown object type (%d) specified in set_attribute" % __pychips_base__.v_object_to_string(obtype)) 
   return result


def get_window(id=None, property=None):
   'Returns a ChipsWindow object containing attribute values of requested window'
   if type(id) == str:
      wid = __pychips_base__.ChipsId()
      wid.set_window_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      wid = id
   elif id == None:
      wid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return proprety structure
      result = ChipsWindow()
      result.display = __pychips_base__.get_window_redraw(wid)
      result.bgcolor = __pychips_base__.get_window_bgcolor(wid)
      return result
   else:
      # return specific property value
      if (property.find('window.') == 0):       # remove preceeding 'window.'
         property = property.replace('window.','')
      if property == 'bgcolor':
         val = __pychips_base__.get_window_bgcolor(wid)
      elif property == 'display':
         val = __pychips_base__.get_window_redraw(wid)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;
      

def set_window(id=None, values=None):
   'Set the attributes of the specified window to the provided values'
   # use __pychips_base__.ChipsId, string window id, or no id (current) to determine window 
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   if type(mid) == str:
      wid = __pychips_base__.ChipsId()
      wid.set_window_id(mid)
   elif isinstance(mid, __pychips_base__.ChipsId):
      wid = mid
   elif mid == None:
      wid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")      
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsWindow object
   wob = ChipsWindow ()

   try:
      wob = __utils__.data_to_attribute_object (wob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (wob)
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_window with %s "%type(wob))


   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(wob, 'bgcolor') and wob.bgcolor != None:
         # handle color as a string or integer (hex value rrggbb)
         __pychips_base__.set_window_bgcolor(wid, __utils__.get_color_as_string (wob.bgcolor))

      if hasattr(wob, 'display') and wob.display != None:
         __pychips_base__.set_window_redraw(wid, __utils__.bool_cast(wob.display))

   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
   return

def load_colormap(arg1, arg2=None, arg3=None, arg4=None, arg5=None):
   """
load_colormap(r, g, b, [alpha], [slot])
load_colormap(filename, [slot])
load_colormap(2d_array, [slot])

Loads the specified colormap into the chips_usercmap1, chips_usercmap2, or chips_usercmap3 slot. Input can be either a file name, four arrays of red, green, blue, and alpha values, or a 2d array containing rows of red, green, blue, and optional alpha values. If the slot is not provided, chips_usercmap1 is used. If an alpha is not provided, a constant level of one is used.
   """
   #Check for a 2D array of values.
   #If it's a 2D array, it's expected to have 3 or 4 columns.
   if hasattr(arg1, "__iter__") and hasattr(arg1[0], "__iter__"):  #2D array of traditional
      #check arg2 to see if a slot is specified
      red = []
      green = []
      blue = []
      alpha = []
      if isinstance(arg2, (int, long)): #slot
         slot = arg2
      elif arg2 == None:
         slot=__pychips_base__.string_to_v_colormap("usercmap1")
      else:
         raise AttributeError('chips ERROR: Expected int or none for second argument if first argument is a 2-D array')
      if len(arg1[0]) == 3:
         for element in arg1:
            red.append(element[0])
            green.append(element[1])
            blue.append(element[2])
            alpha.append(1)
      elif len(arg1[0]) == 4:
         for element in arg1:
            red.append(element[0])
            green.append(element[1])
            blue.append(element[2])
            alpha.append(element[3])
      else:
         raise AttributeError('chips ERROR: Unexpected column width in 2-D array. '\
            'Accepted widths are three or four')
      __pychips_base__.load_colormap(len(red), red, green, blue, alpha, slot)
   elif isinstance(arg1, numpy.ndarray) and len(arg1.shape) != 1:
      red = []
      green = []
      blue = []
      alpha = []
      if isinstance(arg2, (int, long)): #slot
         slot = arg2
      elif arg2 == None:
         slot=__pychips_base__.string_to_v_colormap("usercmap1")
      else:
         raise AttributeError('chips ERROR: Expected int or none for second argument if first argument is a 2-D array')       
      if arg1.shape[1] == 3:
         for element in arg1:
            red.append(element[0])
            green.append(element[1])
            blue.append(element[2])
            alpha.append(1)
      elif arg1.shape[1] == 4:
         for element in arg1:
            red.append(element[0])
            green.append(element[1])
            blue.append(element[2])
            alpha.append(element[3])
      else:
        raise AttributeError('chips ERROR: Unexpected column width in 2-D array. '\
           'Accepted widths are three or four')
      __pychips_base__.load_colormap(len(red), red, green, blue, alpha, slot)
   #Iterable implies some list, array, or tuple stucture
   elif hasattr(arg1, "__iter__"):
      red = arg1
      green = arg2
      blue = arg3
      if not hasattr(arg2, "__iter__"):
         raise AttributeError('chips ERROR: If first argument is an array, second argument is '\
            'expected to be an array.')
      if not hasattr(arg3, "__iter__"):
         raise AttributeError('chips ERROR: If first argument is an array, third argument is '\
            'expected to be an array.')
      try:
         if arg4 == None: #just RGB
            slot=__pychips_base__.string_to_v_colormap("usercmap1")
            alpha = [1] * len(red)
         elif hasattr(arg4, "__iter__"): #Should be a fourth array, alpha
            alpha = arg4
            if arg5:
               slot = arg5
            else:
               slot=__pychips_base__.string_to_v_colormap("usercmap1")
         elif __utils__.isinstance_int(arg4): #slot
            slot = arg4
            alpha = [1] * len(red)
         else:
            raise AttributeError("chips ERROR: Unknown or unexpected parameter type in argument 4. Expected an integer(colormap slot) or array of values (alpha).")
         if len(red) != len(green) or len(red) != len(blue) or len(red) != len(alpha):
            raise AttributeError("chips ERROR: Arrays must be the same length")
         if len(red) == 0:
            raise AttributeError("chips ERROR: Arrays cannot be length zero")
         __pychips_base__.load_colormap(len(red), red, green, blue, alpha, slot)
      except TypeError:
         raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
   elif isinstance(arg1, str): #File
      filename = arg1
      if arg2 == None:
         slot=__pychips_base__.string_to_v_colormap("usercmap1")
      elif __utils__.isinstance_int(arg2):
         slot=arg2
      else:
         raise AttributeError("chips ERROR: Expected int or none as second parameter if first parameter is a filename")
      __pychips_base__.load_colormap(filename, slot)
   else: #Input was not an expected type
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      
def get_frame(fid=None, property=None):
   'Returns a ChipsFrame object containing attribute values of requested frame'
   if type(fid) == str:
      id = __pychips_base__.ChipsId()
      id.set_frame_id(fid)
   elif isinstance(fid, __pychips_base__.ChipsId):
      id = fid
   elif fid == None:
      id = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsFrame()
      result.bgcolor = __pychips_base__.get_frame_bgcolor(id)
      result.border.color = __pychips_base__.get_frame_border_color(id)
      result.border.visible = __pychips_base__.get_frame_border_visible(id)
      result.transparency = __pychips_base__.get_frame_transparency(id)
      result.scale = __pychips_base__.get_frame_scale(id)
      return result
   else:
      # return specific property value
      if (property.find('frame') == 0):       # remove preceeding 'frame.'
         property = property.replace('frame.','')
      if property == 'bgcolor':
         val = __pychips_base__.get_frame_bgcolor(id)
      elif property == 'border.color':
         val = __pychips_base__.get_frame_border_color(id)
      elif property == 'border.visible':
         val = __pychips_base__.get_frame_border_visible(id)
      elif property == 'transparency':
         val = __pychips_base__.get_frame_transparency(id)
      elif property == 'scale':
         val = __pychips_base__.get_frame_scale(id)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;


def set_frame(id=None, values=None):
   'Set the attributes of the specified frame to the provided values'
   # use __pychips_base__.ChipsId, string frame id, or no id (current) to determine which frame

   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   if type(mid) == str:      
      oid = __pychips_base__.ChipsId()      
      oid.set_frame_id(mid)   
   elif isinstance(mid, __pychips_base__.ChipsId):
      oid = mid      
   elif mid == None:      
      oid = __pychips_base__.ChipsId()   
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")      
      return None            

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsFrame object
   ob = ChipsFrame ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob)
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_frame with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'bgcolor') and ob.bgcolor != None:
         # handle color as a string or integer (hex value rrggbb)
         __pychips_base__.set_frame_bgcolor(oid, __utils__.get_color_as_string (ob.bgcolor))
      if hasattr(ob, 'border'):
         if hasattr(ob.border, 'color') and ob.border.color != None:
            # handle color as a string or integer (hex value rrggbb)
            __pychips_base__.set_frame_border_color(oid, __utils__.get_color_as_string (ob.border.color))

         if hasattr(ob.border, 'visible') and ob.border.visible != None:
            if __utils__.bool_cast(ob.border.visible) == True:
               __pychips_base__.display_frame_border(oid)
            else:
               __pychips_base__.hide_frame_border(oid)
      if hasattr(ob, 'transparency') and ob.transparency != None:
         __pychips_base__.set_frame_transparency(oid, __utils__.bool_cast(ob.transparency))
      if hasattr(ob, 'scale') and ob.scale != None:
         __pychips_base__.set_frame_scale(oid, __utils__.bool_cast(ob.scale))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
   return


def get_plot(id=None, property=None):
   'Returns a ChipsPlot object containing attribute values of requested plot'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_plot_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsPlot();
      result.title.xpos = __pychips_base__.get_plot_title_xpos(oid)
      result.title.ypos = __pychips_base__.get_plot_title_ypos(oid)
      result.title.depth = __pychips_base__.get_plot_title_depth(oid)
      result.title.angle = __pychips_base__.get_plot_title_angle(oid)
      result.title.halign = __pychips_base__.get_plot_title_halign(oid)
      result.title.valign = __pychips_base__.get_plot_title_valign(oid)
      result.title.color = __pychips_base__.get_plot_title_color(oid)
      result.title.size = __pychips_base__.get_plot_title_size(oid)
      result.title.font = __pychips_base__.get_plot_title_font(oid)
      result.title.fontstyle = __pychips_base__.get_plot_title_fontstyle(oid)
      result.leftmargin = __pychips_base__.get_plot_leftmargin(oid)
      result.rightmargin = __pychips_base__.get_plot_rightmargin(oid)
      result.topmargin = __pychips_base__.get_plot_topmargin(oid)
      result.bottommargin = __pychips_base__.get_plot_bottommargin(oid)
      result.style = __pychips_base__.get_plot_axisstyle(oid)
      result.corner.style = __pychips_base__.get_plot_cornerstyle(oid)
      result.aspect.ratio = __pychips_base__.get_plot_aspect_ratio(); 
      return result
   else:
      # return specific property value
      if (property.find('plot') == 0):       # remove preceeding 'plot.'
         property = property.replace('plot.','')
      if property == 'title.xpos':
         val = __pychips_base__.get_plot_title_xpos(oid)
      elif property == 'title.ypos':
         val = __pychips_base__.get_plot_title_ypos(oid)
      elif property == 'title.angle':
         val = __pychips_base__.get_plot_title_angle(oid)
      elif property == 'title.depth':
         val = __pychips_base__.get_plot_title_depth(oid)
      elif property == 'title.halign':
         val = __pychips_base__.get_plot_title_halign(oid)
      elif property == 'title.valign':
         val = __pychips_base__.get_plot_title_valign(oid)
      elif property == 'title.color':
         val = __pychips_base__.get_plot_title_color(oid)
      elif property == 'title.size':
         val = __pychips_base__.get_plot_title_size(oid)
      elif property == 'title.font':
         val = __pychips_base__.get_plot_title_font(oid)
      elif property == 'title.fontstyle':
         val = __pychips_base__.get_plot_title_fontstyle(oid)
      elif property == 'leftmargin':
         val = __pychips_base__.get_plot_leftmargin(oid)
      elif property == 'rightmargin':
         val = __pychips_base__.get_plot_rightmargin(oid)
      elif property == 'topmargin':
         val = __pychips_base__.get_plot_topmargin(oid)
      elif property == 'bottommargin':
         val = __pychips_base__.get_plot_bottommargin(oid)
      elif property == 'style':
         val = __pychips_base__.get_plot_axisstyle(oid)
      elif property == 'corner.style':
         val = __pychips_base__.get_plot_cornerstyle(oid)
      elif property == 'aspect.ratio':
         val = __pychips_base__.get_plot_aspect_ratio(oid)
      else:
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
         val = None
      return val;

def set_plot(id=None, values=None):
   'Set the attributes of the specified plot to the provided values'
   # use __pychips_base__.ChipsId, string plot id, or no id (current) to determine plot
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   if type(mid) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_plot_id(mid)
   elif isinstance(mid, __pychips_base__.ChipsId):
      oid = mid
   elif mid == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsPlot object
   ob = ChipsPlot ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_plot with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'leftmargin') and ob.leftmargin != None:
         __pychips_base__.set_plot_leftmargin(oid, float (ob.leftmargin))
      if hasattr(ob, 'rightmargin') and ob.rightmargin != None:
         __pychips_base__.set_plot_rightmargin(oid, float (ob.rightmargin))
      if hasattr(ob, 'topmargin') and ob.topmargin != None:
         __pychips_base__.set_plot_topmargin(oid, float (ob.topmargin))
      if hasattr(ob, 'bottommargin') and ob.bottommargin != None:
         __pychips_base__.set_plot_bottommargin(oid, float (ob.bottommargin))
      if hasattr(ob, 'style') and ob.style != None:
         __pychips_base__.set_plot_axisstyle(oid, ob.style)
      if hasattr(ob, 'corner') and hasattr(ob.corner, 'style') and ob.corner.style != None:
         __pychips_base__.set_plot_cornerstyle(oid, ob.corner.style)
      if hasattr(ob, 'aspect') and hasattr(ob.aspect, 'ratio') and ob.aspect.ratio != None:
         __pychips_base__.set_plot_aspect_ratio(oid, ob.aspect.ratio)
      if hasattr(ob, 'title'):
         if hasattr(ob.title, 'xpos') and ob.title.xpos != None:
            __pychips_base__.set_plot_title_xpos(oid, float (ob.title.xpos))
         if hasattr(ob.title, 'ypos') and ob.title.ypos != None:
            __pychips_base__.set_plot_title_ypos(oid, float (ob.title.ypos))
         if hasattr(ob.title, 'angle') and ob.title.angle != None:
            __pychips_base__.set_plot_title_angle(oid, float(ob.title.angle))
         if hasattr(ob.title, 'depth') and ob.title.depth != None:
            dv = __utils__.convert_depth(ob.title.depth)
            if dv != None:
               __pychips_base__.set_plot_title_depth(oid, dv)
         if hasattr(ob.title, 'halign') and ob.title.halign != None:
            if type(ob.title.halign) == str:
	       alignval = __utils__.convert_offset(ob.title.halign)
	       if alignval != None:
                  __pychips_base__.set_plot_title_halign(oid, alignval)
            else:
               __pychips_base__.set_plot_title_halign(oid, ob.title.halign)
         if hasattr(ob.title, 'valign') and ob.title.valign != None:
            if type(ob.title.valign) == str:
	       alignval = __utils__.convert_offset(ob.title.valign)
	       if alignval != None:
                  __pychips_base__.set_plot_title_valign(oid, alignval)
            else:
               __pychips_base__.set_plot_title_valign(oid, ob.title.valign)
         if hasattr(ob.title, 'color') and ob.title.color != None:
            __pychips_base__.set_plot_title_color(oid, __utils__.get_color_as_string (ob.title.color))
         if hasattr(ob.title, 'size') and ob.title.size != None:
            __pychips_base__.set_plot_title_size(oid, int(ob.title.size))
         if hasattr(ob.title, 'font') and ob.title.font != None:
            __pychips_base__.set_plot_title_font(oid, ob.title.font)
         if hasattr(ob.title, 'fontstyle') and ob.title.fontstyle != None:
            __pychips_base__.set_plot_title_fontstyle(oid, ob.title.fontstyle)
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return


def __get_axis(id=None, property=None, dim=__pychips_base__.XY_AXIS):
   'Returns a ChipsAxis object containing attribute values of requested axis'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      if dim == __pychips_base__.X_AXIS:
         oid.set_xaxis_id(id)
         oid.set_axis_dim(__pychips_base__.X_AXIS)
      elif dim == __pychips_base__.Y_AXIS:
         oid.set_yaxis_id(id)
         oid.set_axis_dim(__pychips_base__.Y_AXIS)
      else:
         oid.set_axis_id(id)
         oid.set_axis_dim(__pychips_base__.XY_AXIS)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = __pychips_base__.ChipsId(id)
      if dim == __pychips_base__.X_AXIS:
         oid.set_axis_dim(__pychips_base__.X_AXIS)
         ai = oid.get_axis_id()
         if ai != "":
            oid.set_axis_id("")
            oid.set_xaxis_id(ai)
      elif dim == __pychips_base__.Y_AXIS:
         oid.set_axis_dim(__pychips_base__.Y_AXIS)
         ai = oid.get_axis_id()
         if ai != "":
            oid.set_axis_id("")
            oid.set_yaxis_id(ai)
      elif dim == __pychips_base__.XY_AXIS:
         oid.set_axis_dim(__pychips_base__.XY_AXIS)
   elif id == None:
      oid = __pychips_base__.ChipsId()
      if dim == __pychips_base__.X_AXIS:
         oid.set_axis_dim(__pychips_base__.X_AXIS)
      elif dim == __pychips_base__.Y_AXIS:
         oid.set_axis_dim(__pychips_base__.Y_AXIS)
      elif dim == __pychips_base__.XY_AXIS:
         oid.set_axis_dim(__pychips_base__.XY_AXIS)
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   if property == None or property == 'all':
      # return property structure
      result = ChipsAxis();
      result.color = __pychips_base__.get_axis_color(oid)
      result.depth = __pychips_base__.get_axis_depth(oid)
      result.thickness = __pychips_base__.get_axis_thickness(oid)
      result.label.color = __pychips_base__.get_axis_label_color(oid)
      result.label.size = __pychips_base__.get_axis_label_size(oid)
      result.label.font = __pychips_base__.get_axis_label_font(oid)
      result.label.fontstyle = __pychips_base__.get_axis_label_fontstyle(oid)
      result.label.halign = __pychips_base__.get_axis_label_halign(oid)
      result.label.valign = __pychips_base__.get_axis_label_valign(oid)
      result.label.visible = __pychips_base__.get_axis_label_visible(oid)
      result.majorgrid.visible = __pychips_base__.get_axis_majorgrid_visible(oid) 
      result.majorgrid.style = __pychips_base__.get_axis_majorgrid_style(oid) 
      result.majorgrid.color = __pychips_base__.get_axis_majorgrid_color(oid) 
      result.majorgrid.thickness = __pychips_base__.get_axis_majorgrid_thickness(oid) 
      result.minorgrid.visible = __pychips_base__.get_axis_minorgrid_visible(oid) 
      result.minorgrid.style = __pychips_base__.get_axis_minorgrid_style(oid) 
      result.minorgrid.color = __pychips_base__.get_axis_minorgrid_color(oid) 
      result.minorgrid.thickness = __pychips_base__.get_axis_minorgrid_thickness(oid) 
      result.automin = __pychips_base__.get_axis_automin(oid) 
      result.automax = __pychips_base__.get_axis_automax(oid) 
      result.pad = __pychips_base__.get_axis_pad(oid)
      result.offset.parallel = __pychips_base__.get_axis_parallel_offset(oid)
      result.offset.perpendicular = __pychips_base__.get_axis_perpendicular_offset(oid)
      result.tickformat = __pychips_base__.get_axis_tickformat(oid)
      result.tickstyle = __pychips_base__.get_axis_tickstyle(oid)
      result.ticklabel.color = __pychips_base__.get_axis_ticklabel_color(oid)
      result.ticklabel.style = __pychips_base__.get_axis_ticklabel_style(oid)
      result.ticklabel.size = __pychips_base__.get_axis_ticklabel_size(oid)
      result.ticklabel.font = __pychips_base__.get_axis_ticklabel_font(oid)
      result.ticklabel.fontstyle = __pychips_base__.get_axis_ticklabel_fontstyle(oid)
      result.ticklabel.angle = __pychips_base__.get_axis_ticklabel_angle(oid)
      result.ticklabel.halign = __pychips_base__.get_axis_ticklabel_halign(oid)
      result.ticklabel.valign = __pychips_base__.get_axis_ticklabel_valign(oid)
      result.ticklabel.visible = __pychips_base__.get_axis_ticklabel_visible(oid)
      result.ticklabel.offset = __pychips_base__.get_axis_ticklabel_offset(oid)
      result.majortick.mode = __pychips_base__.get_axis_major_mode(oid) 
      result.majortick.interval = __pychips_base__.get_axis_major_interval(oid) 
      result.majortick.count = __pychips_base__.get_axis_major_num(oid) 
      result.majortick.thickness = __pychips_base__.get_axis_major_thickness(oid) 
      result.majortick.color = __pychips_base__.get_axis_major_color(oid)  
      result.majortick.style = __pychips_base__.get_axis_major_style(oid) 
      result.majortick.length = __pychips_base__.get_axis_major_length(oid) 
      result.majortick.visible = __pychips_base__.get_major_ticks_visible(oid)
      result.minortick.mode = __pychips_base__.get_axis_minor_mode(oid) 
      result.minortick.interval = __pychips_base__.get_axis_minor_interval(oid) 
      result.minortick.count = __pychips_base__.get_axis_minor_num(oid) 
      result.minortick.thickness = __pychips_base__.get_axis_minor_thickness(oid) 
      result.minortick.color = __pychips_base__.get_axis_minor_color(oid)  
      result.minortick.style = __pychips_base__.get_axis_minor_style(oid) 
      result.minortick.length = __pychips_base__.get_axis_minor_length(oid) 
      result.minortick.visible = __pychips_base__.get_minor_ticks_visible(oid)

      lbang = __pychips_base__.get_axis_label_angle(oid)
      lbtxt = __pychips_base__.get_axis_text(oid)
      dv = oid.get_axis_dim()
      if (dv == __pychips_base__.X_AXIS):
         result.x.label.angle = lbang
         result.x.label.text = lbtxt
      elif (dv == __pychips_base__.Y_AXIS):     
         result.y.label.angle = lbang
         result.y.label.text = lbtxt
      return result
   else:
      # return specific property value
      if (property.find('axis') == 0):       # remove preceeding 'axis.'
         property = property.replace('axis.','')
      if property == 'y.label.angle': 
         val = __pychips_base__.get_axis_ylabel_angle(oid)
      elif property == 'x.label.angle':
         val = __pychips_base__.get_axis_xlabel_angle(oid)
      elif property == 'x.label.text':
         val = __pychips_base__.get_axis_xlabel_text(oid)
      elif property == 'y.label.text':
         val = __pychips_base__.get_axis_ylabel_text(oid)
      elif property == 'color': 
         val = __pychips_base__.get_axis_color(oid)
      elif property == 'thickness': 
         val = __pychips_base__.get_axis_thickness(oid)
      elif property == 'depth': 
         val = __pychips_base__.get_axis_depth(oid)
      elif property == 'label.color': 
         val = __pychips_base__.get_axis_label_color(oid)
      elif property == 'label.size': 
         val = __pychips_base__.get_axis_label_size(oid)
      elif property == 'label.font': 
         val = __pychips_base__.get_axis_label_font(oid)
      elif property == 'label.fontstyle': 
         val = __pychips_base__.get_axis_label_fontstyle(oid)
      elif property == 'label.halign': 
         val = __pychips_base__.get_axis_label_halign(oid)
      elif property == 'label.valign': 
         val = __pychips_base__.get_axis_label_valign(oid)
      elif property == 'label.visible': 
         val = __pychips_base__.get_axis_label_visible(oid)
      elif property == 'majorgrid.visible':
         val = __pychips_base__.get_axis_majorgrid_visible(oid)
      elif property == 'majorgrid.style':
         val = __pychips_base__.get_axis_majorgrid_style(oid)
      elif property == 'majorgrid.color':
         val = __pychips_base__.get_axis_majorgrid_color(oid)
      elif property == 'majorgrid.thickness':
         val = __pychips_base__.get_axis_majorgrid_thickness(oid)
      elif property == 'minorgrid.visible': 
         val = __pychips_base__.get_axis_minorgrid_visible(oid)
      elif property == 'minorgrid.style': 
         val = __pychips_base__.get_axis_minorgrid_style(oid)
      elif property == 'minorrgrid.color': 
         val = __pychips_base__.get_axis_minorgrid_color(oid)
      elif property == 'minorgrid.thickness': 
         val = __pychips_base__.get_axis_minorgrid_thickness(oid)
      elif property == 'automin': 
         val = __pychips_base__.get_axis_automin(oid)
      elif property == 'automax': 
         val = __pychips_base__.get_axis_automax(oid)
      elif property == 'pad': 
         val = __pychips_base__.get_axis_pad(oid)
      elif property == 'offset.parallel': 
         val = __pychips_base__.get_axis_parallel_offset(oid)
      elif property == 'offset.perpendicular': 
         val = __pychips_base__.get_axis_perpendicular_offset(oid)
      elif property == 'tickformat': 
         val = __pychips_base__.get_axis_tickformat(oid)
      elif property == 'tickstyle':
         val = __pychips_base__.get_axis_tickstyle(oid)
      elif property == 'majortick.mode': 
         val = __pychips_base__.get_axis_major_mode(oid)
      elif property == 'majortick.interval': 
         val = __pychips_base__.get_axis_major_interval(oid)
      elif property == 'majortick.count': 
         val = __pychips_base__.get_axis_major_num(oid)
      elif property == 'majortick.thickness': 
         val = __pychips_base__.get_axis_major_thickness(oid)
      elif property == 'majortick.color': 
         val = __pychips_base__.get_axis_major_color(oid)
      elif property == 'majortick.style': 
         val = __pychips_base__.get_axis_major_style(oid)
      elif property == 'majortick.length': 
         val = __pychips_base__.get_axis_major_length(oid)
      elif property == 'majortick.visible': 
         val = __pychips_base__.get_major_ticks_visible(oid)
      elif property == 'minortick.mode':
         val = __pychips_base__.get_axis_minor_mode(oid)
      elif property == 'minortick.interval':
         val = __pychips_base__.get_axis_minor_interval(oid)
      elif property == 'minortick.count':
         val = __pychips_base__.get_axis_minor_num(oid)
      elif property == 'minortick.thickness':
         val = __pychips_base__.get_axis_minor_thickness(oid)
      elif property == 'minortick.color':
         val = __pychips_base__.get_axis_minor_color(oid)
      elif property == 'minortick.style':
         val = __pychips_base__.get_axis_minor_style(oid)
      elif property == 'minortick.length':
         val = __pychips_base__.get_axis_minor_length(oid)
      elif property == 'minortick.visible':
         val = __pychips_base__.get_minor_ticks_visible(oid)
      elif property == 'ticklabel.color':
         val = __pychips_base__.get_axis_ticklabel_color(oid)
      elif property == 'ticklabel.style':
         val = __pychips_base__.get_axis_ticklabel_style(oid)
      elif property == 'ticklabel.size':
         val = __pychips_base__.get_axis_ticklabel_size(oid)
      elif property == 'ticklabel.font':
         val = __pychips_base__.get_axis_ticklabel_font(oid)
      elif property == 'ticklabel.fontstyle':
         val = __pychips_base__.get_axis_ticklabel_fontstyle(oid)
      elif property == 'ticklabel.angle':
         val = __pychips_base__.get_axis_ticklabel_angle(oid)
      elif property == 'ticklabel.halign':
         val = __pychips_base__.get_axis_ticklabel_halign(oid)
      elif property == 'ticklabel.valign':
         val = __pychips_base__.get_axis_ticklabel_valign(oid)
      elif property == 'ticklabel.visible':
         val = __pychips_base__.get_axis_ticklabel_visible(oid)
      elif property == 'ticklabel.offset':
         val = __pychips_base__.get_axis_ticklabel_offset(oid)
      else:
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
         val = None
      return val;


def get_axis(id=None, property=None):
   'Returns a ChipsAxis object containing attribute values of requested axis'
   return __get_axis(id, property)


def get_xaxis(id=None, property=None):
   'Returns a ChipsAxis object containing attribute values of requested x axis'
   return __get_axis(id, property, __pychips_base__.X_AXIS)


def get_yaxis(id=None, property=None):
   'Returns a ChipsAxis object containing attribute values of requested y axis'
   return __get_axis(id, property, __pychips_base__.Y_AXIS)


def __set_axis(values, id=None, dim=__pychips_base__.XY_AXIS):
   'Set attributes of the specified axis/dimension to the provided values'
   # use __pychips_base__.ChipsId, string axis id, or no id (current) to determine axis
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      if dim == __pychips_base__.X_AXIS:
         oid.set_xaxis_id(id)
         oid.set_axis_dim(__pychips_base__.X_AXIS)
      elif dim == __pychips_base__.Y_AXIS:
         oid.set_yaxis_id(id)
         oid.set_axis_dim(__pychips_base__.Y_AXIS)
      else:
         oid.set_axis_id(id)
         oid.set_axis_dim(__pychips_base__.XY_AXIS)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = __pychips_base__.ChipsId(id)
      if dim == __pychips_base__.X_AXIS: 
         oid.set_axis_dim(__pychips_base__.X_AXIS)
	 ai = oid.get_axis_id()
         if ai != "":
	    oid.set_axis_id("")
	    oid.set_xaxis_id(ai)
      elif dim == __pychips_base__.Y_AXIS: 
         oid.set_axis_dim(__pychips_base__.Y_AXIS)
	 ai = oid.get_axis_id()
         if ai != "":
	    oid.set_axis_id("")
	    oid.set_yaxis_id(ai)
      elif dim == __pychips_base__.XY_AXIS: 
         oid.set_axis_dim(__pychips_base__.XY_AXIS)
   elif id == None:
      oid = __pychips_base__.ChipsId()
      if dim == __pychips_base__.X_AXIS: 
         oid.set_axis_dim(__pychips_base__.X_AXIS)
      elif dim == __pychips_base__.Y_AXIS: 
         oid.set_axis_dim(__pychips_base__.Y_AXIS)
      elif dim == __pychips_base__.XY_AXIS: 
         oid.set_axis_dim(__pychips_base__.XY_AXIS)
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(values) == str:
      values = __utils__.string_to_attribute_list(values)

   # convert data structure to ChipsAxis object
   ob = ChipsAxis ()

   try:
      ob = __utils__.data_to_attribute_object (ob, values)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_axis with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'x') and ob.x != None and hasattr(ob.x, 'label') and ob.x.label != None:
         if hasattr(ob.x.label, 'angle') and ob.x.label.angle != None:
            __pychips_base__.set_axis_xlabel_angle(oid, float(ob.x.label.angle))
         if hasattr(ob.x.label, 'text') and ob.x.label.text != None:
            tx = ob.x.label.text
            if tx.endswith('"') and tx.startswith('"'):
               tx=tx[1:len(tx)-1]
            __pychips_base__.set_axis_xlabel_text(oid, tx)
      if hasattr(ob, 'y') and ob.y != None and hasattr(ob.y, 'label') and ob.y.label != None:
         if hasattr(ob.y.label, 'angle') and ob.y.label.angle != None:
            __pychips_base__.set_axis_ylabel_angle(oid, float(ob.y.label.angle))
         if hasattr(ob.y.label, 'text') and ob.y.label.text != None:
            tx = ob.y.label.text
            if tx.endswith('"') and tx.startswith('"'):
               tx=tx[1:len(tx)-1]
            __pychips_base__.set_axis_ylabel_text(oid, tx)
      if hasattr(ob, 'color') and ob.color != None:
         __pychips_base__.set_axis_color(oid, __utils__.get_color_as_string (ob.color))
      if hasattr(ob, 'thickness') and ob.thickness != None:
         __pychips_base__.set_axis_thickness(oid, float(ob.thickness))
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_axis_depth(oid, dv)
      if hasattr(ob, 'label'):
         if hasattr(ob.label, 'color') and ob.label.color != None:
            __pychips_base__.set_axis_label_color(oid, __utils__.get_color_as_string (ob.label.color))
         if hasattr(ob.label, 'size') and ob.label.size != None:
            __pychips_base__.set_axis_label_size(oid, int(ob.label.size))
         if hasattr(ob.label, 'font') and ob.label.font != None:
            __pychips_base__.set_axis_label_font(oid, ob.label.font)
         if hasattr(ob.label, 'fontstyle') and ob.label.fontstyle != None:
            __pychips_base__.set_axis_label_fontstyle(oid, ob.label.fontstyle)
         if hasattr(ob.label, 'halign') and ob.label.halign != None:
            if type(ob.label.halign) == str:
	       alignval = __utils__.convert_offset(ob.label.halign)
	       if alignval != None:
                  __pychips_base__.set_axis_label_halign(oid, alignval)
            else:
               __pychips_base__.set_axis_label_halign(oid, ob.label.halign)
         if hasattr(ob.label, 'valign') and ob.label.valign != None:
            if type(ob.label.valign) == str:
	       alignval = __utils__.convert_offset(ob.label.valign)
	       if alignval != None:
                  __pychips_base__.set_axis_label_valign(oid, alignval)
            else:
               __pychips_base__.set_axis_label_valign(oid, ob.label.valign)
         if hasattr(ob.label, 'visible') and ob.label.visible != None:
            __pychips_base__.set_axis_label_visible(oid, __utils__.bool_cast(ob.label.visible))
   
      if hasattr(ob, 'ticklabel'):
         if hasattr(ob.ticklabel, 'style') and ob.ticklabel.style != None:
            __pychips_base__.set_axis_ticklabel_style(oid, ob.ticklabel.style)
         if hasattr(ob.ticklabel, 'color') and ob.ticklabel.color != None:
            __pychips_base__.set_axis_ticklabel_color(oid, __utils__.get_color_as_string (ob.ticklabel.color))
         if hasattr(ob.ticklabel, 'size') and ob.ticklabel.size != None:
            __pychips_base__.set_axis_ticklabel_size(oid, int(ob.ticklabel.size))
         if hasattr(ob.ticklabel, 'font') and ob.ticklabel.font != None:
            __pychips_base__.set_axis_ticklabel_font(oid, ob.ticklabel.font)
         if hasattr(ob.ticklabel, 'fontstyle') and ob.ticklabel.fontstyle != None:
            __pychips_base__.set_axis_ticklabel_fontstyle(oid, ob.ticklabel.fontstyle)
         if hasattr(ob.ticklabel, 'offset') and ob.ticklabel.offset != None:
            __pychips_base__.set_axis_ticklabel_offset(oid, int(ob.ticklabel.offset))
         if hasattr(ob.ticklabel, 'angle') and ob.ticklabel.angle != None:
            __pychips_base__.set_axis_ticklabel_angle(oid, float(ob.ticklabel.angle))
         if hasattr(ob.ticklabel, 'halign') and ob.ticklabel.halign != None:
            if type(ob.ticklabel.halign) == str:
	       alignval = __utils__.convert_offset(ob.ticklabel.halign)
	       if alignval != None:
                  __pychips_base__.set_axis_ticklabel_halign(oid, alignval)
            else:
               __pychips_base__.set_axis_ticklabel_halign(oid, ob.ticklabel.halign)
         if hasattr(ob.ticklabel, 'valign') and ob.ticklabel.valign != None:
            if type(ob.ticklabel.valign) == str:
	       alignval = __utils__.convert_offset(ob.ticklabel.valign)
	       if alignval != None:
                  __pychips_base__.set_axis_ticklabel_valign(oid, alignval)
            else:
               __pychips_base__.set_axis_ticklabel_valign(oid, ob.ticklabel.valign)
         if hasattr(ob.ticklabel, 'visible') and ob.ticklabel.visible != None:
            __pychips_base__.set_axis_ticklabel_visible(oid, __utils__.bool_cast(ob.ticklabel.visible))

      if hasattr(ob, 'majorgrid'):
         if hasattr(ob.majorgrid, 'visible') and ob.majorgrid.visible != None:
            __pychips_base__.set_axis_majorgrid_visible(oid, __utils__.bool_cast(ob.majorgrid.visible))
         if hasattr(ob.majorgrid, 'style') and ob.majorgrid.style != None:
            if type(ob.majorgrid.style) == str:
               intstyle = __pychips_base__.string_to_v_line(ob.majorgrid.style)
	       if __pychips_base__.valid_line_type(intstyle):
                  __pychips_base__.set_axis_majorgrid_style(oid, intstyle)
	       else:
	          raise AttributeError("chips ERROR: The majorgrid style value (%s) must be one of [noline, solid, dot, dotlongdash, dotshortdash, longdash, shortdash, shortdashlongdash]"%ob.majorgrid.style)
            else:
               __pychips_base__.set_axis_majorgrid_style(oid, ob.majorgrid.style)
         if hasattr(ob.majorgrid, 'color') and ob.majorgrid.color != None:
            __pychips_base__.set_axis_majorgrid_color(oid, __utils__.get_color_as_string (ob.majorgrid.color))
         if hasattr(ob.majorgrid, 'thickness') and ob.majorgrid.thickness != None:
            __pychips_base__.set_axis_majorgrid_thickness(oid, float(ob.majorgrid.thickness))
      if hasattr(ob, 'minorgrid'):
         if hasattr(ob.minorgrid, 'visible') and ob.minorgrid.visible != None:
            __pychips_base__.set_axis_minorgrid_visible(oid, __utils__.bool_cast(ob.minorgrid.visible))
         if hasattr(ob.minorgrid, 'style') and ob.minorgrid.style != None:
            if type(ob.minorgrid.style) == str:
               intstyle = __pychips_base__.string_to_v_line(ob.minorgrid.style)
               if __pychips_base__.valid_line_type(intstyle):
                  __pychips_base__.set_axis_minorgrid_style(oid, intstyle)
	       else:
	          raise AttributeError("chips ERROR: The minorgrid style value (%s) must be one of [noline, solid, dot, dotlongdash, dotshortdash, longdash, shortdash, shortdashlongdash]"%ob.minorgrid.style)
            else:
               __pychips_base__.set_axis_minorgrid_style(oid, ob.minorgrid.style)
         if hasattr(ob.minorgrid, 'color') and ob.minorgrid.color != None:
            __pychips_base__.set_axis_minorgrid_color(oid, __utils__.get_color_as_string (ob.minorgrid.color))
         if hasattr(ob.minorgrid, 'thickness') and ob.minorgrid.thickness != None:
            __pychips_base__.set_axis_minorgrid_thickness(oid, float(ob.minorgrid.thickness))
      if hasattr(ob, 'automin') and ob.automin != None:
         __pychips_base__.set_axis_automin(oid, __utils__.bool_cast(ob.automin))
      if hasattr(ob, 'automax') and ob.automax != None:
         __pychips_base__.set_axis_automax(oid, __utils__.bool_cast(ob.automax))
      if hasattr(ob, 'pad') and ob.pad != None:
         __pychips_base__.set_axis_pad(oid, float(ob.pad))
      if hasattr(ob, 'offset') and hasattr(ob.offset, 'parallel') and ob.offset.parallel != None:
         __pychips_base__.set_axis_parallel_offset(oid, float(ob.offset.parallel))
      if hasattr(ob, 'offset') and hasattr(ob.offset, 'perpendicular') and ob.offset.perpendicular != None:
         __pychips_base__.set_axis_perpendicular_offset(oid, float(ob.offset.perpendicular))
      if hasattr(ob, 'tickformat') and ob.tickformat != None:
         __pychips_base__.set_axis_tickformat(oid, ob.tickformat)
      if hasattr(ob, 'tickstyle') and ob.tickstyle != None:
         __pychips_base__.set_axis_tickstyle (oid, ob.tickstyle)


      # interval and count can internally modify mode- need special handling
      if hasattr(ob, 'majortick'):
         check = 0
         if hasattr(ob.majortick, 'interval') and ob.majortick.interval != None:
            check = check + 1
         if hasattr(ob.majortick, 'count') and ob.majortick.count != None:
            check = check + 2
         if hasattr(ob.majortick, 'mode') and ob.majortick.mode != None:
            if check == 1 or check == 3:
               __pychips_base__.set_axis_major_interval(oid, float(ob.majortick.interval))
            if check == 2 or check == 3:
               __pychips_base__.set_axis_major_num(oid, int(ob.majortick.count))
            __pychips_base__.set_axis_major_mode(oid, ob.majortick.mode)
         elif check == 1:
            __pychips_base__.set_axis_major_interval(oid, float(ob.majortick.interval))
         elif check == 2:
            __pychips_base__.set_axis_major_num(oid, int(ob.majortick.count))
         elif check == 3:
            __pychips_base__.unlock()
            raise AttributeError("chips ERROR: You can not specify majortick.interval and majortick.count without providing majortick.mode")

         if hasattr(ob.majortick, 'style') and ob.majortick.style != None:
            __pychips_base__.set_axis_major_style(oid, ob.majortick.style)
         if hasattr(ob.majortick, 'thickness') and ob.majortick.thickness != None:
            __pychips_base__.set_axis_major_thickness(oid, float(ob.majortick.thickness))
         if hasattr(ob.majortick, 'color') and ob.majortick.color != None:
            __pychips_base__.set_axis_major_color(oid, __utils__.get_color_as_string (ob.majortick.color))
         if hasattr(ob.majortick, 'length') and ob.majortick.length != None:
            __pychips_base__.set_axis_major_length(oid, int(ob.majortick.length))
         if hasattr(ob.majortick, 'visible') and ob.majortick.visible != None:
            if __utils__.bool_cast(ob.majortick.visible) == True:
               __pychips_base__.display_major_ticks(oid)
            else:
               __pychips_base__.hide_major_ticks(oid)
      if hasattr(ob, 'minortick'):
         if hasattr(ob.minortick, 'mode') and ob.minortick.mode != None:
            __pychips_base__.set_axis_minor_mode(oid, ob.minortick.mode)
         if hasattr(ob.minortick, 'interval') and ob.minortick.interval != None:
            __pychips_base__.set_axis_minor_interval(oid, float(ob.minortick.interval))
         if hasattr(ob.minortick, 'count') and ob.minortick.count != None:
            __pychips_base__.set_axis_minor_num(oid, int(ob.minortick.count))
         if hasattr(ob.minortick, 'style') and ob.minortick.style != None:
            __pychips_base__.set_axis_minor_style(oid, ob.minortick.style)
         if hasattr(ob.minortick, 'thickness') and ob.minortick.thickness != None:
            __pychips_base__.set_axis_minor_thickness(oid, float(ob.minortick.thickness))
         if hasattr(ob.minortick, 'color') and ob.minortick.color != None:
            __pychips_base__.set_axis_minor_color(oid, __utils__.get_color_as_string (ob.minortick.color))
         if hasattr(ob.minortick, 'length') and ob.minortick.length != None:
            __pychips_base__.set_axis_minor_length(oid, int(ob.minortick.length))
         if hasattr(ob.minortick, 'visible') and ob.minortick.visible != None:
            if __utils__.bool_cast(ob.minortick.visible) == True:
               __pychips_base__.display_minor_ticks(oid)
            else:
               __pychips_base__.hide_minor_ticks(oid)
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return


def set_axis(id=None, values=None):
   'Set the attributes of the specified axis to the provided values'
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id
   __set_axis(vals, mid);
   return


def set_xaxis(id=None, values=None):
   'Set the attributes of the specified x axis to the provided values'
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id
   __set_axis(vals, mid, __pychips_base__.X_AXIS);
   return


def set_yaxis(id=None, values=None):
   'Set the attributes of the specified y axis to the provided values'
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id
   __set_axis(vals, mid, __pychips_base__.Y_AXIS);
   return


def get_curve(id=None, property=None):
   'Returns a ChipsCurve object containing attribute values of requested curve'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_curve_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsCurve();
      result.symbol.style = __pychips_base__.get_curve_symbolstyle(oid)
      result.symbol.color = __pychips_base__.get_curve_symbolcolor(oid)
      result.symbol.size = __pychips_base__.get_curve_symbolsize(oid)
      result.symbol.angle = __pychips_base__.get_curve_symbolangle(oid)
      result.depth = __pychips_base__.get_curve_depth(oid)
      result.symbol.fill = __pychips_base__.get_curve_symbolfill(oid)
      result.line.color = __pychips_base__.get_curve_linecolor(oid)
      result.line.style = __pychips_base__.get_curve_linestyle(oid)
      result.line.thickness = __pychips_base__.get_curve_linethickness(oid)
      result.err.style = __pychips_base__.get_curve_errstyle(oid)
      result.err.up = __pychips_base__.get_curve_errup(oid)
      result.err.down = __pychips_base__.get_curve_errdown(oid)
      result.err.left = __pychips_base__.get_curve_errleft(oid)
      result.err.right = __pychips_base__.get_curve_errright(oid)
      result.err.color = __pychips_base__.get_curve_errcolor(oid)
      result.err.caplength = __pychips_base__.get_curve_errcaplength(oid)
      result.err.thickness = __pychips_base__.get_curve_errthickness(oid)
      result.limit.length = __pychips_base__.get_curve_limitlength(oid)
      result.limit.override = __pychips_base__.get_curve_limitoverride(oid)
      result.baddata = __pychips_base__.get_curve_baddatamode(oid)
      return result
   else:
      # return specific property value
      if (property.find('curve') == 0):       # remove preceeding 'curve.'
         property = property.replace('curve.','')
      if property == 'symbol.style':
         val = __pychips_base__.get_curve_symbolstyle(oid)
      elif property == 'symbol.color':
         val = __pychips_base__.get_curve_symbolcolor(oid)
      elif property == 'symbol.size':
         val = __pychips_base__.get_curve_symbolsize(oid)
      elif property == 'symbol.angle':
         val = __pychips_base__.get_curve_symbolangle(oid)
      elif property == 'symbol.fill':
         val = __pychips_base__.get_curve_symbolfill(oid)
      elif property == 'line.color':
         val = __pychips_base__.get_curve_linecolor(oid)
      elif property == 'line.style':
         val = __pychips_base__.get_curve_linestyle(oid)
      elif property == 'line.thickness':
         val = __pychips_base__.get_curve_linethickness(oid)
      elif property == 'err.style':
         val = __pychips_base__.get_curve_errstyle(oid)
      elif property == 'err.up':
         val = __pychips_base__.get_curve_errup(oid)
      elif property == 'err.down':
         val = __pychips_base__.get_curve_errdown(oid)
      elif property == 'err.left':
         val = __pychips_base__.get_curve_errleft(oid)
      elif property == 'err.right':
         val = __pychips_base__.get_curve_errright(oid)
      elif property == 'err.color':
         val = __pychips_base__.get_curve_errcolor(oid)
      elif property == 'err.caplength':
         val = __pychips_base__.get_curve_errcaplength(oid)
      elif property == 'err.thickness':
         val = __pychips_base__.get_curve_errthickness(oid)
      elif property == 'limit.length':
         val = __pychips_base__.get_curve_limitlength(oid)
      elif property == 'limit.override':
         val = __pychips_base__.get_curve_limitoverride(oid)
      elif property == 'baddata':
         val = __pychips_base__.get_curve_baddatamode(oid)
      elif property == 'depth':
         val = __pychips_base__.get_curve_depth(oid)
      else:
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
         val = None
      return val;

def set_curve(id=None, values=None):
   'Set the attributes of the specified curve to the provided values'
   # use __pychips_base__.ChipsId, string curve id, or no id (current) to determine curve
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   oid = __utils__.set_id (mid, __pychips_base__.chips_curve)
   __pychips_base__.lock();

   ob = ChipsCurve ()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsCurve object
   ob = ChipsCurve ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_curve with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()
      if hasattr(ob, 'baddata') and ob.baddata != None:
         if type(ob.baddata) == str:
            intmode = __pychips_base__.string_to_v_baddatamode(ob.baddata)
            if __pychips_base__.valid_baddatamode_type(intmode):
               __pychips_base__.set_curve_baddatamode(oid, intmode)
            else:
	       raise AttributeError("chips ERROR: The baddata value (%s) must be one of [ignore, omit]"%ob.baddata)
         else:
            __pychips_base__.set_curve_baddatamode(oid, ob.baddata)
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_curve_depth(oid, dv)
      if hasattr(ob, 'symbol'):
         if hasattr(ob.symbol, 'style') and ob.symbol.style != None:
            if type(ob.symbol.style) == str:
               intstyle = __pychips_base__.string_to_v_symbol(ob.symbol.style)
               if __pychips_base__.valid_symbol_type(intstyle):
                  __pychips_base__.set_curve_symbolstyle(oid, intstyle)
	       else:
	          raise AttributeError("chips ERROR: The symbol style value (%s) must be one of [none, cross, diamond, downtriangle, circle, plus, square, uptriangle, point, arrow]"%ob.symbol.style)
            else:
               __pychips_base__.set_curve_symbolstyle(oid, ob.symbol.style)
         if hasattr(ob.symbol, 'size') and ob.symbol.size != None:
            __pychips_base__.set_curve_symbolsize(oid, int(ob.symbol.size))
         if hasattr(ob.symbol, 'angle') and ob.symbol.angle != None:
            __pychips_base__.set_curve_symbolangle(oid, float(ob.symbol.angle))
         if hasattr(ob.symbol, 'fill') and ob.symbol.fill != None:
            __pychips_base__.set_curve_symbolfill(oid, __utils__.bool_cast(ob.symbol.fill))
         if hasattr(ob.symbol, 'color') and ob.symbol.color != None:
            __pychips_base__.set_curve_symbolcolor(oid, __utils__.get_color_as_string(ob.symbol.color))
      if hasattr(ob, 'line'):
         if hasattr(ob.line, 'color') and ob.line.color != None:
            __pychips_base__.set_curve_linecolor(oid, __utils__.get_color_as_string (ob.line.color))
         if hasattr(ob.line, 'thickness') and ob.line.thickness != None:
            __pychips_base__.set_curve_linethickness(oid, float(ob.line.thickness))
         if hasattr(ob.line, 'style') and ob.line.style != None:
            if type(ob.line.style) == str:
               intstyle = __pychips_base__.string_to_v_line(ob.line.style)
               if __pychips_base__.valid_line_type(intstyle):
                  __pychips_base__.set_curve_linestyle(oid, intstyle)
	       else:
	          raise AttributeError("chips ERROR: The line style value (%s) must be one of [noline, solid, dot, dotlongdash, dotshortdash, longdash, shortdash, shortdashlongdash]"%ob.line.style)
            else:
               __pychips_base__.set_curve_linestyle(oid, ob.line.style)
      if hasattr(ob, 'err'):
         if hasattr(ob.err, 'color') and ob.err.color != None:
            __pychips_base__.set_curve_errcolor(oid, __utils__.get_color_as_string (ob.err.color))
         if hasattr(ob.err, 'caplength') and ob.err.caplength != None:
            __pychips_base__.set_curve_errcaplength(oid, int(ob.err.caplength))
         if hasattr(ob.err, 'thickness') and ob.err.thickness != None:
            __pychips_base__.set_curve_errthickness(oid, float(ob.err.thickness))
         if hasattr(ob.err, 'style') and ob.err.style != None:
            __pychips_base__.set_curve_errstyle(oid, ob.err.style)
         if hasattr(ob.err, 'up') and ob.err.up != None:
            __pychips_base__.set_curve_errup(oid, __utils__.bool_cast(ob.err.up))
         if hasattr(ob.err, 'down') and ob.err.down != None:
            __pychips_base__.set_curve_errdown(oid, __utils__.bool_cast(ob.err.down))
         if hasattr(ob.err, 'left') and ob.err.left != None:
            __pychips_base__.set_curve_errleft(oid, __utils__.bool_cast(ob.err.left))
         if hasattr(ob.err, 'right') and ob.err.right != None:
            __pychips_base__.set_curve_errright(oid, __utils__.bool_cast(ob.err.right))
      if hasattr(ob, 'limit'):
         if hasattr(ob.limit, 'length') and ob.limit.length != None:
            __pychips_base__.set_curve_limitlength(oid, float(ob.limit.length))
         if hasattr(ob.limit, 'override') and ob.limit.override != None:
            __pychips_base__.set_curve_limitoverride(oid, __utils__.bool_cast(ob.limit.override))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return

def get_histogram(id=None, property=None):
   'Returns a ChipsHistogram object with attribute values of requested histogram'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_histogram_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsHistogram();
      result.symbol.style = __pychips_base__.get_histogram_symbolstyle(oid)
      result.symbol.color = __pychips_base__.get_histogram_symbolcolor(oid)
      result.symbol.size = __pychips_base__.get_histogram_symbolsize(oid)
      result.symbol.angle = __pychips_base__.get_histogram_symbolangle(oid)
      result.symbol.fill = __pychips_base__.get_histogram_symbolfill(oid)
      result.line.color = __pychips_base__.get_histogram_linecolor(oid)
      result.line.style = __pychips_base__.get_histogram_linestyle(oid)
      result.line.thickness = __pychips_base__.get_histogram_linethickness(oid)
      result.err.style = __pychips_base__.get_histogram_errstyle(oid)
      result.err.up = __pychips_base__.get_histogram_errup(oid)
      result.err.down = __pychips_base__.get_histogram_errdown(oid)
      result.err.color = __pychips_base__.get_histogram_errcolor(oid)
      result.err.caplength = __pychips_base__.get_histogram_errcaplength(oid)
      result.err.thickness = __pychips_base__.get_histogram_errthickness(oid)
      result.dropline = __pychips_base__.get_histogram_dropline(oid)
      result.depth = __pychips_base__.get_histogram_depth(oid)
      result.fill.color = __pychips_base__.get_histogram_fillcolor(oid)
      result.fill.opacity = __pychips_base__.get_histogram_fillopacity(oid)
      result.fill.style = __pychips_base__.get_histogram_fillstyle(oid)
      return result
   else:
      # return specific property value
      if (property.find('histogram') == 0):    # remove preceeding 'histogram.'
         property = property.replace('histogram.','')
      if property == 'symbol.style':
         val = __pychips_base__.get_histogram_symbolstyle(oid)
      elif property == 'symbol.color':
         val = __pychips_base__.get_histogram_symbolcolor(oid)
      elif property == 'symbol.size':
         val = __pychips_base__.get_histogram_symbolsize(oid)
      elif property == 'symbol.angle':
         val = __pychips_base__.get_histogram_symbolangle(oid)
      elif property == 'symbol.fill':
         val = __pychips_base__.get_histogram_symbolfill(oid)
      elif property == 'line.color':
         val = __pychips_base__.get_histogram_linecolor(oid)
      elif property == 'line.style':
         val = __pychips_base__.get_histogram_linestyle(oid)
      elif property == 'line.thickness':
         val = __pychips_base__.get_histogram_linethickness(oid)
      elif property == 'err.style':
         val = __pychips_base__.get_histogram_errstyle(oid)
      elif property == 'err.up':
         val = __pychips_base__.get_histogram_errup(oid)
      elif property == 'err.down':
         val = __pychips_base__.get_histogram_errdown(oid)
      elif property == 'err.color':
         val = __pychips_base__.get_histogram_errcolor(oid)
      elif property == 'err.caplength':
         val = __pychips_base__.get_histogram_errcaplength(oid)
      elif property == 'err.thickness':
         val = __pychips_base__.get_histogram_errthickness(oid)
      elif property == 'dropline':
         val = __pychips_base__.get_histogram_dropline(oid)
      elif property == 'depth':
         val = __pychips_base__.get_histogram_depth(oid)
      elif property == 'fill.color':
         val = __pychips_base__.get_histogram_fillcolor(oid)
      elif property == 'fill.opacity':
         val = __pychips_base__.get_histogram_fillopacity(oid)
      elif property == 'fill.style':
         val = __pychips_base__.get_histogram_fillstyle(oid)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;

def set_histogram(id=None, values=None):
   'Set the attributes of the specified histogram to the provided values'
   # use __pychips_base__.ChipsId, string histogram id, or no id (current) to determine histogram
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   if type(mid) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_histogram_id(mid)
   elif isinstance(mid, __pychips_base__.ChipsId):
      oid = mid
   elif mid == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsHistogram object
   ob = ChipsHistogram ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_histogram with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()
      if hasattr(ob, 'dropline') and ob.dropline != None:
         __pychips_base__.set_histogram_dropline(oid, __utils__.bool_cast(ob.dropline))
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_histogram_depth(oid, dv)
      if hasattr(ob, 'symbol'):
         if hasattr(ob.symbol, 'style') and ob.symbol.style != None:
            if type(ob.symbol.style) == str:
               intstyle = __pychips_base__.string_to_v_symbol(ob.symbol.style)
               if __pychips_base__.valid_symbol_type(intstyle):
                  __pychips_base__.set_histogram_symbolstyle(oid, intstyle)
	       else:
	          raise AttributeError("chips ERROR: The symbol style value (%s) must be one of [none, cross, diamond, downtriangle, circle, plus, square, uptriangle, point, arrow]"%ob.symbol.style)
            else:
               __pychips_base__.set_histogram_symbolstyle(oid, ob.symbol.style)
         if hasattr(ob.symbol, 'size') and ob.symbol.size != None:
            __pychips_base__.set_histogram_symbolsize(oid, int(ob.symbol.size))
         if hasattr(ob.symbol, 'angle') and ob.symbol.angle != None:
            __pychips_base__.set_histogram_symbolangle(oid, float(ob.symbol.angle))
         if hasattr(ob.symbol, 'fill') and ob.symbol.fill != None:
            __pychips_base__.set_histogram_symbolfill(oid, __utils__.bool_cast(ob.symbol.fill))
         if hasattr(ob.symbol, 'color') and ob.symbol.color != None:
            
            __pychips_base__.set_histogram_symbolcolor(oid, __utils__.get_color_as_string(ob.symbol.color))
      if hasattr(ob, 'line'):
         if hasattr(ob.line, 'color') and ob.line.color != None:
            
            __pychips_base__.set_histogram_linecolor(oid, __utils__.get_color_as_string (ob.line.color))
         if hasattr(ob.line, 'thickness') and ob.line.thickness != None:
            __pychips_base__.set_histogram_linethickness(oid, float(ob.line.thickness))
         if hasattr(ob.line, 'style') and ob.line.style != None:
            if type(ob.line.style) == str:
               intstyle = __pychips_base__.string_to_v_line(ob.line.style)
               if __pychips_base__.valid_line_type(intstyle):
                  __pychips_base__.set_histogram_linestyle(oid, intstyle)
	       else:
	          raise AttributeError("chips ERROR: The line style value (%s) must be one of [noline, solid, dot, dotlongdash, dotshortdash, longdash, shortdash, shortdashlongdash]"%ob.line.style)
            else:
               __pychips_base__.set_histogram_linestyle(oid, ob.line.style)
      if hasattr(ob, 'err'):
         if  hasattr(ob.err, 'color') and ob.err.color != None:
            __pychips_base__.set_histogram_errcolor(oid, __utils__.get_color_as_string (ob.err.color))
         if hasattr(ob.err, 'caplength') and ob.err.caplength != None:
            __pychips_base__.set_histogram_errcaplength(oid, int(ob.err.caplength))
         if hasattr(ob.err, 'thickness') and ob.err.thickness != None:
            __pychips_base__.set_histogram_errthickness(oid, float(ob.err.thickness))
         if hasattr(ob.err, 'style') and ob.err.style != None:
            __pychips_base__.set_histogram_errstyle(oid, ob.err.style)
         if hasattr(ob.err, 'up') and ob.err.up != None:
            __pychips_base__.set_histogram_errup(oid, __utils__.bool_cast(ob.err.up))
         if hasattr(ob.err, 'down') and ob.err.down != None:
            __pychips_base__.set_histogram_errdown(oid, __utils__.bool_cast(ob.err.down))
      if hasattr(ob, 'fill'):
         if hasattr(ob.fill, 'color') and ob.fill.color != None:
            
            __pychips_base__.set_histogram_fillcolor(oid, __utils__.get_color_as_string (ob.fill.color))
         if hasattr(ob.fill, 'opacity') and ob.fill.opacity != None:
            __pychips_base__.set_histogram_fillopacity(oid, float(ob.fill.opacity))
         if hasattr(ob.fill, 'style') and ob.fill.style != None:
            if type(ob.fill.style) == str:
               intstyle = __pychips_base__.string_to_v_fill(ob.fill.style)
               if __pychips_base__.valid_fill_type(intstyle):
                  __pychips_base__.set_histogram_fillstyle(oid, intstyle)
	       else:
                  raise AttributeError("chips ERROR: The fill style value (%s) must be one of [nofill, solidfill, updiagonal, downdiagonal, horizontal, vertical, crisscross, brick, grid, hexagon, polkadot, flower, userfill1, userfill2, userfill3]"%ob.fill.style)
            else:
               __pychips_base__.set_histogram_fillstyle(oid, ob.fill.style)
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
   return


def get_contour(id=None, property=None):
   'Returns a ChipsContour object with attribute values of requested contour'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_contour_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsContour();
      result.color = __pychips_base__.get_contour_color(oid)
      result.thickness = __pychips_base__.get_contour_thickness(oid)
      result.interval =  __pychips_base__.get_contour_interval(oid)
      result.levels =  __pychips_base__.get_contour_levels(oid)
      result.numlevels =  __pychips_base__.get_contour_numlevels(oid)
      result.mode = __pychips_base__.get_contour_mode(oid)
      result.algorithm = __pychips_base__.get_contour_algorithm(oid)
      result.style = __pychips_base__.get_contour_style(oid)
      result.depth = __pychips_base__.get_contour_depth(oid)
      return result
   else:
      # return specific property value
      if (property.find('contour') == 0):    # remove preceeding 'contour.'
         property = property.replace('contour.','')
      if property == 'style':
         val = __pychips_base__.get_contour_style(oid)
      elif property == 'color':
         val = __pychips_base__.get_contour_color(oid)
      elif property == 'thickness':
         val = __pychips_base__.get_contour_thickness(oid)
      elif property == 'depth':
         val = __pychips_base__.get_contour_depth(oid)
      elif property == 'mode':
         val = __pychips_base__.get_contour_mode(oid)
      elif property == 'algorithm':
         val = __pychips_base__.get_contour_algorithm(oid)
      elif property == 'levels':
         val =  __pychips_base__.get_contour_levels(oid)
      elif property == 'numlevels':
         val =  __pychips_base__.get_contour_numlevels(oid)
      elif property == 'interval':
         val = __pychips_base__.get_contour_interval(oid)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;

def set_contour(id=None, values=None):
   'Set the attributes of the specified contour to the provided values'
   # use __pychips_base__.ChipsId, string contour id, or no id (current) to determine contour 
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   oid = __utils__.set_id (mid, __pychips_base__.chips_contour)
   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsContour object
   ob = ChipsContour ()

   try: 
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_contour with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'thickness') and ob.thickness != None:
         __pychips_base__.set_contour_thickness(oid, float(ob.thickness))
      if hasattr(ob, 'style') and ob.style != None:
         __pychips_base__.set_contour_style(oid, ob.style)

      check = 0
      if hasattr(ob, 'interval') and ob.interval != None:
         check = check | 1
      if hasattr(ob, 'numlevels') and ob.numlevels != None:
         check = check | 2
      if hasattr(ob, 'levels') and __utils__.string_to_list(ob.levels) != None:
         check = check | 4
      if hasattr(ob, 'mode') and ob.mode != None:
         if check & 1 != 0:
            __pychips_base__.set_contour_interval(oid, float(ob.interval))
         if check & 2 != 0:
            __pychips_base__.set_contour_numlevels(oid, int(ob.numlevels))
         if check & 4 != 0:
            __pychips_base__.set_contour_levels(oid, ob.levels)
         __pychips_base__.set_contour_mode (oid, ob.mode)
      elif check == 1:
         __pychips_base__.set_contour_interval(oid, float(ob.interval))
      elif check == 2:
         __pychips_base__.set_contour_numlevels(oid, int(ob.numlevels))
      elif check == 4:
         __pychips_base__.set_contour_levels(oid, __utils__.string_to_list (ob.levels, float))
      elif check != 0:
         __pychips_base__.unlock()
         raise AttributeError("chips ERROR: You can not specify contour interval, numlevels and levels without specifying the mode.")

      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_contour_depth(oid, dv)
      if hasattr(ob, 'algorithm') and ob.algorithm != None:
         if type(ob.algorithm) == str: 
            intstyle = __pychips_base__.string_to_v_contour(ob.algorithm)
            if __pychips_base__.valid_contour_type(intstyle):
               __pychips_base__.set_contour_algorithm(oid, intstyle)
	    else:
	       raise AttributeError("chips ERROR: The algorithm value (%s) must be one of [standard, marching]"%ob.algorithm)
         else:
            __pychips_base__.set_contour_algorithm(oid, ob.algorithm)
      if hasattr(ob, 'color') and ob.color != None:
         __pychips_base__.set_contour_color(oid, __utils__.get_color_as_string (ob.color))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return

def get_colorbar(id=None, property=None):
   'Returns a ChipsColorBar object with attribute values of requested colorbar'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_colorbar_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   if property == None or property == 'all':
      # return property structure
      result = ChipsColorBar();
      result.depth = __pychips_base__.get_colorbar_depth(oid)
      result.length = __pychips_base__.get_colorbar_length(oid)
      result.width = __pychips_base__.get_colorbar_width(oid)
      result.border.visible = __pychips_base__.get_colorbar_border_visible(oid)
      result.halign = __pychips_base__.get_colorbar_halign(oid)
      result.valign = __pychips_base__.get_colorbar_valign(oid)
      result.orientation = __pychips_base__.get_colorbar_orientation(oid)
      result.label.text = __pychips_base__.get_colorbar_label_text(oid)
      result.label.color = __pychips_base__.get_colorbar_label_color(oid)
      result.label.size = __pychips_base__.get_colorbar_label_size(oid)
      result.label.font = __pychips_base__.get_colorbar_label_font(oid)
      result.label.fontstyle = __pychips_base__.get_colorbar_label_fontstyle(oid)
      result.label.angle = __pychips_base__.get_colorbar_label_angle(oid)
      result.label.halign = __pychips_base__.get_colorbar_label_halign(oid)
      result.label.valign = __pychips_base__.get_colorbar_label_valign(oid)
      result.label.visible = __pychips_base__.get_colorbar_label_visible(oid)
      result.label.location = __pychips_base__.get_colorbar_label_location(oid)
      result.offset.parallel = __pychips_base__.get_colorbar_parallel_offset(oid)
      result.offset.perpendicular = __pychips_base__.get_colorbar_perpendicular_offset(oid)
      result.tick.mode = __pychips_base__.get_colorbar_tick_mode(oid)
      result.tick.thickness = __pychips_base__.get_colorbar_tick_thickness(oid)
      result.tick.color = __pychips_base__.get_colorbar_tick_color(oid)
      result.tick.style = __pychips_base__.get_colorbar_tick_style(oid)
      result.tick.length = __pychips_base__.get_colorbar_tick_length(oid)
      result.tick.visible = __pychips_base__.get_colorbar_tick_visible(oid)
      result.tick.location = __pychips_base__.get_colorbar_tick_location(oid)
      result.ticklabel.color = __pychips_base__.get_colorbar_ticklabel_color(oid)
      result.ticklabel.size = __pychips_base__.get_colorbar_ticklabel_size(oid)
      result.ticklabel.angle = __pychips_base__.get_colorbar_ticklabel_angle(oid)
      result.ticklabel.valign = __pychips_base__.get_colorbar_ticklabel_valign(oid)
      result.ticklabel.halign = __pychips_base__.get_colorbar_ticklabel_halign(oid)
      result.ticklabel.font = __pychips_base__.get_colorbar_ticklabel_font(oid)
      result.ticklabel.fontstyle = __pychips_base__.get_colorbar_ticklabel_fontstyle(oid)
      result.ticklabel.offset = __pychips_base__.get_colorbar_ticklabel_offset(oid)
      result.ticklabel.visible = __pychips_base__.get_colorbar_ticklabel_visible(oid)
      return result
   else:
      # return specific property value
      if (property.find('colorbar') == 0):       # remove preceeding 'colorbar.'
         property = property.replace('colorbar.','')
      if property == 'depth':
         val = __pychips_base__.get_colorbar_depth(oid)
      elif property == 'length':
         val = __pychips_base__.get_colorbar_length(oid)
      elif property == 'width':
         val = __pychips_base__.get_colorbar_width(oid)
      elif property == 'border.visible':
         val = __pychips_base__.get_colorbar_border_visible(oid)
      elif property == 'valign':
         val = __pychips_base__.get_colorbar_valign(oid)
      elif property == 'orientation':
         val = __pychips_base__.get_colorbar_orientation(oid)
      elif property == 'label.color':
         val = __pychips_base__.get_colorbar_label_color(oid)
      elif property == 'label.size':
         val = __pychips_base__.get_colorbar_label_size(oid)
      elif property == 'label.font':
         val = __pychips_base__.get_colorbar_label_font(oid)
      elif property == 'label.fontstyle':
         val = __pychips_base__.get_colorbar_label_fontstyle(oid)
      elif property == 'label.angle':
         val = __pychips_base__.get_colorbar_label_angle(oid)
      elif property == 'label.text':
         val = __pychips_base__.get_colorbar_label_text(oid)
      elif property == 'label.halign':
         val = __pychips_base__.get_colorbar_label_halign(oid)
      elif property == 'label.valign':
         val = __pychips_base__.get_colorbar_label_valign(oid)
      elif property == 'label.visible':
         val = __pychips_base__.get_colorbar_label_visible(oid)
      elif property == 'label.location':
         val = __pychips_base__.get_colorbar_label_location(oid)
      elif property == 'offset.parallel':
         val = __pychips_base__.get_colorbar_parallel_offset(oid)
      elif property == 'offset.perpendicular':
         val = __pychips_base__.get_colorbar_perpendicular_offset(oid)
      elif property == 'tick.mode':
         val = __pychips_base__.get_colorbar_tick_mode(oid)
      elif property == 'tick.style':
         val = __pychips_base__.get_colorbar_tick_style(oid)
      elif property == 'tick.color':
         val = __pychips_base__.get_colorbar_tick_color(oid)
      elif property == 'tick.length':
         val = __pychips_base__.get_colorbar_tick_length(oid)
      elif property == 'tick.visible':
         val = __pychips_base__.get_colorbar_tick_visible(oid)
      elif property == 'tick.location':
         val = __pychips_base__.get_colorbar_tick_location(oid)
      elif property == 'ticklabel.color':
         val = __pychips_base__.get_colorbar_ticklabel_color(oid)
      elif property == 'ticklabel.size':
         val = __pychips_base__.get_colorbar_ticklabel_size(oid)
      elif property == 'ticklabel.angle':
         val = __pychips_base__.get_colorbar_ticklabel_angle(oid)
      elif property == 'ticklabel.valign':
         val = __pychips_base__.get_colorbar_ticklabel_valign(oid)
      elif property == 'ticklabel.halign':
         val = __pychips_base__.get_colorbar_ticklabel_halign(oid)
      elif property == 'ticklabel.font':
         val = __pychips_base__.get_colorbar_ticklabel_font(oid)
      elif property == 'ticklabel.fontstyle':
         val = __pychips_base__.get_colorbar_ticklabel_fontstyle(oid)
      elif property == 'ticklabel.offset':
         val = __pychips_base__.get_colorbar_ticklabel_offset(oid)
      elif property == 'ticklabel.visible':
         val = __pychips_base__.get_colorbar_ticklabel_visible(oid)
      else:
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
         val = None
      return val;

def set_colorbar(id=None, values=None):
   'Set the attributes of the specified colorbar to the provided values'

   # use __pychips_base__.ChipsId, string colorbar id, or no id (current)
   # to determine colorbar
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   oid = __utils__.set_id (mid, __pychips_base__.chips_colorbar)
   __pychips_base__.lock();

   ob = ChipsColorBar ()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsCurve object
   ob = ChipsColorBar ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_colorbar with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_colorbar_depth(oid, dv)
      if hasattr(ob, 'length') and ob.length != None:
         __pychips_base__.set_colorbar_length(oid, float(ob.length))
      if hasattr(ob, 'width') and ob.width != None:
         __pychips_base__.set_colorbar_width(oid, float(ob.width))
      if hasattr(ob, 'border'):
         if hasattr(ob.border, 'visible') and ob.border.visible != None:
            if __utils__.bool_cast(ob.border.visible) == True:
               __pychips_base__.display_colorbar_border(oid)
            else:
               __pychips_base__.hide_colorbar_border(oid)
      if hasattr(ob, 'halign') and ob.halign != None:
         if type(ob.halign) == str:
            alignval = __utils__.convert_offset(ob.halign)
            if alignval != None:
               __pychips_base__.set_colorbar_halign(oid, alignval)
         else:
            __pychips_base__.set_colorbar_halign(oid, ob.halign)
      if hasattr(ob, 'valign') and ob.valign != None:
         if type(ob.valign) == str:
            alignval = __utils__.convert_offset(ob.valign)
            if alignval != None:
               __pychips_base__.set_colorbar_valign(oid, alignval)
         else:
            __pychips_base__.set_colorbar_valign(oid, ob.valign)
      if hasattr(ob, 'orientation') and ob.orientation != None:
         orientation = __utils__.convert_orientation(ob.orientation)
         if orientation == None:
            raise AttributeError("chips ERROR: The orientation value (%s) must be one of [vertical|horizontal|0|1]"%str(ob.orientation))
         __pychips_base__.set_colorbar_orientation(oid, orientation)

      if hasattr(ob, 'label'):
         if hasattr(ob.label, 'text') and ob.label.text != None:
            __pychips_base__.set_colorbar_label_text(oid, ob.label.text)
         if hasattr(ob.label, 'color') and ob.label.color != None:
            __pychips_base__.set_colorbar_label_color(oid, __utils__.get_color_as_string (ob.label.color))
         if hasattr(ob.label, 'size') and ob.label.size != None:
            __pychips_base__.set_colorbar_label_size(oid, int(ob.label.size))
         if hasattr(ob.label, 'font') and ob.label.font != None:
            __pychips_base__.set_colorbar_label_font(oid, ob.label.font)
         if hasattr(ob.label, 'fontstyle') and ob.label.fontstyle != None:
            __pychips_base__.set_colorbar_label_fontstyle(oid, ob.label.fontstyle)
         if hasattr(ob.label, 'angle') and ob.label.angle != None:
            __pychips_base__.set_colorbar_label_angle(oid, float(ob.label.angle))
         if hasattr(ob.label, 'halign') and ob.label.halign != None:
            if type(ob.label.halign) == str:
               alignval = __utils__.convert_offset(ob.label.halign)
               if alignval != None:
                  __pychips_base__.set_colorbar_label_halign(oid, alignval)
            else:
               __pychips_base__.set_colorbar_label_halign(oid, ob.label.halign)
         if hasattr(ob.label, 'valign') and ob.label.valign != None:
            if type(ob.label.valign) == str:
               alignval = __utils__.convert_offset(ob.label.valign)
               if alignval != None:
                  __pychips_base__.set_colorbar_label_valign(oid, alignval)
            else:
               __pychips_base__.set_colorbar_label_valign(oid, ob.label.valign)
         if hasattr(ob.label, 'visible') and ob.label.visible != None:
            __pychips_base__.set_colorbar_label_visible(oid, __utils__.bool_cast(ob.label.visible))
         if hasattr(ob.label, 'location') and ob.label.location != None:
            __pychips_base__.set_colorbar_label_location(oid, ob.label.location)

      if hasattr(ob, 'offset') and hasattr(ob.offset, 'parallel') and ob.offset.parallel != None:
         __pychips_base__.set_colorbar_parallel_offset(oid, float(ob.offset.parallel))
      if hasattr(ob, 'offset') and hasattr(ob.offset, 'perpendicular') and ob.offset.perpendicular != None:
         __pychips_base__.set_colorbar_perpendicular_offset(oid, float(ob.offset.perpendicular))

      # interval and count can internally modify mode- need special handling
      if hasattr(ob, 'tick'):
         check = 0
         if hasattr(ob.tick, 'mode') and ob.tick.mode != None:
            __pychips_base__.set_colorbar_tick_mode(oid, ob.tick.mode)
         if hasattr(ob.tick, 'style') and ob.tick.style != None:
            __pychips_base__.set_colorbar_tick_style(oid, ob.tick.style)
         if hasattr(ob.tick, 'thickness') and ob.tick.thickness != None:
            __pychips_base__.set_colorbar_tick_thickness(oid, float(ob.tick.thickness))
         if hasattr(ob.tick, 'color') and ob.tick.color != None:
            __pychips_base__.set_colorbar_tick_color(oid, __utils__.get_color_as_string (ob.tick.color))
         if hasattr(ob.tick, 'length') and ob.tick.length != None:
            __pychips_base__.set_colorbar_tick_length(oid, int(ob.tick.length))
         if hasattr(ob.tick, 'visible') and ob.tick.visible != None:
             vis_cast = __utils__.bool_cast(ob.tick.visible)
             __pychips_base__.set_colorbar_tick_visible (oid, vis_cast)
         if hasattr(ob.tick, 'location') and ob.tick.location != None:
            __pychips_base__.set_colorbar_tick_location(oid, ob.tick.location)

      if hasattr(ob, 'ticklabel'):
         if hasattr(ob.ticklabel, 'color') and ob.ticklabel.color != None:
            __pychips_base__.set_colorbar_ticklabel_color(oid, __utils__.get_color_as_string (ob.ticklabel.color))
         if hasattr(ob.ticklabel, 'size') and ob.ticklabel.size != None:
            __pychips_base__.set_colorbar_ticklabel_size(oid, int(ob.ticklabel.size))
         if hasattr(ob.ticklabel, 'font') and ob.ticklabel.font != None:
            __pychips_base__.set_colorbar_ticklabel_font(oid, ob.ticklabel.font)
         if hasattr(ob.ticklabel, 'fontstyle') and ob.ticklabel.fontstyle != None:
            __pychips_base__.set_colorbar_ticklabel_fontstyle(oid, ob.ticklabel.fontstyle)
         if hasattr(ob.ticklabel, 'offset') and ob.ticklabel.offset != None:
            __pychips_base__.set_colorbar_ticklabel_offset(oid, int(ob.ticklabel.offset))
         if hasattr(ob.ticklabel, 'angle') and ob.ticklabel.angle != None:
            __pychips_base__.set_colorbar_ticklabel_angle(oid, float(ob.ticklabel.angle))
         if hasattr(ob.ticklabel, 'halign') and ob.ticklabel.halign != None:
            if type(ob.ticklabel.halign) == str:
               alignval = __utils__.convert_offset(ob.ticklabel.halign)
               if alignval != None:
                  __pychips_base__.set_colorbar_ticklabel_halign(oid, alignval)
               else:
                  __pychips_base__.set_colorbar_ticklabel_halign(oid, ob.ticklabel.halign)
         if hasattr(ob.ticklabel, 'valign') and ob.ticklabel.valign != None:
            if type(ob.ticklabel.valign) == str:
               alignval = __utils__.convert_offset(ob.ticklabel.valign)
               if alignval != None:
                  __pychips_base__.set_colorbar_ticklabel_valign(oid, alignval)
               else:
                  __pychips_base__.set_colorbar(oid, ob.ticklabel.valign)
         if hasattr(ob.ticklabel, 'visible') and ob.ticklabel.visible != None:
            __pychips_base__.set_colorbar_ticklabel_visible(oid, __utils__.bool_cast(ob.ticklabel.visible))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return
   
def get_image(id=None, property=None):
   'Returns a ChipsImage object with attribute values of requested image'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_image_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsImage();
      result.colormap = __pychips_base__.get_image_colormap(oid)
      result.interpolation = __pychips_base__.get_image_interpolation(oid)
      result.invert_colormap =  __pychips_base__.get_image_invert_colormap(oid)
      result.colormap_size =  __pychips_base__.get_image_colormap_size(oid)
      result.colormap_interpolate =  __pychips_base__.get_image_colormap_interpolate(oid)
      result.alpha =  __pychips_base__.get_image_alpha(oid)
      result.threshold =  __pychips_base__.get_image_threshold(oid)
      result.depth = __pychips_base__.get_image_depth(oid)
      result.scale_channels = __pychips_base__.get_image_scale_channels(oid)
      return result
   else:
      # return specific property value
      if (property.find('image') == 0):    # remove preceeding 'image.'
         property = property.replace('image.','')
      if property == 'colormap':
         val = __pychips_base__.get_image_colormap(oid)
      elif property == 'invert_colormap':
         val = __pychips_base__.get_image_invert_colormap(oid)
      elif property == 'colormap_interpolate':
         val = __pychips_base__.get_image_colormap_interpolate(oid)
      elif property == 'colormap_size':
         val = __pychips_base__.get_image_colormap_size(oid)
      elif property == 'scale_channels':
         val = __pychips_base__.get_image_scale_channels(oid)
      elif property == 'interpolation':
         val = __pychips_base__.get_image_interpolation(oid)
      elif property == 'depth':
         val = __pychips_base__.get_image_depth(oid)
      elif property == 'alpha':
         val = __pychips_base__.get_image_alpha(oid)
      elif property == 'threshold':
         val = __pychips_base__.get_image_threshold(oid)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;

def set_image(id=None, values=None):
   'Set the attributes of the specified image to the provided values'
   # use __pychips_base__.ChipsId, string image id, or no id (current) to determine image 
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   oid = __utils__.set_id (mid, __pychips_base__.chips_image)
   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsImage object
   ob = ChipsImage()

   try: 
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_image with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'invert_colormap') and ob.invert_colormap != None:
         __pychips_base__.set_image_invert_colormap(oid, __utils__.bool_cast(ob.invert_colormap))
      if hasattr(ob, 'colormap_interpolate') and ob.colormap_interpolate != None:
         __pychips_base__.set_image_colormap_interpolate(oid, __utils__.bool_cast(ob.colormap_interpolate))
      if hasattr(ob, 'colormap_size') and ob.colormap_size != None:
         __pychips_base__.set_image_colormap_size(oid, int(ob.colormap_size))
      if hasattr(ob, 'scale_channels') and ob.scale_channels != None:
         __pychips_base__.set_image_scale_channels(oid, __utils__.bool_cast(ob.scale_channels))
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_image_depth(oid, dv)

      if hasattr(ob, 'colormap') and ob.colormap != None:
         if type(ob.colormap) == str: 
            intstyle = __pychips_base__.string_to_v_colormap(ob.colormap)
            if __pychips_base__.valid_colormap_type(intstyle):
               __pychips_base__.set_image_colormap(oid, intstyle)
	    else:
	       raise AttributeError("chips ERROR: The colormap value (%s) must be one of [red, green, blue, grayscale, rainbow, hsv, heat, cool, usercmap1, usercmap2, usercmap3]"%ob.colormap)
         else:
            __pychips_base__.set_image_colormap(oid, ob.colormap)

      if hasattr(ob, 'interpolation') and ob.interpolation != None:
         if type(ob.interpolation) == str: 
            intstyle = __pychips_base__.string_to_v_interpolation(ob.interpolation)
            if __pychips_base__.valid_interpolation_type(intstyle):
               __pychips_base__.set_image_interpolation(oid, intstyle)
	    else:
	       raise AttributeError("chips ERROR: The interpolation value (%s) must be one of [neighbor, bilinear, bicubic]"%ob.interpolation)
         else:
            __pychips_base__.set_image_interpolation(oid, ob.interpolation)
      if hasattr(ob, 'alpha') and __utils__.string_to_list(ob.alpha) != None:
	 vals = __utils__.string_to_list(ob.alpha, float)
	 if len(vals) == 1:
            __pychips_base__.set_image_alpha(oid, vals[0], vals[0])
	 elif len(vals) == 2:
            __pychips_base__.set_image_alpha(oid, vals[0], vals[1])
         else:
	    raise AttributeError("chips ERROR: The alpha list should contain at most two values (%s)"%ob.alpha)

      if hasattr(ob, 'threshold') and __utils__.string_to_list(ob.threshold) != None:
	 vals = __utils__.string_to_list(ob.threshold, float)
	 if len(vals) == 2:
            __pychips_base__.set_image_threshold(oid, vals[0], vals[1])
         else:
	    raise AttributeError("chips ERROR: The threshold list should contain two values (%s)"%ob.threshold)
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return


def get_label(id=None, property=None):
   'Returns a ChipsLabel object with attribute values of requested label'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_label_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsLabel();
      result.color = __pychips_base__.get_label_color(oid)
      result.size = __pychips_base__.get_label_size(oid)
      result.angle = __pychips_base__.get_label_angle(oid)
      result.depth = __pychips_base__.get_label_depth(oid)
      result.halign = __pychips_base__.get_label_halign(oid)
      result.valign = __pychips_base__.get_label_valign(oid)
      result.font = __pychips_base__.get_label_font(oid)
      result.fontstyle = __pychips_base__.get_label_fontstyle(oid)
      return result
   else:
      # return specific property value
      if (property.find('label') == 0):    # remove preceeding 'label.'
         property = property.replace('label.','')
      if property == 'size':
         val = __pychips_base__.get_label_size(oid)
      elif property == 'color':
         val = __pychips_base__.get_label_color(oid)
      elif property == 'angle':
         val = __pychips_base__.get_label_angle(oid)
      elif property == 'depth':
         val = __pychips_base__.get_label_depth(oid)
      elif property == 'halign':
         val = __pychips_base__.get_label_halign(oid)
      elif property == 'valign':
         val = __pychips_base__.get_label_valign(oid)
      elif property == 'font':
         val = __pychips_base__.get_label_font(oid)
      elif property == 'fontstyle':
         val = __pychips_base__.get_label_fontstyle(oid)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;

def set_label(id=None, values=None):
   'Set the attributes of the specified label to the provided values'
   # use __pychips_base__.ChipsId, string label id, or no id (current) to determine label 
   if values == None:   # handle values without ids 
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   # setup default object id 
   if type(mid) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_label_id(mid)
   elif isinstance(mid, __pychips_base__.ChipsId):
      oid = mid
   elif mid == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsLabel object
   ob = ChipsLabel ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_label with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'size') and ob.size != None:
         __pychips_base__.set_label_size(oid, int(ob.size))
      if hasattr(ob, 'angle') and ob.angle != None:
         __pychips_base__.set_label_angle(oid, float(ob.angle))
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
	 if dv != None:
            __pychips_base__.set_label_depth(oid, dv)
      if hasattr(ob, 'halign') and ob.halign != None:
         if type(ob.halign) == str:
	    alignval = __utils__.convert_offset(ob.halign)
	    if alignval != None:
               __pychips_base__.set_label_halign(oid, alignval)
         else:
            __pychips_base__.set_label_halign(oid, ob.halign)
      if hasattr(ob, 'valign') and ob.valign != None:
         if type(ob.valign) == str:
	    alignval = __utils__.convert_offset(ob.valign)
	    if alignval != None:
               __pychips_base__.set_label_valign(oid, alignval)
         else:
            __pychips_base__.set_label_valign(oid, ob.valign)
      if hasattr(ob, 'font') and ob.font != None:
         __pychips_base__.set_label_font(oid, ob.font)
      if hasattr(ob, 'fontstyle') and ob.fontstyle != None:
         __pychips_base__.set_label_fontstyle(oid, ob.fontstyle)
      if hasattr(ob, 'color') and ob.color != None:
         __pychips_base__.set_label_color(oid, __utils__.get_color_as_string (ob.color))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return

def get_line(id=None, property=None):
   'Returns a ChipsLine object with attribute values of requested line'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_line_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsLine();
      result.color = __pychips_base__.get_line_color(oid)
      result.depth = __pychips_base__.get_line_depth(oid)
      result.thickness = __pychips_base__.get_line_thickness(oid)
      result.style = __pychips_base__.get_line_style(oid)
      result.extend = __pychips_base__.get_line_extend(oid)
      return result
   else:
      # return specific property value
      if (property.find('line') == 0):    # remove preceeding 'line.'
         property = property.replace('line.','')
      if property == 'style':
         val = __pychips_base__.get_line_style(oid)
      elif property == 'color':
         val = __pychips_base__.get_line_color(oid)
      elif property == 'depth':
         val = __pychips_base__.get_line_depth(oid)
      elif property == 'thickness':
         val = __pychips_base__.get_line_thickness(oid)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;

def set_line(id=None, values=None):
   'Set the attributes of the specified line to the provided values'
   # use __pychips_base__.ChipsId, string line id, or no id (current) to determine line 
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   if type(mid) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_line_id(mid)
   elif isinstance(mid, __pychips_base__.ChipsId):
      oid = mid
   elif mid == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsLine object
   ob = ChipsLine ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_line with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'thickness') and ob.thickness != None:
         __pychips_base__.set_line_thickness(oid, float(ob.thickness))
      if hasattr(ob, 'style') and ob.style != None:
         if type(ob.style) == str:
            intstyle = __pychips_base__.string_to_v_line(ob.style)
            if __pychips_base__.valid_line_type(intstyle):
               __pychips_base__.set_line_style(oid, intstyle)
	    else:
	       raise AttributeError("chips ERROR: The style value (%s) must be one of [noline, solid, dot, dotlongdash, dotshortdash, longdash, shortdash, shortdashlongdash]"%ob.style)
         else:
            __pychips_base__.set_line_style(oid, ob.style)
      if hasattr(ob, 'extend') and ob.extend != None:
         if type(ob.extend) == str:
            intextend = __pychips_base__.string_to_v_line_extension(ob.extend)
            if __pychips_base__.valid_line_extension_type(intextend):
               __pychips_base__.set_line_extend(oid, intextend)
	    else:
	       raise AttributeError("chips ERROR: The extend value (%s) must be one of [none, start, end, both]"%ob.extend)
         else:
            __pychips_base__.set_line_extend(oid, ob.extend)
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_line_depth(oid, dv)
      if hasattr(ob, 'color') and ob.color != None:
         __pychips_base__.set_line_color(oid, __utils__.get_color_as_string (ob.color))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return

def get_point(id=None, property=None):
   'Returns a ChipsPoint object with attribute values of requested point'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_point_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsPoint();
      result.color = __pychips_base__.get_point_color(oid)
      result.size = __pychips_base__.get_point_size(oid)
      result.depth = __pychips_base__.get_point_depth(oid)
      result.style = __pychips_base__.get_point_style(oid)
      result.angle = __pychips_base__.get_point_angle(oid)
      result.fill = __pychips_base__.get_point_fill(oid)
      return result
   else:
      # return specific property value
      if (property.find('point') == 0):    # remove preceeding 'point.'
         property = property.replace('point.','')
      if property == 'style':
         val = __pychips_base__.get_point_style(oid)
      if property == 'depth':
         val = __pychips_base__.get_point_depth(oid)
      elif property == 'color':
         val = __pychips_base__.get_point_color(oid)
      elif property == 'size':
         val = __pychips_base__.get_point_size(oid)
      elif property == 'fill':
         val = __pychips_base__.get_point_fill(oid)
      elif property == 'angle':
         val = __pychips_base__.get_point_angle(oid)
      else:
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
         val = None
      return val;

def set_point(id=None, values=None):
   'Set the attributes of the specified point to the provided values'
   # use __pychips_base__.ChipsId, string point id, or no id (current) to determine point 
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   if type(mid) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_point_id(mid)
   elif isinstance(mid, __pychips_base__.ChipsId):
      oid = mid
   elif mid == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsPoint object
   ob = ChipsPoint ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_point with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'size') and ob.size != None:
         __pychips_base__.set_point_size(oid, int(ob.size))
      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_point_depth(oid, dv)
      if hasattr(ob, 'style') and ob.style != None:
         if type(ob.style) == str:
            intstyle = __pychips_base__.string_to_v_symbol(ob.style)
            if __pychips_base__.valid_symbol_type(intstyle):
               __pychips_base__.set_point_style(oid, intstyle)
	    else:
	       raise AttributeError("chips ERROR: The style value (%s) must be one of [none, cross, diamond, downtriangle, circle, plus, square, uptriangle, point, arrow]"%ob.style)
         else:
            __pychips_base__.set_point_style(oid, ob.style)
      if hasattr(ob, 'color') and ob.color != None:
         __pychips_base__.set_point_color(oid, __utils__.get_color_as_string (ob.color))
      if hasattr(ob, 'angle') and ob.angle != None:
         __pychips_base__.set_point_angle(oid, float(ob.angle))
      if hasattr(ob, 'fill') and ob.fill != None:
         __pychips_base__.set_point_fill(oid, __utils__.bool_cast(ob.fill))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return


def get_region(id=None, property=None):
   'Returns a ChipsRegion object with attribute values of requested region'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_region_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
      return None 

   if property == None or property == 'all':
      # return property structure
      result = ChipsRegion();
      result.angle = __pychips_base__.get_region_angle(oid)
      result.depth = __pychips_base__.get_region_depth(oid)
      result.opacity = __pychips_base__.get_region_opacity(oid)
      result.fill.color = __pychips_base__.get_region_fillcolor(oid)
      result.fill.style = __pychips_base__.get_region_fillstyle(oid)
      result.edge.thickness = __pychips_base__.get_region_edgethickness(oid)
      result.edge.style = __pychips_base__.get_region_edgestyle(oid)
      result.edge.color = __pychips_base__.get_region_edgecolor(oid)
      return result
   else:
      # return specific property value
      if (property.find('region') == 0):    # remove preceeding 'region.'
         property = property.replace('region.','')
      if property == 'edge.style':
         val = __pychips_base__.get_region_edgestyle(oid)
      elif property == 'edge.color':
         val = __pychips_base__.get_region_edgecolor(oid)
      elif property == 'edge.thickness':
         val = __pychips_base__.get_region_edgethickness(oid)
      elif property == 'fill.color':
         val = __pychips_base__.get_region_fillcolor(oid)
      elif property == 'fill.style':
         val = __pychips_base__.get_region_fillstyle(oid)
      elif property == 'angle':
         val = __pychips_base__.get_region_angle(oid)
      elif property == 'depth':
         val = __pychips_base__.get_region_depth(oid)
      elif property == 'opacity':
         val = __pychips_base__.get_region_opacity(oid)
      else:
         val = None
         raise AttributeError("chips ERROR: Unknown or unexpected property name")
      return val;

def set_region(id=None, values=None):
   'Set the attributes of the specified region to the provided values'
   # use __pychips_base__.ChipsId, string region id, or no id (current) to determine region 
   if values == None:   # handle values without ids
      if id == None:
         raise AttributeError("chips ERROR: No attribute values specified")
      mid = None
      vals = id
   else:
      # both provided- need to verify order
      if type(id) == str or isinstance(id, __pychips_base__.ChipsId) or id == None:
         mid = id;
         vals = values;
      else:
         mid = values
         vals = id

   if type(mid) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_region_id(mid)
   elif isinstance(mid, __pychips_base__.ChipsId):
      oid = mid
   elif mid == None:
      oid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
      return None

   __pychips_base__.lock()

   # if input is a string attribute list convert to attribute value pairs
   if type(vals) == str:
      vals = __utils__.string_to_attribute_list(vals)

   # convert data structure to ChipsRegion object
   ob = ChipsRegion ()

   try:
      ob = __utils__.data_to_attribute_object (ob, vals)
   except:
      __pychips_base__.unlock()
      raise

   object_type = __utils__.get_object_type (ob);
   if object_type == None:
      __pychips_base__.unlock()
      raise AttributeError("chips ERROR: You cannot call set_region with %s "%type(ob))

   try:
      __pychips_base__.open_undo_buffer()

      if hasattr(ob, 'depth') and ob.depth != None:
         dv = __utils__.convert_depth(ob.depth)
         if dv != None:
            __pychips_base__.set_region_depth(oid, dv)
      if hasattr(ob, 'opacity') and ob.opacity != None:
         __pychips_base__.set_region_opacity(oid, float(ob.opacity))
      if hasattr(ob, 'fill'):
         if hasattr(ob.fill, 'color') and ob.fill.color != None:
            __pychips_base__.set_region_fillcolor(oid,  __utils__.get_color_as_string (ob.fill.color))
         if hasattr(ob.fill, 'style') and ob.fill.style != None:
            if type(ob.fill.style) == str:
               intstyle = __pychips_base__.string_to_v_fill(ob.fill.style)
               if __pychips_base__.valid_fill_type(intstyle):
                  __pychips_base__.set_region_fillstyle(oid, intstyle)
	       else:
                  raise AttributeError("chips ERROR: The fill style value (%s) must be one of [nofill, solidfill, updiagonal, downdiagonal, horizontal, vertical, crisscross, brick, grid, hexagon, polkadot, flower, userfill1, userfill2, userfill3]"%ob.fill.style)
            else:
               __pychips_base__.set_region_fillstyle(oid, ob.fill.style)
      if hasattr(ob, 'edge'):
         if hasattr(ob.edge, 'color') and ob.edge.color != None:
            __pychips_base__.set_region_edgecolor(oid,  __utils__.get_color_as_string (ob.edge.color))
         if hasattr(ob.edge, 'style') and ob.edge.style != None:
            if type(ob.edge.style) == str:
               intstyle = __pychips_base__.string_to_v_line(ob.edge.style)
               if __pychips_base__.valid_line_type(intstyle):
                  __pychips_base__.set_region_edgestyle(oid, intstyle)
	       else:
                  raise AttributeError("chips ERROR: The edge style value (%s) must be one of [noline, solid, dot, dotlongdash, dotshortdash, longdash, shortdash, shortdashlongdash]"%ob.edge.style)
            else:
               __pychips_base__.set_region_edgestyle(oid, ob.edge.style)
         if hasattr(ob.edge, 'thickness') and ob.edge.thickness != None:
            __pychips_base__.set_region_edgethickness(oid, float(ob.edge.thickness))
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
      return


def get_preferences(item=None):
   'Gets the preference values for the specified item'

   # a list of valid object types
   obj_types = [__pychips_base__.chips_window, 
                __pychips_base__.chips_frame,
                __pychips_base__.chips_plot,
                __pychips_base__.chips_curve,
                __pychips_base__.chips_contour,
                __pychips_base__.chips_histogram,
                __pychips_base__.chips_image,
                __pychips_base__.chips_axis,
                __pychips_base__.chips_colorbar,
                __pychips_base__.chips_line,
                __pychips_base__.chips_label,
                __pychips_base__.chips_point,
                __pychips_base__.chips_region,
                __pychips_base__.chips_not_object]
   if item == None:           # get all preferences  
      prefs = ChipsPreferences()
      mode = __pychips_base__.chips_not_object;
   elif item in obj_types:    # get specified preference object
      if item == __pychips_base__.chips_not_object:
         prefs = ChipsPreferences()
      else:
         prefs = None
      mode = item;
   elif type(item) == str:    # get specified preference value 
      return __pychips_base__.get_preference(item)
   else:
      raise AttributeError("chips ERROR: Unknown item specified in get_preferences")
      return
   if mode == __pychips_base__.chips_window  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.window
      else:
         obj = ChipsWindow()
      obj.height = float(__pychips_base__.get_preference("window.height"))
      obj.width = float(__pychips_base__.get_preference("window.width"))
      obj.units = __pychips_base__.get_preference("window.units")
      obj.scaleheight = float(__pychips_base__.get_preference("window.scaleheight"))
      obj.scalewidth = float(__pychips_base__.get_preference("window.scalewidth"))
      obj.display = __utils__.bool_cast(__pychips_base__.get_preference("window.display"))
      obj.smoothing = __utils__.bool_cast(__pychips_base__.get_preference("window.smoothing"))
      obj.stem = __pychips_base__.get_preference("window.stem")
      obj.bgcolor = __pychips_base__.get_preference("window.bgcolor")
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_frame  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.frame
      else:
         obj = ChipsFrame()
      obj.bgcolor = __pychips_base__.get_preference("frame.bgcolor")
      obj.transparency = __utils__.bool_cast(__pychips_base__.get_preference("frame.transparency"))
      obj.border.visible = __utils__.bool_cast(__pychips_base__.get_preference("frame.border.visible"))
      obj.border.color = __pychips_base__.get_preference("frame.border.color")
      obj.mode = __pychips_base__.get_preference("frame.mode")
      obj.aspect = __utils__.bool_cast(__pychips_base__.get_preference("frame.aspect"))
      obj.scale = __utils__.bool_cast(__pychips_base__.get_preference("frame.scale"))
      obj.stem = __pychips_base__.get_preference("frame.stem")
      if mode != __pychips_base__.chips_not_object:
         return obj  
   if mode == __pychips_base__.chips_plot  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.plot
      else:
         obj = ChipsPlot()
      obj.title.xpos = float(__pychips_base__.get_preference("plot.title.xpos"))
      obj.title.ypos = float(__pychips_base__.get_preference("plot.title.ypos"))
      obj.title.depth = __pychips_base__.get_preference("plot.title.depth")
      obj.title.angle = float(__pychips_base__.get_preference("plot.title.angle"))
      obj.title.halign = __pychips_base__.get_preference("plot.title.halign")
      obj.title.valign = __pychips_base__.get_preference("plot.title.valign")
      obj.title.color = __pychips_base__.get_preference("plot.title.color")
      obj.title.size = int(__pychips_base__.get_preference("plot.title.size"))
      obj.title.font = __pychips_base__.get_preference("plot.title.font")
      obj.title.fontstyle = __pychips_base__.get_preference("plot.title.fontstyle")
      obj.style = __pychips_base__.get_preference("plot.style")
      obj.corner.style = __pychips_base__.get_preference("plot.corner.style")
      obj.aspect.ratio = __pychips_base__.get_preference("plot.aspect.ratio")
      obj.leftmargin = float(__pychips_base__.get_preference("plot.leftmargin"))
      obj.rightmargin = float(__pychips_base__.get_preference("plot.rightmargin"))
      obj.topmargin = float(__pychips_base__.get_preference("plot.topmargin"))
      obj.bottommargin = float(__pychips_base__.get_preference("plot.bottommargin"))
      obj.stem = __pychips_base__.get_preference("plot.stem")
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_axis  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.axis
      else:
         obj = ChipsAxis()
      obj.x.stem = __pychips_base__.get_preference("axis.x.stem")
      obj.y.stem = __pychips_base__.get_preference("axis.y.stem")
      obj.x.label.angle = __pychips_base__.get_preference("axis.x.label.angle")
      obj.x.label.text =  __pychips_base__.get_preference("axis.x.label.text")
      obj.y.label.angle = __pychips_base__.get_preference("axis.y.label.angle")
      obj.y.label.text =  __pychips_base__.get_preference("axis.y.label.text")
      obj.color = __pychips_base__.get_preference("axis.color")
      tmp = __pychips_base__.get_preference("axis.thickness")
      if tmp == '0.5':
         obj.thickness = 0.5
      else:
         obj.thickness = int(tmp)
      obj.depth = __pychips_base__.get_preference("axis.depth")
      obj.label.color = __pychips_base__.get_preference("axis.label.color")
      obj.label.size = int(__pychips_base__.get_preference("axis.label.size"))
      obj.label.font = __pychips_base__.get_preference("axis.label.font")
      obj.label.fontstyle = __pychips_base__.get_preference("axis.label.fontstyle")
      obj.label.halign = __pychips_base__.get_preference("axis.label.halign")
      obj.label.valign = __pychips_base__.get_preference("axis.label.valign")
      obj.majorgrid.visible = __utils__.bool_cast(__pychips_base__.get_preference("axis.majorgrid.visible"))
      obj.majorgrid.color = __pychips_base__.get_preference("axis.majorgrid.color")
      obj.majorgrid.style = __pychips_base__.get_preference("axis.majorgrid.style")
      tmp = __pychips_base__.get_preference("axis.majorgrid.thickness")
      if tmp == '0.5':
         obj.majorgrid.thickness = 0.5
      else:
         obj.majorgrid.thickness = int(tmp)
      obj.minorgrid.visible = __utils__.bool_cast(__pychips_base__.get_preference("axis.minorgrid.visible"))
      obj.minorgrid.color = __pychips_base__.get_preference("axis.minorgrid.color")
      obj.minorgrid.style = __pychips_base__.get_preference("axis.minorgrid.style")
      tmp = __pychips_base__.get_preference("axis.minorgrid.thickness")
      if tmp == '0.5':
         obj.minorgrid.thickness = 0.5
      else:
         obj.minorgrid.thickness = int(tmp)
      obj.automin = __utils__.bool_cast(__pychips_base__.get_preference("axis.automin"))
      obj.automax = __utils__.bool_cast(__pychips_base__.get_preference("axis.automax"))
      obj.pad = float(__pychips_base__.get_preference("axis.pad"))
      obj.offset.parallel = float(__pychips_base__.get_preference("axis.offset.parallel"))
      obj.offset.perpendicular = float(__pychips_base__.get_preference("axis.offset.perpendicular"))
      obj.tickformat = __pychips_base__.get_preference("axis.tickformat")
      obj.ticklabel.style = __pychips_base__.get_preference("axis.ticklabel.style")
      obj.ticklabel.color = __pychips_base__.get_preference("axis.ticklabel.color")
      obj.ticklabel.size = int(__pychips_base__.get_preference("axis.ticklabel.size"))
      obj.ticklabel.angle = float(__pychips_base__.get_preference("axis.ticklabel.angle"))
      obj.ticklabel.halign = __pychips_base__.get_preference("axis.ticklabel.halign")
      obj.ticklabel.valign = __pychips_base__.get_preference("axis.ticklabel.valign")
      obj.ticklabel.font = __pychips_base__.get_preference("axis.ticklabel.font")
      obj.ticklabel.fontstyle = __pychips_base__.get_preference("axis.ticklabel.fontstyle")
      obj.ticklabel.offset = int(__pychips_base__.get_preference("axis.ticklabel.offset"))
      obj.majortick.mode = __pychips_base__.get_preference("axis.majortick.mode")
      obj.majortick.interval = float(__pychips_base__.get_preference("axis.majortick.interval"))
      obj.majortick.count = int(__pychips_base__.get_preference("axis.majortick.count"))
      tmp = __pychips_base__.get_preference("axis.majortick.thickness")
      if tmp == '0.5':
         obj.majortick.thickness = 0.5
      else:
         obj.majortick.thickness = int(tmp)
      obj.majortick.color = __pychips_base__.get_preference("axis.majortick.color")
      obj.majortick.style = __pychips_base__.get_preference("axis.majortick.style")
      obj.majortick.length = int(__pychips_base__.get_preference("axis.majortick.length"))
      obj.majortick.visible = __utils__.bool_cast(__pychips_base__.get_preference("axis.majortick.visible"))
      obj.minortick.mode = __pychips_base__.get_preference("axis.minortick.mode")
      obj.minortick.interval = float(__pychips_base__.get_preference("axis.minortick.interval"))
      obj.minortick.count = int(__pychips_base__.get_preference("axis.minortick.count"))
      tmp = __pychips_base__.get_preference("axis.minortick.thickness")
      if tmp == '0.5':
         obj.minortick.thickness = 0.5
      else:
         obj.minortick.thickness = int(tmp)
      obj.minortick.color = __pychips_base__.get_preference("axis.minortick.color")
      obj.minortick.style = __pychips_base__.get_preference("axis.minortick.style")
      obj.minortick.length = int(__pychips_base__.get_preference("axis.minortick.length"))
      obj.minortick.visible = __utils__.bool_cast(__pychips_base__.get_preference("axis.minortick.visible"))
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_curve  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.curve
      else:
         obj = ChipsCurve()
      obj.stem = __pychips_base__.get_preference("curve.stem")
      obj.baddata = __pychips_base__.get_preference("curve.baddata")
      obj.depth = __pychips_base__.get_preference("curve.depth")
      obj.line.color = __pychips_base__.get_preference("curve.line.color")
      tmp = __pychips_base__.get_preference("curve.line.thickness")
      if tmp == '0.5':
         obj.line.thickness = 0.5
      else:
         obj.line.thickness = int(tmp)
      obj.line.style = __pychips_base__.get_preference("curve.line.style")
      obj.symbol.color = __pychips_base__.get_preference("curve.symbol.color")
      obj.symbol.style = __pychips_base__.get_preference("curve.symbol.style")
      obj.symbol.size = int(__pychips_base__.get_preference("curve.symbol.size"))
      obj.symbol.angle = int(__pychips_base__.get_preference("curve.symbol.angle"))
      obj.symbol.fill = __utils__.bool_cast(__pychips_base__.get_preference("curve.symbol.fill"))
      obj.err.style = __pychips_base__.get_preference("curve.err.style")
      obj.err.up = __utils__.bool_cast(__pychips_base__.get_preference("curve.err.up"))
      obj.err.down = __utils__.bool_cast(__pychips_base__.get_preference("curve.err.down"))
      obj.err.left = __utils__.bool_cast(__pychips_base__.get_preference("curve.err.left"))
      obj.err.right = __utils__.bool_cast(__pychips_base__.get_preference("curve.err.right"))
      obj.err.color = __pychips_base__.get_preference("curve.err.color")
      obj.err.caplength = int(__pychips_base__.get_preference("curve.err.caplength"))
      tmp = __pychips_base__.get_preference("curve.err.thickness")
      if tmp == '0.5':
         obj.err.thickness = 0.5
      else:
         obj.err.thickness = int(tmp)
      obj.limit.length = __pychips_base__.get_preference("curve.limit.length")
      obj.limit.override = __utils__.bool_cast(__pychips_base__.get_preference("curve.limit.override"))
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_histogram  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.histogram
      else:
         obj = ChipsHistogram()
      obj.stem = __pychips_base__.get_preference("histogram.stem")
      obj.depth = __pychips_base__.get_preference("histogram.depth")
      obj.line.color = __pychips_base__.get_preference("histogram.line.color")
      tmp = __pychips_base__.get_preference("histogram.line.thickness")
      if tmp == '0.5':
         obj.line.thickness = 0.5
      else:
         obj.line.thickness = int(tmp)
      obj.line.style = __pychips_base__.get_preference("histogram.line.style")
      obj.symbol.color = __pychips_base__.get_preference("histogram.symbol.color")
      obj.symbol.style = __pychips_base__.get_preference("histogram.symbol.style")
      obj.symbol.size = int(__pychips_base__.get_preference("histogram.symbol.size"))
      obj.symbol.angle = int(__pychips_base__.get_preference("histogram.symbol.angle"))
      obj.symbol.fill = __utils__.bool_cast(__pychips_base__.get_preference("histogram.symbol.fill"))
      obj.err.style = __pychips_base__.get_preference("histogram.err.style")
      obj.err.up = __utils__.bool_cast(__pychips_base__.get_preference("histogram.err.up"))
      obj.err.down = __utils__.bool_cast(__pychips_base__.get_preference("histogram.err.down"))
      obj.err.color = __pychips_base__.get_preference("histogram.err.color")
      obj.err.caplength = int(__pychips_base__.get_preference("histogram.err.caplength"))
      tmp = __pychips_base__.get_preference("histogram.err.thickness")
      if tmp == '0.5':
         obj.err.thickness = 0.5
      else:
         obj.err.thickness = int(tmp)
      obj.dropline = __utils__.bool_cast(__pychips_base__.get_preference("histogram.dropline"))
      obj.fill.color = __pychips_base__.get_preference("histogram.fill.color")
      obj.fill.opacity = float (__pychips_base__.get_preference("histogram.fill.opacity"))
      obj.fill.style = __pychips_base__.get_preference("histogram.fill.style")
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_contour  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.contour
      else:
         obj = ChipsContour()
      obj.stem = __pychips_base__.get_preference("contour.stem")
      obj.depth = __pychips_base__.get_preference("contour.depth")
      obj.color = __pychips_base__.get_preference("contour.color")
      tmp = __pychips_base__.get_preference("contour.thickness")
      if tmp == '0.5':
         obj.thickness = 0.5
      else:
         obj.thickness = int(tmp)
      obj.style = __pychips_base__.get_preference("contour.style")
      obj.levels = __utils__.string_to_list (__pychips_base__.get_preference("contour.levels"),float)
      obj.numlevels = int(__pychips_base__.get_preference("contour.numlevels"))
      obj.algorithm = __pychips_base__.get_preference("contour.algorithm")
      obj.mode = __pychips_base__.get_preference("contour.mode")
      obj.interval = float(__pychips_base__.get_preference("contour.interval"))
      if mode != __pychips_base__.chips_not_object:
         return obj

   if mode == __pychips_base__.chips_label  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.label
      else:
         obj = ChipsLabel()
      obj.stem = __pychips_base__.get_preference("label.stem")
      obj.font = __pychips_base__.get_preference("label.font")
      obj.color = __pychips_base__.get_preference("label.color")
      obj.size = int(__pychips_base__.get_preference("label.size"))
      obj.fontstyle = __pychips_base__.get_preference("label.fontstyle")
      obj.angle = float(__pychips_base__.get_preference("label.angle"))
      obj.halign = __pychips_base__.get_preference("label.halign")
      obj.valign = __pychips_base__.get_preference("label.valign")
      obj.depth = __pychips_base__.get_preference("label.depth")
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_line  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.line
      else:
         obj = ChipsLine()
      obj.stem = __pychips_base__.get_preference("line.stem")
      obj.color = __pychips_base__.get_preference("line.color")
      tmp = __pychips_base__.get_preference("line.thickness")
      if tmp == '0.5':
         obj.thickness = 0.5
      else:
         obj.thickness = int(tmp)
      obj.depth = __pychips_base__.get_preference("line.depth")
      obj.style = __pychips_base__.get_preference("line.style")
      obj.extend = __pychips_base__.get_preference("line.extend")
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_point  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.point
      else:
         obj = ChipsPoint()
      obj.stem = __pychips_base__.get_preference("point.stem")
      obj.color = __pychips_base__.get_preference("point.color")
      obj.style = __pychips_base__.get_preference("point.style")
      obj.depth = __pychips_base__.get_preference("point.depth")
      obj.size = int(__pychips_base__.get_preference("point.size"))
      obj.fill = __utils__.bool_cast(__pychips_base__.get_preference("point.fill"))
      obj.angle = float(__pychips_base__.get_preference("point.angle"))
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_region  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.region
      else:
         obj = ChipsRegion()
      obj.stem = __pychips_base__.get_preference("region.stem")
      obj.fill.color = __pychips_base__.get_preference("region.fill.color")
      obj.fill.style = __pychips_base__.get_preference("region.fill.style")
      tmp = __pychips_base__.get_preference("region.edge.thickness")
      if tmp == '0.5':
         obj.edge.thickness = 0.5
      else:
         obj.edge.thickness = int(tmp)
      obj.edge.style = __pychips_base__.get_preference("region.edge.style")
      obj.edge.color = __pychips_base__.get_preference("region.edge.color")
      obj.angle = float(__pychips_base__.get_preference("region.angle"))
      obj.depth = __pychips_base__.get_preference("region.depth")
      obj.opacity = float(__pychips_base__.get_preference("region.opacity"))
      if mode != __pychips_base__.chips_not_object:
         return obj
   if mode == __pychips_base__.chips_image  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.image
      else:
         obj = ChipsImage ()

      obj.stem = __pychips_base__.get_preference("image.stem") 
      obj.colormap = __pychips_base__.get_preference("image.colormap")
      obj.interpolation = __pychips_base__.get_preference ("image.interpolation");
      obj.depth = __pychips_base__.get_preference ("image.depth")
      try:
         obj.depth = int (obj.depth)
      except:
         pass
      obj.alpha = __pychips_base__.get_preference ("image.alpha")
      obj.alpha = obj.alpha[1:len(obj.alpha)-1]
      obj.alpha = obj.alpha.split (',')
      obj.alpha = [float(ii) for ii in obj.alpha]
      obj.threshold = __pychips_base__.get_preference ("image.threshold")
      obj.threshold = obj.threshold[1:len(obj.threshold)-1]
      obj.threshold = obj.threshold.split (',')
      obj.threshold = [float(ii) for ii in obj.threshold]
      obj.invert_colormap = __pychips_base__.get_preference ("image.invert_colormap")
      obj.colormap_size = int (__pychips_base__.get_preference ("image.colormap_size"))
      obj.colormap_interpolate = __pychips_base__.get_preference ("image.colormap_interpolate")
      obj.scale_channels = __pychips_base__.get_preference ("image.scale_channels");

      if mode != __pychips_base__.chips_not_object:
         return obj

   if mode == __pychips_base__.chips_colorbar  or   mode == __pychips_base__.chips_not_object:
      if mode == __pychips_base__.chips_not_object:
         obj = prefs.colorbar
      else:
         obj = ChipsColorBar ()
      obj.stem = __pychips_base__.get_preference ("colorbar.stem")
      obj.width = float (__pychips_base__.get_preference ("colorbar.width"))
      obj.length = float (__pychips_base__.get_preference ("colorbar.length"))
      obj.border.visible = __pychips_base__.get_preference ("colorbar.border.visible")
      obj.depth = __pychips_base__.get_preference ("colorbar.depth")
      try:
         obj.depth = int(obj.depth)
      except:
         pass
      obj.halign = __pychips_base__.get_preference ("colorbar.halign")
      try:
         obj.halign = float (obj.halign)
      except:
         pass
      obj.valign = __pychips_base__.get_preference ("colorbar.valign")
      try:
         obj.valign = float (obj.valign)
      except:
         pass
      obj.orientation = __pychips_base__.get_preference ("colorbar.orientation")
      obj.tick.mode = __pychips_base__.get_preference ("colorbar.tick.mode")
      obj.tick.location = __pychips_base__.get_preference ("colorbar.tick.location")
      tmp = __pychips_base__.get_preference("colorbar.tick.thickness")
      if tmp == '0.5':
         obj.tick.thickness = 0.5
      else:
         obj.tick.thickness = int(tmp)
      obj.tick.length = float (__pychips_base__.get_preference ("colorbar.tick.length"))
      obj.tick.color = __pychips_base__.get_preference ("colorbar.tick.color")
      obj.tick.visible = __pychips_base__.get_preference ("colorbar.tick.visible")
      obj.tick.style = __pychips_base__.get_preference ("colorbar.tick.style")
      obj.ticklabel.color = __pychips_base__.get_preference ("colorbar.ticklabel.color")
      obj.ticklabel.size = int (__pychips_base__.get_preference ("colorbar.ticklabel.size"))
      obj.ticklabel.angle = float (__pychips_base__.get_preference ("colorbar.ticklabel.angle"))
      obj.ticklabel.halign = __pychips_base__.get_preference ("colorbar.ticklabel.halign")
      try:
         obj.ticklabel.halign = float (obj.ticklabel.halign)
      except:
         pass

      obj.ticklabel.valign = __pychips_base__.get_preference ("colorbar.ticklabel.valign") 
      try:
         obj.ticklabel.valign = float (obj.ticklabel.valign)
      except:
         pass

      obj.ticklabel.font = __pychips_base__.get_preference ("colorbar.ticklabel.font")
      obj.ticklabel.fontstyle = __pychips_base__.get_preference ("colorbar.ticklabel.fontstyle")
      obj.ticklabel.visible = __pychips_base__.get_preference ("colorbar.ticklabel.visible")
      obj.ticklabel.offset = __pychips_base__.get_preference ("colorbar.ticklabel.offset")
      obj.label.text = __pychips_base__.get_preference ("colorbar.label.text")
      obj.label.angle = float (__pychips_base__.get_preference ("colorbar.label.angle"))
      obj.label.halign = __pychips_base__.get_preference ("colorbar.label.halign")
      try:
         obj.label.halign = float (obj.label.halign)
      except:
         pass
      obj.label.valign = __pychips_base__.get_preference ("colorbar.label.valign")
      try:
         obj.label.valign = float (obj.label.valign)
      except:
         pass
      obj.label.location = __pychips_base__.get_preference ("colorbar.label.location")
      obj.label.color = __pychips_base__.get_preference ("colorbar.label.color")
      obj.label.size = int (__pychips_base__.get_preference ("colorbar.label.size"))
      obj.label.font = __pychips_base__.get_preference ("colorbar.label.font")
      obj.label.fontstyle = __pychips_base__.get_preference ("colorbar.label.fontstyle")
      obj.label.visible = __pychips_base__.get_preference ("colorbar.label.visible")
      obj.offset.parallel = __pychips_base__.get_preference ("colorbar.offset.parallel")
      obj.offset.perpendicular = __pychips_base__.get_preference ("colorbar.offset.perpendicular")

      if mode != __pychips_base__.chips_not_object:
         return obj

      
   if mode == __pychips_base__.chips_not_object:
      prefs.foreground.file = __pychips_base__.get_preference("foreground.file")
      prefs.background.file = __pychips_base__.get_preference("background.file")
      prefs.foreground.display = __pychips_base__.get_preference("foreground.display")
      prefs.background.display = __pychips_base__.get_preference("background.display")
      prefs.pick.mask = __pychips_base__.get_preference("pick.mask")
      prefs.pick.color = __pychips_base__.get_preference("pick.color")
      prefs.export.fonts = __pychips_base__.get_preference("export.fonts")
      prefs.export.format = __pychips_base__.get_preference("export.format")
      prefs.export.orientation = __pychips_base__.get_preference("export.orientation")
      prefs.export.colorscheme = __pychips_base__.get_preference("export.colorscheme")
      prefs.export.dpi = __pychips_base__.get_preference("export.dpi")
      prefs.export.leftmargin = float(__pychips_base__.get_preference("export.leftmargin"))
      prefs.export.rightmargin = float(__pychips_base__.get_preference("export.rightmargin"))
      prefs.export.topmargin = float(__pychips_base__.get_preference("export.topmargin"))
      prefs.export.bottommargin = float(__pychips_base__.get_preference("export.bottommargin"))
      prefs.export.pagewidth = float(__pychips_base__.get_preference("export.pagewidth"))
      prefs.export.pageheight = float(__pychips_base__.get_preference("export.pageheight"))
      prefs.export.pagesize = __pychips_base__.get_preference("export.pagesize")
      prefs.export.pageunits = __pychips_base__.get_preference("export.pageunits")
      prefs.export.scaleheight = float(__pychips_base__.get_preference("export.scaleheight"))
      prefs.export.scalewidth = float(__pychips_base__.get_preference("export.scalewidth"))
      prefs.export.fittopage = __utils__.bool_cast(__pychips_base__.get_preference("export.fittopage"))
      prefs.export.clobber = __utils__.bool_cast(__pychips_base__.get_preference("export.clobber"))
      prefs.export.keepaspect = __utils__.bool_cast(__pychips_base__.get_preference("export.keepaspect"))
      prefs.export.printdialog = __utils__.bool_cast(__pychips_base__.get_preference("export.printdialog"))
      prefs.info.current = __utils__.bool_cast(__pychips_base__.get_preference("info.current"))
      prefs.info.coord = __utils__.bool_cast(__pychips_base__.get_preference("info.coord")) 
      prefs.info.depth = __utils__.bool_cast(__pychips_base__.get_preference("info.depth")) 
      prefs.default.depth = __pychips_base__.get_preference("default.depth") 
      prefs.default.verbosity = __pychips_base__.get_preference("default.verbosity") 
   return prefs

def _set_pref_of_object(inp, prefix=''):
   'Sets preference values for object entries'

   from pychips import attribute_objects as attr_objs

   # flatten structure to preference attribute string
   object_type = __utils__.get_object_type (inp);
   if (object_type != None):
      if object_type != 'preferences':
         prefix = prefix + object_type
         if prefix != '':
            prefix = prefix + '.'
   else:
       raise AttributeError("chips ERROR: You cannot call set_preferences with a %s" % type(inp))

   for x in dir(inp):
      xval = getattr(inp,x)
      if xval != None and x[0] != '_':
         if xval.__class__.__name__ in attr_objs._object_types:
            # recursive call to next next depth
            _set_pref_of_object(xval, prefix)
         elif __utils__.check_seq (xval):
            list_attr = '['
            delim = ''
            for subelem in xval:
               list_attr = list_attr + delim + "%s"%subelem
               delim = ','
            list_attr = list_attr + ']'
            __pychips_base__.set_preference(prefix+x, list_attr)

         # check color/bgcolor props (ignore color* such as colormap/colorbar)
         elif x.rstrip().endswith("color"): 
            # color stored as an int- output as a 6 digit hex string (rrggbb)
            __pychips_base__.set_preference(prefix+x, __utils__.get_color_as_string (xval))
         else:
            __pychips_base__.set_preference(prefix+x, '%s' % str(xval))
   return


def set_preferences(values):
   "Sets chips' internal defaults to the specified values"

   __pychips_base__.lock()
   __pychips_base__.open_undo_buffer()

   try:
      if type(values) == str:
         # handle 'a=val' string
         pair = values.split('=')
         # if input is a string attribute list convert to attribute value pairs
         vals = __utils__.string_to_attribute_list(values)
         if len(vals) % 2 == 0:
            for elem in range(0, len(vals), 2):
               __pychips_base__.set_preference(vals[elem], vals[elem+1])
         else:
            raise AttributeError("chips ERROR: attribute string contains an uneven number of attributes and values.")
      else:
         
         # convert data structure to ChipsPreferences object
         pob = ChipsPreferences ()

         pob = __utils__.data_to_attribute_object (pob, values)

         # call recursive routine to set preferences
         _set_pref_of_object(pob)

   except:
      __pychips_base__.discard_undo_buffer ()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
   return




#####################################################
#
#  new set currency functions
#
#####################################################


def set_current(object_type, object_id): 
   "Sets the specified object current"
   if type(object_type) == str:
      raise AttributeError("chips ERROR: expected a V_OBJECT_TYPE enumeration as the first (object_type) parameter")
   if type(object_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_object_id(object_type, object_id)
   elif isinstance(object_id, __pychips_base__.ChipsId):
      id = object_id;
   else:
      raise AttributeError("chips ERROR: Object id  parameter must be a string or ChipsId object")
   if object_type == __pychips_base__.chips_window:
      __pychips_base__.set_current_window(id);
   elif object_type == __pychips_base__.chips_frame:
      __pychips_base__.set_current_frame(id);
   elif object_type == __pychips_base__.chips_plot:
      __pychips_base__.set_current_plot(id);
   elif object_type == __pychips_base__.chips_axis:
      __pychips_base__.set_current_axis(id);
   elif object_type == __pychips_base__.chips_curve:
      __pychips_base__.set_current_curve(id);
   elif object_type == __pychips_base__.chips_histogram:
      __pychips_base__.set_current_histogram(id);
   elif object_type == __pychips_base__.chips_contour:
      __pychips_base__.set_current_contour(id);
   elif object_type == __pychips_base__.chips_image:
      __pychips_base__.set_current_image(id);
   elif object_type == __pychips_base__.chips_colorbar:
      __pychips_base__.set_current_colorbar(id);
   elif object_type == __pychips_base__.chips_label:
      __pychips_base__.set_current_label(id);
   elif object_type == __pychips_base__.chips_line:
      __pychips_base__.set_current_line(id);
   elif object_type == __pychips_base__.chips_point:
      __pychips_base__.set_current_point(id);
   elif object_type == __pychips_base__.chips_region:
      __pychips_base__.set_current_region(id);
   else:
     raise AttributeError("chips ERROR: Unsupported object type (%d) specified in set_current" % __pychips_base__.v_object_to_string(obtype))
   return

def current_window(window_id):
   'Sets the current window to the specified window id'
   if type(window_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_window_id(window_id)
   else:
      id = window_id;
   __pychips_base__.set_current_window(id);
   return

def current_frame(frame_id):
   'Sets the current frame of the current window to the specified frame id'
   if type(frame_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_frame_id(frame_id)
   else:
      id = frame_id;
   __pychips_base__.set_current_frame(id);
   return

def current_plot(plot_id):
   'Sets the current plot of the current frame to the specified plot id'
   if type(plot_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_plot_id(plot_id)
   else:
      id = plot_id;
   __pychips_base__.set_current_plot(id);
   return

def current_axis(axis_id):
   'Sets the current axis of the current plot to the specified axis id'
   if type(axis_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_axis_id(axis_id)
   else:
      id = axis_id;
   __pychips_base__.set_current_axis(id);
   return

def current_curve(curve_id):
   'Sets the current curve of the current plot to the specified curve id'
   if type(curve_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_curve_id(curve_id)
   else:
      id = curve_id;
   __pychips_base__.set_current_curve(id);
   return

def current_image(image_id):
   'Sets the current image of the current plot to the specified image id'
   if type(image_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_image_id(image_id)
   else:
      id = image_id;
   __pychips_base__.set_current_image(id);
   return

def current_colorbar(colorbar_id):
   'Sets the current colorbar of the current plot to the specified colorbar id'
   if type(colorbar_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_colorbar_id(colorbar_id)
   else:
      id = colorbar_id;
   __pychips_base__.set_current_colorbar(id);
   return

def current_contour(contour_id):
   'Sets the current contour of the current plot to the specified contour id'
   if type(contour_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_contour_id(contour_id)
   else:
      id = contour_id;
   __pychips_base__.set_current_contour(id);
   return

def current_histogram(histogram_id):
   'Sets the current histogram of the current plot to the specified histogram id'
   if type(histogram_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_histogram_id(histogram_id)
   else:
      id = histogram_id;
   __pychips_base__.set_current_histogram(id);
   return

def current_label(label_id):
   'Sets the current label of the current frame to the specified label id'
   if type(label_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_label_id(label_id)
   else:
      id = label_id;
   __pychips_base__.set_current_label(id);
   return

def current_line(line_id):
   'Sets the current line of the current frame to the specified line id'
   if type(line_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_line_id(line_id)
   else:
      id = line_id;
   __pychips_base__.set_current_line(id);
   return

def current_point(point_id):
   'Sets the current point of the current frame to the specified point id'
   if type(point_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_point_id(point_id)
   else:
      id = point_id;
   __pychips_base__.set_current_point(id);
   return

def current_region(region_id):
   'Sets the current region of the current frame to the specified region id'
   if type(region_id) == str:
      id = __pychips_base__.ChipsId()
      id.set_region_id(region_id)
   else:
      id = region_id;
   __pychips_base__.set_current_region(id);
   return


#####################################################
#
#  high level deletion routines
#
#####################################################


def clear():
   'Deletes all chips windows'
   __pychips_base__.lock()
   if __pychips_base__.get_object_count(__pychips_base__.chips_window) != 0:
      __pychips_base__.delete_all_windows();
   __pychips_base__.unlock()

def erase(id=None):
   """
   Erases all frames from within the current window(s). A ChipsId or string window
   id may be passed in to specfify which window to operate on.
   """
   if type(id) == str:
      cid = __pychips_base__.ChipsId()
      cid.set_window_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      cid = id
   elif id == None:
      cid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Expected either no arguments or an optional id (ChipsId, None, or string).")

   __pychips_base__.lock()

   if __pychips_base__.get_object_count(__pychips_base__.chips_frame) != 0:
      try:
         __pychips_base__.open_undo_buffer()
         __pychips_base__.delete_all_frames(cid)
      except:
         __pychips_base__.discard_undo_buffer()
         __pychips_base__.unlock()
         raise
      else:
         __pychips_base__.close_undo_buffer()
   __pychips_base__.unlock()


#####################################################
#
#  plot layout functions
#
#####################################################

def strip_chart(entries, orientation=0, gaps=0, ratios=None, bindings=1, plotstyle=__pychips_base__.chips_closed, xlimits=None, ylimits=None, ticklabels=False):
   """
Creates a frame containing a template strip chart containing multiple plots. Entries specifies the number of sections to create. Orientation is either vertical (0) or horizontal (1) stacking. Gaps is a value between 0 and 1. Ratios is a list with 1 value per entry specifying the size ratios of each entry. Bindings specifies which axes to bind (0 = none, 1 = the stacked axis, 2 = both x and y axes). Plotstyle specifies the plot border style (chips_open|chips_closed|chips_box). Xlimits and Ylimts are optional lists specifying the mininum and maximum values to use. Ticklabls specifies whether ticklabels should be drawn for all the stacked axes or not.  
   """
   if entries < 2:
      raise AttributeError("chips ERROR: The number of plot entries (%d) to partition must be greater than 1." % entries)
   if ratios != None:
      if __utils__.check_seq(ratios) == False:
         raise AttributeError("chips ERROR: Ratios must be specified as a list")
      elif len(ratios) != entries: 
         raise AttributeError("chips ERROR: Ratios list must equal the number of entries")
   else:
      ratios = numpy.arange(entries)**0    # set ratio of 1-1
   if bindings == 0:
      bind = 0
   elif bindings == 1:
      if orientation == 0:
         bind = __pychips_base__.X_AXIS
      else:
         bind = __pychips_base__.Y_AXIS
   elif bindings == 2:
      bind = __pychips_base__.XY_AXIS
   if gaps < 0 or gaps > 1:
      raise AttributeError("chips ERROR: Gaps value (%3.2f) must be less than 1 and greater than or equal to 0" % gaps)

   # support optional x and y limits
   if xlimits == None:
      xmin = 0
      xmax = 1
   elif __utils__.check_seq(xlimits) == False:
      raise AttributeError("chips ERROR: X limits must be specified as [xmin xmax] if provided.")
   elif len(xlimits) == 2: 
      xmin = xlimits[0]
      xmax = xlimits[1]
   else:
      raise AttributeError("chips ERROR: Limits list should only contain 2 values.")
   if ylimits == None:
      ymin = 0
      ymax = 1
   elif __utils__.check_seq(ylimits) == False:
      raise AttributeError("chips ERROR: Y limits must be specified as [ymin ymax] if provided.")
   elif len(ylimits) == 2: 
      ymin = ylimits[0]
      ymax = ylimits[1]
   else:
      raise AttributeError("chips ERROR: Limits list should only contain 2 values.")

   __pychips_base__.lock()

   try:
      __pychips_base__.open_undo_buffer()

      # track window currency
      window_current  = __utils__.get_current_window_id()

      add_frame ()
      if orientation == 0: 
         # stacked vertically 
         __pychips_base__.grid_objects(1, entries,0, gaps,1)
         __pychips_base__.adjust_grid_yrelsizes(ratios)
      else:
         # stacked horizontally
         __pychips_base__.grid_objects(entries, 1, gaps, 0, 1)
         __pychips_base__.adjust_grid_xrelsizes(ratios)
      id = ChipsId()
      set_preferences(['plot.style', plotstyle])
      id.plot = "all"
      __pychips_base__.add_axis(id,__pychips_base__.XY_AXIS, 0, xmin, xmax, ymin, ymax, None)
      if __pychips_base__.get_preference('axis.automin')  == 'true':
         __pychips_base__.set_axis_automin(id,1)
      else:
         __pychips_base__.set_axis_automin(id,0)
      if __pychips_base__.get_preference('axis.automax')  == 'true':
         __pychips_base__.set_axis_automax(id,1)
      else:
         __pychips_base__.set_axis_automax(id,0)

      # set up the ids to use based on preference object stems
      aidx = "%s1" % __pychips_base__.get_preference('axis.x.stem')
      aidy = "%s1" % __pychips_base__.get_preference('axis.y.stem')
      lid = __pychips_base__.get_preference('plot.stem')
      lid1 = lid + '1'

      # set up axis bindings 
      if bind != 0:
         if window_current == None or window_current != 'all': 
            if bind == __pychips_base__.X_AXIS or bind == __pychips_base__.XY_AXIS: 
               # bind x axes
               for r in range(2, entries+1):
	          lid2 = lid + '%d' % r
                  __pychips_base__.bind_axes(lid1,aidx,lid2,aidx)
            if bind == __pychips_base__.Y_AXIS or bind == __pychips_base__.XY_AXIS: 
               # bind y axis
               for r in range(2, entries+1):
      	          lid2 = lid + '%d' % r
                  __pychips_base__.bind_axes(lid1,aidy,lid2,aidy)
         else:
   	    # since lib doesn't support 'all' windows for bind, do each separately
            tid = ChipsId()
            if bind == __pychips_base__.X_AXIS or bind == __pychips_base__.XY_AXIS: 
               # bind x axes
               for i in range (0, len(window_list), 2):
                  tid.reset_window_id(window_list[i])
                  for r in range(2, entries+1):
	             tid.reset_plot_id(lid1)
                     tid.set_axis_id(aidx)
	             lid2 = lid + '%d' % r
                     __pychips_base__.bind_axes(tid,lid2,aidx)
            if bind == __pychips_base__.Y_AXIS or bind == __pychips_base__.XY_AXIS: 
               # bind y axis
               for i in range (0, len(window_list), 2):
                  tid.reset_window_id(window_list[i])
                  for r in range(2, entries+1):
	             tid.reset_plot_id(lid1)
                     tid.set_axis_id(aidy)
	             lid2 = lid + '%d' % r
                     __pychips_base__.bind_axes(tid,lid2,aidy)

      # show/hide ticks
      if ticklabels == False:
         if orientation == 0:
   	    id.axis = aidx
            for r in range(1,entries):
   	       id.plot = lid + '%d' % r
	       __pychips_base__.set_axis_ticklabel_visible(id, 0)
         else:
   	    id.axis = aidy
            for r in range(2,entries+1):
	       id.plot = lid + '%d' % r
	       __pychips_base__.set_axis_ticklabel_visible(id, 0)

      current_plot(lid1);
      if window_current != None:
         # since we didn't add a window reset currency to initial state
         current_window(window_current)
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
  
   return


def split(id=None, rows=None, cols=None, ygap=None, xgap=None):
   """
   Creates a grid of rows by cols plot areas in a window. If no parameters are supplied, a pair of vertically stacked drawing areas are provided. If a single parameter is supplied it is treated as the number of rows. When two parameters are passed they are treated as the number of rows and columns. An optional third parameter is used as the gap spacing between rows of plot drawing areas. The optional fourth argument is treated as the gaps between columns.
   """
   idexists = True
   if type(id) == str:
      cid = __pychips_base__.ChipsId()
      cid.set_window_id(id)
      idexists = True
   elif isinstance(id, __pychips_base__.ChipsId):
      cid = id
   elif id == None:
      cid = __pychips_base__.ChipsId()
   elif __utils__.isinstance_int(id):
      idexists = False
      cid = __pychips_base__.ChipsId()
   else:
      raise AttributeError("chips ERROR: Expected either an id (ChipsId, None, or string) or rows (integer) as the first argument.")

   if idexists:
      if rows == None:
         rr = 2
      else:
         rr = rows
      if cols == None:
         cc = 1
      else:
         cc = cols
      if ygap == None:
         yg = 0
      else:
         yg = ygap
      if xgap == None:
         xg = 0
      else:
         xg = xgap
   elif xgap != None:
      raise AttributeError("chips ERROR: Since id was not present, expected up to four arguments in the order: rows, cols, ygaps, xgaps")
   else:
      rr = id
      if rows == None:
         cc = 1
      else:
         cc = rows
      if cols == None:
         yg = 0
      else:
         yg =cols
      if ygap == None:
         xg = 0
      else:
         xg = ygap

   if rr < 1:
      raise AttributeError("chips ERROR: The number of rows must be greater than 1 (%d)." % rr)
   if cc < 1:
      raise AttributeError("chips ERROR: The number of columns must be greater than 1 (%d)." % cc)
   if yg < 0 or yg > 1:
      raise AttributeError("chips ERROR: The y gap value (%3.2f) must be at least 0 but less than 1." % yg)
   if xg < 0 or xg > 1:
      raise AttributeError("chips ERROR: The x gap value (%3.2f) must be at least 0 but less than 1." % xg)

   __pychips_base__.lock()

   try:
      __pychips_base__.open_undo_buffer();
      if __pychips_base__.get_object_count(cid,__pychips_base__.chips_frame) == 0:
         rd = __pychips_base__.get_object_count(cid,__pychips_base__.chips_window)
         __pychips_base__.add_frame(cid)
         if rd == 0:
            __pychips_base__.set_window_redraw(__utils__.bool_cast(__pychips_base__.get_preference('window.display')))
      __pychips_base__.grid_objects(cid,cc,rr,xg,yg,1)
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock()
   return



#####################################################
#
#  move functions
#
#####################################################

def move_frame (*args):
   """
   move_frame ([ChipsId,], xval, yval, [mode])

   Moves the specified frame by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_frame)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_frame, *endargs)

def move_plot (*args):
   """
   move_plot ([ChipsId,], xval, yval, [mode])

   Moves the specified plot by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_plot)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_plot, *endargs)


def move_axis (*args):
   """
   move_axis ([ChipsId,], xval, yval, [mode])

   Moves the specified axis by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_axis)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_axis, *endargs)

def move_label(*args):
   """
   move_label ([ChipsId,], xval, yval, [mode])

   Moves the specified label by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_label)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_label, *endargs)

def move_line (*args):
   """
   move_line ([ChipsId,], xval, yval, [mode])

   Moves the specified line by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_line)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_line, *endargs)

def move_point(*args):
   """
   move_point ([ChipsId,], xval, yval, [mode])

   Moves the specified point by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_point)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_point, *endargs)

def move_region(*args):
   """
   move_region ([ChipsId,], xval, yval, [mode])

   Moves the specified region by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_region)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_region, *endargs)

def move_colorbar(*args):
   """
   move_colorbar ([ChipsId,], xval, yval, [mode])

   Moves the specified colorbar by absolute or relative xval and yval based upon the setting of mode (0=absolute, 1=delta).
   """

   myid = None
   endargs = args
   if (len(args) != 0):
      try:
         myid = __utils__.set_id(args[0], __pychips_base__.chips_colorbar)
         endargs = args[1:]
      except:
         pass
   __pychips_base__.move (myid, __pychips_base__.chips_colorbar, *endargs)



#####################################################
#
#  depth functions
#
#####################################################


def shuffle_axis(id=None, way=None):
   "Shuffles the specified axis's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_axis)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_axis, way)

def shuffle_curve(id=None, way=None):
   "Shuffles the specified curve's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_curve)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_curve, way)

def shuffle_image(id=None, way=None):
   "Shuffles the specified image's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_image)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_image, way)

def shuffle_colorbar(id=None, way=None):
   "Shuffles the specified colorbar's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_colorbar)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_colorbar, way)



def shuffle_contour(id=None, way=None):
   "Shuffles the specified contour's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_contour)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_contour, way)

def shuffle_histogram(id=None, way=None):
   "Shuffles the specified histogram's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_histogram)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_histogram, way)

def shuffle_label(id=None, way=None):
   "Shuffles the specified label's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_label)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_label, way)

def shuffle_line(id=None, way=None):
   "Shuffles the specified line's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_line)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_line, way)

def shuffle_point(id=None, way=None):
   "Shuffles the specified point's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_point)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_point, way)

def shuffle_region(id=None, way=None):
   "Shuffles the specified region's position within its depth based on way- chips_forward(0)|chips_backward(1)|chips_front(2)|chips_back(3)"
   if way == None:
      if __utils__.isinstance_int(id):
         way = id
         id = None
      else:
         way = __pychips_base__.chips_forward
   myid = __utils__.set_id(id, __pychips_base__.chips_region)
   __pychips_base__.shuffle(myid, __pychips_base__.chips_region, way)

def reindex_depth(id=None, olddepth=None, newdepth=None):
   'Moves all items in olddepth to newdepth'
   if type(id) == str:
      oid = __pychips_base__.ChipsId()
      oid.set_frame_id(id)
   elif isinstance(id, __pychips_base__.ChipsId):
      oid = id
   elif id == None:
      oid = __pychips_base__.ChipsId()
   elif __utils__.isinstance_int(id): 
      if newdepth == None:
         if __utils__.isinstance_int(olddepth):
            # treat id as old value and olddepth as new 
            __pychips_base__.collapse_depths(None, [id], olddepth);
            return
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type")
   else:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type") 
   if __utils__.isinstance_int(olddepth):
      if __utils__.isinstance_int(newdepth):
         if olddepth != newdepth:
            __pychips_base__.collapse_depths(oid, [olddepth], newdepth)
      else:
         raise AttributeError("chips ERROR: New depth needs to be an integer value")
   else:
      raise AttributeError("chips ERROR: Old depth needs to be an integer value")


#####################################################
#
#  new object creation functions
#
#####################################################


def add_hline (*args):
   """
   add_hline([ChipsId,] b [,attribute_list])
  
   Adds a horizontal line which intercepts the y axis at 'b'. The
   horizontal line extends indefinitely as limits and ranges
   of a coordinate system change. The call creates a window or
   frame as needed.
   """

   args = __utils__.format_add_argslist (0,True,__utils__.get_object_type(ChipsLine()),*args)

   if (isinstance(args[0], __pychips_base__.ChipsId)):
      id = args[0]
      b = args[1]
      endargs = args[2:]
   else:
      id = ChipsId ();
      b = args[0]
      endargs = args[1:]

   if (len (endargs) == 0):
      endargs = tuple([""])

   #see if a data coordinate system exists; if so find the appropriate
   #min and max for that system
   coord_sys = id.get_coord_sys ()
   if coord_sys == __pychips_base__.chips_none:
      cur = info_current ()
      if cur != None:
         pos1 = cur.find ('Coord Sys')
         pos2 = cur[pos1:].find ('\n')
         cur = cur[pos1+10:pos2+pos1]
         if cur == '[Data]':
            coord_sys = __pychips_base__.DATA


   # if data coordinates find actual end points otherwise just use 0 and 1
   if coord_sys == __pychips_base__.DATA:
      x_min = convert_coordinate (0, __pychips_base__.PLOT_NORM, id, __pychips_base__.X_AXIS)
      x_max = convert_coordinate (1, __pychips_base__.PLOT_NORM, id, __pychips_base__.X_AXIS)
   else:
      x_min = 0
      x_max = 1

   #instead of trying to determine what endargs is (list, string, etc)
   #pass it in at creation and set the line extends in a seperate command
   __pychips_base__.lock ()

   try:
      __pychips_base__.open_undo_buffer ()
      add_line (id, x_min, b, x_max, b, *endargs)
      __pychips_base__.set_line_extend (__pychips_base__.chips_extend_both)
   except:
      __pychips_base__.discard_undo_buffer ()
      __pychips_base__.unlock ()
      raise
   else:
      __pychips_base__.close_undo_buffer ()
      __pychips_base__.unlock ()

   return
   
def add_vline (*args):
   """
   add_vline([ChipsId,] b [,attribute_list])

   Adds a vertical line which intercepts the x axis at 'b'. The
   vertical line extends indefinitely as limits and ranges
   of a coordinate system change. The call creates a window or
   frame as needed.
   """

   args = __utils__.format_add_argslist (0,True,__utils__.get_object_type(ChipsLine()),*args)

   if (isinstance(args[0], __pychips_base__.ChipsId)):
      id = args[0]
      b = args[1]
      endargs = args[2:]
   else:
      id = ChipsId ();
      b = args[0]
      endargs = args[1:]

   if (len (endargs) == 0):
      endargs = tuple([""])

   #see if a data coordinate system exists; if so find the appropriate
   #min and max for that system
   coord_sys = id.get_coord_sys ()
   if coord_sys == __pychips_base__.chips_none:
      cur = info_current ()
      if cur != None:
         pos1 = cur.find ('Coord Sys')
         pos2 = cur[pos1:].find ('\n')
         cur = cur[pos1+10:pos2+pos1]
         if cur == '[Data]':
            coord_sys = __pychips_base__.DATA


   # if data coordinates find actual end points otherwise just use 0 and 1
   if coord_sys == __pychips_base__.DATA:
      y_min = convert_coordinate (0, __pychips_base__.PLOT_NORM, id, __pychips_base__.Y_AXIS)
      y_max = convert_coordinate (1, __pychips_base__.PLOT_NORM, id, __pychips_base__.Y_AXIS)
   else:
      y_min = 0
      y_max = 1


   #instead of trying to determine what endargs is (list, string, etc)
   #pass it in at creation and set the line extends in a seperate command
   __pychips_base__.lock ()

   try:
      __pychips_base__.open_undo_buffer();
      add_line (id, b, y_min, b, y_max, *endargs)
      __pychips_base__.set_line_extend (__pychips_base__.chips_extend_both)
   except:
      __pychips_base__.discard_undo_buffer()
      __pychips_base__.unlock ()
      raise
   else:
      __pychips_base__.close_undo_buffer()
      __pychips_base__.unlock ()

   return


####################################
#  OVERWRITES OF LIBRARY ROUTINES  #
####################################


#####################################################
#
#  overwrite creation functions
#
#####################################################


def add_window (*args):
   __pychips_base__.add_window (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsWindow()),*args)))
add_window.__doc__ = __pychips_base__.add_window.__doc__

def add_frame (*args):
   __pychips_base__.add_frame (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsFrame()),*args)))
add_frame.__doc__ = __pychips_base__.add_frame.__doc__

def add_label (*args):
   __pychips_base__.add_label (*(__utils__.format_add_argslist (1,True,__utils__.get_object_type(ChipsLabel()),*args)))
add_label.__doc__ = __pychips_base__.add_label.__doc__

def add_line (*args):
   __pychips_base__.add_line (*(__utils__.format_add_argslist (0,True,__utils__.get_object_type(ChipsLine()),*args)))
add_line.__doc__ = __pychips_base__.add_line.__doc__

def add_point (*args):
   __pychips_base__.add_point (*(__utils__.format_add_argslist (0,True,__utils__.get_object_type(ChipsPoint()),*args)))
add_point.__doc__ = __pychips_base__.add_point.__doc__

# note this is new function not in the library
def add_plotarea(*args):
   """
   add_plotarea([ChipsId,] [x1, y1, x2, y2] [,attributes])

   Adds a plot to the current frame. If x1,y1,x2,y2 are provided, they are used
   to specify the location of the plot drawing area otherwise default values
   are used.
   """
   __pychips_base__.add_plot (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsPlot()),*args)))

def add_plot (*args):
   __pychips_base__.add_plot (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsPlot()),*args)))
add_plot.__doc__ = __pychips_base__.add_plot.__doc__

def add_curve(*args):
   """
   add_curve([ChipsId], x, y, [error_list], [limits_list], [attributes])'

   adds a curve with optional error bars and/or upper/lower limits. Error
   list may be None, a single array without list brackets, or from one to
   four arrays enclosed in list brackets. Limits_list may be None a single
   array without list brackets, or one or two arrays enclosed in brackets.
   """
   _add_curve_ (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsCurve()),*args)))
   return

def add_contour (*args):
   _add_contour_ (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsContour()),*args)))
add_contour.__doc__ = __pychips_base__.add_contour.__doc__

def add_histogram (*args):
   __pychips_base__.add_histogram (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsHistogram()),*args)))
add_histogram.__doc__ = __pychips_base__.add_histogram.__doc__

def add_axis (*args):
   _add_axis_ (*(__utils__.format_add_argslist (0,True,__utils__.get_object_type(ChipsAxis()),*args)))
add_axis.__doc__ = __pychips_base__.add_axis.__doc__

def add_region (*args):
   __pychips_base__.add_region (*(__utils__.format_add_argslist (0,True,__utils__.get_object_type(ChipsRegion()),*args)))
add_region.__doc__ = __pychips_base__.add_region.__doc__

def add_image (*args):
   _add_image_ (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsImage()),*args)))
add_image.__doc__ = __pychips_base__.add_image.__doc__

def print_window (*args):
   new_args = []
   if len(args) == 3:
      for elem in args:
         new_args.append(elem)
   elif len(args) == 2:
      if isinstance(args[0], __pychips_base__.ChipsId):
         # for object is ChipsId, other could be file or props
         new_args.append(args[0])
         if isinstance (args[1], str):
            # try to create an attribute list. if a list is created
            # then assume that they are lists of attributes otherwise
            # assume that it is a filename
            tmp_list = __utils__.string_to_attribute_list (args[1])
            if len(tmp_list) > 1:
               # if only an att list provided, seed a blank filename
               new_args.append ("");
         else:
            new_args.append ("")
         new_args.append(args[1])
      else:
         # args are filename and props 
         new_args.append(args[0])
         new_args.append(args[1])
   elif len(args) == 1:
      if isinstance(args[0], __pychips_base__.ChipsId):
         # only arg is ChipsId
         new_args.append(args[0])
      elif isinstance (args[0], str):
         # try to create an attribute list. if a list is created
         # then assume that they are lists of attributes otherwise
         # assume that it is a filename
         tmp_list = __utils__.string_to_attribute_list (args[0])
         if len(tmp_list) > 1:
            # if only an att list provided, seed a blank filename
            new_args.append ("") 
         new_args.append(args[0])
      else:
         new_args.append ("");
         new_args.append(args[0])
   __pychips_base__.print_window (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsExport()),*new_args)))
print_window.__doc__ = __pychips_base__.print_window.__doc__


def print_image (*args):
   __pychips_base__.print_image (*(__utils__.format_add_argslist (0,False,__utils__.get_object_type(ChipsExport()),*args)))
print_image.__doc__ = __pychips_base__.print_image.__doc__

# private add colorbar function
def _add_colorbar_ (*args):
   """
   add_colorbar([ChipsId,] x, y [,object_id] [,attribute_list])

   displays a colorbar at the specified plot normalized coordinates. The
   colorbar is associated with the specified object.
   """

   new_args = []
   arg_cnt = 0

   img_dimx = -1
   img_dimy = -1

   image_length = 0

   # pull off the chips id if it exists
   if len(args) != 0 and isinstance(args[0], __pychips_base__.ChipsId):
      new_args.append (args[0])
      arg_cnt = 1

   if len (args[arg_cnt:]) < 2:
      raise AttributeError("chips ERROR: add_colorbar takes at least 2 arguments an x position and a y postion")

   end_arg = args[len (args)-1]
   new_args.append (args[arg_cnt])
   new_args.append (args[arg_cnt+1])
   arg_cnt += 2

#
# *** REMOVE object_type as option for now ***
#
#
#
#   # if there are more than 2 arguments try to parse out
#   # what they are
#   if len (args[arg_cnt:]) == 1:
#
#      # if the last argument is a chips_object 
#      if __utils__.isinstance_int (end_arg):
#   
#         # add in an empty string for the object id
#         new_args.append ("")
#         new_args.append (args[arg_cnt])
#
#         arg_cnt += 1
#
#      # if the last argument is a string then its either an
#      # attribute list or an object id
#      elif isinstance (end_arg, str):
#         # try to create an attribute list. if a list is created
#         # then assume that they are lists of attributes otherwise
#         # assume that it is a object id
#         tmp_list = __utils__.string_to_attribute_list (end_arg)
#      
#         if len(tmp_list) > 1:
#            new_args.append (args[arg_cnt])
#         else:
#            new_args.append (args[arg_cnt])
#            new_args.append (__pychips_base__.chips_not_object)
#         arg_cnt += 1
#      else:
#         raise AttributeError("chips ERROR: Unknown or unexpected parameter type. For argument %d a chips_object type, chips object id, or an attribute list."%(arg_cnt+1))
#
#   elif len (args[arg_cnt:]) == 2:
#      next_to_last = args[len (args)-2]
#
#
#      # if the last argumement is an int assume its a chips_object type
#      # if the last argument is a chips_object
#      if __utils__.isinstance_int (end_arg):
#
#         # add in an empty string for the object id
#         new_args.append (next_to_last)
#         new_args.append (end_arg)
#
#         arg_cnt += 2
#
#      # if the last argument is a string then it must be an
#      # attribute list 
#      elif isinstance (end_arg, str):
#       if isinstance (end_arg, str):
#
#         # if the second to last is an int assume its an object type
#         if __utils__.isinstance_int (next_to_last):
#            # add in an empty chips id
#            new_args.append ("")
#            new_args.append (next_to_last)
#            new_args.append (end_arg)
#         elif isinstance (next_to_last, str):
#            new_args.append (next_to_last)
#            new_args.append (__pychips_base__.chips_not_object)
#            new_args.append (end_arg)
#         else:
#            raise AttributeError("chips ERROR: Unknown or unexpected parameter type. For argument %d expected a chips_object type or a chips object id."%(len (args)-1))
#         arg_cnt += 2
#      else:
#         raise AttributeError("chips ERROR: Unknown or unexpected parameter type. For argument %d a chips_object type or an attribute list."%(len (args)))

# *** code with object_type removed ***
   if len (args[arg_cnt:]) == 1:
      if isinstance (end_arg, str):
         # try to create an attribute list. if a list is created
         # then assume that they are lists of attributes otherwise
         # assume that it is a object id
         tmp_list = __utils__.string_to_attribute_list (end_arg)

         if len(tmp_list) > 1:
            new_args.append (args[arg_cnt])
         else:
            new_args.append (args[arg_cnt])
            new_args.append (__pychips_base__.chips_not_object)
         arg_cnt += 1
      else:
         raise AttributeError("chips ERROR: Unknown or unexpected parameter type. For argument %d a chips object id or an attribute list."%(arg_cnt+1))


# *** end code ***
   elif len (args[arg_cnt:]) == 2:
      new_args.append (args[arg_cnt])
      new_args.append (__pychips_base__.chips_not_object)
      new_args.append (args[arg_cnt+1])
   elif len (args[arg_cnt:]) > 2:
      raise AttributeError("chips ERROR: Unknown or unexpected parameter found. Expected the following parameters [ChipsId,] x, y [,object_id] [,attribute_list]")

   __pychips_base__.add_colorbar (*new_args)

def add_colorbar (*args):
   _add_colorbar_ (*(__utils__.format_add_argslist (0,True,__utils__.get_object_type(ChipsColorBar()),*args)))
add_colorbar.__doc__ = _add_colorbar_.__doc__



# private add_contour function
def _add_contour_(*args):
   new_args = []
   arg_cnt = 0

   img_dimx = -1
   img_dimy = -1

   image_length = 0
  
   if len (args) == 0:
      raise AttributeError("chips ERROR: No arguments supplied. add_contour requires at least one argument.");

   # pull off the chips id if it exists
   if isinstance(args[0], __pychips_base__.ChipsId):
      new_args.append (args[0])
      arg_cnt = 1

   if arg_cnt == len(args) or not __utils__.check_seq(args[arg_cnt]):
      raise AttributeError("chips ERROR: Invalid argument. Expect argument %d to be an array instead of %s"%(arg_cnt,type(args[arg_cnt])))
   
   new_args.append (args[arg_cnt])
   image_length = __utils__.array_length (args[arg_cnt])

   if image_length == 0:
      raise AttributeError ("chips ERROR: Zero length arrays are not supported for the add_contour command")

   if isinstance (args[arg_cnt], numpy.ndarray):
      if args[arg_cnt].ndim == 2:
         img_dimx = args[arg_cnt].shape[1]
         img_dimy = args[arg_cnt].shape[0]
      elif args[arg_cnt].ndim == 1:
         img_dimx = args[arg_cnt].shape[0]
         img_dimy = 1

   arg_cnt += 1

   # if the next array is a list and the array after that is a list assume
   # they are part of the contour data
   if arg_cnt+1 < len(args) and __utils__.check_seq(args[arg_cnt]) and __utils__.check_seq(args[arg_cnt+1]):

         new_args.append (args[arg_cnt])
         new_args.append (args[arg_cnt+1])
         arg_cnt += 2


   # otherwise check to see if the next two arguments are the dimensions
   else:
      if arg_cnt+1 < len(args):
         if __utils__.isinstance_int (args[arg_cnt]) and \
             __utils__.isinstance_int (args[arg_cnt+1]):
            img_dimx = args[arg_cnt]
            img_dimy = args[arg_cnt+1]
            arg_cnt += 2


      if img_dimx == -1 or img_dimy == -1:
         raise AttributeError("chips ERROR: The dimensions of the image have not been supplied.")
      else:
         if image_length != img_dimx * img_dimy:
            raise AttributeError("chips ERROR: The length of the data values (%d) does not match the specified dimensions (%d x %d)"%(image_length,img_dimx,img_dimy))

         new_args.append (img_dimx)
         new_args.append (img_dimy)

   # check for levels
   if arg_cnt != len(args):
      if __utils__.check_seq(args[arg_cnt]):
         new_args.append (args[arg_cnt])
         arg_cnt += 1


   # check for a transform
   if arg_cnt != len(args):
      if isinstance (args[arg_cnt], pytransform.Transform) or args[arg_cnt] == None:
         new_args.append (args[arg_cnt])
         arg_cnt += 1
      else:
         new_args.append (None)
   else:
      new_args.append (None)

   # get the attributes
   if arg_cnt != len(args):
      if not isinstance (args[arg_cnt], str):
         raise AttributeError("chips ERROR: Unexpected argument for argument number %d. Expected this to be the contour attribute list."%(arg_cnt+1))
      new_args.append (args[arg_cnt])
      arg_cnt += 1

   arg_cnt+1
   if arg_cnt != len(args):
      raise AttributeError("chips ERROR: Too many arguments supplied. Did not expect anything after argument %d."%arg_cnt)

   __pychips_base__.add_contour (*new_args)

   
         

# private add image function
def _add_image_(*args):

   new_args = []
   arg_cnt = 0

   img_dimx = -1
   img_dimy = -1

   image_length = 0
  
   # pull off the chips id if it exists
   if isinstance(args[0], __pychips_base__.ChipsId):
      new_args.append (args[0])
      arg_cnt = 1

   # pull out all images, there can be up to 4 images
   array_cnt = 0
   for ii in range (4):

      # if there are no more arguments in the list break
      if arg_cnt == len(args):
         break

      # if the next item is not an array and not null break
      if not __utils__.check_seq(args[arg_cnt]) \
         and arg_cnt < len(args) and args[arg_cnt] is not None:
         break

      tmp_image_length = 0

      if args[arg_cnt] == None:
         tmp_image_length = image_length
      else:
         tmp_image_length = __utils__.array_length (args[arg_cnt])

      if isinstance (args[arg_cnt], numpy.ndarray):
         if args[arg_cnt].ndim == 2:
            img_dimx = args[arg_cnt].shape[1]
            img_dimy = args[arg_cnt].shape[0]
         elif args[arg_cnt].ndim == 1:
            img_dimx = args[arg_cnt].shape[0]
            img_dimy = 1
            

      # verify that the length of the arrays are the same
      if image_length == 0:
         image_length = tmp_image_length
      elif tmp_image_length != image_length:
         raise AttributeError("chips ERROR: Not all the supplied images have equal lengths. Length %d does not match the previous length %d"%(tmp_image_length,image_length))

      new_args.append (args[arg_cnt])
      arg_cnt += 1
      array_cnt += 1

   if image_length == 0:
      raise AttributeError ("chips ERROR: Zero length arrays are not supported for the add_image command")

   # set any images that were not set by the user to NULL
   for ii in range (4-array_cnt):
      new_args.append (None)

   # check to see if the next two arguments are the dimensions
   if arg_cnt+1 < len(args):
      if __utils__.isinstance_int (args[arg_cnt]) and \
          __utils__.isinstance_int (args[arg_cnt+1]):
         img_dimx = args[arg_cnt]
         img_dimy = args[arg_cnt+1]
         arg_cnt += 2
   
   if img_dimx == -1 or img_dimy == -1:
      raise AttributeError("chips ERROR: The dimensions of the image have not been supplied.")
   else:
      if image_length != img_dimx * img_dimy:
         raise AttributeError("chips ERROR: The length of the data values (%d) does not match the specified dimensions (%d x %d)"%(image_length,img_dimx,img_dimy))

      new_args.append (img_dimx)
      new_args.append (img_dimy)

   # check for a transform
   if arg_cnt != len(args):
      if isinstance (args[arg_cnt], pytransform.Transform) or args[arg_cnt] == None:
         new_args.append (args[arg_cnt])
         arg_cnt += 1
      else:
         new_args.append (None)
   else:
      new_args.append (None)

   # get the attributes
   if arg_cnt != len(args):
      if not isinstance (args[arg_cnt], str):
         raise AttributeError("chips ERROR: Unexpected argument for argument number %d. Expected this to be the image attribute list."%(arg_cnt+1))
      new_args.append (args[arg_cnt])
      arg_cnt += 1

   arg_cnt+1
   if arg_cnt != len(args):
      raise AttributeError("chips ERROR: Too many arguments supplied. Did not expect anything after argument %d."%arg_cnt)

   __pychips_base__.add_image (*new_args)

# private add curve function
#def _add_curve_(arg1, arg2, arg3=None, arg4=None, arg5=None, arg6=None, arg7=None):
def _add_curve_ (*args):
   'curve routine that allows error list and/or upper/lower limits list'
   e = [ None, None, None, None]
   l1 = None
   l2 = None
   

   new_args = [None]*11
   arg_index = 0

   if len(args) < 2:
      raise AttributeError ("chips ERROR: Invalid number of arguments. Expected at least two arguments x and y data.")
   
   if isinstance (args[arg_index], __pychips_base__.ChipsId):
      new_args[0] = (args[arg_index])
      arg_index += 1
   

   try:
      datalen = len(args[arg_index])
   except:
      raise AttributeError ("chips ERROR: Insufficient data to plot- one or more missing column(s)")

   new_args[1] = args[arg_index]
   arg_index += 1

   if len(args) == arg_index:
      raise AttributeError ("chips ERROR: Invalid number of arguments. Expected at least x and y data.")

   try:
      local_datalen = len(args[arg_index])
   except:
      raise AttributeError ("chips ERROR: Insufficient data to plot- one or more missing column(s)")

   if datalen != local_datalen:
      raise AttributeError("chips ERROR: All arrays must be of equal length. X array length (%d). Y array length (%d)"%(datalen, local_datalen))

   new_args[2] = args[arg_index]
   arg_index += 1

   arg_info = (('error',4), ('limits',2))

   new_arg_index = 3
   for arg_name,arg_max in arg_info:
      error_count = 0
      while error_count != arg_max and len(args) > arg_index:
         if args[arg_index] is None:
            arg_index += 1
            error_count += 1
         elif __utils__.check_seq(args[arg_index]):
            # check to see if errors are a list of errors

            # if we found a list of within a list and the error count
            # is not 0 then this is no longer an error 
            if __utils__.check_seq (args[arg_index][0]) and error_count != 0:
               error_count = arg_max
            elif __utils__.check_seq (args[arg_index][0]) or args[arg_index][0] is None:
               if len (args[arg_index]) > arg_max:
                  raise AttributeError("chips ERROR: %s list should contain no more than %d columns"%(arg_name, arg_max))
         
               for ii, jj in zip (args[arg_index], range(len(args[arg_index]))):
                  new_args[new_arg_index+jj] = ii
               arg_index += 1
               error_count = arg_max
            elif not __utils__.isinstance_numeric (args[arg_index][0]):
               raise TypeError("chips ERROR: %s lists should contain numerical values"%arg_name)
            elif len (args[arg_index]) != datalen:
               raise AttributeError("chips ERROR: All arrays must be of equal length")
            else:
               new_args[new_arg_index+error_count] = args[arg_index]
               arg_index += 1
               error_count += 1
         else:
            error_count = arg_max
      new_arg_index += arg_max


   if arg_index < len(args):
      if isinstance (args[arg_index], pytransform.Transform) or args[arg_index] == None:
         new_args[9] = args[arg_index]
         arg_index += 1
      else:
         new_args[9] = None
   else:
      new_args[9] = None

   if arg_index+1 == len(args):
      new_args[10] = args[arg_index]
   elif arg_index < len(args):
      raise TypeError("chips ERROR: Too many arguments provided for add_curve")
   else:
      new_args[10] = ""

   __pychips_base__.add_curve (*new_args)

         
def _add_axis_ (*args):
   'overloaded add axis for argument checking'

   # check for the number of arguments
   if len(args) < 4:
      raise AttributeError("chips ERROR: add_axis insufficient number of arguments. add_axis must have a dim, position, min value and max value")

   arg_cnt = 0

   if isinstance (args[arg_cnt], __pychips_base__.ChipsId):
      new_args = [None]*7
      new_args[0] = (args[arg_cnt])
      arg_cnt += 1
   else:
      new_args = [None]*6

   # verify the first 4 arguments
   if not args[arg_cnt] in [__pychips_base__.X_AXIS, __pychips_base__.Y_AXIS, __pychips_base__.XY_AXIS]:
      raise TypeError("chips ERROR: Invalid argument type. The first add_axis argument must be the axis dimension (X_AXIS, Y_AXIS, XY_AXIS)")
 
   if not __utils__.isinstance_numeric (args[arg_cnt+1]):
      raise TypeError("chips ERROR: Invalid argument type. The second add_axis argument must be the axis position")

   if not __utils__.isinstance_numeric (args[arg_cnt+2]):
      raise TypeError("chips ERROR: Invalid argument type. The third add_axis argument must be the axis min value")
   
   if not __utils__.isinstance_numeric (args[arg_cnt+3]):
      raise TypeError("chips ERROR: Invalid argument type. The fourth add_axis argument must be the axis max value")

   new_args[arg_cnt:arg_cnt+2] = args[arg_cnt:arg_cnt+2]
   arg_cnt += 4
   dim = args[0]

   # check additional min/max values
   if arg_cnt != len(args) and __utils__.isinstance_numeric (args[arg_cnt]):
      if arg_cnt+1 != len(args) and __utils__.isinstance_numeric (args[arg_cnt+1]):
         new_args[arg_cnt-2:arg_cnt+2] = args[arg_cnt-2:arg_cnt+2]
         arg_cnt += 2
      else:
         raise AttributeError("chips ERROR: add_axis insufficient number of arguments. The Y axis lower bound was found but not the upper bound.")
   else:
      if dim == __pychips_base__.X_AXIS:
         new_args[arg_cnt-2:arg_cnt] = args[arg_cnt-2:arg_cnt] 
         new_args[arg_cnt:arg_cnt+2] = [0,0]
      elif dim == __pychips_base__.Y_AXIS:
         new_args[arg_cnt-2:arg_cnt] = [0,0]
         new_args[arg_cnt:arg_cnt+2] = args[arg_cnt-2:arg_cnt]
      else:
         new_args[arg_cnt-2:arg_cnt] = args[arg_cnt-2:arg_cnt]
         new_args[arg_cnt:arg_cnt+2] = args[arg_cnt-2:arg_cnt]

   # check for a transform
   if arg_cnt != len(args):
      if isinstance (args[arg_cnt], pytransform.Transform) or args[arg_cnt] == None:
         new_args.append (args[arg_cnt])
         arg_cnt += 1
      else:
         new_args.append (None)
   else:
      new_args.append (None)

   if arg_cnt != len(args):
      new_args += args[arg_cnt:]

   __pychips_base__.add_axis (*new_args)


#####################################################
#
#  scriptable of chips id structure
#
#####################################################

class ChipsId (__pychips_base__.ChipsId):
   def __init__ (self, *args):
      super(ChipsId,self).__init__(*args)

      # store a list of valid properties
      self.attributes = ['window', 'frame', 'plot', 'axis', 'xaxis', \
                   'yaxis', 'curve', 'image', 'colorbar', 'contour', \
                   'histogram', 'label', 'line', 'point', 'region', \
                   'coord', 'coord_sys']
      

   # using property may be more ideal but doesn't seem to
   # work. It ignores the set function. This seems to be
   # due to something done to ChipsId by swig.
   def __setattr__(self,name,value):

      if value == None:
         value = ''

      if name == 'coord_sys':
         self.set_coord_sys (value)
      elif (((type (value) == type (__pychips_base__._pychips.new_ChipsId ())) \
            and (name == 'this')) or \
            (name == 'attributes')):
         self.__dict__[name] = value
      elif name in self.__dict__['attributes']:
         # create a string representation of the attribute
         # function and execute it
         func = "self.set_"+name+"_id"
         eval(func)(value)
      else:
         raise NameError, 'chips ERROR: Invalid member "'+name+'"'

   def __getattr__(self,name):
      if name == 'coord_sys':
         ret_val = self.get_coord_sys ()
      elif name == 'attributes':
         ret_val = self.__dict__[name]
      elif name in self.__dict__['attributes']:
         # create a string representation of the attribute
         # function and execute it
         func = "self.get_"+name+"_id ()"
         ret_val = eval(func)
      else:
         raise NameError, 'chips ERROR: Invalid member "'+name+'"'
      if ret_val == '':
         ret_val = None
      return ret_val

   def __str__(self):
      ret_str = ''
      for p in self.attributes:
         ret_str = ret_str + p + ' = %s' % self.__getattr__ (p)
         ret_str = ret_str + '\n'
      return ret_str

   def __repr__(self):
      ret_str = ''
      for p in self.attributes:
         ret_str = ret_str + p + ' = %s' % self.__getattr__ (p)
         ret_str = ret_str + '\n'
      return ret_str

   


#####################################################
#
#  overwrites to the coordinate functions
#
#####################################################

def convert_coordinate (in_val, in_coord=ChipsId (), out_coord=ChipsId (), axis=__pychips_base__.XY_AXIS):

   """

        convert_coordinate (point, [input coordinate], [output coordinate], [axis])

        Returns the conversion of values from one coordinate system to
        another. Valid coordinate systems are PIXEL, WINDOW_NORM, FRAME_NORM,
        PLOT_NORM, and DATA. Coordinate systems can be passed in as part of a
        chips id, as an integer enumeration, or as a string when specifying
        two DATA coordinates that are part of the same plot. When a chips id
        is not passed in currency rules apply. The input coordinate values
        can be either an array of values representing a point (ex. [X, Y])
        or a single value X, in which case the axis (X_AXIS or Y_AXIS) must
        also be passed.

   """

   cid =  [ChipsId (), ChipsId ()];
   ii = 0;

   #convert the coordinate system to a cid for the C++ API
   for coord in (in_coord, out_coord):
      if isinstance (coord, ChipsId):
         cid[ii] = coord
      elif __utils__.isinstance_int (coord):
         cid[ii].coord_sys = coord
      elif type (coord) == str:
         cid[ii].coord_sys = __pychips_base__.DATA
         cid[ii].coord = coord
      elif coord != None:
         raise AttributeError("chips ERROR: Invalid coordinate type")
      ii = ii + 1

   return __pychips_base__.convert_coordinate (in_val, cid[0], cid[1], axis)


#####################################################
#
#  overwrites to the limit functions
#
#####################################################


def limits (*args):
   """
   limits([ChipsId,] dimension, min, max)
   limits([ChipsId,] object_type, object_id)
   limits(ChipsId, dimension, object_type, ChipsId2)
   limits([ChipsId,] xmin, xmax, ymin, ymax)

   Sets the limits of the axes specified by dimension (X_AXIS|Y_AXIS|XY_AXIS) in
   the plot specified by ChipsId (current if omitted). If min or max is set to
   the string 'AUTO' then that value is automatically set based on the data
   range. The method taking in an object type and object_id will set the X and
   Y axes of the current plot to the limits of the object identified by those
   parameters. The method taking in a ChipsId,a dimension, an object type, and
   a second ChipsId can be used to specify which dimensions to apply the limits
   of the object specified by object _type and ChipsId2
   """

   # if we don't have any arguments or get a None in any position outside of
   # the first (optional ChipsId), then make an invalid limits call to get
   # back the valid api
   nargs = len(args)
   if nargs == 0:
      __pychips_base__.limits (0)
   elif nargs > 1:
      for i in range (1,nargs):
         if args[i] == None:
            __pychips_base__.limits (0)

   if isinstance (args[0], __pychips_base__.ChipsId):
      cid_arg = 1
   else:
      cid_arg = 0

   # check to see if the arguments passed are limits (int, int, string)
   # this could either be limits (dim, chips_object, object id) or
   # limits (dim, min limit, auto). Try the first and then the second.
   if (nargs == 3) and (__utils__.isinstance_int(args[0]) and \
       __utils__.isinstance_int(args[1]) and type(args[2]) == str):
      try:
         myid = __utils__.set_id (args[2], args[1])
         __pychips_base__.limits (args[0], args[1], myid)
      except:
         __pychips_base__.limits (*args)
   elif (nargs == 4) and (isinstance (args[0], __pychips_base__.ChipsId) and \
         __utils__.isinstance_int(args[1]) and __utils__.isinstance_int(args[2]) and \
         type(args[3]) == str):
        try:
           myid = __utils__.set_id (args[3], args[2])
           __pychips_base__.limits (args[0], args[1], args[2], myid)
        except:
           __pychips_base__.limits (*args)

   # check for sequences
   elif len (args[cid_arg:]) == 1 and __utils__.check_seq (args[cid_arg]):
      if len(args[cid_arg]) != 4:
         raise AttributeError("chips ERROR: Expected list argument for limits to have 4 values, min x, max x, min y, max y")
      if cid_arg > 0:
         __pychips_base__.limits (args[cid_arg-1], args[cid_arg][0], args[cid_arg][1], args[cid_arg][2], args[cid_arg][3])
      else:
         __pychips_base__.limits (args[cid_arg][0], args[cid_arg][1], args[cid_arg][2], args[cid_arg][3])
   elif len (args[cid_arg:]) == 3 and __utils__.check_seq (args[cid_arg]):
      if args[cid_arg] != X_AXIS and args[cid_arg] != Y_AXIS and args[cid_arg] != XY_AXIS:
         raise AttributeError("chips ERROR: An invalid argument to the limits command was found, %s. Expecting X_AXIS | Y_AXIS | XY_AXIS"%str (args[cid_arg:]))
      if len(args[cid_arg+1]) != 2:
         raise AttributeError("chips ERROR: Expected list argument for limits to have 2 values, min limit, max limit")
      if cid_arg > 0:
         __pychips_base__.limits (args[cid_arg-1], args[cid_arg], args[cid_arg+1][0], args[cid_arg+1][1])
      else:
         __pychips_base__.limits (args[cid_arg], args[cid_arg+1][0], args[cid_arg+1][1])

         
   else:
      __pychips_base__.limits (*args)




#####################################################
#
#  overwrites to the server specific functions
#
#####################################################


def connect (server_id='', myid=''):

   """
   Attempt to attach to the specified server. If no server is specified and a single server alread exists attempt to attach to the existing server.
   """

   if server_id == '':
      server_ids = __pychips_base__.list_servers()
      if len (server_ids) == 0:
         raise AttributeError("chips ERROR: There are no available servers to connect to.");
      elif len (server_ids) > 1:
         raise AttributeError("chips ERROR: There are multiple servers available to connect to. Please specify a specific server. Use list_servers () to see a list of existing servers.")
      else:
         server_id = server_ids[0]

   __pychips_base__.connect (server_id, myid)


#####################################################
#
#  overwrites to the info functions
#
#####################################################

def info (*args):
   info_string = __pychips_base__.info (*args)

   if (info_string != None):
      info_string = __utils__.ChipsPrintableString (info_string)

   return info_string
info.__doc__ = __pychips_base__.info.__doc__

def info_coordinate (*args):
   info_string = __pychips_base__.info_coordinate (*args)

   if (info_string != None):
      info_string = __utils__.ChipsPrintableString (info_string)

   return info_string
info_coordinate.__doc__ = __pychips_base__.info_coordinate.__doc__


def info_depth (*args):
   info_string = __pychips_base__.info_depth (*args)

   if (info_string != None):
      info_string = __utils__.ChipsPrintableString (info_string)

   return info_string
info_depth.__doc__ = __pychips_base__.info_depth.__doc__


def info_bound_axes (*args):
   info_string = __pychips_base__.info_bound_axes (*args)

   if (info_string != None):
      info_string = __utils__.ChipsPrintableString (info_string)

   return info_string
info_bound_axes.__doc__ = __pychips_base__.info_bound_axes.__doc__

def make_script(filename, overwrite=False):
   """

   make_script(filename, overwrite=False)

   Generates a python script that may be used to replicate the current
   state of the chips Server.
   """
   if type(filename) != str:
      raise AttributeError("chips ERROR: expected a string filename")
   __pychips_base__.save_state(filename, "template")
   tmpname = filename + "_TmP.Py"
   infile=open(filename)
   outfile=open(tmpname, 'w')
   outfile.write("from pychips.all import *  # import chips hlui and advanced commands\n\n")
   header_lines = []
   line = infile.readline()
   added = False

   # write out all the comments at the top
   while line != '' and line.startswith ('#'):
      outfile.write(line)
      line = infile.readline()

   # read in the header
   while line != '' and not line.startswith ('open_undo_block'):
      header_lines.append (line)
      line = infile.readline()

   for ii in range (len(header_lines)):
      line2 = header_lines[ii]
      if (line2.find('MyData_') == 0 and line2.find('= DataObj()') != -1 and not added):
         # define data object class
         outfile.write("class DataObj(): pass\n\n")
         added = True

      # create an ascii transform interpreter
      if (line2.find('MyData_') == 0 and line2.find('insert transform here -') != -1):
         split_line = line2.split ('=')
         line2 = split_line[1]
         pos = line2.find('insert transform here -') + len ('insert transform here -')
         line2 = line2[pos:].strip ()
         header_lines[ii] = split_line[0] + ' = load_ascii_transform ("'+line2+'")\n'

   outfile.writelines (header_lines)

   while line != '':
      outfile.write(line)
      line = infile.readline()


   infile.close()
   outfile.close()
   os.remove(filename)
   os.rename(tmpname, filename)


def info_current (*args):
   info_string = __pychips_base__.info_current (*args)

   if (info_string != None):
      info_string = __utils__.ChipsPrintableString (info_string)

   return info_string
info_current.__doc__ = __pychips_base__.info_current.__doc__

def set_image_channel (arg1, arg2, arg3=None, arg4=None, arg5=None):
   """

   set_image_channel([ChipsId,] image_array, channel)
   set_image_channel([ChipsId,] image_array, xdim, ydim, channel)

   Sets the specified image channel (0=red|1=green|2=blue|3=alpha) with
   the values provided in the image_array. If the optional ChipsId or image id 
   string is not supplied, the routine will operate on the current image.
   """
   my_id = None
   my_vals = None
   my_xdim = None
   my_ydim = None
   my_channel = None 

   my_vals_length = 0
   if isinstance (arg1, __pychips_base__.ChipsId) or \
       isinstance (arg1, str):
      my_id = arg1
      my_vals = arg2
  
      my_vals_length = __utils__.array_length (my_vals)
      if arg3 != None and arg4 != None and arg5 != None:
         my_xdim = arg3
         my_ydim = arg4
         my_channel = arg5
      elif arg3 != None and arg4 == None and arg5 == None:
         my_xdim = 1
         my_ydim = val_length
         my_channel = arg3
      else: 
         raise AttributeError("chips ERROR: invalid number of arguments to set_image_channel")
   else:
      my_vals = arg1
      my_vals_length = __utils__.array_length (my_vals)
      if arg2 != None and arg3 != None and arg4 != None and arg5 == None:
         my_xdim = arg2
         my_ydim = arg3
         my_channel = arg4
      elif arg2 != None and arg3 == None and arg4 == None and arg5 == None:
         my_xdim = 1;
         my_ydim = my_vals_length
         my_channel = arg2 
      else:  
         raise AttributeError("chips ERROR: invalid number of arguments to set_image_channel")

   if my_vals_length != my_xdim * my_ydim:
      raise AttributeError("chips ERROR: The length of the data values (%d) does not match the specified dimensions (%d x %d)"%(my_vals_length,my_xdim,my_ydim))

   if isinstance (my_channel, str):
      intchannel = __pychips_base__.string_to_v_rgba(my_channel)
      if not __pychips_base__.valid_line_type(intchannel):
         raise AttributeError("chips ERROR: The image rgba value (%s) must be one of [red, green, blue, alpha]"%my_channel)
   else:
      intchannel = my_channel
   __pychips_base__.set_image_channel(my_id, my_vals, my_xdim, my_ydim, intchannel)
    
def remove_image_channel (arg1, arg2=None):
   """

   remove_image_channel([ChipsId,] channel)

   Removes the specified channel (0=red|1=green|2=blue|3=alpha) from the
   current image or the one specified by the optional ChipsId structure if
   provided.
   """
   if isinstance (arg1, __pychips_base__.ChipsId) or \
       arg2 != None:
      channel = arg2
   else:
      channel = arg1

   if isinstance (channel, str):
      intchannel = __pychips_base__.string_to_v_rgba(channel)
      if not __pychips_base__.valid_line_type(intchannel):
         raise AttributeError("chips ERROR: The image rgba value (%s) must be one of [red, green, blue, alpha]"%channel)
   else:
      intchannel = channel

   if isinstance (arg1, __pychips_base__.ChipsId) or \
       arg2 != None:
      __pychips_base__.remove_image_channel (arg1, intchannel)
   else:
      __pychips_base__.remove_image_channel (intchannel)



#####################################################
#
#  misc functions
#
#####################################################

# create a class that can read a transform from an ascii file
def load_ascii_transform (filename):
   'Create a tranform from ascii file information'

   transform = None

   try:
      file = open (filename, 'r')
   except:
      raise AttributeError ("chips Error: Could not open file %s for reading"%filename)

   lines = file.readlines ()

   type = None
   name = ''
   flavor = ''
   paramList = []

   # first lines should contain the type, name, and flavor
   # NOTE: flavor is no longer used but is left in for backwards 
   #   compatibility with older files
   index = 0
   for line in lines:
      # remove white spaces
      line = line[:len(line)-1]
      line.strip ()
      if line.startswith ('type: '):
         type = line[6:]
      elif line.startswith ('name: '):
         name = line[6:]
      elif line.startswith ('flavor: '):
         flavor = line[8:]
      elif line.startswith ('parameter list begin'):
         break
      index += 1

   # load the parameter list
   param_name = ''
   param_desc = ''
   param_parent = ''
   param_nvals = 0
   param_datatype = 0
   param_values = []
   for line in lines[index:]:
      # remove spaces
      line = line[:len(line)-1]
      line.strip ()
      if line.startswith ('name: '):
         param_name = line[6:]
      elif line.startswith ('desc: '):
         param_desc = line[6:]
      elif line.startswith ('parent: '):
         param_parent = line[8:]
      elif line.startswith ('nvals: '):
         param_nvals = int(line[7:])
      elif line.startswith ('datatype: '):
         param_datatype = int(line[10:])
      elif line.startswith ('value: '):
         line = line[7:].split (',')
         for val in line:
            if val == 'NULL':
               param_values.append (None)
            elif param_datatype == pytransform.DT_BIT or \
                 param_datatype == pytransform.DT_CHAR:
               param_values.append (val)
            elif param_datatype == pytransform.DT_LOGICAL:
               param_values.append (bool(int(val)))
            elif param_datatype == pytransform.DT_REAL4 or \
                 param_datatype == pytransform.DT_REAL8:
               param_values.append (float(val))
            else:
               param_values.append (int(val))
      elif len(line) == 0 and param_name != '':
         param = pytransform.TransParam (param_name, param_values, param_desc)
         param.set_parent (param_parent)
         paramList.append (param)
         param_name = ''
         param_desc = ''
         param_parent = ''
         param_nvals = 0
         param_datatype = 0
         param_values = []

   # if there's a line left over create a parameter
   if param_name != '':
      param = pytransform.TransParam (param_name, param_values, param_desc)
      param.set_parent (param_parent)
      paramList.append (param)
      

   if type == 'WCSTAN':
      transform = pytransform.WCSTANTransform (name, paramList)
   elif type == 'LINEAR2D':
      transform = pytransform.LINEAR2DTransform (name, paramList)
   elif type == 'SPLIT':
      transform = pytransform.SPLITTransform (name, paramList)
   else:
      transform = pytransform.Transform (name, paramList)



   return transform


def set_arbitrary_tick_positions(id=None, positions=None, labels=None):
   """
   Sets the major tick locations to the values specified in the positions
   array. If a labels array is provided, those values are used as the labels.
   """
   if type(id) == str:
      cid = __pychips_base__.ChipsId()
      cid.set_axis_id(id)
      pos = positions
      labs = labels
   elif isinstance(id, __pychips_base__.ChipsId):
      cid = id
      pos = positions
      labs = labels
   elif id == None:
      cid = __pychips_base__.ChipsId()
      pos = positions
      labs = labels
   elif __utils__.check_seq(id):
      if labels != None:
         raise AttributeError("chips ERROR: Unknown or unexpected parameter type- usage: set_arbitrary_tick_positions([id], position array, [label array]")
      if positions != None:
         if __utils__.check_seq(positions):
            labs = positions
         else:
            raise AttributeError("chips ERROR: Unknown or unexpected parameter type- usage: set_arbitrary_tick_positions([id], position array, [label array]")
      else:
         labs = None
      cid = __pychips_base__.ChipsId()
      pos = id
   else :
      raise AttributeError("chips ERROR: Unknown or unexpected parameter type- expected an axis id, ChipsId object, or the positions array.")
  
   if pos == None:
      raise AttributeError("chips ERROR: The positions array value can not be set to None.")
   elif __utils__.check_seq(pos) == False:
      raise AttributeError("chips ERROR: The positions array value must be a numeric list/array.")
    
   for elem in pos:
      if elem == None:
         raise AttributeError("chips ERROR: The positions array values must be numeric values- detected a None in the array")
      elif type(elem) == str:
         raise AttributeError("chips ERROR: The positions array values must be numeric values- detected a string in the array")

   if labs != None:
      if __utils__.check_seq(labs) == False:
         raise AttributeError("chips ERROR: The labels array value must be a list/array.")
      elif len(labs) != len(pos):
         raise AttributeError("chips ERROR: The labels array must be the same length as the position array.")
      # convert all labels to strings 
      if labs != None:
         lbls = []
         for ii in range(0,len(labs)):
            lbls.append(str(labs[ii]))
      __pychips_base__.set_arbitrary_tick_positions(cid, pos, lbls)
      return
   __pychips_base__.set_arbitrary_tick_positions(cid, pos)


